using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.IO;
using System.Net;

#region using Assetlink includes
using Assetlink;
using System.Configuration;
using Assetlink.IOProxy;
//using Assetlink.Solutions;
#endregion

namespace ALCleanup
{
    
    class ALCleanup
    {
        public System.Diagnostics.EventLog objEventLog = null;
        static void Main(string[] args)
        {
            String InstallationDir   = null;
            String ALCleanupXML      = null;
            String location          = null;
            String DeletePeriod      = null;
            String FileExt           = null;
            String Comments          = null;
            List<String> lst_FoldersToAvoidList = null;
            ALCleanup CleanupObj    = new ALCleanup();

            try
            {
                CleanupObj.Log("Utility started running at : " + DateTime.Now.ToLongTimeString(), "", true);

                System.Net.ServicePointManager.ServerCertificateValidationCallback += ServerValidateCertificate;

                #region Load CleanUpXml

                String configPath = ConfigurationManager.AppSettings["SHAREDDRIVEROOT"];
                if (String.IsNullOrEmpty(configPath) == true)
                {
                    InstallationDir = ALCommon.GetInstallationDir();
                    ALCleanupXML = InstallationDir + "Configuration\\XML\\ALCleanUp.xml";
                }
                else
                {
                    ALCleanupXML = configPath + "\\Configuration\\XML\\ALCleanUp.xml";
                }

                if (!File.Exists(ALCleanupXML))
                {
                    System.Console.WriteLine();
                    System.Console.WriteLine("ALCleanUp.xml could not be found on the specified path - " + ALCleanupXML);
                    System.Console.WriteLine();
                    CleanupObj.Log("ALCleanUp.xml could not be found on the specified path - " + ALCleanupXML, "", true);
                    return;
                }
                XPathDocument XP_Doc_CleanupXML = new XPathDocument(ALCleanupXML);
                XPathNavigator XP_Nav_CleanupXML = XP_Doc_CleanupXML.CreateNavigator();
                #endregion

                XPathNodeIterator XP_Iterator_CleanupXML = XP_Nav_CleanupXML.Select("ALCleanUp//AssetLinkFolders//TempFolder");
                while (XP_Iterator_CleanupXML.MoveNext())
                {
                    location = XP_Iterator_CleanupXML.Current.GetAttribute("Location", "");
                    DeletePeriod = XP_Iterator_CleanupXML.Current.GetAttribute("DeleteOlderThan", "");
                    FileExt = XP_Iterator_CleanupXML.Current.GetAttribute("FileExtList", "");
                    Comments = XP_Iterator_CleanupXML.Current.GetAttribute("Comments", "");

                    XPathNodeIterator iter_AvoidFolders = XP_Iterator_CleanupXML.Current.Select("AVIODFOLDERLIST/AVOIDFOLDER");

                    lst_FoldersToAvoidList = new List<string>();
                    while (iter_AvoidFolders.MoveNext())
                    {
                        lst_FoldersToAvoidList.Add(iter_AvoidFolders.Current.Value);
                    }
                    
                    StringBuilder ContextAdditionalInfo = new StringBuilder(20);
                    ContextAdditionalInfo.AppendFormat("<ADDITIONALINFO><LOCATION>{0}</LOCATION><DELETEOLDERTHAN>{1}</DELETEOLDERTHAN><FILEEXTLIST>{2}</FILEEXTLIST><COMMENTS>{3}</COMMENTS></ADDITIONALINFO>", location, DeletePeriod, FileExt, Comments);
                    Context ContextObj = new Context(0, 0, "", "ALCLEANUP", ContextAdditionalInfo.ToString());

                    if (SASMOMDirectory.CanFolderBeDeleted(location, ContextObj))
                        CleanupObj.CleanUpFolder(location, DeletePeriod, FileExt, Comments, lst_FoldersToAvoidList);
                }
            }            
            catch (Exception ExcepObj)
            {
                System.Diagnostics.EventLog objEventLog = new System.Diagnostics.EventLog("SAS MOM");
                objEventLog.Source = "AlCleanup Utility";
                objEventLog.WriteEntry("Error in ALCleaup. \r\nMessage : " + ExcepObj.Message, System.Diagnostics.EventLogEntryType.Error);
            }

            CleanupObj.Log("Utility stopped running at : " + DateTime.Now.ToLongTimeString(), "", true);
        }

        public void CleanUpFolder(String LocationFolder,String Period,String Ext,String DelComments, List<String> lst_FoldersToAvoid)
        {
            FileInfo[] AllFilesInDir = null;
            int        FileCount      = 0;
            int        SubDirCount    = 0;
            int        ExtCount       = 0;
            int p = Convert.ToInt32(Period);
            TimeSpan   DateDifference;            

            
            try
            {
                String[] ColonSeparator = new String[1] { ";" };
                String[] ArrayOfExt = Ext.Split(ColonSeparator, StringSplitOptions.RemoveEmptyEntries);
                if (Directory.Exists(LocationFolder))
                {
                    DirectoryInfo DirObj = new DirectoryInfo(LocationFolder);
                    DirectoryInfo[] SubDirObj = null;
                    try
                    {
                        SubDirObj = DirObj.GetDirectories();
                    }
                    catch (UnauthorizedAccessException)
                    {
                        return;
                    }

                    #region Delete Files with given Ext

                    for (ExtCount = 0; ExtCount < ArrayOfExt.Length; ExtCount++)
                    {
                        try
                        {
                            AllFilesInDir = DirObj.GetFiles(ArrayOfExt[ExtCount]);
                        }
                        catch (UnauthorizedAccessException)
                        {
                            continue;
                        }

                        for (FileCount = 0; FileCount < AllFilesInDir.Length; FileCount++)
                        {
                            DateDifference = DateTime.Now - AllFilesInDir[FileCount].LastAccessTime;
                            if (DateDifference.Days >= p)
                            {
                                try
                                {
                                    if (AllFilesInDir[FileCount].IsReadOnly)
                                    {
                                        FileAttributes attrs = AllFilesInDir[FileCount].Attributes;
                                        File.SetAttributes(AllFilesInDir[FileCount].FullName, attrs & ~FileAttributes.ReadOnly);
                                    }

                                    AllFilesInDir[FileCount].Delete();
                                    Log("File Deleted Sucessefully", AllFilesInDir[FileCount].FullName, false);
                                }
                                catch (Exception ExpObj)
                                {
                                    Log("Unable to delete file", ExpObj.ToString(), false);
                                }
                            }
                        }
                    }

                    #endregion

                    #region Check SubDirectories of Current Folder

                    for (SubDirCount = 0; SubDirCount < SubDirObj.Length; SubDirCount++)
                    {
                        bool EmptyCheckFlag;
                        String SubDirPath = null;
                        try
                        {
                            SubDirPath = SubDirObj[SubDirCount].FullName;
                        }
                        catch (UnauthorizedAccessException)
                        {
                            continue;
                        }

                        if (lst_FoldersToAvoid.Count > 0 && lst_FoldersToAvoid.Contains(SubDirPath))
                        {
                            lst_FoldersToAvoid.Remove(SubDirPath);
                            continue;
                        }
                        StringBuilder ContextAdditionalInfo = new StringBuilder(20);
                        ContextAdditionalInfo.AppendFormat("<ADDITIONALINFO><LOCATION>{0}</LOCATION><DELETEOLDERTHAN>{1}</DELETEOLDERTHAN><FILEEXTLIST>{2}</FILEEXTLIST><COMMENTS>{3}</COMMENTS></ADDITIONALINFO>", LocationFolder, Period, Ext, DelComments);
                        Context ContextObj = new Context(0, 0, "", "ALCLEANUP", ContextAdditionalInfo.ToString());

                        if (SASMOMDirectory.CanFolderBeDeleted(SubDirPath, ContextObj))
                        {
                            CleanUpFolder(SubDirPath, Period, Ext, DelComments, lst_FoldersToAvoid);

                            EmptyCheckFlag = IsDirectoryEmpty(SubDirPath);
                            if (EmptyCheckFlag.Equals(true))
                            {
                                try
                                {
                                    if ((SubDirObj[SubDirCount].Attributes & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
                                    {
                                        File.SetAttributes(SubDirObj[SubDirCount].FullName, SubDirObj[SubDirCount].Attributes & ~FileAttributes.ReadOnly);
                                    }
                                    SASMOMDirectory.DeleteFolder(SubDirObj[SubDirCount].FullName, ContextObj);
                                    //SubDirObj[SubDirCount].Delete();

                                    Log("SubDirectory Deleted Sucessefully", SubDirPath, false);
                                }
                                catch (Exception ExObj)
                                {
                                    Log("Unable to delete SubDirectory", ExObj.ToString(), false);
                                }
                            }
                        }
                    }
                    #endregion
                }
                else
                {
                    Log("Location Folder dose not exists", LocationFolder, false);
                }
            }
            catch (Exception ExcepObj)
            {
                if (objEventLog == null)
                    objEventLog = new System.Diagnostics.EventLog("SAS MOM");
                objEventLog.Source = "AlCleanup Utility";
                objEventLog.WriteEntry("Error in ALCleaup. \r\nMessage : " + ExcepObj.Message, System.Diagnostics.EventLogEntryType.Error);
            }
        }

        public bool IsDirectoryEmpty(String DirPath)
        {
            bool IsEmpty = true;
            DirectoryInfo Sub_Obj = new DirectoryInfo(DirPath);
            DirectoryInfo[] Dir_SubDirs = null;
            try
            {
                Dir_SubDirs = Sub_Obj.GetDirectories();
            }
            catch (UnauthorizedAccessException)
            {
                return true;
            }

            if (Sub_Obj.GetFiles().Length == 0 && Dir_SubDirs.Length == 0)
            {
                IsEmpty = true;
                return (IsEmpty);
            }
            else
            {
                IsEmpty = false;
                return (IsEmpty);
            }
        }

        private void Log(String LogComment, String LogFilePath, bool IsCompulsory)
        {

            String FlagValue = null;
            FlagValue = Assetlink.RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "EnableLog");

            if (!IsCompulsory && (String.IsNullOrEmpty(FlagValue) || FlagValue != "1"))
            {
                return;
            }

            String FolderPath = ALCommon.GetInstallationDir();
            FolderPath += "\\Logs\\ALCleanup\\";
            if (!Directory.Exists(FolderPath))
            {
                DirectoryInfo DirectoryObj = Directory.CreateDirectory(FolderPath);
            }

            FileStream LogFileStream = null;
            StreamWriter LogStreamWriter = null;
            //String DateForLog = DateTime.Now.ToLongDateString;
            StringBuilder LogFileName = new StringBuilder();//@"C:\" + DateTime.Now.ToLongDateString + "Log.txt";
            LogFileName.AppendFormat("{0}{1}_CleanupLog.txt", FolderPath, DateTime.Now.ToLongDateString());
            StringBuilder LogString = null;
            String LogFile = null;
            LogFile = LogFileName.ToString();
            try
            {
                if (File.Exists(LogFile))
                {

                    LogFileStream = new FileStream(LogFile, FileMode.Append);
                }
                else
                {

                    LogFileStream = new FileStream(LogFile, FileMode.Create);
                }
                LogStreamWriter = new StreamWriter(LogFileStream);
                LogString = new StringBuilder();
                // LogStreamWriter.WriteLine("GUID:                       " + LogGUID);
                LogStreamWriter.WriteLine("Comment:                    " + LogComment);
                LogStreamWriter.WriteLine("Date:                       " + DateTime.Now.Date);
                if (!String.IsNullOrEmpty(LogFilePath))
                    LogStreamWriter.WriteLine("FilePath:                       " + LogFilePath);
                LogStreamWriter.WriteLine();
                LogStreamWriter.WriteLine("-----------------------------------------------------------");
                LogStreamWriter.Close();
                LogFileStream.Close();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        static bool ServerValidateCertificate(object sender, X509Certificate cert, X509Chain chain, SslPolicyErrors sslError)
        {
            return true;
        }
    }
}
