﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Xml.XPath;
using System.Diagnostics;
using Assetlink.WebServices.ALInfrastructureWS;

namespace Assetlink.IOProxy
{
    public class SASMOMDirectory
    {
        public static Boolean DeleteFolder(String FolderPath, Context OperationContext)
        {
            return DeleteFolder(FolderPath, false, OperationContext);
        }

        public static Boolean DeleteFolder(String FolderPath, Boolean Recursive, Context OperationContext)
        {
            Boolean CanDeleteFolder     = false;

            #region Prepare common Log String
            String CommonLog = PrepareCommonLogDetailsString(FolderPath, Recursive);
            #endregion

            #region Folder Path Validation 1
            if (String.IsNullOrEmpty(FolderPath))
                throw (new Exception("Folder path cannot be null or empty"));

            if(!Directory.Exists(FolderPath))
                throw (new System.IO.DirectoryNotFoundException("Could not find a part of the path '"+ FolderPath +"'"));
            #endregion

            try
            {
                CanDeleteFolder = CanFolderBeDeleted(ref FolderPath, OperationContext);
            }
            catch (Exception ExceptionObj)
            {
                WriteLog("Exception Error Processing DeleteFolder Call: " + CommonLog + "\r\n         " + ExceptionObj.Message, OperationContext);
                throw ExceptionObj;
            }

            #region Actual Foder Delete Call
            try
            {
                if (CanDeleteFolder)
                {
                    Utility.DeNormalizeFolderPath(ref FolderPath);
                    if (Directory.Exists(FolderPath))
                    {
                        Directory.Delete(FolderPath, Recursive);
                        //WriteLog("Deleted " + CommonLog, OperationContext);
                        return true;
                    }
                    else
                    {
                        WriteLog("Invalid " + CommonLog, OperationContext);
                        return false;
                    }
                }
                else
                {
                    WriteLog("Attempt to delete " + CommonLog, OperationContext);
                    return false;
                }
            }
            catch (Exception IOExceptionObj)
            {
                WriteLog("Exception : Deleting " + CommonLog + " \r\n         " + IOExceptionObj.Message, OperationContext);
                throw IOExceptionObj;
            }
            #endregion
        }

        /// <summary>
        /// Function which check given folder path canbedeleted or no.
        /// Will validate the path against
        /// 1. File MountPoint
        /// 2. Xml Mount Point
        /// 3. Shadow Folder 
        /// 4. Cutom paths configured in AvoidFolderDeletion.xml
        /// </summary>

        public static Boolean CanFolderBeDeleted(String FolderPath, Context OperationContext)
        {
            return CanFolderBeDeleted(ref FolderPath, OperationContext);
        }

        public static Boolean CanFolderBeDeleted(ref String FolderPath, Context OperationContext)
        {
            List<FolderInfo> AvoidDeleteFolders;
            List<FolderInfo> AllowDeleteFolders;
            List<FolderInfo> CustomAvoidDeleteFolders;
            Boolean          CanDeleteFolder = true;

            try
            {
                #region Folder Path Validation 2
                Utility.NormalizeFolderPath(ref FolderPath);
                if (FolderPath.Contains(@"\..\") || FolderPath.Contains(@"\.\"))
                {
                    throw new Exception("Malicious Folder Path: " + FolderPath);
                }
                #endregion

                #region Handling for MoutPoints and Shadow Folders
                if (ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "WebServicesInstalled") == "1")
                {
                    #region Read Avoid and Allowed to Delete Folder List
                    ReadAllTenantFolderSettings(OperationContext, out AvoidDeleteFolders, out AllowDeleteFolders);
                    #endregion

                    #region Process System Mount Point Folder List (Avoid to Delete folders)
                    foreach (FolderInfo AvoidDeleteFolderInfoObj in AvoidDeleteFolders)
                    {
                        //Path is part of AVOID DELETE FOLDER Hierarchy  OR (Path is child of AVOID DELETE FOLDER Hierarchy AND AVOID DELETE FOLDER child is NOT allowed to be deleted
                        if (AvoidDeleteFolderInfoObj.FolderPath.StartsWith(FolderPath) || (FolderPath.StartsWith(AvoidDeleteFolderInfoObj.FolderPath) && !AvoidDeleteFolderInfoObj.AllowChildDeletion))
                        {
                            CanDeleteFolder = false;
                            break;
                        }
                    }
                    #endregion

                    #region Process System Mount Point Child Folder List (Allowed to Delete Folders)
                    foreach (FolderInfo AllowDeleteFolderInfoObj in AllowDeleteFolders)
                    {
                        //Path is Equal to ALLOWED DELETE FOLDER OR (Path is child of ALLOWED DELETE FOLDER Hierarchy AND ALLOWED DELETE FOLDER allows child to be deleted
                        if ((AllowDeleteFolderInfoObj.FolderPath.Equals(FolderPath) && !AllowDeleteFolderInfoObj.IsShadowFoder) || ((!AllowDeleteFolderInfoObj.FolderPath.Equals(FolderPath)) && FolderPath.StartsWith(AllowDeleteFolderInfoObj.FolderPath) && AllowDeleteFolderInfoObj.AllowChildDeletion))
                            CanDeleteFolder = true;
                    }
                    #endregion
                }
                #endregion

                #region Process custom to Avoid Delete Folder List
                if (CanDeleteFolder)
                {
                    #region Read custom Avoid to Delete Folder List
                    ReadAvoidDeleteFolderSettings(out CustomAvoidDeleteFolders);
                    #endregion

                    foreach (FolderInfo CustomAvoidDeleteFolderInfoObj in CustomAvoidDeleteFolders)
                    {
                        //Path is part of AVOID DELETE FOLDER Hierarchy  OR (Path is child of AVOID DELETE FOLDER Hierarchy AND AVOID DELETE FOLDER child is NOT allowed to be deleted
                        if (CustomAvoidDeleteFolderInfoObj.FolderPath.StartsWith(FolderPath) || (FolderPath.StartsWith(CustomAvoidDeleteFolderInfoObj.FolderPath) && !CustomAvoidDeleteFolderInfoObj.AllowChildDeletion))
                        {
                            CanDeleteFolder = false;
                            break;
                        }
                    }
                }
                #endregion
            }
            catch (Exception ExceptionObj)
            {
                WriteLog("Exception Error Processing CanFolderBeDeleted Call: " + ExceptionObj.Message, OperationContext);
                CanDeleteFolder = false;
                throw ExceptionObj;
            }

            return CanDeleteFolder;
        }

        protected static void ReadAllTenantFolderSettings(Context OperationContext, out List<FolderInfo> AvoidDeleteFolders, out List<FolderInfo> AllowDeleteFolders)
        {
            InfrastructureService InfrastructureWebServiceObj;
            String                ErrorXML      = null;
            String                TenantListXML = null;


            #region set the out Parameters
            AvoidDeleteFolders = new List<FolderInfo>();
            AllowDeleteFolders = new List<FolderInfo>();
            #endregion

            #region Create Infrastructure Web Service Object
            InfrastructureWebServiceObj = new InfrastructureService();
            InfrastructureWebServiceObj.ALServiceHeaderValue = new global::Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader();
            InfrastructureWebServiceObj.ALServiceHeaderValue.SessionID = OperationContext.SessionID;
            InfrastructureWebServiceObj.ALServiceHeaderValue.UserID = OperationContext.UserID;
            InfrastructureWebServiceObj.ALServiceHeaderValue.TenantName = OperationContext.TenantName;
            InfrastructureWebServiceObj.Url = ALCommon.GetWebServiceURL("ALInfrastructureWS");
            #endregion

            #region Tenant Processing
            ErrorXML = InfrastructureWebServiceObj.GetAllTenants(ref TenantListXML);
            if (String.IsNullOrEmpty(ErrorXML))
            {
                XPathDocument       AllTenantxmlDoc             = new XPathDocument(new StringReader(TenantListXML));
                XPathNavigator      AllTenantxmlDocNav          = AllTenantxmlDoc.CreateNavigator();
                XPathNodeIterator   AllTenantxmlDocNavIterator  = AllTenantxmlDocNav.Select("TenantList/Tenant");
                while (AllTenantxmlDocNavIterator.MoveNext())
                    ReadTenantFolderSettings(AllTenantxmlDocNavIterator.Current.GetAttribute("Name", ""), ref InfrastructureWebServiceObj, ref AvoidDeleteFolders, ref AllowDeleteFolders);
            }
            else
            {
                String ErrorMessage = "Error Processing ReadAllTenantFolderSettings Call: " + ErrorXML;
                throw new Exception(ErrorMessage);
            }
            #endregion
        }

        /// <summary>
        /// Read File Mountpoint add it to AvoidDeleteFolders list 
        /// Read XML Mountpoint add it to AvoidDeleteFolders list
        /// Prepare SHADOW_FOLDER path add Mountpoint add it to AllowDeleteFolders list
        /// </summary>
        private static void ReadTenantFolderSettings(String TenantName, ref InfrastructureService InfrastructureWebServiceObj, ref List<FolderInfo> AvoidDeleteFolders, ref List<FolderInfo> AllowDeleteFolders)
        {
            String FileMounPoint                    = null;
            String XmlMountPoint                    = null;
            String FileMounPointShadowFolderPath    = null;
            String ErrorXML                         = null;
            String ErrorMessage                     = null; 

            #region Read File MountPoint
            ErrorXML = InfrastructureWebServiceObj.GetTenantSetting(TenantName, "CONFIGURATION/FILESYSTEM/MOUNTPOINT", ref FileMounPoint);
            if (String.IsNullOrEmpty(ErrorXML))
                AvoidDeleteFolders.Add(new FolderInfo(FileMounPoint, false));
            else
            {
                ErrorMessage = "Error Processing ReadAllTenantFolderSettings Call: " + ErrorXML;
                throw new Exception(ErrorMessage);
            }
            #endregion

            #region Read Xmle MountPoint
            ErrorXML = InfrastructureWebServiceObj.GetTenantSetting(TenantName, "CONFIGURATION/XMLFILESYSTEM/XMLMOUNTPOINT", ref XmlMountPoint);
            if (String.IsNullOrEmpty(ErrorXML))
                AvoidDeleteFolders.Add(new FolderInfo(XmlMountPoint, true));
            else
            {
                ErrorMessage = "Error Processing ReadAllTenantFolderSettings Call: " + ErrorXML;
                throw new Exception(ErrorMessage);
            }
            #endregion

            #region Prepare Shadow Folder Path

            FileMounPointShadowFolderPath = FileMounPoint + @"\SHADOW_FOLDER";
            AllowDeleteFolders.Add(new FolderInfo(FileMounPointShadowFolderPath, true, true));
            #endregion
        }

        /// <summary>
        /// Read To Avoid to Delete folder paths from AvoidFolderDeletion
        /// </summary>
        protected static void ReadAvoidDeleteFolderSettings(out List<FolderInfo> AvoidDeleteFolders)
        {
            String              AvoidFolderDeletionXmlPath  = null;
            String              FolderPath                  = null;
            String              AllowChildDeletion          = null;

            #region set the out Parameters
            AvoidDeleteFolders      = new List<FolderInfo>();
            #endregion

            #region read the custom xml file and prepare Avoid to delete folder list
            AvoidFolderDeletionXmlPath = ALCommon.GetInstallationDir() +@"Binaries\AvoidFolderDeletion.xml";

            if (File.Exists(AvoidFolderDeletionXmlPath))
            {
                XPathDocument AvoidFolderDeletionXMLDoc = null;
                XPathNavigator AvoidFolderDeletionXMLDocNav = null;
                AvoidFolderDeletionXMLDoc = new XPathDocument(AvoidFolderDeletionXmlPath);
                AvoidFolderDeletionXMLDocNav = AvoidFolderDeletionXMLDoc.CreateNavigator();

                XPathNodeIterator FolderIterator = AvoidFolderDeletionXMLDocNav.Select("AvoidFolderDeletion/Folder");
                while (FolderIterator.MoveNext())
                {
                    FolderPath = FolderIterator.Current.GetAttribute("Path", "");
                    AllowChildDeletion = FolderIterator.Current.GetAttribute("AllowChildDeletion", "");
                    if (!String.IsNullOrEmpty(FolderPath))
                    {
                        if (String.IsNullOrEmpty(AllowChildDeletion))
                            AvoidDeleteFolders.Add(new FolderInfo(FolderPath, false));
                        else
                            AvoidDeleteFolders.Add(new FolderInfo(FolderPath, Boolean.Parse(AllowChildDeletion)));
                    }
                }
            }
            else
            {
                String ErrorMessage = "Error Processing ReadAvoidDeleteFolderSettings Call: file " + AvoidFolderDeletionXmlPath + " Not found.";
                throw new Exception(ErrorMessage);
            }
            #endregion
        }

        private static Object WriteFileLock;
        /// <summary>
        /// Function to write the log file.
        /// Will log Details like
        /// TenantName, UserID, SessionId, Solution, Custom Message, StackTrace and AddtionInfromation provided in Content Object
        /// </summary>
        public static void WriteLog(String Message, Context OperationContext)
        {
            DateTime LogDatetime          = DateTime.Now;
            String   MethodName           = null;
            String   StackTraceText       = null;
            String   ProcessName          = System.Diagnostics.Process.GetCurrentProcess().ProcessName;
            String   InstallationDir      = ALCommon.GetInstallationDir();
            String   LogFileFolderPath    = null;
            String   LogFileName          = null;

            ProcessName          = ProcessName.Replace(".", "-");
            LogFileFolderPath    = InstallationDir + "\\Logs\\FolderDeletion";
            LogFileName          = LogFileFolderPath + "\\" + ProcessName + " " + LogDatetime.ToString("dd MMM yyyy") + ".txt";
            
            FileStream   LogFileStream    = null;
            StreamWriter LogStreamWriter  = null;
            StringBuilder LogString       = new StringBuilder(1024);;

            try
            {
                #region Read the stack Details
                StackTrace StackTraceObj = new StackTrace(1,true);
                StackFrame StackFrame    = null;
                System.Reflection.MethodBase MethodInfo;
                if (StackTraceObj != null)
                {
                    StackFrame     = StackTraceObj.GetFrame(1);
                    MethodInfo     = StackFrame.GetMethod();
                    MethodName     = MethodInfo.ReflectedType + ", " + MethodInfo.ToString();
                    
                    StackTraceText = StackTraceObj.ToString().Substring(StackTraceObj.ToString().IndexOf("\r\n") + 2);
                }
                #endregion

                LogString.AppendLine("Time : " + LogDatetime.ToString("HH:mm:ss:fff") + ", TenantName: " + OperationContext.TenantName + ", UserID: " + OperationContext.UserID + ", SessionID: " + OperationContext.SessionID + ", SolutionName: " + OperationContext.SolutionName);
                if (!String.IsNullOrEmpty(Message))
                    LogString.AppendLine("Message: " + Message);

                LogString.AppendLine("Add-Information: " + OperationContext.AdditionalInformation);
                LogString.AppendLine("MethodName: " + MethodName);
                LogString.AppendLine("ThreadID: " + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());
                LogString.AppendLine("Stack Trace: " + StackTraceText);
                LogString.AppendLine("--------------------------------------");

                if (WriteFileLock == null)
                    WriteFileLock = new object();

                lock (WriteFileLock)
                {
                    DirectoryInfo LogFileFolderInfo = new DirectoryInfo(LogFileFolderPath);
                    if (LogFileFolderInfo.Exists == false)
                        LogFileFolderInfo.Create();

                    if (File.Exists(LogFileName))
                        LogFileStream = new FileStream(LogFileName, FileMode.Append);
                    else
                        LogFileStream = new FileStream(LogFileName, FileMode.Create);

                    LogStreamWriter = new StreamWriter(LogFileStream);
                    LogStreamWriter.WriteLine(LogString);
                    LogStreamWriter.Close();
                    LogFileStream.Close();
                }
            }
            catch(Exception ExceptionObj)
            {
                #region Write Event Viewer Log
                Utility.WriteEventLog("Exception in WriteLog : " + ExceptionObj.Message + " \r\n" + LogString.ToString());
                #endregion
            }
        }

        public static String PrepareCommonLogDetailsString(String FolderPath, Boolean Recursive)
        {
            return "Folder: " + FolderPath + ", Recursive: " + Recursive.ToString();
        }
    }


    public class FolderInfo
    {
        public readonly String  FolderPath;
        public readonly Boolean AllowChildDeletion;
        public readonly Boolean IsShadowFoder;

        public FolderInfo(String FolderPath, Boolean AllowChildDeletion)
        {
            Utility.NormalizeFolderPath(ref FolderPath);
            this.FolderPath         = FolderPath;
            this.AllowChildDeletion = AllowChildDeletion;
            this.IsShadowFoder      = false; 
        }

        public FolderInfo(String FolderPath, Boolean AllowChildDeletion, Boolean IsShadowFoder)
        {
            Utility.NormalizeFolderPath(ref FolderPath);
            this.FolderPath         = FolderPath;
            this.AllowChildDeletion = AllowChildDeletion;
            this.IsShadowFoder      = IsShadowFoder;
        }
    }

    public class Context
    {
       public readonly ulong UserID;
       public readonly String TenantName;
       public readonly String SolutionName;
       public readonly ulong SessionID;
       public  String  AdditionalInformation;

       public Context(ulong UserID, ulong SessionID, String TenantName, String SolutionName)
       {
           this.UserID          = UserID;
           this.SessionID       = SessionID;
           this.TenantName      = TenantName;
           this.SolutionName    = SolutionName;
       }

       public Context(ulong UserID, ulong SessionID, String TenantName, String SolutionName, String AdditionalInformation)
       {
           this.UserID                  = UserID;
           this.SessionID               = SessionID;
           this.TenantName              = TenantName;
           this.SolutionName            = SolutionName;
           this.AdditionalInformation   = AdditionalInformation;
       }
    }

    public class Utility
    {
        /// <summary>
        /// Remove extea \\ from path and change toupper
        /// </summary>
        public static void NormalizeFolderPath(ref String FolderPath)
        {
            Boolean IsUNCPath = false;

            #region Remove Extra \\ from path
            FolderPath = FolderPath.Trim(' ');

            if (FolderPath.StartsWith(@"\\"))
                IsUNCPath = true;

            FolderPath += "\\";
            FolderPath = Regex.Replace(FolderPath, @"\\+", "\\").ToUpper();

            if (IsUNCPath)
                FolderPath = "\\" + FolderPath;
            #endregion
        }

        public static void DeNormalizeFolderPath(ref String FolderPath)
        {
            #region Remove Extra \\ from path
            FolderPath = FolderPath.TrimEnd('\\');
            #endregion
        }

        public static void WriteEventLog(String Message)
        {
            #region Write Event Viewer Log
            EventLog EventLogObj;
            String  LogType  = "SAS MOM";
            String  Machine  = ".";
            String  Source   = "SAS MOM Directory";
            String  LogName;

            if (EventLog.SourceExists(Source))
            {
                LogName = EventLog.LogNameFromSourceName(Source, Machine);
                if (String.Compare(LogName, LogType, true) != 0)
                {
                    EventLog.DeleteEventSource(Source);
                    EventLog.CreateEventSource(Source, LogType);
                }
            }
            else
            {
                EventLog.CreateEventSource(Source, LogType);
            }

            EventLogObj = new EventLog(LogType, Machine, Source);
            EventLogObj.ModifyOverflowPolicy(OverflowAction.OverwriteAsNeeded, 1);
            EventLogObj.WriteEntry(Message, EventLogEntryType.Error);
            #endregion
        }
    }

}
