using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.Threading;
using System.Collections.Specialized;
using System.Linq;
using System.Xml.Linq;

using Assetlink;
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.Services.Engines;
using Assetlink.Services.Plugins.PluginManger;
using Assetlink.Services.Engines.InfrastructureEngine;
using Assetlink.WebServices.ALMSOfficeProcessorWS;
using Assetlink.IOProxy;

namespace Assetlink.ProcessManager
{
    using XML      = String;
    using ERRORXML = String;

	enum TimeLineStatus
	{
		YES = 1,
		NO = 0
	}
    public partial class ProcessManagerClass
    {
        private Boolean m_SetBaselineForCreateDeliverable;


        /// <summary>
        /// 
        /// </summary>
        public Boolean SetBaseForCreateDeliverable
        {
            get { return m_SetBaselineForCreateDeliverable; }
            set { m_SetBaselineForCreateDeliverable = value; }
        }
        
        
        /// <summary>
        /// <createDate>10 February 2007</createDate>
        /// <Author>Hemant Kirad</Author>
        /// <Purpose>This is generic method, recursive upto n-level to retrieve information
        /// from OM by processing the Association XML</Purpose>
        /// </summary>
        /// <param name="FilterXML">Represents a list of filters</param>
        /// <param name="AssociationXML">An XML representing target entities and its relationships
        ///     AssociationXML Structure:
        ///         TARGETENTITYNAME @Type="" @TargetEntityType = "" @GetAllFacets=""
        ///             |   - RELATIONSHIPNAME @Type = "RELATION" @GetAllFacets="" 0..*
        ///                         | - $Attributes$
        ///                         | - RELATIONHIPNAME @Type = "RELATION"  0..*      
        ///                                 |- $Attributes$     
        /// </param>
        /// <param name="OutputXML">Complete Collection of data and associated information</param>        
        /// <returns>Error XML if an error ocuurs else null</returns>
        private ERRORXML NSearch(XML FilterXML, XML AssociationXML, ref XML OutputXML)
        {
           ERRORXML         ErrorXML            = null;    // Error XML
           XmlNode          SearchNode          = null;
           XmlNode          FilterListNode      = null;
           StringBuilder    XSLTFilePath        = null;
           StreamReader     Stream_Reader       = null;
           StringReader     String_Reader       = null;
           String           InstalledPath       = null;    //To get the installation path.
           String           NewAssociationXML   = null;
           String           NewFilterXML        = null;
           MemoryStream     Memory_Stream       = null;
           XPathDocument    xpdoc               = null;
           XPathNavigator   NavigatorObject     = null;
           XmlDocument      AssoXMLDocument     = null;    // loads the association XML   
           XmlDocument      FilterXMLDocument   = null;    // contains the filter XML with FilterList nodes
           XmlNode          TargetEntityNode    = null;    // represents the ASSOCIATION node
           OMInterface      OMInterfaceObject   = null;    // OM object to get data

           XslCompiledTransform transformObj = null;  //To load XSLT file and apply Transform.
           
               InstalledPath = ALCommon.GetInstallationDir();

               XSLTFilePath = new StringBuilder();
               XSLTFilePath.AppendFormat("{0}\\XSLT\\PROCESSDESIGNER\\GetHierarchicalData.xslt", InstalledPath);

               transformObj = new XslCompiledTransform();
               transformObj.Load(XSLTFilePath.ToString());

               String_Reader    = new StringReader(AssociationXML);
               xpdoc            = new XPathDocument(String_Reader);
               NavigatorObject  = xpdoc.CreateNavigator();

               if (NavigatorObject != null)
               {
                   Memory_Stream = new MemoryStream();
                   transformObj.Transform(NavigatorObject, null, Memory_Stream);

                   Memory_Stream.Position = 0;
                   Stream_Reader        = new StreamReader(Memory_Stream);
                   NewAssociationXML    = Stream_Reader.ReadToEnd();
                   Memory_Stream.Flush();
                   Stream_Reader.Close();

                   int ListItemStrPos   = NewAssociationXML.IndexOf("\r\n", 0);
                   NewAssociationXML    = NewAssociationXML.Substring(ListItemStrPos);
               }

               if (NewAssociationXML != null)
               {
                   if (FilterXML != null)
                   {
                       AssoXMLDocument = new XmlDocument();
                       AssoXMLDocument.LoadXml(NewAssociationXML);

                       SearchNode       = AssoXMLDocument.SelectSingleNode("SEARCH");
                       TargetEntityNode = SearchNode.SelectSingleNode("TARGETENTITY");
                       if (TargetEntityNode == null)
                       {
                           TargetEntityNode = AssoXMLDocument.CreateNode(XmlNodeType.Element, "TARGETENTITY", "");
                           SearchNode.AppendChild(TargetEntityNode);
                       }

                       transformObj = null;
                       XSLTFilePath.Remove(0, XSLTFilePath.Length);
                       XSLTFilePath.AppendFormat("{0}\\XSLT\\PROCESSDESIGNER\\GetHierarchicalFilter.xslt", InstalledPath);

                       transformObj = new XslCompiledTransform();
                       transformObj.Load(XSLTFilePath.ToString());

                       String_Reader = new StringReader(FilterXML);
                       xpdoc = new XPathDocument(String_Reader);
                       NavigatorObject = xpdoc.CreateNavigator();

                       if (NavigatorObject != null)
                       {
                           Memory_Stream = new MemoryStream();
                           transformObj.Transform(NavigatorObject, null, Memory_Stream);

                           Memory_Stream.Position = 0;
                           Stream_Reader = new StreamReader(Memory_Stream);
                           NewAssociationXML = Stream_Reader.ReadToEnd();
                           Memory_Stream.Flush();
                           Stream_Reader.Close();

                           int ListItemStrPos = NewAssociationXML.IndexOf("\r\n", 0);
                           NewFilterXML = NewAssociationXML.Substring(ListItemStrPos);
                       }

                       if (NewFilterXML != null)
                       {
                           FilterXMLDocument = new XmlDocument();
                           FilterXMLDocument.LoadXml(NewFilterXML);
                           FilterListNode = FilterXMLDocument.SelectSingleNode("FILTERLIST");
                       }

                       TargetEntityNode.AppendChild(AssoXMLDocument.ImportNode(FilterListNode, true));
                   }

                   OMInterfaceObject = new OMInterface(UserID, CurrentLanguage, TenantName);
                   ErrorXML = OMInterfaceObject.GetHierarchialData(AssoXMLDocument.InnerXml, ref OutputXML);

                   if (ErrorXML != null)
                   {
                       return ErrorXML;
                   }
               }
               return null;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ProcessID"></param>
        /// <param name="ProcessStartDate"></param>
        /// <param name="ReturnXML"></param>
        /// <param name="TotalProcessDuration"></param>
        /// <param name="UpdatedProcessDueDate"></param>
		/// <param name="ActivityInfoList"></param>
        /// <returns></returns>
        public ERRORXML CalculateTimeLine_Ex(String ProcessID, DateTime ProcessStartDate, List<ActivityInfo> ActivityInfoList, Boolean IsFirstTime, String str_FilePath, out XML ReturnXML, out String TotalProcessDuration, out DateTime UpdatedProcessDueDate)
        {
            String					ErrorXML					= null;
            String					ProcessInputXML				= null;
            String					ProcessXML					= null;            
            XPathDocument			ProcessDoc					= null;            
            XPathNavigator			ProcessNav					= null;
            XPathNavigator			ActivityNav					= null;            
            XPathNavigator			TempNav						= null;
            XPathNavigator			ProcessInfoNav				= null;
            XPathNavigator			PredActNav					= null;            
            XPathNodeIterator		TempNodeIter				= null;            
			StringReader			StrProcessReader			= null;			
			String					XPathQuery					= null;
			String					SubProcessID				= null;
			String					Level						= "1";
			String					SuccActivityID				= null;			
			String					ActivityInfoID				= null;
			String					ProcessInfoID				= null;
			String					PredActivityID				= null;
			String					ActivityStatus				= null;
			String					SubProcessInfoID			= null;
			int						iLevel						= 1;
			int						Day							= 0;
			int						Month						= 0;
			int						Year						= 0;
			ActivityInfo			SuccessorActivity			= null;
			ActivityInfo			AInfoObj					= null;			
			List<ActivityInfo>		NextActivityList			= null;
			List<ActivityInfo>		NextLevelActivityList		= null;
			StringBuilder			UpdateActivityXML			= null;						
			Hashtable				ActivityDurationTable		= null;
			Hashtable				ActivityDueDateTable    	= null;			
			DateTime				ProcessDueDate				;
			DateTime				ActivityDate				; 
			DateTime				NextStartDate				; 
			DateTime				TempDate					;
			DateTime				TempActivityDueDate			;
			List<ActivityTLStatus>	ActivityTLStatusTable			= null;
			ActivityTLStatus		ActivityStatusObj				= null;
			bool					CalculateTimeLineForActivity	= false;	
			bool					ProcessStartDateLocked			= false;
			bool					ActivityStartDateLocked			= false;
			bool					UpdateLock						= false;
			bool					StartDateExists					= false;			
            OMInterface             OMIObj							= null; 
            TimeLineStatus			TStatus						;
            ALError                 ExcepErrObj                 = null;
            StringBuilder           sb_InputData                = null;

			try
            {
                ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (CalculateTimeLine_Ex):- Entered in CalculateTimeLine_Ex.");
                ReturnXML				= null;
                TotalProcessDuration	= null;
                UpdatedProcessDueDate	= DateTime.Today;
				ActivityDate			= DateTime.Today;
				ProcessDueDate			= DateTime.Today;
				TempDate				= DateTime.Today;
				TempActivityDueDate		= new DateTime(1,1,1);
				UpdateActivityXML		= new StringBuilder(300);
                OMIObj                  = new OMInterface(UserID, CurrentLanguage, TenantName);   
				
				ActivityDurationTable   = new Hashtable();
				ActivityDueDateTable	= new Hashtable();
                
                //HemantK: This change has been done for the performance
                //It will get the 3 level data and keep it in the temp location on if IsFirstTime flag is true
                //In recursion it will refer to this xml from temp location to read the WORKFLOWPROCESS
                //If required WORKFLOWPROCESS is not found in the xml then it will do the 1 levle fetch as it was doing earlier.
                
                if (IsFirstTime)
                {
                    str_FilePath = ALCommon.GetTempPath() + ProcessID + "_CalculateTimeLine_Ex.xml"; 

                    ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (CalculateTimeLine_Ex):- before Get3LevelProcessXML() in CalculateTimeLine_Ex.");
                    ErrorXML = Get3LevelProcessXML(ProcessID, ActivityInfoList, out ProcessXML);
                    ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (CalculateTimeLine_Ex):- after Get3LevelProcessXML() in CalculateTimeLine_Ex.");
                    if (ErrorXML != null)
                    {
                        if (IsFirstTime && File.Exists(str_FilePath))
                            File.Delete(str_FilePath);
                        return ErrorXML;
                    }
                    if (!String.IsNullOrEmpty(ProcessXML))
                        File.WriteAllText(str_FilePath, ProcessXML);
                }

                if ((ActivityInfoList!=null)&&(ActivityInfoList.Count > 0))
                {
                    ProcessXML = GetCurrentProcess(str_FilePath, ProcessID); 

                    if (!String.IsNullOrEmpty(ProcessXML))
                    {
                        StrProcessReader = new StringReader(ProcessXML);
                        ProcessDoc = new XPathDocument(StrProcessReader);
                        ProcessNav = ProcessDoc.CreateNavigator();
                        ProcessNav = ProcessNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                        //In first level get SUBFLOWACTIVITY with the ProcessID is fetched
                        //so GetCurrentProcess() will return the ProcessXML
                        //but this XML will not have all the required data so node count < 3 check has been added
                        if (ProcessNav.SelectChildren(XPathNodeType.All).Count < 3)
                            ProcessXML = null; 
                    }


                    if (String.IsNullOrEmpty(ProcessXML))
                    {
                        ProcessInputXML = PrepareXMLForGetProcess(ProcessID);

                        ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (CalculateTimeLine_Ex):- before GetHierarchialData() in CalculateTimeLine_Ex.");
                        ErrorXML = OMIObj.GetHierarchialData(ProcessInputXML, ref ProcessXML);
                        ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (CalculateTimeLine_Ex):- after GetHierarchialData() in CalculateTimeLine_Ex.");
                        if (ErrorXML != null)
                        {
                            if (IsFirstTime && File.Exists(str_FilePath))
                                File.Delete(str_FilePath);  
                            return ErrorXML;
                        }
                        StrProcessReader = new StringReader(ProcessXML);
                        ProcessDoc = new XPathDocument(StrProcessReader);
                        ProcessNav = ProcessDoc.CreateNavigator();
                        ProcessNav = ProcessNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                    }

					// sync ActivityTimeLine Status List with Status = NO
					ActivityTLStatusTable = new List<ActivityTLStatus>();
					ActivityTLStatusTable = SyncActivityTimeLineStatusList(ActivityInfoList, Level);
                    					
					if (ProcessNav!=null)
					{
						if (ProcessStartDate.Equals(new DateTime(1,1,1)))
						{
							if((ProcessNav.SelectSingleNode("STARTDATE") != null) && (ProcessNav.SelectSingleNode("STARTDATE").HasChildren))
							{									
								Day = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/DAY").Value);
								Month = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/MONTH").Value);
								Year = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/YEAR").Value);
                                ProcessStartDate = new DateTime(Year, Month, Day);
							}	
						}
						if((ProcessNav.SelectSingleNode("DUEDATE") != null) && (ProcessNav.SelectSingleNode("DUEDATE").HasChildren))
						{																
							Day		= Convert.ToInt32(ProcessNav.SelectSingleNode("DUEDATE/DAY").Value);
							Month	= Convert.ToInt32(ProcessNav.SelectSingleNode("DUEDATE/MONTH").Value);
							Year	= Convert.ToInt32(ProcessNav.SelectSingleNode("DUEDATE/YEAR").Value);
                            ProcessDueDate = new DateTime(Year, Month, Day);
						}
					}
					
					// if process has only two activities then set the start and due date of the end activity same as that of the start activity.
					// this is required when the following scenario occurs.
					// A project is created on current date.
					// Deliverable is added on some other day, then the end activity's date is set to end date of deliverable.
					// when the deliverable is deleted the end activity's date of the project must be same as that of start activity.
					
					if(ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY").Count == 2)
					{
						ErrorXML = UpdateProcessWithOnlyStartAndEndActivity(ProcessNav,ProcessStartDate,out TotalProcessDuration ,out UpdatedProcessDueDate);

                        if (IsFirstTime && File.Exists(str_FilePath))
                            File.Delete(str_FilePath);  

						return ErrorXML;
					}
					
					TempNodeIter = ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY[STARTACTIVITY/text() = 'False' and ENDACTIVITY/text() = 'False']");
					while (TempNodeIter.MoveNext())
					{
						TempNav= TempNodeIter.Current;
						ActivityDurationTable.Add(TempNav.SelectSingleNode("ACTIVITYID").Value,TempNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value);
					}
					TempNodeIter = ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY[ENDACTIVITY/text()='False']");
					while(TempNodeIter.MoveNext())
					{
						TempNav = TempNodeIter.Current;
						if ((TempNav.SelectSingleNode("STATUS").Value.ToUpper()=="COMPLETED")||(TempNav.SelectSingleNode("STATUS").Value.ToUpper()=="CANCELLED"))
						{
							Day		= Convert.ToInt32(TempNav.SelectSingleNode("ACTUALENDDATE/DAY").Value);
							Month	= Convert.ToInt32(TempNav.SelectSingleNode("ACTUALENDDATE/MONTH").Value);
							Year	= Convert.ToInt32(TempNav.SelectSingleNode("ACTUALENDDATE/YEAR").Value);
                            TempDate = new DateTime(Year, Month, Day);
						}
						else
						{
							Day		= Convert.ToInt32(TempNav.SelectSingleNode("DUEDATE/DAY").Value);
							Month   = Convert.ToInt32(TempNav.SelectSingleNode("DUEDATE/MONTH").Value);
							Year	= Convert.ToInt32(TempNav.SelectSingleNode("DUEDATE/YEAR").Value);
                            TempDate = new DateTime(Year, Month, Day);
						}
						ActivityDueDateTable.Add(TempNav.SelectSingleNode("ACTIVITYID").Value, TempDate);
					}
					ProcessInfoNav = ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO");
					if(ProcessInfoNav != null)
					{							
						ProcessInfoID = ProcessInfoNav.SelectSingleNode("WORKFLOWPROCESSINFOID").Value;
						ProcessStartDateLocked = (ProcessInfoNav.SelectSingleNode("STARTDATELOCKED").Value=="0"?false:true);
					}
					
						for (int Index=0;Index < ActivityInfoList.Count;Index++)
						{
							if(ActivityInfoList[Index].LevelID == Level) 
							{
								iLevel++;								
								
								CalculateTimeLineForActivity = CheckActivityTimeLineStatus(ProcessNav, ActivityTLStatusTable, ActivityInfoList[Index].ActivityID);
								if (CalculateTimeLineForActivity)
								{
									NextActivityList = GetActivityInfoList(ActivityInfoList, iLevel.ToString());
									if(NextActivityList.Count > 0)
									{
										 NextLevelActivityList = new List<ActivityInfo>();
										NextLevelActivityList = ChangeActivityLevel(NextActivityList);
										
										// remove all the activity ID's of this level from the ActivityInfo List.
										for(int Count = 0; Count < NextLevelActivityList.Count; Count++)
										{
											AInfoObj = new ActivityInfo(NextLevelActivityList[Count].ActivityID, iLevel.ToString());
											int I = ActivityInfoList.IndexOf(AInfoObj);
											 ActivityInfoList.RemoveAt(I);										
										}
									}
									else
										NextLevelActivityList = null;
									// select that activity 
									XPathQuery = "PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + ActivityInfoList[Index].ActivityID + "']";
									ActivityNav = ProcessNav.SelectSingleNode(XPathQuery);
									UpdateLock = false;
									StartDateExists = false;
									if(ActivityNav != null)
									{
										ActivityStatusObj = GetActivityStatusObject(ActivityTLStatusTable,ActivityInfoList[Index].ActivityID);
										ActivityStatus		= ActivityNav.SelectSingleNode("STATUS").Value;
										XPathQuery = "SUBFLOWACTIVITY/WORKFLOWPROCESS";
										TempNav = ActivityNav.SelectSingleNode(XPathQuery);

										ActivityStartDateLocked = (ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value == "0" ? false : true);

										if(TempNav != null)
										{
											SubProcessID = TempNav.SelectSingleNode("WORKFLOWPROCESSID").Value;
											SubProcessInfoID = TempNav.SelectSingleNode("descendant::WORKFLOWPROCESSINFOID").Value;
											// find all the predecessors of the current activity and 
											// get the max due date that must be set as the start date for this activity.
											NextStartDate = ProcessStartDate;

											if(ActivityInfoList[Index].StartDate.Equals(new DateTime(1,1,1)))
												GetMaxDueDateFromPredecessors(ProcessNav, ActivityDueDateTable, ActivityInfoList[Index].ActivityID, out NextStartDate, out PredActivityID);
											else
											{
												NextStartDate = ActivityInfoList[Index].StartDate;
												StartDateExists = true;
											}											
											// if the predecessor is a not start activity then add 1 to the next start date
											if(PredActivityID != null)
											{
												PredActNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + PredActivityID + "']");
												if((PredActNav.SelectSingleNode("STARTACTIVITY").Value.ToUpper() == "FALSE") && (PredActNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "FALSE"))
												{
													long Min = ConvertDaysToMinutes(2);
													TempDate = NextStartDate;
													GetDueDate(TempDate, Min.ToString(), out NextStartDate);
												}
											}
											else if(!StartDateExists)
												NextStartDate = ProcessStartDate;

                                            Year    = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/YEAR").Value);
                                            Month   = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/MONTH").Value);
                                            Day     = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/DAY").Value);
                                            TempDate = new DateTime(Year, Month, Day);
											if((ActivityStartDateLocked) && (NextStartDate <= TempDate))
											{												
												NextStartDate	= new DateTime(1,1,1);
												ErrorXML		= CalculateTimeLine_Ex(SubProcessID, NextStartDate, NextLevelActivityList, false, str_FilePath, out ReturnXML, out TotalProcessDuration, out UpdatedProcessDueDate);
												NextStartDate = TempDate;
											}
											else
											{
												if ((ActivityStatus.ToUpper() == "NEW")||(ActivityStatus.ToUpper() == "DRAFT"))
												{
													ErrorXML = CalculateTimeLine_Ex(SubProcessID, NextStartDate, NextLevelActivityList, false, str_FilePath, out ReturnXML, out TotalProcessDuration, out UpdatedProcessDueDate);												
													UpdateLock = true;
													// The Update lock flag indicates that initially the lock was 1, it would be now be changed to 0
												}
												else
												{
													NextStartDate = new DateTime(1, 1, 1);
													ErrorXML = CalculateTimeLine_Ex(SubProcessID, NextStartDate, NextLevelActivityList, false, str_FilePath, out ReturnXML, out TotalProcessDuration, out UpdatedProcessDueDate);
													NextStartDate = TempDate;
												}
											}
											if(ErrorXML == null)
											{
												XPathQuery = "ACTIVITY_INFORMATION/ACTIVITYINFORMATION";
												TempNav = ActivityNav.SelectSingleNode(XPathQuery);
												if(TempNav != null)
												{
													ActivityInfoID = TempNav.SelectSingleNode("ACTIVITYINFORMATIONID").Value;
												}

												if((ActivityStatus.ToUpper()!="COMPLETED")&&(ActivityStatus.ToUpper()!="CANCELLED"))
												{
													//// Update this activity
                                                    ErrorXML = UpdateActivityForTimeLine(ActivityInfoList[Index].ActivityID, NextStartDate, UpdatedProcessDueDate, ActivityInfoID, TotalProcessDuration, SubProcessInfoID, UpdateLock, ActivityStatus);
                                                    if (ErrorXML != null)
                                                    {
                                                        if (IsFirstTime && File.Exists(str_FilePath))
                                                            File.Delete(str_FilePath);  
                                                        return ErrorXML;
                                                    }
												}
												if(ActivityDurationTable.ContainsKey(ActivityInfoList[Index].ActivityID))
												{
													ActivityDurationTable[ActivityInfoList[Index].ActivityID] = TotalProcessDuration;
												}
												if(ActivityDueDateTable.ContainsKey(ActivityInfoList[Index].ActivityID))
												{
													TempActivityDueDate = (DateTime)ActivityDueDateTable[ActivityInfoList[Index].ActivityID];
													ActivityDueDateTable[ActivityInfoList[Index].ActivityID] = UpdatedProcessDueDate;
												}

												ActivityStatusObj.Status = TimeLineStatus.YES;
												
												// find the successors of this activity.

												XPathQuery = "PROCESS_TRANSITION/TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityInfoList[Index].ActivityID + "']/TOACTIVITY/ACTIVITY";
												TempNodeIter = ProcessNav.Select(XPathQuery);

												while(TempNodeIter.MoveNext())
												{
													TempNav = TempNodeIter.Current;
													SuccActivityID = TempNav.SelectSingleNode("ACTIVITYID").Value;
																					
													SuccessorActivity = GetActivtiyInfoObj(ActivityInfoList, SuccActivityID);
													if(SuccessorActivity != null)
													{
														TStatus = GetTimeLineStatus(ActivityTLStatusTable, SuccessorActivity.ActivityID);
														if (TStatus == TimeLineStatus.YES)														
														{
															// If the successor activity's timeline is already calculated , but as a result of the 
															// calculation of timeline of its predecessor at some later time , the successor will hav to be added
															// in the activityInfo List again with its timeline status as yes.
															//ActivityInfoList.Remove(SuccessorActivity);
															SuccessorActivity = new ActivityInfo(SuccActivityID, Level);
															ActivityInfoList.Add(SuccessorActivity);
															ActivityStatusObj = new ActivityTLStatus(SuccActivityID, TimeLineStatus.NO);
															ActivityTLStatusTable.Add(ActivityStatusObj);
														}														
													}
													else
													{
														SuccessorActivity = new ActivityInfo(SuccActivityID,Level);
														ActivityInfoList.Add(SuccessorActivity);
														ActivityStatusObj = new ActivityTLStatus(SuccActivityID,TimeLineStatus.NO);
														ActivityTLStatusTable.Add(ActivityStatusObj);
														
													}													
												}// end while
											}
										}
										else
										{
											if((ActivityNav.SelectSingleNode("ENDACTIVITY") != null) && (ActivityNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "TRUE"))
											{
												ActivityStatus =  ActivityNav.SelectSingleNode("STATUS").Value;
												
												if ((ActivityStatus.ToUpper() != "COMPLETED")&&(ActivityStatus.ToUpper() != "CANCELLED"))
												{
													ActivityStatusObj.Status = TimeLineStatus.YES;
													XPathQuery = "ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID";
													TempNav = ActivityNav.SelectSingleNode(XPathQuery);
													if(TempNav != null)
														ActivityInfoID = TempNav.Value;
													GetMaxDueDateFromPredecessors(ProcessNav, ActivityDueDateTable, ActivityInfoList[Index].ActivityID, out NextStartDate,out PredActivityID);
													if (PredActivityID!=null)
													{
														PredActNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + PredActivityID + "']");
														if((PredActNav.SelectSingleNode("STATUS").Value.ToUpper() == "COMPLETED") || (PredActNav.SelectSingleNode("STATUS").Value.ToUpper() == "CANCELLED"))
														{
															Year    = Convert.ToInt32(PredActNav.SelectSingleNode("DUEDATE/YEAR").Value);
                                                            Month   = Convert.ToInt32(PredActNav.SelectSingleNode("DUEDATE/MONTH").Value);
                                                            Day     = Convert.ToInt32(PredActNav.SelectSingleNode("DUEDATE/DAY").Value);
                                                            NextStartDate = new DateTime(Year, Month, Day);
														}
													}
                                                    ErrorXML = UpdateActivityForTimeLine(ActivityInfoList[Index].ActivityID, NextStartDate, NextStartDate, ActivityInfoID, "0", null, false, ActivityNav.SelectSingleNode("STATUS").Value);
                                                    if (ErrorXML != null)
                                                    {
                                                        if (IsFirstTime && File.Exists(str_FilePath))
                                                            File.Delete(str_FilePath);  
                                                        return ErrorXML;
                                                    }
													if(ActivityDueDateTable.ContainsKey(ActivityInfoList[Index].ActivityID))
													{
														ActivityDueDateTable[ActivityInfoList[Index].ActivityID] = NextStartDate;
													}
												}
											}
										}// end else.
									}// end activity nav.
								}// end if CalculateTimeLine Activity
							}
						}// end For
						
						// update all the activities in the activity TimeLine Status Table.						
						UpdateDependantActivities(ProcessNav, str_FilePath, ref ActivityTLStatusTable, ref ActivityDueDateTable, ref ActivityDurationTable);						
						// update the project DueDate and Duration.
						ErrorXML = UpdateProcessDurationAndDueDate(ProcessID, ProcessInfoID, ActivityDurationTable, ActivityDueDateTable, out TotalProcessDuration, out UpdatedProcessDueDate);
                        if (ErrorXML != null)
                        {
                            if (IsFirstTime && File.Exists(str_FilePath))
                                File.Delete(str_FilePath);  
                            return ErrorXML;
                        }
					//}		
                }// END IF
                else
                {
					// use the old CalculateTimeLine function.					
					ErrorXML = NewCalculateTimeLine(ProcessID, ProcessStartDate, str_FilePath, out TotalProcessDuration, out UpdatedProcessDueDate);
                    if (ErrorXML != null)
                    {
                        if (IsFirstTime && File.Exists(str_FilePath))
                            File.Delete(str_FilePath);  
                        return ErrorXML;
                    }
                }
                ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (CalculateTimeLine_Ex):- Leaving in CalculateTimeLine_Ex.");
                if (IsFirstTime && File.Exists(str_FilePath))
                    File.Delete(str_FilePath);  
				return ErrorXML;
            }
            catch (Exception ExceptionObject)
            {
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("ProcessID = " + ProcessID + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                             "Exception in CalculateTimeLine_Ex().", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExceptionObject, sb_InputData.ToString());
                ErrorXML = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                ReturnXML				= null;
                TotalProcessDuration	= null;
                UpdatedProcessDueDate	= DateTime.Today;
                if (IsFirstTime && File.Exists(str_FilePath))
                    File.Delete(str_FilePath);  
                return ErrorXML; 
            }
        }


        private ERRORXML Get3LevelProcessXML(String str_ProcessID, List<ActivityInfo> ActivityInfoList, out String str_ProcessXML)
        {
            ERRORXML    ErrorXMLString          = null;
            OMInterface OMIObj                  = null;
            String      str_SearchXML           = null; 
            String      str_1LevelXML           = null; 
            String      str_2LevelXML           = null; 
            String      str_2LevelProcessIDs    = null; 
            String      str_2LevelProcessID     = null; 
            XElement    xEle_1LevelXML          = null; 
            XElement    xEle_2LevelXML          = null; 
            XElement    xEle_WorkFlowProcess    = null; 

            OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
            str_ProcessXML = null;

            //Get 1st level Process XML
            str_SearchXML = PrepareXMLForTimeLineGetProcess(str_ProcessID);
            ErrorXMLString = OMIObj.GetHierarchialData(str_SearchXML, ref str_1LevelXML);
            if (String.IsNullOrEmpty(ErrorXMLString) && !String.IsNullOrEmpty(str_1LevelXML))
            {
                xEle_1LevelXML = XElement.Load(XmlReader.Create(new StringReader(str_1LevelXML))); 
                //Get the ProcessIDs for the 2nd level process 
                if (ActivityInfoList != null && ActivityInfoList.Count > 0)
                {
                    for (int Index = 0; Index < ActivityInfoList.Count; Index++)
                    {
                        if (ActivityInfoList[Index].LevelID == "1")
                        {
                            //str_2LevelProcessIDs = ActivityInfoList[Index].ActivityID + ",";
                            //Get the ProcessIDs

                            XElement XeleAct = xEle_1LevelXML.Element("WORKFLOWPROCESS").Element("PROCESS_ACTIVITY").XPathSelectElement("ACTIVITY[ACTIVITYID=" + ActivityInfoList[Index].ActivityID + "]");
                            if (XeleAct != null)
                            {
                                XElement XeleSubflowAct = XeleAct.Element("SUBFLOWACTIVITY");
                                if (XeleSubflowAct != null)
                                {
                                    xEle_WorkFlowProcess = XeleSubflowAct.Element("WORKFLOWPROCESS").Element("WORKFLOWPROCESSID");
                                }

                            }                            
                            if (xEle_WorkFlowProcess != null)
                                str_2LevelProcessIDs = str_2LevelProcessIDs + xEle_WorkFlowProcess.Value + ",";
                        }
                    }
                }
                else
                {
                    foreach (XElement xEle_Process in xEle_1LevelXML.XPathSelectElements("WORKFLOWPROCESS/PROCESS_ACTIVITY/ACTIVITY/SUBFLOWACTIVITY/WORKFLOWPROCESS/WORKFLOWPROCESSID"))
                    {
                        str_2LevelProcessIDs = str_2LevelProcessIDs + xEle_Process.Value + ",";
                    }
                }

                if (!String.IsNullOrEmpty(str_2LevelProcessIDs))
                {
                    str_2LevelProcessIDs = "(" + str_2LevelProcessIDs.Remove(str_2LevelProcessIDs.Length - 1) + ")";
                    //Get the 2nd Level and below process
                    str_SearchXML = PrepareXMLForTimeLineGetProcess2Level(str_2LevelProcessIDs);
                    ErrorXMLString = OMIObj.GetHierarchialData(str_SearchXML, ref str_2LevelXML);
                    if (String.IsNullOrEmpty(ErrorXMLString) && !String.IsNullOrEmpty(str_2LevelXML))
                    {
                        //Now Merge both XML
                        xEle_2LevelXML = XElement.Load(XmlReader.Create(new StringReader(str_2LevelXML)));
                        foreach (XElement xEle_Process in xEle_2LevelXML.XPathSelectElements("WORKFLOWPROCESS"))
                        {
                            str_2LevelProcessID = xEle_Process.Element("WORKFLOWPROCESSID").Value;
  
                            //Find this workflowprocess in 1st level 
                            xEle_WorkFlowProcess = xEle_1LevelXML.Element("WORKFLOWPROCESS").Element("PROCESS_ACTIVITY").XPathSelectElement("ACTIVITY/SUBFLOWACTIVITY/WORKFLOWPROCESS[WORKFLOWPROCESSID=" + str_2LevelProcessID + "]");
                            if (xEle_WorkFlowProcess != null)
                            {
                                xEle_WorkFlowProcess = xEle_WorkFlowProcess.Parent;
                                xEle_WorkFlowProcess.RemoveNodes();
                                xEle_WorkFlowProcess.Add(xEle_Process); 
                            }
                        }

                    }
                }
                str_ProcessXML = xEle_1LevelXML.ToString();  
            }
            return ErrorXMLString;
        }

        private String GetCurrentProcess(String str_FilePath, String str_ProcessID)
        {
            XDocument xdoc_ProcessCollection    = null; 
            XElement  xele_Process              = null;
            String str_WorkflowProcessXML = null;

            if (!String.IsNullOrEmpty(str_FilePath) && File.Exists(str_FilePath))
            {
                xdoc_ProcessCollection = XDocument.Load(str_FilePath);
                xele_Process = xdoc_ProcessCollection.XPathSelectElement("descendant::WORKFLOWPROCESS[WORKFLOWPROCESSID= '" + str_ProcessID + "']");
                if (xele_Process != null)
                    str_WorkflowProcessXML = "<WORKFLOWPROCESSCOLLECTION>" + xele_Process.ToString() + "</WORKFLOWPROCESSCOLLECTION>";
            }

            return str_WorkflowProcessXML; 
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="ProcessID"></param>
        /// <param name="ProcessStartDate"></param>
        /// <param name="ReturnXML"></param>
        /// <param name="TotalProcessDuration"></param>
        /// <param name="UpdatedProcessDueDate"></param>
		/// <param name="ActivityInfoList"></param>
        /// <param name="ProcessNav"></param> 
        /// <returns></returns>
        public ERRORXML CalculateTimeLine_Ex_ForCreateDeliverable(String ProcessID, DateTime ProcessStartDate, List<ActivityInfo> ActivityInfoList, XPathNavigator ProcessNav,  out XML ReturnXML, out String TotalProcessDuration, out DateTime UpdatedProcessDueDate)
        {
            String					ErrorXML					= null;
            XPathNavigator			ActivityNav					= null;            
            XPathNavigator			TempNav						= null;
            XPathNavigator			ProcessInfoNav				= null;
            XPathNavigator			PredActNav					= null;            
            XPathNodeIterator		TempNodeIter				= null;            
			String					XPathQuery					= null;
			String					SubProcessID				= null;
			String					Level						= "1";
			String					SuccActivityID				= null;			
			String					ActivityInfoID				= null;
			String					ProcessInfoID				= null;
			String					PredActivityID				= null;
			String					ActivityStatus				= null;
			String					SubProcessInfoID			= null;
			int						iLevel						= 1;
			int						Day							= 0;
			int						Month						= 0;
			int						Year						= 0;
            //int                     Hour                        = 0;
            //int                     Min                         = 0;
            //int                     Sec                         = 0;
			ActivityInfo			SuccessorActivity			= null;
			ActivityInfo			AInfoObj					= null;			
			List<ActivityInfo>		NextActivityList			= null;
			List<ActivityInfo>		NextLevelActivityList		= null;
			StringBuilder			UpdateActivityXML			= null;						
			Hashtable				ActivityDurationTable		= null;
			Hashtable				ActivityDueDateTable    	= null;			
			DateTime				ProcessDueDate				;
			DateTime				ActivityDate				; 
			DateTime				NextStartDate				; 
			DateTime				TempDate					;
			DateTime				TempActivityDueDate			;
			List<ActivityTLStatus>	ActivityTLStatusTable			= null;
			ActivityTLStatus		ActivityStatusObj				= null;
			bool					CalculateTimeLineForActivity	= false;	
			bool					ProcessStartDateLocked			= false;
			bool					ActivityStartDateLocked			= false;
			bool					UpdateLock						= false;
			bool					StartDateExists					= false;			
            OMInterface             OMIObj							= null; 
            TimeLineStatus			TStatus						;
            ALError                 ExcepErrObj                 = null;
            StringBuilder           sb_InputData                = null;
                
			try
            {
                ReturnXML				= null;
                TotalProcessDuration	= null;
                UpdatedProcessDueDate	= DateTime.Today;
				ActivityDate			= DateTime.Today;
				ProcessDueDate			= DateTime.Today;
				TempDate				= DateTime.Today;
				TempActivityDueDate		= new DateTime(1,1,1);
				UpdateActivityXML		= new StringBuilder(300);
                OMIObj                  = new OMInterface(UserID, CurrentLanguage, TenantName);   
				
				ActivityDurationTable   = new Hashtable();
				ActivityDueDateTable	= new Hashtable();

                if ((ActivityInfoList!=null)&&(ActivityInfoList.Count > 0))
                {
					// sync ActivityTimeLine Status List with Status = NO
					ActivityTLStatusTable = new List<ActivityTLStatus>();
					ActivityTLStatusTable = SyncActivityTimeLineStatusList(ActivityInfoList, Level);
					
                    ProcessNav = ProcessNav.SelectSingleNode("descendant::WORKFLOWPROCESSID[text()= '" + ProcessID + "']");
                    ProcessNav.MoveToParent(); 
					if (ProcessNav!=null)
					{
						if (ProcessStartDate.Equals(new DateTime(1,1,1)))
						{
							if((ProcessNav.SelectSingleNode("STARTDATE") != null) && (ProcessNav.SelectSingleNode("STARTDATE").HasChildren))
							{									
								Day = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/DAY").Value);
								Month = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/MONTH").Value);
								Year = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/YEAR").Value);
                                ProcessStartDate = new DateTime(Year, Month, Day);
							}	
						}
						if((ProcessNav.SelectSingleNode("DUEDATE") != null) && (ProcessNav.SelectSingleNode("DUEDATE").HasChildren))
						{																
							Day		= Convert.ToInt32(ProcessNav.SelectSingleNode("DUEDATE/DAY").Value);
							Month	= Convert.ToInt32(ProcessNav.SelectSingleNode("DUEDATE/MONTH").Value);
							Year	= Convert.ToInt32(ProcessNav.SelectSingleNode("DUEDATE/YEAR").Value);
                            ProcessDueDate = new DateTime(Year, Month, Day);
						}
					}
					
					// if process has only two activities then set the start and due date of the end activity same as that of the start activity.
					// this is required when the following scenario occurs.
					// A project is created on current date.
					// Deliverable is added on some other day, then the end activity's date is set to end date of deliverable.
					// when the deliverable is deleted the end activity's date of the project must be same as that of start activity.
					
					if(ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY").Count == 2)
					{
						ErrorXML = UpdateProcessWithOnlyStartAndEndActivity(ProcessNav,ProcessStartDate,out TotalProcessDuration ,out UpdatedProcessDueDate);
						return ErrorXML;
					}
					
					TempNodeIter = ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY[STARTACTIVITY/text() = 'False' and ENDACTIVITY/text() = 'False']");
					while (TempNodeIter.MoveNext())
					{
						TempNav= TempNodeIter.Current;
						ActivityDurationTable.Add(TempNav.SelectSingleNode("ACTIVITYID").Value,TempNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value);
					}
					TempNodeIter = ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY[ENDACTIVITY/text()='False']");
					while(TempNodeIter.MoveNext())
					{
						TempNav = TempNodeIter.Current;
						if ((TempNav.SelectSingleNode("STATUS").Value.ToUpper()=="COMPLETED")||(TempNav.SelectSingleNode("STATUS").Value.ToUpper()=="CANCELLED"))
						{
							Day		= Convert.ToInt32(TempNav.SelectSingleNode("ACTUALENDDATE/DAY").Value);
							Month	= Convert.ToInt32(TempNav.SelectSingleNode("ACTUALENDDATE/MONTH").Value);
							Year	= Convert.ToInt32(TempNav.SelectSingleNode("ACTUALENDDATE/YEAR").Value);
                            TempDate = new DateTime(Year, Month, Day);
						}
						else
						{
							Day		= Convert.ToInt32(TempNav.SelectSingleNode("DUEDATE/DAY").Value);
							Month   = Convert.ToInt32(TempNav.SelectSingleNode("DUEDATE/MONTH").Value);
							Year	= Convert.ToInt32(TempNav.SelectSingleNode("DUEDATE/YEAR").Value);
                            TempDate = new DateTime(Year, Month, Day);
						}
						ActivityDueDateTable.Add(TempNav.SelectSingleNode("ACTIVITYID").Value, TempDate);
					}
					ProcessInfoNav = ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO");
					if(ProcessInfoNav != null)
					{							
						ProcessInfoID = ProcessInfoNav.SelectSingleNode("WORKFLOWPROCESSINFOID").Value;
						ProcessStartDateLocked = (ProcessInfoNav.SelectSingleNode("STARTDATELOCKED").Value=="0"?false:true);
					}
					
						for (int Index=0;Index < ActivityInfoList.Count;Index++)
						{
							if(ActivityInfoList[Index].LevelID == Level) 
							{
								iLevel++;								
								
								CalculateTimeLineForActivity = CheckActivityTimeLineStatus(ProcessNav, ActivityTLStatusTable, ActivityInfoList[Index].ActivityID);
								if (CalculateTimeLineForActivity)
								{
									NextActivityList = GetActivityInfoList(ActivityInfoList, iLevel.ToString());
									if(NextActivityList.Count > 0)
									{
										 NextLevelActivityList = new List<ActivityInfo>();
										NextLevelActivityList = ChangeActivityLevel(NextActivityList);
										
										// remove all the activity ID's of this level from the ActivityInfo List.
										for(int Count = 0; Count < NextLevelActivityList.Count; Count++)
										{
											AInfoObj = new ActivityInfo(NextLevelActivityList[Count].ActivityID, iLevel.ToString());
											int I = ActivityInfoList.IndexOf(AInfoObj);
											 ActivityInfoList.RemoveAt(I);										
										}
									}
									else
										NextLevelActivityList = null;
									// select that activity 
									XPathQuery = "PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + ActivityInfoList[Index].ActivityID + "']";
									ActivityNav = ProcessNav.SelectSingleNode(XPathQuery);
									UpdateLock = false;
									StartDateExists = false;
									if(ActivityNav != null)
									{
										ActivityStatusObj = GetActivityStatusObject(ActivityTLStatusTable,ActivityInfoList[Index].ActivityID);
										ActivityStatus		= ActivityNav.SelectSingleNode("STATUS").Value;
										XPathQuery = "SUBFLOWACTIVITY/WORKFLOWPROCESS";
										TempNav = ActivityNav.SelectSingleNode(XPathQuery);

										ActivityStartDateLocked = (ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value == "0" ? false : true);

										if(TempNav != null)
										{
											SubProcessID = TempNav.SelectSingleNode("WORKFLOWPROCESSID").Value;
											SubProcessInfoID = TempNav.SelectSingleNode("descendant::WORKFLOWPROCESSINFOID").Value;
											// find all the predecessors of the current activity and 
											// get the max due date that must be set as the start date for this activity.
											NextStartDate = ProcessStartDate;

											if(ActivityInfoList[Index].StartDate.Equals(new DateTime(1,1,1)))
												GetMaxDueDateFromPredecessors(ProcessNav, ActivityDueDateTable, ActivityInfoList[Index].ActivityID, out NextStartDate, out PredActivityID);
											else
											{
												NextStartDate = ActivityInfoList[Index].StartDate;
												StartDateExists = true;
											}											
											// if the predecessor is a not start activity then add 1 to the next start date
											if(PredActivityID != null)
											{
												PredActNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + PredActivityID + "']");
												if((PredActNav.SelectSingleNode("STARTACTIVITY").Value.ToUpper() == "FALSE") && (PredActNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "FALSE"))
												{
													long l_Min = ConvertDaysToMinutes(2);
													TempDate = NextStartDate;
                                                    GetDueDate(TempDate, l_Min.ToString(), out NextStartDate);
												}
											}
											else if(!StartDateExists)
												NextStartDate = ProcessStartDate;
                                            Year    = Convert.ToInt32( ActivityNav.SelectSingleNode("STARTDATE/YEAR").Value);
                                            Month   = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/MONTH").Value);
                                            Day     = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/DAY").Value);
                                            TempDate = new DateTime(Year, Month, Day);
											if((ActivityStartDateLocked) && (NextStartDate <= TempDate))
											{				
								                //HemantK select the sub process node from the input ProcessNavigator and then pass it
												NextStartDate	= new DateTime(1,1,1);
												ErrorXML		= CalculateTimeLine_Ex_ForCreateDeliverable(SubProcessID, NextStartDate, NextLevelActivityList, TempNav, out ReturnXML, out TotalProcessDuration, out UpdatedProcessDueDate);
												NextStartDate = TempDate;
											}
											else
											{
												if ((ActivityStatus.ToUpper() == "NEW")||(ActivityStatus.ToUpper() == "DRAFT"))
												{
													ErrorXML = CalculateTimeLine_Ex_ForCreateDeliverable(SubProcessID, NextStartDate, NextLevelActivityList, TempNav, out ReturnXML, out TotalProcessDuration, out UpdatedProcessDueDate);												
													UpdateLock = true;
													// The Update lock flag indicates that initially the lock was 1, it would be now be changed to 0
												}
												else
												{
													NextStartDate = new DateTime(1, 1, 1);
													ErrorXML = CalculateTimeLine_Ex_ForCreateDeliverable(SubProcessID, NextStartDate, NextLevelActivityList, TempNav, out ReturnXML, out TotalProcessDuration, out UpdatedProcessDueDate);
													NextStartDate = TempDate;
												}
											}
											if(ErrorXML == null)
											{
												XPathQuery = "ACTIVITY_INFORMATION/ACTIVITYINFORMATION";
												TempNav = ActivityNav.SelectSingleNode(XPathQuery);
												if(TempNav != null)
												{
													ActivityInfoID = TempNav.SelectSingleNode("ACTIVITYINFORMATIONID").Value;
												}

												if((ActivityStatus.ToUpper()!="COMPLETED")&&(ActivityStatus.ToUpper()!="CANCELLED"))
												{
													//// Update this activity
                                                    ErrorXML = UpdateActivityForTimeLine(ActivityInfoList[Index].ActivityID, NextStartDate, UpdatedProcessDueDate, ActivityInfoID, TotalProcessDuration, SubProcessInfoID, UpdateLock, ActivityStatus);
													if(ErrorXML != null)
														return ErrorXML;
												}
												if(ActivityDurationTable.ContainsKey(ActivityInfoList[Index].ActivityID))
												{
													ActivityDurationTable[ActivityInfoList[Index].ActivityID] = TotalProcessDuration;
												}
												if(ActivityDueDateTable.ContainsKey(ActivityInfoList[Index].ActivityID))
												{
													TempActivityDueDate = (DateTime)ActivityDueDateTable[ActivityInfoList[Index].ActivityID];
													ActivityDueDateTable[ActivityInfoList[Index].ActivityID] = UpdatedProcessDueDate;
												}

												ActivityStatusObj.Status = TimeLineStatus.YES;
												
												// find the successors of this activity.

												XPathQuery = "PROCESS_TRANSITION/TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityInfoList[Index].ActivityID + "']/TOACTIVITY/ACTIVITY";
												TempNodeIter = ProcessNav.Select(XPathQuery);

												while(TempNodeIter.MoveNext())
												{
													TempNav = TempNodeIter.Current;
													SuccActivityID = TempNav.SelectSingleNode("ACTIVITYID").Value;
																					
													SuccessorActivity = GetActivtiyInfoObj(ActivityInfoList, SuccActivityID);
													if(SuccessorActivity != null)
													{
														TStatus = GetTimeLineStatus(ActivityTLStatusTable, SuccessorActivity.ActivityID);
														if (TStatus == TimeLineStatus.YES)														
														{
															// If the successor activity's timeline is already calculated , but as a result of the 
															// calculation of timeline of its predecessor at some later time , the successor will hav to be added
															// in the activityInfo List again with its timeline status as yes.
															//ActivityInfoList.Remove(SuccessorActivity);
															SuccessorActivity = new ActivityInfo(SuccActivityID, Level);
															ActivityInfoList.Add(SuccessorActivity);
															ActivityStatusObj = new ActivityTLStatus(SuccActivityID, TimeLineStatus.NO);
															ActivityTLStatusTable.Add(ActivityStatusObj);
														}
													}
													else
													{
														SuccessorActivity = new ActivityInfo(SuccActivityID,Level);
														ActivityInfoList.Add(SuccessorActivity);
														ActivityStatusObj = new ActivityTLStatus(SuccActivityID,TimeLineStatus.NO);
														ActivityTLStatusTable.Add(ActivityStatusObj);
														
													}													
												}// end while
											}
										}
										else
										{
											if((ActivityNav.SelectSingleNode("ENDACTIVITY") != null) && (ActivityNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "TRUE"))
											{
												ActivityStatus =  ActivityNav.SelectSingleNode("STATUS").Value;
												
												if ((ActivityStatus.ToUpper() != "COMPLETED")&&(ActivityStatus.ToUpper() != "CANCELLED"))
												{
													ActivityStatusObj.Status = TimeLineStatus.YES;
													XPathQuery = "ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID";
													TempNav = ActivityNav.SelectSingleNode(XPathQuery);
													if(TempNav != null)
														ActivityInfoID = TempNav.Value;
													GetMaxDueDateFromPredecessors(ProcessNav, ActivityDueDateTable, ActivityInfoList[Index].ActivityID, out NextStartDate,out PredActivityID);
													if (PredActivityID!=null)
													{
														PredActNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + PredActivityID + "']");
														if((PredActNav.SelectSingleNode("STATUS").Value.ToUpper() == "COMPLETED") || (PredActNav.SelectSingleNode("STATUS").Value.ToUpper() == "CANCELLED"))
														{
															NextStartDate = new DateTime(Convert.ToInt32(PredActNav.SelectSingleNode("DUEDATE/YEAR").Value), Convert.ToInt32(PredActNav.SelectSingleNode("DUEDATE/MONTH").Value), Convert.ToInt32(PredActNav.SelectSingleNode("DUEDATE/DAY").Value));
														}
													}
                                                    ErrorXML = UpdateActivityForTimeLine(ActivityInfoList[Index].ActivityID, NextStartDate, NextStartDate, ActivityInfoID, "0", null, false, ActivityNav.SelectSingleNode("STATUS").Value);
													if(ErrorXML != null)
														return ErrorXML;
													if(ActivityDueDateTable.ContainsKey(ActivityInfoList[Index].ActivityID))
													{
														ActivityDueDateTable[ActivityInfoList[Index].ActivityID] = NextStartDate;
													}
												}
											}
										}// end else.
									}// end activity nav.
								}// end if CalculateTimeLine Activity
							}
						}// end For
						
						// update all the activities in the activity TimeLine Status Table.						
						UpdateDependantActivities(ProcessNav, null, ref ActivityTLStatusTable, ref ActivityDueDateTable, ref ActivityDurationTable);						
						// update the project DueDate and Duration.
						ErrorXML = UpdateProcessDurationAndDueDate(ProcessID, ProcessInfoID, ActivityDurationTable, ActivityDueDateTable, out TotalProcessDuration, out UpdatedProcessDueDate);													
						if (ErrorXML!=null)
							return ErrorXML;						
					//}		
                }// END IF
                else
                {
					// use the old CalculateTimeLine function.					
					ErrorXML = NewCalculateTimeLine_ForCreateDeliverable(ProcessID,ProcessStartDate, ProcessNav, out TotalProcessDuration,out UpdatedProcessDueDate);
					if (ErrorXML==null)
					{
					
					}
					else return ErrorXML;
                }
				return ErrorXML;
            }
            catch (Exception ExceptionObject)
            {
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("ProcessID = " + ProcessID + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                             "Exception in CalculateTimeLine_Ex_ForCreateDeliverable().", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExceptionObject, sb_InputData.ToString());
                ErrorXML = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                ReturnXML				= null;
                TotalProcessDuration	= null;
                UpdatedProcessDueDate	= DateTime.Today;
                return ErrorXML;
            }
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="ProcessInstanceID"></param>
        /// <param name="StartDate"></param>
        /// <param name="TotalProcessDuration"></param>
        /// <param name="UpdatedProcessDueDate"></param>
        /// <returns></returns>
        public ERRORXML NewCalculateTimeLine(String ProcessInstanceID,DateTime StartDate, String str_FilePath, out String TotalProcessDuration,out DateTime UpdatedProcessDueDate)
		{
			XPathDocument	ProcessDoc					= null;
			XPathNavigator	ProcessNav					= null;
			XPathNavigator	ProcessActivityNav			= null;
			XPathNavigator	ProcessTransitionNav		= null;
			XPathNavigator	ActivityNav					= null;
			XPathNavigator	StartDateNav				= null;
			XPathNavigator	DueDateNav					= null;
			XPathNavigator	ActEndDateNav				= null;	
			XPathNavigator	ActivityStatusNav			= null;	
			XPathNavigator	TransitionNav				= null;
			XPathNavigator	SubflowProcessNav			= null;
			XPathNodeIterator	TransitionNodeIter		= null;			
			XML				ProcessInputXML				= null;
			XML				ProcessInstanceXML			= null;
			XML				UpdateXML					= null;			
			ERRORXML		ErrorXml					= null;
			StringReader	StrProcess					= null;
			StringBuilder	StartActivity_Succ			= null;			
			String			ProcessInfoID				= null;
			String			ActivityInfoID				= null;
			String			ProcessStatus				= null;
			String			ActivityDuration			= null;
			String			StartActivityID				= null;
			String			ToActivityID				= null;
			String			StartActivity_Successors	= null;
			String			ActivityID					= null;
			String			ActivityStatus				= null;
			String			EndActivity					= null;
			String			PredecessorList				= null;
			String			ActivityIDOfMaxDate			= null;
			String			PredActStatus				= null;			
            String          str_ProcessStatus           = null; 
			String[]		PredecessorListArr          = null;
			List<Activity>	ActivityList				= null;			
			Activity[]		ActivityListArr				= null;
			Activity		ActivityObj					= null;
			Activity		NewActivityObj				= null;
			Queue			Vertices					= null;
            Hashtable		VisitedVertices				= null;
            bool			StartActivtyPred_Flag       = false;
            bool			PredecessorFound            = true;
            bool			IsSubFlowActivity           = false;
            bool			ModifiedFlag				= false;
            bool			ProcessStartDateLocked		= false;
            bool			ActivityStartDateLocked		= false;
            OMInterface		OMI							= null;
			DateTime		ActivityStartDate			 ;
			DateTime		ActualEndDate				;
			DateTime		ActivityDueDate				;
			DateTime		PredecessorActDate			;
            DateTime		StartDateForNextAct			;
            DateTime		DueDateForNextAct			;
            DateTime        Org_ProcessStartDate        ;
            DateTime		Org_ProcessDueDate			;            
            DateTime        TempStartdate;
			long			ProcessDuration				= 0;
			int				Day							= 0;
            int				Month						= 0;
            int				Year						= 0;
			Queue<ActivityDependancy>	ActivityDepQ    = null;
			ActivityDependancy			ActDepObj		= null;
            ALError                     ExcepErrObj     = null;
            StringBuilder               sb_InputData    = null;
            
			try
			{
				
				UpdatedProcessDueDate	= new DateTime();
				DueDateForNextAct		= new DateTime();
				ActualEndDate			= new DateTime(1,1,1);
				ActivityList			= new List<Activity>();
                ActivityDepQ            = new Queue<ActivityDependancy>();
                OMI                     = new OMInterface(UserID, CurrentLanguage, TenantName);   
                
                ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- Entered NewCalculateTimeLine() function.");
                ProcessInstanceXML = GetCurrentProcess(str_FilePath, ProcessInstanceID);
                if (!String.IsNullOrEmpty(ProcessInstanceXML))
                {
                    StrProcess = new StringReader(ProcessInstanceXML);
                    ProcessDoc = new XPathDocument(StrProcess);
                    ProcessNav = ProcessDoc.CreateNavigator();
                    ProcessNav = ProcessNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                    //In first level get SUBFLOWACTIVITY with the ProcessID is fetched
                    //so GetCurrentProcess() will return the ProcessInstanceXML
                    //but this XML will not have all the required data so node count < 3 check has been added
                    if (ProcessNav.SelectChildren(XPathNodeType.All).Count < 3)
                        ProcessInstanceXML = null; 
                }

                if (String.IsNullOrEmpty(ProcessInstanceXML))
                {
                    ProcessInputXML = PrepareXMLForGetProcess(ProcessInstanceID);
                    ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- before GetHierarchialData() in NewCalculateTimeLine.");
                    ErrorXml = OMI.GetHierarchialData(ProcessInputXML, ref ProcessInstanceXML);
                    ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- after GetHierarchialData() in NewCalculateTimeLine.");
                    if (String.IsNullOrEmpty(ErrorXml) && !String.IsNullOrEmpty(ProcessInstanceXML))
                    {
                        StrProcess = new StringReader(ProcessInstanceXML);
                        ProcessDoc = new XPathDocument(StrProcess);
                        ProcessNav = ProcessDoc.CreateNavigator();
                        ProcessNav = ProcessNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                    }
                }

                if (ErrorXml == null)
                {
                    ProcessDuration = 0;

					ProcessStatus    = ProcessNav.SelectSingleNode("STATUS").Value;						
					if ((ProcessStatus.ToUpper()!="COMPLETED")&&(ProcessStatus.ToUpper()!="CANCELLED"))
					{
                        str_ProcessStatus   = ProcessNav.SelectSingleNode("STATUS").Value;   
						ProcessInfoID		= ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").Value;
						ProcessStartDateLocked = (ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/STARTDATELOCKED").Value == "0" ? false : true);
						ProcessActivityNav	= ProcessNav.SelectSingleNode("PROCESS_ACTIVITY");
                        Year    = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/YEAR").Value);
                        Month   = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/MONTH").Value);
                        Day     = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/DAY").Value);
                        Org_ProcessStartDate = new DateTime(Year, Month, Day);
						if(StartDate.Equals(new DateTime(1, 1, 1)))
                        {
                            StartDate = Org_ProcessStartDate;
                        }
                        else if (ProcessStartDateLocked == true)
                        {
                            if (StartDate <= Org_ProcessStartDate)
                            StartDate = Org_ProcessStartDate;
                        }
						if(ProcessActivityNav.Select("ACTIVITY").Count == 2)
						{
							DueDateNav = ProcessNav.SelectSingleNode("DUEDATE");
							if((DueDateNav != null) && (DueDateNav.HasChildren))
							{
                                Year    = Convert.ToInt32(DueDateNav.SelectSingleNode("YEAR").Value);
                                Month   = Convert.ToInt32(DueDateNav.SelectSingleNode("MONTH").Value);
                                Day     = Convert.ToInt32(DueDateNav.SelectSingleNode("DAY").Value);
                                Org_ProcessDueDate = new DateTime(Year, Month, Day);
								ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- Before UpdateProcessWithOnlyStartAndEndActivity in NewCalculateTimeLine() function.");
								ErrorXml = UpdateProcessWithOnlyStartAndEndActivity(ProcessNav, StartDate, out TotalProcessDuration, out UpdatedProcessDueDate);
								ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- After UpdateProcessWithOnlyStartAndEndActivity in NewCalculateTimeLine() function.");
								return ErrorXml;
							}                        						                      
						}
					
						ProcessTransitionNav = ProcessNav.SelectSingleNode("PROCESS_TRANSITION");
					
						ActivityNav			= ProcessActivityNav.SelectSingleNode("ACTIVITY[STARTACTIVITY/text()='True']");
						if (ActivityNav!=null)
						{
							StartActivityID = ActivityNav.SelectSingleNode("ACTIVITYID").Value;
							StartDateNav = ActivityNav.SelectSingleNode("STARTDATE");
							DueDateNav	 = ActivityNav.SelectSingleNode("DUEDATE");
							ActivityStatusNav = ActivityNav.SelectSingleNode("STATUS");
							
							Day			 = Convert.ToInt32(StartDateNav.SelectSingleNode("DAY").Value);
							Month		 = Convert.ToInt32(StartDateNav.SelectSingleNode("MONTH").Value);
							Year		 = Convert.ToInt32(StartDateNav.SelectSingleNode("YEAR").Value);
                            ActivityStartDate = new DateTime(Year, Month, Day);
							if (StartDate!=ActivityStartDate)
								ActivityStartDate = StartDate;

							ActivityDueDate = ActivityStartDate;
							ActivityInfoID	 = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID").Value;
							ActivityDuration = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value;
							ActivityStartDateLocked = (ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value == "0" ? false : true);
							ActivityDuration = "0";
							if(((ActivityStatusNav.Value.ToUpper() == "NEW") || ((ActivityStatusNav.Value.ToUpper() == "DRAFT")))&& (ActivityStartDateLocked == false))
								ModifiedFlag = true;
							else
								ModifiedFlag = false;

							if((ActivityStatusNav.Value.ToUpper() == "COMPLETED") || (ActivityStatusNav.Value.ToUpper() == "CANCELLED"))
							{
								ActEndDateNav = ActivityNav.SelectSingleNode("ACTUALENDDATE");
								if((ActEndDateNav !=null)&&(ActEndDateNav.HasChildren))
								{
									Day			 = Convert.ToInt32(ActEndDateNav.SelectSingleNode("DAY").Value);
									Month		 = Convert.ToInt32(ActEndDateNav.SelectSingleNode("MONTH").Value);
									Year		 = Convert.ToInt32(ActEndDateNav.SelectSingleNode("YEAR").Value);
                                    ActualEndDate = new DateTime(Year, Month, Day);
								}						
							}

                            ActivityObj = new Activity(StartActivityID, ActivityStartDate, ActivityDueDate, ActualEndDate, ActivityInfoID, ActivityDuration, ActivityStatusNav.Value);
							ActivityObj.Modified = ModifiedFlag;
							ActivityList.Add(ActivityObj);																
								
							VisitedVertices = new Hashtable();
							VisitedVertices.Add(StartActivityID, true);
							if (ProcessTransitionNav!=null)
							{							
								TransitionNodeIter = ProcessTransitionNav.Select("TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + StartActivityID + "']");
								if (TransitionNodeIter.Count > 0)
								{
									Vertices			= new Queue();
									StartActivity_Succ	= new StringBuilder();
									while(TransitionNodeIter.MoveNext())
									{
										TransitionNav = TransitionNodeIter.Current;
										ToActivityID = TransitionNav.SelectSingleNode("TOACTIVITY/ACTIVITY/ACTIVITYID").Value;
										Vertices.Enqueue(ToActivityID);
										StartActivity_Succ.AppendFormat(";{0}", ToActivityID);
									}// end while
									StartActivity_Succ.Append(";");
									StartActivity_Successors = StartActivity_Succ.ToString();
									if (Vertices.Count > 0)
									{
										while (Vertices.Count > 0)
										{
											ActivityID = (String)Vertices.Dequeue();                                        
											IsSubFlowActivity = false;
											ModifiedFlag      = false;
											if (!VisitedVertices.ContainsKey(ActivityID))
											{
													VisitedVertices.Add(ActivityID, true);
													ActivityNav = ProcessActivityNav.SelectSingleNode("ACTIVITY[ACTIVITYID/text()='" + ActivityID + "']");
													DueDateNav  = ActivityNav.SelectSingleNode("DUEDATE");
													ActivityStatus = ActivityNav.SelectSingleNode("STATUS").Value;
													ActEndDateNav  = ActivityNav.SelectSingleNode("ACTUALENDDATE");
													StartDateNav	= ActivityNav.SelectSingleNode("STARTDATE");
													if ((StartDateNav!=null)&&(StartDateNav.HasChildren))
													{
														Day			 = Convert.ToInt32(StartDateNav.SelectSingleNode("DAY").Value);
														Month		 = Convert.ToInt32(StartDateNav.SelectSingleNode("MONTH").Value);
														Year		 = Convert.ToInt32(StartDateNav.SelectSingleNode("YEAR").Value);
                                                        ActivityStartDate = new DateTime(Year, Month, Day);
													}
													SubflowProcessNav = ActivityNav.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS/WORKFLOWPROCESSID");
													if(SubflowProcessNav != null)
														IsSubFlowActivity = true;

													if(((ActivityStatus.ToUpper() == "NEW") || ((ActivityStatus.ToUpper() == "DRAFT")) || (IsSubFlowActivity) || (DueDateNav.HasChildren == false) || (ActivityStartDateLocked == false)) && ((ActivityStatus.ToUpper() != "COMPLETED") && (ActivityStatus.ToUpper() != "CANCELLED")))
														ModifiedFlag = true;
													ActivityInfoID	= ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID").Value;
													EndActivity		= ActivityNav.SelectSingleNode("ENDACTIVITY").Value;
													// select the due date and start date nodes												
													ActivityDuration = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value;
													ActivityStartDateLocked = (ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value == "0" ? false : true);
													if((EndActivity.ToUpper() != "TRUE") & (!IsSubFlowActivity))
														ProcessDuration += Convert.ToInt64(ActivityDuration);
													TransitionNodeIter = ProcessTransitionNav.Select("TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityID + "']");
													while(TransitionNodeIter.MoveNext())
													{
														TransitionNav = TransitionNodeIter.Current;
														ToActivityID = TransitionNav.SelectSingleNode("TOACTIVITY/ACTIVITY/ACTIVITYID").Value;
														Vertices.Enqueue(ToActivityID);
													}// end while
													StartActivtyPred_Flag = StartActivity_Successors.Contains(";" + ActivityID + ";");
													if(StartActivtyPred_Flag == false)
													{
														// Find the max duration of the predecessors of this activity.													
														GetPredecessorsAndActivityIDOfMaxDueDate(ActivityList, ActivityID, ProcessNav, out PredecessorList, out ActivityIDOfMaxDate);
														if((PredecessorList != null) && (PredecessorList != ""))
														{
															PredecessorListArr = PredecessorList.Split(';');
															PredecessorFound = true;
															for(int Index = 0; Index < PredecessorListArr.Length; Index++)
															{
																if(PredecessorListArr[Index] != "")
																{
																	if(!VisitedVertices.ContainsKey(PredecessorListArr[Index]))
																	{
																		PredecessorFound = false;
																		break;
																	}
																}
															}
														}
														else
															PredecessorFound = false;
													}
													else
													{
														ActivityIDOfMaxDate = StartActivityID;
													}
													
													if (PredecessorFound)
													{
														ActivityFilter AF = new ActivityFilter(ActivityIDOfMaxDate);
														Predicate<Activity> FindActivityFromList = null;
														FindActivityFromList = new Predicate<Activity>(AF.FilterByActivityID);
														
														ActivityObj = ActivityList.Find(FindActivityFromList);
														if (ActivityObj!=null)
														{
															PredActStatus = ProcessActivityNav.SelectSingleNode("ACTIVITY[ACTIVITYID='"+ActivityObj.ActivityID+"']/STATUS").Value;
															if((PredActStatus.ToUpper() == "COMPLETED") || (PredActStatus.ToUpper() == "CANCELLED"))
																PredecessorActDate = ActivityObj.ActualEndDate;
															else															
																PredecessorActDate = ActivityObj.DueDate;

                                                            if ((ActivityStartDateLocked == true) && ((StartDateNav != null) && (StartDateNav.HasChildren)))
                                                            {
                                                                if (PredecessorActDate > ActivityStartDate)
                                                                    StartDateForNextAct = PredecessorActDate;
                                                                else
                                                                    StartDateForNextAct = ActivityStartDate;
                                                            }
                                                            else
                                                            {
                                                                if ((StartActivtyPred_Flag == false) && (EndActivity.ToUpper() == "FALSE"))
                                                                {
                                                                    long Min = ConvertDaysToMinutes(2);
                                                                    GetDueDate(PredecessorActDate, Min.ToString(), out StartDateForNextAct);
                                                                }
                                                                else
                                                                {
                                                                    StartDateForNextAct = PredecessorActDate;
                                                                }
                                                            }
															if (((StartActivtyPred_Flag == false) && (EndActivity.ToUpper() == "FALSE")))
															{
																// Case: StartActivtyPred_Flag == false and endActivity == false
																if((ActivityDuration != "") || ActivityDuration != null)
																{
                                                                    if (ActivityDuration != "0")
                                                                    {
                                                                        //Bug Fix:S0834866
                                                                        // If the start date of the Step lies on a weekend then include the weekends so that the next contributor
                                                                        // gets the full duration of working days to complete the step.
                                                                        // This is done by getting the next working date in the variable TempStartdate following the weekend date.
                                                                        TempStartdate = GetStartDateFollowingWeekend(StartDateForNextAct);
                                                                        GetDueDate(TempStartdate, ActivityDuration, out DueDateForNextAct);
                                                                    }
                                                                    else
                                                                    {                                                                        
                                                                        DueDateForNextAct = StartDateForNextAct;
                                                                    }
																}
															}
															else
															{
																if (EndActivity.ToUpper() == "FALSE")
																{
																	// Case: StartActivtyPred_Flag == true and endActivity == false																
                                                                    if (ActivityDuration != "0")
                                                                    {
                                                                        //Bug Fix:S0834866
                                                                        // If the start date of the Step lies on a weekend then include the weekends so that the next contributor
                                                                        // gets the full duration of working days to complete the step.
                                                                        // This is done by getting the next working date in the variable TempStartdate following the weekend date.
                                                                        TempStartdate = GetStartDateFollowingWeekend(StartDateForNextAct);
                                                                        GetDueDate(TempStartdate, ActivityDuration, out DueDateForNextAct);
                                                                    }
                                                                    else
                                                                    {                                                                        
                                                                        DueDateForNextAct = StartDateForNextAct;
                                                                    }
																}
																else
																{
																	// Case: StartActivtyPred_Flag == false and endActivity == true or
																	// Case: StartActivtyPred_Flag == true and endActivity == true
																	DueDateForNextAct = StartDateForNextAct;
																}                                                           
															}
															if (IsSubFlowActivity)
															{
																ErrorXml = NewCalculateTimeLine(SubflowProcessNav.Value, StartDateForNextAct, str_FilePath, out TotalProcessDuration, out UpdatedProcessDueDate);
																if(ErrorXml != null)
																	return ErrorXml;
																ActivityDuration = TotalProcessDuration ;
																ProcessDuration += Convert.ToInt64(TotalProcessDuration);															
																DueDateForNextAct = UpdatedProcessDueDate;															
															}
															if ((ActivityStatus.ToUpper()=="COMPLETED")||(ActivityStatus.ToUpper()=="CANCELLED"))
															{
																if((ActEndDateNav != null) && (ActEndDateNav.HasChildren))
																{
																	Day     = Convert.ToInt32(ActEndDateNav.SelectSingleNode("DAY").Value);
																	Month   = Convert.ToInt32(ActEndDateNav.SelectSingleNode("MONTH").Value);
																	Year    = Convert.ToInt32(ActEndDateNav.SelectSingleNode("YEAR").Value);
                                                                    ActualEndDate = new DateTime(Year, Month, Day);
																}
															}
															else
															{
																ActualEndDate = new DateTime(1,1,1);
															}

                                                            NewActivityObj = new Activity(ActivityID, StartDateForNextAct, DueDateForNextAct, ActualEndDate, ActivityInfoID, ActivityDuration, ActivityStatus);
															NewActivityObj.Modified = ModifiedFlag;
															ActivityList.Add(NewActivityObj);
														}
														else
														{
															// keep the activity in dependancy XML
                                                            ActDepObj = new ActivityDependancy(ActivityID, ActivityIDOfMaxDate, ActivityInfoID, ActivityDuration, ActivityStatus);
                                                            ActivityDepQ.Enqueue(ActDepObj);
														}                                                
													}// END IF PREDECESSOR FOUND.
													else
													{
														// keep the activity in dependancy XML
                                                        ActDepObj = new ActivityDependancy(ActivityID, ActivityIDOfMaxDate, ActivityInfoID, ActivityDuration, ActivityStatus);
                                                        ActivityDepQ.Enqueue(ActDepObj);
													}
											}
										} // end while Vertices.Count
	                                    
										// Check if the dependency xml has and dependent Activities.If Yes then Update the CreatedActivityXML
                                        if (ActivityDepQ.Count > 0)
										{
                                            UpdateDependantActivities(ProcessNav, str_FilePath, ActivityList, ActivityDepQ);
										}
										if ((ProcessStatus.ToUpper()!="COMPLETED")&&(ProcessStatus.ToUpper()!="CANCELLED"))
											SortDueDates(ActivityList);
									} // end If Vertices.Count
								}
							}
							
						}// end if ActivityNav                
						ActivityListArr = ActivityList.ToArray();
						UpdatedProcessDueDate = ActivityListArr[ActivityListArr.Length - 1].DueDate;

						TotalProcessDuration = ProcessDuration.ToString();						
						ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- Before  UpdateProcessAndActivityDates() function.");
                        UpdateXML = UpdateProcessAndActivityDates(ActivityList, ProcessInstanceID, ProcessInfoID, TotalProcessDuration, StartDate, UpdatedProcessDueDate, str_ProcessStatus);
						if(UpdateXML != null)
						{
							OMI = new OMInterface(UserID, CurrentLanguage, TenantName);
							ErrorXml = OMI.Execute(ref UpdateXML);
							if(ErrorXml != null)
							{
								ErrorXml = PrepareErrorXML(ErrorXml, ErrorCodes.PROCESS_MODIFICATION_FAILED, ErrorObjectType.PROCESS);
								return ErrorXml;
							}						
						}
						ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- After UpdateProcessAndActivityDates() function.");
						return ErrorXml;																
					}
					else
					{
						TotalProcessDuration = ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/DURATION").Value;
						ActEndDateNav = ProcessNav.SelectSingleNode("ACTUALENDDATE");
						if((ActEndDateNav != null) && (ActEndDateNav.HasChildren))
						{
							Day     = Convert.ToInt32(ActEndDateNav.SelectSingleNode("DAY").Value);
							Month   = Convert.ToInt32(ActEndDateNav.SelectSingleNode("MONTH").Value);
							Year    = Convert.ToInt32(ActEndDateNav.SelectSingleNode("YEAR").Value);
                            UpdatedProcessDueDate = new DateTime(Year, Month, Day);
						}	
					}					
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Process Manager (NewCalculateTimeLine):- leaving NewCalculateTimeLine() function.");
					return ErrorXml;
                }
                else
                {
					TotalProcessDuration  = null;
					return ErrorXml;	
                }				
			}
			catch(Exception ExceptionObj)
			{
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("ProcessID = " + ProcessInstanceID + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                             "Exception in NewCalculateTimeLine().", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExceptionObj, sb_InputData.ToString());
                ErrorXml = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXml);
                UpdatedProcessDueDate = new DateTime();
                TotalProcessDuration = null;
                return ErrorXml;
			}
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ProcessInstanceID"></param>
        /// <param name="StartDate"></param>
        /// <param name="TotalProcessDuration"></param>
        /// <param name="UpdatedProcessDueDate"></param>
        /// <param name="ProcessNav"></param> 
        /// <returns></returns>
        public ERRORXML NewCalculateTimeLine_ForCreateDeliverable(String ProcessInstanceID, DateTime StartDate, XPathNavigator ProcessNav, out String TotalProcessDuration,out DateTime UpdatedProcessDueDate)
		{
			XPathNavigator	ProcessActivityNav			= null;
			XPathNavigator	ProcessTransitionNav		= null;
			XPathNavigator	ActivityNav					= null;
			XPathNavigator	StartDateNav				= null;
			XPathNavigator	DueDateNav					= null;
			XPathNavigator	ActEndDateNav				= null;	
			XPathNavigator	ActivityStatusNav			= null;	
			XPathNavigator	TransitionNav				= null;
			XPathNavigator	SubflowProcessNav			= null;
			XPathNodeIterator	TransitionNodeIter		= null;			
			XML				UpdateXML					= null;			
			ERRORXML		ErrorXml					= null;
			StringBuilder	StartActivity_Succ			= null;			
			String			ProcessInfoID				= null;
			String			ActivityInfoID				= null;
			String			ProcessStatus				= null;
			String			ActivityDuration			= null;
			String			StartActivityID				= null;
			String			ToActivityID				= null;
			String			StartActivity_Successors	= null;
			String			ActivityID					= null;
			String			ActivityStatus				= null;
			String			EndActivity					= null;
			String			PredecessorList				= null;
			String			ActivityIDOfMaxDate			= null;
			String			PredActStatus				= null;			
            String          str_ProcessStatus           = null; 
			String[]		PredecessorListArr          = null;
			List<Activity>	ActivityList				= null;			
			Activity[]		ActivityListArr				= null;
			Activity		ActivityObj					= null;
			Activity		NewActivityObj				= null;
			Queue			Vertices					= null;
            Hashtable		VisitedVertices				= null;
            bool			StartActivtyPred_Flag       = false;
            bool			PredecessorFound            = true;
            bool			IsSubFlowActivity           = false;
            bool			ModifiedFlag				= false;
            bool			ProcessStartDateLocked		= false;
            bool			ActivityStartDateLocked		= false;
            OMInterface		OMI							= null;
			DateTime		ActivityStartDate			 ;
			DateTime		ActualEndDate				;
			DateTime		ActivityDueDate				;
			DateTime		PredecessorActDate			;
            DateTime		StartDateForNextAct			;
            DateTime		DueDateForNextAct			;
            DateTime        Org_ProcessStartDate        ;
            DateTime		Org_ProcessDueDate			;            
			long			ProcessDuration				= 0;
			int				Day							= 0;
            int				Month						= 0;
            int				Year						= 0;
			Queue<ActivityDependancy>	ActivityDepQ = null;
			ActivityDependancy			ActDepObj		= null;
            ALError                     ExcepErrObj     = null;
            StringBuilder               sb_InputData    = null;
            
			try
			{
				UpdatedProcessDueDate	= new DateTime();
				DueDateForNextAct		= new DateTime();
				ActualEndDate			= new DateTime(1,1,1);
				ActivityList			= new List<Activity>();
                ActivityDepQ            = new Queue<ActivityDependancy>();
                OMI                     = new OMInterface(UserID, CurrentLanguage, TenantName);   
               
                ProcessDuration = 0;
                ProcessNav = ProcessNav.SelectSingleNode("descendant::WORKFLOWPROCESSID[text()= '" + ProcessInstanceID + "']");
                ProcessNav.MoveToParent(); 

				ProcessStatus    = ProcessNav.SelectSingleNode("STATUS").Value;						
				if ((ProcessStatus.ToUpper()!="COMPLETED")&&(ProcessStatus.ToUpper()!="CANCELLED"))
				{
                    str_ProcessStatus   = ProcessNav.SelectSingleNode("STATUS").Value; 
					ProcessInfoID		= ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").Value;
					ProcessStartDateLocked = (ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/STARTDATELOCKED").Value == "0" ? false : true);
					ProcessActivityNav	= ProcessNav.SelectSingleNode("PROCESS_ACTIVITY");					

            		Year  = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/YEAR").Value);
                    Month = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/MONTH").Value);
                    Day   = Convert.ToInt32(ProcessNav.SelectSingleNode("STARTDATE/DAY").Value);
                    Org_ProcessStartDate = new DateTime(Year, Month, Day);

					if(StartDate.Equals(new DateTime(1, 1, 1)) || (ProcessStartDateLocked == true))
						StartDate = Org_ProcessStartDate;
				
					if(ProcessActivityNav.Select("ACTIVITY").Count == 2)
					{
						DueDateNav = ProcessNav.SelectSingleNode("DUEDATE");
						if((DueDateNav != null) && (DueDateNav.HasChildren))
						{
							Year  = Convert.ToInt32(DueDateNav.SelectSingleNode("YEAR").Value);
                            Month = Convert.ToInt32(DueDateNav.SelectSingleNode("MONTH").Value);
                            Day   = Convert.ToInt32(DueDateNav.SelectSingleNode("DAY").Value);
                            Org_ProcessDueDate = new DateTime(Year, Month, Day);
							ErrorXml = UpdateProcessWithOnlyStartAndEndActivity(ProcessNav, StartDate, out TotalProcessDuration, out UpdatedProcessDueDate);
							return ErrorXml;
						}
					}
				
					ProcessTransitionNav = ProcessNav.SelectSingleNode("PROCESS_TRANSITION");
				
					ActivityNav			= ProcessActivityNav.SelectSingleNode("ACTIVITY[STARTACTIVITY/text()='True']");
					if (ActivityNav!=null)
					{
						StartActivityID = ActivityNav.SelectSingleNode("ACTIVITYID").Value;
						StartDateNav = ActivityNav.SelectSingleNode("STARTDATE");
						DueDateNav	 = ActivityNav.SelectSingleNode("DUEDATE");
						ActivityStatusNav = ActivityNav.SelectSingleNode("STATUS");
						
						Day			 = Convert.ToInt32(StartDateNav.SelectSingleNode("DAY").Value);
						Month		 = Convert.ToInt32(StartDateNav.SelectSingleNode("MONTH").Value);
						Year		 = Convert.ToInt32(StartDateNav.SelectSingleNode("YEAR").Value);
                        ActivityStartDate = new DateTime(Year, Month, Day);
						if (StartDate!=ActivityStartDate)
							ActivityStartDate = StartDate;

						ActivityDueDate = ActivityStartDate;
						ActivityInfoID	 = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID").Value;
						ActivityDuration = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value;
						ActivityStartDateLocked = (ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value == "0" ? false : true);
						ActivityDuration = "0";
						if(((ActivityStatusNav.Value.ToUpper() == "NEW") || ((ActivityStatusNav.Value.ToUpper() == "DRAFT")))&& (ActivityStartDateLocked == false))
							ModifiedFlag = true;
						else
							ModifiedFlag = false;

						if((ActivityStatusNav.Value.ToUpper() == "COMPLETED") || (ActivityStatusNav.Value.ToUpper() == "CANCELLED"))
						{
							ActEndDateNav = ActivityNav.SelectSingleNode("ACTUALENDDATE");
							if((ActEndDateNav !=null)&&(ActEndDateNav.HasChildren))
							{
								Day			 = Convert.ToInt32(ActEndDateNav.SelectSingleNode("DAY").Value);
								Month		 = Convert.ToInt32(ActEndDateNav.SelectSingleNode("MONTH").Value);
								Year		 = Convert.ToInt32(ActEndDateNav.SelectSingleNode("YEAR").Value);
                                ActualEndDate = new DateTime(Year, Month, Day);
							}						
						}

                        ActivityObj = new Activity(StartActivityID, ActivityStartDate, ActivityDueDate, ActualEndDate, ActivityInfoID, ActivityDuration, ActivityStatusNav.Value);
						ActivityObj.Modified = ModifiedFlag;
						ActivityList.Add(ActivityObj);																
							
						VisitedVertices = new Hashtable();
						VisitedVertices.Add(StartActivityID, true);
						if (ProcessTransitionNav!=null)
						{		
							TransitionNodeIter = ProcessTransitionNav.Select("TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + StartActivityID + "']");
							if (TransitionNodeIter.Count > 0)
							{
								Vertices			= new Queue();
								StartActivity_Succ	= new StringBuilder();
								while(TransitionNodeIter.MoveNext())
								{
									TransitionNav = TransitionNodeIter.Current;
									ToActivityID = TransitionNav.SelectSingleNode("TOACTIVITY/ACTIVITY/ACTIVITYID").Value;
									Vertices.Enqueue(ToActivityID);
									StartActivity_Succ.AppendFormat(";{0}", ToActivityID);
								}// end while
								StartActivity_Succ.Append(";");
								StartActivity_Successors = StartActivity_Succ.ToString();
								if (Vertices.Count > 0)
								{
									while (Vertices.Count > 0)
									{
										ActivityID = (String)Vertices.Dequeue();                                        
										IsSubFlowActivity = false;
										ModifiedFlag      = false;
										if (!VisitedVertices.ContainsKey(ActivityID))
										{
												VisitedVertices.Add(ActivityID, true);
												ActivityNav = ProcessActivityNav.SelectSingleNode("ACTIVITY[ACTIVITYID/text()='" + ActivityID + "']");
												DueDateNav  = ActivityNav.SelectSingleNode("DUEDATE");
												ActivityStatus = ActivityNav.SelectSingleNode("STATUS").Value;
												ActEndDateNav  = ActivityNav.SelectSingleNode("ACTUALENDDATE");
												StartDateNav	= ActivityNav.SelectSingleNode("STARTDATE");
												if ((StartDateNav!=null)&&(StartDateNav.HasChildren))
												{
													Day			 = Convert.ToInt32(StartDateNav.SelectSingleNode("DAY").Value);
													Month		 = Convert.ToInt32(StartDateNav.SelectSingleNode("MONTH").Value);
													Year		 = Convert.ToInt32(StartDateNav.SelectSingleNode("YEAR").Value);
                                                    ActivityStartDate = new DateTime(Year, Month, Day);
												}
												SubflowProcessNav = ActivityNav.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS/WORKFLOWPROCESSID");
												if(SubflowProcessNav != null)
													IsSubFlowActivity = true;

												if(((ActivityStatus.ToUpper() == "NEW") || ((ActivityStatus.ToUpper() == "DRAFT")) || (IsSubFlowActivity) || (DueDateNav.HasChildren == false) || (ActivityStartDateLocked == false)) && ((ActivityStatus.ToUpper() != "COMPLETED") && (ActivityStatus.ToUpper() != "CANCELLED")))
													ModifiedFlag = true;
												ActivityInfoID	= ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID").Value;
												EndActivity		= ActivityNav.SelectSingleNode("ENDACTIVITY").Value;
												// select the due date and start date nodes												
												ActivityDuration = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value;
												ActivityStartDateLocked = (ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value == "0" ? false : true);
												if((EndActivity.ToUpper() != "TRUE") & (!IsSubFlowActivity))
													ProcessDuration += Convert.ToInt64(ActivityDuration);
												TransitionNodeIter = ProcessTransitionNav.Select("TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityID + "']");
												while(TransitionNodeIter.MoveNext())
												{
													TransitionNav = TransitionNodeIter.Current;
													ToActivityID = TransitionNav.SelectSingleNode("TOACTIVITY/ACTIVITY/ACTIVITYID").Value;
													Vertices.Enqueue(ToActivityID);
												}// end while
												StartActivtyPred_Flag = StartActivity_Successors.Contains(";" + ActivityID + ";");
												if(StartActivtyPred_Flag == false)
												{
													// Find the max duration of the predecessors of this activity.													
													GetPredecessorsAndActivityIDOfMaxDueDate(ActivityList, ActivityID, ProcessNav, out PredecessorList, out ActivityIDOfMaxDate);
													if((PredecessorList != null) && (PredecessorList != ""))
													{
														PredecessorListArr = PredecessorList.Split(';');
														PredecessorFound = true;
														for(int Index = 0; Index < PredecessorListArr.Length; Index++)
														{
															if(PredecessorListArr[Index] != "")
															{
																if(!VisitedVertices.ContainsKey(PredecessorListArr[Index]))
																{
																	PredecessorFound = false;
																	break;
																}
															}
														}
													}
													else
														PredecessorFound = false;
												}
												else
												{
													ActivityIDOfMaxDate = StartActivityID;
												}
												
												if (PredecessorFound)
												{
													ActivityFilter AF = new ActivityFilter(ActivityIDOfMaxDate);
													Predicate<Activity> FindActivityFromList = null;
													FindActivityFromList = new Predicate<Activity>(AF.FilterByActivityID);
													
													ActivityObj = ActivityList.Find(FindActivityFromList);
													if (ActivityObj!=null)
													{
														PredActStatus = ProcessActivityNav.SelectSingleNode("ACTIVITY[ACTIVITYID='"+ActivityObj.ActivityID+"']/STATUS").Value;
														if((PredActStatus.ToUpper() == "COMPLETED") || (PredActStatus.ToUpper() == "CANCELLED"))
															PredecessorActDate = ActivityObj.ActualEndDate;
														else															
															PredecessorActDate = ActivityObj.DueDate;
													
														if ((ActivityStartDateLocked == true)&&((StartDateNav!=null)&&(StartDateNav.HasChildren)))
															StartDateForNextAct = ActivityStartDate;
														else
														{	
															if((StartActivtyPred_Flag == false) && (EndActivity.ToUpper() == "FALSE"))
															{													
																long Min = ConvertDaysToMinutes(2);
																GetDueDate(PredecessorActDate, Min.ToString(), out StartDateForNextAct);
															}
															else
															{
																StartDateForNextAct = PredecessorActDate;
															}
														}
														if (((StartActivtyPred_Flag == false) && (EndActivity.ToUpper() == "FALSE")))
														{
															// Case: StartActivtyPred_Flag == false and endActivity == false
															if((ActivityDuration != "") || ActivityDuration != null)
															{
																if(ActivityDuration != "0")
																{
																	GetDueDate(StartDateForNextAct, ActivityDuration, out DueDateForNextAct);
																}
																else
																	DueDateForNextAct = StartDateForNextAct;                                                                
															}
														}
														else
														{
															if (EndActivity.ToUpper() == "FALSE")
															{
																// Case: StartActivtyPred_Flag == true and endActivity == false																
																if(ActivityDuration != "0")
																{																	
																	GetDueDate(StartDateForNextAct, ActivityDuration, out DueDateForNextAct);
																}
																else
																	DueDateForNextAct = StartDateForNextAct;
															}
															else
															{
																// Case: StartActivtyPred_Flag == false and endActivity == true or
																// Case: StartActivtyPred_Flag == true and endActivity == true
																DueDateForNextAct = StartDateForNextAct;
															}                                                           
														}
														if (IsSubFlowActivity)
														{
//															UpdateActDuration = true;
                                                            String TempSubflowID = null;
                                                            TempSubflowID = SubflowProcessNav.Value;  
                                                            SubflowProcessNav.MoveToParent();
															ErrorXml = NewCalculateTimeLine_ForCreateDeliverable(TempSubflowID, StartDateForNextAct, SubflowProcessNav, out TotalProcessDuration, out UpdatedProcessDueDate);
															if(ErrorXml != null)
																return ErrorXml;
															ActivityDuration = TotalProcessDuration ;
															ProcessDuration += Convert.ToInt64(TotalProcessDuration);															
															DueDateForNextAct = UpdatedProcessDueDate;															
														}
														if ((ActivityStatus.ToUpper()=="COMPLETED")||(ActivityStatus.ToUpper()=="CANCELLED"))
														{
															if((ActEndDateNav != null) && (ActEndDateNav.HasChildren))
															{
																Day     = Convert.ToInt32(ActEndDateNav.SelectSingleNode("DAY").Value);
																Month   = Convert.ToInt32(ActEndDateNav.SelectSingleNode("MONTH").Value);
																Year    = Convert.ToInt32(ActEndDateNav.SelectSingleNode("YEAR").Value);
                                                                ActualEndDate = new DateTime(Year, Month, Day);
															}
														}
														else
														{
															ActualEndDate = new DateTime(1,1,1);
														}

                                                        NewActivityObj = new Activity(ActivityID, StartDateForNextAct, DueDateForNextAct, ActualEndDate, ActivityInfoID, ActivityDuration, ActivityStatus);
														NewActivityObj.Modified = ModifiedFlag;
														ActivityList.Add(NewActivityObj);
													}
													else
													{
														// keep the activity in dependancy XML
                                                        ActDepObj = new ActivityDependancy(ActivityID, ActivityIDOfMaxDate, ActivityInfoID, ActivityDuration, ActivityStatus);
														//ActivityDepList.Add(ActDepObj);
                                                        ActivityDepQ.Enqueue(ActDepObj);
													}                                                
												}// END IF PREDECESSOR FOUND.
												else
												{
													// keep the activity in dependancy XML
                                                    ActDepObj = new ActivityDependancy(ActivityID, ActivityIDOfMaxDate, ActivityInfoID, ActivityDuration, ActivityStatus);
													//ActivityDepList.Add(ActDepObj);
                                                    ActivityDepQ.Enqueue(ActDepObj);
												}
										}
									} // end while Vertices.Count
                                    
									// Check if the dependency xml has and dependent Activities.If Yes then Update the CreatedActivityXML
                                    if (ActivityDepQ.Count > 0)
									{
                                        UpdateDependantActivities(ProcessNav, null, ActivityList, ActivityDepQ);
									}
									if ((ProcessStatus.ToUpper()!="COMPLETED")&&(ProcessStatus.ToUpper()!="CANCELLED"))
										SortDueDates(ActivityList);
								} // end If Vertices.Count
							}
						}
						
					}// end if ActivityNav                
					ActivityListArr = ActivityList.ToArray();
					UpdatedProcessDueDate = ActivityListArr[ActivityListArr.Length - 1].DueDate;

					TotalProcessDuration = ProcessDuration.ToString();
                    UpdateXML = UpdateProcessAndActivityDates(ActivityList, ProcessInstanceID, ProcessInfoID, TotalProcessDuration, StartDate, UpdatedProcessDueDate, str_ProcessStatus);
					if(UpdateXML != null)
					{
						OMI = new OMInterface(UserID, CurrentLanguage, TenantName);
						ErrorXml = OMI.Execute(ref UpdateXML);
						if(ErrorXml != null)
						{
							ErrorXml = PrepareErrorXML(ErrorXml, ErrorCodes.PROCESS_MODIFICATION_FAILED, ErrorObjectType.PROCESS);
							return ErrorXml;
						}						
					}
					return ErrorXml;																
				}
				else
				{
					TotalProcessDuration = ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/DURATION").Value;
					ActEndDateNav = ProcessNav.SelectSingleNode("ACTUALENDDATE");
					if((ActEndDateNav != null) && (ActEndDateNav.HasChildren))
					{
						Day     = Convert.ToInt32(ActEndDateNav.SelectSingleNode("DAY").Value);
						Month   = Convert.ToInt32(ActEndDateNav.SelectSingleNode("MONTH").Value);
						Year    = Convert.ToInt32(ActEndDateNav.SelectSingleNode("YEAR").Value);
                        UpdatedProcessDueDate = new DateTime(Year, Month, Day);
					}	
				}					
				return ErrorXml;
			}
			catch(Exception ExceptionObj)
			{
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("ProcessID = " + ProcessInstanceID + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                             "Exception in NewCalculateTimeLine_ForCreateDeliverable().", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExceptionObj, sb_InputData.ToString());
                ErrorXml = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXml);
                UpdatedProcessDueDate = new DateTime();
                TotalProcessDuration = null; 
                return ErrorXml;
			}
		}
        
		/// <summary>
		/// 
		/// </summary>
		/// <param name="PackageDefinitionID"></param>
		/// <param name="ProcessDefinitionID"></param>
		/// <param name="InputXML"></param>
		/// <param name="ProcessInstanceXML"></param>
		/// <param name="InstantiateRDF"></param>
		/// <param name="XpdlDoc"></param>
        /// <param name="str_SolutionName"></param>
		/// <returns></returns>
		public ERRORXML CreateProcess(String PackageDefinitionID, String ProcessDefinitionID,Boolean InstantiateRDF, String str_SolutionName, out XmlDocument XpdlDoc, ref XML InputXML, out XML ProcessInstanceXML)
		{			
			XmlDocument			ReturnOutputDocument		= null;
			XPathDocument		InputDocument				= null;
			XML					ProcessDefinitionXML		= null;			
			XML					ProcessInfoXML				= null;
			XML					OperationXML				= null;
			ERRORXML            ErrorXML                    = null;
			XML					RDFXML						= null;
			XPathNavigator      XPDLProcessNavigator        = null; 
			XPathNavigator		XPDLNavigator				= null;
			XPathNavigator		ProcessInstanceNavigator	= null;	
			XPathNavigator		InputDocumentNavigator		= null;		
			XmlNode				ProcessInstanceNode			= null;
			String				XpathQuery					= null;
			StringBuilder		OperationInputXML			= null;
			OMInterface         OMIObj                      = null;
			StringReader		InputDataReader				= null;
			String				ProcessInfoID				= null;
			DateTime			ProcessStartDate			;
            Boolean             b_AddPCAUGNotificationEvent = false;
            //MediaRequestRouter  Obj_MediaRequestRouter      = null;
            EventMessageEngine  Obj_EventEngine     = null;
            String              EventID                     = null;
            //ulong               ulong_RequestID             = 0;
            ALError             ExcepErrObj                 = null;
            StringBuilder       sb_InputData                = null;
			
			try
			{
				OperationInputXML = new StringBuilder(1000);
				ProcessInstanceXML = null;

				if(InputXML != null)
				{
					InputDataReader = new StringReader(InputXML);
					InputDocument = new XPathDocument(InputDataReader);
					InputDocumentNavigator = InputDocument.CreateNavigator();
					XpathQuery = "WORKFLOWPROCESS[@ProcessDefinitionID='" + ProcessDefinitionID + "']";
					ProcessInstanceNavigator = InputDocumentNavigator.SelectSingleNode(XpathQuery);    
				}

				OMIObj			  = new OMInterface(UserID,CurrentLanguage,TenantName);
				ErrorXML = GetProcessDefinitionByID(Convert.ToUInt64(PackageDefinitionID), ProcessDefinitionID, out ProcessDefinitionXML);
				if (ErrorXML == null)
                {
					XpdlDoc = new XmlDocument();
					XpdlDoc.LoadXml(ProcessDefinitionXML);
					XPDLNavigator = XpdlDoc.CreateNavigator();
					XpathQuery = "WORKFLOWPROCESS[WORKFLOWPROCESSID='" + ProcessDefinitionID + "']";
					XPDLProcessNavigator = XPDLNavigator.SelectSingleNode(XpathQuery);
                    ErrorXML = CreateProcess_ProcessInfoXML(PackageDefinitionID, ProcessDefinitionID, XPDLProcessNavigator, ProcessInstanceNavigator, str_SolutionName, out ProcessStartDate, out ProcessInfoXML, out b_AddPCAUGNotificationEvent);
					if (ErrorXML==null)
					{						
						OperationInputXML.AppendFormat(@"<INPUTXML>
							                                <TRANSACTION>TRUE</TRANSACTION>
							                                <OPERATION>
								                                <OBJECTNAME>ENTITY</OBJECTNAME>
								                                <METHOD>CREATEDATA</METHOD>
								                                <PARAM Name='INPUTXML'>{0}</PARAM>
							                                </OPERATION>
						                                </INPUTXML>", ProcessInfoXML);
						OperationXML = OperationInputXML.ToString();
						ErrorXML = OMIObj.Execute(ref OperationXML);
                        if (String.IsNullOrEmpty(ErrorXML))
                        {
                            //Check if notification is to be send to Process Complete Access User(s).
                            if (b_AddPCAUGNotificationEvent)
                            {
                                //Add event in the message queue.
                                OperationInputXML.Remove(0, OperationInputXML.Length);
                                OperationInputXML.Append(@"<EVENT TYPE='SENDNOTIFICATION'>
                                                            <ACTION>PCAUG</ACTION>
                                                          </EVENT>");
                                String temp = OperationXML;
                                int i_startIndex = temp.IndexOf("<WORKFLOWPROCESSID>") ;
                                int i_EndIndex = temp.LastIndexOf("</WORKFLOWPROCESSID>") + "</WORKFLOWPROCESSID>".Length;

                                temp = temp.Substring(i_startIndex, i_EndIndex-i_startIndex);
                                temp += "<PROCESSDEFID>" + ProcessDefinitionID + "</PROCESSDEFID>";
                                temp += "<PACKAGEDEFID>" + PackageDefinitionID + "</PACKAGEDEFID>";
                                if (String.Compare(str_SolutionName, "Marketing Workbench",true)==0)
                                {
                                    str_SolutionName = "MarketingWorkbench";
                                }
                                Obj_EventEngine = new EventMessageEngine(SessionID, UserID, CurrentLanguage, TenantName);
                                ErrorXML = Obj_EventEngine.StampEvent(str_SolutionName, "SENDNOTIFICATION", OperationInputXML.ToString(), temp, out EventID);
                                //Obj_MediaRequestRouter = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), CurrentLanguage, str_SolutionName);
                                //UniqueID.Read(ref ulong_RequestID);
                                //ErrorXML = Obj_MediaRequestRouter.SendRequestToMediaProcessor(OperationInputXML.ToString(), "GenericEvent", ulong_RequestID.ToString(), temp);
                            }
                        }

						if(InstantiateRDF)
						{
                            ReturnOutputDocument    = new XmlDocument();
                            ReturnOutputDocument.LoadXml(OperationXML);
                            ProcessInstanceNode     = ReturnOutputDocument.SelectSingleNode("INPUTXML/OPERATION/PARAM/WORKFLOWPROCESS");
							ProcessInfoID = ProcessInstanceNode.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").InnerText;
							RDFXML = GetRDFXMLWithUpdatedPrimaryKeyValues(ProcessInstanceNode);
                            
                            
							if((!String.IsNullOrEmpty(RDFXML)) && (!String.IsNullOrEmpty(ProcessInfoXML)))
							{
								ProcessInfoXML = "<WORKFLOWPROCESSINFO MODIFIED='TRUE'><WORKFLOWPROCESSINFOID>" + ProcessInfoID + "</WORKFLOWPROCESSINFOID><RELEVANTDATAFIELDS>" + RDFXML + "</RELEVANTDATAFIELDS></WORKFLOWPROCESSINFO>";
								OperationInputXML.Remove(0, OperationInputXML.Length);
                                OperationInputXML.Append(@"<INPUTXML>
							                                <TRANSACTION>TRUE</TRANSACTION>
							                                <OPERATION>
								                                <OBJECTNAME>FACET</OBJECTNAME>
								                                <METHOD>UPDATE</METHOD>
								                                <PARAM Name='INPUTXML'>" + ProcessInfoXML + @"</PARAM>
							                                </OPERATION>
						                                </INPUTXML>");
								OperationXML = OperationInputXML.ToString();
								ErrorXML = OMIObj.Execute(ref OperationXML);
								if(ErrorXML == null)
								{
									// update the rdf node.
									ProcessInstanceNode.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/RELEVANTDATAFIELDS").InnerXml = RDFXML;
								}
							}
							ProcessInstanceXML = ProcessInstanceNode.OuterXml;
						}
						else
						{
							if(ErrorXML == null)
							{
								int startindex = 0;
								int endindex = 0;
								startindex = OperationXML.IndexOf("<WORKFLOWPROCESS>");
								endindex = OperationXML.IndexOf("</PARAM>");
								ProcessInstanceXML = OperationXML.Substring(startindex, endindex - startindex);
							}
						}
					}
                }
                else
                {
					XpdlDoc				= null;
					ProcessInstanceXML  = null;
				}
                return ErrorXML;
			}
			catch(Exception ExcepObj)
			{
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("PackageDefinitionID = " + PackageDefinitionID + " ProcessDefinitionID = " + ProcessDefinitionID + " InputXML = " + InputXML + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_CREATION_FAILED,
                                             "Exception while creating Process.", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExcepObj, sb_InputData.ToString());
                ErrorXML = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                XpdlDoc = new XmlDocument();
                ProcessInstanceXML = null; 
                return ErrorXML;
			}
		}

		/// <summary>
		/// <createddate></createddate>
		/// </summary>
		/// <param name="ProcessID"></param>
		/// <param name="ProcessInfoID"></param>
		/// <param name="ProcessDuration"></param>
		/// <returns></returns>
		public ERRORXML UpdateProcessDuration(String ProcessID, String ProcessInfoID, String ProcessDuration)
		{
			StringBuilder	ProcessXML				= null;
			OMInterface		OMIObj                  = null;
            OperationQueue	QMQueueObj              = null;
            Operation       OperObj                 = null;
			XML				TempXML					= null;
			ERRORXML		ErrorXml				= null;
			StringBuilder	SearchXML				= null;
			XML				ResultXML				= null;
			XPathDocument	ProcessDoc				= null;
			XPathNavigator	xpNav					= null;
            ALError         ExcepErrObj             = null;
            StringBuilder   sb_InputData            = null;

			try
			{
				ProcessXML  = new StringBuilder(250);
				OperObj		= new Operation("ENTITY", "UPDATEDATA");
				QMQueueObj	= new OperationQueue(false);
				OMIObj		= new OMInterface(UserID,CurrentLanguage,TenantName);

				if(ProcessInfoID == null)
				{
					// get the process information.
					SearchXML = new StringBuilder(250);
					SearchXML.AppendFormat(@"<SEARCH>
	                                            <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
	                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                            <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                            <TARGETENTITY>
		                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <SELECTLIST>
													    <EXPRESSION>
			                                                <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
			                                                <ALIAS>WORKFLOWPROCESSID</ALIAS>
		                                                </EXPRESSION>		                                              
	                                                </SELECTLIST>		                                            
		                                            <RELATIONSHIP NAME='PROCESS_INFORMATION'>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
					                                            <ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
				                                            </EXPRESSION>
														</SELECTLIST>	                                           
		                                            </RELATIONSHIP>
		                                            <FILTERLIST>
			                                            <FILTER>
				                                            <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
				                                            <OPERATOR>EQUALTO</OPERATOR>
				                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
				                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            <FILTERID>1</FILTERID>
			                                            </FILTER>
		                                            </FILTERLIST>
	                                            </TARGETENTITY>
                                            </SEARCH>", ProcessID);
					ErrorXml = OMIObj.GetHierarchialData(SearchXML.ToString(), ref ResultXML);
					if(ErrorXml != null)
						return ErrorXml;
					ProcessDoc = new XPathDocument(new StringReader(ResultXML));
					xpNav = ProcessDoc.CreateNavigator();
					ProcessInfoID = xpNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS/PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").Value;
				}

				ProcessXML.AppendFormat(@"<WORKFLOWPROCESS MODIFIED='TRUE'>
											<WORKFLOWPROCESSID>{0}</WORKFLOWPROCESSID>
													<PROCESS_INFORMATION TYPE='RELATION'>
														<WORKFLOWPROCESSINFO MODIFIED='TRUE'>
															<WORKFLOWPROCESSINFOID>{1}</WORKFLOWPROCESSINFOID>
															<DURATION>{2}</DURATION>
														</WORKFLOWPROCESSINFO>
													</PROCESS_INFORMATION>
										</WORKFLOWPROCESS>", ProcessID, ProcessInfoID, ProcessDuration);
				OperObj.AddParameter("INPUTXML", ProcessXML.ToString());
				QMQueueObj.AddOperation(OperObj);
				TempXML = QMQueueObj.Xml;
				ErrorXml = OMIObj.Execute(ref TempXML);
				return ErrorXml;
			}
			catch(Exception ExcepObj)
			{
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("ProcessID = " + ProcessID + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                             "Exception while updating process duration.", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExcepObj, sb_InputData.ToString());
                ErrorXml = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXml);
                return ErrorXml;
			}
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
        private XML PrepareXMLForTimeLineGetProcess2Level(String ProcessIDs)
        {
			StringBuilder   AssociationXML          = null;

			AssociationXML = new StringBuilder(1000);
            AssociationXML.Append(@"<SEARCH>
                                      <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
                                      <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                      <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
                                      <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                      <TARGETENTITY>
                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                        <SELECTLIST>
                                          <EXPRESSION>
                                            <FIELDNAME>STARTDATE</FIELDNAME>
                                            <ALIAS>STARTDATE</ALIAS>
                                          </EXPRESSION>
                                          <EXPRESSION>
                                            <FIELDNAME>DUEDATE</FIELDNAME>
                                            <ALIAS>DUEDATE</ALIAS>
                                          </EXPRESSION>
                                          <EXPRESSION>
                                            <FIELDNAME>ACTUALSTARTDATE</FIELDNAME>
                                            <ALIAS>ACTUALSTARTDATE</ALIAS>
                                          </EXPRESSION>
                                          <EXPRESSION>
                                            <FIELDNAME>ACTUALENDDATE</FIELDNAME>
                                            <ALIAS>ACTUALENDDATE</ALIAS>
                                          </EXPRESSION>
                                          <EXPRESSION>
                                            <FIELDNAME>STATUS</FIELDNAME>
                                            <ALIAS>STATUS</ALIAS>
                                          </EXPRESSION>
                                          <EXPRESSION>
                                            <FIELDNAME>PROCESSNAME</FIELDNAME>
                                            <ALIAS>PROCESSNAME</ALIAS>
                                          </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='PROCESS_ACTIVITY'>
                                          <SELECTLIST>
                                            <EXPRESSION>
                                              <FIELDNAME>ACTIVITYNAME</FIELDNAME>
                                              <ALIAS>ACTIVITYNAME</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>STARTDATE</FIELDNAME>
                                              <ALIAS>STARTDATE</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>DUEDATE</FIELDNAME>
                                              <ALIAS>DUEDATE</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>STARTACTIVITY</FIELDNAME>
                                              <ALIAS>STARTACTIVITY</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>ENDACTIVITY</FIELDNAME>
                                              <ALIAS>ENDACTIVITY</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>STATUS</FIELDNAME>
                                              <ALIAS>STATUS</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>ACTUALENDDATE</FIELDNAME>
                                              <ALIAS>ACTUALENDDATE</ALIAS>
                                            </EXPRESSION>
                                          </SELECTLIST>
                                          <RELATIONSHIP NAME='ACTIVITY_INFORMATION'>
                                            <SELECTLIST>
                                              <EXPRESSION>
                                                <FIELDNAME>DURATION</FIELDNAME>
                                                <ALIAS>DURATION</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>STARTDATELOCKED</FIELDNAME>
                                                <ALIAS>STARTDATELOCKED</ALIAS>
                                              </EXPRESSION>
                                            </SELECTLIST>
                                          </RELATIONSHIP>
                                          <RELATIONSHIP NAME='SUBFLOWACTIVITY'>
                                            <SELECTLIST>
                                              <EXPRESSION>
                                                <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
                                                <ALIAS>WORKFLOWPROCESSID</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>STARTDATE</FIELDNAME>
                                                <ALIAS>STARTDATE</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>DUEDATE</FIELDNAME>
                                                <ALIAS>DUEDATE</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>ACTUALSTARTDATE</FIELDNAME>
                                                <ALIAS>ACTUALSTARTDATE</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>ACTUALENDDATE</FIELDNAME>
                                                <ALIAS>ACTUALENDDATE</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>STATUS</FIELDNAME>
                                                <ALIAS>STATUS</ALIAS>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <FIELDNAME>PROCESSNAME</FIELDNAME>
                                                <ALIAS>PROCESSNAME</ALIAS>
                                              </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='PROCESS_INFORMATION'>
                                              <SELECTLIST>
                                                <EXPRESSION>
                                                  <FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
                                                  <ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>DURATION</FIELDNAME>
                                                  <ALIAS>DURATION</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>STARTDATELOCKED</FIELDNAME>
                                                  <ALIAS>STARTDATELOCKED</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>PROCESSDEFINITIONID</FIELDNAME>
                                                  <ALIAS>PROCESSDEFINITIONID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>PACKAGEDEFINITIONID</FIELDNAME>
                                                  <ALIAS>PACKAGEDEFINITIONID</ALIAS>
                                                </EXPRESSION>
                                              </SELECTLIST>
                                            </RELATIONSHIP>
                                            <RELATIONSHIP NAME='PROCESS_TRANSITION'>
                                              <SELECTLIST>
                                                <EXPRESSION>
                                                  <FIELDNAME>TRANSITIONID</FIELDNAME>
                                                  <ALIAS>TRANSITIONID</ALIAS>
                                                </EXPRESSION>
                                              </SELECTLIST>
                                              <RELATIONSHIP NAME='FROMACTIVITY'>
                                                <SELECTLIST>
                                                  <EXPRESSION>
                                                    <FIELDNAME>ACTIVITYID</FIELDNAME>
                                                    <ALIAS>ACTIVITYID</ALIAS>
                                                  </EXPRESSION>
                                                </SELECTLIST>
                                              </RELATIONSHIP>
                                              <RELATIONSHIP NAME='TOACTIVITY'>
                                                <SELECTLIST>
                                                  <EXPRESSION>
                                                    <FIELDNAME>ACTIVITYID</FIELDNAME>
                                                    <ALIAS>ACTIVITYID</ALIAS>
                                                  </EXPRESSION>
                                                </SELECTLIST>
                                              </RELATIONSHIP>
                                            </RELATIONSHIP>
                                            <RELATIONSHIP NAME='PROCESS_ACTIVITY'>
                                              <SELECTLIST>
                                                <EXPRESSION>
                                                  <FIELDNAME>ACTIVITYNAME</FIELDNAME>
                                                  <ALIAS>ACTIVITYNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>STARTDATE</FIELDNAME>
                                                  <ALIAS>STARTDATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>DUEDATE</FIELDNAME>
                                                  <ALIAS>DUEDATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>STARTACTIVITY</FIELDNAME>
                                                  <ALIAS>STARTACTIVITY</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>ENDACTIVITY</FIELDNAME>
                                                  <ALIAS>ENDACTIVITY</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>STATUS</FIELDNAME>
                                                  <ALIAS>STATUS</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>ACTUALENDDATE</FIELDNAME>
                                                  <ALIAS>ACTUALENDDATE</ALIAS>
                                                </EXPRESSION>
                                              </SELECTLIST>
                                              <RELATIONSHIP NAME='ACTIVITY_INFORMATION'>
                                                <SELECTLIST>
                                                  <EXPRESSION>
                                                    <FIELDNAME>DURATION</FIELDNAME>
                                                    <ALIAS>DURATION</ALIAS>
                                                  </EXPRESSION>
                                                  <EXPRESSION>
                                                    <FIELDNAME>STARTDATELOCKED</FIELDNAME>
                                                    <ALIAS>STARTDATELOCKED</ALIAS>
                                                  </EXPRESSION>
                                                </SELECTLIST>
                                              </RELATIONSHIP>
                                              <RELATIONSHIP NAME='SUBFLOWACTIVITY'>
                                                <SELECTLIST>
                                                  <EXPRESSION>
                                                    <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
                                                    <ALIAS>WORKFLOWPROCESSID</ALIAS>
                                                  </EXPRESSION>              
                                                </SELECTLIST>            
                                              </RELATIONSHIP>
                                            </RELATIONSHIP>
                                          </RELATIONSHIP>
                                        </RELATIONSHIP>
                                        <RELATIONSHIP NAME='PROCESS_TRANSITION'>
                                          <SELECTLIST>
                                            <EXPRESSION>
                                              <FIELDNAME>TRANSITIONID</FIELDNAME>
                                              <ALIAS>TRANSITIONID</ALIAS>
                                            </EXPRESSION>
                                          </SELECTLIST>
                                          <RELATIONSHIP NAME='FROMACTIVITY'>
                                            <SELECTLIST>
                                              <EXPRESSION>
                                                <FIELDNAME>ACTIVITYID</FIELDNAME>
                                                <ALIAS>ACTIVITYID</ALIAS>
                                              </EXPRESSION>
                                            </SELECTLIST>
                                          </RELATIONSHIP>
                                          <RELATIONSHIP NAME='TOACTIVITY'>
                                            <SELECTLIST>
                                              <EXPRESSION>
                                                <FIELDNAME>ACTIVITYID</FIELDNAME>
                                                <ALIAS>ACTIVITYID</ALIAS>
                                              </EXPRESSION>
                                            </SELECTLIST>
                                          </RELATIONSHIP>
                                        </RELATIONSHIP>
                                        <RELATIONSHIP NAME='PROCESS_INFORMATION'>
                                          <SELECTLIST>
                                            <EXPRESSION>
                                              <FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
                                              <ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>DURATION</FIELDNAME>
                                              <ALIAS>DURATION</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>STARTDATELOCKED</FIELDNAME>
                                              <ALIAS>STARTDATELOCKED</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>PROCESSDEFINITIONID</FIELDNAME>
                                              <ALIAS>PROCESSDEFINITIONID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                              <FIELDNAME>PACKAGEDEFINITIONID</FIELDNAME>
                                              <ALIAS>PACKAGEDEFINITIONID</ALIAS>
                                            </EXPRESSION>
                                          </SELECTLIST>
                                        </RELATIONSHIP>
                                        <FILTERLIST>
                                          <FILTER>
                                            <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
                                            <OPERATOR>IN</OPERATOR>
                                            <EXPRESSIONVALUE>" + ProcessIDs + @"</EXPRESSIONVALUE>
                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                            <FILTERID>1</FILTERID>
                                          </FILTER>
                                        </FILTERLIST>
                                      </TARGETENTITY>
                                    </SEARCH>");

			return AssociationXML.ToString();
        }

        private XML PrepareXMLForTimeLineGetProcess(String ProcessID)
        {
            StringBuilder AssociationXML = null;

            AssociationXML = new StringBuilder(1000);

            AssociationXML.Append(@"<SEARCH>
                                        <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <SELECTLIST>
										        <EXPRESSION>
                                                    <FIELDNAME>STARTDATE</FIELDNAME>
                                                    <ALIAS>STARTDATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>DUEDATE</FIELDNAME>
                                                    <ALIAS>DUEDATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ACTUALSTARTDATE</FIELDNAME>
                                                    <ALIAS>ACTUALSTARTDATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ACTUALENDDATE</FIELDNAME>
                                                    <ALIAS>ACTUALENDDATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>STATUS</FIELDNAME>
                                                    <ALIAS>STATUS</ALIAS>
                                                </EXPRESSION>
	                                        </SELECTLIST>
                                            <RELATIONSHIP NAME='PROCESS_ACTIVITY'>
	                                            <SELECTLIST>
													<EXPRESSION>
			                                            <FIELDNAME>ACTIVITYNAME</FIELDNAME>
			                                            <ALIAS>ACTIVITYNAME</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>STARTDATE</FIELDNAME>
			                                            <ALIAS>STARTDATE</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>DUEDATE</FIELDNAME>
			                                            <ALIAS>DUEDATE</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>STARTACTIVITY</FIELDNAME>
			                                            <ALIAS>STARTACTIVITY</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>ENDACTIVITY</FIELDNAME>
			                                            <ALIAS>ENDACTIVITY</ALIAS>
		                                            </EXPRESSION>
		                                             <EXPRESSION>
			                                            <FIELDNAME>STATUS</FIELDNAME>
			                                            <ALIAS>STATUS</ALIAS>
		                                            </EXPRESSION>
													<EXPRESSION>
			                                            <FIELDNAME>ACTUALENDDATE</FIELDNAME>
			                                            <ALIAS>ACTUALENDDATE</ALIAS>
		                                            </EXPRESSION>
	                                            </SELECTLIST>
	                                            <RELATIONSHIP NAME='ACTIVITY_INFORMATION'>
                                                    <SELECTLIST>
													    <EXPRESSION>
			                                                <FIELDNAME>DURATION</FIELDNAME>
			                                                <ALIAS>DURATION</ALIAS>
		                                                </EXPRESSION>
		                                                <EXPRESSION>
			                                                <FIELDNAME>STARTDATELOCKED</FIELDNAME>
			                                                <ALIAS>STARTDATELOCKED</ALIAS>
		                                                </EXPRESSION>
	                                                </SELECTLIST>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='SUBFLOWACTIVITY'>
	                                                <SELECTLIST>
		                                                <EXPRESSION>
			                                                <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
			                                                <ALIAS>WORKFLOWPROCESSID</ALIAS>
		                                                </EXPRESSION>
				                                    </SELECTLIST>
				                                    <RELATIONSHIP NAME='PROCESS_INFORMATION'>
														<SELECTLIST>
															<EXPRESSION>
																<FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
																<ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
															</EXPRESSION>																    
														</SELECTLIST>
				                                    </RELATIONSHIP>
			                                    </RELATIONSHIP>
		                                    </RELATIONSHIP>
                                            <RELATIONSHIP NAME='PROCESS_TRANSITION'>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>TRANSITIONID</FIELDNAME>
			                                            <ALIAS>TRANSITIONID</ALIAS>
		                                            </EXPRESSION>
	                                            </SELECTLIST>
	                                            <RELATIONSHIP NAME='FROMACTIVITY'>
		                                            <SELECTLIST>
			                                            <EXPRESSION>
				                                            <FIELDNAME>ACTIVITYID</FIELDNAME>
				                                            <ALIAS>ACTIVITYID</ALIAS>
			                                            </EXPRESSION>
		                                            </SELECTLIST>
	                                            </RELATIONSHIP>
	                                            <RELATIONSHIP NAME='TOACTIVITY'>
		                                            <SELECTLIST>
			                                            <EXPRESSION>
				                                            <FIELDNAME>ACTIVITYID</FIELDNAME>
				                                            <ALIAS>ACTIVITYID</ALIAS>
			                                            </EXPRESSION>
		                                            </SELECTLIST>
	                                            </RELATIONSHIP>
                                            </RELATIONSHIP>
                                            <RELATIONSHIP NAME='PROCESS_INFORMATION'>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
			                                            <ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>DURATION</FIELDNAME>
			                                            <ALIAS>DURATION</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>STARTDATELOCKED</FIELDNAME>
			                                            <ALIAS>STARTDATELOCKED</ALIAS>
		                                            </EXPRESSION>				                                           
			                                    </SELECTLIST>
		                                    </RELATIONSHIP>
		                                    <FILTERLIST>
	                                            <FILTER>
		                                            <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
		                                            <OPERATOR>EQUALTO</OPERATOR>
		                                            <EXPRESSIONVALUE>" + ProcessID + @"</EXPRESSIONVALUE>
		                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
		                                            <FILTERID>1</FILTERID>
	                                            </FILTER>
                                            </FILTERLIST>
	                                    </TARGETENTITY>
                                    </SEARCH>");

            return AssociationXML.ToString();
        }


		private List<ActivityInfo> GetActivityInfoList(List<ActivityInfo> TempActivityInfoList, String Level)
		{
			List<ActivityInfo>ReturnActivityInfo = null;
			ActivityInfo AInfo = null;
			
				ReturnActivityInfo = new List<ActivityInfo>();
				for (int Index=0;Index<TempActivityInfoList.Count;Index++)
				{
					if (TempActivityInfoList[Index].LevelID == Level)
					{
						AInfo = new ActivityInfo(TempActivityInfoList[Index]);
						ReturnActivityInfo.Add(AInfo);
					}
				}		
				return 	ReturnActivityInfo;
		}								

		private List<ActivityInfo> ChangeActivityLevel(List<ActivityInfo> ActivityInfoObjList)
		{
			int Level;
			Level = Convert.ToInt32(ActivityInfoObjList[0].LevelID);
			Level--;
			for (int Index = 0;Index<ActivityInfoObjList.Count;Index++)
				ActivityInfoObjList[Index].LevelID = Level.ToString();
			return ActivityInfoObjList;
		}

		private ActivityInfo GetActivtiyInfoObj(List<ActivityInfo> ActivityInfoList, String SuccessorActivityID)
		{
			for(int Index=0;Index<ActivityInfoList.Count ; Index++)
			{
				if (ActivityInfoList[Index].ActivityID == SuccessorActivityID)
					return ActivityInfoList[Index];
			}
			return null;
		}

		private ERRORXML UpdateActivityForTimeLine(String ActivityID, DateTime StartDate, DateTime DueDate, String ActivityInfoID, String ProcessDuration, String ProcessInfoID, bool UpdateLock, String str_ActivityStatus)
		{
			OMInterface                 OMIObj                  = null;
            OperationQueue              QMQueueObj              = null;
            Operation                   OperObj                 = null;
			XML							TempXML					= null;
			StringBuilder				UpdateActivityXML		= null;
			ERRORXML					ErrorXML				= null;
			
				OperObj				= new Operation("ENTITY", "UPDATEDATA");
				QMQueueObj			= new OperationQueue(false);
				OMIObj				= new OMInterface(UserID,CurrentLanguage,TenantName);
				UpdateActivityXML	= new StringBuilder(500);
				if (!UpdateLock)
				{
                    UpdateActivityXML.Append(@"<ACTIVITY MODIFIED='TRUE'>
						                                <ACTIVITYID>" + ActivityID + "</ACTIVITYID>");
                    if ((!String.IsNullOrEmpty(str_ActivityStatus)) && ((str_ActivityStatus.ToUpper() == "NEW") || (str_ActivityStatus.ToUpper() == "DRAFT")))
                    {
                        UpdateActivityXML.Append(@"<STARTDATE>
				                                       <DAY>" + StartDate.Day.ToString() + "</DAY>"
                                                    + "<MONTH>" + StartDate.Month.ToString() + "</MONTH>"
                                                    + "<YEAR>" + StartDate.Year.ToString() + "</YEAR>"                            
                                                + "</STARTDATE>");

                        if (m_SetBaselineForCreateDeliverable == true)
                        {
                            UpdateActivityXML.Append(@"<BASESTARTDATE>
				                                       <DAY>" + StartDate.Day.ToString() + "</DAY>"
                                                        + "<MONTH>" + StartDate.Month.ToString() + "</MONTH>"
                                                        + "<YEAR>" + StartDate.Year.ToString() + "</YEAR>"
                                                    + @"</BASESTARTDATE>
                                                        <BASEENDDATE>
				                                           <DAY>" + DueDate.Day.ToString() + "</DAY>"
                                                        + "<MONTH>" + DueDate.Month.ToString() + "</MONTH>"
                                                        + "<YEAR>" + DueDate.Year.ToString() + "</YEAR>"
                                                    + "</BASEENDDATE>");
                        }
                    }

                    UpdateActivityXML.Append(@"<DUEDATE>
						                                <DAY>" + DueDate.Day.ToString() + "</DAY>"
                                                    + "<MONTH>" + DueDate.Month.ToString() + "</MONTH>"
                                                    + "<YEAR>" + DueDate.Year.ToString() + "</YEAR>"
                                                + @"</DUEDATE>
						                                <ACTIVITY_INFORMATION TYPE='RELATION'>
							                                <ACTIVITYINFORMATION MODIFIED='TRUE'>
								                                <ACTIVITYINFORMATIONID>" + ActivityInfoID + "</ACTIVITYINFORMATIONID>"
                                                            + "<DURATION>" + ProcessDuration + "</DURATION>"
                                                        + @"</ACTIVITYINFORMATION>
						                                </ACTIVITY_INFORMATION>
					                                </ACTIVITY>");
					// Update this activity
					OperObj.AddParameter("INPUTXML", UpdateActivityXML.ToString());
					QMQueueObj.AddOperation(OperObj);
					TempXML = QMQueueObj.Xml;		

				}
				else
				{
					UpdateActivityXML.Append(@"<INPUTXML>
				                                <OPERATION>
				                                <OBJECTNAME>ENTITY</OBJECTNAME>
				                                <METHOD>UPDATEDATA</METHOD>
				                                <PARAM Name='INPUTXML'>
					                                <ACTIVITY MODIFIED='TRUE'>
						                                <ACTIVITYID>"+ActivityID+"</ACTIVITYID>");
                    if ((!String.IsNullOrEmpty(str_ActivityStatus)) && ((str_ActivityStatus.ToUpper() == "NEW") || (str_ActivityStatus.ToUpper() == "DRAFT")))
                    {
                        UpdateActivityXML.Append(@"<STARTDATE>
							                           <DAY>" + StartDate.Day.ToString() + "</DAY>"
                                                    + "<MONTH>" + StartDate.Month.ToString() + "</MONTH>"
                                                    + "<YEAR>" + StartDate.Year.ToString() + "</YEAR>"
                                                + "</STARTDATE>");
                        if (m_SetBaselineForCreateDeliverable == true)
                        {
                            UpdateActivityXML.Append(@"<BASESTARTDATE>
				                                       <DAY>" + StartDate.Day.ToString() + "</DAY>"
                                                        + "<MONTH>" + StartDate.Month.ToString() + "</MONTH>"
                                                        + "<YEAR>" + StartDate.Year.ToString() + "</YEAR>"
                                                    + @"</BASESTARTDATE>
                                                        <BASEENDDATE>
				                                           <DAY>" + DueDate.Day.ToString() + "</DAY>"
                                                        + "<MONTH>" + DueDate.Month.ToString() + "</MONTH>"
                                                        + "<YEAR>" + DueDate.Year.ToString() + "</YEAR>"
                                                    + "</BASEENDDATE>");
                        }

                    }

                    UpdateActivityXML.Append(@"<DUEDATE>
									                    <DAY>" + DueDate.Day.ToString() + "</DAY>"
                                                    + "<MONTH>" + DueDate.Month.ToString() + "</MONTH>"
                                                    + "<YEAR>" + DueDate.Year.ToString() + "</YEAR>"                                                    
                                                + @"</DUEDATE>
								                                <ACTIVITY_INFORMATION TYPE='RELATION'>
									                                <ACTIVITYINFORMATION MODIFIED='TRUE'>
										                                <ACTIVITYINFORMATIONID>" + ActivityInfoID + "</ACTIVITYINFORMATIONID>"
                                                                        + "<DURATION>" + ProcessDuration + "</DURATION>"
                                                                        + @"<STARTDATELOCKED>0</STARTDATELOCKED>
									                                </ACTIVITYINFORMATION>
								                                </ACTIVITY_INFORMATION>
							                                </ACTIVITY>
						                                </PARAM>
					                                </OPERATION>
					                                <OPERATION>
						                                <OBJECTNAME>FACET</OBJECTNAME>
						                                <METHOD>UPDATE</METHOD>
						                                <PARAM Name='INPUTXML'>
							                                <WORKFLOWPROCESSINFO MODIFIED='TRUE'>
								                                <WORKFLOWPROCESSINFOID>" + ProcessInfoID + "</WORKFLOWPROCESSINFOID>"
                                                                + @"<STARTDATELOCKED>0</STARTDATELOCKED>
							                                </WORKFLOWPROCESSINFO>
						                                </PARAM>
					                                </OPERATION>
					                                </INPUTXML>");
					TempXML = UpdateActivityXML.ToString();
				}		
				ErrorXML = OMIObj.Execute(ref TempXML);
				return ErrorXML;
		}		

		private void GetMaxDueDateFromPredecessors(XPathNavigator ProcessNav, Hashtable ActivityTable, String ActivityID, out DateTime NextStartDate,out String PredActivityIDOfMaxDueDate)
		{
			XPathNavigator		TempNav							= null;
			XPathNavigator		ActivityNav						= null;
			XPathNodeIterator	TempIter						= null;
			String				PredActID						= null;
			int					Index							= 1;
			int					Day								= 0;
			int					Month							= 0;
			int					Year							= 0;
			DateTime			TempDate						;
			DateTime			StartDateForNextAct				;

				TempDate			= DateTime.Today;
				NextStartDate		= DateTime.Today;
                StartDateForNextAct = DateTime.Today;
				PredActivityIDOfMaxDueDate = null;
				TempIter = ProcessNav.Select("PROCESS_TRANSITION/TRANSITION[TOACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityID + "']/FROMACTIVITY/ACTIVITY/ACTIVITYID");
				while(TempIter.MoveNext())
				{
					TempNav		= TempIter.Current;
					ActivityNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='"+TempNav.Value+"' and STARTACTIVITY='False' and ENDACTIVITY='False']");
					if ((Index == 1)&(ActivityNav !=null))
					{
						PredActID					= ActivityNav.SelectSingleNode("ACTIVITYID").Value;
						TempDate					= (DateTime)ActivityTable[PredActID];
						StartDateForNextAct			= TempDate;
						PredActivityIDOfMaxDueDate	= PredActID;						
					}
					else
					{					
						if(ActivityNav != null)
						{
							PredActID = ActivityNav.SelectSingleNode("ACTIVITYID").Value;
							TempDate  = (DateTime)ActivityTable[PredActID];
						}
					}
					Index++;
					if(TempDate > StartDateForNextAct)
					{
						StartDateForNextAct = TempDate;
						PredActivityIDOfMaxDueDate = PredActID;
					}
				}
				if (PredActivityIDOfMaxDueDate!=null)
				{
					NextStartDate = StartDateForNextAct;
					NextStartDate = (DateTime)ActivityTable[PredActivityIDOfMaxDueDate];
				}
				else
				{
					ActivityNav		= ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='"+ ActivityID +"']");
					Day				= Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/DAY").Value);
					Month			= Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/MONTH").Value);
					Year			= Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/YEAR").Value);
                    NextStartDate = new DateTime(Year, Month, Day);
					 
				}
		}
		
		private  List<ActivityTLStatus> SyncActivityTimeLineStatusList(List<ActivityInfo> AInfoList,String Level)
		{
			List<ActivityTLStatus> ATLList		= null;
			ActivityTLStatus ActivityStatusObj	= null;
			String			 ActivityID			= null;	
			
                ATLList = new List<ActivityTLStatus>();
				foreach(ActivityInfo AInfo in AInfoList)
				{
					 if (AInfo.LevelID == Level)
					 {
						 ActivityID		   = AInfo.ActivityID; 
						 ActivityStatusObj = new ActivityTLStatus(ActivityID,TimeLineStatus.NO);
						 ATLList.Add(ActivityStatusObj);
					 }
				}
				return ATLList;
		}
		
		private bool CheckActivityTimeLineStatus(XPathNavigator ProcessNav ,List<ActivityTLStatus> ATLList,String ActivityID)
		{
			XPathNodeIterator PredecessorIter = null;			
			XPathNavigator	  PredecessorNav  = null;
			XPathNavigator	  ActNav		  = null;	
			TimeLineStatus	  TStatus		;
			
				PredecessorIter = ProcessNav.Select("PROCESS_TRANSITION/TRANSITION[TOACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityID + "']/FROMACTIVITY/ACTIVITY/ACTIVITYID");
				TStatus = TimeLineStatus.YES;
				while (PredecessorIter.MoveNext())
				{
					PredecessorNav	= PredecessorIter.Current;
					ActNav			= ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + PredecessorNav.Value + "']");
					TStatus = GetTimeLineStatus(ATLList, PredecessorNav.Value);
					if(TStatus == TimeLineStatus.NO)
						return false;
				}
				return (TStatus == TimeLineStatus.YES?true:false);
		}
		
		private TimeLineStatus GetTimeLineStatus(List<ActivityTLStatus> ATList,String ActivityID)
		{
				for (int Index = 0;Index< ATList.Count;Index++)
				{
					if (ATList[Index].ActivityID == ActivityID)
						return ATList[Index].Status;
				}
				return TimeLineStatus.YES;
		}
		
		private ActivityTLStatus GetActivityStatusObject(List<ActivityTLStatus> ATList,String ActivityID)
		{
				foreach (ActivityTLStatus ActObj in ATList)
				{
					if (ActObj.ActivityID == ActivityID)
						return ActObj;
				}
				return null;
		}
		
		private ERRORXML UpdateProcessDurationAndDueDate(String ProcessID,String ProcessInfoID,Hashtable DurationTable,Hashtable DueDateTable,out String TotalProcessDuration,out DateTime UpdatedProcessDueDate)
		{
			IDictionaryEnumerator ActivityDurationEnum = null;
			IDictionaryEnumerator ActivityDueDateEnum  = null;					
			StringBuilder		 UpdatedProcessXML	   = null;
			StringBuilder		 AssociationXML		   = null;
			XML					 TempXML			   = null;
			ERRORXML			 ErrorXML			   = null;
			OMInterface			 OMIObj				   = null;
            OperationQueue		 QMQueueObj			   = null;
            Operation			 OperObj			   = null;
            long				  uProcessDuration     = 0;
			DateTime			 TempDate			   ;
			DateTime			 ActivityDate		   ;
			
					ActivityDate = new DateTime(1,1,1);
					TempDate	 = new DateTime(1,1,1);
					AssociationXML= new StringBuilder();
					
					OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
					ActivityDurationEnum = DurationTable.GetEnumerator();
					while(ActivityDurationEnum.MoveNext())
					{
						uProcessDuration += Convert.ToInt64(ActivityDurationEnum.Value);
					}
					// Update the process duration and set the start date and due date:
					// Compute the max due date.					
					ActivityDueDateEnum = DueDateTable.GetEnumerator();
					int Count = 1;
					while(ActivityDueDateEnum.MoveNext())
					{
						if(Count == 1)
						{
							TempDate = (DateTime)ActivityDueDateEnum.Value;
						}
						else
						{
							ActivityDate = (DateTime)ActivityDueDateEnum.Value;
						}
						if (ActivityDate > TempDate)
						{
							TempDate = ActivityDate;
						}
						Count++;
					}
                    
					UpdatedProcessXML = new StringBuilder(250);
					UpdatedProcessXML.AppendFormat(@"<WORKFLOWPROCESS MODIFIED='TRUE'>
				                                        <WORKFLOWPROCESSID>{0}</WORKFLOWPROCESSID>
				                                        <DUEDATE>
					                                        <DAY>"+TempDate.Day.ToString() +"</DAY>"
					                                        +"<MONTH>" + TempDate.Month.ToString() +"</MONTH>"
					                                        + "<YEAR>"+ TempDate.Year.ToString()+ "</YEAR>"
				                                        + @"</DUEDATE>", ProcessID );
                    if (m_SetBaselineForCreateDeliverable)
                    {
                        UpdatedProcessXML.AppendFormat(@"<BASEENDDATE>
                                                            <DAY>" + TempDate.Day.ToString() + "</DAY>"
                                                         + "<MONTH>" + TempDate.Month.ToString() + "</MONTH>"
                                                         + "<YEAR>" + TempDate.Year.ToString() + "</YEAR>"
                                                 + @"</BASEENDDATE>");
                    }

				              UpdatedProcessXML.AppendFormat(@"<PROCESS_INFORMATION TYPE='RELATION'>
					                                        <WORKFLOWPROCESSINFO MODIFIED='TRUE'>
						                                        <WORKFLOWPROCESSINFOID>{0}</WORKFLOWPROCESSINFOID>
						                                        <DURATION>{1}</DURATION>
					                                        </WORKFLOWPROCESSINFO>
				                                        </PROCESS_INFORMATION>
			                                        </WORKFLOWPROCESS>", ProcessInfoID, uProcessDuration.ToString());					
					OperObj		= new Operation("ENTITY", "UPDATEDATA");
					QMQueueObj	= new OperationQueue(false);
					OperObj.AddParameter("INPUTXML", UpdatedProcessXML.ToString());
					QMQueueObj.AddOperation(OperObj);
					TempXML = QMQueueObj.Xml;						
					ErrorXML				= OMIObj.Execute(ref TempXML);
					TotalProcessDuration	= uProcessDuration.ToString();
					UpdatedProcessDueDate	= TempDate;			
					return ErrorXML;
		}

		private ERRORXML UpdateDependantActivities(XPathNavigator ProcessNav, String str_FilePath, ref List<ActivityTLStatus> ActivityTLStatusTable, ref Hashtable ActivityDueDateTable, ref Hashtable ActivityDurationTable)
		{
			String				PredActivityID			= null;
			String				SubProcessID			= null;
			String				XPathQuery				= null;
			String				ActivityInfoID			= null;
			String				TotalProcessDuration	= null;
			XPathNavigator		ActivityNav				= null;
			XPathNavigator		TempNav					= null;
			XPathNavigator		PredActNav				= null;
			XPathNodeIterator	TempNodeIter			= null;
			XML					ReturnXML				= null;
			ERRORXML			ErrorXML				= null;				
			DateTime			UpdatedProcessDueDate	;
			DateTime			NextStartDate			;
			DateTime			TempDate				;
			bool				StartDateLocked			= false;			
			bool				SuccessorPresent		= false;
			String				ActivityLock			= null;			
			String				SuccActivityID			= null;
			ActivityTLStatus	ATLObj					= null;
			ActivityTLStatus    ATLStatusObj			= null;
			TimeLineStatus		TStatus					= TimeLineStatus.NO;
            int                 Year                        = 0;
            int                 Month                       = 0;
            int                 Day                         = 0;
            
				TempDate		= DateTime.Today;
				NextStartDate	= DateTime.Today;
                UpdatedProcessDueDate = DateTime.Today;
				
				for (int Index=0;Index < ActivityTLStatusTable.Count;Index++)
				{
					ATLStatusObj = ActivityTLStatusTable[Index];
					if (ATLStatusObj.Status == TimeLineStatus.NO)
					{
						ActivityNav  = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + ATLStatusObj.ActivityID + "']");
						ActivityLock = ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/STARTDATELOCKED").Value;
						StartDateLocked = (ActivityLock == "0"?false:true);
						if (!StartDateLocked)
						{						
							GetMaxDueDateFromPredecessors(ProcessNav, ActivityDueDateTable, ATLStatusObj.ActivityID, out NextStartDate, out PredActivityID);
							if(PredActivityID != null)
							{
								PredActNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID='" + PredActivityID + "']");
								if((PredActNav.SelectSingleNode("STARTACTIVITY").Value.ToUpper() == "FALSE") && (PredActNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "FALSE"))
								{
									long Min = ConvertDaysToMinutes(2);
									TempDate = NextStartDate;
									GetDueDate(TempDate, Min.ToString(), out NextStartDate);
								}
							}
						}
						else
						{
							
                            Year = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/YEAR").Value);
                            Month = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/MONTH").Value);
                            Day = Convert.ToInt32(ActivityNav.SelectSingleNode("STARTDATE/DAY").Value);
                            NextStartDate = new DateTime(Year, Month, Day);
						}						
						XPathQuery = "SUBFLOWACTIVITY/WORKFLOWPROCESS";
						TempNav = ActivityNav.SelectSingleNode(XPathQuery);
						if(TempNav != null)
						{
							SubProcessID = TempNav.SelectSingleNode("WORKFLOWPROCESSID").Value;
							CalculateTimeLine_Ex(SubProcessID,NextStartDate,null, false, str_FilePath, out ReturnXML,out TotalProcessDuration,out UpdatedProcessDueDate);
							if(ErrorXML == null)
							{
								XPathQuery = "ACTIVITY_INFORMATION/ACTIVITYINFORMATION";
								TempNav = ActivityNav.SelectSingleNode(XPathQuery);
								if(TempNav != null)
								{
									ActivityInfoID = TempNav.SelectSingleNode("ACTIVITYINFORMATIONID").Value;
								}
								//// Update this activity
                                ErrorXML = UpdateActivityForTimeLine(ATLStatusObj.ActivityID, NextStartDate, UpdatedProcessDueDate, ActivityInfoID, TotalProcessDuration, null, false, ActivityNav.SelectSingleNode("STATUS").Value);
								if(ErrorXML != null)
									return ErrorXML;

								if(ActivityDurationTable.ContainsKey(ATLStatusObj.ActivityID))
								{
									ActivityDurationTable[ATLStatusObj.ActivityID] = TotalProcessDuration;
								}
								if(ActivityDueDateTable.ContainsKey(ATLStatusObj.ActivityID))
								{
									ActivityDueDateTable[ATLStatusObj.ActivityID] = UpdatedProcessDueDate;
								}

								ATLStatusObj.Status = TimeLineStatus.YES;
							}
						}
						else
						{
							if((ActivityNav.SelectSingleNode("ENDACTIVITY") != null) && (ActivityNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "TRUE"))
							{
								XPathQuery = "ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID";
								TempNav = ActivityNav.SelectSingleNode(XPathQuery);
								if(TempNav != null)
									ActivityInfoID = TempNav.Value;
                                ErrorXML = UpdateActivityForTimeLine(ATLStatusObj.ActivityID, NextStartDate, NextStartDate, ActivityInfoID, "0", null, false, ActivityNav.SelectSingleNode("STATUS").Value);
								if(ErrorXML != null)
									return ErrorXML;
								if(ActivityDueDateTable.ContainsKey(ATLStatusObj.ActivityID))
								{
									ActivityDueDateTable[ATLStatusObj.ActivityID] = NextStartDate;
								}
								ATLStatusObj.Status = TimeLineStatus.YES;
							}
						}
						// FIND ALL THE SUCCESSORS OF THE CURRENT ACTIVTIY, AND ADD THIS ACTIVITY TO TLSTATUS LIST.
						XPathQuery = "PROCESS_TRANSITION/TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + ATLStatusObj.ActivityID + "']/TOACTIVITY/ACTIVITY";
						TempNodeIter = ProcessNav.Select(XPathQuery);

						while(TempNodeIter.MoveNext())
						{
							TempNav = TempNodeIter.Current;
							SuccActivityID = TempNav.SelectSingleNode("ACTIVITYID").Value;

							//if successor is present in the ActivityTLStatus , then do not.
							SuccessorPresent = GetActivtiyInfo(ActivityTLStatusTable, SuccActivityID);
							if(SuccessorPresent == false)
							{
								ATLObj = new ActivityTLStatus(SuccActivityID, TimeLineStatus.NO);
								ActivityTLStatusTable.Add(ATLObj);
							}
							else
							{
								TStatus = GetTimeLineStatus(ActivityTLStatusTable, SuccActivityID);
								if(TStatus == TimeLineStatus.YES)
								{
									// If the successor activity's timeline is already calculated , but as a result of the 
									// calculation of timeline of its predecessor at some later time , the successor will hav to be added
									// in the activityInfo List again with its timeline status as yes.									
									ATLObj = new ActivityTLStatus(SuccActivityID, TimeLineStatus.NO);
									ActivityTLStatusTable.Add(ATLObj);
								}
							}
						}// end while
					}
				}
				return ErrorXML;	
		}

		private XML UpdateProcessAndActivityDates(List<Activity> ActList, String ProcessInstanceID, String ProcessInfoID, String TotalProcessDuration, DateTime ProcessStartDate, DateTime UpdatedProcessDueDate, String str_ProcessStatus)
		{
			StringBuilder	UpdateXML = null;
			
					UpdateXML = new StringBuilder(500);
					UpdateXML.Append(@"<INPUTXML>
				                        <OPERATION>
				                        <OBJECTNAME>ENTITY</OBJECTNAME>
				                        <METHOD>UPDATEDATA</METHOD>
				                        <PARAM Name='INPUTXML'>
					                        <WORKFLOWPROCESS MODIFIED='TRUE'>
						                        <WORKFLOWPROCESSID>"+ProcessInstanceID+"</WORKFLOWPROCESSID>");
                    if ((!String.IsNullOrEmpty(str_ProcessStatus)) && ((str_ProcessStatus.ToUpper() == "NEW") || (str_ProcessStatus.ToUpper() == "DRAFT")))
                    {
                        UpdateXML.Append(@"<STARTDATE>
								               <DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                                + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                                + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>"                    
                                          + "</STARTDATE>");

                        if (m_SetBaselineForCreateDeliverable)
                        {

                            UpdateXML.Append(@"<BASESTARTDATE>
								               <DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                                    + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                                    + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>"
                                             + @"</BASESTARTDATE>
                                             <BASEENDDATE>
								               <DAY>" + UpdatedProcessDueDate.Day.ToString() + "</DAY>"
                                                    + "<MONTH>" + UpdatedProcessDueDate.Month.ToString() + "</MONTH>"
                                                    + "<YEAR>" + UpdatedProcessDueDate.Year.ToString() + "</YEAR>"
                                              + "</BASEENDDATE>");
                        }
                    }

                    UpdateXML.Append(@"<DUEDATE>
									       <DAY>" + UpdatedProcessDueDate.Day.ToString() + "</DAY>"
                                        + "<MONTH>" + UpdatedProcessDueDate.Month.ToString() + "</MONTH>"
                                        + "<YEAR>" + UpdatedProcessDueDate.Year.ToString() + "</YEAR>"                                        
                                    + @"</DUEDATE>
				                        <PROCESS_INFORMATION TYPE='RELATION'>
					                        <WORKFLOWPROCESSINFO MODIFIED='TRUE'><WORKFLOWPROCESSINFOID>" + ProcessInfoID + "</WORKFLOWPROCESSINFOID>"
                                + @"<DURATION>" + TotalProcessDuration + "</DURATION>"
                        + @"</WORKFLOWPROCESSINFO>
				                        </PROCESS_INFORMATION>
			                        </WORKFLOWPROCESS>
		                        </PARAM>
		                        </OPERATION>");

							foreach(Activity ActivityObj in ActList)
							{
								if (ActivityObj.Modified)
								{
                                    UpdateXML.Append(@"<OPERATION>
								                            <OBJECTNAME>ENTITY</OBJECTNAME>
								                            <METHOD>UPDATEDATA</METHOD>
								                            <PARAM Name='INPUTXML'>
									                            <ACTIVITY MODIFIED='TRUE'>
										                            <ACTIVITYID>" + ActivityObj.ActivityID + "</ACTIVITYID>");
                                    if ((!String.IsNullOrEmpty(ActivityObj.Status)) && ((ActivityObj.Status.ToUpper() == "NEW") || (ActivityObj.Status.ToUpper() == "DRAFT")))
                                    {
                                        UpdateXML.Append(@"<STARTDATE>
								                               <DAY>" + ActivityObj.StartDate.Day.ToString() + "</DAY>"
                                                            + "<MONTH>" + ActivityObj.StartDate.Month.ToString() + "</MONTH>"
                                                            + "<YEAR>" + ActivityObj.StartDate.Year.ToString() + "</YEAR>"
                                                        + "</STARTDATE>");
                                        if (m_SetBaselineForCreateDeliverable)
                                        {
                                            UpdateXML.Append(@"<BASESTARTDATE>
								                               <DAY>" + ActivityObj.StartDate.Day.ToString() + "</DAY>"
                                                                + "<MONTH>" + ActivityObj.StartDate.Month.ToString() + "</MONTH>"
                                                                + "<YEAR>" + ActivityObj.StartDate.Year.ToString() + "</YEAR>"
                                                         + @"</BASESTARTDATE>
                                                        <BASEENDDATE>
						                                    <DAY>" + ActivityObj.DueDate.Day.ToString() + "</DAY>"
                                                             + "<MONTH>" + ActivityObj.DueDate.Month.ToString() + "</MONTH>"
                                                             + "<YEAR>" + ActivityObj.DueDate.Year.ToString() + "</YEAR>"
                                                          + "</BASEENDDATE>");
                                        }

                                    }

                                        UpdateXML.Append(@"<DUEDATE>
												               <DAY>" + ActivityObj.DueDate.Day.ToString() + "</DAY>"
                                                            + "<MONTH>" + ActivityObj.DueDate.Month.ToString() + "</MONTH>"
                                                            + "<YEAR>" + ActivityObj.DueDate.Year.ToString() + "</YEAR>"
                                                        + @"</DUEDATE>
								                            <ACTIVITY_INFORMATION TYPE='RELATION'>
								                            <ACTIVITYINFORMATION MODIFIED='TRUE'><ACTIVITYINFORMATIONID>" + ActivityObj.ActivityInfoID + "</ACTIVITYINFORMATIONID>"
                                                        + "<DURATION>" + ActivityObj.Duration + "</DURATION>"
                                                        + @"</ACTIVITYINFORMATION>
								                            </ACTIVITY_INFORMATION>
								                            </ACTIVITY>
						                                </PARAM>
						                            </OPERATION>");
								}
							}
							UpdateXML.Append(@"<TRANSACTION>TRUE</TRANSACTION>
					                        </INPUTXML>");
					return UpdateXML.ToString();
		}

		private int CompareActivityDueDates(Activity AObj1, Activity AObj2)
		{
			return (AObj1.DueDate.CompareTo(AObj2.DueDate));
		}

		private int CompareActivityEndDates(Activity AObj1, Activity AObj2)
		{
			return (AObj1.DueDate.CompareTo(AObj2.ActualEndDate));
		}

		private void GetPredecessorsAndActivityIDOfMaxDueDate(List<Activity> ActivityList, String ActivityID, XPathNavigator ProcessNav, out String PredecessorList, out String ActivityIDOfMaxDueDate)
        {
			XPathNavigator	ProcessTransitionNav	= null;
			XPathNavigator ProcessActivityNav		= null;
			XPathNavigator TempNav					= null;
			XPathNavigator	FromActNav				= null;            
            XPathNodeIterator	ActivityIDNodeIter	= null;            
            StringBuilder ActivityIDList			= null;
            StringBuilder ActPredecessorList		= null;			
			String			XPathQuery				= null;
			String			FromActivityID			= null;
			Activity		ActObj					= null;
			Activity		ActivityObj				= null;
			Activity[]		ActivityArr				= null;
			List<Activity>	TempActivityList		= null;
            DateTime      MaxDate;            
            int			  TotalCount			   = 0;

                ActivityIDOfMaxDueDate = null;
                PredecessorList         = null;                
                ActivityIDList          = new StringBuilder();
                ActPredecessorList      = new StringBuilder();
				TempActivityList		= new List<Activity>();

				ProcessTransitionNav	= ProcessNav.SelectSingleNode("PROCESS_TRANSITION");
				ProcessActivityNav		= ProcessNav.SelectSingleNode("PROCESS_ACTIVITY");
				
				if(ProcessTransitionNav != null)
                {
					XPathQuery         = "TRANSITION[TOACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityID + "']/FROMACTIVITY/ACTIVITY";
                    int i = 0;
                    ActivityIDNodeIter = ProcessTransitionNav.Select(XPathQuery);
					
					TotalCount	= ActivityIDNodeIter.Count;
					i = 0;
					while(ActivityIDNodeIter.MoveNext())
					{
						TempNav = ActivityIDNodeIter.Current;
						if(i < TotalCount - 1)
						{
							ActPredecessorList.AppendFormat("{0};", TempNav.SelectSingleNode("ACTIVITYID").Value);
						}
						else
							ActPredecessorList.AppendFormat("{0}", TempNav.SelectSingleNode("ACTIVITYID").Value);
						i++;
						
						FromActNav = ActivityIDNodeIter.Current;
						FromActivityID = FromActNav.SelectSingleNode("ACTIVITYID").Value;
						ActivityFilter AF = new ActivityFilter(ActivityIDNodeIter.Current.Value);
						Predicate<Activity> FindActivityFromList = null;
						FindActivityFromList = new Predicate<Activity>(AF.FilterByActivityID);
						ActivityObj			 = ActivityList.Find(FindActivityFromList);
                        if (ActivityObj != null)
                        {
                            ActObj = ActivityObj;
                            if (ActObj.ActualEndDate != new DateTime(1, 1, 1))
                                ActObj.DueDate = ActObj.ActualEndDate;
                            TempActivityList.Add(ActObj);
                        }
                        else
                        { 
                            // Predecessor not found, return null;
                            return;
                        }
					}
					if (TempActivityList.Count > 0)
					{
						SortDueDates(TempActivityList);
						// find the predecessor activity of max duedate
						ActivityArr = TempActivityList.ToArray();
						ActObj		= ActivityArr[ActivityArr.Length  - 1];
						MaxDate		= ActObj.DueDate;
						ActivityIDOfMaxDueDate = ActObj.ActivityID;
						PredecessorList = ActPredecessorList.ToString();
					}	
                }
        }

		private static Activity FindActivityIDInList(Activity AObj, String ActivityID)
		{
				if (AObj.ActivityID == ActivityID )
					return AObj;
				return null;					
		}

		private void SortDueDates(List<Activity> ActList)
		{
			Comparison<Activity> CompareDates = null;				
			
				CompareDates = new Comparison<Activity>(CompareActivityDueDates);
				ActList.Sort(CompareDates);
		}
		
		private void SortEndDates(List<Activity> ActList)
		{
			Comparison<Activity> CompareDates = null;
			
            CompareDates = new Comparison<Activity>(CompareActivityDueDates);
            ActList.Sort(CompareDates);
		}

		private ERRORXML UpdateProcessWithOnlyStartAndEndActivity(XPathNavigator ProcessNav, DateTime ProcessStartDate,out String Duration, out DateTime ProcessDueDate)
		{
			StringBuilder		UpdateXML			= null;			
			XPathNavigator		ActivityNav			= null;			
			XPathNodeIterator	ActivityNodeIter	= null;
			String				ProcessInfoID		= null;			
			ERRORXML			ErrorXml			= null;
			XML					UpDateProcessXML	= null;			
			OMInterface			OMI					= null;			
			
				UpdateXML = new StringBuilder(400);
				ProcessDueDate = ProcessStartDate;
				Duration = "0";
				
                ProcessInfoID = ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").Value;				
				if(ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/DURATION") != null)
					Duration = ProcessNav.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/DURATION").Value;
				GetDueDate(ProcessStartDate, Duration, out ProcessDueDate);
                UpdateXML.Append(@"<INPUTXML>
					                    <OPERATION>
					                    <OBJECTNAME>ENTITY</OBJECTNAME>
					                    <METHOD>UPDATEDATA</METHOD>
					                    <PARAM Name='INPUTXML'>
						                    <WORKFLOWPROCESS MODIFIED='TRUE'>
							                    <WORKFLOWPROCESSID>" + ProcessNav.SelectSingleNode("WORKFLOWPROCESSID").Value + @"</WORKFLOWPROCESSID>
							                    <STARTDATE>
								                    <DAY>" + ProcessStartDate.Day.ToString() + @"</DAY>
								                    <MONTH>" + ProcessStartDate.Month.ToString() + @"</MONTH>
								                    <YEAR>" + ProcessStartDate.Year.ToString() + @"</YEAR>
							                    </STARTDATE>
							                    <DUEDATE>
								                    <DAY>" + ProcessDueDate.Day.ToString() + @"</DAY>
								                    <MONTH>" + ProcessDueDate.Month.ToString() + @"</MONTH>
								                    <YEAR>" + ProcessDueDate.Year.ToString() + @"</YEAR>					
							                    </DUEDATE>");
                if (m_SetBaselineForCreateDeliverable)
                {

                    UpdateXML.Append(@"<BASESTARTDATE>
						                    <DAY>" + ProcessStartDate.Day.ToString() + @"</DAY>
						                    <MONTH>" + ProcessStartDate.Month.ToString() + @"</MONTH>
						                    <YEAR>" + ProcessStartDate.Year.ToString() + @"</YEAR>
					                    </BASESTARTDATE>
					                    <BASEENDDATE>
						                    <DAY>" + ProcessDueDate.Day.ToString() + @"</DAY>
						                    <MONTH>" + ProcessDueDate.Month.ToString() + @"</MONTH>
						                    <YEAR>" + ProcessDueDate.Year.ToString() + @"</YEAR>
					                    </BASEENDDATE>");
                }

                UpdateXML.Append(@"<PROCESS_INFORMATION TYPE='RELATION'>
									                    <WORKFLOWPROCESSINFO MODIFIED='TRUE'>
                                                            <WORKFLOWPROCESSINFOID>" + ProcessInfoID + @"</WORKFLOWPROCESSINFOID>
									                        <DURATION>" + Duration + @"</DURATION>
								                    </WORKFLOWPROCESSINFO>
							                    </PROCESS_INFORMATION>
						                    </WORKFLOWPROCESS>
					                    </PARAM>
				                    </OPERATION>");
						
						ActivityNodeIter = ProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY");
						while(ActivityNodeIter.MoveNext())
						{
							ActivityNav = ActivityNodeIter.Current;
                            UpdateXML.Append(@"<OPERATION>
							                            <OBJECTNAME>ENTITY</OBJECTNAME>
							                            <METHOD>UPDATEDATA</METHOD>
							                            <PARAM Name='INPUTXML'>
								                            <ACTIVITY MODIFIED='TRUE'>
									                            <ACTIVITYID>" + ActivityNav.SelectSingleNode("ACTIVITYID").Value + @"</ACTIVITYID>
									                            <STARTDATE>
										                            <DAY>" + ProcessStartDate.Day.ToString() + @"</DAY>
										                            <MONTH>" + ProcessStartDate.Month.ToString() + @"</MONTH>
										                            <YEAR>" + ProcessStartDate.Year.ToString() + @"</YEAR>
									                            </STARTDATE>
									                            <DUEDATE>
										                            <DAY>" + ProcessStartDate.Day.ToString() + @"</DAY>
										                            <MONTH>" + ProcessStartDate.Month.ToString() + @"</MONTH>
										                            <YEAR>" + ProcessStartDate.Year.ToString() + @"</YEAR>
									                            </DUEDATE>");
                                    if (m_SetBaselineForCreateDeliverable)
                                    {

                                        UpdateXML.Append(@"<BASESTARTDATE>
						                                        <DAY>" + ProcessStartDate.Day.ToString() + @"</DAY>
						                                        <MONTH>" + ProcessStartDate.Month.ToString() + @"</MONTH>
						                                        <YEAR>" + ProcessStartDate.Year.ToString() + @"</YEAR>
					                                        </BASESTARTDATE>
					                                        <BASEENDDATE>
						                                        <DAY>" + ProcessDueDate.Day.ToString() + @"</DAY>
						                                        <MONTH>" + ProcessDueDate.Month.ToString() + @"</MONTH>
						                                        <YEAR>" + ProcessDueDate.Year.ToString() + @"</YEAR>
					                                        </BASEENDDATE>");
                                    }
                                    UpdateXML.Append(@"<ACTIVITY_INFORMATION TYPE='RELATION'>
										                            <ACTIVITYINFORMATION MODIFIED='TRUE'>
                                                                        <ACTIVITYINFORMATIONID>" + ActivityNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID").Value + @"</ACTIVITYINFORMATIONID>
											                            <DURATION>0</DURATION>
										                            </ACTIVITYINFORMATION>
									                            </ACTIVITY_INFORMATION>
								                            </ACTIVITY>
								                            </PARAM>
								                         </OPERATION>");			
						}
						UpdateXML.Append(@"<TRANSACTION>TRUE</TRANSACTION>
				                        </INPUTXML>");
				UpDateProcessXML = UpdateXML.ToString();
				if(UpDateProcessXML != null)
				{
					if(ErrorXml == null)
					{
						OMI = new OMInterface(UserID, CurrentLanguage, TenantName);
						ErrorXml = OMI.Execute(ref UpDateProcessXML);
						if(ErrorXml != null)
						{
							ErrorXml = PrepareErrorXML(ErrorXml, ErrorCodes.PROCESS_MODIFICATION_FAILED, ErrorObjectType.PROCESS);
							return ErrorXml;
						}
					}
				}
				return ErrorXml;
		}
				
        private void UpdateDependantActivities(XPathNavigator ProcessNav, String str_FilePath, List<Activity> ActivityList,Queue<ActivityDependancy> ActivityDepQ)
		{
			String				ActivityID		= null;
			String				XPathQuery		= null;
			String				FromActivityID	= null;
			XPathNavigator		ProcessActNav	= null;
			XPathNavigator		ActNav			= null;
			XPathNavigator		ProcessTranNav	= null;
			XPathNavigator		FromActNav		= null;
            XPathNavigator      SubflowProcessNav=null;
			XPathNodeIterator	PredActIter		= null;
			List<Activity>		TempActivityList= null;	
			Activity			ActObj			= null;	
			Activity			ActivityObj		= null;
			Activity			NewActivity		= null;
			Activity[]			ActivityArr		= null;
            String              ProcessDuration = null;
            ERRORXML            ErrorXml        = null;
            Boolean             PredFound       = true;
			DateTime			NexStartDate	;
			DateTime			NexDueDate		;
			ActivityDependancy  ADObj           = null;
				
				ProcessActNav = ProcessNav.SelectSingleNode("PROCESS_ACTIVITY");
				ProcessTranNav = ProcessNav.SelectSingleNode("PROCESS_TRANSITION");


                while (ActivityDepQ.Count > 0)
                {
                    PredFound = true;
                    ADObj = ActivityDepQ.Dequeue();
                    TempActivityList = new List<Activity>();
                    ActivityID = ADObj.DependantActivityID;
                    XPathQuery = "TRANSITION[TOACTIVITY/ACTIVITY/ACTIVITYID='" + ActivityID + "']/FROMACTIVITY/ACTIVITY";
                    // get all the predecessors of this activity.
                    PredActIter = ProcessTranNav.Select(XPathQuery);
                    while (PredActIter.MoveNext())
                    {
                        FromActNav = PredActIter.Current;
                        FromActivityID = FromActNav.SelectSingleNode("ACTIVITYID").Value;
                        ActivityFilter AF = new ActivityFilter(PredActIter.Current.Value);
                        Predicate<Activity> FindActivityFromList = null;
                        FindActivityFromList = new Predicate<Activity>(AF.FilterByActivityID);
                        ActivityObj = ActivityList.Find(FindActivityFromList);
                        if (ActivityObj != null)
                        {
                            ActObj = ActivityObj;
                            TempActivityList.Add(ActObj);                            
                        }
                        else
                        {
                            ActivityDepQ.Enqueue(ADObj);
                            PredFound = false;
                            break;
                        }                       
                    }

                    if (!PredFound)
                        continue;

                    SortDueDates(TempActivityList);
                    // find the predecessor activity of max duedate
                    ActivityArr = TempActivityList.ToArray();
                    ActObj = ActivityArr[ActivityArr.Length - 1];
                    // check whether the dependant activity is an end activity, if yes then the start and due dates
                    // of the dependant activity are same as that of the predecessor.
                    ActNav = ProcessActNav.SelectSingleNode("ACTIVITY[ACTIVITYID='" + ActivityID + "']");
                    if ((ActNav != null) && (ActNav.SelectSingleNode("ENDACTIVITY").Value.ToUpper() == "TRUE"))
                    {
                        NewActivity = new Activity(ActivityID, ActObj.StartDate, ActObj.DueDate, ActObj.ActivityInfoID, "0", ActNav.SelectSingleNode("STATUS").Value);
                    }
                    else
                    {
                        if (ActNav != null)
                        {
                            long Min = ConvertDaysToMinutes(2);
                            GetDueDate(ActObj.DueDate, Min.ToString(), out NexStartDate);
                            // Check if the Activity is Subflow, if yes then call timeline again
                            SubflowProcessNav = ActNav.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS/WORKFLOWPROCESSID");
                            if (SubflowProcessNav != null)
                            {
                                ErrorXml = NewCalculateTimeLine(SubflowProcessNav.Value, NexStartDate, str_FilePath, out ProcessDuration, out NexDueDate);
                                NewActivity = new Activity(ActivityID, NexStartDate, NexDueDate, ActObj.ActivityInfoID, ProcessDuration, ActNav.SelectSingleNode("STATUS").Value);
                            }
                            else
                            {
                                GetDueDate(NexStartDate, ActNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value, out NexDueDate);
                                NewActivity = new Activity(ActivityID, NexStartDate, NexDueDate, ActObj.ActivityInfoID, ActNav.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value, ActNav.SelectSingleNode("STATUS").Value);
                            }
                        }
                    }
                    NewActivity.Modified = true;
                    ActivityList.Add(NewActivity);
                }				
		}
		
		private bool GetActivity_ModifiedFlagValue(Activity AObj)
		{
			return AObj.Modified;
		}
		
		private void GetModifiedActivities(List<Activity> ActList)
		{
			Predicate<Activity> ActivityModified = null;
			
            ActivityModified = new Predicate<Activity>(GetActivity_ModifiedFlagValue);
			ActList.FindAll(ActivityModified);
		}

		private bool GetActivtiyInfo(List<ActivityTLStatus> ActivityStatusList, String SuccessorActivityID)
		{
			for(int Index = 0; Index < ActivityStatusList.Count; Index++)
			{
				if(ActivityStatusList[Index].ActivityID == SuccessorActivityID)
					return true;
			}
			return false;
		}
       
        /// <summary>
        /// This function will return field information for given ObjectType
        /// </summary>
        /// <Author> Priyanka Gunjal</Author>
        /// <Date>25 Sept 2006</Date>
        /// <param name="EntityTypeXML">Comma seperated list of Entities</param>
        /// <param name="FacetTypeXML">Comma seperated list of Facets</param>
        /// <param name="RelationshipTypeXML">Comma seperated list of Relationships</param>
        /// <param name="SchemaInfoDetails">Schema details of Entities/Facets and Relationships</param>
        /// <returns></returns>
        public ERRORXML GetTypeInfobyName(XML EntityTypeXML, XML FacetTypeXML, XML RelationshipTypeXML, ref XML SchemaInfoDetails)
        {
            #region variables
            OperationQueue  OpQueObj            = null;
            Operation       OperObj             = null;
            OMInterface     OMIObj              = null;
            XML             OperationXML        = null;
            ERRORXML        ErrorXML            = null;                       
            String[]        EntityList          = null;
            String[]        FacetList           = null;
            String[]        RelationshipList    = null;
            String          EntityName          = null;
            String          FacetName           = null;
            String          RelationshipName    = null;   
            StringBuilder   TypeInfoDetails     = null;  
            ALEngineError	ALEngineErrorObj    = null;
            #endregion

            OpQueObj        = new OperationQueue(false);
            TypeInfoDetails = new StringBuilder(400);
            TypeInfoDetails.Append("<TYPEINFOCOLLECTION>");

            if (!String.IsNullOrEmpty(EntityTypeXML))
            {
                EntityList = EntityTypeXML.Split(',');
                for (int Index = 0; Index < EntityList.Length; Index++)
                {
                    EntityName      = EntityList[Index];
                    OperObj         = new Operation("ENTITYTYPE", "READBYNAME");
                    OperObj.AddParameter("ENTITYTYPENAME", EntityName);
                    OpQueObj.AddOperation(OperObj);
                    OperationXML    = OpQueObj.Xml;
                    OMIObj          = new OMInterface(UserID, CurrentLanguage, TenantName);
                    ErrorXML        = OMIObj.GetTypeData(OperationXML, ref SchemaInfoDetails);
                    if (ErrorXML != null)
                    {                        
                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_ENTITYTYPEINFO_FAILED", this.GetType().ToString(), "GetTypeInfobyName()", null, ErrorXML);
                        ErrorXML        = ALEngineErrorObj.ErrorCollectionXML;
                        return ErrorXML;                        
                    }
                    else
                    {
                        TypeInfoDetails.Append(SchemaInfoDetails);
                    }
                }
            }
            if(!String.IsNullOrEmpty(FacetTypeXML))
            {
                FacetList       = FacetTypeXML.Split(',');
                for (int Index = 0; Index < FacetList.Length; Index++)
                {
                    FacetName       = FacetList[Index];
                    OperObj         = new Operation("FACETTYPE", "READBYNAME");
                    OperObj.AddParameter("FACETTYPENAME", FacetName);
                    OpQueObj.AddOperation(OperObj);
                    OperationXML    = OpQueObj.Xml;
                    OMIObj          = new OMInterface(UserID, CurrentLanguage, TenantName);
                    ErrorXML        = OMIObj.GetTypeData(OperationXML, ref SchemaInfoDetails);                             
                    if (ErrorXML != null)
                    {
                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_FACETTYPEINFO_FAILED", this.GetType().ToString(), "GetTypeInfobyName()", null, ErrorXML);
                        ErrorXML         = ALEngineErrorObj.ErrorCollectionXML;
                        return ErrorXML;
                    }
                    else
                    {
                        TypeInfoDetails.Append(SchemaInfoDetails);
                    }
                }
            }

            if (!String.IsNullOrEmpty(RelationshipTypeXML))
            {
                RelationshipList = RelationshipTypeXML.Split(',');
                for (int Index = 0; Index < RelationshipList.Length; Index++)
                {
                    RelationshipName    = RelationshipList[Index];
                    OpQueObj            = new OperationQueue(false);
                    OperObj             = new Operation("RELATION", "READRELATIONSHIPBYNAME");
                    OperObj.AddParameter("RELATIONSHIPNAME", RelationshipName);
                    OpQueObj.AddOperation(OperObj);
                    OperationXML        = OpQueObj.Xml;
                    OMIObj              = new OMInterface(UserID, CurrentLanguage, TenantName);
                    ErrorXML            = OMIObj.GetTypeData(OperationXML, ref SchemaInfoDetails);
                    if (ErrorXML != null)
                    {
                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_RELATIONSHIPTYPEINFO_FAILED", this.GetType().ToString(), "GetTypeInfobyName()", null, ErrorXML);
                        ErrorXML         = ALEngineErrorObj.ErrorCollectionXML;
                        return ErrorXML;
                    }
                    else
                    {
                        TypeInfoDetails.Append(SchemaInfoDetails);
                    }
                }
            }
            TypeInfoDetails.Append("</TYPEINFOCOLLECTION>");
            SchemaInfoDetails = TypeInfoDetails.ToString();
            return null;            
        }               

        /// <summary>
        /// Consolidate the files for the route process.
        /// </summary>
        /// <param name="TaskID"></param>
        /// <returns></returns>
        public ERRORXML ConsolidateDocument(String TaskID)
        {
            String              TaskXML                 = null;
            string[]            FilePathSplit           = null;
            ERRORXML            ErrorXML                = null;
            String              DestFileName            = null;
            String              TaskInitFilePath        = null;
            String              ReviewerFilePath        = null;
            String              ConsolidatedFilePath    = null;
            String              DestinationFilePath     = null;
            String              ApprovalsMode           = null;
            StringBuilder       DestFilePath            = null;
            StringReader        String_Reader           = null;
            XPathDocument       OutputXMLDoc            = null;
            XPathNavigator      OutputXMLNav            = null;
            XPathNavigator      ActivityNode            = null;
            XPathNavigator      WorkflowProcessNav      = null;
            XPathNodeIterator   ActivityNodeList        = null;
            XPathNavigator      TaskInitiatorsFileNav   = null;
            XPathNavigator      ReviewersFileNav        = null;
            StringBuilder       ProcessXML              = null;
            String              UpdateProcessXML        = null;
            String              FileExtension           = null;
            String              EndActivitySeqNo        = null;
            String              ConsolidatedFileName    = null;
            String              FieldName               = null;
            XPathNavigator      FileSystemNav           = null;
            XPathNavigator      ConsolidatedFileNav     = null;
            StringBuilder		SearchXML				= null;            
            OMInterface			OMInterfaceObj			= null;
            String              TempFileName            = null;
            StringBuilder       TempReviewerFileName    = null;
            String              TempPath                = null; 
            String[]            TempReviewerPath        = null;
            String              str_InputXML            = null;
            StringDictionary    strdict_AdditionalInfo  = null;
            ALEngineError       ALEngineErrorObj        = null;

            String              DestionationFolderPath          = null;
            String              SourceFolderPath                = null;
            String              DestionationFolderNetowrkPath   = null;
            String              SourceFolderNetowrkPath         = null;
            String              DestinationFileNeworkPath       = null;
            
            String              MediaProcessorNetWorkPath   = null;
            String              MediaProcessorPath          = null;
            String              MediaProcessSubFolderPath   = null;
            MediaRequestRouter  MediaRequestRouterObj       = null;
            Context             ContextObj                  = null;
            StringBuilder       sb_AdditionalInfo           = null;
            try
            {                
				SearchXML = new StringBuilder(100);
                sb_AdditionalInfo = new StringBuilder(100);
				SearchXML.AppendFormat(@"<SEARCH>
                                            <SESSIONINFO UserID='{0}' TenantName='{1}' Language='{2}' SessionID='{3}'/>
                                            <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
                                                <PRIVILEGENAME>READ</PRIVILEGENAME>	                                                    
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
                                                        <ALIAS>WORKFLOWPROCESSID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME FileRequired='YES'>PROCESSFILE</FIELDNAME>
                                                        <ALIAS>PROCESSFILE</ALIAS>
                                                    </EXPRESSION>		                                                    
                                                </SELECTLIST>	                                                    		                                                    
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <FILTERID>1</FILTERID>
                                                        <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                                
                                                    </FILTER>
                                                </FILTERLIST>
                                                <RELATIONSHIP NAME='PROCESS_ACTIVITY'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ACTIVITYID</FIELDNAME>
                                                            <ALIAS>ACTIVITYID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>SEQUENCENUMBER</FIELDNAME>
                                                            <ALIAS>SEQUENCENUMBER</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ENDACTIVITY</FIELDNAME>
                                                            <ALIAS>ENDACTIVITY</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='YES'>ACTIVITYFILE</FIELDNAME>
                                                            <ALIAS>ACTIVITYFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                    <FILTERLIST>
                                                        <FILTER>
                                                            <FILTERID>2</FILTERID>
                                                            <EXPRESSION>STARTACTIVITY</EXPRESSION>
                                                            <OPERATOR>EQUALTO</OPERATOR>
                                                            <EXPRESSIONVALUE>False</EXPRESSIONVALUE>
                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                                
                                                        </FILTER>                                                        
                                                    </FILTERLIST>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='WORKFLOWPROCESS_WORKFLOWINFORMATION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WORKFLOWINFORMATIONID</FIELDNAME>
                                                            <ALIAS>WORKFLOWINFORMATIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WORKFLOWCATEGORY</FIELDNAME>
                                                            <ALIAS>WORKFLOWCATEGORY</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WORKFLOWSUBCATEGORY</FIELDNAME>
                                                            <ALIAS>WORKFLOWSUBCATEGORY</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ROUTEMETHOD</FIELDNAME>
                                                            <ALIAS>ROUTEMETHOD</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </TARGETENTITY>
                                        </SEARCH>", UserID.ToString(), TenantName, CurrentLanguage, SessionID.ToString(), TaskID);


                OMInterfaceObj = new OMInterface(UserID, CurrentLanguage, TenantName);
				ErrorXML = OMInterfaceObj.GetHierarchialData(SearchXML.ToString(), ref TaskXML);
                if (ErrorXML != null)
                {
                    strdict_AdditionalInfo = new StringDictionary();
                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                    strdict_AdditionalInfo.Add("MOREINFO", "Error while reading process details for consolidation.");

                    ALEngineErrorObj = new ALEngineError();
                    ALEngineErrorObj.AppendError("WORKFLOWPROCESS_READ_FAILED", this.GetType().ToString(),
                                                    "ConsolidateDocument()", null, strdict_AdditionalInfo,
                                                    null, ErrorXML);
                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ErrorXML; 
                }
                else
                {
                    if (TaskXML != null)
                    {
                        sb_AdditionalInfo.Append(@"<ADDITIONALINFO>
                                                        <APINAME>ProcessManagerClass.ConsolidateDocument()</APINAME>
                                                        <THREADID>" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + "</THREADID>"
                                                        + TaskXML
                                                + "</ADDITIONALINFO>");
                        ContextObj = new Context(UserID, SessionID, TenantName, "ProcessManager", sb_AdditionalInfo.ToString());
                        //Step2:- Make a copy of the file uploaded by the task initiator.
                        String_Reader = new StringReader(TaskXML);
                        OutputXMLDoc  = new XPathDocument(String_Reader);
                        OutputXMLNav  = OutputXMLDoc.CreateNavigator();

						WorkflowProcessNav	  = OutputXMLNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                        TaskInitiatorsFileNav = WorkflowProcessNav.SelectSingleNode("FILESYSTEM/FILE[FIELDNAME='PROCESSFILE']");

                        TaskInitFilePath = TaskInitiatorsFileNav.SelectSingleNode("PATH").Value;

                        FileExtension = Path.GetExtension(TaskInitFilePath);

                        FilePathSplit = TaskInitFilePath.Split('\\');

                        DestFileName  = FilePathSplit[FilePathSplit.Length - 1];

                        #region Prepare the Folder Paths
                        MediaRequestRouterObj = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), CurrentLanguage, "ASSETLINKSYSTEM");
                        MediaRequestRouterObj.GetNetworkPath("MSOFFICEPROCESSOR", out MediaProcessorNetWorkPath, out MediaProcessorPath);

                        MediaProcessSubFolderPath   = "MSOfficeProcessor\\" + TenantName + "\\" + Guid.NewGuid();
                        DestionationFolderPath      = MediaProcessorPath + "\\" + MediaProcessSubFolderPath;
                        SourceFolderPath            = DestionationFolderPath + "\\SourceFiles";

                        DestionationFolderNetowrkPath = MediaProcessorNetWorkPath + "\\" + MediaProcessSubFolderPath;
                        SourceFolderNetowrkPath       = DestionationFolderNetowrkPath  + "\\SourceFiles";

                        DirectoryInfo DirectoryInfoObj = new DirectoryInfo(DestionationFolderPath);
                        if (!DirectoryInfoObj.Exists)
                            DirectoryInfoObj.Create();

                        DirectoryInfoObj = new DirectoryInfo(SourceFolderPath);
                        if (!DirectoryInfoObj.Exists)
                            DirectoryInfoObj.Create();
                        #endregion

                        DestFilePath = new StringBuilder();
                        DestFilePath.Append(DestionationFolderPath);
                        ApprovalsMode = WorkflowProcessNav.SelectSingleNode("WORKFLOWPROCESS_WORKFLOWINFORMATION/WORKFLOWINFORMATION/ROUTEMETHOD").Value;

                        if (ApprovalsMode.ToUpper() == "SIMULTANEOUS")
                        {
                            DestFilePath.AppendFormat("\\{0}", DestFileName);
                            DestinationFilePath = DestFilePath.ToString();

                            DestinationFileNeworkPath = DestionationFolderNetowrkPath + "\\" + DestFileName;
                            File.Move(TaskInitFilePath, DestFilePath.ToString());

                            ActivityNodeList = WorkflowProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY[ENDACTIVITY='False']");

                            //Step3:- Prepare an array of Strings which contains the file paths of all the files uploaded by the reviewer.
                            if (ActivityNodeList.Count > 0)
                            {
                                while (ActivityNodeList.MoveNext())
                                {
                                    ActivityNode = ActivityNodeList.Current;
                                    ReviewersFileNav = ActivityNode.SelectSingleNode("FILESYSTEM/FILE[FIELDNAME='ACTIVITYFILE']");

                                    //Step4:- Call the consolidate API with the correct parameters.
                                    if (ReviewersFileNav != null)
                                    {
                                        ReviewerFilePath        = ReviewersFileNav.SelectSingleNode("PATH").Value;
                                        FileInfo FileInfoObj    = new FileInfo(ReviewerFilePath);
                                        if (FileExtension.ToUpper() == ".PDF" || (FileExtension.ToUpper() == ".DOC") || (FileExtension.ToUpper() == ".DOCX"))
                                        {
                                            File.Move(ReviewerFilePath, SourceFolderPath + "\\" + FileInfoObj.Name.Replace(FileInfoObj.Extension, "") + ActivityNode.SelectSingleNode("ACTIVITYID").Value + FileInfoObj.Extension);
                                        }
                                        else if ((FileExtension.ToUpper() == ".XLS") || (FileExtension.ToUpper() == ".XLSX"))
                                        {
                                            //Hemant Karkande 23 Oct 2007 
                                            //Following code was added because excel does not allow to open 2 files with same name
                                            //even if they are in different folders.
                                            TempReviewerFileName = new StringBuilder();

                                            TempReviewerPath = ReviewerFilePath.Split('\\');
                                            TempFileName = TempReviewerPath[TempReviewerPath.Length - 1];

                                            TempPath = ReviewerFilePath.Substring(0, ReviewerFilePath.Length - TempFileName.Length);
                                            
                                            TempFileName = TempFileName.Substring(0, TempFileName.Length - 4);
                                            if (FileExtension.ToUpper() == ".XLSX")
                                            {
                                                TempReviewerFileName.AppendFormat("{0}\\{1}_{2}XLSX", SourceFolderPath, TempFileName, ActivityNode.SelectSingleNode("ACTIVITYID").Value);
                                            }
                                            else
                                            {
                                                TempReviewerFileName.AppendFormat("{0}\\{1}_{2}.XLS", SourceFolderPath, TempFileName, ActivityNode.SelectSingleNode("ACTIVITYID").Value);
                                            }
                                            
                                            //Here actualy it was required to be renamed but .net does not allow to rename so move is used.
                                            //Rename and Copy
                                            File.Move(ReviewerFilePath, TempReviewerFileName.ToString());

                                            //till here and now pass this moved file to consolidation.
                                        }

                                    }

                                    ReviewersFileNav = null;
                                }

                                //Invoke the Web Service
                                ALMSOfficeProcessorWS ALMSOfficeProcessorWSObj  = new ALMSOfficeProcessorWS();
                                ALMSOfficeProcessorWSObj.Url                    = ALCommon.GetWebServiceURL("ALMSOfficeProcessorWS");
                                
                                switch(FileExtension.ToUpper())
                                {
                                    case ".PDF":
                                        ErrorXML = ALMSOfficeProcessorWSObj.ConsolidatePDFDocument(SourceFolderNetowrkPath, DestinationFileNeworkPath);
                                        break;

                                    case ".DOC":
                                    case ".DOCX":
                                        ErrorXML = ALMSOfficeProcessorWSObj.ConsolidateWordDocument(SourceFolderNetowrkPath, DestinationFileNeworkPath);
                                        break;

                                    case ".XLSX":
                                    case ".XLS":
                                        ErrorXML = ALMSOfficeProcessorWSObj.ConsolidateExcelSpreadSheet(SourceFolderNetowrkPath, DestinationFileNeworkPath);
                                        break;
                                }

                                if (String.IsNullOrEmpty(ErrorXML))
                                {
                                    //Step5:- Associate the consolidated document with the WORKFLOWPROCESS entity.
                                    ProcessXML = new StringBuilder(50);
                                    ProcessXML.Append(@"<WORKFLOWPROCESS>
                                                            <WORKFLOWPROCESSID>" + TaskID + "</WORKFLOWPROCESSID>"
                                                                    + @"<FILESYSTEM>
                                                                <FILE Operation='CREATE'>
                                                                <FIELDNAME>CONSOLIDATEDFILE</FIELDNAME>
                                                                <NAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + "</NAME>"
                                                                     + "<DISPLAYNAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + "</DISPLAYNAME>"
                                                                     + "<PATH>" + ALCommon.TrimAndReplaceSpecialChars(DestinationFilePath) + "</PATH>"
                                                                     + "</FILE></FILESYSTEM></WORKFLOWPROCESS>");

                                    UpdateProcessXML = ProcessXML.ToString();
                                    ErrorXML = UpdateProcess(false, false, ref UpdateProcessXML);
                                    if (ErrorXML != null)
                                    {
                                        strdict_AdditionalInfo = new StringDictionary();
                                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                        strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode); 
                                        strdict_AdditionalInfo.Add("INPUTXML", UpdateProcessXML);                                        
                                        strdict_AdditionalInfo.Add("MOREINFO", "Error while updating Route Process for consolidation.");

                                        ALEngineErrorObj = new ALEngineError();
                                        ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                        "ConsolidateDocument()", null, strdict_AdditionalInfo,
                                                                        null, ErrorXML);
                                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                    }
                                }
                                else
                                {
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                    strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode); 
                                    strdict_AdditionalInfo.Add("INPUTXML", str_InputXML);                                    
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while invoking consolidation plugin.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                    "ConsolidateDocument()", null, strdict_AdditionalInfo,
                                                                    null, ErrorXML);
                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                }
                                //if (Directory.Exists(DestionationFolderPath))
                                //    Directory.Delete(DestionationFolderPath,true);
                                // 13 July 2013. Now Using the Common API to delete Folder.
                                SASMOMDirectory.DeleteFolder(DestionationFolderPath, true, ContextObj);

                            }
                        }//End of (ApprovalsMode.ToUpper() == "SIMULTANEOUS")
                        else if (ApprovalsMode.ToUpper() == "SEQUENTIAL")
                        {
                            ActivityNodeList = WorkflowProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY");
                            if (ActivityNodeList.Count > 0)
                            {
                                EndActivitySeqNo = WorkflowProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ENDACTIVITY='True']/SEQUENCENUMBER").Value;
                                GetLastFileUploadedForApprovals(WorkflowProcessNav, EndActivitySeqNo, out FileSystemNav, out FieldName);

                                ConsolidatedFileNav = FileSystemNav.SelectSingleNode("FILE[FIELDNAME='" + FieldName + "']");

                                ConsolidatedFilePath = ConsolidatedFileNav.SelectSingleNode("PATH").Value;
                                FilePathSplit = ConsolidatedFilePath.Split('\\');

                                ConsolidatedFileName = FilePathSplit[FilePathSplit.Length - 1];
                                DestFilePath.AppendFormat("\\{0}", ConsolidatedFileName);
                                DestinationFilePath = DestFilePath.ToString();

                                File.Copy(ConsolidatedFilePath, DestFilePath.ToString(), true);

                                ProcessXML = new StringBuilder(50);
                                ProcessXML.Append(@"<WORKFLOWPROCESS>
                                                    <WORKFLOWPROCESSID>" + TaskID + "</WORKFLOWPROCESSID>"
                                                    + @"<FILESYSTEM>
                                                        <FILE Operation='CREATE'>
                                                        <FIELDNAME>CONSOLIDATEDFILE</FIELDNAME>
                                                        <NAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + "</NAME>"
                                                     + "<DISPLAYNAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + "</DISPLAYNAME>"
                                                     + "<PATH>" + ALCommon.TrimAndReplaceSpecialChars(DestinationFilePath) + "</PATH>"
                                                     + "</FILE></FILESYSTEM></WORKFLOWPROCESS>");

                                UpdateProcessXML = ProcessXML.ToString();
                                ErrorXML = UpdateProcess(false, false, ref UpdateProcessXML);
                                if (ErrorXML != null)
                                {
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                    strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode); 
                                    strdict_AdditionalInfo.Add("INPUTXML", UpdateProcessXML);
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while updating Route Process for consolidation.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                    "ConsolidateDocument()", null, strdict_AdditionalInfo,
                                                                    null, ErrorXML);
                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                }

                                if (File.Exists(DestFilePath.ToString()))
                                    File.Delete(DestFilePath.ToString());
                            }
                        }

                    }//End of (TaskXML != null)
                }
                return ErrorXML;
            }
            catch (Exception ExceptionObject)
            {
                strdict_AdditionalInfo = new StringDictionary();
                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                strdict_AdditionalInfo.Add("MOREINFO", "Exception while consolidating document");

                ALEngineErrorObj = new ALEngineError();
                ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                "ConsolidateDocument()", null, strdict_AdditionalInfo,
                                                ExceptionObject, null);
                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;

                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
        }

        private void GetLastFileUploadedForApprovals(XPathNavigator WorkflowProcessNav, String ActivitySeqNo, out XPathNavigator FileSystemNav, out String FieldName)
        {
            XPathNavigator ProcessActivityNode = null;
            XPathNavigator ActivityNode        = null;
            XPathNavigator ActFileSystemNode   = null;
            StringBuilder  XPathQuery          = null;

                FileSystemNav = WorkflowProcessNav.SelectSingleNode("FILESYSTEM");
                FieldName     = "PROCESSFILE";

                ProcessActivityNode = WorkflowProcessNav.SelectSingleNode("PROCESS_ACTIVITY");
                XPathQuery = new StringBuilder();

                for (int loopCnt = Convert.ToInt16(ActivitySeqNo) - 1; loopCnt > 1; loopCnt--)
                {
                    XPathQuery.AppendFormat("ACTIVITY[SEQUENCENUMBER='{0}']", loopCnt);
                    ActivityNode = ProcessActivityNode.SelectSingleNode(XPathQuery.ToString());

                    if (ActivityNode != null)
                    {
                        ActFileSystemNode = ActivityNode.SelectSingleNode("FILESYSTEM");
                        if ((ActFileSystemNode != null) && (ActFileSystemNode.HasChildren))
                        {
                            FileSystemNav = ActFileSystemNode;
                            FieldName     = "ACTIVITYFILE";
                            break;
                        }
                    }
                    XPathQuery.Remove(0, XPathQuery.Length);
                }
        }
    }     
    
    class ActivityTLStatus
    {		
		private String			m_ActivityID;
		private TimeLineStatus m_ActivityStatus;
		 
		public String ActivityID
		{
			get { return m_ActivityID;}
			set { m_ActivityID = value;}
		}
		
		public TimeLineStatus Status
		{
			get {return m_ActivityStatus;}
			set {m_ActivityStatus = value;}
		}
		
		public ActivityTLStatus()
		{
		}

		public ActivityTLStatus(String AID, TimeLineStatus Status)
		{
			m_ActivityID = AID;
			m_ActivityStatus = Status;
		}		
    }     
    
	/// <summary>
	/// 
	/// </summary>
	public class Activity
    {
		/// <summary>
		/// 
		/// </summary>
		protected  String	m_ActivityID;
		/// <summary>
		/// 
		/// </summary>
		protected DateTime m_StartDate;
		/// <summary>
		/// 
		/// </summary>
		protected DateTime m_DueDate;
		/// <summary>
		/// 
		/// </summary>
		protected String  m_ActivityInfoID;
		/// <summary>
		/// 
		/// </summary>
		protected String  m_Duration;		
		/// <summary>
		/// 
		/// </summary>
		protected Boolean m_Modified;		
		/// <summary>
		/// 
		/// </summary>
		protected DateTime m_ActualEndDate;
        /// <summary>
        /// 
        /// </summary>
        protected String m_Status;

		/// <summary>
		/// 
		/// </summary>
		public Activity()
		{
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="ActivityID"></param>
		/// <param name="StartDate"></param>
		/// <param name="DueDate"></param>
		/// <param name="ActivityInfoID"></param>
		/// <param name="Duration"></param>
        /// <param name="Status"></param> 
		public Activity(String ActivityID,DateTime StartDate,DateTime DueDate,String ActivityInfoID,String Duration, String Status)
		{
			try
			{
					m_ActivityID = ActivityID;
					m_StartDate  = StartDate;
					m_DueDate	 = DueDate;	
					m_ActivityInfoID= ActivityInfoID;
					m_Duration		= Duration;
				    m_Status        = Status; 	
			}
			catch(Exception ExceptionObj) 
			{
				throw ExceptionObj;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ActivityID"></param>
		/// <param name="StartDate"></param>
		/// <param name="DueDate"></param>
		/// <param name="ActEndDate"></param>
		/// <param name="ActivityInfoID"></param>
		/// <param name="Duration"></param>
        /// <param name="Status"></param> 
		public Activity(String ActivityID, DateTime StartDate, DateTime DueDate, DateTime ActEndDate, String ActivityInfoID, String Duration, String Status)
		{
			try
			{
				m_ActivityID = ActivityID;
				m_StartDate = StartDate;
				m_DueDate = DueDate;
				m_ActivityInfoID = ActivityInfoID;
				m_Duration = Duration;
				m_ActualEndDate = ActEndDate;
                m_Status = Status;
			}
			catch(Exception ExceptionObj)
			{
				throw ExceptionObj;
			}
		}
		 
		 /// <summary>
		 /// 
		 /// </summary>
		public String ActivityID
		{
			get { return m_ActivityID;}
			set	{ m_ActivityID = value;}
		}

		/// <summary>
		/// 
		/// </summary>
		public DateTime StartDate
		{
			get
			{
				return m_StartDate;
			}
			set
			{
				m_StartDate = (DateTime)value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public DateTime DueDate
		{
			get
			{
				return m_DueDate;
			}
			set
			{
				m_DueDate = (DateTime)value;
			}
		}
		
		/// <summary>
		/// 
		/// </summary>
		public String ActivityInfoID
		{
			get {return m_ActivityInfoID ;}
			set { m_ActivityInfoID  = value;}
		}
		
		/// <summary>
		/// 
		/// </summary>
		public String Duration
		{
			get { return m_Duration;}
			set { m_Duration = value;}
		}			
		
		/// <summary>
		/// 
		/// </summary>
		public Boolean Modified
		{
			get {return m_Modified;}
			set {m_Modified = value;}
		}
		
		/// <summary>
		/// 
		/// </summary>
		public DateTime ActualEndDate
		{
			get
			{
				return m_ActualEndDate;
			}
		}

        /// <summary>
        /// 
        /// </summary>
        public String Status
        {
            get
            {
                return m_Status; 
            }
        }
    }    
    
    class ActivityFilter
    {
		private String m_ActivityID;
		
		public ActivityFilter(String ActivityID)
		{
			m_ActivityID = ActivityID;
		}
		
		public bool FilterByActivityID(Activity AObj)
		{
			return (AObj.ActivityID == m_ActivityID?true:false);
		}		
    }

	class ActivityDependancy : Activity
    {
		private String m_DependantActivityID;
		private String m_SourceActivityID;
		
		public ActivityDependancy():base ()
		{
			
		}
		
		public ActivityDependancy(String DependantActivityID,String SourceActivityID,String ActivityInfoID,	String Duration, String Status)
            : base(DependantActivityID, DateTime.Today, DateTime.Today, ActivityInfoID, Duration, Status)
		{
			m_SourceActivityID = SourceActivityID;
			m_DependantActivityID = DependantActivityID;
		}
		
		public String SourceActivityID
		{
			get {return m_SourceActivityID;}
			set { m_SourceActivityID = value;}
		}
		
		public String DependantActivityID
		{
			get { return m_DependantActivityID;}
			set {m_DependantActivityID= value;}
		}
    }
}

