using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.IO;
using Assetlink.Services.Engines;
using Assetlink.Diagnostic;
using Assetlink.Framework;
using Assetlink.Services.Plugins.PluginManger;
using System.Threading;
using System.Collections.Specialized; 
using System.Drawing;
using System.Drawing.Imaging;

using iTextSharp.text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.annotations;
using Assetlink.WebServices.ALMSOfficeProcessorWS;
using log4net;
using log4net.Config;
using Assetlink.IOProxy;
namespace Assetlink.ProcessManager
{
    using XML      = String;
    using ERRORXML = String;

    public partial class ProcessManagerClass
    {
        private static readonly ILog ProcessManager_LogObj = LogManager.GetLogger("ProcessManager");

        #region Public Methods

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method invokes the appropriate ReverseTimeline method based on the Mode Parameter,
        /// The mode can be either Edit or Create</Purpose>
        /// </summary>
        /// <param name="InputXML">The Input XML contains the Process details containing start and due dates of activities and subprocesses</param>
        /// <param name="Mode">The mode of the process can be either edit or create</param>
        /// <param name="ProcessDueDate">The new due date of process used for reverse calculation</param>
        /// <param name="d_ProcessDuration">The new duration after the reverse time line is calculated</param>
        /// <param name="ProcessStartDate"></param>
        /// <param name="UpdatedDateXML">if the </param>
        /// <returns>An Error XML if an error occurs</returns>
        public ERRORXML CalculateReverseTimeLine(ref XML InputXML, String Mode, DateTime ProcessDueDate, out Double d_ProcessDuration, out DateTime ProcessStartDate, out XML UpdatedDateXML)
        {
            ERRORXML        ErrorXml        = null;
            ALError         ExcepErrObj     = null;
            StringBuilder   sb_InputData    = null;

            try
            {
                UpdatedDateXML = String.Empty;
                if (Mode.ToUpper() == "CREATE")
                {
                    ErrorXml = CalculateReverseTimeLine_Create(ref InputXML, ProcessDueDate, out d_ProcessDuration, out ProcessStartDate);
                }
                else
                {
                    ErrorXml = CalculateReverseTimeLine_Edit(InputXML, ProcessDueDate, out UpdatedDateXML, out d_ProcessDuration, out ProcessStartDate);
                }
                return ErrorXml;
            }
            catch (Exception ExcepObj)
            {
                sb_InputData = new StringBuilder(10);
                sb_InputData.Append("InputXML = " + InputXML + " Mode = " + Mode + "");
                ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                             "Exception in Calculate Reverse TimeLine.", ErrorObjectType.PROCESS,
                                             "Entity", ErrorType.EXCEPTION, ExcepObj, sb_InputData.ToString());
                ErrorXml = ExcepErrObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXml);
                d_ProcessDuration = 0;
                ProcessStartDate = new DateTime();
                UpdatedDateXML = null; 
                return ErrorXml;
            }
        }

        /// <summary>
        /// <Purpose>Consolidate the multiple files for the Multi-FileRoute process.</Purpose>
        /// <Author>Parita Harniya</Author>
        /// </summary>
        /// <param name="TaskID">Task ID</param>
        /// <param name="ReviewFileID">Review File ID</param>
        /// <returns>Error if any else null</returns>
        public ERRORXML ConsolidateDocument_MultiFile(String TaskID, String ReviewFileID)
        {
            String              TaskXML                 = null;
            string[]            FilePathSplit           = null;
            ERRORXML            ErrorXML                = null;
            String              DestFileName            = null;
            String              TaskInitFilePath        = null;
            String              ReviewerFilePath        = null;
            String              ConsolidatedFilePath    = null;
            String              DestinationFilePath     = null;
            String              ApprovalsMode           = null;
            StringBuilder       DestFilePath            = null;
            StringReader        String_Reader           = null;
            XPathDocument       OutputXMLDoc            = null;
            XPathNavigator      OutputXMLNav            = null;
            XPathNavigator      ResponseNode            = null;
            XPathNavigator      WorkflowProcessNav      = null;
            XPathNodeIterator   ActivityNodeList        = null;
            XPathNavigator      xpNav_FileForApproval   = null;
            XPathNavigator      ReviewersFileNav        = null;
            StringBuilder       ProcessXML              = null;
            String              UpdateProcessXML        = null;
            String              FileExtension           = null;
            String              EndActivitySeqNo        = null;
            String              ConsolidatedFileName    = null;
            String              FieldName               = null;
            XPathNavigator      FileSystemNav           = null;
            XPathNavigator      ConsolidatedFileNav     = null;
            StringBuilder		SearchXML				= null;            
            OMInterface			OMInterfaceObj			= null;
            String              TempFileName            = null;
            StringBuilder       TempReviewerFileName    = null;
            String              TempPath                = null; 
            String[]            TempReviewerPath        = null;
            String              str_InputXML            = null;
            XPathNodeIterator   xpNodeIte_ReviewFiles   = null;
            String              str_ReviewFileID        = null;
            String              str_WebDavLocation      = null;
            OperationQueue      QMQueueObj              = null;
            Operation           OperObj                 = null;
            OMInterface         OMIObj                  = null;
            String              OperationXML            = null;
            String              str_ErrorType           = null;
            String              str_SolutionName        = null;
            StringDictionary    strdict_AdditionalInfo  = null;
            ALEngineError       ALEngineErrorObj        = null;
			String              processFileName         = null;
            String              MediaType               = null;
            bool                isMediaFile             = true;
            string              PDFProcessor            = "";
			
            String              DestionationFolderPath          = null;
            String              SourceFolderPath                = null;
            String              DestionationFolderNetowrkPath   = null;
            String              SourceFolderNetowrkPath         = null;
            String              DestinationFileNeworkPath       = null;
            
            String              MediaProcessorNetWorkPath   = null;
            String              MediaProcessorPath          = null;
            String              MediaProcessSubFolderPath   = null;
            MediaRequestRouter  MediaRequestRouterObj       = null;
            Context             ContextObj                  = null;
            StringBuilder       sb_AdditionalInfo           = null;
            try
            {
                //Step1:- Get the process details.
                SearchXML = new StringBuilder(100);
                sb_AdditionalInfo = new StringBuilder(100);
                
                SearchXML.Append(@"<SEARCH>
                                        <SESSIONINFO UserID='" + UserID + @"' TenantName='" + TenantName + @"' Language='" + CurrentLanguage + @"' SessionID='" + SessionID.ToString() + @"'/>
                                        <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>	                                                    
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
                                                    <ALIAS>WORKFLOWPROCESSID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
		                                            <FIELDNAME>SOLUTIONNAME</FIELDNAME>
		                                            <ALIAS>SOLUTIONNAME</ALIAS>
	                                            </EXPRESSION>
                                            </SELECTLIST>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <FILTERID>1</FILTERID>
                                                    <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>" + TaskID + @"</EXPRESSIONVALUE>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                                
                                                </FILTER>
                                            </FILTERLIST>
                                            <RELATIONSHIP NAME='WORKFLOWPROCESS_PROCESSREVIEWDETAILS'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>REVIEWFILEID</FIELDNAME>
                                                        <ALIAS>REVIEWFILEID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>WEBDAVLOCATION</FIELDNAME>
                                                        <ALIAS>WEBDAVLOCATION</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
			                                            <FIELDNAME>ANNOTATIONMETHOD</FIELDNAME>
			                                            <ALIAS>ANNOTATIONMETHOD</ALIAS>
		                                            </EXPRESSION> 
                                                    <EXPRESSION>
                                                        <FIELDNAME FileRequired='YES'>FILEFORAPPROVAL</FIELDNAME>
                                                        <ALIAS>FILEFORAPPROVAL</ALIAS>
                                                    </EXPRESSION>                                                    
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='TOPIC_REVIEWDETAILS'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>TOPICID</FIELDNAME>
                                                            <ALIAS>TOPICID</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                 </RELATIONSHIP>");
                if (!String.IsNullOrEmpty(ReviewFileID))
                {
                    SearchXML.Append(@"<FILTERLIST>
                                            <FILTER>
                                                <FILTERID>1</FILTERID>
                                                <EXPRESSION>REVIEWFILEID</EXPRESSION>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>" + ReviewFileID + @"</EXPRESSIONVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                                
                                            </FILTER>
                                        </FILTERLIST>");
                }

                    SearchXML.Append(@"</RELATIONSHIP>
                                            <RELATIONSHIP NAME='PROCESS_ACTIVITY'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>ACTIVITYID</FIELDNAME>
                                                        <ALIAS>ACTIVITYID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>SEQUENCENUMBER</FIELDNAME>
                                                        <ALIAS>SEQUENCENUMBER</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>ENDACTIVITY</FIELDNAME>
                                                        <ALIAS>ENDACTIVITY</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <FILTERID>2</FILTERID>
                                                        <EXPRESSION>STARTACTIVITY</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>False</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                                
                                                    </FILTER>                                                        
                                                </FILTERLIST>
                                                <RELATIONSHIP NAME='ACTIVITY_RESPONSE'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>RESPONSEID</FIELDNAME>
                                                            <ALIAS>RESPONSEID</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                    <RELATIONSHIP NAME='RESPONSE_ACTIVITYREVIEWDETAILS'>
                                                        <SELECTLIST>
                                                            <EXPRESSION>
                                                                <FIELDNAME>REVIEWDETAILSID</FIELDNAME>
                                                                <ALIAS>REVIEWDETAILSID</ALIAS>
                                                            </EXPRESSION>
                                                            <EXPRESSION>
                                                                <FIELDNAME FileRequired='YES'>FILEWITHCOMMENTS</FIELDNAME>
                                                                <ALIAS>FILEWITHCOMMENTS</ALIAS>
                                                            </EXPRESSION>
                                                        </SELECTLIST>
                                                    </RELATIONSHIP>
                                                    <RELATIONSHIP NAME='RESPONSE_PROCESSREVIEWDETAILS'>
                                                        <SELECTLIST>
                                                            <EXPRESSION>
                                                                <FIELDNAME>REVIEWFILEID</FIELDNAME>
                                                                <ALIAS>REVIEWFILEID</ALIAS>
                                                            </EXPRESSION>
                                                        </SELECTLIST>                                                            
                                                    </RELATIONSHIP>");
                if (!String.IsNullOrEmpty(ReviewFileID))
                {
                    SearchXML.Append(@"<FILTERLIST>
                                            <FILTER>
                                                <FILTERID>1</FILTERID>
                                                <EXPRESSION>REVIEWFILEID</EXPRESSION>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>" + ReviewFileID + @"</EXPRESSIONVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                                
						                        <RELATIONSHIPNAME>RESPONSE_PROCESSREVIEWDETAILS</RELATIONSHIPNAME>                            
                                            </FILTER>
                                        </FILTERLIST>");
                }
                                SearchXML.Append(@"</RELATIONSHIP>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='WORKFLOWPROCESS_WORKFLOWINFORMATION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WORKFLOWINFORMATIONID</FIELDNAME>
                                                            <ALIAS>WORKFLOWINFORMATIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WORKFLOWCATEGORY</FIELDNAME>
                                                            <ALIAS>WORKFLOWCATEGORY</ALIAS>
                                                        </EXPRESSION>			                                            
                                                         <EXPRESSION>
				                                            <FIELDNAME>ANNOTATIONMETHOD</FIELDNAME>
				                                            <ALIAS>ANNOTATIONMETHOD</ALIAS>
			                                            </EXPRESSION>       
                                                        <EXPRESSION>
                                                            <FIELDNAME>WORKFLOWSUBCATEGORY</FIELDNAME>
                                                            <ALIAS>WORKFLOWSUBCATEGORY</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ROUTEMETHOD</FIELDNAME>
                                                            <ALIAS>ROUTEMETHOD</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </TARGETENTITY>
                                        </SEARCH>");

                OMInterfaceObj = new OMInterface(UserID, CurrentLanguage, TenantName);
				ErrorXML = OMInterfaceObj.GetHierarchialData(SearchXML.ToString(), ref TaskXML);                
                if (ErrorXML != null)
                {
                    strdict_AdditionalInfo = new StringDictionary();
                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                    strdict_AdditionalInfo.Add("INPUTXML", SearchXML.ToString());
                    strdict_AdditionalInfo.Add("MOREINFO", "Error while reading process details for multifile consolidation.");

                    ALEngineErrorObj = new ALEngineError();
                    ALEngineErrorObj.AppendError("WORKFLOWPROCESS_READ_FAILED", this.GetType().ToString(),
                                                    "ConsolidateDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                    null, ErrorXML);
                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                }
                else
                {
                    if (TaskXML != null)
                    {
                        sb_AdditionalInfo.Append(@"<ADDITIONALINFO>
                                                        <APINAME>ProcessManagerClass.ConsolidateDocument_MultiFile()</APINAME>
                                                        <THREADID>" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + "</THREADID>"
                                                        + TaskXML 
                                                + "</ADDITIONALINFO>");
                        ContextObj = new Context(UserID, SessionID, TenantName, "ProcessManager", sb_AdditionalInfo.ToString());

                        String_Reader = new StringReader(TaskXML);
                        OutputXMLDoc  = new XPathDocument(String_Reader);
                        OutputXMLNav  = OutputXMLDoc.CreateNavigator();

						WorkflowProcessNav	  = OutputXMLNav.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                        str_SolutionName      = WorkflowProcessNav.SelectSingleNode("SOLUTIONNAME").Value;
                        xpNodeIte_ReviewFiles = WorkflowProcessNav.Select("WORKFLOWPROCESS_PROCESSREVIEWDETAILS/PROCESSREVIEWDETAILS");
                        PDFProcessor = WorkflowProcessNav.SelectSingleNode("WORKFLOWPROCESS_WORKFLOWINFORMATION/WORKFLOWINFORMATION/ANNOTATIONMETHOD").Value;
                        
                        ApprovalsMode = WorkflowProcessNav.SelectSingleNode("WORKFLOWPROCESS_WORKFLOWINFORMATION/WORKFLOWINFORMATION/ROUTEMETHOD").Value;

                        //Step2:- Iterate on process review details.
                        foreach (XPathNavigator xpNav_ProcessReviewDetails in xpNodeIte_ReviewFiles)
                        {
                            str_WebDavLocation = xpNav_ProcessReviewDetails.SelectSingleNode("WEBDAVLOCATION").Value;
                            xpNav_FileForApproval = xpNav_ProcessReviewDetails.SelectSingleNode("FILESYSTEM/FILE[FIELDNAME='FILEFORAPPROVAL']");
                            processFileName = xpNav_FileForApproval.SelectSingleNode("NAME").Value;
                            
                            //HemantK
                            //Read the annotation method from the process review details facet
                            if (xpNav_ProcessReviewDetails.SelectSingleNode("ANNOTATIONMETHOD") != null && !String.IsNullOrEmpty(xpNav_ProcessReviewDetails.SelectSingleNode("ANNOTATIONMETHOD").Value))
                                PDFProcessor = xpNav_ProcessReviewDetails.SelectSingleNode("ANNOTATIONMETHOD").Value;

                            isMediaFile = ALCommon.IsMediaFile(TenantName, processFileName, PDFProcessor, ref MediaType);
                            //if (isMediaFile && MediaType != null && MediaType.ToUpper() != "VIDEO")
                            if (isMediaFile && MediaType != null && MediaType.ToUpper() != "VIDEO" && PDFProcessor.ToUpper() == "ASSETLINK PROPRIETARY")
                            {
                                ErrorXML = CreateMediaConsolidatedDocument(TaskID, xpNav_ProcessReviewDetails, xpNav_FileForApproval, str_WebDavLocation, MediaType, true);
                                if (!String.IsNullOrEmpty(ErrorXML))
                                {
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                    strdict_AdditionalInfo.Add("INPUTXML", str_InputXML);
                                    strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode);
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while creating consolidated media file.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("ERROR_CREATING_CONSOLIDATED_MEDIA_FILE", this.GetType().ToString(),
                                                                    "ConsolidateDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                                    null, ErrorXML);
                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                }
                            }
                            else
                            {
                                
                                if (String.IsNullOrEmpty(str_WebDavLocation))
                                {
                                    str_ReviewFileID = xpNav_ProcessReviewDetails.SelectSingleNode("REVIEWFILEID").InnerXml;

                                    TaskInitFilePath = xpNav_FileForApproval.SelectSingleNode("PATH").Value;

                                    FileExtension = Path.GetExtension(TaskInitFilePath);

                                    FilePathSplit = TaskInitFilePath.Split('\\');

                                    DestFileName = FilePathSplit[FilePathSplit.Length - 1];

                                    #region Prepare the Folder Paths
                                    MediaRequestRouterObj = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), CurrentLanguage, "ASSETLINKSYSTEM");
                                    MediaRequestRouterObj.GetNetworkPath("MSOFFICEPROCESSOR", out MediaProcessorNetWorkPath, out MediaProcessorPath);

                                    MediaProcessSubFolderPath   = "MSOfficeProcessor\\" + TenantName + "\\" + Guid.NewGuid();
                                    DestionationFolderPath      = MediaProcessorPath + "\\" + MediaProcessSubFolderPath;
                                    SourceFolderPath            = DestionationFolderPath + "\\SourceFiles";

                                    DestionationFolderNetowrkPath = MediaProcessorNetWorkPath + "\\" + MediaProcessSubFolderPath;
                                    SourceFolderNetowrkPath = DestionationFolderNetowrkPath + "\\SourceFiles";

                                    DirectoryInfo DirectoryInfoObj = new DirectoryInfo(DestionationFolderPath);
                                    if (!DirectoryInfoObj.Exists)
                                        DirectoryInfoObj.Create();

                                    DirectoryInfoObj = new DirectoryInfo(SourceFolderPath);
                                    if (!DirectoryInfoObj.Exists)
                                        DirectoryInfoObj.Create();
                                    #endregion

                                    DestFilePath = new StringBuilder();
                                    DestFilePath.Append(DestionationFolderPath);

                                    // Step3:- For Simultaneous Route Method
                                    if (ApprovalsMode.ToUpper() == "SIMULTANEOUS")
                                    {
                                        DestFilePath.AppendFormat("\\{0}", DestFileName);
                                        DestinationFilePath = DestFilePath.ToString();

                                        DestinationFileNeworkPath = DestionationFolderNetowrkPath + "\\" + DestFileName;
                                        File.Move(TaskInitFilePath, DestFilePath.ToString());

                                        ActivityNodeList = WorkflowProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY[ENDACTIVITY='False']/ACTIVITY_RESPONSE/RESPONSE[RESPONSE_PROCESSREVIEWDETAILS/PROCESSREVIEWDETAILS/REVIEWFILEID='"
                                            + str_ReviewFileID + "']");

                                        //Step4:- Iterate on file responses.
                                        if (ActivityNodeList.Count > 0)
                                        {
                                            while (ActivityNodeList.MoveNext())
                                            {
                                                ResponseNode = ActivityNodeList.Current;
                                                ReviewersFileNav = ResponseNode.SelectSingleNode("RESPONSE_ACTIVITYREVIEWDETAILS/ACTIVITYREVIEWDETAILS/FILESYSTEM/FILE[FIELDNAME='FILEWITHCOMMENTS']");

                                                if (ReviewersFileNav != null)
                                                {
                                                    ReviewerFilePath = ReviewersFileNav.SelectSingleNode("PATH").Value;

                                                    FileInfo FileInfoObj = new FileInfo(ReviewerFilePath);
                                                    if (FileExtension.ToUpper() == ".PDF" || (FileExtension.ToUpper() == ".DOC") || (FileExtension.ToUpper() == ".DOCX"))
                                                    {
                                                        File.Move(ReviewerFilePath, SourceFolderPath + "\\" + FileInfoObj.Name.Replace(FileInfoObj.Extension, "") + ResponseNode.SelectSingleNode("RESPONSEID").Value + FileInfoObj.Extension);
                                                    }
                                                    else if ((FileExtension.ToUpper() == ".XLS") || (FileExtension.ToUpper() == ".XLSX"))
                                                    {
                                                        //Following code was added because excel does not allow to open 2 files with same name
                                                        //even if they are in different folders.
                                                        TempReviewerFileName = new StringBuilder();

                                                        TempReviewerPath = ReviewerFilePath.Split('\\');
                                                        TempFileName = TempReviewerPath[TempReviewerPath.Length - 1];

                                                        TempPath = ReviewerFilePath.Substring(0, ReviewerFilePath.Length - TempFileName.Length);

                                                        TempFileName = TempFileName.Substring(0, TempFileName.Length - 4);
                                                        if (FileExtension.ToUpper() == ".XLSX")
                                                        {
                                                            TempReviewerFileName.AppendFormat("{0}\\{1}_{2}XLSX", SourceFolderPath, TempFileName, ResponseNode.SelectSingleNode("RESPONSEID").Value);
                                                        }
                                                        else
                                                        {
                                                            TempReviewerFileName.AppendFormat("{0}\\{1}_{2}.XLS", SourceFolderPath, TempFileName, ResponseNode.SelectSingleNode("RESPONSEID").Value);
                                                        }

                                                        //Here actualy it was required to be renamed but .net does not allow to rename so move is used.
                                                        //Rename and Copy
                                                        File.Move(ReviewerFilePath, TempReviewerFileName.ToString());
                                                        //till here and now pass this moved file to consolidation.
                                                    }

                                                }

                                                ReviewersFileNav = null;
                                            }

                                            //Step5:- Call the consolidate API with the correct parameters.
                                            //Invoke plugin service
                                            if ((FileExtension.ToUpper() == ".PDF") && (ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "SolutionsInstalled") == "1"))
                                            {
                                                Thread.Sleep(10000);
                                            }

                                            //Invoke the Web Service
                                            ALMSOfficeProcessorWS ALMSOfficeProcessorWSObj  = new ALMSOfficeProcessorWS();
                                            ALMSOfficeProcessorWSObj.Url                    = ALCommon.GetWebServiceURL("ALMSOfficeProcessorWS");
                                            
                                            switch(FileExtension.ToUpper())
                                            {
                                                case ".PDF":
                                                    ErrorXML = ALMSOfficeProcessorWSObj.ConsolidatePDFDocument(SourceFolderNetowrkPath, DestinationFileNeworkPath);
                                                    break;

                                                case ".DOC":
                                                case ".DOCX":
                                                    ErrorXML = ALMSOfficeProcessorWSObj.ConsolidateWordDocument(SourceFolderNetowrkPath, DestinationFileNeworkPath);
                                                    break;

                                                case ".XLSX":
                                                case ".XLS":
                                                    ErrorXML = ALMSOfficeProcessorWSObj.ConsolidateExcelSpreadSheet(SourceFolderNetowrkPath, DestinationFileNeworkPath);
                                                    break;
                                            }
                                            

                                            if (String.IsNullOrEmpty(ErrorXML))
                                            {
                                                String str_FileSize = new FileInfo(DestinationFilePath).Length.ToString();

                                                //Step6:- Update the facet process review details to update consolidated file.
                                                ProcessXML = new StringBuilder(50);
                                                ProcessXML.Append(@"<PROCESSREVIEWDETAILS MODIFIED='TRUE'>
                                                                <REVIEWFILEID>" + str_ReviewFileID + @"</REVIEWFILEID>
                                                                <FILESYSTEM>
                                                                    <FILE Operation='CREATE'>
                                                                        <FIELDNAME>CONSOLIDATEDFILE</FIELDNAME>
                                                                        <NAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</NAME>
                                                                        <DISPLAYNAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</DISPLAYNAME>
                                                                        <PATH>" + ALCommon.TrimAndReplaceSpecialChars(DestinationFilePath) + @"</PATH>
                                                                        <FILESIZE>" + str_FileSize + @"</FILESIZE>
                                                                    </FILE>
                                                                </FILESYSTEM>
                                                            </PROCESSREVIEWDETAILS>");
                                                UpdateProcessXML = ProcessXML.ToString();

                                                OMIObj = null;
                                                QMQueueObj = null;
                                                OperObj = null;
                                                OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
                                                QMQueueObj = new OperationQueue(false);
                                                OperObj = new Operation("FACET", "UPDATEDATA");
                                                OperObj.AddParameter("INPUTXML", UpdateProcessXML);
                                                QMQueueObj.AddOperation(OperObj);
                                                OperationXML = QMQueueObj.Xml;

                                                ErrorXML = OMIObj.Execute(ref OperationXML);
                                                if (ErrorXML != null)
                                                {
                                                    str_ErrorType = ALCommon.GetInnerValue(ref ErrorXML, "<ERRORTYPE>", "</ERRORTYPE>");
                                                    if (!String.IsNullOrEmpty(str_ErrorType) && !str_ErrorType.ToUpper().Equals("WARNING"))
                                                    {
                                                        strdict_AdditionalInfo = new StringDictionary();
                                                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                                        strdict_AdditionalInfo.Add("INPUTXML", OperationXML);
                                                        strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode);
                                                        strdict_AdditionalInfo.Add("MOREINFO", "Error while updating Mulit-File Route Process for consolidation.");

                                                        ALEngineErrorObj = new ALEngineError();
                                                        ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                                        "ConsolidateDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                                                        null, ErrorXML);
                                                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                                    }
                                                    else
                                                        ErrorXML = null; 
                                                }
                                            }
                                            else
                                            {
                                                strdict_AdditionalInfo = new StringDictionary();
                                                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                                strdict_AdditionalInfo.Add("INPUTXML", str_InputXML);
                                                strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode);
                                                strdict_AdditionalInfo.Add("MOREINFO", "Error while invoking consolidation plugin.");

                                                ALEngineErrorObj = new ALEngineError();
                                                ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                                "ConsolidateDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                                                null, ErrorXML);
                                                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                            }

                                            //if (Directory.Exists(DestionationFolderPath))
                                            //    Directory.Delete(DestionationFolderPath, true);
                                            // 13 July 2013 . Will be Using CommonDeleteDirectory Function.
                                            SASMOMDirectory.DeleteFolder(DestionationFolderPath, true, ContextObj);
                                        }
                                    }//End of (ApprovalsMode.ToUpper() == "SIMULTANEOUS")
                                    //Step7:- If Sequential Route Method
                                    else if (ApprovalsMode.ToUpper() == "SEQUENTIAL")
                                    {
                                        ActivityNodeList = WorkflowProcessNav.Select("PROCESS_ACTIVITY/ACTIVITY");
                                        //Step8:- Iterate on Activity.
                                        if (ActivityNodeList.Count > 0)
                                        {
                                            //Step9:- Get the last uploaded file.
                                            EndActivitySeqNo = WorkflowProcessNav.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ENDACTIVITY='True']/SEQUENCENUMBER").Value;
                                            GetLastFileUploadedForApprovals_MultiFile(WorkflowProcessNav, str_ReviewFileID, EndActivitySeqNo, out FileSystemNav, out FieldName);

                                            if ((FileSystemNav != null) && (!String.IsNullOrEmpty(FieldName)))
                                                ConsolidatedFileNav = FileSystemNav.SelectSingleNode("FILE[FIELDNAME='" + FieldName + "']");
                                            else
                                            {
                                                //FileSystemNav   = xpNav_FileForApproval ;
                                                ConsolidatedFileNav = xpNav_FileForApproval;
                                                FieldName           = "FILEFORAPPROVAL";
                                            }

                                            ConsolidatedFilePath = ConsolidatedFileNav.SelectSingleNode("PATH").Value;
                                            FilePathSplit = ConsolidatedFilePath.Split('\\');

                                            ConsolidatedFileName = FilePathSplit[FilePathSplit.Length - 1];
                                            DestFilePath.AppendFormat("\\{0}", ConsolidatedFileName);
                                            DestinationFilePath = DestFilePath.ToString();

                                            File.Copy(ConsolidatedFilePath, DestFilePath.ToString(), true);

                                            //Step10:- Update the facet process review details to update consolidated file.
                                            ProcessXML = new StringBuilder(50);
                                            ProcessXML.Append(@"<PROCESSREVIEWDETAILS MODIFIED='TRUE'>
                                                            <REVIEWFILEID>" + str_ReviewFileID + @"</REVIEWFILEID>
                                                            <FILESYSTEM>
                                                                <FILE Operation='CREATE'>
                                                                    <FIELDNAME>CONSOLIDATEDFILE</FIELDNAME>
                                                                    <NAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</NAME>
                                                                    <DISPLAYNAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</DISPLAYNAME>
                                                                    <PATH>" + ALCommon.TrimAndReplaceSpecialChars(DestinationFilePath) + @"</PATH>
                                                                </FILE>
                                                            </FILESYSTEM>
                                                        </PROCESSREVIEWDETAILS>");

                                            UpdateProcessXML = ProcessXML.ToString();

                                            OMIObj = null;
                                            QMQueueObj = null;
                                            OperObj = null;
                                            OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
                                            QMQueueObj = new OperationQueue(false);
                                            OperObj = new Operation("FACET", "UPDATEDATA");
                                            OperObj.AddParameter("INPUTXML", UpdateProcessXML);
                                            QMQueueObj.AddOperation(OperObj);
                                            OperationXML = QMQueueObj.Xml;

                                            ErrorXML = OMIObj.Execute(ref OperationXML);

                                            if (ErrorXML != null)
                                            {
                                                str_ErrorType = ALCommon.GetInnerValue(ref ErrorXML, "<ERRORTYPE>", "</ERRORTYPE>");
                                                if (!String.IsNullOrEmpty(str_ErrorType) && !str_ErrorType.ToUpper().Equals("WARNING"))
                                                {
                                                    strdict_AdditionalInfo = new StringDictionary();
                                                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                                                    strdict_AdditionalInfo.Add("INPUTXML", OperationXML);
                                                    strdict_AdditionalInfo.Add("APPROVALMODE", ApprovalsMode);
                                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while updating Mulit-File Route Process for consolidation.");

                                                    ALEngineErrorObj = new ALEngineError();
                                                    ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                                    "ConsolidateDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                                                    null, ErrorXML);
                                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                                }
                                                else
                                                    ErrorXML = null; 
                                            }
                                            if (File.Exists(DestFilePath.ToString()))
                                                File.Delete(DestFilePath.ToString());
                                        }
                                    }//End of (ApprovalsMode.ToUpper() == "SEQUENTIAL")
                                }//End of (String.IsNullOrEmpty(str_WebDavLocation))
                            }
                        }//End of foreach
                    }//End of (TaskXML != null)
                }
                return ErrorXML;
            }
            catch (Exception ExceptionObject)
            {
                strdict_AdditionalInfo = new StringDictionary();
                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", TaskID);
                strdict_AdditionalInfo.Add("MOREINFO", "Exception while consolidating document");

                ALEngineErrorObj = new ALEngineError();
                ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                "ConsolidateDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                ExceptionObject, null);
                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;

                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
        }

        /// <summary>
        /// CreateMediaConsolidatedDocument
        /// </summary>
        /// <param name="ObjectID"></param>
        /// <param name="ObjectNavigator"></param>
        /// <param name="xpNav_File"></param>
        /// <param name="tempFieldValue"></param>
        /// <param name="MediaType"></param>
        /// <param name="isMultiFile"></param>
        /// <returns></returns>
        public ERRORXML CreateMediaConsolidatedDocument(String ObjectID, XPathNavigator ObjectNavigator, XPathNavigator xpNav_File, String tempFieldValue, String MediaType, Boolean isMultiFile)
        {
            // 29 Sep 10 : Ankit : For Creating consolidated media file
            ERRORXML ErrorXMLString = null; //To Return Error.
            ALEngineError ALErrObj = null; //To Catch the error.
            String TaskInitFilePath = null;
            String DestinationFilePath = null;
            StringBuilder DestFilePath = null;
            string[] FilePathSplit = null;
            String DestFileName = null;
            String FileExtension = null;
            XPathNavigator xpNav_Topic = null;
            String TopicID = null;
            StringDictionary strdict_AdditionalInfo = null;
            Boolean PostAdded = false;

            try
            {
                if (!isMultiFile)
                    xpNav_Topic = ObjectNavigator.SelectSingleNode("WORKFLOWPROCESS_MEDIATOPIC/TOPIC/TOPICID");
                else
                    xpNav_Topic = ObjectNavigator.SelectSingleNode("TOPIC_REVIEWDETAILS/TOPIC/TOPICID");

                if (xpNav_Topic != null)
                {
                    TopicID = xpNav_Topic.Value;

                    TaskInitFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                    FileExtension = Path.GetExtension(TaskInitFilePath);
                    FilePathSplit = TaskInitFilePath.Split('\\');
                    //DestFileName = FilePathSplit[FilePathSplit.Length - 1];
                    DestFileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                    DestFilePath = new StringBuilder();

                    DestFilePath.Append(ALCommon.GetTempPath() + ObjectID);
                    DirectoryInfo D = new DirectoryInfo(DestFilePath.ToString());
                    if (!D.Exists)
                        D.Create();

                    DestFilePath.AppendFormat("\\{0}", DestFileName);
                    DestinationFilePath = DestFilePath.ToString();

                    ErrorXMLString = GetMediaConsolidatedDocument(TopicID, xpNav_File, MediaType, ref DestinationFilePath, ref PostAdded);                    
                    if (String.IsNullOrEmpty(ErrorXMLString))
                    {
                        String str_FileSize = new FileInfo(DestinationFilePath).Length.ToString();

                        if (!String.IsNullOrEmpty(MediaType) && MediaType.ToUpper() == "IMAGE" && PostAdded)
                            DestFileName = DestFileName.Substring(0, DestFileName.LastIndexOf(".")) + ".pdf";

                        if (!isMultiFile)
                            ErrorXMLString = StoreMediaConsolidatedDocument(ObjectID, tempFieldValue, DestFileName, DestinationFilePath, str_FileSize);
                        else
                        {
                            ObjectID = ObjectNavigator.SelectSingleNode("REVIEWFILEID").InnerXml;
                            ErrorXMLString = StoreMediaConsolidatedDocument_MultiFile(ObjectID, tempFieldValue, DestFileName, DestinationFilePath, str_FileSize);
                        }
                    }

                    if (File.Exists(DestinationFilePath))
                        File.Delete(DestinationFilePath);
                }
            }
            catch (Exception ex)
            {
                ErrorXMLString = ex.Message;
                strdict_AdditionalInfo = new StringDictionary();
                strdict_AdditionalInfo.Add("OBJECTID", ObjectID);
                strdict_AdditionalInfo.Add("ISMULTIFILEAPPROVAL", isMultiFile.ToString());
                strdict_AdditionalInfo.Add("MOREINFO", "Error while creating media consolidated file.");

                ALErrObj = new ALEngineError();
                ALErrObj.AppendError("CREATE_MEDIA_CONSOLIDATED_DOCUMENT_FAILED", this.GetType().ToString(),
                                                "CreateMediaConsolidatedDocument()", null, strdict_AdditionalInfo,
                                                null, ErrorXMLString);
                ErrorXMLString = ALErrObj.ErrorCollectionXML;
            }
            return ErrorXMLString;
        }

        private ERRORXML StoreMediaConsolidatedDocument(String ObjectID, String str_SolutionName, String DestFileName, String DestinationFilePath, String str_FileSize)
        {
            ERRORXML ErrorXMLString = null; //To Return Error.
            ALEngineError ALErrObj = null; //To Catch the error.
            OMInterface OMIObj = null;
            OperationQueue QMQueueObj = null;
            Operation OperObj = null;
            String OperationXML = null;
            StringBuilder ProcessXML = null;
            StringDictionary strdict_AdditionalInfo = null;

            //Update the entity workflowprocess to update consolidated file.
            ProcessXML = new StringBuilder(50);
            //Solution is also added below to avoid OM Error of Deferred Query Execution
            ProcessXML.Append(@"<WORKFLOWPROCESS>
                                            <WORKFLOWPROCESSID>" + ObjectID + @"</WORKFLOWPROCESSID>
                                            <SOLUTIONNAME>" + str_SolutionName + @"</SOLUTIONNAME>
                                            <FILESYSTEM>
                                                <FILE Operation='CREATE'>
                                                    <FIELDNAME>CONSOLIDATEDFILE</FIELDNAME>
                                                    <NAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</NAME>
                                                    <DISPLAYNAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</DISPLAYNAME>
                                                    <PATH>" + ALCommon.TrimAndReplaceSpecialChars(DestinationFilePath) + @"</PATH>
                                                    <FILESIZE>" + str_FileSize + @"</FILESIZE>
                                                </FILE>
                                            </FILESYSTEM>
                                        </WORKFLOWPROCESS>");

            OMIObj = null;
            QMQueueObj = null;
            OperObj = null;
            OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
            QMQueueObj = new OperationQueue(false);
            OperObj = new Operation("ENTITY", "UPDATEDATA");
            OperObj.AddParameter("INPUTXML", ProcessXML.ToString());
            QMQueueObj.AddOperation(OperObj);
            OperationXML = QMQueueObj.Xml;

            ErrorXMLString = OMIObj.Execute(ref OperationXML);
            if (ErrorXMLString != null)
            {
                strdict_AdditionalInfo = new StringDictionary();
                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", ObjectID);
                strdict_AdditionalInfo.Add("INPUTXML", ProcessXML.ToString());               
                strdict_AdditionalInfo.Add("MOREINFO", "Error while storing consolidated file in Database for Single File Approval.");

                ALErrObj = new ALEngineError();
                ALErrObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                "StoreMediaConsolidatedDocument()", null, strdict_AdditionalInfo,
                                                null, ErrorXMLString);
                ErrorXMLString = ALErrObj.ErrorCollectionXML;
            }
            return ErrorXMLString;
        }

        private ERRORXML StoreMediaConsolidatedDocument_MultiFile(String ObjectID, String tempStr_WDLocation, String DestFileName, String DestinationFilePath, String str_FileSize)
        {
            ERRORXML ErrorXML = null; //To Return Error.
            ALEngineError ALErrObj = null; //To Catch the error.
            OMInterface OMIObj = null;
            OperationQueue QMQueueObj = null;
            Operation OperObj = null;
            String OperationXML = null;
            StringBuilder ProcessXML = null;
            StringDictionary strdict_AdditionalInfo = null;

            //Update the facet process review details to update consolidated file.
            ProcessXML = new StringBuilder(50);
            ProcessXML.Append(@"<PROCESSREVIEWDETAILS MODIFIED='TRUE'>
                        <REVIEWFILEID>" + ObjectID + @"</REVIEWFILEID>
                        <WEBDAVLOCATION>" + tempStr_WDLocation + @"</WEBDAVLOCATION>
                        <FILESYSTEM>
                            <FILE Operation='CREATE'>
                                <FIELDNAME>CONSOLIDATEDFILE</FIELDNAME>
                                <NAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</NAME>
                                <DISPLAYNAME>" + ALCommon.TrimAndReplaceSpecialChars(DestFileName) + @"</DISPLAYNAME>
                                <PATH>" + ALCommon.TrimAndReplaceSpecialChars(DestinationFilePath) + @"</PATH>
                                <FILESIZE>" + str_FileSize + @"</FILESIZE>
                            </FILE>
                        </FILESYSTEM>
                    </PROCESSREVIEWDETAILS>");

            OMIObj = null;
            QMQueueObj = null;
            OperObj = null;
            OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
            QMQueueObj = new OperationQueue(false);
            OperObj = new Operation("FACET", "UPDATEDATA");
            OperObj.AddParameter("INPUTXML", ProcessXML.ToString());
            QMQueueObj.AddOperation(OperObj);
            OperationXML = QMQueueObj.Xml;

            ErrorXML = OMIObj.Execute(ref OperationXML);
            if (ErrorXML != null)
            {
                strdict_AdditionalInfo = new StringDictionary();
                strdict_AdditionalInfo.Add("REVIEWFILEID", ObjectID);
                strdict_AdditionalInfo.Add("INPUTXML", ProcessXML.ToString());
                strdict_AdditionalInfo.Add("MOREINFO", "Error while storing consolidated file in Database for Multi File Approval.");

                ALErrObj = new ALEngineError();
                ALErrObj.AppendError("PROCESSREVIEWDETAILS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                "StoreMediaConsolidatedDocument_MultiFile()", null, strdict_AdditionalInfo,
                                                null, ErrorXML);
                ErrorXML = ALErrObj.ErrorCollectionXML;
            }
            return ErrorXML;
        }

        /// <summary>
        /// GetMediaConsolidatedDocument
        /// </summary>
        /// <param name="DiscussionId"></param>
        /// <param name="xpNav_File"></param>
        /// <param name="MediaType"></param>
        /// <param name="DestinationFilePath"></param>
        /// <param name="PostAdded"></param>
        /// <returns></returns>
        public ERRORXML GetMediaConsolidatedDocument(String DiscussionId, XPathNavigator xpNav_File, String MediaType, ref String DestinationFilePath, ref Boolean PostAdded)
        {
            ERRORXML            ErrorXML                = null;
            String              str_TopicXML            = null;
            StringBuilder       SearchBuilder           = null;
            OMInterface         OMInterfaceObj          = null;
            XPathDocument       xpDoc                   = null;
            XPathNavigator      xpNav                   = null;
            String              srcPath                 = null;
            bool                IsImage                 = false;
            String              annotations             = null;
            XPathNodeIterator   PostList                = null;
            XPathNavigator      xpPostNav               = null;
            XPathNavigator      xpPostAttachmentsNav    = null;
            String              tFile                   = null;
            String              tempFile                = null;
            ALEngineError       ALErrObj                = null; //To Catch the error.
            StringDictionary    strdict_AdditionalInfo  = null;

            try
            {
                if (!String.IsNullOrEmpty(MediaType) && MediaType.ToUpper() == "IMAGE")
                    IsImage = true;

                SearchBuilder = new StringBuilder();
                SearchBuilder.Append("<SEARCH>");
                SearchBuilder.AppendFormat("<SESSIONINFO TenantName=\"{0}\" Language=\"{1}\" UserID=\"{2}\" SessionID=\"{3}\"/>", TenantName, CurrentLanguage, UserID, SessionID);
                SearchBuilder.Append(@"<TARGETOBJECT>TOPIC</TARGETOBJECT>
                               <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                               <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                               <TARGETENTITY>
                               <FILTERLIST>
                               <FILTER>
                                <FILTERID>1</FILTERID>
                                <EXPRESSION>TOPICID</EXPRESSION>
                                <OPERATOR>EQUALTO</OPERATOR>");
                SearchBuilder.AppendFormat("<EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>", DiscussionId.ToString());
                SearchBuilder.Append(@"<ISCASESENSITIVE>False</ISCASESENSITIVE>
                               </FILTER>
                               </FILTERLIST>
                               <SELECTLIST>
                                <EXPRESSION><FIELDNAME>TOPICID</FIELDNAME><ALIAS>TOPICID</ALIAS></EXPRESSION>");
                if (IsImage)
                    SearchBuilder.Append("<EXPRESSION><FIELDNAME FileRequired='YES'>TOPICATTACHMENTS</FIELDNAME><ALIAS>TOPICATTACHMENTS</ALIAS></EXPRESSION>");
                SearchBuilder.Append(@"</SELECTLIST>
                               <RELATIONSHIP NAME='TOPICPOST'>
                                <SELECTLIST><EXPRESSION><FIELDNAME FileRequired='YES'>POSTATTACHMENTS</FIELDNAME><ALIAS>POSTATTACHMENTS</ALIAS></EXPRESSION>
                                </SELECTLIST>
                               </RELATIONSHIP>
                               </TARGETENTITY>
                               </SEARCH>");

                OMInterfaceObj = new OMInterface(UserID, CurrentLanguage, TenantName);
                ErrorXML = OMInterfaceObj.GetHierarchialData(SearchBuilder.ToString(), ref str_TopicXML);

                xpDoc = new XPathDocument(new StringReader(str_TopicXML));
                xpNav = xpDoc.CreateNavigator();

                if (IsImage)
                    xpNav_File = xpNav.SelectSingleNode("TOPICCOLLECTION/TOPIC/FILESYSTEM/BINDER/FILE");

                srcPath = xpNav_File.SelectSingleNode("PATH").Value;

                PostList = xpNav.Select("TOPICCOLLECTION/TOPIC/TOPICPOST/POST");
                String postAttachmentPath = null;
                String AppendAnnotationsXML = "";
                String postAttachmentContent = "";                
                String postAttachmentAnnotations = "<annotations>";
                String postAttachmentComments = "<comments>";
                String printWidth = null;
                String printHeight = null;
                XPathDocument xpDoc_PostAttachment = null;
                XPathNavigator xpNav_PostAttachment = null;

                if (PostList != null && PostList.Count > 0)
                {
                    PostAdded = true;
                    while (PostList.MoveNext())
                    {
                        xpPostNav = PostList.Current;
                        xpPostAttachmentsNav = xpPostNav.SelectSingleNode("FILESYSTEM/BINDER/FILE/PATH");
                        if (xpPostAttachmentsNav != null)
                            postAttachmentPath = xpPostAttachmentsNav.Value;
                        if (!String.IsNullOrEmpty(postAttachmentPath))
                        {
                            xpDoc_PostAttachment = new XPathDocument(new StreamReader(postAttachmentPath));
                            xpNav_PostAttachment = xpDoc_PostAttachment.CreateNavigator();
                            xpNav_PostAttachment = xpNav_PostAttachment.SelectSingleNode("POST");

                            if (String.IsNullOrEmpty(printWidth))
                                printWidth = xpNav_PostAttachment.GetAttribute("w", "");

                            if (String.IsNullOrEmpty(printHeight))
                                printHeight = xpNav_PostAttachment.GetAttribute("h", "");

                            AppendAnnotationsXML += xpNav_PostAttachment.OuterXml;
                            postAttachmentContent = xpNav_PostAttachment.InnerXml;
                            postAttachmentAnnotations += ALCommon.GetInnerValue(ref postAttachmentContent, "<annotations>", "</annotations>");
                            postAttachmentComments += ALCommon.GetInnerValue(ref postAttachmentContent, "<comments>", "</comments>");
                        }
                    }

                    postAttachmentAnnotations += "</annotations>";
                    postAttachmentComments += "</comments>";


                    annotations = "<POST w='" + printWidth + "' h='" + printHeight + "'>" + postAttachmentAnnotations + postAttachmentComments + "</POST>";

                    XmlDocument doc = new XmlDocument();
                    doc.LoadXml(annotations);

                    XmlNodeList annotationList = doc.GetElementsByTagName("annotations")[0].ChildNodes;

                    XmlNodeList commentList = doc.GetElementsByTagName("comments")[0].ChildNodes;

                    UTF8Encoding encoding = new UTF8Encoding();

                    String annotationValue = "";
                    Hashtable localComments = new Hashtable();

                    String sourcePath = srcPath;

                    if (!IsImage)
                        tempFile = DestinationFilePath;
                    else
                    {
                        tempFile = DestinationFilePath.Substring(0, DestinationFilePath.LastIndexOf(".")) + ".pdf";
                        DestinationFilePath = tempFile;
                    }

                    if (IsImage)
                    {

                        float pageW = float.Parse(doc.DocumentElement.Attributes["w"].Value);
                        float pageH = float.Parse(doc.DocumentElement.Attributes["h"].Value);

                        Document imagePDF = new Document(new iTextSharp.text.Rectangle(pageW + 5f, pageH + 5f), 2.5f, 2.5f, 2.5f, 2.5f);
                        tFile = ALCommon.GetTempPath() + SessionID.ToString() + "_" + DiscussionId + "_temp.pdf"; ;

                        try
                        {
                            PdfWriter.GetInstance(imagePDF, new FileStream(tFile, FileMode.Create));
                            imagePDF.Open();

                            using (FileStream fs = new FileStream(sourcePath, FileMode.Open))
                            {
                                iTextSharp.text.Image jpg = iTextSharp.text.Image.GetInstance(System.Drawing.Image.FromStream(fs), ImageFormat.Jpeg);
                                jpg.BorderWidth = 0f;
                                imagePDF.Add(jpg);
                            }

                        }
                        catch (DocumentException dex)
                        {
                            ErrorXML = dex.Message;
                        }

                        catch (IOException ioex)
                        {
                            ErrorXML = ioex.Message;
                        }

                        catch (Exception ex)
                        {
                            ErrorXML = ex.Message;
                        }

                        finally
                        {
                            imagePDF.Close();
                            sourcePath = tFile;
                        }

                    }
                    else
                    {
                        FileStream inputFileStream = new FileStream(srcPath, FileMode.Open, FileAccess.Read);
                        BinaryReader binaryReader = null;
                        binaryReader = new BinaryReader(inputFileStream);

                        //Remove Annotations if Present
                        if (IsConsolidatedFile(binaryReader))
                        {
                            sourcePath = srcPath.Substring(0, srcPath.LastIndexOf(".")) + "_temp.pdf";
                            using (Stream inputPdfStream = new FileStream(srcPath, FileMode.Open, FileAccess.Read, FileShare.Read))
                            using (Stream outputPdfStream = new FileStream(sourcePath, FileMode.Create, FileAccess.Write, FileShare.None))
                            {
                                var reader = new PdfReader(inputPdfStream);
                                reader.RemoveAnnotations();
                                var stamper = new PdfStamper(reader, outputPdfStream, '\0');
                                stamper.Close();
                            }
                        }
                    }

                    using (Stream inputPdfStream = new FileStream(sourcePath, FileMode.Open, FileAccess.Read, FileShare.Read))
                    using (Stream outputPdfStream = new FileStream(tempFile, FileMode.Create, FileAccess.Write, FileShare.None))
                    {
                        var reader = new PdfReader(inputPdfStream);

                        var pdf = new PdfStamper(reader, outputPdfStream, '\0');

                        foreach (XmlNode comment in commentList)
                        {

                            Hashtable o = null;
                            XmlAttribute data = comment.Attributes["data"];

                            string commentValue = encoding.GetString(Convert.FromBase64String(data.Value));

                            o = (Hashtable)Procurios.Public.JSON.JsonDecode(commentValue);

                            if (localComments[o["annoId"]] == null)
                            {
                                localComments[o["annoId"]] = new Hashtable();
                            }
                            (localComments[o["annoId"]] as Hashtable)[Convert.ToInt32((o["orderIndex"]))] = o;


                        }


                        foreach (XmlNode ann in annotationList)
                        {
                            Hashtable o = null;

                            XmlAttribute data = ann.Attributes["data"];

                            annotationValue = encoding.GetString(Convert.FromBase64String(data.Value));

                            o = (Hashtable)Procurios.Public.JSON.JsonDecode(annotationValue);

                            if (localComments[o["guid"]] == null)
                                continue;

                            if (ann.Name == "anchor")
                            {
                                AddAnchorAnnotation(o, localComments[o["guid"]] as Hashtable, pdf, reader);
                            }
                            if (ann.Name == "area")
                            {
                                AddAreaAnnotation(o, localComments[o["guid"]] as Hashtable, pdf, reader);
                            }
                            if (ann.Name == "strikeout")
                            {
                                AddStrikeoutAnnotation(o, localComments[o["guid"]] as Hashtable, pdf, reader);

                            }
                            if (ann.Name == "highlight")
                            {
                                AddHighlightAnnotation(o, localComments[o["guid"]] as Hashtable, pdf, reader);
                            }
                            if (ann.Name == "text")
                            {
                                continue;
                            }
                        }

                        pdf.Close();
                    }

                    //Delete Temporary Files
                    if (!String.IsNullOrEmpty(tFile))
                    {
                        if (File.Exists(tFile))
                            File.Delete(tFile);
                    }

                    BinaryWriter binaryWriter = new BinaryWriter(new FileStream(tempFile, FileMode.Append, FileAccess.Write));

                    //Append Annotations in PDF Binary
                    String AppendAnnotations = "$ALSEP$" + AppendAnnotationsXML + "$ALSEP$";
                    AppendAnnotations += AppendAnnotations.Length.ToString() + "$ALSEP$";
                    binaryWriter.Write(AppendAnnotations);
                    if (binaryWriter != null)
                    {
                        binaryWriter.Close();
                        binaryWriter = null;
                    }
                }
                else
                    DestinationFilePath = srcPath;   

            }
            catch (Exception e)
            {
                ErrorXML = e.Message;                
            }
            
            if (ErrorXML != null)
            {
                strdict_AdditionalInfo = new StringDictionary();
                strdict_AdditionalInfo.Add("TOPICID", DiscussionId);
                strdict_AdditionalInfo.Add("MOREINFO", "Error occurd while creating the consolidated document for Media file.");

                ALErrObj = new ALEngineError();
                ALErrObj.AppendError("CONSOLIDATION_DOCUMENT_GENERATION_FAILED", this.GetType().ToString(),
                                                "GetMediaConsolidatedDocument()", null, strdict_AdditionalInfo,
                                                null, ErrorXML);
                ErrorXML = ALErrObj.ErrorCollectionXML;
            }

            return ErrorXML;
        }

        private bool IsConsolidatedFile(BinaryReader binaryReader)
        {
            string SEPERATOR_TEXT = "$ALSEP$";
            int SEPERATOR_SIZE = 7;

            binaryReader.BaseStream.Position = binaryReader.BaseStream.Seek(-SEPERATOR_SIZE, SeekOrigin.End);

            byte[] buffer = new byte[SEPERATOR_SIZE];
            buffer = binaryReader.ReadBytes(SEPERATOR_SIZE);
            string almark = Encoding.UTF8.GetString(buffer);
            return (almark == SEPERATOR_TEXT);
        }

        public XML GenerateGraphicInfoNodeForTransition(XPathNavigator XPDLProcessNavigator, String SA_DefinitionID, String EA_DefinitionID)
        {
            String          XpathQuery          = null;
            XPathNavigator  XPDLTransitionNode  = null;
            XPathNavigator  GraphicInfoNode     = null;
            XPathNavigator  ActivityNode        = null;
            Boolean         IsStartActivity     = false;
            Boolean         IsEndActivity       = false;
            String          StrGraphicInfoNode  = null;
            XPathNavigator  StartActivityNode   = null;
            XPathNavigator  EndActivityNode     = null;

            XpathQuery          = "//PROCESS_TRANSITION/TRANSITION[FROMACTIVITY/ACTIVITYID=" + SA_DefinitionID + " and TOACTIVITY/ACTIVITYID=" + EA_DefinitionID + "]";
            XPDLTransitionNode  = XPDLProcessNavigator.SelectSingleNode(XpathQuery);
            if (XPDLTransitionNode != null)
            {
                GraphicInfoNode = XPDLTransitionNode.SelectSingleNode("ConnectorGraphicsInfos");
                if (GraphicInfoNode!=null)
                    StrGraphicInfoNode = "<GRAPHICSINFO>" + GraphicInfoNode.OuterXml+ "</GRAPHICSINFO>";
            }

            if (GraphicInfoNode == null)
            {
                XpathQuery      = "PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID="+ SA_DefinitionID+ "]";
                ActivityNode    = XPDLProcessNavigator.SelectSingleNode(XpathQuery);

                if (ActivityNode != null)
                {
                    StartActivityNode   = ActivityNode.SelectSingleNode("STARTACTIVITY");
                    if (StartActivityNode.Value.ToUpper() == "TRUE")
                        IsStartActivity = true;
                }

                XpathQuery      = "PROCESS_ACTIVITY/ACTIVITY[ACTIVITYID="+ EA_DefinitionID+ "]";
                ActivityNode    = XPDLProcessNavigator.SelectSingleNode(XpathQuery);
                if (ActivityNode != null)
                {
                    EndActivityNode     = ActivityNode.SelectSingleNode("ENDACTIVITY");
                    if (EndActivityNode.Value.ToUpper() == "TRUE")
                        IsEndActivity   = true;
                }
                    
                StrGraphicInfoNode = @"<GRAPHICSINFO>
                                                <ConnectorGraphicsInfos>
                                                    <ConnectorGraphicsInfo>";
                if ((IsStartActivity) && (!IsEndActivity))
                {
                    StrGraphicInfoNode += @"<Coordinates XCoordinate='" + SA_IA_Tansition[0, 0] + @"' YCoordinate='" + SA_IA_Tansition[0, 1] + @"'/>
                                           <Coordinates XCoordinate='" + SA_IA_Tansition[1, 0] + @"' YCoordinate='" + SA_IA_Tansition[1, 1] + @"'/>";

                }
                else if ((IsStartActivity) && (IsEndActivity))
                {
                    StrGraphicInfoNode += @"<Coordinates XCoordinate='" + SA_EA_Transition[0, 0] + @"' YCoordinate='" + SA_EA_Transition[0, 1] + @"'/>
                                           <Coordinates XCoordinate='" + SA_EA_Transition[1, 0] + @"' YCoordinate='" + SA_EA_Transition[1, 1] + @"'/>";
                }
                else if ((!IsStartActivity) && (!IsEndActivity))
                {
                    StrGraphicInfoNode += @"<Coordinates XCoordinate='" + IA_IA_Tansition[0, 0] + @"' YCoordinate='" + IA_IA_Tansition[0, 1] + @"'/>
                                           <Coordinates XCoordinate='" + IA_IA_Tansition[1, 0] + @"' YCoordinate='" + IA_IA_Tansition[1, 1] + @"'/>";
                }
                else if ((!IsStartActivity) && (IsEndActivity))
                {
                    StrGraphicInfoNode += @"<Coordinates XCoordinate='" + IA_EA_Tansition[0, 0] + @"' YCoordinate='" + IA_EA_Tansition[0, 1] + @"'/>
                                           <Coordinates XCoordinate='" + IA_EA_Tansition[1, 0] + @"' YCoordinate='" + IA_EA_Tansition[1, 1] + @"'/>";
                }


                 StrGraphicInfoNode += @"        </ConnectorGraphicsInfo>
                                            </ConnectorGraphicsInfos>
                                        </GRAPHICSINFO>";
            }

            return StrGraphicInfoNode;
        }

        public XML GetActivityGraphicInformationNode(XPathNavigator InputActivityNode, Boolean IsStartActivity, Boolean IsEndActivity)
        {
            XPathNavigator  NodeGraphicsInfoNav = null;
            StringBuilder   SBGraphicInfoNode   = null;
            String          Width               = null;
            String          Height              = null;
            XmlDocument     GraphicInfoNodeDoc  = null;
            XmlNode         GraphicInfoNode     = null;
            XmlNode         NodeGraphicsInfoNode= null;
            XmlNode         CoordinatesNode     = null;

            SBGraphicInfoNode = new StringBuilder(30);

            if (IsStartActivity)
            {
                Width = SA_Width;
                Height = SA_Height;
            }
            else if (IsEndActivity)
            {
                Width = EA_Width;
                Height = EA_Height;
            }
            else
            {
                Width = ActivityWidth;
                Height = ActivityHeight;
            }

            if (InputActivityNode != null)
                NodeGraphicsInfoNav = InputActivityNode.SelectSingleNode("NodeGraphicsInfos");

            if ((NodeGraphicsInfoNav != null) && (NodeGraphicsInfoNav.HasChildren))
            {
                GraphicInfoNodeDoc          = new XmlDocument();
                GraphicInfoNode             = GraphicInfoNodeDoc.CreateElement("GRAPHICSINFO");
                GraphicInfoNode.InnerXml    = NodeGraphicsInfoNav.OuterXml;

                //Incase user has not traversed to process designer's VFW UI Width and Height parameters are empty.
                NodeGraphicsInfoNode        = GraphicInfoNode.SelectSingleNode("NodeGraphicsInfos/NodeGraphicsInfo");
                if (NodeGraphicsInfoNode.Attributes.GetNamedItem("Width", "") == null)
                    ((XmlElement)NodeGraphicsInfoNode).SetAttribute("Width", Width.ToString());

                if (NodeGraphicsInfoNode.Attributes.GetNamedItem("Height", "") == null)
                    ((XmlElement)NodeGraphicsInfoNode).SetAttribute("Height", Height.ToString());

                CoordinatesNode             = NodeGraphicsInfoNode.SelectSingleNode("Coordinates");
                if (CoordinatesNode.Attributes.GetNamedItem("XCoordinate", "") == null)
                    ((XmlElement)CoordinatesNode).SetAttribute("XCoordinate", Activity_XCoord.ToString());

                if (CoordinatesNode.Attributes.GetNamedItem("YCoordinate", "") == null)
                    ((XmlElement)CoordinatesNode).SetAttribute("YCoordinate", Activity_YCoord.ToString());

                SBGraphicInfoNode.Append(GraphicInfoNode.OuterXml);
            }
            else
            {
                SBGraphicInfoNode.AppendFormat(@"<GRAPHICSINFO>
                                                    <NodeGraphicsInfos>
	                                                    <NodeGraphicsInfo Width='{0}' Height='{1}'>
		                                                    <Coordinates XCoordinate='{2}' YCoordinate='{3}'/>
	                                                    </NodeGraphicsInfo>
                                                    </NodeGraphicsInfos>
                                                 </GRAPHICSINFO>", Width, Height, Activity_XCoord, Activity_YCoord);
            }

            return SBGraphicInfoNode.ToString();
        }

        public XML GenerateGraphicsInfoNodeFor_SrcActivityToDestActivity(XPathNavigator SourceActivityGraphicsInfoNode, XPathNavigator DestActivityGraphicsInfoNode)
        {
            StringBuilder       SB_GraphicsInfo             = null;
            XPathNavigator      SrcXYCoordNode              = null;
            XPathNavigator      DestXYCoordNode             = null;
            int                 SrcXCoord                   = 0;
            int                 SrcYCoord                   = 0;
            int                 DestXCoord                  = 0;
            int                 DestYCoord                  = 0;
            XPathNavigator      SrcNodeGraphicsInfo         = null;
            XPathNavigator      DestNodeGraphicsInfo        = null;
            int                 SrcActWidth                 = 0;
            int                 SrcActHeight                = 0;
            int                 DestActWidth                = 0;
            int                 DestActHeight               = 0;

            SB_GraphicsInfo = new StringBuilder(50);

            if (((SourceActivityGraphicsInfoNode != null) && (DestActivityGraphicsInfoNode != null))
                && ((SourceActivityGraphicsInfoNode.HasChildren) && (DestActivityGraphicsInfoNode.HasChildren)))
            {
                SrcNodeGraphicsInfo     = SourceActivityGraphicsInfoNode.SelectSingleNode("NodeGraphicsInfos/NodeGraphicsInfo");
                SrcXYCoordNode          = SourceActivityGraphicsInfoNode.SelectSingleNode("NodeGraphicsInfos/NodeGraphicsInfo/Coordinates");
                DestXYCoordNode         = DestActivityGraphicsInfoNode.SelectSingleNode("NodeGraphicsInfos/NodeGraphicsInfo/Coordinates");
                DestNodeGraphicsInfo    = DestActivityGraphicsInfoNode.SelectSingleNode("NodeGraphicsInfos/NodeGraphicsInfo");

                SrcXCoord               = Convert.ToInt32(SrcXYCoordNode.GetAttribute("XCoordinate",""));
                SrcYCoord               = Convert.ToInt32(SrcXYCoordNode.GetAttribute("YCoordinate",""));
                SrcActWidth             = Convert.ToInt32(SrcNodeGraphicsInfo.GetAttribute("Width",""));
                SrcActHeight            = Convert.ToInt32(SrcNodeGraphicsInfo.GetAttribute("Height",""));
                
                DestXCoord              = Convert.ToInt32(DestXYCoordNode.GetAttribute("XCoordinate",""));
                DestYCoord              = Convert.ToInt32(DestXYCoordNode.GetAttribute("YCoordinate",""));
                DestActWidth            = Convert.ToInt32(DestNodeGraphicsInfo.GetAttribute("Width",""));
                DestActHeight           = Convert.ToInt32(DestNodeGraphicsInfo.GetAttribute("Height",""));

                SrcXCoord   += SrcActWidth;
                SrcYCoord   += (SrcActHeight / 2);
                DestYCoord  += (DestActHeight / 2);
                
                SB_GraphicsInfo.AppendFormat(@"<ConnectorGraphicsInfos>
		                                            <ConnectorGraphicsInfo>
			                                            <Coordinates XCoordinate='{0}' YCoordinate='{1}'/>
			                                            <Coordinates XCoordinate='{2}' YCoordinate='{3}'/>
		                                            </ConnectorGraphicsInfo>
	                                            </ConnectorGraphicsInfos>
                                        ", SrcXCoord, SrcYCoord, DestXCoord, DestYCoord);
            }

            return SB_GraphicsInfo.ToString();
        }


        #endregion

        #region Private Methods

        #region Reverse Time Line Mode - Create

        /// <alsummary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method is a recursive function that calculates the 
        /// timeline of a process using the given duedate.i.e it finds out the start date of the process. 
        /// The timeline calculation begins with the end activity and ends at the start activity using 
        /// the "FROM" transitions for an Activity</Purpose>
        /// </alsummary>
        /// <param name="InputXML">The InputXML contains the process details including the Activities, 
        /// their durations and transitions between the Activities</param>
        /// <param name="ProcessDueDate">The new Process Due Date</param>
        /// <param name="ProcessStartDate">The new Process Start Date</param>        
        /// <param name="d_ProcessDuration"></param>
        /// <returns>An ErrorXML if an Error occurs, else returns null</returns>
        private ERRORXML CalculateReverseTimeLine_Create(ref XML InputXML, DateTime ProcessDueDate, out Double d_ProcessDuration, out DateTime ProcessStartDate)
        {
            String              sb_DateXML      = null;
            XmlDocument         Process_Doc     = null;
            XmlNode             ProcessNode     = null;
            XmlNode             DueDateNode     = null;
            XmlNode             StartDateNode   = null;
            XmlNode             ProcessActNode  = null;
            XmlNode             ProcessTrNode   = null;
            XmlNode             ActNode         = null;
            XmlNode             SuccActNode     = null;
            XmlNode             SubflowNode     = null;
            XmlNode             ActInfoNode     = null;
            XmlNode             ActDurNode      = null;
            XmlNode             ProcessInfoNode = null;
            XmlNode             ProcessDurNode  = null;
            XmlNodeList          FromAct_Lst    = null;
            String              str_ProcessID   = null;
            String              str_ActivityID  = null;
            String              str_ActDuration = null;
            String              str_MinActID    = null;
            String              str_ProcessDur  = null;            
            ERRORXML            ErrorXml        = null;
            XML                 UpdatedDateXML  = null;
            Queue               Vertices        = null;
            Double              d_SubprocessDur = 0;
            Hashtable           VisitedVertices = null;
            Boolean             b_IsSuccEnd     = false;
            Boolean             b_IsActStart    = false;
            Boolean             b_IsActEnd      = false;
            DateTime            MinDate         ;
            DateTime            NewStartDate    ;
            String[]            sArr_DependantAct = null;
            Int32               Index             = 0;
                            
                d_ProcessDuration   = 0;
                Process_Doc         = new XmlDocument();
                Process_Doc.LoadXml(InputXML);                                
                ProcessNode = Process_Doc.SelectSingleNode("WORKFLOWPROCESS");
                str_ProcessID = ((XmlElement)ProcessNode).GetAttribute("ProcessDefinitionID");
               
                Vertices            = new Queue();
                VisitedVertices     = new Hashtable();
                ProcessStartDate = DateTime.Today;

                ProcessInfoNode = ProcessNode.SelectSingleNode("PROCESS_INFORMATION");
                if (ProcessInfoNode == null)
                {
                    StringBuilder sb_ProcessInfo = new StringBuilder(100);
                    sb_ProcessInfo.Append(@"<PROCESS_INFORMATION TYPE='RELATION'>
                                                <WORKFLOWPROCESSINFO>
                                                    <DURATION>0</DURATION>
                                                </WORKFLOWPROCESSINFO>
                                             </PROCESS_INFORMATION>");
                    ProcessNode.InnerXml += sb_ProcessInfo.ToString();
                    ProcessInfoNode = ProcessNode.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO");
                }
                else
                    ProcessInfoNode = ProcessNode.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO");
                
                ProcessDurNode = ProcessInfoNode.SelectSingleNode("DURATION");
                if (ProcessDurNode == null)
                {
                    ProcessDurNode = Process_Doc.CreateElement("DURATION");
                    ProcessDurNode.InnerText = "0";
                    ProcessInfoNode.AppendChild(ProcessDurNode);
                }

                DueDateNode = ProcessNode.SelectSingleNode("DUEDATE");
                
                // First set the start date and due date of end Activity to the new process due date
                ProcessActNode  = ProcessNode.SelectSingleNode("PROCESS_ACTIVITY");
                ProcessTrNode   = ProcessNode.SelectSingleNode("PROCESS_TRANSITION");

                // Check if the process contains only start and end Activity, then directly prepare the updated date XML
                if (ProcessActNode.SelectNodes("ACTIVITY").Count == 2)
                {
                    PrepareProcessDateXML_Create(ref ProcessNode, ProcessDueDate,out ProcessStartDate, out str_ProcessDur);                    
                    d_ProcessDuration   = Convert.ToDouble(str_ProcessDur);
                    InputXML = ProcessNode.OuterXml; 
                    return ErrorXml;
                }
                sArr_DependantAct   = new String[ProcessActNode.SelectNodes("ACTIVITY").Count];
                ActNode             = ProcessActNode.SelectSingleNode("ACTIVITY[translate(ENDACTIVITY,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='TRUE']");
               
                str_ActivityID = ((XmlElement)ActNode).GetAttribute("ActivityDefinitionID");
               
                StartDateNode = ActNode.SelectSingleNode("STARTDATE");
                if (StartDateNode == null)
                {
                    StartDateNode = Process_Doc.CreateElement("STARTDATE");
                    ActNode.AppendChild(StartDateNode);
                }

                sb_DateXML = @"<DAY>" + ProcessDueDate.Day.ToString() + "</DAY>"
                           + "<MONTH>" + ProcessDueDate.Month.ToString() + "</MONTH>"
                           + "<YEAR>" + ProcessDueDate.Year.ToString() + "</YEAR>";

                StartDateNode.InnerXml = sb_DateXML;

                DueDateNode = ActNode.SelectSingleNode("DUEDATE");
                if (DueDateNode == null)
                {
                    DueDateNode = Process_Doc.CreateElement("DUEDATE");
                    ActNode.AppendChild(DueDateNode);
                }
                DueDateNode.InnerXml = sb_DateXML;

                VisitedVertices.Add(str_ActivityID, ProcessDueDate.ToString());

                    // Traverse the Process using the reverse BFS
                FromAct_Lst = ProcessTrNode.SelectNodes("TRANSITION[TOACTIVITY/ACTIVITYID='" + str_ActivityID + "']/FROMACTIVITY/ACTIVITYID");

                foreach (XmlNode FromActNode in FromAct_Lst)
                {                    
                    Vertices.Enqueue(FromActNode.InnerText);                    
                }                
                while (Vertices.Count > 0)
                {
                    str_ActivityID  = Vertices.Dequeue().ToString();                    
                    ActNode = ProcessActNode.SelectSingleNode("ACTIVITY[@ActivityDefinitionID='" + str_ActivityID + "']");
                    
                    if (ActNode != null)
                    {
                        ActInfoNode     = ActNode.SelectSingleNode("ACTIVITY_INFORMATION");
                        if (ActInfoNode == null)
                        {
                            StringBuilder sb_ActivityInfo = new StringBuilder(100);
                            sb_ActivityInfo.Append(@"<ACTIVITY_INFORMATION TYPE='RELATION'>
                                                        <ACTIVITYINFORMATION>
                                                            <DURATION>0</DURATION>
                                                        </ACTIVITYINFORMATION>
                                                     </ACTIVITY_INFORMATION>");
                            ActNode.InnerXml += sb_ActivityInfo.ToString();
                            ActInfoNode = ActNode.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION");
                        }
                        else
                            ActInfoNode = ActInfoNode.SelectSingleNode("ACTIVITYINFORMATION");

                        ActDurNode = ActInfoNode.SelectSingleNode("DURATION");
                        if (ActDurNode == null)
                        {
                            Process_Doc.CreateElement("DURATION");
                            ActInfoNode.AppendChild(ActDurNode);
                        }
                        else
                            str_ActDuration = ActNode.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").InnerText;

                        GetActivityIDWithMinimumStartDate_Create(VisitedVertices, ProcessTrNode, str_ActivityID, out str_MinActID, out MinDate);
                        if (!String.IsNullOrEmpty(str_MinActID))
                        {                            
                            SuccActNode = ProcessActNode.SelectSingleNode("ACTIVITY[@ActivityDefinitionID='" + str_MinActID + "']");                            
                            b_IsSuccEnd     = Convert.ToBoolean(SuccActNode.SelectSingleNode("ENDACTIVITY").InnerText);
                            b_IsActStart    = Convert.ToBoolean(ActNode.SelectSingleNode("STARTACTIVITY").InnerText);
                            b_IsActEnd      = Convert.ToBoolean(ActNode.SelectSingleNode("ENDACTIVITY").InnerText);

                            // Check if the Activity is Subflow, if yes then give a recursive call
                            SubflowNode = ActNode.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS");

                            if (SubflowNode != null)
                            {
                                if ((!b_IsSuccEnd) && (!b_IsActStart))
                                {
                                    long Min = ConvertDaysToMinutes(2);
                                    GetStartDate(Min.ToString(), MinDate, out MinDate);
                                }
                                UpdatedDateXML = SubflowNode.OuterXml;
                                CalculateReverseTimeLine_Create(ref UpdatedDateXML, MinDate, out d_SubprocessDur, out ProcessStartDate);
                                NewStartDate = ProcessStartDate;
                                str_ActDuration = d_SubprocessDur.ToString();
                            }
                            else
                            {
                                if ((!b_IsSuccEnd) && (!b_IsActStart))
                                {
                                    long Min = ConvertDaysToMinutes(2);
                                    GetStartDate(Min.ToString(), MinDate, out MinDate);
                                }
                                GetStartDate(str_ActDuration, MinDate, out NewStartDate);
                            }
                            if ((!b_IsActEnd) && (!b_IsActStart))
                            {
                                d_ProcessDuration += Convert.ToDouble(str_ActDuration);
                            }

                            if (VisitedVertices.ContainsKey(str_ActivityID) == false)
                            {
                                VisitedVertices.Add(str_ActivityID, NewStartDate.ToString());
                            }

                            StartDateNode = ActNode.SelectSingleNode("STARTDATE");
                            if (StartDateNode == null)
                            {
                                StartDateNode = Process_Doc.CreateElement("STARTDATE");
                                ActNode.AppendChild(StartDateNode);
                            }
                            DueDateNode = ActNode.SelectSingleNode("DUEDATE");
                            if (DueDateNode == null)
                            {
                                DueDateNode = Process_Doc.CreateElement("DUEDATE");
                                ActNode.AppendChild(DueDateNode);
                            }

                            // Update the new Start Date and Due Date
                            sb_DateXML = @"<DAY>" + NewStartDate.Day.ToString() + "</DAY>"
                                        + "<MONTH>" + NewStartDate.Month.ToString() + "</MONTH>"
                                        + "<YEAR>" + NewStartDate.Year.ToString() + "</YEAR>";

                            StartDateNode.InnerXml = sb_DateXML;

                            sb_DateXML = @"<DAY>" + MinDate.Day.ToString() + "</DAY>"
                                        + "<MONTH>" + MinDate.Month.ToString() + "</MONTH>"
                                        + "<YEAR>" + MinDate.Year.ToString() + "</YEAR>";
                            DueDateNode.InnerXml = sb_DateXML;

                            if ((UpdatedDateXML != null) && (SubflowNode != null))
                            {
                                SubflowNode.ParentNode.InnerXml = UpdatedDateXML;
                            }                            
                            ActDurNode.InnerText = str_ActDuration;
                              
                            // Get the predecessors of current Activity.
                            FromAct_Lst = ProcessTrNode.SelectNodes("TRANSITION[TOACTIVITY/ACTIVITYID='" + str_ActivityID + "']/FROMACTIVITY/ACTIVITYID");
                            foreach (XmlNode FromActNode in FromAct_Lst)
                            {
                                if (!Vertices.Contains(FromActNode.InnerText))
                                {
                                    Vertices.Enqueue(FromActNode.InnerText);
                                }
                            }
                        }
                        else
                        {
                            // The Activity is dependant...Add the Activity to the dependancy List
                            sArr_DependantAct[Index++] = str_ActivityID;
                        }
                    }
                }
                // Update if Any dependant Activities
                if ((sArr_DependantAct.Length > 0) && (Index > 0))
                {
                    ErrorXml = UpdateDependantActivities_Create(sArr_DependantAct,ref ProcessActNode, ProcessTrNode, VisitedVertices);
                    if (ErrorXml != null)
                        return ErrorXml;
                    //sb_DateXML.Append(DependantActXML);
                }
                // find the Process Start date                    
                ProcessStartDate = GetProcessStartDate(VisitedVertices);
                              
            // The Process start date will be the minimum of all start dates. i.e. the start date of the Start Activity

                StartDateNode = ProcessNode.SelectSingleNode("STARTDATE");
                if (StartDateNode == null)
                {
                    StartDateNode = Process_Doc.CreateElement("STARTDATE");
                    ProcessNode.AppendChild(StartDateNode);
                }
                sb_DateXML = @"<DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                    + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                    + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>";

                StartDateNode.InnerXml = sb_DateXML;

                ProcessDurNode.InnerText = d_ProcessDuration.ToString();
                InputXML = ProcessNode.OuterXml;
                return ErrorXml;
        }

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method returns the Predecessor Activity with the earliest start date</Purpose>        
        /// </summary>
        /// <param name="VisitedVertices">The has table of visited vertices</param>
        /// <param name="ProcessTrNode">The Process Transition Node</param>
        /// <param name="str_ActivityID">The Activity ID whose dependant Activity will be returned</param>
        /// <param name="str_MinActID">The Activity ID with earliest start date</param>
        /// <param name="MinDate">Date of the Activity with minimum start date</param>
        private void GetActivityIDWithMinimumStartDate_Create(Hashtable VisitedVertices, XmlNode ProcessTrNode, String str_ActivityID, out String str_MinActID, out DateTime MinDate)
        {
            XmlNodeList SuccNodeList= null;            
            DateTime    TempDate;
            String      str_ToActID = null;            
            
                SuccNodeList = ProcessTrNode.SelectNodes("TRANSITION[FROMACTIVITY/ACTIVITYID='" + str_ActivityID + "']/TOACTIVITY/ACTIVITYID");
                TempDate = new DateTime(1, 1, 1);
                MinDate = new DateTime(1, 1, 1);
                str_MinActID = String.Empty;

                foreach( XmlNode SuccNode in SuccNodeList)
                {
                    str_ToActID = SuccNode.InnerText;
                    
                    if (VisitedVertices.ContainsKey(str_ToActID))
                    {
                        MinDate = Convert.ToDateTime(VisitedVertices[str_ToActID]);
                        if (TempDate.Equals(new DateTime(1, 1, 1)))
                        {
                            TempDate = MinDate;
                            str_MinActID = str_ToActID;
                        }
                        else
                        {
                            if (TempDate < MinDate)
                            {
                                MinDate = TempDate;
                                str_MinActID = str_ToActID;
                            }
                        }
                    }
                    else
                    {
                        str_MinActID = String.Empty;
                        return;
                    }
                }
        }

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method prepares the new date xml for process with start and end Activity</Purpose>
        /// </summary>
        /// <param name="ProcessNode">XML Node that refers to the process</param>
        /// <param name="ProcessDueDate">The due date of the process</param>
        /// <param name="ProcessStartDate">the start date of the process</param>
        /// <param name="str_Duration">The duration of the process</param>
        private void PrepareProcessDateXML_Create(ref XmlNode ProcessNode, DateTime ProcessDueDate,out DateTime ProcessStartDate,out String str_Duration)
        {
            XmlNodeList ActNodeList = null;
            XmlNode ProcessInfoNode = null;
            XmlNode StartDateNode   = null;
            XmlNode DueDateNode     = null;            
            String  sb_DateXML      = null;
            
                ProcessInfoNode = ProcessNode.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO");
                str_Duration = ProcessInfoNode.SelectSingleNode("DURATION").InnerText;

                StartDateNode = ProcessNode.SelectSingleNode("STARTDATE");
                if (StartDateNode == null)
                {
                    StartDateNode = ProcessNode.OwnerDocument.CreateElement("STARTDATE");
                    ProcessNode.AppendChild(StartDateNode);
                }
                DueDateNode = ProcessNode.SelectSingleNode("DUEDATE");
                if (DueDateNode == null)
                {
                    DueDateNode = ProcessNode.OwnerDocument.CreateElement("DUEDATE");
                    ProcessNode.AppendChild(DueDateNode);
                }
                GetStartDate(str_Duration, ProcessDueDate, out ProcessStartDate);

                sb_DateXML = @"<DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                            + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                            + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>";

                StartDateNode.InnerXml = sb_DateXML;

                sb_DateXML = @"<DAY>" + ProcessDueDate.Day.ToString() + "</DAY>"
                            + "<MONTH>" + ProcessDueDate.Month.ToString() + "</MONTH>"
                            + "<YEAR>" + ProcessDueDate.Year.ToString() + "</YEAR>";

                DueDateNode.InnerXml = sb_DateXML;

                ActNodeList = ProcessNode.SelectNodes("PROCESS_ACTIVITY/ACTIVITY");

                foreach (XmlNode ActNode in ActNodeList)
                {
                    StartDateNode = ActNode.SelectSingleNode("STARTDATE");
                    if (StartDateNode == null)
                    {
                        StartDateNode = ProcessNode.OwnerDocument.CreateElement("STARTDATE");
                        ActNode.AppendChild(StartDateNode);
                    }
                    DueDateNode = ActNode.SelectSingleNode("DUEDATE");
                    if (DueDateNode == null)
                    {
                        DueDateNode = ProcessNode.OwnerDocument.CreateElement("DUEDATE");
                        ActNode.AppendChild(DueDateNode);
                    }

                    sb_DateXML = @"<DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>";

                    StartDateNode.InnerXml = sb_DateXML;

                    sb_DateXML = @"<DAY>" + ProcessDueDate.Day.ToString() + "</DAY>"
                           + "<MONTH>" + ProcessDueDate.Month.ToString() + "</MONTH>"
                           + "<YEAR>" + ProcessDueDate.Year.ToString() + "</YEAR>";

                    DueDateNode.InnerXml = sb_DateXML;                                                             
                }
        }

        /// <summary>
        /// <Purpose>This method updates the dependant Activities of the Proeces</Purpose>
        /// <Author>Mugdha Moolay Gadre</Author>        
        /// </summary>
        /// <param name="sArr_DependantAct">The array of the dependant Activities</param>
        /// <param name="ProcessActNode">XML node that refers to the PROCESS_ACTIVITY node</param>
        /// <param name="PrTrNode">XML node that refers to the PROCESS_TRANSITION node</param>
        /// <param name="VisitedVertices">Hastable consists of the visited vertices of the BFS</param>
        private ERRORXML UpdateDependantActivities_Create(String[] sArr_DependantAct,ref XmlNode ProcessActNode ,XmlNode PrTrNode, Hashtable VisitedVertices)
        {
            String              str_DepActID        = null;
            String              str_ActDuration     = null;
            String              sb_DateXML          = null;            
            XmlNode             ActNode             = null;
            XmlNode             SuccActNode         = null;
            XmlNode             SubflowNode         = null;
            XmlNode             StartDateNode       = null;
            XmlNode             DueDateNode         = null;            
            DateTime            MinDate             ; 
            DateTime            ProcessStartDate    ;
            DateTime            NewStartDate        ;
            String              str_MinActID        = null;
            Boolean             b_IsSuccEnd         = false;
            Boolean             b_IsActStart        = false;
            Boolean             b_IsActEnd          = false;
            XML                 UpdatedDateXML      = null;
            double              d_SubprocessDur     = 0;
            ERRORXML            ErrorXml            = null;
                            
                for (int Index = 0; Index < sArr_DependantAct.Length; Index++)
                {
                    str_DepActID    = sArr_DependantAct[Index];
                    ActNode = ProcessActNode.SelectSingleNode("ACTIVITY[@ActivityDefinitionID='" + str_DepActID + "']");
                    
                    if (ActNode != null)
                    {
                        StartDateNode = ActNode.SelectSingleNode("STARTDATE");
                        if (StartDateNode == null)
                            StartDateNode = ProcessActNode.OwnerDocument.CreateElement("STARTDATE");

                        DueDateNode = ActNode.SelectSingleNode("DUEDATE");
                        if (DueDateNode == null)
                            DueDateNode = ProcessActNode.OwnerDocument.CreateElement("DUEDATE");
                        
                        GetActivityIDWithMinimumStartDate_Create(VisitedVertices, PrTrNode, str_DepActID, out str_MinActID, out MinDate);
                        if (!String.IsNullOrEmpty(str_MinActID))
                        {
                            SuccActNode = ProcessActNode.SelectSingleNode("ACTIVITY[ACTIVITYID='" + str_MinActID + "']");

                            if (SuccActNode == null)
                            {
                                SuccActNode = ProcessActNode.SelectSingleNode("ACTIVITY[@ActivityDefinitionID='" + str_MinActID + "']");
                            }
                            b_IsSuccEnd     = Convert.ToBoolean(SuccActNode.SelectSingleNode("ENDACTIVITY").InnerText);
                            b_IsActStart    = Convert.ToBoolean(ActNode.SelectSingleNode("STARTACTIVITY").InnerText);
                            b_IsActEnd      = Convert.ToBoolean(ActNode.SelectSingleNode("ENDACTIVITY").InnerText);

                            // Check if the Activity is Subflow, if yes then give a recursive call
                            SubflowNode = ActNode.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS");

                            if (SubflowNode != null)
                            {
                                if ((!b_IsSuccEnd) && (!b_IsActStart))
                                {
                                    long Min = ConvertDaysToMinutes(2);
                                    GetStartDate(Min.ToString(), MinDate, out MinDate);
                                    //MinDate = MinDate.AddDays(-1);
                                }
                                UpdatedDateXML = SubflowNode.OuterXml;
                                CalculateReverseTimeLine_Create(ref UpdatedDateXML, MinDate, out d_SubprocessDur, out ProcessStartDate);
                                NewStartDate = ProcessStartDate;
                                str_ActDuration = d_SubprocessDur.ToString();
                            }
                            else
                            {
                                if ((!b_IsSuccEnd) && (!b_IsActStart))
                                {
                                    long Min = ConvertDaysToMinutes(2);
                                    GetStartDate(Min.ToString(), MinDate, out MinDate);
                                }
                                GetStartDate(str_ActDuration, MinDate, out NewStartDate);
                            }
                            sb_DateXML = @"<DAY>" + NewStartDate.Day.ToString() + "</DAY>"
                                       + "<MONTH>" + NewStartDate.Month.ToString() + "</MONTH>"
                                       + "<YEAR>" + NewStartDate.Year.ToString() + "</YEAR>";

                            StartDateNode.InnerXml = sb_DateXML;

                            sb_DateXML = @"<DAY>" + MinDate.Day.ToString() + "</DAY>"
                                       + "<MONTH>" + MinDate.Month.ToString() + "</MONTH>"
                                       + "<YEAR>" + MinDate.Year.ToString() + "</YEAR>";

                            DueDateNode.InnerXml = sb_DateXML;

                            if ((UpdatedDateXML != null) && (SubflowNode != null))
                            {
                                SubflowNode.ParentNode.InnerXml = UpdatedDateXML;
                            }
                        }
                    }
                }// End For
                return ErrorXml;
        }

        #endregion

        #region Reverse Time Line Mode - Edit

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method is a recursive function that calculates the 
        /// timeline of a process using the given duedate.i.e it finds out the start date of the process. 
        /// The timeline calculation begins with the end activity and ends at the start activity using 
        /// the "FROM" transitions for an Activity</Purpose>
        /// </summary>
        /// <param name="ProcessDueDate">The new Process Due Date</param>
        /// <param name="ProcessStartDate">The new Process Start Date</param>        
        /// <param name="d_ProcessDuration"></param>
        /// <param name="InputXML">The InputXML contains the process details including the Activities, 
        /// their durations and transitions between the Activities</param>
        /// <param name="UpdatedDateXML"></param>
        /// <returns>An ErrorXML if an Error occurs, else returns null</returns>
        private ERRORXML CalculateReverseTimeLine_Edit(XML InputXML, DateTime ProcessDueDate, out XML UpdatedDateXML, out Double d_ProcessDuration, out DateTime ProcessStartDate)
        {
            StringBuilder       sb_DateXML      = null;
            String              DependantActXML = null;
            XPathDocument       xpDoc_Process   = null;
            XPathNavigator      xpNav_Process   = null;
            XPathNavigator      xpNav_ProcColl   = null;
            XPathNavigator      xpNav_Activity  = null;
            XPathNavigator      xpNav_ProcAct   = null;
            XPathNavigator      xpNav_ProcTr    = null;
            XPathNavigator      xpNav_Subflow   = null;
            XPathNavigator      xpNav_SuccAct   = null;
            XPathNodeIterator   xpIter_FromAct  = null;
            String              str_ProcessID   = null;
            String              str_ActivityID  = null;
            String              str_ActDuration = null;
            String              str_MinActID    = null;
            String              str_ProcessDur  = null;
            String              str_ActInfoID   = null;
            String              str_ProcInfoID  = null;
            ERRORXML            ErrorXml        = null;
            Queue               Vertices        = null;
            Double              d_SubprocessDur = 0;
            Hashtable           VisitedVertices = null;
            Boolean             b_IsSuccEnd     = false;
            Boolean             b_IsActStart    = false;
            Boolean             b_IsActEnd      = false;
            DateTime            MinDate         ;
            DateTime            NewStartDate    ;
            String[]            sArr_DependantAct = null;
            Int32               Index             = 0;
            
                sb_DateXML          = new StringBuilder(500);
                d_ProcessDuration   = 0;
                xpDoc_Process       = new XPathDocument(new StringReader(InputXML));
                xpNav_ProcColl      = xpDoc_Process.CreateNavigator();
                xpNav_Process       = xpNav_ProcColl.SelectSingleNode("WORKFLOWPROCESS");               
                str_ProcessID       = xpNav_Process.SelectSingleNode("WORKFLOWPROCESSID").Value;
                Vertices            = new Queue();
                VisitedVertices     = new Hashtable();
                ProcessStartDate    = DateTime.Today;
                UpdatedDateXML      = String.Empty;
                str_ProcInfoID      = xpNav_Process.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").Value; 

                sb_DateXML.Append("<WORKFLOWPROCESS MODIFIED='TRUE'>");
                sb_DateXML.AppendFormat("<WORKFLOWPROCESSID>{0}</WORKFLOWPROCESSID>", str_ProcessID);
                sb_DateXML.AppendFormat(@"<DUEDATE>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                        </DUEDATE>", ProcessDueDate.Day.ToString(), ProcessDueDate.Month.ToString(), ProcessDueDate.Year.ToString());
                sb_DateXML.Append("<PROCESS_ACTIVITY TYPE='RELATION'>");
                
                // First set the start date and due date of end Activity to the new process due date

                xpNav_ProcAct   = xpNav_Process.SelectSingleNode("PROCESS_ACTIVITY");
                xpNav_ProcTr    = xpNav_Process.SelectSingleNode("PROCESS_TRANSITION");

                // Check if the process contains only start and end Activity, then directly prepare the updated date XML
                if (xpNav_ProcAct.Select("ACTIVITY").Count == 2)
                {
                    UpdatedDateXML      = PrepareProcessDateXML_Edit(xpNav_Process, ProcessDueDate,out ProcessStartDate, out str_ProcessDur);
                    d_ProcessDuration   = Convert.ToDouble(str_ProcessDur);
                    return ErrorXml;
                }
                sArr_DependantAct = new String[xpNav_ProcAct.Select("ACTIVITY").Count];
                xpNav_Activity  = xpNav_ProcAct.SelectSingleNode("ACTIVITY[translate(ENDACTIVITY,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='TRUE']");

                str_ActivityID  = xpNav_Activity.SelectSingleNode("ACTIVITYID").Value;

                sb_DateXML.Append("<ACTIVITY MODIFIED='TRUE'>");
                    sb_DateXML.AppendFormat("<ACTIVITYID>{0}</ACTIVITYID>", str_ActivityID);
                    sb_DateXML.AppendFormat(@"<STARTDATE>
                                                <DAY>{0}</DAY>
                                                <MONTH>{1}</MONTH>
                                                <YEAR>{2}</YEAR>
                                               </STARTDATE>", ProcessDueDate.Day.ToString(), ProcessDueDate.Month.ToString(), ProcessDueDate.Year.ToString());
                    sb_DateXML.AppendFormat(@"<DUEDATE>
                                                <DAY>{0}</DAY>
                                                <MONTH>{1}</MONTH>
                                                <YEAR>{2}</YEAR>
                                            </DUEDATE>", ProcessDueDate.Day.ToString(), ProcessDueDate.Month.ToString(), ProcessDueDate.Year.ToString());

                    sb_DateXML.Append("</ACTIVITY>");
                    VisitedVertices.Add(str_ActivityID, ProcessDueDate.ToString());

                    // Traverse the Process using the reverse BFS
                    xpIter_FromAct = xpNav_ProcTr.Select("TRANSITION[TOACTIVITY/ACTIVITY/ACTIVITYID='" + str_ActivityID + "']/FROMACTIVITY/ACTIVITY");
                    while (xpIter_FromAct.MoveNext())
                    { 
                        Vertices.Enqueue(xpIter_FromAct.Current.SelectSingleNode("ACTIVITYID").Value);
                    }

                    while (Vertices.Count > 0)
                    {
                        str_ActivityID = Vertices.Dequeue().ToString();
                        xpNav_Activity = xpNav_ProcAct.SelectSingleNode("ACTIVITY[ACTIVITYID='" + str_ActivityID + "']");
                        if (xpNav_Activity != null)
                        {
                            str_ActDuration = xpNav_Activity.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/DURATION").Value;
                            str_ActInfoID   = xpNav_Activity.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYINFORMATIONID").Value;
                            GetActivityIDWithMinimumStartDate_Edit(VisitedVertices, xpNav_ProcTr, str_ActivityID, out str_MinActID, out MinDate);
                            if (!String.IsNullOrEmpty(str_MinActID))
                            {
                                xpNav_SuccAct = xpNav_ProcAct.SelectSingleNode("ACTIVITY[ACTIVITYID='" + str_MinActID + "']");

                                b_IsSuccEnd = Convert.ToBoolean(xpNav_SuccAct.SelectSingleNode("ENDACTIVITY").Value);
                                b_IsActStart = Convert.ToBoolean(xpNav_Activity.SelectSingleNode("STARTACTIVITY").Value);
                                b_IsActEnd = Convert.ToBoolean(xpNav_Activity.SelectSingleNode("ENDACTIVITY").Value);

                                // Check if the Activity is Subflow, if yes then give a recursive call
                                xpNav_Subflow = xpNav_Activity.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS");

                                if (xpNav_Subflow != null)
                                {
                                    if ((!b_IsSuccEnd) && (!b_IsActStart))
                                    {
                                        long Min = ConvertDaysToMinutes(2);
                                        GetStartDate(Min.ToString(), MinDate, out MinDate);
                                    }
                                    CalculateReverseTimeLine_Edit(xpNav_Subflow.OuterXml, MinDate, out UpdatedDateXML, out d_SubprocessDur, out ProcessStartDate);
                                    NewStartDate = ProcessStartDate;
                                    str_ActDuration = d_SubprocessDur.ToString();
                                }
                                else
                                {
                                    if ((!b_IsSuccEnd) && (!b_IsActStart))
                                    {
                                        long Min = ConvertDaysToMinutes(2);
                                        GetStartDate(Min.ToString(), MinDate, out MinDate);
                                    }
                                    GetStartDate(str_ActDuration, MinDate, out NewStartDate);
                                }
                                if ((!b_IsActEnd) && (!b_IsActStart))
                                {
                                    d_ProcessDuration += Convert.ToDouble(str_ActDuration);
                                }

                                if (VisitedVertices.ContainsKey(str_ActivityID) == false)
                                {
                                    VisitedVertices.Add(str_ActivityID, NewStartDate.ToString());
                                }
                                // Update the new Start Date and Due Date
                                sb_DateXML.AppendFormat(@"<ACTIVITY MODIFIED='TRUE'>
                                                        <ACTIVITYID>" + str_ActivityID + "</ACTIVITYID>"
                                                            + @"<STARTDATE>
                                                            <DAY>{0}</DAY>
                                                            <MONTH>{1}</MONTH>
                                                            <YEAR>{2}</YEAR>
                                                        </STARTDATE>", NewStartDate.Day.ToString(), NewStartDate.Month.ToString(), NewStartDate.Year.ToString());
                                sb_DateXML.AppendFormat(@"<DUEDATE>
                                                            <DAY>{0}</DAY>
                                                            <MONTH>{1}</MONTH>
                                                            <YEAR>{2}</YEAR>
                                                        </DUEDATE>", MinDate.Day.ToString(), MinDate.Month.ToString(), MinDate.Year.ToString());

                                if ((UpdatedDateXML != null) && (xpNav_Subflow != null))
                                {
                                    sb_DateXML.Append("<SUBFLOWACTIVITY TYPE='RELATION'>" + UpdatedDateXML + "</SUBFLOWACTIVITY>");
                                }

                                sb_DateXML.AppendFormat(@"<ACTIVITY_INFORMATION TYPE='RELATION'><ACTIVITYINFORMATION MODIFIED='TRUE'>
                                                        <ACTIVITYINFORMATIONID>{0}</ACTIVITYINFORMATIONID>
                                                        <DURATION>{1}</DURATION>
                                                    </ACTIVITYINFORMATION></ACTIVITY_INFORMATION>", str_ActInfoID, str_ActDuration);
                                sb_DateXML.Append("</ACTIVITY>");
                                // Get the predecessors of current Activity.
                                xpIter_FromAct = xpNav_ProcTr.Select("TRANSITION[TOACTIVITY/ACTIVITY/ACTIVITYID='" + str_ActivityID + "']/FROMACTIVITY/ACTIVITY");
                                while (xpIter_FromAct.MoveNext())
                                {
                                    if (!Vertices.Contains(xpIter_FromAct.Current.SelectSingleNode("ACTIVITYID").Value))
                                    {
                                        Vertices.Enqueue(xpIter_FromAct.Current.SelectSingleNode("ACTIVITYID").Value);
                                    }
                                }
                            }
                            else
                            {
                                // The Activity is dependant...Add the Activity to the dependancy List
                                sArr_DependantAct[Index++] = str_ActivityID;
                            }
                        }
                    }
                    // Update if Any dependant Activities
                    if ((sArr_DependantAct.Length > 0) && (Index > 0))
                    {
                        ErrorXml = UpdateDependantActivities_Edit(sArr_DependantAct,xpNav_ProcAct,xpNav_ProcTr, VisitedVertices, out DependantActXML);
                        if (ErrorXml != null)
                            return ErrorXml;
                        sb_DateXML.Append(DependantActXML);
                    }
                    // find the Process Start date                    
                    ProcessStartDate = GetProcessStartDate(VisitedVertices);
                    
                    sb_DateXML.Append("</PROCESS_ACTIVITY>");

                // The Process start date will be the minimum of all start dates. i.e. the start date of the Start Activity

                    sb_DateXML.Append(@"<STARTDATE>
                                        <DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                        + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                        + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>"
                                        + "</STARTDATE>");

                    sb_DateXML.AppendFormat(@"<PROCESS_INFORMATION TYPE='RELATION'><WORKFLOWPROCESSINFO MODIFIED='TRUE'>
                                                <WORKFLOWPROCESSINFOID>{0}</WORKFLOWPROCESSINFOID>
                                                <DURATION>{1}</DURATION>
                                            </WORKFLOWPROCESSINFO></PROCESS_INFORMATION>",str_ProcInfoID, d_ProcessDuration);
                    sb_DateXML.Append("</WORKFLOWPROCESS>");

                    UpdatedDateXML = sb_DateXML.ToString();
                return ErrorXml;
        }

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method returns the Predecessor Activity with the earliest start date</Purpose>        
        /// </summary>
        /// <param name="VisitedVertices">The has table of visited vertices</param>
        /// <param name="xpNav_ProcessTr">The Process Transition Node</param>
        /// <param name="str_ActivityID">The Activity ID whose dependant Activity will be returned</param>
        /// <param name="str_MinActID">The Activity ID with earliest start date</param>
        /// <param name="MinDate">Date of the Activity with minimum start date</param>
        private void GetActivityIDWithMinimumStartDate_Edit(Hashtable VisitedVertices, XPathNavigator xpNav_ProcessTr, String str_ActivityID, out String str_MinActID,out DateTime MinDate)
        {
            XPathNodeIterator   xpIter_SuccAct = null;
            DateTime            TempDate        ;            
            String              str_ToActID      = null;            
            
                xpIter_SuccAct = xpNav_ProcessTr.Select("TRANSITION[FROMACTIVITY/ACTIVITY/ACTIVITYID='" + str_ActivityID + "']/TOACTIVITY/ACTIVITY");
                TempDate       = new DateTime(1, 1, 1);
                MinDate        = new DateTime(1, 1, 1);
                str_MinActID   = String.Empty;
                while (xpIter_SuccAct.MoveNext())
                {
                    str_ToActID     = xpIter_SuccAct.Current.SelectSingleNode("ACTIVITYID").Value;
                    if (VisitedVertices.ContainsKey(str_ToActID))
                    {
                        MinDate = Convert.ToDateTime(VisitedVertices[str_ToActID]);
                        if (TempDate.Equals(new DateTime(1, 1, 1)))
                        {
                            TempDate = MinDate;
                            str_MinActID = str_ToActID;
                        }
                        else
                        {
                            if (TempDate < MinDate)
                            {
                                MinDate = TempDate;
                                str_MinActID = str_ToActID;
                            }
                        }
                    }
                    else
                    {
                        str_MinActID = String.Empty;
                        return;
                    }
                 
                }
        }

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method prepares the new date xml for process with start and end Activity</Purpose>
        /// </summary>
        /// <param name="xpNav_Process">XML Node that refers to the process</param>
        /// <param name="ProcessDueDate">The due date of the process</param>
        /// <param name="ProcessStartDate">the start date of the process</param>
        /// <param name="str_Duration">The duration of the process</param>
        private XML PrepareProcessDateXML_Edit(XPathNavigator xpNav_Process, DateTime ProcessDueDate, out DateTime ProcessStartDate, out String str_Duration)
        {
            StringBuilder   sb_DateXML      = null;
            XPathNodeIterator xpIter_Act    = null;
            String          str_ProcInfoID  = null;            
            
                sb_DateXML = new StringBuilder(100);
                str_Duration = xpNav_Process.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/DURATION").Value;
                str_ProcInfoID = xpNav_Process.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/WORKFLOWPROCESSINFOID").Value;
                GetStartDate(str_Duration, ProcessDueDate, out ProcessStartDate);
                sb_DateXML.Append("<WORKFLOWPROCESS MODIFIED='TRUE'>");
                sb_DateXML.AppendFormat("<WORKFLOWPROCESSID>{0}</WORKFLOWPROCESSID>", xpNav_Process.SelectSingleNode("WORKFLOWPROCESSID").Value);
                                        
                sb_DateXML.Append(@"<STARTDATE>
                                        <DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                        + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                        + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>"
                                 + @"</STARTDATE>
                                    <DUEDATE>
                                        <DAY>" + ProcessDueDate.Day.ToString() + "</DAY>"
                                        + "<MONTH>" + ProcessDueDate.Month.ToString() + "</MONTH>"
                                        + "<YEAR>" + ProcessDueDate.Year.ToString() + "</YEAR>"
                                 + "</DUEDATE>");
                                       
                xpIter_Act = xpNav_Process.Select("PROCESS_ACTIVITY/ACTIVITY");
                sb_DateXML.Append("<PROCESS_ACTIVITY TYPE='RELATION'>");
                while (xpIter_Act.MoveNext())
                {
                    sb_DateXML.Append(@"<ACTIVITY MODIFIED='TRUE'>
                                            <ACTIVITYID>" + xpIter_Act.Current.SelectSingleNode("ACTIVITYID").Value + "</ACTIVITYID>"
                                         + @"<STARTDATE>
                                               <DAY>" + ProcessStartDate.Day.ToString() + "</DAY>"
                                            + "<MONTH>" + ProcessStartDate.Month.ToString() + "</MONTH>"
                                            + "<YEAR>" + ProcessStartDate.Year.ToString() + "</YEAR>"
                                         + "</STARTDATE>"
                                         + @"<DUEDATE>
                                                   <DAY>" + ProcessDueDate.Day.ToString() + "</DAY>"
                                                + "<MONTH>" + ProcessDueDate.Month.ToString() + "</MONTH>"
                                                + "<YEAR>" + ProcessDueDate.Year.ToString() + "</YEAR>"
                                           + "</DUEDATE>"
                                      + "</ACTIVITY>");
                }
                sb_DateXML.AppendFormat(@"</PROCESS_ACTIVITY>
                                    <PROCESS_INFORMATION TYPE='RELATION'>
                                        <WORKFLOWPROCESSINFO MODIFIED='TRUE'>
                                            <WORKFLOWPROCESSINFOID>{0}</WORKFLOWPROCESSINFOID>
                                            <DURATION>{1}</DURATION>
                                        </WORKFLOWPROCESSINFO>
                                    </PROCESS_INFORMATION>
                                </WORKFLOWPROCESS>", str_ProcInfoID, str_Duration);
                return sb_DateXML.ToString();
        }

        /// <summary>
        /// <Purpose>This method updates the dependant Activities of the Proeces</Purpose>
        /// <Author>Mugdha Moolay Gadre</Author>        
        /// </summary>
        /// <param name="sArr_DependantAct">The array of the dependant Activities</param>
        /// <param name="xpNav_ProcessAct">XML node that refers to the PROCESS_ACTIVITY node</param>
        /// <param name="xpNav_ProcTr">XML node that refers to the PROCESS_TRANSITION node</param>
        /// <param name="VisitedVertices">Hastable consists of the visited vertices of the BFS</param>
        /// <param name="DependantActXML">The updated date XML for Dependant Activities</param>
        private ERRORXML UpdateDependantActivities_Edit(String[] sArr_DependantAct,XPathNavigator xpNav_ProcessAct,XPathNavigator xpNav_ProcTr, Hashtable VisitedVertices, out String DependantActXML)
        {
            String              str_DepActID        = null;
            String              str_ActDuration     = null;
            StringBuilder       sb_DateXML          = null;
            XPathNavigator      xpNav_Act           = null;
            XPathNavigator      xpNav_SuccAct       = null;
            XPathNavigator      xpNav_Subflow       = null;
            DateTime            MinDate             ; 
            DateTime            ProcessStartDate    ;
            DateTime            NewStartDate        ;
            String              str_MinActID        = null;
            Boolean             b_IsSuccEnd         = false;
            Boolean             b_IsActStart        = false;
            Boolean             b_IsActEnd          = false;
            XML                 UpdatedDateXML      = null;
            double              d_SubprocessDur     = 0;
            ERRORXML            ErrorXml            = null;
            
                sb_DateXML = new StringBuilder(100);
                for (int Index = 0; Index < sArr_DependantAct.Length; Index++)
                {
                    str_DepActID    = sArr_DependantAct[Index];
                    xpNav_Act       = xpNav_ProcessAct.SelectSingleNode("ACTIVITY[ACTIVITYID='" + str_DepActID + "']");
                    if (xpNav_Act != null)
                    {
                        GetActivityIDWithMinimumStartDate_Edit(VisitedVertices, xpNav_ProcTr, str_DepActID, out str_MinActID, out MinDate);
                        if (!String.IsNullOrEmpty(str_MinActID))
                        {
                            xpNav_SuccAct = xpNav_ProcessAct.SelectSingleNode("ACTIVITY[ACTIVITYID='" + str_MinActID + "']");

                            b_IsSuccEnd = Convert.ToBoolean(xpNav_SuccAct.SelectSingleNode("ENDACTIVITY").Value);
                            b_IsActStart = Convert.ToBoolean(xpNav_Act.SelectSingleNode("STARTACTIVITY").Value);
                            b_IsActEnd = Convert.ToBoolean(xpNav_Act.SelectSingleNode("ENDACTIVITY").Value);

                            // Check if the Activity is Subflow, if yes then give a recursive call
                            xpNav_Subflow = xpNav_Act.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS");

                            if (xpNav_Subflow != null)
                            {
                                if ((!b_IsSuccEnd) && (!b_IsActStart))
                                {
                                    long Min = ConvertDaysToMinutes(2);
                                    GetStartDate(Min.ToString(), MinDate, out MinDate);
                                    //MinDate = MinDate.AddDays(-1);
                                }
                                CalculateReverseTimeLine_Edit(xpNav_Subflow.OuterXml, MinDate, out UpdatedDateXML, out d_SubprocessDur, out ProcessStartDate);
                                NewStartDate = ProcessStartDate;
                                str_ActDuration = d_SubprocessDur.ToString();
                            }
                            else
                            {
                                if ((!b_IsSuccEnd) && (!b_IsActStart))
                                {
                                    long Min = ConvertDaysToMinutes(2);
                                    GetStartDate(Min.ToString(), MinDate, out MinDate);
                                }
                                GetStartDate(str_ActDuration, MinDate, out NewStartDate);
                            }
                            sb_DateXML.AppendFormat(@"<ACTIVITY MODIFIED='TRUE'>
                                                        <ACTIVITYID>" + str_DepActID + "</ACTIVITYID>"
                                                            + @"<STARTDATE>
                                                            <DAY>{0}</DAY>
                                                            <MONTH>{1}</MONTH>
                                                            <YEAR>{2}</YEAR>
                                                        </STARTDATE>", NewStartDate.Day.ToString(), NewStartDate.Month.ToString(), NewStartDate.Year.ToString() );
                            sb_DateXML.AppendFormat(@"<DUEDATE>
                                                            <DAY>{0}</DAY>
                                                            <MONTH>{1}</MONTH>
                                                            <YEAR>{2}</YEAR>
                                                        </DUEDATE>", MinDate.Day.ToString(), MinDate.Month.ToString(), MinDate.Year.ToString());

                            if ((UpdatedDateXML != null) && (xpNav_Subflow != null))
                            {
                                sb_DateXML.Append("<SUBFLOWACTIVITY TYPE='RELATION'>" + UpdatedDateXML + "</SUBFLOWACTIVITY>");
                            }
                            sb_DateXML.Append("</ACTIVITY>");
                        }
                    }
                }// End For
                DependantActXML = sb_DateXML.ToString();
                return ErrorXml;
        }        

        #endregion
        
        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method returns the start date given a due date</Purpose>
        /// </summary>
        /// <param name="str_Duration">The duration will be used to calculate the new start date</param>
        /// <param name="DueDate">The due date will be used to calculate the new start date</param>
        /// <param name="Startdate">The calculated start date will be returned.</param>
        private void GetStartDate(String str_Duration, DateTime DueDate, out DateTime Startdate)
        {
            String          InstalledPath       = null; //To get the installation path.
            StringBuilder   ConfigXMLPath       = null; //To build XML path.            
            XPathNavigator  DateCalculationNode = null; //To Select the DateCalculation Node.
            XPathNavigator  WeekDaysNode        = null; //To Select the WeekDays Node.
            XPathNavigator  DayHoursNode        = null; //To Select the Day Hours Node.
            String          WeekDays            = null; //To Get the WeekDays. 
            String          DayHours            = null; //To Get the Day Hours.
            int             MinutesPerDay       = 0;    //To Get the Minutes per day.
            Double          NoOfDaysByGivenDur  = 0;    //To Get the Total number of days required according to given duration in minutes.
            Double          RoundedNoOfDays     = 0;    //To Store Rounded Number of Days.
            DateTime        NewStartDate;                 //To Calculate new due date.

            //After Change
            XPathDocument   xp_ConfigXMLDoc     = null; 
            XPathNavigator  Root_ConfigXMLDoc   = null; 
            String          cachedItemWeekDays  = null;
            String          cachedItemWorkingHr = null; 
            
                cachedItemWeekDays = TenantName.ToUpper() + "_MWB_WEEKDAYS";
                cachedItemWorkingHr = TenantName.ToUpper() + "_MWB_WORKINGHOURS";
                if ((CacheManager.IsCached(cachedItemWeekDays.ToUpper())) && (CacheManager.IsCached(cachedItemWorkingHr.ToUpper())))
                {
                    WeekDays = CacheManager.GetCacheItem(cachedItemWeekDays).ToString();
                    DayHours = CacheManager.GetCacheItem(cachedItemWorkingHr).ToString();
                }
                else 
                { 
                    ConfigXMLPath = new StringBuilder();

                    InstalledPath = ALCommon.GetInstallationDir();
					ConfigXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(TenantName));

                    xp_ConfigXMLDoc = new XPathDocument(ConfigXMLPath.ToString());
                    Root_ConfigXMLDoc = xp_ConfigXMLDoc.CreateNavigator();

                    DateCalculationNode = Root_ConfigXMLDoc.SelectSingleNode("CONFIGURATION/DATECALCULATION");
                    if ((DateCalculationNode != null) && (DateCalculationNode.HasChildren))
                    {
                        WeekDaysNode = DateCalculationNode.SelectSingleNode("WORKINGDAYS");
                        if ((WeekDaysNode != null) && (!String.IsNullOrEmpty(WeekDaysNode.Value)))
                            WeekDays = WeekDaysNode.Value;
                        else
                            WeekDays = "5";

                        DayHoursNode = DateCalculationNode.SelectSingleNode("WORKINGHOURS");
                        if ((DayHoursNode != null) && (!String.IsNullOrEmpty(DayHoursNode.Value)))
                            DayHours = DayHoursNode.Value;
                        else
                            DayHours = "8";
                    }
                    else
                    {
                        WeekDays = "5";
                        DayHours = "8";
                    }
                    CacheManager.AddCacheItem(cachedItemWeekDays, WeekDays, ConfigXMLPath.ToString());
                    CacheManager.AddCacheItem(cachedItemWorkingHr, DayHours, ConfigXMLPath.ToString());   
                }
                //Calculate the minutes per day
                MinutesPerDay = Convert.ToInt32(DayHours) * 60;

                //Get the number of working days.
                if (Convert.ToInt32(str_Duration) > 0)
                {
                    NoOfDaysByGivenDur = Convert.ToDouble(str_Duration) / Convert.ToDouble(MinutesPerDay);
                    RoundedNoOfDays = Math.Round(NoOfDaysByGivenDur);
                    if (NoOfDaysByGivenDur > RoundedNoOfDays)
                        NoOfDaysByGivenDur = RoundedNoOfDays + 1;
                    else
                        NoOfDaysByGivenDur = RoundedNoOfDays;

                    NoOfDaysByGivenDur--;
                }
                else
                    NoOfDaysByGivenDur = 0;

                NewStartDate = DueDate;
                //Calculate Due date for this duration.
                if (WeekDays == "5")
                {
                    while (NoOfDaysByGivenDur > 0)
                    {
                        NewStartDate = NewStartDate.AddDays(-1);
                       
                        if ((NewStartDate.DayOfWeek != DayOfWeek.Sunday) && (NewStartDate.DayOfWeek != DayOfWeek.Saturday))
                            NoOfDaysByGivenDur--;
                    }
                }
                else if (WeekDays == "6")
                {
                    while (NoOfDaysByGivenDur > 0)
                    {
                        NewStartDate = NewStartDate.AddDays(-1);
                        if (NewStartDate.DayOfWeek != DayOfWeek.Sunday)
                            NoOfDaysByGivenDur--;
                    }
                }
                Startdate = NewStartDate;
        }        

        /// <summary>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>This method returns the start date with minimum value</Purpose>
        /// </summary>
        /// <param name="ht_StartDates">Hash table of Start dates of Activities within the process.</param>
        /// <returns>An Error XML </returns>
        private DateTime GetProcessStartDate(Hashtable ht_StartDates)
        {
            IDictionaryEnumerator ienum_Date = null;
            DateTime MinDate = new DateTime(1,1,1);
            DateTime TempDate ;
            
                ienum_Date = ht_StartDates.GetEnumerator();
                while (ienum_Date.MoveNext())
                {
                    if (MinDate.Equals(new DateTime(1, 1, 1)))
                        MinDate = Convert.ToDateTime(ienum_Date.Value);
                    else
                    { 
                        TempDate = Convert.ToDateTime(ienum_Date.Value);
                        if (TempDate < MinDate)
                            MinDate = TempDate;
                    }
                }
                return MinDate;
        }
        
        /// <summary>
        /// <Purpose>Consolidate the files for the route process.</Purpose>
        /// <Author>Parita Harniya</Author>
        /// </summary>
        /// <param name="WorkflowProcessNav">Process Navigator</param>
        /// <param name="str_ReviewFileID">Review file ID</param>
        /// <param name="ActivitySeqNo">Activity Seq. no.</param>
        /// <param name="FileSystemNav">File System Nav</param>
        /// <param name="FieldName">Field Name</param>
        private void GetLastFileUploadedForApprovals_MultiFile(XPathNavigator WorkflowProcessNav, String str_ReviewFileID, String ActivitySeqNo, out XPathNavigator FileSystemNav, out String FieldName)
        {
            XPathNavigator ProcessActivityNode = null;
            XPathNavigator ActivityNode        = null;
            XPathNavigator ActFileSystemNode   = null;
            StringBuilder  XPathQuery          = null;

                //FileSystemNav = WorkflowProcessNav.SelectSingleNode("FILESYSTEM");
                //FieldName     = "PROCESSFILE";
                FileSystemNav   = null;
                FieldName       = null;
                ProcessActivityNode = WorkflowProcessNav.SelectSingleNode("PROCESS_ACTIVITY");
                XPathQuery = new StringBuilder();

                for (int loopCnt = Convert.ToInt16(ActivitySeqNo) - 1; loopCnt > 1; loopCnt--)
                {
                    XPathQuery.AppendFormat("ACTIVITY[SEQUENCENUMBER='{0}']", loopCnt);
                    ActivityNode = ProcessActivityNode.SelectSingleNode(XPathQuery.ToString());

                    if (ActivityNode != null)
                    {
                        ActFileSystemNode = ActivityNode.SelectSingleNode("ACTIVITY_RESPONSE/RESPONSE[RESPONSE_PROCESSREVIEWDETAILS/PROCESSREVIEWDETAILS/REVIEWFILEID='"
                                    + str_ReviewFileID + 
                                    "']/RESPONSE_ACTIVITYREVIEWDETAILS/ACTIVITYREVIEWDETAILS/FILESYSTEM");
                        if ((ActFileSystemNode != null) && (ActFileSystemNode.HasChildren))
                        {
                            FileSystemNav = ActFileSystemNode;
                            FieldName     = "FILEWITHCOMMENTS";
                            break;
                        }
                    }
                    XPathQuery.Remove(0, XPathQuery.Length);
                }
        }

        /// <summary>
        /// <CreatedOn>11 Feb 2013</CreatedOn>
        /// <CreatedBy>Mugdha Gadre</CreatedBy>
        /// <Purpose>This methods initiates the next processes in the workflow after a process is completed</Purpose>
        /// </summary>
        /// <param name="str_ActivityID">The Activity ID of the process that is completed</param>
        /// <returns></returns>
        private ERRORXML StartNextProcess(String str_ActivityID)
        { 
            OMInterface     obj_OMI                     = null;
            XML             OutputXML                   = null;
            XML             SearchXML                   = null;
            ERRORXML        ErrorXml                    = null;
            XPathDocument   xpDoc_Activity              = null;
            XPathNavigator  xpNav_Activity              = null;
            XPathNavigator  xpNav_ParentProcess         = null;
            String          ParentProcessID             = null;

            SearchXML       = getSearchXMLForActivity(str_ActivityID);
            obj_OMI         = new OMInterface(UserID, CurrentLanguage, TenantName);
            ErrorXml        = obj_OMI.GetHierarchialData(SearchXML, ref OutputXML);
            if (ErrorXml == null)
            {
                xpDoc_Activity = new XPathDocument(new StringReader(OutputXML));
                xpNav_Activity = xpDoc_Activity.CreateNavigator();
                xpNav_Activity = xpNav_Activity.SelectSingleNode("ACTIVITYCOLLECTION/ACTIVITY");
                if (xpNav_Activity != null)
                {
                    xpNav_ParentProcess = xpNav_Activity.SelectSingleNode("PROCESS_ACTIVITY/WORKFLOWPROCESS");
                    if (xpNav_ParentProcess != null)
                    {
                        ParentProcessID = xpNav_ParentProcess.SelectSingleNode("WORKFLOWPROCESSID").Value;
                        if (xpNav_ParentProcess.HasChildren)
                        {
                            ErrorXml = StartNextActivities(Convert.ToUInt64(str_ActivityID), xpNav_ParentProcess);
                        }
                    }
                }
            }
            return ErrorXml;
        }

       
        private void AppendLog(String LogXML)
        {
            ProcessManager_LogObj.Info(LogXML);
        }

        private XML PrepareErrorXML(String InstanceID, String InstanceType, Exception ExcepObj, String ErrorXML)
        {
            StringBuilder   sb_ErrorXML = new StringBuilder(200);
            
            String StackTrace = String.Empty;
            String ErrorType = (ExcepObj == null ? "ERROR" : "EXCEPTION" );

            if (ExcepObj != null)
                StackTrace = ExcepObj.StackTrace;

            sb_ErrorXML.AppendFormat(@"<ERRORINFO>
                                        <TENANTNAME>{0}</TENANTNAME>
                                        <ERRORTYPE>{1}</ERRORTYPE>
                                        <INSTANCETYPE>{2}</INSTANCETYPE>
                                        <INSTANCEID>{3}</INSTANCEID>
                                        <ADDITIONALINFO>{4}</ADDITIONALINFO>                                        
                                        <STACKTRACE>{5}</STACKTRACE>
                                    </ERRORINFO>",TenantName, ErrorType, InstanceType, InstanceID, ErrorXML, StackTrace);

            return sb_ErrorXML.ToString();
            
        }

        # region Consolidated Media File Helper Functions
        private iTextSharp.text.Rectangle getBounds(PdfStamper stamper, int pageno, float W, float H, iTextSharp.text.Rectangle rect)
        {
            /* var llx = float.Parse(ann["x"].ToString());
              var lly = H - (float.Parse(ann["y"].ToString()) + float.Parse(ann["height"].ToString()));


              var urx = float.Parse(ann["x"].ToString()) + float.Parse(ann["width"].ToString());
              var ury = H - float.Parse(ann["y"].ToString());
             */

            var llx = rect.Left;
            var lly = H - rect.Bottom;


            var urx = rect.Right;
            var ury = H - rect.Top;



            float cbOriginX = 0;
            float cbOriginY = 0;
            if (stamper.Reader.GetBoxSize(pageno, "crop") != null)
            {
                cbOriginX = stamper.Reader.GetBoxSize(pageno, "crop").GetLeft(0);
                cbOriginY = stamper.Reader.GetBoxSize(pageno, "crop").GetBottom(0);
            }

            float mbOriginX = 0;
            float mbOriginY = 0;
            if (stamper.Reader.GetBoxSize(pageno, "media") != null)
            {
                mbOriginX = stamper.Reader.GetBoxSize(pageno, "media").GetLeft(0);
                mbOriginY = stamper.Reader.GetBoxSize(pageno, "media").GetBottom(0);
            }

            var originX = (cbOriginX > mbOriginX ? cbOriginX : mbOriginX);
            var originY = (cbOriginY > mbOriginY ? cbOriginY : mbOriginY);

            return new iTextSharp.text.Rectangle(originX + llx, originY + lly, originX + urx, originY + ury);
        }

        private void AddComments(PdfAnnotation ann, iTextSharp.text.Rectangle rect, Hashtable comments, PdfStamper stamper, int pageno)
        {
            for (int i = 1; i < comments.Count; i++)
            {
                Color replyclr = Color.FromArgb(Convert.ToInt32((comments[i] as Hashtable)["color"]));




                PdfAnnotation replyAnn = new PdfAnnotation(stamper.Writer, rect);
                replyAnn.Put(PdfName.SUBTYPE, PdfName.TEXT);
                replyAnn.Put(PdfName.TYPE, PdfName.ANNOT);
                replyAnn.Color = new BaseColor(replyclr);
                replyAnn.Put(PdfName.T, new PdfString(Convert.ToString((comments[i] as Hashtable)["authorName"]), PdfObject.TEXT_UNICODE));
                replyAnn.Put(PdfName.CONTENTS, new PdfString(Convert.ToString((comments[i] as Hashtable)["content"]), PdfObject.TEXT_UNICODE));


                string mydate = Convert.ToString((comments[i] as Hashtable)["lastModified"]).Replace("GMT", "");
                replyAnn.Put(PdfName.M, new PdfDate(DateTime.ParseExact(mydate, "ddd MMM dd HH:mm:ss zzzz yyyy", null)));
                replyAnn.Put(PdfName.IRT, ann.IndirectReference);

                // we add a reference to the text annotation to the popup annotation
                stamper.AddAnnotation(replyAnn, pageno);
            }

        }

        private void AddAnchorAnnotation(Hashtable o, Hashtable comments, PdfStamper stamper, PdfReader reader)
        {
            int pageno = Convert.ToInt16(o["PageNo"]);

            iTextSharp.text.Rectangle PageSize = reader.GetPageSize(reader.GetPageNRelease(pageno));
            float W = PageSize.Width;
            float H = PageSize.Height;

            iTextSharp.text.Rectangle orgRect = new iTextSharp.text.Rectangle(float.Parse(o["x"].ToString()),
                                                        float.Parse(o["y"].ToString()) + float.Parse(o["height"].ToString()),
                                                        float.Parse(o["x"].ToString()) + float.Parse(o["width"].ToString()),
                                                        float.Parse(o["y"].ToString()));
            iTextSharp.text.Rectangle rect = getBounds(stamper, pageno, W, H, orgRect);

            var cb = stamper.GetOverContent(pageno);

            Color clr = Color.FromArgb(Convert.ToInt32(o["color"]));

            PdfAnchorAnnotation anchorAnn = new PdfAnchorAnnotation(stamper.Writer);
            anchorAnn.Rectangle = rect;
            anchorAnn.Color = new BaseColor(clr);
            anchorAnn.Put(PdfName.T, new PdfString(Convert.ToString((comments[0] as Hashtable)["authorName"]), PdfObject.TEXT_UNICODE));
            anchorAnn.Contents = Convert.ToString((comments[0] as Hashtable)["content"]);
            anchorAnn.Open = true;
            string mydate = Convert.ToString((comments[0] as Hashtable)["lastModified"]).Replace("GMT", "");



            anchorAnn.DateTime = DateTime.ParseExact(mydate, "ddd MMM dd HH:mm:ss zzzz yyyy", null);

            // anchorAnn.Opacity = 0.001;
            //  anchorAnn.Flags = PdfAnnotation.FLAGS_READONLY;



            var x = (float.Parse(o["width"].ToString())) / 2 + 1;
            var y = (float.Parse(o["height"].ToString())) / 2 + 1;
            var r = float.Parse(o["width"].ToString()) / 2;

            PdfTemplate template = PdfTemplate.CreateTemplate(stamper.Writer, float.Parse(o["width"].ToString()) + 2f, float.Parse(o["height"].ToString()) + 2f);

            PdfShadingPattern pattern = new PdfShadingPattern(PdfShading.SimpleRadial(stamper.Writer, x, y, r - 2, x, y, r + 1, BaseColor.LIGHT_GRAY, BaseColor.WHITE));

            template.SetShadingFill(pattern);
            template.Circle(x, y, r + 1);
            template.Fill();
            template.SetRGBColorStroke(Color.White.R, Color.White.G, Color.White.B);
            template.SetRGBColorFill(clr.R, clr.G, clr.B);
            template.Circle(x, y, r);
            template.FillStroke();
            anchorAnn.SetAppearance(PdfAnnotation.APPEARANCE_NORMAL, template);

            stamper.AddAnnotation(anchorAnn, pageno);
            AddComments(anchorAnn, rect, comments, stamper, pageno);

        }

        private void AddAreaAnnotation(Hashtable o, Hashtable comments, PdfStamper stamper, PdfReader reader)
        {


            int pageno = Convert.ToInt16(o["PageNo"]);

            iTextSharp.text.Rectangle PageSize = reader.GetPageSize(reader.GetPageNRelease(pageno));
            float W = PageSize.Width;
            float H = PageSize.Height;

            iTextSharp.text.Rectangle orgRect = new iTextSharp.text.Rectangle(float.Parse(o["x"].ToString()),
                                                                   float.Parse(o["y"].ToString()) + float.Parse(o["height"].ToString()),
                                                                   float.Parse(o["x"].ToString()) + float.Parse(o["width"].ToString()),
                                                                   float.Parse(o["y"].ToString()));
            iTextSharp.text.Rectangle rect = getBounds(stamper, pageno, W, H, orgRect);

            var cb = stamper.GetOverContent(pageno);

            Color clr = Color.FromArgb(Convert.ToInt32(o["color"]));


            cb.SetRGBColorStroke(clr.R, clr.G, clr.B);

            rect.Border = iTextSharp.text.Rectangle.BOX;
            rect.BorderWidth = 1;
            rect.BorderColor = new BaseColor(clr);// BaseColor.RED;
            cb.Rectangle(rect);
            cb.Stroke();


            PdfRectAnnotation areaAnn = new PdfRectAnnotation(stamper.Writer);
            areaAnn.Rectangle = rect;
            areaAnn.Color = new BaseColor(clr);
            areaAnn.Put(PdfName.T, new PdfString(Convert.ToString((comments[0] as Hashtable)["authorName"]), PdfObject.TEXT_UNICODE));

            areaAnn.Contents = Convert.ToString((comments[0] as Hashtable)["content"]);
            areaAnn.Open = true;
            string mydate = Convert.ToString((comments[0] as Hashtable)["lastModified"]).Replace("GMT", "");
            areaAnn.DateTime = DateTime.ParseExact(mydate, "ddd MMM dd HH:mm:ss zzzz yyyy", null);

            stamper.AddAnnotation(areaAnn, pageno);
            AddComments(areaAnn, rect, comments, stamper, pageno);



        }

        private void AddHighlightAnnotation(Hashtable o, Hashtable comments, PdfStamper stamper, PdfReader reader)
        {

            int pageno = Convert.ToInt16(o["PageNo"]);

            iTextSharp.text.Rectangle PageSize = reader.GetPageSize(reader.GetPageNRelease(pageno));
            float W = PageSize.Width;
            float H = PageSize.Height;

            iTextSharp.text.Rectangle orgRect = new iTextSharp.text.Rectangle(float.Parse(o["x"].ToString()),
                                                                  float.Parse(o["y"].ToString()) + float.Parse(o["height"].ToString()),
                                                                  float.Parse(o["x"].ToString()) + float.Parse(o["width"].ToString()),
                                                                  float.Parse(o["y"].ToString()));
            iTextSharp.text.Rectangle rect = getBounds(stamper, pageno, W, H, orgRect);


            ArrayList rectangles = o["rectangles"] as ArrayList;


            var cb = stamper.GetOverContent(pageno);

            Color clr = Color.FromArgb(Convert.ToInt32(o["color"]));

            PdfArray quadPoints = new PdfArray();
            if (rectangles != null && rectangles.Count > 0)
            {
                PdfHighlightAnnotation highlightAnn = new PdfHighlightAnnotation(stamper.Writer);
                highlightAnn.Rectangle = rect;
                highlightAnn.Subject = Convert.ToString((comments[0] as Hashtable)["authorName"]);
                highlightAnn.Put(PdfName.T, new PdfString(Convert.ToString((comments[0] as Hashtable)["authorName"]), PdfObject.TEXT_UNICODE));
                highlightAnn.Contents = Convert.ToString((comments[0] as Hashtable)["content"]);
                highlightAnn.Open = true;
                highlightAnn.Color = new BaseColor(clr);


                foreach (Hashtable markup in rectangles)
                {

                    iTextSharp.text.Rectangle tRect = new iTextSharp.text.Rectangle(float.Parse(markup["x"].ToString()),
                                                       float.Parse(markup["y"].ToString()) + float.Parse(markup["height"].ToString()),
                                                       float.Parse(markup["x"].ToString()) + float.Parse(markup["width"].ToString()),
                                                       float.Parse(markup["y"].ToString()));

                    iTextSharp.text.Rectangle updatedRect = getBounds(stamper, pageno, W, H, tRect);

                    quadPoints.Add(new PdfNumber(updatedRect.GetLeft(0))); //0,0
                    quadPoints.Add(new PdfNumber(updatedRect.GetTop(0)));

                    quadPoints.Add(new PdfNumber(updatedRect.GetRight(0))); //1,0
                    quadPoints.Add(new PdfNumber(updatedRect.GetTop(0)));

                    quadPoints.Add(new PdfNumber(updatedRect.GetLeft(0))); //0,1
                    quadPoints.Add(new PdfNumber(updatedRect.GetBottom(0)));

                    quadPoints.Add(new PdfNumber(updatedRect.GetRight(0))); //1,1
                    quadPoints.Add(new PdfNumber(updatedRect.GetBottom(0)));
                }

                string mydate = Convert.ToString((comments[0] as Hashtable)["lastModified"]).Replace("GMT", "");
                highlightAnn.DateTime = DateTime.ParseExact(mydate, "ddd MMM dd HH:mm:ss zzzz yyyy", null);

                //highlightAnn.Flags = PdfAnnotation.FLAGS_READONLY;

                highlightAnn.Put(PdfName.QUADPOINTS, quadPoints);
                stamper.AddAnnotation(highlightAnn, pageno);
                AddComments(highlightAnn, rect, comments, stamper, pageno);
            }
        }

        private void AddStrikeoutAnnotation(Hashtable o, Hashtable comments, PdfStamper stamper, PdfReader reader)
        {

            int pageno = Convert.ToInt16(o["PageNo"]);

            iTextSharp.text.Rectangle PageSize = reader.GetPageSize(reader.GetPageNRelease(pageno));
            float W = PageSize.Width;
            float H = PageSize.Height;

            iTextSharp.text.Rectangle orgRect = new iTextSharp.text.Rectangle(float.Parse(o["x"].ToString()),
                                                                  float.Parse(o["y"].ToString()) + float.Parse(o["height"].ToString()),
                                                                  float.Parse(o["x"].ToString()) + float.Parse(o["width"].ToString()),
                                                                  float.Parse(o["y"].ToString()));
            iTextSharp.text.Rectangle rect = getBounds(stamper, pageno, W, H, orgRect);


            ArrayList rectangles = o["rectangles"] as ArrayList;


            var cb = stamper.GetOverContent(pageno);

            Color clr = Color.FromArgb(Convert.ToInt32(o["color"]));

            PdfArray quadPoints = new PdfArray();
            if (rectangles != null && rectangles.Count > 0)
            {
                PdfStrikeoutAnnotation strikeoutAnn = new PdfStrikeoutAnnotation(stamper.Writer);
                strikeoutAnn.Rectangle = rect;
                strikeoutAnn.Subject = Convert.ToString((comments[0] as Hashtable)["authorName"]);
                strikeoutAnn.Put(PdfName.T, new PdfString(Convert.ToString((comments[0] as Hashtable)["authorName"]), PdfObject.TEXT_UNICODE));
                strikeoutAnn.Contents = Convert.ToString((comments[0] as Hashtable)["content"]);
                strikeoutAnn.Open = true;
                strikeoutAnn.Color = new BaseColor(clr);


                foreach (Hashtable markup in rectangles)
                {

                    iTextSharp.text.Rectangle tRect = new iTextSharp.text.Rectangle(float.Parse(markup["x"].ToString()),
                                                       float.Parse(markup["y"].ToString()) + float.Parse(markup["height"].ToString()),
                                                       float.Parse(markup["x"].ToString()) + float.Parse(markup["width"].ToString()),
                                                       float.Parse(markup["y"].ToString()));

                    iTextSharp.text.Rectangle updatedRect = getBounds(stamper, pageno, W, H, tRect);

                    quadPoints.Add(new PdfNumber(updatedRect.GetLeft(0))); //0,0
                    quadPoints.Add(new PdfNumber(updatedRect.GetTop(0)));

                    quadPoints.Add(new PdfNumber(updatedRect.GetRight(0))); //1,0
                    quadPoints.Add(new PdfNumber(updatedRect.GetTop(0)));

                    quadPoints.Add(new PdfNumber(updatedRect.GetLeft(0))); //0,1
                    quadPoints.Add(new PdfNumber(updatedRect.GetBottom(0)));

                    quadPoints.Add(new PdfNumber(updatedRect.GetRight(0))); //1,1
                    quadPoints.Add(new PdfNumber(updatedRect.GetBottom(0)));
                }

                string mydate = Convert.ToString((comments[0] as Hashtable)["lastModified"]).Replace("GMT", "");
                strikeoutAnn.DateTime = DateTime.ParseExact(mydate, "ddd MMM dd HH:mm:ss zzzz yyyy", null);

                // strikeoutAnn.Flags = PdfAnnotation.FLAGS_READONLY;

                strikeoutAnn.Put(PdfName.QUADPOINTS, quadPoints);
                stamper.AddAnnotation(strikeoutAnn, pageno);
                AddComments(strikeoutAnn, rect, comments, stamper, pageno);
            }


            // CreateAnnotation(comment, BaseColor.RED, bounds, stamper, pageno);

        }

        #endregion

        #endregion

    }
}
