using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.ObjectManager;
using Assetlink.DataAccessLayer;
using System.Globalization;
using Assetlink.IOProxy;

namespace Assetlink.SessionManager
{
    using ERRORXML = String;
    /// <summary>
    /// <CreatedDate>18 Sept 2006</CreatedDate>
    /// <Author>Mr. Rahul Pawar</Author>
    /// <Purpose>
    /// This class will have all the methods defined for SessionManager.
    /// </Purpose>
    /// </summary>
    public partial class SessionManagerClass : AssetlinkBase
    {
        #region Private Members

        TenantSettings              TenantSetting                   = null;

        enum USERSTATUS : int
        {
            ACTIVE      = 0,
            DELETED     = 1,
            PENDING     = 2,
            REJECTED    = 3,
            DISABLED    = 4,
            LOCKED      = 5,
        }

        #endregion Private Members
        
        #region Constructor
        /// <summary>
        /// 
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="CurrentLanguage"></param>
        /// <param name="TenantName"></param>
        public SessionManagerClass(ulong SessionID, ulong UserID, String CurrentLanguage, String TenantName)
            : base(SessionID, UserID, CurrentLanguage, TenantName)
        {
            if(!String.IsNullOrEmpty(TenantName))
                TenantSetting = new TenantSettings(TenantName);
        }    
        #endregion Constructor

        # region Public Methods

        /// <summary>
        /// <CreateDate>6 Oct 2006</CreateDate>
        /// <Author>Sagar Dasalkar</Author>
        /// <Purpose>This method removes temporary files and session created for user (Signs out from Assetlink). </Purpose>
        /// </summary>
        /// <param name="UserID">Unique UserID</param>
        /// <param name="SessionID">SessionID of the User</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        public ERRORXML SignOut(ulong UserID,ulong SessionID)
        {
            String          InstallationPath    = null;
            String          ErrorXML            = null;
            StringBuilder   TemporaryPath       = null;
            StringBuilder   InputData           = null;
            StringBuilder   StrQuery            = null;
            ALError         ALErrorObj          = null;
            DataManager     OMDataManager       = null;
            OMError         ErrorObj            = null;
            Context         ContextObj          = null;
            DirectoryInfo   TemporaryDir;

            try
            {                
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>SessionManager.SessionManagerClass.SignOut</APINAME><USERID>{0}</USERID><SESSIONID>{2}</SESSIONID></ADDITIOALINFO>", UserID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), SessionID);

                ContextObj = new Context(UserID, SessionID, TenantName, "SessionManager", ContextAdditionalInfo.ToString());
                // Read Installation Path
                //InstallationPath    = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");
                //InstallationPath read changed on 16 October 2006
                InstallationPath = ALCommon.GetInstallationDir();
                TemporaryPath = new StringBuilder();
                StrQuery = new StringBuilder();

                //TemporaryPath.AppendFormat("{0}{1}\\Data\\{2}\\{3}", InstallationPath, TenantName, UserID.ToString(), SessionID.ToString());
                //TemporaryPath changed on 16 October 2006
                TemporaryPath.AppendFormat("{0}\\Temp\\{1}", InstallationPath, SessionID.ToString());
                TemporaryDir = new DirectoryInfo(TemporaryPath.ToString());
                OMDataManager = new DataManager(this.TenantName, false);

                //Delete Session from ALOM_SESSION
                DynamicSQLParameter m_parameter_1 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(), SessionID.ToString(), "BIGINT");

                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_1);

                StrQuery.AppendFormat("DELETE FROM ALOM_SESSION WHERE SESSIONID= {0} ", m_parameter_1.m_ParameterName);

                

                //Execute Query
                ErrorObj = OMDataManager.DataProvider.Execute(StrQuery.ToString());
                if (ErrorObj != null)
                {
                   return ErrorObj.Serialize();
                }

                if (TemporaryDir.Exists)
                {
                    //TemporaryDir.Delete(true);                    
                    SASMOMDirectory.DeleteFolder(TemporaryDir.FullName, true, ContextObj);                                        
                }                
            }
            catch (Exception ExceptionObject)
            {
               

                InputData = new StringBuilder();
                InputData.AppendFormat("SessionID = {0}", SessionID);
                ALErrorObj = new ALError(ErrorCodes.USER_SIGNOUT_FAILED,
                                             "Exception while signing out.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
            finally
            {                
                if (OMDataManager != null && OMDataManager.DataProvider.ConnectionObj.State == System.Data.ConnectionState.Open)
                    OMDataManager.DataProvider.Close();
            }

            return null;
        }

        /// <summary>
        /// <CreateDate>18 Sept 2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// <Purpose>This method authenticates User against Assetlink or External Mode. </Purpose>
        /// </summary>
        /// <param name="LoginName">LoginName to be validated</param>
        /// <param name="Password">Password of that User</param>
        /// <param name="IPAddress">IP Address of the host making request.</param>
        /// <param name="Mode">Authentication mode (LOGIN/EMAIL/INTERSOLUTION/INTERNAL).</param>
        /// <param name="CallerIdentity">Identity of caller.</param>
        /// <param name="SessionID">out parameter which is returned if user is authenticated</param>
        /// <param name="UserInfoXML">out parameter which is returned if user is authenticated with UserinfoXML</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        public ERRORXML AuthenticateUser(String LoginName, String Password, String IPAddress, String Mode, String CallerIdentity,
                                        ref ulong SessionID, ref String UserInfoXML, out bool bAccountLockedNow)
        {
            String                      OutputXML                       = null;
            String                      ErrorXML                        = null;
            String                      ErrorXMLString                  = null;
            String                      EncryptedPassword               = null;
            String                      ExistingDBPassword              = null;
            String                      AuthenticationType              = null;
            String                      UserStatus                      = null;
            String                      AdminAccountPassword            = null;
            String                      str_FieldCollectionXml          = null;
            String                      str_TenantAccUnlockedPassword   = null;
            StringBuilder               InputData                       = null;
            
            XmlDocument                 xmlDoc_SearchUserXml            = null;
            XmlNode                     xmlNode_UserCollection          = null;
            XmlNode                     xmlNode_User                    = null;
            XmlNode                     xmlNode_StatusNode              = null;
            XmlNode                     xmlNode_AuthenticationType      = null;
            
            XPathDocument               xpDoc_AdminUserDetails          = null;
            XPathNavigator              xpNav_User                      = null;

            ALError                     ALErrorObj                      = null;
            ObjectManagerClass          obj_ObjectManagerClass          = null;

            ulong                       UserID                          = 0;
            bool                        b_UserPasswordAdminPassword     = false;

            bAccountLockedNow                                           = false;

            try
            {
                UserInfoXML = null;
                obj_ObjectManagerClass = new ObjectManagerClass(this.UserID, this.CurrentLanguage, this.TenantName, false);
                
                ErrorXML = GetUser(LoginName, null, obj_ObjectManagerClass, ref OutputXML);
                
                if (!String.IsNullOrEmpty(ErrorXML))
                    return ErrorXML;

                if (!String.IsNullOrEmpty(OutputXML))
                {
                    xmlDoc_SearchUserXml = new XmlDocument();
                    xmlDoc_SearchUserXml.LoadXml(OutputXML);
                    xmlNode_UserCollection = xmlDoc_SearchUserXml.SelectSingleNode("USERCOLLECTION");
                    xmlNode_User = xmlNode_UserCollection.SelectSingleNode("USER");

                    if (xmlNode_User == null)
                    {
                        return GetErrorXmlForErrorCode(ErrorCodes.USER_DOESNOT_EXIST);
                    }
                    else
                    {
                        // This will authenticate user against LDAP
                    }
                    xmlNode_AuthenticationType = xmlNode_User.SelectSingleNode("AUTHENTICATIONMODE");
                    AuthenticationType = xmlNode_AuthenticationType.InnerText;
                    UserID =Convert.ToUInt64(xmlNode_User.SelectSingleNode("USERID").InnerText); 
                    //Authentication type decides how authentication will be performed. 
                    //Authentication type MOM authenticates user by fetching data from database
                    //Other type can be LDAP
                    if (AuthenticationType.ToUpper() == "MOM")
                    {
                        xmlNode_StatusNode = xmlNode_User.SelectSingleNode("STATUS");
                        UserStatus = xmlNode_StatusNode.InnerText.ToUpper();

                        //New Code: No need to encrypt the proivded password. 
                        //This is due to the dot net's new encryption-decryption code added. Compare the plain (decrypted) password strings.
                        EncryptedPassword   = Password;
                        ExistingDBPassword  = xmlNode_User.SelectSingleNode("PASSWORD").InnerText;
                        ExistingDBPassword  = ALCommon.Decrypt(ExistingDBPassword);

                        if (UserStatus == "LOCKED")
                        {
                            //Case - Unlock password entered and user is admin: Unlock admin account
                            //New Code: Decrypt the unlock password stored into account policy settings and then compare it.
                            str_TenantAccUnlockedPassword = ALCommon.Decrypt(TenantSetting.AccountPolicySettings.UnlockPassword);
                            if ((EncryptedPassword.CompareTo(str_TenantAccUnlockedPassword) == 0) && (IsUserAdmin(LoginName)))
                            {
                                ErrorXMLString = CreateUserSession(UserID, IPAddress, Mode, CallerIdentity, false, false, xmlNode_User, obj_ObjectManagerClass, ref SessionID);
                                if (!String.IsNullOrEmpty(ErrorXMLString))
                                    return ErrorXMLString;
                                
                                //Update LOGINFAILURECOUNT, STATUS, LOGINFAILURECOUNT 
                                ErrorXML = UpdateUser(UserID, USERSTATUS.ACTIVE, 0, true, false, obj_ObjectManagerClass, ref xmlNode_User);
                                if (!String.IsNullOrEmpty(ErrorXML))
                                    return PrepareErrorXML(ErrorXML, ErrorCodes.UPDATE_FAILED, ErrorObjectType.AUTHENTICATION);

                                UserInfoXML = xmlNode_User.OuterXml;

                                //Check for password expiry - Password expiry check will always be performed after authentication and 
                                //session creation because if password expires, user need to change password and
                                //change password interaction requires valid session.
                                if (IsUserPasswordExpired(xmlNode_User))
                                    return GetErrorXmlForErrorCode(ErrorCodes.USER_PASSWORD_EXPIRED);

                                return null;
                            }
                            else
                            {
                                //Case - auto unlock : Unlock user account if unlock hours are elapsed (from account locked date)
                                if (ExistingDBPassword == EncryptedPassword)
                                {
                                    TimeSpan TimeSpan_Diff = DateTime.Now.Subtract(ALCommon.XmlToDateTime(xmlNode_User.SelectSingleNode("ACCOUNTLOCKEDDATE")));
                                    if (TimeSpan_Diff.TotalHours >= TenantSetting.AccountPolicySettings.AutoUnlockHours)
                                    {
                                        ErrorXMLString = CreateUserSession(UserID, IPAddress, Mode, CallerIdentity, false, false, 
                                                                        xmlNode_User, obj_ObjectManagerClass, ref SessionID);
                                        if (!String.IsNullOrEmpty(ErrorXMLString))
                                            return ErrorXMLString;

                                        //Unlock account of admin user, update LOGINFAILURECOUNT, LASTLOGINDATE, STATUS
                                        ErrorXML = UpdateUser(UserID, USERSTATUS.ACTIVE, 0, true, false, obj_ObjectManagerClass, ref xmlNode_User);
                                        if (!String.IsNullOrEmpty(ErrorXML))
                                            return PrepareErrorXML(ErrorXML, ErrorCodes.UPDATE_FAILED, ErrorObjectType.AUTHENTICATION);

                                        UserInfoXML = xmlNode_User.OuterXml;

                                        //Check for password expiry - Password expiry check will always be performed after authentication and 
                                        //session creation because if password expires, user need to change password and
                                        //change password interaction requires valid session.
                                        if (IsUserPasswordExpired(xmlNode_User))
                                            return GetErrorXmlForErrorCode(ErrorCodes.USER_PASSWORD_EXPIRED);
                                        
                                        return null;
                                    }
                                }
                            }
                        }

                        if (UserStatus == "LOCKED" || UserStatus == "DISABLED" || UserStatus == "DELETED" || UserStatus == "REJECTED" || UserStatus == "PENDING")
                        {
                            ALErrorObj = new ALError(ErrorCodes.AUTHENTICATION_FAILED,
                                     "User Account is " + xmlNode_StatusNode.InnerText, ErrorObjectType.AUTHENTICATION,
                                     "Entity", ErrorType.ERROR, null, "Authentication Failed");
                            return ALErrorObj.Serialize();
                        }

                        if (EncryptedPassword != ExistingDBPassword)
                        {
                            //Case - Impersonation by admin (using admin password) : create a session
                            if (!IsUserAdmin(LoginName))
                            {
                                #region Field collection Xml to fetch user password
                                str_FieldCollectionXml = @"<FIELDCOLLECTION>
                                                                <TARGETENTITYFIELDS>
                                                                    <FIELD>USERID</FIELD>
                                                                    <FIELD>PASSWORD</FIELD>
                                                                </TARGETENTITYFIELDS>
                                                           </FIELDCOLLECTION>";
                                #endregion Field collection Xml to fetch user password

                                ErrorXML = GetUser("admin", str_FieldCollectionXml, obj_ObjectManagerClass, ref OutputXML);

                                if (!String.IsNullOrEmpty(ErrorXML))
                                    return ErrorXML;

                                if (!String.IsNullOrEmpty(OutputXML))
                                {
                                    xpDoc_AdminUserDetails = new XPathDocument(new StringReader(OutputXML));
                                    xpNav_User = xpDoc_AdminUserDetails.CreateNavigator().SelectSingleNode("USERCOLLECTION/USER");
                                    if (xpNav_User == null)
                                        return GetErrorXmlForErrorCode(ErrorCodes.USER_DOESNOT_EXIST);

                                    AdminAccountPassword = xpNav_User.SelectSingleNode("PASSWORD").Value;

                                    //If the Admin account password is blank, don't allow impersonation by admin.
                                    if (String.IsNullOrEmpty(AdminAccountPassword))
                                    {
                                        /*If User password is not same as Admin account password, set this flag to false, so that 
                                         * the LOGINFAILURECOUNT can be updated and the user account can be locked if login failure count exceeds
                                         * the login failure count setting.*/
                                        b_UserPasswordAdminPassword = false;
                                    }
                                    else
                                    {
                                        /*If User password is same as Admin password, set this flag to true. This means that admin could successfully
                                         * login to any other user account using admin account password*/
                                        //New Code: Decrypt the admin account password and then compare it.
                                        AdminAccountPassword = ALCommon.Decrypt(AdminAccountPassword);
                                        if (EncryptedPassword.CompareTo(AdminAccountPassword) == 0)
                                            b_UserPasswordAdminPassword = true;
                                    }

                                    if (b_UserPasswordAdminPassword)
                                    {
                                        ErrorXMLString = CreateUserSession(UserID, IPAddress, Mode, CallerIdentity, false, true, xmlNode_User, obj_ObjectManagerClass, ref SessionID);
                                        if (!String.IsNullOrEmpty(ErrorXMLString))
                                            return ErrorXMLString;

                                        //Update LASTLOGINDATE, LOGINFAILURECOUNT, STATUS
                                        ErrorXML = UpdateUser(UserID, USERSTATUS.ACTIVE, 0, true, false, obj_ObjectManagerClass, ref xmlNode_User);
                                        if (!String.IsNullOrEmpty(ErrorXML))
                                            return PrepareErrorXML(ErrorXML, ErrorCodes.UPDATE_FAILED, ErrorObjectType.AUTHENTICATION);
                                        
                                        UserInfoXML = xmlNode_User.OuterXml;

                                        //Check for password expiry - Password expiry check will always be performed after authentication and 
                                        //session creation because if password expires, user need to change password and
                                        //change password interaction requires valid session.
                                        if (IsUserPasswordExpired(xmlNode_User))
                                            return GetErrorXmlForErrorCode(ErrorCodes.USER_PASSWORD_EXPIRED);

                                        return null;
                                    }
                                }
                            }

                            //Case - Password doesn't match : Lock user account (if number of login attemps > login failure count) and return error
                            //we also need to send mail to user about ACCOUNT LOCK
                            return LockUserAccount(UserID, xmlNode_User, obj_ObjectManagerClass, out bAccountLockedNow);
                        }
                        else
                        {
                            //Case - Entered password and password from database match
                            ErrorXMLString = CreateUserSession(UserID, IPAddress, Mode, CallerIdentity, false, false, xmlNode_User, obj_ObjectManagerClass, ref SessionID);
                            if (!String.IsNullOrEmpty(ErrorXMLString))
                                return ErrorXMLString;

                            // Update LASTLOGINDATE, LOGINFAILURECOUNT, STATUS
                            ErrorXML = UpdateUser(UserID, USERSTATUS.ACTIVE, 0, true, false , obj_ObjectManagerClass, ref xmlNode_User);
                            if (!String.IsNullOrEmpty(ErrorXML))
                                return PrepareErrorXML(ErrorXML, ErrorCodes.UPDATE_FAILED, ErrorObjectType.AUTHENTICATION);

                            UserInfoXML = xmlNode_User.OuterXml;

                            //Check for password expiry - Password expiry check will always be performed after authentication and 
                            //session creation because if password expires, user need to change password and
                            //change password interaction requires valid session.
                            if (IsUserPasswordExpired(xmlNode_User))
                                return GetErrorXmlForErrorCode(ErrorCodes.USER_PASSWORD_EXPIRED);
                        }
                    }
                    else
                    {
                        // Authenticate User against LDAP 
                    }
                }
                else
                {
                    return GetErrorXmlForErrorCode(ErrorCodes.USER_DOESNOT_EXIST);
                }
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("Username = {0}", LoginName);
                ALErrorObj = new ALError(ErrorCodes.AUTHENTICATION_FAILED,
                                             "Exception while authenticating user.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
            return null;
        }

        /// <summary>
        /// <CreateDate>18 Sept 2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// <Purpose>This method is used to change password if password has expired depending on password policy settings</Purpose>
        /// </summary>
        /// <param name="UserID">Unique UserID</param>
        /// <param name="OldUserPassword">Old Password to be changed</param>
        /// <param name="NewUserPassword">NewPassword which would be stored against that User</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        public ERRORXML ChangeUserPassword(ulong UserID, String OldUserPassword, String NewUserPassword)
        {
            StringBuilder SearchXML = null;            
            StringBuilder InputData = null;
            StringBuilder SBInputXML = null;

            String InputXML = null;
            String OutputXML = null;
            String FieldListXML = null;
            String ErrorXML = null;
            String ErrorXMLString = null;
            String EncNewUserPassword = null;
            ALError ALErrorObj = null;

            XmlDocument SearchUserXMlDoc = null;
            XmlNode PasswordNode = null;
            XmlNode PasswordChangedDateNode = null;
            DateTime CurrentTime;
            String str_PasswordInDB = null;

            try
            {
                ObjectManagerClass OM = new ObjectManagerClass(this.UserID, this.CurrentLanguage, this.TenantName, false);

                // Prepare Inputxml for search 
                SearchXML = new StringBuilder();
                SearchXML.Remove(0, SearchXML.Length);

                SearchXML.Append("<SEARCH><TARGETOBJECT>USER</TARGETOBJECT><TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE><SORTCOLUMN>USERID</SORTCOLUMN>");
                SearchXML.Append("<SORTORDER>ASC</SORTORDER><ISCASESENSITIVE>False</ISCASESENSITIVE><MAXRESULTSIZE>1</MAXRESULTSIZE><SEARCHALLFACETS>False</SEARCHALLFACETS>");                
                SearchXML.Append("<FILTERLIST>");
                SearchXML.AppendFormat("<FILTER><FIELDNAME>USERID</FIELDNAME><OPERATOR>EQUALTO</OPERATOR><FIELDVALUE>{0}</FIELDVALUE><ISCASESENSITIVE>False</ISCASESENSITIVE><NAVIGATIONPATH>USER</NAVIGATIONPATH></FILTER>", UserID);
                SearchXML.Append("</FILTERLIST>");
                SearchXML.Append("</SEARCH>");

                FieldListXML = "<FIELDCOLLECTION><TARGETENTITYFIELDS><FIELD>PASSWORD</FIELD><FIELD>PASSWORDCHANGEDDATE</FIELD></TARGETENTITYFIELDS></FIELDCOLLECTION>";                
                ErrorXML = OM.GetData(SearchXML.ToString(), FieldListXML, ref OutputXML);
                if (ErrorXML != null)
                {
                    ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.USER_READ_FAILED, ErrorObjectType.AUTHENTICATION);
                    return ErrorXML;
                }

                if (OutputXML != null)
                {
                    SearchUserXMlDoc = new XmlDocument();
                    SearchUserXMlDoc.LoadXml(OutputXML);

                    if (SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION").HasChildNodes)
                    {
                        EncNewUserPassword  = Assetlink.ALCommon.Encrypt(NewUserPassword);                        
                        PasswordNode        = SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION/USER/PASSWORD");
                        str_PasswordInDB    = ALCommon.Decrypt(PasswordNode.InnerText);
                        if (str_PasswordInDB.CompareTo(OldUserPassword) == 0)
                        {
                            //OldUserPassword passed to this method is equal to Current UserPassword
                            SBInputXML = new StringBuilder();

                            //Change LastModifiedTime
                            PasswordChangedDateNode = SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION/USER/PASSWORDCHANGEDDATE");
                            CurrentTime = DateTime.Now;

                            PasswordChangedDateNode.SelectSingleNode("DAY").InnerText = CurrentTime.Day.ToString();
                            PasswordChangedDateNode.SelectSingleNode("MONTH").InnerText = CurrentTime.Month.ToString();
                            PasswordChangedDateNode.SelectSingleNode("YEAR").InnerText = CurrentTime.Year.ToString();
                            PasswordChangedDateNode.SelectSingleNode("HOUR").InnerText = CurrentTime.Hour.ToString();
                            PasswordChangedDateNode.SelectSingleNode("MINUTE").InnerText = CurrentTime.Minute.ToString();
                            PasswordChangedDateNode.SelectSingleNode("SECOND").InnerText = CurrentTime.Second.ToString();

                            SBInputXML.Append("<INPUTXML><TRANSACTION>True</TRANSACTION><OPERATION>");
                            SBInputXML.Append("<OBJECTNAME>ENTITY</OBJECTNAME><METHOD>UPDATE</METHOD><PARAM Name=\"INPUTXML\">");
                            PasswordNode.InnerText = EncNewUserPassword;
                            SBInputXML.Append(SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION/USER").OuterXml);
                            SBInputXML.Append("</PARAM></OPERATION></INPUTXML>");

                            InputXML = SBInputXML.ToString();

                            ErrorXML = OM.Execute(ref InputXML);

                            if (ErrorXML != null)
                            {
                                ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.UPDATE_FAILED, ErrorObjectType.AUTHENTICATION);
                                return ErrorXML;
                            }
                            return null;
                           
                        }
                        else
                        {
                            //User Old Password is invalid
                            ALErrorObj = new ALError(ErrorCodes.INVALID_USER_CREDENTIALS,
                                "User credentials are invalid.", ErrorObjectType.AUTHENTICATION,
                                "Entity", ErrorType.ERROR, null, "Change User Password Failed");
                            ErrorXMLString = ALErrorObj.Serialize();
                            return ErrorXMLString;
                        }
                    }
                    else
                    {
                        //Error User does not exists
                        ALErrorObj = new ALError(ErrorCodes.USER_DOESNOT_EXIST,
                                 "User does not exists.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "Change User Password Failed");
                        ErrorXMLString = ALErrorObj.Serialize();
                        return ErrorXMLString;
                    }
                }
                else
                {
                    ALErrorObj = new ALError(ErrorCodes.USER_DOESNOT_EXIST,
                                 "UserID is Incorrect.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "Change User Password Failed");
                    ErrorXMLString = ALErrorObj.Serialize();
                    return ErrorXMLString;
                }
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("UserID = {0}", UserID);
                ALErrorObj = new ALError(ErrorCodes.AUTHENTICATION_FAILED,
                                             "Exception while changing user password.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }



            
        }

        /// <summary>
        /// <CreateDate>18 Sept 2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// <Purpose>This method is used to validate a Session against UserID</Purpose>
        /// </summary>
        /// <param name="IsSessionValid">Boolean value which returns TRUE if session is valid else FALSE</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        public ERRORXML IsSessionValid(ref Boolean IsSessionValid)
        {
            DataManager OMDataManager = null;

            StringBuilder StrQuery = null;
            StringBuilder InputData = null;

            String ErrorXML = null;

            ALError ALErrorObj = null;
            OMError ErrorObj = null;

            DateTime LastUpdatedTime;
            DateTime CurrentTime;
            TimeSpan TimeSpan;
            DateTimeFormatInfo DTFI = null;
            DynamicSQLParameter m_parameter_1, m_parameter_2, m_parameter_3;
            try
            {
                OMDataManager = new DataManager(this.TenantName, false);
                StrQuery = new StringBuilder();

                //Read Record from Database for given UserID and SessionID
                m_parameter_1 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(), UserID.ToString(), "BIGINT");
                m_parameter_2 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),SessionID.ToString(),"BIGINT");

                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_1);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_2);

                StrQuery.AppendFormat("SELECT LASTUPDATEDTIME FROM ALOM_SESSION WHERE USERID= {0}  AND SESSIONID= {1} ", m_parameter_1.m_ParameterName, m_parameter_2.m_ParameterName);
                ErrorObj = OMDataManager.DataProvider.RetrieveResult(StrQuery.ToString());

                if (ErrorObj != null)
                {
                    IsSessionValid = false;
                    return ErrorObj.Serialize();
                }

                CurrentTime = DateTime.Now;
                LastUpdatedTime = CurrentTime;

                if (OMDataManager.DataProvider.DataReaderObj.Read())
                {
                    LastUpdatedTime = Convert.ToDateTime(OMDataManager.DataProvider.DataReaderObj.GetValue(0));
                }
                else
                {
                    //Record not found
                    IsSessionValid = false;
                    return null;
                }

                OMDataManager.DataProvider.DataReaderObj.Close();

                //Add SessionTimeOut in LastUpdatedTime
                LastUpdatedTime = LastUpdatedTime.AddSeconds(Convert.ToDouble(OMDataManager.SessionTimeOut));

                TimeSpan = CurrentTime.Subtract(LastUpdatedTime);

                if (TimeSpan.TotalSeconds < 0)
                {
                    //Session Valid -- Update LastUpdatedTime with CurrentTime
                    StrQuery.Remove(0, StrQuery.Length);
                    DTFI = new CultureInfo(System.Threading.Thread.CurrentThread.CurrentCulture.Name, false).DateTimeFormat;
                    DTFI.DateSeparator = "/";
                    DTFI.TimeSeparator = ":";

                    m_parameter_1 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),OMDataManager.DataProvider.ConvertDate(CurrentTime.ToString(DTFI)),"DATETIME");
                    m_parameter_2 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),UserID.ToString(),"BIGINT");
                    m_parameter_3 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),SessionID.ToString(), "BIGINT");
                    
                    OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_1);
                    OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_2);
                    OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_3);


                    StrQuery.AppendFormat("UPDATE ALOM_SESSION SET LASTUPDATEDTIME=  {0}  WHERE USERID= {1} AND SESSIONID= {2} ", m_parameter_1.m_ParameterName, m_parameter_2.m_ParameterName, m_parameter_3.m_ParameterName);
                    ErrorObj = OMDataManager.DataProvider.Execute(StrQuery.ToString());

                    if (ErrorObj != null)
                    {
                        IsSessionValid = false;
                        return ErrorObj.Serialize();
                    }

                }
                else
                {
                    //Session InValid -- Delete Entry from Database
                    StrQuery.Remove(0, StrQuery.Length);

                    m_parameter_1 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),UserID.ToString(),"DATETIME");
                    m_parameter_2 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(), SessionID.ToString(), "BIGINT");

                    OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_1);
                    OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_2);

                    StrQuery.AppendFormat("DELETE FROM ALOM_SESSION WHERE USERID= {0} AND SESSIONID= {1} ", m_parameter_1.m_ParameterName, m_parameter_2.m_ParameterName);
                    ErrorObj = OMDataManager.DataProvider.Execute(StrQuery.ToString());

                    IsSessionValid = false;

                    if (ErrorObj != null)
                    {
                        return ErrorObj.Serialize();
                    }
                    return null;
                }

                IsSessionValid = true;
                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("SessionID = {0}", SessionID);
                ALErrorObj = new ALError(ErrorCodes.SESSION_VALIDATION_FAILED,
                                             "Exception while validating session.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
            finally
            {
                if (OMDataManager != null && OMDataManager.DataProvider.ConnectionObj.State == System.Data.ConnectionState.Open)
                    OMDataManager.DataProvider.Close(); //Added to Close the Connection;
            }
        }

        /// <summary>
        /// <CreateDate>18 Sept 2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// <Purpose>This method Registers a User</Purpose>
        /// </summary>
        /// <param name="InputXML">Input XML which contains User Registration Details</param>        
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORXML: If an Error occurred.
        /// </returns>
        public ERRORXML RegisterUser(String InputXML)
        {
            StringBuilder InputData = null;
            String ErrorXML = null;
            ALError ALErrorObj = null;
            XmlDocument InputXMLDocument = null;
            XmlNode LoginNameNode = null;
            StringBuilder SearchXML = null;
            String OutputXML = null;
            XmlDocument SearchUserXMlDoc = null;

            try
            {
                //Check whether user already exists
                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                LoginNameNode = InputXMLDocument.SelectSingleNode("INPUTXML/OPERATION/PARAM/USER/LOGINNAME");

                SearchXML = new StringBuilder();
                SearchXML.Remove(0, SearchXML.Length);

                SearchXML.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>USER</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                        <SEARCHALLFACETS>False</SEARCHALLFACETS>
                                        <FILTERLIST>
                                            <FILTER>
                                                <FIELDNAME>LOGINNAME</FIELDNAME>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <FIELDVALUE>{0}</FIELDVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <NAVIGATIONPATH>USER</NAVIGATIONPATH>
                                            </FILTER>
                                            <FILTER>
                                                <FIELDNAME>STATUS</FIELDNAME>
                                                <OPERATOR>NOTEQUALTO</OPERATOR>
                                                <FIELDVALUE>REJECTED</FIELDVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <NAVIGATIONPATH>USER</NAVIGATIONPATH>
                                            </FILTER>

                                        </FILTERLIST>
                                    </SEARCH>", LoginNameNode.InnerText);

                ObjectManagerClass OM = new ObjectManagerClass(this.UserID, this.CurrentLanguage, this.TenantName, false);

                ErrorXML = OM.GetData(SearchXML.ToString(), null, ref OutputXML);
                if (ErrorXML != null)
                {
                    //User Read failed
                    ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.USER_READ_FAILED, ErrorObjectType.AUTHENTICATION);
                    return ErrorXML;
                }

                if (OutputXML != null)
                {
                    SearchUserXMlDoc = new XmlDocument();
                    SearchUserXMlDoc.LoadXml(OutputXML);

                    if (SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION").HasChildNodes)
                    {
                        //User already exists
                        ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.USER_ALREADY_EXISTS, ErrorObjectType.AUTHENTICATION);
                        return ErrorXML;
                    }
                }

                ErrorXML = OM.Execute(ref InputXML);

                if (ErrorXML != null)
                {
                    ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.USER_REGISTRATION_FAILED, ErrorObjectType.AUTHENTICATION);
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("InputXML = {0}", InputXML);
                ALErrorObj = new ALError(ErrorCodes.USER_REGISTRATION_FAILED,
                                             "Exception while registering user.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }

        }

        /// <summary>
        /// <CreateDate>18 Sept 2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// <Purpose>This method Retrives Users Password</Purpose>
        /// </summary>
        /// <param name="EmailID">Users EmailID</param>    
        /// <param name="UserPassword">User Password</param> 
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORXML: If an Error occurred.
        /// </returns>
        public ERRORXML GetUserPassword(String EmailID, ref String UserPassword)
        {
            StringBuilder SearchXML = null;
            StringBuilder InputData = null;

            String OutputXML = null;
            String ErrorXML = null;
            String ErrorXMLString = null;
            ALError ALErrorObj = null;

            XmlDocument SearchUserXMlDoc = null;
            XmlNode PasswordNode = null;

            try
            {
                UserPassword = null;

                ObjectManagerClass OM = new ObjectManagerClass(this.UserID, this.CurrentLanguage, this.TenantName, false);

                // Prepare Inputxml for search 
                SearchXML = new StringBuilder();
                SearchXML.Remove(0, SearchXML.Length);

                SearchXML.Append("<SEARCH><TARGETOBJECT>USER</TARGETOBJECT><TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE><SORTCOLUMN>USERID</SORTCOLUMN>");
                SearchXML.Append("<SORTORDER>ASC</SORTORDER><ISCASESENSITIVE>False</ISCASESENSITIVE><MAXRESULTSIZE>1</MAXRESULTSIZE><SEARCHALLFACETS>False</SEARCHALLFACETS>");
                SearchXML.Append("<FILTERLIST>");
                SearchXML.AppendFormat("<FILTER><FIELDNAME>EMAILID</FIELDNAME><OPERATOR>EQUALTO</OPERATOR><FIELDVALUE>{0}</FIELDVALUE><ISCASESENSITIVE>False</ISCASESENSITIVE><NAVIGATIONPATH>USER</NAVIGATIONPATH></FILTER>", EmailID);
                SearchXML.Append("</FILTERLIST>");
                SearchXML.Append("</SEARCH>");

                ErrorXML = OM.GetData(SearchXML.ToString(), null, ref OutputXML);
                if (ErrorXML != null)
                {
                    ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.PASSWORD_READ_FAILED, ErrorObjectType.AUTHENTICATION);
                    return ErrorXML;
                }

                if (OutputXML != null)
                {
                    SearchUserXMlDoc = new XmlDocument();
                    SearchUserXMlDoc.LoadXml(OutputXML);

                    if (SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION").HasChildNodes)
                    {
                        
                        PasswordNode = SearchUserXMlDoc.SelectSingleNode("USERCOLLECTION/USER/PASSWORD");

                        if (PasswordNode != null)
                        {
                            if (PasswordNode.InnerText == null)
                                UserPassword = "";
                            else
                                UserPassword = PasswordNode.InnerText;
                        }
                        else
                        {
                            //User Old Password is invalid
                            ALErrorObj = new ALError(ErrorCodes.PASSWORD_READ_FAILED,
                                "User credentials are invalid.", ErrorObjectType.AUTHENTICATION,
                                "Entity", ErrorType.ERROR, null, "Get User Password Failed");
                            ErrorXMLString = ALErrorObj.Serialize();
                            return ErrorXMLString;
                        }
                    }
                    else
                    {
                        //Error User does not exists
                        ALErrorObj = new ALError(ErrorCodes.USER_DOESNOT_EXIST,
                                 "User does not exists.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "Get User Password Failed");
                        ErrorXMLString = ALErrorObj.Serialize();
                        return ErrorXMLString;
                    }
                }
                else
                {
                    ALErrorObj = new ALError(ErrorCodes.INVALID_USER_CREDENTIALS,
                                 "EmailID is Incorrect.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "Get User Password Failed");
                    ErrorXMLString = ALErrorObj.Serialize();
                    return ErrorXMLString;
                }

                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("UserID = {0}", UserID);
                ALErrorObj = new ALError(ErrorCodes.AUTHENTICATION_FAILED,
                                             "Exception while reading user password.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
        }

        /// <summary>
        /// <CreateDate>19 Sept 2006</CreateDate>
        /// <Author>Sagar Dasalkar</Author>
        /// <Purpose>This method returns password policies for Tenant</Purpose>
        /// </summary>
        /// <param name="OutputXML">OutputXML which is generated.</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORXML: If an Error occurred.
        /// </returns>
        public ERRORXML GetPasswordPolicies(ref String OutputXML)
        {
            XmlDocument TenantXMLDocument = null;
            XmlNode PasswordPolicyNode = null;
            StringBuilder AppXMLPath = null;
            StringBuilder InputData = null;
            ALError ALErrorObj = null;
            String ErrorXML = null;

            try
            {
                AppXMLPath = new StringBuilder();
                AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(TenantName));
                TenantXMLDocument = new XmlDocument();
                TenantXMLDocument.Load(AppXMLPath.ToString());
                PasswordPolicyNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/PASSWORDPOLICIES");

                if (PasswordPolicyNode == null)
                {
                    ErrorXML = PrepareErrorXML(ErrorXML, ErrorCodes.PASSWORD_POLICIES_READ_FAILED, ErrorObjectType.AUTHENTICATION);
                    return ErrorXML;
                }

                OutputXML = PasswordPolicyNode.OuterXml;
                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("TenantName = {0}", TenantName);
                ALErrorObj = new ALError(ErrorCodes.PASSWORD_POLICIES_READ_FAILED,
                                             "Exception while reading password policies.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }

        }

        /// <summary>
        /// <CreateDate>14 Nov 2006</CreateDate>
        /// <Author>Sagar Dasalkar</Author>
        /// <Purpose>This method impersonates User against given credentials</Purpose>
        /// </summary>
        /// <param name="LoginName">LoginName to be validated</param>
        /// <param name="TenantName">TenantName of that User</param>
        /// <param name="ImpersonationToken">ImpersonationToken returned by SystemConnect</param>
        /// <param name="IPAddress">IP address of the host making request.</param>
        /// <param name="Mode">Impersonation mode (LOGIN/INTERNAL/EMAIL)</param>
        /// <param name="CallerIdentity">Identity of caller.</param>
        /// <param name="SessionID">out parameter which returns newly generated SessionID when user gets impersonated</param>
        /// <param name="UserXML">out parameter which is returned if user is authenticated with UserinfoXML</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        public ERRORXML ImpersonateUser(String LoginName, String TenantName, String ImpersonationToken, String IPAddress, String Mode,
                        String CallerIdentity, ref ulong SessionID, ref String UserXML)
        {
            ALError             ALErrorObj              = null;
            String              ErrorXML                = null;
            StringBuilder       InputData               = null;
            StringBuilder       SearchXML               = null;
            XmlDocument         xmlDoc_UserDetails      = null;
            XmlNode             xmlNode_User            = null;
            ulong               ul_UserID               = 0;
            String              OutputXML               = null;
            String              str_FilterOn            = null;
            String              str_FilterValue         = null;

            try
            {
                UserXML = null;
                SessionID = 0;
                if (ImpersonationToken.CompareTo("111111133311111") != 0)
                    return GetErrorXmlForErrorCode(ErrorCodes.INVALID_IMPERSONATION_TOKEN);

                if (!String.IsNullOrEmpty(Mode) && Mode.ToUpper().CompareTo("INTERNAL") == 0)
                {
                    if (String.IsNullOrEmpty(CallerIdentity))
                        return GetErrorXmlForErrorCode(ErrorCodes.IMPERSONATE_USER_FAILED);
                }

                ObjectManagerClass OM = new ObjectManagerClass(TenantName, false);
                SearchXML = new StringBuilder(200);
                /* Cases: 
                 * 1. If login name is specified, user will be searched based on login name
                 * 2. If login name is not specified, user will be searched based on User Id 
                 * 3. If both login name and user id are not specified, an error will be returned.
                 * */
                if (!String.IsNullOrEmpty(LoginName))
                {
                    str_FilterOn = "LOGINNAME";
                    str_FilterValue = LoginName;
                }
                else if (this.UserID > 0)
                {
                    str_FilterOn = "USERID";
                    str_FilterValue = this.UserID.ToString();
                }
                else
                {
                    ALErrorObj = new ALError(ErrorCodes.USER_DOESNOT_EXIST,
                                 "User credentials are invalid.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "ImpersonateUser Failed");
                    return ALErrorObj.Serialize();
                }

                SearchXML.Append(@"<SEARCH>
                                        <TARGETOBJECT>USER</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <FILTERLIST>
                                            <FILTER>
                                                <FIELDNAME>" + str_FilterOn + @"</FIELDNAME>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <FIELDVALUE>" + str_FilterValue + @"</FIELDVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <NAVIGATIONPATH>USER</NAVIGATIONPATH>
                                            </FILTER>
                                            <FILTER>
                                                <FIELDNAME>STATUS</FIELDNAME>
                                                <OPERATOR>IN</OPERATOR>
                                                <FIELDVALUE>('ACTIVE','LOCKED')</FIELDVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <NAVIGATIONPATH>USER</NAVIGATIONPATH>
                                            </FILTER>
                                        </FILTERLIST>
                                    </SEARCH>");

                ErrorXML = OM.GetData(SearchXML.ToString(), null, ref OutputXML);

                if (!String.IsNullOrEmpty(ErrorXML))
                    return ErrorXML;

                if (!String.IsNullOrEmpty(OutputXML))
                {
                    //XmlDocument is used since user details might be updated in case when impersonation by admin is not allowed
                    //and login attempts were made for impersonation by admin. In this case, user accout might get locked if number attempts
                    //exceed login failure count.
                    xmlDoc_UserDetails = new XmlDocument();
                    xmlDoc_UserDetails.LoadXml(OutputXML);
                    xmlNode_User = xmlDoc_UserDetails.SelectSingleNode("USERCOLLECTION/USER");

                    if (xmlNode_User == null)
                    {
                        ALErrorObj = new ALError(ErrorCodes.USER_DOESNOT_EXIST,
                                 "User credentials are invalid.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "ImpersonateUser Failed");
                        return ALErrorObj.Serialize();
                    }

                    ul_UserID = Convert.ToUInt64(xmlNode_User.SelectSingleNode("USERID").InnerText);
                    //Create Session for User
                    ErrorXML = CreateUserSession(ul_UserID, IPAddress, Mode, CallerIdentity, true, false,xmlNode_User, OM, ref SessionID);

                    if (!String.IsNullOrEmpty(ErrorXML))
                        return ErrorXML;

                    UserXML = xmlNode_User.OuterXml;
                }
                else
                {
                    ALErrorObj = new ALError(ErrorCodes.USER_DOESNOT_EXIST,
                                 "User credentials are invalid.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "ImpersonateUser Failed");
                    return ALErrorObj.Serialize();
                }

                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("Username = {0}", LoginName);
                ALErrorObj = new ALError(ErrorCodes.IMPERSONATE_USER_FAILED,
                                             "Exception while Impersonating user.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
            finally
            {
                InputData               = null;
                SearchXML               = null;
                xmlNode_User            = null;
                xmlDoc_UserDetails      = null;
            }
        }

        /// <summary>
        /// <CreateDate>16 Nov 2006</CreateDate>
        /// <Author>Sagar Dasalkar</Author>
        /// <Purpose>This method connects to assetlink using SystemUser</Purpose>
        /// </summary>
        /// <param name="SystemUserLoginName">SystemUserLoginName to be validated</param>
        /// <param name="SystemUserPassword">SystemUserPassword of that SystemUser</param>
        /// <param name="ImpersonationToken">out paremeter ImpersonationToken, which is returned if SystemUser credentials are valid</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        public ERRORXML SystemConnect(String SystemUserLoginName, String SystemUserPassword, ref String ImpersonationToken)
        {
            ALError         ALErrorObj              = null;
            String          DeccSystemUserLoginName = null;
            String          DecSystemUserPassword   = null;
            String          ErrorXMLString          = null;
            String          ErrorXML                = null;
            StringBuilder   InputData               = null;
            
            try
            {
                //Check SystemUserLoginName & SystemUserPassword
                DeccSystemUserLoginName = Assetlink.ALCommon.Decrypt(SystemUserLoginName);
                if (DeccSystemUserLoginName.CompareTo("MePhIsToPhElEs") != 0)
                {
                    ImpersonationToken = null;
                    ALErrorObj = new ALError(ErrorCodes.INVALID_SYSTEM_USER_CREDENTIALS,
                                 "SystemUser credentials are invalid", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "SystemConnect Failed");
                    ErrorXMLString = ALErrorObj.Serialize();
                    return ErrorXMLString;
                }

                DecSystemUserPassword = Assetlink.ALCommon.Decrypt(SystemUserPassword);
                if (DecSystemUserPassword.CompareTo("ErAtOsThEnEs") != 0)
                {
                    ImpersonationToken = null;
                    ALErrorObj = new ALError(ErrorCodes.INVALID_SYSTEM_USER_CREDENTIALS,
                                 "SystemUser credentials are invalid", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "SystemConnect Failed");
                    ErrorXMLString = ALErrorObj.Serialize();
                    return ErrorXMLString;
                }

                ImpersonationToken = "111111133311111";
                
                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("SystemUserLoginName = {0}, ", SystemUserLoginName);
                InputData.AppendFormat("SystemUserPassword = {0}", SystemUserPassword);
                ALErrorObj = new ALError(ErrorCodes.SYSTEM_CONNECT_FAILED,
                                             "Exception in SystemConnect", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                ErrorXML = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, ErrorXML);
                return ErrorXML;
            }
        }

         
        #endregion

        # region Private Methods
        /// <summary>
        /// <CreateDate>18 Sept 2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// <Purpose>This method is used to create a Session against UserID</Purpose>
        /// </summary>
        /// <param name="UserID">Unique UserID</param>
        /// <param name="IPAddress">IP address of the host making request.</param>
        /// <param name="Mode">Impersonation or Authenticaton mode.</param>
        /// <param name="CallerIdentity">Identity of caller.</param>
        /// <param name="Impersonation">Specifies whether user is to be impersonated.</param>
        /// <param name="ImpersonationByAdmin">Specifies whether user is to be imperosnated by admin.</param>
        /// <param name="xmlNode_UserDetails">Navigator to user details XML.</param>
        /// <param name="obj_OMClass">Required to update user.</param>
        /// <param name="SessionID">ref unique SessionID for that User</param>
        /// <returns>
        /// NULL: If the API was successful.
        /// ERRORRXML: If an Error occurred.
        /// </returns>
        private ERRORXML CreateUserSession(ulong UserID, String IPAddress, String Mode, String CallerIdentity, 
                                bool Impersonation, bool ImpersonationByAdmin, XmlNode xmlNode_UserDetails, ObjectManagerClass obj_OMClass, ref ulong SessionID)
        {
            DataManager             OMDataManager               = null;
            OMError                 ErrorObj                    = null;

            StringBuilder           sb_Query                    = null;
            StringBuilder           InputData                   = null;

            ALError                 ALErrorObj                  = null;

            String                  str_ErrorXml                = null;
            DateTimeFormatInfo      DTFI                        = null;
            Boolean                 ConcurrentSessionAllowed    = false;
            DynamicSQLParameter    m_parameter_1 = null, m_parameter_2 = null, m_parameter_3 = null, m_parameter_4 = null, m_parameter_5 = null, m_parameter_6 = null, m_parameter_7 = null, m_parameter_8 = null; 

            try
            {
                OMDataManager = new DataManager(this.TenantName, false);

                str_ErrorXml = IsConcurrentSessionAllowed(IPAddress, TenantName, Mode, UserID, Impersonation, ImpersonationByAdmin, xmlNode_UserDetails, obj_OMClass, ref ConcurrentSessionAllowed, ref SessionID);

                //If concurrent login is not allowed then error would always be returned. Return error.
                if (!String.IsNullOrEmpty(str_ErrorXml) && !ConcurrentSessionAllowed)
                    return str_ErrorXml;

                //Get New SessionID
                Assetlink.UniqueID.Read(ref SessionID);

                sb_Query = new StringBuilder();

                DTFI = new CultureInfo(System.Threading.Thread.CurrentThread.CurrentCulture.Name, false).DateTimeFormat;
                DTFI.DateSeparator = "/";
                DTFI.TimeSeparator = ":";

                m_parameter_1 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),UserID.ToString(),"BIGINT");
                m_parameter_2 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),SessionID.ToString(),"BIGINT");
                m_parameter_3 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),OMDataManager.DataProvider.ConvertDate(DateTime.Now.ToString(DTFI)),"DATETIME");
                m_parameter_4 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),OMDataManager.DataProvider.GetNextParameterizedVariable(),"NVARCHAR(100)");
                m_parameter_5 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),(!String.IsNullOrEmpty(Mode) ? "'" + Mode + "'" : "NULL"),"NVARCHAR(20)");
                m_parameter_6 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),(Impersonation ? "'1'" : "'0'"),"NVARCHAR(1)");
                m_parameter_7 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(),(ImpersonationByAdmin ? "'1'" : "'0'"),"NVARCHAR(1)");

                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_1);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_2);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_3);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_4);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_5);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_6);
                OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_7);

                sb_Query.AppendFormat("INSERT INTO ALOM_SESSION({0},{1},{2},{3},{4},{5},{6}",
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("USERID"),
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("SESSIONID"),
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("LASTUPDATEDTIME"),
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("IPADDRESS"),
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("MODE"),
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("IMPERSONATION"),
                                OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("IMPERSONATIONBYADMIN"));

                if (!String.IsNullOrEmpty(CallerIdentity))
                {
                    m_parameter_8 = new DynamicSQLParameter(OMDataManager.DataProvider.GetNextParameterizedVariable(), "'" + CallerIdentity + "'", "NVARCHAR(100)");

                    OMDataManager.DataProvider.DynamicParameters.Add(m_parameter_8);

                    sb_Query.AppendFormat(",{0}", OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved("CALLERIDENTITY"));
                }
                
                sb_Query.Append(")");

                sb_Query.AppendFormat(" VALUES( {0} , {1} , {2} , {3} , {4} , {5} , {6} ",
                                m_parameter_1.m_ParameterName,
                                m_parameter_2.m_ParameterName,
                                m_parameter_3.m_ParameterName,
                                m_parameter_4.m_ParameterName,
                                m_parameter_5.m_ParameterName,
                                m_parameter_6.m_ParameterName,
                                m_parameter_7.m_ParameterName);

                if (!String.IsNullOrEmpty(CallerIdentity))
                {
                    sb_Query.AppendFormat(", {0} ", m_parameter_8.m_ParameterName);
                }

                sb_Query.Append(")");

                ErrorObj = OMDataManager.DataProvider.Execute(sb_Query.ToString());

                if (ErrorObj != null)
                {
                    SessionID = 0;
                    return ErrorObj.Serialize();
                }
               
                return null;
            }
            catch (Exception ExceptionObject)
            {
                InputData = new StringBuilder();
                InputData.AppendFormat("SessionID = {0}", SessionID);
                ALErrorObj = new ALError(ErrorCodes.SESSION_CREATION_FAILED,
                                             "Exception while creating session.", ErrorObjectType.AUTHENTICATION,
                                             "USER", ErrorType.EXCEPTION, ExceptionObject, InputData.ToString());
                str_ErrorXml = ALErrorObj.Serialize();
                ExceptionTracing.LogError(TenantName, str_ErrorXml);
                return str_ErrorXml;
            }
            finally
            {
                if (OMDataManager != null && OMDataManager.DataProvider.ConnectionObj.State == System.Data.ConnectionState.Open)
                    OMDataManager.DataProvider.Close();
            }

        }

        /// <summary>
        /// <CreateDate>18-Sept-2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// </summary>
        /// <param name="ErrorXML"> Error XML containing error description</param>
        /// <param name="ErrorCode">The error code that must be set</param>
        /// <param name="ObjectType">The type of object that returned an error</param>
        /// <returns></returns>
        private ERRORXML PrepareErrorXML(ERRORXML ErrorXML, ErrorCodes ErrorCode, ErrorObjectType ObjectType)
        {
            XmlDocument ErrorDocument = null;
            XmlNode ErrorNode = null;
            ALError ErrorObject = null;
            ERRORXML ReturnErrorXML = null;
            // load the XML and change 
            try
            {
                ErrorDocument = new XmlDocument();
                ErrorDocument.LoadXml(ErrorXML);
                ErrorNode = ErrorDocument.SelectSingleNode("ERRORCOLLECTION/ERROR/METHODNAME");
                ErrorNode.InnerText = GetExecutingMethodName();
                ErrorNode = null;
                ErrorNode = ErrorDocument.SelectSingleNode("ERRORCOLLECTION/ERROR/ERRORCODE");
                ErrorNode.InnerText = ErrorCode.ToString();
                ErrorNode = null;
                ErrorNode = ErrorDocument.SelectSingleNode("ERRORCOLLECTION/ERROR/OBJECTTYPE");
                ErrorNode.InnerText = ObjectType.ToString();
                
                ReturnErrorXML = ErrorDocument.OuterXml;
                return ReturnErrorXML;
            }
            catch (Exception ExceptionObject)
            {
                ErrorObject = new ALError(ErrorCode, "Error", ObjectType, ObjectType.ToString(), ErrorType.EXCEPTION, ExceptionObject, ErrorXML);
                ReturnErrorXML = ErrorObject.Serialize();
                return ReturnErrorXML;
            }
        }

        /// <summary>
        /// <CreateDate>19-Spet-2006</CreateDate>
        /// <Author>Rahul Pawar</Author>
        /// </summary>
        /// <returns>Method Name that generated an error</returns>
        /// 
        private string GetExecutingMethodName()
        {
            string MethodName = "Unknown";

            StackTrace Trace = new StackTrace(false);

            for (int index = 0; index < Trace.FrameCount; ++index)
            {
                StackFrame frame = Trace.GetFrame(index);
                MethodBase method = frame.GetMethod();
                if (method.DeclaringType != GetType())
                {
                    MethodName = method.ToString();
                    break;
                }
            }

            return MethodName;
        }

        /// <summary>
        /// Identifies whether concurrent sessions are allowed. This function performs checks for following cases
        /// 1. To identify whether concurrent sessions are allowed
        /// 2. To identify whether impersonation by admin is allowed
        /// </summary>
        /// <param name="IPAddress">IP address of the host making request.</param>
        /// <param name="TenantName">Tenant name.</param>
        /// <param name="Mode">Impersonation or Authentication mode.</param>
        /// <param name="UserId">Id of user account.</param>
        /// <param name="Impersonation">Specifies whether user is to be impersonated.</param>
        /// <param name="ImpersonationByAdmin">Specifies whether user is to be imperosnated by admin.</param>
        /// <param name="xmlNode_UserDetails">Navigator to user details XML.</param>
        /// <param name="obj_OMClass">Required to update user.</param>
        /// <param name="ConcurrentSessionAllowed">Output parameter - Specifies whether concurrent sessions are allowed.</param>
        /// <param name="SessionId">Output parameter - If user already has an active session
        /// error will be returned along with active session ID. System will terminate the session. Otherwise, session id left as it is.</param>
        /// <returns>Error XML if any error occurs, othewise null.</returns>
        private String IsConcurrentSessionAllowed(String IPAddress, String TenantName, String Mode, ulong UserId, 
                                            bool Impersonation, bool ImpersonationByAdmin,
                                            XmlNode xmlNode_UserDetails, ObjectManagerClass obj_OMClass,
                                            ref bool ConcurrentSessionAllowed, ref ulong SessionId)
        {
            #region Variables

            StringBuilder               sb_Query                    = null;                     //Used to prepare query
            DataManager                 obj_DataManager             = null;                     //Used to execute queries on database
            OMError                     obj_OMError                 = null;                     //Used to prepare OMError
            ALError                     obj_ALError                 = null;                     //Used to prepare ALError
            DateTime                    dtNow;                                                  //Specifies current datetime
            DateTime                    dtSessionValidFrom;                                     //Specifies the start datetime of session
            const string                str_SqlDateTimeFormat       = "MM/dd/yyyy HH:mm:ss";    //Date time format for SQL 
            const string                str_TeradataDateTimeFormat  = "MM-dd-yyyy HH:mm:ss";    //Date time format for teradata
            String                      str_Now                     = null;                     //Used to store Current datetime
            String                      str_SessionValidFrom        = null;                     //Used to store start datetime of session
            StringBuilder               sb_ActiveSessionXml         = null;                     //Used to prepare active session XML
            
            XPathDocument               xpDoc_ActiveSessions        = null;                     //Used to load active session XML
            XPathNavigator              xpNav_ActiveSessionsRoot    = null;                     //Used to navigate through active session XML
            XPathNodeIterator           xpNavItr_SameMachine        = null;                     //Used to navigate through active sessions present on same machine
            XPathNodeIterator           xpNavItr_DifferentMachine   = null;                     //Used to navigate through active sessions present on different machine

            String                      str_XPathForSameIP          = null;                     //Used to prepare XPath for fetching active sessions on same machine 
            String                      str_XPathForDifferentIP     = null;                     //Used to prepare XPath for fetching active sessions on different machine             
            bool                        bAccountLockedNow;
            
            #endregion Variables

            #region Check whether concurrent login is allowed

            try
            {
                ConcurrentSessionAllowed = false;                

                #region Validate Mode value

                if (String.IsNullOrEmpty(Mode))
                {
                    obj_ALError = new ALError(ErrorCodes.CONCURRENT_LOGIN_NOT_ALLOWED,
                                             "Invalid parameter.", ErrorObjectType.AUTHENTICATION,
                                             "Entity", ErrorType.ERROR, null, "Authentication Failed");

                    return obj_ALError.Serialize();
                }

                switch (Mode.ToUpper())
                {
                    //Mode value should be one of the following
                    case "LOGIN":
                    case "INTERNAL":
                    case "EMAIL":
                    case "INTERSOLUTION":
                    case "WEBSERVICE":
                    case "DESKTOP":
                        break;

                    //Invalid value for Mode, return error
                    default:
                        obj_ALError = new ALError(ErrorCodes.CONCURRENT_LOGIN_NOT_ALLOWED,
                                             "Invalid parameter.", ErrorObjectType.AUTHENTICATION,
                                             "Entity", ErrorType.ERROR, null, "Authentication Failed");
                        return obj_ALError.Serialize();
                }

                #endregion Validate Mode value

                //Case - Impersonation by admin and Impersonation by admin is not allowed : Return error
                if (!TenantSetting.ConcurrentLoginSettings.ImpersonationByAdminAllowed && ImpersonationByAdmin)
                {
                    return LockUserAccount(UserId, xmlNode_UserDetails, obj_OMClass, out bAccountLockedNow);
                }

                //Case - Impersonation by admin and Impersonation by admin is allowed
                if (TenantSetting.ConcurrentLoginSettings.ImpersonationByAdminAllowed && ImpersonationByAdmin)
                {
                    ConcurrentSessionAllowed = true;
                    return null;
                }

                //Case - Concurrent logins allowed
                if (TenantSetting.ConcurrentLoginSettings.ConcurrentLoginsAllowed)
                {
                    ConcurrentSessionAllowed = true;
                    return null;
                }

                if (!String.IsNullOrEmpty(Mode))
                {
                    switch (Mode.ToUpper())
                    {
                        //Session will always be created for following cases. These cases are excluded from concurrent login scenario.
                        //1. Interactive login using email link (i.e. Mode=EMAIL)
                        //2. Interactive login using published site link (i.e. Mode=INTERSOLUTION)
                        //3. Authentication via thrrd party integration application (i.e. Mode=WEBSERVICE)
                        //4. Authentication from assetlink desktop (i.e. Mode=DESKTOP)
                        case "EMAIL":
                        case "INTERSOLUTION":
                        case "WEBSERVICE":
                        case "DESKTOP":
                            ConcurrentSessionAllowed = true;
                            break;

                        //Session will always be created when impersonated by backend processes like Cataloger (Mode=INTERNAL) and not allowed
                        //when impersonated from login page when session in cookie is invalid
                        case "INTERNAL":
                            if (Impersonation)
                                ConcurrentSessionAllowed = true;
                            else
                                ConcurrentSessionAllowed = false;
                            break;
                    }
                }

                if (ConcurrentSessionAllowed)
                    return null;

                dtNow = DateTime.Now;
                dtSessionValidFrom = dtNow.Subtract(new TimeSpan(0, 0, TenantSetting.SessionSettings.Timeout));

                //Summary: All active sessions (that are eligible for concurrent login scenario) will be fetched. Consider following cases
                //1. If there are active sessions present which were created from different machine, and the system does not allow concurrent login, 
                //then a session created from different machine will be selected and will be passed to login handler for termination. 
                //Login handler will terminate active session based user input.
                //2. If there are no active sessions created from different machine, system will check whether there are sessions created 
                //from same machine and if multiple session are not allowed on same machine, then a session will be selected and passed
                //to login handler for termination. Login handler will terminate active session based user input.
                
                //Step1: Get active sessions for user
                sb_Query = new StringBuilder(100);
                sb_Query.AppendFormat(@"SELECT * FROM ALOM_SESSION WHERE USERID={0} 
                                        AND IMPERSONATIONBYADMIN='0' 
                                        AND (IMPERSONATION='0' OR (IMPERSONATION='1' AND MODE='LOGIN')) 
                                        AND ((MODE<>'EMAIL' AND MODE<>'INTERSOLUTION' AND MODE<>'WEBSERVICE' AND MODE<>'DESKTOP') 
                                                OR MODE IS NULL) ", UserId);

                obj_DataManager = new DataManager(TenantName, false);

                switch (obj_DataManager.DataBaseType.ToUpper())
                {
                    case "SQLEXPRESS":
                    case "SQLSERVER":
                        //Format: CONVERT(DATETIME,'MM/dd/yyyy HH:mm:ss',101)
                        str_SessionValidFrom = "CONVERT(DATETIME,'" + dtSessionValidFrom.ToString(str_SqlDateTimeFormat) + "',101)";
                        str_Now = "CONVERT(DATETIME,'" + dtNow.ToString(str_SqlDateTimeFormat) + "',101)";

                        sb_Query.AppendFormat(" AND LASTUPDATEDTIME BETWEEN {0} AND {1}", str_SessionValidFrom, str_Now);
                        break;

                    case "TERADATA":
                        //Format: 'MM-dd-yyyy HH:mm:ss'
                        str_SessionValidFrom = "'" + dtSessionValidFrom.ToString(str_TeradataDateTimeFormat) + "'";
                        str_Now = "'" + dtNow.ToString(str_TeradataDateTimeFormat) + "'";

                        sb_Query.AppendFormat(" AND LASTUPDATEDTIME BETWEEN {0} AND {1}", str_SessionValidFrom, str_Now);
                        break;

                    case "ORACLE":
                    case "ORACLEEXPRESS":
                        sb_Query.Remove(0, sb_Query.Length);
                        sb_Query.AppendFormat(@"SELECT * FROM ALOM_SESSION WHERE USERID={0}", UserId);
                        sb_Query.Append("AND IMPERSONATIONBYADMIN='0' AND (IMPERSONATION='0' OR (IMPERSONATION='1' AND \"MODE\"='LOGIN'))");
                        sb_Query.Append("AND ((\"MODE\"<>'EMAIL' AND \"MODE\"<>'INTERSOLUTION' AND \"MODE\"<>'WEBSERVICE' AND \"MODE\"<>'DESKTOP') OR \"MODE\" IS NULL)"); 
                                                 
                        //Format: 'MM-dd-yyyy HH:mm:ss'
                        str_SessionValidFrom = obj_DataManager.DataProvider.ConvertDate(dtSessionValidFrom.ToString());
                        str_Now = obj_DataManager.DataProvider.ConvertDate(dtNow.ToString());

                        sb_Query.AppendFormat(" AND LASTUPDATEDTIME BETWEEN {0} AND {1}", "to_timestamp(" + str_SessionValidFrom + ",'DD-MON-YY HH24.MI.SSXFF')", "to_timestamp(" + str_Now + ",'DD-MON-YY HH24.MI.SSXFF')");
                        break;

                    default:
                        return GetErrorXmlForErrorCode(ErrorCodes.DATABASE_NOT_SUPPORTED);
                }

                obj_OMError = obj_DataManager.DataProvider.RetrieveResult(sb_Query.ToString());

                if (obj_OMError != null)
                    return obj_OMError.Serialize();

                //Step2: Prepare active sessions XML
                while (obj_DataManager.DataProvider.DataReaderObj.Read())
                {
                    if (sb_ActiveSessionXml == null)
                    {
                        sb_ActiveSessionXml = new StringBuilder(200);
                        sb_ActiveSessionXml.Append("<ACTIVESESSIONS>");
                    }

                    sb_ActiveSessionXml.Append("<ACTIVESESSION>");

                    sb_ActiveSessionXml.AppendFormat("<SESSIONID>{0}</SESSIONID>",
                        obj_DataManager.DataProvider.DataReaderObj.GetValue(obj_DataManager.DataProvider.DataReaderObj.GetOrdinal("SESSIONID")));

                    sb_ActiveSessionXml.AppendFormat("<IPADDRESS>{0}</IPADDRESS>",
                     obj_DataManager.DataProvider.DataReaderObj.GetValue(obj_DataManager.DataProvider.DataReaderObj.GetOrdinal("IPADDRESS")));

                    sb_ActiveSessionXml.AppendFormat("<MODE>{0}</MODE>",
                     obj_DataManager.DataProvider.DataReaderObj.GetValue(obj_DataManager.DataProvider.DataReaderObj.GetOrdinal("MODE")));
                    
                    sb_ActiveSessionXml.Append("</ACTIVESESSION>");
                }

                if(sb_ActiveSessionXml == null)
                {
                    ConcurrentSessionAllowed = true;
                    return null;
                }

                if (sb_ActiveSessionXml != null)
                {
                    sb_ActiveSessionXml.Append("</ACTIVESESSIONS>");
                }

                xpDoc_ActiveSessions = new XPathDocument(new StringReader(sb_ActiveSessionXml.ToString()));
                xpNav_ActiveSessionsRoot = xpDoc_ActiveSessions.CreateNavigator().SelectSingleNode("ACTIVESESSIONS");

                str_XPathForSameIP = "ACTIVESESSION[IPADDRESS[translate(text(),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')=" + ALXmlUtil.GetXpathValueString(IPAddress) + "]]";
                str_XPathForDifferentIP = "ACTIVESESSION[IPADDRESS[translate(text(),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')!=" + ALXmlUtil.GetXpathValueString(IPAddress) + "]]";

                xpNavItr_SameMachine = xpNav_ActiveSessionsRoot.Select(str_XPathForSameIP);
                xpNavItr_DifferentMachine = xpNav_ActiveSessionsRoot.Select(str_XPathForDifferentIP);

                //Step3: Terminate active session from different machine
                if (xpNavItr_DifferentMachine.Count > 0) 
                {
                    if (xpNavItr_DifferentMachine.MoveNext())
                    {
                        SessionId = Convert.ToUInt64(xpNavItr_DifferentMachine.Current.SelectSingleNode("SESSIONID").Value);

                        obj_ALError = new ALError(ErrorCodes.CONCURRENT_LOGIN_NOT_ALLOWED,
                                             "Concurrent login is not allowed.", ErrorObjectType.AUTHENTICATION,
                                             "Entity", ErrorType.ERROR, null, "Authentication Failed");
                        return obj_ALError.Serialize();
                    }
                }

                //Step3: Terminate active session from same machine
                if (!TenantSetting.ConcurrentLoginSettings.MultipleSessionsAllowdOnSameMachine && xpNavItr_SameMachine.Count > 0)
                {
                    if (xpNavItr_SameMachine.MoveNext())
                    {
                        SessionId = Convert.ToUInt64(xpNavItr_SameMachine.Current.SelectSingleNode("SESSIONID").Value);

                        obj_ALError = new ALError(ErrorCodes.CONCURRENT_LOGIN_NOT_ALLOWED,
                                             "Concurrent login is not allowed.", ErrorObjectType.AUTHENTICATION,
                                             "Entity", ErrorType.ERROR, null, "Authentication Failed");
                        return obj_ALError.Serialize();
                    }
                }
            }
            finally
            {
                sb_Query                    = null;
                obj_OMError                 = null;
                obj_ALError                 = null;
                str_Now                     = null;
                str_SessionValidFrom        = null;
                sb_ActiveSessionXml         = null;

                xpDoc_ActiveSessions        = null;
                xpNav_ActiveSessionsRoot    = null;
                xpNavItr_SameMachine        = null;
                xpNavItr_DifferentMachine   = null;

                str_XPathForSameIP          = null;
                str_XPathForDifferentIP     = null;

                if(obj_DataManager != null && obj_DataManager.DataProvider.ConnectionObj.State == System.Data.ConnectionState.Open)
                    obj_DataManager.DataProvider.Close();
            }

            ConcurrentSessionAllowed = true;
            return null;

            #endregion Check whether concurrent login is allowed
        }

        /// <summary>
        /// Fetches user details.
        /// </summary>
        /// <param name="strLoginName">Loginname of the user.</param>
        /// <param name="FieldCollectionXml">XML that specifies which fields are to be fetched.</param>
        /// <param name="objObjectManagerClass">Object Manager class instance to fetch user details.</param>
        /// <param name="strOutputXml">Output parameter - User details will be returned in this parameter.</param>
        /// <returns>Returns error XML if any error occurs, otherwise null.</returns>
        private String GetUser(String strLoginName, String FieldCollectionXml, ObjectManagerClass objObjectManagerClass, ref String strOutputXml)
        {   
            StringBuilder           sb_SearchXml            = null;         //Used to prepare search input XML
            
            sb_SearchXml = new StringBuilder(200);
            sb_SearchXml.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>USER</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <FILTERLIST>
                                            <FILTER>
                                                <FIELDNAME>LOGINNAME</FIELDNAME>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <FIELDVALUE>{0}</FIELDVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <NAVIGATIONPATH>USER</NAVIGATIONPATH>
                                            </FILTER>
                                        </FILTERLIST>
                                    </SEARCH>", strLoginName);

            return objObjectManagerClass.GetData(sb_SearchXml.ToString(), FieldCollectionXml, ref strOutputXml);
        }

        /// <summary>
        /// Identifies whether user password is expired.
        /// </summary>
        /// <param name="xmlNode_UserDetails">Navigator to User details.</param>
        /// <returns>true if user password is expired, otherwise false.</returns>
        private bool IsUserPasswordExpired(XmlNode xmlNode_UserDetails)
        {
            if (TenantSetting.PasswordPolicySettings.EnablePasswordExpiry)
            {
                if (TenantSetting.PasswordPolicySettings.ExpiryAfterDays <
                    (DateTime.Now - ALCommon.XmlToDateTime(xmlNode_UserDetails.SelectSingleNode("PASSWORDCHANGEDDATE"))).Days)
                    return true;
            }
            return false;
        }

        

        /// <summary>
        /// Updates user details.
        /// </summary>
        /// <param name="strUserXml">User details XML.</param>
        /// <param name="obj_OMClass">Object Manager Class instance required to update user.</param>
        /// <returns>Error xml if any error occurs, otherwise null.</returns>
        private String UpdateUser(String strUserXml, ObjectManagerClass obj_OMClass)
        {
            StringBuilder       sb_UserXml          = null;         //Used to prepare input XML to update user
            String              str_InputXml        = null;         //Used to prepare input XML to update user

            sb_UserXml = new StringBuilder(500);
            sb_UserXml.AppendFormat(@"<INPUTXML>
                                    <TRANSACTION>FALSE</TRANSACTION>
                                    <OPERATION>
                                        <OBJECTNAME>ENTITY</OBJECTNAME>
                                        <METHOD>UPDATE</METHOD>
                                        <PARAM Name=""InputXML"">{0}</PARAM>
                                    </OPERATION>
                                </INPUTXML>",strUserXml);

            str_InputXml = sb_UserXml.ToString();

            return obj_OMClass.Execute(ref str_InputXml);
        }

        /// <summary>
        /// Updates user details.
        /// </summary>
        /// <param name="UserId">Id of the user account.</param>
        /// <param name="userStatus">Status of user account.</param>
        /// <param name="LoginFailureCount">Login failure count for user account.</param>
        /// <param name="UpdateLastLoginDate">Specifies whether last login date is to be updated for user.</param>
        /// <param name="UpdateAccountLockedDate">Specifies whether account locked date is to be udpated for user.</param>
        /// <param name="obj_OMClass">Object Manager Class instance required to update user.</param>
        /// <param name="xmlNode_User">Navigator to iterate through user details.</param>
        /// <returns>Error Xml if any error occurs, otherwise null.</returns>
        private String UpdateUser(ulong UserId , USERSTATUS userStatus, int LoginFailureCount, bool UpdateLastLoginDate, 
                               bool UpdateAccountLockedDate, ObjectManagerClass obj_OMClass, ref XmlNode xmlNode_User)
        {
            StringBuilder               sb_UserXml              = null;         //Used to prepare user details XML to update user
            String                      str_CurrentDateTimeXml  = null;         //Used to store current datetime in XML format

            sb_UserXml = new StringBuilder(100);
            sb_UserXml.AppendFormat("<USER><USERID>{0}</USERID><LOGINFAILURECOUNT>{1}</LOGINFAILURECOUNT><STATUS>{2}</STATUS>",
                UserId, LoginFailureCount, userStatus.ToString());

            xmlNode_User.SelectSingleNode("LOGINFAILURECOUNT").InnerText = LoginFailureCount.ToString();
            xmlNode_User.SelectSingleNode("STATUS").InnerText = userStatus.ToString();
            
            str_CurrentDateTimeXml = ALCommon.CurrentDateTimeToXml();
            //At any point of time, either LASTLOGINDATE could be updated or ACCOUNTLOCKEDDATE
            if (UpdateLastLoginDate)
            {
                sb_UserXml.AppendFormat("<LASTLOGINDATE>{0}</LASTLOGINDATE>", str_CurrentDateTimeXml);
                xmlNode_User.SelectSingleNode("LASTLOGINDATE").InnerXml = str_CurrentDateTimeXml;
            }
            else if (UpdateAccountLockedDate)
            {
                sb_UserXml.AppendFormat("<ACCOUNTLOCKEDDATE>{0}</ACCOUNTLOCKEDDATE>", str_CurrentDateTimeXml);
                xmlNode_User.SelectSingleNode("ACCOUNTLOCKEDDATE").InnerXml = str_CurrentDateTimeXml;
            }

            sb_UserXml.Append("</USER>");

            return UpdateUser(sb_UserXml.ToString(), obj_OMClass);
        }

        /// <summary>
        /// Returns error XML for given error code.
        /// </summary>
        /// <param name="errorCode">Error code.</param>
        /// <returns>Error Xml prepared from error code.</returns>
        private String GetErrorXmlForErrorCode(ErrorCodes errorCode)
        {
            String          str_ErrorXml        = null;
            ALError         obj_ALError         = null;

            switch (errorCode)
            {
                case ErrorCodes.USER_PASSWORD_EXPIRED:
                    obj_ALError = new ALError(errorCode,
                                     "User Password Expired.", ErrorObjectType.AUTHENTICATION,
                                     "Entity", ErrorType.ERROR, null, "Authentication Successful");
                    break;

                case ErrorCodes.USER_DOESNOT_EXIST:
                    obj_ALError = new ALError(errorCode,
                                 "Username or password is incorrect.", ErrorObjectType.AUTHENTICATION,
                                 "Entity", ErrorType.ERROR, null, "Authentication Failed");
                    break;
                case ErrorCodes.INVALID_IMPERSONATION_TOKEN :
                    obj_ALError = new ALError(errorCode,
                                "Impersonation token is invalid.", ErrorObjectType.AUTHENTICATION,
                                "Entity", ErrorType.ERROR, null, "ImpersonateUser Failed");
                    break;

                case ErrorCodes.IMPERSONATE_USER_FAILED :
                    obj_ALError = new ALError(errorCode,
                                "Caller identity cannot be null or emtpy.", ErrorObjectType.AUTHENTICATION,
                                "Entity", ErrorType.ERROR, null, "ImpersonateUser Failed");
                    break;

                case ErrorCodes.DATABASE_NOT_SUPPORTED:
                    obj_ALError = new ALError(errorCode,
                                "Database is not supported.", ErrorObjectType.AUTHENTICATION,
                                "Entity", ErrorType.ERROR, null, "Failed to create session.");
                    break;
            }

            if(obj_ALError != null)
                str_ErrorXml = obj_ALError.Serialize();

            return str_ErrorXml;
        }

        /// <summary>
        /// Identifies whether user is admin.
        /// </summary>
        /// <param name="LoginName">Loginname of user.</param>
        /// <returns>true if the user is admin, otherwise false.</returns>
        private bool IsUserAdmin(String LoginName)
        {
            if (String.IsNullOrEmpty(LoginName))
                return false;

            if (LoginName.ToUpper().CompareTo("ADMIN") == 0)
                return true;

            return false;
        }

        /// <summary>
        /// Locks user account according to the login failure count setting.
        /// </summary>
        /// <param name="UserId">Id of user account.</param>
        /// <param name="xmlNode_UserDetails">Navigator to access user details XML.</param>
        /// <param name="obj_OMClass">Object Manager Class instance required to update user.</param>
        /// <returns>Error XML if any error occurs, otherwise null.</returns>
        private String LockUserAccount(ulong UserId, XmlNode xmlNode_UserDetails, ObjectManagerClass obj_OMClass, out bool bAccountLockedNow)
        {
            int                     i_LoginFailureCount             = 0;
            USERSTATUS              enum_UserStatus                 = USERSTATUS.ACTIVE;
            ALError                 obj_ALError                     = null;
            String                  str_ErrorXml                    = null;
            bool                    b_UpdateAccountLockedDate       = false;

            bAccountLockedNow                                       = false;

            if (TenantSetting.AccountPolicySettings.EnableAccountLocking)
            {
                i_LoginFailureCount = Convert.ToInt32(xmlNode_UserDetails.SelectSingleNode("LOGINFAILURECOUNT").InnerText);
                i_LoginFailureCount++;
                if (i_LoginFailureCount > (TenantSetting.AccountPolicySettings.LoginFailureCount - 1))
                {
                    //Lock user account, change the account locked date
                    b_UpdateAccountLockedDate = true;
                    enum_UserStatus = USERSTATUS.LOCKED;
                    bAccountLockedNow = true;
                }
                else
                {
                    enum_UserStatus = USERSTATUS.ACTIVE;
                }
                //Update STATUS, LOGINFAILURECOUNT, ACCOUNTLOCKEDDATE if user account is to be locked
                str_ErrorXml = UpdateUser(UserId, enum_UserStatus, i_LoginFailureCount, false, b_UpdateAccountLockedDate, obj_OMClass, ref xmlNode_UserDetails);
                if (!String.IsNullOrEmpty(str_ErrorXml))
                    return PrepareErrorXML(str_ErrorXml, ErrorCodes.UPDATE_FAILED, ErrorObjectType.AUTHENTICATION);
            }

            obj_ALError = new ALError(ErrorCodes.INVALID_USER_CREDENTIALS,
                     "Username or password is incorrect.", ErrorObjectType.AUTHENTICATION,
                     "Entity", ErrorType.ERROR, null, "Authentication Failed");
            return obj_ALError.Serialize();

        }

        #endregion
    }
}