#region System Include
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Threading;
using System.Collections.Specialized;
#endregion

#region Assetlink Include
using Assetlink.Services.Engines;
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.Services.Engines.DAMEngine;
using Assetlink.IOProxy;
#endregion

namespace Assetlink.Services.Engines.ArtworkProducerEngine
{
    using XML = System.String;
    using ERRORXML = System.String;

    public class ArtworkProducerCache : ALBaseEngine
    {
        #region variable
        ALErrorCommon objErrorCommon = null;
        String ReturnErrorXML = null;
        public String ArtworkCachePath = null;
        public String FileDimensions = null;
        #endregion

        System.Security.Principal.WindowsIdentity OrgIdentity;

        public enum CacheType
        {
            FormatCache,
            ArtworkCache
        }

        public enum GraphicTagUpdateType
        {
            UPLOAD,
            ASSET,
            DEFAULT,
            NOIMAGE
        }

        public enum GraphicTagUpdateStatus
        {
            DONE,
            PENDING
        }

        #region Constructor

        public ArtworkProducerCache(UInt64 UserID, String Language, String TenantName)
            : base(UserID, Language, TenantName)
        {            
        }
                
        #endregion

        #region Public Methods

        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="FormatXml","FormatID","Mode"></param>
        /// <returns></returns>
        public ERRORXML CreateCacheForFormat(XML FormatXml, String FormatID, String Mode)
        {
            #region Member Variables
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;
            String CacheFolderPath = null;
            String FormatIDFolder = null;
            String OrgFolder = null;
            String OrgFilePath = null;
            String FileName = null;
            String OutputXML = null;
            String AdditionalFilePath = null;
            StringBuilder sbCacheFormatXml = null;
            StringBuilder sbSearchXml = null;
            XPathDocument xpDoc_FormatXml = null;
            XPathNavigator xpNav_FormatXml = null;
            XPathNavigator xpNav_File = null;
            XPathNavigator xpNav_Modified = null;
            XmlDocument xmlDoc_OutputXml = null;
            XmlDocument XmlDoc_CacheFormatInfoXML = null;
            XmlDocument xmlDoc_FXml = null;
            XmlNode FormatNode = null;
            XPathNodeIterator xpItr_AdditionalFiles = null;
            XPathNodeIterator FileNodeList = null;
            XPathNodeIterator TempFileNodeListForCount = null;
            String AdditionalFilesFolder = null;
            Context ContextObj = null;
            #endregion

            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{3}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.CreateCacheForFormat</APINAME><FORMATXML>{0}</FORMATXML><FORMATID>{1}</FORMATID><MODE>{2}</MODE></ADDITIOALINFO>", FormatXml, FormatID, Mode, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                objErrorCommon = new ALErrorCommon();
                CacheFolderPath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
                FormatIDFolder = CacheFolderPath;
                OrgFolder = FormatIDFolder + "\\Original";
                sbCacheFormatXml = new StringBuilder(16);
                xpDoc_FormatXml = new XPathDocument(new StringReader(FormatXml));
                xpNav_FormatXml = xpDoc_FormatXml.CreateNavigator();
                XmlDoc_CacheFormatInfoXML = new XmlDocument();
                AdditionalFilesFolder = FormatIDFolder + "\\AdditionalFiles";

                if (Mode.ToUpper() == "CREATE")
                {
                    if (!String.IsNullOrEmpty(FormatXml))
                    {
                        #region Append Modifiedon to each file
                        xmlDoc_FXml = new XmlDocument();
                        xmlDoc_FXml.LoadXml(FormatXml);
                        StringBuilder sbDateXml = new StringBuilder(16);
                        sbDateXml.AppendFormat(@"<MODIFIEDON>
                                                    <DAY>{0}</DAY> 
                                                    <MONTH>{1}</MONTH>
                                                    <YEAR>{2}</YEAR>
                                                    <HOUR>{3}</HOUR>
                                                    <MINUTE>{4}</MINUTE>   
                                                    <SECOND>{5}</SECOND>
                                                </MODIFIEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                        FileNodeList = xmlDoc_FXml.CreateNavigator().Select("//FILE");
                        if (FileNodeList.Count > 0)
                        {
                            while (FileNodeList.MoveNext())
                            {
                                xpNav_Modified = FileNodeList.Current.SelectSingleNode("MODIFIEDON");
                                if (xpNav_Modified == null)
                                    FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                else
                                {
                                    if (!xpNav_Modified.HasChildren)
                                    {
                                        xpNav_Modified.DeleteSelf();
                                        FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                    }
                                }
                            }
                        }
                        #endregion

                        #region Create FormatInfo Xml and Copy Original file to folder
                        sbCacheFormatXml.AppendFormat(@"<FORMATDETAILS>{0}
                                                            <GRAPHICELEMENTCOLLECTION></GRAPHICELEMENTCOLLECTION>
                                                        </FORMATDETAILS>", xmlDoc_FXml.OuterXml);
                        xpNav_File = xpNav_FormatXml.SelectSingleNode("FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                        if (xpNav_File != null)
                        {
                            OrgFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                            //FileName = OrgFilePath.Substring(OrgFilePath.LastIndexOf("\\") + 1);
                            FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                        }
                        Directory.CreateDirectory(FormatIDFolder);
                        if (!String.IsNullOrEmpty(OrgFilePath))
                        {
                            Directory.CreateDirectory(OrgFolder);
                            File.Move(OrgFilePath, OrgFolder + "\\" + FileName);
                        }
                        xpItr_AdditionalFiles = xpNav_FormatXml.Select("FORMAT/FILESYSTEM/BINDER/FILE");
                        if (xpItr_AdditionalFiles.Count > 0)
                        {
                            Directory.CreateDirectory(FormatIDFolder + "\\AdditionalFiles");
                            while (xpItr_AdditionalFiles.MoveNext())
                            {
                                AdditionalFilePath = xpItr_AdditionalFiles.Current.SelectSingleNode("PATH").Value;
                                FileName = AdditionalFilePath.Substring(AdditionalFilePath.LastIndexOf("\\") + 1);

                                TempFileNodeListForCount = xpNav_FormatXml.Select("//FILE[PATH/text()='" + AdditionalFilePath + "']");
                                if (TempFileNodeListForCount.Count > 1)
                                    File.Copy(AdditionalFilePath, FormatIDFolder + "\\AdditionalFiles" + "\\" + FileName, true);
                                else
                                    File.Move(AdditionalFilePath, FormatIDFolder + "\\AdditionalFiles" + "\\" + FileName);
                            }
                        }
                        File.WriteAllText(FormatIDFolder + "\\FormatInfo.xml", sbCacheFormatXml.ToString());
                        #endregion
                    }
                }
                else
                {
                    #region Get FormatDetails using FormatID in Edit mode
                    sbSearchXml = new StringBuilder(16);

                    sbSearchXml.Append(@"<SEARCH>
                                    <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                    <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                    <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>FORMATID</EXPRESSION>
                                                     <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>" + FormatID + @"</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                             </FILTERLIST>
                                             <RELATIONSHIP NAME='FORMAT_FORMATINFO'>                                                            
                                             </RELATIONSHIP>
                                     </TARGETENTITY>
                                  </SEARCH>");
                    OMIObj = new OMInterface(UserID, Language, TenantName);
                    ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref OutputXML);
                    #endregion

                    if (ErrorXML == null)
                    {
                        if (Directory.Exists(FormatIDFolder))
                        {
                            #region Update FormatInfo Xml
                            if (File.Exists(FormatIDFolder + "\\FormatInfo.xml"))
                            {
                                XmlDoc_CacheFormatInfoXML.Load(FormatIDFolder + "\\FormatInfo.xml");
                                xmlDoc_OutputXml = new XmlDocument();
                                xmlDoc_OutputXml.LoadXml(OutputXML);                                
                                if (xmlDoc_OutputXml != null)
                                {
                                    FileNodeList = xmlDoc_OutputXml.CreateNavigator().Select("//FILE");
                                    if (FileNodeList.Count > 0)
                                    {
                                        StringBuilder sbDateXml = new StringBuilder(16);
                                        sbDateXml.AppendFormat(@"<MODIFIEDON>
                                                    <DAY>{0}</DAY> 
                                                    <MONTH>{1}</MONTH>
                                                    <YEAR>{2}</YEAR>
                                                    <HOUR>{3}</HOUR>
                                                    <MINUTE>{4}</MINUTE>   
                                                    <SECOND>{5}</SECOND>
                                                </MODIFIEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                                        while (FileNodeList.MoveNext())
                                        {
                                            xpNav_Modified = FileNodeList.Current.SelectSingleNode("MODIFIEDON");
                                            if (xpNav_Modified == null)
                                            {
                                                String FileDisplayName = FileNodeList.Current.SelectSingleNode("DISPLAYNAME").InnerXml;
                                                XmlNode ModifiedOn = XmlDoc_CacheFormatInfoXML.SelectSingleNode("//FILE[DISPLAYNAME/text()='" + FileDisplayName + "']/MODIFIEDON");
                                                if ( ModifiedOn == null)
                                                    FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                                else
                                                    FileNodeList.Current.AppendChild(ModifiedOn.OuterXml);
                                            }
                                            else
                                            {
                                                if (!xpNav_Modified.HasChildren)
                                                {
                                                    String FileDisplayName = FileNodeList.Current.SelectSingleNode("DISPLAYNAME").InnerXml;
                                                    XmlNode ModifiedOn = XmlDoc_CacheFormatInfoXML.SelectSingleNode("//FILE[DISPLAYNAME/text()='" + FileDisplayName + "']/MODIFIEDON");
                                                    XmlNode MNode = xmlDoc_OutputXml.SelectSingleNode("//FILE[DISPLAYNAME/text()='" + FileDisplayName + "']/MODIFIEDON");
                                                    XmlNode FileNode = xmlDoc_OutputXml.SelectSingleNode("//FILE[DISPLAYNAME/text()='" + FileDisplayName + "']");
                                                    FileNode.RemoveChild(MNode);
                                                    if ( ModifiedOn == null)
                                                        FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                                    else
                                                        FileNodeList.Current.AppendChild(ModifiedOn.OuterXml);
                                                }
                                            }
                                        }
                                    }
                                    OutputXML = xmlDoc_OutputXml.OuterXml;
                                }

                                
                                FormatNode = XmlDoc_CacheFormatInfoXML.SelectSingleNode("FORMATDETAILS/FORMAT");
                                FormatNode.InnerXml = ALCommon.GetInnerValue(ref OutputXML, "<FORMAT>", "</FORMAT>");
                                XmlDoc_CacheFormatInfoXML.Save(FormatIDFolder + "\\FormatInfo.xml");
                            }
                            #endregion

                            #region Update Original file if original file is uploaded while editing format
                            xpNav_File = xpNav_FormatXml.SelectSingleNode("FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                            if (xpNav_File != null)
                            {
                                OrgFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                                //FileName = OrgFilePath.Substring(OrgFilePath.LastIndexOf("\\") + 1);
                                FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                if (Directory.Exists(OrgFolder))
                                {
                                    //Directory.Delete(OrgFolder, true);                                    
                                    SASMOMDirectory.DeleteFolder(OrgFolder, true, ContextObj);
                                    Directory.CreateDirectory(OrgFolder);
                                    if (!File.Exists(OrgFolder + "\\" + FileName))
                                        File.Move(OrgFilePath, OrgFolder + "\\" + FileName);
                                    else
                                        File.Copy(OrgFilePath, OrgFolder + "\\" + FileName, true);
                                }
                            }
                            #endregion

                            #region Update Additional files if changed while editing format

                            XPathNodeIterator xpIter_Files = xpNav_FormatXml.Select("FORMAT/FILESYSTEM/BINDER/FILE [translate(@Operation, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')='DELETE']");
                            while (xpIter_Files.MoveNext())
                            {
                                xpNav_File = xpIter_Files.Current;
                                if (xpNav_File != null)
                                {
                                    FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                    String FilePath = AdditionalFilesFolder + "\\" + FileName;
                                    if (File.Exists(FilePath))
                                    {
                                        try
                                        {
                                            File.Delete(FilePath);
                                        }
                                        catch
                                        { }
                                    }
                                }
                            }

                            xpIter_Files = xpNav_FormatXml.Select("FORMAT/FILESYSTEM/BINDER/FILE [translate(@Operation, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')='CREATE']");
                            while (xpIter_Files.MoveNext())
                            {
                                xpNav_File = xpIter_Files.Current;
                                if (xpNav_File != null)
                                {
                                    OrgFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                                    FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                    if (!Directory.Exists(AdditionalFilesFolder))
                                    {
                                        Directory.CreateDirectory(AdditionalFilesFolder);
                                    }
                                    if (!File.Exists(AdditionalFilesFolder + "\\" + FileName))
                                        File.Move(OrgFilePath, AdditionalFilesFolder + "\\" + FileName);
                                    else
                                        File.Copy(OrgFilePath, AdditionalFilesFolder + "\\" + FileName, true);
                                }
                            }

                            #endregion

                        }
                    }
                    else
                    {
                        objErrorCommon.AppendError("APENGINE_FORMATCACHE_CREATECACHEFORFORMAT_GETFORMATDETAILS_FAILED", this.GetType().ToString(), "CreateCacheForFormat", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                    }
                }
                return ErrorXML;

            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_FORMATCACHE_CREATECACHEFORFORMAT_EXCEPTION", this.GetType().ToString(), "CreateCacheForFormat", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="GraphicElementXml","GraphicElementID","FormatID"></param>
        /// <returns></returns>
        public ERRORXML CreateCacheForGraphicElement(XML GraphicElementXml, String GraphicElementID, String FormatID)
        {
            #region Member Variables
            ERRORXML ErrorXML = null;
            String CacheFolderPath = null;
            String FormatIDFolder = null;
            String OrgGEFolder = null;
            String OrgFilePath = null;
            String FileName = null;
            String GEID = null;
            String TagName = null;
            String FileXml = null;
            StringBuilder sbCacheFormatXml = null;
            StringBuilder sbGEXml = null;
            //XPathDocument xpDoc_GEXml = null;
            XmlDocument xpDoc_GEXml = null;
            XPathNavigator xpNav_GEXml = null;
            XPathNavigator xpNav_File = null;
            XmlDocument XmlDoc_CacheFormatInfoXML = null;
            XmlNode GECollectionNode = null;
            XmlNode GENode = null;
            XmlNode GENodeInCacheXml = null;
            Context ContextObj = null;
            #endregion

            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{3}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.CreateCacheForGraphicElement</APINAME><GRAPHICELEMENTXML>{0}</GRAPHICELEMENTXML><GRAPHICELEMENTID>{1}</GRAPHICELEMENTID><FORMATID>{2}</FORMATID></ADDITIOALINFO>", GraphicElementXml, GraphicElementID, FormatID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                objErrorCommon = new ALErrorCommon();
                CacheFolderPath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
                FormatIDFolder = CacheFolderPath;
                OrgGEFolder = FormatIDFolder + "\\GE_" + GraphicElementID + "\\Original";
                sbCacheFormatXml = new StringBuilder(16);
                sbGEXml = new StringBuilder(16);
                //xpDoc_GEXml = new XPathDocument(new StringReader(GraphicElementXml));
                xpDoc_GEXml = new XmlDocument();
                xpDoc_GEXml.LoadXml(GraphicElementXml);
                xpNav_GEXml = xpDoc_GEXml.CreateNavigator();
                XmlDoc_CacheFormatInfoXML = new XmlDocument();

                if (!String.IsNullOrEmpty(GraphicElementXml))
                {
                    if (Directory.Exists(FormatIDFolder))
                    {
                        if (File.Exists(FormatIDFolder + "\\FormatInfo.xml"))
                        {
                            xpNav_File = xpNav_GEXml.SelectSingleNode("GRAPHICELEMENT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                            if (xpNav_File != null)
                            {
                                #region Update FormatInfo Xml
                                GEID = xpNav_GEXml.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENTID").InnerXml;
                                TagName = xpNav_GEXml.SelectSingleNode("GRAPHICELEMENT/GRAPHICTAGNAME").InnerXml;
                                FileXml = xpNav_File.OuterXml;
                                StringBuilder DateXml = new StringBuilder(16);
                                DateXml.AppendFormat(@"<MODIFIEDON>
                                                            <DAY>{0}</DAY> 
                                                            <MONTH>{1}</MONTH>
                                                            <YEAR>{2}</YEAR>
                                                            <HOUR>{3}</HOUR>
                                                            <MINUTE>{4}</MINUTE>   
                                                            <SECOND>{5}</SECOND>
                                                        </MODIFIEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year,
                                                                                     DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                                XPathNavigator xpNav_Modified = xpNav_File.SelectSingleNode("MODIFIEDON");
                                if (xpNav_Modified == null)
                                    FileXml = FileXml.Insert(FileXml.IndexOf("</FILE>"), DateXml.ToString());
                                else
                                {
                                    xpNav_Modified.DeleteSelf();
                                    xpNav_File.AppendChild(DateXml.ToString());
                                    FileXml = xpNav_File.OuterXml;
                                }

                                sbGEXml.AppendFormat(@"<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>
                                                        <GRAPHICTAGNAME>{1}</GRAPHICTAGNAME>                                                        
                                                        <FILESYSTEM>{2}</FILESYSTEM>", GEID, TagName, FileXml);
                                GENode = (XmlNode)XmlDoc_CacheFormatInfoXML.CreateElement("GRAPHICELEMENT");
                                GENode.InnerXml = sbGEXml.ToString();

                                XmlDoc_CacheFormatInfoXML.Load(FormatIDFolder + "\\FormatInfo.xml");
                                GECollectionNode = XmlDoc_CacheFormatInfoXML.SelectSingleNode("FORMATDETAILS/GRAPHICELEMENTCOLLECTION");
                                if (GECollectionNode != null)
                                {
                                    GENodeInCacheXml = GECollectionNode.SelectSingleNode("GRAPHICELEMENT[GRAPHICELEMENTID/text()='" + GEID + "']");
                                    if (GENodeInCacheXml != null)
                                    {
                                        GENodeInCacheXml = GECollectionNode.RemoveChild(GENodeInCacheXml);
                                    }
                                    GECollectionNode.AppendChild(GENode);
                                    XmlDoc_CacheFormatInfoXML.Save(FormatIDFolder + "\\FormatInfo.xml");
                                }
                                #endregion

                                #region Copy GE Original file to folder
                                OrgFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                                FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                //FileName = OrgFilePath.Substring(OrgFilePath.LastIndexOf("\\") + 1);
                                if (Directory.Exists(OrgGEFolder))
                                {
                                    //Directory.Delete(OrgGEFolder, true);
                                    SASMOMDirectory.DeleteFolder(OrgGEFolder, true, ContextObj);
                                    Directory.CreateDirectory(OrgGEFolder);
                                    File.Copy(OrgFilePath, OrgGEFolder + "\\" + FileName, true);
                                }
                                else
                                {
                                    Directory.CreateDirectory(OrgGEFolder);
                                    File.Copy(OrgFilePath, OrgGEFolder + "\\" + FileName, true);
                                }
                                #endregion
                            }
                        }
                    }
                }
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_FORMATCACHE_CREATECACHEFORGRAPHICELEMENT_EXCEPTION", this.GetType().ToString(), "CreateCacheForGraphicElement", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            return ErrorXML;
        }

        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="FormatID","ArtworkID"></param>
        /// <returns></returns>
        public ERRORXML CopyFormatCacheToArtworkCache(String FormatID, String ArtworkID)
        {
            #region Member Variables
            String FormatCachePath = null;
            ERRORXML ErrorXML = null;
            #endregion

            ArtworkCachePath = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache);
            FormatCachePath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);

            if (Directory.Exists(FormatCachePath))
            {
                if (Directory.Exists(ArtworkCachePath + "\\Input"))
                {
                    CopyFiles(FormatCachePath, ArtworkCachePath + "\\Input");
                }
                else
                {
                    Directory.CreateDirectory(ArtworkCachePath + "\\Input");
                    CopyFiles(FormatCachePath, ArtworkCachePath + "\\Input");
                }
            }
            else
            {
                objErrorCommon.AppendError("APENGINE_FORMATCACHE_COPYFORMATCACHETOARTWORKCACHE_FORMATCACHEPATH_FAILED", this.GetType().ToString(), "CopyFormatCacheToArtworkCache", null, "Format Cache folder does not exists.");
                ErrorXML = objErrorCommon.ErrorCollectionXML;
            }
            return ErrorXML;
        }

        public static String GetCacheFolderPath(String TenantName, String ObjectID, CacheType Type)
        {
            StringBuilder sbCacheFolderPath = null;

            sbCacheFolderPath = new StringBuilder(30);
            if (Type == ArtworkProducerCache.CacheType.FormatCache)
            {
                sbCacheFolderPath.AppendFormat("{0}{2}\\{1}", System.Configuration.ConfigurationManager.AppSettings["AP_FormatCachePath"], ObjectID, TenantName);
                String str_Path = sbCacheFolderPath.ToString();
                if (!Directory.Exists(str_Path))
                {
                    Directory.CreateDirectory(str_Path);
                }
            }
            else
            {
                sbCacheFolderPath.AppendFormat("{0}{2}\\{1}", System.Configuration.ConfigurationManager.AppSettings["AP_ArtworkCachePath"], ObjectID, TenantName);
                String str_Path = sbCacheFolderPath.ToString();
                if (!Directory.Exists(str_Path))
                {
                    Directory.CreateDirectory(str_Path);
                }
            }

            return sbCacheFolderPath.ToString();
        }

        public static String GetFormatDetailsXMLPath(String TenantName, String FormatID)
        {
            return GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache) + "\\FormatInfo.xml";
        }

        public static String GetArtworkValuesXMLPath(String TenantName, String ArtworkID)
        {
            return GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache) + "\\Artwork.xml";
        }

        public static String GetGraphicElementOriginalFilePath(String TenantName, String FormatID, String GraphicElementID)
        {
            String GEFolder = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache) + "\\GE_" + GraphicElementID + "\\Original";
            string[] arr_Files = Directory.GetFiles(GEFolder);
            if (arr_Files != null && !String.IsNullOrEmpty(arr_Files[0]))
            {
                if (!String.IsNullOrEmpty(arr_Files[0]))
                {
                    return arr_Files[0];
                }
                throw new Exception("Could not find the requested Graphic Element file.");
            }
            else
                throw new Exception("Could not find the requested Graphic Element file.");
        }

        public static String GetFormatOriginalFilePath(String TenantName, String FormatID)
        {
            String FormatFolder = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache) + "\\Original";
            string[] arr_Files = Directory.GetFiles(FormatFolder);
            if (arr_Files != null && !String.IsNullOrEmpty(arr_Files[0]))
            {
                if (!String.IsNullOrEmpty(arr_Files[0]))
                {
                    return arr_Files[0];
                }
                throw new Exception("Could not find the requested Graphic Element file.");
            }
            else
                throw new Exception("Could not find the requested Graphic Element file.");
        }

        public static String GetArtworkOriginalFilePath(String TenantName, String ArtworkID)
        {
            String CacheXMLPath = GetArtworkValuesXMLPath(TenantName, ArtworkID);
            String CacheXML = File.ReadAllText(CacheXMLPath);
            String OrgFilePath = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref CacheXML, "<ORIGINALFILEPATH>", "</ORIGINALFILEPATH>"));
            return OrgFilePath;
        }

        public bool ModifyTextTagValue(String ArtworkID, String TEID, String TagValue)
        {
            String str_ArtworkTagValuesXMLPath = GetArtworkValuesXMLPath(TenantName, ArtworkID);
            XmlDocument xDoc = new XmlDocument();
            xDoc.Load(str_ArtworkTagValuesXMLPath);
            XmlNode xNode_TE = xDoc.SelectSingleNode("ARTWORK/ARTWORKVALUES/TEXTTAGS/TAG[@ID='" + TEID + "']");
            if (xNode_TE != null)
            {
                ((XmlElement)xNode_TE).SetAttribute("ISHIDDEN", "0");
                xNode_TE.SelectSingleNode("VALUE").InnerXml = TagValue;
                xDoc.Save(str_ArtworkTagValuesXMLPath);
            }
            else
                return false;
            return true;
        }

        public bool ModifyGraphicTagValue(String ArtworkID, String GEID, String TagValue, GraphicTagUpdateType UpdateType, GraphicTagUpdateStatus Status, bool MakeAsyncCall, out bool DisAssociateArtworkDigitalAsset)
        {
            String          JobPath                  = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache) + "\\Input\\";
            String          ArtworkXMLPath           = GetArtworkValuesXMLPath(TenantName, ArtworkID);
            XmlDocument     xDoc_ArtworkValuesXML    = new XmlDocument();
            int             RetryCount               = 3;
            String          str_IntermidiateFilePath = null;

            DisAssociateArtworkDigitalAsset = true;

            while (RetryCount > 0)
            {
                try
                {
                    RetryCount--;
            xDoc_ArtworkValuesXML.Load(ArtworkXMLPath);
                    break;
                }
                catch (Exception)
                {                    
                    Thread.Sleep(1000);                    
                }
            }
            XmlNode xNode_GE = xDoc_ArtworkValuesXML.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@ID='" + GEID + "']");
            if (xNode_GE != null)
            {
                switch (UpdateType)
                {
                    case GraphicTagUpdateType.DEFAULT:
                        {
                            #region Check Previous State and delete old files
                            XmlNode ValueTypeNode = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                            XmlNode StatusNode = xNode_GE.Attributes.GetNamedItem("STATUS");
                            DeleteOldGEFiles(ValueTypeNode.Value, StatusNode.Value, ((XmlElement)xNode_GE).InnerText);
                            #endregion

                            if (Status != GraphicTagUpdateStatus.DONE)
                            {                               
                                String FormatID = GetFormatID(TenantName, ArtworkID);
                                if (String.IsNullOrEmpty(FormatID))
                                    return false;
                                String str_GEOrgFilePath = GetGraphicElementOriginalFilePath(TenantName, FormatID, GEID);
                                String DestPath = JobPath + Path.GetFileName(str_GEOrgFilePath);
                                //String DestPath = JobPath + Guid.NewGuid() + Path.GetExtension(str_GEOrgFilePath);                                
                                
                                //Copy Original file to some Temp location so that the processing will be performed
                                //and the processed file will be copied to job folder
                                if (!String.IsNullOrEmpty(FileDimensions))
                                {
                                    str_IntermidiateFilePath = ALCommon.GetTempPath() + Guid.NewGuid() + Path.GetExtension(str_GEOrgFilePath);
                                    File.Copy(str_GEOrgFilePath, str_IntermidiateFilePath, true);
                                    
                                    str_IntermidiateFilePath        = processAndGetFileToArtworkJobFolder(str_IntermidiateFilePath, Path.GetFileName(str_IntermidiateFilePath), FileDimensions, "SERVICES");
                                    File.Copy(str_IntermidiateFilePath, DestPath, true);
                                }
                                else
                                    File.Copy(str_GEOrgFilePath, DestPath, true);

                                ((XmlElement)xNode_GE).InnerText = DestPath;
                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "DEFAULT";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "DONE";
                            }
                            else
                            {                               

                                ((XmlElement)xNode_GE).InnerText = TagValue;
                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "DEFAULT";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "DONE";
                            }

                            xDoc_ArtworkValuesXML.Save(ArtworkXMLPath);

                            break;
                        }
                    case GraphicTagUpdateType.UPLOAD:
                        {
                            #region Check Previous State and delete old files
                            XmlNode ValueTypeNode = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                            XmlNode StatusNode = xNode_GE.Attributes.GetNamedItem("STATUS");
                            DeleteOldGEFiles(ValueTypeNode.Value, StatusNode.Value, ((XmlElement)xNode_GE).InnerText);
                            #endregion

                            if (Status != GraphicTagUpdateStatus.DONE)
                            {
                                String FormatID = GetFormatID(TenantName, ArtworkID);
                                String GEFilePath = GetGraphicElementOriginalFilePath(TenantName, FormatID, GEID);
                                String FileName = Path.GetFileName(GEFilePath);
                                //String DestPath = JobPath + Path.GetFileName(FileName);
                                String DestPath = JobPath + Guid.NewGuid() + Path.GetExtension(TagValue);

                                //Copy Original file to some Temp location so that the processing will be performed
                                //and the processed file will be copied to job folder
                                if (!String.IsNullOrEmpty(FileDimensions))
                                {
                                    str_IntermidiateFilePath = ALCommon.GetTempPath() + Guid.NewGuid() + Path.GetExtension(TagValue);
                                    File.Copy(TagValue, str_IntermidiateFilePath, true);
                                    
                                    str_IntermidiateFilePath = processAndGetFileToArtworkJobFolder(str_IntermidiateFilePath, Path.GetFileName(str_IntermidiateFilePath), FileDimensions, "SERVICES");
                                    File.Copy(str_IntermidiateFilePath, DestPath, true);
                                }
                                else
                                    File.Copy(TagValue, DestPath, true);

                                ((XmlElement)xNode_GE).InnerText = DestPath;
                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "UPLOAD";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "DONE";
                            }
                            else
                            {
                                ((XmlElement)xNode_GE).InnerText = TagValue;
                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "UPLOAD";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "DONE";
                            }
                            xDoc_ArtworkValuesXML.Save(ArtworkXMLPath);

                            break;
                        }
                    case GraphicTagUpdateType.ASSET:
                        {
                            #region Check Previous State
                            XmlNode ValueTypeNode = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                            XmlNode StatusNode = xNode_GE.Attributes.GetNamedItem("STATUS");
                            String GEFilePath = ((XmlElement)xNode_GE).InnerText;
                            String str_Status = StatusNode.Value;
                            #endregion

                            if (Status != GraphicTagUpdateStatus.DONE)
                            {
                                if (String.IsNullOrEmpty(TagValue))
                                    return false;                               

                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "ASSET";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "PENDING";
                                ((XmlElement)xNode_GE).SetAttribute("STARTTIME", DateTime.Now.ToString());

                                ((XmlElement)xNode_GE).InnerText = TagValue;
                                xDoc_ArtworkValuesXML.Save(ArtworkXMLPath);

                                GetArtworkAsset(ArtworkID, GEID, TagValue, MakeAsyncCall);
                            }
                            else
                            {
                                ((XmlElement)xNode_GE).InnerText = TagValue;
                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "ASSET";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "DONE";
                                xDoc_ArtworkValuesXML.Save(ArtworkXMLPath);
                            }
                            #region Check and delete old files
                            String tmpGEFilePath = GEFilePath;
                            int pos = tmpGEFilePath.IndexOf("_");
                            if (pos >= 0)
                                tmpGEFilePath = tmpGEFilePath.Substring(pos, tmpGEFilePath.Length - pos);                            
                            if (xDoc_ArtworkValuesXML.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[ " + ALXmlUtil.GetXpathValueString(tmpGEFilePath) + "=substring(text(),string-length(text()) - string-length(" + ALXmlUtil.GetXpathValueString(tmpGEFilePath) + ") +1 ) ]") == null)
                            {
                                DeleteOldGEFiles(ValueTypeNode.Value, str_Status, GEFilePath);
                                DisAssociateArtworkDigitalAsset = true;
                            }
                            else
                                DisAssociateArtworkDigitalAsset = false;

                            #endregion
                            break;
                        }
                    case GraphicTagUpdateType.NOIMAGE:
                        {
                            #region Check Previous State and delete old files
                            XmlNode ValueTypeNode = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                            XmlNode StatusNode = xNode_GE.Attributes.GetNamedItem("STATUS");
                            DeleteOldGEFiles(ValueTypeNode.Value, StatusNode.Value, ((XmlElement)xNode_GE).InnerText);
                            #endregion

                            if (Status == GraphicTagUpdateStatus.DONE)
                            {
                                String FormatID = GetFormatID(TenantName, ArtworkID);
                                String GEFilePath = GetGraphicElementOriginalFilePath(TenantName, FormatID, GEID);
                                String FileName = Path.GetFileName(GEFilePath);
                                String DestPath = JobPath + Path.GetFileName(FileName);
                                //String DestPath = JobPath + Guid.NewGuid() + Path.GetExtension(GEFilePath);
                                if (File.Exists(DestPath))
                                {
                                    File.Delete(DestPath);
                                }

                                ((XmlElement)xNode_GE).InnerText = TagValue;
                                XmlNode xAttr = xNode_GE.Attributes.GetNamedItem("VALUETYPE");
                                xAttr.Value = "NOIMAGE";
                                xAttr = xNode_GE.Attributes.GetNamedItem("STATUS");
                                xAttr.Value = "DONE";
                            }                            
                            xDoc_ArtworkValuesXML.Save(ArtworkXMLPath);

                            break;
                        }
                }
            }
            else
                return false;

            return true;
        }

        /// <alsummary>
        /// This is an overload of the existing function to allow 
        /// control over SYNCHRONOUS/ASYNCHRONOUS call made to update artwork.xml   
        /// </alsummary>
        public bool ModifyGraphicTagValue(String ArtworkID, String GEID, String TagValue, GraphicTagUpdateType UpdateType, GraphicTagUpdateStatus Status, bool MakeAsyncCall)
        {
            bool DisAssociateArtworkDigitalAsset;
            return ModifyGraphicTagValue(ArtworkID, GEID, TagValue, UpdateType, Status, MakeAsyncCall, out DisAssociateArtworkDigitalAsset);
        }

        public bool ModifyGraphicTagValue(String ArtworkID, String GEID, String TagValue, GraphicTagUpdateType UpdateType, GraphicTagUpdateStatus Status)
        {            
            return ModifyGraphicTagValue(ArtworkID, GEID, TagValue, UpdateType, Status, true);
        }
        
        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="ArtworkID"></param>
        /// <returns></returns>
        public ERRORXML PrepareArtworkCache(String ArtworkID, String Mode)
        {
            ERRORXML ErrorXML = null;
            Context ContextObj = null;
            StringBuilder ContextAdditionalInfo = null;
            ContextAdditionalInfo = new StringBuilder(20);
            ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{2}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.PrepareArtworkCache</APINAME><ARTWORKID>{0}</ARTWORKID><MODE>{1}</MODE></ADDITIOALINFO>", ArtworkID, Mode, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());


            ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
            ArtworkCachePath = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache);
            if (Directory.Exists(ArtworkCachePath))
            {
                //Directory.Delete(ArtworkCachePath, true);
                SASMOMDirectory.DeleteFolder(ArtworkCachePath, true, ContextObj);
            }
            if (Mode.ToUpper() == "CREATE")
            {
                ErrorXML = ArtworkCacheForCreateMode(ArtworkID);
                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_APCACHE_PREPAREARTWORKCACHE_ARTWORKCACHEFORCREATEMODE_FAILED", this.GetType().ToString(), "ArtworkCacheForCreateMode", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
            }
            else
            {
                ErrorXML = ArtworkCacheForEditMode(ArtworkID);
                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_APCACHE_PREPAREARTWORKCACHE_ARTWORKCACHEFOREDITMODE_FAILED", this.GetType().ToString(), "ArtworkCacheForEditMode", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
            }
            return ErrorXML;
        }

        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="FormatID"></param>
        /// <returns></returns>
        public ERRORXML PrepareFormatCache(String FormatID)
        {
            #region Member Variables
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;
            String CacheFolderPath = null;
            String FormatIDFolder = null;
            String OrgFolder = null;
            String OrgFilePath = null;
            String FileName = null;
            String OutputXML = null;
            String AdditionalFilePath = null;
            String FormatXml = null;
            String FileSystemXML = null;
            String OnlyFormatXml = null;
            String GEXml = null;
            String GEID = null;
            String TagName = null;
            String OrgGEFolder = null;
            String ObjectID = null;
            StringBuilder sbCacheFormatXml = null;
            StringBuilder sbSearchXml = null;
            XPathDocument xpDoc_FormatXml = null;
            XPathNavigator xpNav_FormatXml = null;
            XPathNavigator xpNav_File = null;
            XPathNodeIterator xpItr_AdditionalFiles = null;
            XPathNodeIterator xpItr_GE = null;
            XPathDocument xpDoc_FileSystemXml = null;
            XPathNavigator xpNav_FileSystemXml = null;
            Context ContextObj = null;            
            #endregion

            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.PrepareFormatCache</APINAME><FORMATID>{0}</FORMATID></ADDITIOALINFO>", FormatID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                objErrorCommon = new ALErrorCommon();
                CacheFolderPath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
                FormatIDFolder = CacheFolderPath;
                OrgFolder = FormatIDFolder + "\\Original";
                sbCacheFormatXml = new StringBuilder(16);

                if (Directory.Exists(FormatIDFolder))
                {
                    //Directory.Delete(FormatIDFolder, true);
                    SASMOMDirectory.DeleteFolder(FormatIDFolder, true, ContextObj);
                }
                #region Get FormatDetails using FormatID
                sbSearchXml = new StringBuilder(16);

                sbSearchXml.Append(@"<SEARCH>
                            <TARGETOBJECT>FORMAT</TARGETOBJECT>
                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                            <TARGETENTITY>
                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                    <FILTERLIST>
                                        <FILTER>
                                            <EXPRESSION>FORMATID</EXPRESSION>
                                             <OPERATOR>EQUALTO</OPERATOR>
                                            <EXPRESSIONVALUE>" + FormatID + @"</EXPRESSIONVALUE>
                                            <FILTERID>1</FILTERID>
                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                        </FILTER>
                                     </FILTERLIST>
                                     <RELATIONSHIP NAME='FORMAT_FORMATINFO'>                                                            
                                     </RELATIONSHIP>
                                     <RELATIONSHIP NAME='FORMAT_GRAPHICELEMENT'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                <ALIAS>GRAPHICELEMENTID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                <ALIAS>GRAPHICTAGNAME</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                <ALIAS>ORIGINALFILE</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>                                                            
                                     </RELATIONSHIP>
                             </TARGETENTITY>
                          </SEARCH>");
                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref OutputXML);
                #endregion

                if (ErrorXML == null)
                {
                    xpDoc_FormatXml = new XPathDocument(new StringReader(OutputXML));
                    xpNav_FormatXml = xpDoc_FormatXml.CreateNavigator();

                    if (!String.IsNullOrEmpty(OutputXML))
                    {
                        Directory.CreateDirectory(FormatIDFolder);

                        #region Append Modifiedon to each file
                        XmlDocument xmlDoc_FXml = new XmlDocument();
                        xmlDoc_FXml.LoadXml(OutputXML);
                        StringBuilder sbDateXml = new StringBuilder(16);
                        sbDateXml.AppendFormat(@"<MODIFIEDON>
                                                    <DAY>{0}</DAY> 
                                                    <MONTH>{1}</MONTH>
                                                    <YEAR>{2}</YEAR>
                                                    <HOUR>{3}</HOUR>
                                                    <MINUTE>{4}</MINUTE>   
                                                    <SECOND>{5}</SECOND>
                                                </MODIFIEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                        XPathNodeIterator FileNodeList = xmlDoc_FXml.CreateNavigator().Select("//FILE");
                        if (FileNodeList.Count > 0)
                        {
                            XPathNavigator xpNav_Modified = null;
                            while (FileNodeList.MoveNext())
                            {
                                //FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                xpNav_Modified = FileNodeList.Current.SelectSingleNode("MODIFIEDON");
                                if (xpNav_Modified == null)
                                    FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                else
                                {
                                    if (!xpNav_Modified.HasChildren)
                                    {
                                        xpNav_Modified.DeleteSelf();
                                        FileNodeList.Current.AppendChild(sbDateXml.ToString());
                                    }
                                }
                            }
                        }
                        OutputXML = xmlDoc_FXml.OuterXml;
                        #endregion
                        #region Create FormatInfo Xml and Copy Original file to folder
                        FormatXml = ALCommon.GetInnerValue(ref OutputXML, "<FORMATCOLLECTION>", "</FORMATCOLLECTION>");
                        if (FormatXml.Contains("<FORMAT_GRAPHICELEMENT>"))
                        {
                            OnlyFormatXml = FormatXml.Substring(0, FormatXml.IndexOf("<FORMAT_GRAPHICELEMENT>"));
                            OnlyFormatXml = OnlyFormatXml + FormatXml.Substring(FormatXml.IndexOf("</FORMAT_GRAPHICELEMENT>") + 24);
                            GEXml = ALCommon.GetInnerValue(ref OutputXML, "<FORMAT_GRAPHICELEMENT>", "</FORMAT_GRAPHICELEMENT>");
                        }
                        else
                        {
                            OnlyFormatXml = FormatXml;
                        }

                        //if (xpNav_FormatXml.SelectSingleNode("FORMATCOLLECTION/FORMAT/FORMAT_GRAPHICELEMENT") != null)
                        //{
                        //    GEXml = xpNav_FormatXml.SelectSingleNode("FORMATCOLLECTION/FORMAT/FORMAT_GRAPHICELEMENT").InnerXml;
                        //}

                        if (!String.IsNullOrEmpty(OnlyFormatXml))
                        {
                            sbCacheFormatXml.AppendFormat(@"<FORMATDETAILS>{0}", OnlyFormatXml);

                            if (!String.IsNullOrEmpty(GEXml))
                            {
                                sbCacheFormatXml.AppendFormat("<GRAPHICELEMENTCOLLECTION>{0}</GRAPHICELEMENTCOLLECTION>", GEXml);
                            }

                            sbCacheFormatXml.AppendFormat("</FORMATDETAILS>");
                        }

                        /*if (!String.IsNullOrEmpty(OnlyFormatXml) && !String.IsNullOrEmpty(GEXml))
                        {
                            sbCacheFormatXml.AppendFormat(@"<FORMATDETAILS>{0}
                                                        <GRAPHICELEMENTCOLLECTION>{1}</GRAPHICELEMENTCOLLECTION>
                                                    </FORMATDETAILS>", OnlyFormatXml, GEXml);
                        }*/

                        #region Get FormatFile
                        xpNav_File = xpNav_FormatXml.SelectSingleNode("FORMATCOLLECTION/FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                        if (xpNav_File != null)
                        {
                            xpNav_File.MoveToParent();
                            FileSystemXML = xpNav_File.OuterXml;
                            ALFile objFile = new ALFile(UserID, Language, TenantName);
                            ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                            ObjectID = ObjectID.Replace("<OBJECTID>", "");
                            FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                            FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                            ErrorXML = objFile.GetFile(ref FileSystemXML);
                            if (ErrorXML != null)
                            {

                            }
                            else
                            {
                                xpDoc_FileSystemXml = new XPathDocument(new StringReader(FileSystemXML));
                                xpNav_FileSystemXml = xpDoc_FileSystemXml.CreateNavigator();

                                #region Processing for Additional Files
                                xpItr_AdditionalFiles = xpNav_FileSystemXml.Select("FILESYSTEM/FILESYSTEM/BINDER/FILE");
                                if (xpItr_AdditionalFiles.Count > 0)
                                {
                                    Directory.CreateDirectory(FormatIDFolder + "\\AdditionalFiles");
                                    while (xpItr_AdditionalFiles.MoveNext())
                                    {
                                        AdditionalFilePath = xpItr_AdditionalFiles.Current.SelectSingleNode("PATH").Value;
                                        FileName = xpItr_AdditionalFiles.Current.SelectSingleNode("DISPLAYNAME").Value;
                                        if (!String.IsNullOrEmpty(AdditionalFilePath))
                                        {
                                            File.Copy(AdditionalFilePath, FormatIDFolder + "\\AdditionalFiles\\" + FileName, true);
                                        }
                                    }
                                }
                                #endregion

                                #region Processing for Format Original file
                                xpNav_File = xpNav_FileSystemXml.SelectSingleNode("FILESYSTEM/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                                if (xpNav_File != null)
                                {
                                    Directory.CreateDirectory(OrgFolder);
                                    OrgFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                                    FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                    if (!String.IsNullOrEmpty(OrgFilePath))
                                    {
                                        File.Copy(OrgFilePath, OrgFolder + "\\" + FileName, true);
                                    }
                                }
                                #endregion
                            }
                        }
                        #endregion

                        #region Get GraphicElements File
                        xpItr_GE = xpNav_FormatXml.Select("FORMATCOLLECTION/FORMAT/FORMAT_GRAPHICELEMENT/GRAPHICELEMENT");
                        while (xpItr_GE.MoveNext())
                        {
                            GEID = xpItr_GE.Current.SelectSingleNode("GRAPHICELEMENTID").Value;
                            TagName = xpItr_GE.Current.SelectSingleNode("GRAPHICTAGNAME").Value;
                            FileSystemXML = xpItr_GE.Current.SelectSingleNode("FILESYSTEM").OuterXml;
                            ALFile objFile = new ALFile(UserID, Language, TenantName);
                            ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                            ObjectID = ObjectID.Replace("<OBJECTID>", "");
                            FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                            FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                            ErrorXML = objFile.GetFile(ref FileSystemXML);
                            if (ErrorXML != null)
                            {

                            }
                            else
                            {
                                xpDoc_FileSystemXml = new XPathDocument(new StringReader(FileSystemXML));
                                xpNav_FileSystemXml = xpDoc_FileSystemXml.CreateNavigator();

                                #region Processing for GE Original file
                                xpNav_File = xpNav_FileSystemXml.SelectSingleNode("FILESYSTEM/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                                if (xpNav_File != null)
                                {
                                    OrgFilePath = xpNav_File.SelectSingleNode("PATH").Value;
                                    FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                    if (!String.IsNullOrEmpty(OrgFilePath))
                                    {
                                        OrgGEFolder = FormatIDFolder + "\\GE_" + GEID + "\\Original";
                                        Directory.CreateDirectory(OrgGEFolder);
                                        File.Copy(OrgFilePath, OrgGEFolder + "\\" + FileName, true);
                                    }
                                }
                                #endregion
                            }
                        }
                        #endregion

                        File.WriteAllText(FormatIDFolder + "\\FormatInfo.xml", sbCacheFormatXml.ToString());
                        #endregion
                    }
                }
                else
                {
                    objErrorCommon.AppendError("APENGINE_FORMATCACHE_CREATECACHEFORFORMAT_GETFORMATDETAILS_FAILED", this.GetType().ToString(), "CreateCacheForFormat", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }

                return ErrorXML;

            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_FORMATCACHE_CREATECACHEFORFORMAT_EXCEPTION", this.GetType().ToString(), "CreateCacheForFormat", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="FormatID"></param>
        /// <returns></returns>
        public bool ValidateFormatCache(String FormatID)
        {
            #region Member Variables
            String FormatCachePath = null;
            String GEID = null;
            String FileName = null;
            bool CheckExists = false;
            XPathDocument xpDoc_FormatInfo = null;
            XPathNavigator xpNav_FormatInfo = null;
            XPathNavigator xpNav_File = null;
            XPathNodeIterator xpItr_GE = null;
            XPathNodeIterator xpItr_AdditionalFiles = null;
            #endregion

            FormatCachePath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
            if (Directory.Exists(FormatCachePath))
            {
                if (File.Exists(FormatCachePath + "\\FormatInfo.xml"))
                {
                    xpDoc_FormatInfo = new XPathDocument(FormatCachePath + "\\FormatInfo.xml");
                    xpNav_FormatInfo = xpDoc_FormatInfo.CreateNavigator();

                    #region Validate GraphicElement files
                    xpItr_GE = xpNav_FormatInfo.Select("FORMATDETAILS/GRAPHICELEMENTCOLLECTION/GRAPHICELEMENT");
                    if (xpItr_GE.Count > 0)
                    {
                        while (xpItr_GE.MoveNext())
                        {
                            if (xpItr_GE.Current.SelectSingleNode("GRAPHICELEMENTID") != null)
                            {
                                GEID = xpItr_GE.Current.SelectSingleNode("GRAPHICELEMENTID").Value;
                            }
                            if (xpItr_GE.Current.SelectSingleNode("FILESYSTEM/FILE/DISPLAYNAME") != null)
                            {
                                FileName = xpItr_GE.Current.SelectSingleNode("FILESYSTEM/FILE/DISPLAYNAME").Value;
                            }
                            if (!String.IsNullOrEmpty(GEID) && !String.IsNullOrEmpty(FileName))
                            {
                                CheckExists = ValidateGraphicElementFile(TenantName, FormatID, GEID, FileName);
                                if (!CheckExists)
                                {
                                    return false;
                                }
                            }

                        }
                    }
                    #endregion

                    #region Validate Format Original file
                    xpNav_File = xpNav_FormatInfo.SelectSingleNode("FORMATDETAILS/FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                    if (xpNav_File != null)
                    {
                        FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                        if (!File.Exists(FormatCachePath + "\\Original\\" + FileName))
                        {
                            return false;
                        }
                    }
                    #endregion

                    #region Validate Additional Files
                    xpItr_AdditionalFiles = xpNav_FormatInfo.Select("FORMATDETAILS/FORMAT/FILESYSTEM/BINDER/FILE");
                    if (xpItr_AdditionalFiles.Count > 0)
                    {
                        while (xpItr_AdditionalFiles.MoveNext())
                        {
                            FileName = xpItr_AdditionalFiles.Current.SelectSingleNode("DISPLAYNAME").Value;
                            if (!File.Exists(FormatCachePath + "\\AdditionalFiles\\" + FileName))
                            {
                                return false;
                            }
                        }
                    }
                    #endregion

                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        /// <alsummary>
        ///     <createddate>9 December 2008</createddate>
        ///     <author>Bhavana Chavan</author>
        /// </alsummary>
        /// <param name="ArtworkID"></param>
        /// <returns></returns>
        public bool ValidateArtworkCacheWithFormatCache(String ArtworkID, String FormatID)
        {
            #region Member Variables
            String FormatCachePath = null;            
            #endregion

            ArtworkCachePath = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache);
            FormatCachePath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
            if (Directory.Exists(ArtworkCachePath))
            {
                if (File.Exists(ArtworkCachePath + "\\Input\\FormatInfo.xml"))
                {
                    if (File.Exists(FormatCachePath + "\\FormatInfo.xml"))
                    {
                        if (ValidateFormatInfoXml(ArtworkID, FormatID))
                        {
                            if (ValidateArtworkJob(ArtworkID))
                            {
                                return true;
                            }
                            else
                                return false;
                        }
                        else
                            return false;
                    }
                    return false;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        public static bool ValidateGraphicElementFile(String TenantName, String FormatID, String GraphicElementID, String FileName)
        {
            String GEFolder = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache) + "\\GE_" + GraphicElementID + "\\Original\\";
            if (File.Exists(GEFolder + FileName))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public String GetFormatID(String TenantName, String ArtworkID)
        {
            String FormatXML = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache) + "\\Input\\FormatInfo.xml";

            if (!File.Exists(FormatXML))
                PrepareArtworkCache(ArtworkID, "EDIT");

            XmlReader objReader = XmlReader.Create(FormatXML);
            objReader.MoveToContent();
            if (!objReader.ReadToDescendant("FORMAT"))
                return null;

            if (!objReader.ReadToDescendant("FORMATID"))
                return null;

            String FormatID = objReader.ReadElementContentAsString();
            objReader.Close();
            return FormatID;
        }

        public ERRORXML DeleteArtworkCache(String ArtworkIDList)
        {
            ERRORXML ErrorXML = null;
            String[] ArtworkIDArr = ArtworkIDList.Split(';');
            objErrorCommon = new ALErrorCommon();
            Context ContextObj = null;
            
            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.DeleteArtworkCache</APINAME><ARTWORKIDLIST>{0}</ARTWORKIDLIST></ADDITIOALINFO>", ArtworkIDList, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                for (int Index = 0; Index < ArtworkIDArr.Length; Index++)
                {
                    if (ArtworkIDArr[Index] != "" && ArtworkIDArr[Index] != null)
                    {
                        ArtworkCachePath = GetCacheFolderPath(TenantName, ArtworkIDArr[Index], CacheType.ArtworkCache);
                        if (Directory.Exists(ArtworkCachePath))
                        {
                            //Directory.Delete(ArtworkCachePath, true);
                            SASMOMDirectory.DeleteFolder(ArtworkCachePath, true, ContextObj);
                        }
                    }
                }
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_APCACHE_DELETEARTWORKCACHE_EXCEPTION", this.GetType().ToString(), "DeleteArtworkCache", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        public ERRORXML DeleteFormatCache(String FormatIDList)
        {
            ERRORXML ErrorXML = null;
            String FormatCachePath = null;
            String[] FormatIDArr = FormatIDList.Split(';');
            Context ContextObj = null;

            objErrorCommon = new ALErrorCommon();

            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.DeleteFormatCache</APINAME><FORMATIDLIST>{0}</FORMATIDLIST></ADDITIOALINFO>", FormatIDList, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                for (int Index = 0; Index < FormatIDArr.Length; Index++)
                {
                    if (FormatIDArr[Index] != "" && FormatIDArr[Index] != null)
                    {
                        FormatCachePath = GetCacheFolderPath(TenantName, FormatIDArr[Index], CacheType.FormatCache);
                        if (Directory.Exists(FormatCachePath))
                        {
                            //Directory.Delete(FormatCachePath, true);
                            SASMOMDirectory.DeleteFolder(FormatCachePath, true, ContextObj);
                        }
                    }
                }
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_APCACHE_DELETEFORMATCACHE_EXCEPTION", this.GetType().ToString(), "DeleteFormatCache", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        public ERRORXML DeleteGraphicElementFromFormatCache(String GraphicElementIDList, String FormatID)
        {
            ERRORXML ErrorXML = null;
            String FormatCachePath = null;
            XmlDocument XmlDoc_CacheFormatInfoXML = null;
            String[] GraphicElementIDArr = GraphicElementIDList.Split(';');
            Context ContextObj = null;
            objErrorCommon = new ALErrorCommon();

            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerEngine.ArtworkProducerCache.DeleteGraphicElementFromFormatCache</APINAME><GRAPHICELEMENTIDLIST>{0}</GRAPHICELEMENTIDLIST><FORMATID>{2}</FORMATID></ADDITIOALINFO>", GraphicElementIDList, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), FormatID);

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                if (!String.IsNullOrEmpty(FormatID))
                {
                    FormatCachePath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
                    XmlDoc_CacheFormatInfoXML = new XmlDocument();
                    XmlDoc_CacheFormatInfoXML.Load(FormatCachePath + "\\FormatInfo.xml");
                }
                for (int Index = 0; Index < GraphicElementIDArr.Length; Index++)
                {
                    if (GraphicElementIDArr[Index] != "" && GraphicElementIDArr[Index] != null)
                    {
                        if (Directory.Exists(FormatCachePath + "\\GE_" + GraphicElementIDArr[Index]))
                        {
                            //Directory.Delete(FormatCachePath + "\\GE_" + GraphicElementIDArr[Index], true);
                            SASMOMDirectory.DeleteFolder(FormatCachePath + "\\GE_" + GraphicElementIDArr[Index], true, ContextObj);
                        }
                        if (XmlDoc_CacheFormatInfoXML != null)
                        {
                            XmlNode GENode = XmlDoc_CacheFormatInfoXML.SelectSingleNode("FORMATDETAILS/GRAPHICELEMENTCOLLECTION/GRAPHICELEMENT[GRAPHICELEMENTID/text()='" + GraphicElementIDArr[Index] + "']");
                            if (GENode != null)
                            {
                                XmlDoc_CacheFormatInfoXML.SelectSingleNode("FORMATDETAILS/GRAPHICELEMENTCOLLECTION").RemoveChild(GENode);
                            }
                        }
                    }
                }
                XmlDoc_CacheFormatInfoXML.Save(FormatCachePath + "\\FormatInfo.xml");
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_APCACHE_DELETEGRAPHICELEMENTFROMFORMATCACHE_EXCEPTION", this.GetType().ToString(), "DeleteGraphicElementFromFormatCache", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        public ERRORXML UpdateArtworkDetails(String ArtworkXML, String ArtworkID)
        {
            ERRORXML ErrorXML = null;
            String str_ArtworkXMLPath = GetArtworkValuesXMLPath(TenantName, ArtworkID);
            bool bArtworkInfoCreated = false;
            if (!File.Exists(str_ArtworkXMLPath))
            {
                ErrorXML = PrepareArtworkCache(ArtworkID, "EDIT");
                return ErrorXML;
            }

            XmlDocument xDoc_CacheXML = null;
            XPathDocument xpDoc_ArtworkXML = new XPathDocument(new StringReader(ArtworkXML));
            XPathNavigator xpNav_RootArtwork = xpDoc_ArtworkXML.CreateNavigator();

            XPathNavigator xpNav_Temp = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORKNAME");
            if (xpNav_Temp != null)
            {
                if (xDoc_CacheXML == null)
                {
                    xDoc_CacheXML = new XmlDocument();
                    xDoc_CacheXML.Load(str_ArtworkXMLPath);
                }
                XmlElement xEle_ArtworkName = (XmlElement)xDoc_CacheXML.SelectSingleNode("ARTWORK/ARTWORKNAME");
                if (xEle_ArtworkName != null)
                {
                    xEle_ArtworkName.InnerXml = xpNav_Temp.InnerXml;
                }
            }

            xpNav_Temp = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORKDESCRIPTION");
            if (xpNav_Temp != null)
            {
                if (xDoc_CacheXML == null)
                {
                    xDoc_CacheXML = new XmlDocument();
                    xDoc_CacheXML.Load(str_ArtworkXMLPath);
                }

                XmlElement xEle_ArtworkDesc = (XmlElement)xDoc_CacheXML.SelectSingleNode("ARTWORK/ARTWORKDESCRIPTION");
                if (xEle_ArtworkDesc != null)
                {
                    xEle_ArtworkDesc.InnerXml = xpNav_Temp.InnerXml;
                }
            }

            xpNav_Temp = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/CUSTOM_HEIGHT");
            if (xpNav_Temp != null)
            {
                if (xDoc_CacheXML == null)
                {
                    xDoc_CacheXML = new XmlDocument();
                    xDoc_CacheXML.Load(str_ArtworkXMLPath);
                }

                XmlElement xEle_ArtworkInfo = (XmlElement)xDoc_CacheXML.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/CUSTOM_HEIGHT");
                if (xEle_ArtworkInfo != null)
                {
                    xEle_ArtworkInfo.InnerXml = xpNav_Temp.InnerXml;
                }
                else
                {
                    CreateArtworkInfoNode(xpNav_RootArtwork, ref xDoc_CacheXML);
                    bArtworkInfoCreated = true;
                }
            }

            if (!bArtworkInfoCreated)
            {
                xpNav_Temp = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/CUSTOM_WIDTH");
                if (xpNav_Temp != null)
                {
                    if (xDoc_CacheXML == null)
                    {
                        xDoc_CacheXML = new XmlDocument();
                        xDoc_CacheXML.Load(str_ArtworkXMLPath);
                    }

                    XmlElement xEle_ArtworkInfo = (XmlElement)xDoc_CacheXML.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/CUSTOM_WIDTH");
                    if (xEle_ArtworkInfo != null)
                    {
                        xEle_ArtworkInfo.InnerXml = xpNav_Temp.InnerXml;
                    }
                    else
                    {
                        CreateArtworkInfoNode(xpNav_RootArtwork, ref xDoc_CacheXML);
                        bArtworkInfoCreated = true;
                    }
                }
            }

            if (!bArtworkInfoCreated)
            {
                xpNav_Temp = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/ARTWORKDIMENSIONUNIT");
                if (xpNav_Temp != null)
                {
                    if (xDoc_CacheXML == null)
                    {
                        xDoc_CacheXML = new XmlDocument();
                        xDoc_CacheXML.Load(str_ArtworkXMLPath);
                    }

                    XmlElement xEle_ArtworkInfo = (XmlElement)xDoc_CacheXML.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/ARTWORKDIMENSIONUNIT");
                    if (xEle_ArtworkInfo != null)
                    {
                        xEle_ArtworkInfo.InnerXml = xpNav_Temp.InnerXml;
                    }
                    else
                    {
                        CreateArtworkInfoNode(xpNav_RootArtwork, ref xDoc_CacheXML);
                        bArtworkInfoCreated = true;
                    }
                }
            }

            if (xDoc_CacheXML != null)
            {
                xDoc_CacheXML.Save(str_ArtworkXMLPath);
            }

            return null;
        }

        public ERRORXML GetAllTagValues(String ArtworkID, out String TagValues)
        {
            StringBuilder sbTagValues = new StringBuilder();
            TagValues = null;
            try
            {
                sbTagValues.Append("<ARTWORK>");
                String FilePath = GetArtworkValuesXMLPath(TenantName, ArtworkID);
                if (!File.Exists(FilePath))
                    PrepareArtworkCache(ArtworkID, "EDIT");

                XPathDocument xpDoc_CacheXML = new XPathDocument(FilePath);
                XPathNavigator xpNav_Root = xpDoc_CacheXML.CreateNavigator().SelectSingleNode("ARTWORK");
                XPathNodeIterator xpIter_Tags = xpNav_Root.Select("ARTWORKVALUES/TEXTTAGS/TAG");

                if (xpIter_Tags.Count > 0)
                {
                    sbTagValues.Append("<TEXTTAGS>");
                }
                while (xpIter_Tags.MoveNext())
                {
                    sbTagValues.AppendFormat("<TAG NAME=\"{0}\" ISHIDDEN=\"{1}\" ID='{2}' AUTOPOPULATE='{3}'>", xpIter_Tags.Current.GetAttribute("NAME", ""), xpIter_Tags.Current.GetAttribute("ISHIDDEN", ""), xpIter_Tags.Current.GetAttribute("ID", ""), xpIter_Tags.Current.GetAttribute("AUTOPOPULATE", ""));                    
                    sbTagValues.Append(xpIter_Tags.Current.InnerXml);
                    sbTagValues.Append("</TAG>");
                }
                if (xpIter_Tags.Count > 0)
                {
                    sbTagValues.Append("</TEXTTAGS>");
                }

                xpIter_Tags = xpNav_Root.Select("ARTWORKVALUES/GRAPHICTAGS/TAG");

                if (xpIter_Tags.Count > 0)
                {
                    sbTagValues.Append("<GRAPHICTAGS>");
                }
                while (xpIter_Tags.MoveNext())
                {                    
                    sbTagValues.AppendFormat("<TAG NAME=\"{0}\" ISHIDDEN=\"{1}\" ID='{2}' AUTOPOPULATE='{3}'>", xpIter_Tags.Current.GetAttribute("NAME", ""), xpIter_Tags.Current.GetAttribute("ISHIDDEN", ""), xpIter_Tags.Current.GetAttribute("ID", ""), xpIter_Tags.Current.GetAttribute("AUTOPOPULATE", ""));
                    sbTagValues.Append(xpIter_Tags.Current.InnerXml);
                    sbTagValues.Append("</TAG>");
                }
                if (xpIter_Tags.Count > 0)
                {
                    sbTagValues.Append("</GRAPHICTAGS>");
                }

                sbTagValues.Append("</ARTWORK>");

                TagValues = sbTagValues.ToString();

                return null;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_APCACHE_GETALLTAGVALUE_EXCEPTION", this.GetType().ToString(), "GetAllTagValue", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        public static bool IsArtworkPartOfBatch(ulong UserID, String Language, String TenantName, String ArtworkID, out String BatchJobID)
        {
            String ValuesXMLPath = GetArtworkValuesXMLPath(TenantName, ArtworkID);
            BatchJobID = null;
            if (File.Exists(ValuesXMLPath))
            {
                XmlReader xReader = XmlReader.Create(ValuesXMLPath);
                xReader.MoveToContent();
                if (xReader.ReadToDescendant("BATCHJOBID"))
                {
                    BatchJobID = xReader.ReadElementContentAsString();
                    return true;
                }
                xReader.Close();
            }
            else
            {
                StringBuilder sbSearch = new StringBuilder();
                sbSearch.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ARTWORKID</EXPRESSION>
                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>ARTWORKID</FIELDNAME>
                                                    <ALIAS>ARTWORKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='ARTWORKJOB_ARTWORK'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>JOBID</FIELDNAME>
                                                        <ALIAS>JOBID</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                            </RELATIONSHIP>
                                        </TARGETENTITY>
                                      </SEARCH>", ArtworkID);

                String OutputXML = null;
                OMInterface objOMI = new OMInterface(UserID, Language, TenantName);
                ERRORXML ErrorXML = objOMI.GetHierarchialData(sbSearch.ToString(), ref OutputXML);
                if (OutputXML != null)
                    BatchJobID = ALCommon.GetInnerValue(ref OutputXML, "<JOBID>", "</JOBID>");
                if (String.IsNullOrEmpty(BatchJobID))
                    return false;
                else
                    return true;
            }
            return false;
        }

        public static bool IsBookArtwork(ulong UserID, String Language, String TenantName, String ArtworkID, out String BookID)
        {
            String ValuesXMLPath    = GetArtworkValuesXMLPath(TenantName, ArtworkID);
            BookID                  = null;
            if (File.Exists(ValuesXMLPath))
            {
                XmlReader xReader   = XmlReader.Create(ValuesXMLPath);
                xReader.MoveToContent();
                if (xReader.ReadToDescendant("BOOKID"))
                {
                    BookID = xReader.ReadElementContentAsString();
                    xReader.Close();
                    return true;
                }
                xReader.Close();
            }
            else
            {
                StringBuilder sbSearch = new StringBuilder();
                sbSearch.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ARTWORKID</EXPRESSION>
                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>ARTWORKID</FIELDNAME>
                                                    <ALIAS>ARTWORKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='SPREAD_ARTWORK'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>SPREADID</FIELDNAME>
                                                        <ALIAS>SPREADID</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='BOOK_SPREAD'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>BOOKID</FIELDNAME>
                                                            <ALIAS>BOOKID</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </RELATIONSHIP>
                                        </TARGETENTITY>
                                      </SEARCH>", ArtworkID);

                String      OutputXML   = null;
                OMInterface objOMI      = new OMInterface(UserID, Language, TenantName);
                ERRORXML    ErrorXML    = objOMI.GetHierarchialData(sbSearch.ToString(), ref OutputXML);
                if (OutputXML != null)
                    BookID = ALCommon.GetInnerValue(ref OutputXML, "<BOOKID>", "</BOOKID>");
                if (String.IsNullOrEmpty(BookID))
                    return false;
                else
                    return true;
            }
            return false;
        }

        #endregion

        private void CopyFiles(String SourceFolderPath, String DestinationFolderPath)
        {
            #region Member Variables
            FileInfo[] AllFilesInDir = null;
            int FileCount = 0;
            int SubDirCount = 0;
            #endregion

            if (Directory.Exists(SourceFolderPath))
            {
                DirectoryInfo DirObj = new DirectoryInfo(SourceFolderPath);
                DirectoryInfo[] SubDirObj = DirObj.GetDirectories();
                #region Copy Files to Artwork Folder
                AllFilesInDir = DirObj.GetFiles();

                for (FileCount = 0; FileCount < AllFilesInDir.Length; FileCount++)
                {
                    try
                    {
                        //if (File.Exists(DestinationFolderPath + "\\" + AllFilesInDir[FileCount]))
                        //{
                        //    File.Delete(DestinationFolderPath + "\\" + AllFilesInDir[FileCount]);
                        //}
                        File.Copy(AllFilesInDir[FileCount].FullName, DestinationFolderPath + "\\" + AllFilesInDir[FileCount], true);
                    }
                    catch (Exception ExpObj)
                    {
                        throw ExpObj;
                    }

                }
                #endregion

                #region Check SubDirectories of Current Folder
                for (SubDirCount = 0; SubDirCount < SubDirObj.Length; SubDirCount++)
                {
                    String SubDirPath = SubDirObj[SubDirCount].FullName;
                    CopyFiles(SubDirPath, DestinationFolderPath);
                }
                #endregion
            }
        }

        private ERRORXML ArtworkCacheForCreateMode(String ArtworkID)
        {
            #region Member Variables
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;
            String ArtworkXML = null;
            String FormatID = null;
            String strArtworkDesc = null;
            String strArtworkName = null;
            String FileName = null;
            StringBuilder sbSearchXml = null;
            StringBuilder ArtworkCacheXml = null;
            XPathDocument xpDoc_ArtworkXml = null;
            XPathNavigator xpNav_ArtworkXml = null;
            XPathNavigator xpNav_File = null;
            XPathNavigator xpNav_ArtworkInfo = null;
            XPathNodeIterator xpItr_TextTags = null;
            XPathNodeIterator xpItr_GraphicTags = null;
            #endregion

            try
            {
                #region Get all artwork details
                sbSearchXml = new StringBuilder(16);
                sbSearchXml.Append(@"<SEARCH>
                            <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                            <TARGETENTITY>
                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                    <FILTERLIST>
                                        <FILTER>
                                            <EXPRESSION>ARTWORKID</EXPRESSION>
                                             <OPERATOR>EQUALTO</OPERATOR>
                                            <EXPRESSIONVALUE>" + ArtworkID + @"</EXPRESSIONVALUE>
                                            <FILTERID>1</FILTERID>
                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                        </FILTER>
                                     </FILTERLIST>
                                     <SELECTLIST>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKID</FIELDNAME>
                                            <ALIAS>ARTWORKID</ALIAS>
                                        </EXPRESSION>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKNAME</FIELDNAME>
                                            <ALIAS>ARTWORKNAME</ALIAS>
                                        </EXPRESSION>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKDESCRIPTION</FIELDNAME>
                                            <ALIAS>ARTWORKDESCRIPTION</ALIAS>
                                        </EXPRESSION>
                                    </SELECTLIST>
                                    <RELATIONSHIP NAME='ARTWORKJOB_ARTWORK'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>JOBID</FIELDNAME>
                                                <ALIAS>JOBID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                    </RELATIONSHIP>
                                    <RELATIONSHIP NAME='SPREAD_ARTWORK'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>SPREADID</FIELDNAME>
                                                <ALIAS>SPREADID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='BOOK_SPREAD'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>BOOKID</FIELDNAME>
                                                    <ALIAS>BOOKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </RELATIONSHIP>
                                     </RELATIONSHIP>
                                     <RELATIONSHIP NAME='ARTWORK_ARTWORKINFO'>                                                            
                                     </RELATIONSHIP>
                                     <RELATIONSHIP NAME='FORMAT_ARTWORK'>
                                        <RELATIONSHIP NAME='FORMAT_TEXTELEMENT'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                    <ALIAS>TEXTELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                                    <ALIAS>TEXTTAGNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ISHIDDEN</FIELDNAME>
                                                    <ALIAS>ISHIDDEN</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                    <ALIAS>AUTOPOPULATE</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='TEXTELEMENT_OPTION'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>OPTIONVALUE</FIELDNAME>
                                                        <ALIAS>OPTIONVALUE</ALIAS>
                                                    </EXPRESSION>                                                    
                                                </SELECTLIST>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>ISDEFAULTOPTION</EXPRESSION>
                                                         <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
                                                        <FILTERID>1</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                 </FILTERLIST>                                                                                                     
                                            </RELATIONSHIP>                                                                                                     
                                        </RELATIONSHIP> 
                                        <RELATIONSHIP NAME='FORMAT_GRAPHICELEMENT'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                    <ALIAS>AUTOPOPULATE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                    <ALIAS>ORIGINALFILE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ALLOWEXTERNALFILEUPLOAD</FIELDNAME>
                                                    <ALIAS>ALLOWEXTERNALFILEUPLOAD</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ALLOWASSETSELECTION</FIELDNAME>
                                                    <ALIAS>ALLOWASSETSELECTION</ALIAS>
                                                </EXPRESSION>                                                
                                            </SELECTLIST>                                                                                                        
                                        </RELATIONSHIP>                                                              
                                     </RELATIONSHIP>
                             </TARGETENTITY>
                          </SEARCH>");
                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref ArtworkXML);
                #endregion

                if (ErrorXML == null)
                {
                    FormatID = ALCommon.GetInnerValue(ref ArtworkXML, "<FORMATID>", "</FORMATID>");

                    #region Validate Format cache

                    bool bretVal = ValidateFormatCache(FormatID);
                    if (!bretVal)
                    {
                        PrepareFormatCache(FormatID);
                    }

                    #endregion


                    if (!String.IsNullOrEmpty(FormatID))
                    {
                        Directory.CreateDirectory(ArtworkCachePath + "\\Input");
                        #region Copy FormatCache to ArtworkCache
                        ErrorXML = CopyFormatCacheToArtworkCache(FormatID, ArtworkID);
                        #endregion

                        if (ErrorXML == null)
                        {
                            #region Prepare Artwork xml
                            xpDoc_ArtworkXml = new XPathDocument(new StringReader(ArtworkXML));
                            xpNav_ArtworkXml = xpDoc_ArtworkXml.CreateNavigator();
                            if (xpNav_ArtworkXml != null)
                            {
                                ArtworkCacheXml = new StringBuilder(16);
                                ArtworkCacheXml.Append("<ARTWORK>");
                                strArtworkName = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORKNAME").OuterXml;
                                strArtworkDesc = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORKDESCRIPTION").OuterXml;
                                ArtworkCacheXml.AppendFormat(@"<ARTWORKID>{0}</ARTWORKID>{1}{2}", ArtworkID, strArtworkName, strArtworkDesc);
                                
                                XPathNavigator xpNav_BatchJob = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORKJOB_ARTWORK/ARTWORKJOB/JOBID");
                                if (xpNav_BatchJob != null)
                                    ArtworkCacheXml.AppendFormat(@"<BATCHJOBID>{0}</BATCHJOBID>", xpNav_BatchJob.InnerXml);

                                XPathNavigator xpNav_BookID     = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/SPREAD_ARTWORK/SPREAD/BOOK_SPREAD/BOOK/BOOKID");
                                if (xpNav_BookID!=null)
                                    ArtworkCacheXml.AppendFormat(@"<BOOKID>{0}</BOOKID>", xpNav_BookID.Value);

                                #region Append ArtworkInfo xml
                                xpNav_ArtworkInfo = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORK_ARTWORKINFO");
                                if (xpNav_ArtworkInfo != null)
                                {
                                    ArtworkCacheXml.Append(xpNav_ArtworkInfo.OuterXml);
                                }
                                #endregion

                                #region Append Original file path
                                xpNav_File = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                                if (xpNav_File != null)
                                {
                                    FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                    if (File.Exists(ArtworkCachePath + "\\Input\\" + FileName))
                                    {
                                        ArtworkCacheXml.AppendFormat("<ORIGINALFILEPATH>{0}</ORIGINALFILEPATH>", ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName));
                                    }
                                }
                                #endregion

                                #region Append Text Tags
                                xpItr_TextTags = xpNav_ArtworkXml.Select("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT/FORMAT_TEXTELEMENT/TEXTELEMENT");
                                if (xpItr_TextTags.Count > 0)
                                {
                                    ArtworkCacheXml.Append("<ARTWORKVALUES><TEXTTAGS>");
                                    while (xpItr_TextTags.MoveNext())
                                    {
                                        String TextTagName = xpItr_TextTags.Current.SelectSingleNode("TEXTTAGNAME").InnerXml;
                                        ALSpecialCharHandling.EncodeSpecialCharacters(ref TextTagName);
                                        XPathNavigator xpNav_TEOption = xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENT_OPTION/TEXTELEMENTOPTION/OPTIONVALUE");
                                        String TEOptionValue = "";
                                        if (xpNav_TEOption != null)
                                        {
                                            TEOptionValue = xpNav_TEOption.InnerXml;
                                        }                                        
                                        ArtworkCacheXml.AppendFormat("<TAG NAME='{0}' ID='{1}' ISHIDDEN='{2}' AUTOPOPULATE='{4}'><VALUE>{3}</VALUE></TAG>", TextTagName, xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENTID").InnerXml, xpItr_TextTags.Current.SelectSingleNode("ISHIDDEN").InnerXml, TEOptionValue, xpItr_TextTags.Current.SelectSingleNode("AUTOPOPULATE").InnerXml);
                                    }
                                    ArtworkCacheXml.Append("</TEXTTAGS>");
                                }
                                #endregion

                                #region Append Graphic Tags
                                xpItr_GraphicTags = xpNav_ArtworkXml.Select("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT/FORMAT_GRAPHICELEMENT/GRAPHICELEMENT");
                                if (xpItr_GraphicTags.Count > 0)
                                {
                                    if (ArtworkCacheXml.ToString().Contains("<ARTWORKVALUES>"))
                                    {
                                        ArtworkCacheXml.Append("<GRAPHICTAGS>");
                                    }
                                    else
                                    {
                                        ArtworkCacheXml.Append("<ARTWORKVALUES><GRAPHICTAGS>");
                                    }
                                    while (xpItr_GraphicTags.MoveNext())
                                    {
                                        xpNav_File = xpItr_GraphicTags.Current.SelectSingleNode("FILESYSTEM/FILE");
                                        if (xpNav_File != null)
                                        {
                                            FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                            if (File.Exists(ArtworkCachePath + "\\Input\\" + FileName))
                                            {
                                                String TagName = xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICTAGNAME").Value;
                                                ALSpecialCharHandling.EncodeSpecialCharacters(ref TagName);                                                
                                                ArtworkCacheXml.AppendFormat("<TAG NAME='{0}' ID='{1}' VALUETYPE='DEFAULT' STATUS='DONE' AUTOPOPULATE='{3}'>{2}</TAG>", TagName, xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml, ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName), xpItr_GraphicTags.Current.SelectSingleNode("AUTOPOPULATE").InnerXml);
                                            }
                                        }
                                    }
                                    ArtworkCacheXml.Append("</GRAPHICTAGS>");
                                }
                                #endregion

                                if (ArtworkCacheXml.ToString().Contains("<ARTWORKVALUES>"))
                                {
                                    ArtworkCacheXml.Append("</ARTWORKVALUES>");
                                }
                                ArtworkCacheXml.Append("</ARTWORK>");
                                if (!String.IsNullOrEmpty(ArtworkCacheXml.ToString()))
                                {
                                    File.WriteAllText(ArtworkCachePath + "\\Artwork.xml", ArtworkCacheXml.ToString());
                                }
                            }
                            #endregion
                        }
                        else
                        {
                            objErrorCommon.AppendError("APENGINE_APCACHE_ARTWORKCACHEFORCREATEMODE_COPYFORMATCACHETOARTWORKCACHE_FAILED", this.GetType().ToString(), "CopyFormatCacheToArtworkCache", null, ErrorXML);
                            ErrorXML = objErrorCommon.ErrorCollectionXML;
                        }
                    }
                }
                else
                {
                    objErrorCommon.AppendError("APENGINE_APCACHE_ARTWORKCACHEFORCREATEMODE_GETARTWORKDETAILS_FAILED", this.GetType().ToString(), "ArtworkCacheForCreateMode", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_FORMATCACHE_ARTWORKCACHEFORCREATEMODE_EXCEPTION", this.GetType().ToString(), "ArtworkCacheForCreateMode", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }

        }

        public ERRORXML ArtworkCacheForEditMode(String ArtworkID)
        {
            #region Member Variables
            ERRORXML ErrorXML = null;
            OMInterface OMIObj = null;
            String ArtworkXML = null;
            String FormatID = null;
            String GEID = null;
            String ObjectID = null;
            String GEFilePath = null;
            String DAFilePath = null;
            String FileSystemXML = null;
            String FormatXML = null;
            String FormatCachePath = null;
            String FileName = null;
            String strArtworkName = null;
            String strArtworkDesc = null;
            int TECnt = 0;
            int GECnt = 0;
            StringBuilder sbSearchXml = null;
            StringBuilder sbNOTINFilterTE = null;
            StringBuilder sbNOTINFilterGE = null;
            StringBuilder sbTEIDForNotIN = null;
            StringBuilder sbGEIDForNotIN = null;
            StringBuilder sbFormatSearchXML = null;
            StringBuilder ArtworkCacheXml = null;
            XPathDocument xpDoc_ArtworkXml = null;
            XPathDocument xpDoc_FormatXml = null;
            XPathNavigator xpNav_ArtworkXml = null;
            XPathNavigator xpNav_FormatXml = null;
            XPathNavigator xpNav_ArtworkInfo = null;
            XPathNavigator xpNav_GEActual = null;
            XPathNodeIterator xpItr_GEActual = null;
            XPathNavigator xpNav_File = null;
            XPathNodeIterator xpItr_TextTags = null;
            XPathNodeIterator xpItr_GraphicTags = null;
            XPathNodeIterator xpItr_DigitalAsset = null;
            XPathNavigator  xpNav_DADimensions      = null;
            String          str_processedFile       = null;
            String          str_FileFormat          = null;
            String          str_FilePresentOn       = null;
            StringBuilder   sb_GetFileCollectionXML = new StringBuilder(100);
            FileTransferInfrastructure ObjFT_MS     = null;
            #endregion

            sbTEIDForNotIN              = new StringBuilder(16);
            sbGEIDForNotIN              = new StringBuilder(16);
            ObjFT_MS                    = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
            ObjFT_MS.ToUseServiceURL    = ALCommon.GetWebServiceURL("ALMEDIASERVERMTOMWS");
            try
            {
                #region Get Artwork Details
                sbSearchXml = new StringBuilder(16);
                sbSearchXml.Append(@"<SEARCH>
                            <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                            <TARGETENTITY>
                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                    <FILTERLIST>
                                        <FILTER>
                                            <EXPRESSION>ARTWORKID</EXPRESSION>
                                             <OPERATOR>EQUALTO</OPERATOR>
                                            <EXPRESSIONVALUE>" + ArtworkID + @"</EXPRESSIONVALUE>
                                            <FILTERID>1</FILTERID>
                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                        </FILTER>
                                     </FILTERLIST>
                                     <SELECTLIST>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKID</FIELDNAME>
                                            <ALIAS>ARTWORKID</ALIAS>
                                        </EXPRESSION>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKNAME</FIELDNAME>
                                            <ALIAS>ARTWORKNAME</ALIAS>
                                        </EXPRESSION>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKDESCRIPTION</FIELDNAME>
                                            <ALIAS>ARTWORKDESCRIPTION</ALIAS>
                                        </EXPRESSION>
                                    </SELECTLIST>
                                    <RELATIONSHIP NAME='ARTWORKJOB_ARTWORK'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>JOBID</FIELDNAME>
                                                <ALIAS>JOBID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                    </RELATIONSHIP>
                                    <RELATIONSHIP NAME='SPREAD_ARTWORK'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>SPREADID</FIELDNAME>
                                                <ALIAS>SPREADID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='BOOK_SPREAD'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>BOOKID</FIELDNAME>
                                                    <ALIAS>BOOKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </RELATIONSHIP>
                                     </RELATIONSHIP>
                                     <RELATIONSHIP NAME='ARTWORK_ARTWORKINFO'>                                                            
                                     </RELATIONSHIP>
                                     <RELATIONSHIP NAME='FORMAT_ARTWORK'>
                                        <SELECTLIST>
                                            <EXPRESSION>
	                                            <FIELDNAME>FORMATID</FIELDNAME>
	                                            <ALIAS>FORMATID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>FORMATTYPE</FIELDNAME>
                                                <ALIAS>FORMATTYPE</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                    </RELATIONSHIP>
                                     <RELATIONSHIP NAME='ARTWORK_TEXTELEMENTVALUE'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>TEXTELEMENTVALUEID</FIELDNAME>
                                                <ALIAS>TEXTELEMENTVALUEID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>TEXTELEMENTACTUALVALUE</FIELDNAME>
                                                <ALIAS>TEXTELEMENTACTUALVALUE</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                    <ALIAS>TEXTELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                                    <ALIAS>TEXTTAGNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ISHIDDEN</FIELDNAME>
                                                    <ALIAS>ISHIDDEN</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                    <ALIAS>AUTOPOPULATE</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </RELATIONSHIP>
                                    </RELATIONSHIP>
                                    <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENT'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                <ALIAS>GRAPHICELEMENTID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                <ALIAS>GRAPHICTAGNAME</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                <ALIAS>ORIGINALFILE</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                <ALIAS>AUTOPOPULATE</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='GRAPHICELEMENT_REPLACEDFILE' HIERARCHY='BOTTOMUP'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                    <ALIAS>ORIGINALFILE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                    <ALIAS>AUTOPOPULATE</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </RELATIONSHIP>
                                    </RELATIONSHIP>
                                    <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENTVALUEINFO'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>GRAPHICELEMENTVALUEID</FIELDNAME>
                                                <ALIAS>GRAPHICELEMENTVALUEID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                       <FILTERLIST>
                                            <FILTER>
                                                <EXPRESSION>NOIMAGE</EXPRESSION>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                
                                            </FILTER>
                                        </FILTERLIST>
                                        <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                    <ALIAS>AUTOPOPULATE</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST> 
                                        </RELATIONSHIP>
                                    </RELATIONSHIP>
                                    <RELATIONSHIP NAME='ARTWORK_DIGITALASSET'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                <ALIAS>DIGITALASSETID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='DIGITALASSET_CURRENTREVISION'>
                                            <RELATIONSHIP NAME='CONTENTREVISION_RENDITION'>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>RENDITIONTYPE</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>ORIGINAL</EXPRESSIONVALUE>
                                                        <FILTERID>1</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>ACTUALFILE</FIELDNAME>
                                                        <ALIAS>ACTUALFILE</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>RENDITIONTYPE</FIELDNAME>
                                                        <ALIAS>RENDITIONTYPE</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                            </RELATIONSHIP>
                                        </RELATIONSHIP>
                                         <!--<RELATIONSHIP NAME='GRAPHICELEMENT_DIGITALASSET'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                    <ALIAS>ORIGINALFILE</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>                                                                                                            
                                        </RELATIONSHIP>-->
                                        <RELATIONSHIP NAME='GRAPHICELEMENTVALUEINFO_DIGITALASSET'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>GRAPHICELEMENTVALUEID</FIELDNAME>
                                                    <ALIAS>GRAPHICELEMENTVALUEID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>DIMENSIONS</FIELDNAME>
                                                    <ALIAS>DIMENSIONS</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                           <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ARTWORKID</EXPRESSION>
                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>" + ArtworkID + @"</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    <RELATIONSHIPNAME>ARTWORK_GRAPHICELEMENTVALUEINFO</RELATIONSHIPNAME>
                                                </FILTER>
                                            </FILTERLIST>
                                            <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                        <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                        <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                        <ALIAS>ORIGINALFILE</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                        <ALIAS>AUTOPOPULATE</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST> 
                                            </RELATIONSHIP>
                                        </RELATIONSHIP>
                                    </RELATIONSHIP>
                             </TARGETENTITY>
                          </SEARCH>");
                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref ArtworkXML);
                #endregion

                if (ErrorXML == null)
                {
                    Directory.CreateDirectory(ArtworkCachePath + "\\Input");
                    FormatID = ALCommon.GetInnerValue(ref ArtworkXML, "<FORMATID>", "</FORMATID>");

                    #region Validate Format cache

                    bool bretVal = ValidateFormatCache(FormatID);
                    if (!bretVal)
                    {
                        PrepareFormatCache(FormatID);
                    }

                    #endregion

                    #region Prepare Artwork xml
                    xpDoc_ArtworkXml = new XPathDocument(new StringReader(ArtworkXML));
                    xpNav_ArtworkXml = xpDoc_ArtworkXml.CreateNavigator();
                    ArtworkCacheXml = new StringBuilder(16);
                    ArtworkCacheXml.Append("<ARTWORK>");
                    strArtworkName = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORKNAME").OuterXml;
                    strArtworkDesc = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORKDESCRIPTION").OuterXml;
                    ArtworkCacheXml.AppendFormat(@"<ARTWORKID>{0}</ARTWORKID>{1}{2}", ArtworkID, strArtworkName, strArtworkDesc);
                    
                    XPathNavigator xpNav_BatchJob = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORKJOB_ARTWORK/ARTWORKJOB/JOBID");
                    if (xpNav_BatchJob != null)
                        ArtworkCacheXml.AppendFormat(@"<BATCHJOBID>{0}</BATCHJOBID>", xpNav_BatchJob.InnerXml);

                    XPathNavigator xpNav_BookID    = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/SPREAD_ARTWORK/SPREAD/BOOK_SPREAD/BOOK/BOOKID");
                    if (xpNav_BookID!=null)
                        ArtworkCacheXml.AppendFormat(@"<BOOKID>{0}</BOOKID>", xpNav_BookID.Value);
                    #endregion

                    #region Append ArtworkInfo xml
                    xpNav_ArtworkInfo = xpNav_ArtworkXml.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORK_ARTWORKINFO");
                    if (xpNav_ArtworkInfo != null)
                    {
                        ArtworkCacheXml.Append(xpNav_ArtworkInfo.OuterXml);
                    }
                    #endregion

                    #region Append Text Tags from ArtworkXml
                    xpItr_TextTags = xpNav_ArtworkXml.Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_TEXTELEMENTVALUE/TEXTELEMENTVALUE");
                    if (xpItr_TextTags.Count > 0)
                    {
                        sbTEIDForNotIN.Append("(");
                        ArtworkCacheXml.Append("<ARTWORKVALUES><TEXTTAGS>");
                        while (xpItr_TextTags.MoveNext())
                        {
                            XPathNavigator xpNavTE = xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT");
                            if (xpNavTE == null)
                                continue;

                            String TextTagName = xpNavTE.SelectSingleNode("TEXTTAGNAME").Value;
                            ALSpecialCharHandling.EncodeSpecialCharacters(ref TextTagName);
                            ArtworkCacheXml.AppendFormat(@"<TAG NAME='{0}' ID='{1}' ISHIDDEN='{3}' AUTOPOPULATE='{4}'>
                                                        <VALUE>{2}</VALUE>
                                                      </TAG>", TextTagName,
                                                                 xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/TEXTELEMENTID").InnerXml,
                                                                 xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENTACTUALVALUE").InnerXml,
                                                                 xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/ISHIDDEN").InnerXml,
                                                                 xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/AUTOPOPULATE").InnerXml);
                            sbTEIDForNotIN.Append(xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/TEXTELEMENTID").InnerXml);
                            if (TECnt < (xpItr_TextTags.Count - 1))
                            {
                                sbTEIDForNotIN.Append(",");
                            }
                            TECnt = TECnt + 1;
                        }
                        sbTEIDForNotIN.Append(")");
                        ArtworkCacheXml.Append("</TEXTTAGS>");
                    }
                    #endregion

                    #region Append Graphic Tags for Artwork_GraphicElement relationship
                    xpItr_GraphicTags = xpNav_ArtworkXml.Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_GRAPHICELEMENT/GRAPHICELEMENT");
                    if (xpItr_GraphicTags.Count > 0)
                    {
                        sbGEIDForNotIN.Append("(");
                        if (ArtworkCacheXml.ToString().Contains("<ARTWORKVALUES>"))
                        {
                            ArtworkCacheXml.Append("<GRAPHICTAGS>");
                        }
                        else
                        {
                            ArtworkCacheXml.Append("<ARTWORKVALUES><GRAPHICTAGS>");
                        }
                        while (xpItr_GraphicTags.MoveNext())
                        {
                            xpNav_GEActual = xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT");

                            if (xpNav_GEActual == null)
                                continue;

                            if (xpNav_GEActual.SelectSingleNode("GRAPHICELEMENTID") != null)
                            {
                                GEID = xpNav_GEActual.SelectSingleNode("GRAPHICELEMENTID").Value;
                                sbGEIDForNotIN.Append(GEID);
                            }

                            String ValueTagName = xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICTAGNAME").InnerXml;
                            if (ValueTagName != "NOPHOTO")
                            {
                                xpNav_File = xpItr_GraphicTags.Current.SelectSingleNode("FILESYSTEM");
                                if (xpNav_File != null)
                                {
                                    FileSystemXML = xpNav_File.OuterXml;
                                    ALFile objFile = new ALFile(UserID, Language, TenantName);
                                    ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                                    ObjectID = ObjectID.Replace("<OBJECTID>", "");
                                    FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                                    FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                                    ErrorXML = objFile.GetFile(ref FileSystemXML);
                                    if (ErrorXML != null)
                                    {
                                        objErrorCommon.AppendError("APENGINE_APCACHE_ARTWORKCACHEFOREDITMODE_GETGEFILE_FAILED", this.GetType().ToString(), "ArtworkCacheForEditMode", null, ErrorXML);
                                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                                    }
                                    else
                                    {
                                        //for External file Always use file name instade of displayname
                                        //FileName = ALCommon.GetInnerValue(ref FileSystemXML, "<DISPLAYNAME>", "</DISPLAYNAME>");
                                        GEFilePath = ALCommon.GetInnerValue(ref FileSystemXML, "<PATH>", "</PATH>");
                                        GEFilePath = ALSpecialCharHandling.DecodeSpecialCharacters(GEFilePath);
                                        FileName = Guid.NewGuid().ToString() + Path.GetExtension(GEFilePath);
                                        if (GEFilePath != null)
                                        {
                                            File.Copy(GEFilePath, ArtworkCachePath + "\\Input\\" + FileName, true);
                                        }
                                        if (File.Exists(ArtworkCachePath + "\\Input\\" + FileName))
                                        {
                                            String TagName = xpNav_GEActual.SelectSingleNode("GRAPHICTAGNAME").Value;
                                            ALSpecialCharHandling.EncodeSpecialCharacters(ref TagName);                                            
                                            ArtworkCacheXml.AppendFormat("<TAG NAME=\"{0}\" ID='{1}' VALUETYPE='UPLOAD' STATUS='DONE' AUTOPOPULATE='{3}'>{2}</TAG>", TagName, GEID, ArtworkCachePath + "\\Input\\" + FileName, xpNav_GEActual.SelectSingleNode("AUTOPOPULATE").Value);
                                        }
                                    }
                                }
                            }
                            else
                            {
                                String TagName = xpNav_GEActual.SelectSingleNode("GRAPHICTAGNAME").Value;
                                ALSpecialCharHandling.EncodeSpecialCharacters(ref TagName);                                
                                ArtworkCacheXml.AppendFormat("<TAG NAME='{0}' ID='{1}' VALUETYPE='NOIMAGE' STATUS='DONE' AUTOPOPULATE='{2}'></TAG>", TagName, GEID, xpNav_GEActual.SelectSingleNode("AUTOPOPULATE").Value);
                            }
                            if (GECnt < (xpItr_GraphicTags.Count - 1))
                            {
                                sbGEIDForNotIN.Append(",");
                            }
                            GECnt = GECnt + 1;
                        }

                    }
                    #endregion

                    #region Append Graphic Tags for Artwork_GraphicElementInfo relationship for NoImage
                    xpItr_GraphicTags = xpNav_ArtworkXml.Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE");
                    if (xpItr_GraphicTags.Count > 0)
                    {
                        if (GECnt == 0)
                        {
                            sbGEIDForNotIN.Append("(");
                            if (ArtworkCacheXml.ToString().Contains("<ARTWORKVALUES>"))
                            {                                
                                ArtworkCacheXml.Append("<GRAPHICTAGS>");
                            }
                            else
                            {
                                ArtworkCacheXml.Append("<ARTWORKVALUES><GRAPHICTAGS>");
                            }
                        }
                        while (xpItr_GraphicTags.MoveNext())
                        {
                            xpNav_GEActual = xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT");

                            if (xpNav_GEActual == null)
                                continue;

                            if (xpNav_GEActual.SelectSingleNode("GRAPHICELEMENTID") != null)
                            {
                                GEID = xpNav_GEActual.SelectSingleNode("GRAPHICELEMENTID").Value;
                                if (GECnt > 0)
                                {
                                    sbGEIDForNotIN.Append("," + GEID);
                                }
                                else
                                {
                                    sbGEIDForNotIN.Append(GEID);
                                }
                            }                                                        
                            
                            String TagName = xpNav_GEActual.SelectSingleNode("GRAPHICTAGNAME").Value;
                            ALSpecialCharHandling.EncodeSpecialCharacters(ref TagName);
                            ArtworkCacheXml.AppendFormat("<TAG NAME='{0}' ID='{1}' VALUETYPE='NOIMAGE' STATUS='DONE' AUTOPOPULATE='{2}'></TAG>", TagName, GEID, xpNav_GEActual.SelectSingleNode("AUTOPOPULATE").Value);

                            GECnt = GECnt + 1;
                        }

                    }
                    #endregion

                    #region Append Graphic Tags for Artwork_DigitalAsset relationship
                    xpItr_DigitalAsset = xpNav_ArtworkXml.Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_DIGITALASSET/DIGITALASSET");
                    if (xpItr_DigitalAsset.Count > 0)
                    {
                        if (GECnt == 0)
                        {
                            sbGEIDForNotIN.Append("(");
                            //ArtworkCacheXml.Append("<GRAPHICTAGS>");
                            if (ArtworkCacheXml.ToString().Contains("<ARTWORKVALUES>"))
                            {
                                ArtworkCacheXml.Append("<GRAPHICTAGS>");
                            }
                            else
                            {
                                ArtworkCacheXml.Append("<ARTWORKVALUES><GRAPHICTAGS>");
                            }
                        }

                        while (xpItr_DigitalAsset.MoveNext())
                        {                            
                            xpItr_GEActual = xpItr_DigitalAsset.Current.Select("GRAPHICELEMENTVALUEINFO_DIGITALASSET/GRAPHICELEMENTVALUE/GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT");
                            while (xpItr_GEActual.MoveNext())
                            {
                                sb_GetFileCollectionXML.Remove(0, sb_GetFileCollectionXML.Length);
                                xpNav_GEActual = xpItr_GEActual.Current;
                                if (xpNav_GEActual != null)
                                {
                                    if (xpNav_GEActual.SelectSingleNode("GRAPHICELEMENTID") != null)
                                    {
                                        GEID = xpNav_GEActual.SelectSingleNode("GRAPHICELEMENTID").Value;
                                        if (GECnt > 0)
                                        {
                                            sbGEIDForNotIN.Append("," + GEID);
                                        }
                                        else
                                        {
                                            sbGEIDForNotIN.Append(GEID);
                                        }
                                    }


                                    xpNav_File = xpItr_DigitalAsset.Current.SelectSingleNode("DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE/text()='Original' or RENDITIONTYPE/text()='ORIGINAL']/FILESYSTEM");
                                    if (xpNav_File != null)
                                    {
                                        FileSystemXML = xpNav_File.OuterXml;
                                        ALFile objFile = new ALFile(UserID, Language, TenantName);
                                        ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                                        ObjectID = ObjectID.Replace("<OBJECTID>", "");
                                        FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                                        FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                                        ErrorXML = objFile.GetFile(ref FileSystemXML);
                                        if (ErrorXML != null)
                                        {
                                            objErrorCommon.AppendError("APENGINE_APCACHE_ARTWORKCACHEFOREDITMODE_GETDAFILE_FAILED", this.GetType().ToString(), "ArtworkCacheForEditMode", null, ErrorXML);
                                            ErrorXML = objErrorCommon.ErrorCollectionXML;
                                        }
                                        else
                                        {

                                            FileName = ALCommon.GetInnerValue(ref FileSystemXML, "<DISPLAYNAME>", "</DISPLAYNAME>");

                                            if (String.IsNullOrEmpty(FileName))
                                                FileName = ALCommon.GetInnerValue(ref FileSystemXML, "<NAME>", "</NAME>");


                                            DAFilePath = ALCommon.GetInnerValue(ref FileSystemXML, "<PATH>", "</PATH>");
                                            if (DAFilePath != null)
                                            {
                                                str_processedFile = DAFilePath;
                                                str_FileFormat = FileName.Substring(FileName.LastIndexOf(".") + 1);

                                                str_FilePresentOn = ALCommon.GetInnerValue(ref FileSystemXML, "<ONMEDIASERVER>", "</ONMEDIASERVER>");
                                                if (String.Compare(str_FilePresentOn, "FALSE", true) == 0)
                                                    str_FilePresentOn = "SERVICES";
                                                else
                                                    str_FilePresentOn = "MEDIASERVER";

                                                xpNav_DADimensions = xpItr_DigitalAsset.Current.SelectSingleNode("GRAPHICELEMENTVALUEINFO_DIGITALASSET/GRAPHICELEMENTVALUE/DIMENSIONS");
                                                if (xpNav_DADimensions != null && !String.IsNullOrEmpty(xpNav_DADimensions.InnerXml))
                                                {

                                                    ErrorXML = processFile(ref str_processedFile, xpNav_DADimensions.Value, str_FileFormat, str_FilePresentOn);
                                                    if (String.IsNullOrEmpty(ErrorXML))
                                                    {
                                                        if (ALCommon.IsMediaServerActivated(TenantName) && !IsMediaServerAndServicesTogether())
                                                        {
                                                            //Get File to artwork cache folder from Media server
                                                            sb_GetFileCollectionXML.AppendFormat(@"<FILECOLLECTION><FILE>                                                                                
                                                                                    <SOURCEPATH>{0}</SOURCEPATH>
                                                                                    <DESTINATIONPATH>{1}\\Input\\{3}_{2}</DESTINATIONPATH>
                                                                                </FILE></FILECOLLECTION>", ALCommon.TrimAndReplaceSpecialChars(str_processedFile), ArtworkCachePath, ALCommon.TrimAndReplaceSpecialChars(FileName), GEID);

                                                            String str_FileXML = sb_GetFileCollectionXML.ToString();
                                                            ErrorXML = ObjFT_MS.GetFile(ref str_FileXML);
                                                        }
                                                        else
                                                            File.Copy(str_processedFile, ArtworkCachePath + "\\Input\\" + GEID + "_" + FileName, true);
                                                    }
                                                }
                                                else
                                                {
                                                    if (ALCommon.IsMediaServerActivated(TenantName) && (String.Compare(str_FilePresentOn, "MEDIASERVER", true) == 0) && !IsMediaServerAndServicesTogether())
                                                    {
                                                        //Get File to artwork cache folder from Media server
                                                        sb_GetFileCollectionXML.AppendFormat(@"<FILECOLLECTION><FILE>                                                                                
                                                                                    <SOURCEPATH>{0}</SOURCEPATH>
                                                                                    <DESTINATIONPATH>{1}\\Input\\{3}_{2}</DESTINATIONPATH>
                                                                                </FILE></FILECOLLECTION>", ALCommon.TrimAndReplaceSpecialChars(DAFilePath), ArtworkCachePath, ALCommon.TrimAndReplaceSpecialChars(FileName), GEID);

                                                        String str_FileXML = sb_GetFileCollectionXML.ToString();
                                                        ErrorXML = ObjFT_MS.GetFile(ref str_FileXML);
                                                    }
                                                    else
                                                    {
                                                        DAFilePath = ALSpecialCharHandling.DecodeSpecialCharacters(DAFilePath);
                                                        FileName = ALSpecialCharHandling.DecodeSpecialCharacters(FileName);
                                                        File.Copy(DAFilePath, ArtworkCachePath + "\\Input\\" + GEID + "_" + FileName, true);
                                                    }
                                                }
                                            }

                                            if (File.Exists(ArtworkCachePath + "\\Input\\" + GEID + "_" + FileName))
                                            {
                                                String TagName = xpNav_GEActual.SelectSingleNode("GRAPHICTAGNAME").Value;
                                                ALSpecialCharHandling.EncodeSpecialCharacters(ref TagName);
                                                ArtworkCacheXml.AppendFormat("<TAG NAME='{0}' ID='{1}' VALUETYPE='ASSET' STATUS='DONE' AUTOPOPULATE='{3}'>{2}</TAG>", TagName, GEID, ArtworkCachePath + "\\Input\\" + GEID + "_" + FileName, xpNav_GEActual.SelectSingleNode("AUTOPOPULATE").Value);
                                            }
                                        }
                                    }
                                    //if (xpItr_DigitalAsset.CurrentPosition < xpItr_TextTags.Count)
                                    //{
                                    //    sbGEIDForNotIN.Append(",");
                                    //}
                                    GECnt = GECnt + 1;
                                }
                            }
                        }
                    }
                    //if (GECnt > 0)
                    if (ArtworkCacheXml.ToString().Contains("<GRAPHICTAGS>"))
                    {
                        ArtworkCacheXml.Append("</GRAPHICTAGS>");
                    }

                    if (GECnt > 0)
                        sbGEIDForNotIN.Append(")");

                    if (ArtworkCacheXml.ToString().Contains("<ARTWORKVALUES>"))
                        ArtworkCacheXml.Append("</ARTWORKVALUES>");
                    else
                        ArtworkCacheXml.Append("<ARTWORKVALUES></ARTWORKVALUES>");
                    #endregion

                    #region Serach FormatDetails using formatID and TextElementIDList,GraphicelementIDList
                    sbNOTINFilterTE = new StringBuilder(16);
                    sbNOTINFilterGE = new StringBuilder(16);
                    if (!String.IsNullOrEmpty(sbTEIDForNotIN.ToString()))
                    {
                        sbNOTINFilterTE.AppendFormat(@"<FILTERLIST>
		                                            <FILTER>
			                                            <EXPRESSION>TEXTELEMENTID</EXPRESSION>
			                                            <OPERATOR>NOTIN</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <FILTERID>1</FILTERID>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
		                                            </FILTER>
	                                            </FILTERLIST>", sbTEIDForNotIN.ToString());
                    }
                    //if (!String.IsNullOrEmpty(sbGEIDForNotIN.ToString()))
                    if (GECnt > 0)
                    {
                        sbNOTINFilterGE.AppendFormat(@"<FILTERLIST>
		                                            <FILTER>
			                                            <EXPRESSION>GRAPHICELEMENTID</EXPRESSION>
			                                            <OPERATOR>NOTIN</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <FILTERID>1</FILTERID>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
		                                            </FILTER>
	                                            </FILTERLIST>", sbGEIDForNotIN.ToString());
                    }

                    String strNOTINFilterGE = sbNOTINFilterGE.ToString();
                    sbFormatSearchXML = new StringBuilder(16);
                    sbFormatSearchXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
	                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                <FILTERLIST>
		                                            <FILTER>
			                                            <EXPRESSION>FORMATID</EXPRESSION>
			                                            <OPERATOR>EQUALTO</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <FILTERID>1</FILTERID>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
		                                            </FILTER>
	                                            </FILTERLIST>                                                
                                                <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>FORMATID</FIELDNAME>
			                                            <ALIAS>FORMATID</ALIAS>
		                                            </EXPRESSION>                                                    
		                                            <EXPRESSION>
			                                            <FIELDNAME>ORIGINALFILE</FIELDNAME>
			                                            <ALIAS>ORIGINALFILE</ALIAS>
		                                            </EXPRESSION>                                                    		                                            
	                                            </SELECTLIST>                                                
                                                <RELATIONSHIP NAME='FORMAT_TEXTELEMENT'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                            <ALIAS>TEXTELEMENTID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                                            <ALIAS>TEXTTAGNAME</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ISHIDDEN</FIELDNAME>
                                                            <ALIAS>ISHIDDEN</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                            <ALIAS>AUTOPOPULATE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>{1}                                                    
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='FORMAT_GRAPHICELEMENT'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
	                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
	                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
	                                                        <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
	                                                        <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                        </EXPRESSION>                                                        
                                                        <EXPRESSION>
	                                                        <FIELDNAME>ORIGINALFILE</FIELDNAME>
	                                                        <ALIAS>ORIGINALFILE</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>AUTOPOPULATE</FIELDNAME>
                                                            <ALIAS>AUTOPOPULATE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>{2}                                                    
                                                 </RELATIONSHIP>                                               
                                             </TARGETENTITY>   
                                          </SEARCH>", FormatID, sbNOTINFilterTE.ToString(), strNOTINFilterGE);
                    OMIObj = new OMInterface(UserID, Language, TenantName);
                    ErrorXML = OMIObj.GetHierarchialData(sbFormatSearchXML.ToString(), ref FormatXML);
                    #endregion

                    if (ErrorXML == null)
                    {
                        xpDoc_FormatXml = new XPathDocument(new StringReader(FormatXML));
                        xpNav_FormatXml = xpDoc_FormatXml.CreateNavigator();
                        FormatCachePath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);
                        if (xpNav_FormatXml != null)
                        {
                            #region Append Original file path
                            xpNav_File = xpNav_FormatXml.SelectSingleNode("FORMATCOLLECTION/FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                            if (xpNav_File != null)
                            {
                                FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                if (File.Exists(FormatCachePath + "\\Original\\" + FileName))
                                {
                                    File.Copy(FormatCachePath + "\\Original\\" + FileName, ArtworkCachePath + "\\Input\\" + FileName, true);
                                }
                                if (File.Exists(ArtworkCachePath + "\\Input\\" + FileName))
                                {
                                    ArtworkCacheXml.AppendFormat("<ORIGINALFILEPATH>{0}</ORIGINALFILEPATH>", ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName));
                                }
                            }
                            #endregion

                            #region Append remaining TextTags
                            xpItr_TextTags = xpNav_FormatXml.Select("FORMATCOLLECTION/FORMAT/FORMAT_TEXTELEMENT/TEXTELEMENT");
                            if (xpItr_TextTags.Count > 0)
                            {
                                StringBuilder TagXml = new StringBuilder(16);
                                while (xpItr_TextTags.MoveNext())
                                {
                                    String isHidden = xpItr_TextTags.Current.SelectSingleNode("ISHIDDEN").Value;
                                    String TextTagName = xpItr_TextTags.Current.SelectSingleNode("TEXTTAGNAME").Value;
                                    ALSpecialCharHandling.EncodeSpecialCharacters(ref TextTagName);
                                    TagXml.AppendFormat("<TAG NAME='{0}' ID='{1}' ISHIDDEN='{2}' AUTOPOPULATE='{3}'><VALUE></VALUE></TAG>", TextTagName, xpItr_TextTags.Current.SelectSingleNode("TEXTELEMENTID").InnerXml, isHidden, xpItr_TextTags.Current.SelectSingleNode("AUTOPOPULATE").Value);
                                }
                                if (ArtworkCacheXml.ToString().Contains("</TEXTTAGS>"))
                                {
                                    ArtworkCacheXml = ArtworkCacheXml.Insert(ArtworkCacheXml.ToString().IndexOf("</TEXTTAGS>"), TagXml.ToString());
                                }
                                else
                                {
                                    ArtworkCacheXml = ArtworkCacheXml.Insert(ArtworkCacheXml.ToString().IndexOf("</ARTWORKVALUES>"), "<TEXTTAGS>" + TagXml.ToString() + "</TEXTTAGS>");
                                }
                            }
                            #endregion

                            #region Append remaining Graphic Tags
                            xpItr_GraphicTags = xpNav_FormatXml.Select("FORMATCOLLECTION/FORMAT/FORMAT_GRAPHICELEMENT/GRAPHICELEMENT");
                            if (xpItr_GraphicTags.Count > 0)
                            {
                                String GEOrgPath = null;
                                StringBuilder GETagXml = new StringBuilder(16);
                                while (xpItr_GraphicTags.MoveNext())
                                {
                                    GEID = xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICELEMENTID").Value;
                                    xpNav_File = xpItr_GraphicTags.Current.SelectSingleNode("FILESYSTEM/FILE");
                                    if (xpNav_File != null)
                                    {
                                        FileName = xpNav_File.SelectSingleNode("DISPLAYNAME").Value;
                                        GEOrgPath = GetGraphicElementOriginalFilePath(TenantName, FormatID, GEID);
                                        if (File.Exists(GEOrgPath))
                                        {
                                            File.Copy(GEOrgPath, ArtworkCachePath + "\\Input\\" + FileName, true);
                                        }
                                        if (File.Exists(ArtworkCachePath + "\\Input\\" + FileName))
                                        {
                                            String TagName = xpItr_GraphicTags.Current.SelectSingleNode("GRAPHICTAGNAME").Value;
                                            ALSpecialCharHandling.EncodeSpecialCharacters(ref TagName);
                                            GETagXml.AppendFormat("<TAG NAME='{0}' ID='{1}' VALUETYPE='DEFAULT' STATUS='DONE' AUTOPOPULATE='{3}'>{2}</TAG>", TagName, GEID, ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName), xpItr_GraphicTags.Current.SelectSingleNode("AUTOPOPULATE").Value);
                                        }
                                    }
                                }
                                if (ArtworkCacheXml.ToString().Contains("</GRAPHICTAGS>"))
                                {
                                    ArtworkCacheXml = ArtworkCacheXml.Insert(ArtworkCacheXml.ToString().IndexOf("</GRAPHICTAGS>"), GETagXml.ToString());
                                }
                                else
                                {
                                    ArtworkCacheXml = ArtworkCacheXml.Insert(ArtworkCacheXml.ToString().IndexOf("</ARTWORKVALUES>"), "<GRAPHICTAGS>" + GETagXml.ToString() + "</GRAPHICTAGS>");
                                }
                            }
                            #endregion

                            #region Move FormatInfo xml from FormatCache to ArtworkCache
                            if (File.Exists(FormatCachePath + "\\FormatInfo.xml"))
                            {
                                File.Copy(FormatCachePath + "\\FormatInfo.xml", ArtworkCachePath + "\\Input\\FormatInfo.xml",true);
                            }
                            #endregion

                            #region Move AdditionalFiles from FormatCache to ArtworkCache
                            if (Directory.Exists(FormatCachePath + "\\AdditionalFiles"))
                            {
                                string[] arr_Files = Directory.GetFiles(FormatCachePath + "\\AdditionalFiles");
                                for (int FileCnt = 0; FileCnt < arr_Files.Length; FileCnt++)
                                {
                                    FileName = arr_Files[FileCnt].Substring(arr_Files[FileCnt].LastIndexOf("\\") + 1);
                                    File.Copy(arr_Files[FileCnt], ArtworkCachePath + "\\Input\\" + FileName,true);
                                }
                            }
                            #endregion
                        }
                    }
                    ArtworkCacheXml.Append("</ARTWORK>");
                    if (!String.IsNullOrEmpty(ArtworkCacheXml.ToString()))
                    {
                        File.WriteAllText(ArtworkCachePath + "\\Artwork.xml", ArtworkCacheXml.ToString());
                    }
                }
                else
                {
                    objErrorCommon.AppendError("APENGINE_APCACHE_ARTWORKCACHEFOREDITMODE_GETARTWORKDETAILS_FAILED", this.GetType().ToString(), "ArtworkCacheForEditMode", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_FORMATCACHE_ARTWORKCACHEFOREDITMODE_EXCEPTION", this.GetType().ToString(), "ArtworkCacheForEditMode", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
        }

        public ERRORXML CleanUpArtwokCache()
        {
            ERRORXML                        ErrorXML                            = null;
            XML                             OutputXML                           = null;            
            StringBuilder                   SearchInputXML                      = null;
            XPathDocument                   ArtworkListXML                      = null;
            XPathNavigator                  xpNav_ArtworkList                   = null;
            XPathNodeIterator               xpIter_ArtworkIds                   = null;            
            string                          ArtworkID                           = null;
            StringCollection                ArtworkIDCollection                 = null;
            StringBuilder                   sbCacheFolderPath                   = null;
            string                          str_Path                            = null;
            string[]                        ArtworkCacheMemberList              = null;
            long                            result                              = 0;
            string                          DeleteList                          = "";
            string                          DirName                             = null;
            OMInterface                     objOMI                              = null;

            try
            {
                SearchInputXML = new StringBuilder(100);
                objOMI = new OMInterface(UserID, Language, TenantName);

                SearchInputXML.AppendFormat(@"<SEARCH>
                                                    <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>                                                    
                                                    <SESSIONINFO UserID='{0}' TenantName='{1}' Language='{2}'/>
                                                    <TARGETENTITY>
                                                        <SELECTLIST>
                                                          <EXPRESSION>
                                                            <FIELDNAME>ARTWORKID</FIELDNAME>
                                                            <ALIAS>ARTWORKID</ALIAS>
                                                          </EXPRESSION>
                                                        </SELECTLIST>
                                                     </TARGETENTITY>
                                                </SEARCH>
                                                        ", UserID, TenantName, Language);
                ErrorXML = objOMI.GetHierarchialData(SearchInputXML.ToString(), ref OutputXML);

                if (ErrorXML == null)
                {
                    ArtworkListXML = new XPathDocument(new StringReader(OutputXML));
                    xpNav_ArtworkList = ArtworkListXML.CreateNavigator();

                    xpIter_ArtworkIds = xpNav_ArtworkList.Select("ARTWORKCOLLECTION/ARTWORK/ARTWORKID");
                    if (xpIter_ArtworkIds.Count > 0)
                    {
                        ArtworkIDCollection = new StringCollection();
                        sbCacheFolderPath = new StringBuilder(30);
                        while (xpIter_ArtworkIds.MoveNext())
                        {
                            ArtworkID = xpIter_ArtworkIds.Current.Value;
                            ArtworkIDCollection.Add(ArtworkID);
                        }

                        sbCacheFolderPath.AppendFormat("{0}{1}", System.Configuration.ConfigurationManager.AppSettings["AP_ArtworkCachePath"], TenantName);
                        str_Path = sbCacheFolderPath.ToString();

                        if (Directory.Exists(str_Path))
                        {
                            ArtworkCacheMemberList = Directory.GetDirectories(str_Path, "*", SearchOption.TopDirectoryOnly);

                            if ((ArtworkCacheMemberList != null) && (ArtworkCacheMemberList.Length > 0))
                            {
                                foreach (string ArtCacheMbr in ArtworkCacheMemberList)
                                {
                                    DirName = ArtCacheMbr.Substring(ArtCacheMbr.LastIndexOf("\\")+1);
                                    if (long.TryParse(DirName, out result))
                                    {
                                        if (!ArtworkIDCollection.Contains(DirName))
                                        {
                                            DeleteList += DirName + ";";
                                        }
                                    }
                                }
                            }
                        }

                        if (!String.IsNullOrEmpty(DeleteList))
                        {
                            
                            ErrorXML = DeleteArtworkCache(DeleteList);

                            if (ErrorXML != null)
                            {
                                objErrorCommon.AppendError("APENGINE_BOOK_CACHE_CLEANUP_FAILED", this.GetType().ToString(), "CleanUpArtwokCacheForBooks", null, ErrorXML);
                                return ErrorXML;
                            }
                        }

                    }
                }
                else
                {
                    objErrorCommon.AppendError("APENGINE_BOOK_CACHE_CLEANUP_FAILED", this.GetType().ToString(), "CleanUpArtwokCacheForBooks", null, ErrorXML);                    
                    return ErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_BOOK_CACHE_CLEANUP_FAILED", this.GetType().ToString(), "CleanUpArtwokCacheForBooks", ExcepObj, null);
                return objErrorCommon.ErrorCollectionXML;
            }

            return null;
        }

        private void GetArtworkAsset(String ArtworkID, String GEID, String DAID, bool Async)
        {
            List<String> objData = new List<string>();
            objData.Add(ArtworkID);
            objData.Add(GEID);
            objData.Add(DAID);

            OrgIdentity = System.Security.Principal.WindowsIdentity.GetCurrent();

            if (Async)
            {
                System.Threading.ThreadPool.QueueUserWorkItem(Handle_GetArtworkAsset, objData);
            }
            else
            {
                Handle_GetArtworkAsset(objData);
            }
        }

        private void Handle_GetArtworkAsset(object data)
        {
            System.Security.Principal.WindowsImpersonationContext NewContext = null;
            try
            {
                System.Threading.Thread.CurrentPrincipal = new System.Security.Principal.WindowsPrincipal(OrgIdentity);
                NewContext = OrgIdentity.Impersonate();

                String str_Dimensions       = null;
                String str_FilePresentOn    = null;
                String str_TargetFormat     = null;
                bool IsMediaServerActivated = false;
                StringBuilder sb_GetFileCollectionXML = null;
                String str_AssetOutBoxPath  = null;
                String str_AssetName        = null;

                List<String> objData = (List<String>)data;
                String ArtworkID = objData[0];
                String GEID = objData[1];
                String AssetID = objData[2];

                StringBuilder sbSearch = new StringBuilder();
                sbSearch.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
	                                                <EXPRESSION>DIGITALASSETID</EXPRESSION>
	                                                <OPERATOR>EQUALTO</OPERATOR>
	                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
	                                                <FILTERID>1</FILTERID>
	                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                    <ALIAS>DIGITALASSETID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='DIGITALASSET_CURRENTREVISION'>
                                                <RELATIONSHIP NAME='CONTENTREVISION_RENDITION'>
                                                    <FILTERLIST>
                                                        <FILTER>
                                                            <EXPRESSION>RENDITIONTYPE</EXPRESSION>
                                                            <OPERATOR>EQUALTO</OPERATOR>
                                                            <EXPRESSIONVALUE>ORIGINAL</EXPRESSIONVALUE>
                                                            <FILTERID>1</FILTERID>
                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        </FILTER>
                                                    </FILTERLIST>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>RENDITIONTYPE</FIELDNAME>
                                                            <ALIAS>RENDITIONTYPE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </RELATIONSHIP>
                                            <RELATIONSHIP NAME='GRAPHICELEMENTVALUEINFO_DIGITALASSET'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>DIMENSIONS</FIELDNAME>
                                                        <ALIAS>DIMENSIONS</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>ARTWORKID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>
                                                        <FILTERID>1</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>ARTWORK_GRAPHICELEMENTVALUEINFO</RELATIONSHIPNAME>
                                                    </FILTER>
                                                    <FILTER>
                                                        <EXPRESSION>GRAPHICELEMENTID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{2}</EXPRESSIONVALUE>
                                                        <FILTERID>2</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO</RELATIONSHIPNAME>
                                                    </FILTER>
                                                </FILTERLIST>
                                            </RELATIONSHIP>
                                        </TARGETENTITY>
                                    </SEARCH>", AssetID, ArtworkID, GEID);

                String AssetDetailsXML = null;

                OMInterface objOMI = new OMInterface(UserID, Language, TenantName);
                String ErrorXML = objOMI.GetHierarchialData(sbSearch.ToString(), ref AssetDetailsXML);
                if (ErrorXML != null)
                {
                    return;
                }

                String AssetFileXML = ALCommon.GetInnerValue(ref AssetDetailsXML, "<FILE>", "</FILE>");
                AssetFileXML = "<FILE>" + AssetFileXML + "</FILE>";

                String ObjectID = AssetFileXML.Substring(AssetFileXML.LastIndexOf("<OBJECTID>"), AssetFileXML.IndexOf("</OBJECTID>") - AssetFileXML.IndexOf("<OBJECTID>"));
                ObjectID = ObjectID.Replace("<OBJECTID>", "");
                AssetFileXML = AssetFileXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                AssetFileXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + AssetFileXML + "</FILESYSTEM>";


                //FileRetrival FileRetrivalObj = new FileRetrival(SessionID.ToString(), UserID.ToString(), TenantName, Language);
               
                ErrorXML = objOMI.GetFiles(ref AssetFileXML);
                if (ErrorXML != null)
                {
                    return;
                }

                String FormatID     = GetFormatID(TenantName, ArtworkID);
                String GEFilePath   = GetGraphicElementOriginalFilePath(TenantName, FormatID, GEID);
                String FileName     = Path.GetFileName(GEFilePath);                
                String FilePath     = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref AssetFileXML, "<PATH>", "</PATH>"));

                IsMediaServerActivated = ALCommon.IsMediaServerActivated(TenantName);
                if (IsMediaServerActivated && String.Compare(ALCommon.GetInnerValue(ref AssetFileXML, "<ONMEDIASERVER>", "</ONMEDIASERVER>"), "TRUE", true) == 0)
                    str_FilePresentOn = "MEDIASERVER";
                else
                    str_FilePresentOn = "SERVICES";

                str_AssetOutBoxPath = FilePath.Substring(0, FilePath.LastIndexOf("\\"));
                str_AssetName = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref AssetFileXML, "<DISPLAYNAME>", "</DISPLAYNAME>"));
                if (String.IsNullOrEmpty(str_AssetName))
                    str_AssetName = ALCommon.GetInnerValue(ref AssetFileXML, "<NAME>", "</NAME>");

                FileTransferInfrastructure ObjFT_MS = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                ObjFT_MS.ToUseServiceURL = ALCommon.GetWebServiceURL("ALMEDIASERVERMTOMWS");

                sb_GetFileCollectionXML = new StringBuilder(100);

                #region Process file according to the crop dimensions specified  
                str_Dimensions          = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref AssetDetailsXML, "<DIMENSIONS>", "</DIMENSIONS>"));
                if (!String.IsNullOrEmpty(str_Dimensions))
                {
                    str_TargetFormat = str_AssetName.Substring(str_AssetName.LastIndexOf(".") + 1);

                    ErrorXML = processFile(ref FilePath, str_Dimensions, str_TargetFormat, str_FilePresentOn);
                    if (String.IsNullOrEmpty(ErrorXML))
                    {
                        if (IsMediaServerActivated && !IsMediaServerAndServicesTogether())
                        {
                            //Get File to artwork cache folder from Media server
                            sb_GetFileCollectionXML.AppendFormat(@"<FILECOLLECTION>
                                                                    <FILE>                                                                                
                                                                        <SOURCEPATH>{0}</SOURCEPATH>
                                                                        <DESTINATIONPATH>{1}\\{3}_{2}</DESTINATIONPATH>
                                                                    </FILE>
                                                                    </FILECOLLECTION>", ALCommon.TrimAndReplaceSpecialChars(FilePath),
                                                              str_AssetOutBoxPath, ALCommon.TrimAndReplaceSpecialChars(str_AssetName), GEID);

                            String str_FileXML = sb_GetFileCollectionXML.ToString();
                            ErrorXML = ObjFT_MS.GetFile(ref str_FileXML);
                            if (String.IsNullOrEmpty(ErrorXML))
                                FilePath = str_AssetOutBoxPath + "\\" + GEID + "_" + str_AssetName;
                        }
                        else
                        {
                            File.Copy(FilePath, str_AssetOutBoxPath + "\\" + GEID + "_" + str_AssetName, true);
                            FilePath = str_AssetOutBoxPath + "\\" + GEID + "_" + str_AssetName;
                        }

                    }
                }
                #endregion
                else
                {
                    if (ALCommon.IsMediaServerActivated(TenantName) && (String.Compare(str_FilePresentOn, "MEDIASERVER", true) == 0) && !IsMediaServerAndServicesTogether())
                    {
                        //Get File to artwork cache folder from Media server
                        sb_GetFileCollectionXML.AppendFormat(@"<FILECOLLECTION>
                                                                    <FILE>                                                                                
                                                                    <SOURCEPATH>{0}</SOURCEPATH>
                                                                    <DESTINATIONPATH>{1}\\{3}_{2}</DESTINATIONPATH>
                                                                </FILE></FILECOLLECTION>",
                                                               ALCommon.TrimAndReplaceSpecialChars(FilePath), str_AssetOutBoxPath, ALCommon.TrimAndReplaceSpecialChars(str_AssetName), GEID);

                        String str_FileXML = sb_GetFileCollectionXML.ToString();
                        ErrorXML = ObjFT_MS.GetFile(ref str_FileXML);
                        if (String.IsNullOrEmpty(ErrorXML))
                            FilePath = str_AssetOutBoxPath + "\\" + GEID + "_" + str_AssetName;

                    }

                }

                String ArtworkXMLPath = GetArtworkValuesXMLPath(TenantName, ArtworkID);
                XmlDocument xDoc = new XmlDocument();
                xDoc.Load(ArtworkXMLPath);
                XmlNode xNode = xDoc.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@ID='" + GEID + "']");
                xNode.Attributes.GetNamedItem("STATUS").Value = "IN_OUTBOX";
                ((XmlElement)xNode).InnerText = FilePath;
                xDoc.Save(ArtworkXMLPath);
            }
            catch (Exception)
            {
                return;
            }
            finally
            {
                if (NewContext != null)
                    NewContext.Undo();
            }
        }

        private bool ValidateFormatInfoXml(String ArtworkID, String FormatID)
        {
            XPathDocument xpDoc_FormatInfoInArt = null;
            XPathDocument xpDoc_FormatInfoInFormat = null;
            XmlDocument xmlDoc_ArtworkXml = null;
            XPathNavigator xpNav_FormatInfoInArt = null;
            XPathNavigator xpNav_FormatInfoInFormat = null;
            XPathNavigator xpNav_GEFileInArt = null;
            XPathNavigator xpNav_GEFile = null;
            String FormatCachePath = null;
            String Day = null;
            String Month = null;
            String Year = null;
            String Hour = null;
            String Minute = null;
            String Second = null;
            String FileName = null;
            StringBuilder sbGEIDList = null;
            bool CopyFormaInfo = false;

            try
            {
                ArtworkCachePath = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache);
                FormatCachePath = GetCacheFolderPath(TenantName, FormatID, CacheType.FormatCache);

                xpDoc_FormatInfoInArt = new XPathDocument(ArtworkCachePath + "\\Input\\FormatInfo.xml");
                xpNav_FormatInfoInArt = xpDoc_FormatInfoInArt.CreateNavigator();

                xpDoc_FormatInfoInFormat = new XPathDocument(FormatCachePath + "\\FormatInfo.xml");
                xpNav_FormatInfoInFormat = xpDoc_FormatInfoInFormat.CreateNavigator();

                #region Load Artwork xml
                xmlDoc_ArtworkXml = new XmlDocument();
                xmlDoc_ArtworkXml.Load(ArtworkCachePath + "\\Artwork.xml");
                #endregion

                #region Check Original file date
                XPathNavigator xpNav_OrgFile = xpNav_FormatInfoInFormat.SelectSingleNode("FORMATDETAILS/FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                if (xpNav_OrgFile != null)
                {
                    Day = xpNav_OrgFile.SelectSingleNode("MODIFIEDON/DAY").Value;
                    Month = xpNav_OrgFile.SelectSingleNode("MODIFIEDON/MONTH").Value;
                    Year = xpNav_OrgFile.SelectSingleNode("MODIFIEDON/YEAR").Value;
                    Hour = xpNav_OrgFile.SelectSingleNode("MODIFIEDON/HOUR").Value;
                    Minute = xpNav_OrgFile.SelectSingleNode("MODIFIEDON/MINUTE").Value;
                    Second = xpNav_OrgFile.SelectSingleNode("MODIFIEDON/SECOND").Value;

                    DateTime objFormatDate = new DateTime(Convert.ToInt32(Year), Convert.ToInt32(Month), Convert.ToInt32(Day), Convert.ToInt32(Hour), Convert.ToInt32(Minute), Convert.ToInt32(Second));

                    XPathNavigator xpNav_ArtFile = xpNav_FormatInfoInArt.SelectSingleNode("FORMATDETAILS/FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']");
                    if (xpNav_ArtFile != null)
                    {
                        Day = xpNav_ArtFile.SelectSingleNode("MODIFIEDON/DAY").Value;
                        Month = xpNav_ArtFile.SelectSingleNode("MODIFIEDON/MONTH").Value;
                        Year = xpNav_ArtFile.SelectSingleNode("MODIFIEDON/YEAR").Value;
                        Hour = xpNav_ArtFile.SelectSingleNode("MODIFIEDON/HOUR").Value;
                        Minute = xpNav_ArtFile.SelectSingleNode("MODIFIEDON/MINUTE").Value;
                        Second = xpNav_ArtFile.SelectSingleNode("MODIFIEDON/SECOND").Value;

                        DateTime objArtDate = objArtDate = new DateTime(Convert.ToInt32(Year), Convert.ToInt32(Month), Convert.ToInt32(Day), Convert.ToInt32(Hour), Convert.ToInt32(Minute), Convert.ToInt32(Second));

                        if (objFormatDate != objArtDate)
                        {
                            return false;
                        }
                    }
                }
                #endregion

                #region Check GE files form FormatInfo in FormatCache
                XPathNodeIterator xpItr_GE = xpNav_FormatInfoInFormat.Select("FORMATDETAILS/GRAPHICELEMENTCOLLECTION/GRAPHICELEMENT");
                if (xpItr_GE.Count > 0)
                {
                    int IDCnt = 0;

                    sbGEIDList = new StringBuilder(16);
                    sbGEIDList.Append("FORMATDETAILS/GRAPHICELEMENTCOLLECTION/GRAPHICELEMENT[");

                    while (xpItr_GE.MoveNext())
                    {
                        String GEID = xpItr_GE.Current.SelectSingleNode("GRAPHICELEMENTID").Value;
                        String TagName = xpItr_GE.Current.SelectSingleNode("GRAPHICTAGNAME").Value;

                        sbGEIDList.AppendFormat("GRAPHICELEMENTID/text() !='{0}'", GEID);
                        if (IDCnt < (xpItr_GE.Count - 1))
                        {
                            sbGEIDList.Append(" and ");
                        }
                        IDCnt++;

                        xpNav_GEFile = xpItr_GE.Current.SelectSingleNode("FILESYSTEM/FILE");
                        if (xpNav_GEFile != null)
                        {
                            Day = xpNav_GEFile.SelectSingleNode("MODIFIEDON/DAY").Value;
                            Month = xpNav_GEFile.SelectSingleNode("MODIFIEDON/MONTH").Value;
                            Year = xpNav_GEFile.SelectSingleNode("MODIFIEDON/YEAR").Value;
                            Hour = xpNav_GEFile.SelectSingleNode("MODIFIEDON/HOUR").Value;
                            Minute = xpNav_GEFile.SelectSingleNode("MODIFIEDON/MINUTE").Value;
                            Second = xpNav_GEFile.SelectSingleNode("MODIFIEDON/SECOND").Value;

                            DateTime objGEFormatDate = new DateTime(Convert.ToInt32(Year), Convert.ToInt32(Month), Convert.ToInt32(Day), Convert.ToInt32(Hour), Convert.ToInt32(Minute), Convert.ToInt32(Second));

                            xpNav_GEFileInArt = xpNav_FormatInfoInArt.SelectSingleNode("FORMATDETAILS/GRAPHICELEMENTCOLLECTION/GRAPHICELEMENT[GRAPHICELEMENTID/text()='" + GEID + "']/FILESYSTEM/FILE");
                            if (xpNav_GEFileInArt != null)
                            {
                                Day = xpNav_GEFileInArt.SelectSingleNode("MODIFIEDON/DAY").Value;
                                Month = xpNav_GEFileInArt.SelectSingleNode("MODIFIEDON/MONTH").Value;
                                Year = xpNav_GEFileInArt.SelectSingleNode("MODIFIEDON/YEAR").Value;
                                Hour = xpNav_GEFileInArt.SelectSingleNode("MODIFIEDON/HOUR").Value;
                                Minute = xpNav_GEFileInArt.SelectSingleNode("MODIFIEDON/MINUTE").Value;
                                Second = xpNav_GEFileInArt.SelectSingleNode("MODIFIEDON/SECOND").Value;

                                DateTime objGEArtDate = new DateTime(Convert.ToInt32(Year), Convert.ToInt32(Month), Convert.ToInt32(Day), Convert.ToInt32(Hour), Convert.ToInt32(Minute), Convert.ToInt32(Second));
                                if (objGEFormatDate != objGEArtDate)
                                {
                                    CopyFormaInfo = true;

                                    FileName = xpNav_GEFile.SelectSingleNode("DISPLAYNAME").Value;

                                    #region Copy from FormatCache to ArtworkCache
                                    if (File.Exists(FormatCachePath + "\\GE_" + GEID + "\\Original\\" + FileName))
                                    {
                                        File.Copy(FormatCachePath + "\\GE_" + GEID + "\\Original\\" + FileName, ArtworkCachePath + "\\Input\\" + FileName, true);
                                    }
                                    #endregion

                                    #region Update Artwork Xml
                                    XmlNode GETag = xmlDoc_ArtworkXml.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@ID='" + GEID + "']");
                                    if (GETag != null)
                                    {
                                        GETag.InnerXml = ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName);
                                    }
                                    #endregion
                                }
                            }
                            else
                            {
                                CopyFormaInfo = true;

                                FileName = xpNav_GEFile.SelectSingleNode("DISPLAYNAME").Value;

                                #region Copy from FormatCache to ArtworkCache
                                if (File.Exists(FormatCachePath + "\\GE_" + GEID + "\\Original\\" + FileName))
                                {
                                    File.Copy(FormatCachePath + "\\GE_" + GEID + "\\Original\\" + FileName, ArtworkCachePath + "\\Input\\" + FileName, true);
                                }
                                #endregion

                                #region Update Artwork Xml
                                XmlNode GETag = xmlDoc_ArtworkXml.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@ID='" + GEID + "']");
                                if (GETag != null)
                                {
                                    GETag.InnerXml = ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName);
                                }
                                else
                                {
                                    XmlNode GraphicTagNode = xmlDoc_ArtworkXml.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS");
                                    if (GraphicTagNode != null)
                                    {
                                        GETag = xmlDoc_ArtworkXml.CreateElement("TAG");
                                        GETag.InnerXml = ArtworkCachePath + "\\Input\\" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(FileName);
                                        ((XmlElement)GETag).SetAttribute("NAME", TagName);
                                        ((XmlElement)GETag).SetAttribute("ID", GEID);
                                        ((XmlElement)GETag).SetAttribute("VALUETYPE", "DEFAULT");
                                        ((XmlElement)GETag).SetAttribute("STATUS", "DONE");

                                        GraphicTagNode.AppendChild(GETag);
                                    }
                                }
                                #endregion
                            }
                        }
                    }
                    sbGEIDList.Append("]");
                    //else
                    //    return false;                

                    #region Check GE files from FormatInfo in ArtworkCache
                    xpItr_GE = xpNav_FormatInfoInArt.Select(sbGEIDList.ToString());
                    if (xpItr_GE.Count > 0)
                    {
                        CopyFormaInfo = true;

                        while (xpItr_GE.MoveNext())
                        {
                            String GEID = xpItr_GE.Current.SelectSingleNode("GRAPHICELEMENTID").Value;
                            String TagName = xpItr_GE.Current.SelectSingleNode("GRAPHICTAGNAME").Value;

                            FileName = xpItr_GE.Current.SelectSingleNode("FILESYSTEM/FILE/DISPLAYNAME").Value;
                            #region Remove File from ArtworkCache
                            if (File.Exists(ArtworkCachePath + "\\Input\\" + FileName))
                            {
                                File.Delete(ArtworkCachePath + "\\Input\\" + FileName);
                            }
                            #endregion

                            #region Remove Graphic Tag from Artwork Xml
                            XmlNode GETagInArt = xmlDoc_ArtworkXml.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@ID='" + GEID + "']");
                            if (GETagInArt != null)
                            {
                                XmlNode GraphicTagNodeInArt = xmlDoc_ArtworkXml.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS");
                                GraphicTagNodeInArt.RemoveChild(GETagInArt);
                            }
                            #endregion

                        }
                    }
                    #endregion
                }
                #endregion

                xmlDoc_ArtworkXml.Save(ArtworkCachePath + "\\Artwork.xml");

                #region Check AdditionalFiles
                XPathNodeIterator xpItr_AdditionalFiles = xpNav_FormatInfoInFormat.Select("FORMATDETAILS/FORMAT/FILESYSTEM/BINDER/FILE");
                if (xpItr_AdditionalFiles.Count > 0)
                {
                    XPathNavigator xpNav_AdditionalFileInArt = null;
                    while (xpItr_AdditionalFiles.MoveNext())
                    {
                        FileName = xpItr_AdditionalFiles.Current.SelectSingleNode("DISPLAYNAME").Value;

                        Day = xpItr_AdditionalFiles.Current.SelectSingleNode("MODIFIEDON/DAY").Value;
                        Month = xpItr_AdditionalFiles.Current.SelectSingleNode("MODIFIEDON/MONTH").Value;
                        Year = xpItr_AdditionalFiles.Current.SelectSingleNode("MODIFIEDON/YEAR").Value;
                        Hour = xpItr_AdditionalFiles.Current.SelectSingleNode("MODIFIEDON/HOUR").Value;
                        Minute = xpItr_AdditionalFiles.Current.SelectSingleNode("MODIFIEDON/MINUTE").Value;
                        Second = xpItr_AdditionalFiles.Current.SelectSingleNode("MODIFIEDON/SECOND").Value;

                        DateTime objFormatDate = new DateTime(Convert.ToInt32(Year), Convert.ToInt32(Month), Convert.ToInt32(Day), Convert.ToInt32(Hour), Convert.ToInt32(Minute), Convert.ToInt32(Second));

                        xpNav_AdditionalFileInArt = xpNav_FormatInfoInArt.SelectSingleNode("FORMATDETAILS/FORMAT/FILESYSTEM/BINDER/FILE[DISPLAYNAME/text()='" + FileName + "']");
                        if (xpNav_AdditionalFileInArt != null)
                        {
                            Day = xpNav_AdditionalFileInArt.SelectSingleNode("MODIFIEDON/DAY").Value;
                            Month = xpNav_AdditionalFileInArt.SelectSingleNode("MODIFIEDON/MONTH").Value;
                            Year = xpNav_AdditionalFileInArt.SelectSingleNode("MODIFIEDON/YEAR").Value;
                            Hour = xpNav_AdditionalFileInArt.SelectSingleNode("MODIFIEDON/HOUR").Value;
                            Minute = xpNav_AdditionalFileInArt.SelectSingleNode("MODIFIEDON/MINUTE").Value;
                            Second = xpNav_AdditionalFileInArt.SelectSingleNode("MODIFIEDON/SECOND").Value;

                            DateTime objArtDate = new DateTime(Convert.ToInt32(Year), Convert.ToInt32(Month), Convert.ToInt32(Day), Convert.ToInt32(Hour), Convert.ToInt32(Minute), Convert.ToInt32(Second));
                            if (objFormatDate != objArtDate)
                            {
                                CopyFormaInfo = true;

                                #region Copy files
                                if (File.Exists(FormatCachePath + "\\AdditionalFiles\\" + FileName))
                                {
                                    File.Copy(FormatCachePath + "\\AdditionalFiles\\" + FileName, ArtworkCachePath + "\\Input\\" + FileName, true);
                                }
                                #endregion
                            }
                        }
                    }
                }
                #endregion

                if (CopyFormaInfo)
                {
                    File.Copy(FormatCachePath + "\\FormatInfo.xml", ArtworkCachePath + "\\Input\\FormatInfo.xml", true);
                }
                return true;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_APCACHE_VALIDATEFORMATINFOXML_EXCEPTION", this.GetType().ToString(), "ValidateFormatInfoXml", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return false;
            }            
        }

        private bool ValidateArtworkJob(String ArtworkID)
        {
            XPathDocument xpDoc_ArtworkXml = null;
            XPathNavigator xpNav_ArtworkXml = null;
            XPathNodeIterator xpItr_Tag = null;
            String OrgFilePath = null;
            String GEFilePath = null;

            ArtworkCachePath = GetCacheFolderPath(TenantName, ArtworkID, CacheType.ArtworkCache);

            if (Directory.Exists(ArtworkCachePath))
            {
                if (File.Exists(ArtworkCachePath + "\\Artwork.xml"))
                {
                    xpDoc_ArtworkXml = new XPathDocument(ArtworkCachePath + "\\Artwork.xml");
                    xpNav_ArtworkXml = xpDoc_ArtworkXml.CreateNavigator();
                    if (xpNav_ArtworkXml != null)
                    {
                        if (xpNav_ArtworkXml.SelectSingleNode("ARTWORK/ORIGINALFILEPATH") != null)
                        {
                            OrgFilePath = xpNav_ArtworkXml.SelectSingleNode("ARTWORK/ORIGINALFILEPATH").Value;
                            if (!File.Exists(OrgFilePath))
                            {
                                return false;
                            }
                        }
                        else
                            return false;
                        xpItr_Tag = xpNav_ArtworkXml.Select("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG");
                        if (xpItr_Tag.Count > 0)
                        {
                            while (xpItr_Tag.MoveNext())
                            {
                                GEFilePath = xpItr_Tag.Current.InnerXml;

                                String str_status = xpItr_Tag.Current.GetAttribute("STATUS", "");
                                String str_Type = xpItr_Tag.Current.GetAttribute("VALUETYPE", "");
                                if (str_Type.ToUpper() != "NOIMAGE")
                                {
                                    if (String.Compare(str_status, "DONE", true) == 0 && !File.Exists(GEFilePath))
                                    {
                                        return false;
                                    }
                                }
                                else if (File.Exists(GEFilePath))
                                {
                                    File.Delete(GEFilePath);
                                }
                            }
                        }
                    }
                    else
                        return false;
                }
                else
                    return false;
                return true;
            }
            else
                return false;
        }

        private void CreateArtworkInfoNode(XPathNavigator xpNav_RootArtwork, ref XmlDocument xDoc_CacheXML)
        {
            XmlElement xEle_Artwork = (XmlElement)xDoc_CacheXML.SelectSingleNode("ARTWORK");
            XmlElement xEle_ArtworkInfo = (XmlElement)xEle_Artwork.SelectSingleNode("ARTWORK_ARTWORKINFO");
            if (xEle_ArtworkInfo != null)
                xEle_Artwork.RemoveChild(xEle_ArtworkInfo);

            XmlElement Node_Reln = xDoc_CacheXML.CreateElement("ARTWORK_ARTWORKINFO");
            XmlElement Node_Entity = xDoc_CacheXML.CreateElement("ARTWORKINFO");
            XmlElement Node_ARTWORKINFOID = xDoc_CacheXML.CreateElement("ARTWORKINFOID");
            XmlElement Node_CUSTOM_HEIGHT = xDoc_CacheXML.CreateElement("CUSTOM_HEIGHT");
            XmlElement Node_CUSTOM_WIDTH = xDoc_CacheXML.CreateElement("CUSTOM_WIDTH");
            XmlElement Node_ARTWORKDIMENSIONUNIT = xDoc_CacheXML.CreateElement("ARTWORKDIMENSIONUNIT");

            XPathNavigator xpNav_Temp1 = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/CUSTOM_HEIGHT");
            if (xpNav_Temp1 != null)
                Node_CUSTOM_HEIGHT.InnerXml = xpNav_Temp1.InnerXml;

            xpNav_Temp1 = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/CUSTOM_WIDTH");
            if (xpNav_Temp1 != null)
                Node_CUSTOM_WIDTH.InnerXml = xpNav_Temp1.InnerXml;

            xpNav_Temp1 = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/ARTWORKINFOID");
            if (xpNav_Temp1 != null)
                Node_ARTWORKINFOID.InnerXml = xpNav_Temp1.InnerXml;

            xpNav_Temp1 = xpNav_RootArtwork.SelectSingleNode("ARTWORK/ARTWORK_ARTWORKINFO/ARTWORKINFO/ARTWORKDIMENSIONUNIT");
            if (xpNav_Temp1 != null)
                Node_ARTWORKDIMENSIONUNIT.InnerXml = xpNav_Temp1.InnerXml;

            Node_Entity.AppendChild(Node_ARTWORKINFOID);
            Node_Entity.AppendChild(Node_CUSTOM_HEIGHT);
            Node_Entity.AppendChild(Node_CUSTOM_WIDTH);
            Node_Entity.AppendChild(Node_ARTWORKDIMENSIONUNIT);
            Node_Reln.AppendChild(Node_Entity);
            xEle_Artwork.AppendChild(Node_Reln);

            return;
        }

        private void DeleteOldGEFiles(String ValueType, String Status, String GEValue)
        {
            if (!String.IsNullOrEmpty(ValueType))
            {
                switch (ValueType.ToUpper())
                {
                    case "UPLOAD":
                    case "ASSET":
                        if (Status.ToUpper() == "DONE")
                        {
                            File.Delete(GEValue);
                        }
                        break;                    
                }
            }
        }

        /// <summary>
        /// This method is used to process the specified file and perform the actions crop/resize/flip etc
        /// as specified by the dimensions.
        /// </summary>
        /// <param name="InputPath"></param>
        /// <param name="Dimensions"></param>
        /// <param name="TargetFormat"></param>
        /// <param name="str_FilePresentOn"></param>
        /// <returns></returns>
        public String processFile(ref String InputPath, String Dimensions, String TargetFormat, String str_FilePresentOn)
        {
            #region member variable
            String          str_OutputPath  = null;
            StringBuilder   sb_ActionList   = null;
            Int32           SequenceNumber  = 1;
            AssetRendition  ObjRendition    = null;
            String          ErrorXML        = null;
            XPathDocument   xp_ImageActions = null;
            XPathNavigator  xpNav_ImageActionList = null;
            String          str_ActionList  = null;
            String          str_DestPath    = null;
            StringBuilder sb_FileCollectionXML = new StringBuilder(100);
            #endregion

            str_ActionList          = String.Format("<ImageActionList>{0}</ImageActionList>", Dimensions);
            xp_ImageActions         = new XPathDocument(new StringReader(str_ActionList));
            xpNav_ImageActionList   = xp_ImageActions.CreateNavigator();
            xpNav_ImageActionList   = xpNav_ImageActionList.SelectSingleNode("ImageActionList");

            sb_ActionList           = new StringBuilder(100);
            sb_ActionList.Append("<ACTIONLIST>");
            foreach (XPathNavigator xpNav_Action in xpNav_ImageActionList.Select("*"))
            {
                switch (xpNav_Action.LocalName.ToUpper())
                {
                    case "FLIPACTION":
                        if (xpNav_Action.GetAttribute("Direction","") == "0")
                            sb_ActionList.AppendFormat("<ACTION SEQUENCENUMBER='{0}' NAME='flip' ARGUMENT='0'/>", SequenceNumber);                                
                        else
                            sb_ActionList.AppendFormat("<ACTION SEQUENCENUMBER='{0}' NAME='flip' ARGUMENT='1'/>", SequenceNumber);     
                        SequenceNumber++;
                        break;
                    case "ROTATEACTION":
                        sb_ActionList.AppendFormat("<ACTION SEQUENCENUMBER='{0}' NAME='rotate' ARGUMENT='{1}'/>", SequenceNumber, xpNav_Action.GetAttribute("Angle",""));
                        SequenceNumber++;
                        break;
                    case "CROPACTION":
                        sb_ActionList.AppendFormat("<ACTION SEQUENCENUMBER='{0}' NAME='crop' ARGUMENT='{1}x{2}+{3}+{4}'/>", SequenceNumber, xpNav_Action.GetAttribute("width", ""),
                            xpNav_Action.GetAttribute("height", ""), xpNav_Action.GetAttribute("x", ""), xpNav_Action.GetAttribute("y", ""));
                        SequenceNumber++;
                        break;
                    case "RESIZEACTION":
                        sb_ActionList.AppendFormat("<ACTION SEQUENCENUMBER='{0}' NAME='resize' ARGUMENT='{1}x{2}'/>", SequenceNumber, xpNav_Action.GetAttribute("width", ""), xpNav_Action.GetAttribute("height", ""));
                        SequenceNumber++;
                        break;
                }
                
            }
            sb_ActionList.Append("</ACTIONLIST>");

            //copy file to Media server if it is present on webservices machine
            if (ALCommon.IsMediaServerActivated(TenantName) && String.Compare(str_FilePresentOn, "MEDIASERVER", true) != 0 
                && !IsMediaServerAndServicesTogether())
            {
                FileTransferInfrastructure objFileTransfer  = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                objFileTransfer.ToUseServiceURL             = ALCommon.GetWebServiceURL("ALMEDIASERVERMTOMWS");
                sb_FileCollectionXML.AppendFormat(@"<FILECOLLECTION><FILE>
                                                        <NAME>{1}.jpg</NAME>
                                                        <PATH>{0}</PATH>
                                                        <DESTINATIONPATH>{0}</DESTINATIONPATH>
                                                    </FILE></FILECOLLECTION>", ALCommon.TrimAndReplaceSpecialChars(InputPath),ALCommon.TrimAndReplaceSpecialChars(Path.GetFileName(InputPath)));

                String str_FileXML      = sb_FileCollectionXML.ToString();
                ErrorXML                = objFileTransfer.SaveFile(ref str_FileXML);
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    str_DestPath    = ALCommon.GetInnerValue(ref str_FileXML, "<NEWPATH>", "</NEWPATH>");
                    InputPath       = str_DestPath;
                }
                else
                    return ErrorXML;
            }

            ObjRendition    = new AssetRendition(UserID, Language, TenantName);
            ObjRendition.ProcessingConditions = "<USE_FALTTEN_ATTRIBUTE>FALSE</USE_FALTTEN_ATTRIBUTE>";
            ErrorXML        = ObjRendition.DynamicRendering(InputPath, 0, 0, 0, TargetFormat, false, sb_ActionList.ToString(), out str_OutputPath);
            if (String.IsNullOrEmpty(ErrorXML))
                InputPath = str_OutputPath;

            return ErrorXML;
        }

        /// <summary>
        /// This method will check whether Media Server and Services are installed
        /// on same machine or not
        /// </summary>
        /// <returns></returns>
        public bool IsMediaServerAndServicesTogether()
        {
            bool IsMSAndServicesTogether        = false;
            String str_WebServicesInstalled     = null;
            String str_MediaServer_Installed    = null;

            str_MediaServer_Installed   = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "MediaServer Installed");
            str_WebServicesInstalled    = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "WebServicesInstalled");

            if (str_MediaServer_Installed == "1" && str_WebServicesInstalled == "1")
                IsMSAndServicesTogether = true;

            return IsMSAndServicesTogether;
        }

        private String processAndGetFileToArtworkJobFolder(String FilePath, String str_FileName, String str_Dimensions, String str_FilePresentOn)
        {
            #region Member Variable
            Boolean     IsMediaServerActivated  = false;
            String      str_FolderPath          = null;
            String      ErrorXML                = null;
            String      str_TargetFormat        = null;
            StringBuilder sb_GetFileCollectionXML = null;
            FileTransferInfrastructure ObjFT_MS = null;
            String      str_FilePath            = null;
            #endregion

            #region Process file according to the crop dimensions specified
            //str_Dimensions = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref AssetDetailsXML, "<DIMENSIONS>", "</DIMENSIONS>"));
            str_FilePath            = FilePath;
            ObjFT_MS                = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
            sb_GetFileCollectionXML = new StringBuilder(100);
            IsMediaServerActivated  = ALCommon.IsMediaServerActivated(TenantName);
            str_FolderPath          = FilePath.Substring(0, FilePath.LastIndexOf("\\"));
            str_TargetFormat        = str_FileName.Substring(str_FileName.LastIndexOf(".") + 1);

            ErrorXML                = processFile(ref str_FilePath, str_Dimensions, str_TargetFormat, str_FilePresentOn);
            if (String.IsNullOrEmpty(ErrorXML))
            {
                if (IsMediaServerActivated && !IsMediaServerAndServicesTogether())
                {
                    //Get File to artwork cache folder from Media server
                    ObjFT_MS.ToUseServiceURL    = ALCommon.GetWebServiceURL("ALMEDIASERVERMTOMWS");
                    sb_GetFileCollectionXML.AppendFormat(@"<FILECOLLECTION>
                                                                <FILE>                                                                                
                                                                    <SOURCEPATH>{0}</SOURCEPATH>
                                                                    <DESTINATIONPATH>{1}\\{2}</DESTINATIONPATH>
                                                                </FILE>
                                                                </FILECOLLECTION>", ALCommon.TrimAndReplaceSpecialChars(str_FilePath),
                                                      str_FolderPath, ALCommon.TrimAndReplaceSpecialChars(str_FileName));

                    String str_FileXML      = sb_GetFileCollectionXML.ToString();
                    ErrorXML                = ObjFT_MS.GetFile(ref str_FileXML);
                    if (String.IsNullOrEmpty(ErrorXML))
                        FilePath = str_FolderPath + "\\" + str_FileName;
                }
                else
                {
                    File.Copy(str_FilePath, str_FolderPath + "\\" + str_FileName, true);
                    FilePath = str_FolderPath + "\\" + str_FileName;
                }

            }
            #endregion
            return FilePath;
        }

        public ERRORXML UpdateArtworkCacheOriginalFile(String ArtworkID)
        {
            ERRORXML ErrorXML = null;
            String ArtworkCacheFolderPath = null;
            String FormatCacheFolderPath = null;
            String FormatId = null;

            Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkEngine ArtworkEngineObj = new Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkEngine(UserID, Language, TenantName);
            ArtworkEngineObj.GetFormatIDForArtwork(ArtworkID, out FormatId);

            ArtworkCacheFolderPath = GetCacheFolderPath(TenantName, ArtworkID, ArtworkProducerCache.CacheType.ArtworkCache) + "\\Input";
            FormatCacheFolderPath = GetCacheFolderPath(TenantName, FormatId, ArtworkProducerCache.CacheType.FormatCache);

            if (!Directory.Exists(ArtworkCacheFolderPath))
            {
                bool bretVal = ValidateFormatCache(FormatId);
                if (bretVal)
                {
                    bretVal = ValidateArtworkCacheWithFormatCache(ArtworkID, FormatId);
                    if (!bretVal)
                        ErrorXML = PrepareArtworkCache(ArtworkID, "EDIT");
                }
                else
                {
                    PrepareFormatCache(FormatId);
                    ErrorXML = PrepareArtworkCache(ArtworkID, "EDIT");
                }
            }
            else
            {
                ArtworkCacheFolderPath = ArtworkCacheFolderPath + "\\Main.html";
                FormatCacheFolderPath = FormatCacheFolderPath + "\\Original\\Main.html";
                if (System.IO.File.Exists(FormatCacheFolderPath))
                    System.IO.File.Copy(FormatCacheFolderPath, ArtworkCacheFolderPath, true);
            }

            return ErrorXML;
        }
    }
}
