#region Standard Include
using System;
using System.Text;
using System.Xml;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
# endregion
#region Assetlink Include
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.Services.Engines;
using Assetlink.Services.Engines.DirectoryEngine;
using Assetlink.IOProxy;
#endregion 



//--------------------------------------------------------------------------------------------
//  Class   Name: Vault
//  Created On  : 12th September 2006
//  Author      : 
//
//
//
//-------------------------------------------------------------------------------------------

namespace Assetlink.Services.Engines.DAMEngine
{
    using XML       = System.String;
    using ERRORXML  = System.String;
    using ENTITYID  = System.UInt64;

    public class Vault : ALBaseEngine
    {
        #region Attributes
        
        private OperationQueue      QMQueueObj             = null;
        private Operation           OperObj                = null;
        private OMInterface         OMIObj                 = null;
        private ERRORXML            ReturnErrorXML         = null;
        private SearchHelper        SearchObj              = null;
        private ALEngineError       ALEngineErrorObj       = null;
        private CommonFunctions     CommonFunctionsObj     = null;        
        private StringDictionary    strdict_AdditionalInfo = null;
        private Boolean             isMediaServerActivated = false;
        private String              DeleteFolder_ErrorXml  = null;
        #endregion

        public enum ObjectType
        {
            SECTION =0,
            VAULT   =1
        }        

        #region Constructors
        public Vault(UInt64 UserID, String Language, String TenantName):base(UserID,Language,TenantName)
        {
            ALEngineErrorObj        = new ALEngineError();
            CommonFunctionsObj      = new CommonFunctions(UserID, Language, TenantName);
            isMediaServerActivated  = ALCommon.IsMediaServerActivated(TenantName);
        }
        #endregion
       
        #region Methods
        
        /// <alsummary>
        /// <createddate> 12th September 2006 </createddate>
        /// <author> Mr. Anand Mahadik </author>
        /// <purpose> Create System/Non-System Vaults. </purpose>
        /// <xmlstructure><![CDATA[
        /// <VAULT>
        ///  <NAME/>
        ///  <MOUNTPOINT/>
        ///  <MAXALLOWEDDISKSPACE/>
        ///  <VAULTTYPE/>
        ///  <CONFIGUARATIONXML/>
        ///  <VAULT_SECURITYPOLICY TYPE="RELATION"/>
        /// </VAULT>
        /// ]]></xmlstructure>
        /// </alsummary>
        public ERRORXML CreateVault(ref XML VaultXML)
        {
            XML                 OperationXML            = null;
            ERRORXML            ErrorXML                = null;
            XmlDocument         VaultXMLDoc             = null;
            XmlDocument         SectionXMLDoc           = null;
            XmlNode             VaultNode               = null;
            XmlNode             VaultIDNode             = null;
            XmlNode             VaultSectionRelNode     = null;
            XmlNode             SectionNameNode         = null;
            XmlNode             SectionNode             = null;
            XmlNode             SectionIDNode           = null;
            XmlNode             SectionPathNode         = null;
            XmlNode             SectionVaultRelNode     = null;
            XmlNode             IsRootSectionNode       = null;
            XmlNode             SectionOwnerRelNode     = null;
            XmlNode             TotalUsedDiskSpaceNode  = null;
            String              VaultName               = null;
            String              SecurityPolicyID        = null;
            ulong               SectionID               = 0;
            ulong               VaultID                 = 0;
            XML                 SectionXML              = null;
            String              MountPoint              = null;
            XmlNode             DescriptionNode         = null;
            Section             SectionObj              = null;
            XmlNode             MarkedAsDeletedNode     = null;
            XmlNode             IsSystemVaultNode       = null;
            String              vaultType               = null;
            StringBuilder       MountPointWithoutSpace  = null;
            String[]            FolderNames             = null;
            XmlNode             MountPointNode          = null;
            XmlNode             AssetCountInPathNode    = null;
            XmlNode             AssetCountInSectionNode = null;
            XmlNode             OnMediaServerNode       = null;
            Boolean             UseMediaServer          = false;
            Boolean             VaultFolderCreatedInAPI = false;
            DirectoryOperations DirectoryOperationsObj  = null; 
                     
            try
            {
                VaultXMLDoc             = new XmlDocument();
                SectionXMLDoc           = new XmlDocument();
                QMQueueObj              = new OperationQueue(true);
                DeleteFolder_ErrorXml   = null;
                VaultXMLDoc.LoadXml(VaultXML);
                VaultNode               = VaultXMLDoc.DocumentElement;

                TotalUsedDiskSpaceNode  = VaultXMLDoc.CreateElement("TOTALUSEDSPACE");
                TotalUsedDiskSpaceNode.InnerText = "0";
                VaultNode.AppendChild(TotalUsedDiskSpaceNode);
                
                VaultName               = VaultNode.SelectSingleNode("NAME").InnerText;
                vaultType               = VaultNode.SelectSingleNode("VAULTTYPE").InnerText;

                #region Check Whether to create vault on Media Server or Not
                if (isVaultForArtworkProducer(VaultNode))
                    UseMediaServer = false;
                else
                    UseMediaServer = CheckforUseMediaServer(VaultNode);

                DirectoryOperationsObj  = new DirectoryOperations(UserID, Language, TenantName, UseMediaServer);

                if (VaultNode.SelectSingleNode("ONMEDIASERVER") == null)
                {
                    OnMediaServerNode           = VaultXMLDoc.CreateElement("ONMEDIASERVER");
                    OnMediaServerNode.InnerText = UseMediaServer.ToString().ToUpper();
                    VaultNode.AppendChild(OnMediaServerNode);
                }
                #endregion

                VaultXML                = VaultNode.OuterXml;

                if (VaultAlreadyExists(VaultName))
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTWITHTHISNAMEEXISTS", this.GetType().ToString(), "CreateVault", null, null);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                MountPointNode          = VaultNode.SelectSingleNode("MOUNTPOINT");
                MountPoint              = MountPointNode.InnerText;

                // Remove Leading and Traling space characters form any of the folder name.
                if (MountPoint.Contains(" \\") || MountPoint.Contains("\\ "))
                {
                    MountPointWithoutSpace  = new StringBuilder();
                    FolderNames             = MountPoint.Split('\\');
                    for (int i = 0; i < FolderNames.Length; i++)
                    {
                        if (((i + 1) == FolderNames.Length) && (!MountPoint.EndsWith("\\")))
                        {
                            MountPointWithoutSpace.AppendFormat("{0}", FolderNames[i].Trim());
                        }
                        else
                        {
                            MountPointWithoutSpace.AppendFormat("{0}\\", FolderNames[i].Trim());
                        }
                    }
                    MountPoint                  = MountPointWithoutSpace.ToString();
                    MountPointNode.InnerText    = MountPoint;
                }

                // This check has been added to see whether the MountPoint for the new vault that is getting
                // create is not a sub dir of any existing vaults MountPoint. Because if the vault gets deleted then both the 
                // MountPoints get deleted.
                if (CheckIfMountPointAlreadyUsed(MountPoint))
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_MOUNTPOINTALREADYUSED", this.GetType().ToString(), "CreateVault", null, null);
                    ReturnErrorXML          = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                if (DirectoryOperationsObj.IsFolderExists(MountPoint))
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_MOUNTPOINTALREADYEXSISTS", this.GetType().ToString(), "CreateVault", null, null);
                    ReturnErrorXML          = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                SecurityPolicyID            = VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY").InnerText;
                VaultNode.RemoveChild(VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY"));
                if (VaultNode.SelectSingleNode("VAULTID") == null)
                {
                    UniqueID.Read(ref VaultID);
                    VaultIDNode             = VaultXMLDoc.CreateElement("VAULTID");
                    VaultIDNode.InnerText   = VaultID.ToString();
                    VaultNode.AppendChild(VaultIDNode);
                }
                else
                {
                    VaultID                 = Convert.ToUInt64(VaultNode.SelectSingleNode("VAULTID").InnerText);
                }

                SectionNode             = SectionXMLDoc.CreateElement("SECTION");
                SectionIDNode           = SectionXMLDoc.CreateElement("SECTIONID");
                UniqueID.Read(ref SectionID);
                SectionIDNode.InnerText = SectionID.ToString();
                SectionNode.AppendChild(SectionIDNode);

                SectionNameNode             = SectionXMLDoc.CreateElement("NAME");
                SectionNameNode.InnerText   = VaultName;
                SectionNode.AppendChild(SectionNameNode);

                if (VaultNode.SelectSingleNode("DESCRIPTION") != null)
                {
                    DescriptionNode           = SectionXMLDoc.CreateElement("DESCRIPTION");
                    DescriptionNode.InnerText = VaultNode.SelectSingleNode("DESCRIPTION").InnerText;
                    SectionNode.AppendChild(DescriptionNode);
                }

                if (VaultNode.SelectSingleNode("CONFIGURATIONXML") != null)
                {
                    SectionNode.AppendChild(SectionXMLDoc.ImportNode(VaultNode.SelectSingleNode("CONFIGURATIONXML"),true));
                    VaultNode.RemoveChild(VaultNode.SelectSingleNode("CONFIGURATIONXML"));
                }

                SectionPathNode                 = SectionXMLDoc.CreateElement("PATH");
                SectionPathNode.InnerText       = VaultName;
                SectionNode.AppendChild(SectionPathNode);

                IsRootSectionNode               = SectionXMLDoc.CreateElement("ISROOTSECTION");
                IsRootSectionNode.InnerText     = "1";
                SectionNode.AppendChild(IsRootSectionNode);

                MarkedAsDeletedNode             = SectionXMLDoc.CreateElement("MARKEDASDELETED");
                MarkedAsDeletedNode.InnerText   = "0";
                SectionNode.AppendChild(MarkedAsDeletedNode);

                IsSystemVaultNode               =  SectionXMLDoc.CreateElement("ISSYSTEMVAULT");
                IsSystemVaultNode.InnerText     = CommonFunctionsObj.GetVaultTypeEnum(vaultType);
                SectionNode.AppendChild(IsSystemVaultNode);

                AssetCountInPathNode            = SectionXMLDoc.CreateElement("ASSETCOUNTINPATH");
                AssetCountInPathNode.InnerText  = "0";
                SectionNode.AppendChild(AssetCountInPathNode);

                AssetCountInSectionNode             = SectionXMLDoc.CreateElement("ASSETCOUNTINSECTION");
                AssetCountInSectionNode.InnerText   = "0";
                SectionNode.AppendChild(AssetCountInSectionNode);

                VaultSectionRelNode             = SectionXMLDoc.CreateElement("VAULT_ROOTSECTION");
                ((XmlElement)VaultSectionRelNode).SetAttribute("TYPE", "RELATION");
                ((XmlElement)VaultSectionRelNode).SetAttribute("PARENTID", "TRUE");
                VaultSectionRelNode.InnerText   = VaultID.ToString();
                SectionNode.AppendChild(VaultSectionRelNode);

                SectionVaultRelNode             = SectionXMLDoc.CreateElement("SECTION_VAULT");
                ((XmlElement)SectionVaultRelNode).SetAttribute("TYPE", "RELATION");
                ((XmlElement)SectionVaultRelNode).SetAttribute("PARENTID", "TRUE");
                SectionVaultRelNode.InnerText   = VaultID.ToString();
                SectionNode.AppendChild(SectionVaultRelNode);

                SectionOwnerRelNode             = SectionXMLDoc.CreateElement("SECTION_OWNER");
                ((XmlElement)SectionOwnerRelNode).SetAttribute("TYPE", "RELATION");
                SectionOwnerRelNode.InnerText   = UserID.ToString();
                SectionNode.AppendChild(SectionOwnerRelNode);

                //// Create root section for the vault
                // Here we have to add Operation for Section creation before operations nodes of security policy
                // because when section is created OM adds access control entry for Everyone and this we need to remove and add our entries.
                // So 1st section must get created and then Access control entries must be added.
                SectionXML  = SectionNode.OuterXml;
                OperObj     = new Operation("ENTITY", "CREATEDATA");
                OperObj.AddParameter("INPUTXML", SectionXML);
                QMQueueObj.AddOperation(OperObj);

                SectionObj  = new Section(UserID, Language, TenantName);
                ErrorXML    = SectionObj.SetAccessRights(SectionID,UserID.ToString(), Convert.ToUInt64(SecurityPolicyID), DAM_SecurityPolicyLevel.CURRENTSECTION, ref QMQueueObj,false);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_SETTINGSECURITYPOLICYFAILED", this.GetType().ToString(), "CreateVault", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                ErrorXML = CreateFolder(ref DirectoryOperationsObj, MountPoint, "CreateVault");
                if (ErrorXML != null)
                    return ErrorXML;
                else
                    VaultFolderCreatedInAPI = true;

                //// Create vault
                VaultXML    = VaultNode.OuterXml;
                OperObj     = new Operation("ENTITY", "CREATEDATA");
                OperObj.AddParameter("INPUTXML", VaultXML);
                QMQueueObj.AddOperation(OperObj);

                OperationXML= QMQueueObj.Xml;

                OMIObj      = new OMInterface(UserID, Language, TenantName);
                ErrorXML    = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    //No need to change this call, since it will call the WindowsAssetManager delete call on Media Server and we are making a call to
                    //ALCommonExtension Delete call from WindowsAssetManager                    
                    DeleteFolder_ErrorXml = DirectoryOperationsObj.DeleteFolder(MountPoint);
                    if (!String.IsNullOrEmpty(DeleteFolder_ErrorXml))
                    { 
                        //Log Error details using common Logging for Delete Folder
                        LogErrorForDeleteFolder("CreateVault", "", VaultName, MountPoint, UseMediaServer.ToString(), DeleteFolder_ErrorXml);
                    }
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTCREATIONFAILED", this.GetType().ToString(), "CreateVault", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                if (VaultFolderCreatedInAPI)
                {
                    //No need to change this call, since it will call the WindowsAssetManager delete call on Media Server and we are making a call to
                    //ALCommonExtension Delete call from WindowsAssetManager                    
                    DeleteFolder_ErrorXml = DirectoryOperationsObj.DeleteFolder(MountPoint);
                    if (!String.IsNullOrEmpty(DeleteFolder_ErrorXml))
                    {
                        //Log Error details using common Logging for Delete Folder
                        LogErrorForDeleteFolder("CreateVault", "", VaultName, MountPoint, UseMediaServer.ToString(), DeleteFolder_ErrorXml);
                    }                    
                }

                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTCREATIONFAILED", this.GetType().ToString(), "CreateVault", ExcepObj, null);

                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);

                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            return null;                   
        }

        /// <summary>
        /// This function check if the vault name is of Artwork producer application.
        /// </summary>
        /// <param name="vaultName">Name of the vault</param>
        /// <returns>return True if valur is of of Artwork producer application else false</returns>
        private bool isVaultForArtworkProducer(XmlNode VaultNode)
        {
            String      vaultName   = null;
            ALCommon    aLCommonObj = null;
            String      apVaultName = null;
            try
            {
                
                aLCommonObj = new ALCommon();
                vaultName   = VaultNode.SelectSingleNode("NAME").InnerText;
                aLCommonObj.GetTenantSetting(TenantName, "CONFIGURATION/ARTWORKPRODUCER/VAULTNAME", out apVaultName);
                
                if (apVaultName.ToUpper() == vaultName.ToUpper())
                    return true;
                else
                    return false;
                
            }
            catch 
            {
                return false;
            }

        }

        /// <alsummary>
        /// <createddate> 12th September 2006 </createddate>
        /// <author> Mr. Anand Mahadik </author>
        /// <purpose> Deletes System/Non-System Vaults. </purpose>
        /// <xmlstructure><![CDATA[
        /// <VAULTCOLLECTION>
        ///  <VAULT>
        ///   <VAULTID/>
        ///  </VAULT>
        /// <VAULTCOLLECTION/>
        /// ]]></xmlstructure>
        /// </alsummary>
        public ERRORXML DeleteVault(ENTITYID VaultID)
        {
            #region Member Variable
            XML             ErrorXML                    = null;
            XML             RootSectionXML              = null;
            XML             OperationXML                = null;
            XmlDocument     RootSectionXMLDoc           = null;
            XmlNode         RootSectionNode             = null;
            String          MountPoint                  = null;
            XmlNode         VaultNode                   = null;
            StringBuilder   sb_NSearchInputXML          = null;
            DirectoryOperations DirectoryOperationsObj  = null;
            String          OnMediaServer               = null;            
            #endregion

            try
            {
                OMIObj = new OMInterface(UserID, Language, TenantName);
                DeleteFolder_ErrorXml = null;
                # region Retriving RootSection(along with its Immidiate Subsections and DigitalAssets) of a vault
                sb_NSearchInputXML          = new StringBuilder(1000);
                sb_NSearchInputXML.AppendFormat(@"<SEARCH>
	                                                <SESSIONINFO TenantName='{0}' Language='{1}' UserID='{2}' SessionID='{3}'/>
	                                                <TARGETOBJECT>SECTION</TARGETOBJECT>
	                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                                <TARGETENTITY>
		                                                <FILTERLIST>
			                                                <FILTER>
				                                                <FILTERID>1</FILTERID>
				                                                <EXPRESSION>VAULTID</EXPRESSION>
				                                                <OPERATOR>EQUALTO</OPERATOR>
				                                                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
				                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                <RELATIONSHIPNAME>VAULT_ROOTSECTION</RELATIONSHIPNAME>
			                                                </FILTER>				                                            
		                                                </FILTERLIST>                                                        
		                                                <RELATIONSHIP NAME='SECTION_VAULT'>
			                                                <SELECTLIST>
				                                                <EXPRESSION>
					                                                <FIELDNAME>MOUNTPOINT</FIELDNAME>
					                                                <ALIAS>MOUNTPOINT</ALIAS>
				                                                </EXPRESSION>	
                                                                <EXPRESSION>
					                                                <FIELDNAME>ONMEDIASERVER</FIELDNAME>
					                                                <ALIAS>ONMEDIASERVER</ALIAS>
				                                                </EXPRESSION>
			                                                </SELECTLIST>                                                            
		                                                </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='SECTION_SUBSECTION'>
			                                                <SELECTLIST>
				                                                <EXPRESSION>
					                                                <FIELDNAME>SECTIONID</FIELDNAME>
					                                                <ALIAS>SECTIONID</ALIAS>
				                                                </EXPRESSION>
                                                                <EXPRESSION>
					                                                <FIELDNAME>MARKEDASDELETED</FIELDNAME>
					                                                <ALIAS>MARKEDASDELETED</ALIAS>
				                                                </EXPRESSION>
			                                                </SELECTLIST>                                                             
		                                                </RELATIONSHIP>		                                                
	                                                    <RELATIONSHIP NAME='SECTION_DIGITALASSET'>
			                                                <SELECTLIST>
				                                                <EXPRESSION>
					                                                <FIELDNAME>DIGITALASSETID</FIELDNAME>
					                                                <ALIAS>DIGITALASSETID</ALIAS>
				                                                </EXPRESSION>
                                                                <EXPRESSION>
					                                                <FIELDNAME>MARKEDASDELETED</FIELDNAME>
					                                                <ALIAS>MARKEDASDELETED</ALIAS>
				                                                </EXPRESSION>                                                                
			                                                </SELECTLIST>                                                            
		                                                </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='SECTION_COLLECTION'>
			                                                <SELECTLIST>
				                                                <EXPRESSION>
					                                                <FIELDNAME>COLLECTIONID</FIELDNAME>
					                                                <ALIAS>COLLECTIONID</ALIAS>
				                                                </EXPRESSION>
                                                                <EXPRESSION>
					                                                <FIELDNAME>MARKEDASDELETED</FIELDNAME>
					                                                <ALIAS>MARKEDASDELETED</ALIAS>
				                                                </EXPRESSION>                                                                
			                                                </SELECTLIST>                                                            
		                                                </RELATIONSHIP>", TenantName, Language, UserID.ToString(), SessionID.ToString(), VaultID.ToString());
                           sb_NSearchInputXML.Append(@"</TARGETENTITY>
                                                    </SEARCH>");
                ErrorXML         = OMIObj.GetHierarchialData(sb_NSearchInputXML.ToString(),ref RootSectionXML);                
                if (ErrorXML != null)
                {
                    ReturnErrorXML  = ALEngineError.PrepareErrorXML(ErrorXML);
                    ExceptionTracing.TraceLog("Error in DeleteVault", "DAMEngine",
                                           TenantName, DateTime.Now);
                    return ReturnErrorXML;
                }
                else
                {
                    RootSectionXMLDoc   = new XmlDocument();
                    RootSectionXMLDoc.LoadXml(RootSectionXML);
                    RootSectionNode     = RootSectionXMLDoc.SelectSingleNode("SECTIONCOLLECTION/SECTION");

                    if ((RootSectionNode.SelectSingleNode("SECTION_SUBSECTION/SECTION") != null) || (RootSectionNode.SelectSingleNode("SECTION_DIGITALASSET/DIGITALASSET") != null)
                        || (RootSectionNode.SelectSingleNode("SECTION_COLLECTION/COLLECTION")!= null))
                    {
                        if ((RootSectionNode.SelectNodes("SECTION_SUBSECTION/SECTION/MARKEDASDELETED[text()='0']").Count > 0) || (RootSectionNode.SelectNodes("SECTION_DIGITALASSET/DIGITALASSET/MARKEDASDELETED[text()='0']").Count > 0)
                                   || (RootSectionNode.SelectNodes("SECTION_COLLECTION/COLLECTION/MARKEDASDELETED[text()='0']").Count > 0))                        
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTNOTEMPTY", this.GetType().ToString(), "DeleteVault", null, null);
                        else 
                            if ((RootSectionNode.SelectNodes("SECTION_SUBSECTION/SECTION/MARKEDASDELETED[text()='1']").Count > 0) || (RootSectionNode.SelectNodes("SECTION_DIGITALASSET/DIGITALASSET/MARKEDASDELETED[text()='1']").Count > 0)
                                   || (RootSectionNode.SelectNodes("SECTION_COLLECTION/COLLECTION/MARKEDASDELETED[text()='1']").Count > 0))
                                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_CONTENTSMARKEDFORDELETE", this.GetType().ToString(), "DeleteVault", null, null);
                        else 
                            if ((RootSectionNode.SelectNodes("SECTION_SUBSECTION/SECTION/MARKEDASDELETED[text()='2']").Count > 0) || (RootSectionNode.SelectNodes("SECTION_DIGITALASSET/DIGITALASSET/MARKEDASDELETED[text()='2']").Count > 0)
                                   || (RootSectionNode.SelectNodes("SECTION_COLLECTION/COLLECTION/MARKEDASDELETED[text()='2']").Count > 0))
                                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_CONTENTSMARKEDFORPERMANANTDELETE", this.GetType().ToString(), "DeleteVault", null, null);

                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }

                    VaultNode = RootSectionNode.SelectSingleNode("SECTION_VAULT/VAULT");
                    if (VaultNode != null)
                    {
                        DirectoryOperationsObj = new DirectoryOperations(UserID, Language, TenantName, CheckforUseMediaServer(VaultNode));
                        MountPoint = VaultNode.SelectSingleNode("MOUNTPOINT").InnerText;
                        OnMediaServer = VaultNode.SelectSingleNode("ONMEDIASERVER").InnerText;
                    }         
                    
                }                
                # endregion

                # region Delete Vault

                QMQueueObj      = new OperationQueue(true);
                OperObj         = new Operation("ENTITY", "DELETE");
                OperObj.AddParameter("ENTITYID",VaultID.ToString());
                OperObj.AddParameter("ENTITYTYPENAME","VAULT");
                QMQueueObj.AddOperation(OperObj);

                OperationXML    = QMQueueObj.Xml;

               
                ErrorXML        = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTDELETIONFAILED", this.GetType().ToString(), "DeleteVault", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
                                
                //File.AppendAllText(ALCommon.GetInstallationDir() + "\\Logs\\Vault.txt", "DeleteVault Before DeleteFolderCall MountPoint: " + MountPoint);
                
                //No need to change this call, since it will call the WindowsAssetManager delete call on Media Server and we are making a call to
                //ALCommonExtension Delete call from WindowsAssetManager                
                ErrorXML = DirectoryOperationsObj.DeleteFolder(MountPoint, true);

                //File.AppendAllText(ALCommon.GetInstallationDir() + "\\Logs\\Vault.txt", "DeleteVault after DeleteFolderCall ErrorXML: " + ErrorXML);
                //Log Error details using common Logging for Delete Folder
                //LogErrorForDeleteFolder("DeleteVault", VaultID.ToString(), "", MountPoint, OnMediaServer, ErrorXML);

                if (ErrorXML != null)
                {                    
                    //Log Error details using common Logging for Delete Folder
                    LogErrorForDeleteFolder("DeleteVault", VaultID.ToString(), "", MountPoint, OnMediaServer, ErrorXML);
                    
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTDELETIONFAILED", this.GetType().ToString(), "DeleteVault", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
                # endregion
            }
            catch (Exception ExcepObj)
            {
                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTDELETIONFAILED", this.GetType().ToString(), "DeleteVault", ExcepObj, null);
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                File.AppendAllText(ALCommon.GetInstallationDir() + "\\Logs\\Vault.txt", ReturnErrorXML);
                
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                               
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return null;
        }

        /// <alsummary>
        /// <createddate> 12th September 2006 </createddate>
        /// <author> Mr. Anand Mahadik </author>
        /// <purpose> Updates vault information.</purpose>
        /// <xmlstructure><![CDATA[
        /// <VAULT>
        ///  <VAULTID/>
        ///  <NAME/>
		///  <MOUNTPOINT/>
        ///  <VAULTTYPE/>
        ///  <MAXALLOWEDDISKSPACE/>
        /// </VAULT>
        /// ]]></xmlstructure>
        /// </alsummary>
        public ERRORXML UpdateVault(XML VaultXML)
        {
            XML                 OperationXML                = null;
            ERRORXML            ErrorXML                    = null;
            ERRORXML            ReturnErrorXML              = null;
            XmlDocument         VaultXMLDoc                 = null;
            XmlDocument         SectionXMLDoc               = null;
            XmlNode             VaultNode                   = null;
            XmlNode             SectionDescriptionNode      = null;
            XmlNode             SecurityPolicyNode          = null;
            String              VaultID                     = null;
            XML                 RootSectionXML              = null;
            XmlDocument         RootSectionXMLDoc           = null;
            XmlNode             RootSectionNode             = null;
            String              MountPoint                  = null;
            Section             SectionObj                  = null;
            XmlNode             SectionNameNode             = null;
            String              RootSectionID               = null;
            String              SecurityPolicyID            = null;
            XmlNode             OwnerIDNode                 = null;
            String              OldVaultType                = null;
            String              NewVaultType                = null;
            String              SectionPath                 = null;
            DirectoryOperations DirectoryOperationsObj      = null;
            bool                OnMediaServer               = false;
            try
            {
                VaultXMLDoc             = new XmlDocument();
                SectionXMLDoc           = new XmlDocument();
                QMQueueObj              = new OperationQueue(true);
                strdict_AdditionalInfo  = new StringDictionary();
                DeleteFolder_ErrorXml   = null;
                VaultXMLDoc.LoadXml(VaultXML);
                VaultNode           = VaultXMLDoc.DocumentElement;

                VaultID             = VaultNode.SelectSingleNode("VAULTID").InnerText;
                
                if (VaultNode.SelectSingleNode("MOUNTPOINT") != null)
                {
                    MountPoint = VaultNode.SelectSingleNode("MOUNTPOINT").InnerText;
                    // This check has been added to see whether the MountPoint of the vault getting
                    // updated is not a sub dir of any existing vault's MountPoint. Because if the vault gets deleted then both the 
                    // MountPoints get deleted.
                    if (CheckIfMountPointAlreadyUsed(MountPoint))
                    {
                        ALEngineErrorObj.AppendError("DAMENGINE_VAULT_MOUNTPOINTALREADYUSED", this.GetType().ToString(), "UpdateVault", null, null);
                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }
                }

                if (VaultNode.SelectSingleNode("VAULTTYPE") != null)
                {
                    NewVaultType = VaultNode.SelectSingleNode("VAULTTYPE").InnerText;
                }

                # region Get the root section of the vault
                SearchObj                   = new SearchHelper(UserID, Language, TenantName);
                SearchObj.SearchAllFacets   = false;
                SearchObj.SortColumnName    = "SECTIONID";
                SearchObj.SortOrder         = "ASC";
                SearchObj.MaxResultSize     = 1;

                SearchObj.Init("SECTION", "ENTITY");
                SearchObj.AddFilter("VAULTID", "EQUALTO", VaultID,
                                    false, "SECTION.VAULT_ROOTSECTION");

                SearchObj.AddFields("ENTITY",null,"SECTION");
                SearchObj.AddFields("USERID",  "SECTION.SECTION_OWNER");
                SearchObj.AddFields("VAULTTYPE", "SECTION.SECTION_VAULT");
                SearchObj.AddFields("ONMEDIASERVER", "SECTION.SECTION_VAULT");
                SearchObj.AddFields("PATH","SECTION");
                SearchObj.AddFields("NAME", "SECTION");
                SearchObj.AddFields("ENTITY", null, "SECTION.SECTION_SECURITYPOLICY");

                RootSectionXML              = null;
                ErrorXML                    = SearchObj.DoSearch(ref RootSectionXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_ROOTSECTIONREADFAILED", this.GetType().ToString(), "UpdateVault", null, ErrorXML);
                    
                    ReturnErrorXML = ALEngineError.PrepareErrorXML(ErrorXML);
                    ExceptionTracing.TraceLog("Error in UpdateVault", "DAMEngine",
                                           TenantName, DateTime.Now);

                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
                else
                {
                    RootSectionXMLDoc       = new XmlDocument();
                    RootSectionXMLDoc.LoadXml(RootSectionXML);
                    RootSectionNode         = RootSectionXMLDoc.SelectSingleNode("SECTIONCOLLECTION/SECTION");                    

                    #region Check Modify access on Root section
                    if (RootSectionNode != null)
                    {
                        RootSectionID       = RootSectionNode.SelectSingleNode("SECTIONID").InnerText;
                        OwnerIDNode         = RootSectionNode.SelectSingleNode("SECTION_OWNER/USER/USERID");
                        OldVaultType        = RootSectionNode.SelectSingleNode("SECTION_VAULT/VAULT/VAULTTYPE").InnerText;
                        SectionPath         = RootSectionNode.SelectSingleNode("PATH").InnerText;
                        SecurityPolicyNode  = RootSectionNode.SelectSingleNode("SECTION_SECURITYPOLICY/SECURITYPOLICY");

                        ((XmlElement)RootSectionNode).SetAttribute("MODIFIED", "TRUE");
                        SectionObj = new Section(UserID, Language, TenantName);

                        if (OwnerIDNode != null)
                        {
                            if (SecurityPolicyNode != null)
                            {
                                if (!SectionObj.IsAccessAllowed(OwnerIDNode.InnerText, SecurityPolicyNode.SelectSingleNode("POLICYDETAILS").InnerXml, "DW_CREATE_PERMISSION"))
                                {
                                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_CREATEACCESSDENIED", this.GetType().ToString(), "UpdateVault", null, null);
                                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                    return ReturnErrorXML;
                                }
                            }
                            else
                            {
                                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_SECURITYPOLICYNOTFOUND", this.GetType().ToString(), "UpdateVault", null, null);
                                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                return ReturnErrorXML;
                            }
                        }
                        else
                        {
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_ROOTSECTIONOWNERNOTFOUND", this.GetType().ToString(), "UpdateVault", null, null);
                            ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                            return ReturnErrorXML;
                        }
                    }
                    else
                    {
                        ALEngineErrorObj.AppendError("DAMENGINE_VAULT_ROOTSECTIONNOTFOUND", this.GetType().ToString(), "UpdateVault", null, null);
                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }
                    #endregion

                    OnMediaServer = CheckforUseMediaServer(RootSectionNode.SelectSingleNode("SECTION_VAULT/VAULT"));
                    DirectoryOperationsObj = new DirectoryOperations(UserID, Language, TenantName, CheckforUseMediaServer(RootSectionNode.SelectSingleNode("SECTION_VAULT/VAULT")));

                    if (DirectoryOperationsObj.IsFolderExists(MountPoint))
                    {
                        ALEngineErrorObj.AppendError("DAMENGINE_VAULT_MOUNTPOINTALREADYEXSISTS", this.GetType().ToString(), "UpdateVault", null, null);
                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }

                    if(VaultNode.SelectSingleNode("NAME")!= null)
                    {
                        if (RootSectionNode.SelectSingleNode("NAME").InnerText.ToUpper() != VaultNode.SelectSingleNode("NAME").InnerText.ToUpper())
                        {
                            SectionObj.FormOperationNodesForUpdatingPath(RootSectionNode.SelectSingleNode("SECTIONID").InnerText, VaultNode.SelectSingleNode("NAME").InnerText, ref QMQueueObj);

                            if (RootSectionNode.SelectSingleNode("PATH") != null)
                            {
                                RootSectionNode.RemoveChild(RootSectionNode.SelectSingleNode("PATH"));
                            }

                            SectionNameNode = RootSectionNode.SelectSingleNode("NAME");
                            SectionNameNode.InnerText = VaultNode.SelectSingleNode("NAME").InnerText;
                        }

                    }
                    if (VaultNode.SelectSingleNode("DESCRIPTION") != null)
                    {
                        SectionDescriptionNode                  = RootSectionNode.SelectSingleNode("DESCRIPTION");
                        if (SectionDescriptionNode != null)
                        {
                            SectionDescriptionNode.InnerText = VaultNode.SelectSingleNode("DESCRIPTION").InnerText;
                        }
                        else
                        {
                            SectionDescriptionNode              = RootSectionXMLDoc.CreateElement("DESCRIPTION");
                            SectionDescriptionNode.InnerText    = VaultNode.SelectSingleNode("DESCRIPTION").InnerText;
                            RootSectionNode.AppendChild(SectionDescriptionNode);
                        }
                    }

                    //// Add update root section operation
                    //// Root Section update should be done prior to setting new access control on the section entity. 
                    //// As the logged in user may have less access on the security policy being applied.

                    OperObj = new Operation("ENTITY", "UPDATEDATA");
                    OperObj.AddParameter("INPUTXML", RootSectionNode.OuterXml);
                    QMQueueObj.AddOperation(OperObj);

                    if (VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY[@DISASSOCIATE]") != null)
                    {
                        VaultNode.RemoveChild(VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY[@DISASSOCIATE]"));
                    }
                    
                    if (VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY[@TYPE=\"RELATION\"]") != null)
                    {                        
                        SecurityPolicyID    = VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY[@TYPE=\"RELATION\"]").InnerText;

                        VaultNode.RemoveChild(VaultNode.SelectSingleNode("VAULT_SECURITYPOLICY[@TYPE=\"RELATION\"]"));
                        if (OwnerIDNode != null)
                        {
                            ErrorXML        = SectionObj.SetAccessRights(Convert.ToUInt64(RootSectionID),OwnerIDNode.InnerText, Convert.ToUInt64(SecurityPolicyID), DAM_SecurityPolicyLevel.CURRENTSECTION, ref QMQueueObj,false);

                            if (ErrorXML != null)
                            {
                                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_COMPLETEACCESSDENIED", this.GetType().ToString(), "UpdateVault", null, ErrorXML);
                                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                return ReturnErrorXML;
                            }
                        }
                        else
                        {
                            CommonFunctionsObj.FormExtraInformationNode(RootSectionID,
                                                                   SectionNameNode.InnerText,
                                                                   "Section owner does not exists");

                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("SECTIONNAME", SectionNameNode.InnerText);

                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTUPDATIONFAILED", this.GetType().ToString(), "UpdateVault",CommonFunctionsObj.ExtraInformationNode,strdict_AdditionalInfo, null, ErrorXML);
                            ReturnErrorXML  = ALEngineErrorObj.ErrorCollectionXML;
                            return ReturnErrorXML;
                        }
                    }

                    //Pravin.
                    //Updates section's vault type.
                    if  ((!String.IsNullOrEmpty(NewVaultType)) && (String.Compare(OldVaultType, NewVaultType, true) != 0))
                    {
                        FormOperationNodesForUpdatingVaultTypeofSection(SectionPath, NewVaultType, ref QMQueueObj);
                    }
                    //End.
                }

                # endregion

                if (!String.IsNullOrEmpty(MountPoint))
                {
                    ErrorXML = CreateFolder(ref DirectoryOperationsObj, MountPoint, "UpdateVault");
                    if (ErrorXML != null)
                        return ErrorXML;
                }

                //// Add update vault operation
                VaultXML        = VaultNode.OuterXml;
                OperObj         = new Operation("ENTITY", "UPDATEDATA");
                OperObj.AddParameter("INPUTXML", VaultXML);
                QMQueueObj.AddOperation(OperObj);
                               
                // Here we need to impersonate user as admin because user may have access on parent section but could
                // not have access on subsequent section's. As only Admin user can update vault check is not present.

                OperationXML    = QMQueueObj.Xml;
                OMIObj          = new OMInterface(UserID, Language, TenantName);
                ErrorXML        = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    if (MountPoint != null)
                    {
                        //No need to change this call, since it will call the WindowsAssetManager delete call on Media Server and we are making a call to
                        //ALCommonExtension Delete call from WindowsAssetManager                        
                        DeleteFolder_ErrorXml = DirectoryOperationsObj.DeleteFolder(MountPoint);
                        if (!String.IsNullOrEmpty(DeleteFolder_ErrorXml))
                        {
                            //Log Error details using common Logging for Delete Folder
                            LogErrorForDeleteFolder("DeleteVault", VaultID, VaultNode.SelectSingleNode("NAME").InnerText, MountPoint, OnMediaServer.ToString(), DeleteFolder_ErrorXml);
                        }                        
                    }

                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTUPDATIONFAILED", this.GetType().ToString(), "UpdateVault", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
                
            }
            catch (Exception ExcepObj)
            {
                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTUPDATIONFAILED", this.GetType().ToString(), "UpdateVault", ExcepObj, null);
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);

                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return null;
        }

        public ERRORXML PrepareTotalUsedDiskSpaceXml(ENTITYID EntityID,ObjectType Type, ref XML VaultXML)
        {
            ERRORXML            ErrorXML                    = null;
            ERRORXML            ReturnErrorXML              = null;
            XmlDocument         VaultXMLDoc                 = null;
            XmlNode             VaultNode                   = null;
            XmlNode             PathNode                    = null;
            XmlNode             TotalUsedSpaceNode          = null;
            String              Path                        = null;
            DirectoryOperations DirectoryOperationsObj      = null;
            XML                 TempVaultXML                = null;
            ulong               ValutSize                   = 0;
       
            try
            {
                VaultXMLDoc                 = new XmlDocument();
                               
                //// Get vault for the section
                SearchObj                   = new SearchHelper(UserID, Language, TenantName);
                SearchObj.SearchAllFacets   = false;
                SearchObj.SortColumnName    = "VAULTID";
                SearchObj.SortOrder         = "ASC";
                SearchObj.MaxResultSize     = 1;

                SearchObj.Init("VAULT", "ENTITY");

                if(Type == ObjectType.SECTION)
                    SearchObj.AddFilter("SECTIONID", "EQUALTO", EntityID.ToString(),
                                    false, "VAULT.SECTION_VAULT");
                else
                    SearchObj.AddFilter("VAULTID", "EQUALTO", EntityID.ToString(),
                                    false, "VAULT");

                SearchObj.AddFields("ENTITY","MOUNTPOINT","VAULT");
                SearchObj.AddFields("ENTITY", "TOTALUSEDSPACE", "VAULT");
                SearchObj.AddFields("ENTITY", "ONMEDIASERVER", "VAULT");                                

                TempVaultXML                = null;
                ErrorXML                    = SearchObj.DoSearch(ref TempVaultXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTREADFAILED", this.GetType().ToString(), "PrepareUpdateVaultsTotalUsedDiskSpaceXml", null, ErrorXML);

                    ReturnErrorXML = ALEngineError.PrepareErrorXML(ErrorXML);
                    ExceptionTracing.TraceLog("Error in PrepareUpdateVaultsTotalUsedDiskSpaceXml", "DAMEngine",
                                           TenantName, DateTime.Now);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
                else
                {
                    VaultXMLDoc                     = new XmlDocument();
                    VaultXMLDoc.LoadXml(TempVaultXML);
                    VaultNode                       = VaultXMLDoc.SelectSingleNode("VAULTCOLLECTION/VAULT");
                    ((XmlElement)VaultNode).SetAttribute("MODIFIED", "TRUE");
                    PathNode                        = VaultNode.SelectSingleNode("MOUNTPOINT");
                    Path                            = PathNode.InnerText;
                    TotalUsedSpaceNode              = VaultNode.SelectSingleNode("TOTALUSEDSPACE");

                    DirectoryOperationsObj          = new DirectoryOperations(UserID, Language, TenantName, CheckforUseMediaServer(VaultNode));

                    ErrorXML = DirectoryOperationsObj.GetFolderSize(Path, out ValutSize);
                    if (ErrorXML != null)
                    {
                        ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTSIZEFAILED", this.GetType().ToString(), "CreateAssetRendition", null, ErrorXML);
                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }
                    TotalUsedSpaceNode.InnerText    = ValutSize.ToString();
                    
                }

                PathNode = VaultNode.SelectSingleNode("MOUNTPOINT");
                if (PathNode != null)
                    VaultNode.RemoveChild(PathNode);

                VaultXML = VaultNode.OuterXml;

              }  
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);

                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            return null;
        }

        static internal Boolean CheckIfDiskSpaceAvailable(ulong FileSize, ref XmlNode VaultNode)
        {
            ulong   MaxAllowedDiskSpace     = 0;
            ulong   TotalUsedDiskSpace      = 0;
            XmlNode MaxAllowedDiskSpaceNode = null;
            XmlNode TotalUsedDiskSpaceNode  = null;
            try
            {
                       
                MaxAllowedDiskSpaceNode = VaultNode.SelectSingleNode("MAXALLOWEDDISKSPACE");
                if (MaxAllowedDiskSpaceNode != null)
                    MaxAllowedDiskSpace = Convert.ToUInt64(MaxAllowedDiskSpaceNode.InnerText);
                
                TotalUsedDiskSpaceNode = VaultNode.SelectSingleNode("TOTALUSEDSPACE");
                if (TotalUsedDiskSpaceNode!=null)
                    TotalUsedDiskSpace = Convert.ToUInt64(TotalUsedDiskSpaceNode.InnerText);

                if (MaxAllowedDiskSpace > 0)
                {
                    if ((TotalUsedDiskSpace + FileSize) > MaxAllowedDiskSpace)
                    {
                        return false;
                    }
                    else
                    {
                        TotalUsedDiskSpaceNode.InnerText = Convert.ToString((TotalUsedDiskSpace + FileSize));
                    }
                }
                return true;
            }
            catch (Exception )
            {
                return false;
            }
            
        }

      
        /// <alsummary>
        /// <createddate> 11th October 2007 </createddate>
        /// <author> Mr.Pravin Shinde</author>
        /// <purpose>Prepares Operation nodes to update vault type of section and its subsections.</purpose>
        ///<param name="vaultType">System/Non-System</param>
        ///<param name="SectionPath">Section path to update vault type </param>
        ///<param name="QMQueueObj">Operation Queue</param>
        internal ERRORXML FormOperationNodesForUpdatingVaultTypeofSection(String SectionPath, String vaultType, ref OperationQueue QMQueueObj)
        {

            #region Local Variables            
            StringBuilder   PathForSubsections          = null;
            String          vaultTypeEnum               = null;
            StringBuilder   sb_BatchUpdateXml           = null;
            #endregion

            try
            {
                if (!String.IsNullOrEmpty(SectionPath))
                {                    
                    sb_BatchUpdateXml   = new StringBuilder(200);
                    PathForSubsections  = new StringBuilder();
                    PathForSubsections.AppendFormat("{0}\\", SectionPath);
                    vaultTypeEnum       = CommonFunctionsObj.GetVaultTypeEnum(vaultType);
                    
                    sb_BatchUpdateXml.Append("<ENTITYCOLLECTION AuthorizationRequired='FALSE'>");
                    sb_BatchUpdateXml.AppendFormat(@"<SECTION>
                                                        <FILTERLIST>
                                                            <FILTER>
                                                                <EXPRESSION>PATH</EXPRESSION>
                                                                <OPERATOR>EQUALTO</OPERATOR>
                                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                                <ISCASESENSITIVE>NO</ISCASESENSITIVE>
                                                            </FILTER>
                                                            <FILTER>
                                                                <EXPRESSION>PATH</EXPRESSION>
                                                                <OPERATOR>STARTSWITH</OPERATOR>
                                                                <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>
                                                                <ISCASESENSITIVE>NO</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>
                                                        <ISSYSTEMVAULT>{2}</ISSYSTEMVAULT>
                                                    </SECTION>", ALCommon.TrimAndReplaceSpecialChars(SectionPath),
                                                    ALCommon.TrimAndReplaceSpecialChars(PathForSubsections.ToString()), vaultTypeEnum.ToString());
                    sb_BatchUpdateXml.Append("</ENTITYCOLLECTION>");

                    OperObj         = new Operation("ENTITY", "BATCHUPDATE");
                    OperObj.AddParameter("INPUTXML", sb_BatchUpdateXml.ToString());
                    QMQueueObj.AddOperation(OperObj);
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }
               

        private void LogErrorForDeleteFolder(String MethodName, String VaultID, String VaultName, String VaultLocation, String OnMediaServer, String ErrorXml)
        {
            StringBuilder LogXML = new StringBuilder(200);
            Context ObjContext   = null;
            String str_Message   = null;
            LogXML.AppendFormat(@"<ADDITIONALINFO UserID='{0}' SessionID='{1}' Language='{2}' TenantName='{3}'>
                                        <METHODNAME>{4}</METHODNAME>                                            
                                        <VAULTID>{5}</VAULTID>   
                                        <VAULTNAME>{6}</VAULTNAME>
                                        <VAULTLOCATION>{7}</VAULTLOCATION>
                                        <ONMEDIASERVER>{8}</ONMEDIASERVER>
                                        <ERRORXML>{9}</ERRORXML>
                                        <THREADID>{10}</THREADID>
                                  </ADDITIONALINFO>",
                                  UserID, SessionID, Language, TenantName,MethodName, VaultID, VaultName, VaultLocation,
                                  OnMediaServer, ErrorXml, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

            str_Message = String.Format("Attempting to delete folder from path: {0} from Method: {1}" , VaultLocation, MethodName);
            ObjContext = new Context(UserID, SessionID, TenantName, SolutionName, LogXML.ToString());
            SASMOMDirectory.WriteLog(str_Message, ObjContext);
        }
        #endregion

        #region Private Methods


        private Boolean VaultAlreadyExists(String VaultName)
        {
            ERRORXML    ErrorXML        = null;
            XmlDocument VaultXMLDoc     = null;
            XmlNode     VaultNode       = null;
            String      VaultXML        = null;

            try
            {
                SearchObj                   = new SearchHelper(UserID, Language, TenantName);
                SearchObj.SearchAllFacets   = false;
                SearchObj.SortColumnName    = "VAULTID";
                SearchObj.SortOrder         = "ASC";
                SearchObj.MaxResultSize     = 1;

                SearchObj.Init("VAULT", "ENTITY");
                SearchObj.AddFilter("NAME", "EQUALTO", VaultName,
                                    false, "VAULT");

                SearchObj.AddFields("VAULTID","VAULT");
                             
                VaultXML                    = null;
                ErrorXML                    = SearchObj.DoSearch(ref VaultXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTREADFAILED", this.GetType().ToString(), "VaultAlreadyExists", null, ErrorXML);
                    return true;
                }
                else
                {
                    VaultXMLDoc = new XmlDocument();
                    VaultXMLDoc.LoadXml(VaultXML);
                    VaultNode   = VaultXMLDoc.SelectSingleNode("VAULTCOLLECTION/VAULT");
                    if (VaultNode != null)
                        return true;
                    else
                        return false;
                }
                
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private Boolean CheckIfMountPointAlreadyUsed(String MountPoint)
        {
            ERRORXML        ErrorXML        = null;
            XmlDocument     VaultXMLDoc     = null;
            String          VaultXML        = null;
            XmlNodeList     MountPointNodes = null;
            StringBuilder   UsedMountPoint  = null;

            try
            {
                UsedMountPoint              = new StringBuilder();
                SearchObj                   = new SearchHelper(UserID, Language, TenantName);
                SearchObj.SearchAllFacets   = false;
                SearchObj.SortColumnName    = "VAULTID";
                SearchObj.SortOrder         = "ASC";
               
                SearchObj.Init("VAULT", "ENTITY");
                SearchObj.AddFields("VAULTID","VAULT");
                SearchObj.AddFields("MOUNTPOINT", "VAULT");
                             
                VaultXML                    = null;
                ErrorXML                    = SearchObj.DoSearch(ref VaultXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("DAMENGINE_VAULT_VAULTREADFAILED", this.GetType().ToString(), "CheckIfMountPointAlreadyUsed", null, ErrorXML);
                    return true;
                }
                else
                {
                    VaultXMLDoc     = new XmlDocument();
                    VaultXMLDoc.LoadXml(VaultXML);
                    MountPointNodes = VaultXMLDoc.SelectNodes("VAULTCOLLECTION/VAULT/MOUNTPOINT");
                    foreach (XmlNode MountPointNode in MountPointNodes)
                    {
                        UsedMountPoint.Remove(0,UsedMountPoint.Length);
                        UsedMountPoint.AppendFormat("{0}\\", MountPointNode.InnerText);
                        if (MountPoint.StartsWith(UsedMountPoint.ToString(), StringComparison.CurrentCultureIgnoreCase))
                        {
                            return true;
                        }
                    }
                }
                return false;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// Function check if the vault belongs to Media server or not.
        /// </summary>
        /// <param name="VaultNode">Dam VaultNode</param>
        /// <returns>Return True if vault belongs to Media server else Return False</returns>
        private Boolean CheckforUseMediaServer(XmlNode VaultNode)
        {
            if (VaultNode.SelectSingleNode("ONMEDIASERVER") == null)
                return isMediaServerActivated;
            else
                return Boolean.Parse(VaultNode.SelectSingleNode("ONMEDIASERVER").InnerText);
        }

        /// <summary>
        /// This function take care of creating folder on file system,
        /// by using appropriate File system Interface Object.
        /// </summary>
        /// <param name="DirectoryOperationsObj">File System Interface Obj to be use</param>
        /// <param name="FolderPath">File system folder path</param>
        /// <param name="FunctionName">Function from which the call is made</param>
        /// <returns>Null On success else return ErrorXML</returns>
        private ERRORXML CreateFolder(ref DirectoryOperations DirectoryOperationsObj, String FolderPath, String FunctionName)
        {
            ERRORXML    ErrorXml            = null;
            XmlDocument ErrorXmlDoc         = null;
            XmlNode     ExceptionTypeNode   = null;


            if (Path.IsPathRooted(FolderPath))
            {
                ErrorXml = DirectoryOperationsObj.CreateFolder(FolderPath);
                if (ErrorXml != null)
                {
                    ErrorXmlDoc = new XmlDocument();
                    ErrorXmlDoc.LoadXml(ErrorXml);

                    ExceptionTypeNode = ErrorXmlDoc.SelectSingleNode("ERRORCOLLECTION/ERROR/INNERERROR/ERROR/EXCEPTIONTYPE");

                    switch (ExceptionTypeNode.InnerText.ToUpper())
                    {
                        case "PATHTOOLONGEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_TOOLONGPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "DIRECTORYNOTFOUNDEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_INVALIDPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "UNAUTHORIZEDACCESSEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_UNAUTHORIZEDPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "ARGUMENTNULLEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_EMPTYPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "ARGUMENTEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_INVALIDPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "NOTSUPPORTEDEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_INVALIDPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "IOEXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_READONLYPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        case "EXCEPTION":
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_INVALIDPATHFORMOUNTPOINT", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;

                        default:
                            ALEngineErrorObj.AppendError("DAMENGINE_VAULT_UNKNOWNERROR", this.GetType().ToString(), FunctionName, null, ErrorXml);
                            break;
                    }

                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            else
            {
                ALEngineErrorObj.AppendError("DAMENGINE_VAULT_CREATEFOLDER", this.GetType().ToString(), FunctionName, null, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            return null;
        }
      
        #endregion

    }
}

