﻿#region System Include
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml.XPath;
using System.Xml;
using System.Diagnostics;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Threading;
using System.Configuration;
using System.Web.Services;
using System.Reflection;
#endregion


#region Assetlink Include
using Assetlink;
using Assetlink.Services.MediaProcessor.ManagedMediaProcessorWrapper;
using Assetlink.WebServices.ALDAMWS;
using Assetlink.WebServices.ALInfrastructureWS;
using Assetlink.WebServices.ALSearchWS;
using Assetlink.Services.WindowsService;
using Assetlink.Services.WindowsService.CatalogerHelper;
using Assetlink.IOProxy;
#endregion

namespace Assetlink.MediaProcessors
{
    using XML       = System.String;
    using ERRORXML  = System.String;
    using ENTITYID  = System.UInt64;

    public class AssetProcessor : ALQueueComponent
    {
        #region Member Varibales        
        Assetlink.WebServices.ALDAMWS.ALServiceHeader            RequestHeader                  = null;
        Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader InfrastructureRequestHeader    = null;
        InfrastructureService                                    InfrastructureWSObj            = null;
        ALDAMWS                                                  DAMWSObj                       = null;
        Search                                                   ObjSearch                      = null;

        StringDictionary        strdict_AdditionalInfo       = null;
        public String           ServiceName                  = null;
        public String           RequestXML                   = null;
        public String           ErrorFolderName              = null;
        ulong                   ul_UserID;
        ulong                   ul_SessionID;
        String                  str_TenantName          = null;
        String                  str_UserID              = null;
        String                  str_Language            = null;
        String                  str_SolutionName        = "DAM";
        String                  str_SessionID           = null;
        String                  JobType                 = null;             
        String                  RequestToken            = null;

        String                  RequestID               = null;
        String                  CatalogJobID            = null;
        String                  DigitalAssetID          = null;
        bool                    CatalogerAndOMTogether  = false;

        XmlDocument             ErrorXMLDoc             = null;
        XmlDocument             JobProcessingDetails    = null;
        XmlNode                 JobNode                 = null;
        ThreadContext           ThreadContextObj        = null;
        StringBuilder           ErrorMessage            = null;
        CommonVariables         ThreadObj               = null;
        Boolean                 IsErrorOccured          = false;
        StringBuilder           sb_ErrorCollection      = new StringBuilder(200);
        String                  CurrentProcessingFileName = null;
        String                  Is_GenerateThb_From_Preview_Call = null; //If the Current Request is sent for Generate Thumbnail from Preview call
        String                  MountPoint              = null;
        String                  CatalogerMountPointUNCPath = null;
        String                  JobPath                 = null;
        String                  JobName                 = null;

        Int32                   NumberOfProcessedAssets = 0;        
        UserCATNotificationPreferences UserNotificationPref     = null;
        XML                     TenantCatalogingPreferencesXML  = null;
        CatalogerContext        CatalogerContextObj             = null;

        XPathDocument           PluginsXMLDoc           = null;
        String                  NetworkFolderPath       = null;
        String                  SenderName              = null;
        String                  SenderEmail             = null;
        String                  SenderID                = null;
        String                  LoginName               = null;

        String                  str_CallerIdentity      = "ASSETPROCESSOR";
        String                  ImpersonationToken      = null;
        String                  EncryptedSystemUserName = null;
        String                  EncryptedSystemUserPwd  = null;
        const ulong             SystemUserID            = 111111122211111;
        const String            SystemUserName          = "MePhIsToPhElEs";
        const String            SystemUserPwd           = "ErAtOsThEnEs";

        XmlDocument             FileFormatXMLDoc        = null;
        bool                    GetPreviewAttributesExplicitly = false;
        bool                    GetThumbnailAttributesExplicitly = false;
        String                  LoggedERRORID           = null;
        bool                    IsJobSuccesseded        = false;
        bool                    IsJobProcessingComplete = false;
        XPathNavigator          xpNav_ErrorXml          = null;
        XPathNavigator          xpNav_Error             = null;
        String                  ErrorCode               = null;
        #endregion

        public override bool InitializeComponent()
        {
            return base.InitializeComponent();
        }

        #region Enums
        internal enum RenditionType
        {
            ORIGINAL = 0,
            PREVIEW = 1,
            THUMBNAIL = 2
        }

        enum ErrorType
        {
            ERROR = 0,
            EXCEPTION = 1
        }

        enum AssetStatus
        {
            ATTRIBUTESUPDATED = 0,
            NOTGENERATED = 0,
            GENERATED = 1,
            GENERATOR_NOT_PRESENT = 2,
            FAILED_TO_GENERATE = 3
        }
        #endregion

        #region internal Class
        //These are same as internal Class's in Cataloger, if any addition or update of these class will have to be reflected here also.
        internal class ThumbnailAttributes
        {
           public   String      ThumbnailFileName           = null;
           public   String      ThumbnailFilePath           = null;
           public   String      ThumbnailFileExtension      = null;
           public   String      ThumbnailFileFormat         = null;
           public   String      ThumbnailGenerator          = null;
           public   long        ThumbnailSize               = 0;
           public   int         ThumbnailHeight             = 0;
           public   int         ThumbnailWidth              = 0;
           public   int         ThumbnailDepth              = 0;
           public   int         ThumbnailDPI                = 0;
           public   String      IntermidiateThumFilePath    = null;
           public   int         ThumbnailQuality            = 0;
           public   String      TempThumbFilePath           = null;
           public   String      ThumbMacType                = null;
           public   String      ThumbMacCreator             = null;
           public   Boolean     ManagedGenerator            = false;
           public Boolean       IsFileCopiedToTempLocation  = true;
        }
        
        internal class PreviewAttributes
        {
           public   String      PreviewFileName             = null;
           public   String      PreviewFilePath             = null;
           public   String      PreviewFileExtension        = null;
           public   String      PreviewFileFormat           = null;
           public   String      PreviewGenerator            = null;
           public   long        PreviewSize                 = 0;
           public   int         PreviewHeight               = 0;
           public   int         PreviewWidth                = 0;
           public   int         PreviewDepth                = 0;
           public   int         PreviewDPI                  = 0;
           public   String      IntermidiatePrvFilePath     = null;
           public   String      TempPrvFilePath             = null;
           public   String      PrvMacType                  = null;
           public   String      PrvMacCreator               = null;
           public   Boolean     ManagedGenerator            = false;
           public   Boolean     IsFileCopiedToTempLocation  = true;
        }

        internal class OriginalAttributes
        {
           public   String      OriginalFileName            = null;
           public   String      OriginalFilePath            = null;
           public   String      OriginalFileExtension       = null;
           public   String      OriginalFileFormat          = null;
           public   long        OriginalSize                = 0;
           public   int         OriginalHeight              = 0;
           public   int         OriginalWidth               = 0;
           public   int         OriginalDepth               = 0;
           public   int         OriginalDPI                 = 0;
           public   String      IntermidiateOrgFilePath     = null;
           public   String      TempOrgFilePath             = null;
           public   String      OrgMacType                  = null;
           public   String      OrgMacCreator               = null;
           public   String      OriginalGenerator           = null;
           public   Boolean     ManagedGenerator            = false;
           public   String      MetadataProcessor           = null;
           public   Boolean     IsFileCopiedToTempLocation  = true;
        }

        internal class ThreadContext
        {
           public   XmlNode             ClonedDigitalAssetNode  = null;
           public   Boolean             UseOriginalAsPreview    = false;
           public   Boolean             PreviewExists           = false;
           public   Boolean             ThumbnailExists         = false;
           public   Boolean             IsBinHex                = false;
           public   Boolean             IsThumbBinHex           = false;
           public   Boolean             IsPrvBinHex             = false;
           public   Boolean             GenThumbFromPreview     = false;
           public   String              ThbFromOrgGenerator     = null;
           public   XmlNode             FormatDetails           = null;
           public   PreviewAttributes   PreviewAttrObj          = null;
           public   ThumbnailAttributes ThumbnailAttrObj        = null;
           public   OriginalAttributes  OriginalAttrObj         = null;
           public   ENTITYID            DigitalAssetID          = 0;
           public   String              ContentRevisionID       = null;
           public   StringBuilder       ErrorMessage            = null;
           public   ENTITYID            SectionID               = 0;
           public   Boolean             OnMediaServer           = false;
           public   XmlNode             JobNode                 = null;
           public   String              TempFolderPath          = null;
           public   Boolean             IsSuccess               = true;
           public   String              ParentDigitalAssetName  = null;
           public   String              str_SectionPath         = null;
           public   String              MediaProcessor          = null;

           public ThreadContext()
           {
               PreviewAttrObj   = new PreviewAttributes();
               ThumbnailAttrObj = new ThumbnailAttributes();
               OriginalAttrObj  = new OriginalAttributes();
               ErrorMessage     = new StringBuilder();
           }
        }
        #endregion

        #region public methods
        public override void ProcessMessage(String MediaRequestXML)
        {
            #region Member Variables            
            XPathNavigator      xpNav_MediaResponseXML      = null;
            String              str_RequesteGenerator       = null;
            MediaRequestRouter  obj_MediaRequestRouter      = null;
            String              MediaResponseXML            = null;
            String              str_JobHandler              = null;
            String              str_CatalogerAndOMTogether  = null;           
            Boolean             MediaProcessorLogAdded      = false;            
            XPathNavigator      xpNav_LogFile               = null;
            String              ErrorXML                    = null;
            String              str_Renditiontype           = null;
            StringBuilder       CatalogerPluginsFilePath    = null;
            XmlDocument         xmlDoc_Request              = null;
            #endregion

            try
            {
                #region Initialization
                ThreadObj               = new CommonVariables();
                MediaProcessorLogAdded  = false;
                IsErrorOccured          = false;
                sb_ErrorCollection      = new StringBuilder(200);
                CatalogerContextObj     = new CatalogerContext();
                UserNotificationPref    = new UserCATNotificationPreferences();
                CatalogerPluginsFilePath = new StringBuilder(100);
                ThreadContextObj        = new ThreadContext();
                IsJobProcessingComplete = false;
                #endregion

                #region Read Request
                MediaResponseXML        = MediaRequestXML;
                obj_MediaRequestRouter  = new MediaRequestRouter(str_TenantName, str_UserID, str_SessionID, str_Language, str_SolutionName);
                obj_MediaRequestRouter.TransformResponseXML(ref MediaResponseXML);

                xmlDoc_Request          = new XmlDocument();
                xmlDoc_Request.LoadXml(MediaRequestXML);

                xpNav_MediaResponseXML  = xmlDoc_Request.CreateNavigator();
                xpNav_MediaResponseXML  = xpNav_MediaResponseXML.SelectSingleNode("/REQUEST");

                str_TenantName          = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/TENANTNAME").Value;
                str_UserID              = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/USERID").Value;
                ul_UserID               = ulong.Parse(str_UserID);
                str_SessionID           = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/SESSIONID").Value;
                ul_SessionID            = ulong.Parse(str_SessionID);
                str_Language            = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/LANGUAGE").Value;
                str_SolutionName        = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/SOLUTIONNAME").Value;
                str_JobHandler          = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBHANDLER").Value;
                str_RequesteGenerator   = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/REQUESTGENERATOR").Value;
                str_RequesteGenerator   = str_RequesteGenerator.ToUpper();
                JobType                 = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBTYPE").Value;
                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/REQUESTTOKEN") != null)
                    RequestToken        = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/REQUESTTOKEN").Value;
                RequestID               = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/REQUESTID").Value;

                LoginName               = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/LOGINNAME").Value;
                NetworkFolderPath       = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/NETWORKFOLDERPATH").Value;
                CatalogJobID            = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBID").Value;

                DigitalAssetID          = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/DIGITALASSETID").Value;
                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/CATALOGERANDOMTOGETHER") != null)
                {
                    str_CatalogerAndOMTogether = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/CATALOGERANDOMTOGETHER").Value;
                    bool.TryParse(str_CatalogerAndOMTogether, out CatalogerAndOMTogether);
                }

                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/RENDITIONTYPE") != null)
                    str_Renditiontype = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/RENDITIONTYPE").Value;

                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/GEN_THB_FRM_PRV_CALL") != null)
                    Is_GenerateThb_From_Preview_Call = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/GEN_THB_FRM_PRV_CALL").Value;

                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/MOUNTPOINT") != null)
                    MountPoint = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/MOUNTPOINT").Value;
                               

                if (!MountPoint.StartsWith("\\"))
                {
                    StringBuilder sb_CatalogerMountPointUNCPath = new StringBuilder(100);
                    sb_CatalogerMountPointUNCPath.AppendFormat("\\\\{0}\\{1}", GetCatlogerMachineIPAddress(), Path.GetFileName(MountPoint));
                    CatalogerMountPointUNCPath = sb_CatalogerMountPointUNCPath.ToString();
                }
                else
                {
                    CatalogerMountPointUNCPath = MountPoint;
                }

                InitializeVariables(MediaRequestXML);                
                #endregion

                #region Prepare Client log file
                xpNav_LogFile = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT/LOGFILE");
                if (xpNav_LogFile == null)
                {
                    String str_FirstOutputFile = null;
                    if (xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT") != null)
                        str_FirstOutputFile = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT").GetAttribute("FILENAME", "");
                    else
                        str_FirstOutputFile = ALCommon.GetTempPath();

                    StringBuilder sbTemp        = new StringBuilder(100);
                    sbTemp.AppendFormat("{0}\\AssetProcessor_Log.txt", Path.GetDirectoryName(str_FirstOutputFile));
                    ThreadObj.ClientLogPath     = sbTemp.ToString();
                }
                else
                {
                    ThreadObj.ClientLogPath     = xpNav_LogFile.GetAttribute("FILENAME", "");
                }

                if (!Directory.Exists(Path.GetDirectoryName(ThreadObj.ClientLogPath)))
                    ThreadObj.ISClientLogAccessible = false;

                #endregion

                #region Check if any plugins are registered
                CatalogerPluginsFilePath.AppendFormat("{0}\\Binaries\\CatalogerPlugins.xml", ALCommon.GetInstallationDir());
                if (File.Exists(CatalogerPluginsFilePath.ToString()))
                {
                    PluginsXMLDoc = new XPathDocument(CatalogerPluginsFilePath.ToString());
                }
                #endregion

                if(IsSessionValid())
                {
                    AppendToClientLog("Start Processing RequestID:" + ThreadObj.RequestID, ThreadObj);
                    ErrorXML    = GetCatalogJobProcessingDetails();
                    if (String.IsNullOrEmpty(ErrorXML))
                    {
                        SetCatalogerContextObject(xpNav_MediaResponseXML);

                        #region Process Request
                        switch (JobType.ToUpper())
                        {
                            case "GETINFO":
                                #region Update Original Attributes
                                switch (str_Renditiontype.ToUpper())
                                {
                                    case "ORIGINAL":
                                        AppendToClientLog("Updating Original Attributes for RequestID:" + ThreadObj.RequestID, ThreadObj);
                                        UpdateOriginalAttributes(xpNav_MediaResponseXML);
                                        AppendToClientLog("Updating Original Attributes for RequestID is Complete:" + ThreadObj.RequestID, ThreadObj);
                                        break;

                                    case "PREVIEW":
                                    case "THUMBNAIL":
                                        AppendToClientLog("Updating Attributes for "+ str_Renditiontype +" RequestID:" + ThreadObj.RequestID, ThreadObj);
                                        UpdateAttributes(xpNav_MediaResponseXML, str_Renditiontype.ToUpper());
                                        AppendToClientLog("Updating Attributes for " + str_Renditiontype + " is Complete RequestID:" + ThreadObj.RequestID, ThreadObj);
                                        break;
                                }
                                
                                #endregion
                                break;

                            case "GENERATETHUMBNAILFILE":
                                #region Create ThumbnailRendition
                                AppendToClientLog("Generating Thumbnail for RequestID:" + ThreadObj.RequestID, ThreadObj);
                                GenerateThumbnailFile(xpNav_MediaResponseXML);
                                AppendToClientLog("Generating Thumbnail for RequestID is Complete:" + ThreadObj.RequestID, ThreadObj);
                                #endregion
                                break;

                            case "GENERATEPREVIEWFILE":
                                #region Create Preview Renditon
                                AppendToClientLog("Generating Preview for RequestID:" + ThreadObj.RequestID, ThreadObj);
                                GeneratePreviewFile(xpNav_MediaResponseXML);
                                AppendToClientLog("Generating Preview for RequestID is Complete:" + ThreadObj.RequestID, ThreadObj);
                                #endregion
                                break;
                        }
                        #endregion
                        AppendToClientLog("End Processing RequestID:" + ThreadObj.RequestID, ThreadObj);
                    }
                    else
                    {
                        IsErrorOccured = true;
                        sb_ErrorCollection.AppendFormat(@"<ERROR><ERRORMESSAGE>Error while GetCatalogJobProcessingDetail </ERRORMESSAGE>{0}</ERROR>", ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                    }

                    #region Update CatalogJob
                    if (!UpdateCatalogJobStatus())
                    {
                        //Error in Updating CatalogJob processing Details
                        IsErrorOccured = true;
                        sb_ErrorCollection.AppendFormat(@"<ERROR><ERRORMESSAGE>Error while UpdateCatalogJobStatus</ERRORMESSAGE>{0}</ERROR>"                                          
                                          , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                    }
                    #endregion

                    #region Append Log
                    if (IsErrorOccured)
                    {
                        ThreadObj.ErrorXML = sb_ErrorCollection.ToString();
                        UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, CurrentProcessingFileName);
                    }
                    if (String.IsNullOrEmpty(ThreadObj.ErrorXML) && !IsErrorOccured)
                    {
                        UpdateRequestXML("DONE", xpNav_MediaResponseXML, ThreadObj, CurrentProcessingFileName);
                    }

                    AddMediaProcessorLog(xpNav_MediaResponseXML.OuterXml, ThreadObj);
                    MediaProcessorLogAdded = true;
                    #endregion

                }//IsSessionValid

                AppendToClientLog("Processing completed for RequestID:" + ThreadObj.RequestID, ThreadObj);

                #region Cleanup of Temp files
                if (IsJobProcessingComplete && IsJobSuccesseded)
                {
                    try
                    {
                        //Incase of shared network folder job check whether job 
                        //cleanup is specified.
                        if (!String.IsNullOrEmpty(NetworkFolderPath) &&
                            JobPath.StartsWith(NetworkFolderPath,
                            StringComparison.CurrentCultureIgnoreCase))
                        {
                            String JobClenupRequired = ConfigurationManager.AppSettings["SharedNetworkFolder_Cleanup_Required"];
                            if ((!String.IsNullOrEmpty(JobClenupRequired)) && (JobClenupRequired.ToUpper() == "YES"))
                                NTFSJobCleanUp();
                        }
                        else
                        {
                            NTFSJobCleanUp();
                        }                        
                    }
                    catch (Exception ExceptionObj)
                    {
                        IsJobSuccesseded = false;
                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.AppendFormat("Exception thrown during Job clean up. Message : {0}", ExceptionObj.Message);
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_JOBCLEANUPFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);
                    }
                }
                #endregion
            }
            catch (ThreadAbortException ExceptionObj)
            {
                AppendToClientLog("Start of thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.ErrorXML = "<MESSAGE>" + ExceptionObj.Message.ToString() + "</MESSAGE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORTYPE>EXCEPTION</ERRORTYPE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORDETAILS>" + ExceptionObj.StackTrace + "</ERRORDETAILS>";

                UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, CurrentProcessingFileName);

                AppendToClientLog("After UpdateRequestXML of Thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.IsExceptionOccured = true;
                if (MediaProcessorLogAdded)
                    UpdateMediaProcessorLog("Thread abort Exception in ProcessFile", ThreadObj);
                else
                {
                    if (xpNav_MediaResponseXML != null)
                    {
                        AddMediaProcessorLog(xpNav_MediaResponseXML.OuterXml, ThreadObj);
                        MediaProcessorLogAdded = true;
                    }
                }

                AppendToClientLog("After AddMediaProcessorLog/UpdateMediaProcessorLog of Thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);
                ReportErrorInEventViewer(ExceptionObj, MediaRequestXML);
                AppendToClientLog("After ReportErrorInEventViewer of Thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);
            }
            catch (Exception ExceptionObj)
            {
                AppendToClientLog("Start of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.ErrorXML = ExceptionObj.Message;
                UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, CurrentProcessingFileName);
                AppendToClientLog("After UpdateRequestXML of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.IsExceptionOccured = true;
                if (MediaProcessorLogAdded)
                    UpdateMediaProcessorLog("Exception in ProcessFile", ThreadObj);
                else
                {
                    if (xpNav_MediaResponseXML != null)
                    {
                        AddMediaProcessorLog(xpNav_MediaResponseXML.OuterXml, ThreadObj);
                        MediaProcessorLogAdded = true;
                    }
                }
                
                AppendToClientLog("After AddMediaProcessorLog/UpdateMediaProcessorLog of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);
                ReportErrorInEventViewer(ExceptionObj, MediaRequestXML);
                AppendToClientLog("After ReportErrorInEventViewer of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.ErrorXML = "<MESSAGE>" + ExceptionObj.Message.ToString() + "</MESSAGE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORTYPE>EXCEPTION</ERRORTYPE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORDETAILS>" + ExceptionObj.StackTrace + "</ERRORDETAILS>";
                UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, CurrentProcessingFileName);
            }
            finally
            {
                if (IsJobProcessingComplete)
                {
                    // Code to clean up the session
                    Infrastructure.SignOut(ul_UserID, ul_SessionID, str_Language, str_TenantName);
                }
            }
        }
        #endregion

        #region Private Methods - Asset Processing
        private void InitializeVariables(String str_RequestXML)
        { 
            RequestHeader                   = new Assetlink.WebServices.ALDAMWS.ALServiceHeader();
            InfrastructureRequestHeader     = new Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader();
            SetALServiceHeaderValue(ul_SessionID, ul_UserID, str_TenantName, str_Language);

            DAMWSObj                        = new ALDAMWS();
            InfrastructureWSObj             = new InfrastructureService();
            DAMWSObj.ALServiceHeaderValue   = RequestHeader;
            InfrastructureWSObj.ALServiceHeaderValue = InfrastructureRequestHeader;
            DAMWSObj.Url                    = ConfigurationManager.AppSettings["ALDAMWS"];
            InfrastructureWSObj.Url         = ConfigurationManager.AppSettings["ALInfrastructureWS"];            
            strdict_AdditionalInfo          = new StringDictionary();            
            RequestXML                      = str_RequestXML;                      
            JobProcessingDetails            = new XmlDocument();
            ObjSearch                       = new Search();
            ObjSearch.Url                   = ConfigurationManager.AppSettings["ALSearchWS"];
            ErrorMessage                    = new StringBuilder();
            FileFormatXMLDoc                = new XmlDocument();
        }

        private ERRORXML SetALServiceHeaderValue(ulong SessionID, ulong UserID, String TenantName, String Language)
        {
            try
            {
                RequestHeader.SessionID     = SessionID;
                RequestHeader.UserID        = UserID;
                RequestHeader.TenantName    = TenantName;
                RequestHeader.Language      = Language;

                InfrastructureRequestHeader.SessionID   = SessionID;
                InfrastructureRequestHeader.UserID      = UserID;
                InfrastructureRequestHeader.TenantName  = TenantName;
                InfrastructureRequestHeader.Language    = Language;
                return null;
            }
            catch (Exception ExceptionObj)
            {                
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// This method is used to update the Original Attributes in Database for specified Asset
        /// After the Attributes are updated properly, it will check for JobProcessing Status for that Asset
        /// If all the Other processing is DOne then, it will mark the status for that Asset as "DONE"
        /// This method will also update Catalog Job table if processing of all the Assets in DONE or completed
        /// </summary>
        /// <param name="xpNav_MediaResponseXML"></param>
        private void UpdateOriginalAttributes(XPathNavigator xpNav_MediaResponseXML)
        {
            #region member variables            
            XmlNode         JobProcessingDetailsNode    = null;
            XPathNavigator  xpNav_JobProcessingXMLNode  = null;
            XPathNavigator  OriginalObjNode             = null;
            String          OriginalRenditionID         = null;
            XmlNode         RenditionNode               = null;
            String          AssetRenditionXML           = null;
            String          ErrorXML                    = null;
            XmlNode         OrgAttributeUpdated         = null;
            XmlNode         RelationNode                = null;
            XPathNavigator  xpNav_Output                = null;
            XmlNode         PrvAttributeUpdated         = null;
            XPathNavigator  PreviewObjNode              = null;
            String          PreviewRenditionID          = null;
            #endregion

            try
            {
                if (JobNode != null)
                {
                    JobProcessingDetailsNode = JobNode.SelectSingleNode("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS[ASSETID='" + DigitalAssetID + "']");
                    if (JobProcessingDetailsNode != null)
                    {   
                        xpNav_JobProcessingXMLNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA");
                         if (xpNav_JobProcessingXMLNode != null)
                         {
                             xpNav_Output = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT");

                             SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.ORIGINAL, xpNav_Output, xpNav_MediaResponseXML);

                             if (xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "SUCCESS", true) == 0)
                             {                                
                                 OriginalObjNode = xpNav_JobProcessingXMLNode.SelectSingleNode("ORIGINALATTROBJ");
                                 if (OriginalObjNode != null)
                                 {
                                     OriginalRenditionID = OriginalObjNode.SelectSingleNode("ASSETRENDITIONID").Value;
                                     if (GenerateAssetRenditionNode(ThreadContextObj, OriginalRenditionID, RenditionType.ORIGINAL, ref RenditionNode))
                                     {
                                         #region Update Original Atrributes
                                         //As Original File is already added in the Database and Only the File attaributes are to be updated.
                                         ((XmlElement)RenditionNode).SetAttribute("ISFILEUPDATED", "False");

                                         AssetRenditionXML  = RenditionNode.OuterXml;
                                         ErrorXML           = DAMWSObj.UpdateAssetRendition(AssetRenditionXML);

                                         if (!String.IsNullOrEmpty(ErrorXML))
                                         {
                                             strdict_AdditionalInfo.Clear();
                                             strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                             ErrorMessage.AppendFormat("Original attribute update failed for {0}", ThreadContextObj.ParentDigitalAssetName);
                                             LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORIGINALATTRIBUTEUPDATEFAILED", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);

                                             IsErrorOccured = true;
                                             sb_ErrorCollection.AppendFormat("<ERROR FileName='{0}'><ERRORMESSAGE>{1}</ERRORMESSAGE>{2}</ERROR>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                 , ErrorMessage.ToString(), ErrorXML);

                                             //Update Job Status as Error
                                             OrgAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("ORGATTRIBUTEUPDATED");
                                             if (OrgAttributeUpdated != null)
                                                 OrgAttributeUpdated.InnerText = "3";
                                             
                                             //Should we continue generating preview and thubnail information?????                                        
                                             //return false;
                                         }
                                         else
                                         { 
                                            OrgAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("ORGATTRIBUTEUPDATED");
                                            if (OrgAttributeUpdated != null)
                                                OrgAttributeUpdated.InnerText = "1";
                                         }
                                         #endregion

                                         #region Update Preview Attributes in case UseOriginalAsPreview Flag is True
                                         if (ThreadContextObj.UseOriginalAsPreview)
                                         {
                                             RenditionNode      = null;
                                             PreviewObjNode     = xpNav_JobProcessingXMLNode.SelectSingleNode("PREVIEWATTROBJ");
                                             if (PreviewObjNode != null)
                                             {
                                                 PreviewRenditionID = PreviewObjNode.SelectSingleNode("ASSETRENDITIONID").Value;
                                                 if (GenerateAssetRenditionNode(ThreadContextObj, PreviewRenditionID, RenditionType.PREVIEW, ref RenditionNode))
                                                 {
                                                     ((XmlElement)RenditionNode).SetAttribute("ISFILEUPDATED", "False");

                                                     AssetRenditionXML  = RenditionNode.OuterXml;
                                                     ErrorXML           = DAMWSObj.UpdateAssetRendition(AssetRenditionXML);

                                                     if (!String.IsNullOrEmpty(ErrorXML))
                                                     {
                                                         strdict_AdditionalInfo.Clear();
                                                         strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                                         ErrorMessage.AppendFormat("Preview attribute update failed for {0}", ThreadContextObj.ParentDigitalAssetName);
                                                         LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWATTRIBUTEUPDATEFAILED", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);

                                                         IsErrorOccured = true;
                                                         sb_ErrorCollection.AppendFormat("<ERROR FileName='{0}'><ERRORMESSAGE>{1}</ERRORMESSAGE>{2}</ERROR>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                             , ErrorMessage.ToString(), ErrorXML);

                                                         //Update Job Status as Error
                                                         PrvAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                                                         if (PrvAttributeUpdated != null)
                                                             PrvAttributeUpdated.InnerText = "3";

                                                         //Should we continue generating preview and thubnail information?????                                        
                                                         //return false;
                                                     }
                                                     else
                                                     {
                                                         PrvAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                                                         if (PrvAttributeUpdated != null)
                                                             PrvAttributeUpdated.InnerText = "1";
                                                     }
                                                 }           
                                             }
                                         }// if UseOriginalAsPreview
                                         #endregion

                                         #region Update CatalogJob Processing Details
                                         //1. Update Original Attribute Updated Flag and Check if processing is done for the current Asset
                                         XmlDocument TempDocument    = new XmlDocument();
                                         RelationNode                = TempDocument.CreateElement("CATALOGJOB_PROCESSINGDETAILS");
                                         ((XmlElement)RelationNode).SetAttribute("TYPE", "RELATION");
                                         ((XmlElement)JobProcessingDetailsNode).SetAttribute("MODIFIED", "TRUE");
                                         RelationNode.InnerXml       = JobProcessingDetailsNode.OuterXml;

                                         if (!UpdateCatalogJobProcessingDetails(ref RelationNode))
                                         {   
                                             //Error in Updating CatalogJob processing Details
                                             IsErrorOccured = true;
                                             sb_ErrorCollection.AppendFormat(@"<ERROR FileName='{0}'>
                                                                                <ERRORMESSAGE>Error while UpdateCatalogJobProcessingDetails</ERRORMESSAGE>
                                                                                {1}
                                                                            </ERROR>"
                                                                            , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                                            , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                                         }                                         
                                         #endregion
                                     }
                                 }
                                 else
                                 {
                                     //Error Original Details not found
                                 }                             
                             }//If SUCCESS
                             else if ((xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "ERROR", true) == 0) || xpNav_Output == null)
                             {
                                 ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_ORGATTRIBUTESREADFAILED";
                                 xpNav_ErrorXml = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/ERRORXML");
                                 if (xpNav_ErrorXml != null)
                                 {
                                     xpNav_Error = xpNav_ErrorXml.SelectSingleNode("//ERRORCODE");
                                     if (xpNav_Error != null && !String.IsNullOrEmpty(xpNav_Error.Value))
                                         ErrorCode = xpNav_Error.Value;
                                 }

                                 strdict_AdditionalInfo.Clear();
                                 strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                 ErrorMessage.AppendFormat("Original Attributes read failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                 LogError(ErrorCode, ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);
                                 MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.ORIGINAL, true);
                             }
                        }// if (xpNav_JobProcessingXMLNode != null)                                           
                    }
                }
                else
                { 
                    //Erorr Handling for Job Node Not found
                }
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in Update Original Attributes : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORIGINALATTRIBUTEUPDATEFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);

                ThreadObj.sb_Error.AppendFormat(ExceptionObj.Message.ToString());
                ThreadObj.sb_Error.AppendFormat(ExceptionObj.StackTrace);
                AppendToClientLog("Exception in Updating Original File attributes. Exception: " + ExceptionObj.Message, ThreadObj);
                if (JobProcessingDetailsNode != null)
                    MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.ORIGINAL, true);               
            }
        }
        
        private void UpdateAttributes(XPathNavigator xpNav_MediaResponseXML, String str_Renditiontype)
        {
             #region member variables            
            XmlNode         JobProcessingDetailsNode    = null;
            XPathNavigator  xpNav_JobProcessingXMLNode  = null;
            XPathNavigator  ThumbnailObjNode            = null;
            String          ThumbnailRenditionID        = null;
            XmlNode         RenditionNode               = null;
            String          AssetRenditionXML           = null;
            String          ErrorXML                    = null;
            XmlNode         ThbAttributeUpdated         = null;
            XmlNode         RelationNode                = null;
            XPathNavigator  xpNav_Output                = null;
            XmlNode         PrvAttributeUpdated         = null;
            XPathNavigator  PreviewObjNode              = null;
            String          PreviewRenditionID          = null;
            #endregion

            try
            {
                if (JobNode != null)
                {  
                    JobProcessingDetailsNode = JobNode.SelectSingleNode("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS[ASSETID='" + DigitalAssetID + "']");
                    if (JobProcessingDetailsNode != null)
                    {                       
                        xpNav_Output = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT");
                        if (xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "SUCCESS", true) == 0)
                        {
                            xpNav_JobProcessingXMLNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA");
                            if (xpNav_JobProcessingXMLNode != null)
                            {
                                if (String.Compare(str_Renditiontype, "THUMBNAIL", true) == 0)
                                {
                                    #region Update Thumbnail Atrributes
                                    SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.THUMBNAIL, xpNav_Output, xpNav_MediaResponseXML);

                                    ThumbnailObjNode = xpNav_JobProcessingXMLNode.SelectSingleNode("THUMBNAILATTROBJ");
                                    if (ThumbnailObjNode != null)
                                    {
                                        ThumbnailRenditionID = ThumbnailObjNode.SelectSingleNode("ASSETRENDITIONID").Value;
                                        if (GenerateAssetRenditionNode(ThreadContextObj, ThumbnailRenditionID, RenditionType.THUMBNAIL, ref RenditionNode))
                                        {
                                            //As Thumbnail File is already added in the Database and Only the File attaributes are to be updated.
                                            ((XmlElement)RenditionNode).SetAttribute("ISFILEUPDATED", "False");

                                            AssetRenditionXML   = RenditionNode.OuterXml;
                                            ErrorXML            = DAMWSObj.UpdateAssetRendition(AssetRenditionXML);

                                            if (!String.IsNullOrEmpty(ErrorXML))
                                            {
                                                strdict_AdditionalInfo.Clear();
                                                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                                ErrorMessage.AppendFormat("Original attribute update failed for {0}", ThreadContextObj.ParentDigitalAssetName);
                                                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORIGINALATTRIBUTEUPDATEFAILED", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);

                                                IsErrorOccured = true;
                                                sb_ErrorCollection.AppendFormat("<ERROR FileName='{0}'><ERRORMESSAGE>{1}</ERRORMESSAGE>{2}</ERROR>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                    , ErrorMessage.ToString(), ErrorXML);

                                                //Update Job Status as Error
                                                ThbAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("THBATTRIBUTEUPDATED");
                                                if (ThbAttributeUpdated != null)
                                                    ThbAttributeUpdated.InnerText = "3";
                                                
                                                //Should we continue generating preview and thubnail information?????                                        
                                                //return false;
                                            }
                                            else
                                            {
                                                ThbAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("THBATTRIBUTEUPDATED");
                                                if (ThbAttributeUpdated != null)
                                                    ThbAttributeUpdated.InnerText = "1";
                                            }
                                        }                                    
                                    }
                                    #endregion
                                }
                                else if (String.Compare(str_Renditiontype, "PREVIEW", true) == 0)
                                {
                                    #region Update Preview Attribute
                                    SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.PREVIEW, xpNav_Output, xpNav_MediaResponseXML);

                                    RenditionNode   = null;
                                    PreviewObjNode  = xpNav_JobProcessingXMLNode.SelectSingleNode("PREVIEWATTROBJ");
                                    if (PreviewObjNode != null)
                                    {
                                        PreviewRenditionID = PreviewObjNode.SelectSingleNode("ASSETRENDITIONID").Value;
                                        if (GenerateAssetRenditionNode(ThreadContextObj, PreviewRenditionID, RenditionType.PREVIEW, ref RenditionNode))
                                        {
                                            ((XmlElement)RenditionNode).SetAttribute("ISFILEUPDATED", "False");

                                            AssetRenditionXML   = RenditionNode.OuterXml;
                                            ErrorXML            = DAMWSObj.UpdateAssetRendition(AssetRenditionXML);

                                            if (!String.IsNullOrEmpty(ErrorXML))
                                            {
                                                strdict_AdditionalInfo.Clear();
                                                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                                ErrorMessage.AppendFormat("Preview attribute update failed for {0}", ThreadContextObj.ParentDigitalAssetName);
                                                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWATTRIBUTEUPDATEFAILED", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);

                                                IsErrorOccured = true;
                                                sb_ErrorCollection.AppendFormat("<ERROR FileName='{0}'><ERRORMESSAGE>{1}</ERRORMESSAGE>{2}</ERROR>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                    , ErrorMessage.ToString(), ErrorXML);

                                                //Update Job Status as Error
                                                PrvAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                                                if (PrvAttributeUpdated != null)
                                                    PrvAttributeUpdated.InnerText = "3";


                                                //Should we continue generating preview and thubnail information?????                                        
                                                //return false;
                                            }
                                            else
                                            {
                                                PrvAttributeUpdated = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                                                if (PrvAttributeUpdated != null)
                                                    PrvAttributeUpdated.InnerText = "1";
                                            }
                                        }
                                    }
                                    #endregion
                                }

                                #region Update CatalogJob Processing Details
                                //1. Update Preview/Thumbnail Attribute Updated Flag and Check if processing is done for the current Asset
                                XmlDocument TempDocument    = new XmlDocument();
                                RelationNode                = TempDocument.CreateElement("CATALOGJOB_PROCESSINGDETAILS");
                                ((XmlElement)RelationNode).SetAttribute("TYPE", "RELATION");
                                ((XmlElement)JobProcessingDetailsNode).SetAttribute("MODIFIED", "TRUE");
                                RelationNode.InnerXml       = JobProcessingDetailsNode.OuterXml;

                                if (!UpdateCatalogJobProcessingDetails(ref RelationNode))
                                {
                                    //Error in Updating CatalogJob processing Details
                                    IsErrorOccured = true;
                                    sb_ErrorCollection.AppendFormat(@"<ERROR FileName='{0}'>
                                                                <ERRORMESSAGE>Error while UpdateCatalogJobProcessingDetails</ERRORMESSAGE>
                                                                {1}
                                                            </ERROR>"
                                                                   , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                                   , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                                }

                                #endregion

                            }//if (xpNav_JobProcessingXMLNode != null)
                        }//If SUCCESS
                        else if ((xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "ERROR", true) == 0) || xpNav_Output == null)
                        {
                            if (String.Compare(str_Renditiontype, "THUMBNAIL", true) == 0)
                            {
                                ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_THBATTRIBUTESREADFAILED";
                                SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.THUMBNAIL, xpNav_Output, xpNav_MediaResponseXML);
                            }
                            else if (String.Compare(str_Renditiontype, "PREVIEW", true) == 0)
                            {
                                ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_PRVATTRIBUTESREADFAILED";
                                SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.PREVIEW, xpNav_Output, xpNav_MediaResponseXML);
                            }

                            #region Error handling
                            xpNav_ErrorXml = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/ERRORXML");
                            if (xpNav_ErrorXml != null)
                            {
                                xpNav_Error = xpNav_ErrorXml.SelectSingleNode("//ERRORCODE");
                                if (xpNav_Error != null && !String.IsNullOrEmpty(xpNav_Error.Value))
                                    ErrorCode = xpNav_Error.Value;
                            }

                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);                            
                            if (String.Compare(str_Renditiontype, "PREVIEW", true) == 0)
                            {
                                ErrorMessage.AppendFormat("Preview Attributes read failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                LogError(ErrorCode, ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);
                                MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.PREVIEW, true);
                            }
                            else
                            {
                                ErrorMessage.AppendFormat("Thumbnail Attributes read failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                LogError(ErrorCode, ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);
                                MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.PREVIEW, true);
                            }
                            #endregion
                        }
                       
                    }                   
                }
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in Update Original Attributes : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORIGINALATTRIBUTEUPDATEFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);

                ThreadObj.sb_Error.AppendFormat(ExceptionObj.Message.ToString());
                ThreadObj.sb_Error.AppendFormat(ExceptionObj.StackTrace);
                AppendToClientLog("Exception in Updating Original File attributes. Exception: " + ExceptionObj.Message, ThreadObj);
                if (JobProcessingDetailsNode != null)
                {
                    if (String.Compare(str_Renditiontype, "PREVIEW", true) == 0)                    
                        MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.PREVIEW, true);
                    else
                        MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.THUMBNAIL, true);

                }
            }
        }

        /// <summary>
        /// This method is used to update the Preview File and Attributes in Database for specified Asset
        /// After the Attributes are updated properly, it will check for JobProcessing Status for that Asset
        /// If all the Other processing is DOne then, it will mark the status for that Asset as "DONE"
        /// This method will also update Catalog Job table if processing of all the Assets in DONE or completed
        /// </summary>
        /// <param name="xpNav_MediaResponseXML"></param>
        private void GeneratePreviewFile(XPathNavigator xpNav_MediaResponseXML)
        {
            #region member variables            
            XmlNode         JobProcessingDetailsNode    = null;
            XPathNavigator  xpNav_JobProcessingXMLNode  = null;            
            XmlNode         RenditionNode               = null;
            String          AssetRenditionXML           = null;
            String          ErrorXML                    = null;
            XPathNavigator  xpNav_Output                = null; 
            XmlNode         TempDataNode                = null;
            XmlNode         ContentRevisionRenditionRelNode = null;
            XmlNode         RelationNode                = null;
            #endregion

            try
            {
                GetPreviewAttributesExplicitly = false;
                if (JobNode != null)
                {
                    JobProcessingDetailsNode = JobNode.SelectSingleNode("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS[ASSETID='" + DigitalAssetID + "']");
                    if (JobProcessingDetailsNode != null)
                    {
                        xpNav_JobProcessingXMLNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA");
                        if (xpNav_JobProcessingXMLNode != null)
                        {                       
                            xpNav_Output = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT");
                            SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.PREVIEW, xpNav_Output, xpNav_MediaResponseXML);

                            if (xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "SUCCESS", true) == 0)
                            {                                
                                if (GenerateAssetRenditionNode(ThreadContextObj, null, RenditionType.PREVIEW, ref RenditionNode))
                                {
                                    #region Create Preview Rendition
                                    if (!CatalogerAndOMTogether)
                                        TransferFileAndUpdatePath(RenditionNode);

                                    ContentRevisionRenditionRelNode             = RenditionNode.OwnerDocument.CreateElement("CONTENTREVISION_RENDITION");
                                    ((XmlElement)ContentRevisionRenditionRelNode).SetAttribute("TYPE", "RELATION");
                                    ((XmlElement)ContentRevisionRenditionRelNode).SetAttribute("PARENTID", "TRUE");
                                    ContentRevisionRenditionRelNode.InnerText   = ThreadContextObj.ContentRevisionID;
                                    RenditionNode.AppendChild(ContentRevisionRenditionRelNode);

                                    AssetRenditionXML                           = RenditionNode.OuterXml;
                                    ErrorXML                                    = DAMWSObj.CreateAssetRendition(ref AssetRenditionXML);

                                    if (!String.IsNullOrEmpty(ErrorXML))
                                    {
                                        strdict_AdditionalInfo.Clear();
                                        strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                        ErrorMessage.AppendFormat("Preview Rendition creation failed for  {0}", ThreadContextObj.ParentDigitalAssetName);
                                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWCREATIONEFAILED", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);
                                        //Update Job Status as Error
                                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                                        if (TempDataNode != null)
                                            TempDataNode.InnerText = "3"; //Failed to Generate

                                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("PREVIEWGENERATED");
                                        if (TempDataNode != null)
                                            TempDataNode.InnerText = "3"; //Failed to Generate

                                        IsErrorOccured = true;
                                        sb_ErrorCollection.AppendFormat("<ERROR FileName='{0}'><ERRORMESSAGE>{1}</ERRORMESSAGE>{2}</ERROR>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                            , ErrorMessage.ToString(), ErrorXML);
                                    }
                                    else
                                    {
                                        //Update CatalogJob Processing Details
                                        //1. Update Original Attribute Updated Flag and Check if processing is done for the current Asset
                                        if (!GetPreviewAttributesExplicitly)
                                        {
                                            TempDataNode = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                                            if (TempDataNode != null)
                                                TempDataNode.InnerText = "1";
                                        }

                                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("PREVIEWGENERATED");
                                        if (TempDataNode != null)
                                            TempDataNode.InnerText = "1";
                                    }
                                    #endregion

                                    #region Update Processing Status
                                    XmlDocument TempDocument    = new XmlDocument();
                                    RelationNode                = TempDocument.CreateElement("CATALOGJOB_PROCESSINGDETAILS");
                                    ((XmlElement)RelationNode).SetAttribute("TYPE", "RELATION");
                                    ((XmlElement)JobProcessingDetailsNode).SetAttribute("MODIFIED", "TRUE");
                                    RelationNode.InnerXml       = JobProcessingDetailsNode.OuterXml;

                                    if (!UpdateCatalogJobProcessingDetails(ref RelationNode))
                                    {
                                        //Error in Updating CatalogJob processing Details
                                        IsErrorOccured = true;
                                        sb_ErrorCollection.AppendFormat(@"<ERROR FileName='{0}'><ERRORMESSAGE>Error while UpdateCatalogJobProcessingDetails</ERRORMESSAGE>{1}</ERROR>"
                                                                       , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.PreviewFilePath)
                                                                       , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                                    }

                                    if (GetPreviewAttributesExplicitly)
                                    {
                                        //Append Preview RenditionID Node to PreviewAttributesObj
                                        String str_PrvAssetRenditionID = ALCommon.GetInnerValue(ref AssetRenditionXML, "<ASSETRENDITIONID>", "</ASSETRENDITIONID>");
                                        if (xpNav_MediaResponseXML.SelectSingleNode("/REQUEST/EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ") != null)
                                            xpNav_MediaResponseXML.SelectSingleNode("/REQUEST/EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ").AppendChildElement("", "ASSETRENDITIONID", "", str_PrvAssetRenditionID);

                                        GetAttributes(ThreadContextObj.PreviewAttrObj.PreviewFilePath, ThreadContextObj, RenditionType.PREVIEW, xpNav_MediaResponseXML);
                                    }
                                    #endregion                                   
                                }  
                                else                                    
                                {
                                    IsErrorOccured = true;
                                    sb_ErrorCollection.AppendFormat(@"<ERROR FileName='{0}'><ERRORMESSAGE>Error while GenerateAssetRenditionNode</ERRORMESSAGE>{1}</ERROR>"
                                                                   , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.PreviewFilePath)
                                                                   , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));

                                    strdict_AdditionalInfo.Clear();
                                    strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                    strdict_AdditionalInfo.Add("MESSAGE", ThreadObj.sb_Error.ToString());
                                    ErrorMessage.AppendFormat("Preview generation failed for {0}", ThreadContextObj.ParentDigitalAssetName);
                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);
                                    MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.PREVIEW, false);
                                }
                            }
                            else if ((xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "ERROR", true) == 0) || xpNav_Output == null)
                            {
                                #region Error Handling
                                ErrorCode        = "CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED";
                                
                                xpNav_ErrorXml  = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/ERRORXML");
                                if (xpNav_ErrorXml != null)
                                {
                                    xpNav_Error = xpNav_ErrorXml.SelectSingleNode("//ERRORCODE");
                                    if (xpNav_Error != null && !String.IsNullOrEmpty(xpNav_Error.Value))
                                        ErrorCode = xpNav_Error.Value;
                                }

                                
                                strdict_AdditionalInfo.Clear();
                                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                strdict_AdditionalInfo.Add("MESSAGE", GetErrorMessage(xpNav_MediaResponseXML));
                                ErrorMessage.AppendFormat("Preview generation failed for {0} : Message : {1}", ThreadContextObj.ParentDigitalAssetName, "");
                                
                                LogError(ErrorCode, ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);

                                MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.PREVIEW, false);
                                #endregion             
                            }
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in Updating Preview Attributes : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);
                if(JobProcessingDetailsNode != null)
                    MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.PREVIEW, false);
            }
        }
                
        /// <summary>
        /// This method is used to update the Thumbnail File and Attributes in Database for specified Asset
        /// After the Attributes are updated properly, it will check for JobProcessing Status for that Asset
        /// If all the Other processing is DOne then, it will mark the status for that Asset as "DONE"
        /// This method will also update Catalog Job table if processing of all the Assets in DONE or completed
        /// </summary>
        /// <param name="xpNav_MediaResponseXML"></param>
        private void GenerateThumbnailFile(XPathNavigator xpNav_MediaResponseXML)
        {
            #region member variables            
            XmlNode         JobProcessingDetailsNode    = null;
            XPathNavigator  xpNav_JobProcessingXMLNode  = null;            
            XmlNode         RenditionNode               = null;
            String          AssetRenditionXML           = null;
            String          ErrorXML                    = null;
            XPathNavigator  xpNav_Output                = null; 
            XmlNode         TempDataNode                = null;
            XmlNode         ContentRevisionRenditionRelNode = null;
            XmlNode         RelationNode                = null;
            MediaProcessorWrapper MediaProcessorWrapperObj = null;
            String          DataForkName                = null;
            String          CurrentFileName             = null;
            String          ActualFileName              = null;
            String          str_ExtraInformationForMediaProcessor = null;
            String          DestinationName             = null;
            XmlDocument     TempDocument                = null;
            #endregion

            try
            {
                GetThumbnailAttributesExplicitly = false;
                if (JobNode != null)
                {
                    JobProcessingDetailsNode = JobNode.SelectSingleNode("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS[ASSETID='" + DigitalAssetID + "']");
                    if (JobProcessingDetailsNode != null)
                    {
                        xpNav_JobProcessingXMLNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA");
                        if (xpNav_JobProcessingXMLNode != null)
                        {
                            xpNav_Output = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/FILECONVERTEROUTPUT/OUTPUTLIST/OUTPUT");
                            SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.THUMBNAIL, xpNav_Output, xpNav_MediaResponseXML);

                            if (xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "SUCCESS", true) == 0)
                            {                                
                                if (GenerateAssetRenditionNode(ThreadContextObj, null, RenditionType.THUMBNAIL, ref RenditionNode))
                                {
                                    #region Create Thumbnail Rendition
                                    if (!CatalogerAndOMTogether)
                                        TransferFileAndUpdatePath(RenditionNode);

                                    ContentRevisionRenditionRelNode             = RenditionNode.OwnerDocument.CreateElement("CONTENTREVISION_RENDITION");
                                    ((XmlElement)ContentRevisionRenditionRelNode).SetAttribute("TYPE", "RELATION");
                                    ((XmlElement)ContentRevisionRenditionRelNode).SetAttribute("PARENTID", "TRUE");
                                    ContentRevisionRenditionRelNode.InnerText   = ThreadContextObj.ContentRevisionID;
                                    RenditionNode.AppendChild(ContentRevisionRenditionRelNode);

                                    AssetRenditionXML                           = RenditionNode.OuterXml;
                                    ErrorXML                                    = DAMWSObj.CreateAssetRendition(ref AssetRenditionXML);

                                    if (!String.IsNullOrEmpty(ErrorXML))
                                    {
                                        strdict_AdditionalInfo.Clear();
                                        strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                        ErrorMessage.AppendFormat("Preview Rendition creation failed for  {0}", ThreadContextObj.ParentDigitalAssetName);
                                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWCREATIONEFAILED", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);
                                        
                                        //Update CatalogJob Processing Details                                        
                                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("THBATTRIBUTEUPDATED");
                                        if (TempDataNode != null)
                                            TempDataNode.InnerText = "3";

                                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("THUMBNAILGENERATED");
                                        if (TempDataNode != null)
                                            TempDataNode.InnerText = "3";                                      
                                        
                                    }
                                    else
                                    {
                                        //Update CatalogJob Processing Details
                                        //1. Update Original Attribute Updated Flag and Check if processing is done for the current Asset
                                        if (!GetThumbnailAttributesExplicitly)
                                        {
                                            TempDataNode = JobProcessingDetailsNode.SelectSingleNode("THBATTRIBUTEUPDATED");
                                            if (TempDataNode != null)
                                                TempDataNode.InnerText = "1";
                                        }

                                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("THUMBNAILGENERATED");
                                        if (TempDataNode != null)
                                            TempDataNode.InnerText = "1";
                                    }
                                    #endregion

                                    #region Update Processing Status
                                    TempDocument            = new XmlDocument();
                                    RelationNode            = TempDocument.CreateElement("CATALOGJOB_PROCESSINGDETAILS");
                                    ((XmlElement)RelationNode).SetAttribute("TYPE", "RELATION");
                                    ((XmlElement)JobProcessingDetailsNode).SetAttribute("MODIFIED", "TRUE");
                                    RelationNode.InnerXml   = JobProcessingDetailsNode.OuterXml;

                                    if (!UpdateCatalogJobProcessingDetails(ref RelationNode))
                                    {
                                        //Error in Updating CatalogJob processing Details
                                        IsErrorOccured = true;
                                        sb_ErrorCollection.AppendFormat(@"<ERROR FileName='{0}'>
                                                                            <ERRORMESSAGE>Error while UpdateCatalogJobProcessingDetails</ERRORMESSAGE>
                                                                            {1}
                                                                        </ERROR>"
                                                                       , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath)
                                                                       , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                                    }
                                    #endregion

                                    if (GetThumbnailAttributesExplicitly)
                                    {
                                        String str_ThbAssetRenditionID = ALCommon.GetInnerValue(ref AssetRenditionXML, "<ASSETRENDITIONID>", "</ASSETRENDITIONID>");
                                        if (xpNav_MediaResponseXML.SelectSingleNode("/REQUEST/EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ") != null)
                                            xpNav_MediaResponseXML.SelectSingleNode("/REQUEST/EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ").AppendChildElement("", "ASSETRENDITIONID", "", str_ThbAssetRenditionID);

                                        GetAttributes(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath, ThreadContextObj, RenditionType.THUMBNAIL, xpNav_MediaResponseXML);
                                    }

                                }
                                else
                                {
                                    strdict_AdditionalInfo.Clear();
                                    strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                    ErrorMessage.AppendFormat("Thumbnail generation failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_THUMBNAILGENERATIONFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);
                                    MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.THUMBNAIL, false);
                                }
                            }
                            else if ((xpNav_Output != null && String.Compare(xpNav_Output.GetAttribute("STATUS", ""), "ERROR", true) == 0) || xpNav_Output == null)
                            {                 
             
                                //In case, Thumbnial generation from Preview is failed then make a call to generate Thumbnail Using Original file
                                if (ThreadContextObj.PreviewExists && ThreadContextObj.GenThumbFromPreview
                                                    && String.Compare(Is_GenerateThb_From_Preview_Call, "TRUE", true) == 0)
                                {
                                    SetThreadContextObject(xpNav_JobProcessingXMLNode, RenditionType.THUMBNAIL, xpNav_Output, xpNav_MediaResponseXML);

                                    #region Generate Thumbnail from Original
                                    //Is Binhex processing
                                    if (ThreadContextObj.IsBinHex)
                                        DataForkName = ThreadContextObj.OriginalAttrObj.TempOrgFilePath;

                                    CurrentFileName = ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath;
                                    ActualFileName = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                                    DestinationName = ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath;

                                    GetDestinationFileName(ThreadContextObj, "JPEG", RenditionType.ORIGINAL, ThreadContextObj.ThbFromOrgGenerator, ref CurrentFileName, ref DestinationName);
                                    str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.ORIGINAL, ThreadContextObj.ThbFromOrgGenerator, xpNav_MediaResponseXML);

                                    if (ThreadContextObj.OriginalAttrObj.ManagedGenerator)
                                    {
                                        MediaProcessorWrapperObj = new MediaProcessorWrapper(ul_SessionID, ul_UserID, str_TenantName, str_Language);
                                        MediaProcessorWrapperObj.SetFileAttributes(CurrentFileName, DataForkName, DestinationName, "JPEG", "THUMBNAIL",
                                            ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight, ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth, ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth,
                                            ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI, ThreadContextObj.ThumbnailAttrObj.ThumbnailQuality, ActualFileName, str_ExtraInformationForMediaProcessor);

                                    }
                                    if ((ThreadContextObj.ThbFromOrgGenerator != null) && (ThreadContextObj.ThbFromOrgGenerator.Length > 0))
                                    {
                                        if (ThreadContextObj.OriginalAttrObj.ManagedGenerator)
                                        {
                                            MediaProcessorWrapperObj.GeneratorName = ThreadContextObj.ThbFromOrgGenerator;
                                            MediaProcessorWrapperObj.ThreadTimeout = 3600000; //Cata ThreadTimeOut;
                                            MediaProcessorWrapperObj.GenerateThumbnailFile();
                                        }
                                    }
                                    #endregion
                                }
                                else
                                { 
                                    #region ERROR Handling
                                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_THUMBNAILGENERATIONFAILED";

                                    xpNav_ErrorXml = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/ERRORXML");
                                    if (xpNav_ErrorXml != null)
                                    {
                                        xpNav_Error = xpNav_ErrorXml.SelectSingleNode("//ERRORCODE");
                                        if (xpNav_Error != null && !String.IsNullOrEmpty(xpNav_Error.Value))
                                            ErrorCode = xpNav_Error.Value;
                                    }

                                    strdict_AdditionalInfo.Clear();
                                    strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                    ErrorMessage.AppendFormat("Thumbnail generation failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                                    LogError(ErrorCode, ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);

                                    MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.THUMBNAIL, false);
                                    #endregion
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in Updating Preview Attributes : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);
                
                if(JobProcessingDetailsNode != null)
                    MarkAssetProcessingStatusAsError(JobProcessingDetailsNode, RenditionType.THUMBNAIL, false);
            }
        }

        private Boolean GenerateAssetRenditionNode(ThreadContext ThreadContextObj, String AssetRenditionID, RenditionType RenditionTypeValue, ref XmlNode RenditionNode)
        {
            #region Member Variables
            XmlDocument     DigitalAssetDoc             = null; 
            XmlNode         ColoumnNode                 = null;
            //XmlNode         RenditionNode               = null;
            XmlNode         FileSystemNode              = null;
            XmlNode         FileNode                    = null;
            StringBuilder   ErrorMessage                = null;
            String          ErrorCode                   = null;           
            #endregion

            try
            {                
                #region Generate Rendition node 
                ErrorMessage                = new StringBuilder();
                DigitalAssetDoc             = new XmlDocument();               
                RenditionNode               = DigitalAssetDoc.CreateElement("ASSETRENDITION");
                DigitalAssetDoc.AppendChild(RenditionNode);

                if (!String.IsNullOrEmpty(AssetRenditionID))
                {
                    ColoumnNode = DigitalAssetDoc.CreateElement("ASSETRENDITIONID");
                    ColoumnNode.InnerText = AssetRenditionID;
                    RenditionNode.AppendChild(ColoumnNode);
                }

                ColoumnNode                 = DigitalAssetDoc.CreateElement("NAME");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = ThreadContextObj.OriginalAttrObj.OriginalFileName;
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = ThreadContextObj.PreviewAttrObj.PreviewFileName;
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName;
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("RENDITIONTYPE");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = "Original";
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = "Preview";
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = "Thumbnail";
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode = DigitalAssetDoc.CreateElement("FILEFORMAT");
                if (RenditionTypeValue == RenditionType.ORIGINAL)
                   ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalFileFormat;
                else if (RenditionTypeValue == RenditionType.PREVIEW)
                   ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewFileFormat;
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                   ColoumnNode.InnerText = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat;

                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("FILESIZE");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = ThreadContextObj.OriginalAttrObj.OriginalSize.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = ThreadContextObj.PreviewAttrObj.PreviewSize.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailSize.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("DPI");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalDPI.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewDPI.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("WIDTH");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalWidth.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewWidth.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("HEIGHT");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalHeight.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewHeight.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("DEPTH");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalDepth.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewDepth.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                //For Original: File is already created and added in Vault
                if (((RenditionTypeValue == RenditionType.THUMBNAIL && !ThreadContextObj.ThumbnailExists) 
                    || (RenditionTypeValue == RenditionType.PREVIEW && !ThreadContextObj.UseOriginalAsPreview && !ThreadContextObj.PreviewExists))
                    && (String.Compare(JobType, "GETINFO", true)!=0))
                {   
                    FileSystemNode = DigitalAssetDoc.CreateElement("FILESYSTEM");
                    FileNode = DigitalAssetDoc.CreateElement("FILE");

                    ColoumnNode = DigitalAssetDoc.CreateElement("NAME");
                    if (RenditionTypeValue == RenditionType.ORIGINAL)
                        ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalFileName;
                    else if (RenditionTypeValue == RenditionType.PREVIEW)
                        ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewFileName;
                    else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                        ColoumnNode.InnerText = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName;
                    FileNode.AppendChild(ColoumnNode);

                    ColoumnNode = DigitalAssetDoc.CreateElement("PATH");
                    if (RenditionTypeValue == RenditionType.ORIGINAL)
                        ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                    else if (RenditionTypeValue == RenditionType.PREVIEW)
                        ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewFilePath;
                    else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                        ColoumnNode.InnerText = ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath;
                    FileNode.AppendChild(ColoumnNode);

                    ColoumnNode = DigitalAssetDoc.CreateElement("FIELDNAME");
                    ColoumnNode.InnerText = "ACTUALFILE";
                    FileNode.AppendChild(ColoumnNode);

                    ColoumnNode = DigitalAssetDoc.CreateElement("ONMEDIASERVER");
                    ColoumnNode.InnerText = ThreadContextObj.OnMediaServer.ToString().ToUpper();
                    FileNode.AppendChild(ColoumnNode);

                    FileSystemNode.AppendChild(FileNode);
                    RenditionNode.AppendChild(FileSystemNode);                    
                 }
                
                #endregion
                return true;
            }
            catch (Exception ExceptionObj)
            {
                if (RenditionTypeValue == RenditionType.ORIGINAL)
                {
                    ErrorMessage.AppendFormat("Exception thrown in GenerateAssetRenditionNode for ORIGINAL : Message : {0}", ExceptionObj.Message);
                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETRENDITIONNODEFORORGFAILED";
                }
                if (RenditionTypeValue == RenditionType.PREVIEW)
                {
                    ErrorMessage.AppendFormat("Exception thrown in GenerateAssetRenditionNode for PREVIEW : Message : {0}", ExceptionObj.Message);
                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETRENDITIONNODEFORPREVFAILED";
                }
                if (RenditionTypeValue == RenditionType.THUMBNAIL)
                {
                    ErrorMessage.AppendFormat("Exception thrown in GenerateAssetRenditionNode for THUMBNAIL : Message : {0}", ExceptionObj.Message);
                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETRENDITIONNODEFORTHUMBFAILED";
                }

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError(ErrorCode,ErrorMessage.ToString(),null,strdict_AdditionalInfo, ErrorType.EXCEPTION);				
                return false;
            }
        }        
        
        /// <summary>
        /// This method is used to get the CatalogJob Processing Details
        /// </summary>
        private String GetCatalogJobProcessingDetails()
        {
            #region member varibales
            String      str_SearchInputXML      = null;
            String      str_SearchResultXML     = null;
            String      ErrorXML                = null;
            XmlNode     DataNode                = null;
            #endregion

            try
            {
                #region Initialization
                #endregion

                #region Prepare SearchXML
                str_SearchInputXML = String.Format(@"<SEARCH>
                                                    <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}'/>
		                                            <TARGETOBJECT>CATALOGJOB</TARGETOBJECT>
		                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
		                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
		                                            <TARGETENTITY>
			                                            <PRIVILEGENAME>READ</PRIVILEGENAME>				                                            
			                                            <FILTERLIST>
				                                            <FILTER>
					                                            <FILTERID>1</FILTERID>
					                                            <EXPRESSION>CATALOGJOBID</EXPRESSION>
					                                            <OPERATOR>EQUALTO</OPERATOR>
					                                            <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
					                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            </FILTER>
			                                            </FILTERLIST>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>CATALOGJOBID</FIELDNAME>
					                                            <ALIAS>CATALOGJOBID</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>JOBNAME</FIELDNAME>
					                                            <ALIAS>JOBNAME</ALIAS>
				                                            </EXPRESSION>
                                                            <EXPRESSION>
					                                            <FIELDNAME>STATUS</FIELDNAME>
					                                            <ALIAS>STATUS</ALIAS>
				                                            </EXPRESSION>
                                                            <EXPRESSION>
					                                            <FIELDNAME>JOBPATH</FIELDNAME>
					                                            <ALIAS>JOBPATH</ALIAS>
				                                            </EXPRESSION>	
                                                            <EXPRESSION>
					                                            <FIELDNAME>TOTALASSETS</FIELDNAME>
					                                            <ALIAS>TOTALASSETS</ALIAS>
				                                            </EXPRESSION>	
                                                            <EXPRESSION>
					                                            <FIELDNAME>NOOFPROCESSEDASSETS</FIELDNAME>
					                                            <ALIAS>NOOFPROCESSEDASSETS</ALIAS>
				                                            </EXPRESSION>						                                           
			                                            </SELECTLIST>
                                                        <RELATIONSHIP NAME='CATALOGJOB_PROCESSINGDETAILS'/>					
		                                            </TARGETENTITY>
	                                            </SEARCH>", str_SessionID, str_UserID,str_TenantName,str_Language, CatalogJobID);
                #endregion

                #region perform search
                ErrorXML        = ObjSearch.GetHierarchialData(str_SearchInputXML, ref str_SearchResultXML);
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    JobProcessingDetails = new XmlDocument();
                    JobProcessingDetails.LoadXml(str_SearchResultXML);
                    JobNode              = JobProcessingDetails.SelectSingleNode("CATALOGJOBCOLLECTION/CATALOGJOB");

                    if(JobNode != null)
                    {
                        DataNode = JobNode.SelectSingleNode("JOBPATH");
                        if(DataNode != null && !String.IsNullOrEmpty(DataNode.InnerText))
                            JobPath = DataNode.InnerText;

                        DataNode = JobNode.SelectSingleNode("JOBNAME");
                        if(DataNode != null && !String.IsNullOrEmpty(DataNode.InnerText))
                            JobName = DataNode.InnerText;
                        
                        SetNumberOfProcessedAsset(JobNode);

                        CatalogerContextObj.JobInformationObj           = new CatalogerContext.JobInformation();
                        CatalogerContextObj.JobInformationObj.JobNode   = JobNode;
                        CatalogerContextObj.JobInformationObj.JobPath   = JobPath;
                    }
                }               
                #endregion
            }
            catch (Exception ExceptionObj)
            {
                ThreadObj.sb_Error.Append(ExceptionObj.Message);
                ThreadObj.sb_Error.Append(ExceptionObj.StackTrace);
                return ThreadObj.sb_Error.ToString();
            }
            return ErrorXML;
        }

        private void SetThreadContextObject(XPathNavigator JobDetailsNode, RenditionType RenditionTypeValue, XPathNavigator xpNav_Output, XPathNavigator xpNav_MediaResponseXML)
        {
            #region member variablss
            bool        isExists        = false;
            bool        isBinHex        = false;
            bool        ObjBool         = false;
            ulong       EntityID        = 0;
            long        ObjLong         = 0;                        
            int         height          = 0;
            int         Width           = 0;
            int         Depth           = 0;
            int         DPI             = 0;
            String      DestinationName = null;
            XPathNavigator xpNav_DataNode = null;
            #endregion

            try
            {
                ThreadContextObj = new ThreadContext();
                
                #region Set Common attributes
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWEXISTS");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out isExists);
                    ThreadContextObj.PreviewExists = isExists;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILEXISTS");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out isExists);
                    ThreadContextObj.ThumbnailExists = isExists;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ISBINHEX");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out isBinHex);
                    ThreadContextObj.IsBinHex = isBinHex;
                }
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ISTHUMBBINHEX");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out isBinHex);
                    ThreadContextObj.IsThumbBinHex = isBinHex;
                }
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ISPRVBINHEX");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out isBinHex);
                    ThreadContextObj.IsPrvBinHex = isBinHex;
                }
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/GENTHUMBFROMPREVIEW");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out ObjBool);
                    ThreadContextObj.GenThumbFromPreview = ObjBool;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THBFROMORGGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ThbFromOrgGenerator = xpNav_DataNode.Value;

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/FORMATDETAILS");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.InnerXml))
                {
                    XmlDocument TempXML             = new XmlDocument();
                    XmlNode FormatDetailsNode       = TempXML.CreateElement("FORMATDETAILS");
                    FormatDetailsNode.InnerXml      = xpNav_DataNode.InnerXml;
                    ThreadContextObj.FormatDetails  = FormatDetailsNode;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/DIGITALASSETID");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    ulong.TryParse(xpNav_DataNode.Value, out EntityID);
                    ThreadContextObj.DigitalAssetID = EntityID;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/CONTENTREVISIONID");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ContentRevisionID = xpNav_DataNode.Value;

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/SECTIONID");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    ulong.TryParse(xpNav_DataNode.Value, out EntityID);
                    ThreadContextObj.SectionID = EntityID;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ONMEDIASERVER");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out ObjBool);
                    ThreadContextObj.OnMediaServer = ObjBool;
                }

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/TEMPFOLDERPATH");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.TempFolderPath = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PARENTDIGITALASSETNAME");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ParentDigitalAssetName = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/SECTIONPATH");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.str_SectionPath = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/MEDIAPROCESSOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.MediaProcessor = xpNav_DataNode.Value;

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/USEORIGINALASPREVIEW");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    Boolean.TryParse(xpNav_DataNode.Value, out ObjBool);
                    ThreadContextObj.UseOriginalAsPreview = ObjBool;
                }
                
                #endregion

                #region Set Original attributes
                ThreadContextObj.OriginalAttrObj = new OriginalAttributes();

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORIGINALFILENAME");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OriginalFileName = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORIGINALFILEPATH");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OriginalFilePath = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORIGINALFILEEXTENSION");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OriginalFileExtension = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORIGINALFILEFORMAT");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OriginalFileFormat = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORIGINALSIZE");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    long.TryParse(xpNav_DataNode.Value, out ObjLong);
                    ThreadContextObj.OriginalAttrObj.OriginalSize = ObjLong;
                }
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORGMACTYPE");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OrgMacType = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORGMACCREATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OrgMacCreator = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/ORIGINALGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.OriginalGenerator = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/MANAGEDGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    bool.TryParse(xpNav_DataNode.Value, out ObjBool);
                    ThreadContextObj.OriginalAttrObj.ManagedGenerator = ObjBool;
                }
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/ORIGINALATTROBJ/INTERMIDIATEORGFILEPATH");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath = xpNav_DataNode.Value;                
                #endregion

                #region Set Preview attributes
                ThreadContextObj.PreviewAttrObj = new PreviewAttributes();
                //if (ThreadContextObj.PreviewExists || ThreadContextObj.UseOriginalAsPreview)
                //{
                    xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PREVIEWFILENAME");
                    if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                        ThreadContextObj.PreviewAttrObj.PreviewFileName = xpNav_DataNode.Value;
                    xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PREVIEWFILEPATH");
                    if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                        ThreadContextObj.PreviewAttrObj.PreviewFilePath = xpNav_DataNode.Value;
                    xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PREVIEWFILEEXTENSION");
                    if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                        ThreadContextObj.PreviewAttrObj.PreviewFileExtension = xpNav_DataNode.Value;                   
                //}

                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PREVIEWFILEFORMAT");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.PreviewAttrObj.PreviewFileFormat = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PREVIEWGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.PreviewAttrObj.PreviewGenerator = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PRVMACTYPE");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.PreviewAttrObj.PrvMacType = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/PRVMACCREATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.PreviewAttrObj.PrvMacCreator = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/PREVIEWATTROBJ/MANAGEDGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    bool.TryParse(xpNav_DataNode.Value, out ObjBool);
                    ThreadContextObj.PreviewAttrObj.ManagedGenerator = ObjBool;
                }
                #endregion

                #region Set Thumbnail attributes
                ThreadContextObj.ThumbnailAttrObj = new ThumbnailAttributes();
                //if (ThreadContextObj.ThumbnailExists)
                //{
                    xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBNAILFILENAME");
                    if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName = xpNav_DataNode.Value;
                    xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBNAILFILEPATH");
                    if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath = xpNav_DataNode.Value;
                    xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBNAILFILEEXTENSION");
                    if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension = xpNav_DataNode.Value;
                //}
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBNAILFILEFORMAT");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBNAILGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBMACTYPE");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ThumbnailAttrObj.ThumbMacType = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/THUMBMACCREATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                    ThreadContextObj.ThumbnailAttrObj.ThumbMacCreator = xpNav_DataNode.Value;
                xpNav_DataNode = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/THUMBNAILATTROBJ/MANAGEDGENERATOR");
                if (xpNav_DataNode != null && !String.IsNullOrEmpty(xpNav_DataNode.Value))
                {
                    bool.TryParse(xpNav_DataNode.Value, out ObjBool);
                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = ObjBool;
                }
                #endregion

                #region Set Fileattributes              
                if (xpNav_Output != null)
                {
                    int.TryParse(xpNav_Output.GetAttribute("HEIGHT", ""), out height);
                    int.TryParse(xpNav_Output.GetAttribute("WIDTH", ""), out Width);
                    int.TryParse(xpNav_Output.GetAttribute("DEPTH", ""), out Depth);
                    int.TryParse(xpNav_Output.GetAttribute("DPI", ""), out DPI);

                    DestinationName         = xpNav_Output.GetAttribute("DESTINATIONFILENAME", "");
                    FileInfo FileInfoObj    = new FileInfo(DestinationName);
                    switch (RenditionTypeValue)
                    {
                        case RenditionType.ORIGINAL:
                            ThreadContextObj.OriginalAttrObj.OriginalHeight     = height;
                            ThreadContextObj.OriginalAttrObj.OriginalWidth      = Width;
                            ThreadContextObj.OriginalAttrObj.OriginalDepth      = Depth;
                            ThreadContextObj.OriginalAttrObj.OriginalDPI        = DPI;
                            CurrentProcessingFileName                           = ThreadContextObj.OriginalAttrObj.OriginalFilePath;

                            if (ThreadContextObj.UseOriginalAsPreview)
                            { 
                                ThreadContextObj.PreviewAttrObj.PreviewHeight   = height;
                                ThreadContextObj.PreviewAttrObj.PreviewWidth    = Width;
                                ThreadContextObj.PreviewAttrObj.PreviewDepth    = Depth;
                                ThreadContextObj.PreviewAttrObj.PreviewDPI      = DPI;
                            }
                            break;
                         case RenditionType.PREVIEW:
                            ThreadContextObj.PreviewAttrObj.PreviewHeight       = height;
                            ThreadContextObj.PreviewAttrObj.PreviewWidth        = Width;
                            ThreadContextObj.PreviewAttrObj.PreviewDepth        = Depth;
                            ThreadContextObj.PreviewAttrObj.PreviewDPI          = DPI;
                            if (FileInfoObj.Exists)
                            {
                                ThreadContextObj.PreviewAttrObj.PreviewSize = FileInfoObj.Length;
                                if (String.Compare(DestinationName, ThreadContextObj.PreviewAttrObj.PreviewFilePath, true) != 0)
                                    if (!File.Exists(ThreadContextObj.PreviewAttrObj.PreviewFilePath))
                                        FileInfoObj.MoveTo(ThreadContextObj.PreviewAttrObj.PreviewFilePath);
                            }
                            if (ThreadContextObj.PreviewAttrObj.PreviewHeight <= 0 || ThreadContextObj.PreviewAttrObj.PreviewWidth <= 0)
                                GetPreviewAttributesExplicitly = true;

                            CurrentProcessingFileName                           = DestinationName;
                            break;
                        case RenditionType.THUMBNAIL:
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight   = height;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth    = Width;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth    = Depth;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI      = DPI;                            
                            if (FileInfoObj.Exists)
                            {
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailSize = FileInfoObj.Length;
                                if (String.Compare(DestinationName, ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath, true) != 0)
                                    FileInfoObj.MoveTo(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath);
                            }
                            
                            if ((ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight <= 0) && (ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth <= 0))
                                GetThumbnailAttributesExplicitly = true;
                            CurrentProcessingFileName                           = DestinationName;
                            break;
                    }
                }
                #endregion

                if (CatalogerContextObj.AssetInformationObj != null)
                {
                    CatalogerContextObj.AssetInformationObj.OriginalFileName    = ThreadContextObj.OriginalAttrObj.OriginalFileName;
                    CatalogerContextObj.AssetInformationObj.OriginalFilePath    = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                    CatalogerContextObj.AssetInformationObj.PreviewFileName     = ThreadContextObj.PreviewAttrObj.PreviewFileName;
                    CatalogerContextObj.AssetInformationObj.ThumbnailFileName   = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName;
                }
            }
            catch (Exception)
            {
            }
        }

        private Boolean TransferFileAndUpdatePath(XmlNode AssetRenditionNode)
        {
            FileTransferInfrastructure  FileTransferInfrastructureObj   = null;
            XML                         FileListXML                     = null;
            XmlNode                     FileCollectionNode              = null;
            XmlDocument                 FileTransferXMLDoc              = null;
            ERRORXML                    ErrorXML                        = null;
            StringBuilder               ErrorMessage                    = null;
            XmlNode                     FileNode                        = null;
            try
            {
                FileNode = AssetRenditionNode.SelectSingleNode("FILESYSTEM/FILE");
                if (FileNode != null)
                {
                    //If Destination loaction for file Media Server then don't transfer the
                    //file to services Machine. (Since the will copied form catalger mount point
                    //location to vault using Media server at OM Level.
                    if (FileNode.SelectSingleNode("ONMEDIASERVER") != null)
                    {
                        if (Boolean.Parse(FileNode.SelectSingleNode("ONMEDIASERVER").InnerText))
                            return true; //No need to Physically Transefer the File
                    }

                    FileTransferXMLDoc              = new XmlDocument();
                    ErrorMessage                    = new StringBuilder();
                    FileTransferInfrastructureObj   = new FileTransferInfrastructure(str_SessionID, str_UserID, str_TenantName, str_Language);
                    FileCollectionNode              = FileTransferXMLDoc.CreateElement("FILECOLLECTION");
                    FileTransferInfrastructureObj   = new FileTransferInfrastructure(str_SessionID, str_UserID, str_TenantName, str_Language);
                   
                    FileCollectionNode.InnerXml = FileNode.OuterXml;
                    FileListXML                 = FileCollectionNode.OuterXml;
                    if ((ErrorXML = FileTransferInfrastructureObj.SaveFile(ref FileListXML)) == null)
                    {
                        FileCollectionNode.InnerXml = FileListXML;
                        AssetRenditionNode.SelectSingleNode("FILESYSTEM").InnerXml = FileCollectionNode.SelectSingleNode("FILECOLLECTION").InnerXml;
                    }
                    else
                    {
                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("FILENAME", FileNode.SelectSingleNode("NAME").InnerText);
                        ErrorMessage.Remove(0,ErrorMessage.Length);
                        ErrorMessage.AppendFormat("Failed to transfer file {0}",AssetRenditionNode.SelectSingleNode("FILESYSTEM/FILE/NAME").InnerText);
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FAILEDTOTRANSFERFILE", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);
                    }
                }
                return true;

            }
            catch (Exception)
            {
				
				return false;
            }
        }
        
        private ERRORXML LogError(String ErrorCode, String ErrorMessage,String InnerError, StringDictionary strdict_AdditionalInfo,  ErrorType ErrorTypeValue)
        {
            ERRORXML        ErrorXML                        = null;
            XmlNode         JobErrorNode                    = null;
            XmlNode         ColoumnNode                     = null;
            XmlNode         ErrorDetailsNode                = null;
            XML             LogErrorXML                     = null;
            XmlNode         MessageNode                     = null;
            XmlNode         ErrorNode                       = null;
            XmlDocument     InnerErrorXMLDoc                = null;
            XmlNode         ErrorCollectionNode             = null;
            XmlNode         xmlnod_AdditionalInfo           = null;
            XmlNode         xmlnod_AdditionalInfoField      = null;
            XmlNode         xmlnod_ErrorCode                = null;

            LoggedERRORID = null;

           try
			{
		       if (InnerError != null)
                {
                    InnerErrorXMLDoc                    = new XmlDocument();
                    InnerErrorXMLDoc.LoadXml(InnerError);
                    if (InnerErrorXMLDoc.SelectSingleNode("ERRORCOLLECTION") == null)
                    {
                        ErrorCollectionNode             = InnerErrorXMLDoc.CreateElement("ERRORCOLLECTION");
                        ErrorCollectionNode.InnerXml    = InnerError;
                        InnerError                      = ErrorCollectionNode.OuterXml;
                    }
                }
                else
                {
                    InnerError = String.Empty;
                }

                if (ErrorXMLDoc == null)
                {
                    ErrorXMLDoc                 = new XmlDocument();
                    JobErrorNode                = ErrorXMLDoc.CreateElement("JOBERROR");
                    
                    ColoumnNode                 = ErrorXMLDoc.CreateElement("APPLICATIONNAME");
                    ColoumnNode.InnerText       = "Cataloger";
                    JobErrorNode.AppendChild(ColoumnNode);

                    ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORTYPE");
                    if(ErrorTypeValue==ErrorType.ERROR)
                        ColoumnNode.InnerText   = "ERROR";
                    else
                        ColoumnNode.InnerText   = "EXCEPTION";
                    JobErrorNode.AppendChild(ColoumnNode);

                    ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORMESSAGE");
                    ErrorNode                   = ErrorXMLDoc.CreateElement("ERROR");
                    MessageNode                 = ErrorXMLDoc.CreateElement("MESSAGE");
                    xmlnod_ErrorCode            = ErrorXMLDoc.CreateElement("ERRORCODE");

                    if (!String.IsNullOrEmpty(ErrorCode))
                    {
                        xmlnod_ErrorCode.InnerText = ErrorCode;
                    }
                    ErrorNode.AppendChild(xmlnod_ErrorCode);

                    MessageNode.InnerText       = ErrorMessage;
                    ErrorNode.AppendChild(MessageNode);

                    if (InnerError != null)
                    {
                        ErrorDetailsNode            = ErrorXMLDoc.CreateElement("ERRORDETAILS");
                        ErrorDetailsNode.InnerXml   = InnerError;
                        ErrorNode.AppendChild(ErrorDetailsNode);
                    }

                    if (strdict_AdditionalInfo != null)
                    {
                        xmlnod_AdditionalInfo = ErrorXMLDoc.CreateElement("ADDITIONALINFO");
                        IEnumerator ienum_AdditionalInfo = strdict_AdditionalInfo.GetEnumerator();

                        foreach (DictionaryEntry obj_DictionaryEntry in strdict_AdditionalInfo)
                        {

                            xmlnod_AdditionalInfoField = ErrorXMLDoc.CreateElement(obj_DictionaryEntry.Key.ToString().ToUpper());
                            if (obj_DictionaryEntry.Value != null)
                            {
                                xmlnod_AdditionalInfoField.InnerXml = ALCommon.TrimAndReplaceSpecialChars(obj_DictionaryEntry.Value.ToString());
                            }
                            xmlnod_AdditionalInfo.AppendChild(xmlnod_AdditionalInfoField);
                        }

                        ErrorNode.AppendChild(xmlnod_AdditionalInfo);
                    }

                    ColoumnNode.AppendChild(ErrorNode);
                    JobErrorNode.AppendChild(ColoumnNode);

                    ColoumnNode                 = ErrorXMLDoc.CreateElement("CATALOGJOB_JOBERROR");
                    ((XmlElement)ColoumnNode).SetAttribute("TYPE", "RELATION");
                    ((XmlElement)ColoumnNode).SetAttribute("PARENTID", "TRUE");
                    ColoumnNode.InnerText       = CatalogJobID;
                    JobErrorNode.AppendChild(ColoumnNode);

                    ErrorXMLDoc.AppendChild(JobErrorNode);
                }
                else
                {
                    if (ErrorTypeValue == ErrorType.ERROR)
                        ErrorXMLDoc.SelectSingleNode("//ERRORTYPE").InnerText           = "ERROR";
                    else
                        ErrorXMLDoc.SelectSingleNode("//ERRORTYPE").InnerText           = "EXCEPTION";

                    if (strdict_AdditionalInfo != null)
                    {
                        xmlnod_AdditionalInfo = ErrorXMLDoc.SelectSingleNode("//ADDITIONALINFO");
                        if (xmlnod_AdditionalInfo == null)
                        {
                            ErrorNode = ErrorXMLDoc.SelectSingleNode("//ERROR");
                            xmlnod_AdditionalInfo = ErrorXMLDoc.CreateElement("ADDITIONALINFO");
                            ErrorNode.AppendChild(xmlnod_AdditionalInfo);
                        }
                        else
                        {
                            xmlnod_AdditionalInfo.RemoveAll();
                        }

                        IEnumerator ienum_AdditionalInfo = strdict_AdditionalInfo.GetEnumerator();
                        foreach (DictionaryEntry obj_DictionaryEntry in strdict_AdditionalInfo)
                        {

                            xmlnod_AdditionalInfoField = ErrorXMLDoc.CreateElement(obj_DictionaryEntry.Key.ToString().ToUpper());
                            if (obj_DictionaryEntry.Value != null)
                            {
                                xmlnod_AdditionalInfoField.InnerXml = obj_DictionaryEntry.Value.ToString();
                            }
                            xmlnod_AdditionalInfo.AppendChild(xmlnod_AdditionalInfoField);
                        }
                    }
                    else
                    {
                        xmlnod_AdditionalInfo = ErrorXMLDoc.SelectSingleNode("//ADDITIONALINFO");
                        if (xmlnod_AdditionalInfo != null)
                        {
                            ErrorNode = ErrorXMLDoc.SelectSingleNode("//ERROR");
                            ErrorNode.RemoveChild(xmlnod_AdditionalInfo);
                        }
                    }

                    ErrorXMLDoc.SelectSingleNode("//ERRORCODE").InnerText               = ErrorCode;
                    ErrorXMLDoc.SelectSingleNode("//MESSAGE").InnerText                 = ErrorMessage;
                    ErrorXMLDoc.SelectSingleNode("//ERRORDETAILS").InnerXml             = InnerError;
                    ErrorXMLDoc.SelectSingleNode("//CATALOGJOB_JOBERROR").InnerText     = CatalogJobID; 
                }

                LogErrorXML = ErrorXMLDoc.InnerXml;
                ErrorXML    = DAMWSObj.LogError(ref LogErrorXML);
                if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(LogErrorXML))
                    LoggedERRORID = ALCommon.GetInnerValue(ref LogErrorXML, "<ERRORID>", "</ERRORID>");
                return ErrorXML;
			}
		   catch(Exception e)
			{
				//CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
				return e.Message;
			}
        }

        private bool UpdateCatalogJobProcessingDetails(ref XmlNode JobDetailsNode)
        {
            #region member variables
            ERRORXML     ErrorXML               = null;
            String       JobDetailsXML          = null;
            XmlNode      DataNode               = null;
            String      ProcessingStatus        = null;
            XmlNode     CatJobDetailsNode       = null;
            bool        AssetProcessed          = false;
            FileInfo    FileInfoObj             = null;
            String      BatchUpdateXML          = null;
            XmlNode     ErrorIDList             = null;
            String      str_ErrorIDList         = null;
            #endregion

            try
            {                              

                #region Check Asset procesing status   
                CatJobDetailsNode = JobDetailsNode.SelectSingleNode("CATALOGJOBDETAILS");
                if (IsAssetProcessingComplete(ref CatJobDetailsNode, ref ProcessingStatus))
                {
                    DataNode = CatJobDetailsNode.SelectSingleNode("PROCESSINGSTATUS");
                    if (DataNode != null)
                        DataNode.InnerText = ProcessingStatus;

                    AssetProcessed  = true;
                    NumberOfProcessedAssets++;
                }
                #endregion

                #region Update Details    
                //Update ERRORIDLIST if present
                if (!String.IsNullOrEmpty(LoggedERRORID))
                {
                    ErrorIDList = JobDetailsNode.SelectSingleNode("CATALOGJOBDETAILS/ERRORIDLIST");
                    if (ErrorIDList == null)
                    {
                        ErrorIDList = JobDetailsNode.OwnerDocument.CreateElement("ERRORIDLIST");
                        JobDetailsNode.SelectSingleNode("CATALOGJOBDETAILS").AppendChild(ErrorIDList);
                    }                    
                    str_ErrorIDList         = String.Format("{0},{1}", ErrorIDList.InnerText, LoggedERRORID);
                    ErrorIDList.InnerText   = str_ErrorIDList;
                    LoggedERRORID           = null;
                }

                XmlNode ClonedDANode = JobDetailsNode.SelectSingleNode("CATALOGJOBDETAILS/JOBDETAILSXML/JOBDATA/CLONEDDIGITALASSETNODE");
                if (ClonedDANode != null)
                    ClonedDANode.ParentNode.RemoveChild(ClonedDANode);

                JobDetailsXML   = String.Format(@"<CATALOGJOB>
                                                  <CATALOGJOBID>{0}</CATALOGJOBID>  
                                                  <STATUS>In-Progress</STATUS>                                                 
                                                  {1}  
                                                </CATALOGJOB>", CatalogJobID, JobDetailsNode.OuterXml);

                ErrorXML        = DAMWSObj.UpdateCatalogJob(JobDetailsXML);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    //Error in Updating CatalogJob processing Details                    
                    ThreadObj.sb_Error.Append(ErrorXML);
                    return false;
                }

                //Update No of Processed Asset count
                if (AssetProcessed)
                {
                    BatchUpdateXML = String.Format(@"<ENTITYCOLLECTION>
                                                   <CATALOGJOB AuthorizationRequired='FALSE'>
                                                     <FILTERLIST>
                                                        <FILTER>
                                                            <EXPRESSION>CATALOGJOBID</EXPRESSION>
                                                            <OPERATOR>EQUALTO</OPERATOR>
                                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                            <ISCASESENSITIVE>NO</ISCASESENSITIVE>
                                                        </FILTER>
                                                     </FILTERLIST>                                                                                                  
                                                     <NOOFPROCESSEDASSETS>NOOFPROCESSEDASSETS+1</NOOFPROCESSEDASSETS>                                                          
                                                  </CATALOGJOB>                                                                                      
                                                  </ENTITYCOLLECTION>", CatalogJobID);                                                         

                    ErrorXML = InfrastructureWSObj.BatchUpdateEntityFacetInstances(BatchUpdateXML, "ENTITY", true);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        //Error in Updating CatalogJob processing Details                    
                        ThreadObj.sb_Error.Append(ErrorXML);
                        //String str_Log1 = String.Format("<ASSET>Processing Complete for Asset: {0} BUT UPDATE COUNT FAILED</ASSET>", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                        //File.AppendAllText(@"c:\Assetlink_R13\COUNTUPDATE_AssetLog.txt", str_Log1);
                        return false;
                    }

                    //String str_Log = String.Format("<ASSET>Processing Complete for Asset: {0} </ASSET>", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                    //File.AppendAllText(@"c:\Assetlink_R13\Processed_AssetLog.txt", str_Log);
                }
                
                if(AssetProcessed)
                    InvokePlugin("AFTERASSETPROCESSING", CatalogerContextObj);
                
                #endregion

                #region CleanUp temporary files
                if(AssetProcessed)
                {
                    if (ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath != null)
                    {
                        if (File.Exists(ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath) && ThreadContextObj.PreviewAttrObj.IsFileCopiedToTempLocation)
                        {
                            try
                            {
                                FileInfoObj = new FileInfo(ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath);
                                FileInfoObj.Delete();
                            }
                            catch (Exception)
                            {
                                // This could occur of the Temp file is still being used by some generator and we try to delete it
                                // as wrapper return when thred times out. This will throw an exception but we should not stamp the error
                                // bec Its not very critical to delete temp file.
                            }
                        }
                    }

                    if (ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath != null)
                    {
                        if (File.Exists(ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath) && ThreadContextObj.ThumbnailAttrObj.IsFileCopiedToTempLocation)
                        {
                            try
                            {
                                FileInfoObj = new FileInfo(ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath);
                                FileInfoObj.Delete();
                            }
                            catch (Exception)
                            {
                                // This could occur of the Temp file is still being used by some generator and we try to delete it
                                // as wrapper return when thred times out. This will throw an exception but we should not stamp the error
                                // bec Its not very critical to delete temp file.
                            }
                        }
                    }
                    if (ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath != null && ThreadContextObj.OriginalAttrObj.IsFileCopiedToTempLocation)
                    {
                        try
                        {
                            FileInfoObj = new FileInfo(ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath);
                            FileInfoObj.Delete();
                        }
                        catch (Exception)
                        {
                            // This could occur of the Temp file is still being used by some generator and we try to delete it
                            // as wrapper return when thred times out. This will throw an exception but we should not stamp the error
                            // bec Its not very critical to delete temp file.
                        }                       
                    }     
                }
                #endregion
                return true;
            }
            catch (Exception ExceptionObj)
            {
                ThreadObj.sb_Error.Append(ExceptionObj.Message);
                ThreadObj.sb_Error.Append(ExceptionObj.StackTrace);
                return false;
            }
        }

        private bool UpdateCatalogJobStatus()
        {
            #region member variables
            XmlNodeList AssetInJobList          = null;
            XmlNode     AssetNode               = null;
            XmlNode     ProcessingStatusNode    = null;
            Int32       ProcessedAssetCnt       = 0;            
            String      ErrorXML                = null;
            String      ProcessingStatus        = null;
            bool        IsSuccess               = false;
            bool        IsProcessingComplete    = false;
            String      TotalAssets             = "";
            int         TotalAssetsCnt          = 0;
            Assetlink.Services.WindowsService.CatalogerHelper.SendNotification NotificationObj = null;            
            XmlNode     TempNode                = null;
            #endregion

            try
            {
                GetCatalogJobProcessingDetails();
                if (JobNode != null)
                {
                    #region Check if Processing of all Assets from the Job is complete
                    AssetInJobList = JobNode.SelectNodes("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS");
                    
                    if (JobNode.SelectSingleNode("TOTALASSETS") != null)
                    {
                        TotalAssets = JobNode.SelectSingleNode("TOTALASSETS").InnerText;
                        Int32.TryParse(TotalAssets, out TotalAssetsCnt);
                    }

                    for (Int32 AssetCnt = 0; AssetCnt < AssetInJobList.Count; AssetCnt++)
                    {
                        AssetNode = AssetInJobList[AssetCnt];
                        if (IsAssetProcessingComplete(ref AssetNode, ref ProcessingStatus))
                        {
                            ProcessingStatusNode = AssetNode.SelectSingleNode("PROCESSINGSTATUS");
                            if (ProcessingStatusNode != null)
                                ProcessingStatusNode.InnerText = ProcessingStatus;

                            ProcessedAssetCnt++;
                        }
                    }
                    #endregion

                    //Update Catalog JobStatus to Done/Error If all assets are processed.
                    if (ProcessedAssetCnt == TotalAssetsCnt)
                    {
                        #region Update Catalog job table
                        IsProcessingComplete    = true;
                        IsJobProcessingComplete = true;
                        AssetInJobList          = JobNode.SelectNodes("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS[PROCESSINGSTATUS='Error']");
                        if (AssetInJobList.Count > 0)
                        {
                            //Mark the JobStatus as Error
                            JobNode.SelectSingleNode("STATUS").InnerText    = "Error";
                            IsSuccess                                       = false;
                            IsJobSuccesseded = false;
                        }
                        else
                        {
                            //Mark the JobStatus as Done
                            JobNode.SelectSingleNode("STATUS").InnerText    = "Done";
                            IsSuccess                                       = true;
                            IsJobSuccesseded                                = true;
                        }

                        TempNode = JobNode.SelectSingleNode ("CATALOGJOB_PROCESSINGDETAILS");
                        if (TempNode != null)
                            TempNode.ParentNode.RemoveChild(TempNode);

                        ErrorXML = DAMWSObj.UpdateCatalogJob(JobNode.OuterXml);
                        if (!String.IsNullOrEmpty(ErrorXML))
                        {
                            ThreadObj.sb_Error.Append(ErrorXML);
                            return false;
                        }
                        #endregion

                        #region Invoke Cataloger Extension
                        if (IsProcessingComplete)
                        {                           
                           InvokePlugin("JOBPROCESSINGEND", CatalogerContextObj);
                        }
                        #endregion
                                                  
                        
                    }//Processing All Asset's is complete
                }
            }
            catch (Exception ExceptionObj)
            {
                ThreadObj.sb_Error.Append(ExceptionObj.Message);
                ThreadObj.sb_Error.Append(ExceptionObj.StackTrace);
                return false;
            }
            finally
            {
                try
                {
                    #region Stamp Notification for Job Processing Over
                    if (IsProcessingComplete)
                    {
                        bool ErrorFlag = false;
                          // Send mail
                        if (!IsSuccess)
                            ErrorFlag = true;

                          if (!string.IsNullOrEmpty(UserNotificationPref.SendMailTo))
                              if ((UserNotificationPref.SendMailSettings.ToString().CompareTo("ALWAYS") == 0) || ((UserNotificationPref.SendMailSettings.ToString().CompareTo("ONERROR") == 0) && ErrorFlag))
                              {
                                  GetSenderDetails();                                 

                                  NotificationObj                   = new Assetlink.Services.WindowsService.CatalogerHelper.SendNotification();
                                  NotificationObj.SenderName        = SenderName;
                                  NotificationObj.SenderEmail       = SenderEmail;
                                  NotificationObj.TenantName        = str_TenantName;

                                  ErrorXML                          = NotificationObj.StampNotification(SenderID, JobName, JobName, UserNotificationPref.SendMailTo, JobName, ErrorFlag);
                                  if (!String.IsNullOrEmpty(ErrorXML))
                                  {
                                      ErrorMessage.Append("Error in stamping notification");
                                      LogError("CATALOGJOBPROCESSOR_JOBHANDLER_STAMPNOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML,null, ErrorType.ERROR);
                                  }
                              }
                          
                          // Clean-Up the job folder
                          /*if (Directory.Exists(JobPath))
                          {
                              Directory.Delete(JobPath, true);
                          }*/                                               
                    }
                    #endregion
                }
				catch(Exception ExceptionObj)
				{
                    ThreadObj.sb_Error.Append("Exception Occured while stamping Notification after Job Processing Over: " + ExceptionObj.Message);
                    ThreadObj.sb_Error.Append(ExceptionObj.StackTrace);                    
				}                
            }            
            return true;
        }

        private bool IsAssetProcessingComplete(ref XmlNode JobDetailsNode, ref String ProcessingStatus)
        {
            XmlNode      DataNode               = null;
            String[]    StatusFields            = { "ORGATTRIBUTEUPDATED", "PREVIEWGENERATED", "PRVATTRIBUTEUPDATED", "THUMBNAILGENERATED", "THBATTRIBUTEUPDATED", "ISVALIDBUSINESSINFO" };
            bool         isProcessingComplete   = true;

            ProcessingStatus = "";
            //Check if all processing is complete for the current Asset, if Yes then Update status for the Current Asset in Processing Details Table
            foreach (String Field in StatusFields)
            {
                DataNode = JobDetailsNode.SelectSingleNode(Field);
                if (DataNode != null && !String.IsNullOrEmpty(DataNode.InnerText) && DataNode.InnerText != "0")
                {
                    if (DataNode.InnerText == "1" || DataNode.InnerText == "2")
                    {
                        if (String.Compare(ProcessingStatus,"Error", true) != 0)
                            ProcessingStatus = "Done";
                    }
                    else if (DataNode.InnerText == "3")
                        ProcessingStatus = "Error";
                    continue;
                }
                else
                {
                    ProcessingStatus = "In-Progress";
                    isProcessingComplete = false;
                    break;
                }
            }
            return isProcessingComplete;
        }

        private void SetCatalogerContextObject(XPathNavigator xpNav_MediaResponseXML)
        {
            String      AssetInfoPath               = null;
            XmlNode     ClonedDigitalAssetNode      = null;
            XmlDocument TempDocument                = null;
            try
            {
                CatalogerContextObj.AssetInformationObj     = new CatalogerContext.AssetInformation();
                //CatalogerContextObj.JobInformationObj       = new CatalogerContext.JobInformation(); //Initialized previouly
                CatalogerContextObj.UserPreferencesObj      = new UserPreferences();
                CatalogerContextObj.SessionInformationObj   = new CatalogerContext.SessionInformation();

                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/TENANTCATALOGINGPREFERENCESXML") != null
                       && !String.IsNullOrEmpty(xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/TENANTCATALOGINGPREFERENCESXML").InnerXml))
                    TenantCatalogingPreferencesXML = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/TENANTCATALOGINGPREFERENCESXML").OuterXml;

                GetUserPreferences(CatalogerContextObj);

                CatalogerContextObj.SessionInformationObj.SessionID     = ul_SessionID;
                CatalogerContextObj.SessionInformationObj.UserID        = ul_UserID;
                CatalogerContextObj.SessionInformationObj.TenantName    = str_TenantName;
                CatalogerContextObj.SessionInformationObj.LoginName     = "";

                CatalogerContextObj.JobInformationObj.JobPath           = JobPath;
                AssetInfoPath                                           = String.Format("{0}\\AssetInfo", JobPath);
                CatalogerContextObj.JobInformationObj.AssetInfoDocument = new XmlDocument();
                CatalogerContextObj.JobInformationObj.AssetInfoDocument.Load(AssetInfoPath);
                CatalogerContextObj.JobInformationObj.JobNode           = JobNode;

                TempDocument            = new XmlDocument();
                ClonedDigitalAssetNode  = TempDocument.CreateElement("DIGITALASSET");
                if (xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/CLONEDDIGITALASSETNODE") != null
                       && !String.IsNullOrEmpty(xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/CLONEDDIGITALASSETNODE").InnerXml))
                    ClonedDigitalAssetNode.InnerXml = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION/JOBDETAILS/JOBPROCESSINGDETAILS/JOBDATA/CLONEDDIGITALASSETNODE").InnerXml;

                CatalogerContextObj.AssetInformationObj.AssetNode       = ClonedDigitalAssetNode;                
            }
            catch (Exception)
            { 
            }

        }

        private void SetNumberOfProcessedAsset(XmlNode JobNode)
        {
            XmlNode ProcessedAssetNode = null;
            ProcessedAssetNode = JobNode.SelectSingleNode("NOOFPROCESSEDASSETS");
            if (ProcessedAssetNode != null && !String.IsNullOrEmpty(ProcessedAssetNode.InnerText))
            {
                Int32.TryParse(ProcessedAssetNode.InnerText, out NumberOfProcessedAssets);
            }

        }

        private bool IsSessionValid()
        {
            String  ErrorXML    = null;
            bool    IsValid     = false;
           
            ErrorXML = Infrastructure.IsSessionValid(ul_UserID, ul_SessionID, str_Language, str_TenantName, ref IsValid);
            if (!IsValid)
            {
                if (!Connect(LoginName))
                    return false;
            }           
            return IsValid;
        }

        private void GetSenderDetails()
        {
            String  ErrorXML    = null;
            ulong   SessionID   = 0;
            String  UserXML = null;
            XPathDocument xp_UserXMLDoc = null;
            XPathNavigator xpNav_UserXML = null;

            EncryptedSystemUserName         = ALCommon.Encrypt(SystemUserName);
            EncryptedSystemUserPwd          = ALCommon.Encrypt(SystemUserPwd);
            ErrorXML                        = Infrastructure.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, ref ImpersonationToken);

            if (String.IsNullOrEmpty(ErrorXML))
            {
                ErrorXML = Infrastructure.ImpersonateUser("Admin", str_TenantName, ImpersonationToken,str_CallerIdentity, ref SessionID, ref UserXML);            
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    xp_UserXMLDoc   = new XPathDocument(new System.IO.StringReader(UserXML));
                    xpNav_UserXML   = xp_UserXMLDoc.CreateNavigator();
                    SenderID        = xpNav_UserXML.SelectSingleNode("USER/USERID").Value;
                    SenderEmail     = xpNav_UserXML.SelectSingleNode("USER/EMAILID").Value;
                    SenderName      = ALCommon.GetFormattedUserName(xpNav_UserXML);
                }
            }
        }

        private void MarkAssetProcessingStatusAsError(XmlNode JobProcessingDetailsNode, RenditionType RenditionTypeValue, bool OnlyAttributeReadFailed)
        {
            XmlNode     TempDataNode = null;
            XmlDocument TempDocument = null;
            XmlNode     RelationNode = null;
            String      FilePath     = null;

            try
            {
                switch (RenditionTypeValue)
                {
                    case RenditionType.THUMBNAIL:
                        //Update CatalogJob Processing Details                                        
                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("THBATTRIBUTEUPDATED");
                        if (TempDataNode != null)
                            TempDataNode.InnerText = "3";

                        if (!OnlyAttributeReadFailed)
                        {
                            TempDataNode = JobProcessingDetailsNode.SelectSingleNode("THUMBNAILGENERATED");
                            if (TempDataNode != null)
                                TempDataNode.InnerText = "3";
                        }
                        FilePath = ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath;
                        break;

                    case RenditionType.PREVIEW:
                        //Update CatalogJob Processing Details                                        
                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("PRVATTRIBUTEUPDATED");
                        if (TempDataNode != null)
                            TempDataNode.InnerText = "3";
                        if (!OnlyAttributeReadFailed)
                        {
                            TempDataNode = JobProcessingDetailsNode.SelectSingleNode("PREVIEWGENERATED");
                            if (TempDataNode != null)
                                TempDataNode.InnerText = "3";
                        }
                        FilePath = ThreadContextObj.PreviewAttrObj.PreviewFilePath;
                        break;

                    case RenditionType.ORIGINAL:
                        //Update CatalogJob Processing Details                                        
                        TempDataNode = JobProcessingDetailsNode.SelectSingleNode("ORGATTRIBUTEUPDATED");
                        if (TempDataNode != null)
                            TempDataNode.InnerText = "3";

                        FilePath = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                        break;
                }

                #region Update Processing Status
                TempDocument            = new XmlDocument();
                RelationNode            = TempDocument.CreateElement("CATALOGJOB_PROCESSINGDETAILS");
                ((XmlElement)RelationNode).SetAttribute("TYPE", "RELATION");
                ((XmlElement)JobProcessingDetailsNode).SetAttribute("MODIFIED", "TRUE");
                RelationNode.InnerXml   = JobProcessingDetailsNode.OuterXml;

                if (!UpdateCatalogJobProcessingDetails(ref RelationNode))
                {
                    //Error in Updating CatalogJob processing Details
                    IsErrorOccured = true;
                    sb_ErrorCollection.AppendFormat(@"<ERROR FileName='{0}'>
                                                        <ERRORMESSAGE>Error while UpdateCatalogJobProcessingDetails</ERRORMESSAGE>
                                                        {1}
                                                    </ERROR>"
                                                   , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath)
                                                   , ALCommon.TrimAndReplaceSpecialChars(ThreadObj.sb_Error.ToString()));
                }
                #endregion

            }
            catch(Exception)
            {}
        }

        private void GetFileFormatDetails()
        {
            String  FileFormatXML   = null;
            String  ErrorXML        = null;
            String  str_SearchKey   = null;

            try
            {
                FileFormatXMLDoc    = new XmlDocument();
                str_SearchKey       = String.Format("{0}_FileFormatXML", str_TenantName);
                if (CacheManager.IsCached(str_SearchKey))
                {
                    FileFormatXML   = (String)CacheManager.GetCacheItem(str_SearchKey);
                    FileFormatXMLDoc.LoadXml(FileFormatXML);
                }
                else
                {
                    ErrorXML = DAMWSObj.GetAllFileFormats(out FileFormatXML);
                    if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(FileFormatXML))
                    {
                        FileFormatXMLDoc.LoadXml(FileFormatXML);
                        CacheManager.AddCacheItem(str_SearchKey, FileFormatXML);
                    }
                }
            }
            catch (Exception)
            { }
        }

        private String GetErrorMessage(XPathNavigator xpNav_MediaResponseXML)
        {
            String ErrorXml = null;
            if (xpNav_MediaResponseXML.SelectSingleNode("//ERRORXML") != null)
            {
                ErrorXml = xpNav_MediaResponseXML.SelectSingleNode("//ERRORXML").Value;
            }
            return ErrorXml;
        }
        #endregion

        #region Cataloger Common Functions
        internal void GetDestinationFileName(ThreadContext ThreadContextObj, String DestinationFormat, RenditionType enum_RenditionType, String GeneratorName, ref String CurrentFileName, ref String DestinationFileName)
        {
            ENTITYID FileID = 0;
            StringBuilder TemporaryFilePath = null;
            StringBuilder sb_CatalogerMountPointUNCPath = null;

            sb_CatalogerMountPointUNCPath = new StringBuilder(100);
            TemporaryFilePath = new StringBuilder();
            try
            {
                ALCommon.GenerateUniqueID(ref FileID);
                if (!String.IsNullOrEmpty(GeneratorName))
                {
                    switch (GeneratorName)
                    {
                        case "IllustratorCS3Wrapper.dll":
                        case "MSOfficeProcessorWrapper.dll":
                        case "CatalogerGenericQWrapper.dll":
                            {
                                if (IsFilePathContainsSpecialCharacters(DestinationFileName))
                                {

                                    TemporaryFilePath.AppendFormat("{0}\\Temp\\{1}.{2}", MountPoint, FileID, DestinationFormat);
                                    DestinationFileName = TemporaryFilePath.ToString();
                                }
                                DestinationFileName = GetUNCPath(DestinationFileName, ThreadContextObj.MediaProcessor);
                                switch (enum_RenditionType)
                                {
                                    case RenditionType.ORIGINAL:
                                        {
                                            if (IsFilePathContainsSpecialCharacters(ThreadContextObj.OriginalAttrObj.OriginalFilePath))
                                                CurrentFileName = GetUNCPath(ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath, ThreadContextObj.MediaProcessor);
                                            else
                                                CurrentFileName = GetUNCPath(ThreadContextObj.OriginalAttrObj.OriginalFilePath, ThreadContextObj.MediaProcessor);
                                        } break;
                                    case RenditionType.PREVIEW:
                                        {
                                            if (IsFilePathContainsSpecialCharacters(ThreadContextObj.PreviewAttrObj.PreviewFilePath))
                                                CurrentFileName = GetUNCPath(ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath, ThreadContextObj.MediaProcessor);
                                            else
                                                CurrentFileName = GetUNCPath(ThreadContextObj.PreviewAttrObj.PreviewFilePath, ThreadContextObj.MediaProcessor);
                                        }
                                        break;
                                }
                            } break;
                        default:
                            {
                                ALCommon.GenerateUniqueID(ref FileID);
                                TemporaryFilePath.AppendFormat("{0}\\{1}.{2}", ThreadContextObj.TempFolderPath, FileID, DestinationFormat);
                                DestinationFileName = TemporaryFilePath.ToString();

                                switch (enum_RenditionType)
                                {
                                    case RenditionType.PREVIEW:
                                        CurrentFileName = ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath;
                                        break;
                                    case RenditionType.ORIGINAL:
                                    default:
                                        CurrentFileName = ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath;
                                        break;
                                }
                            } break;
                    }
                }
            }
            catch (Exception)
            {
                //CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
            }
        }

        private Boolean IsFilePathContainsSpecialCharacters(String SourceFilePath)
        {
            String FileName                 = Path.GetFileName(SourceFilePath);
            String FileNameValidatorRegex   = ConfigurationManager.AppSettings["FileNameValidatorRegexString"];//[^A-Za-z0-9-_&.:\\\\]
            Match match                     = Regex.Match(SourceFilePath, FileNameValidatorRegex);
            if (!string.IsNullOrEmpty(match.Value))
            {
                return true;
            }
            return false;
        }

        private String GetUNCPath(String FilePath, String ApplicationType)
        {
            if (!IsMediaProcessorInstalled(ApplicationType))
            {
                if (IsMediaProcessorOnSameNetwork(ApplicationType))
                {
                    FilePath = FilePath.Replace(MountPoint, CatalogerMountPointUNCPath);
                }
            }
            return FilePath;
        }

        private Boolean IsMediaProcessorInstalled(String ApplicationName)
        {
            String IsApplicationInstalled = null;
            StringBuilder sb_ApplicationInstalledKey = new StringBuilder(100);
            sb_ApplicationInstalledKey.AppendFormat("{0} MediaProcessor Installed", ApplicationName);

            IsApplicationInstalled = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", sb_ApplicationInstalledKey.ToString());

            if (!String.IsNullOrEmpty(IsApplicationInstalled))
            {
                if (IsApplicationInstalled == "1")
                    return true;
                else
                    return false;
            }
            return false;
        }

        private Boolean IsMediaProcessorOnSameNetwork(String ApplicationName)
        {
            String IsMediaProcessorOnSameNetwork = null;
            StringBuilder sb_IsMediaProcessorOnSameNetworkKey = new StringBuilder(100);
            sb_IsMediaProcessorOnSameNetworkKey.AppendFormat("{0}_ISONSAMENETWORK", ApplicationName);
            IsMediaProcessorOnSameNetwork = ConfigurationManager.AppSettings[sb_IsMediaProcessorOnSameNetworkKey.ToString()];

            if (!String.IsNullOrEmpty(IsMediaProcessorOnSameNetwork))
            {
                if (string.Compare(IsMediaProcessorOnSameNetwork, "TRUE", true) == 0)
                    return true;
                else
                    return false;
            }
            return false;
        }

        private String GetCatlogerMachineIPAddress()
        {
            String str_CatMachineIPAddress = null;
            str_CatMachineIPAddress = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerMachineIP");
            return str_CatMachineIPAddress;
        }

        private String GenerateExtraInformationForMediaProcessor(ThreadContext ThreadContextObj, RenditionType enum_RenditionType, String GeneratorName, XPathNavigator xpNav_MediaResponseXML)
        {
            StringBuilder  sb_ExtraInformationForMediaProcessor = null;
            String         str_OutputFolderPath                 = null;            
            String         str_MediaProcessor                   = null;
            String         str_UpdateDetailsQPath               = null;
            String         str_JobProcessingDetails             = null;
            XPathNavigator xpNav_ExtraInfo                      = null;

            sb_ExtraInformationForMediaProcessor = new StringBuilder(500);
            try
            {

                xpNav_ExtraInfo             = xpNav_MediaResponseXML.SelectSingleNode("EXTRAINFORMATION");
                str_UpdateDetailsQPath      = (String)ConfigurationManager.AppSettings["UpdateAssetProcessJobQueue"];
                if (xpNav_ExtraInfo != null)
                {
                    str_JobProcessingDetails = xpNav_ExtraInfo.SelectSingleNode("JOBDETAILS/JOBPROCESSINGDETAILS").InnerXml;
                }

                if (!String.IsNullOrEmpty(GeneratorName))
                {
                    switch (GeneratorName)
                    {
                        case "MSOfficeProcessorWrapper.dll":
                        case "CatalogerGenericQWrapper.dll":
                            {
                                if (IsFilePathContainsSpecialCharacters(JobPath))
                                {
                                    str_OutputFolderPath = MountPoint + "\\Temp";
                                }
                                else
                                {
                                    str_OutputFolderPath = JobPath + "\\Temp";
                                }

                                if (!Directory.Exists(str_OutputFolderPath))
                                    Directory.CreateDirectory(str_OutputFolderPath);

                                sb_ExtraInformationForMediaProcessor.Append("<EXTRAINFORMATION>");
                                sb_ExtraInformationForMediaProcessor.Append("<REQUESTGENERATOR>CATALOGER</REQUESTGENERATOR>");
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<OUTPUTFOLDERPATH>{0}</OUTPUTFOLDERPATH>", ALCommon.TrimAndReplaceSpecialChars(str_OutputFolderPath));
                                switch (GeneratorName)
                                {
                                    case "MSOfficeProcessorWrapper.dll":
                                        {
                                            str_MediaProcessor = "MSOFFICEPROCESSOR";
                                        } break;
                                    case "CatalogerGenericQWrapper.dll":
                                        {
                                            str_MediaProcessor = "IMAGEPROCESSOR";

                                        } break;
                                }

                                sb_ExtraInformationForMediaProcessor.AppendFormat("<APPLICATIONTYPE>{0}</APPLICATIONTYPE>", ALCommon.TrimAndReplaceSpecialChars(str_MediaProcessor));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<ISMEDIAPROCESSORONSAMENETWORK>{0}</ISMEDIAPROCESSORONSAMENETWORK>", IsMediaProcessorOnSameNetwork(str_MediaProcessor));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBID>{0}</JOBID>", CatalogJobID);
                                
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERUNCPATH>{0}</CATALOGERUNCPATH>", ALCommon.TrimAndReplaceSpecialChars(CatalogerMountPointUNCPath));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBPATH>{0}</JOBPATH>", ALCommon.TrimAndReplaceSpecialChars(JobPath));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<DIGITALASSETID>{0}</DIGITALASSETID>", ThreadContextObj.DigitalAssetID.ToString());
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERANDOMTOGETHER>{0}</CATALOGERANDOMTOGETHER>", CatalogerAndOMTogether.ToString());
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<UPDATEPROCESSQPATH>{0}</UPDATEPROCESSQPATH>", str_UpdateDetailsQPath);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<RENDITIONTYPE>{0}</RENDITIONTYPE>", enum_RenditionType.ToString());
                                sb_ExtraInformationForMediaProcessor.Append("<GEN_THB_FRM_PRV_CALL>FALSE</GEN_THB_FRM_PRV_CALL>");
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<MOUNTPOINT>{0}</MOUNTPOINT>", ALCommon.TrimAndReplaceSpecialChars(MountPoint));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBPROCESSINGDETAILS>{0}</JOBPROCESSINGDETAILS>", str_JobProcessingDetails);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<LOGINNAME>{0}</LOGINNAME>", LoginName);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<NETWORKFOLDERPATH>{0}</NETWORKFOLDERPATH>", NetworkFolderPath);                            
                                sb_ExtraInformationForMediaProcessor.Append("</EXTRAINFORMATION>");
                            } break;
                        case "IllustratorCS3Wrapper.dll":
                            {
                                if (IsFilePathContainsSpecialCharacters(JobPath))
                                {
                                    str_OutputFolderPath = MountPoint + "\\Temp";
                                }
                                else
                                {
                                    str_OutputFolderPath = JobPath + "\\Temp";
                                }

                                if (!Directory.Exists(str_OutputFolderPath))
                                    Directory.CreateDirectory(str_OutputFolderPath);

                                sb_ExtraInformationForMediaProcessor.Append("<EXTRAINFORMATION>");
                                sb_ExtraInformationForMediaProcessor.Append("<REQUESTGENERATOR>CATALOGER</REQUESTGENERATOR>");
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<OUTPUTFOLDERPATH>{0}</OUTPUTFOLDERPATH>", ALCommon.TrimAndReplaceSpecialChars(str_OutputFolderPath));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERINSTALLED>{0}</CATALOGERINSTALLED>", ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerInstalled"));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<WEBSERVICESINSTALLED>{0}</WEBSERVICESINSTALLED>", ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "WebServicesInstalled"));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<ISMEDIAPROCESSORONSAMENETWORK>{0}</ISMEDIAPROCESSORONSAMENETWORK>", ConfigurationManager.AppSettings["ISMEDIAPROCESSORONSAMENETWORK"]);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<THUMBNAILEXISTS>{0}</THUMBNAILEXISTS>", ThreadContextObj.ThumbnailExists.ToString().ToLower());
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<MOUNTPOINT>{0}</MOUNTPOINT>", ALCommon.TrimAndReplaceSpecialChars(MountPoint));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<DIGITALASSETID>{0}</DIGITALASSETID>", ThreadContextObj.DigitalAssetID.ToString());
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERANDOMTOGETHER>{0}</CATALOGERANDOMTOGETHER>", CatalogerAndOMTogether.ToString());
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<UPDATEPROCESSQPATH>{0}</UPDATEPROCESSQPATH>", str_UpdateDetailsQPath);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<RENDITIONTYPE>{0}</RENDITIONTYPE>", enum_RenditionType.ToString());
                                sb_ExtraInformationForMediaProcessor.Append("<GEN_THB_FRM_PRV_CALL>FALSE</GEN_THB_FRM_PRV_CALL>");
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<MOUNTPOINT>{0}</MOUNTPOINT>", ALCommon.TrimAndReplaceSpecialChars(MountPoint));
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBPROCESSINGDETAILS>{0}</JOBPROCESSINGDETAILS>", str_JobProcessingDetails);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<LOGINNAME>{0}</LOGINNAME>", LoginName);
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<NETWORKFOLDERPATH>{0}</NETWORKFOLDERPATH>", NetworkFolderPath);                            
                                sb_ExtraInformationForMediaProcessor.Append("</EXTRAINFORMATION>");
                            } break;
                    }

                }
            }
            catch (Exception)
            {
                //CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
            }
            return sb_ExtraInformationForMediaProcessor.ToString();
        }

        private Boolean GetUserPreferences( CatalogerContext CatalogerContextObj)
        {
            XML             UserCatalogerPreferences        = null;
            XmlDocument     UserCatalogerPreferencesDoc     = null;
            XmlNode         CatalogerPreferencesNode        = null;
            ERRORXML        ErrorXML                        = null;
            StringBuilder   ErrorMessage                    = null;
            XPathDocument   TenantCatalogingPreferencesDoc  = null;
            XPathNavigator  RootNode                        = null;
            XPathNavigator  PreferenceValueNode             = null;  

            try
            {

                ErrorMessage            = new StringBuilder();
                ErrorXML                = DAMWSObj.GetUserCatalogerPreferences(ref UserCatalogerPreferences);

                if ((ErrorXML == null) && (UserCatalogerPreferences.Length > 0))
                {
                    UserCatalogerPreferencesDoc         = new XmlDocument();
                    UserCatalogerPreferencesDoc.LoadXml(UserCatalogerPreferences);
                    
                    CatalogerPreferencesNode            = UserCatalogerPreferencesDoc.SelectSingleNode("//USER_DAMPERSONALISATION/DAMPERSONALIZATION/CONFIGURATIONXML/APPLICATIONSETTINGS/CATALOGERPREFERENCES");
                    // User preferences
                    if (CatalogerPreferencesNode != null)
                    {
                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWHEIGHT")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewHeight = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWHEIGHT").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewHeight = 768;

                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWWIDTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewWidth = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWWIDTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewWidth = 1024;

                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWDEPTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewDepth     = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWDEPTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewDepth     = 24;

                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWDPI")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewDPI       = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWDPI").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewDPI       = 72;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILHEIGHT")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILHEIGHT").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailHeight = 125;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILWIDTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILWIDTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDEPTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDEPTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;

                        if(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILQUALITY")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailQuality  = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILQUALITY").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailQuality  = 75;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDPI") != null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDPI      = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDPI").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDPI      = 72;

                        if (CatalogerPreferencesNode.SelectSingleNode("EMAILNOTIFICATION") != null)
                        {                            
                            switch  (CatalogerPreferencesNode.SelectSingleNode("EMAILNOTIFICATION").InnerText)
                            {
                                case "ALWAYS":
                                            UserNotificationPref.SendMailSettings = MailSettings.ALWAYS;
                                            break ;
                                case "ONERROR":
                                            UserNotificationPref.SendMailSettings = MailSettings.ONERROR;
                                            break ;
                                case "NEVER":
                                default     : UserNotificationPref.SendMailSettings = MailSettings.NEVER;
                                            break;
                            }
                        }

                        if (CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS") != null)
                            UserNotificationPref.SendMailTo = CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS").InnerText;
                        
                    }
                    else if(!String.IsNullOrEmpty(TenantCatalogingPreferencesXML))
                    {
                        // Tenant preferences
                        TenantCatalogingPreferencesDoc  = new XPathDocument(new StringReader(TenantCatalogingPreferencesXML));
                        RootNode                        = TenantCatalogingPreferencesDoc.CreateNavigator();

                        if (RootNode != null)
                        {
                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWHEIGHT"))!= null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewHeight    = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewHeight    = 768;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewHeight        = 768;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWWIDTH")) != null)
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewWidth = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewWidth = 1024;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewWidth     = 1024;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWDEPTH")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewDepth = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewDepth = 24;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewDepth     = 24;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWDPI")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewDPI   = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewDPI   = 72;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewDPI       = 72;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILHEIGHT")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = 125;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailHeight      = 125;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILWIDTH")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = 125;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailWidth       = 125;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILDEPTH")) != null) 
                                 if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                     CatalogerContextObj.UserPreferencesObj.ThumbnailDepth  = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailDepth       = 24;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILQUALITY")) != null)
                                 if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailQuality = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailQuality = 75;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailQuality     = 75;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILDPI")) != null)
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailDPI     = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailDPI     = 72;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailDPI         = 72;
                        }

                    }
                    else
                    {
                        CatalogerContextObj.UserPreferencesObj.PreviewHeight    = 768;
                        CatalogerContextObj.UserPreferencesObj.PreviewWidth     = 1024;
                        CatalogerContextObj.UserPreferencesObj.PreviewDepth     = 24;
                        CatalogerContextObj.UserPreferencesObj.PreviewDPI       = 72;

                        CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = 125;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = 125;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailDPI     = 72;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailQuality = 75;
                    }
                }
                else
                {
                    ErrorMessage.Append("Getting user preferences failed.");
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_USERPREFERENCESREADFAILED", ErrorMessage.ToString(), ErrorXML,null, ErrorType.ERROR);
                    return false;
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in GetUserPreferences : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_GETUSERPREFERENCESFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);				
                return false;
            }
        }

        private Boolean InvokePlugin(String CodePoint,CatalogerContext CatalogerContextObj)
        {
            Assembly            PluginAssembly          = null;
            Type[]              Types                   = null;
            Type                ClassType               = null;
            Object              ClassInstance           = null;
            XPathNavigator      XPathNavigatorObj       = null;
            XPathNodeIterator   PluginNodes             = null; 
            XPathNavigator      PluginNode              = null;
            StringBuilder       XPath                   = null;
            XPathExpression     XPathExpressionObj      = null;
            XPathNavigator      CodePointNode           = null;
            String              MethodeToInvoke         = null;
            String              ClassName               = null;
          
            try
            {
                XPath                       = new StringBuilder();
                if (PluginsXMLDoc != null)
                {
					XPathNavigatorObj       = PluginsXMLDoc.CreateNavigator();
                    XPath.AppendFormat("CODEPOINTCOLLECTION/CODEPOINT[(translate(@Name, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='{0}')]", CodePoint.ToUpper());
                    XPathExpressionObj      = XPathExpression.Compile(XPath.ToString());

                    if ((CodePointNode      = XPathNavigatorObj.SelectSingleNode(XPathExpressionObj)) != null)
                    {
                        PluginNodes         = CodePointNode.Select("PLUGIN");
                        while (PluginNodes.MoveNext())
                        {
                            PluginNode      = PluginNodes.Current; ;

                            //Load the Database Assembly
                            PluginAssembly  = Assembly.LoadFrom(PluginNode.SelectSingleNode("DLLPATH").Value);
                            MethodeToInvoke = PluginNode.SelectSingleNode("METHODTOINVOKE").Value;
                            Types           = PluginAssembly.GetTypes();

                            ClassName       = PluginNode.SelectSingleNode("TYPENAME").Value;
                            foreach (Type assmType in Types)
                            {
                                if (assmType.Name.ToUpper() == ClassName.ToUpper())
                                {
                                    ClassType = assmType;
                                    break;
                                }
                            }

                            if (ClassType != null)
                            {
                                ClassInstance = Activator.CreateInstance(ClassType);

                                //Invoke InitializeDatabaseSetup method of the Database Library
                                ClassInstance.GetType().InvokeMember(MethodeToInvoke,
                                                       BindingFlags.InvokeMethod, null, ClassInstance,
                                                       new Object[] { CodePoint, CatalogerContextObj });
                            }
                        }
                    }
                }
                return true;
            }
            catch (Exception)
            {
				//CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				throw;
            }
        }

        private void NTFSJobCleanUp()
        {
            StringBuilder sb_AdditionalInfoXml = new StringBuilder(200);
            sb_AdditionalInfoXml.AppendFormat(@"<ADDITIONALINFO UserID='{1}' SessionID='{2}' Language='{3}' TenantName='{4}'>
                                                    <THREADID>{0}</THREADID>
                                                    <METHODNAME>AssetProcessor.AssetProcessor.NTFSJobCleanUp()</METHODNAME>    
                                                    <CATALOGJOBID>{5}</CATALOGJOBID>                                           
                                                    <JOBPATH>{6}</JOBPATH>    
                                              </ADDITIONALINFO>",
                                             System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), ul_UserID.ToString(), ul_SessionID.ToString(), "English", str_TenantName,
                                             CatalogJobID, ALCommon.TrimAndReplaceSpecialChars(JobPath));

            Context ObjContext = new Context(ul_UserID, ul_SessionID, str_TenantName, str_SolutionName, sb_AdditionalInfoXml.ToString());
            SASMOMDirectory.DeleteFolder(JobPath, true, ObjContext); 
        }

        private Boolean Connect(String UserLoginName)
        {
            ERRORXML        ErrorXML        = null;
            XML             UserXML         = null;
            XmlDocument     UserXMLDoc      = null; 
            StringBuilder   ErrorMessage    = null;

            try
            {
                EncryptedSystemUserName = ALCommon.Encrypt(SystemUserName);
                EncryptedSystemUserPwd  = ALCommon.Encrypt(SystemUserPwd);
                ErrorXML                = Infrastructure.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, ref ImpersonationToken);

                ErrorMessage            = new StringBuilder();
                ErrorXML                = Infrastructure.ImpersonateUser(UserLoginName, str_TenantName, ImpersonationToken, str_CallerIdentity, ref ul_SessionID, ref UserXML);
                if(ErrorXML == null)
                {
                    UserXMLDoc  = new XmlDocument();
                    UserXMLDoc.LoadXml(UserXML);
                    if (UserXMLDoc.SelectSingleNode("//USERID") != null)
                    {
                        ul_UserID  = Convert.ToUInt64(UserXMLDoc.SelectSingleNode("//USERID").InnerText);
                        SetALServiceHeaderValue(ul_SessionID, ul_UserID, str_TenantName, str_Language);
                    }
                    else
                    {
                        strdict_AdditionalInfo.Add("USERLOGINNAME", UserLoginName);
                        ErrorMessage.AppendFormat("Failed to connect as : {0}", UserLoginName);
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FAILEDTOCONNECT", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                        return false;
                    }
                }
                else
                {
                    strdict_AdditionalInfo.Add("USERLOGINNAME", UserLoginName);
                    ErrorMessage.AppendFormat("Failed to connect as : {0}", UserLoginName);
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FAILEDTOCONNECT",ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                    return false;
                }
                return true;
            }
            catch (Exception)
            {
				//CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean GetAttributes(String DestinationName, ThreadContext ThreadContextObj, RenditionType RenditionTypeValue, XPathNavigator xpNav_MediaResponseXML)
        {
            StringBuilder               XPathQuery                      = null;
            XmlNode                     FileFormatNode                  = null;
            StringBuilder               ErrorMessage                    = null;
            String                      PartialQuery                    = null;                        
            MediaProcessorWrapper       MediaProcessorWrapperObj        = null; 
            Boolean                     ManagedGenerator                = false;
            String                      str_ExtraInformationForMediaProcessor = null;
            String                      previewExt                      = null;
            String                      GeneratorName                   = null;
            try
            {
               
                XPathQuery                      = new StringBuilder();
                ErrorMessage                    = new StringBuilder();
                previewExt                      = DestinationName.Substring(DestinationName.LastIndexOf(".") + 1);

                GetFileFormatDetails();
                if (FileFormatXMLDoc.SelectSingleNode("//FILEFORMAT") == null)
                {
                    ErrorMessage.Append("FileFormats not found.");
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FILEFORMATSNOTFOUND",ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                    return false;
                }

                PartialQuery        = "//FILEFORMAT[FILEEXTENSION[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')=";
                XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, previewExt.ToLower());
                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                {
                    switch (RenditionTypeValue)
                    {
                        case RenditionType.PREVIEW:
                            GeneratorName = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            break;
                        case RenditionType.THUMBNAIL:
                            GeneratorName = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                            break;
                    }
                    if (FileFormatNode != null)
                    {
                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                        {
                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                ManagedGenerator = true;
                            else
                                ManagedGenerator = false;
                        }
                    }
                }

                if (ManagedGenerator)
                {
                    switch (GeneratorName)
                    {
                        case "IllustratorCS3Wrapper.dll":
                        case "MSOfficeProcessorWrapper.dll":
                        case "CatalogerGenericQWrapper.dll":
                        case "FLVWrapper.dll":
                            {
                                str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionTypeValue, GeneratorName, xpNav_MediaResponseXML);
                            } break;
                    }

                    MediaProcessorWrapperObj                                    = new MediaProcessorWrapper(ul_SessionID,ul_UserID,str_TenantName,str_Language);
                    MediaProcessorWrapperObj.SetFileAttributes(DestinationName, null,
                    null, null, null, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                    
                    MediaProcessorWrapperObj.GeneratorName         = GeneratorName;
                    MediaProcessorWrapperObj.ThreadTimeout         = 3600000;
                    MediaProcessorWrapperObj.GetInfo();
                }                
                return true;
            }
            catch (Exception)
            {				
				return false;
            }
        }
        #endregion        

        #region Helper Methods - Execution Logging Methods
        public override void StopApplication()
        {
            base.StopApplication();
        }

        private void AppendToClientLog(String LogContent, CommonVariables ThreadObj)
        {
            if (!ThreadObj.ISClientLogAccessible)
                return;

            StringBuilder sbClientLog = new StringBuilder(200);
            sbClientLog.AppendFormat("Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6} :-  {7} \r\n", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond, LogContent);

            try
            {
                File.AppendAllText(ThreadObj.ClientLogPath, sbClientLog.ToString());
            }
            catch
            {
                ThreadObj.ISClientLogAccessible = false;
                throw;
            }
        }

        private void UpdateRequestXML(String Status, XPathNavigator xpNav_MediaRequest, CommonVariables ThreadObj, String InputFileName)
        {
            XPathNavigator xpNav_Status = null;
                        
            if (xpNav_MediaRequest != null)
            {
                if (ThreadObj.sb_OutputXML != null && ThreadObj.sb_OutputXML.Length > 0)
                {
                    ThreadObj.sb_OutputXML.Insert(0, "<FILECONVERTEROUTPUT><OUTPUTLIST>");
                    ThreadObj.sb_OutputXML.Append("</OUTPUTLIST></FILECONVERTEROUTPUT>");
                }

                xpNav_Status = xpNav_MediaRequest.SelectSingleNode("/REQUEST/REQUESTHEADER/REQUESTSTATUS");
                if (xpNav_Status != null)
                {
                    xpNav_Status.InnerXml = Status;
                }

                if (String.Compare(Status, "ERROR", true) == 0)
                {
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST").AppendChildElement("", "ERRORXML", "", null);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/ERRORXML").InnerXml = ThreadObj.ErrorXML;
                }

                if (xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION") != null)
                {
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "APPLICATION", "", ThreadObj.ApplicationName);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "INPUTFILEPATH", "", InputFileName);
                }
                else
                {
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST").AppendChildElement("", "EXTRAINFORMATION", "", null);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "APPLICATION", "", ThreadObj.ApplicationName);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "INPUTFILEPATH", "", InputFileName);
                }

                xpNav_MediaRequest.SelectSingleNode("/REQUEST").AppendChildElement("", "OUTPUTXML", "", null);

                if (ThreadObj.sb_OutputXML != null)
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/OUTPUTXML").InnerXml = ThreadObj.sb_OutputXML.ToString();
            }

        }

        private void AddMediaProcessorLog(String RequestXML, CommonVariables ThreadObj)
        {
            if (!String.IsNullOrEmpty(RequestXML))
            {
                LogInfrastructure LogInfraObj   = new LogInfrastructure();
                LogInfraObj.TenantName          = str_TenantName;
                LogInfraObj.Application         = "ASSETPROCESSOR";
                LogInfraObj.AddMediaLog(RequestXML, this.MessageArrivalTime, DateTime.Now);
            }
        }

        private void UpdateMediaProcessorLog(String RequestStatus, CommonVariables ThreadObj)
        {
            if (!String.IsNullOrEmpty(ThreadObj.RequestID) && !String.IsNullOrEmpty(RequestStatus))
            {
                LogInfrastructure LogInfraObj   = new LogInfrastructure();
                LogInfraObj.TenantName          = str_TenantName;
                LogInfraObj.Application         = "ASSETPROCESSOR";
                LogInfraObj.UpdateMediaLogStatus(ThreadObj.RequestID, RequestStatus);
            }
        }

        private void SendResponseToSolution(String str_ResponseMessage, CommonVariables ThreadObj)
        {
            if (String.IsNullOrEmpty(ThreadObj.OutputQueuePath))
                return;

            MSMQWrapper objQueue = new MSMQWrapper(ThreadObj.OutputQueuePath, QueueAccessMode.SendMessage);
            objQueue.AdministrationQueuePath = UndeliveredQueuePath;
            if (ResponseTimeout != 0)
                objQueue.TimeToReachQueue = TimeSpan.FromSeconds(ResponseTimeout);
            else
                objQueue.TimeToReachQueue = new TimeSpan(24, 0, 0);
            ThreadObj.ErrorXML = objQueue.SendMessage("Asset Processor Response", str_ResponseMessage);

            if (ThreadObj.ErrorXML != null)
            {
                if (!String.IsNullOrEmpty(ThreadObj.ClientLogPath))
                    AppendToClientLog("ERROR WHILE RESPONDING : " + ThreadObj.ErrorXML, ThreadObj);
                if (ThreadObj.IsExceptionOccured)
                    UpdateMediaProcessorLog("Error while Responding after exception", ThreadObj);
                else
                    UpdateMediaProcessorLog("Error while Responding", ThreadObj);
            }

        }

        private void ReportErrorInEventViewer(Exception ExceObj, String AdditionalInfo)
        {
            if (EventLogObj == null)
                EventLogObj = new System.Diagnostics.EventLog("SAS MOM");

            StringBuilder sbErrorDetails = new StringBuilder(500);

            sbErrorDetails.AppendFormat("Asset Processor Component Exception:  {0}", ExceObj.Message);
            sbErrorDetails.AppendLine(" Stack Trace: " + ExceObj.StackTrace);
            sbErrorDetails.AppendLine(" AdditionalDetails: " + AdditionalInfo);

            EventLogObj.WriteEntry(sbErrorDetails.ToString(), System.Diagnostics.EventLogEntryType.Error);
        }

        private void ReportErrorInEventViewer(String str_Error, String AdditionalInfo)
        {
            if (EventLogObj == null)
                EventLogObj = new System.Diagnostics.EventLog("SAS MOM");

            StringBuilder sbErrorDetails = new StringBuilder(500);

            sbErrorDetails.AppendFormat("Asset Processor Component Error:  {0}", str_Error);
            sbErrorDetails.AppendLine(" AdditionalDetails: " + AdditionalInfo);

            EventLogObj.WriteEntry(sbErrorDetails.ToString(), System.Diagnostics.EventLogEntryType.Error);
        }
        #endregion
    }

    class CommonVariables
    {
        #region Public member variables
        public StringBuilder        sb_Error         = new StringBuilder(200);
        public StringBuilder        sb_OutputXML     = new StringBuilder(200);
        public StringBuilder        sb_ConsoleOutput = new StringBuilder();
        public String               TenantName       = null;
        public String               OutputQueuePath  = null;
        public String               RequestID        = null;
        public String               ClientLogPath    = null;
        public ERRORXML             ErrorXML         = null;
        public Boolean              ISClientLogAccessible = true;
        public Boolean              IsExceptionOccured = false;
        public Boolean              UseFlatten       = true;
        public String               ApplicationName  = "IMAGEMAGICK";
        public String               IntermidiateFileName = null;
        #endregion

        public CommonVariables()
        {   
        }
    }
}
