﻿#region System Include
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml.XPath;
using System.Xml;
using System.Diagnostics;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Threading;
using System.Configuration;
using System.Web.Services;
using System.Reflection;
#endregion


#region Assetlink Include
using Assetlink;
using Assetlink.Services.MediaProcessor.ManagedMediaProcessorWrapper;
using Assetlink.WebServices.ALDAMWS;
using Assetlink.WebServices.ALSearchWS;
using Assetlink.Services.Plugins.ManagedPlugInsWrapper;
using Assetlink.Metadata;
using Assetlink.IOProxy;
#endregion

namespace Assetlink.MediaProcessors
{
    using XML       = System.String;
    using ERRORXML  = System.String;
    using ENTITYID  = System.UInt64;

    public class CheckoutProcessor : ALQueueComponent
    {
        #region member variables
        String                 JobPath                  = null;
        String                 JobName                  = null;
        String                 DAMJobID                 = null;
        String                 ZipFileName              = null;
        String                 ZipFilePath              = null;
        CommonVariables        ThreadObj                = null;
        Boolean                IsErrorOccured           = false;
        StringBuilder          sb_ErrorCollection       = new StringBuilder(200);
        ulong                  ul_UserID;
        ulong                  ul_SessionID;
        String                 str_TenantName          = null;
        String                 str_UserID              = null;
        String                 str_Language            = null;
        String                 str_SolutionName        = "DAM";
        String                 str_SessionID           = null;
        Search                 SearchObj               = null;
        String                 strCulture              = null;
        //String                 LogFileName             = null;
        DateTime               JobSubmissionTime;
        String                 m_JobID                 = null;
        #endregion

        public override bool InitializeComponent()
        {
            return base.InitializeComponent();
        }

        #region public methods
        public override void ProcessMessage(String MediaRequestXML)
        { 
            #region Member Variables            
            XPathNavigator      xpNav_MediaResponseXML      = null;
            //String              str_RequesteGenerator       = null;
            MediaRequestRouter  obj_MediaRequestRouter      = null;
            String              MediaResponseXML            = null;            
            Boolean             MediaProcessorLogAdded      = false;            
            XPathNavigator      xpNav_LogFile               = null;
            String              ErrorXML                    = null;            
            XmlDocument         xmlDoc_Request              = null;
            String              JobPath                     = null;
            String              OutputXML                   = null;
            String             inputXmlPath                 = null;
            String             configurationXmlPath         = null;
            XPathNavigator      xpNav_DataXml               = null;
            String              ProcessorName               = null;
            #endregion

            try
            {
                #region Initialization
                ThreadObj               = new CommonVariables();
                MediaProcessorLogAdded  = false;
                IsErrorOccured          = false;
                sb_ErrorCollection      = new StringBuilder(200);                
                #endregion

                #region Read Request
                MediaResponseXML = MediaRequestXML;
                obj_MediaRequestRouter = new MediaRequestRouter(str_TenantName, str_UserID, str_SessionID, str_Language, str_SolutionName);
                obj_MediaRequestRouter.TransformResponseXML(ref MediaResponseXML);

                xmlDoc_Request          = new XmlDocument();
                xmlDoc_Request.LoadXml(MediaRequestXML);

                xpNav_MediaResponseXML  = xmlDoc_Request.CreateNavigator();
                xpNav_MediaResponseXML  = xpNav_MediaResponseXML.SelectSingleNode("/REQUEST");
                ThreadObj.RequestID     = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/REQUESTID").Value;
                str_TenantName          = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/TENANTNAME").Value;
                str_UserID              = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/USERID").Value;
                ul_UserID               = ulong.Parse(str_UserID);
                str_SessionID           = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/SESSIONID").Value;
                ul_SessionID            = ulong.Parse(str_SessionID);
                str_Language            = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/LANGUAGE").Value;
                str_SolutionName        = xpNav_MediaResponseXML.SelectSingleNode("REQUESTHEADER/SOLUTIONNAME").Value;
                JobPath                 = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/DAMJOBINFO/JOBPATH").Value;
                ProcessorName           = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/DAMJOBINFO/PROCESSORNAME").Value;
                ThreadObj.ApplicationName = ProcessorName;
                inputXmlPath            = String.Format("{0}\\Input.xml", JobPath);
                configurationXmlPath    = String.Format("{0}\\Configuration.xml", JobPath);
                xpNav_DataXml           = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/DAMJOBINFO/BATCHDOWNLOAD");
                if (xpNav_DataXml != null && !string.IsNullOrEmpty(xpNav_DataXml.InnerXml))
                    File.WriteAllText(inputXmlPath, xpNav_DataXml.OuterXml);
                else
                {
                    //Log Error
                    return;
                }

                xpNav_DataXml = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/DAMJOBINFO/CONFIGURATION");
                if (xpNav_DataXml != null && !string.IsNullOrEmpty(xpNav_DataXml.InnerXml))
                    File.WriteAllText(configurationXmlPath, xpNav_DataXml.OuterXml);
                else
                {
                    //Log Error
                    return;
                }               
                #endregion

                #region Prepare Client log file
                xpNav_LogFile = xpNav_MediaResponseXML.SelectSingleNode("INPUTXML/DAMJOBINFO/LOGFILE");
                if (xpNav_LogFile == null)
                {                    
                    StringBuilder sbTemp        = new StringBuilder(100);
                    sbTemp.AppendFormat("{0}\\CheckoutProcessor_Log.txt", JobPath);
                    ThreadObj.ClientLogPath     = sbTemp.ToString();
                }
                else
                {
                    ThreadObj.ClientLogPath = xpNav_LogFile.Value;
                }

                if (!Directory.Exists(Path.GetDirectoryName(ThreadObj.ClientLogPath)))
                    ThreadObj.ISClientLogAccessible = false;
                #endregion
                
                #region Process Request
                AppendToClientLog("Start Processing " + ThreadObj.ApplicationName + " RequestID:" + ThreadObj.RequestID, ThreadObj);
                switch (ProcessorName)
                {
                    case "DAMCOMPRESSION":
                        ErrorXML = InvokeZIPPlugin(JobPath, out OutputXML, str_SessionID, str_UserID, str_TenantName, str_Language);
                        break;
                    case "DAMFTP":
                        ErrorXML = InvokeFTPPlugin(JobPath, out OutputXML, str_SessionID, str_UserID, str_TenantName, str_Language);
                        break;
                    case "DAMCOPY":
                        ErrorXML = InvokeCOPYPlugin(JobPath, out OutputXML, str_SessionID, str_UserID, str_TenantName, str_Language);
                        break;
                }
                AppendToClientLog("End Processing " + ThreadObj.ApplicationName + " RequestID:" + ThreadObj.RequestID, ThreadObj);
                #endregion

                #region Append Log
                if (IsErrorOccured)
                {
                    ThreadObj.ErrorXML = sb_ErrorCollection.ToString();
                    UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, JobPath);
                }
                if (String.IsNullOrEmpty(ThreadObj.ErrorXML) && !IsErrorOccured)
                {
                    UpdateRequestXML("DONE", xpNav_MediaResponseXML, ThreadObj, JobPath);
                }

                AddMediaProcessorLog(xpNav_MediaResponseXML.OuterXml, ThreadObj);
                MediaProcessorLogAdded = true;
                #endregion
            }
            catch (ThreadAbortException ExceptionObj)
            {
                AppendToClientLog("Start of thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.ErrorXML = "<MESSAGE>" + ExceptionObj.Message.ToString() + "</MESSAGE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORTYPE>EXCEPTION</ERRORTYPE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORDETAILS>" + ExceptionObj.StackTrace + "</ERRORDETAILS>";

                UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, JobPath);

                AppendToClientLog("After UpdateRequestXML of Thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.IsExceptionOccured = true;
                if (MediaProcessorLogAdded)
                    UpdateMediaProcessorLog("Thread abort Exception in ProcessFile", ThreadObj);
                else
                {
                    if (xpNav_MediaResponseXML != null)
                    {
                        AddMediaProcessorLog(xpNav_MediaResponseXML.OuterXml, ThreadObj);
                        MediaProcessorLogAdded = true;
                    }
                }

                AppendToClientLog("After AddMediaProcessorLog/UpdateMediaProcessorLog of Thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);
                ReportErrorInEventViewer(ExceptionObj, MediaRequestXML);
                AppendToClientLog("After ReportErrorInEventViewer of Thread abort exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);
            }
            catch (Exception ExceptionObj)
            {
                AppendToClientLog("Start of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.ErrorXML = ExceptionObj.Message;
                UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, JobPath);
                AppendToClientLog("After UpdateRequestXML of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.IsExceptionOccured = true;
                if (MediaProcessorLogAdded)
                    UpdateMediaProcessorLog("Exception in ProcessFile", ThreadObj);
                else
                {
                    if (xpNav_MediaResponseXML != null)
                    {
                        AddMediaProcessorLog(xpNav_MediaResponseXML.OuterXml, ThreadObj);
                        MediaProcessorLogAdded = true;
                    }
                }

                AppendToClientLog("After AddMediaProcessorLog/UpdateMediaProcessorLog of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);
                ReportErrorInEventViewer(ExceptionObj, MediaRequestXML);
                AppendToClientLog("After ReportErrorInEventViewer of exception in ProcessMessage - " + ExceptionObj.Message, ThreadObj);

                ThreadObj.ErrorXML = "<MESSAGE>" + ExceptionObj.Message.ToString() + "</MESSAGE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORTYPE>EXCEPTION</ERRORTYPE>";
                ThreadObj.ErrorXML = ThreadObj.ErrorXML + "<ERRORDETAILS>" + ExceptionObj.StackTrace + "</ERRORDETAILS>";
                UpdateRequestXML("ERROR", xpNav_MediaResponseXML, ThreadObj, JobPath);
            }
            finally
            {
                AppendToClientLog("Processing completed for RequestID:" + ThreadObj.RequestID, ThreadObj);
            }
        }

        public ERRORXML InvokeZIPPlugin(String JobLocation, out String OutPutXml,
                                        String SessionID, String UserID, String TenantName, String Language)
        {
            #region Member Variables            
            String                  JobID                       = null;
            Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper ZipWrapperObj = null;
            StringBuilder           SourcePath                  = null;
            StringBuilder           DestinationPath             = null;
            StringBuilder           FilePath                    = null;
            XmlDocument             ConfigXMLDoc                = null;           
            String                  CompressionLevel            = null;
            String                  CreateSEA                   = null;
            StringBuilder           ErrorMessage                = null;
            XmlDocument             InputXMLDoc                 = null;
            Boolean                 Error                       = false;
            XML                     DamJobXML                   = null;
            ERRORXML                ErrorXML                    = null;
            XPathDocument           ErrorXMLDoc                 = null;
            XPathNavigator          XPathNav                    = null;
            XPathNodeIterator       ErrorNodes                  = null;
            XPathNavigator          ColumnNav                   = null;
            ALDAMWS                 DAMWSObj                    = null;
            ALServiceHeader         RequestHeader               = null; 
            String                  MetadataRequired            = null;   
            OutPutXml                                           = null;            
            Boolean                 ErrorInXMPEmbedding         = false;
            String                  str_FolderName              = null;
            String                  NotifyUser                  = null;
            XmlNode                 NotifyUserNode              = null;
            #endregion

            try
            {
                //System.Net.ServicePointManager.CertificatePolicy = new ALHttps_Policy();
                str_FolderName  = "PluginServer_" + JobLocation.Substring(JobLocation.LastIndexOf("\\") + 1) + "_" + Guid.NewGuid().ToString();
                ALCommon.WriteLog(TenantName, "PluginService", str_FolderName, SessionID, "InvokePlugin Method Start. JobLocation: " + JobLocation);

                FilePath                    = new StringBuilder();
                DestinationPath             = new StringBuilder();
                SourcePath                  = new StringBuilder();
                ConfigXMLDoc                = new XmlDocument();
                InputXMLDoc                 = new XmlDocument();
                ErrorMessage                = new StringBuilder();
                ZipWrapperObj               = new Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper();
                
                JobPath                     = JobLocation;
                JobID                       = JobPath.Substring(JobPath.LastIndexOf("\\") + 1);
                m_JobID                     = JobID;
                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Configuration.xml", JobPath);
                ConfigXMLDoc.Load(FilePath.ToString());

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Input.xml", JobPath);
                InputXMLDoc.Load(FilePath.ToString());

                UserID                      = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/USERID").InnerText;
                Language                    = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/LANGUAGE").InnerText;
                TenantName                  = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/TENANTNAME").InnerText;
                SessionID                   = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/SESSIONID").InnerText;
                JobName                     = InputXMLDoc.SelectSingleNode("BATCHDOWNLOAD/JOBINFO/JOBNAME").InnerText;
                DAMJobID                    = InputXMLDoc.SelectSingleNode("BATCHDOWNLOAD/JOBINFO/DAMJOBID").InnerText;

                RequestHeader               = new ALServiceHeader();
                RequestHeader.UserID        = Convert.ToUInt64(UserID);
                RequestHeader.Language      = Language;
                RequestHeader.TenantName    = TenantName;
                RequestHeader.SessionID     = Convert.ToUInt64(SessionID);

                DAMWSObj                    = new ALDAMWS();
                DAMWSObj.ALServiceHeaderValue = RequestHeader;
                DAMWSObj.Url                = ConfigurationManager.AppSettings["ALDAMWS"];

                ZipFileName                 = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/ZIPFILENAME").InnerText;
                CompressionLevel            = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/COMPRESSIONLEVEL").InnerText;
                CreateSEA                   = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/CREATESEA").InnerText;
                
                if (CreateSEA.ToUpper() == "TRUE")
                    ZipWrapperObj.CreateSEA = true;
                else
                    ZipWrapperObj.CreateSEA = false;

                NotifyUserNode = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/NOTIFYUSER");
                if (NotifyUserNode != null)
                    NotifyUser = NotifyUserNode.InnerText;

                switch (CompressionLevel)
                {
                    case "0": ZipWrapperObj.CompressionLevel = Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper.CompressionLevelEnum.None; break;
                    case "1": ZipWrapperObj.CompressionLevel = Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper.CompressionLevelEnum.Low; break;
                    case "6": ZipWrapperObj.CompressionLevel = Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper.CompressionLevelEnum.Medium; break;
                    case "9": ZipWrapperObj.CompressionLevel = Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper.CompressionLevelEnum.High; break;
                    default: ZipWrapperObj.CompressionLevel = Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper.CompressionLevelEnum.High; break;
                }

                #region Embedd XMP into Original files if required
                MetadataRequired            = InputXMLDoc.DocumentElement.GetAttribute("METADATAREQUIRED");
                if (!String.IsNullOrEmpty(MetadataRequired) && (MetadataRequired.ToUpper() == "TRUE"))
                {
                    AppendToClientLog("Start Processing EmbeddXMPInToOriginalFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                    ErrorXML                = EmbeddXMPInToOriginalFiles(InputXMLDoc, JobPath);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        IsErrorOccured = true;                        
                        ErrorInXMPEmbedding = true;
                        ErrorXMLDoc         = new XPathDocument(new StringReader(ErrorXML));
                        XPathNav            = ErrorXMLDoc.CreateNavigator();
                        ErrorNodes          = XPathNav.Select("ERRORCOLLECTION/ERROR");
                        while (ErrorNodes.MoveNext())
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            if ((ColumnNav  = ErrorNodes.Current.SelectSingleNode("DIGITALASSETNAME")) != null)
                                ErrorMessage.AppendFormat("Failed to Embedd XMP into {0}.", ColumnNav.Value);
                            else
                                ErrorMessage.Append("Failed to Embedd XMP.");

                            sb_ErrorCollection.Append(ErrorMessage.ToString());
                            ColumnNav       = ErrorNodes.Current.SelectSingleNode("ERRORMESSAGE");
                            LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(), ColumnNav.Value, null, ""), ErrorType.ERROR);
                            AppendToClientLog("EmbeddXMPInToOriginalFiles Error:" + ErrorMessage.ToString(), ThreadObj);
                        }
                    }
                    AppendToClientLog("End Processing EmbeddXMPInToOriginalFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                }
                #endregion

                AppendToClientLog("Start Processing ZipFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                SourcePath.AppendFormat("{0}\\INPUT", JobPath);
                DestinationPath.AppendFormat("{0}\\OUTPUT", JobPath);
                if (!Directory.Exists(DestinationPath.ToString()))
                    Directory.CreateDirectory(DestinationPath.ToString());
                ErrorXML = ZipWrapperObj.ZipFiles(SourcePath.ToString(), DestinationPath.ToString(), ZipFileName);
                if (!String.IsNullOrEmpty(ErrorXML) && String.Compare(ErrorXML, "SUCCESS", true) != 0)
                {
                    Error = true;
                    IsErrorOccured = true;
                    sb_ErrorCollection.Append(ErrorXML);
                    LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorXML, ErrorXML, null, ""), ErrorType.ERROR);
                }
                AppendToClientLog("End Processing ZipFiles for RequestID:" + ThreadObj.RequestID + " ErrorXML:" + ErrorXML, ThreadObj);

                if(Error)
                    DamJobXML   = FormDamJobXML(JobID, "Error", InputXMLDoc.InnerXml);
                else if (ErrorInXMPEmbedding)
                    DamJobXML = FormDamJobXML(JobID, "Done with Warnings", InputXMLDoc.InnerXml);
                else
                {
                    NTFSCleanup(SourcePath.ToString());
                    DamJobXML = FormDamJobXML(JobID, "Done", InputXMLDoc.InnerXml);
                }

                ErrorXML        = DAMWSObj.UpdateJob(DamJobXML);
                if (ErrorXML != null)
                {
                    IsErrorOccured  = true;                    
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to update DamJob entry");

                    sb_ErrorCollection.AppendFormat("{0}", ErrorMessage.ToString());
                    LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(),ErrorMessage.ToString(), "ZIP_EXE_JOBUPDATIONFAILED", ""), ErrorType.ERROR); 
                    return ErrorMessage.ToString();
                }

                #region Stamp Notification
                NotifyUser = "TRUE";
                if (String.Compare(NotifyUser, "TRUE", true) == 0)
                {
                    AppendToClientLog("Start Processing StampNotification for RequestID:" + ThreadObj.RequestID, ThreadObj);
                    ZipFilePath = String.Format("{0}\\{1}.zip", DestinationPath, ZipFileName);
                    ErrorXML    = StampNotification(!Error, ErrorXML, InputXMLDoc, ConfigXMLDoc, SessionID, UserID, TenantName, Language, ZipFileName);
                    if (ErrorXML != null)
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Failed to stamp notification");
                        IsErrorOccured = true;
                        sb_ErrorCollection.Append(ErrorMessage.ToString());
                        LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(), "", "ZIP_EXE_STAMPNOTIFICATIONFAILED", ""), ErrorType.ERROR);
                    }
                    AppendToClientLog("End Processing StampNotification for RequestID:" + ThreadObj.RequestID + " ErrorXML:" + ErrorXML, ThreadObj);
                }
                #endregion

            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Failed to create the ZIP file : {0}", ExceptionObj.Message);
                IsErrorOccured = true;
                sb_ErrorCollection.Append(ErrorMessage.ToString());
                LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ExceptionObj.Message, ErrorMessage.ToString(), null, ""), ErrorType.EXCEPTION);
                return ErrorMessage.ToString();
            }
            ALCommon.WriteLog(TenantName, "PluginService", str_FolderName, SessionID, "InvokePlugin Method End. JobLocation: " + JobLocation);
            return null;
        }

        public ERRORXML InvokeFTPPlugin(String JobLocation, out String OutPutXml,
                                        String SessionID, String UserID, String TenantName, String Language)
        {
            #region Member Variables            
            String                  JobID                       = null;            
            FTPClient               FTPClientObj                = null;
            StringBuilder           SourcePath                  = null;
            StringBuilder           DestinationPath             = null;
            StringBuilder           FilePath                    = null;
            XmlDocument             ConfigXMLDoc                = null;
            String                  HostName                    = null;
            String                  UserName                    = null;
            String                  Password                    = null;
            String                  DestinationFolder           = null;
            String                  NotificationOnError         = null;
            String                  NotificationOnCompletion    = null;
            StringBuilder           ErrorMessage                = null;
            XmlDocument             InputXMLDoc                 = null;
            String                  FailedFileList              = null;
            String[]                FailedFiles                 = null;
            XmlNode                 ParentNode                  = null;
            XmlNode                 FileNameNode                = null;
            StringBuilder           XPath                       = null;
            Boolean                 Error                       = false;
            XML                     DamJobXML                   = null;
            ERRORXML                ErrorXML                    = null;
            ALDAMWS                 DAMWSObj                    = null;
            ALServiceHeader         RequestHeader               = null;
            String                  NotifyUserList              = null;
            Boolean                 IsSuccess                   = false;
            StringBuilder           sb_JobDetails               = null;
            
            String                  MetadataRequired            = null;   
            OutPutXml                                           = null;            
            XPathDocument           ErrorXMLDoc                 = null;
            XPathNavigator          XPathNav                    = null;
            XPathNodeIterator       ErrorNodes                  = null;
            XPathNavigator          ColumnNav                   = null;
            Boolean                 ErrorInXMPEmbedding         = false;
            #endregion

            try
            {
                //System.Net.ServicePointManager.CertificatePolicy = new ALHttps_Policy();
                FilePath                    = new StringBuilder();
                SourcePath                  = new StringBuilder();
                DestinationPath             = new StringBuilder();
                XPath                       = new StringBuilder();
                ConfigXMLDoc                = new XmlDocument();
                InputXMLDoc                 = new XmlDocument();
                ErrorMessage                = new StringBuilder();                
                FTPClientObj                = new FTPClient();
                sb_JobDetails               = new StringBuilder();
                               
                JobPath                     = JobLocation;
                JobID                       = JobPath.Substring(JobPath.LastIndexOf("\\") + 1);
                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Configuration.xml", JobPath);
                ConfigXMLDoc.Load(FilePath.ToString());

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Input.xml", JobPath);
                InputXMLDoc.Load(FilePath.ToString());

                UserID                      = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/USERID").InnerText;
                Language                    = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/LANGUAGE").InnerText;
                TenantName                  = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/TENANTNAME").InnerText;
                JobName                     = InputXMLDoc.DocumentElement.SelectSingleNode("JOBINFO/JOBNAME").InnerText;
                SessionID                   = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/SESSIONID").InnerText;

                RequestHeader               = new ALServiceHeader();
                RequestHeader.UserID        = Convert.ToUInt64(UserID);
                RequestHeader.Language      = Language;
                RequestHeader.TenantName    = TenantName;
                RequestHeader.SessionID     = Convert.ToUInt64(SessionID);
                DAMWSObj                    = new ALDAMWS();
                DAMWSObj.ALServiceHeaderValue = RequestHeader;
                DAMWSObj.Url = ConfigurationManager.AppSettings["ALDAMWS"];

                HostName                    = ConfigXMLDoc.SelectSingleNode("//HOSTNAME").InnerText;
                UserName                    = ConfigXMLDoc.SelectSingleNode("//USERNAME").InnerText;
                Password                    = ConfigXMLDoc.SelectSingleNode("//PASSWORD").InnerText;
                DestinationFolder           = ConfigXMLDoc.SelectSingleNode("//DESTINATIONDIRECTORY").InnerText;
                NotificationOnError         = ConfigXMLDoc.SelectSingleNode("//NOTIFICATIONONERROR").InnerText;
                NotificationOnCompletion    = ConfigXMLDoc.SelectSingleNode("//NOTIFICATIONONCOMPLETION").InnerText;
                NotifyUserList              = ConfigXMLDoc.SelectSingleNode("//NOTIFYUSERSLIST").InnerText;

               
                FTPClientObj.HostName               = HostName;
                FTPClientObj.UserName               = UserName;
                FTPClientObj.Password               = Password;
                FTPClientObj.DestinationFolderName  = DestinationFolder;
                FTPClientObj.JobID                  = JobID;
                FTPClientObj.JobName                = JobName.Substring(JobName.LastIndexOf("_") + 1);

                
                #region Embedd XMP into Original files if required
                MetadataRequired            = InputXMLDoc.DocumentElement.GetAttribute("METADATAREQUIRED");
                if (!String.IsNullOrEmpty(MetadataRequired) && (MetadataRequired.ToUpper() == "TRUE"))
                {
                    AppendToClientLog("Start Processing EmbeddXMPInToOriginalFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                    ErrorXML                = EmbeddXMPInToOriginalFiles(InputXMLDoc, JobPath);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        IsErrorOccured      = true;
                        ErrorInXMPEmbedding = true;
                        ErrorXMLDoc         = new XPathDocument(new StringReader(ErrorXML));
                        XPathNav            = ErrorXMLDoc.CreateNavigator();
                        ErrorNodes          = XPathNav.Select("ERRORCOLLECTION/ERROR");
                        while (ErrorNodes.MoveNext())
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            if ((ColumnNav = ErrorNodes.Current.SelectSingleNode("DIGITALASSETNAME")) != null)
                                ErrorMessage.AppendFormat("Failed to Embedd XMP into {0}.", ColumnNav.Value);
                            else
                                ErrorMessage.Append("Failed to Embedd XMP.");
                            sb_ErrorCollection.Append(ErrorMessage.ToString());
                            ColumnNav       = ErrorNodes.Current.SelectSingleNode("ERRORMESSAGE");
                            LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(), ColumnNav.Value, null, ""), ErrorType.ERROR);
                            AppendToClientLog("EmbeddXMPInToOriginalFiles Error:" + ErrorMessage.ToString(), ThreadObj);
                        }
                    }
                    AppendToClientLog("End Processing EmbeddXMPInToOriginalFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                }
                #endregion

                AppendToClientLog("Start Processing FTP Transfer for RequestID:" + ThreadObj.RequestID, ThreadObj);
                SourcePath.AppendFormat("{0}\\INPUT", JobPath);
                ErrorXML = FTPClientObj.Transfer(SourcePath.ToString());

                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    Error = true;

                    if (!String.IsNullOrEmpty(FTPClientObj.FailedFileList))
                    {
                        Error = true;
                        FailedFileList = FTPClientObj.FailedFileList;
                        FailedFiles = FailedFileList.Split(',');
                        for (int Count = 0; Count < FailedFiles.Length; Count++)
                        {
                            XPath.Remove(0, XPath.Length);
                            XPath.AppendFormat("//DOWNLOADFILENAME[text()='{0}']", FailedFiles[Count].Trim());
                            FileNameNode    = InputXMLDoc.SelectSingleNode(XPath.ToString());
                            ParentNode      = FileNameNode.ParentNode;
                            ((XmlElement)ParentNode).SetAttribute("Status", "Error");
                        }
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.AppendFormat("Failed to transfer following files : {0}", ALCommon.TrimAndReplaceSpecialChars(FTPClientObj.FailedFileList));
                    }
                    LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(), ErrorXML, null, ""), ErrorType.ERROR);
                }

                AppendToClientLog("End Processing FTP Transfer for RequestID:" + ThreadObj.RequestID, ThreadObj);

                if(Error)
                    DamJobXML   = FormDamJobXML(JobID, "Error", InputXMLDoc.InnerXml);
                else if (ErrorInXMPEmbedding)
                    DamJobXML   = FormDamJobXML(JobID, "Done with Warnings", InputXMLDoc.InnerXml);
                else
                    DamJobXML   = FormDamJobXML(JobID, "Done", InputXMLDoc.InnerXml);

                //Stamp notification
                sb_JobDetails = PrepareJobDetailsXML(ConfigXMLDoc, InputXMLDoc);                
                if ((!String.IsNullOrEmpty(NotificationOnError) && NotificationOnError.Equals("TRUE"))
                     || (!String.IsNullOrEmpty(NotificationOnCompletion) && NotificationOnCompletion.Equals("TRUE")))
                {
                    if (!String.IsNullOrEmpty(NotifyUserList))
                    {
                        AppendToClientLog("Start Processing StampFTPNotification for RequestID:" + ThreadObj.RequestID, ThreadObj);
                        IsSuccess = !Error;                        
                        ErrorXML = StampFTPNotification(IsSuccess, ErrorMessage.ToString(), NotifyUserList, sb_JobDetails, SessionID, UserID, TenantName, Language);
                        if (ErrorXML != null)
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.Append("Failed to stamp notification");
                            IsErrorOccured = true;
                            sb_ErrorCollection.Append(ErrorMessage.ToString());
                            LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(),"", "FTP_EXE_STAMPNOTIFICATIONFAILED", ""), ErrorType.ERROR);                            
                        }
                        AppendToClientLog("End Processing StampFTPNotification for RequestID:" + ThreadObj.RequestID + " ErrorXML: " + ErrorXML, ThreadObj);
                    }
                }                

                ErrorXML        = DAMWSObj.UpdateJob(DamJobXML);
                if (ErrorXML != null)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to update DamJob entry");
                    IsErrorOccured = true;
                    sb_ErrorCollection.Append(ErrorMessage.ToString());
                    LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ErrorMessage.ToString(),"", "FTP_EXE_JOBUPDATIONFAILED", ""), ErrorType.ERROR);                    
                }
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Failed to transfer files : {0}", ExceptionObj.Message);
                IsErrorOccured = true;
                sb_ErrorCollection.Append(ErrorMessage.ToString());                
                LogError(DAMWSObj, JobID, PrepareErrorMessageXml(ExceptionObj.Message, "", null, ""), ErrorType.EXCEPTION);                
                return ErrorMessage.ToString();
            }

            return null;
        }

         public ERRORXML InvokeCOPYPlugin(String JobLocation, out String OutPutXml,
                                        String SessionID, String UserID, String TenantName, String Language)
        {
            #region member Variables            
            String                  JobID                       = null;
            StringBuilder           SourcePath                  = null;
            StringBuilder           FilePath                    = null;
            XmlDocument             ConfigXMLDoc                = null;
            String                  DestinationFolder           = null;
            String                  UserNameList                = null;
            String                  NotifyUserList              = null;
            String                  NotifySender                = null;
            StringBuilder           ErrorMessage                = null;
            XmlDocument             InputXMLDoc                 = null;
            Boolean                 Error                       = false;
            XML                     DamJobXML                   = null;
            ERRORXML                ErrorXML                    = null;
            ALDAMWS                 DAMWSObj                    = null;
            ALServiceHeader         RequestHeader               = null;
            String[]                Separator                   = { "," };
            String[]                UserNameValues              = null;
            String[]                UserEmailIDValues           = null;
            String                  FolderName                  = null;
            String                  NotificationXML             = null;
            StringBuilder           SenderNotificationXML       = null;
            String                  UserFolderPath              = null;
            Int32                   CntUsers                    = 0;
            String                  Subject                     = null;
            bool                    OverwriteFiles              = false;
            XPathDocument           xSelectedUserXML            = null;
            XPathNavigator          xNavSelectedUser            = null;
            XPathNavigator          xNavUser                    = null;
            String                  MetadataRequired            = null;   
            OutPutXml                                           = null;            
            XPathDocument           ErrorXMLDoc                 = null;
            XPathNavigator          XPathNav                    = null;
            XPathNodeIterator       ErrorNodes                  = null;
            XPathNavigator          ColumnNav                   = null;
            XPathNavigator          ErrMsgNav                   = null; 
            Boolean                 ErrorInXMPEmbedding         = false;
            #endregion

            try
            {
                FilePath                    = new StringBuilder(50);
                SourcePath                  = new StringBuilder(50);
                ConfigXMLDoc                = new XmlDocument();
                InputXMLDoc                 = new XmlDocument();
                ErrorMessage                = new StringBuilder(100);
                SenderNotificationXML       = new StringBuilder (100);
                //NotificationXML             = new StringBuilder (100);
                
                JobPath                     = JobLocation;
                JobID                       = JobPath.Substring(JobPath.LastIndexOf("\\") + 1);

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Configuration.xml", JobPath);
                ConfigXMLDoc.Load(FilePath.ToString());

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Input.xml", JobPath);
                InputXMLDoc.Load(FilePath.ToString());

                UserID                      = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/USERID").InnerText;
                Language                    = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/LANGUAGE").InnerText;
                TenantName                  = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/TENANTNAME").InnerText;
                SessionID                   = InputXMLDoc.DocumentElement.SelectSingleNode("SESSIONINFO/SESSIONID").InnerText;

                RequestHeader               = new ALServiceHeader();
                RequestHeader.UserID        = Convert.ToUInt64(UserID);
                RequestHeader.Language      = Language;
                RequestHeader.TenantName    = TenantName;
                RequestHeader.SessionID     = Convert.ToUInt64(SessionID);

                DAMWSObj                    = new ALDAMWS();
                DAMWSObj.ALServiceHeaderValue = RequestHeader;
                DAMWSObj.Url                = ConfigurationManager.AppSettings["ALDAMWS"];

                DestinationFolder           = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/DESTINATIONDIRECTORY").InnerText;
                UserNameList                = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/USERNAME").InnerText;
                NotifyUserList              = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/NOTIFYUSERSLIST").InnerText;
                NotifySender                = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/NOTIFYSENDER").InnerText;
                FolderName                  = ConfigXMLDoc.SelectSingleNode("CONFIGURATION/FOLDERNAME").InnerText;
                xSelectedUserXML            = new XPathDocument(new StringReader(ConfigXMLDoc.SelectSingleNode("CONFIGURATION/USERLIST").OuterXml));

                if (xSelectedUserXML != null)
                {
                    xNavSelectedUser = xSelectedUserXML.CreateNavigator();
                }


                if (ConfigXMLDoc.SelectSingleNode("CONFIGURATION/OVERWRITEFILES") != null)
                    if (ConfigXMLDoc.SelectSingleNode("CONFIGURATION/OVERWRITEFILES").InnerText == "TRUE") OverwriteFiles = true;
                
                SourcePath.AppendFormat("{0}\\INPUT", JobPath);
                //DestinationPath.AppendFormat("{0}\\OUTPUT", JobPath);
                //if (!Directory.Exists(DestinationPath.ToString()))
                  //  Directory.CreateDirectory(DestinationPath.ToString());

                #region Embedd XMP into Original files if required
                MetadataRequired            = InputXMLDoc.DocumentElement.GetAttribute("METADATAREQUIRED");
                if (!String.IsNullOrEmpty(MetadataRequired) && (MetadataRequired.ToUpper() == "TRUE"))
                {
                    AppendToClientLog("Start Processing EmbeddXMPInToOriginalFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                    ErrorXML                = EmbeddXMPInToOriginalFiles(InputXMLDoc, JobPath);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        IsErrorOccured      = true;
                        ErrorInXMPEmbedding = true;
                        ErrorXMLDoc         = new XPathDocument(new StringReader(ErrorXML));
                        XPathNav            = ErrorXMLDoc.CreateNavigator();
                        ErrorNodes          = XPathNav.Select("ERRORCOLLECTION/ERROR");
                        while (ErrorNodes.MoveNext())
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrMsgNav       = ErrorNodes.Current.SelectSingleNode("ERRORMESSAGE");
                            if ((ColumnNav  = ErrorNodes.Current.SelectSingleNode("DIGITALASSETNAME")) != null)
                                ErrorMessage.AppendFormat("Failed to Embedd XMP into {0}. Error: {1}", ColumnNav.Value,ErrMsgNav.Value);
                            else
                                ErrorMessage.Append("Failed to Embedd XMP.");
                            sb_ErrorCollection.Append(ErrorMessage.ToString());                     
                            LogError(DAMWSObj, JobID, ErrorMessage.ToString(), ErrorType.ERROR);                    
                        }
                    }
                    AppendToClientLog("End Processing EmbeddXMPInToOriginalFiles for RequestID:" + ThreadObj.RequestID, ThreadObj);
                }
                #endregion

                /*
                 * 1. Split the Users from UserNameList
                 * 2. iterate on UserNameList
                 * 3. call CopyFiles 
                 * 4. If ErrorXML , append it to collection
                 */
                AppendToClientLog("Start Processing copy files to Network folder for RequestID:" + ThreadObj.RequestID, ThreadObj);
                UserNameValues    = UserNameList.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                UserEmailIDValues = NotifyUserList.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                SenderNotificationXML.Append("<USERLIST>");
                foreach (String UserName in UserNameValues)
                {
                    NotificationXML = InputXMLDoc.SelectSingleNode("BATCHDOWNLOAD/JOBINFO").OuterXml;
                    ErrorXML        = CopyFiles(UserName, SourcePath.ToString(), DestinationFolder, FolderName, ref InputXMLDoc, ref NotificationXML,out UserFolderPath, OverwriteFiles);
                    Subject         = FolderName.Substring(FolderName.LastIndexOf("\\")+ "\\".Length);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {  
                        Error = true;
                        ErrorMessage.AppendFormat("Failed to Send Files to Folder : {0}", ErrorXML);
                    }
                    else
                    {
                        xNavUser        = xNavSelectedUser.SelectSingleNode("USERLIST/USER[LOGINNAME='" + UserName + "']");
                        if (xNavUser != null)
                        {
                            NotificationXML = NotificationXML.Replace("</JOBINFO>", "<USERNAME>" + ALCommon.GetFormattedUserName(xNavUser) + "</USERNAME></JOBINFO>");
                            if (UserEmailIDValues.Length > 0)
                                ErrorXML = StampCOPYNotification("5607", Subject, UserEmailIDValues[CntUsers].ToString(), NotificationXML, SessionID, UserID, TenantName, Language);
                            if (ErrorXML != null)
                            {
                                Error = true;
                                
                                ErrorMessage.AppendFormat("Failed to stamp notification: {0}", ErrorXML);
                               
                            }
                        }
                    }
                    CntUsers++;
                    SenderNotificationXML.AppendFormat("<USER>{0} : {1}</USER>", UserName, UserFolderPath);
                }
                SenderNotificationXML.Append("</USERLIST>");
                SenderNotificationXML.AppendFormat("<DOWNLOADFOLDER>{0}</DOWNLOADFOLDER>", FolderName.Substring(FolderName.LastIndexOf("\\") + "\\".Length));

                if (ErrorMessage.Length > 0)
                {
                    IsErrorOccured = true;
                    sb_ErrorCollection.Append(ErrorMessage.ToString());
                    LogError(DAMWSObj, JobID, ErrorMessage.ToString(), ErrorType.ERROR);
                    AppendToClientLog("End Processing copy files to Network folder for RequestID:" + ThreadObj.RequestID + "Error: " + ErrorMessage.ToString(), ThreadObj);
                }
                else
                {
                    // send notification to Sender
                    if (!String.IsNullOrEmpty(NotifySender))
                    {
                        AppendToClientLog("Start Processing StampCOPYNotificationfor RequestID:" + ThreadObj.RequestID, ThreadObj);
                        Subject = InputXMLDoc.SelectSingleNode("BATCHDOWNLOAD/JOBINFO/JOBNAME").InnerText;
                        ErrorXML = StampCOPYNotification("5608", Subject, NotifySender, InputXMLDoc.OuterXml + SenderNotificationXML, SessionID, UserID, TenantName, Language);
                        if (ErrorXML != null)
                        {
                            Error = true;
                            ErrorMessage.AppendFormat("Failed to stamp notification for Sender: {0}", ErrorXML);
                            IsErrorOccured = true;
                            sb_ErrorCollection.Append(ErrorMessage.ToString());
                        }
                        AppendToClientLog("Start Processing StampCOPYNotification for RequestID:" + ThreadObj.RequestID + " ErrorXML: " + ErrorXML, ThreadObj);
                    }
                }
                if(Error)
                    DamJobXML = FormDamJobXML(JobID, "Error", InputXMLDoc.InnerXml);
                else if (ErrorInXMPEmbedding)
                    DamJobXML = FormDamJobXML(JobID, "Done with Warnings", InputXMLDoc.InnerXml);
                else
                    DamJobXML = FormDamJobXML(JobID, "Done", InputXMLDoc.InnerXml);

               
                ErrorXML     = DAMWSObj.UpdateJob(DamJobXML);
                if (ErrorXML != null)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to update DamJob entry");
                    IsErrorOccured = true;
                    sb_ErrorCollection.Append(ErrorMessage.ToString());
                    LogError(DAMWSObj,JobID, ErrorMessage.ToString(), ErrorType.ERROR);
                    return ErrorMessage.ToString();
                }
                
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Failed to Send Files to Folder : {0}", ExceptionObj.Message);
                IsErrorOccured = true;
                sb_ErrorCollection.Append(ErrorMessage.ToString());
                LogError(DAMWSObj,JobID, ErrorMessage.ToString(), ErrorType.EXCEPTION);
                return ErrorMessage.ToString();
            }
            
            return null;
        }

        #endregion

        #region Private Methods
        private ERRORXML LogError(ALDAMWS DAMWSObj, String DAMJobID, String ErrorMessage, ErrorType ErrorTypeValue)
        {
            XmlDocument     ErrorXMLDoc         = null;
            ERRORXML        ErrorXML            = null;
            XmlNode         ErrorNode           = null;
            XmlNode         ColoumnNode         = null;
            XmlNode         ErrorDetailsNode    = null;
            XML             LogErrorXML         = null;
                        
            ErrorXMLDoc                 = new XmlDocument();
            ErrorNode                   = ErrorXMLDoc.CreateElement("JOBERROR");
            
            ColoumnNode                 = ErrorXMLDoc.CreateElement("APPLICATIONNAME");
            ColoumnNode.InnerText       = ThreadObj.ApplicationName;
            ErrorNode.AppendChild(ColoumnNode);

            ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORTYPE");
            if(ErrorTypeValue == ErrorType.ERROR)
                ColoumnNode.InnerText   = "ERROR";
            else
                ColoumnNode.InnerText   = "EXCEPTION";
            ErrorNode.AppendChild(ColoumnNode);

            ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORMESSAGE");
            ErrorDetailsNode            = ErrorXMLDoc.CreateElement("ERROR");
            ErrorDetailsNode.InnerXml   = ALCommon.GetInnerValue(ref ErrorMessage, "<ERROR>", "</ERROR>");
            ColoumnNode.AppendChild(ErrorDetailsNode);
            ErrorNode.AppendChild(ColoumnNode);

            ColoumnNode                 = ErrorXMLDoc.CreateElement("DAMJOB_JOBERROR");
            ((XmlElement)ColoumnNode).SetAttribute("TYPE", "RELATION");
            ((XmlElement)ColoumnNode).SetAttribute("PARENTID", "TRUE");
            ColoumnNode.InnerText       = DAMJobID;
            ErrorNode.AppendChild(ColoumnNode);

            ErrorXMLDoc.AppendChild(ErrorNode);
            LogErrorXML                 = ErrorXMLDoc.InnerXml;
            DAMWSObj.LogError(ref LogErrorXML);
            return ErrorXML;
        }

        private XML FormDamJobXML(String JobID, String Status, XML InputXML)
        {
            XmlDocument DamJobXMLDoc = null;
            XmlNode     RootNode     = null;
            XmlNode     ColoumnNode  = null;

            try
            {
                DamJobXMLDoc            = new XmlDocument();
                RootNode                = DamJobXMLDoc.CreateElement("DAMJOB");
                ColoumnNode             = DamJobXMLDoc.CreateElement("DAMJOBID");
                ColoumnNode.InnerText   = JobID;
                RootNode.AppendChild(ColoumnNode);
                ColoumnNode             = DamJobXMLDoc.CreateElement("STATUS");
                ColoumnNode.InnerText   = Status;
                RootNode.AppendChild(ColoumnNode);
                ColoumnNode             = DamJobXMLDoc.CreateElement("INPUTXML");
                ColoumnNode.InnerXml    = InputXML;
                RootNode.AppendChild(ColoumnNode);
                ColoumnNode             = DamJobXMLDoc.CreateElement("COMPLETEDON");
                ColoumnNode.InnerXml    = ALCommon.CurrentDateTimeToXml();
                RootNode.AppendChild(ColoumnNode);

                JobSubmissionTime       = DateTime.Now;
                return RootNode.OuterXml;
            }
            catch (Exception)
            {
               throw;
            }
        }

        private String PrepareErrorMessageXml(String str_ErrorMessage,String ErrorDetails, String str_ErrorCode, String str_AdditionalInformation)
        {
            #region Local Variables
            XmlDocument     ErrorMessageXmlDoc  = null;
            XmlNode         ErrorNode           = null;
            XmlNode         ColumnNode          = null;
            #endregion

            ErrorMessageXmlDoc          = new XmlDocument();
            ErrorNode                   = ErrorMessageXmlDoc.CreateElement("ERROR");

            if (!String.IsNullOrEmpty(str_ErrorCode))
            {
                ColumnNode              = ErrorMessageXmlDoc.CreateElement("ERRORCODE");
                ColumnNode.InnerText    = str_ErrorCode;
                ErrorNode.AppendChild(ColumnNode);
            }
            ColumnNode                  = ErrorMessageXmlDoc.CreateElement("MESSAGE");
            ColumnNode.InnerText        = str_ErrorMessage;
            ErrorNode.AppendChild(ColumnNode);

            ColumnNode                  = ErrorMessageXmlDoc.CreateElement("ERRORDETAILS");
            ColumnNode.InnerText        = ErrorDetails;
            ErrorNode.AppendChild(ColumnNode);

            if (!String.IsNullOrEmpty(str_AdditionalInformation))
            {
                ColumnNode              = ErrorMessageXmlDoc.CreateElement("ADDITIONALINFO");
                ColumnNode.InnerXml     = str_AdditionalInformation;
                ErrorNode.AppendChild(ColumnNode);
            }

            return ErrorNode.OuterXml;
        }

        public ERRORXML EmbeddXMPInToOriginalFiles(XmlDocument BatchDownloadXMLDoc, String str_JobPath)
        {
            StringBuilder       SearchInputXML                  = null;
            XmlNodeList         FacetNodes                      = null;
            XmlNodeList         FieldNodes                      = null;
            XmlNodeList         DigitalAssetIDNodes             = null;
            StringBuilder       DigitalAssetIDs                 = null;
            String              Separator                       = null;
            String              XPath                           = null;
            String              FilePath                        = null; 
            String              DigitalAssetCollectionXML       = null;
            XPathDocument       DigitalAssetCollectionXMLDoc    = null; 
            XPathNavigator      XPathNav                        = null;
            XPathNavigator      DigitalAssetNode                = null;
            XMPHelper           XMPHelperObj                    = null;
            String              XMPProcessor                    = null;
            XmlDocument         ErrorXMLDoc                     = null;
            XmlNode             ErrorCollectionNode             = null;
            XmlNode             ErrorNode                       = null;
            XmlNode             ColumnNode                      = null;
            ERRORXML            ErrorXML                        = null;
            String              ErrorMsg                        = null;
            XmlNode             RelationshipNameNode            = null;
            XmlNode             FieldNameNode                   = null;
            XmlNode             MetadataFacetMappingNode        = null;
            
            try
            {
                #region Get AssetID's of corresponding Original renditions
                DigitalAssetIDs         = new StringBuilder(100);
                DigitalAssetIDNodes     = BatchDownloadXMLDoc.SelectNodes("BATCHDOWNLOAD/ASSET[RENDITIONTYPE='Original']/DIGITALASSETID");
                foreach (XmlNode DigitalAssetIDNode in DigitalAssetIDNodes)
                {
                    DigitalAssetIDs.Append(Separator + DigitalAssetIDNode.InnerText);
                    if (Separator == null)
                        Separator = ",";
                }
                #endregion

                #region Get Facets based on XMPFacetFieldMapping configuration
                MetadataFacetMappingNode    = BatchDownloadXMLDoc.SelectSingleNode("BATCHDOWNLOAD/MetadataFacetMapping");
                FacetNodes                  = MetadataFacetMappingNode.SelectNodes("Solution/FacetToXMP/Facet");
                if (FacetNodes.Count > 0)
                {
                    SetUpSearchObj();
                    SearchInputXML          = new StringBuilder(256);
                    SearchInputXML.AppendFormat(@"<SEARCH>
                                                    <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <SESSIONINFO Language='{0}' UserID='{1}' TenantName='{2}' SessionID='{3}'/>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                    <TARGETENTITY>
                                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                        <SELECTLIST>
                                                            <EXPRESSION>
	                                                            <FIELDNAME>DIGITALASSETID</FIELDNAME>
	                                                            <ALIAS>DIGITALASSETID</ALIAS>
                                                            </EXPRESSION>
                                                         </SELECTLIST>
                                		                 <FILTERLIST>
                                                            <FILTER>
                                				                <EXPRESSION>DIGITALASSETID</EXPRESSION>
	                            				                <OPERATOR>IN</OPERATOR>
	                            				                <EXPRESSIONVALUE>({4})</EXPRESSIONVALUE>
	                            				                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                            				                </FILTER>
                        				                 </FILTERLIST>", str_Language, str_UserID, str_TenantName, str_SessionID, DigitalAssetIDs.ToString());
                        foreach(XmlNode FacetNode in FacetNodes)
                        {
                            RelationshipNameNode    = FacetNode.Attributes.GetNamedItem("RelationshipName");
                            SearchInputXML.Append("<RELATIONSHIP NAME='" + RelationshipNameNode.InnerText + "'>");
                            FieldNodes              = FacetNode.SelectNodes("XMP/XMPProperty");
                            if(FieldNodes.Count > 0)
                            {
                                SearchInputXML.Append("<SELECTLIST>");
                                foreach (XmlNode FieldNode in FieldNodes)
                                {
                                    FieldNameNode   = FieldNode.Attributes.GetNamedItem("FieldName");
                                    SearchInputXML.AppendFormat(@"<EXPRESSION>
                                                                        <FIELDNAME>{0}</FIELDNAME>
	                                                                    <ALIAS>{0}</ALIAS>
                                                                  </EXPRESSION>", FieldNameNode.InnerText);
                                }
                                SearchInputXML.Append("</SELECTLIST>");
                            }
                             SearchInputXML.Append("</RELATIONSHIP>");
                        }

                        SearchInputXML.Append(@"        </TARGETENTITY>
                				                    </SEARCH>");


                    SearchObj.GetHierarchialData(SearchInputXML.ToString(),ref DigitalAssetCollectionXML);
                }
                #endregion

                #region Embedd XMP into Original files
                DigitalAssetCollectionXMLDoc    = new XPathDocument(new StringReader(DigitalAssetCollectionXML));
                XPathNav                        = DigitalAssetCollectionXMLDoc.CreateNavigator();
                foreach (XmlNode DigitalAssetIDNode in DigitalAssetIDNodes)
                {
                    XPath               = "DIGITALASSETCOLLECTION/DIGITALASSET[DIGITALASSETID="+DigitalAssetIDNode.InnerText +"]";
                    DigitalAssetNode    = XPathNav.SelectSingleNode(XPath);
                    if (DigitalAssetNode != null)
                    {
                        XMPHelperObj    = new XMPHelper(ul_SessionID, ul_UserID, str_Language, str_TenantName);
                        FilePath        = str_JobPath + "\\INPUT\\" + DigitalAssetIDNode.ParentNode.SelectSingleNode("DOWNLOADFILENAME").InnerText;
                        XMPHelperObj.GetXMPProcessor(FilePath, ref XMPProcessor);
                        ErrorMsg        = MetadataHandler.EmbedXMPIntoFile(XMPProcessor, FilePath, DigitalAssetNode.OuterXml, MetadataFacetMappingNode.OuterXml);
                        if (!String.IsNullOrEmpty(ErrorMsg))
                        {
                            if (ErrorXMLDoc == null)
                            {
                                ErrorXMLDoc             = new XmlDocument();
                                ErrorCollectionNode     = ErrorXMLDoc.CreateElement("ERRORCOLLECTION");
                            }

                            ErrorNode                   = ErrorXMLDoc.CreateElement("ERROR");
                            ErrorCollectionNode.AppendChild(ErrorNode);

                            ColumnNode                  = ErrorXMLDoc.CreateElement("DIGITALASSETNAME");
                            ColumnNode.InnerText        = DigitalAssetIDNode.ParentNode.SelectSingleNode("NAME").InnerText;
                            ErrorNode.AppendChild(ColumnNode);

                            ColumnNode                  = ErrorXMLDoc.CreateElement("ERRORMESSAGE");
                            ColumnNode.InnerText        = ErrorMsg;
                            ErrorNode.AppendChild(ColumnNode);
                        }
                    }
                }
                if(ErrorCollectionNode != null)
                    ErrorXML = ErrorCollectionNode.OuterXml;
                #endregion
            }
            catch (Exception ExceptionObj)
            {
                ErrorXML = "<ERRORCOLLECTION><ERROR><ERRORMESSAGE>" + ExceptionObj.Message + "</ERRORMESSAGE></ERROR></ERRORCOLLECTION>";
            }
            return ErrorXML;
        }

        void SetUpSearchObj()
        {
            SearchObj       = new Search();
            SearchObj.Url   = ConfigurationManager.AppSettings.Get("ALSearchWS");
        }

        //ZIP Functions
        private ERRORXML StampNotification(bool IsSuccess,String ErrorMessage, XmlDocument InputXML, XmlDocument ConfigXML,
                                            String SessionID, String UserID, String TenantName, String Language, String ZipFileName)
        {
            #region Member Variables
            StringBuilder       sb_MessageString        = null;
            String              ErrorXml                = "";
            StringBuilder       str_Subject             = null;            
            String              str_EmailStampNotxml    = "";
            SendNotification    obj_Notification        = null;
            NotificationHelper  obj_NotificationHelper  = null;
            EmailChannel        obj_EmailChannel        = null;            
            String              NotificationTypeID      = "5611";            
            String              str_EmailID             = null;
            String              str_LinkValidity        = null;
            String              LoginName               = null;
            String              str_InputXML            = null;
            String              FileSize                = null;
            double              TimeRequiredtoDownlaod  = 0;
            XmlNode             TempNode                = null;
            XmlNode             JobInfoNode             = null;
            String              str_NumDecimalSeperator = null;
            #endregion

            # region Add Notificatio to Standard Email Infrastructure
            str_Subject             = new StringBuilder();
            sb_MessageString        = new StringBuilder();
            obj_NotificationHelper  = new NotificationHelper();
            str_InputXML            = InputXML.OuterXml;

            if (ConfigXML.SelectSingleNode("CONFIGURATION/LINKVALIDITYDURATION") != null)
                str_LinkValidity = ConfigXML.SelectSingleNode("CONFIGURATION/LINKVALIDITYDURATION").InnerText;
            else
                str_LinkValidity = "5";

            if (ConfigXML.SelectSingleNode("CONFIGURATION/NUMBERDECIMALSEPERATOR") != null)
                str_NumDecimalSeperator = ConfigXML.SelectSingleNode("CONFIGURATION/NUMBERDECIMALSEPERATOR").InnerText;
            else
                str_NumDecimalSeperator = ".";

            if (ConfigXML.SelectSingleNode("CONFIGURATION/CULTURE") != null)
                strCulture = ConfigXML.SelectSingleNode("CONFIGURATION/CULTURE").InnerText;
            else
                strCulture = "en-US";

            obj_NotificationHelper.Init(NotificationTypeID, UserID);            
            
              //Initialize channel
            obj_EmailChannel = new EmailChannel(PriorityType.NORMAL, false);

            //Add Recepient      
            str_EmailID = GetRecipient(SessionID, UserID, TenantName, Language, ref LoginName);
            if (String.IsNullOrEmpty(str_EmailID))
                return PrepareErrorMessageXml("Error while sending email, receipent email ID is null.", null, null, null);

            obj_EmailChannel.AddRecipient("TO", str_EmailID.Trim(), "", "");
            obj_NotificationHelper.AddChannel(obj_EmailChannel);

            //Prepare Message            
            sb_MessageString.AppendFormat("<MESSAGE IsSuccess='{0}'>", IsSuccess.ToString().ToLower());
            if (IsSuccess)
            {                
                sb_MessageString.AppendFormat("{0}","The files have been compressed successfully.");
                str_Subject.AppendFormat("Zip file created successfully: {0}", ZipFileName);
            }
            else
            {             
                sb_MessageString.AppendFormat("{0}", "There are errors in compressing files.");
                str_Subject.AppendFormat("Compression Job {0}: Error", JobName);
                if(!String.IsNullOrEmpty(ErrorMessage))
                    sb_MessageString.Append("<ERRORMESSAGE>"+ ErrorMessage +"</ERRORMESSAGE>");
            }

            GetFileSize(ZipFilePath, str_NumDecimalSeperator, ref FileSize, ref TimeRequiredtoDownlaod);
            JobInfoNode     = InputXML.SelectSingleNode("BATCHDOWNLOAD/JOBINFO");
            if (JobInfoNode != null)
            {
                TempNode            = InputXML.CreateElement("FILESIZE");
                TempNode.InnerText  = FileSize;
                JobInfoNode.AppendChild(TempNode);

                //TempNode            = InputXML.CreateElement("TIMEREQUIREDTODOWNLOAD");
                //TempNode.InnerText  = GetFormattedDownloadedTime((Int64)TimeRequiredtoDownlaod, TenantName);
                //JobInfoNode.AppendChild(TempNode);
            }

            sb_MessageString.AppendFormat(@"<JOBDETAILS>
                                            {0}
                                            {1}
                                            <LINKVALIDITYDURATION>{2}</LINKVALIDITYDURATION>                                
                                           </JOBDETAILS>", InputXML.OuterXml, ConfigXML.OuterXml, GetZipFileValidityDate(str_LinkValidity));
            sb_MessageString.Append("</MESSAGE>");

            //add link to notification
            ErrorXml = obj_NotificationHelper.AddLink("CompressionJobStatusPage", GetZipJobStatusEmailPageUrl(SessionID, UserID, TenantName, Language, LoginName));
            ErrorXml = obj_NotificationHelper.AddSubject("<SUBJECT>" + str_Subject.ToString() + "</SUBJECT>");
            ErrorXml = obj_NotificationHelper.AddMessage(sb_MessageString.ToString());
            

            ErrorXml = obj_NotificationHelper.GetNotificationXML(ref str_EmailStampNotxml);
            if (!String.IsNullOrEmpty(str_EmailStampNotxml))
            {
                obj_Notification = new SendNotification(SessionID, UserID, TenantName, Language);
                ErrorXml = obj_Notification.StampNotification(str_EmailStampNotxml);
            }
            return ErrorXml;
            #endregion
        }

        /// <alsummary>
        /// <createddate>29 Feb 2012</createddate>        
        /// <purpose>To Get Url of Compression Status page </purpose>
        /// </alsummary>
        private String GetZipJobStatusEmailPageUrl(String SessionID, String UserID, String TenantName, String Language, String LoginName)
        {
            # region Local VAriables
            StringBuilder               ZipJobStatusEmailPageUrl        = null;
            String                      DefaultPageMode             = "EMAIL";
            String                      CommunicationProtocol       = null;
            String                      InternetAddress             = null;        
            StringBuilder               SolutionPageParams          = new StringBuilder();            
            StringBuilder               SolutionPageUrl             = null;
            ALCommon                    ObjALCommon                 = null;
            #endregion

            ObjALCommon             = new ALCommon();
            CommunicationProtocol   = ALCommon.GetCommunicationProtocol();
            InternetAddress         = ALCommon.GetInternetAddress();
            ZipJobStatusEmailPageUrl= new StringBuilder();
            SolutionPageUrl         = new StringBuilder();

            SolutionPageUrl.AppendFormat("/{0}/Widgets/{1}/Aspx/CompressionOutput.aspx", TenantName, "DAM_AssetListWidget");
            SolutionPageParams.AppendFormat("SenderID={0}&DAMJobID={1}&SolutionName=DAM&FromEmail=TRUE", UserID, DAMJobID);

            ZipJobStatusEmailPageUrl.AppendFormat("{0}{1}/{2}/default.aspx?", CommunicationProtocol, InternetAddress, TenantName);
            ZipJobStatusEmailPageUrl.AppendFormat("Language={0}", Language);
            ZipJobStatusEmailPageUrl.AppendFormat("&Mode={0}", DefaultPageMode);
            ZipJobStatusEmailPageUrl.AppendFormat("&Imp=N");
            ZipJobStatusEmailPageUrl.AppendFormat("&UserName={0}", LoginName);
            ZipJobStatusEmailPageUrl.AppendFormat("&SolutionPage={0}", SolutionPageUrl.ToString());
            ZipJobStatusEmailPageUrl.AppendFormat("&ThemeName=Default");
            ZipJobStatusEmailPageUrl.AppendFormat("&Param={0}", ALCommon.Encrypt(SolutionPageParams.ToString()));
            ZipJobStatusEmailPageUrl = ZipJobStatusEmailPageUrl.Replace("&", "%26");

            return ZipJobStatusEmailPageUrl.ToString();
        }

        private String GetRecipient(String SessionID, String UserID, String TenantName, String Language, ref String LoginName)
        {
            #region member variables
            
            Search          ObjSearch           = null;
            String          str_SearchInput     = null;
            String          SearchResult        = null;
            String          ErrorXML            = null;
            String          EmailID             = null;
            XPathDocument   xp_SearchResult     = null;
            XPathNavigator  xpNav_SearchRoot    = null;
            XPathNavigator  xpNav_Data          = null;
            #endregion

            try
            {
                #region Initialization            
                ObjSearch       = new Search();
                #endregion

                #region Prepare SearchXML and perform search
                str_SearchInput = String.Format(@"<SEARCH>
                                                    <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}'/>
                                                    <TARGETOBJECT>USER</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>TRUE</AUTHORIZATIONREQUIRED>
                                                    <MAXRESULTSIZE>1</MAXRESULTSIZE>
                                                    <TARGETENTITY>
                                                      <FILTERLIST>
                                                        <FILTER>
                                                          <EXPRESSION>USERID</EXPRESSION>
                                                          <OPERATOR>EQUALTO</OPERATOR>
                                                          <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>
                                                        </FILTER>                                                        
                                                      </FILTERLIST>
                                                      <SELECTLIST>
                                                        <EXPRESSION>
                                                          <FIELDNAME>USERID</FIELDNAME>
                                                          <ALIAS>USERID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                          <FIELDNAME>LOGINNAME</FIELDNAME>
                                                          <ALIAS>LOGINNAME</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                          <FIELDNAME>EMAILID</FIELDNAME>
                                                          <ALIAS>EMAILID</ALIAS>
                                                        </EXPRESSION>
                                                      </SELECTLIST>                                                      
                                                    </TARGETENTITY>
                                                  </SEARCH>", SessionID, UserID, TenantName, Language);

                ObjSearch.Url   = ConfigurationManager.AppSettings["ALSearchWS"];
                ErrorXML        = ObjSearch.GetHierarchialData(str_SearchInput, ref SearchResult);
                if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(SearchResult))
                {
                    xp_SearchResult     = new XPathDocument(new StringReader(SearchResult));
                    xpNav_SearchRoot    = xp_SearchResult.CreateNavigator();

                    xpNav_Data = xpNav_SearchRoot.SelectSingleNode("USERCOLLECTION/USER/EMAILID");
                    if (xpNav_Data != null)
                        EmailID = xpNav_Data.Value;

                    xpNav_Data = xpNav_SearchRoot.SelectSingleNode("USERCOLLECTION/USER/LOGINNAME");
                    if (xpNav_Data != null)
                        LoginName = xpNav_Data.Value;                    
                }
                #endregion
            }
            catch (Exception)
            {
                return null;
            }
            return EmailID;
        }

        private void GetFileSize(String FilePath, String str_NumberDecimalSeparator, ref String FileSize, ref double TimeReqrdToDownload)
        {            
            double numFileSize = 0;            
            FileInfo FileInfoObj = new FileInfo(FilePath);
            if (FileInfoObj.Exists)
            {
                numFileSize = FileInfoObj.Length;
                if (numFileSize != 0)
                {
                    FileSize                = GetFileSize(numFileSize.ToString(), str_NumberDecimalSeparator);
                    numFileSize             = numFileSize / 1024;
                    TimeReqrdToDownload     = Math.Ceiling(numFileSize / 7);                    
                }                
            }
        }

        public String GetFileSize(String FileSizeInBytes, String str_NumberDecimalSeparator)
        {
            UInt64 SizeInBytes;
            String ConvertedValue = "";            
            string suffix = string.Empty;
            String DummyTenantName = "";

            try
            {
                String IsMediaServerInstalled = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK\", "MediaServer Installed");
                if (IsMediaServerInstalled == "1")
                    DummyTenantName = "ALMediaServices";
                else
                    DummyTenantName = str_TenantName;

                //This is done because For English, it should not append the Culture in Resource file name.
                if (String.Compare(strCulture, "en-US", true) == 0)
                    strCulture = "";

                SizeInBytes = Convert.ToUInt64(FileSizeInBytes);
                double d = 0;

                if (SizeInBytes < (1ul << 10)) return SizeInBytes.ToString() + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_Bytes_Shared");                     // byte

                else if (SizeInBytes < (1ul << 20)) { d = SizeInBytes / (double)(1ul << 10); suffix = " " + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_KB_Shared")/*" KB"*/; } // kilobyte

                else if (SizeInBytes < (1ul << 30)) { d = SizeInBytes / (double)(1ul << 20); suffix = " " + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_MB_Shared")/*" MB"*/; } // megabyte

                else if (SizeInBytes < (1ul << 40)) { d = SizeInBytes / (double)(1ul << 30); suffix = " " + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_GB_Shared")/*" GB"*/; } // gigabyte

                else if (SizeInBytes < (1ul << 50)) { d = SizeInBytes / (double)(1ul << 40); suffix = " " + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_TB_Shared")/*" TB"*/; } // terabyte

                else if (SizeInBytes < (1ul << 60)) { d = SizeInBytes / (double)(1ul << 50); suffix = " " + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_PB_Shared")/*" PB"*/; } // petabyte

                else { d = SizeInBytes / (double)(1ul << 60); suffix = " " + Localization.GetStringByPath("ALResource", DummyTenantName, str_Language, strCulture, "FileSize_EB_Shared")/*" EB"*/; }                                // exabyte

                //Convert it to a value with 2 digits after .
                ConvertedValue = d.ToString();


                /* For different culture NumberDecimalSeparator is different (e.g. For en-US culture NumberDecimalSeparator is . (dot) whereas
                 * for fr-BE culture [French] it is , (comma)). Previous statement d.ToString() converts value to culture specific number format. 
                 * In that case NumberDecimalSeparator can be used to truncate data from string.
                 */                
                if (ConvertedValue.IndexOf(str_NumberDecimalSeparator) != -1)
                {
                    if (ConvertedValue.Length - ConvertedValue.IndexOf(str_NumberDecimalSeparator) >= 3)
                        ConvertedValue = ConvertedValue.Substring(0, ConvertedValue.IndexOf(str_NumberDecimalSeparator) + 3);
                }
            }
            catch (Exception)
            {
                return "-";
            }
            return ConvertedValue + suffix;
        }

        public String GetFormattedDownloadedTime(Int64 DownloadTime, String TenantName)
        {
            TimeSpan ObjTimeSpan;
            StringBuilder strTimeFormat = null;

            ObjTimeSpan = new TimeSpan(TimeSpan.TicksPerSecond * DownloadTime);
            strTimeFormat = new StringBuilder(100);

            //if (ObjTimeSpan.Hours > 0)
            //    strTimeFormat.AppendFormat("{0}{1} ", ObjTimeSpan.Hours, Localization.GetString("ALResource", TenantName, "Hours_Shared"));

            //if (ObjTimeSpan.Minutes > 0)
            //    strTimeFormat.AppendFormat("{0}{1} ", ObjTimeSpan.Minutes, Localization.GetString("ALResource", TenantName, "Minutes_Shared"));

            //if (ObjTimeSpan.Seconds > 0)
            //    strTimeFormat.AppendFormat("{0}{1}", ObjTimeSpan.Seconds, Localization.GetString("ALResource", TenantName, "Seconds_Shared"));

            //if (ObjTimeSpan.Hours == 0 && ObjTimeSpan.Minutes == 0 && ObjTimeSpan.Seconds == 0)
            //{
            //    strTimeFormat.AppendFormat("{0}{1}", ObjTimeSpan.Seconds, Localization.GetString("ALResource", TenantName, "Seconds_Shared"));
            //}

            return strTimeFormat.ToString();
        }

        public String GetZipFileValidityDate(String str_LinkValidity)
        {
            #region Member Varibales            
            Int32           Duration                = 0;
            DateTime        ValidTill;            
            #endregion

            if (!String.IsNullOrEmpty(str_LinkValidity) && JobSubmissionTime != null)
            {
                Int32.TryParse(str_LinkValidity, out Duration);
                ValidTill           = JobSubmissionTime.AddDays(Duration);
                return ALCommon.DateTimeToXml(ValidTill);
            }
            return null;
        }

        private void NTFSCleanup(String FolderPath)
        {
            try
            {
                StringBuilder sb_AdditionalInfoXml = new StringBuilder(200);
                sb_AdditionalInfoXml.AppendFormat(@"<ADDITIONALINFO UserID='{1}' SessionID='{2}' Language='{3}' TenantName='{4}'>
                                                    <THREADID>{0}</THREADID>
                                                    <METHODNAME>AssetProcessor.AssetProcessor.NTFSJobCleanUp()</METHODNAME>    
                                                    <JOBID>{5}</JOBID>                                           
                                                    <JOBPATH>{6}</JOBPATH>    
                                              </ADDITIONALINFO>",
                                                 System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), ul_UserID.ToString(), ul_SessionID.ToString(), "English", str_TenantName,
                                                 m_JobID, ALCommon.TrimAndReplaceSpecialChars(JobPath));

                Context ObjContext = new Context(ul_UserID, ul_SessionID, str_TenantName, str_SolutionName, sb_AdditionalInfoXml.ToString());
                SASMOMDirectory.DeleteFolder(FolderPath, true, ObjContext); 
            }
            catch (Exception)
            { }
        }

        //FTP Functions
        private StringBuilder PrepareJobDetailsXML(XmlDocument ConfigDocument, XmlDocument InputDocument)
        {
            StringBuilder   JobDetails              = null;
            XmlNodeList     TranseferedFileNodeList = null;
            XmlNodeList     FailedFileNodeList      = null;
            XmlNode         JobInfoNode             = null;

            JobDetails               = new StringBuilder();
            JobDetails.AppendFormat("{0}", "<JOBDETAILS>");
            JobDetails.AppendFormat("{0}", ConfigDocument.DocumentElement.InnerXml);
            
            TranseferedFileNodeList = InputDocument.SelectNodes("descendant::ASSET");
            foreach (XmlNode FileNode in TranseferedFileNodeList)
                JobDetails.AppendFormat("{0}", FileNode.OuterXml);

            FailedFileNodeList = InputDocument.SelectNodes("descendant::ASSET[@Status='Error']");
            if (FailedFileNodeList.Count > 0 )
                JobDetails.AppendFormat("{0}", "<FAILEDFILELIST TransferFailed='true'>");
            else 
                JobDetails.AppendFormat("{0}", "<FAILEDFILELIST TransferFailed='false'>");

            foreach (XmlNode FileNode in FailedFileNodeList)
                JobDetails.AppendFormat("{0}", FileNode.OuterXml);

            JobDetails.AppendFormat("{0}", "</FAILEDFILELIST>");

            JobInfoNode = InputDocument.SelectSingleNode("descendant::JOBINFO");
            if (JobInfoNode != null)
                JobDetails.Append(JobInfoNode.OuterXml);
            JobDetails.AppendFormat("{0}", "</JOBDETAILS>");
            return JobDetails;

        }

        private ERRORXML StampFTPNotification(bool IsSuccess,String ErrorMessage, String NotifyUserlist, StringBuilder JobDetailsXML, 
                                            String SessionID, String UserID, String TenantName, String Language)
        {
            StringBuilder       sb_MessageString        = null;
            String              ErrorXml                = "";
            StringBuilder       str_Subject             = null;            
            String              str_EmailStampNotxml    = "";
            SendNotification    obj_Notification        = null;
            NotificationHelper  obj_NotificationHelper  = null;
            EmailChannel        obj_EmailChannel        = null;
            Char[]              chr_delimiter           = null;
            String[]            str_EmailIDList         = null;
            String              str_delimStr            = null;
            
                        
            # region Add Notificatio to Standard Email Infrastructure
            str_Subject = new StringBuilder();
            sb_MessageString        = new StringBuilder();

            obj_NotificationHelper  = new NotificationHelper();
            obj_NotificationHelper.Init("5603", UserID);
            
            //Initialize channel
            obj_EmailChannel = new EmailChannel(PriorityType.NORMAL, false);

            //Add Recepient      
            str_delimStr            = ",";
            chr_delimiter           = str_delimStr.ToCharArray();
            str_EmailIDList         = NotifyUserlist.Split(chr_delimiter);

            foreach (String str_EmailID in str_EmailIDList)
                obj_EmailChannel.AddRecipient("TO", str_EmailID.Trim(), "", "");

            obj_NotificationHelper.AddChannel(obj_EmailChannel);

            //Prepare Message            
            sb_MessageString.AppendFormat("<MESSAGE>");
            if (IsSuccess)
            {                
                sb_MessageString.AppendFormat("{0}","The files have been transferred successfully.");
                str_Subject.AppendFormat("FTP Job {0}: Successful",JobName);
            }
            else
            {             
                sb_MessageString.AppendFormat("{0}", "There are errors in transfering files.");
                str_Subject.AppendFormat("FTP Job {0}: Error", JobName);
                if(!String.IsNullOrEmpty(ErrorMessage))
                    sb_MessageString.Append("<ERRORMESSAGE>"+ ErrorMessage +"</ERRORMESSAGE>");
            }
        
            sb_MessageString.AppendFormat("{0}", JobDetailsXML.ToString());
            sb_MessageString.Append("</MESSAGE>");

            ErrorXml = obj_NotificationHelper.AddSubject("<SUBJECT>" + str_Subject.ToString() + "</SUBJECT>");
            ErrorXml = obj_NotificationHelper.AddMessage(sb_MessageString.ToString());
            ErrorXml = obj_NotificationHelper.GetNotificationXML(ref str_EmailStampNotxml);
            if (!String.IsNullOrEmpty(str_EmailStampNotxml))
            {
                obj_Notification = new SendNotification(SessionID, UserID, TenantName, Language);
                ErrorXml = obj_Notification.StampNotification(str_EmailStampNotxml);
            }
            return ErrorXml;
            #endregion
        }

        //COPY To Network Folder Function
        private ERRORXML CopyFiles(String UserName, String SourcePath, String DestPath, String DownloadFolderName, ref XmlDocument InputXMLDoc,ref String NotificationXML,out String UserFolderPath , bool OverwriteFiles)
        {
            StringBuilder DestinationFolder     = null;
            String        FileName              = null;
            DirectoryInfo objSourceDirInfo      = null;
            XmlNode       SelectedFileNode      = null;
            StringBuilder NotificationDetails   = null;
            String        SectionPath           = null;
            String        OriginalFileName      = null;
            String        IsFileNameChanged     = "FALSE";          

            DestinationFolder   = new StringBuilder (50);
            NotificationDetails = new StringBuilder(100);

            try
            {
                DestinationFolder.AppendFormat("{0}\\{1}\\{2}", DestPath, UserName, DownloadFolderName);
                if (!Directory.Exists(DestinationFolder.ToString()))
                    Directory.CreateDirectory(DestinationFolder.ToString());

                UserFolderPath = DestinationFolder.ToString();
                NotificationDetails.AppendFormat("<DESTINATIONFOLDER>{0}</DESTINATIONFOLDER>", DestinationFolder);
                objSourceDirInfo = new DirectoryInfo(SourcePath);

                foreach (FileInfo sourceFile in objSourceDirInfo.GetFiles())
                {
                    FileName = sourceFile.Name;
                    NotificationDetails.AppendFormat("<ORIGINALFILENAME>{0}</ORIGINALFILENAME>", ALCommon.TrimAndReplaceSpecialChars(FileName));

                    SelectedFileNode = InputXMLDoc.SelectSingleNode("BATCHDOWNLOAD/ASSET/DOWNLOADFILENAME[text()=" + ALXmlUtil.GetXpathValueString(FileName) + "]").ParentNode;

                    if (File.Exists(Path.Combine(DestinationFolder.ToString(), sourceFile.Name)) && OverwriteFiles==false)
                    {
                        OriginalFileName = FileName;
                        FileName = getUniqueFileName(DestinationFolder.ToString(), sourceFile.Name);
                        if (SelectedFileNode != null)
                        {
                            SelectedFileNode.InnerXml = SelectedFileNode.InnerXml + "<ACTUALFILENAME USERNAME=\"" + UserName + "\">" + ALCommon.TrimAndReplaceSpecialChars(FileName) + "</ACTUALFILENAME>";
                            SectionPath               = SelectedFileNode.SelectSingleNode("SECTIONPATH").InnerText;
                            IsFileNameChanged = "TRUE";
                        }
                        NotificationDetails.AppendFormat("<RENAMEDFILE><SOURCE>{0}</SOURCE><DESTINATION>{1}</DESTINATION></RENAMEDFILE>", ALCommon.TrimAndReplaceSpecialChars(OriginalFileName), ALCommon.TrimAndReplaceSpecialChars(FileName));
                    }
                    else
                    {
                        if (SelectedFileNode != null)
                        {
                            if ((SelectedFileNode.SelectSingleNode("ACTUALFILENAME") == null) || ((SelectedFileNode.SelectSingleNode("ACTUALFILENAME").Attributes.Count == 0)))
                                SelectedFileNode.InnerXml = SelectedFileNode.InnerXml + "<ACTUALFILENAME>" + ALCommon.TrimAndReplaceSpecialChars(FileName) + "</ACTUALFILENAME>";
                            SectionPath = SelectedFileNode.SelectSingleNode("SECTIONPATH").InnerText;
                        }
                    }
                    sourceFile.CopyTo(Path.Combine(DestinationFolder.ToString(), FileName), OverwriteFiles);
                    NotificationDetails.AppendFormat("<DOWNLOADFILENAME>{0}</DOWNLOADFILENAME>", ALCommon.TrimAndReplaceSpecialChars(FileName));
                    NotificationDetails.AppendFormat("<DOWNLOADFOLDER>{0}</DOWNLOADFOLDER>", DownloadFolderName.Substring(DownloadFolderName.LastIndexOf("\\") + "\\".Length));
                }
                NotificationDetails.AppendFormat("<FILERENAMED>{0}</FILERENAMED>", IsFileNameChanged);
                NotificationDetails.Append("</JOBINFO>");
                NotificationXML = NotificationXML.Replace("</JOBINFO>", NotificationDetails.ToString());

            }
            catch (Exception objExcep)
            {
                UserFolderPath = "";
                return objExcep.Message;
            }
            return null;
        }

        // Usage: if file with the name is already exists, then this function returns the unique name of the file, which is not present on NTFS.
        private String getUniqueFileName(String DestPath, String FileName)
        {
            bool    flag = true;
            int     cnt = 1;
            String  name, ext;
            String uniquename = "";

            if (FileName.Contains("."))
            {
                name = FileName.Substring(0, FileName.LastIndexOf('.'));
                ext  = FileName.Substring(FileName.LastIndexOf('.'));
            }
            else
            {
                name = FileName;
                ext = "";
            }

            while (flag)
            {
                uniquename = name + "(" + cnt + ")" + ext;
                if (!File.Exists(Path.Combine(DestPath,uniquename)))
                    flag = false;
                cnt++;
            }
            return uniquename;
        }

        // stamp the notification for all the users selected.
        private ERRORXML StampCOPYNotification(String NotificationID,String Subject,String NotifyUserlist, String NotificationDetailsXML, 
                                            String SessionID, String UserID, String TenantName, String Language)
        {
            StringBuilder       sb_MessageString        = null;
            String              ErrorXml                = "";
            String              str_EmailStampNotxml    = "";
            SendNotification    obj_Notification        = null;
            NotificationHelper  obj_NotificationHelper  = null;
            EmailChannel        obj_EmailChannel        = null;
            Char[]              chr_delimiter           = null;
            String[]            str_EmailIDList         = null;
            String              str_delimStr            = null;
            
                        
            # region Add Notification to Standard Email Infrastructure
            sb_MessageString        = new StringBuilder();

            obj_NotificationHelper  = new NotificationHelper();
            obj_NotificationHelper.Init(NotificationID, UserID);
            
            //Initialize channel
            obj_EmailChannel = new EmailChannel(PriorityType.NORMAL, false);

            //Add Recepient      
            str_delimStr            = ",";
            chr_delimiter           = str_delimStr.ToCharArray();
            str_EmailIDList         = NotifyUserlist.Split(chr_delimiter);

            foreach (String str_EmailID in str_EmailIDList)
                obj_EmailChannel.AddRecipient("TO", str_EmailID.Trim(), "", "");

            obj_NotificationHelper.AddChannel(obj_EmailChannel);

            //Prepare Message            
            sb_MessageString.AppendFormat("<MESSAGE>");
            sb_MessageString.Append(NotificationDetailsXML);
            sb_MessageString.Append("</MESSAGE>");

            ErrorXml = obj_NotificationHelper.AddSubject("<SUBJECT>" + Subject + "</SUBJECT>");
            ErrorXml = obj_NotificationHelper.AddMessage(sb_MessageString.ToString());
            ErrorXml = obj_NotificationHelper.GetNotificationXML(ref str_EmailStampNotxml);
            if (!String.IsNullOrEmpty(str_EmailStampNotxml))
            {
                obj_Notification = new SendNotification(SessionID, UserID, TenantName, Language);
                ErrorXml = obj_Notification.StampNotification(str_EmailStampNotxml);
            }
            return ErrorXml;
            #endregion
        }
        #endregion      

        #region Media Processor Logging Functions
        public override void StopApplication()
        {
            base.StopApplication();
        }

        private void AppendToClientLog(String LogContent, CommonVariables ThreadObj)
        {
            if (!ThreadObj.ISClientLogAccessible)
                return;

            StringBuilder sbClientLog = new StringBuilder(200);
            sbClientLog.AppendFormat("Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6} :-  {7} \r\n", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond, LogContent);

            try
            {
                File.AppendAllText(ThreadObj.ClientLogPath, sbClientLog.ToString());
            }
            catch
            {
                ThreadObj.ISClientLogAccessible = false;
                throw;
            }
        }

        private void UpdateRequestXML(String Status, XPathNavigator xpNav_MediaRequest, CommonVariables ThreadObj, String InputFileName)
        {
            XPathNavigator xpNav_Status = null;
                        
            if (xpNav_MediaRequest != null)
            {
                if (ThreadObj.sb_OutputXML != null && ThreadObj.sb_OutputXML.Length > 0)
                {
                    ThreadObj.sb_OutputXML.Insert(0, "<FILECONVERTEROUTPUT><OUTPUTLIST>");
                    ThreadObj.sb_OutputXML.Append("</OUTPUTLIST></FILECONVERTEROUTPUT>");
                }

                xpNav_Status = xpNav_MediaRequest.SelectSingleNode("/REQUEST/REQUESTHEADER/REQUESTSTATUS");
                if (xpNav_Status != null)
                {
                    xpNav_Status.InnerXml = Status;
                }

                if (String.Compare(Status, "ERROR", true) == 0)
                {
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST").AppendChildElement("", "ERRORXML", "", null);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/ERRORXML").InnerXml = ThreadObj.ErrorXML;
                }

                if (xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION") != null)
                {
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "APPLICATION", "", ThreadObj.ApplicationName);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "INPUTFILEPATH", "", InputFileName);
                }
                else
                {
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST").AppendChildElement("", "EXTRAINFORMATION", "", null);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "APPLICATION", "", ThreadObj.ApplicationName);
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/EXTRAINFORMATION").AppendChildElement("", "INPUTFILEPATH", "", InputFileName);
                }

                xpNav_MediaRequest.SelectSingleNode("/REQUEST").AppendChildElement("", "OUTPUTXML", "", null);

                if (ThreadObj.sb_OutputXML != null)
                    xpNav_MediaRequest.SelectSingleNode("/REQUEST/OUTPUTXML").InnerXml = ThreadObj.sb_OutputXML.ToString();
            }

        }

        private void AddMediaProcessorLog(String RequestXML, CommonVariables ThreadObj)
        {
            if (!String.IsNullOrEmpty(RequestXML))
            {
                LogInfrastructure LogInfraObj   = new LogInfrastructure();
                LogInfraObj.TenantName          = str_TenantName;
                LogInfraObj.Application         = ThreadObj.ApplicationName;
                LogInfraObj.AddMediaLog(RequestXML, this.MessageArrivalTime, DateTime.Now);
            }
        }

        private void UpdateMediaProcessorLog(String RequestStatus, CommonVariables ThreadObj)
        {
            if (!String.IsNullOrEmpty(ThreadObj.RequestID) && !String.IsNullOrEmpty(RequestStatus))
            {
                LogInfrastructure LogInfraObj   = new LogInfrastructure();
                LogInfraObj.TenantName          = str_TenantName;
                LogInfraObj.Application         = ThreadObj.ApplicationName;
                LogInfraObj.UpdateMediaLogStatus(ThreadObj.RequestID, RequestStatus);
            }
        }

        private void SendResponseToSolution(String str_ResponseMessage, CommonVariables ThreadObj)
        {
            if (String.IsNullOrEmpty(ThreadObj.OutputQueuePath))
                return;

            MSMQWrapper objQueue = new MSMQWrapper(ThreadObj.OutputQueuePath, QueueAccessMode.SendMessage);
            objQueue.AdministrationQueuePath = UndeliveredQueuePath;
            if (ResponseTimeout != 0)
                objQueue.TimeToReachQueue = TimeSpan.FromSeconds(ResponseTimeout);
            else
                objQueue.TimeToReachQueue = new TimeSpan(24, 0, 0);
            ThreadObj.ErrorXML = objQueue.SendMessage("Asset Processor Response", str_ResponseMessage);

            if (ThreadObj.ErrorXML != null)
            {
                if (!String.IsNullOrEmpty(ThreadObj.ClientLogPath))
                    AppendToClientLog("ERROR WHILE RESPONDING : " + ThreadObj.ErrorXML, ThreadObj);
                if (ThreadObj.IsExceptionOccured)
                    UpdateMediaProcessorLog("Error while Responding after exception", ThreadObj);
                else
                    UpdateMediaProcessorLog("Error while Responding", ThreadObj);
            }

        }

        private void ReportErrorInEventViewer(Exception ExceObj, String AdditionalInfo)
        {
            if (EventLogObj == null)
                EventLogObj = new System.Diagnostics.EventLog("SAS MOM");

            StringBuilder sbErrorDetails = new StringBuilder(500);

            sbErrorDetails.AppendFormat("Asset Processor Component Exception:  {0}", ExceObj.Message);
            sbErrorDetails.AppendLine(" Stack Trace: " + ExceObj.StackTrace);
            sbErrorDetails.AppendLine(" AdditionalDetails: " + AdditionalInfo);

            EventLogObj.WriteEntry(sbErrorDetails.ToString(), System.Diagnostics.EventLogEntryType.Error);
        }

        private void ReportErrorInEventViewer(String str_Error, String AdditionalInfo)
        {
            if (EventLogObj == null)
                EventLogObj = new System.Diagnostics.EventLog("SAS MOM");

            StringBuilder sbErrorDetails = new StringBuilder(500);

            sbErrorDetails.AppendFormat("Asset Processor Component Error:  {0}", str_Error);
            sbErrorDetails.AppendLine(" AdditionalDetails: " + AdditionalInfo);

            EventLogObj.WriteEntry(sbErrorDetails.ToString(), System.Diagnostics.EventLogEntryType.Error);
        }
        #endregion
    }

    class CommonVariables
    {
        #region Public member variables
        public StringBuilder        sb_Error         = new StringBuilder(200);
        public StringBuilder        sb_OutputXML     = new StringBuilder(200);
        public StringBuilder        sb_ConsoleOutput = new StringBuilder();
        public String               TenantName       = null;
        public String               OutputQueuePath  = null;
        public String               RequestID        = null;
        public String               ClientLogPath    = null;
        public ERRORXML             ErrorXML         = null;
        public Boolean              ISClientLogAccessible = true;
        public Boolean              IsExceptionOccured = false;        
        public String               ApplicationName  = "DAMCOMPRESSION";        
        #endregion      
    }
}
