#region Using System Include
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.IO;
using System.Collections.Specialized;
using System.Threading;
#endregion

#region Using Assetlink Include
using Assetlink;
using Assetlink.Solutions.Facade.ArtworkProducerFacade;
using Assetlink.WebServices.ALInfrastructureWS;
using Assetlink.WebServices.ALArtworkProducerWS;
using Assetlink.WebServices.ALSearchWS;
using Assetlink.IOProxy;
#endregion

namespace Assetlink.QueueComponents
{
    using XML       = System.String;
    using ERRORXML  = System.String;

    public class BatchArtworkGenerator : ALQueueComponent
    {
        ulong                   UserID                          = 0;
        String                  TenantName                      = null;
        ulong                   SessionID                       = 0;
        String                  Language                        = "English";
        String                  SolutionName                    = "ArtworkProducer";
        const String            SystemUserName                  = "MePhIsToPhElEs";
        const String            SystemUserPwd                   = "ErAtOsThEnEs";
        String                  EncryptedSystemUserName         = null;
        String                  EncryptedSystemUserPwd          = null;
        String                  NTFSJobPath                     = null;
        String                  BatchArtworkGenerationXMLPath   = null;
        ALArtworkProducerWS     ArtworkProducerWSobj            = null;
        XML                     AdditionalInfo                  = null;
        Assetlink.WebServices.ALArtworkProducerWS.ALServiceHeader   RequestHeader   = null;
        const String            CallerIdentity                  = "AL_AP_BatchArtworkGenerator";
 
        public override bool InitializeComponent()
        {
            ERRORXML                        ErrorXML                        = null;
            InfrastructureService           InfrastructureServiceObj        = null;
            Search                          SearchObj                       = null;
            String                          ImpersonationToken              = null;            
            ulong                           AdminSessionID                  = 0;
            ulong                           JobCreatorSessionID             = 0;
            String                          JobCreatorLoginName             = null;
            ulong                           AdminID                         = 0;
            ulong                           JobCreatorID                    = 0;
            StringBuilder                   sb_SearchXML                    = null;
            XML                             TenantListXML                   = null;
            XPathDocument                   TenantListXMLDoc                = null;
            XPathNavigator                  XPathNodeNav                    = null;
            XPathNodeIterator               TenantNodes                     = null;
            XML                             UserXML                         = null;
            XML                             ArtworkJobCollectionXML         = null;
            XPathDocument                   ArtworkJobCollectionXMLDoc      = null;
            XPathNavigator                  ArtworkJobNodeNav               = null;
            XPathNodeIterator               ArtworkJobNodes                 = null;
            StringBuilder                   ErrorMessage                    = null;
            String                          ArtworkJobID                    = null;
            Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader        ALInfrastructureRequestHeader   = null;
           
            try
            {
                ALInfrastructureRequestHeader   = new Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader();
                InfrastructureServiceObj        = new InfrastructureService();    
                InfrastructureServiceObj.Url    = ALCommon.GetWebServiceURL("ALInfrastructureWS");
                InfrastructureServiceObj.ALServiceHeaderValue = ALInfrastructureRequestHeader;
                
                ErrorMessage                    = new StringBuilder(50);
                sb_SearchXML                    = new StringBuilder(100);
                ErrorXML                        = InfrastructureServiceObj.GetAllTenants(ref TenantListXML);

                if (String.IsNullOrEmpty(ErrorXML))
                {
                    TenantListXMLDoc            = new XPathDocument(new StringReader(TenantListXML));
                    XPathNodeNav                = TenantListXMLDoc.CreateNavigator();
                    TenantNodes                 = XPathNodeNav.Select("TenantList/Tenant");
                    while (TenantNodes.MoveNext())
                    {
                        TenantName              = TenantNodes.Current.GetAttribute("Name", "");

                        EncryptedSystemUserName = ALCommon.Encrypt(SystemUserName);
                        EncryptedSystemUserPwd  = ALCommon.Encrypt(SystemUserPwd);
                        ErrorXML                = InfrastructureServiceObj.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, 
                                                   ref ImpersonationToken);

                        ErrorXML                = InfrastructureServiceObj.ImpersonateUser("Admin", TenantName, ImpersonationToken,
                         ALCommon.GetLocalMachineIPAddress(), "INTERNAL", CallerIdentity,ref AdminSessionID, ref UserXML);

                        if (!String.IsNullOrEmpty(ErrorXML))
                        {
                            ErrorMessage.AppendFormat("Error occured while impersonating user as Admin. TenantName: {0}. Error XML: {1}", TenantName, ErrorXML);
                            LogErrorDetails(ErrorMessage.ToString());
                            continue;
                        }

                        AdminID                 = Convert.ToUInt64(ALCommon.GetInnerValue(ref UserXML, "<USERID>", "</USERID>"));
                        sb_SearchXML.Remove(0, sb_SearchXML.Length);
                        sb_SearchXML.AppendFormat(@"<SEARCH>
                                                        <SESSIONINFO TenantName='{0}' Language='{1}' UserID='{2}' SessionID='{3}'/>
                                                        <TARGETOBJECT>ARTWORKJOB</TARGETOBJECT>
                                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                        <TARGETENTITY>
                                                            <FILTERLIST>
                                                                <FILTER>
                                                                    <EXPRESSION>JOBSTATUS</EXPRESSION>
                                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                                    <EXPRESSIONVALUE>In Progress</EXPRESSIONVALUE>
                                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>      
                                                                </FILTER>
                                                                <FILTER>
                                                                    <EXPRESSION>JOBTYPE</EXPRESSION>
                                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                                    <EXPRESSIONVALUE>BATCHARTWORKGENERATION</EXPRESSIONVALUE>
                                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>      
                                                                </FILTER>
                                                            </FILTERLIST>		                                                
                                                            <SELECTLIST>
                                                                <EXPRESSION>
	                                                                <FIELDNAME>JOBID</FIELDNAME>
	                                                                <ALIAS>JOBID</ALIAS>
                                                                </EXPRESSION>
                                                           </SELECTLIST>
                                                        <RELATIONSHIP NAME='ARTWORKJOB_CREATOR'>
                                                            <SELECTLIST>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>USERID</FIELDNAME>
                                                                    <ALIAS>USERID</ALIAS>
                                                                </EXPRESSION>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>LOGINNAME</FIELDNAME>
                                                                    <ALIAS>LOGINNAME</ALIAS>
                                                                </EXPRESSION>
                                                            </SELECTLIST>                                              
                                                    </RELATIONSHIP>
                                                    </TARGETENTITY> 
                                                </SEARCH>
                                                ", TenantName, Language, AdminID, AdminSessionID.ToString());
                        SearchObj               = new Search();
                        SearchObj.Url           = ALCommon.GetWebServiceURL("ALSearchWS");
                        ArtworkJobCollectionXML = null;
                        ErrorXML                = SearchObj.GetHierarchialData(sb_SearchXML.ToString(), ref ArtworkJobCollectionXML);

                        if (ErrorXML != null)
                        {
                            ErrorMessage.AppendFormat("Failed to retrive In-Progress jobs.TenantName:{0}. Error XML: {1}",TenantName, ErrorXML);
                            LogErrorDetails(ErrorMessage.ToString());
                            continue;
                        }
                        else
                        {
                            ArtworkJobCollectionXMLDoc  = new XPathDocument(new StringReader(ArtworkJobCollectionXML));
                            ArtworkJobNodeNav           = ArtworkJobCollectionXMLDoc.CreateNavigator();
                            ArtworkJobNodes             = ArtworkJobNodeNav.Select("ARTWORKJOBCOLLECTION/ARTWORKJOB");
                            while (ArtworkJobNodes.MoveNext())
                            {
                                ArtworkJobID            = ArtworkJobNodes.Current.SelectSingleNode("JOBID").Value;
                                JobCreatorID            = Convert.ToUInt64(ArtworkJobNodes.Current.SelectSingleNode("ARTWORKJOB_CREATOR/USER/USERID").Value);
                                JobCreatorLoginName     = ArtworkJobNodes.Current.SelectSingleNode("ARTWORKJOB_CREATOR/USER/LOGINNAME").Value;
                                ErrorXML                = InfrastructureServiceObj.ImpersonateUser(JobCreatorLoginName, TenantName, ImpersonationToken,
                                                          ALCommon.GetLocalMachineIPAddress(), "INTERNAL", CallerIdentity, ref JobCreatorSessionID, ref UserXML);

                                if (String.IsNullOrEmpty(ErrorXML))
                                {
                                    UserID              = JobCreatorID;
                                    SessionID           = JobCreatorSessionID;
                                    ProcessJob(ArtworkJobID);

                                    SetRequestHeader(ref ALInfrastructureRequestHeader, JobCreatorSessionID, JobCreatorID, Language, TenantName);
                                    ErrorXML            = InfrastructureServiceObj.SignOut();
                                    if (ErrorXML != null)
                                    {
                                        ErrorMessage.AppendFormat("Error occured while signing out(Job owner).TenantName:{0}. ArtworkJobID: {1}. Error XML: {2}", TenantName, ArtworkJobID, ErrorXML);
                                        LogErrorDetails(ErrorMessage.ToString());
                                    }
                                }
                                else
                                {
                                    ErrorMessage.AppendFormat("Error occured while impersonating as job creator.TenantName:{0}. ArtworkJobID: {1}. Error XML: {2}", TenantName, ArtworkJobID, ErrorXML);
                                    LogErrorDetails(ErrorMessage.ToString());
                                }
                            }

                            SetRequestHeader(ref ALInfrastructureRequestHeader, AdminSessionID, AdminID, Language, TenantName);
                            ErrorXML                    = InfrastructureServiceObj.SignOut();
                            if (ErrorXML != null)
                            {
                                ErrorMessage.AppendFormat("Error occured while signing out(Admin session). TenantName:{0}. Error XML: {1}", TenantName, ErrorXML);
                                LogErrorDetails(ErrorMessage.ToString());
                            }
                        }
                    }
                }
                else
                {
                    ReportErrorInEventViewer("Failed to retrieve Tenants.", null, ErrorXML);
                    return false;
                }
                IntitializationRequired = false;
                return true;
            }
            catch (Exception ExceptionObj)
            {
                ReportErrorInEventViewer(null,ExceptionObj, null);
                return false;
            }
        }

        public override void ProcessMessage(String InputXML)
        {
            XPathDocument                   InputXMLDoc             = null;
            XPathNavigator                  XPathNav                = null;
            XPathNavigator                  ConfigNode              = null;
            StringBuilder                   ErrorMessage            = null;
            StringBuilder                   sb_SearchXML            = null;
            String                          ArtworkJobID            = null;            
            try
            {
                ErrorMessage            = new StringBuilder(50);
                sb_SearchXML            = new StringBuilder(100);
                InputXMLDoc             = new XPathDocument(new StringReader(InputXML));
                XPathNav                = InputXMLDoc.CreateNavigator();
                XPathNav                = XPathNav.SelectSingleNode("REQUEST/INPUTXML/INPUT");
                ConfigNode              = XPathNav.SelectSingleNode("ARTWORKJOBID");
                ArtworkJobID            = ConfigNode.Value;
                UserID                  = Convert.ToUInt64(XPathNav.SelectSingleNode("USERID").Value);
                TenantName              = XPathNav.SelectSingleNode("TENANTNAME").Value;
                SessionID               = Convert.ToUInt64(XPathNav.SelectSingleNode("SESSIONID").Value);
                Language                = XPathNav.SelectSingleNode("LANGUAGE").Value;
                SolutionName            = XPathNav.SelectSingleNode("SOLUTIONNAME").Value;

                ValidateSessionAndProcessJob(ArtworkJobID);
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.AppendFormat("Failed to process batch artwork generation request.ArtworkJobID:{0} Exception: {1}", ArtworkJobID, ExceptionObj.Message);
                ReportErrorInEventViewer(null, ExceptionObj, ErrorMessage.ToString());
            }
        }

        private void ProcessJob(String JobID)
        {
            XmlDocument             BatchArtworkGenerationXMLDoc    = null;
            XmlNodeList             ArtworkNodes                    = null;
            String                  XPath                           = null;
            StringBuilder           ErrorMessage                    = null;
            Assetlink.Solutions.Facade.ArtworkProducerFacade.ArtworkProducerFacade APFacadeObj = null;
            Context                 ContextObj                      = null;
            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>AL_AP_BatchArtworkGenerator.BatchArtworkGenerator.ProcessJob</APINAME><JOBID>{0}</JOBID></ADDITIOALINFO>", JobID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj                      = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                APFacadeObj                     = new Assetlink.Solutions.Facade.ArtworkProducerFacade.ArtworkProducerFacade(SessionID.ToString(),UserID.ToString(),TenantName,Language);
                ErrorMessage                    = new StringBuilder(50);
                ArtworkProducerWSobj            = new ALArtworkProducerWS();
                ArtworkProducerWSobj.Url        = ALCommon.GetWebServiceURL("ALArtworkProducerWS");
                RequestHeader                   = new Assetlink.WebServices.ALArtworkProducerWS.ALServiceHeader();
                RequestHeader.SessionID         = SessionID;
                RequestHeader.UserID            = UserID;
                RequestHeader.TenantName        = TenantName;
                RequestHeader.Language          = Language;
                RequestHeader.SolutionName      = SolutionName;
                ArtworkProducerWSobj.ALServiceHeaderValue = RequestHeader;
               
                BatchArtworkGenerationXMLDoc    = new XmlDocument();
                NTFSJobPath                     = APFacadeObj.GetArtworkJobPath(JobID);
                BatchArtworkGenerationXMLPath   = NTFSJobPath + "\\BatchArtworkGeneration.xml";

                // Defensive code added so that even if ArtworkJob not marked as "Done" or "Error". i.e Job is "In Progress"
                // But the NTFS JOB clean up has been done in previous execution cycle. 
                if (!File.Exists(BatchArtworkGenerationXMLPath))
                    return;
                BatchArtworkGenerationXMLDoc.Load(BatchArtworkGenerationXMLPath);

                XPath           = "ARTWORK[not(@FilesTransferred)]";
                ArtworkNodes    = BatchArtworkGenerationXMLDoc.DocumentElement.SelectNodes(XPath);
                foreach (XmlNode ArtworkNode in ArtworkNodes)
                {
                    TransferFileAndUpdateArtwork(ArtworkNode);
                }

                #region NTFS job clean-up
                //Directory.Delete(NTFSJobPath,true);
                SASMOMDirectory.DeleteFolder(NTFSJobPath, true, ContextObj);
                #endregion
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.AppendFormat("Exception occured while processing batch artwork generation request.ArtworkJobID:{0}", JobID);
                ReportErrorInEventViewer(null, ExceptionObj, ErrorMessage.ToString());
            }
        }

        private ERRORXML TransferFileAndUpdateArtwork(XmlNode ArtworkNode)
        {
            ERRORXML                        ErrorXML                            = null;
            XmlDocument                     FileCollectionXMLDoc                = null;
            XmlNode                         FileCollectionNode                  = null;
            XmlNode                         FileNode                            = null;
            XmlNode                         FieldNode                           = null;
            StringBuilder                   FilePath                            = null;
            String                          XPath                               = null;
            String                          UplodedFilesFolderPath              = null;
            String                          FormatID                            = null;
            XmlNodeList                     GraphicElementNodes                 = null;
            XmlNodeList                     TextElementNodes                    = null;
            FileTransferInfrastructure      FileTransferInfrastructureObj       = null;
            XML                             FileCollectionXML                   = null;
            XmlNode                         RootNode                            = null;
            StringBuilder                   SB_TEGECollectionXML                = null;
            String                          TextElementID                       = null;
            String                          GraphicElementID                    = null;
            String                          NewFilePath                         = null;
            String                          ArtworkXMLToUpdate                  = null;
            XmlNode                         ClonedArtworkNode                   = null;
            XmlNode                         ActualValueNode                     = null;
            XmlNode                         DisplayValueNode                    = null;
            String                          DisplayFileName                     = null;
            String                          ArtworkID                           = null;
            String                          GEValue                             = null;
            String                          GEValue_DARelStr                    = null;
            XmlNode                         UploadedFromAttr                    = null; 
            String                          UploadedIn                          = null;

            try
            {
                FileCollectionXMLDoc    = new XmlDocument();
                FilePath                = new StringBuilder(100);
                SB_TEGECollectionXML    = new StringBuilder(255);
                FormatID                = ArtworkNode.SelectSingleNode("FORMATID").InnerText;
                ArtworkID               = ArtworkNode.SelectSingleNode("ARTWORKID").InnerText;
                UplodedFilesFolderPath  = NTFSJobPath + "\\" + FormatID + "\\UploadedFiles\\";

                GraphicElementNodes     = ArtworkNode.SelectNodes("*[@TagType='GRAPHIC' and @ValueType='UPLOAD']");

                #region Transfer files to Webservice machine
                if (GraphicElementNodes.Count > 0)
                {
                    RootNode                = FileCollectionXMLDoc.CreateElement("ROOT");
                    FileCollectionXMLDoc.AppendChild(RootNode);
                    FileCollectionNode      = FileCollectionXMLDoc.CreateElement("FILECOLLECTION");
                    RootNode.AppendChild(FileCollectionNode);

                    foreach (XmlNode GraphicElementNode in GraphicElementNodes)
                    {
                        ActualValueNode     = GraphicElementNode.SelectSingleNode("ACTUALVALUE");
                        DisplayValueNode    = GraphicElementNode.SelectSingleNode("DISPLAYVALUE");

                        if ((ActualValueNode != null) && (!String.IsNullOrEmpty(ActualValueNode.InnerText)))
                        {
                            FilePath.Remove(0, FilePath.Length);
                            UploadedIn              = "BinderFiles\\" ;           
                            if ((UploadedFromAttr   = GraphicElementNode.Attributes.GetNamedItem("UploadedFrom", ""))!=null)
                                if(UploadedFromAttr.Value == "WIDGET")
                                    UploadedIn      = "WidgetFiles\\";

                            FilePath.Append(UplodedFilesFolderPath + UploadedIn + ActualValueNode.InnerText);

                            if (File.Exists(FilePath.ToString()))
                            {
                                FileNode            = FileCollectionXMLDoc.CreateElement("FILE");
                                FileCollectionNode.AppendChild(FileNode);

                                FieldNode           = FileCollectionXMLDoc.CreateElement("NAME");
                                FieldNode.InnerText = ActualValueNode.InnerText;
                                FileNode.AppendChild(FieldNode);

                                if ((DisplayValueNode != null) && (!String.IsNullOrEmpty(DisplayValueNode.InnerText)))
                                {
                                    FieldNode           = FileCollectionXMLDoc.CreateElement("DISPLAYNAME");
                                    FieldNode.InnerText = DisplayValueNode.InnerText;
                                    FileNode.AppendChild(FieldNode);
                                }

                                FieldNode           = FileCollectionXMLDoc.CreateElement("PATH");
                                FieldNode.InnerText = FilePath.ToString();
                                FileNode.AppendChild(FieldNode);
                            }
                            else
                            {
                                AdditionalInfo = @"<DICTIONARY>
                                                        <ITEM>
                                                            <NAME>FILEPATH</NAME>
                                                            <VALUE>" + ALCommon.TrimAndReplaceSpecialChars(FilePath.ToString()) + "</VALUE>" +
                                                       @"</ITEM>
                                                   </DICTIONARY>";
                                ArtworkProducerWSobj.LogArtworkError(ArtworkID, ErrorObjectType.Artwork,"APBAG_ARTWORK_FILE_NOT_FOUND_UNDER_JOBFOLDER",
                                    "File does not exists under job folder.", null, ErrorType.ERROR, AdditionalInfo, true);
                                return "APBAG_ARTWORK_FILE_NOT_FOUND_UNDER_JOBFOLDER";
                            }
                        }
                    }

                    // Transfer files to Webservice machine
                    if (FileCollectionNode.SelectSingleNode("FILE") != null)
                    {
                        FileCollectionXML               = FileCollectionNode.OuterXml;
                        FileTransferInfrastructureObj   = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                        ErrorXML                        = FileTransferInfrastructureObj.SaveFile(ref FileCollectionXML);
                        if (String.IsNullOrEmpty(ErrorXML))
                        {
                            RootNode.InnerXml           = FileCollectionXML;
                            ((XmlElement)ArtworkNode).SetAttribute("FilesTransferred", "Yes");
                            ArtworkNode.OwnerDocument.Save(BatchArtworkGenerationXMLPath);
                        }
                        else
                        {
                            AdditionalInfo = @"<DICTIONARY>
                                                    <ITEM>
                                                        <NAME>FILEPATH</NAME>
                                                        <VALUE>" + ALCommon.TrimAndReplaceSpecialChars(FilePath.ToString()) + "</VALUE>" +
                                                   @"</ITEM>
                                                </DICTIONARY>";
                            ArtworkProducerWSobj.LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APBAG_ARTWORK_FILE_TRANSFER_FAILED",
                                "File transfer failed.", ErrorXML, ErrorType.ERROR, AdditionalInfo, true);
                            return "APBAG_ARTWORK_FILE_TRANSFER_FAILED";
                        }
                    }
                }
                #endregion

                #region Form Text-Graphic element collection xml
                SB_TEGECollectionXML.Append("<PAGE>");
                SB_TEGECollectionXML.Append("<GROUPELEMENT>");

                #region Text Elements
                TextElementNodes = ArtworkNode.SelectNodes("*[@TagType='TEXT']");
                foreach (XmlNode TextElementNode in TextElementNodes)
                {
                    TextElementID   = TextElementNode.Name.Substring(TextElementNode.Name.LastIndexOf('_')+1);
                    ActualValueNode = TextElementNode.SelectSingleNode("ACTUALVALUE");
                    if (ActualValueNode != null)
                    {
                        SB_TEGECollectionXML.AppendFormat(@"<TEXTELEMENT MODIFIED='TRUE'>
                                                            <TEXTELEMENTID>{0}</TEXTELEMENTID>
	                                                        <TEXTELEMENT_TEXTELEMENTVALUE TYPE='RELATION'>
	                                                            <TEXTELEMENTVALUE>
		                                                            <TEXTELEMENTACTUALVALUE>{1}</TEXTELEMENTACTUALVALUE>
	                                                            </TEXTELEMENTVALUE>
                                                            </TEXTELEMENT_TEXTELEMENTVALUE>
		                                                 </TEXTELEMENT>", TextElementID, ActualValueNode.InnerXml);
                    }
                }
                #endregion

                #region Graphic element where user has uploaded files
                foreach (XmlNode GraphicElementNode in GraphicElementNodes)
                {
                    ActualValueNode         = GraphicElementNode.SelectSingleNode("ACTUALVALUE");
                    DisplayValueNode        = GraphicElementNode.SelectSingleNode("DISPLAYVALUE");

                    if ((ActualValueNode!=null)&& (!String.IsNullOrEmpty(ActualValueNode.InnerText)))
                    {
                        GraphicElementID    = GraphicElementNode.Name.Substring(GraphicElementNode.Name.LastIndexOf('_') + 1);
                        XPath               = "FILECOLLECTION/FILE[NAME=" + ALXmlUtil.GetXpathValueString(ActualValueNode.InnerText) + "]/NEWPATH";
                        if (RootNode.SelectSingleNode(XPath)!=null)
                        {
                            NewFilePath         = RootNode.SelectSingleNode(XPath).InnerText;
                            if ((DisplayValueNode != null) && (!String.IsNullOrEmpty(DisplayValueNode.InnerText)))
                                DisplayFileName = DisplayValueNode.InnerText;
                            else
                                DisplayFileName = ActualValueNode.InnerText;

                            SB_TEGECollectionXML.AppendFormat(@"<GRAPHICELEMENT MODIFIED='TRUE'>
			                                                        <GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>
			                                                        <GRAPHICELEMENT_REPLACEDFILE TYPE='RELATION'>
				                                                        <GRAPHICELEMENT>
					                                                        <FILESYSTEM>
						                                                        <FILE>
							                                                        <NAME>{1}</NAME>
							                                                        <DISPLAYNAME>{2}</DISPLAYNAME>
							                                                        <PATH>{3}</PATH>
							                                                        <FIELDNAME>ORIGINALFILE</FIELDNAME>
						                                                        </FILE>
					                                                        </FILESYSTEM>
				                                                        </GRAPHICELEMENT>
			                                                        </GRAPHICELEMENT_REPLACEDFILE>
                                                                    <GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>{4}
			                                                        </GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>
		                                                        </GRAPHICELEMENT>
                                                        ", GraphicElementID, ALCommon.TrimAndReplaceSpecialChars(ActualValueNode.InnerText), 
                                                         ALCommon.TrimAndReplaceSpecialChars(DisplayFileName), ALCommon.TrimAndReplaceSpecialChars(NewFilePath),
                                                         GetGraphicElementValueInfo(GraphicElementNode));
                        }
                    }
                }
                #endregion

                #region Graphic element where user has associated DigitalAsset
                GraphicElementNodes = ArtworkNode.SelectNodes("*[@TagType='GRAPHIC' and @ValueType='ASSET']");
                foreach (XmlNode GraphicElementNode in GraphicElementNodes)
                {
                    ActualValueNode         = GraphicElementNode.SelectSingleNode("ACTUALVALUE");
                    GraphicElementID        = GraphicElementNode.Name.Substring(GraphicElementNode.Name.LastIndexOf('_') + 1);
                    GEValue                 = GetGraphicElementValueInfo(GraphicElementNode);
                    GEValue_DARelStr        = "<GRAPHICELEMENTVALUEINFO_DIGITALASSET TYPE='RELATION'>" + ActualValueNode.InnerText + "</GRAPHICELEMENTVALUEINFO_DIGITALASSET>";
                    GEValue                 = ALCommon.InsertBefore(ref GEValue, "</GRAPHICELEMENTVALUE>", ref GEValue_DARelStr);
                    SB_TEGECollectionXML.AppendFormat(@"<GRAPHICELEMENT MODIFIED='TRUE'>
		                                                    <GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>
                                                            <GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>{1}
		                                                    </GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>
	                                                    </GRAPHICELEMENT>
                                                        ", GraphicElementID, GEValue);
                }
                #endregion

                #region Create GraphicElementValueInfo for Default Graphic Elements
                GraphicElementNodes = ArtworkNode.SelectNodes("*[@TagType='GRAPHIC' and @ValueType='DEFAULT']");
                foreach (XmlNode GraphicElementNode in GraphicElementNodes)
                {
                    GraphicElementID        = GraphicElementNode.Name.Substring(GraphicElementNode.Name.LastIndexOf('_') + 1);
                    GEValue                 = GetGraphicElementValueInfo(GraphicElementNode);
                    SB_TEGECollectionXML.AppendFormat(@"<GRAPHICELEMENT MODIFIED='TRUE'>
		                                                    <GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>
                                                            <GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>{1}
		                                                    </GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>
	                                                    </GRAPHICELEMENT>
                                                        ", GraphicElementID, GEValue);
                }
                #endregion

                SB_TEGECollectionXML.Append("</GROUPELEMENT>");
                SB_TEGECollectionXML.Append("</PAGE>");
                #endregion

                ClonedArtworkNode           = ArtworkNode.CloneNode(true);
                ClonedArtworkNode.InnerXml  = ClonedArtworkNode.InnerXml + SB_TEGECollectionXML.ToString();

                ArtworkXMLToUpdate      = ClonedArtworkNode.OuterXml;
                ErrorXML                = ArtworkProducerWSobj.UpdateTEGEForArtworkAndStampMediaJob(ref ArtworkXMLToUpdate);
                // Error logging not required because it is already done in Engine.
            }
            catch (Exception ExceptionObj)
            {
                AdditionalInfo = @"<DICTIONARY>
                                        <ITEM>
                                            <NAME>APINAME</NAME>
                                            <VALUE>TransferFileAndUpdateArtwork()</VALUE>
                                        </ITEM>
                                        <ITEM>
                                            <NAME>MESSAGE</NAME>
                                            <VALUE>" + ALCommon.TrimAndReplaceSpecialChars(ExceptionObj.Message) + "</VALUE>" +
                                      @"</ITEM>
                                    </DICTIONARY>";
                ArtworkProducerWSobj.LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "AP_BATCH_ARTWORK_GENERATOR_EXCEPTION_OCCURED",
                    "Exception occured.", null, ErrorType.EXCEPTION, AdditionalInfo, true);
            }
            return ErrorXML;
        }

        private void LogErrorDetails(String Message)
        {
            StringBuilder LogFilePath     = null;
            StringBuilder LogDetails      = null;
            FileStream    LogFileStream   = null;
            StreamWriter  LogStreamWriter = null;

            try
            {
                LogFilePath = new StringBuilder(50);
                LogDetails  = new StringBuilder(100);

                LogFilePath.AppendFormat("{0}Logs\\ALQueueManager\\BatchArtworkGenerator\\", ALCommon.GetInstallationDir());
                if (!Directory.Exists(LogFilePath.ToString()))
                    Directory.CreateDirectory(LogFilePath.ToString());

                LogFilePath.AppendFormat("\\{0}_{1}.txt",TenantName, DateTime.Now.ToLongDateString());

                if (File.Exists(LogFilePath.ToString()))
                    LogFileStream   = new FileStream(LogFilePath.ToString(), FileMode.Append);// Append the changes if file exists
                else
                    LogFileStream   = new FileStream(LogFilePath.ToString(), FileMode.Create); // if not present create file

                LogStreamWriter     = new StreamWriter(LogFileStream);
                LogDetails.AppendFormat("{0} .....{1}", Message, DateTime.Now.ToString());
                LogStreamWriter.WriteLine(LogDetails.ToString());
                LogStreamWriter.WriteLine("-----------------------------------------------------------------------------------------------------------------");
              
                LogStreamWriter.Close();
                LogFileStream.Close();
            }
            catch (Exception)
            { 
            }
        }

        private void SetRequestHeader(ref Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader ALInfrastructureRequestHeader,
                                        ulong SessionID, ulong UserID, String Language, String TenantName)
        {
            ALInfrastructureRequestHeader.SessionID     = SessionID;
            ALInfrastructureRequestHeader.UserID        = UserID;
            ALInfrastructureRequestHeader.TenantName    = TenantName;
            ALInfrastructureRequestHeader.Language      = Language;
        }

        private void ValidateSessionAndProcessJob(String ArtworkJobID)
        {
            InfrastructureService   InfrastructureServiceObj                                        = null;
            String                  ImpersonationToken                                              = null;
            ERRORXML                ErrorXML                                                        = null;
            ulong                   AdminID                                                         = 0;
            ulong                   AdminSessionID                                                  = 0;
            ulong                   JobCreatorID                                                    = 0;
            XML                     ArtworkJobCreatorXML                                            = null;
            XML                     UserXML                                                         = null;
            Boolean                 IsSessionValid                                                  = false;  
            StringBuilder           ErrorMessage                                                    = null;
            StringBuilder           sb_SearchXML                                                    = null;  
            Search                  SearchObj                                                       = null;
            String                  JobCreatorLoginName                                             = null;
            ulong                   JobCreatorSessionID                                             = 0;
            Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader ALInfrastructureRequestHeader  = null;

            try
            {
                sb_SearchXML                                    = new StringBuilder(100);
                ErrorMessage                                    = new StringBuilder(30);
                InfrastructureServiceObj                        = new InfrastructureService();
                JobCreatorID                                    = UserID;
                ALInfrastructureRequestHeader                   = new Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader();
                InfrastructureServiceObj.Url                    = ALCommon.GetWebServiceURL("ALInfrastructureWS");
                InfrastructureServiceObj.ALServiceHeaderValue   = ALInfrastructureRequestHeader;
                SetRequestHeader(ref ALInfrastructureRequestHeader, SessionID, UserID, Language, TenantName);

                InfrastructureServiceObj.IsSessionValid(ref IsSessionValid);

                if (IsSessionValid)
                {
                    ProcessJob(ArtworkJobID);
                }
                else
                {
                    EncryptedSystemUserName = ALCommon.Encrypt(SystemUserName);
                    EncryptedSystemUserPwd  = ALCommon.Encrypt(SystemUserPwd);
                    ErrorXML                = InfrastructureServiceObj.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd,
                                               ref ImpersonationToken);

                    ErrorXML                = InfrastructureServiceObj.ImpersonateUser("Admin", TenantName, ImpersonationToken,
                     ALCommon.GetLocalMachineIPAddress(), "INTERNAL", CallerIdentity, ref AdminSessionID, ref UserXML);

                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        ErrorMessage.AppendFormat("Error occured while impersonating as Admin. ArtworkJobID:{0} TenantName: {1}. Error XML: {2}",ArtworkJobID, TenantName, ErrorXML);
                        LogErrorDetails(ErrorMessage.ToString());
                    }

                    AdminID                 = Convert.ToUInt64(ALCommon.GetInnerValue(ref UserXML, "<USERID>", "</USERID>"));
                    sb_SearchXML.Remove(0, sb_SearchXML.Length);
                    sb_SearchXML.AppendFormat(@"<SEARCH>
                                                    <SESSIONINFO TenantName='{0}' Language='{1}' UserID='{2}' SessionID='{3}'/>
                                                    <TARGETOBJECT>USER</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                    <TARGETENTITY>
                                                        <FILTERLIST>
                                                            <FILTER>
                                                                <EXPRESSION>USERID</EXPRESSION>
                                                                <OPERATOR>EQUALTO</OPERATOR>
                                                                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
                                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>      
                                                            </FILTER>
                                                        </FILTERLIST>		                                                
                                                        <SELECTLIST>
                                                            <EXPRESSION>
                                                                <FIELDNAME>USERID</FIELDNAME>
                                                                <ALIAS>USERID</ALIAS>
                                                            </EXPRESSION>
                                                            <EXPRESSION>
                                                                <FIELDNAME>LOGINNAME</FIELDNAME>
                                                                <ALIAS>LOGINNAME</ALIAS>
                                                            </EXPRESSION>
                                                        </SELECTLIST>    
                                                    </TARGETENTITY> 
                                                </SEARCH>
                                                    ", TenantName, Language, AdminID, AdminSessionID, JobCreatorID);
                    SearchObj       = new Search();
                    SearchObj.Url   = ALCommon.GetWebServiceURL("ALSearchWS");
                    ArtworkJobCreatorXML = null;
                    ErrorXML        = SearchObj.GetHierarchialData(sb_SearchXML.ToString(), ref ArtworkJobCreatorXML);

                    if (ErrorXML != null)
                    {
                        ErrorMessage.AppendFormat("Failed to retrive Job Creator. ArtworkJobID:{0} TenantName:{1}. Error XML: {2}", ArtworkJobID, TenantName, ErrorXML);
                        LogErrorDetails(ErrorMessage.ToString());
                    }
                    else
                    {
                        JobCreatorLoginName = ALCommon.GetInnerValue(ref ArtworkJobCreatorXML, "<LOGINNAME>", "</LOGINNAME>");
                        ErrorXML            = InfrastructureServiceObj.ImpersonateUser(JobCreatorLoginName, TenantName, ImpersonationToken,
                                                  ALCommon.GetLocalMachineIPAddress(), "INTERNAL", CallerIdentity, ref JobCreatorSessionID, ref UserXML);

                        if (String.IsNullOrEmpty(ErrorXML))
                        {
                            SessionID       = JobCreatorSessionID;
                            ProcessJob(ArtworkJobID);

                            SetRequestHeader(ref ALInfrastructureRequestHeader, JobCreatorSessionID, UserID, Language, TenantName);
                            ErrorXML        = InfrastructureServiceObj.SignOut();
                            if (ErrorXML != null)
                            {
                                ErrorMessage.AppendFormat("Error occured while signing out(Job Creator). ArtworkJobID:{0} TenantName:{0}. Error XML: {1}",ArtworkJobID, TenantName, ErrorXML);
                                LogErrorDetails(ErrorMessage.ToString());
                            }

                        }
                        else
                        {
                            ErrorMessage.AppendFormat("Error occured while impersonating as job creator.TenantName:{0}. ArtworkJobID: {1}. Error XML: {2}", TenantName, ArtworkJobID, ErrorXML);
                            LogErrorDetails(ErrorMessage.ToString());
                        }

                        SetRequestHeader(ref ALInfrastructureRequestHeader, AdminSessionID, AdminID, Language, TenantName);
                        ErrorXML            = InfrastructureServiceObj.SignOut();
                        if (ErrorXML != null)
                        {
                            ErrorMessage.AppendFormat("Error occured while signing out(Admin session). TenantName:{0}. Error XML: {1}", TenantName, ErrorXML);
                            LogErrorDetails(ErrorMessage.ToString());
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.AppendFormat("Exception occured in ValidateSessionAndProcessJob API. ArtworkJobID:{0}. TenantName:{1}.",ArtworkJobID, TenantName);
                ReportErrorInEventViewer(null, ExceptionObj, ErrorMessage.ToString());
            }
        }

        private XML GetGraphicElementValueInfo(XmlNode GraphicElementNode)
        {
            XmlNode     FieldNode               = null;               
            String      ShowURL                 = String.Empty;
            String      URL                     = String.Empty;
            String      URLTarget               = String.Empty;
            String      URLType                 = String.Empty;
            String      URLProtocol             = String.Empty;
            String      StrGraphicElementValue  = null;

            FieldNode           = GraphicElementNode.SelectSingleNode("EXTRAINFO/SHOWURL");
            if (FieldNode != null)
                ShowURL         = FieldNode.InnerText;

            FieldNode           = GraphicElementNode.SelectSingleNode("EXTRAINFO/URL");
            if (FieldNode != null)
                URL             = FieldNode.InnerText;

            FieldNode           = GraphicElementNode.SelectSingleNode("EXTRAINFO/URLTARGET");
            if (FieldNode != null)
                URLTarget       = FieldNode.InnerText;

            FieldNode           = GraphicElementNode.SelectSingleNode("EXTRAINFO/URLTYPE");
            if (FieldNode != null)
                URLType         = FieldNode.InnerText;

            FieldNode           = GraphicElementNode.SelectSingleNode("EXTRAINFO/URLPROTOCOL");
            if (FieldNode != null)
                URLProtocol     = FieldNode.InnerText;


            StrGraphicElementValue = @"<GRAPHICELEMENTVALUE>
                                            <SHOWURL>" + ShowURL + @"</SHOWURL>
                                            <URL>" + URL + @"</URL>
                                            <URLTARGET>" + URLTarget + @"</URLTARGET>
                                            <URLTYPE>" + URLType + @"</URLTYPE>
                                            <URLPROTOCOL>" + URLProtocol + @"</URLPROTOCOL>
                                       </GRAPHICELEMENTVALUE>";

            return StrGraphicElementValue;
        }

        private void ReportErrorInEventViewer(String ErrorMessage, Exception ExceObj, String AdditionalInfo)
        {
            if (EventLogObj == null)
                EventLogObj = new System.Diagnostics.EventLog("SAS MOM");

            StringBuilder sbErrorDetails = new StringBuilder(500);

            if (ExceObj != null)
            {
                sbErrorDetails.AppendFormat("Batch Artwork Generator Component : Message : {0}", ExceObj.Message);
                sbErrorDetails.AppendLine(" Stack Trace : " + ExceObj.StackTrace);
                sbErrorDetails.AppendLine(" AdditionalDetails : " + AdditionalInfo);
            }
            else
            {
                sbErrorDetails.AppendFormat("Batch Artwork Generator Component : Message : {0}", ErrorMessage);
                sbErrorDetails.AppendLine(" AdditionalDetails : " + AdditionalInfo);
            }

            EventLogObj.WriteEntry(sbErrorDetails.ToString(), System.Diagnostics.EventLogEntryType.Error);
        }
    }
}
