using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.XPath;
using System.IO;
using Assetlink.IOProxy; 

//--------------------------------------------//
// Author:      Mr. Hemant Karkande.          //
// Class:       AL_WorkbenchComponent         //
// Created On:  24th November 2009            //  
// Purpose: This Q component will be used on  //
//          the solution side and will be     //  
//          invoked from the Process Manager  //
//--------------------------------------------//

namespace Assetlink.QueueComponents
{
    using XML       = String;
    using ERRORXML  = String;

    public class AL_WorkbenchComponent : ALQueueComponent
    {
        ulong       ul_UserID;
        ulong       ul_SessionID;
        String      str_TenantName          = null;
        String      str_UserID              = null;
        String      str_Language            = null;
        String      str_SessionID           = null;
        String      str_SolutionName        = null; 

        #region Public Methods
            public override void ProcessMessage(String ProcessMessageXML)
            {
                XPathDocument       xpDocMessageDoc        = null;
                XPathNavigator      xpNavMessage           = null;
                XPathNavigator      xpNav_Request          = null;
                XPathNavigator      xpNav_InputXML         = null;
                try
                {
                    xpDocMessageDoc = new XPathDocument(new StringReader(ProcessMessageXML));
                    xpNavMessage    = xpDocMessageDoc.CreateNavigator();
                    xpNav_Request   = xpNavMessage.SelectSingleNode("REQUEST");
                    str_TenantName  = xpNav_Request.SelectSingleNode("REQUESTHEADER/TENANTNAME").Value;
                    str_UserID      = xpNav_Request.SelectSingleNode("REQUESTHEADER/USERID").Value;
                    ul_UserID       = ulong.Parse(str_UserID);
                    str_SessionID   = xpNav_Request.SelectSingleNode("REQUESTHEADER/SESSIONID").Value;
                    ul_SessionID    = ulong.Parse(str_SessionID);
                    str_Language    = xpNav_Request.SelectSingleNode("REQUESTHEADER/LANGUAGE").Value;
                    str_SolutionName= xpNav_Request.SelectSingleNode("REQUESTHEADER/SOLUTIONNAME").Value;
                    xpNav_InputXML  = xpNav_Request.SelectSingleNode("INPUTXML");

                    ProcesInputXML(xpNav_InputXML);
                }
                catch (Exception ExcepObj)
                {
                    LogErrorEvent("EXCEPTION", ExcepObj.Message, xpNav_InputXML.OuterXml, ExcepObj.StackTrace);
                }
            }
        #endregion

        #region Private Methods
            private void ProcesInputXML(XPathNavigator xpNav_InputXML)
            {
                String      str_ProcessID       = null;
                String      str_Event           = null;
                String      str_WebDAVLocation  = null; 
                ERRORXML    ErrorXml            = null; 
     
                try
                {
                    str_Event           = xpNav_InputXML.SelectSingleNode("EVENT").Value;

                    switch (str_Event.ToUpper())
                    {
                        case "CLEAR_WEBDAV_LOCATION":
                            str_ProcessID       = xpNav_InputXML.SelectSingleNode("PROCESSID").Value;
                            str_WebDAVLocation  = xpNav_InputXML.SelectSingleNode("WEBDAVLOCATION").Value;   
                            ClearWebDAVLocation(str_ProcessID, str_WebDAVLocation); 
                            break;
                    }
                    if (ErrorXml != null)
                    {
                        LogErrorEvent("ERROR", ErrorXml, xpNav_InputXML.OuterXml, null);
                    }
                }
                catch (Exception ExceptionObj)
                {
                    LogErrorEvent("EXCEPTION", ExceptionObj.Message, xpNav_InputXML.OuterXml, ExceptionObj.StackTrace);
                }
            }

            /// <summary>
            /// <CreatedOn>24 November 2009</CreatedOn>
            /// <Author>Mr. Hemant Karkande</Author>
            /// <Purpose>To clear WebDAV and fdf file location for approvals Task and Job</Purpose>
            /// </summary>
            /// <param name="str_ProcessID">Process ID to clear fdf file path</param>
            /// <param name="str_WebDAVLocation">WebDAV Location to clear</param> 
            private void ClearWebDAVLocation(String str_ProcessID, String str_WebDAVLocation)
            {
                String          str_MD5Name             = null;
                String          str_WebDAVPhysicalPath  = null;
                String          str_FDFPath             = null; 
                StringBuilder   sb_AssetlinkXMLPath     = null; 
                XPathDocument   xp_AssetlinkDoc         = null;
                XPathNavigator  nav_WebDAVLocation      = null; 
                Context         ContextObj              = null;
                StringBuilder   sb_AdditionalInfo       = null;
                try
                {
                    sb_AdditionalInfo = new StringBuilder(100);
                    
                    if ((!String.IsNullOrEmpty(str_WebDAVLocation)) && (File.Exists(str_WebDAVLocation)))
                    {
                        sb_AdditionalInfo.Append(@"<ADDITIONALINFO>
                                                        <APINAME>AL_WorkbenchComponent.ClearWebDAVLocation()</APINAME>
                                                        <THREADID>" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @"</THREADID>
                                                         <WORKFLOWPROCESSID>" + str_ProcessID + @"</WORKFLOWPROCESSID>
                                                        <WEBDAVLOCATION>" + str_WebDAVLocation + "</WEBDAVLOCATION>"
                                                   + "</ADDITIONALINFO>");
                        ContextObj = new Context(ul_UserID, ul_SessionID, str_TenantName, str_SolutionName, sb_AdditionalInfo.ToString());    
                        str_MD5Name = Path.GetDirectoryName(str_WebDAVLocation);

                        //Directory.Delete(str_MD5Name, true);
                        // 13 July 2013. Will Now use common Directory.Delete due to defect S0991814 
                        SASMOMDirectory.DeleteFolder(str_MD5Name, true, ContextObj);
                        sb_AssetlinkXMLPath = new StringBuilder(25);
                        sb_AssetlinkXMLPath.Append(ALCommon.GetConfigurationXMLPath() + "\\Assetlink.xml");

                        xp_AssetlinkDoc = new XPathDocument(sb_AssetlinkXMLPath.ToString());
                        nav_WebDAVLocation = xp_AssetlinkDoc.CreateNavigator().SelectSingleNode("Assetlink/Settings/add[@name=\"WEBDAVLOCATION\"]");
                        if (nav_WebDAVLocation != null)
                        {
                            str_WebDAVPhysicalPath = nav_WebDAVLocation.GetAttribute("value", "");
                        }
                        str_FDFPath = str_WebDAVPhysicalPath + "\\" + str_TenantName + "\\" + str_ProcessID;
                        if (Directory.Exists(str_FDFPath))
                        {
                            //Directory.Delete(str_FDFPath, true);
                            // 13 July 2013. Will Now use common Directory.Delete due to defect S0991814 
                            SASMOMDirectory.DeleteFolder(str_FDFPath, true, ContextObj);
                        }
                    }
                }
                catch (Exception ExceptionObj)
                {
                    LogErrorEvent("EXCEPTION", ExceptionObj.Message, str_ProcessID, ExceptionObj.StackTrace);
                }
            }

            private void LogErrorEvent(String Type, String ErrorDetails, XML InputXML, String StackTrace)
            {
                try
                {
                    if (EventLogObj == null)
                        EventLogObj = new System.Diagnostics.EventLog("SAS MOM");                
                    StringBuilder sbError = new StringBuilder(50);

                    sbError.AppendFormat("AL_WorkbenchComponent : InputXML \r:- {0} \r\n", InputXML);

                    sbError.AppendFormat("Message :- {0} \r\n", ErrorDetails);
                    if (!String.IsNullOrEmpty(StackTrace))
                        sbError.AppendFormat("Stack Trace :- {0} \r\n", StackTrace);

                    EventLogObj.WriteEntry(sbError.ToString(), System.Diagnostics.EventLogEntryType.Error);
                }
                catch { }
            }
        #endregion
    }
}
