using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Assetlink;
using System.Xml;
using System.Xml.XPath;
using System.Configuration;
using Assetlink.IOProxy;
namespace Assetlink.ReplicationAgent
{
    public class ReplicationAgent:ALQueueComponent
    {
        public override bool InitializeComponent()
        {
            return true;
        }

        public override void ProcessMessage(String MediaRequestXML)
        {
            String         SourcePath           = null;
            String         DestinationPath      = null;
            String         ChangeName           = null;
            String         ChangeElementName    = null;
            String         EventType            = null;
            String         strOldPath           = null;
            XmlDocument    xInputDoc            = null;

            try
            {
                xInputDoc = new XmlDocument();
                xInputDoc.LoadXml(MediaRequestXML);

                EventType         = xInputDoc.SelectSingleNode("REQUEST/INPUTXML/EVENTTYPE").InnerText;
                DestinationPath   = xInputDoc.SelectSingleNode("REQUEST/INPUTXML/REPLICATIONPATH").InnerText;
                SourcePath        = xInputDoc.SelectSingleNode("REQUEST/INPUTXML/REPLICATIONSERVERTEMPPATH").InnerText;

                ChangeName        = xInputDoc.SelectSingleNode("REQUEST/INPUTXML/ACTUALNAME").InnerText;
                ChangeElementName = xInputDoc.SelectSingleNode("REQUEST/INPUTXML/CHANGEELEMENT").InnerText;
                if (xInputDoc.SelectSingleNode("REQUEST/INPUTXML/OLDPATH") != null)
                    strOldPath = xInputDoc.SelectSingleNode("REQUEST/INPUTXML/OLDPATH").InnerText;

                LogDetails("Start", EventType, SourcePath, strOldPath);
                updateNTFS(EventType, ChangeElementName, SourcePath, DestinationPath, strOldPath, ChangeName);
                LogDetails("End", EventType, SourcePath, strOldPath);

            }
            catch (Exception objException)
            {
                // Log the Exception
                LogDetails("Exception in ProcessMessage of Replication Agent Response Handler...." + objException.Message, "", "", "ReplicationAgentResponseExceptionLog");
            }
        }

        public override void StopApplication()
        {
            base.StopApplication();
        }

        #region Helper functions

        private void updateNTFS(String EventType, String ChangeElementName, String SourcePath, String DestinationPath, String strOldPath, String ActualName)
        {
            String ErrorXML = null;
            String  Lock    = "d7f2460f-1ab9-4771-ae30-e164b390972e"; // ID get from System.Guid.NewGuid().ToString()

            Context         ContextObj        = null;
            StringBuilder   sb_AdditionalInfo = null;

            sb_AdditionalInfo = new StringBuilder(100);
            sb_AdditionalInfo.Append(@"<ADDITIONALINFO>
                                        <THREADID>" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @"</THREADID>
                                        <APINAME>ReplicationAgent.updateNTFS()</APINAME>
                                        <EVENTTYPE>" + EventType + @"</EVENTTYPE>
                                        <CHANGEELEMENTNAME>" + ChangeElementName + @"</CHANGEELEMENTNAME>
                                        <SOURCEPATH>" + SourcePath + @"</SOURCEPATH>
                                        <DESTINATIONPATH>" + DestinationPath + @"</DESTINATIONPATH>
                                        <OLDPATH>" + strOldPath + @"</OLDPATH>
                                        <ACTUALNAME>" + ActualName + @"</ACTUALNAME>
                                      </ADDITIONALINFO>");

            ContextObj = new Context(0, 0, "", "ReplicationAgent", sb_AdditionalInfo.ToString());
            switch (EventType.ToUpper())
            { 
                case "CREATED":
                    if (ChangeElementName.ToUpper() == "DIRECTORY")
                    {
                        Directory.CreateDirectory(DestinationPath + ".bkp"); // this create blank directory
                        Directory.CreateDirectory(DestinationPath);// directory created
                        //Directory.Delete(DestinationPath + ".bkp"); // delete the created directory
                        // 13 July 2013. Use Common Delete Folder API.
                        SASMOMDirectory.DeleteFolder(DestinationPath + ".bkp", ContextObj);
                    }
                    else
                    {
                        if (File.Exists(DestinationPath + ".tmp"))
                            File.Delete(DestinationPath + ".tmp");

                        System.Threading.Monitor.Enter(Lock); // critical section 
                        try
                        {
                            ErrorXML = getFileFromReplicationServer(SourcePath, DestinationPath, ActualName); // get file into .bkp

                            if (String.IsNullOrEmpty(ErrorXML))
                            {
                                if (File.Exists(DestinationPath + ".bkp"))
                                {
                                    if (File.Exists(DestinationPath)) // in case of create DestinationPath might not present on NTFS
                                        File.Move(DestinationPath, DestinationPath + ".tmp"); // Move original file to tmp
                                    File.Move(DestinationPath + ".bkp", DestinationPath); // Move bkp to original file
                                }
                                else
                                {
                                    LogDetails("Failed to generate File :" + DestinationPath + ".bkp", EventType, SourcePath, strOldPath);
                                }

                                if (File.Exists(DestinationPath + ".bkp"))
                                    File.Delete(DestinationPath + ".bkp"); // delte bkp
                                if (File.Exists(DestinationPath + ".tmp"))
                                    File.Delete(DestinationPath + ".tmp"); // delte tmp
                            }
                            else
                                LogDetails("Error in MTOM GetFile.." + ErrorXML, EventType, SourcePath, strOldPath);
                        }
                        catch (Exception objException)
                        {
                            LogDetails("Exception.." + objException.Message, EventType, SourcePath, strOldPath);
                        }
                        finally
                        {
                            System.Threading.Monitor.Exit(Lock);
                        }
                    }
                    break;
                case "CHANGED":
                    if (ChangeElementName.ToUpper() == "DIRECTORY")
                    {// directory created
                    }
                    else
                    {
                        if (File.Exists(DestinationPath + ".tmp"))
                            File.Delete(DestinationPath + ".tmp");

                       System.Threading.Monitor.Enter(Lock); // critical section 

                        try
                        {
                            ErrorXML = getFileFromReplicationServer(SourcePath, DestinationPath, ActualName); // get file into .bkp

                            if (String.IsNullOrEmpty(ErrorXML))
                            {
                                if (File.Exists(DestinationPath) && File.Exists(DestinationPath + ".bkp"))
                                {
                                    File.Move(DestinationPath, DestinationPath + ".tmp"); // Move original file to tmp
                                    File.Move(DestinationPath + ".bkp", DestinationPath); // Move bkp to original file
                                }
                                else
                                {
                                    if (!File.Exists(DestinationPath))
                                        LogDetails("File Not Present.." + DestinationPath, EventType, SourcePath, strOldPath);
                                    else if (!File.Exists(DestinationPath + ".bkp"))
                                        LogDetails("Failed to generate File :" + DestinationPath + ".bkp", EventType, SourcePath, strOldPath);
                                }

                                if (File.Exists(DestinationPath + ".bkp"))
                                    File.Delete(DestinationPath + ".bkp"); // delte bkp
                                if (File.Exists(DestinationPath + ".tmp"))
                                    File.Delete(DestinationPath + ".tmp"); // delte tmp
                            }
                            else
                                LogDetails("Error in MTOM GetFile.." + ErrorXML, EventType, SourcePath, strOldPath);
                        }
                        catch (Exception objException)
                        {
                            LogDetails("Exception.." + objException.Message, EventType, SourcePath, strOldPath);
                        }
                        finally
                        {
                            System.Threading.Monitor.Exit(Lock);
                        }
                    }
                    break;
                case "RENAMED":
                    if (ChangeElementName.ToUpper() == "DIRECTORY")
                        Directory.Move(strOldPath, DestinationPath);// directory renamed
                    else
                        File.Move(strOldPath, DestinationPath);// file remaned
                    break;
                case "DELETED":
                    // changeelement will not be proper as from source location it was deleted, so check on destination
                    if (Directory.Exists(DestinationPath))
                    {
                        // 13 July 2013. Use the Common Delete Folder API.s
                        //Directory.Delete(DestinationPath, true);// directory created
                        SASMOMDirectory.DeleteFolder(DestinationPath, true, ContextObj);
                    }
                    else
                    {
                        if (File.Exists(DestinationPath))
                        {
                            File.Create(DestinationPath + ".bkp").Close();// create blank file
                            File.Delete(DestinationPath);// file deleted
                        }
                    }
                    break;
            }
            
        }

        private String getFileFromReplicationServer(String PathonServer, String DestinationPath, String ActualName)
        {
            FileTransferInfrastructure      objFileTransfer     = null;
            StringBuilder                   FileCollectionXML   = null;
            String                          FileListXML         = null;
            String                          ErrorXML            = null;

            FileCollectionXML   = new StringBuilder(100);
            objFileTransfer     = new FileTransferInfrastructure(null, null, null, null);// no need of Tenent Name Session ID, UserID, Language
            objFileTransfer.FileTransferServiceName = ConfigurationManager.AppSettings["ReplicationServerMachineFileTransferUrl"].ToString();

            FileCollectionXML.Append("<FILECOLLECTION>");
            FileCollectionXML.Append("<FILE>");
            FileCollectionXML.Append("<NAME>$FILENAME$</NAME>");
            FileCollectionXML.Append("<PATH>$SOURCEPATH$</PATH>");
            FileCollectionXML.Append("<DESTINATIONPATH>$DESTPATH$.bkp</DESTINATIONPATH>");
            FileCollectionXML.Append("</FILE>");
            FileCollectionXML.Append("</FILECOLLECTION>");

            FileListXML     = FileCollectionXML.Replace("$FILENAME$",ActualName).Replace("$SOURCEPATH$",PathonServer).Replace("$DESTPATH$",DestinationPath).ToString();
            ErrorXML        = objFileTransfer.GetFile(ref FileListXML); // transfer file on Replication Server
            return ErrorXML;
        }


        private void LogDetails(String Message, String Opertation, String FilePath, String OldPath)
        {
            StringBuilder LogFilePath     = null;
            StringBuilder LogDetails      = null;
            FileStream    LogFileStream   = null;
            StreamWriter  LogStreamWriter = null;

            try
            {
                LogFilePath = new StringBuilder(50);
                LogDetails  = new StringBuilder(100);


                LogFilePath.AppendFormat("{0}Logs\\Replication", ALCommon.GetInstallationDir());
                if (!Directory.Exists(LogFilePath.ToString()))
                    Directory.CreateDirectory(LogFilePath.ToString());

                LogFilePath.AppendFormat("\\{0}_ReplicationAgentResponseHandlerLog.txt", DateTime.Now.ToLongDateString());

                if (File.Exists(LogFilePath.ToString()))
                    LogFileStream = new FileStream(LogFilePath.ToString(), FileMode.Append);// Append the changes if file exists
                else
                    LogFileStream = new FileStream(LogFilePath.ToString(), FileMode.Create); // if not present create file

                LogStreamWriter = new StreamWriter(LogFileStream);
                if (Opertation.ToUpper() == "RENAMED")
                    LogDetails.AppendFormat("{0} .....{1}        {2}         {3}     to    {4}", Message, DateTime.Now.ToString(), Opertation, OldPath, FilePath);
                else
                    LogDetails.AppendFormat("{0} .....{1}        {2}         {3}", Message, DateTime.Now.ToString(), Opertation, FilePath);
                LogStreamWriter.WriteLine(LogDetails.ToString());
                LogStreamWriter.WriteLine("-----------------------------------------------------------------------------------------------------------------");
              
                LogStreamWriter.Close();
                LogFileStream.Close();
            }
            catch (Exception)
            { 
            }
        }
        #endregion
    }
}
