#region Standard Include
using System;
using System.Text;
using System.Xml;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Threading;
using System.ServiceProcess;
using System.Diagnostics;
using System.Xml.XPath;
# endregion

#region Assetlink Include
using Assetlink.Services.WindowsService;
using Assetlink.WebServices.ALDAMWS;
using Assetlink.WebServices.ALInfrastructureWS;
#endregion


namespace Assetlink.Services.WindowsService.DAMJobCopyService
{
    using XML       = System.String;
    using ERRORXML  = System.String;
    using ENTITYID  = System.UInt64;
 
    public struct TenantInfo
    {
        public String ExternalMountPoint;
        public String MountPoint;
        public String TenantName;
        public String AssetInfoRetryCount;
        public String AssetInfoRetryInterval;
        public String ServiceName;
    }

    public class FWProcessor
    {
         const ulong            SystemUserID            = 111111122211111;
         const String           SystemUserName          = "MePhIsToPhElEs";
         const String           SystemUserPwd           = "ErAtOsThEnEs";
         public String          ServiceName             = null;
         String                 EncryptedSystemUserName = null;
         String                 EncryptedSystemUserPwd  = null;
         ERRORXML               ErrorXML                = null;
         TenantInfo             TenantInfoObj;
         public Boolean         IsStopped               = false;                  
         ulong                  SessionID               = 0;
         String                 ImpersonationToken      = null;
         ALDAMWS                DAMWSObj                = null;
         InfrastructureService  InfraWSObj              = null;
         ServiceController      ServiceControllerObj    = null;
         public EventLog        EventLogObj             = null;

        global::Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader   RequestHeader2    = null;
        global::Assetlink.WebServices.ALDAMWS.ALServiceHeader              RequestHeader1    = null;
        
        public FWProcessor()
        {
            RequestHeader1                  = new global::Assetlink.WebServices.ALDAMWS.ALServiceHeader();
            DAMWSObj                        = new ALDAMWS();
            DAMWSObj.ALServiceHeaderValue   = RequestHeader1;
            DAMWSObj.Url                    = ConfigurationManager.AppSettings["DAMWSURL"];

            RequestHeader2                  = new global::Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader();
            InfraWSObj                      = new InfrastructureService();
            InfraWSObj.ALServiceHeaderValue = RequestHeader2;
            InfraWSObj.Url                  = ConfigurationManager.AppSettings["ALInfrastructureWS"];
        }

        public void Start()
        {
            Thread MainThread               = new Thread(new ThreadStart(this.StartProcessingDAMJobCopy));
            MainThread.Name                 = "MainThread";
            ServiceControllerObj            = new ServiceController(ServiceName);
            MainThread.Start();
        }

        

        #region Private Methods

        private void StartProcessingDAMJobCopy()
        {   
            XML                     ExternalMountPointXML               = null;
            XPathDocument           xpDoc_ExternalMountPoints           = null;
            XPathNavigator          xpNav_ExternalMountPoint            = null;
            XPathNodeIterator       xpItr_ExternalMountPoint            = null;            
            String                  TenantName                          = null;
            String                  MountPoint                          = null;
            String                  ExternalMountPoint                  = null;
            FWProcessor             FWProcessorObj                      = null;
            FileSystemWatcher       FileSystemWatcherObj                = null;
            StringBuilder           ErrorMessage                        = null;
            bool                    f_AtleastOneTenantFound             = false;
            XPathDocument           FileNotificationCollectionXMLDoc    = null;
            XML                     FileNotificationCollectionXML       = null;
            XPathNavigator          XPathNav                            = null;
            XPathNodeIterator       UnProcessedJobNav                   = null;
            StringBuilder           AssetinfoFilePath                   = null;
            String                  OriginalFolderPath                  = null;
            StringBuilder           XPath                               = null;
            ProcessFileNotification ProcessFileNotificationObj          = null;
            String                  JobID                               = null;
            Thread                  ThreadObj                           = null;
            
            try
            {                
                ErrorMessage            = new StringBuilder();
                XPath                   = new StringBuilder(20);
                AssetinfoFilePath       = new StringBuilder(50);    
                EncryptedSystemUserName = ALCommon.Encrypt(SystemUserName);
                EncryptedSystemUserPwd  = ALCommon.Encrypt(SystemUserPwd);
                ErrorXML                = Infrastructure.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, ref ImpersonationToken);
                
                if (ErrorXML == null)
                {        
                    SetALServiceHeaderValue(SessionID, SystemUserID, "Assetlink", "English");                    
                    ErrorXML            = GetAllExternalMoutPoints(out ExternalMountPointXML );
                    if (ErrorXML == null)
                    {
                        #region Get all Un-Processed jobs
                        ErrorXML = DAMWSObj.GetUnProcessedJobs("DAMJOBCOPY",out FileNotificationCollectionXML);
                        if (ErrorXML == null)
                        {
                            FileNotificationCollectionXMLDoc = new XPathDocument(new StringReader(FileNotificationCollectionXML));
                            XPathNav = FileNotificationCollectionXMLDoc.CreateNavigator();
                        }
                        else
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.Append("Failed to retrieve unprocessed jobs. ErrorXML: " + ErrorXML);
                            EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);
                        }
                        #endregion

                        xpDoc_ExternalMountPoints = new XPathDocument(new StringReader(ExternalMountPointXML));
                        xpNav_ExternalMountPoint  = xpDoc_ExternalMountPoints.CreateNavigator();

                        xpItr_ExternalMountPoint  = xpNav_ExternalMountPoint.Select("EXTERNALMOUNTPOINTCOLLECTION/EXTERNALMOUNTPOINT");
                        while (xpItr_ExternalMountPoint.MoveNext())
                        {
                            xpNav_ExternalMountPoint = xpItr_ExternalMountPoint.Current;
                            if (xpNav_ExternalMountPoint.SelectSingleNode("APPLICATION").Value.ToUpper() == "CATALOGER")
                            {
                                TenantName = xpNav_ExternalMountPoint.SelectSingleNode("TENANTNAME").Value;
                                if (xpNav_ExternalMountPoint.SelectSingleNode("EXTERNALCATALOGERMOUNTPOINT") != null)
                                {
                                    ExternalMountPoint = xpNav_ExternalMountPoint.SelectSingleNode("EXTERNALCATALOGERMOUNTPOINT").Value;
                                    if (!String.IsNullOrEmpty(ExternalMountPoint))                                  
                                    {                                        
                                        ErrorXML = CreateExternalMountPoint(ExternalMountPoint);
                                        if (ErrorXML == null)
                                        {
                                            if (xpNav_ExternalMountPoint.SelectSingleNode("MOUNTPOINT") != null)
                                            {
                                                MountPoint = xpNav_ExternalMountPoint.SelectSingleNode("MOUNTPOINT").Value;
                                                if (String.IsNullOrEmpty(MountPoint))
                                                {
                                                    continue;                                                    
                                                }
                                                else
                                                {
                                                    f_AtleastOneTenantFound = true;
                                                    FileSystemWatcherObj    = new FileSystemWatcher();
                                                    FWProcessorObj          = new FWProcessor();
                                                    FWProcessorObj.TenantInfoObj.ExternalMountPoint = ExternalMountPoint;
                                                    FWProcessorObj.TenantInfoObj.MountPoint         = MountPoint;
                                                    FWProcessorObj.TenantInfoObj.TenantName         = TenantName;
                                                    FWProcessorObj.TenantInfoObj.ServiceName        = EventLogObj.Source;

                                                    if (xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYCOUNT") != null)
                                                        FWProcessorObj.TenantInfoObj.AssetInfoRetryCount    = xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYCOUNT").Value;
                                                    if (xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYINTERVAL") != null)
                                                        FWProcessorObj.TenantInfoObj.AssetInfoRetryInterval = xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYINTERVAL").Value;

                                                    MonitorFileSystem(FileSystemWatcherObj, FWProcessorObj);

                                                    #region Spawn thread for processing unprocessed jobs

                                                    if (XPathNav != null)
                                                    {
                                                        XPath.Remove(0, XPath.Length);
                                                        XPath.AppendFormat("FILENOTIFICATIONCOLLECTION/FILENOTIFICATION[translate(@TenantName, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')='{0}']", TenantName.ToLower());
                                                        UnProcessedJobNav = XPathNav.Select(XPath.ToString());
                                                        while (UnProcessedJobNav.MoveNext())
                                                        {
                                                            AssetinfoFilePath.Remove(0, AssetinfoFilePath.Length);
                                                            JobID               = UnProcessedJobNav.Current.SelectSingleNode("NOTIFICATIONID").Value;
                                                            OriginalFolderPath  = UnProcessedJobNav.Current.SelectSingleNode("ORIGINALPATH").Value;
                                                            if ((!String.IsNullOrEmpty(OriginalFolderPath)) && ((OriginalFolderPath.IndexOf("Originals", StringComparison.CurrentCultureIgnoreCase)) >= 0))
                                                            {
                                                                FWProcessorObj  = new FWProcessor();
                                                                AssetinfoFilePath.AppendFormat("{0}Assetinfo", OriginalFolderPath.Substring(0, OriginalFolderPath.LastIndexOf("Originals", StringComparison.CurrentCultureIgnoreCase)));

                                                                FWProcessorObj.TenantInfoObj.ExternalMountPoint = ExternalMountPoint;
                                                                FWProcessorObj.TenantInfoObj.MountPoint         = MountPoint;
                                                                FWProcessorObj.TenantInfoObj.TenantName         = TenantName;
                                                                FWProcessorObj.TenantInfoObj.ServiceName        = EventLogObj.Source;

                                                                if (xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYCOUNT") != null)
                                                                    FWProcessorObj.TenantInfoObj.AssetInfoRetryCount    = xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYCOUNT").Value;
                                                                if (xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYINTERVAL") != null)
                                                                    FWProcessorObj.TenantInfoObj.AssetInfoRetryInterval = xpNav_ExternalMountPoint.SelectSingleNode("ASSETINFORETRYINTERVAL").Value;

                                                                ProcessFileNotificationObj = new ProcessFileNotification(FWProcessorObj.TenantInfoObj, AssetinfoFilePath.ToString(), JobID);
                                                                ThreadObj = new Thread(new ThreadStart(ProcessFileNotificationObj.ProcessNewJob));
                                                                ThreadObj.Start();
                                                            }
                                                        }
                                                    }

                                                    #endregion
                                                }
                                            }
                                            else
                                            {
                                                ErrorMessage.Remove(0, ErrorMessage.Length);
                                                ErrorMessage.AppendFormat("FileWatcher mount point not specified for {0}.", TenantName);
                                                EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);
                                                ServiceControllerObj.Stop();
                                                return;
                                            }
                                        }
                                    }
                                }                              
                            }
                        }

                        if (!f_AtleastOneTenantFound)
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.Append("None of the Tenant has External Mount Point or File Watcher Mount point configured. Hence stopping the service.");
                            EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);
                            ServiceControllerObj.Stop();
                            return;
                        }
                    }
                    else
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Failed to retrieve External mount points.");
                        EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);
                        ServiceControllerObj.Stop();
                        return;
                    }
                }
                else
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("System connect failed.");
                    EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);
                    ServiceControllerObj.Stop();
                    return;
                }
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown by DAMJobCopy. Message : {0}.", ExceptionObj.Message);
                EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);
                ServiceControllerObj.Stop();
                return;
            }
        }

        private ERRORXML CreateExternalMountPoint(String ExternalMountPoint)
        {
            try
            {
                if (Directory.Exists(ExternalMountPoint))
                    return null;
                Directory.CreateDirectory(ExternalMountPoint);
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            return null;
        }
        
        private void MonitorFileSystem(FileSystemWatcher FileSystemWatcherObj, FWProcessor FWProcessorObj)
        {
            try
            {
                FileSystemWatcherObj.Path                   = FWProcessorObj.TenantInfoObj.ExternalMountPoint;
                FileSystemWatcherObj.EnableRaisingEvents    = true;
                FileSystemWatcherObj.IncludeSubdirectories  = true;
                FileSystemWatcherObj.Filter                 = "AssetInfo*";
                FileSystemWatcherObj.Created                += new FileSystemEventHandler(FWProcessorObj.HandleDirectoryChanges);
                
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        protected void HandleDirectoryChanges(object sender, FileSystemEventArgs EventArgs)
        {
            ProcessFileNotification ProcessFileNotificationObj  = null;
            Thread                  ThreadObj                   = null;
            try
            {
                if (((EventArgs.Name.EndsWith("AssetInfo", StringComparison.OrdinalIgnoreCase)) || (EventArgs.Name.EndsWith("AssetInfo.hqx", StringComparison.OrdinalIgnoreCase))) && (EventArgs.ChangeType == WatcherChangeTypes.Created))
                {
                    ProcessFileNotificationObj              = new ProcessFileNotification(TenantInfoObj, EventArgs.FullPath);
                    //ProcessFileNotificationObj.EventLogObj  = this.EventLogObj;
                    ThreadObj                               = new Thread(new ThreadStart(ProcessFileNotificationObj.ProcessNewJob));
                    ThreadObj.Start();
                }
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private ERRORXML SetALServiceHeaderValue(ulong SessionID, ulong UserID, String TenantName, String Language)
        {
           try
            {
                RequestHeader1.SessionID     = SessionID;
                RequestHeader1.UserID        = UserID;
                RequestHeader1.TenantName    = TenantName;
                RequestHeader1.Language      = Language;
                
                RequestHeader2.SessionID     = SessionID;
                RequestHeader2.UserID        = UserID;
                RequestHeader2.TenantName    = TenantName;
                RequestHeader2.Language      = Language;
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private ERRORXML GetAllExternalMoutPoints(out XML ExternalMountPointXML)
        {           
            String              strErrorXml                     = null;            
            XML                 TenantListXml                   = null;
            String              strTenantName                   = null;                       
            XmlDocument         TenantXMLDoc                    = null;
            XmlNode             ExternalMountPointCollNode      = null;
            XmlNode             CatalogerNode                   = null;            
            XmlNode             ExternalMountPointNode          = null;
            XmlNode             TempNode                        = null;            
            XPathDocument       xpDoc_TenantList                = null;
            XPathNavigator      xpNav_Tenant                    = null;
            XPathNodeIterator   xpItr_Tenant                    = null;
                        
            ExternalMountPointXML         = null;            
            TenantXMLDoc                  = new XmlDocument();

            strErrorXml             =   InfraWSObj.GetAllTenants(ref TenantListXml);
            if (String.IsNullOrEmpty(strErrorXml))
            {
                xpDoc_TenantList    =   new XPathDocument(new StringReader(TenantListXml));
                xpNav_Tenant        =   xpDoc_TenantList.CreateNavigator();
                xpItr_Tenant        =   xpNav_Tenant.Select("TenantList/Tenant");
                
                if(xpItr_Tenant.Count > 0)
                    ExternalMountPointCollNode = TenantXMLDoc.CreateElement("EXTERNALMOUNTPOINTCOLLECTION");        
                while(xpItr_Tenant.MoveNext())
                {                    
                    strTenantName   =   xpItr_Tenant.Current.GetAttribute("Name", "");

                    TenantXMLDoc.Load(ALCommon.GetTenantXMLPath(strTenantName));

                    ExternalMountPointNode = TenantXMLDoc.CreateElement("EXTERNALMOUNTPOINT");

                    TempNode            = TenantXMLDoc.CreateElement("TENANTNAME");
                    TempNode.InnerText  = strTenantName;
                    ExternalMountPointNode.AppendChild(TempNode);

                    TempNode            = TenantXMLDoc.CreateElement("APPLICATION");
                    TempNode.InnerText  = "CATALOGER";
                    ExternalMountPointNode.AppendChild(TempNode);                                  
                    
                    CatalogerNode = TenantXMLDoc.SelectSingleNode("CONFIGURATION/CATALOGER");
                    if (CatalogerNode != null)
                    {
                        TempNode            = TenantXMLDoc.CreateElement("EXTERNALCATALOGERMOUNTPOINT");
                        if (CatalogerNode.SelectSingleNode("EXTERNALCATALOGERMOUNTPOINT") != null)
                            TempNode.InnerText = CatalogerNode.SelectSingleNode("EXTERNALCATALOGERMOUNTPOINT").InnerText;
                        else
                            TempNode.InnerText = "";
                        ExternalMountPointNode.AppendChild(TempNode);

                        TempNode            = TenantXMLDoc.CreateElement("MOUNTPOINT");
                        if (CatalogerNode.SelectSingleNode("MOUNTPOINT") != null)
                            TempNode.InnerText = CatalogerNode.SelectSingleNode("MOUNTPOINT").InnerText;
                        else
                            TempNode.InnerText = "";                        
                        ExternalMountPointNode.AppendChild(TempNode);

                        TempNode            = TenantXMLDoc.CreateElement("ASSETINFORETRYCOUNT");
                        TempNode.InnerText  = CatalogerNode.SelectSingleNode("ASSETINFORETRYCOUNT").InnerText;
                        ExternalMountPointNode.AppendChild(TempNode);

                        TempNode            = TenantXMLDoc.CreateElement("ASSETINFORETRYINTERVAL");
                        TempNode.InnerText  = CatalogerNode.SelectSingleNode("ASSETINFORETRYINTERVAL").InnerText;
                        ExternalMountPointNode.AppendChild(TempNode);
                    }

                    if (ExternalMountPointNode != null)
                        ExternalMountPointCollNode.AppendChild(ExternalMountPointNode);
                }
                if (xpItr_Tenant.Count > 0)
                    ExternalMountPointXML = ExternalMountPointCollNode.OuterXml;
                else
                    ExternalMountPointXML = null;
            }            
            return null;            
        }
        #endregion
    }

    public class ProcessFileNotification 
    {
        const ulong     SystemUserID            = 111111122211111;
        const String    SystemUserName          = "MePhIsToPhElEs";
        String          EncryptedSystemUserName = null;
        const String    SystemUserPwd           = "ErAtOsThEnEs";
        String          EncryptedSystemUserPwd  = null;
        ERRORXML        ErrorXML                = null;         
        String          FilePath                = null;
        XmlDocument     FileNotificationXmlDoc  = null;
        XmlNode         FileNotificationNode    = null;
        ALDAMWS         DAMWSObj                = null;
        ENTITYID        NotificationID          = 0;
        ENTITYID        CurrentUserID           = 0;
        TenantInfo      TenantInfoObj;
        StringBuilder   OriginalsPath           = null;
        StringBuilder   CatalogerJobPath        = null;
        Int32           FileCount               = 0;
        DateTime        StartTime;
        ulong           SessionID               = 0;
        XML             UserXML                 = null;
        XPathDocument   xp_UserXMLDoc           = null;
        ulong           UserID                  = 0;
        XmlDocument     ErrorXMLDoc             = null;
        String          ImpersonationToken      = null;        
        StringCollection OriginalsFileNames     = null;
        String          Language                = "English";
        XPathNavigator  xpNav_UserRoot          = null;
        StringDictionary strdict_AdditionalInfo = null;
        String           SendMailTo             = null;
        String           SenderName             = null;
        String           SenderEmail            = null;
        String           VendorEmail            = null;
        String           SendMailSettings       = "ALWAYS";
        String           JobName                = null;
        String           LoginName              = null;
        String           str_AdminID            = null;
        String           str_NewJobName         = null;
        String           JobPath                = null;

        public EventLog  EventLogObj            = null;
        String           JobID                  = null;
        const String     str_CallerIdentity     = "DAMJOBCOPYSERVICE"; 
        global::Assetlink.WebServices.ALDAMWS.ALServiceHeader RequestHeader = null;
        enum ErrorType
        {
            ERROR = 0,
            EXCEPTION = 1
        }

        public ProcessFileNotification(TenantInfo TenantInfoObj, String FilePath)
        {
            InitializeDataMembers(TenantInfoObj, FilePath, null);
        }

        public ProcessFileNotification(TenantInfo TenantInfoObj, String FilePath, String JobID)
        {
            InitializeDataMembers(TenantInfoObj, FilePath, JobID);
        }

        private void InitializeDataMembers(TenantInfo TenantInfoObj, String FilePath,String JobID)
        {
            this.TenantInfoObj              = new TenantInfo();
            this.TenantInfoObj              = TenantInfoObj;
            this.FilePath                   = FilePath;
            this.JobID                      = JobID;
            FileNotificationXmlDoc          = new XmlDocument();
            RequestHeader                   = new global::Assetlink.WebServices.ALDAMWS.ALServiceHeader();
            DAMWSObj                        = new ALDAMWS();
            DAMWSObj.ALServiceHeaderValue   = RequestHeader;
            DAMWSObj.Url                    = ConfigurationManager.AppSettings["DAMWSURL"];
            strdict_AdditionalInfo          = new StringDictionary();
            EventLogObj                     = CreateEventLog();
        }

        public void ProcessNewJob()
        {
            DAMJobCopyErrorCodes   Obj_DAMJobCopyErrorCodes; 
            StringBuilder          ErrorMessage         = null;
            String                 ErrorXML             = null;
            bool                   IsJobContextCreated  = false; // This flag will determine whether job is created for the request or not. In case of exception, based on the flag value exception will either be logged in JOB table or in the Event Viewer
            
            try
            {                
                EncryptedSystemUserName = ALCommon.Encrypt(SystemUserName);
                EncryptedSystemUserPwd  = ALCommon.Encrypt(SystemUserPwd);
                ErrorMessage            = new StringBuilder(100);

                ErrorXML                = Infrastructure.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, ref ImpersonationToken);
                if (ErrorXML == null)
                {
                    ErrorXML = Infrastructure.ImpersonateUser("Admin", TenantInfoObj.TenantName, ImpersonationToken,
                         str_CallerIdentity, ref SessionID, ref UserXML);

                    if ((ErrorXML == null) && (UserXML != null))
                    {
                        xp_UserXMLDoc   = new XPathDocument(new System.IO.StringReader(UserXML));
                        xpNav_UserRoot  = xp_UserXMLDoc.CreateNavigator();
                        UserID          = Convert.ToUInt64(xpNav_UserRoot.SelectSingleNode("USER/USERID").Value);
                        
                        //Set Notification settings like SenderName which will be always Admin of that Tenant, SenderEmailID
                        str_AdminID     = UserID.ToString();
                        SetNotificationSettings(xpNav_UserRoot);

                        SetALServiceHeaderValue(SessionID, UserID, TenantInfoObj.TenantName, "English");
                        
                        // Add entry for Job Arrived
                        if (AddFileNotificationInfo() == DAMJobCopyErrorCodes.OK)
                        {
                            IsJobContextCreated = true;
                            Obj_DAMJobCopyErrorCodes = ValidateJobPath();
                            if (Obj_DAMJobCopyErrorCodes == DAMJobCopyErrorCodes.OK)
                            {
                                // Change status to new job
                                //FileNotificationNode.SelectSingleNode("STATUS").InnerText = "New Job";
                                //if ((UpdateFileNotification(ref ErrorXML)) == DAMJobCopyErrorCodes.OK)
                                //{
                                    Obj_DAMJobCopyErrorCodes = MoveJobToCatalogerMachine(ref ErrorXML);                                    
                                    if(Obj_DAMJobCopyErrorCodes != DAMJobCopyErrorCodes.OK)
                                    {                                        
                                        ErrorMessage.Remove(0, ErrorMessage.Length);
                                        ErrorMessage.Append("Error while moving job to cataloger mount point.");
                                        LogError("DAMJOBCOPY_FWPROCESSOR_PROCESSNEWJOB_MOVEJOBFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo);
                                        FileNotificationNode.SelectSingleNode("STATUS").InnerText = "Error";
                                        UpdateFileNotification(ref ErrorXML);

                                        // In case of Move failure, two email should be sent. One to Admin and second to Vendor
                                        StampNotification(Obj_DAMJobCopyErrorCodes,"Admin");

                                        if (!SenderEmail.Equals(VendorEmail))
                                        {
                                            SendMailTo = VendorEmail;
                                            StampNotification(Obj_DAMJobCopyErrorCodes, "vendor");
                                        }
                                    }
                                    else
                                    {
                                        // In case of successful MoveJon File notification entry should be removed from the syste. 
                                        // Otherwise system will show 2 entries for the same job in catalog Job status

                                        if ((DeleteFileNotification(ref ErrorXML)) != DAMJobCopyErrorCodes.OK)
                                        {
                                            FileNotificationNode.SelectSingleNode("STATUS").InnerText = "Error";
                                            UpdateFileNotification(ref ErrorXML);

                                            ErrorMessage.Remove(0, ErrorMessage.Length);
                                            ErrorMessage.Append("Error while deleting FileNotification.");
                                            LogError("DAMJOBCOPY_FWPROCESSOR_PROCESSNEWJOBFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                                        }
                                        else
                                        {
                                            //Job clean up
                                            try
                                            {
                                                NTFSJobCleanUp();
                                            }
                                            catch (Exception ExceptionObj)
                                            {
                                                ErrorMessage.Remove(0, ErrorMessage.Length);
                                                ErrorMessage.AppendFormat("Exception thrown during NTFS job cleanup. Message : {0}", ExceptionObj.Message);
                                                LogErrorDetails(TenantInfoObj.TenantName, ErrorMessage.ToString());
                                            }
                                        }
                                    }   
                                //}
                            }
                            else
                            {
                                FileNotificationNode.SelectSingleNode("STATUS").InnerText = "Error";
                                UpdateFileNotification(ref ErrorXML);
                                StampNotification(Obj_DAMJobCopyErrorCodes, "Admin");
                            }
                            // Code to clean up the session
                            Infrastructure.SignOut(UserID, SessionID, Language, TenantInfoObj.TenantName);
                        }
                    }                   
                }
          }
          catch(Exception  ExceptionObj)
          {
              if (IsJobContextCreated)
              {
                  FileNotificationNode.SelectSingleNode("STATUS").InnerText = "Error";
                  UpdateFileNotification(ref ErrorXML);

                  ErrorMessage.Remove(0, ErrorMessage.Length);
                  ErrorMessage.AppendFormat("Exception thrown by ProcessNewJob. Message: {0}", ExceptionObj.Message);
                  LogError("DAMJOBCOPY_FWPROCESSOR_PROCESSNEWJOBFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.EXCEPTION, strdict_AdditionalInfo);                                    
              }
              else
              {
                  ErrorMessage.Remove(0, ErrorMessage.Length);
                  ErrorMessage.AppendFormat("Exception thrown by ProcessNewJob. Message: {0}", ExceptionObj.Message);
                  EventLogObj.WriteEntry(ErrorMessage.ToString(), EventLogEntryType.Error);                      
              }
          }
        }

        private void NTFSJobCleanUp()
        {
            StringBuilder sb_AdditionalInfoXml = new StringBuilder(200);
            sb_AdditionalInfoXml.AppendFormat(@"<ADDITIONALINFO UserID='{1}' SessionID='{2}' Language='{3}' TenantName='{4}'>
                                                    <THREADID>{0}</THREADID>
                                                    <METHODNAME>DAMJobCopyService.ProcessFileNotification.NTFSJobCleanUp()</METHODNAME>                                                 
                                                    <JOBPATH>{5}</JOBPATH>    
                                              </ADDITIONALINFO>",
                                             System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), UserID, SessionID, Language, TenantInfoObj.TenantName,
                                             ALCommon.TrimAndReplaceSpecialChars(JobPath));
            Infrastructure.DeleteFolder(JobPath, true, UserID, SessionID, Language, TenantInfoObj.TenantName, sb_AdditionalInfoXml.ToString());
        }

        private void LogErrorDetails(String TenantName, String Message)
        {
            StringBuilder LogFilePath     = null;
            StringBuilder LogDetails      = null;
            FileStream    LogFileStream   = null;
            StreamWriter  LogStreamWriter = null;

            try
            {
                LogFilePath = new StringBuilder(50);
                LogDetails  = new StringBuilder(100);

                LogFilePath.AppendFormat("{0}{1}\\Logs\\DAM Job Copy",ALCommon.GetInstallationDir(),TenantName);
                if (!Directory.Exists(LogFilePath.ToString()))
                    Directory.CreateDirectory(LogFilePath.ToString());

                LogFilePath.AppendFormat("\\{0}_ErrorLog.txt", DateTime.Now.ToLongDateString());

                if (File.Exists(LogFilePath.ToString()))
                    LogFileStream   = new FileStream(LogFilePath.ToString(), FileMode.Append);// Append the changes if file exists
                else
                    LogFileStream   = new FileStream(LogFilePath.ToString(), FileMode.Create); // if not present create file

                LogStreamWriter     = new StreamWriter(LogFileStream);
                LogDetails.AppendFormat("{0} .....{1}", Message, DateTime.Now.ToString());
                LogStreamWriter.WriteLine(LogDetails.ToString());
                LogStreamWriter.WriteLine("-----------------------------------------------------------------------------------------------------------------");
              
                LogStreamWriter.Close();
                LogFileStream.Close();
            }
            catch (Exception)
            { 
            }
        }

        private DAMJobCopyErrorCodes MoveJobToCatalogerMachine(ref String ErrorXML)
        {                         
            Char[]                  CharSearchFilePath  = null;            
            String[]                FileList            = null;
            String                  SearchFilePath      = null;                                     
            String                  strFileListXml      = null;
            StringBuilder           ErrorMessage        = null;
            StringBuilder           sb_FileList         = null;            
            DAMJobCopyErrorCodes    DAMJobCopyErrorCodesObj;           
            StringBuilder           sb_TempJobName      = null;
            FileTransferInfrastructure MTOMObj          = null; 
            String                  str_CatalogerMTOMWSUrl = null;
            bool                    IsJobAlreadyExists  = false;
            bool                    IsJobRenamed        = false;
            StringBuilder           sb_TempNewJobName   = null;

            try
            {
                ErrorMessage        = new StringBuilder(300);
                sb_TempJobName      = new StringBuilder(300);
                sb_TempNewJobName   = new StringBuilder(300);
                MTOMObj             = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantInfoObj.TenantName, Language);

                if ((DAMJobCopyErrorCodesObj = CheckAllOriginals()) == DAMJobCopyErrorCodes.OK)
                {
                    #region prepare FileListXml
                    sb_FileList             = new StringBuilder(200);
                    CharSearchFilePath      = new char[FilePath.LastIndexOf("\\")];
                    FilePath.CopyTo(0, CharSearchFilePath, 0, CharSearchFilePath.Length);
                    SearchFilePath          = new String(CharSearchFilePath);

                    #region check if job with the same name already exists on cataloger mount point
                    sb_TempJobName.AppendFormat("{0}\\{1}\\{2}", TenantInfoObj.MountPoint, LoginName, JobName);
                    str_NewJobName      = sb_TempJobName.ToString();
                    GetCatalogerMTOMWS(ref str_CatalogerMTOMWSUrl);
                    IsJobAlreadyExists  = MTOMObj.IsFolderExists(str_NewJobName, str_CatalogerMTOMWSUrl);

                    while (IsJobAlreadyExists)
                    {
                        sb_TempNewJobName.Remove(0, sb_TempNewJobName.Length);
                        sb_TempNewJobName.AppendFormat("{0}_{1}{2}{3}{4}{5}{6}", JobName,DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                        sb_TempJobName.Remove(0, sb_TempJobName.Length);
                        sb_TempJobName.AppendFormat("{0}\\{1}\\{2}", TenantInfoObj.MountPoint, LoginName, sb_TempNewJobName.ToString());
                        str_NewJobName      = sb_TempNewJobName.ToString();
                        IsJobAlreadyExists  = MTOMObj.IsFolderExists(sb_TempJobName.ToString(), str_CatalogerMTOMWSUrl);
                        IsJobRenamed        = true;                        
                    }

                    if (IsJobRenamed)
                    {
                        sb_TempJobName.Remove(0, sb_TempJobName.Length);
                        sb_TempJobName.AppendFormat("{0}\\{1}\\{2}", TenantInfoObj.ExternalMountPoint, LoginName, str_NewJobName);
                        DirectoryInfo ObjJob = new DirectoryInfo(SearchFilePath);
                        ObjJob.MoveTo(sb_TempJobName.ToString());
                        
                        //Send notification to vendor regarding job rename
                        StampNotification(DAMJobCopyErrorCodes.JOB_RENAMED, "JobWarnings");

                        JobName         = str_NewJobName;
                        FilePath        = FilePath.Replace(SearchFilePath, sb_TempJobName.ToString());
                        SearchFilePath  = sb_TempJobName.ToString();
                    }                   
                    #endregion

                    JobPath = SearchFilePath;
                    sb_FileList.AppendFormat("<FOLDERCOLLECTION><FOLDER Name='{0}'>", JobName);

                    if (Directory.Exists(SearchFilePath + "\\Originals"))
                    {
                        sb_FileList.Append("<SUBFOLDER Name='Originals'>");
                        FileList = Directory.GetFiles(SearchFilePath + "\\Originals");
                        IEnumerator Iterator = FileList.GetEnumerator();
                        while (Iterator.MoveNext())
                        {
                            sb_FileList.AppendFormat("<FILE>{0}</FILE>", Iterator.Current);
                        }
                        sb_FileList.Append("</SUBFOLDER>");
                    }
                    else
                    {
                        return DAMJobCopyErrorCodes.ORG_FOLDER_NOT_FOUND;
                    }

                    if (Directory.Exists(SearchFilePath + "\\Previews"))
                    {
                        FileList = null;
                        sb_FileList.Append("<SUBFOLDER Name='Previews'>");
                        FileList = Directory.GetFiles(SearchFilePath + "\\Previews");
                        IEnumerator Iterator = FileList.GetEnumerator();
                        while (Iterator.MoveNext())
                        {
                            sb_FileList.AppendFormat("<FILE>{0}</FILE>", Iterator.Current);
                        }
                        sb_FileList.Append("</SUBFOLDER>");
                    }

                    if (Directory.Exists(SearchFilePath + "\\Thumbnails"))
                    {
                        FileList = null;
                        sb_FileList.Append("<SUBFOLDER Name='Thumbnails'>");
                        FileList = Directory.GetFiles(SearchFilePath + "\\Thumbnails");
                        IEnumerator Iterator = FileList.GetEnumerator();
                        while (Iterator.MoveNext())
                        {
                            sb_FileList.AppendFormat("<FILE>{0}</FILE>", Iterator.Current);
                        }
                        sb_FileList.Append("</SUBFOLDER>");
                    }

                    sb_FileList.AppendFormat("<FILE>{0}</FILE>", FilePath);
                    sb_FileList.Append("</FOLDER></FOLDERCOLLECTION>");
                    strFileListXml = sb_FileList.ToString();
                    #endregion
                    
                    ErrorXML = MTOMObj.MoveJob(ref strFileListXml, TenantInfoObj.MountPoint, LoginName);
                    if (String.IsNullOrEmpty(ErrorXML))
                        return DAMJobCopyErrorCodes.OK;
                    else
                        return DAMJobCopyErrorCodes.ERROR_IN_MOVEJOB;
                }
                else
                {
                    if (DAMJobCopyErrorCodesObj == DAMJobCopyErrorCodes.FILECOUNT_DOES_NOT_MATCH)
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Number of Files specified by the FileCount in AssetInfo not Arrive in time");
                        LogError("DAMJOBCOPY_FWPROCESSOR_FILECOUNT_DOES_NOT_MATCH", ErrorMessage.ToString(), null, ErrorType.ERROR, null);
                    }
                    if (DAMJobCopyErrorCodesObj == DAMJobCopyErrorCodes.ORG_FOLDER_NOT_FOUND)
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Originals Folder Not Arrived in time");
                        LogError("DAMJOBCOPY_FWPROCESSOR_ORG_FOLDER_NOT_FOUND", ErrorMessage.ToString(), null, ErrorType.ERROR, null);
                    }
                    return DAMJobCopyErrorCodesObj;
                }
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);

                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in MoveJobToCatalogerMachine. Message: {0}", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_MOVEJOBTOCATALOGERMACHINE", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }
        }
        
        #region Private Methods

        private ERRORXML LogError(String ErrorCode, String ErrorMessage, String InnerError, ErrorType ErrorTypeValue, StringDictionary strdict_AdditionalInfo)
        {
            ERRORXML        ErrorXML                        = null;
            XmlNode         JobErrorNode                    = null;
            XmlNode         ColoumnNode                     = null;
            XmlNode         ErrorDetailsNode                = null;
            XML             LogErrorXML                     = null;
            XmlNode         MessageNode                     = null;
            XmlNode         ErrorNode                       = null;
            XmlDocument     InnerErrorXMLDoc                = null;
            XmlNode         ErrorCollectionNode             = null;
            XmlNode         xmlnod_AdditionalInfo           = null;
            XmlNode         xmlnod_AdditionalInfoField      = null;
            XmlNode         xmlnod_ErrorCode                = null;

            if (InnerError != null)
            {
                InnerErrorXMLDoc                    = new XmlDocument();
                InnerErrorXMLDoc.LoadXml(InnerError);
                if (InnerErrorXMLDoc.SelectSingleNode("ERRORCOLLECTION") == null)
                {
                    ErrorCollectionNode             = InnerErrorXMLDoc.CreateElement("ERRORCOLLECTION");
                    ErrorCollectionNode.InnerXml    = InnerError;
                    InnerError                      = ErrorCollectionNode.OuterXml;
                }
            }
            else
            {
                InnerError = String.Empty;
            }

            if (ErrorXMLDoc == null)
            {
                ErrorXMLDoc                 = new XmlDocument();
                JobErrorNode                = ErrorXMLDoc.CreateElement("JOBERROR");
                
                ColoumnNode                 = ErrorXMLDoc.CreateElement("APPLICATIONNAME");
                ColoumnNode.InnerText       = "DAMJobCopy";
                JobErrorNode.AppendChild(ColoumnNode);

                ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORTYPE");
                if(ErrorTypeValue==ErrorType.ERROR)
                    ColoumnNode.InnerText   = "ERROR";
                else
                    ColoumnNode.InnerText   = "EXCEPTION";
                JobErrorNode.AppendChild(ColoumnNode);

                ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORMESSAGE");
                ErrorNode                   = ErrorXMLDoc.CreateElement("ERROR");
                MessageNode                 = ErrorXMLDoc.CreateElement("MESSAGE");
                xmlnod_ErrorCode            = ErrorXMLDoc.CreateElement("ERRORCODE");

                if (!String.IsNullOrEmpty(ErrorCode))
                {
                    xmlnod_ErrorCode.InnerText = ErrorCode;
                }
                ErrorNode.AppendChild(xmlnod_ErrorCode);

                MessageNode.InnerText       = ErrorMessage;
                ErrorNode.AppendChild(MessageNode);

                if (InnerError != null)
                {
                    ErrorDetailsNode            = ErrorXMLDoc.CreateElement("ERRORDETAILS");
                    ErrorDetailsNode.InnerXml   = InnerError;
                    ErrorNode.AppendChild(ErrorDetailsNode);
                }

                if (strdict_AdditionalInfo != null)
                {
                    xmlnod_AdditionalInfo = ErrorXMLDoc.CreateElement("ADDITIONALINFO");
                    IEnumerator ienum_AdditionalInfo = strdict_AdditionalInfo.GetEnumerator();

                    foreach (DictionaryEntry obj_DictionaryEntry in strdict_AdditionalInfo)
                    {

                        xmlnod_AdditionalInfoField = ErrorXMLDoc.CreateElement(obj_DictionaryEntry.Key.ToString().ToUpper());
                        if (obj_DictionaryEntry.Value != null)
                        {
                            xmlnod_AdditionalInfoField.InnerXml = ALCommon.TrimAndReplaceSpecialChars(obj_DictionaryEntry.Value.ToString());
                        }
                        xmlnod_AdditionalInfo.AppendChild(xmlnod_AdditionalInfoField);
                    }

                    ErrorNode.AppendChild(xmlnod_AdditionalInfo);
                }

                ColoumnNode.AppendChild(ErrorNode);
                JobErrorNode.AppendChild(ColoumnNode);

                ColoumnNode                 = ErrorXMLDoc.CreateElement("FILENOTIFICATION_JOBERROR");
                ((XmlElement)ColoumnNode).SetAttribute("TYPE", "RELATION");
                ((XmlElement)ColoumnNode).SetAttribute("PARENTID", "TRUE");
                ColoumnNode.InnerText       = NotificationID.ToString();
                JobErrorNode.AppendChild(ColoumnNode);

                ErrorXMLDoc.AppendChild(JobErrorNode);
            }
            else
            {
                if (ErrorTypeValue == ErrorType.ERROR)
                    ErrorXMLDoc.SelectSingleNode("//ERRORTYPE").InnerText               = "ERROR";
                else
                    ErrorXMLDoc.SelectSingleNode("//ERRORTYPE").InnerText               = "EXCEPTION";

                if (strdict_AdditionalInfo != null)
                {
                    xmlnod_AdditionalInfo = ErrorXMLDoc.SelectSingleNode("//ADDITIONALINFO");
                    if (xmlnod_AdditionalInfo == null)
                    {
                        ErrorNode = ErrorXMLDoc.SelectSingleNode("//ERROR");
                        xmlnod_AdditionalInfo = ErrorXMLDoc.CreateElement("ADDITIONALINFO");
                        ErrorNode.AppendChild(xmlnod_AdditionalInfo);
                    }
                    else
                    {
                        xmlnod_AdditionalInfo.RemoveAll();
                    }

                    IEnumerator ienum_AdditionalInfo = strdict_AdditionalInfo.GetEnumerator();
                    foreach (DictionaryEntry obj_DictionaryEntry in strdict_AdditionalInfo)
                    {

                        xmlnod_AdditionalInfoField = ErrorXMLDoc.CreateElement(obj_DictionaryEntry.Key.ToString().ToUpper());
                        if (obj_DictionaryEntry.Value != null)
                        {
                            xmlnod_AdditionalInfoField.InnerXml = obj_DictionaryEntry.Value.ToString();
                        }
                        xmlnod_AdditionalInfo.AppendChild(xmlnod_AdditionalInfoField);
                    }
                }
                else
                {
                    xmlnod_AdditionalInfo = ErrorXMLDoc.SelectSingleNode("//ADDITIONALINFO");
                    if (xmlnod_AdditionalInfo != null)
                    {
                        ErrorNode = ErrorXMLDoc.SelectSingleNode("//ERROR");
                        ErrorNode.RemoveChild(xmlnod_AdditionalInfo);
                    }
                }
                                
                ErrorXMLDoc.SelectSingleNode("//ERRORCODE").InnerText               	= ErrorCode;
                ErrorXMLDoc.SelectSingleNode("//MESSAGE").InnerText                 	= ErrorMessage;
                ErrorXMLDoc.SelectSingleNode("//ERRORDETAILS").InnerXml             	= InnerError;
                ErrorXMLDoc.SelectSingleNode("//FILENOTIFICATION_JOBERROR").InnerText   = NotificationID.ToString();
            }

            LogErrorXML = ErrorXMLDoc.InnerXml;
            ErrorXML    = DAMWSObj.LogError(ref LogErrorXML);
            return ErrorXML;
        }       
        
        private DAMJobCopyErrorCodes CheckAllOriginals()
        {
            DirectoryInfo           DirInfoObj                  = null;
            FileInfo[]              Files                       = null;
            DirectoryInfo[]         Directories                 = null;
            DAMJobCopyErrorCodes    DAMJobCopyErrorCodesObj;
            StringBuilder           ErrorMessage                = null;
            try
            {
                ErrorMessage = new StringBuilder();
                while (true)
                {
                    if (!Directory.Exists(OriginalsPath.ToString()))
                    {
                        if ((DAMJobCopyErrorCodesObj = CheckForTimeOut()) == DAMJobCopyErrorCodes.TIME_OUT)
                        {
                            DAMJobCopyErrorCodesObj = DAMJobCopyErrorCodes.ORG_FOLDER_NOT_FOUND;
                            break;
                        }
                        Thread.Sleep(10000);
                        continue;
                    }
                    DirInfoObj  = new DirectoryInfo(OriginalsPath.ToString());
                    Files       = DirInfoObj.GetFiles();
                    foreach (FileInfo File in Files)
                    {
                        if (OriginalsFileNames.Count > 0)
                        {
                            if (OriginalsFileNames.Contains(File.Name.ToUpper()))
                                OriginalsFileNames.Remove(File.Name.ToUpper());
                        }
                        else
                            break;
                    }

                    if (OriginalsFileNames.Count > 0)
                    {
                        if ((DAMJobCopyErrorCodesObj = CheckForTimeOut()) == DAMJobCopyErrorCodes.TIME_OUT)
                        {
                            DAMJobCopyErrorCodesObj = DAMJobCopyErrorCodes.FILECOUNT_DOES_NOT_MATCH;
                            break;
                        }
                        Thread.Sleep(10000);
                    }
                    else
                    {
                        DAMJobCopyErrorCodesObj = DAMJobCopyErrorCodes.OK;
                        break;
                    }
                    
                }

                if ((DAMJobCopyErrorCodesObj != DAMJobCopyErrorCodes.FILECOUNT_DOES_NOT_MATCH) || (DAMJobCopyErrorCodesObj != DAMJobCopyErrorCodes.ORG_FOLDER_NOT_FOUND) )
                {
                    return DAMJobCopyErrorCodesObj;
                }
                else
                {
                    Directories = DirInfoObj.GetDirectories();
                    foreach (DirectoryInfo Dir in Directories)
                    {
                        if (Dir.Name != "." && Dir.Name != "..")
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.AppendFormat("Extra Folder Named {0} Present in Originals Folder", Dir.Name);

                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("EXTRAFOLDERNAME", Dir.Name);
                            LogError("DAMJOBCOPY_FWPROCESSOR_EXTRAFOLDERINORIGINALSFOLDER", ErrorMessage.ToString(), null, ErrorType.ERROR, strdict_AdditionalInfo);
                        }
                    }
                }
                return DAMJobCopyErrorCodes.OK;
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in CheckAllOriginals. Message: {0}", ExceptionObj.Message);

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_EXCEPTION_CHECKALLORIGINALSFAILED", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }
        }

        private DAMJobCopyErrorCodes CheckForTimeOut()
        {
            TimeSpan TimeSpanObj = new TimeSpan(1, 0, 0, 0, 0);

            if ((StartTime + TimeSpanObj) < DateTime.Now)
                return DAMJobCopyErrorCodes.TIME_OUT;
            else
                return DAMJobCopyErrorCodes.OK;
        }

        private DAMJobCopyErrorCodes AddFileNotificationInfo()
        {
            XmlNode         ColoumnNode         = null;
            XmlNode         DateNode            = null;
            XML             FileNotificationXML = null;
            StringBuilder   ErrorMessage        = null;

            try
            {
                ErrorMessage            = new StringBuilder();
                FileNotificationNode    = FileNotificationXmlDoc.CreateElement("FILENOTIFICATION");
                
                ColoumnNode             =  FileNotificationXmlDoc.CreateElement("NOTIFICATIONID");
                UniqueID.Read(ref NotificationID); 
                ColoumnNode.InnerText   = NotificationID.ToString();
                FileNotificationNode.AppendChild(ColoumnNode);
                
                ColoumnNode             =  FileNotificationXmlDoc.CreateElement("ORIGINALSFILECOUNT");
                ColoumnNode.InnerText   = "0";
                FileNotificationNode.AppendChild(ColoumnNode);

                ColoumnNode             =  FileNotificationXmlDoc.CreateElement("STATUS");
                ColoumnNode.InnerText   = "Job Arrived";
                FileNotificationNode.AppendChild(ColoumnNode);

                ColoumnNode             =  FileNotificationXmlDoc.CreateElement("ORIGINALPATH");
                ColoumnNode.InnerText   = "";
                FileNotificationNode.AppendChild(ColoumnNode);

                ColoumnNode             =  FileNotificationXmlDoc.CreateElement("STARTDATE");

                StartTime               = DateTime.Now;
                DateNode                 = FileNotificationXmlDoc.CreateElement("DAY");
                DateNode.InnerText       = DateTime.Now.Day.ToString();
                ColoumnNode.AppendChild(DateNode);

                DateNode                 = FileNotificationXmlDoc.CreateElement("MONTH");
                DateNode.InnerText       = DateTime.Now.Month.ToString();
                ColoumnNode.AppendChild(DateNode);

                DateNode                 = FileNotificationXmlDoc.CreateElement("YEAR");
                DateNode.InnerText       = DateTime.Now.Year.ToString();
                ColoumnNode.AppendChild(DateNode);

                DateNode                 = FileNotificationXmlDoc.CreateElement("HOUR");
                DateNode.InnerText       = DateTime.Now.Hour.ToString();
                ColoumnNode.AppendChild(DateNode);

                DateNode                 = FileNotificationXmlDoc.CreateElement("MINUTE");
                DateNode.InnerText       = DateTime.Now.Minute.ToString();
                ColoumnNode.AppendChild(DateNode);

                DateNode                 = FileNotificationXmlDoc.CreateElement("SECOND");
                DateNode.InnerText       = DateTime.Now.Second.ToString();
                ColoumnNode.AppendChild(DateNode);

                FileNotificationNode.AppendChild(ColoumnNode);

                ColoumnNode             =  FileNotificationXmlDoc.CreateElement("JOBNAME");
                ColoumnNode.InnerText   = "";
                FileNotificationNode.AppendChild(ColoumnNode);


                FileNotificationXML     = FileNotificationNode.OuterXml;
                ErrorXML                = DAMWSObj.AddFileNotification(ref FileNotificationXML);
                if (ErrorXML != null)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to add FileNotification entry");
                    LogError("DAMJOBCOPY_FWPROCESSOR_ADDFILENOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, null);
                    return DAMJobCopyErrorCodes.ERROR;
                }
                return DAMJobCopyErrorCodes.OK;
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in AddFileNotificationInfo : Message : {0}", ExceptionObj.Message);

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_ADDFILENOTIFICATIONFAILED", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }
        }

        private DAMJobCopyErrorCodes UpdateFileNotification(ref String ErrorXml)
        {
            XML             FileNotificationXML = null;
            StringBuilder   ErrorMessage        = null;
            try
            {
                ErrorMessage            = new StringBuilder();
                FileNotificationXML     = FileNotificationNode.OuterXml;
                ErrorXml                = DAMWSObj.UpdateFileNotification(FileNotificationXML);
                if (ErrorXml != null)
                {
                   return DAMJobCopyErrorCodes.ERROR;
                }
                return DAMJobCopyErrorCodes.OK;
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in UpdateFileNotification : Message : {0}", ExceptionObj.Message);

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_UPDATEFILENOTIFICATIONFAILED", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }
        }

        private DAMJobCopyErrorCodes DeleteFileNotification(ref String ErrorXml)
        {
            XML             FileNotificationXML     = null;
            StringBuilder   sb_FileNotificationColl = null;
            StringBuilder   ErrorMessage            = null;
            String          FileNotificationIDXml   = null;
            XPathDocument   xpDoc_FileNotification  = null;
            XPathNavigator  xpNav_FileNotification  = null;

            try
            {
                ErrorMessage            = new StringBuilder(200);
                sb_FileNotificationColl = new StringBuilder(100);

                FileNotificationXML     = FileNotificationNode.OuterXml;
                xpDoc_FileNotification  = new XPathDocument(new StringReader(FileNotificationXML));
                xpNav_FileNotification  = xpDoc_FileNotification.CreateNavigator();
                
                sb_FileNotificationColl.Append("<FILENOTIFICATIONCOLLECTION><FILENOTIFICATION>");
                sb_FileNotificationColl.AppendFormat("<NOTIFICATIONID>{0}</NOTIFICATIONID>",xpNav_FileNotification.SelectSingleNode("FILENOTIFICATION/NOTIFICATIONID").Value);
                sb_FileNotificationColl.Append("</FILENOTIFICATION></FILENOTIFICATIONCOLLECTION>");

                FileNotificationIDXml = sb_FileNotificationColl.ToString();
                ErrorXml              = DAMWSObj.DeleteFileNotificationJobs(FileNotificationIDXml);
                if (ErrorXml != null)
                {
                    return DAMJobCopyErrorCodes.ERROR;
                }
                return DAMJobCopyErrorCodes.OK;
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in DeleteFileNotification : Message : {0}", ExceptionObj.Message);

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_DELETEFILENOTIFICATIONFAILED", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }
        }

        private DAMJobCopyErrorCodes GetFileCount()
        {            
            XPathDocument       AssetinfoDoc            = null;
            XPathNavigator      xpnav_AssetInfoRoot     = null;
            XPathNavigator      FileCountNode           = null;
            StringBuilder       ErrorMessage            = null;
            XPathNodeIterator   DigitalAssetNameNodes   = null;
            try
            {
                ErrorMessage        = new StringBuilder();                
                OriginalsFileNames  = new StringCollection();
                
                AssetinfoDoc = new XPathDocument(FilePath);
                xpnav_AssetInfoRoot = AssetinfoDoc.CreateNavigator();

                FileCountNode = xpnav_AssetInfoRoot.SelectSingleNode("ASSETINFO/DIGITALASSETCOUNT");
                FileCount     = Convert.ToInt32(FileCountNode.Value);

                DigitalAssetNameNodes = xpnav_AssetInfoRoot.Select("ASSETINFO/SECTION/DIGITALASSET/NAME");
                foreach (XPathNavigator DigitalAssetNameNode in DigitalAssetNameNodes)
                {
                    OriginalsFileNames.Add(DigitalAssetNameNode.Value.ToUpper().Trim());
                }
                return DAMJobCopyErrorCodes.OK;
            }
            catch (FileNotFoundException)
            {
                return DAMJobCopyErrorCodes.COULDNOTOPENASSETINFO;
            }
            catch (IOException)
            {
                return DAMJobCopyErrorCodes.COULDNOTOPENASSETINFO;
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in GetFileCount : Message : {0}", ExceptionObj.Message);

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_EXCEPTION_GETFILECOUNTFAILED", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }            
        }

        
        /// <summary>
        /// This function validates given JOb and stamp the job with status as "New Job"
        /// </summary>              
        /// <returns></returns>
        private DAMJobCopyErrorCodes ValidateJobPath()
        {
            String[]                Paths                       = null;
            String[]                Delimeter                   = null;
            String                  RestOfThePath               = null;                        
            XML                     UserXML                     = null;            
            String                  SearchFilePath              = null;    
            int                     RetryCount                  = 0;
            int                     RetryInterval               = 0;
            String                  TempRestOfThePath           = null;                        
            StringBuilder           TempMountPoint              = null;
            StringBuilder           ErrorMessage                = null;            
            XPathNavigator          xpNav_Root                  = null;
            StringBuilder           FilenotificationCollXML     = null;
            DAMJobCopyErrorCodes    Obj_DAMJobCopyErrorCodes    = DAMJobCopyErrorCodes.OK;
           
            try
            {
                ErrorMessage        = new StringBuilder();
                OriginalsPath       = new StringBuilder();
                TempMountPoint      = new StringBuilder();
                CatalogerJobPath    = new StringBuilder();

                SearchFilePath      = FilePath.Substring(0, FilePath.LastIndexOf("\\"));

                //If Cataloger mount point specified has "\\" (i.e. c:\Assetlink\CatMP\ )appended at the last position then no need to append
                // "\\" to Tenant mount point 
                TempMountPoint = new StringBuilder();
                if (TenantInfoObj.ExternalMountPoint.LastIndexOf("\\") == (TenantInfoObj.ExternalMountPoint.Length - 1))
                    TempMountPoint.AppendFormat("{0}", TenantInfoObj.ExternalMountPoint);
                else
                    TempMountPoint.AppendFormat("{0}\\", TenantInfoObj.ExternalMountPoint);

                Delimeter           = new string[1];
                Delimeter[0]        = TempMountPoint.ToString();
                Paths               = FilePath.Split(Delimeter, StringSplitOptions.RemoveEmptyEntries);
                RestOfThePath       = Paths[0];

                LoginName           = RestOfThePath.Substring(0, RestOfThePath.IndexOf("\\"));
                TempRestOfThePath   = RestOfThePath.Remove(0, RestOfThePath.IndexOf("\\") + 1);

                # region Associate User with the FileNotification
                if (String.IsNullOrEmpty(LoginName))
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.AppendFormat("User folder found under the Mount Point Named {0}", RestOfThePath);

                    strdict_AdditionalInfo.Clear();
                    strdict_AdditionalInfo.Add("MOUNTPOINT", RestOfThePath);
                    LogError("DAMJOBCOPY_FWPROCESSOR_INVALIDUSER", ErrorMessage.ToString(), null, ErrorType.ERROR, strdict_AdditionalInfo);
                    return DAMJobCopyErrorCodes.ERROR;
                }
                else
                {
                    #region Update Jobname
                    if (TempRestOfThePath.Contains("\\"))
                    {
                        TempRestOfThePath = TempRestOfThePath.Substring(0, TempRestOfThePath.LastIndexOf("\\"));
                        JobName = TempRestOfThePath.Substring(TempRestOfThePath.LastIndexOf("\\") + 1);

                        if (!String.IsNullOrEmpty(JobName))
                        {
                            FileNotificationNode.SelectSingleNode("JOBNAME").InnerText = JobName;
                            if ((UpdateFileNotification(ref ErrorXML)) != DAMJobCopyErrorCodes.OK)
                            {
                                ErrorMessage.Remove(0, ErrorMessage.Length);
                                ErrorMessage.Append("Failed to update jobname for the FileNotification");
                                LogError("DAMJOBCOPY_FWPROCESSOR_UPDATEFILENOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, null);
                                return DAMJobCopyErrorCodes.ERROR;
                            }
                        }
                        else
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.Append("Job Name not found under the User Folder");
                            LogError("DAMJOBCOPY_FWPROCESSOR_JOBNAMENOTFOUNDERUNDERUSERFOLDER", ErrorMessage.ToString(), null, ErrorType.ERROR, null);
                            return DAMJobCopyErrorCodes.ERROR;
                        }
                    }
                    else
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Job Name not found under the User Folder");
                        LogError("DAMJOBCOPY_FWPROCESSOR_JOBNAMENOTFOUNDERUNDERUSERFOLDER", ErrorMessage.ToString(), null, ErrorType.ERROR, null);
                        return DAMJobCopyErrorCodes.ERROR;
                    }
                    #endregion

                    ulong AdminSessionID    = SessionID;

                    ErrorXML                = Infrastructure.ImpersonateUser(LoginName, TenantInfoObj.TenantName, ImpersonationToken, 
                                                str_CallerIdentity, ref SessionID, ref UserXML);
                
                    if (ErrorXML == null) 
                    {
                        if(UserXML!=null)
                        {
                            // Code to clean up the Admin session
                            Infrastructure.SignOut(UserID, AdminSessionID, Language, TenantInfoObj.TenantName);

                            xp_UserXMLDoc       = new XPathDocument(new System.IO.StringReader(UserXML));
                            xpNav_Root          = xp_UserXMLDoc.CreateNavigator();   

                            if (xpNav_Root.SelectSingleNode("USER") != null)
                            {
                                UserID          = Convert.ToUInt64(xpNav_Root.SelectSingleNode("USER/USERID").Value);
                                VendorEmail     = xpNav_Root.SelectSingleNode("USER/EMAILID").Value;
                                CurrentUserID   = UserID;

                                //set Request header with new SessionID and UserID
                                SetALServiceHeaderValue(SessionID, UserID, TenantInfoObj.TenantName,  "English");

                                GetUserCatalogerEmailPreferences();
                                ErrorXML        = DAMWSObj.AssociateUserWithFileNotification(CurrentUserID, NotificationID);
                                if (ErrorXML != null)
                                {
                                    ErrorMessage.Remove(0, ErrorMessage.Length);
                                    ErrorMessage.AppendFormat("User-FileNotification association failed for user {0}.", LoginName);

                                    strdict_AdditionalInfo.Clear();
                                    strdict_AdditionalInfo.Add("LOGINNAME", LoginName);
                                    LogError("DAMJOBCOPY_FWPROCESSOR_ASSOCIATEUSERWITHFILENOTIFICATIONFAILED", ErrorMessage.ToString(), null, ErrorType.ERROR, strdict_AdditionalInfo);
                                    return DAMJobCopyErrorCodes.ERROR;
                                }
                            }
                            else
                            {
                                ErrorMessage.Remove(0, ErrorMessage.Length);
                                ErrorMessage.AppendFormat("Invalid User found under the Mount Point Named {0}", LoginName);
                                
                                strdict_AdditionalInfo.Clear();
                                strdict_AdditionalInfo.Add("MOUNTPOINT", LoginName);
                                LogError("DAMJOBCOPY_FWPROCESSOR_INVALIDUSER", ErrorMessage.ToString(), null, ErrorType.ERROR, strdict_AdditionalInfo);
                                return DAMJobCopyErrorCodes.ERROR;                                
                            }
                        }
                        else
                        {
                            SessionID = AdminSessionID; // Old SessionID should be used as impersanation is faild. In this for logging error Admin's sessionID should be used.
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.AppendFormat("Invalid User found under the Mount Point Named {0}", LoginName);                            

                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("MOUNTPOINT", LoginName);
                            LogError("DAMJOBCOPY_FWPROCESSOR_INVALIDUSER", ErrorMessage.ToString(), null, ErrorType.ERROR, strdict_AdditionalInfo);
                            return DAMJobCopyErrorCodes.ERROR;
                        }
                    }
                    else
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.AppendFormat("User Impersonation failed for user {0}.", LoginName);

                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("LOGINNAME", LoginName);
                        LogError("DAMJOBCOPY_FWPROCESSOR_IMPERSONATEUSERFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo);
                        return DAMJobCopyErrorCodes.ERROR;
                    }                   
                   
                }
                #endregion

                #region Update Originals file path

                OriginalsPath.AppendFormat("{0}\\Originals", SearchFilePath);
                FileNotificationNode.SelectSingleNode("ORIGINALPATH").InnerText = OriginalsPath.ToString();
                if ((UpdateFileNotification(ref ErrorXML)) != DAMJobCopyErrorCodes.OK)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to update original path for the FileNotification");
                    LogError("DAMJOBCOPY_FWPROCESSOR_UPDATEFILENOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, null);
                    return DAMJobCopyErrorCodes.ERROR;
                }
                
                #endregion

                #region If JobID is not null then delete the previous job entry
                if (!String.IsNullOrEmpty(JobID))
                {
                    FilenotificationCollXML = new StringBuilder(30);
                    FilenotificationCollXML.Append("<FILENOTIFICATIONCOLLECTION>");
                    FilenotificationCollXML.Append("<FILENOTIFICATION>");
                    FilenotificationCollXML.Append("<NOTIFICATIONID>" + JobID + "</NOTIFICATIONID>");
                    FilenotificationCollXML.Append("</FILENOTIFICATION>");
                    FilenotificationCollXML.Append("</FILENOTIFICATIONCOLLECTION>");
                    ErrorXML = DAMWSObj.DeleteFileNotificationJobs(FilenotificationCollXML.ToString());
                    if (ErrorXML != null)
                    {
                        return DAMJobCopyErrorCodes.ERROR;
                    }
                }
                #endregion

                #region Prepare Cataloger JobPath
                CatalogerJobPath.Append(SearchFilePath);
                #endregion
                #region Update Filecount

                if ((Obj_DAMJobCopyErrorCodes = GetFileCount()) == DAMJobCopyErrorCodes.COULDNOTOPENASSETINFO)
                {
                    if (TenantInfoObj.AssetInfoRetryInterval == null)
                        RetryInterval = 5000;
                    else
                    {
                        RetryInterval = Convert.ToInt32(TenantInfoObj.AssetInfoRetryInterval);
                        if (RetryInterval < 0)
                            RetryInterval = 5000;
                    }

                    if (TenantInfoObj.AssetInfoRetryCount == null)
                        RetryCount = 1;
                    else
                    {
                        RetryCount = Convert.ToInt32(TenantInfoObj.AssetInfoRetryCount);
                        if (RetryCount < 0)
                            RetryCount = 1;
                    }

                    while (RetryCount >= 1)
                    {
                        Thread.Sleep(RetryInterval);
                        if ((Obj_DAMJobCopyErrorCodes = GetFileCount()) == DAMJobCopyErrorCodes.COULDNOTOPENASSETINFO)
                        {
                            Thread.Sleep(RetryInterval);
                            RetryCount--;
                        }
                        else if (Obj_DAMJobCopyErrorCodes == DAMJobCopyErrorCodes.ERROR)
                        {
                            return Obj_DAMJobCopyErrorCodes;
                        }
                        else
                        {
                            break;
                        }
                    }
                    if (RetryCount == 0)
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Failed to load AssetInfo File");
                        LogError("DAMJOBCOPY_FWPROCESSOR_LOADING_ASSETINFO_FAILED", ErrorMessage.ToString(), null, ErrorType.ERROR, null);                        
                        return DAMJobCopyErrorCodes.COULDNOTOPENASSETINFO;
                    }
                   
                }
                else if (Obj_DAMJobCopyErrorCodes == DAMJobCopyErrorCodes.ERROR)
                {
                    return Obj_DAMJobCopyErrorCodes;
                }
                FileNotificationNode.SelectSingleNode("ORIGINALSFILECOUNT").InnerText = FileCount.ToString();

                /*if ((UpdateFileNotification(ref ErrorXML)) != DAMJobCopyErrorCodes.OK)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to update original file count for the FileNotification");
                    LogError("DAMJOBCOPY_FWPROCESSOR_UPDATEFILENOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, null);
                    return DAMJobCopyErrorCodes.ERROR;
                }*/

                // Stamp JOB status as "New Job"
                FileNotificationNode.SelectSingleNode("STATUS").InnerText = "New Job";
                if ((UpdateFileNotification(ref ErrorXML)) != DAMJobCopyErrorCodes.OK)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Failed to update FileNotification.");
                    LogError("DAMJOBCOPY_FWPROCESSOR_VALIDATEJOBPATH_UPDATEFILENOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML, ErrorType.ERROR, null);
                    return DAMJobCopyErrorCodes.ERROR;
                }
                            
                #endregion
            
                return  DAMJobCopyErrorCodes.OK;
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in ValidateJobPath. Message: {0}", ExceptionObj.Message);

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError("DAMJOBCOPY_FWPROCESSOR_EXCEPTION_VALIDATEJOBPATHFAILED", ErrorMessage.ToString(), null, ErrorType.EXCEPTION, strdict_AdditionalInfo);
                return DAMJobCopyErrorCodes.ERROR;
            }
        }

        private ERRORXML SetALServiceHeaderValue(ulong SessionID, ulong UserID, String TenantName, String Language)
        {
           try
            {
                RequestHeader.SessionID         = SessionID;
                RequestHeader.UserID            = UserID;
                RequestHeader.TenantName        = TenantName;
                RequestHeader.Language          = Language;
                return null;
            }
            catch (Exception)
            {
                return null;
            }
        }

        /// <summary>
        /// This method is used to stamp notification for DAMJobCopy validation Errors.
        /// </summary>        
        /// <param name="Obj_DAMJobCopyErrorCodes">DAMJobCopy Error code</param>
        /// <param name="usertype">Either Admin or Vendor</param>
        /// <returns></returns>
        public ERRORXML StampNotification(DAMJobCopyErrorCodes Obj_DAMJobCopyErrorCodes, string usertype)
        {
            #region Member Variables
            ERRORXML            ErrorXML                    = null;
            String              NotificationTypeID          = null;
            const String        Descriminant                = "0";
            const String        ChannelType                 = "EMAIL";
            const String        ReceipientType              = "TO";
            XmlDocument         NotificationXmlDoc          = null;
            XmlNode             NotificationNode            = null;
            XmlNode             MessageXmlNode              = null;      
            XmlNode             ParamNode                   = null;
            XmlNode             NotificationMessageNode     = null;
            XmlNode             ChannelCollectionNode       = null;
            XmlNode             ChannelNode                 = null;
            XmlNode             ReceipientCollectionNode    = null;
            XmlNode             ReceipientNode              = null;
            String              NotificationXML             = null;
            Notification        NotificationProcessorObj    = null;
            #endregion

            try
            {
                if (usertype.Equals("Admin"))
                    NotificationTypeID = "5650";
                else if (usertype.Equals("JobWarnings"))
                    NotificationTypeID = "5652";
                else
                    NotificationTypeID = "5651";

                switch (Obj_DAMJobCopyErrorCodes)
                { 
                    case DAMJobCopyErrorCodes.COULDNOTOPENASSETINFO:
                    case DAMJobCopyErrorCodes.FILECOUNT_DOES_NOT_MATCH:
                    case DAMJobCopyErrorCodes.ERROR_IN_MOVEJOB:
                    case DAMJobCopyErrorCodes.ORG_FOLDER_NOT_FOUND:
                    case DAMJobCopyErrorCodes.JOB_RENAMED:
                        #region Stamp Notification
                        if ((String.Compare(SendMailSettings,"NEVER",true) != 0) && !String.IsNullOrEmpty(SendMailTo))
                        {
                            NotificationXmlDoc      = new XmlDocument();
                            NotificationNode        = NotificationXmlDoc.CreateElement("NOTIFICATION");

                            ParamNode               = NotificationXmlDoc.CreateElement("NOTIFICATIONTYPEID");
                            ParamNode.InnerText     = NotificationTypeID;
                            NotificationNode.AppendChild(ParamNode);

                            ParamNode               = NotificationXmlDoc.CreateElement("DESCRIMINANT");
                            ParamNode.InnerText     = Descriminant;
                            NotificationNode.AppendChild(ParamNode);

                            ParamNode               = NotificationXmlDoc.CreateElement("SENDERID");
                            ParamNode.InnerText     = str_AdminID;
                            NotificationNode.AppendChild(ParamNode);

                            MessageXmlNode          = NotificationXmlDoc.CreateElement("MESSAGE_XML");
                            NotificationNode.AppendChild(MessageXmlNode);

                            NotificationMessageNode = NotificationXmlDoc.CreateElement("NOTIFICATION_MESSAGE");
                            MessageXmlNode.AppendChild(NotificationMessageNode);

                            ParamNode               = NotificationXmlDoc.CreateElement("SUBJECT");
                            ParamNode.InnerText     = "";
                            ((XmlElement)ParamNode).SetAttribute("JobName", JobName);
                            NotificationMessageNode.AppendChild(ParamNode);

                            ParamNode               = NotificationXmlDoc.CreateElement("MESSAGE");                    
                            ((XmlElement)ParamNode).SetAttribute("JOBHASERROR", "true");
                            ((XmlElement)ParamNode).SetAttribute("JobName", JobName);
                            ((XmlElement)ParamNode).SetAttribute("SenderName", SenderName);
                            ((XmlElement)ParamNode).SetAttribute("SenderEmailID", SenderEmail);
                            ((XmlElement)ParamNode).SetAttribute("ERRORCODE", Obj_DAMJobCopyErrorCodes.ToString());
                            ((XmlElement)ParamNode).SetAttribute("RenamedJobName", str_NewJobName);
                            NotificationMessageNode.AppendChild(ParamNode);

                            ChannelCollectionNode   = NotificationXmlDoc.CreateElement("CHANNELCOLLECTION");
                            NotificationNode.AppendChild(ChannelCollectionNode);

                            ChannelNode             = NotificationXmlDoc.CreateElement("CHANNEL");
                            ((XmlElement)ChannelNode).SetAttribute("CHANNELTYPE", ChannelType);
                            ChannelCollectionNode.AppendChild(ChannelNode);

                            ReceipientCollectionNode = NotificationXmlDoc.CreateElement("RECIPIENTCOLLECTION");
                            ChannelNode.AppendChild(ReceipientCollectionNode);

                            ReceipientNode          = NotificationXmlDoc.CreateElement("RECIPIENT");
                            ReceipientCollectionNode.AppendChild(ReceipientNode);

                            ParamNode               = NotificationXmlDoc.CreateElement("EMAILID");
                            ParamNode.InnerText     = SendMailTo;
                            ReceipientNode.AppendChild(ParamNode);

                            ParamNode               = NotificationXmlDoc.CreateElement("RECIPIENTTYPE");
                            ParamNode.InnerText     = ReceipientType;
                            ReceipientNode.AppendChild(ParamNode);

                            NotificationXML         = NotificationNode.OuterXml;
                            
                            //Stamp Notification
                            NotificationProcessorObj    = new Notification(TenantInfoObj.TenantName);
                            ErrorXML                    = NotificationProcessorObj.StampNotification(NotificationXML);
                        }
                        #endregion
                        break;
                    default:
                        break;
                }               
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            return ErrorXML;
        }

        /// <summary>
        /// Used to get User preferences
        /// </summary>       
        private void GetUserCatalogerEmailPreferences()
        {
            #region Member Variables
            XML             str_UserPreferences             = null;
            XPathDocument   xp_UserPreferencesDoc           = null;
            XPathNavigator  CatalogerPreferencesNode        = null;
            ERRORXML        ErrorXML                        = null;
            StringBuilder   ErrorMessage                    = null;           
            XPathNavigator  xpNav_RootNode                  = null;
            #endregion

            try
            {

                ErrorMessage            = new StringBuilder();
                strdict_AdditionalInfo  = new StringDictionary();

                ErrorXML                = DAMWSObj.GetUserCatalogerPreferences(ref str_UserPreferences);
                if (String.IsNullOrEmpty(ErrorXML) && (str_UserPreferences.Length > 0))
                {
                    xp_UserPreferencesDoc       = new XPathDocument(new System.IO.StringReader(str_UserPreferences));
                    xpNav_RootNode              = xp_UserPreferencesDoc.CreateNavigator();                    

                    CatalogerPreferencesNode    = xpNav_RootNode.SelectSingleNode("USERCOLLECTION/USER/USER_DAMPERSONALISATION/DAMPERSONALIZATION/CONFIGURATIONXML/APPLICATIONSETTINGS/CATALOGERPREFERENCES");
                    
                    // User preferences
                    if (CatalogerPreferencesNode != null)
                    {
                        if (CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS") != null && !String.IsNullOrEmpty(CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS").Value))
                            SendMailTo = CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS").Value;
                        else
                            SendMailTo = SenderEmail;

                        if (CatalogerPreferencesNode.SelectSingleNode("EMAILNOTIFICATION") != null)
                        {
                            switch (CatalogerPreferencesNode.SelectSingleNode("EMAILNOTIFICATION").Value)
                            {
                                case "NEVER":
                                    SendMailSettings = "NEVER";
                                    break;
                                case "ONERROR":
                                    SendMailSettings = "ONERROR";
                                    break;
                                case "ALWAYS":
                                default: 
                                    SendMailSettings = "ALWAYS";
                                    break;
                            }
                        }
                    }
                    else                    
                        SendMailTo = SenderEmail;                    
                }
            }
            catch(Exception)
            {}
         }

        /// <summary>
        /// This method is used to set SenderName. SenderEmail for Stamping Notification.
        /// </summary>
        /// <param name="xpNav_UserXML"></param>
        private void SetNotificationSettings(XPathNavigator xpNav_UserXML)
        {            
            SenderName      = ALCommon.GetFormattedUserName(xpNav_UserXML);
            if (xpNav_UserXML.SelectSingleNode("USER/EMAILID") != null)
                SenderEmail = xpNav_UserXML.SelectSingleNode("USER/EMAILID").Value;
        }

        private EventLog CreateEventLog()
        {
            String strLogType = null;
            String strMachine = null;
            String strSource = null;
            String LogName = null;

            strLogType = "Application";
            strMachine = ".";
            strSource = TenantInfoObj.ServiceName;

            if (EventLog.SourceExists(strSource))
            {
                LogName = EventLog.LogNameFromSourceName(strSource, strMachine);
                if (LogName.ToUpper() != "APPLICATION")
                {
                    EventLog.DeleteEventSource(strSource);
                    EventLog.CreateEventSource(strSource, strLogType);
                }
            }
            else
            {
                EventLog.CreateEventSource(strSource, strLogType);
            }
            return new EventLog(strLogType, strMachine, strSource);
        }

        public ERRORXML GetCatalogerMTOMWS(ref String CatalogerMTOMWS)
        {
            ERRORXML        ErrorXml                        = null;
            String          CommunicationProtocol           = null; 
            String          CatalogerMachineIP              = null;
            String          IsCatalogerOnSolutionMachine    = "1";
            StringBuilder   ErrorBuilder                    = null;
            StringBuilder   CatalogerMTOMWSUrl              = new StringBuilder();

            try
            {
               IsCatalogerOnSolutionMachine = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerInstalled");
                
               if (IsCatalogerOnSolutionMachine.Trim().Equals("0"))
               {
                       CommunicationProtocol    = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK\CATALOGER", "Communication Protocol");
                       CatalogerMachineIP       = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK\CATALOGER", "Server Address");
               }
               else
               {
                   CommunicationProtocol        = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "Communication Protocol");
                   CatalogerMachineIP           = "localhost";
               }

               if ((! String.IsNullOrEmpty(CommunicationProtocol)) && (! String.IsNullOrEmpty(CatalogerMachineIP)))
               {
                   CatalogerMTOMWSUrl.AppendFormat("{0}{1}/alservices/ALMTOMWS.asmx", CommunicationProtocol,CatalogerMachineIP);
                   CatalogerMTOMWS      = CatalogerMTOMWSUrl.ToString();
               }
            }
            catch (Exception ExcepObj)
            {
                ErrorBuilder            = new StringBuilder();
                ErrorBuilder.AppendFormat("<ERROR><MESSAGE>{0}</MESSAGE></ERROR>", ExcepObj.Message);
                return ErrorBuilder.ToString();
            }
            return ErrorXml;
        }
        #endregion

    }
}
