using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;

using System.Web.UI.WebControls;
using System.IO;
using System.Collections.Specialized;
using System.Collections;
using System.Threading;
using System.Diagnostics;
using System.Reflection;
using Microsoft.Win32;
using System.Runtime.InteropServices;
using System.Security.Cryptography;
using System.Globalization;
using System.Configuration;
using System.Configuration.Internal;
using System.Runtime.CompilerServices;
using Microsoft.Win32.SafeHandles;
using Assetlink.IOProxy;

namespace Assetlink.CustomControls
{
    public class UploadComponent : WebControl
    {
        internal class DataStore
        {
            // Methods
            public DataStore()
            {
                this.mData = null;
                this.mForm = null;
                this.mThreadContinue = false;
                this.mThreadData = null;
                this.mThread = null;
            }



            public byte[] Add(byte[] theBuffer)
            {
                if (this.mThread != null)
                {
                    this.AddAsync(theBuffer);
                }
                else
                {
                    this.AddSync(theBuffer);
                }
                if (this.mForm != null)
                {
                    return this.mForm.CurrentFile;
                }
                return new byte[0];
            }


            private void AddAsync(byte[] theBuffer)
            {
                lock (this.mThreadData.SyncRoot)
                {
                    this.mThreadData.Add(theBuffer);
                }
            }


            private void AddSync(byte[] theBuffer)
            {
                if (theBuffer != null)
                {
                    if (this.mData != null)
                    {
                        this.mData.Add(theBuffer);
                    }
                    if (this.mForm != null)
                    {
                        this.mForm.AddData(theBuffer);
                    }
                }
            }


            public void Clear()
            {
                this.Flush();
                if (this.mData != null)
                {
                    this.mData.Clear();
                }
                if (this.mForm != null)
                {
                    this.mForm.Clear();
                }
            }


            private void ContinuousFlush()
            {
                while (this.mThreadContinue)
                {
                    Thread.Sleep(150);
                    this.FlushThreadData();
                }
                this.FlushThreadData();
            }


            ~DataStore()
            {
                try
                {
                    this.Flush();
                }
                catch
                {
                    return;
                }
            }


            public void FinishUp(HttpApplication inApp, Settings inSet)
            {
                this.Flush();
                if (this.mForm != null)
                {
                    this.mForm.ClearData();
                    if (inSet.Advanced.EnableLargeUploads)
                    {
                        inApp.Context.Items.Add(Multipart.Form.KeyName, this.mForm);
                    }
                }
            }


            private static byte[] Flatten(ArrayList inList)
            {
                int num1 = 0;
                int num2 = 0;
                int num3 = 0;
                int num4 = inList.Count;
                num2 = 0;
                while (num2 < num4)
                {
                    num3 += ((byte[])inList[num2]).Length;
                    num2++;
                }
                byte[] buffer1 = new byte[num3];
                for (num2 = 0; num2 < num4; num2++)
                {
                    byte[] buffer2 = (byte[])inList[num2];
                    buffer2.CopyTo(buffer1, num1);
                    num1 += buffer2.Length;
                    inList[num2] = null;
                }
                return buffer1;
            }


            private void Flush()
            {
                if (this.mThread != null)
                {
                    this.mThreadContinue = false;
                    if (!this.mThread.Join(0x2710))
                    {
                        Log.UniqueError("Thread failed to stop after ten seconds.");
                        this.mThread.Abort();
                    }
                    this.mThread = null;
                }
            }


            private void FlushThreadData()
            {
            Label_0000:
                try
                {
                    byte[] buffer1;
                    lock (this.mThreadData.SyncRoot)
                    {
                        if (this.mThreadData.Count <= 0)
                        {
                            return;
                        }
                        buffer1 = (byte[])this.mThreadData[0];
                        this.mThreadData.RemoveAt(0);
                    }
                    if (buffer1 != null)
                    {
                        this.AddSync(buffer1);
                    }
                    goto Label_0000;
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                }
            }


            public string GetBoundary()
            {
                this.Flush();
                string text1 = "";
                if ((this.mData != null) && (this.mData.Count > 0))
                {
                    text1 = Multipart.Boundary.GetContentType(new MemoryStream((byte[])this.mData[0]));
                }
                if (this.mForm != null)
                {
                    text1 = this.mForm.ContentType;
                }
                return text1;
            }


            public byte[] GetUploadData(int inBytesTotalHint)
            {
                this.Flush();
                if (this.mData != null)
                {
                    return DataStore.Flatten(this.mData);
                }
                if (this.mForm != null)
                {
                    return this.mForm.GetData(inBytesTotalHint);
                }
                return null;
            }


            public void Init(Settings inSet, int inCodePage)
            {
                if (inSet.Advanced.EnableLargeUploads || inSet.Advanced.FixCorruptUploads)
                {
                    this.mForm = new Multipart.Form(inCodePage, inSet.Advanced.EnableLargeUploads, false, inSet.Advanced.Optimization);
                    if (inSet.Advanced.MultiThreaded)
                    {
                        this.mThreadContinue = true;
                        this.mThreadData = new ArrayList();
                        this.mThread = new Thread(new ThreadStart(this.ContinuousFlush));
                        this.mThread.Start();
                    }
                }
                else
                {
                    this.mData = new ArrayList();
                }
            }



            // Fields
            private ArrayList mData;
            private Multipart.Form mForm;
            private Thread mThread;
            private bool mThreadContinue;
            private ArrayList mThreadData;
        }
        internal class FileUtils
        {


            // Methods
            public FileUtils()
            {
            }

            [DllImport("kernel32", CharSet = CharSet.Unicode, SetLastError = true)]
            private static extern int CreateFile(string filename, uint desiredAccess, uint shareMode, uint attributes, uint creationDisposition, uint flagsAndAttributes, uint templateFile);
            public static FileStream CreateFileStream(string inPath, FileMode inMode, FileAccess inAccess)
            {
                int num1 = inPath.LastIndexOf(':');
                if ((num1 > 2) && (num1 > inPath.LastIndexOf('\\')))
                {
                    return FileUtils.MakeFileStream(inPath, inMode, inAccess);
                }
                return new FileStream(inPath, inMode, inAccess, FileShare.Read);
            }


            public static FileStream MakeFileStream(string inPath, FileMode inMode, FileAccess inAccess)
            {
                uint num1;
                if (inAccess == FileAccess.Read)
                {
                    num1 = 0x80000000;
                }
                else if (inAccess == FileAccess.Write)
                {
                    num1 = 0x40000000;
                }
                else
                {
                    num1 = 0xc0000000;
                }
                int num2 = FileUtils.CreateFile(inPath, num1, 1, 0, (uint)inMode, 0x80, 0);
                if (num2 != -1)
                {

                    IntPtr iPtr = new IntPtr(num2);
                    SafeFileHandle handleValue = new SafeFileHandle(iPtr, true);

                    return new FileStream(handleValue, FileAccess.Write);


                }
                int num3 = Marshal.GetLastWin32Error();
                if (num3 == 2)
                {
                    throw new FileNotFoundException("File stream not found.", inPath);
                }
                if (num3 == 0x7b)
                {
                    throw new IOException("Unable to create named stream. Check that volume is NTFS formatted.");
                }
                throw new Win32Exception(num3);
            }


            public static byte[] ReadDataFromFile(string inPath)
            {
                FileStream stream1 = FileUtils.CreateFileStream(inPath, FileMode.Open, FileAccess.Read);
                byte[] buffer1 = new byte[stream1.Length];
                int num1 = stream1.Read(buffer1, 0, buffer1.Length);
                stream1.Close();
                if (num1 != buffer1.Length)
                {
                    throw new IOException("Could not read all data from file.");
                }
                return buffer1;
            }


            public static byte[] ReadDataFromStream(Stream inStream)
            {
                byte[] buffer1 = new byte[inStream.Length];
                int num1 = inStream.Read(buffer1, 0, buffer1.Length);
                if (num1 != buffer1.Length)
                {
                    throw new IOException("Could not read all data from stream.");
                }
                return buffer1;
            }


            public static void StreamToStream(Stream inSrc, Stream inDst)
            {
                FileUtils.StreamToStream(inSrc, (long)0, inDst, (long)0, inSrc.Length - inSrc.Position, 0x10000);
            }


            public static void StreamToStream(Stream inSrc, long inSrcPos, Stream inDst, long inDstPos, long inLen, int inChunk)
            {
                byte[] buffer1 = new byte[inChunk];
                while (inLen > 0)
                {
                    if (inLen < inChunk)
                    {
                        inChunk = (int)inLen;
                    }
                    inSrc.Read(buffer1, 0, inChunk);
                    inDst.Write(buffer1, 0, inChunk);
                    inLen -= inChunk;
                }
            }


            public static void WriteDataToFile(string inPath, bool inOverwrite, Stream inSource)
            {
                FileMode mode1 = inOverwrite ? FileMode.Create : FileMode.CreateNew;
                FileStream stream1 = FileUtils.CreateFileStream(inPath, mode1, FileAccess.Write);
                long num1 = inSource.Position;
                inSource.Position = 0;
                FileUtils.StreamToStream(inSource, stream1);
                inSource.Position = num1;
                stream1.Close();
            }

            public static void WriteDataToFile(string inPath, bool inOverwrite, byte[] inSource, int inStart, int inLength)
            {
                MemoryStream stream1 = new MemoryStream(inSource, inStart, inLength, false, false);
                FileUtils.WriteDataToFile(inPath, inOverwrite, stream1);
                stream1.Close();
            }

        }

        internal class HTTPUtils
        {
            // Methods
            public HTTPUtils()
            {
            }

            public static string Decode(string inValue)
            {
                string text1 = HttpUtility.UrlDecode(inValue, Encoding.ASCII);
                if (inValue != text1)
                {
                    return text1;
                }
                text1 = HttpUtility.HtmlDecode(inValue);
                if (inValue != text1)
                {
                    return text1;
                }
                return inValue;
            }

            public static string Decode(byte[] inValue, int inCodePage)
            {
                if (inValue == null)
                {
                    return "";
                }
                return HTTPUtils.Decode(Encoding.GetEncoding(inCodePage).GetString(inValue));
            }


            public static string GetRawPath(byte[] inPath, int inCodePage)
            {
                return HTTPUtils.Decode(inPath, inCodePage);
            }


            public static string GetURLSafeFileName(byte[] inPath, int inCodePage)
            {
                string text1 = HTTPUtils.Decode(inPath, inCodePage);
                if (HTTPUtils.IsMacPath(text1))
                {
                    return HTTPUtils.MakeURLSafeFileName(text1);
                }
                return HTTPUtils.MakeURLSafeFileName(Utilities.FindFileName(text1));
            }


            public static string GetWinSafeFileName(byte[] inPath, int inCodePage)
            {
                string text1 = HTTPUtils.Decode(inPath, inCodePage);
                if (HTTPUtils.IsMacPath(text1))
                {
                    return HTTPUtils.MakeMacFileName(text1);
                }
                return HTTPUtils.MakeSafeFileName(Utilities.FindFileName(text1));
            }


            private static bool IsAlphaNum(char inChar)
            {
                if ((inChar >= 'a') && (inChar <= 'z'))
                {
                    return true;
                }
                if ((inChar >= 'A') && (inChar <= 'Z'))
                {
                    return true;
                }
                if ((inChar >= '0') && (inChar <= '9'))
                {
                    return true;
                }
                return false;
            }


            private static bool IsMacPath(string inPath)
            {
                if (((inPath != null) && (inPath.Length > 0)) && (inPath[0] != '\\'))
                {
                    return (inPath.IndexOf(':') < 0);
                }
                return false;
            }


            private static string MakeMacFileName(string inName)
            {
                int num1 = 0;
                char[] chArray1 = inName.ToCharArray();
                num1 = 0;
                while (num1 < chArray1.Length)
                {
                    char ch1 = chArray1[num1];
                    if (ch1 <= '/')
                    {
                        if (ch1 == '"')
                        {
                            goto Label_0069;
                        }
                        if (ch1 == '*')
                        {
                            goto Label_0073;
                        }
                        if (ch1 == '/')
                        {
                            goto Label_007D;
                        }
                        goto Label_00CD;
                    }
                    if (ch1 <= '\\')
                    {
                        switch (ch1)
                        {
                            case ':':
                                {
                                    chArray1[num1] = (char)0xf022;
                                    goto Label_00E1;
                                }
                            case ';':
                            case '=':
                                {
                                    goto Label_00CD;
                                }
                            case '<':
                                {
                                    chArray1[num1] = (char)0xf023;
                                    goto Label_00E1;
                                }
                            case '>':
                                {
                                    chArray1[num1] = (char)0xf024;
                                    goto Label_00E1;
                                }
                            case '?':
                                {
                                    chArray1[num1] = (char)0xf025;
                                    goto Label_00E1;
                                }
                            case '\\':
                                {
                                    goto Label_00A5;
                                }
                        }
                        goto Label_00CD;
                    }
                    if (ch1 == '|')
                    {
                        goto Label_00AF;
                    }
                    if (ch1 == '\x00f0')
                    {
                        goto Label_00B9;
                    }
                    goto Label_00CD;
                Label_0069:
                    chArray1[num1] = (char)0xf020;
                    goto Label_00E1;
                Label_0073:
                    chArray1[num1] = (char)0xf021;
                    goto Label_00E1;
                Label_007D:
                    chArray1[num1] = (char)0xf022;
                    goto Label_00E1;
                Label_00A5:
                    chArray1[num1] = (char)0xf026;
                    goto Label_00E1;
                Label_00AF:
                    chArray1[num1] = (char)0xf027;
                    goto Label_00E1;
                Label_00B9:
                    chArray1[num1] = (char)0xf02a;
                    goto Label_00E1;
                Label_00CD:
                    if (chArray1[num1] <= '\x001f')
                    {
                        chArray1[num1] += (char)0xf000;
                    }
                Label_00E1:
                    num1++;
                }
                if ((num1 > 0) && (chArray1[num1 - 1] == ' '))
                {
                    chArray1[num1 - 1] = (char)0xf028;
                }
                else if ((num1 > 0) && (chArray1[num1 - 1] == '.'))
                {
                    chArray1[num1 - 1] = (char)0xf029;
                }
                return new string(chArray1);
            }



            private static string MakeSafeFileName(string inName)
            {
                int num1 = 0;
                char[] chArray1 = inName.ToCharArray();
                num1 = 0;
                while (num1 < chArray1.Length)
                {
                    char ch1 = chArray1[num1];
                    if (ch1 <= '/')
                    {
                        if (((ch1 == '"') || (ch1 == '*')) || (ch1 == '/'))
                        {
                            goto Label_0052;
                        }
                        goto Label_0059;
                    }
                    switch (ch1)
                    {
                        case ':':
                        case '<':
                        case '>':
                        case '?':
                        case '\\':
                            {
                                goto Label_0052;
                            }
                        case ';':
                        case '=':
                            {
                                goto Label_0059;
                            }
                        default:
                            {
                                if (ch1 != '|')
                                {
                                    goto Label_0059;
                                }
                                goto Label_0052;
                            }
                    }
                Label_0052:
                    chArray1[num1] = '_';
                    goto Label_0065;
                Label_0059:
                    if (chArray1[num1] < ' ')
                    {
                        chArray1[num1] = '_';
                    }
                Label_0065:
                    num1++;
                }
                num1 = chArray1.Length - 1;
                while (num1 >= 0)
                {
                    if ((chArray1[num1] != ' ') && (chArray1[num1] != '.'))
                    {
                        break;
                    }
                    num1--;
                }
                return new string(chArray1, 0, num1 + 1);
            }


            private static string MakeURLSafeFileName(string inName)
            {
                int num1 = 0;
                char[] chArray1 = inName.ToCharArray();
                for (num1 = 0; num1 < chArray1.Length; num1++)
                {
                    if (HTTPUtils.IsAlphaNum(chArray1[num1]))
                    {
                        chArray1[num1] = char.ToLower(chArray1[num1]);
                    }
                    else if ((chArray1[num1] != '-') && (chArray1[num1] != '.'))
                    {
                        chArray1[num1] = '_';
                    }
                }
                return new string(chArray1);
            }

        }

        internal class Log
        {
            // Methods
            static Log()
            {
                Log.sEventLog = null;
                Log.sErrorList = new ArrayList();
            }



            public Log()
            {
            }



            public static void Error(string inMsg)
            {
                Log.WriteEntry(inMsg, EventLogEntryType.Error);
            }


            public static void ExceptionDetails(Exception e)
            {
                try
                {
                    string text1 = string.Format("{0} threw exception '{1}'.\r\n\r\n{2}", e.Source, e.Message, e.StackTrace);
                    Log.WriteEntry(text1, EventLogEntryType.Error);
                }
                catch
                {
                }
            }


            private static EventLog GetEventLog()
            {
                if (Log.sEventLog == null)
                {
                    string text1 = Assembly.GetExecutingAssembly().GetName().Name;
                    if (!EventLog.SourceExists(text1))
                    {
                        EventLog.CreateEventSource(text1, "Application");
                    }
                    Log.sEventLog = new EventLog("Application", ".", text1);
                }
                return Log.sEventLog;
            }


            public static void Success(string inMsg)
            {
                Log.WriteEntry(inMsg, EventLogEntryType.SuccessAudit);
            }


            public static void UniqueError(string inMsg)
            {
                Log.UniqueError(inMsg, "");
            }


            public static void UniqueError(string inUniqueMsg, string inDescription)
            {
                if ((Log.sErrorList.Count <= 15) && !Log.sErrorList.Contains(inUniqueMsg))
                {
                    Log.sErrorList.Add(inUniqueMsg);
                    Log.Error(inUniqueMsg + inDescription);
                    if (Log.sErrorList.Count > 15)
                    {
                        Log.Error("Many errors have been raised. To avoid cluttering the Event Log no more will be recorded.");
                    }
                }
            }


            private static void WriteEntry(string inMsg, EventLogEntryType inType)
            {
                try
                {
                    Log.GetEventLog().WriteEntry(inMsg, inType);
                }
                catch
                {
                }
            }



            // Fields
            private static ArrayList sErrorList;
            private static EventLog sEventLog;
        }

        internal class MacBinary
        {
            // Methods
            public MacBinary()
            {
                SuperStream stream1;
                SuperStream stream2;
                int num1;
                int num2;
                int num3;
                byte num4;
                uint num5;
                this.mRezStream = (SuperStream)(stream1 = null);
                this.mDataStream = stream2 = stream1;
                this.mStream = stream2;
                this.mMIMEType = null;
                this.mRezLen = num1 = -1;
                this.mDataLen = num2 = num1;
                this.mRezFork = num3 = num2;
                this.mDataFork = num3;
                this.mAttr2 = (byte)(num4 = 0);
                this.mAttr1 = num4;
                this.mMacCreator = num5 = 0;
                this.mMacType = num5;
            }



            private int FindRezFork()
            {
                RezFork fork1 = new RezFork();
                int num1 = 0;
                int num2 = 0;
                int num3 = (this.mDataFork + this.mDataLen) + this.mPadding;
                fork1.SetData(new SuperStream(this.mStream, num3, this.mRezLen));
                if (fork1.IsOK())
                {
                    return num3;
                }
                num3 = this.mDataFork + this.mDataLen;
                num2 = this.mPadding + 0x80;
                if (num2 > ((this.Length - 0x80) - this.mDataLen))
                {
                    num2 = (this.Length - 0x80) - this.mDataLen;
                }
                for (num1 = 1; num1 <= num2; num1++)
                {
                    fork1.SetData(new SuperStream(this.mStream, num3, this.mRezLen));
                    if (fork1.IsOK())
                    {
                        return num3;
                    }
                    num3++;
                }
                return 0;
            }


            private SuperStream GetDataStream()
            {
                int num1 = this.mDataFork;
                int num2 = this.mDataLen;
                if ((num1 < 0) || (num2 < 0))
                {
                    num1 = num2 = 0;
                }
                return new SuperStream(this.mStream, num1, num2);
            }


            public string GetFileXt()
            {
                return MacFileType.InferFileType(this.mMacType, this.mMacCreator);
            }


            public int GetRezFork()
            {
                if (this.mRezFork < 0)
                {
                    this.mRezFork = this.FindRezFork();
                }
                return this.mRezFork;
            }


            private SuperStream GetRezStream()
            {
                int num1 = this.GetRezFork();
                int num2 = this.mRezLen;
                if ((num1 < 0) || (num2 < 0))
                {
                    num1 = num2 = 0;
                }
                return new SuperStream(this.mStream, num1, num2);
            }


            public bool IsOK()
            {
                if (((this.mMIMEType != "application/x-macbinary") && (this.mMIMEType != "")) && (this.mMIMEType != null))
                {
                    return false;
                }
                if (this.mStream == null)
                {
                    return false;
                }
                if (this.Length < 0x80)
                {
                    return false;
                }
                if (this.mDataLen < 0)
                {
                    return false;
                }
                if (this.mRezLen < 0)
                {
                    return false;
                }
                if ((this.mDataFork + this.mDataLen) > this.Length)
                {
                    return false;
                }
                int num1 = this.GetRezFork();
                if (num1 == 0)
                {
                    return false;
                }
                if ((num1 + this.mRezLen) > this.Length)
                {
                    return false;
                }
                return true;
            }


            public void SetData(SuperStream inStream, string inMIMEType)
            {
                SuperStream stream1;
                this.mStream = inStream;
                this.mRezStream = (SuperStream)(stream1 = null);
                this.mDataStream = stream1;
                this.mMIMEType = inMIMEType;
                if (this.mStream.Length >= 0x80)
                {
                    this.mDataFork = 0x80;
                    this.mRezFork = -1;
                    this.mStream.Position = 0x53;
                    this.mDataLen = this.mStream.GetMacLONG();
                    this.mStream.Position = 0x57;
                    this.mRezLen = this.mStream.GetMacLONG();
                    this.mPadding = ((this.mDataLen % 0x80) != 0) ? (0x80 - (this.mDataLen % 0x80)) : 0;
                    this.mStream.Position = 0x49;
                    this.mAttr1 = this.mStream.GetBYTE();
                    this.mStream.Position = 0x65;
                    this.mAttr2 = this.mStream.GetBYTE();
                    this.mStream.Position = 0x41;
                    this.mMacType = this.mStream.GetOSType();
                    this.mStream.Position = 0x45;
                    this.mMacCreator = this.mStream.GetOSType();
                }
                else
                {
                    int num1;
                    int num2;
                    int num3;
                    int num4;
                    byte num5;
                    uint num6;
                    this.mPadding = num1 = -1;
                    this.mRezLen = num2 = num1;
                    this.mDataLen = num3 = num2;
                    this.mRezFork = num4 = num3;
                    this.mDataFork = num4;
                    this.mAttr2 = (byte)(num5 = 0);
                    this.mAttr1 = num5;
                    this.mMacCreator = num6 = 0;
                    this.mMacType = num6;
                }
            }



            // Properties
            public byte Attr1
            {
                get
                {
                    return this.mAttr1;
                }
            }

            public byte Attr2
            {
                get
                {
                    return this.mAttr2;
                }
            }

            public Stream DataStream
            {
                get
                {
                    if (this.mDataStream == null)
                    {
                        this.mDataStream = this.GetDataStream();
                    }
                    return this.mDataStream;
                }
            }

            public int Length
            {
                get
                {
                    return (int)this.mStream.Length;
                }
            }

            public uint MacCreator
            {
                get
                {
                    return this.mMacCreator;
                }
            }

            public uint MacType
            {
                get
                {
                    return this.mMacType;
                }
            }

            public Stream RezStream
            {
                get
                {
                    if (this.mRezStream == null)
                    {
                        this.mRezStream = this.GetRezStream();
                    }
                    return this.mRezStream;
                }
            }


            // Fields
            private byte mAttr1;
            private byte mAttr2;
            private int mDataFork;
            private int mDataLen;
            private SuperStream mDataStream;
            private uint mMacCreator;
            private uint mMacType;
            private string mMIMEType;
            private int mPadding;
            private int mRezFork;
            private int mRezLen;
            private SuperStream mRezStream;
            private SuperStream mStream;
        }

        internal class MacFile
        {
            // Methods
            public MacFile(MacBinary inMacBinary)
            {
                this.mMacBinary = inMacBinary;
                this.mUseStreams = true;
                this.mOverwrite = false;
            }


            public void Save(string inPath)
            {
                this.SaveData(inPath);
                try
                {
                    if (this.mUseStreams)
                    {
                        this.SaveStreams(inPath);
                    }
                }
                catch (Exception exception1)
                {
                    File.Delete(inPath);
                    throw exception1;
                }
            }


            protected void SaveData(string inPath)
            {
                string text1 = inPath;
                FileUtils.WriteDataToFile(text1, this.mOverwrite, this.mMacBinary.DataStream);
            }


            protected void SaveStreams(string inPath)
            {
                string text1 = inPath + ":AFP_Resource";
                string text2 = inPath + ":AFP_AfpInfo";
                Stream stream1 = this.mMacBinary.RezStream;
                if (stream1.Length > 0)
                {
                    FileUtils.WriteDataToFile(text1, this.mOverwrite, stream1);
                }
                MacFile.AfpInfo info1 = new MacFile.AfpInfo();
                try
                {
                    info1.SetData(FileUtils.ReadDataFromFile(text2));
                }
                catch
                {
                }
                info1.SetData(this.mMacBinary.MacType, this.mMacBinary.MacCreator, this.mMacBinary.Attr1, this.mMacBinary.Attr2);
                FileUtils.WriteDataToFile(text2, this.mOverwrite, info1.data, 0, info1.size);
            }


            public void SetOverwrite(bool inOverwrite)
            {
                this.mOverwrite = inOverwrite;
            }


            public void SetStreams(bool inUseStreams)
            {
                this.mUseStreams = inUseStreams;
            }



            // Fields
            private MacBinary mMacBinary;
            private bool mOverwrite;
            private bool mUseStreams;

            // Nested Types
            public class AfpInfo
            {
                // Methods
                public AfpInfo()
                {
                    byte num1;
                    byte num2;
                    byte num3;
                    byte num4;
                    byte num5;
                    byte num6;
                    byte num7;
                    byte num8;
                    byte num9;
                    this.size = 60;
                    this.data = new byte[this.size];
                    this.data[0] = 0x41;
                    this.data[1] = 70;
                    this.data[2] = 80;
                    this.data[3] = 0;
                    this.data[7] = (byte)(num1 = 0);
                    this.data[6] = num2 = num1;
                    this.data[5] = num3 = num2;
                    this.data[4] = num3;
                    this.data[6] = 1;
                    this.data[11] = (byte)(num4 = 0);
                    this.data[10] = num5 = num4;
                    this.data[9] = num6 = num5;
                    this.data[8] = num6;
                    this.data[15] = (byte)(num7 = 0);
                    this.data[14] = num8 = num7;
                    this.data[13] = num9 = num8;
                    this.data[12] = num9;
                    this.data[15] = 0x80;
                }


                public void SetData(byte[] inData)
                {
                    int num1 = (inData.Length > this.size) ? this.size : inData.Length;
                    Array.Copy(inData, 0, this.data, 0, num1);
                }


                public void SetData(uint inType, uint inCreator, byte inAttr1, byte inAttr2)
                {
                    byte[] buffer1 = SuperStream.OSTypeToBytes(inType);
                    for (int num1 = 0; num1 < 4; num1++)
                    {
                        this.data[0x10 + num1] = buffer1[num1];
                    }
                    buffer1 = SuperStream.OSTypeToBytes(inCreator);
                    for (int num2 = 0; num2 < 4; num2++)
                    {
                        this.data[20 + num2] = buffer1[num2];
                    }
                    this.data[0x18] = (byte)(inAttr1 & 0xfc);
                    this.data[0x19] = (byte)(inAttr2 & 0xfc);
                }

                // Fields
                public byte[] data;
                public int size;
            }
        }

        internal class MacFileType
        {
            // Methods
            static MacFileType()
            {
                MacFileType.FileTypes = MacFileType.Init(new ArrayList());
                MacFileType.BlankType = SuperStream.StringToOSType("----");
            }



            public MacFileType()
            {
            }

            private static void AddEntry(ArrayList inList, string inType, string inCreator, string inXt, string inDesc)
            {
                MacFileType.FileTypeEntry entry1 = new MacFileType.FileTypeEntry();
                entry1.type = SuperStream.StringToOSType(inType);
                entry1.creator = SuperStream.StringToOSType(inCreator);
                entry1.xt = inXt;
                entry1.desc = inDesc;
                inList.Add(entry1);
            }


            public static string InferFileType(string inType, string inCreator)
            {
                return MacFileType.InferFileType(SuperStream.StringToOSType(inType), SuperStream.StringToOSType(inCreator));
            }


            public static string InferFileType(uint inType, uint inCreator)
            {
                string text1 = "";
                try
                {
                    text1 = MacFileType.InferFileTypeSFM(inType, inCreator);
                }
                catch (Exception exception1)
                {
                    Log.UniqueError("Unable to read Services for Macintosh prefrences from Registry. " + exception1.Message);
                }
                if (text1 == "")
                {
                    text1 = MacFileType.InferFileTypeHW(inType, inCreator);
                    if (text1 == "")
                    {
                        text1 = SuperStream.OSTypeToString(inType);
                    }
                }
                return text1;
            }


            private static string InferFileTypeHW(uint inType, uint inCreator)
            {
                for (int num1 = 0; num1 < MacFileType.FileTypes.Count; num1++)
                {
                    MacFileType.FileTypeEntry entry1 = (MacFileType.FileTypeEntry)MacFileType.FileTypes[num1];
                    bool flag1 = (entry1.type == inType) || (entry1.type == MacFileType.BlankType);
                    bool flag2 = (entry1.creator == inCreator) || (entry1.creator == MacFileType.BlankType);
                    if (flag1 && flag2)
                    {
                        return entry1.xt;
                    }
                }
                return "";
            }


            private static string InferFileTypeSFM(uint inType, uint inCreator)
            {
                string text1 = "TYPE=" + SuperStream.OSTypeToString(inType).Trim();
                string text2 = "CREATOR=" + SuperStream.OSTypeToString(inCreator).Trim();
                string text3 = null;
                string text4 = "";
                RegistryKey key1 = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\Services\MacFile\Parameters\Type_Creators\");
                string[] textArray1 = key1.GetValueNames();
                string[] textArray2 = textArray1;
                for (int num1 = 0; num1 < textArray2.Length; num1++)
                {
                    string text5 = textArray2[num1];
                    try
                    {
                        Array array1 = (Array)key1.GetValue(text5);
                        if ((((string)array1.GetValue(0)) == text2) && (((string)array1.GetValue(1)) == text1))
                        {
                            text3 = "ID=" + text5;
                            break;
                        }
                    }
                    catch
                    {
                    }
                }
                key1.Close();
                if (text3 != null)
                {
                    key1 = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\Services\MacFile\Parameters\Extensions\");
                    textArray1 = key1.GetValueNames();
                    string[] textArray3 = textArray1;
                    for (int num2 = 0; num2 < textArray3.Length; num2++)
                    {
                        string text6 = textArray3[num2];
                        try
                        {
                            Array array2 = (Array)key1.GetValue(text6);
                            if (((string)array2.GetValue(0)) == text3)
                            {
                                text4 = text6;
                                break;
                            }
                        }
                        catch
                        {
                        }
                    }
                    key1.Close();
                }
                return text4;
            }



            private static ArrayList Init(ArrayList inList)
            {
                MacFileType.AddEntry(inList, "AB65", "AD65", "P65", "Pagemaker 6.5");
                MacFileType.AddEntry(inList, "ALB4", "ALD4", "PM4", "Pagemaker 4");
                MacFileType.AddEntry(inList, "ALB5", "ALD5", "PM5", "Pagemaker 5");
                MacFileType.AddEntry(inList, "ALB6", "ALD6", "PM6", "Pagemaker 6");
                MacFileType.AddEntry(inList, "ART3", "TEXT", "AI", "Illustrator 3");
                MacFileType.AddEntry(inList, "ART4", "TEXT", "AI", "Illustrator");
                MacFileType.AddEntry(inList, "ART5", "TEXT", "AI", "Illustrator");
                MacFileType.AddEntry(inList, "DPRO", "dDOC", "MDP", "MacDraw Pro");
                MacFileType.AddEntry(inList, "FH50", "AGD1", "FH5", "Freehand 5");
                MacFileType.AddEntry(inList, "FHD3", "FHA3", "FH3", "Freehand 3");
                MacFileType.AddEntry(inList, "FMP3", "FMP3", "FP3", "FileMaker Pro");
                MacFileType.AddEntry(inList, "FXTC", "EggP", "AEP", "After Effects Project");
                MacFileType.AddEntry(inList, "InDd", "inDn", "IND", "Indesign Document");
                MacFileType.AddEntry(inList, "MV93", "MD93", "DIR", "Macromedia Director");
                MacFileType.AddEntry(inList, "StMI", "TEXT", "HTM", "PageMill HTML page");
                MacFileType.AddEntry(inList, "TEXT", "BNHQ", "HQX", "BinHex");
                MacFileType.AddEntry(inList, "WP42", "SSIW", "WP4", "Wordperfect 4");
                MacFileType.AddEntry(inList, "WP50", "SSIW", "WP5", "Wordperfect 5");
                MacFileType.AddEntry(inList, "ZIP ", "ZIP ", "ZIP", "Zip Archive");
                MacFileType.AddEntry(inList, "PICT", "----", "PCT", "Generic PICT");
                MacFileType.AddEntry(inList, "JPEG", "----", "JPG", "Generic JPEG");
                MacFileType.AddEntry(inList, "GIFf", "----", "GIF", "Generic GIF");
                MacFileType.AddEntry(inList, "PNGf", "----", "PNG", "Generic PNG");
                MacFileType.AddEntry(inList, "BMP ", "----", "BMP", "Generic BMP");
                MacFileType.AddEntry(inList, "TIFF", "----", "TIF", "Generic TIFF");
                MacFileType.AddEntry(inList, "EPSF", "----", "EPS", "Generic EPS");
                MacFileType.AddEntry(inList, "MooV", "----", "MOV", "Generic Movie");
                MacFileType.AddEntry(inList, "TEXT", "----", "TXT", "Generic Text");
                MacFileType.AddEntry(inList, "HTML", "----", "HTM", "Generic HTML");
                MacFileType.AddEntry(inList, "PDF ", "----", "PDF", "Portable Document Format");
                MacFileType.AddEntry(inList, "RTF ", "----", "RTF", "Rich Text Format");
                MacFileType.AddEntry(inList, "FMP3", "----", "FP3", "FileMaker Pro");
                MacFileType.AddEntry(inList, "XLBN", "----", "XLS", "Excel 1");
                MacFileType.AddEntry(inList, "XLS4", "----", "XLS", "Excel 4");
                MacFileType.AddEntry(inList, "XLS5", "----", "XLS", "Excel 5");
                MacFileType.AddEntry(inList, "XLS8", "----", "XLS", "Excel 98");
                MacFileType.AddEntry(inList, "sLS8", "----", "XLS", "Excel 98 Template");
                MacFileType.AddEntry(inList, "SLD3", "----", "PPT", "Power Point 3");
                MacFileType.AddEntry(inList, "SLD8", "----", "PPT", "Power Point 98");
                MacFileType.AddEntry(inList, "PPOT", "----", "PPT", "Power Point 98 Template");
                MacFileType.AddEntry(inList, "XDOC", "----", "QXD", "Quark Xpress (all versions)");
                MacFileType.AddEntry(inList, "W6BN", "----", "DOC", "Microsoft Word 6");
                MacFileType.AddEntry(inList, "W8BN", "----", "DOC", "Microsoft Word 98");
                MacFileType.AddEntry(inList, "WDBN", "----", "DOC", "Microsoft Word 3/4/5");
                MacFileType.AddEntry(inList, "WTBN", "----", "DOT", "Microsoft Word Template");
                MacFileType.AddEntry(inList, "8BIM", "----", "PSD", "Photoshop 3");
                MacFileType.AddEntry(inList, "8BPS", "----", "PSD", "Photoshop 3 (calques)");
                MacFileType.AddEntry(inList, "----", "BOBO", "CWK", "Claris Works");
                MacFileType.AddEntry(inList, "----", "FMP3", "FP3", "FileMaker Pro");
                MacFileType.AddEntry(inList, "----", "XCEL", "XLS", "Excel");
                MacFileType.AddEntry(inList, "----", "PPT3", "PPT", "Power Point");
                MacFileType.AddEntry(inList, "----", "XPR3", "QXD", "Quark Xpress");
                MacFileType.AddEntry(inList, "----", "MSWD", "DOC", "Microsoft Word");
                MacFileType.AddEntry(inList, "----", "8BIM", "PSD", "Photoshop Document");
                return inList;
            }




            // Fields
            private static uint BlankType;
            private static ArrayList FileTypes;
            private const string RegPathMacFileExts = @"SYSTEM\CurrentControlSet\Services\MacFile\Parameters\Extensions\";
            private const string RegPathMacFilePars = @"SYSTEM\CurrentControlSet\Services\MacFile\Parameters\";
            private const string RegPathMacFileType = @"SYSTEM\CurrentControlSet\Services\MacFile\Parameters\Type_Creators\";

            // Nested Types
            public class FileTypeEntry
            {
                // Methods
                public FileTypeEntry()
                {
                }

                // Fields
                public uint creator;
                public string desc;
                public uint type;
                public string xt;
            }
        }

        internal class Multipart
        {
            // Methods
            static Multipart()
            {
                Multipart.sCRLF = Encoding.ASCII.GetBytes("\r\n");
                Multipart.sDashDash = Encoding.ASCII.GetBytes("--");
                Multipart.sFileReplacement = Encoding.ASCII.GetBytes("Uploaded File At:");
                Multipart.sBoundary = Encoding.ASCII.GetBytes("boundary=");
                Multipart.sContentType = "multipart/form-data; boundary=";
            }



            public Multipart()
            {
            }

            // Fields
            private static byte[] sBoundary;
            private static string sContentType;
            private static byte[] sCRLF;
            private static byte[] sDashDash;
            private static byte[] sFileReplacement;

            // Nested Types
            internal class Boundary
            {
                // Methods
                public Boundary()
                {
                }
                public static string GetContentType(byte[] inBoundary)
                {
                    if ((inBoundary == null) || (inBoundary.Length <= 2))
                    {
                        return "";
                    }
                    byte[] buffer1 = new byte[inBoundary.Length - 2];
                    Array.Copy(inBoundary, 2, buffer1, 0, inBoundary.Length - 2);
                    inBoundary = buffer1;
                    return (Multipart.sContentType + Encoding.ASCII.GetString(inBoundary));
                }


                public static string GetContentType(Stream inStream)
                {
                    return Multipart.Boundary.GetContentType(Multipart.Boundary.GetMultipartBoundary(inStream));
                }


                public static byte[] GetMultipartBoundary(Stream inStream)
                {
                    SuperStream stream1 = new SuperStream(inStream);
                    byte[] buffer1 = new byte[0];
                    long num1 = 0;
                    long num2 = 0;
                    long num3 = 0;
                    bool flag1 = false;
                    num1 = stream1.FindStr(Multipart.sDashDash);
                    if (num1 >= 0)
                    {
                        num2 = stream1.FindStr(Multipart.sCRLF);
                        if (num2 < 0)
                        {
                            return buffer1;
                        }
                        if (num1 >= Multipart.sBoundary.Length)
                        {
                            stream1.Position = num1 - Multipart.sBoundary.Length;
                            if (stream1.EqualStr(Multipart.sBoundary))
                            {
                                flag1 = true;
                            }
                        }
                        stream1.Position = num1;
                        num3 = num2 - num1;
                        if (flag1)
                        {
                            num3 += 2;
                        }
                        buffer1 = new byte[num3];
                        if (flag1)
                        {
                            Multipart.sDashDash.CopyTo(buffer1, 0);
                            stream1.Read(buffer1, 2, ((int)num3) - 2);
                            return buffer1;
                        }
                        stream1.Read(buffer1, 0, (int)num3);
                    }
                    return buffer1;
                }


                public static byte[] GetSafeMultipartBoundary(Stream inStream)
                {
                    SuperStream stream1 = new SuperStream(inStream);
                    byte[] buffer1 = Multipart.Boundary.GetMultipartBoundary(inStream);
                    long num1 = 0;
                    if (buffer1.Length > 0x20)
                    {
                        byte[] buffer2 = new byte[0x20];
                        Array.Copy(buffer1, 0, buffer2, 0, 0x20);
                        buffer1 = buffer2;
                    }
                    stream1.Position = 0;
                    num1 = stream1.FindStr(buffer1);
                    if ((num1 < 0) && (buffer1.Length > 2))
                    {
                        byte[] buffer3 = new byte[buffer1.Length - 2];
                        Array.Copy(buffer1, 2, buffer3, 0, buffer1.Length - 2);
                        buffer1 = buffer3;
                        stream1.Position = 0;
                        num1 = stream1.FindStr(buffer1);
                        if (num1 < 0)
                        {
                            buffer1 = new byte[0];
                        }
                    }
                    return buffer1;
                }



            }

            internal class Field
            {
                // Methods
                public Field(bool inDumpFiles, bool inReserializeFiles)
                {
                    this.mDumpFiles = inDumpFiles;
                    this.mReserialize = inReserializeFiles;
                    this.mFileName = null;
                    this.mFieldName = null;
                    this.mMIMEType = null;
                    this.mAttributes = new ArrayList();
                    this.mData = new ArrayList();
                    this.mTempFile = null;
                    this.mDataLen = 0;
                }



                public void AddAttribute(byte[] inAttribute, int inCodePage)
                {
                    this.mAttributes.Add(inAttribute);
                    this.ProcessAttribute(inAttribute, inCodePage);
                    this.MakeTempFile();
                }


                public void AddData(byte[] inData)
                {
                    this.mDataLen += inData.Length;
                    this.mData.Add(inData);
                    if (this.mTempFile != null)
                    {
                        this.WriteData();
                    }
                    if ((this.mDumpFiles && (this.mTempFile == null)) && (this.mDataLen > 0x4000000))
                    {
                        Log.UniqueError("Large corrupt upload detected. Flushing data store.");
                        this.mData.Clear();
                    }
                }


                public void AddData(SuperStream inStream, long inLen)
                {
                    if (inLen > 0)
                    {
                        byte[] buffer1 = new byte[inLen];
                        inStream.Read(buffer1, 0, (int)inLen);
                        this.AddData(buffer1);
                    }
                }


                public void ClearData()
                {
                    this.mAttributes.Clear();
                    this.mData.Clear();
                }


                public void ClearFiles()
                {
                    this.mAttributes.Clear();
                    if (this.mTempFile != null)
                    {
                        this.mTempFile.Dispose();
                    }
                    this.mTempFile = null;
                }


                public void Finish()
                {
                    if (this.mTempFile != null)
                    {
                        this.mTempFile.CloseStream();
                    }
                }


                private byte[] GetAttributeValue(SuperStream inStream, string inValueName)
                {
                    byte[] buffer1 = Encoding.ASCII.GetBytes(inValueName);
                    byte[] buffer2 = null;
                    long num1 = inStream.FindStr(buffer1);
                    num1 = inStream.FindStr(Encoding.ASCII.GetBytes("="));
                    num1 = inStream.FindStr(Encoding.ASCII.GetBytes("\""));
                    if (num1 < 0)
                    {
                        return null;
                    }
                    long num2 = inStream.FindStr(Encoding.ASCII.GetBytes("\";"));
                    if (num2 < 0)
                    {
                        num2 = inStream.Length - 1;
                    }
                    num1++;
                    num2--;
                    long num3 = (num2 - num1) + 1;
                    if (num3 < 0)
                    {
                        return null;
                    }
                    buffer2 = new byte[num3];
                    inStream.Position = num1;
                    inStream.Read(buffer2, 0, (int)num3);
                    return buffer2;
                }


                private void MakeTempFile()
                {
                    if ((this.mDumpFiles && (this.mFileName != null)) && (this.mTempFile == null))
                    {
                        this.mTempFile = new TempFile();
                        if (!this.mTempFile.Writable)
                        {
                            this.mDumpFiles = false;
                            this.mTempFile = null;
                            Log.UniqueError("Unable to create temporary file. ");
                        }
                    }
                }


                private void ProcessAttribute(byte[] inData, int inCodePage)
                {
                    SuperStream stream1 = new SuperStream(new MemoryStream(inData));
                    string text1 = stream1.ToString().ToLower();
                    if (text1.StartsWith("content-disposition:"))
                    {
                        this.mFieldName = HTTPUtils.Decode(this.GetAttributeValue(stream1, "name"), inCodePage);
                        this.mFileName = this.GetAttributeValue(stream1, "filename");
                    }
                    else if (text1.StartsWith("content-type:"))
                    {
                        this.mMIMEType = text1.Substring(text1.IndexOf(":") + 1).Trim();
                    }
                }


                public void Serialize(Stream inStream)
                {
                    byte[] buffer1;
                    int num1 = 0;
                    while (num1 < this.mAttributes.Count)
                    {
                        buffer1 = (byte[])this.mAttributes[num1];
                        inStream.Write(buffer1, 0, buffer1.Length);
                        inStream.Write(Multipart.sCRLF, 0, Multipart.sCRLF.Length);
                        num1++;
                    }
                    inStream.Write(Multipart.sCRLF, 0, Multipart.sCRLF.Length);
                    if (this.mData.Count > 0)
                    {
                        for (num1 = 0; num1 < this.mData.Count; num1++)
                        {
                            buffer1 = (byte[])this.mData[num1];
                            inStream.Write(buffer1, 0, buffer1.Length);
                            this.mData[num1] = null;
                        }
                    }
                    else if (this.mTempFile != null)
                    {
                        if (this.mReserialize)
                        {
                            FileUtils.StreamToStream(this.mTempFile.FileStream, inStream);
                        }
                        else
                        {
                            inStream.Write(Multipart.sFileReplacement, 0, Multipart.sFileReplacement.Length);
                            buffer1 = Encoding.ASCII.GetBytes(this.mTempFile.Path);
                            inStream.Write(buffer1, 0, buffer1.Length);
                        }
                    }
                    inStream.Write(Multipart.sCRLF, 0, Multipart.sCRLF.Length);
                }


                private void WriteData()
                {
                    try
                    {
                        FileStream stream1 = this.mTempFile.FileStream;
                        for (int num1 = 0; num1 < this.mData.Count; num1++)
                        {
                            byte[] buffer1 = (byte[])this.mData[num1];
                            stream1.Write(buffer1, 0, buffer1.Length);
                        }
                        this.mData.Clear();
                    }
                    catch
                    {
                    }
                }



                // Properties
                public string FieldName
                {
                    get
                    {
                        return this.mFieldName;
                    }
                }

                public byte[] FileName
                {
                    get
                    {
                        return this.mFileName;
                    }
                }

                public bool IsFileField
                {
                    get
                    {
                        return (this.mFileName != null);
                    }
                }

                public string MIMEType
                {
                    get
                    {
                        return this.mMIMEType;
                    }
                }

                public TempFile TempFile
                {
                    get
                    {
                        return this.mTempFile;
                    }
                }


                // Fields
                private ArrayList mAttributes;
                private ArrayList mData;
                private long mDataLen;
                private bool mDumpFiles;
                private string mFieldName;
                private byte[] mFileName;
                private string mMIMEType;
                private bool mReserialize;
                private TempFile mTempFile;
            }

            internal class Form
            {
                // Methods
                public Form(int inCodePage, bool inDumpFiles, bool inReserializeFiles, bool inOptimize)
                {
                    this.mCodePage = inCodePage;
                    this.mDumpFiles = inDumpFiles;
                    this.mReserialize = inReserializeFiles;
                    this.mOptimize = inOptimize;
                    this.mFields = new ArrayList();
                    this.mBoundaryFull = null;
                    this.mBoundarySafe = null;
                    this.mBuffer = null;
                    this.mState = Multipart.Form.State.GetBoundary;
                    this.mCurrentItem = null;
                }


                private void AddBuffers(byte[] inBuffer)
                {
                    if (this.mBuffer == null)
                    {
                        this.mBuffer = inBuffer;
                    }
                    else
                    {
                        byte[] buffer1 = new byte[this.mBuffer.Length + inBuffer.Length];
                        this.mBuffer.CopyTo(buffer1, 0);
                        inBuffer.CopyTo(buffer1, this.mBuffer.Length);
                        this.mBuffer = buffer1;
                    }
                }


                public void AddData(byte[] inBuffer)
                {
                    bool flag1 = true;
                    this.AddBuffers(inBuffer);
                    SuperStream stream1 = new SuperStream(new MemoryStream(this.mBuffer));
                    if ((this.mOptimize && (this.mState == Multipart.Form.State.ReadData)) && this.IsAllData(this.mBoundarySafe, this.mBuffer, 0))
                    {
                        this.CurrentField.AddData(this.mBuffer);
                        stream1.Position = stream1.Length;
                        flag1 = false;
                    }
                    while (flag1)
                    {
                        long num1;
                        long num2;
                        switch (this.mState)
                        {
                            case Multipart.Form.State.Invalid:
                                {
                                    stream1.Position = 0;
                                    flag1 = false;
                                    continue;
                                }
                            case Multipart.Form.State.GetBoundary:
                                {
                                    this.mBoundaryFull = Multipart.Boundary.GetMultipartBoundary(stream1);
                                    this.mBoundarySafe = Multipart.Boundary.GetSafeMultipartBoundary(stream1);
                                    flag1 = (this.mBoundaryFull.Length > 0) && (this.mBoundarySafe.Length > 0);
                                    stream1.Position = 0;
                                    if (flag1)
                                    {
                                        this.mState = Multipart.Form.State.FindFirstBoundary;
                                    }
                                    continue;
                                }
                            case Multipart.Form.State.FindFirstBoundary:
                                {
                                    num2 = stream1.FindStr(this.mBoundarySafe);
                                    this.mState = (num2 >= 0) ? Multipart.Form.State.FindStartOfData : Multipart.Form.State.Invalid;
                                    continue;
                                }
                            case Multipart.Form.State.FindStartOfData:
                                {
                                    num1 = stream1.Position;
                                    num2 = stream1.FindStr(Multipart.sCRLF);
                                    if (num2 < num1)
                                    {
                                        goto Label_0138;
                                    }
                                    this.mState = Multipart.Form.State.ReadAttribute;
                                    continue;
                                }
                            case Multipart.Form.State.ReadAttribute:
                                {
                                    num1 = stream1.Position;
                                    num2 = stream1.FindStr(Multipart.sCRLF);
                                    if (num2 != num1)
                                    {
                                        goto Label_0169;
                                    }
                                    this.mState = Multipart.Form.State.ReadData;
                                    continue;
                                }
                            case Multipart.Form.State.ReadData:
                                {
                                    goto Label_01C2;
                                }
                            default:
                                {
                                    continue;
                                }
                        }
                    Label_0138:
                        flag1 = false;
                        stream1.Position = num1;
                        continue;
                    Label_0169:
                        if (num2 > num1)
                        {
                            byte[] buffer1 = new byte[num2 - num1];
                            stream1.Position = num1;
                            stream1.Read(buffer1, 0, buffer1.Length);
                            stream1.Position += 2;
                            this.CurrentField.AddAttribute(buffer1, this.mCodePage);
                            continue;
                        }
                        flag1 = false;
                        stream1.Position = num1;
                        continue;
                    Label_01C2:
                        num1 = stream1.Position;
                        num2 = stream1.FindStr(this.mBoundarySafe);
                        if (num2 >= num1)
                        {
                            stream1.Position = num1;
                            num2 -= num1 + 2;
                            if (num2 > 0)
                            {
                                this.CurrentField.AddData(stream1, num2);
                            }
                            this.CurrentField.Finish();
                            stream1.Position += this.mBoundarySafe.Length + 2;
                            this.mCurrentItem = null;
                            this.mState = Multipart.Form.State.FindStartOfData;
                            continue;
                        }
                        stream1.Position = num1;
                        num2 = ((stream1.Length - num1) - this.mBoundarySafe.Length) - 2;
                        if (num2 > 0)
                        {
                            this.CurrentField.AddData(stream1, num2);
                        }
                        flag1 = false;
                    }
                    this.TidyBuffers((int)stream1.Position);
                }


                public void Clear()
                {
                    this.ClearData();
                    this.ClearFiles();
                }

                public void ClearData()
                {
                    for (int num1 = 0; num1 < this.mFields.Count; num1++)
                    {
                        ((Multipart.Field)this.mFields[num1]).ClearData();
                    }
                }


                public void ClearFiles()
                {
                    for (int num1 = 0; num1 < this.mFields.Count; num1++)
                    {
                        ((Multipart.Field)this.mFields[num1]).ClearFiles();
                    }
                }


                ~Form()
                {
                    this.ClearData();
                    this.ClearFiles();
                }


                private void Finish()
                {
                    if (this.mCurrentItem != null)
                    {
                        this.mCurrentItem.Finish();
                        this.mCurrentItem = null;
                    }
                }



                public byte[] GetData(int inExpectedSize)
                {
                    if (this.mDumpFiles && !this.mReserialize)
                    {
                        inExpectedSize = 0;
                    }
                    this.Finish();
                    MemoryStream stream1 = new MemoryStream(inExpectedSize);
                    this.Serialize(stream1);
                    stream1.Close();
                    return stream1.GetBuffer();
                }


                private bool IsAllData(byte[] inBoundary, byte[] inBuffer, int inStart)
                {
                    int num2 = inBoundary.Length;
                    int num1 = 0;
                    while (num1 < num2)
                    {
                        if (inBoundary[num1] != 0x2d)
                        {
                            break;
                        }
                        num1++;
                    }
                    if (num1 < 5)
                    {
                        return false;
                    }
                    num2 = inBuffer.Length;
                    if (num2 < 10)
                    {
                        return false;
                    }
                    int num3 = num2 - 2;
                    for (int num4 = inStart + 2; num4 < num3; num4 += num1)
                    {
                        if (inBuffer[num4] != 0x2d)
                        {
                            //num4 += num1;
                            continue;
                        }
                        if ((inBuffer[num4 + 1] == 0x2d) && (inBuffer[num4 + 2] == 0x2d))
                        {
                            return false;
                        }
                        if ((inBuffer[num4 - 1] == 0x2d) && (inBuffer[num4 - 2] == 0x2d))
                        {
                            return false;
                        }
                    }
                    if (inBuffer[num2 - 1] == 0x2d)
                    {
                        if (inBuffer[num2 - 2] == 0x2d)
                        {
                            return false;
                        }
                        if (inBuffer[num2 - 2] == 10)
                        {
                            return false;
                        }
                    }
                    else if (inBuffer[num2 - 1] == 10)
                    {
                        if (inBuffer[num2 - 2] == 13)
                        {
                            return false;
                        }
                    }
                    else if (inBuffer[num2 - 1] == 13)
                    {
                        return false;
                    }
                    return true;
                }


                public void Serialize(Stream inStream)
                {
                    if (this.IsMultipart)
                    {
                        for (int num1 = 0; num1 < this.mFields.Count; num1++)
                        {
                            inStream.Write(this.mBoundaryFull, 0, this.mBoundaryFull.Length);
                            inStream.Write(Multipart.sCRLF, 0, Multipart.sCRLF.Length);
                            Multipart.Field field1 = (Multipart.Field)this.mFields[num1];
                            field1.Serialize(inStream);
                        }
                        inStream.Write(this.mBoundaryFull, 0, this.mBoundaryFull.Length);
                        inStream.Write(Multipart.sDashDash, 0, Multipart.sDashDash.Length);
                        inStream.Write(Multipart.sCRLF, 0, Multipart.sCRLF.Length);
                    }
                    else if (this.mBuffer != null)
                    {
                        inStream.Write(this.mBuffer, 0, this.mBuffer.Length);
                    }
                }


                private void TidyBuffers(int inBytesUsed)
                {
                    byte[] buffer1 = null;
                    int num1 = this.mBuffer.Length - inBytesUsed;
                    if (num1 > 0)
                    {
                        buffer1 = new byte[num1];
                        Array.Copy(this.mBuffer, inBytesUsed, buffer1, 0, num1);
                    }
                    this.mBuffer = buffer1;
                }



                // Properties
                public string ContentType
                {
                    get
                    {
                        return Multipart.Boundary.GetContentType(this.mBoundaryFull);
                    }
                }

                public int Count
                {
                    get
                    {
                        return this.mFields.Count;
                    }
                }

                private Multipart.Field CurrentField
                {
                    get
                    {
                        if (this.mCurrentItem == null)
                        {
                            this.mCurrentItem = new Multipart.Field(this.mDumpFiles, this.mReserialize);
                            this.mFields.Add(this.mCurrentItem);
                        }
                        return this.mCurrentItem;
                    }
                }

                public byte[] CurrentFile
                {
                    get
                    {
                        if (this.Count > 0)
                        {
                            return this[this.Count - 1].FileName;
                        }
                        return new byte[0];
                    }
                }

                public bool IsMultipart
                {
                    get
                    {
                        if (this.mState != Multipart.Form.State.Invalid)
                        {
                            return (this.mState != Multipart.Form.State.GetBoundary);
                        }
                        return false;
                    }
                }

                public Multipart.Field this[int inIndex]
                {
                    get
                    {
                        return (Multipart.Field)this.mFields[inIndex];
                    }
                }

                public static string KeyName
                {
                    get
                    {
                        return "UploadComponent.Multipart.Form";
                    }
                }


                // Fields
                private byte[] mBoundaryFull;
                private byte[] mBoundarySafe;
                private byte[] mBuffer;
                private int mCodePage;
                private Multipart.Field mCurrentItem;
                private bool mDumpFiles;
                private ArrayList mFields;
                private bool mOptimize;
                private bool mReserialize;
                private State mState;

                // Nested Types
                private enum State
                {
                    Invalid,
                    GetBoundary,
                    FindFirstBoundary,
                    FindStartOfData,
                    ReadAttribute,
                    ReadData
                }
            }
        }

        public class Progress
        {
            // Methods
            internal Progress()
            {
                throw new InvalidOperationException();
            }


            public Progress(int ID)
            {
                //Registration.CheckValid();
                try
                {
                    this.mID = ID;
                    if (HttpContext.Current != null)
                    {
                        this.mCodePage = HttpContext.Current.Request.ContentEncoding.CodePage;
                    }
                    else
                    {
                        this.mCodePage = 0;
                    }
                    this.mInfo = SharedState.Find(this.mID);
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }


            public Progress(string ID)
            {
                //Registration.CheckValid();
                try
                {
                    try
                    {
                        this.mID = Convert.ToInt32(ID);
                    }
                    catch
                    {
                        this.mID = 0;
                    }
                    if (HttpContext.Current != null)
                    {
                        this.mCodePage = HttpContext.Current.Request.ContentEncoding.CodePage;
                    }
                    else
                    {
                        this.mCodePage = 0;
                    }
                    this.mInfo = SharedState.Find(this.mID);
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }


            public static int UniqueID()
            {
                int num1;
                try
                {
                    byte[] buffer1 = new byte[4];
                    RNGCryptoServiceProvider provider1 = new RNGCryptoServiceProvider();
                    provider1.GetNonZeroBytes(buffer1);
                    num1 = Math.Abs((int)(((buffer1[0] + (buffer1[1] << 8)) + (buffer1[2] << 0x10)) + (buffer1[3] << 0x18)));
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
                return num1;
            }



            // Properties
            public int BytesDone
            {
                get
                {
                    int num1;
                    try
                    {
                        num1 = this.mInfo.BytesDone;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public double BytesPerSecondAverage
            {
                get
                {
                    double num1;
                    try
                    {
                        num1 = this.mInfo.BytesPerSecondAverage();
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public double BytesPerSecondCurrent
            {
                get
                {
                    double num1;
                    try
                    {
                        num1 = this.mInfo.BytesPerSecondCurrent();
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public int BytesTotal
            {
                get
                {
                    int num1;
                    try
                    {
                        num1 = this.mInfo.BytesTotal;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public int CodePage
            {
                get
                {
                    int num1;
                    try
                    {
                        num1 = this.mCodePage;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
                set
                {
                    try
                    {
                        this.mCodePage = value;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                }
            }

            public double ElapsedSeconds
            {
                get
                {
                    double num1;
                    try
                    {
                        num1 = this.mInfo.ElapsedMS / 1000;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public string FileName
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = HTTPUtils.Decode(this.mInfo.FileName, this.mCodePage);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }

            public bool Finished
            {
                get
                {
                    bool flag1;
                    try
                    {
                        flag1 = this.mInfo.Finished;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return flag1;
                }
            }

            public int ID
            {
                get
                {
                    int num1;
                    try
                    {
                        num1 = this.mID;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
                set
                {
                    try
                    {
                        this.mID = value;
                        this.mInfo = SharedState.Find(this.mID);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                }
            }

            public static int[] IDs
            {
                get
                {
                    int[] numArray1;
                    try
                    {
                        numArray1 = SharedState.GetIDs();
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return numArray1;
                }
            }

            public string Note
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = this.mInfo.Note;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
                set
                {
                    try
                    {
                        this.mInfo.Note = value;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                }
            }

            public double PercentDone
            {
                get
                {
                    double num1;
                    try
                    {
                        num1 = this.mInfo.PercentDone();
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public double SecondsLeft
            {
                get
                {
                    double num1;
                    try
                    {
                        num1 = this.mInfo.SecondsLeft();
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public bool Started
            {
                get
                {
                    bool flag1;
                    try
                    {
                        flag1 = this.mInfo.Started;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return flag1;
                }
            }


            // Fields
            private int mCodePage;
            private int mID;
            private StateInfo mInfo;
        }

        public class ProgressEventArgs : EventArgs
        {
            // Methods
            public ProgressEventArgs()
            {
            }

            // Fields
            public Progress State;
        }




        internal class ProgressUtils
        {
            // Methods
            public ProgressUtils()
            {
            }

            public static StateInfo CreateState(string inID)
            {
                int num1 = 0;
                try
                {
                    num1 = Convert.ToInt32(inID);
                }
                catch
                {
                    num1 = SharedState.GetNewID();
                }
                return SharedState.Create(num1);
            }


            public static void DoUpload(ProgressModule inModule, HttpApplication inApp, StateInfo inState, Settings inSet, string inDumpPath)
            {
                int num3;
                int num1 = inSet.Advanced.ChunkSize;
                HttpWorkerRequest request1 = ProgressUtils.GetWorkerRequest(inApp);
                DataStore store1 = new DataStore();
                inState.Note = inSet.Notes.GettingSize;
                inModule.DoUpdateProgress(inState.ID);
                int num2 = num3 = inApp.Request.ContentLength;
                int num6 = inApp.Request.ContentEncoding.CodePage;
                inState.SetState(0, num3);
                inModule.DoUpdateProgress(inState.ID);
                inState.Note = inSet.Notes.CheckingSize;
                inModule.DoUpdateProgress(inState.ID);
                int num4 = ProgressUtils.GetMaxRequestLength(inApp);
                if (num3 > num4)
                {
                    inState.Note = inSet.Notes.UploadTooLarge;
                    inModule.DoUpdateProgress(inState.ID);
                }
                else
                {
                    byte[] buffer1;
                    store1.Init(inSet, num6);
                    inState.Note = inSet.Notes.Preloading;
                    inModule.DoUpdateProgress(inState.ID);
                    if (num2 > 0)
                    {
                        buffer1 = request1.GetPreloadedEntityBody();
                        if (buffer1 != null)
                        {
                            if (num1 <= 10)
                            {
                                num1 = buffer1.Length;
                            }
                            num2 -= buffer1.Length;
                            inState.FileName = store1.Add(buffer1);
                            inState.SetState(buffer1.Length, num3);
                            inModule.DoUpdateProgress(inState.ID);
                        }
                    }
                    inState.Note = inSet.Notes.Transferring;
                    inModule.DoUpdateProgress(inState.ID);
                    while (num2 > 0)
                    {
                        if (num1 > num2)
                        {
                            num1 = num2;
                        }
                        buffer1 = new byte[num1];
                        int num5 = request1.ReadEntityBody(buffer1, num1);
                        if (num5 <= 0)
                        {
                            inState.Note = inSet.Notes.UploadStopped;
                            inModule.DoUpdateProgress(inState.ID);
                            store1.Clear();
                            return;
                        }
                        num2 -= num5;
                        if (num5 < num1)
                        {
                            Array.Resize<byte>(ref buffer1, num5);
                        }
                        inState.SetState(num3 - num2, num3);
                        inState.FileName = store1.Add(buffer1);
                        inModule.DoUpdateProgress(inState.ID);
                        buffer1 = null;
                    }
                    if (inDumpPath != null)
                    {
                        store1.Clear();
                        string text1 = ProgressUtils.ReadUploadData(inDumpPath, store1, num1);
                        ProgressUtils.SetContentType(inApp.Request, text1);
                    }
                    buffer1 = store1.GetUploadData(num3);
                    ProgressUtils.SetUploadData(request1, buffer1);
                    store1.FinishUp(inApp, inSet);
                    inState.Note = inSet.Notes.Finishing;
                    inModule.DoUpdateProgress(inState.ID);
                }
            }


            private static int GetMaxRequestLength(HttpApplication inApp)
            {
                int num1 = 0x800000;
                try
                {
                    System.Configuration.Configuration configuration1 = System.Web.Configuration.WebConfigurationManager.OpenWebConfiguration("~");
                    System.Web.Configuration.HttpRuntimeSection section1 = (System.Web.Configuration.HttpRuntimeSection)configuration1.GetSection("system.web/httpRuntime");
                    num1 = section1.MaxRequestLength * 0x400;
                }
                catch (Exception exception1)
                {
                    Log.UniqueError("Unable to get maximum request length.", exception1.Message);
                }
                return num1;
            }


            private static HttpWorkerRequest GetWorkerRequest(HttpApplication inApp)
            {
                BindingFlags flags1 = BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance;
                Type type1 = inApp.Context.GetType();
                HttpWorkerRequest request1 = null;
                try
                {
                    request1 = (HttpWorkerRequest)type1.GetProperty("WorkerRequest", flags1).GetValue(inApp.Context, null);
                }
                catch (Exception exception1)
                {
                    Log.UniqueError("Unable to get worker thread.", exception1.Message);
                }
                return request1;
            }


            public static string ReadUploadData(string inFilePath, DataStore inData, int inChunkSize)
            {
                byte[] buffer1;
                FileStream stream1 = new FileStream(inFilePath, FileMode.Open, FileAccess.Read, FileShare.Read);
                for (long num1 = stream1.Length; num1 > 0; num1 -= buffer1.Length)
                {
                    if (num1 < inChunkSize)
                    {
                        buffer1 = new byte[num1];
                    }
                    else
                    {
                        buffer1 = new byte[inChunkSize];
                    }
                    stream1.Read(buffer1, 0, buffer1.Length);
                    inData.Add(buffer1);
                }
                stream1.Close();
                return inData.GetBoundary();
            }


            private static void SetContentType(HttpRequest inReq, string inValue)
            {
                BindingFlags flags1 = BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance;
                Type type1 = inReq.GetType();
                try
                {
                    type1.GetField("_contentType", flags1).SetValue(inReq, inValue);
                }
                catch (Exception exception1)
                {
                    Log.UniqueError("Unable to set content type.", exception1.Message);
                }
            }


            private static byte[] SetUploadData(HttpWorkerRequest inWR, byte[] inData)
            {
                BindingFlags flags1 = BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance;
                Type type1 = inWR.GetType();
                int num1 = inData.Length;
                while (true)
                {
                    if (((type1 == null) || (type1.FullName == "System.Web.Hosting.ISAPIWorkerRequest")) || (type1.FullName == "Microsoft.VisualStudio.WebHost.Request"))
                    {
                        break;
                    }
                    type1 = type1.BaseType;
                }
                if (type1 == null)
                {
                    Log.UniqueError("Unable to set upload data.");
                }
                if ((type1 != null) && (type1.FullName == "System.Web.Hosting.ISAPIWorkerRequest"))
                {
                    try
                    {
                        type1.GetField("_contentAvailLength", flags1).SetValue(inWR, num1);
                        type1.GetField("_contentTotalLength", flags1).SetValue(inWR, num1);
                        type1.GetField("_preloadedContent", flags1).SetValue(inWR, inData);
                        type1.GetField("_preloadedContentRead", flags1).SetValue(inWR, true);
                    }
                    catch (Exception exception1)
                    {
                        Log.UniqueError("Unable to set upload data properties.", exception1.Message);
                    }
                }
                if ((type1 != null) && (type1.FullName == "Microsoft.VisualStudio.WebHost.Request"))
                {
                    try
                    {
                        type1.GetField("_contentLength", flags1).SetValue(inWR, num1);
                        type1.GetField("_preloadedContent", flags1).SetValue(inWR, inData);
                        type1.GetField("_preloadedContentLength", flags1).SetValue(inWR, num1);
                    }
                    catch (Exception exception2)
                    {
                        Log.UniqueError("Unable to set debug upload data properties.", exception2.Message);
                    }
                }
                return inData;
            }



        }

        /*public class Registration
        {
            // Methods
            public Registration()
            {
            }


            internal static void CheckValid()
            {
                try
                {
                    if (Registration.IsValid(Registration.SerialKey))
                    {
                        return;
                    }
                    TimeSpan span1 = Registration.GetTimeLeft();
                    if (span1.TotalDays <= 0)
                    {
                        throw new Exception("Your license has expired.");
                    }
                }
                catch
                {
                }
            }


            private static string Decode(string inString)
            {
                return Registration.EncodeDecode(inString);
            }


            private static string Encode(string inString)
            {
                return Registration.EncodeDecode(inString);
            }


            private static string EncodeDecode(string inString)
            {
                if (inString == null)
                {
                    return "";
                }
                char[] chArray1 = inString.ToCharArray();
                for (int num1 = 0; num1 < chArray1.Length; num1++)
                {
                    chArray1[num1] ^= (char)(0x17fd + num1);
                }
                StringBuilder builder1 = new StringBuilder(chArray1.Length);
                builder1.Append(chArray1);
                return builder1.ToString();
            }


            private static DateTime GetInstallDate(string inPath)
            {
                RegistryKey key1;
                string text1 = "";
                try
                {
                    key1 = Registration.OpenRegistryPath(inPath, true);
                }
                catch
                {
                    key1 = Registration.OpenRegistryPath(inPath, false);
                }
                if (key1 == null)
                {
                    throw new Exception("Unable to access expiration date.");
                }
                string text2 = "id";
                if (key1.GetValue(text2) == null)
                {
                    string text3 = Registration.Encode(Convert.ToString(DateTime.Now, CultureInfo.InvariantCulture.DateTimeFormat));
                    key1.SetValue(text2, text3);
                    Log.Success(Registration.Name + " registered successfully.");
                }
                object obj1 = key1.GetValue(text2);
                if (obj1 == null)
                {
                    throw new Exception("Unable to find expiration date.");
                }
                text1 = Registration.Decode((string)obj1);
                key1.Close();
                DateTime time1 = DateTime.MaxValue;
                try
                {
                    time1 = Convert.ToDateTime(text1, CultureInfo.InvariantCulture.DateTimeFormat);
                }
                catch
                {
                    throw new Exception("Unable to establish expiration date.");
                }
                return time1;
            }


            private static string GetRegistryString(string inPath, string inKey)
            {
                string text1 = "";
                object obj1 = null;
                RegistryKey key1 = Registration.OpenRegistryPath(inPath, false);
                if (key1 != null)
                {
                    obj1 = key1.GetValue(inKey);
                }
                if (obj1 is string)
                {
                    text1 = (string)obj1;
                }
                return text1;
            }


            private static TimeSpan GetTimeLeft()
            {
                DateTime time1 = Registration.InstallDate;
                TimeSpan span1 = TimeSpan.Zero;
                if (time1 == DateTime.MaxValue)
                {
                    return span1;
                }
                span1 = DateTime.Now.Subtract(time1);
                if ((span1.TotalDays >= 0) && (span1.TotalDays <= 30))
                {
                    TimeSpan span2 = new TimeSpan(30, 0, 0, 0);
                    return span2.Subtract(span1);
                }
                return TimeSpan.Zero;
            }


            public static void Install()
            {
                string text1 = Registration.License;
            }


            internal static bool IsValid(string inString)
            {
                if (Registration.IsValidPro(inString))
                {
                    return true;
                }
                if (Registration.IsValidLite(inString))
                {
                    return true;
                }
                return false;
            }


            internal static bool IsValidLite(string inString)
            {
                try
                {
                    if (inString.Length < 12)
                    {
                        return false;
                    }
                    string text1 = inString.Substring(0, 12).Replace("-", "");
                    int num1 = Convert.ToInt32(text1);
                    if ((((0x41c64e6d * num1) + 0x3039) & 0x7fff) != 0x3bd7)
                    {
                        return false;
                    }
                    if ((((0x3d41c64e * num1) + 0x3a18) & 0x7fff) != 0x4a4c)
                    {
                        return false;
                    }
                    if ((((0x2e3039c6 * num1) + 0x41c6) & 0x7fff) != 0x244a)
                    {
                        return false;
                    }
                    if ((((0x164e6d41 * num1) + 0x44a7) & 0x7fff) != 0x577d)
                    {
                        return false;
                    }
                    Registration.sLite = true;
                    return true;
                }
                catch
                {
                }
                return false;
            }


            internal static bool IsValidPro(string inString)
            {
                try
                {
                    if (inString.Length < 12)
                    {
                        return false;
                    }
                    string text1 = inString.Substring(0, 12).Replace("-", "");
                    int num1 = Convert.ToInt32(text1);
                    if ((((0x41c64e6d * num1) + 0x3039) % 0x7fff) != 0x4291)
                    {
                        return false;
                    }
                    if ((((0x3d41c64e * num1) + 0x3a18) % 0x7fff) != 0x55b5)
                    {
                        return false;
                    }
                    if ((((0x1e3039c6 * num1) + 0x41c6) % 0x7fff) != 0x5fe7)
                    {
                        return false;
                    }
                    if ((((0x164e6d41 * num1) + 0x44a7) % 0x7fff) != 0x3359)
                    {
                        return false;
                    }
                    Registration.sLite = false;
                    return true;
                }
                catch
                {
                }
                return false;
            }


            private static RegistryKey OpenRegistryPath(string inPath, bool inWritable)
            {
                string[] textArray1 = inPath.Split(new char[] { '\\' });
                RegistryKey key1 = null;
                for (int num1 = 0; num1 < textArray1.Length; num1++)
                {
                    if (num1 == 0)
                    {
                        try
                        {
                            key1 = Registry.LocalMachine.OpenSubKey(textArray1[num1], inWritable);
                        }
                        catch
                        {
                        }
                        if (key1 == null)
                        {
                            key1 = Registry.LocalMachine.OpenSubKey(textArray1[num1]);
                        }
                    }
                    else
                    {
                        RegistryKey key2 = key1.OpenSubKey(textArray1[num1], inWritable);
                        if (key2 == null)
                        {
                            key2 = key1.CreateSubKey(textArray1[num1]);
                        }
                        key1.Close();
                        key1 = key2;
                    }
                    if (key1 == null)
                    {
                        break;
                    }
                }
                return key1;
            }


            private static void SetRegistryString(string inPath, string inKey, string inValue)
            {
                RegistryKey key1 = Registration.OpenRegistryPath(inPath, true);
                if (key1 != null)
                {
                    key1.SetValue(inKey, inValue);
                }
            }


            public static void Uninstall()
            {
            }



            // Properties
            internal static DateTime InstallDate
            {
                get
                {
                    DateTime time1 = DateTime.MaxValue;
                    try
                    {
                        string text1 = HttpUtility.UrlDecode(Settings.Current.Advanced.Serial);
                        text1 = Registration.Decode(text1);
                        if (text1 != "")
                        {
                            time1 = Convert.ToDateTime(text1, CultureInfo.InvariantCulture.DateTimeFormat);
                        }
                    }
                    catch
                    {
                    }
                    try
                    {
                        if (time1 == DateTime.MaxValue)
                        {
                            time1 = Registration.GetInstallDate(Registration.Path);
                        }
                    }
                    catch
                    {
                    }
                    return time1;
                }
            }

            internal static bool IsLite
            {
                get
                {
                    if (!Registration.sChecked)
                    {
                        Registration.IsValid(Registration.SerialKey);
                        Registration.sChecked = true;
                    }
                    return Registration.sLite;
                }
            }

            public static string License
            {
                get
                {
                    string text1;
                    try
                    {
                        if (Registration.IsValidPro(Registration.SerialKey))
                        {
                            return string.Format("Full License ({0})", Registration.Serial);
                        }
                        if (Registration.IsValidLite(Registration.SerialKey))
                        {
                            return string.Format("Lite License ({0})", Registration.Serial);
                        }
                        TimeSpan span1 = Registration.GetTimeLeft();
                        if (span1.TotalDays <= 0)
                        {
                            return "Trial License - Expired";
                        }
                        text1 = string.Format("Trial License - {0:D2} days {1:D2}:{2:D2}:{3:D2} Left", new object[] { span1.Days, span1.Hours, span1.Minutes, span1.Seconds });
                    }
                    catch (Exception exception1)
                    {
                        text1 = string.Format("No License. {0}", exception1.Message);
                    }
                    return text1;
                }
            }

            private static string Name
            {
                get
                {
                    if (Registration.sName == null)
                    {
                        Registration.sName = Assembly.GetExecutingAssembly().GetName().Name;
                    }
                    return Registration.sName;
                }
            }

            private static string Path
            {
                get
                {
                    if (Registration.sPath == null)
                    {
                        Registration.sPath = @"SOFTWARE\WebSupergoo\" + Registration.Name + @"\" + Registration.Version;
                    }
                    return Registration.sPath;
                }
            }

            public static string Serial
            {
                get
                {
                    string text1 = Registration.SerialKey;
                    if (text1.Length < 12)
                    {
                        return "";
                    }
                    return text1.Substring(12, text1.Length - 12);
                }
                set
                {
                    Registration.SerialKey = value;
                }
            }

            internal static string SerialKey
            {
                get
                {
                    string text1 = "";
                    try
                    {
                        text1 = Registration.GetRegistryString(Registration.Path, "serial");
                    }
                    catch
                    {
                    }
                    try
                    {
                        if (text1 == "")
                        {
                            text1 = Settings.Current.Advanced.Serial;
                        }
                    }
                    catch
                    {
                    }
                    return text1;
                }
                set
                {
                    if (Registration.IsValid(value))
                    {
                        Registration.SetRegistryString(Registration.Path, "serial", value);
                    }
                }
            }

            public static string TrialKey
            {
                get
                {
                    string text1 = "";
                    try
                    {
                        DateTime time1 = Registration.GetInstallDate(Registration.Path);
                        string text2 = Convert.ToString(time1, CultureInfo.InvariantCulture.DateTimeFormat);
                        text2 = Registration.Encode(text2);
                        text2 = HttpUtility.UrlEncode(text2);
                        text1 = text2;
                    }
                    catch
                    {
                    }
                    return text1;
                }
            }

            private static string Version
            {
                get
                {
                    if (Registration.sVers == null)
                    {
                        Registration.sVers = Assembly.GetExecutingAssembly().GetName().Version.Major.ToString();
                    }
                    return Registration.sVers;
                }
            }


            // Fields
            private static bool sChecked;
            private static bool sLite;
            private static string sName;
            private static string sPath;
            private static string sVers;
        }*/

        internal class RezFork
        {
            // Methods
            public RezFork()
            {
                this.mStream = null;
                this.mHeader = new RezFork.ResHeader();
                this.mMap = new RezFork.ResMap();
                this.mRTL = new RezFork.ResType();
                this.mRFL = new RezFork.ResRef();
                this.mData = new RezFork.ResData();
            }


            public void GetRezByID(uint inType, ushort inID, out int outData, out int outLen)
            {
                int num5;
                int num1 = ((int)this.mStream.Length) - 1;
                int num2 = 0;
                int num3 = 0;
                int num4 = 0;
                outLen = num5 = 0;
                outData = num5;
                this.mMap.SetData(this.mStream, this.mHeader.resmap_offset);
                if ((this.mMap.offset + this.mHeader.resmap_length) <= num1)
                {
                    this.mRTL.SetData(this.mStream, (this.mMap.offset + this.mMap.resourcetypelist_offset) + 2);
                    if (this.mRTL.offset <= num1)
                    {
                        num2 = this.mMap.numtypes_zbased + 1;
                        if ((this.mRTL.offset + (this.mRTL.size * num2)) <= num1)
                        {
                            this.mRFL.offset = 0;
                            num4 = 0;
                            while (num4 < num2)
                            {
                                if (this.mRTL.restype == inType)
                                {
                                    this.mRFL.SetData(this.mStream, (this.mRTL.offset + this.mRTL.referencelist_offset) - 4);
                                    num3 = this.mRTL.numres_zbased + 1;
                                    break;
                                }
                                this.mRTL.SetData(this.mStream, this.mRTL.offset + this.mRTL.size);
                                num4++;
                            }
                            if ((this.mRFL.offset != 0) && ((this.mRFL.offset + (this.mRFL.size * num3)) <= num1))
                            {
                                this.mData.offset = 0;
                                for (num4 = 0; num4 < num3; num4++)
                                {
                                    if (this.mRFL.resid == inID)
                                    {
                                        this.mData.SetData(this.mStream, this.mHeader.resdata_offset + this.mRFL.attr_data_offset);
                                        break;
                                    }
                                    this.mRFL.SetData(this.mStream, this.mRFL.offset + this.mRFL.size);
                                }
                                if (((this.mData.offset != 0) && (this.mData.res_length >= 0)) && (((this.mData.offset + this.mData.size) + this.mData.res_length) <= num1))
                                {
                                    outData = this.mData.offset + this.mData.size;
                                    outLen = this.mData.res_length;
                                }
                            }
                        }
                    }
                }
            }


            public bool IsOK()
            {
                if (this.mStream.Length < this.mHeader.size)
                {
                    return false;
                }
                if (this.mHeader.resdata_offset < this.mHeader.size)
                {
                    return false;
                }
                if (this.mHeader.resmap_offset < this.mHeader.size)
                {
                    return false;
                }
                if (this.mHeader.resdata_length < 0)
                {
                    return false;
                }
                if (this.mHeader.resmap_length < 0)
                {
                    return false;
                }
                if (this.mHeader.resdata_offset > this.mHeader.resmap_offset)
                {
                    if ((this.mHeader.resmap_offset + this.mHeader.resmap_length) > this.mHeader.resdata_offset)
                    {
                        return false;
                    }
                    if ((this.mHeader.resdata_offset + this.mHeader.resdata_length) != this.mStream.Length)
                    {
                        return false;
                    }
                }
                else
                {
                    if ((this.mHeader.resdata_offset + this.mHeader.resdata_length) > this.mHeader.resmap_offset)
                    {
                        return false;
                    }
                    if ((this.mHeader.resmap_offset + this.mHeader.resmap_length) != this.mStream.Length)
                    {
                        return false;
                    }
                }
                return true;
            }


            public void SetData(SuperStream inStream)
            {
                this.mStream = inStream;
                this.mHeader.SetData(inStream, 0);
            }



            // Fields
            private ResData mData;
            private ResHeader mHeader;
            private ResMap mMap;
            private ResRef mRFL;
            private ResType mRTL;
            private SuperStream mStream;

            // Nested Types
            private class ResData
            {
                // Methods
                public ResData()
                {
                }


                public void SetData(SuperStream inStream, int inOffset)
                {
                    this.size = 8;
                    this.offset = inOffset;
                    inStream.Position = this.offset;
                    this.res_length = inStream.GetMacLONG();
                }



                // Fields
                public int offset;
                public int res_length;
                public int size;
            }

            private class ResHeader
            {
                // Methods
                public ResHeader()
                {
                }


                public void SetData(SuperStream inStream, int inOffset)
                {
                    this.size = 0x10;
                    this.offset = inOffset;
                    inStream.Position = this.offset;
                    this.resdata_offset = inStream.GetMacLONG();
                    inStream.Position = this.offset + 4;
                    this.resmap_offset = inStream.GetMacLONG();
                    inStream.Position = this.offset + 8;
                    this.resdata_length = inStream.GetMacLONG();
                    inStream.Position = this.offset + 12;
                    this.resmap_length = inStream.GetMacLONG();
                }



                // Fields
                public int offset;
                public int resdata_length;
                public int resdata_offset;
                public int resmap_length;
                public int resmap_offset;
                public int size;
            }

            private class ResMap
            {
                // Methods
                public ResMap()
                {
                }


                public void SetData(SuperStream inStream, int inOffset)
                {
                    this.offset = inOffset;
                    inStream.Position = this.offset + 0x18;
                    this.resourcetypelist_offset = inStream.GetMacWORD();
                    inStream.Position = this.offset + 0x1c;
                    this.numtypes_zbased = inStream.GetMacWORD();
                }



                // Fields
                public ushort numtypes_zbased;
                public int offset;
                public ushort resourcetypelist_offset;
            }

            private class ResRef
            {
                // Methods
                public ResRef()
                {
                }


                public void SetData(SuperStream inStream, int inOffset)
                {
                    this.size = 12;
                    this.offset = inOffset;
                    inStream.Position = this.offset;
                    this.resid = inStream.GetMacWORD();
                    inStream.Position = this.offset + 4;
                    this.attr_data_offset = inStream.GetMacLONG();
                    this.attr_data_offset = (this.attr_data_offset & 0xffffff) << 8;
                }



                // Fields
                public int attr_data_offset;
                public int offset;
                public ushort resid;
                public int size;
            }

            private class ResType
            {
                // Methods
                public ResType()
                {
                }


                public void SetData(SuperStream inStream, int inOffset)
                {
                    this.size = 8;
                    this.offset = inOffset;
                    inStream.Position = this.offset;
                    this.restype = inStream.GetMacDWORD();
                    inStream.Position = this.offset + 4;
                    this.numres_zbased = inStream.GetMacWORD();
                    inStream.Position = this.offset + 6;
                    this.referencelist_offset = inStream.GetMacWORD();
                }



                // Fields
                public ushort numres_zbased;
                public int offset;
                public ushort referencelist_offset;
                public uint restype;
                public int size;
            }
        }

        internal class Settings
        {
            // Methods
            static Settings()
            {
                Settings.sSettings = new Settings();
            }


            private Settings()
            {
                this.mAdvanced = null;
                this.mQueryFields = null;
                this.mNotes = null;
            }


            private static bool GetBool(IDictionary theConfig, string inKey, bool inDefault)
            {
                try
                {
                    if (theConfig != null)
                    {
                        inDefault = Convert.ToBoolean(theConfig[inKey].ToString());
                    }
                }
                catch
                {
                }
                return inDefault;
            }


            private static int GetInt(IDictionary theConfig, string inKey, int inDefault)
            {
                try
                {
                    if (theConfig != null)
                    {
                        inDefault = Convert.ToInt32(theConfig[inKey].ToString());
                    }
                }
                catch
                {
                }
                return inDefault;
            }


            private static string GetString(IDictionary theConfig, string inKey, string inDefault)
            {
                try
                {
                    if (theConfig != null)
                    {
                        inDefault = theConfig[inKey].ToString();
                    }
                }
                catch
                {
                }
                return inDefault;
            }



            // Properties
            public Settings.AdvancedSettings Advanced
            {
                get
                {
                    if (this.mAdvanced == null)
                    {
                        this.mAdvanced = new Settings.AdvancedSettings();
                    }
                    return this.mAdvanced;
                }
            }

            public static Settings Current
            {
                get
                {
                    return Settings.sSettings;
                }
            }

            public Settings.NotesSettings Notes
            {
                get
                {
                    if (this.mNotes == null)
                    {
                        this.mNotes = new Settings.NotesSettings();
                    }
                    return this.mNotes;
                }
            }

            public Settings.QueryFieldsSettings QueryFields
            {
                get
                {
                    if (this.mQueryFields == null)
                    {
                        this.mQueryFields = new Settings.QueryFieldsSettings();
                    }
                    return this.mQueryFields;
                }
            }


            // Fields
            private AdvancedSettings mAdvanced;
            private NotesSettings mNotes;
            private QueryFieldsSettings mQueryFields;
            private static Settings sSettings;

            // Nested Types
            public class AdvancedSettings
            {
                // Methods
                public AdvancedSettings()
                {
                    this.EnableProgressBar = true;
                    this.FixCorruptUploads = true;
                    this.EnableLargeUploads = true;
                    this.MultiThreaded = true;
                    this.Optimization = true;
                    this.ChunkSize = 0x2000;
                    this.TempDirectory = "";
                    this.Serial = "";
                    string text1 = "UploadComponent.ALUpload/advanced";
                    IDictionary dictionary1 = (IDictionary)HttpContext.Current.GetSection(text1);
                    this.EnableProgressBar = Settings.GetBool(dictionary1, "enableProgressBar", this.EnableProgressBar);
                    this.FixCorruptUploads = Settings.GetBool(dictionary1, "fixCorruptUploads", this.FixCorruptUploads);
                    this.EnableLargeUploads = Settings.GetBool(dictionary1, "enableLargeUploads", this.EnableLargeUploads);
                    this.MultiThreaded = Settings.GetBool(dictionary1, "multiThreaded", this.MultiThreaded);
                    this.Optimization = Settings.GetBool(dictionary1, "optimization", this.Optimization);
                    this.ChunkSize     = int.Parse(ConfigurationManager.AppSettings["BrowserChunkSize"].ToString());
                    this.TempDirectory = Settings.GetString(dictionary1, "tempDirectory", this.TempDirectory);
                    this.LogExceptions = Settings.GetBool(dictionary1, "logExceptions", this.LogExceptions);
                    this.Serial = Settings.GetString(dictionary1, "serial", this.Serial);
                }




                // Fields
                public int ChunkSize;
                public bool EnableLargeUploads;
                public bool EnableProgressBar;
                public bool FixCorruptUploads;
                public bool LogExceptions;
                public bool MultiThreaded;
                public bool Optimization;
                public string Serial;
                public string TempDirectory;
            }

            public class NotesSettings
            {
                // Methods
                public NotesSettings()
                {
                    this.Exception = "CC_UploadComponent_ProgrssBar_Error";
                    this.Connecting = "CC_UploadComponent_ProgrssBar_Connecting";
                    this.GettingSize = "CC_UploadComponent_ProgrssBar_GettingUploadSize";
                    this.CheckingSize = "CC_UploadComponent_ProgrssBar_CheckingUploadSize...";
                    this.Preloading = "CC_UploadComponent_ProgrssBar_GettingPreloadedData";
                    this.Transferring = "CC_UploadComponent_ProgrssBar_TransferringData";
                    this.Finishing = "CC_UploadComponent_ProgrssBar_Finishing";
                    this.UploadStopped = "CC_UploadComponent_ProgrssBar_UploadStoppedUnexpectedly";
                    this.UploadTooLarge = "CC_UploadComponent_ProgrssBar_UploadSizeisGreaterThanMaxAllowed";
                    string text1 = "UploadComponent.ALUpload/notes";
                    IDictionary dictionary1 = (IDictionary)HttpContext.Current.GetSection(text1);
                    this.Exception = Settings.GetString(dictionary1, "exception", this.Exception);
                    this.Connecting = Settings.GetString(dictionary1, "connecting", this.Connecting);
                    this.GettingSize = Settings.GetString(dictionary1, "gettingSize", this.GettingSize);
                    this.CheckingSize = Settings.GetString(dictionary1, "checkingSize", this.CheckingSize);
                    this.Preloading = Settings.GetString(dictionary1, "preloading", this.Preloading);
                    this.Transferring = Settings.GetString(dictionary1, "transferring", this.Transferring);
                    this.Finishing = Settings.GetString(dictionary1, "finishing", this.Finishing);
                    this.UploadStopped = Settings.GetString(dictionary1, "uploadStopped", this.UploadStopped);
                    this.UploadTooLarge = Settings.GetString(dictionary1, "uploadTooLarge", this.UploadTooLarge);
                }




                // Fields
                public string CheckingSize;
                public string Connecting;
                public string Exception;
                public string Finishing;
                public string GettingSize;
                public string Preloading;
                public string Transferring;
                public string UploadStopped;
                public string UploadTooLarge;
            }

            public class QueryFieldsSettings
            {
                // Methods
                public QueryFieldsSettings()
                {
                    this.UploadID = "UploadID";
                    this.DumpPath = "";
                    string text1 = "UploadComponent.ALUpload/queryfields";

                    IDictionary dictionary1 = (IDictionary)HttpContext.Current.GetSection(text1);
                    this.UploadID = Settings.GetString(dictionary1, "uploadID", this.UploadID);
                    this.DumpPath = Settings.GetString(dictionary1, "dumpPath", this.DumpPath);
                }




                // Fields
                public string DumpPath;
                public string UploadID;
            }
        }

        internal class SharedState
        {
            // Methods
            static SharedState()
            {
                SharedState.sStateInfoDB = new Hashtable();
            }


            public SharedState()
            {
            }


            public static StateInfo Create(int inID)
            {
               /* if (sStateInfoDB[inID] != null)
                {
                    if (((StateInfo)sStateInfoDB[inID]).Finished == true)
                    {
                        sStateInfoDB[inID]=new StateInfo(inID,"",Settings.Current.Advanced.ChunkSize);

                    }
                }*/
                SharedState.Purge();
                StateInfo info1 = new StateInfo(inID, "", Settings.Current.Advanced.ChunkSize);
                lock (SharedState.sStateInfoDB.SyncRoot)
                {
                    SharedState.sStateInfoDB[inID] = info1;
                }
                return info1;
            }


            public static void Delete(int inID)
            {
                lock (SharedState.sStateInfoDB.SyncRoot)
                {
                    SharedState.sStateInfoDB.Remove(inID);
                }
                SharedState.Purge();
            }


            public static StateInfo Find(int inID)
            {
                StateInfo info1 = (StateInfo)SharedState.sStateInfoDB[inID];
                if (info1 == null)
                {
                    info1 = new StateInfo(inID, Settings.Current.Notes.Connecting, Settings.Current.Advanced.ChunkSize);
                }
                return info1;
            }


            public static int[] GetIDs()
            {
                int[] numArray1;
                lock (SharedState.sStateInfoDB.SyncRoot)
                {
                    numArray1 = new int[SharedState.sStateInfoDB.Count];
                    SharedState.sStateInfoDB.Keys.CopyTo(numArray1, 0);
                }
                return numArray1;
            }


            public static int GetNewID()
            {
                return new Random().Next();
            }


            private static void Purge()
            {
                lock (SharedState.sStateInfoDB.SyncRoot)
                {
                    ArrayList list1 = new ArrayList();
                    foreach (DictionaryEntry entry1 in SharedState.sStateInfoDB)
                    {
                        StateInfo info1 = (StateInfo)entry1.Value;
                        if (info1.Expired())
                        {
                            list1.Add(entry1.Key);
                        }
                    }
                    foreach (object obj1 in list1)
                    {
                        SharedState.sStateInfoDB.Remove(obj1);
                    }
                }
            }



            // Fields
            private static Hashtable sStateInfoDB;
        }

        internal class StateInfo
        {
            // Methods
            public StateInfo(int inID, string inNote, int inChunkSize)
            {
                this.mID = inID;
                this.mBytesTotal = 0;
                this.mBytesDone = 0;
                this.mStarted = false;
                this.mFinished = false;
                this.mStartTime = DateTime.Now;
                this.mTouchTime = DateTime.Now;
                this.mNote = inNote;
                this.mFileName = new byte[0];
                int num2 = 1;
                if ((inChunkSize > 0) && (inChunkSize < 0x10000))
                {
                    num2 = 0x10000 / inChunkSize;
                }
                this.mDataIndex = 0;
                this.mDataTime = new DateTime[num2];
                int num1 = 0;
                while (num1 < num2)
                {
                    this.mDataTime[num1] = this.mTouchTime;
                    num1++;
                }
                this.mDataSize = new int[num2];
                for (num1 = 0; num1 < num2; num1++)
                {
                    this.mDataSize[num1] = 0;
                }
            }


            public double BytesPerSecondAverage()
            {
                double num1 = 0;
                double num2 = this.ElapsedMS / 1000;
                if (num2 > 0)
                {
                    num1 = ((double)this.mBytesDone) / num2;
                }
                return num1;
            }


            public double BytesPerSecondCurrent()
            {
                double num3 = 0;
                int num1 = this.mDataIndex - 1;
                if (num1 < 0)
                {
                    num1 = this.mDataTime.Length - 1;
                }
                int num2 = num1 + 1;
                if (num2 >= this.mDataTime.Length)
                {
                    num2 = 0;
                }
                TimeSpan span1 = this.mDataTime[num1].Subtract(this.mDataTime[num2]);
                double num4 = span1.TotalSeconds;
                if (num4 <= 0)
                {
                    return num3;
                }
                int num5 = 0;
                int num6 = this.mDataSize.Length;
                for (int num7 = 0; num7 < num6; num7++)
                {
                    num5 += this.mDataSize[num7];
                }
                return (((double)num5) / num4);
            }


            public bool Expired()
            {
                DateTime time1 = this.mTouchTime.AddSeconds(60);
                return (DateTime.Compare(time1, DateTime.Now) < 0);
            }


            public double PercentDone()
            {
                double num1 = 0;
                if ((this.mBytesDone == this.mBytesTotal) && this.mFinished)
                {
                    num1 = 100;
                }
                else if (this.mBytesTotal > 0)
                {
                    num1 = (((double)this.mBytesDone) / ((double)this.mBytesTotal)) * 95;
                }
                return (double)((int)num1);
            }


            public double SecondsLeft()
            {
                double num1 = 0;
                double num2 = this.ElapsedMS;
                if (this.mBytesDone > 0)
                {
                    num1 = ((((double)this.mBytesTotal) / ((double)this.mBytesDone)) * num2) - num2;
                }
                return (num1 / 1000);
            }


            public void SetState(int inDone, int inTotal)
            {
                int num1 = inDone - this.mBytesDone;
                this.mBytesDone = inDone;
                this.mBytesTotal = inTotal;
                this.mStarted = true;
                this.mTouchTime = DateTime.Now;
                this.mDataTime[this.mDataIndex] = this.mTouchTime;
                this.mDataSize[this.mDataIndex] = num1;
                if (++this.mDataIndex >= this.mDataTime.Length)
                {
                    this.mDataIndex = 0;
                }
            }


            public void Touch()
            {
                this.mTouchTime = DateTime.Now;
            }



            // Properties
            public int BytesDone
            {
                get
                {
                    this.Touch();
                    return this.mBytesDone;
                }
            }

            public int BytesTotal
            {
                get
                {
                    this.Touch();
                    return this.mBytesTotal;
                }
            }

            public double ElapsedMS
            {
                get
                {
                    this.Touch();
                    TimeSpan span1 = DateTime.Now.Subtract(this.mStartTime);
                    return span1.TotalMilliseconds;
                }
            }

            public byte[] FileName
            {
                get
                {
                    this.Touch();
                    return this.mFileName;
                }
                set
                {
                    this.mFileName = value;
                }
            }

            public bool Finished
            {
                get
                {
                    this.Touch();
                    return this.mFinished;
                }
                set
                {
                    this.mFinished = value;
                }
            }

            public int ID
            {
                get
                {
                    return this.mID;
                }
            }

            public string Note
            {
                get
                {
                    this.Touch();
                    return this.mNote;
                }
                set
                {
                    this.mNote = value;
                }
            }

            public bool Started
            {
                get
                {
                    this.Touch();
                    return this.mStarted;
                }
            }


            // Fields
            private int mBytesDone;
            private int mBytesTotal;
            private int mDataIndex;
            private int[] mDataSize;
            private DateTime[] mDataTime;
            private byte[] mFileName;
            private bool mFinished;
            private int mID;
            private string mNote;
            private bool mStarted;
            private DateTime mStartTime;
            private DateTime mTouchTime;
        }

        internal class SubStream : Stream
        {
            // Methods
            public SubStream(Stream inStream)
            {
                this.mStream = inStream;
                this.mStart = 0;
                this.mLength = inStream.Length;
                this.mPosition = 0;
                this.mLockPos = 0;
            }


            public SubStream(SubStream inStream, int inStart, int inLength)
            {
                if (inStart < 0)
                {
                    inStart = 0;
                }
                if (inLength < 0)
                {
                    inLength = 0;
                }
                if (inStart > inStream.Length)
                {
                    inStart = (int)inStream.Length;
                }
                if (inLength > (inStream.Length - inStart))
                {
                    inLength = ((int)inStream.Length) - inStart;
                }
                this.mStream = inStream.mStream;
                this.mStart = inStart + inStream.mStart;
                this.mLength = inLength;
                this.mPosition = 0;
            }


            public override void Flush()
            {
                this.mStream.Flush();
            }


            public Stream Lock()
            {
                this.mLockPos = this.mStream.Position;
                this.mStream.Position = this.mStart + this.mPosition;
                return this.mStream;
            }


            public override int Read(byte[] buffer, int offset, int count)
            {
                long num1 = this.mLength - this.mPosition;
                long num2 = this.mStart + this.mPosition;
                this.mPosition += count;
                if (num1 < count)
                {
                    count = (int)num1;
                }
                if (count < 0)
                {
                    count = 0;
                }
                if (count > 0)
                {
                    long num3 = this.mStream.Position;
                    this.mStream.Position = num2;
                    count = this.mStream.Read(buffer, offset, count);
                    this.mStream.Position = num3;
                }
                return count;
            }


            public override long Seek(long offset, SeekOrigin origin)
            {
                switch (origin)
                {
                    case SeekOrigin.Begin:
                        {
                            this.mPosition = offset;
                            return this.mPosition;
                        }
                    case SeekOrigin.Current:
                        {
                            this.mPosition += offset;
                            return this.mPosition;
                        }
                    case SeekOrigin.End:
                        {
                            this.mPosition = this.mLength - offset;
                            return this.mPosition;
                        }
                }
                return (long)0;
            }


            public override void SetLength(long value)
            {
                throw new NotSupportedException();
            }


            public override string ToString()
            {
                long num1 = this.Position;
                byte[] buffer1 = new byte[this.Length];
                this.Read(buffer1, 0, (int)this.Length);
                this.Position = num1;
                return Encoding.ASCII.GetString(buffer1);
            }


            public void Unlock()
            {
                this.mStream.Position = this.mLockPos;
            }


            public override void Write(byte[] buffer, int offset, int count)
            {
                throw new NotSupportedException();
            }



            // Properties
            public override bool CanRead
            {
                get
                {
                    return true;
                }
            }

            public override bool CanSeek
            {
                get
                {
                    return true;
                }
            }

            public override bool CanWrite
            {
                get
                {
                    return false;
                }
            }

            public override long Length
            {
                get
                {
                    return this.mLength;
                }
            }

            public override long Position
            {
                get
                {
                    return this.mPosition;
                }
                set
                {
                    this.mPosition = value;
                }
            }

            public long Start
            {
                get
                {
                    return this.mStart;
                }
            }


            // Fields
            private long mLength;
            private long mLockPos;
            private long mPosition;
            private long mStart;
            private Stream mStream;
        }

        internal class SuperStream : SubStream
        {
            // Methods
            public SuperStream(Stream inStream)
                : base(inStream)
            {
            }


            public SuperStream(SubStream inStream, int inStart, int inLength)
                : base(inStream, inStart, inLength)
            {
            }


            public bool EqualStr(byte[] inSearch)
            {
                bool flag1;
                if (inSearch.Length > (this.Length - this.Position))
                {
                    return false;
                }
                Stream stream1 = base.Lock();
                try
                {
                    flag1 = SuperStream.EqualStr(inSearch, stream1);
                }
                finally
                {
                    base.Unlock();
                }
                return flag1;
            }


            private static bool EqualStr(byte[] inSearch, Stream inStream)
            {
                bool flag1 = false;
                if ((inSearch != null) && (inSearch.Length >= 1))
                {
                    long num2 = 0;
                    long num3 = inStream.Position;
                    long num4 = inSearch.Length - 1;
                    while (!flag1)
                    {
                        int num1 = inStream.ReadByte();
                        if ((num1 < 0) || (inSearch[(int)((IntPtr)num2)] != num1))
                        {
                            break;
                        }
                        num2++;
                        if (num2 >= num4)
                        {
                            flag1 = true;
                        }
                    }
                    inStream.Position = num3;
                }
                return flag1;
            }


            public long FindStr(byte[] inSearch)
            {
                if ((inSearch != null) && (inSearch.Length >= 1))
                {
                    long num2 = (this.Length - this.Position) - inSearch.Length;
                    int num3 = inSearch[0];
                    Stream stream1 = base.Lock();
                    try
                    {
                        int num4;
                        goto Label_0086;
                    Label_002C:
                        num4 = stream1.ReadByte();
                        if (num3 == num4)
                        {
                            long num1 = stream1.Position;
                            stream1.Position--;
                            if (SuperStream.EqualStr(inSearch, stream1))
                            {
                                this.Position = (stream1.Position - base.Start) + inSearch.Length;
                                return (this.Position - inSearch.Length);
                            }
                            stream1.Position = num1;
                        }
                    Label_0086:
                        num2--;
                        if (num2 >= 0)
                        {
                            goto Label_002C;
                        }
                    }
                    finally
                    {
                        base.Unlock();
                    }
                    this.Position = this.Length;
                }
                return (long)(-1);
            }


            public byte GetBYTE()
            {
                return (byte)this.ReadByte();
            }


            public uint GetMacDWORD()
            {
                uint num1 = (uint)this.ReadByte();
                uint num2 = (uint)this.ReadByte();
                uint num3 = (uint)this.ReadByte();
                uint num4 = (uint)this.ReadByte();
                return ((((num1 << 0x18) | (num2 << 0x10)) | (num3 << 8)) | num4);
            }


            public int GetMacLONG()
            {
                int num1 = this.ReadByte();
                int num2 = this.ReadByte();
                int num3 = this.ReadByte();
                int num4 = this.ReadByte();
                return ((((num1 << 0x18) | (num2 << 0x10)) | (num3 << 8)) | num4);
            }


            public short GetMacSHORT()
            {
                int num1 = this.ReadByte();
                int num2 = this.ReadByte();
                return (short)((num1 << 0x18) | (num2 << 0x10));
            }


            public ushort GetMacWORD()
            {
                uint num1 = (uint)this.ReadByte();
                uint num2 = (uint)this.ReadByte();
                return (ushort)((num1 << 0x18) | (num2 << 0x10));
            }


            public uint GetOSType()
            {
                return this.GetMacDWORD();
            }


            public static byte[] OSTypeToBytes(uint inType)
            {
                return new byte[] { ((byte)((inType & 0xff000000) >> 0x18)), ((byte)((inType & 0xff0000) >> 0x10)), ((byte)((inType & 0xff00) >> 8)), ((byte)(inType & 0xff)) };
            }


            public static string OSTypeToString(uint inType)
            {
                return Encoding.ASCII.GetString(SuperStream.OSTypeToBytes(inType));
            }


            public static uint StringToOSType(string inStr)
            {
                Stream stream1 = new MemoryStream(Encoding.ASCII.GetBytes(inStr));
                SuperStream stream2 = new SuperStream(stream1);
                return stream2.GetOSType();
            }



        }

        internal class TempFile
        {
            // Methods
            public TempFile()
            {
                this.mPath = null;
                this.mStream = null;
            }


            public void CloseStream()
            {
                if (this.mStream != null)
                {
                    try
                    {
                        this.mStream.Close();
                    }
                    catch (Exception exception1)
                    {
                        try
                        {
                            Log.UniqueError("Unable to close temporary file \"", this.mPath + "\". " + exception1.Message);
                        }
                        catch
                        {
                        }
                    }
                    this.mStream = null;
                }
            }


            private void DeleteFile()
            {
                if (this.mPath != null)
                {
                    try
                    {
                        File.Delete(this.mPath);
                    }
                    catch (Exception exception1)
                    {
                        try
                        {
                            if (File.Exists(this.mPath))
                            {
                                Log.UniqueError("Unable to delete temporary file \"", this.mPath + "\". " + exception1.Message);
                            }
                        }
                        catch
                        {
                        }
                    }
                    this.mPath = null;
                }
            }


            public string Detach()
            {
                string text1 = this.mPath;
                this.CloseStream();
                this.mPath = null;
                return text1;
            }


            public void Dispose()
            {
                try
                {
                    this.CloseStream();
                    this.DeleteFile();
                }
                catch
                {
                }
            }


            ~TempFile()
            {
                this.Dispose();
            }


            private static string GetTempFileName()
            {
                Guid guid1 = Guid.NewGuid();
                return (TempFile.GetTempPath() + guid1.ToString() + ".txt");
            }


            private static string GetTempPath()
            {
                lock (typeof(TempFile))
                {
                    if ((TempFile.sTempPath == null) || (TempFile.sTempPath == ""))
                    {
                        TempFile.sTempPath = System.IO.Path.GetTempPath() + Assembly.GetExecutingAssembly().GetName().Name + @"\";
                        try
                        {
                            Context ContextObj = null;
                            StringBuilder ContextAdditionalInfo = null;
                            ContextAdditionalInfo = new StringBuilder(20);
                            ContextAdditionalInfo.Append("<ADDITIOALINFO><APINAME>CustomControls.ALCC_UploadComponent.UploadComponent.GetTempPath</APINAME></ADDITIOALINFO>");

                            ContextObj = new Context(0, 0, "", "", ContextAdditionalInfo.ToString());

                            SASMOMDirectory.DeleteFolder(TempFile.sTempPath, true, ContextObj);
                            //Directory.Delete(TempFile.sTempPath, true);
                        }
                        catch
                        {
                        }
                        try
                        {
                            Directory.CreateDirectory(TempFile.sTempPath);
                        }
                        catch (Exception exception1)
                        {
                            Log.UniqueError("Unable to create temporary directory \"", TempFile.sTempPath + "\". " + exception1.Message);
                            TempFile.sTempPath = System.IO.Path.GetTempPath();

                        }
                    }
                    //Following code is added by Kiran Abhang on date 12th March 2009 to resolve bug (bugzilla bug ID-6076).
                    else if (!(Directory.Exists(TempFile.sTempPath)))
                    {
                        try
                        {
                            Directory.CreateDirectory(TempFile.sTempPath);
                        }
                        catch (Exception exception1)
                        {
                            Log.UniqueError("Unable to create temporary directory \"", TempFile.sTempPath + "\". " + exception1.Message);
                            TempFile.sTempPath = System.IO.Path.GetTempPath();
                        }
                    }
                    //End: Kiran's code.
                }
                return TempFile.sTempPath;
            }


            public static void SetTempPath(string inPath, bool inTest)
            {
                if (inPath.Length > 0)
                {
                    if (inPath[inPath.Length - 1] != '\\')
                    {
                        inPath = inPath + @"\";
                    }
                    inTest = inTest && (TempFile.sTempPath != inPath);
                    TempFile.sTempPath = inPath;
                    if (inTest)
                    {
                        TempFile file1 = new TempFile();
                        FileStream stream1 = file1.FileStream;
                    }
                }
            }



            // Properties
            public FileStream FileStream
            {
                get
                {
                    if (this.mStream == null)
                    {
                        this.mStream = new FileStream(this.Path, FileMode.OpenOrCreate, FileAccess.ReadWrite);
                    }
                    return this.mStream;
                }
            }

            public string Path
            {
                get
                {
                    if (this.mPath == null)
                    {
                        this.mPath = TempFile.GetTempFileName();
                    }
                    return this.mPath;
                }
            }

            public bool Writable
            {
                get
                {
                    FileStream stream1 = null;
                    try
                    {
                        stream1 = this.FileStream;
                    }
                    catch
                    {
                    }
                    return (stream1 != null);
                }
            }


            // Fields
            private string mPath;
            private FileStream mStream;
            private static string sTempPath;
        }

        public class Upload
        {
            // Methods
            
            public Upload()
            {
                //Registration.CheckValid();
                try
                {
                    this.mContext = HttpContext.Current;
                    this.mOverwrite = true;
                    this.mCodePage = this.mContext.Request.ContentEncoding.CodePage;
                    this.mForm = (Multipart.Form)this.mContext.Items[Multipart.Form.KeyName];
                    this.mFiles = new UploadedFileCollection();
                    if (this.mForm == null)
                    {
                        foreach (string text1 in this.mContext.ApplicationInstance.Request.Files.Keys)
                        {
                            UploadedFile file1 = new UploadedFile(this, this.mContext.ApplicationInstance.Request.Files[text1]);
                            this.mFiles.AddFile(text1, file1);
                        }
                    }
                    else
                    {
                        for (int num1 = 0; num1 < this.mForm.Count; num1++)
                        {
                            if (this.mForm[num1].IsFileField)
                            {
                                UploadedFile file2 = new UploadedFile(this, this.mForm[num1]);
                                this.mFiles.AddFile(this.mForm[num1].FieldName, file2);
                            }
                        }
                    }
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }

            public static event EventHandler upload_error = null;
            
            public void Dispose()
            {
                try
                {
                    if (this.mForm != null)
                    {
                        this.mForm.Clear();
                    }
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }

              public static void error_event(object sender, EventArgs e)
              {
                if (upload_error != null)
                {
                    upload_error(sender, e);
                }
              }
            internal string ResolvePath(string path)
            {
                bool flag1 = path.IndexOf(':') < 0;
                bool flag2 = path.IndexOf('\\') < 0;
                bool flag3 = (path[0] != '/') || (path[1] != '/');
                if ((flag1 && flag2) && flag3)
                {
                    try
                    {
                        string text1 = Utilities.FindURLFileName(path);
                        path = path.Substring(0, path.Length - text1.Length);
                        path = this.Context.ApplicationInstance.Server.MapPath(path);
                        path = path + @"\" + text1;
                    }
                    catch (Exception exception1)
                    {
                        throw new Exception("Unable to map virtual to physical pathname.", exception1);
                    }
                }
                if (Utilities.FindFileName(path) == "")
                {
                    throw new Exception("No file name provided.");
                }
                return path;
            }


            public void Save(string path)
            {
                try
                {
                    path = this.ResolvePath(path);
                    FileUtils.WriteDataToFile(path, this.mOverwrite, this.mContext.ApplicationInstance.Request.InputStream);
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }



            // Properties
            public int CodePage
            {
                get
                {
                    int num1;
                    try
                    {
                        num1 = this.mCodePage;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
                set
                {
                    try
                    {
                        this.mCodePage = value;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                }
            }

            public HttpContext Context
            {
                get
                {
                    HttpContext context1;
                    try
                    {
                        context1 = this.mContext;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return context1;
                }
            }

            public UploadedFileCollection Files
            {
                get
                {
                    UploadedFileCollection collection1;
                    try
                    {
                        collection1 = this.mFiles;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return collection1;
                }
            }

            public bool Overwrite
            {
                get
                {
                    bool flag1;
                    try
                    {
                        flag1 = this.mOverwrite;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return flag1;
                }
                set
                {
                    try
                    {
                        this.mOverwrite = value;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                }
            }


            // Fields
            private int mCodePage;
            private HttpContext mContext;
            private UploadedFileCollection mFiles;
            private Multipart.Form mForm;
            private bool mOverwrite;
        }

        public class UploadedFile
        {
            // Methods
            internal UploadedFile(Upload upload, HttpPostedFile file)
            {
                //Registration.CheckValid();
                this.mInputStream = file.InputStream;
                this.mContentType = file.ContentType;
                this.mRawFileName = Encoding.GetEncoding(upload.CodePage).GetBytes(file.FileName);
                this.mTempFile = null;
                this.mUpload = upload;
                this.UpdateStreams();
            }


            internal UploadedFile(Upload upload, Multipart.Field file)
            {
                //Registration.CheckValid();
                this.mInputStream = file.TempFile.FileStream;
                this.mContentType = file.MIMEType;
                this.mRawFileName = file.FileName;
                this.mTempFile = file.TempFile;
                this.mUpload = upload;
                this.UpdateStreams();
            }

          
            public string Detach()
            {
                string text2;
                try
                {
                    string text1 = "";
                    if (this.mTempFile != null)
                    {
                        text1 = this.mTempFile.Detach();
                        this.mTempFile = null;
                    }
                    text2 = text1;
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
                return text2;
            }


            public void SaveAs(string path)
            {
                try
                {
                    path = this.mUpload.ResolvePath(path);
                    if (this.mMacBinary != null)
                    {
                        MacFile file1 = new MacFile(this.mMacBinary);
                        file1.SetStreams(false);
                        file1.SetOverwrite(this.mUpload.Overwrite);
                        file1.Save(path);
                    }
                    else
                    {
                        FileUtils.WriteDataToFile(path, this.mUpload.Overwrite, this.mDataStream);
                    }
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }


            public void SaveSFM(string path)
            {
                try
                {
                    path = this.mUpload.ResolvePath(path);
                    if (this.mMacBinary != null)
                    {
                        MacFile file1 = new MacFile(this.mMacBinary);
                        file1.SetStreams(true);
                        file1.SetOverwrite(this.mUpload.Overwrite);
                        file1.Save(path);
                    }
                    else
                    {
                        FileUtils.WriteDataToFile(path, this.mUpload.Overwrite, this.mDataStream);
                    }
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
            }


            internal void UpdateStreams()
            {
                long num1;
                long num2;
                this.mRawStream = new SuperStream(this.mInputStream);
                this.mMacBinary = new MacBinary();
                this.mMacBinary.SetData(new SuperStream(this.mRawStream), this.mContentType);
                if (this.mMacBinary.IsOK())
                {
                    this.mDataStream = this.mMacBinary.DataStream;
                    this.mRezStream = this.mMacBinary.RezStream;
                }
                else
                {
                    this.mMacBinary = null;
                    this.mDataStream = new SuperStream(this.mRawStream);
                    this.mRezStream = new MemoryStream(0);
                }
                this.mRezStream.Position = num1 = 0;
                this.mDataStream.Position = num2 = num1;
                this.mRawStream.Position = num2;
            }



            // Properties
            public long ContentLength
            {
                get
                {
                    long num1;
                    try
                    {
                        num1 = this.mInputStream.Length;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return num1;
                }
            }

            public string ContentType
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = this.mContentType;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }

            public Stream DataStream
            {
                get
                {
                    Stream stream1;
                    try
                    {
                        stream1 = this.mDataStream;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return stream1;
                }
            }

            public bool Exists
            {
                get
                {
                    bool flag1;
                    try
                    {
                        flag1 = this.mRawFileName.Length > 0;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return flag1;
                }
            }

            public string Extension
            {
                get
                {
                    string text2;
                    try
                    {
                        string text1 = "";
                        if (this.mMacBinary != null)
                        {
                            text1 = this.mMacBinary.GetFileXt();
                        }
                        else
                        {
                            text1 = Utilities.FindFileExtension(this.FileName);
                        }
                        if (text1 != "")
                        {
                            text1 = "." + text1;
                        }
                        text2 = text1;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text2;
                }
            }

            public string FileName
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = HTTPUtils.GetRawPath(this.mRawFileName, this.mUpload.CodePage);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }

            public Stream InputStream
            {
                get
                {
                    Stream stream1;
                    try
                    {
                        stream1 = this.mInputStream;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return stream1;
                }
            }

            public bool MacBinary
            {
                get
                {
                    bool flag1;
                    try
                    {
                        flag1 = this.mMacBinary != null;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return flag1;
                }
            }

            public byte[] RawFileName
            {
                get
                {
                    byte[] buffer1;
                    try
                    {
                        buffer1 = this.mRawFileName;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return buffer1;
                }
            }

            public Stream RawStream
            {
                get
                {
                    Stream stream1;
                    try
                    {
                        stream1 = this.mRawStream;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return stream1;
                }
            }

            public Stream RezStream
            {
                get
                {
                    Stream stream1;
                    try
                    {
                        stream1 = this.mRezStream;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return stream1;
                }
            }

            public string TempFile
            {
                get
                {
                    string text1;
                    try
                    {
                        if ((this.mTempFile == null) || (this.mTempFile.Path == null))
                        {
                            return "";
                        }
                        text1 = this.mTempFile.Path;
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }

            public string Text
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = Utilities.StreamToString(this.DataStream, this.mUpload.CodePage);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }

            public string URLSafeFileName
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = HTTPUtils.GetURLSafeFileName(this.mRawFileName, this.mUpload.CodePage);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }

            public string WinSafeFileName
            {
                get
                {
                    string text1;
                    try
                    {
                        text1 = HTTPUtils.GetWinSafeFileName(this.mRawFileName, this.mUpload.CodePage);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return text1;
                }
            }


            // Fields
            private string mContentType;
            private Stream mDataStream;
            private Stream mInputStream;
            private MacBinary mMacBinary;
            private byte[] mRawFileName;
            private Stream mRawStream;
            private Stream mRezStream;
            private TempFile mTempFile;
            private Upload mUpload;
        }

        public class UploadedFileCollection : NameObjectCollectionBase
        {
            // Methods
            internal UploadedFileCollection()
            {
            }


            internal void AddFile(string field, UploadedFile file)
            {
                base.BaseAdd(field, file);
            }


            public UploadedFile Get(int index)
            {
                UploadedFile file1;
                try
                {
                    file1 = (UploadedFile)base.BaseGet(index);
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
                return file1;
            }


            public UploadedFile Get(string key)
            {
                UploadedFile file1;
                try
                {
                    file1 = (UploadedFile)base.BaseGet(key);
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
                return file1;
            }


            public string GetKey(int index)
            {
                string text1;
                try
                {
                    text1 = base.BaseGetKey(index);
                }
                catch (Exception exception1)
                {
                    if (Settings.Current.Advanced.LogExceptions)
                    {
                        Log.ExceptionDetails(exception1);
                    }
                    throw exception1;
                }
                return text1;
            }



            // Properties
            public string[] AllKeys
            {
                get
                {
                    string[] textArray1;
                    try
                    {
                        textArray1 = base.BaseGetAllKeys();
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return textArray1;
                }
            }

            public UploadedFile this[string key]
            {
                get
                {
                    UploadedFile file1;
                    try
                    {
                        file1 = (UploadedFile)base.BaseGet(key);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return file1;
                }
            }

            public UploadedFile this[int index]
            {
                get
                {
                    UploadedFile file1;
                    try
                    {
                        file1 = (UploadedFile)base.BaseGet(index);
                    }
                    catch (Exception exception1)
                    {
                        if (Settings.Current.Advanced.LogExceptions)
                        {
                            Log.ExceptionDetails(exception1);
                        }
                        throw exception1;
                    }
                    return file1;
                }
            }
        }

        internal class Utilities
        {
            // Methods
            public Utilities()
            {
            }


            public static string ArraySummary(byte[] inArray)
            {
                return Utilities.StreamSummary(new MemoryStream(inArray));
            }


            public static string EqualStream(Stream inS1, Stream inS2)
            {
                long num7;
                if (inS1.Length != inS2.Length)
                {
                    return string.Concat(new string[] { "Length mismatch - ", inS1.Length.ToString(), " vs ", inS2.Length.ToString(), "." });
                }
                long num1 = inS1.Position;
                long num2 = inS2.Position;
                long num3 = inS1.Length;
                inS2.Position = num7 = 0;
                inS1.Position = num7;
                for (long num4 = 0; num4 < num3; num4++)
                {
                    if (inS1.ReadByte() != inS2.ReadByte())
                    {
                        return ("Difference detected at position " + num4.ToString() + ".");
                    }
                }
                inS1.Position = num1;
                inS2.Position = num2;
                return "";
            }


            public static string FindFileExtension(string inPath)
            {
                int num1 = inPath.LastIndexOf('.');
                if (num1 >= 0)
                {
                    return inPath.Substring(num1 + 1, (inPath.Length - num1) - 1);
                }
                return "";
            }


            public static string FindFileName(string inPath)
            {
                int num1 = inPath.LastIndexOf('\\');
                if (num1 >= 0)
                {
                    return inPath.Substring(num1 + 1, (inPath.Length - num1) - 1);
                }
                return inPath;
            }


            public static string FindURLFileName(string inPath)
            {
                int num1 = inPath.LastIndexOf('/');
                if (num1 >= 0)
                {
                    return inPath.Substring(num1 + 1, (inPath.Length - num1) - 1);
                }
                return inPath;
            }


            public static string StreamSummary(Stream inStream)
            {
                return Utilities.StreamSummary(inStream, 0x10);
            }


            public static string StreamSummary(Stream inStream, int theNum)
            {
                int num1 = (int)inStream.Position;
                int num2 = 0;
                string text1 = "";
                string text2 = "";
                text1 = string.Concat(new object[] { "At ", num1.ToString(), " of ", inStream.Length, "." });
                inStream.Position = 0;
                if (inStream.Length < theNum)
                {
                    theNum = (int)inStream.Length;
                }
                for (int num3 = 1; num3 <= theNum; num3++)
                {
                    num2 = inStream.ReadByte();
                    text2 = text2 + num2.ToString();
                    if (num3 != theNum)
                    {
                        text2 = text2 + ", ";
                    }
                }
                inStream.Position = num1;
                if (theNum == inStream.Length)
                {
                    return (text1 + " [" + text2 + "]");
                }
                return (text1 + " [" + text2 + ",...]");
            }


            public static string StreamToString(Stream inStream, int inCodePage)
            {
                long num1 = inStream.Position;
                byte[] buffer1 = new byte[inStream.Length];
                inStream.Read(buffer1, 0, (int)inStream.Length);
                inStream.Position = num1;
                return Encoding.GetEncoding(inCodePage).GetString(buffer1);
            }



        }

    }
    class ProgressModule : IHttpModule
    {

        public delegate void UpdateProgressHandler(object source, EventArgs e);
        public event ProgressModule.UpdateProgressHandler UpdateProgress;



        // Methods
        public ProgressModule()
        {
        }
        private void Application_BeginRequest(object src, EventArgs e)
        {

            UploadComponent.Settings settings1 = UploadComponent.Settings.Current;
            if (settings1.Advanced.EnableProgressBar)
            {
                HttpApplication application1 = (HttpApplication)src;
                string text1 = application1.Request.QueryString[settings1.QueryFields.UploadID];
                string text2 = application1.Request.QueryString[settings1.QueryFields.DumpPath];
                if ((text1 != null) || (text2 != null))
                {

                    UploadComponent.StateInfo info1 = UploadComponent.ProgressUtils.CreateState(text1);
                    this.DoUpdateProgress(info1.ID);
                    try
                    {

                        UploadComponent.TempFile.SetTempPath(settings1.Advanced.TempDirectory, true);
                        UploadComponent.ProgressUtils.DoUpload(this, application1, info1, settings1, text2);
                    }
                    catch (Exception exception1)
                    {
                        if (UploadComponent.Settings.Current.Advanced.LogExceptions)
                        {
                            UploadComponent.Log.ExceptionDetails(exception1);
                        }
                        info1.Note = string.Format(settings1.Notes.Exception, exception1.Message);
                        this.DoUpdateProgress(info1.ID);
                        if (exception1 is UnauthorizedAccessException)
                        {
                            throw exception1;
                        }
                        return;
                    }
                    finally
                    {
                        info1.Finished = true;
                    }
                }
            }
        }



        public void Dispose()
        {
        }

        internal void DoUpdateProgress(int inID)
        {
            if ((this.UpdateProgress != null))
            {
                UploadComponent.ProgressEventArgs args1 = new UploadComponent.ProgressEventArgs();
                args1.State = new UploadComponent.Progress(inID);
                this.UpdateProgress(this, args1);
            }
        }


        public void Init(HttpApplication context)
        {
            context.BeginRequest += new EventHandler(this.Application_BeginRequest);
            UpdateProgress += new ProgressModule.UpdateProgressHandler (UploadComponent.Upload.error_event);
        }


        // Properties
        public string ModuleName
        {
            get
            {
                return "ProgressModule";
            }
        }

    }
}
