using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Web;
using System.Net;
using Microsoft.Win32;
using System.Threading;
using System.Security.Principal;
using System.Configuration;

#region AssetlinkIncludes
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Solutions.Facade.TMSFacade;
using Assetlink.WebServices.ALSearchWS;
using Assetlink.WebServices.ALDAMWS;
using Assetlink.WebServices.ALInfrastructureWS;
using Assetlink.Widgets.WidgetCommon;
using Assetlink;
using Assetlink.IOProxy;
#endregion


namespace Assetlink.Solutions.Facade.DAMFacade
{
    public class DAM_CheckIn : DamFacadeBase
    {
        public String   ItemName                    = "";
        StringBuilder   JobNamePath                 = null;
        StringBuilder   JobName                     = null;        
        private String  TempJobFolderName           = "TmpJobs";//"DAM_TemperoryJobs";        
        Thread          ThreadCatalogAssets         = null;
        const Int32     MAX_ALLOWED_CHAR_FOR_CATALOGJOB_FILE = 249;

        /// <summary>
        /// This struct is used to support network balance scenario
        /// An Object of this struct is passed as input to "CatalogAssetsAsynch" function.
        /// This was implemented because the function "CatalogAssetsAsynch" is executed in a thread that cannot access the key from
        /// web.config present in the "DAM" solution.
        /// <remarks>
        /// NLB_SECURITY (Debasish Pramanik): The thread created for CatalogAssetsAsynch is created with the
        ///                                   default context irrespective of the parent thread context. This becomes 
        ///                                   visible when the system was tested on a Tenant virtual directory 
        ///                                   running with different user apart from NETWORKSERVICE. The NETWORKSERVICE
        ///                                   a/c has limited access on the network resource on shared drive, which is
        ///                                   being used in case of Load Balancing.
        ///                                   To overcome this issue we need to impersonate with the current thread 
        ///                                   identity. This is done in CatalogAssetsAsynch.
        /// 
        /// </remarks>
        /// </summary>
        public struct CatalogDigitalAssetInput
        {
            public String DigitalAssetCollection;
            public String SectionID;
            public String AdditionalInfoXml;            
            public String SharedDrivePath;
            public WindowsIdentity User;   //Refer to comment NLB_SECURITY
        }

        #region Constructure
        public DAM_CheckIn(String SessionID, String UserID,
                    String TenantName, String CurrentLanguage)
            : base(SessionID, UserID, TenantName, CurrentLanguage)
        {
            JobName = new StringBuilder();
            JobNamePath = new StringBuilder();
        }
        #endregion

        #region public methods
        public String PrepareJob(String FacetInformationXML, XPathNavigator xNavigator, ref String JobPath, ref String FolderHierarchy, String AdditionalInfoXml, String DigitalAssetID, bool ForCheckIn, String Section_Path)
        {
            #region variables
            XPathNodeIterator   xIterator       = null;
            XPathNavigator      xFileNode       = null;
            XPathNavigator      xRootNode       = null;
            String              ErrorXml        = "";
            String              SavedFilePath   = "";
            String              OriginalFileName = "";
            String              FilePath        = "";
            String              RenditionType   = "";
            String              SectionID       = "";
            String              SectionPath     = "";
            String              FileName        = "";
            String              m_ItemName      = "";            
            String              tempPath        = "";
            int                 Index           = 0;
            String              EncodedString   = "";
            String              str_Dttime      = null;
            #endregion

            try
            {
                if (String.IsNullOrEmpty(Section_Path))
                {
                    #region Get Section Path
                    xRootNode = xNavigator.SelectSingleNode("descendant::UploadedFiles");
                    SectionID = xRootNode.GetAttribute("SectionID", "");
                    SectionPath = GetSectionPath(SectionID);
                    #endregion
                }
                else
                    SectionPath = Section_Path;
                //prepare Job Name.
                xFileNode = xNavigator.SelectSingleNode("descendant::File[@RenditionType='Original']");
                if (xFileNode != null)
                {
                    FilePath = xFileNode.InnerXml;
                    OriginalFileName = FilePath.Substring(FilePath.LastIndexOf("\\") + 1);
                    ALSolutionCommon.DecodeSpecialCharacters(ref OriginalFileName);
                    Index = OriginalFileName.LastIndexOf('.');
                    if (Index != -1)
                        m_ItemName = OriginalFileName.Substring(0, Index);
                    else
                        m_ItemName = OriginalFileName;

                    ItemName = m_ItemName;                                                            

                    #region Prepare Jobname and JobPath      
                    str_Dttime = String.Format("{0}{1}{2}{3}{4}{5}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    JobName.AppendFormat("{0}_{1}", m_ItemName,str_Dttime);
                    InstallationDir = ALSolutionCommon.GetInstallationDir();
                    JobNamePath.AppendFormat("{0}\\{1}\\{2}\\{3}", ALSolutionCommon.GetTempPath(SessionID), UserID, TempJobFolderName, JobName);

                    bool IsValidJob=  GetJobNameifLengthyFileName(m_ItemName, JobName, JobNamePath, str_Dttime,OriginalFileName);
                    //If After reducing the jobname, the total length is > 260 then show error.
                    if (!IsValidJob)
                    {
                        ErrorXml = @"<ERRORCOLLECTION>
                                        <ERROR>
                                            <ERRORCODE>CheckIn_LongFilePath</ERRORCODE>
                                        </ERROR>
                                    </ERRORCOLLECTION>";

                        return ErrorXml;
                    }
                    #endregion

                    #region Create JobName directory
                    try
                    {
                        if (!Directory.Exists(JobNamePath.ToString()))
                            Directory.CreateDirectory(JobNamePath.ToString());
                    }
                    catch (Exception ex)
                    { return ALSolutionCommon.PrepareErrorXML("Cannot create Job directory: " + ex.Message); }

                    #endregion

                    #region Move the files under Jobname Folder.
                    //prepare folder collection Xml.
                    EncodedString = JobName.ToString();
                    ALSolutionCommon.EncodeSpecialCharacters(ref EncodedString);
                    StringBuilder FolderCollectionXml = new StringBuilder();
                    FolderCollectionXml.Append("<FOLDERCOLLECTION>");
                    FolderCollectionXml.Append("<FOLDER Name=\"" + EncodedString + "\">");

                    xIterator = xNavigator.Select("descendant::File");
                    foreach (XPathNavigator x_FileNode in xIterator)
                    {
                        RenditionType = x_FileNode.GetAttribute("NTFSFolder", "");
                        //Rahul Bug Fix 3806
                        SavedFilePath = x_FileNode.GetAttribute("SavedFilePath", "");
                        //prepare separate folder for each rendition.
                        if (!Directory.Exists(JobNamePath.ToString() + "\\" + RenditionType))
                            Directory.CreateDirectory(JobNamePath.ToString() + "\\" + RenditionType);

                        tempPath = x_FileNode.InnerXml;
                        FileName = x_FileNode.InnerXml;
                        FileName = FileName.Substring(FileName.LastIndexOf("\\") + 1);

                        /*
                         * Prepare Preview and Thumbnail files like OriginalFileName.Extension.
                         * Ex. Original file : Test.jpg
                         *     Preview file  : a.gif  => Test.jpg.gif
                         *     Thumbnail file  : b.xmm  => Test.jpg.xmmm
                        */

                        if (!RenditionType.ToUpper().Contains("ORIGINAL"))
                        {
                            FileName = OriginalFileName + Path.GetExtension(tempPath);
                        }

                        ALSolutionCommon.DecodeSpecialCharacters(ref FileName);
                        FilePath = JobNamePath.ToString() + "\\" + RenditionType + "\\" + FileName;
                        String UploadedFile_Name = x_FileNode.Value;
                        //Check for '' in file name
                        String UploadedFileName = UploadedFile_Name.Substring(UploadedFile_Name.LastIndexOf("\\") + 1);
                        if (UploadedFileName.IndexOf("''") != -1)
                        {
                            UploadedFileName = UploadedFileName.Replace("''", "'");
                            UploadedFile_Name = UploadedFile_Name.Substring(0, UploadedFile_Name.LastIndexOf("\\")) + UploadedFileName;
                        }
                        //Rahul Bug Fix 3806
                        ALSolutionCommon.DecodeSpecialCharacters(ref SavedFilePath);
                        File.Move(SavedFilePath, FilePath);

                        EncodedString = FilePath;
                        ALSolutionCommon.EncodeSpecialCharacters(ref EncodedString);

                        if (EncodedString.IndexOf("''") != -1)
                        {
                            EncodedString = EncodedString.Replace("''","'");
                        }

                        FolderCollectionXml.Append("<SUBFOLDER Name='" + RenditionType + "'>");
                        FolderCollectionXml.Append("<FILE>" + EncodedString + "</FILE>");
                        FolderCollectionXml.Append("</SUBFOLDER>");
                    }

                    #endregion

                    #region Prepare AssetInfo File.
                    //if (FacetInformationXML.IndexOf("<FACETINFORMATION>") != -1)
                    //{
                    //    FacetInformationXML = FacetInformationXML.Replace("<FACETINFORMATION>", "");
                    //    FacetInformationXML = FacetInformationXML.Replace("</FACETINFORMATION>", "");
                    //}
                    ErrorXml = PrepareAssetInfo(FacetInformationXML, SectionPath, OriginalFileName, AdditionalInfoXml, DigitalAssetID, ForCheckIn);

                    EncodedString = JobNamePath.ToString();
                    ALSolutionCommon.EncodeSpecialCharacters(ref EncodedString);
                    if (EncodedString.IndexOf("''") != -1)
                        EncodedString = EncodedString.Replace("''", "'");                    

                    FolderCollectionXml.Append("<FILE>" + EncodedString + "\\AssetInfo" + "</FILE>");
                    FolderCollectionXml.Append("</FOLDER>");
                    FolderCollectionXml.Append("</FOLDERCOLLECTION>");

                    FolderHierarchy = FolderCollectionXml.ToString();
                    JobPath = JobNamePath.ToString();

                    #endregion
                }
                else
                    return ALSolutionCommon.PrepareErrorXML("Original File must not be empty.");
            }
            catch (Exception ex)
            {
                return ALSolutionCommon.PrepareErrorXML(ex.Message);
            }
            return ErrorXml;
        }

        public String MoveJob(String JobPath, ref String FileListXML)
        {
            #region variables
            String              MountPoint                      = "";
            String              UserName                        = "";
            InfrastructureFacade.InfrastructureFacadeClass obj_InfrastructureFacadeClass = null;
            ulong ul_userid = 0;
            ulong ul_sessionid = 0;
            StringBuilder AdditionalInfo = new StringBuilder(200);
            #endregion

            try
            {

                obj_InfrastructureFacadeClass = new Assetlink.Solutions.Facade.InfrastructureFacade.InfrastructureFacadeClass(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML =  obj_InfrastructureFacadeClass.GetTenantSetting("CATALOGER/MOUNTPOINT", ref MountPoint);
                if (!String.IsNullOrEmpty(ErrorXML))
                    return ErrorXML;

                if (String.IsNullOrEmpty(MountPoint))
                    return ALSolutionCommon.PrepareErrorXML("CHECKIN_MOVEJOB_FILEWATCHERMOUNTPOINTNOTEXISTS","");                   

                UserName = GetUserName(UserID);
                
                JobNamePath.Remove(0, JobNamePath.Length);
                JobNamePath.Append(JobPath);

                CommonFacade.FileTransfer ObjFT = new FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML                        = ObjFT.MoveJob(ref FileListXML, MountPoint, UserName);
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    //Directory.Delete(JobPath, true);
                    ulong.TryParse(UserID, out ul_userid);
                    ulong.TryParse(SessionID, out ul_sessionid);
                    AdditionalInfo.AppendFormat(@"<ADDITIONALINFO UserID='{0}' SessionID='{1}' Language='{2}' TenantName='{3}'>
                                        <METHODNAME>MoveJob</METHODNAME>                                                                                    
                                        <JOBPATH>{4}</JOBPATH>                                                                                
                                  </ADDITIONALINFO>",
                                  UserID, SessionID, CurrentLanguage, TenantName, JobPath);
                    Context ObjContext = new Context(ul_userid, ul_sessionid, TenantName, "DAM", AdditionalInfo.ToString());
                    SASMOMDirectory.DeleteFolder(JobPath, true, ObjContext);
                }
            }
            catch (Exception ex)
            {
                ErrorXML = "Exception occured while moving job to catalog folder. Details are as follows,$BR$";
                ErrorXML = ErrorXML + ex.Message;
                return ALSolutionCommon.PrepareErrorXML(ErrorXML);                       
            }
            finally
            {

            }
            return ErrorXML;

        }

        public String PrepareAssetInfo(String FacetInformationXML, String SectionPath, String OriginalFileName, String AdditionalInfoXml, String DigitalAssetID, bool ForCheckIn)
        {
            StringBuilder AssetInfo = null;
            //XPathDocument xpathDoc_FacetInfoXml = null;
            //XPathDocument xpathDoc_AdditionInfoXml = null;
            XmlDocument     xmlDoc_FacetInfoXml      = null;
            XmlDocument     xmlDoc_AdditionalInfoXml = null;
            StringBuilder   sb_AdditionalInfo        = null; 
            try
            {
                //Author : Bhalchandra Kadam
                //Date   : 05 February 2009
                //Comment: Remove the duplicate facet information from AdditionalInfoXml and update FacetInformationXML with new data.
                if ((!String.IsNullOrEmpty(FacetInformationXML)) && (!String.IsNullOrEmpty(AdditionalInfoXml)) )
                {
                    xmlDoc_FacetInfoXml      = new XmlDocument();
                    xmlDoc_AdditionalInfoXml = new XmlDocument();
                    xmlDoc_FacetInfoXml.LoadXml(FacetInformationXML);
                    sb_AdditionalInfo = new StringBuilder(200);
                    sb_AdditionalInfo.Append("<ADDITIONALINFO>" + AdditionalInfoXml + "</ADDITIONALINFO>");
                    xmlDoc_AdditionalInfoXml.LoadXml(sb_AdditionalInfo.ToString());
                    foreach (XmlNode xmlNode_FacetRelationship in xmlDoc_AdditionalInfoXml.SelectNodes("/ADDITIONALINFO/*[@TYPE]"))
                    {
                        if (xmlDoc_FacetInfoXml.DocumentElement.SelectSingleNode(xmlNode_FacetRelationship.Name) != null)
                        {
                            foreach (XmlNode xmlNode_FacetField in xmlNode_FacetRelationship.ChildNodes[0].ChildNodes)
                            {
                                if (xmlDoc_FacetInfoXml.DocumentElement.SelectSingleNode(xmlNode_FacetRelationship.Name + "/" + xmlNode_FacetRelationship.ChildNodes[0].Name + "/" + xmlNode_FacetField.Name) != null)
                                {
                                    xmlDoc_FacetInfoXml.DocumentElement.SelectSingleNode(xmlNode_FacetRelationship.Name + "/" + xmlNode_FacetRelationship.ChildNodes[0].Name + "/" + xmlNode_FacetField.Name).InnerXml = xmlNode_FacetField.InnerXml;
                                }
                                else
                                {
                                    xmlDoc_FacetInfoXml.DocumentElement.SelectSingleNode(xmlNode_FacetRelationship.Name + "/" + xmlNode_FacetRelationship.ChildNodes[0].Name ).InnerXml += xmlNode_FacetField.OuterXml;
                                }
                            }
                            //xmlDoc_FacetInfoXml.DocumentElement.SelectSingleNode(xmlNode_FacetRelationship.Name).InnerXml = xmlNode_FacetRelationship.InnerXml;
                            xmlNode_FacetRelationship.ParentNode.RemoveChild(xmlNode_FacetRelationship);
                        }
                    }
                    FacetInformationXML = xmlDoc_FacetInfoXml.DocumentElement.InnerXml;
                    AdditionalInfoXml   = xmlDoc_AdditionalInfoXml.DocumentElement.InnerXml;
                }
                else
                {
                    if (FacetInformationXML.IndexOf("<FACETINFORMATION>") != -1)
                    {
                        FacetInformationXML = FacetInformationXML.Replace("<FACETINFORMATION>", "");
                        FacetInformationXML = FacetInformationXML.Replace("</FACETINFORMATION>", "");
                    }
                }

                ALSolutionCommon.EncodeSpecialCharacters(ref OriginalFileName);
                if (OriginalFileName.IndexOf("''") != -1)
                    OriginalFileName = OriginalFileName.Replace("''","'");

                AssetInfo = new StringBuilder();
                if (ForCheckIn)
                    AssetInfo.Append("<ASSETINFO ForCheckin='TRUE'>");
                else
                    AssetInfo.Append("<ASSETINFO ForCheckin='TRUE' EnableVersioning='TRUE'>");

                ALSolutionCommon.EncodeSpecialCharacters(ref SectionPath);
                AssetInfo.AppendFormat("<SECTION Path=\"{0}\">", SectionPath);
                AssetInfo.Append("<DIGITALASSET>");
                AssetInfo.AppendFormat("<NAME>{0}</NAME>", OriginalFileName);
                if (!String.IsNullOrEmpty(DigitalAssetID))
                    AssetInfo.AppendFormat("<DIGITALASSETID>{0}</DIGITALASSETID>", DigitalAssetID);
                if (!String.IsNullOrEmpty(FacetInformationXML))
                    AssetInfo.Append(FacetInformationXML);
                if (!String.IsNullOrEmpty(AdditionalInfoXml))
                    AssetInfo.Append(AdditionalInfoXml);
                AssetInfo.Append("</DIGITALASSET>");
                AssetInfo.Append("</SECTION>");
                AssetInfo.AppendFormat("<DIGITALASSETCOUNT>1</DIGITALASSETCOUNT>");
                AssetInfo.Append("</ASSETINFO>");
                
                StreamWriter writer = new StreamWriter(JobNamePath.ToString() + "\\AssetInfo");
                writer.Write(AssetInfo.ToString());
                writer.Close();
            }
            catch (Exception ex)
            {
                return ALSolutionCommon.PrepareErrorXML("Error occured while preparing AssetInfo file :" + ex.Message);
            }
            return "";
        }

        public void CatalogDigitalAssets(String DigitalAssetCollection, String SectionID, String AdditionalInfoXml)
        {
            CatalogDigitalAssetInput ObjCreateDigitalAssetInput;
            String                   SharedDrivePath        = null;
            ParameterizedThreadStart parameterizedThreadStart = null;           

            //Here the struct object is used to store the value of InputXML and also the path
            //and the objct is passed to UpdateSecurityPolicy.
            //This mathod was implemented because when "UpdateSecurityPolicy" method is executed as a polling thread
            //it was not able to read the keys required to support Network load balance.
            SharedDrivePath                                     = ALSolutionCommon.GetTempPath(SessionID);
            ObjCreateDigitalAssetInput                          = new CatalogDigitalAssetInput();
            ObjCreateDigitalAssetInput.AdditionalInfoXml        = AdditionalInfoXml;
            ObjCreateDigitalAssetInput.SectionID                = SectionID;
            ObjCreateDigitalAssetInput.DigitalAssetCollection   = DigitalAssetCollection;
            ObjCreateDigitalAssetInput.SharedDrivePath          = SharedDrivePath;
            ObjCreateDigitalAssetInput.User                     = WindowsIdentity.GetCurrent() as WindowsIdentity;

            parameterizedThreadStart                            = new ParameterizedThreadStart(CatalogAssetsAsynch);
            ThreadCatalogAssets                                 = new Thread(parameterizedThreadStart);
            ThreadCatalogAssets.IsBackground                    = true;
            ThreadCatalogAssets.Name                            = "CatalogAssetsAsynch";            
            ThreadCatalogAssets.Start(ObjCreateDigitalAssetInput);
        }

        /// <summary>
        /// This API will prepare job and catalogs Digital Assets 
        /// </summary>
        /// <param name="DigitalAssetCollection"></param>
        /// <param name="SectionID"></param>
        /// <param name="AdditionalInfoXml"></param>
        //public void CatalogAssetsAsynch(String DigitalAssetCollection, String SectionID, String AdditionalInfoXml)
        public void CatalogAssetsAsynch(Object ObjCreateDigitalAssetInput)
        {
            #region member varibles
            XPathDocument               xDocument                   = null;
            XPathNavigator              xFileNavigator              = null;
            XPathNodeIterator           xOriginalFileIterator       = null;
            StringBuilder               FileInformation             = null;
            String                      FileInfoXml                 = "";
            String                      JobPathAsynch               = "";
            String                      FolderHierarchy             = "";
            String                      configKey                   = null;
            WindowsIdentity             userIdentity                = null;
            bool                        isLoadBalance               = false;
            WindowsImpersonationContext impersonationContext        = null;
            CatalogDigitalAssetInput    Obj_CreateDigitalAssetInput;
            #endregion

            Obj_CreateDigitalAssetInput = (CatalogDigitalAssetInput)ObjCreateDigitalAssetInput;
            configKey                   = ConfigurationManager.AppSettings["SHAREDDRIVEROOT"];
            if (String.IsNullOrEmpty(configKey) != true)
            {
                userIdentity            = Obj_CreateDigitalAssetInput.User;
                impersonationContext    = userIdentity.Impersonate();
                isLoadBalance           = true;
            }

            String DigitalAssetCollection   = Obj_CreateDigitalAssetInput.DigitalAssetCollection;
            String SectionID                = Obj_CreateDigitalAssetInput.SectionID;
            String AdditionalInfoXml        = Obj_CreateDigitalAssetInput.AdditionalInfoXml;
            ErrorXML                        = "";


            #region Get Files for each digital asset, prepare uploaded files structure
            /*                
                <UploadedFiles SectionID='1212'>
                     <File RenditionType=\"{0}\" NTFSFolder=\"{1}s\">{2}</File>
                </UploadedFiles>                
             */
            FileInformation         = new StringBuilder();
            FileInfoXml             = GetFileInformation(DigitalAssetCollection);
            if (!String.IsNullOrEmpty(FileInfoXml))
            {
                FileInformation.Append("<UploadedFiles SectionID='" + SectionID + "'>");
                FileInformation.Append(FileInfoXml);
                FileInformation.Append("</UploadedFiles>");
            }
            #endregion

            if (!String.IsNullOrEmpty(FileInformation.ToString()))            
            {
                #region Prepare CatalogJob
                xDocument           = new XPathDocument(new System.IO.StringReader(FileInformation.ToString()));
                xFileNavigator      = xDocument.CreateNavigator();
                ErrorXML            = PrepareJobAsynch(xFileNavigator, ref JobPathAsynch, ref FolderHierarchy, AdditionalInfoXml, true);
                #endregion

                #region Move Job to cataloger mount point
                if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(FolderHierarchy))
                {
                    xOriginalFileIterator = xFileNavigator.Select("descendant::File[@RenditionType='Original']");
                    ErrorXML = MoveJob(JobPathAsynch, ref FolderHierarchy);
                }
                #endregion
            }

            if (isLoadBalance == true)
                impersonationContext.Undo();

            ThreadCatalogAssets.Abort();
        }

        public Boolean IsServiceRunning(String ServiceName)
        {
            String                 CatalogerInstalled                   = null;
            String                 CommunicationProtocol                = null;
            String                 CatalogerMachineIP                   = null;
            String                 CatalogerInfrastructureServiceUrl    = null;
            InfrastructureService  InfrastructureServiceObj             = null;
            Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader ServiceHeader = null;

            CatalogerInstalled = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK\", "CatalogerInstalled");
            if (CatalogerInstalled == "1")
            {
                return ALCommon.IsServiceRunning(ServiceName);
            }
            else if (CatalogerInstalled == "0")
            {
                //If Cataloger is installed on Other machine
                CommunicationProtocol               = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK\CATALOGER", "Communication Protocol");
                CatalogerMachineIP                  = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK\CATALOGER", "Server Address");
                CatalogerInfrastructureServiceUrl   = CommunicationProtocol + CatalogerMachineIP + "/ALServices/ALInfrastructureWS.asmx";

                InfrastructureServiceObj            = new InfrastructureService();
                InfrastructureServiceObj.Url        = CatalogerInfrastructureServiceUrl;
                ServiceHeader                       = new Assetlink.WebServices.ALInfrastructureWS.ALServiceHeader();
                ServiceHeader.UserID                = Convert.ToUInt64(UserID);
                ServiceHeader.SessionID             = Convert.ToUInt64(SessionID);
                ServiceHeader.TenantName            = TenantName;
                ServiceHeader.Language              = CurrentLanguage;
                InfrastructureServiceObj.ALServiceHeaderValue = ServiceHeader;

                return InfrastructureServiceObj.IsServiceRunning(ServiceName);
            }
                
            return false;
        }

        #endregion

        #region private methods

        private String GetUserName(String UserID)
        {
            Search ObjSearch = null;
            String WebServiceUrl = "";
            XPathDocument OutputDocument = null;
            XPathNavigator xNavigator = null;

            #region prepare search input xml
            StringBuilder sbInput = new StringBuilder();
            sbInput.Append("<INPUTXML>");
            sbInput.Append("<SESSIONINFO TenantName=\"" + TenantName + "\" Language=\"" + CurrentLanguage + "\" UserID=\"" + UserID + "\" SessionID=\"" + SessionID + "\"/>");
            sbInput.Append("<SEARCHCOLLECTION>");
            sbInput.Append("<SEARCH>");
            sbInput.Append("<TARGETOBJECT>USER</TARGETOBJECT>");
            sbInput.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
            sbInput.Append("<SORTCOLUMN>USERID</SORTCOLUMN>");
            sbInput.Append("<SORTORDER>ASC</SORTORDER>");
            sbInput.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
            sbInput.Append("<SEARCHSTRING/>");
            sbInput.Append("<SEARCHALLFACETS>true</SEARCHALLFACETS>");
            sbInput.Append("<FILTERMODE ApplyFilter=\"\"/>");
            sbInput.Append("<FILTERLIST>");
            sbInput.Append("<FILTER>");
            sbInput.Append("<FIELDNAME>USERID</FIELDNAME>");
            sbInput.Append("<OPERATOR>EQUALTO</OPERATOR>");
            sbInput.Append("<FIELDVALUE>" + UserID + "</FIELDVALUE>");
            sbInput.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
            sbInput.Append("<NAVIGATIONPATH>USER</NAVIGATIONPATH>");
            sbInput.Append("</FILTER>");
            sbInput.Append("</FILTERLIST>");
            sbInput.Append("<FIELDCOLLECTION>");
            sbInput.Append("<TARGETENTITYFIELDS>");
            sbInput.Append("<FIELD>USERID</FIELD>");
            sbInput.Append("<FIELD>LOGINNAME</FIELD>");
            sbInput.Append("</TARGETENTITYFIELDS>");
            sbInput.Append("</FIELDCOLLECTION>");
            sbInput.Append("</SEARCH>");
            sbInput.Append("</SEARCHCOLLECTION>");
            sbInput.Append("</INPUTXML>");
            #endregion

            ObjSearch = new Search();
            WebServiceUrl = GetWebServiceURL("ALSearchWS");
            if (!String.IsNullOrEmpty(WebServiceUrl))
            {
                ObjSearch.Url = WebServiceUrl;
                ErrorXML = ObjSearch.DoSearch(sbInput.ToString(), ref OutputXml);
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    OutputDocument = new XPathDocument(new System.IO.StringReader(OutputXml));
                    xNavigator = OutputDocument.CreateNavigator();
                    xNavigator = xNavigator.SelectSingleNode("descendant::LOGINNAME");
                    if (xNavigator != null)
                        return xNavigator.Value;
                }
            }
            return UserID;
        }

        private String GetSectionPath(String SectionID)
        {
            SearchConfiguration ObjSearch = null;
            XPathDocument xSearchOutput = null;
            XPathNavigator xNavigator = null;
            XPathNavigator xIterator = null;
            System.IO.StringReader reader = null;
            String SearchOutput = "";
            String ErrorXml = "";

            try
            {
                ObjSearch = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXml = ObjSearch.DoSearch(PrepareSerchXml(SectionID), ref SearchOutput);
                if (String.IsNullOrEmpty(ErrorXml))
                {
                    reader = new StringReader(SearchOutput);
                    xSearchOutput = new XPathDocument(reader);
                    xNavigator = xSearchOutput.CreateNavigator();
                    xIterator = xNavigator.SelectSingleNode("descendant::PATH");
                    if (xIterator != null)
                        return xIterator.Value;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return "";
        }

        private String PrepareSerchXml(String SectinID)
        {
            StringBuilder sbInput = new StringBuilder();
            sbInput.Append("<INPUTXML>");
            sbInput.Append("<SESSIONINFO TenantName=\"" + TenantName + "\" Language=\"" + CurrentLanguage + "\" UserID=\"" + UserID + "\" SessionID=\"" + SessionID + "\"/>");
            sbInput.Append("<SEARCHCOLLECTION>");
            sbInput.Append("<SEARCH>");
            sbInput.Append("<TARGETOBJECT>SECTION</TARGETOBJECT>");
            sbInput.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
            sbInput.Append("<SORTCOLUMN>SECTIONID</SORTCOLUMN>");
            sbInput.Append("<SORTORDER>ASC</SORTORDER>");
            sbInput.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
            sbInput.Append("<SEARCHSTRING/>");
            sbInput.Append("<SEARCHALLFACETS>true</SEARCHALLFACETS>");
            sbInput.Append("<FILTERMODE ApplyFilter=\"\"/>");
            sbInput.Append("<FILTERLIST>");
            sbInput.Append("<FILTER>");
            sbInput.Append("<FIELDNAME>SECTIONID</FIELDNAME>");
            sbInput.Append("<OPERATOR>EQUALTO</OPERATOR>");
            sbInput.Append("<FIELDVALUE>" + SectinID + "</FIELDVALUE>");
            sbInput.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
            sbInput.Append("<NAVIGATIONPATH>SECTION</NAVIGATIONPATH>");
            sbInput.Append("</FILTER>");
            sbInput.Append("</FILTERLIST>");
            sbInput.Append("<FIELDCOLLECTION>");
            sbInput.Append("<TARGETENTITYFIELDS>");
            sbInput.Append("<FIELD>SECTIONID</FIELD>");
            sbInput.Append("<FIELD>PATH</FIELD>");
            sbInput.Append("</TARGETENTITYFIELDS>");
            sbInput.Append("</FIELDCOLLECTION>");
            sbInput.Append("</SEARCH>");
            sbInput.Append("</SEARCHCOLLECTION>");
            sbInput.Append("</INPUTXML>");
            return sbInput.ToString();
        }

        public String ReadRegistryKey(String Key, String StringKey)
        {
            RegistryKey RKey = null;
            String Value = null;
            Object obj = null;

            try
            {
                if (Key == null || StringKey == null)
                    return null;
                RKey = Registry.LocalMachine.OpenSubKey(Key);
                if (RKey == null)
                    return null;

                obj = RKey.GetValue(StringKey);
                if (obj == null)
                    return null;
                Value = Convert.ToString(obj);
                RKey.Close();
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return Value;
        }

        private String PrepareJobAsynch(XPathNavigator xNavigator, ref String JobPath, ref String FolderHierarchy, String AdditionalInfoXml, bool ForCheckIn)
        {
            #region variables
            XPathNodeIterator xIterator = null;
            XPathNavigator xFileNode = null;
            XPathNavigator xRootNode = null;
            String ErrorXml = "";
            String OriginalFileName = "";
            String FilePath = "";
            String RenditionType = "";
            String SectionID = "";
            String SectionPath = "";
            String FileName = "";
            String m_ItemName = "";
            String tempPath = "";
            String EncodedString = "";

            #endregion

            try
            {
                xRootNode   = xNavigator.SelectSingleNode("descendant::UploadedFiles");
                SectionID   = xRootNode.GetAttribute("SectionID", "");
                if (!String.IsNullOrEmpty(SectionID))
                {
                    #region Get Section Path
                    SectionPath = GetSectionPath(SectionID);
                    #endregion

                    //prepare Job Name.
                    xFileNode               = xNavigator.SelectSingleNode("descendant::File[@RenditionType='Original']");
                    if (xFileNode != null)
                    {
                        FilePath            = xFileNode.InnerXml;
                        OriginalFileName    = FilePath.Substring(FilePath.LastIndexOf("\\") + 1);
                        ALSolutionCommon.DecodeSpecialCharacters(ref OriginalFileName);

                        #region Prepare Jobname and JobPath
                        m_ItemName          = "CatalogJob";                        
                        JobName.AppendFormat("{0}_{1}{2}{3}{4}{5}{6}", m_ItemName, DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                        InstallationDir     = ALSolutionCommon.GetInstallationDir();
                        JobNamePath.AppendFormat("{0}\\{1}\\{2}\\{3}", ALSolutionCommon.GetTempPath(SessionID), UserID, TempJobFolderName, JobName);
                        #endregion

                        #region Create JobName directory
                        try
                        {
                            if (!Directory.Exists(JobNamePath.ToString()))
                                Directory.CreateDirectory(JobNamePath.ToString());
                        }
                        catch (Exception ex)
                        { return ALSolutionCommon.PrepareErrorXML("Cannot create Job directory: " + ex.Message); }

                        #endregion

                        #region Move the files under Jobname Folder.
                        //prepare folder collection Xml.
                        EncodedString                       = JobName.ToString();
                        ALSolutionCommon.EncodeSpecialCharacters(ref EncodedString);
                        StringBuilder FolderCollectionXml   = new StringBuilder();
                        FolderCollectionXml.Append("<FOLDERCOLLECTION>");
                        FolderCollectionXml.Append("<FOLDER Name=\"" + EncodedString + "\">");

                        xIterator                           = xNavigator.Select("descendant::File");
                        foreach (XPathNavigator x_FileNode in xIterator)
                        {
                            RenditionType = x_FileNode.GetAttribute("NTFSFolder", "");
                            //prepare separate folder for each rendition.
                            if (!Directory.Exists(JobNamePath.ToString() + "\\" + RenditionType))
                                Directory.CreateDirectory(JobNamePath.ToString() + "\\" + RenditionType);

                            tempPath = x_FileNode.InnerXml;
                            FileName = x_FileNode.InnerXml;
                            //FileName = FileName.Substring(FileName.LastIndexOf("\\") + 1);
                            FileName = x_FileNode.GetAttribute("OriginalFileName","");

                            /*
                             * Prepare Preview and Thumbnail files like OriginalFileName.Extension.
                             * Ex. Original file : Test.jpg
                             *     Preview file  : a.gif  => Test.jpg.gif
                             *     Thumbnail file  : b.xmm  => Test.jpg.xmmm
                            

                            if (!RenditionType.ToUpper().Contains("ORIGINAL"))
                            {
                                FileName = OriginalFileName + Path.GetExtension(tempPath);
                            }
                            */

                            ALSolutionCommon.DecodeSpecialCharacters(ref FileName);
                            FilePath = JobNamePath.ToString() + "\\" + RenditionType + "\\" + FileName;
                            File.Move(x_FileNode.Value, FilePath);

                            EncodedString = FilePath;
                            ALSolutionCommon.EncodeSpecialCharacters(ref EncodedString);
                            FolderCollectionXml.Append("<SUBFOLDER Name='" + RenditionType + "'>");
                            FolderCollectionXml.Append("<FILE>" + EncodedString + "</FILE>");
                            FolderCollectionXml.Append("</SUBFOLDER>");
                        }

                        #endregion

                        #region Prepare AssetInfo File.
                        ErrorXml = PrepareBatchCatalogAssetInfo(SectionPath, AdditionalInfoXml, xNavigator, ForCheckIn);
                        EncodedString = JobNamePath.ToString();
                        ALSolutionCommon.EncodeSpecialCharacters(ref EncodedString);
                        FolderCollectionXml.Append("<FILE>" + EncodedString + "\\AssetInfo" + "</FILE>");
                        FolderCollectionXml.Append("</FOLDER>");
                        FolderCollectionXml.Append("</FOLDERCOLLECTION>");

                        FolderHierarchy = FolderCollectionXml.ToString();
                        JobPath = JobNamePath.ToString();
                        #endregion
                    }
                    else
                    {
                        //Dump Error for Asynch check In.
                        //return ALSolutionCommon.PrepareErrorXML("Original File must not be empty.");
                    }
                }
                else
                {
                    //Dump Error for Asynch check In.
                    //return ALSolutionCommon.PrepareErrorXML("Must provide section to catalog.");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return ErrorXml;
        }

        private String GetFileInformation(String AssetIDList)
        {
            #region member variables
            StringBuilder SearchInputXml = null;
            String SearchResult = "";
            String ErrorXml = "";
            SearchConfiguration ObjSearch = null;
            XmlDocument SearchResultXml = null;
            XmlNode FileFormatNode = null;
            XmlNode MetaDataCollectionNode = null;
            XmlNodeList RevisionNodeList = null;
            XmlNodeList RenditionNodeList = null;
            XmlNode RevisionNode = null;
            XmlNode RenditionNode = null;
            XmlNode FileSystemNode = null;
            String RenditionType = "";
            String OriginalFileName = "";
            String FilePath = "";
            String FileFormat = "";
            String DigitalAssetID = "";
            StringBuilder FileInformationXML = null;
            /* This will contain list of information of all file of assets depending on rendition type
             *  <File RenditionType="Original" NTFSFolder="Originals">D:\ASSETLINK58\\Temp\133296667925935\\10149517.jpg</File>
             *  <File RenditionType="Original" NTFSFolder="Originals">D:\ASSETLINK58\\Temp\133296667925935\\xcx.jpg</File>
             */
            #endregion

            try
            {
                #region prepare searchInput Xml
                SearchInputXml = new StringBuilder();
                SearchInputXml.Append("<INPUTXML>");
                SearchInputXml.Append("<SESSIONINFO TenantName='" + TenantName + "' Language='" + CurrentLanguage + "' UserID='" + UserID + "' SessionID='" + SessionID + "'/>");
                SearchInputXml.Append("<SEARCHCOLLECTION>");
                SearchInputXml.Append("<SEARCH>");
                SearchInputXml.Append("<TARGETOBJECT>CONTENTREVISION</TARGETOBJECT>");
                SearchInputXml.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
                SearchInputXml.Append("<SORTCOLUMN>CONTENTREVISIONID</SORTCOLUMN>");
                SearchInputXml.Append("<SORTORDER>ASC</SORTORDER>");
                SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                SearchInputXml.Append("<SEARCHSTRING/>");
                SearchInputXml.Append("<SEARCHALLFACETS>true</SEARCHALLFACETS>");
                SearchInputXml.Append("<FILTERMODE ApplyFilter=''/>");
                SearchInputXml.Append("<FILTERLIST>");
                SearchInputXml.Append("<FILTER>");
                SearchInputXml.Append("<FIELDNAME>DIGITALASSETID</FIELDNAME>");
                SearchInputXml.Append("<OPERATOR>IN</OPERATOR>");
                SearchInputXml.Append("<FIELDVALUE>(" + AssetIDList + ")</FIELDVALUE>");
                SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                SearchInputXml.Append("<NAVIGATIONPATH>CONTENTREVISION.DIGITALASSET_CURRENTREVISION</NAVIGATIONPATH>");
                SearchInputXml.Append("</FILTER>");
                SearchInputXml.Append("<FILTER>");
                SearchInputXml.Append("<FIELDNAME>RENDITIONTYPE</FIELDNAME>");
                SearchInputXml.Append("<OPERATOR>LIKE</OPERATOR>");
                SearchInputXml.Append("<FIELDVALUE>Thumbnail</FIELDVALUE>");
                SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                SearchInputXml.Append("<NAVIGATIONPATH>CONTENTREVISION.CONTENTREVISION_RENDITION</NAVIGATIONPATH>");
                SearchInputXml.Append("</FILTER>");
                SearchInputXml.Append("<FILTER>");
                SearchInputXml.Append("<FIELDNAME>RENDITIONTYPE</FIELDNAME>");
                SearchInputXml.Append("<OPERATOR>LIKE</OPERATOR>");
                SearchInputXml.Append("<FIELDVALUE>Original</FIELDVALUE>");
                SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                SearchInputXml.Append("<NAVIGATIONPATH>CONTENTREVISION.CONTENTREVISION_RENDITION</NAVIGATIONPATH>");
                SearchInputXml.Append("</FILTER>");
                SearchInputXml.Append("<FILTER>");
                SearchInputXml.Append("<FIELDNAME>RENDITIONTYPE</FIELDNAME>");
                SearchInputXml.Append("<OPERATOR>LIKE</OPERATOR>");
                SearchInputXml.Append("<FIELDVALUE>Preview</FIELDVALUE>");
                SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                SearchInputXml.Append("<NAVIGATIONPATH>CONTENTREVISION.CONTENTREVISION_RENDITION</NAVIGATIONPATH>");
                SearchInputXml.Append("</FILTER>");
                SearchInputXml.Append("</FILTERLIST>");
                SearchInputXml.Append("<FIELDCOLLECTION>");
                SearchInputXml.Append("<TARGETENTITYFIELDS>");
                SearchInputXml.Append("<FIELD>CONTENTREVISIONID</FIELD>");
                SearchInputXml.Append("</TARGETENTITYFIELDS>");
                SearchInputXml.Append("<RELATEDENTITYFIELDS NavigationPath='CONTENTREVISION.CONTENTREVISION_RENDITION'>");
                SearchInputXml.Append("<FIELD>ASSETRENDITIONID</FIELD>");
                SearchInputXml.Append("<FIELD>NAME</FIELD>");
                SearchInputXml.Append("<FIELD>RENDITIONTYPE</FIELD>");
                SearchInputXml.Append("<FIELD>FILENAME</FIELD>");
                SearchInputXml.Append("<FIELD>ACTUALFILE</FIELD>");
                SearchInputXml.Append("<FIELD>FILEFORMAT</FIELD>");
                SearchInputXml.Append("<FIELD>FILESIZE</FIELD>");
                SearchInputXml.Append("</RELATEDENTITYFIELDS>");
                SearchInputXml.Append("<RELATEDENTITYFIELDS NavigationPath='CONTENTREVISION.DIGITALASSET_CURRENTREVISION'>");
                SearchInputXml.Append("<FIELD>DIGITALASSETID</FIELD>");
                SearchInputXml.Append("<FIELD>NAME</FIELD>");
                SearchInputXml.Append("</RELATEDENTITYFIELDS>");
                SearchInputXml.Append("</FIELDCOLLECTION>");
                SearchInputXml.Append("</SEARCH>");
                String MetaDataSearchXml = PrepareSearchXMLForMetaData(AssetIDList);
                if (!String.IsNullOrEmpty(MetaDataSearchXml))
                    SearchInputXml.Append(MetaDataSearchXml);
                SearchInputXml.Append("</SEARCHCOLLECTION>");
                SearchInputXml.Append("</INPUTXML>");
                #endregion

                FileInformationXML = new StringBuilder();

                #region perform search
                ObjSearch = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXml = ObjSearch.DoSearch(SearchInputXml.ToString(), ref SearchResult);
                if (String.IsNullOrEmpty(ErrorXml) && !String.IsNullOrEmpty(SearchResult))
                {
                    #region Get file for each file - each rendition
                    SearchResultXml = new XmlDocument();
                    SearchResultXml.LoadXml(SearchResult);
                    RevisionNodeList = SearchResultXml.SelectNodes("descendant::CONTENTREVISIONCOLLECTION/CONTENTREVISION");
                    for (int RevisionCnt = 0; RevisionCnt < RevisionNodeList.Count; RevisionCnt++)
                    {
                        OriginalFileName = "";
                        RevisionNode = RevisionNodeList[RevisionCnt];

                        DigitalAssetID = RevisionNode.SelectSingleNode("DIGITALASSET_CURRENTREVISION/DIGITALASSET/DIGITALASSETID").InnerText;
                        if (RevisionNode.SelectSingleNode("DIGITALASSET_CURRENTREVISION/DIGITALASSET/NAME") != null)
                        {
                            OriginalFileName = RevisionNode.SelectSingleNode("DIGITALASSET_CURRENTREVISION/DIGITALASSET/NAME").InnerText;
                            if (OriginalFileName.IndexOf(".") == -1)
                            {
                                FileFormatNode = RevisionNode.SelectSingleNode("CONTENTREVISION_RENDITION/ASSETRENDITION/RENDITIONTYPE[text()='Original']/FILEFORMAT");
                                if (FileFormatNode != null)
                                {
                                    FileFormat = FileFormatNode.InnerText;
                                    OriginalFileName = OriginalFileName + "." + FileFormat;
                                }
                            }
                        }

                        RenditionNodeList = RevisionNode.SelectNodes("CONTENTREVISION_RENDITION/ASSETRENDITION");
                        for (int RenditionCnt = 0; RenditionCnt < RenditionNodeList.Count; RenditionCnt++)
                        {
                            RenditionNode = RenditionNodeList[RenditionCnt];
                            if (RenditionNode.SelectSingleNode("RENDITIONTYPE") != null)
                            {
                                RenditionType = RenditionNode.SelectSingleNode("RENDITIONTYPE").InnerText;                                
                            }
                            FileFormatNode = RenditionNode.SelectSingleNode("FILEFORMAT");
                            if (FileFormatNode != null)
                                FileFormat = FileFormatNode.InnerText;

                            //get file from server
                            FileSystemNode = RenditionNode.SelectSingleNode("FILESYSTEM");
                            if (FileSystemNode != null)
                            {
                                /*
                                 * Prepare Preview and Thumbnail files like OriginalFileName.Extension.
                                 * Ex. Original file : Test.jpg
                                 *     Preview file  : a.gif  => Test.jpg.gif
                                 *     Thumbnail file  : b.xmm  => Test.jpg.xmmm
                                 */
                                String GetFileName = "";
                                if (!RenditionType.ToUpper().Equals("ORIGINAL"))
                                    GetFileName = OriginalFileName + "." + FileFormat;
                                else
                                    GetFileName = OriginalFileName;

                                String File_Name = FileSystemNode.FirstChild.SelectSingleNode("OBJECTID").InnerText + "." + DamCommon.GetFileExtension(FileSystemNode.FirstChild.SelectSingleNode("NAME").InnerText);                                
                                String TempFilePath = ALSolutionCommon.GetDownloadFilePath(TenantName,SessionID) + File_Name;
                                
                                if (!File.Exists(TempFilePath))
                                    FilePath = GetFileAsynch(FileSystemNode, RenditionType, GetFileName);
                                else
                                    FilePath = TempFilePath;
                                
                                if (!String.IsNullOrEmpty(FilePath))
                                {
                                    FileInformationXML.Append("<File RenditionType='" + RenditionType + "' NTFSFolder='" + RenditionType + "s' DigitalAssetID='" + DigitalAssetID + "' OriginalFileName='" + ALSolutionCommon.TrimAndReplaceSpecialChars(GetFileName) + "'>");
                                    FileInformationXML.Append(ALSolutionCommon.TrimAndReplaceSpecialChars(FilePath));
                                    FileInformationXML.Append("</File>");
                                }
                            }
                        }
                    }

                    MetaDataCollectionNode = SearchResultXml.SelectSingleNode("descendant::DIGITALASSETCOLLECTION");
                    if (MetaDataCollectionNode != null)
                    {                        
                        String MetadataPath = ALSolutionCommon.GetDownloadFilePath(TenantName,SessionID) + SessionID + "_AssetMetaData.xml";
                        StreamWriter writer = new StreamWriter(MetadataPath);
                        writer.Write(MetaDataCollectionNode.OuterXml);
                        writer.Close();
                    }
                    #endregion
                }
                else
                    return null;
                #endregion
            }
            catch (Exception )
            {
                return null;
            }
            return FileInformationXML.ToString();
        }

        /// This function will return file for perticuler asset
        /// </summary>
        /// <Author>Meenal Joshi</Author>
        /// <Date>10 Nov 2006</Date>
        /// <param name="FileSystemNode">FileSystemNode</param>        
        /// <returns>File Path, null otherwise</returns>
        public String GetFileAsynch(XmlNode FileSystemNode, String RenditionType,String GetFileName)
        {
            String          InputXml                    = null;
            XmlNode         SrcPathNode                 = null;
            XmlNode         ObjectIDNode                = null;
            Assetlink.Solutions.Facade.CommonFacade.FileTransfer FTObj = null;            
            String          DownloadedFilePathSessionId = null;
            bool            IsFileOnMediaServer         = false;

            try
            {
                if (FileSystemNode != null)
                {
                    ErrorXML = null;
                    ObjFileFacade = new FileFacade(SessionID, UserID, this.TenantName, this.CurrentLanguage);
                    ((XmlElement)FileSystemNode.FirstChild).SetAttribute("FileRequired", "YES");

                    ObjectIDNode = FileSystemNode.SelectSingleNode("descendant::OBJECTID");
                    if (ObjectIDNode != null)
                        ((XmlElement)FileSystemNode).SetAttribute("ObjectID", ObjectIDNode.InnerText);

                    InputXml = FileSystemNode.OuterXml;
                    ErrorXML = ObjFileFacade.GetFile(ref InputXml);
                    if (ErrorXML != null && ErrorXML != "")
                    {
                        return "";
                    }

                    XmlDocument OutXmldocument = new XmlDocument();
                    OutXmldocument.LoadXml(InputXml);
                    FileSystemNode = OutXmldocument.SelectSingleNode("//FILESYSTEM");

                    SrcPathNode                 = FileSystemNode.OwnerDocument.CreateElement("SOURCEPATH");
                    SrcPathNode.InnerText       = FileSystemNode.FirstChild.SelectSingleNode("PATH").InnerText;
                    FileSystemNode.FirstChild.AppendChild(SrcPathNode);                    
                    DownloadedFilePathSessionId = ALSolutionCommon.GetDownloadFilePath(TenantName);

                    if (FileSystemNode.FirstChild.SelectSingleNode("ONMEDIASERVER") != null)
                        IsFileOnMediaServer = Boolean.Parse(FileSystemNode.FirstChild.SelectSingleNode("ONMEDIASERVER").InnerText);

                    FTObj                       = new Assetlink.Solutions.Facade.CommonFacade.FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);                    
                    FTObj.GetFilePath           = DownloadedFilePathSessionId;
                    FTObj.UseMediaServer        = IsFileOnMediaServer;
                    OutputXml                   = FileSystemNode.OuterXml;

                    ErrorXML = null;
                    FTObj.DownloadFileName = FileSystemNode.FirstChild.SelectSingleNode("OBJECTID").InnerText + "." + DamCommon.GetFileExtension(FileSystemNode.FirstChild.SelectSingleNode("NAME").InnerText);                    
                    ErrorXML = FTObj.GetFile(ref OutputXml);

                    if (ErrorXML == null)
                    {
                        return DownloadedFilePathSessionId + FileSystemNode.FirstChild.SelectSingleNode("OBJECTID").InnerText + "." + DamCommon.GetFileExtension(FileSystemNode.FirstChild.SelectSingleNode("NAME").InnerText);
                    }

                }
            }
            catch (Exception )
            {
                return "";
            }
            return "";
        }

        public String PrepareBatchCatalogAssetInfo(String SectionPath, String AdditionalInfoXml, XPathNavigator xNavigator,bool ForCheckIn)
        {
            StringBuilder AssetInfo                     = null;
            XmlDocument AssetInfoDoc                    = null;
            XPathDocument xAssociatedFacetsDoc          = null;
            XmlNode DigitalAssetCollNode                = null;
            XPathNavigator xAssociatedFacetsNav         = null;
            XPathNodeIterator xAssociatedFacetsIterator = null;
            String OriginalFileName                     = "";            
            XmlNode MetaDataAssetNode                   = null;
            XmlNode MetaDataAssetIDNode                 = null;   
            XmlNode FacetPrimaryKeyNode                 = null;
            XmlNode FacetNode                           = null;
            XPathNodeIterator xOriginalFileIterator     = null;            
            String DigitalAssetID                       = "";
            String RelationshipName                     = "";
            String TypeName                             = "";
            String PrimaryKey                           = "";
            XmlDocument     xmlDoc_AdditionalInfoXml    = null;
            StringBuilder   sb_AdditionalInfoXml        = null;
            XmlNodeList     BinderFileNodes             = null;
            XmlNodeList     FileNodes                   = null;

            String InstallationDir = ALSolutionCommon.GetInstallationDir();
                        
            String MetadataPath             = ALSolutionCommon.GetDownloadFilePath(TenantName,SessionID) + "\\" + SessionID + "_AssetMetaData.xml";            
            String AssociatedSchemaXMLPath = ALSolutionCommon.GetDownloadFilePath(TenantName,SessionID) + "\\" + "FacetsForDigitalAsset.xml";

            try
            {
                if (File.Exists(AssociatedSchemaXMLPath))
                {
                    xAssociatedFacetsDoc = new XPathDocument(AssociatedSchemaXMLPath);
                    xAssociatedFacetsNav = xAssociatedFacetsDoc.CreateNavigator();
                    
                    if (xAssociatedFacetsNav != null)
	                {
                        xmlDoc_AdditionalInfoXml = new XmlDocument();
                        sb_AdditionalInfoXml = new StringBuilder (200);
                        sb_AdditionalInfoXml.Append("<ADDITIONALINFO>" + AdditionalInfoXml + "</ADDITIONALINFO>");
	                }
                }
                
                if (File.Exists(MetadataPath))
                {
                    AssetInfoDoc = new XmlDocument();
                    AssetInfoDoc.Load(MetadataPath);
                    DigitalAssetCollNode = AssetInfoDoc.SelectSingleNode("DIGITALASSETCOLLECTION");
                }
                xOriginalFileIterator = xNavigator.Select("descendant::UploadedFiles/File[@RenditionType='Original']");
                ALSolutionCommon.EncodeSpecialCharacters(ref OriginalFileName);
                AssetInfo = new StringBuilder();
                if (ForCheckIn)
                    AssetInfo.Append("<ASSETINFO ForCheckin='TRUE'>");
                else
                    AssetInfo.Append("<ASSETINFO ForCheckin='FALSE'>");

                ALSolutionCommon.EncodeSpecialCharacters(ref SectionPath);
                AssetInfo.AppendFormat("<SECTION Path=\"{0}\">", SectionPath);

                #region If DigitalAsset contains FILESYSTEM
                // This code is added , in case of copy assets having BINDER field in businessinfo
                // PATH is given as MOUTPOINT path, this PATH  = PATH + FILEID
                BinderFileNodes = DigitalAssetCollNode.SelectNodes("DIGITALASSET/*/*/FILESYSTEM/BINDER/FILE");
                foreach (XmlNode BinderFile in BinderFileNodes)
                {
                    if (BinderFile.SelectSingleNode("FILEID") != null)
                        BinderFile.SelectSingleNode("PATH").InnerText = BinderFile.SelectSingleNode("PATH").InnerText + "\\" + BinderFile.SelectSingleNode("FILEID").InnerText;
                }

                // This code is added , in case of copy assets having FILE field in businessinfo
                // PATH is given as MOUTPOINT path, this PATH  = PATH + FILEID
                FileNodes = DigitalAssetCollNode.SelectNodes("DIGITALASSET/*/*/FILESYSTEM/FILE");
                foreach (XmlNode FileNode in FileNodes)
                {
                    if (FileNode.SelectSingleNode("FILEID") != null)
                        FileNode.SelectSingleNode("PATH").InnerText = FileNode.SelectSingleNode("PATH").InnerText + "\\" + FileNode.SelectSingleNode("FILEID").InnerText;
                }
                #endregion

                foreach (XPathNavigator xAssetNode in xOriginalFileIterator)
                {
                    DigitalAssetID = xAssetNode.GetAttribute("DigitalAssetID","");                    
                    OriginalFileName = xAssetNode.GetAttribute("OriginalFileName", "");
                    AssetInfo.Append("<DIGITALASSET>");
                    AssetInfo.AppendFormat("<NAME>{0}</NAME>", ALSolutionCommon.TrimAndReplaceSpecialChars(OriginalFileName));
                    
                    #region Add MetaInformation with Asset

                    if (DigitalAssetCollNode != null)
                    {
                        MetaDataAssetIDNode = DigitalAssetCollNode.SelectSingleNode("descendant::DIGITALASSETID[text()='" + DigitalAssetID + "']");
                        if (MetaDataAssetIDNode != null)
                        {
                            MetaDataAssetNode = MetaDataAssetIDNode.ParentNode;

                            AssetInfo.AppendFormat("<DESCRIPTION>{0}</DESCRIPTION>", ALSolutionCommon.TrimAndReplaceSpecialChars(MetaDataAssetNode.SelectSingleNode("DESCRIPTION").InnerText));
                            AssetInfo.AppendFormat("<EXPIRYDATE>{0}</EXPIRYDATE>", MetaDataAssetNode.SelectSingleNode("EXPIRYDATE").InnerXml);
                       
							if(xAssociatedFacetsNav != null)
							{
                                xmlDoc_AdditionalInfoXml.LoadXml(sb_AdditionalInfoXml.ToString());
								xAssociatedFacetsIterator = xAssociatedFacetsNav.Select("TYPEINFOCOLLECTION/TYPE");
								foreach(XPathNavigator xAssociatedFacetNode in xAssociatedFacetsIterator)
								{
									RelationshipName = xAssociatedFacetNode.GetAttribute("RelationshipName", "");
									TypeName = xAssociatedFacetNode.SelectSingleNode("TYPENAME").Value;
									PrimaryKey = xAssociatedFacetNode.SelectSingleNode("PRIMARYKEY").Value;

                                    //Author : Bhalchandra Kadam
                                    //Date   : 05 February 2009
                                    //Comment: Remove the duplicate facet information from AdditionalInfoXml and update FacetInformationXML with new data.
                                    //Check if Facet is present in AdditionInfoXml.
                                    FacetNode = xmlDoc_AdditionalInfoXml.SelectSingleNode("ADDITIONALINFO/" + RelationshipName);
                                    if (FacetNode != null)
                                    {
                                        //Check if the DigitalAsset's facet information also have facets associated.
                                        XmlNode xmlNode_AssetFacet = MetaDataAssetNode.SelectSingleNode(RelationshipName);
                                        if (xmlNode_AssetFacet != null)
                                        {
                                            foreach (XmlNode xmlNode_FacetField in FacetNode.ChildNodes[0].ChildNodes)
                                            {
                                                if (xmlNode_AssetFacet.SelectSingleNode(FacetNode.ChildNodes[0].Name + "/" + xmlNode_FacetField.Name) != null)
                                                {
                                                    xmlNode_AssetFacet.SelectSingleNode(FacetNode.ChildNodes[0].Name + "/" + xmlNode_FacetField.Name).InnerXml = xmlNode_FacetField.InnerXml;
                                                }
                                                else
                                                {
                                                    xmlNode_AssetFacet.SelectSingleNode(FacetNode.ChildNodes[0].Name).InnerXml += xmlNode_FacetField.OuterXml;
                                                }
                                            }
                                            FacetPrimaryKeyNode = xmlNode_AssetFacet.SelectSingleNode(FacetNode.ChildNodes[0].Name + "/" + PrimaryKey);
                                            if (FacetPrimaryKeyNode != null)
                                            {
                                                FacetPrimaryKeyNode.ParentNode.RemoveChild(FacetPrimaryKeyNode);
                                            }
                                            ((XmlElement)xmlNode_AssetFacet).SetAttribute("TYPE", "RELATION");
                                            AssetInfo.Append(xmlNode_AssetFacet.OuterXml);
                                        }
                                        else
                                        {
                                            AssetInfo.Append(FacetNode.OuterXml);
                                        }
                                        FacetNode.ParentNode.RemoveChild(FacetNode);
                                    }
                                    else
                                    {
                                        FacetPrimaryKeyNode = MetaDataAssetNode.SelectSingleNode(RelationshipName + "/" + TypeName + "/" + PrimaryKey);
                                        if (FacetPrimaryKeyNode != null)
                                        {
                                            FacetNode = FacetPrimaryKeyNode.ParentNode;
                                            FacetNode.RemoveChild(FacetPrimaryKeyNode);

                                            ((XmlElement)FacetNode.ParentNode).SetAttribute("TYPE", "RELATION");
                                            AssetInfo.Append(FacetNode.ParentNode.OuterXml);
                                        }
                                    }
								}
                                AdditionalInfoXml = xmlDoc_AdditionalInfoXml.DocumentElement.InnerXml;
							}
                        }
                    }                   
                    
                    #endregion

                    if (!String.IsNullOrEmpty(AdditionalInfoXml))
                    {
                        //AssetInfo.Append(AdditionalInfoXml);
                        String TmpAssetInfo = "";
                        XmlDocument tDoc = new XmlDocument();
                        tDoc.LoadXml(AssetInfo.ToString() + "</DIGITALASSET></SECTION></ASSETINFO>");
                        XPathNodeIterator xpitr = tDoc.CreateNavigator().Select("ASSETINFO/SECTION/DIGITALASSET");
                        Boolean flag = false;
                        while(xpitr.MoveNext())
                        {
                            if (xpitr.Current.SelectSingleNode("NAME").Value == OriginalFileName)
                            {
                                TmpAssetInfo = ProcessAdditionalInfoNode(xpitr.Current.OuterXml, "<ADDITIONALINFO>" + AdditionalInfoXml + "</ADDITIONALINFO>");
                                xpitr.Current.InnerXml = TmpAssetInfo;
                                flag = true;
                                break;
                            }
                        }
                        if (flag)
                        {
                            AssetInfo.Remove(0, AssetInfo.Length);
                            String TmpStr = tDoc.OuterXml;
                            TmpStr = TmpStr.Remove(TmpStr.LastIndexOf("</SECTION>"));
                            AssetInfo.Append(TmpStr);
                        }
                    }
                    else    
                        AssetInfo.Append("</DIGITALASSET>");                    
                }
                AssetInfo.Append("</SECTION>");
                AssetInfo.AppendFormat("<DIGITALASSETCOUNT>" + xOriginalFileIterator.Count + "</DIGITALASSETCOUNT>");
                AssetInfo.Append("</ASSETINFO>");

                StreamWriter writer = new StreamWriter(JobNamePath.ToString() + "\\AssetInfo");
                writer.Write(AssetInfo.ToString());
                writer.Close();
            }
            catch (Exception ex)
            {
                return ALSolutionCommon.PrepareErrorXML("Error occured while preparing AssetInfo file :" + ex.Message);
            }
            return "";
        }

        private String ProcessAdditionalInfoNode(String TmpAssetInfo,String AddInfoXML)
        {
            
            XmlDocument tmpAssetInfoDoc = new XmlDocument();
            tmpAssetInfoDoc.LoadXml(TmpAssetInfo);

            XmlDocument tmpAddInfoDoc = new XmlDocument();
            tmpAddInfoDoc.LoadXml(AddInfoXML);

            XmlNodeList NonRelNodes = tmpAddInfoDoc.SelectSingleNode("ADDITIONALINFO").ChildNodes;

            XmlNodeList RelNodes = tmpAddInfoDoc.DocumentElement.SelectNodes("//* [@TYPE='RELATION'] | //* [@TYPE='Relation']");


            //Non-Relationship nodes
            foreach (XmlNode TNonRelNode in NonRelNodes)
            {
                if (TNonRelNode.Attributes["TYPE"] == null)
                {
                    if (tmpAssetInfoDoc.SelectSingleNode("DIGITALASSET/" + TNonRelNode.Name) != null)
                        tmpAssetInfoDoc.SelectSingleNode("DIGITALASSET/" + TNonRelNode.Name).InnerXml += TNonRelNode.InnerXml;
                    else
                        tmpAssetInfoDoc.SelectSingleNode("DIGITALASSET").InnerXml += TNonRelNode.InnerXml;
                }
            }

            //Relationship nodes
            foreach (XmlNode TRelNode in RelNodes)
            {
                if (tmpAssetInfoDoc.SelectSingleNode(TRelNode.Name) == null)
                    tmpAssetInfoDoc.SelectSingleNode("DIGITALASSET").InnerXml += TRelNode.OuterXml;
                
                /* DO NOT DELETE -- Code to handle child nodes of Relationship node in Additional info XML
                 * THIS CODE IS COMMENTED AS THIS CODE PATH IS CURRENTLY NOT IS USE, BUT IN FUTURE IT MAY BE
                 * REQUIRED.
                 * else
                {
                    XmlNode tmpRelNode = tmpAssetInfoDoc.SelectSingleNode("DIGITALASSET/" + TRelNode.Name);
                    ProcessChild(TRelNode, tmpRelNode);
                }*/
            }

            return tmpAssetInfoDoc.DocumentElement.InnerXml;
        }

        /* DO NOT DELETE -- Code to handle child nodes of Relationship node in Additional info XML
        private void ProcessChild(XmlNode tnode, XmlNode AssetInfonode)
        {

            if (tnode.ChildNodes.Count == 0)
            {
                AssetInfonode.InnerText = tnode.InnerText;
            }

            XmlNodeList nlist = tnode.ChildNodes;
            foreach (XmlNode t in nlist)
            {
                if (AssetInfonode.SelectSingleNode(t.Name) == null)
                    AssetInfonode.InnerXml += t.OuterXml;
                else
                {
                    XmlNode trnode = AssetInfonode.SelectSingleNode(t.Name);
                    ProcessChild(t, trnode);
                }
            }
        }*/

        private String PrepareSearchXMLForMetaData(String AssetIDList)
        {
            StringBuilder SearchInputXml = null;
            TMFacade ObjTMSFacade = null;
            String AssociatedSchemaXML = "";
            XPathDocument xAssociationXML = null;
            XPathNavigator xAssociationNav = null;
            XPathNodeIterator xAssoIterator = null;
            String AssociationName = "";
            String ErrorXML = "";

            SearchInputXml = new StringBuilder();
            SearchInputXml.Append("<SEARCH>");
            SearchInputXml.Append("<TARGETOBJECT>DIGITALASSET</TARGETOBJECT>");
            SearchInputXml.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
            SearchInputXml.Append("<SORTCOLUMN>DIGITALASSETID</SORTCOLUMN>");
            SearchInputXml.Append("<SORTORDER>ASC</SORTORDER>");
            SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");            
            SearchInputXml.Append("<SEARCHSTRING/>");            
            SearchInputXml.Append("<FILTERMODE ApplyFilter=\"\"/>");
            SearchInputXml.Append("<FILTERLIST>");
            SearchInputXml.Append("<FILTER>");
            SearchInputXml.Append("<FIELDNAME>DIGITALASSETID</FIELDNAME>");
            SearchInputXml.Append("<OPERATOR>IN</OPERATOR>");
            SearchInputXml.Append("<FIELDVALUE>(" + AssetIDList + ")</FIELDVALUE>");
            SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
            SearchInputXml.Append("<NAVIGATIONPATH>DIGITALASSET</NAVIGATIONPATH>");
            SearchInputXml.Append("</FILTER>");
            SearchInputXml.Append("</FILTERLIST>");
            SearchInputXml.Append("<FIELDCOLLECTION>");
            SearchInputXml.Append("<TARGETENTITYFIELDS>");
            SearchInputXml.Append("<FIELD>DIGITALASSETID</FIELD>");
            SearchInputXml.Append("<FIELD>DESCRIPTION</FIELD>");
            SearchInputXml.Append("<FIELD>EXPIRYDATE</FIELD>");
            SearchInputXml.Append("</TARGETENTITYFIELDS>");

            #region Get Associated Facets with DigitalAsset and Get the data if available.
            /*
             <RELATEDFACETFIELDS NavigationPath="DIGITALASSET.DIGITALASSET_SIMPLESEARCH"/>
            */

            ObjTMSFacade = new TMFacade(SessionID, UserID, TenantName, CurrentLanguage);
            ErrorXML = ObjTMSFacade.GetAssociatedSchemaByName("ENTITY", "DIGITALASSET", "FACET", "DESCENDENT", ref AssociatedSchemaXML);
            if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(AssociatedSchemaXML))
            {                
                String AssociatedSchemaXMLPath = ALSolutionCommon.GetDownloadFilePath(TenantName,SessionID) + "\\" + "FacetsForDigitalAsset.xml";

                File.WriteAllText(AssociatedSchemaXMLPath, AssociatedSchemaXML);

                xAssociationXML = new XPathDocument(new System.IO.StringReader(AssociatedSchemaXML));
                xAssociationNav = xAssociationXML.CreateNavigator();
                xAssoIterator = xAssociationNav.Select("descendant::TYPE");
                foreach (XPathNavigator xFacetInfoNode in xAssoIterator)
                {
                    AssociationName = xFacetInfoNode.GetAttribute("RelationshipName", "");
                    SearchInputXml.Append("<RELATEDFACETFIELDS NavigationPath=\"DIGITALASSET." + AssociationName + "\"/>");
                }
            }
            #endregion
            SearchInputXml.Append("</FIELDCOLLECTION>");
            SearchInputXml.Append("</SEARCH>");       
            return SearchInputXml.ToString();
        }

        private bool GetJobNameifLengthyFileName(String m_ItemName, StringBuilder JobName, StringBuilder JobNamePath, String str_Dttime, String OriginalFileName)
        {   
            Int32 endIndex = 0;
            bool IsValidJobPath = true;
            String str_TmpJobFilePath = null;

            //11 because : 11 + DateTimeString = 25
            if (m_ItemName.Length > 11)
                endIndex = 11;
            else
                endIndex = m_ItemName.Length;

            if ((JobNamePath.ToString().Length + OriginalFileName.Length) >= MAX_ALLOWED_CHAR_FOR_CATALOGJOB_FILE)
            {                
                JobName.Remove(0, JobName.Length);
                JobName.AppendFormat("{0}.._{1}", m_ItemName.Substring(0, endIndex), str_Dttime); 
                JobNamePath.Remove(0, JobNamePath.Length);
                JobNamePath.AppendFormat("{0}\\{1}\\{2}\\{3}", ALSolutionCommon.GetTempPath(SessionID), UserID, TempJobFolderName, JobName);
                str_TmpJobFilePath = String.Format("{0}\\{1}\\{2}\\{3}\\Originals\\", ALSolutionCommon.GetTempPath(SessionID), UserID, TempJobFolderName, JobName);
                if ((str_TmpJobFilePath.Length + OriginalFileName.Length) >= MAX_ALLOWED_CHAR_FOR_CATALOGJOB_FILE)
                    IsValidJobPath = false;

            }
            return IsValidJobPath;
        }
        #endregion

    }
}
