#region Standard Include
using System;
using System.Text;
using System.Xml;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Threading;
using System.Configuration;
using System.Reflection;
using System.Xml.XPath;
using System.Text.RegularExpressions;
using System.Runtime.InteropServices;
using System.Diagnostics;
# endregion

#region Assetlink Include
using Assetlink;
using Assetlink.WebServices.ALDAMWS;
using Assetlink.WebServices.ALSearchWS;
using Assetlink.Services.WindowsService;
using MEDIAPROCESSORWRAPPERLib;
using BINHEXTRACTLib;
using Assetlink.Services.WindowsService.CatalogerHelper;
using Assetlink.Services.MediaProcessor.ManagedMediaProcessorWrapper;
using Assetlink.Metadata;
#endregion

namespace Assetlink.Services.WindowsService.CatalogJobProcessor
{
    using XML       = System.String;
    using ERRORXML  = System.String;
    using ENTITYID  = System.UInt64;
    
    public class JobProcessor
    {
         const ulong        SystemUserID            = 111111122211111;
         const String       SystemUserName          = "MePhIsToPhElEs";
         const String       SystemUserPwd           = "ErAtOsThEnEs";
         const String       Language                = "English";
         ulong              SessionID               = 0;
         String             EncryptedSystemUserName = null;
         String             EncryptedSystemUserPwd  = null;
         String             ImpersonationToken      = null;
         ALDAMWS            DAMWSObj                = null;
         ALServiceHeader    RequestHeader           = null;
         Hashtable          TenantInfo              = null;
         XmlDocument        FileFormatXMLDoc        = null;
         XPathDocument      PluginsXMLDoc           = null;
         XPathDocument      XMPFacetConfigXMLDoc    = null;


        public JobProcessor(Hashtable TenantInfo, XmlDocument FileFormatXMLDoc, XPathDocument PluginsXMLDoc, XPathDocument XMPFacetConfigXMLDoc)
         {
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
			this.TenantInfo             = TenantInfo;
            this.PluginsXMLDoc          = PluginsXMLDoc;
            DAMWSObj                    = new ALDAMWS();
            RequestHeader               = new ALServiceHeader();
            this.FileFormatXMLDoc       = FileFormatXMLDoc;
            this.XMPFacetConfigXMLDoc   = XMPFacetConfigXMLDoc;
            DAMWSObj.Url                = ConfigurationManager.AppSettings["DAMWSURL"];
         }

		void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
			String Message = "Is terminating: " + e.IsTerminating;
			Message += Environment.NewLine + "Stack Trace: " + e.ExceptionObject.ToString();
			CatalogerHelper.CatalogerHelper.WriteLogToNTFS(Message, new StackTrace(true));	

            //Explicitly Restart the Cataloger Service
            Message = "Restarting Cataloger Service... ";
            CatalogerHelper.CatalogerHelper.WriteLogToNTFS(Message, null);
            CatalogerHelper.CatalogerHelper.Restart("SAS MOM Cataloger Service");
		}

        public void ProcessReadyJobs()
        {
            
            ERRORXML        ErrorXML                = null;
            XmlDocument     CatlogJobXMLDoc         = null;
            XML             CatlogJobXML            = null;
            XmlNodeList     InProgressJobs          = null;
            XmlNodeList     ReleasedJobs            = null;
            JobHandler      JobHandlerObj           = null;
            String          TenantName              = null;
            String          strIsPrimaryCataloger   = null;
            String          LocalMachineIPAdd       = null;
            Boolean         IsPrimaryCataloger      = false;

            try
            {
                EncryptedSystemUserName = ALCommon.Encrypt(SystemUserName);
                EncryptedSystemUserPwd  = ALCommon.Encrypt(SystemUserPwd);
                ErrorXML                = Infrastructure.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, ref ImpersonationToken);
               
                if (ErrorXML == null)
                {
					SetALServiceHeaderValue(SessionID, SystemUserID, "Assetlink", "English");
					CatalogerHelper.CatalogerHelper.WriteTraceLogs("Query to get all jobs started.", null, MethodInfo.GetCurrentMethod().DeclaringType.ToString() + "." + MethodInfo.GetCurrentMethod().Name);

                    #region Check whether primary cataloger and fetch CatalogerMacIP
                    strIsPrimaryCataloger   = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "IsPrimaryCataloger");
                    
                    if ((String.IsNullOrEmpty(strIsPrimaryCataloger)) || (strIsPrimaryCataloger == "1"))
                    {
                        LocalMachineIPAdd   = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerMachineIP");
                        IsPrimaryCataloger  = true;
                    }
                    else
                    {
                        LocalMachineIPAdd   = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "SecondaryCatalogerMachineIP");

                        //Case: SecondaryCataloger on seprate machine
                        if (String.IsNullOrEmpty(LocalMachineIPAdd))
                        {
                            LocalMachineIPAdd   = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerMachineIP");
                        }
                    }
                    #endregion

					ErrorXML            = DAMWSObj.GetNewCatalogerJobs(LocalMachineIPAdd,IsPrimaryCataloger, out CatlogJobXML);
                    if (ErrorXML == null &&	!String.IsNullOrEmpty(CatlogJobXML))
                    {
						CatalogerHelper.CatalogerHelper.WriteTraceLogs("Job processing started.", null, MethodInfo.GetCurrentMethod().DeclaringType.ToString() + "." + MethodInfo.GetCurrentMethod().Name);
						CatlogJobXMLDoc = new XmlDocument();
                        CatlogJobXMLDoc.LoadXml(CatlogJobXML);
                        InProgressJobs  = CatlogJobXMLDoc.SelectNodes("//CATALOGJOB[STATUS='In-Progress']");
                        foreach (XmlNode InProgressJob in InProgressJobs)
                        {
                            TenantName                  = InProgressJob.Attributes.GetNamedItem("TenantName").Value;
                            JobHandlerObj               = new JobHandler(ImpersonationToken, TenantInfo, PluginsXMLDoc, FileFormatXMLDoc.DocumentElement.SelectSingleNode(TenantName).OuterXml, XMPFacetConfigXMLDoc);                            
                            JobHandlerObj.SenderName    = InProgressJob.Attributes.GetNamedItem("SenderName").Value;
                            JobHandlerObj.SenderEmail   = InProgressJob.Attributes.GetNamedItem("SenderEmail").Value;
                            JobHandlerObj.SenderID      = InProgressJob.Attributes.GetNamedItem("SenderID").Value;
                            
                            JobHandlerObj.ProcessInprogressJob(InProgressJob);
                        }
                        ReleasedJobs        = CatlogJobXMLDoc.SelectNodes("//CATALOGJOB[STATUS='Released']");
                        foreach (XmlNode ReleasedJob in ReleasedJobs)
                        {
                            TenantName                  = ReleasedJob.Attributes.GetNamedItem("TenantName").Value;
                            JobHandlerObj               = new JobHandler(ImpersonationToken, TenantInfo, PluginsXMLDoc, FileFormatXMLDoc.DocumentElement.SelectSingleNode(TenantName).OuterXml, XMPFacetConfigXMLDoc);
                            JobHandlerObj.SenderName    = ReleasedJob.Attributes.GetNamedItem("SenderName").Value;
                            JobHandlerObj.SenderEmail   = ReleasedJob.Attributes.GetNamedItem("SenderEmail").Value;
                            JobHandlerObj.SenderID      = ReleasedJob.Attributes.GetNamedItem("SenderID").Value;

                            JobHandlerObj.ProcessReleasedJob(ReleasedJob);
                        }
						CatalogerHelper.CatalogerHelper.WriteTraceLogs("Job processing completed.", null, MethodInfo.GetCurrentMethod().DeclaringType.ToString() + "." + MethodInfo.GetCurrentMethod().Name);
                    }
					else
					if(!String.IsNullOrEmpty(ErrorXML))
					{
						CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ErrorXML, new StackTrace(true));
					}
					else
					CatalogerHelper.CatalogerHelper.WriteTraceLogs("No jobs found for processing.", null, MethodInfo.GetCurrentMethod().DeclaringType.ToString() + "." + MethodInfo.GetCurrentMethod().Name);
                }
            }
            catch (Exception e)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
            }
        }

        #region Private Functions
        
        private ERRORXML SetALServiceHeaderValue(ulong SessionID, ulong UserID, String TenantName, String Language)
        {
           try
            {
                RequestHeader.SessionID         = SessionID;
                RequestHeader.UserID            = UserID;
                RequestHeader.TenantName        = TenantName;
                RequestHeader.Language          = Language;
                DAMWSObj.ALServiceHeaderValue   = RequestHeader;
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

       
        #endregion

    }

    internal class JobHandler
    {
         const	int                         MAX_WT_ATTRIBUTE                = 1024;
         const	int                         MAX_HT_ATTRIBUTE                = 768;
         const	int                         MIN_WT_ATTRIBUTE                = 0;
         const  int                         MIN_HT_ATTRIBUTE                = 0;
         const String                       Language                        = "English";
         Hashtable                          TenantInfo                      = null;
         String                             ImpersonationToken              = null;
         const UInt64                       MaxRetryCount                   = 5;
         ALDAMWS                            DAMWSObj                        = null;
         ALServiceHeader                    RequestHeader                   = null;
         ENTITYID                           UserID                          = 0;
         String                             LoginName                       = null;
         UInt64                             SessionID                       = 0;
         String                             TenantName                      = null;
         XmlDocument                        ErrorXMLDoc                     = null;
         String                             CatalogJobID                    = null;
         String                             JobPath                         = null;
         String                             AssetInfoPath                   = null;
         XmlNodeList                        DigitalAssetNodes               = null;
         Hashtable                          SectionDetailsCache             = null;
         XmlDocument                        FileFormatXMLDoc                = null;
         String                             MountPoint                      = null;
         Boolean                            EnableVersoning                 = true;
         int                                ThreadTimeOut                   = 0;
         private static Object              Lock                            = new Object();
         Boolean                            CatalogerAndOMTogether          = false;
         CatalogerContext                   CatalogerContextObj             = null;
         XPathDocument                      PluginsXMLDoc                   = null;
         UserCATNotificationPreferences     UserNotificationPref            = null;
         public String                      SenderName                      = null;
         public String                      SenderEmail                     = null;
         Boolean                            IsBusinessInfoValid             = true;
         XML                                TenantCatalogingPreferencesXML  = null;
         StringDictionary                   strdict_AdditionalInfo          = null;
         public String                      SenderID                        = null;
         XPathNavigator                     XMPFacetConfigXMLDocNav         = null;
         String                             XMPSupportRequired              = null;
         public String                       CatalogerMountPointUNCPath     = null;
         String                             NetworkFolderPath               = null;
         const String                       str_CallerIdentity              = "CATALOGJOBPROCESSOR";
         bool                               Is_GenerateThb_From_Preview_Call = false;
         Int32                              NoOfProcessedAssets             = 0;
         Int32                              iTotalAssets                    = 0;
         bool                               IsProcessingComplete            = false;
         bool                               isJobSuccessed                  = false;
         String                             LoggedERRORID                   = null;
        internal enum RenditionType
        {
            ORIGINAL    = 0,
            PREVIEW     = 1,
            THUMBNAIL   = 2
        }

        enum ErrorType
        {
            ERROR       = 0,
            EXCEPTION   =1
        }

        internal class ThumbnailAttributes
        {
           public   String      ThumbnailFileName           = null;
           public   String      ThumbnailFilePath           = null;
           public   String      ThumbnailFileExtension      = null;
           public   String      ThumbnailFileFormat         = null;
           public   String      ThumbnailGenerator          = null;
           public   long        ThumbnailSize               = 0;
           public   int         ThumbnailHeight             = 0;
           public   int         ThumbnailWidth              = 0;
           public   int         ThumbnailDepth              = 0;
           public   int         ThumbnailDPI                = 0;
           public   String      IntermidiateThumFilePath    = null;
           public   int         ThumbnailQuality            = 0;
           public   String      TempThumbFilePath           = null;
           public   String      ThumbMacType                = null;
           public   String      ThumbMacCreator             = null;
           public   Boolean     ManagedGenerator            = false;
           public Boolean       IsFileCopiedToTempLocation  = true;
        }
        
        internal class PreviewAttributes
        {
           public   String      PreviewFileName             = null;
           public   String      PreviewFilePath             = null;
           public   String      PreviewFileExtension        = null;
           public   String      PreviewFileFormat           = null;
           public   String      PreviewGenerator            = null;
           public   long        PreviewSize                 = 0;
           public   int         PreviewHeight               = 0;
           public   int         PreviewWidth                = 0;
           public   int         PreviewDepth                = 0;
           public   int         PreviewDPI                  = 0;
           public   String      IntermidiatePrvFilePath     = null;
           public   String      TempPrvFilePath             = null;
           public   String      PrvMacType                  = null;
           public   String      PrvMacCreator               = null;
           public   Boolean     ManagedGenerator            = false;
           public   Boolean     IsFileCopiedToTempLocation  = true;
        }

        internal class OriginalAttributes
        {
           public   String      OriginalFileName            = null;
           public   String      OriginalFilePath            = null;
           public   String      OriginalFileExtension       = null;
           public   String      OriginalFileFormat          = null;
           public   long        OriginalSize                = 0;
           public   int         OriginalHeight              = 0;
           public   int         OriginalWidth               = 0;
           public   int         OriginalDepth               = 0;
           public   int         OriginalDPI                 = 0;
           public   String      IntermidiateOrgFilePath     = null;
           public   String      TempOrgFilePath             = null;
           public   String      OrgMacType                  = null;
           public   String      OrgMacCreator               = null;
           public   String      OriginalGenerator           = null;
           public   Boolean     ManagedGenerator            = false;
           public   String      MetadataProcessor           = null;
           public   Boolean     IsFileCopiedToTempLocation  = true;
        }

        internal class ThreadContext
        {
           public   XmlNode             ClonedDigitalAssetNode  = null;
           public   Boolean             UseOriginalAsPreview    = false;
           public   Boolean             PreviewExists           = false;
           public   Boolean             ThumbnailExists         = false;
           public   Boolean             IsBinHex                = false;
           public   Boolean             IsThumbBinHex           = false;
           public   Boolean             IsPrvBinHex             = false;
           public   Boolean             GenThumbFromPreview     = false;
           public   String              ThbFromOrgGenerator     = null;
           public   XmlNode             FormatDetails           = null;
           public   PreviewAttributes   PreviewAttrObj          = null;
           public   ThumbnailAttributes ThumbnailAttrObj        = null;
           public   OriginalAttributes  OriginalAttrObj         = null;
           public   ENTITYID            DigitalAssetID          = 0;
           public   String              ContentRevisionID       = null;
           public   StringBuilder       ErrorMessage            = null;
           public   ENTITYID            SectionID               = 0;
           public   Boolean             OnMediaServer           = false;
           public   XmlNode             JobNode                 = null;
           public   String              TempFolderPath          = null;
           public   Boolean             IsSuccess               = true;
           public   String              ParentDigitalAssetName  = null;
           public   String              str_SectionPath         = null;
           public   String              MediaProcessor          = null;
           public ThreadContext()
           {
               PreviewAttrObj   = new PreviewAttributes();
               ThumbnailAttrObj = new ThumbnailAttributes();
               OriginalAttrObj  = new OriginalAttributes();
               ErrorMessage     = new StringBuilder();
           }
        }

        /// <summary>
        /// Internal class to Represent section Details.
        /// </summary>
        internal class SectionDetails
        {
            public UInt64   SectionID       = 0;
            public Boolean  OnMediaServer   = false;

            public SectionDetails(UInt64 SectionID, Boolean OnMediaServer)
            {
                this.SectionID      = SectionID;
                this.OnMediaServer  = OnMediaServer;
            }
        }

        public JobHandler(String ImpersonationToken, Hashtable TenantInfo, XPathDocument PluginsXMLDoc, XML FileFormatXML, XPathDocument XMPFacetConfigXMLDoc)
        {
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
			this.TenantInfo                 = TenantInfo;
            this.PluginsXMLDoc              = PluginsXMLDoc;
            this.ImpersonationToken         = ImpersonationToken;
            if (XMPFacetConfigXMLDoc != null)
                this.XMPFacetConfigXMLDocNav = XMPFacetConfigXMLDoc.CreateNavigator();
            RequestHeader                   = new ALServiceHeader();
            SectionDetailsCache             = new Hashtable();
            FileFormatXMLDoc                = new XmlDocument();
            FileFormatXMLDoc.LoadXml(FileFormatXML);
            DAMWSObj                        = new ALDAMWS();
            DAMWSObj.ALServiceHeaderValue   = RequestHeader;
            DAMWSObj.Timeout                = GetWebServiceTimeout();
            DAMWSObj.Url                    = ConfigurationManager.AppSettings["DAMWSURL"];
            CatalogerContextObj             = new CatalogerContext();
            UserNotificationPref            = new UserCATNotificationPreferences();
            strdict_AdditionalInfo          = new StringDictionary();

        }

		void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
			String Message = "Is terminating: " + e.IsTerminating;
			Message += Environment.NewLine + "Stack Trace: " + e.ExceptionObject.ToString();
			CatalogerHelper.CatalogerHelper.WriteLogToNTFS(Message, new StackTrace(true));

            //Explicitly Restart the Cataloger Service  
            Message = "Restarting Cataloger Service... ";
            CatalogerHelper.CatalogerHelper.WriteLogToNTFS(Message, null);
            CatalogerHelper.CatalogerHelper.Restart("SAS MOM Cataloger Service");
		}

		internal void ProcessInprogressJob(XmlNode InProgressJob)
        {
            UInt64              RetryCount          = 0;
            XmlNode             RetryCountNode      = null;
            StringBuilder       ErrorMessage        = null;
            StringBuilder       AssetInfoFilePath   = null; 
            String              Seprator            = null;
            String              JobName             = null;
            Boolean             Success             = false;
            StringCollection    TenantSettings      = null;
            StringBuilder       sb_AssetFileList    = null;
            XmlNode             ProcessedAssets     = null;
                
            try
            {
                if (InProgressJob.SelectSingleNode("RETRYCOUNT") != null)
                {
                    CatalogJobID    = InProgressJob.SelectSingleNode("CATALOGJOBID").InnerText;
                    TenantName      = InProgressJob.Attributes.GetNamedItem("TenantName").Value;

                    RetryCountNode  = InProgressJob.SelectSingleNode("RETRYCOUNT");
                    RetryCount      = Convert.ToUInt64(RetryCountNode.InnerText);

                    ProcessedAssets = InProgressJob.SelectSingleNode("NOOFPROCESSEDASSETS");
                    if (ProcessedAssets != null && !String.IsNullOrEmpty(ProcessedAssets.InnerText))
                        Int32.TryParse(ProcessedAssets.InnerText, out NoOfProcessedAssets);


                    if (RetryCount <= MaxRetryCount)
                    {
                       Success      = ProcessJob(InProgressJob);
                    }
                    else
                    {
                        AssetInfoFilePath       = new StringBuilder();
                        ErrorMessage            = new StringBuilder();
                        TenantSettings          = new StringCollection();
                        sb_AssetFileList        = new StringBuilder();
                       
                        if ((InProgressJob.SelectSingleNode("CATALOGJOB_CREATOR/USER/LOGINNAME") != null) )
                        {
                            LoginName       = InProgressJob.SelectSingleNode("CATALOGJOB_CREATOR/USER/LOGINNAME").InnerText;
                            JobName         = InProgressJob.SelectSingleNode("JOBNAME").InnerText;
                        }
                        TenantSettings      = (StringCollection)TenantInfo[TenantName];
                        MountPoint          = TenantSettings[0];

                        if (!MountPoint.StartsWith("\\"))
                        {
                            StringBuilder sb_CatalogerMountPointUNCPath = new StringBuilder(100);
                            sb_CatalogerMountPointUNCPath.AppendFormat("\\\\{0}\\{1}", GetCatlogerMachineIPAddress(), Path.GetFileName(MountPoint));
                            CatalogerMountPointUNCPath = sb_CatalogerMountPointUNCPath.ToString();
                        }
                        else
                        {
                            CatalogerMountPointUNCPath = MountPoint;
                        }
                        //Note: Defensive handling is done, in case there are job in "In-progress" status in previous system for which JOBPATH 
                        //is not populated. Cataloging will be done in normal manner in this case.
                        if (InProgressJob.SelectSingleNode("JOBPATH") != null && !String.IsNullOrEmpty(InProgressJob.SelectSingleNode("JOBPATH").InnerText))                        
                            AssetInfoFilePath.AppendFormat("{0}\\AssetInfo", InProgressJob.SelectSingleNode("JOBPATH").InnerText);
                        else
                            AssetInfoFilePath.AppendFormat("{0}\\{1}\\{2}\\AssetInfo", MountPoint, LoginName, JobName);

                        if (File.Exists(AssetInfoFilePath.ToString()))
                        {
                            CatalogerContextObj.JobInformationObj.AssetInfoDocument = new XmlDocument();
                            CatalogerContextObj.JobInformationObj.AssetInfoDocument.Load(AssetInfoFilePath.ToString());
                            DigitalAssetNodes                                       = CatalogerContextObj.JobInformationObj.AssetInfoDocument.SelectNodes("//DIGITALASSET[not(@Cataloged)]");
                            ErrorMessage.Append("Max retry count reached for the job,marking the job as Cancelled. Following files did not get Cataloged : ");

                            foreach (XmlNode DigitalAssetNode in DigitalAssetNodes)
                            {
                                ErrorMessage.AppendFormat("{0}{1}", Seprator, DigitalAssetNode.SelectSingleNode("NAME").InnerText);
                                sb_AssetFileList.AppendFormat("{0}{1}", Seprator, DigitalAssetNode.SelectSingleNode("NAME").InnerText);
                                if (Seprator==null)
                                    Seprator = ",";
                            }
                            strdict_AdditionalInfo.Add("DIGITALASSETFILENAMELIST", sb_AssetFileList.ToString());
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_MAXRETRYCOUNTREACHED", ErrorMessage.ToString(), null, strdict_AdditionalInfo,ErrorType.ERROR);
                        }

                        // Update job status to Cancelled
                        InProgressJob.SelectSingleNode("STATUS").InnerText = "Cancelled";
                        if (!UpdateCatalogJob(InProgressJob))
                        {
                            ErrorMessage.Remove(0, ErrorMessage.Length);
                            ErrorMessage.Append("Marking job status as Cancelled failed.");
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_MARKINGJOBSTATUSASCANCELEDFAILED", ErrorMessage.ToString(), null, null, ErrorType.ERROR);
                        }
                    }                
                }
 
            }
            catch (Exception e)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
            }
           
        }

        internal void ProcessReleasedJob(XmlNode ReleasedJob)
        {
            Boolean     Success     = false;
            try
            {
				CatalogJobID    = ReleasedJob.SelectSingleNode("CATALOGJOBID").InnerText;
                TenantName      = ReleasedJob.Attributes.GetNamedItem("TenantName").Value;
                Success         = ProcessJob(ReleasedJob);
                
                //Meenal: Will be done in Update Asset Process                    
                /*
                // Code to clean up the session
                Infrastructure.SignOut(UserID, SessionID, Language, TenantName);
                */
            }
            catch (Exception e)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
            }
        }

        private Boolean Connect(String UserLoginName)
        {
            ERRORXML        ErrorXML        = null;
            XML             UserXML         = null;
            XmlDocument     UserXMLDoc      = null; 
            StringBuilder   ErrorMessage    = null;

            try
            {
                ErrorMessage            = new StringBuilder();
                ErrorXML                = Infrastructure.ImpersonateUser(UserLoginName, TenantName, ImpersonationToken, str_CallerIdentity, ref SessionID, ref UserXML);
                if(ErrorXML == null)
                {
                    UserXMLDoc  = new XmlDocument();
                    UserXMLDoc.LoadXml(UserXML);
                    if (UserXMLDoc.SelectSingleNode("//USERID") != null)
                    {
                        UserID  = Convert.ToUInt64(UserXMLDoc.SelectSingleNode("//USERID").InnerText);
                        SetALServiceHeaderValue(SessionID, UserID, TenantName, Language);
                    }
                    else
                    {
                        strdict_AdditionalInfo.Add("USERLOGINNAME", UserLoginName);
                        ErrorMessage.AppendFormat("Failed to connect as : {0}", UserLoginName);
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FAILEDTOCONNECT", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                        return false;
                    }
                }
                else
                {
                    strdict_AdditionalInfo.Add("USERLOGINNAME", UserLoginName);
                    ErrorMessage.AppendFormat("Failed to connect as : {0}", UserLoginName);
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FAILEDTOCONNECT",ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                    return false;
                }
                return true;
            }
            catch (Exception e)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean ProcessJob(XmlNode JobNode)
        {
            ERRORXML        ErrorXML            = null;
            UInt64          RetryCount          = 0;
            XmlNode         RetryCountNode      = null;
            StringBuilder   ErrorMessage        = null;
            String          TypeOfJob           = null;
            Boolean         IsSuccess           = false;
            String          JobName             = null;
            String          Subject             = null;
            String          Message             = null;
            String          JobClenupRequired   = null;
            
            try
            {               

                ErrorMessage                    = new StringBuilder();
                JobName                         = JobNode.SelectSingleNode("JOBNAME").InnerText;
                IsProcessingComplete            = false;	

                if (JobNode.SelectSingleNode("RETRYCOUNT") != null)
                {
                    RetryCountNode              = JobNode.SelectSingleNode("RETRYCOUNT");
                    RetryCount                  = Convert.ToUInt64(RetryCountNode.InnerText);
                    RetryCount++;
                    RetryCountNode.InnerText    = RetryCount.ToString();
                }
                else
                {
                    RetryCountNode              = JobNode.OwnerDocument.CreateElement("RETRYCOUNT");
                    RetryCountNode.InnerText    = "0";
                    JobNode.AppendChild(RetryCountNode);
                }
                
                JobNode.SelectSingleNode("STATUS").InnerText = "In-Progress";
                if ((JobNode.SelectSingleNode("CATALOGJOB_CREATOR/USER/LOGINNAME") != null) && (JobNode.SelectSingleNode("CATALOGJOB_CREATOR/USER/USERID") != null))
                {
                    UserID      = Convert.ToUInt64(JobNode.SelectSingleNode("CATALOGJOB_CREATOR/USER/USERID").InnerText);
                    LoginName   = JobNode.SelectSingleNode("CATALOGJOB_CREATOR/USER/LOGINNAME").InnerText;
                    JobNode.RemoveChild(JobNode.SelectSingleNode("CATALOGJOB_CREATOR"));
                }
                else
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Job creator not found");
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_JOBCREATORNOTFOUND", ErrorMessage.ToString(), null,null,ErrorType.ERROR);
                }

                // Update RetryCount and Status
                if (!(IsSuccess = UpdateCatalogJob(JobNode)))
                    return false;

                CatalogerContextObj.SessionInformationObj.LoginName = LoginName;
                CatalogerContextObj.SessionInformationObj.SessionID = SessionID;
                CatalogerContextObj.SessionInformationObj.UserID    = UserID;
                CatalogerContextObj.SessionInformationObj.TenantName = TenantName;

                CatalogerContextObj.JobInformationObj.JobNode       = JobNode;

                InvokePlugin("JOBPROCESSINGSTART",CatalogerContextObj);
                // code to send Job Change notification(Invoke extension)
                
                if(CatalogerContextObj.JobInformationObj.JobNode.SelectSingleNode("TYPEOFJOB")!=null)
                   TypeOfJob = CatalogerContextObj.JobInformationObj.JobNode.SelectSingleNode("TYPEOFJOB").InnerText;
                else
                   TypeOfJob = "CATALOGING";

                switch(TypeOfJob.ToUpper())
                {

                   default     :  if ((IsSuccess = ProcessAssetInfoFile(CatalogerContextObj.JobInformationObj.JobNode)))
                                    {
                                        InvokePlugin("BEFOREASSETLISTPROCESSING",CatalogerContextObj);
                                        IsSuccess = ProcessDigitalAssetList(CatalogerContextObj.JobInformationObj.JobNode);
                                    }
                                    break;
                }

                return true;
            }
            catch (Exception ExceptionObj)
            {
                IsSuccess = false;
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in ProcessJob : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PROCESSJOBFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);

                IsProcessingComplete    = true;
                isJobSuccessed          = false;
                return false;
            }
            finally
            {
                /* Marking the Job status as Done will be done in "Update Asset Process" service as it is updating Digital Asset with Data and Preview/Thumbnail*/
                #region Stamp Notification in case of Success/Error as per User Preferences
                try
				{                    
				    if (TypeOfJob.ToUpper()!="COPY")
                    {
                        Subject = "";
                        Message = "";
                        bool ErrorFlag = false;
                        if (IsProcessingComplete)
                        {
                            if (isJobSuccessed)
                            {
                                InvokePlugin("JOBPROCESSINGEND", CatalogerContextObj);
                                try
                                {
                                    //Incase of shared network folder job check whether job 
                                    //cleanup is specified.
                                    if (!String.IsNullOrEmpty(NetworkFolderPath) &&
                                        JobPath.StartsWith(NetworkFolderPath,
                                        StringComparison.CurrentCultureIgnoreCase))
                                    {
                                        JobClenupRequired = ConfigurationManager.AppSettings["SharedNetworkFolder_Cleanup_Required"];
                                        if ((!String.IsNullOrEmpty(JobClenupRequired)) && (JobClenupRequired.ToUpper() == "YES"))
                                            NTFSJobCleanUp();
                                    }
                                    else
                                    {
                                        NTFSJobCleanUp();
                                    }

                                }
                                catch (Exception ExceptionObj)
                                {
                                    IsSuccess = false;
                                    strdict_AdditionalInfo.Clear();
                                    strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                                    ErrorMessage.Remove(0, ErrorMessage.Length);
                                    ErrorMessage.AppendFormat("Exception thrown during Job clean up. Message : {0}", ExceptionObj.Message);
                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_JOBCLEANUPFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);                                    
                                }
                            }
                            else if (!isJobSuccessed)
                            {
                                ErrorFlag = true;
                                InvokePlugin("JOBPROCESSINGEND", CatalogerContextObj);
                            }

                            if (((UserNotificationPref.SendMailSettings.ToString().CompareTo("ALWAYS") == 0) || (UserNotificationPref.SendMailSettings.ToString().CompareTo("ONERROR") == 0 && ErrorFlag)) 
                                && !string.IsNullOrEmpty(UserNotificationPref.SendMailTo))
                            {
                                SendNotification NotificationObj    = new SendNotification();
                                NotificationObj.SenderName          = SenderName;
                                NotificationObj.SenderEmail         = SenderEmail;
                                NotificationObj.TenantName          = TenantName;
                                Subject                             = JobName;
                                Message                             = JobName;
                                ErrorXML                            = NotificationObj.StampNotification(SenderID, Subject, Message, UserNotificationPref.SendMailTo, JobName, ErrorFlag);
                                if (!String.IsNullOrEmpty(ErrorXML))
                                {
                                    ErrorMessage.Append("Error in stamping notification");
                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_STAMPNOTIFICATIONFAILED", ErrorMessage.ToString(), ErrorXML, null, ErrorType.ERROR);
                                }
                            }
                        }                       
                    }
				}
				catch(Exception exc)
				{
					CatalogerHelper.CatalogerHelper.WriteLogToNTFS(exc.Message, new StackTrace(true));
                }
                #endregion
            }
        }

        private void NTFSJobCleanUp()
        {   
            StringBuilder sb_AdditionalInfoXml = new StringBuilder(200);
            sb_AdditionalInfoXml.AppendFormat(@"<ADDITIONALINFO UserID='{1}' SessionID='{2}' Language='{3}' TenantName='{4}'>
                                                    <THREADID>{0}</THREADID>
                                                    <METHODNAME>Cataloger.CatalogJobProcessor.NTFSJobCleanUp()</METHODNAME>    
                                                    <CATALOGJOBID>{5}</CATALOGJOBID>                                           
                                                    <JOBPATH>{6}</JOBPATH>    
                                              </ADDITIONALINFO>",
                                             System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), UserID, SessionID, Language, TenantName,
                                             CatalogJobID, ALCommon.TrimAndReplaceSpecialChars(JobPath));

            Infrastructure.DeleteFolder(JobPath, true, UserID, SessionID, Language, TenantName, sb_AdditionalInfoXml.ToString());
        }

        private Boolean UpdateCatalogJob(XmlNode JobNode)
        {
            ERRORXML    ErrorXML    = null;
            Boolean     IsValid     = false;
            
            try
            {
                if (SessionID == 0)
                {
                    if (!Connect(LoginName))
                        return false;
                }
                else
                {
                    ErrorXML    = Infrastructure.IsSessionValid(UserID, SessionID, Language, TenantName, ref IsValid);
                    if(!IsValid)
                        if (!Connect(LoginName))
                            return false;
                }
                ErrorXML   = DAMWSObj.UpdateCatalogJob(JobNode.OuterXml);

                if (ErrorXML != null)
                {
                    return false;
                }
                return true;
            }
            catch (Exception e)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
				return false;
            }
        }

       private ERRORXML LogError(String ErrorCode, String ErrorMessage,String InnerError, StringDictionary strdict_AdditionalInfo,  ErrorType ErrorTypeValue)
        {
            ERRORXML        ErrorXML                        = null;
            XmlNode         JobErrorNode                    = null;
            XmlNode         ColoumnNode                     = null;
            XmlNode         ErrorDetailsNode                = null;
            XML             LogErrorXML                     = null;
            XmlNode         MessageNode                     = null;
            XmlNode         ErrorNode                       = null;
            XmlDocument     InnerErrorXMLDoc                = null;
            XmlNode         ErrorCollectionNode             = null;
            XmlNode         xmlnod_AdditionalInfo           = null;
            XmlNode         xmlnod_AdditionalInfoField      = null;
            XmlNode         xmlnod_ErrorCode                = null;

            LoggedERRORID = null;

           try
			{
		   if (InnerError != null)
            {
                InnerErrorXMLDoc                    = new XmlDocument();
                InnerErrorXMLDoc.LoadXml(InnerError);
                if (InnerErrorXMLDoc.SelectSingleNode("ERRORCOLLECTION") == null)
                {
                    ErrorCollectionNode             = InnerErrorXMLDoc.CreateElement("ERRORCOLLECTION");
                    ErrorCollectionNode.InnerXml    = InnerError;
                    InnerError                      = ErrorCollectionNode.OuterXml;
                }
            }
            else
            {
                InnerError = String.Empty;
            }

            if (ErrorXMLDoc == null)
            {
                ErrorXMLDoc                 = new XmlDocument();
                JobErrorNode                = ErrorXMLDoc.CreateElement("JOBERROR");
                
                ColoumnNode                 = ErrorXMLDoc.CreateElement("APPLICATIONNAME");
                ColoumnNode.InnerText       = "Cataloger";
                JobErrorNode.AppendChild(ColoumnNode);

                ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORTYPE");
                if(ErrorTypeValue==ErrorType.ERROR)
                    ColoumnNode.InnerText   = "ERROR";
                else
                    ColoumnNode.InnerText   = "EXCEPTION";
                JobErrorNode.AppendChild(ColoumnNode);

                ColoumnNode                 = ErrorXMLDoc.CreateElement("ERRORMESSAGE");
                ErrorNode                   = ErrorXMLDoc.CreateElement("ERROR");
                MessageNode                 = ErrorXMLDoc.CreateElement("MESSAGE");
                xmlnod_ErrorCode            = ErrorXMLDoc.CreateElement("ERRORCODE");

                if (!String.IsNullOrEmpty(ErrorCode))
                {
                    xmlnod_ErrorCode.InnerText = ErrorCode;
                }
                ErrorNode.AppendChild(xmlnod_ErrorCode);

                MessageNode.InnerText       = ErrorMessage;
                ErrorNode.AppendChild(MessageNode);

                if (InnerError != null)
                {
                    ErrorDetailsNode            = ErrorXMLDoc.CreateElement("ERRORDETAILS");
                    ErrorDetailsNode.InnerXml   = InnerError;
                    ErrorNode.AppendChild(ErrorDetailsNode);
                }

                if (strdict_AdditionalInfo != null)
                {
                    xmlnod_AdditionalInfo = ErrorXMLDoc.CreateElement("ADDITIONALINFO");
                    IEnumerator ienum_AdditionalInfo = strdict_AdditionalInfo.GetEnumerator();

                    foreach (DictionaryEntry obj_DictionaryEntry in strdict_AdditionalInfo)
                    {

                        xmlnod_AdditionalInfoField = ErrorXMLDoc.CreateElement(obj_DictionaryEntry.Key.ToString().ToUpper());
                        if (obj_DictionaryEntry.Value != null)
                        {
                            xmlnod_AdditionalInfoField.InnerXml = ALCommon.TrimAndReplaceSpecialChars(obj_DictionaryEntry.Value.ToString());
                        }
                        xmlnod_AdditionalInfo.AppendChild(xmlnod_AdditionalInfoField);
                    }

                    ErrorNode.AppendChild(xmlnod_AdditionalInfo);
                }

                ColoumnNode.AppendChild(ErrorNode);
                JobErrorNode.AppendChild(ColoumnNode);

                ColoumnNode                 = ErrorXMLDoc.CreateElement("CATALOGJOB_JOBERROR");
                ((XmlElement)ColoumnNode).SetAttribute("TYPE", "RELATION");
                ((XmlElement)ColoumnNode).SetAttribute("PARENTID", "TRUE");
                ColoumnNode.InnerText       = CatalogJobID;
                JobErrorNode.AppendChild(ColoumnNode);

                ErrorXMLDoc.AppendChild(JobErrorNode);
            }
            else
            {
                if (ErrorTypeValue == ErrorType.ERROR)
                    ErrorXMLDoc.SelectSingleNode("//ERRORTYPE").InnerText           = "ERROR";
                else
                    ErrorXMLDoc.SelectSingleNode("//ERRORTYPE").InnerText           = "EXCEPTION";

                if (strdict_AdditionalInfo != null)
                {
                    xmlnod_AdditionalInfo = ErrorXMLDoc.SelectSingleNode("//ADDITIONALINFO");
                    if (xmlnod_AdditionalInfo == null)
                    {
                        ErrorNode = ErrorXMLDoc.SelectSingleNode("//ERROR");
                        xmlnod_AdditionalInfo = ErrorXMLDoc.CreateElement("ADDITIONALINFO");
                        ErrorNode.AppendChild(xmlnod_AdditionalInfo);
                    }
                    else
                    {
                        xmlnod_AdditionalInfo.RemoveAll();
                    }

                    IEnumerator ienum_AdditionalInfo = strdict_AdditionalInfo.GetEnumerator();
                    foreach (DictionaryEntry obj_DictionaryEntry in strdict_AdditionalInfo)
                    {

                        xmlnod_AdditionalInfoField = ErrorXMLDoc.CreateElement(obj_DictionaryEntry.Key.ToString().ToUpper());
                        if (obj_DictionaryEntry.Value != null)
                        {
                            xmlnod_AdditionalInfoField.InnerXml = obj_DictionaryEntry.Value.ToString();
                        }
                        xmlnod_AdditionalInfo.AppendChild(xmlnod_AdditionalInfoField);
                    }
                }
                else
                {
                    xmlnod_AdditionalInfo = ErrorXMLDoc.SelectSingleNode("//ADDITIONALINFO");
                    if (xmlnod_AdditionalInfo != null)
                    {
                        ErrorNode = ErrorXMLDoc.SelectSingleNode("//ERROR");
                        ErrorNode.RemoveChild(xmlnod_AdditionalInfo);
                    }
                }

                ErrorXMLDoc.SelectSingleNode("//ERRORCODE").InnerText               = ErrorCode;
                ErrorXMLDoc.SelectSingleNode("//MESSAGE").InnerText                 = ErrorMessage;
                ErrorXMLDoc.SelectSingleNode("//ERRORDETAILS").InnerXml             = InnerError;
                ErrorXMLDoc.SelectSingleNode("//CATALOGJOB_JOBERROR").InnerText     = CatalogJobID; 
            }

            LogErrorXML = ErrorXMLDoc.InnerXml;
            ErrorXML    = DAMWSObj.LogError(ref LogErrorXML);
            if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(LogErrorXML))
            {
                LoggedERRORID = ALCommon.GetInnerValue(ref LogErrorXML, "<ERRORID>", "</ERRORID>");
            }
            return ErrorXML;
			}
		   catch(Exception e)
			{
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(e.Message, new StackTrace(true));
				return e.Message;
			}
        }

        private ERRORXML SetALServiceHeaderValue(ulong SessionID, ulong UserID, String TenantName, String Language)
        {
           try
            {
                RequestHeader.SessionID         = SessionID;
                RequestHeader.UserID            = UserID;
                RequestHeader.TenantName        = TenantName;
                RequestHeader.Language          = Language;
                return null;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				throw ExceptionObj;
            }
        }

        private Boolean ProcessAssetInfoFile(XmlNode JobNode)
        {
            StringBuilder       PathString          = null;
            int                 DigitalAssetCount   = 0;
            DirectoryInfo       DirectoryInfoObj    = null;
            StringBuilder       OriginalFolderPath  = null;
            StringBuilder       ErrorMessage        = null;
            StringCollection    TenantSettings      = null;
            XmlNode             AssetInfoNode       = null;
              
            try
            {
                PathString              = new StringBuilder();
                OriginalFolderPath      = new StringBuilder();
                ErrorMessage            = new StringBuilder();
                TenantSettings          = new StringCollection();
                
                TenantSettings                  = (StringCollection)TenantInfo[TenantName];
                MountPoint                      = TenantSettings[0];
                TenantCatalogingPreferencesXML  = TenantSettings[4];
                XMPSupportRequired              = TenantSettings[5];
                NetworkFolderPath               = TenantSettings[6];

                if (TenantSettings[1].ToUpper() == "TRUE")
                    EnableVersoning = true;
                else
                    EnableVersoning = false;

                ThreadTimeOut       = Convert.ToInt32(TenantSettings[2]);
                if ((TenantSettings[3] != null) && (TenantSettings[3] == "1"))
                {
                    CatalogerAndOMTogether = true;
                }

                if (!MountPoint.StartsWith("\\"))
                {
                    StringBuilder sb_CatalogerMountPointUNCPath = new StringBuilder(100);
                    sb_CatalogerMountPointUNCPath.AppendFormat("\\\\{0}\\{1}", GetCatlogerMachineIPAddress(), Path.GetFileName(MountPoint));
                    CatalogerMountPointUNCPath = sb_CatalogerMountPointUNCPath.ToString();
                }
                else
                {
                    CatalogerMountPointUNCPath = MountPoint;
                }
                //Note: Defensive handling is done, in case there are job in "In-progress" status in previous system for which JOBPATH 
                //is not populated. Cataloging will be done in normal manner in this case.
                if (JobNode.SelectSingleNode("JOBPATH") != null && !String.IsNullOrEmpty(JobNode.SelectSingleNode("JOBPATH").InnerText))
                    JobPath = JobNode.SelectSingleNode("JOBPATH").InnerText;
                else
                {
                    PathString.AppendFormat("{0}\\{1}\\{2}", MountPoint, LoginName, JobNode.SelectSingleNode("JOBNAME").InnerText);
                    JobPath             = PathString.ToString();
                }

                OriginalFolderPath.AppendFormat("{0}\\Originals", JobPath);
                PathString.Remove(0,PathString.Length);
                
                PathString.AppendFormat("{0}\\AssetInfo", JobPath);
                AssetInfoPath       = PathString.ToString();
                PathString.Remove(0,PathString.Length);

                CatalogerContextObj.JobInformationObj.AssetInfoDocument     = new XmlDocument();
                CatalogerContextObj.JobInformationObj.AssetInfoDocument.Load(AssetInfoPath);

                CatalogerContextObj.JobInformationObj.JobPath               =   JobPath;

                //Invoke Plugin here Before AssetInfo Processing  code point. Used for Tag Extraction plugin. 
                InvokePlugin("BEFOREASSETINFOPROCESSING", CatalogerContextObj);

                AssetInfoNode                                               = CatalogerContextObj.JobInformationObj.AssetInfoDocument.DocumentElement;
                if (AssetInfoNode.Attributes.GetNamedItem("EnableVersioning")!= null)
                {
                    if (AssetInfoNode.Attributes.GetNamedItem("EnableVersioning").Value.ToUpper() == "FALSE")
                        EnableVersoning = false;
                    else
                        EnableVersoning = true;

                }
                DigitalAssetCount   = Convert.ToInt32(AssetInfoNode.SelectSingleNode("DIGITALASSETCOUNT").InnerText);

                if (Directory.Exists(OriginalFolderPath.ToString()))
                {
                    DirectoryInfoObj = new DirectoryInfo(OriginalFolderPath.ToString());
                    if (DigitalAssetCount > DirectoryInfoObj.GetFiles().LongLength)
                    {
                        ErrorMessage.Append("File count in originals folder does not match Assetinfo file count, job processing Aborted.");
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORGFILECOUNTNOTMATCHWITHASSETINFO", ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                        return false;
                    }
                }
                else
                {
                        ErrorMessage.Append("Originals folder not found, job processing Aborted.");
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORGFOLDERNOTFOUND", ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                        return false;
                }

                // Create Preview and Thumbnail folders
                PathString.Remove(0, PathString.Length);
                PathString.AppendFormat("{0}\\Previews", JobPath);
                if (!Directory.Exists(PathString.ToString()))
                    Directory.CreateDirectory(PathString.ToString());

                PathString.Remove(0, PathString.Length);
                PathString.AppendFormat("{0}\\Thumbnails", JobPath);
                if (!Directory.Exists(PathString.ToString()))
                    Directory.CreateDirectory(PathString.ToString());

                DigitalAssetNodes = CatalogerContextObj.JobInformationObj.AssetInfoDocument.SelectNodes("//DIGITALASSET");
                if (DigitalAssetCount == DigitalAssetNodes.Count)
                {
                    foreach (XmlNode DigitalAssetNode in DigitalAssetNodes) 
                    {
                        PathString.Remove(0,PathString.Length);
                        PathString.AppendFormat("{0}\\{1}",OriginalFolderPath,DigitalAssetNode.SelectSingleNode("NAME").InnerText);
                        if (!File.Exists(PathString.ToString()))
                        {
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("DIGITALASSETNAME", DigitalAssetNode.SelectSingleNode("NAME").InnerText);
                            ErrorMessage.AppendFormat("The file name {0} provided in AssetInfo file is not in the Originals folder, job processing Aborted.", DigitalAssetNode.SelectSingleNode("NAME").InnerText);
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORGFILENOTPRESENT",ErrorMessage.ToString(),null,strdict_AdditionalInfo, ErrorType.ERROR);
                            return false;
                        }
                    }
                }
                else
                {
                    ErrorMessage.Append("AssetInfo filecount and the number of DigitalAsset node does not match, job processing Aborted.");
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FILECOUNTNOTMATCHWITHASSETINFO",ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                    return false;
                }
                // So that only DigitalAsset that are not cataloged gets cataloged during retry.
                DigitalAssetNodes = CatalogerContextObj.JobInformationObj.AssetInfoDocument.DocumentElement.SelectNodes("SECTION/DIGITALASSET[not(@Cataloged)]");
                //DigitalAssetNodes = CatalogerContextObj.JobInformationObj.AssetInfoDocument.DocumentElement.SelectNodes("SECTION/DIGITALASSET[not(@Cataloged) and not(@GetOrgInfoRequestSent) and not(@GeneratePrvRequestSent) and not(@GenerateThbRequestSent)]");
                
                InvokePlugin("AFTERASSETINFOPROCESSING", CatalogerContextObj);
                return true;

            }
            catch (Exception ExceptionObj) 
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Exception thrown in ProcessAssetInfoFile : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PROCESSASSETINFOFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }

        private Boolean ProcessDigitalAssetList(XmlNode JobNode)
        {
            ERRORXML            ErrorXML                    = null;
            XmlDocument         RetErrorXMLDoc              = null;
            String              SectionPath                 = null;
            ENTITYID            SectionID                   = 0; 
            ThreadContext       ThreadContextObj;
            StringBuilder       ErrorMessage                = null;
            StringCollection    AccessDeniedSections        = null;
            Boolean             IsSuccess                   = true;
            Boolean             RequestedAccessRightGranted = false;
            StringCollection    SectionNameCollection       = null;
            StringBuilder       SectionPathWithoutSpace     = null;
            String[]            SplitOn                     = {"\\"};
            Boolean             OnMediaServer               = false;

            try
            {
                ErrorMessage                            = new StringBuilder();
                AccessDeniedSections                    = new StringCollection();
                
                if(GetUserPreferences( CatalogerContextObj))
                {
                    foreach (XmlNode DigitalAssetNode in DigitalAssetNodes)
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        // Send extention notification and also handle ship functionality.

                        SectionPath         = DigitalAssetNode.ParentNode.Attributes.GetNamedItem("Path").Value;                       

                        #region Remove invalid spaces from path
                        SectionPath                 = SectionPath.Trim();
                        if (SectionPath.Contains(" \\") || SectionPath.Contains("\\ "))
                        {
                            SectionNameCollection   = new StringCollection();
                            SectionPathWithoutSpace = new StringBuilder();
                            SectionNameCollection.AddRange(SectionPath.Split(SplitOn, StringSplitOptions.RemoveEmptyEntries));
                            for (int Count = 0; Count < SectionNameCollection.Count; Count++)
                            {
                                if (Count == 0)
                                    SectionPathWithoutSpace.AppendFormat("{0}", SectionNameCollection[Count].Trim());
                                else
                                    SectionPathWithoutSpace.AppendFormat("\\{0}", SectionNameCollection[Count].Trim());
                            }
                            SectionPath = SectionPathWithoutSpace.ToString();
                        }
                        #endregion

                        ThreadContextObj                                    = new ThreadContext();
                        ThreadContextObj.str_SectionPath                    = SectionPath;
                        ThreadContextObj.OriginalAttrObj.OriginalFileName   = DigitalAssetNode.SelectSingleNode("NAME").InnerText;

                        #region Check Access on Section
                        if (!SectionDetailsCache.ContainsKey(SectionPath))
                        {
                            if (!AccessDeniedSections.Contains(SectionPath))
                            {
                                ErrorXML = DAMWSObj.CreateSectionsOnPathEx(SectionPath, out SectionID, out OnMediaServer);
                                if (ErrorXML == null)
                                {
                                    if (SectionID != 0)
                                    {
                                        ErrorXML = DAMWSObj.IsAccessAllowed(SectionID, "DW_CREATE_PERMISSION", out RequestedAccessRightGranted);
                                        if (ErrorXML != null)
                                        {
                                            strdict_AdditionalInfo.Clear();
                                            strdict_AdditionalInfo.Add("TARGETSECTIONPATH", SectionPath);
                                            ErrorMessage.AppendFormat("IsAccessAllowed failed for TargetSection {0}", SectionPath);
                                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ISACCESSALLOWEDFAILEDFORTARGETSECTION", ErrorMessage.ToString(), ErrorXML,strdict_AdditionalInfo, ErrorType.ERROR);
                                            UpdateCatalogJobProcessingDetails(ThreadContextObj, "", false);
                                            IsSuccess = false;
                                            continue;
                                        }
                                        if (!RequestedAccessRightGranted)
                                        {
                                            if (EnableVersoning)
                                            {
                                                ErrorXML = DAMWSObj.IsAccessAllowed(SectionID, "DW_MODIFY_PERMISSION", out RequestedAccessRightGranted);
                                                if (ErrorXML != null)
                                                {
                                                    strdict_AdditionalInfo.Clear();
                                                    strdict_AdditionalInfo.Add("TARGETSECTIONPATH", SectionPath);
                                                    ErrorMessage.AppendFormat("IsAccessAllowed failed for TargetSection {0}", SectionPath);
                                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ISACCESSALLOWEDFAILEDFORTARGETSECTION", ErrorMessage.ToString(), ErrorXML,strdict_AdditionalInfo, ErrorType.ERROR);
                                                    UpdateCatalogJobProcessingDetails(ThreadContextObj, "", false);
                                                    IsSuccess = false;
                                                    continue;
                                                }
                                                if (RequestedAccessRightGranted)
                                                {
                                                    SectionDetailsCache.Add(SectionPath, new SectionDetails(SectionID, OnMediaServer));
                                                }
                                                else
                                                {
                                                    AccessDeniedSections.Add(SectionPath);
                                                }
                                            }
                                            else
                                            {
                                                AccessDeniedSections.Add(SectionPath);
                                            }
                                        }
                                        else
                                        {
                                            SectionDetailsCache.Add(SectionPath, new SectionDetails(SectionID, OnMediaServer));
                                        }
                                    }
                                }
                                else
                                {
                                    RetErrorXMLDoc = new XmlDocument();
                                    RetErrorXMLDoc.LoadXml(ErrorXML);
                                    if (RetErrorXMLDoc.SelectSingleNode("//ERRORCODE[text()='DAMENGINE_SECTION_ACCESSDENIED']") != null)
                                    {
                                        AccessDeniedSections.Add(SectionPath);
                                        strdict_AdditionalInfo.Clear();
                                        strdict_AdditionalInfo.Add("TARGETSECTIONPATH", SectionPath);
                                        ErrorMessage.AppendFormat("User does not have create item permission on section {0}", SectionPath);
                                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_CRAETEITEMACCESSDENIED", ErrorMessage.ToString(), ErrorXML,strdict_AdditionalInfo, ErrorType.ERROR);
                                        UpdateCatalogJobProcessingDetails(ThreadContextObj, "", false);
                                        IsSuccess = false;                                  
                                        continue;
                                    }
                                    else
                                    {
                                        strdict_AdditionalInfo.Clear();
                                        strdict_AdditionalInfo.Add("TARGETSECTIONPATH", SectionPath);
                                        ErrorMessage.AppendFormat("Unable To Create TargetSection {0}", SectionPath);
                                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_UNABLETOCREATETARGETSECTION", ErrorMessage.ToString(), ErrorXML,strdict_AdditionalInfo, ErrorType.ERROR);
                                        UpdateCatalogJobProcessingDetails(ThreadContextObj, "", false);
                                        IsSuccess = false;
                                        continue;
                                    }
                                }
                            }
                            else
                            {
                                continue;
                            }
                        }
                        else
                        {
                            SectionID       = ((SectionDetails)SectionDetailsCache[SectionPath]).SectionID;
                            OnMediaServer   = ((SectionDetails)SectionDetailsCache[SectionPath]).OnMediaServer;
                        }
                        #endregion

                        // This is done because changes made to DigitalAssetNode after this 
                        // Should not be reflected into AssetInfo stored on NTFS                         
                        ThreadContextObj.SectionID              = SectionID;
                        ThreadContextObj.OnMediaServer          = OnMediaServer;
                        ThreadContextObj.ClonedDigitalAssetNode = DigitalAssetNode.Clone();
                        ThreadContextObj.JobNode                = JobNode;                        
                        
                        if (!IsSuccess)
                            ThreadContextObj.IsSuccess = false;

                        CatalogerContextObj.AssetInformationObj.AssetNode           = ThreadContextObj.ClonedDigitalAssetNode;
                        CatalogerContextObj.AssetInformationObj.OriginalFileName    = ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("NAME").InnerText;
                        InvokePlugin("BEFOREASSETPROCESSING", CatalogerContextObj);

                        ProcessDigitalAsset(ThreadContextObj,CatalogerContextObj);
                        if (!ThreadContextObj.IsSuccess)
                            IsSuccess = false;
                      }
                }
                else
                {
                    IsSuccess = false;
                }

                return IsSuccess;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in ProcessDigitalAssetList : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PROCESSDIGITALASSETLISTFAILED", ErrorMessage.ToString(),null, strdict_AdditionalInfo, ErrorType.EXCEPTION);
                MarkJobAsError(JobNode);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }

        // Return type has to be void as we can implement Multi-Threading later.
        private void ProcessDigitalAsset(ThreadContext ThreadContextObj, CatalogerContext CatalogerContextObj)
        {
            StringBuilder       ErrorMessage                        = null;
            Boolean             IsSuccess                           = true;
            //FileInfo            FileInfoObj                         = null;
            XmlNodeList         ChildDigitalAssetNodes              = null;
            Boolean             LastChild                           = false;
            XmlNode             ParentDigitalAssetNode              = null;
            StringCollection    CleanUpTempFiles                    = null;
            ThreadContext[]     ThreadContextObjForEachChildAsset   = null;   
            Boolean             XMPReadSuccessfully                 = true;
            
            try
            {
                ErrorMessage            = new StringBuilder(100);
                CleanUpTempFiles        = new StringCollection();
                              
                ParentDigitalAssetNode                  = ThreadContextObj.ClonedDigitalAssetNode;
                ThreadContextObj.ParentDigitalAssetName = ParentDigitalAssetNode.SelectSingleNode("NAME").InnerText;
                ChildDigitalAssetNodes                  = ParentDigitalAssetNode.SelectNodes("PARENT_CHILDASSET/DIGITALASSET");
               
                ThreadContextObjForEachChildAsset       = new ThreadContext[ChildDigitalAssetNodes.Count + 1];
                for (int Count = 0; Count <= ChildDigitalAssetNodes.Count; Count++)
                {
                    ThreadContextObjForEachChildAsset[Count] = new ThreadContext();
                    if (Count == ChildDigitalAssetNodes.Count)
                        LastChild = true;

                    if (LastChild)
                    {
                        ThreadContextObjForEachChildAsset[Count].ClonedDigitalAssetNode = ParentDigitalAssetNode;
                    }
                    else
                    {
                        ThreadContextObjForEachChildAsset[Count].ClonedDigitalAssetNode = ChildDigitalAssetNodes[Count];
                    }

                    ThreadContextObjForEachChildAsset[Count].JobNode                = ThreadContextObj.JobNode;
                    ThreadContextObjForEachChildAsset[Count].SectionID              = ThreadContextObj.SectionID;
                    ThreadContextObjForEachChildAsset[Count].OnMediaServer          = ThreadContextObj.OnMediaServer;
                    ThreadContextObjForEachChildAsset[Count].ParentDigitalAssetName = ThreadContextObj.ParentDigitalAssetName;
                    ThreadContextObjForEachChildAsset[Count].str_SectionPath        = ThreadContextObj.str_SectionPath;                    
        
                    if (GetDigitalAssetInfo(ThreadContextObjForEachChildAsset[Count]))
                    {

                        if (FillUserPreferences(ThreadContextObjForEachChildAsset[Count], CatalogerContextObj))
                        {
                            if (LoadGenerator(ThreadContextObjForEachChildAsset[Count]))
                            {                               
                                #region Fill XMP Data
                                if (XMPSupportRequired.ToUpper() == "YES")
                                {
                                    if (!String.IsNullOrEmpty(ThreadContextObjForEachChildAsset[Count].OriginalAttrObj.MetadataProcessor))
                                    {
                                        XMPReadSuccessfully = ExtractXMP(ThreadContextObjForEachChildAsset[Count], XMPFacetConfigXMLDocNav);
                                    }
                                }
                                #endregion

                                if (!CreateDigitalAsset(ThreadContextObjForEachChildAsset[Count], LastChild))
                                {
                                    IsSuccess = false;
                                }
                                //Call to Get Original Attributes
                                if(IsSuccess)
                                    GetAttributes(ThreadContextObjForEachChildAsset[Count], RenditionType.ORIGINAL);                                
                            }
                            else
                            {                                
                                #region Fill XMP Data
                                if (XMPSupportRequired.ToUpper() == "YES")
                                {
                                    if (!String.IsNullOrEmpty(ThreadContextObjForEachChildAsset[Count].OriginalAttrObj.MetadataProcessor))
                                    {
                                        XMPReadSuccessfully = ExtractXMP(ThreadContextObjForEachChildAsset[Count], XMPFacetConfigXMLDocNav);
                                    }
                                }
                                #endregion

                                if (!CreateDigitalAsset(ThreadContextObjForEachChildAsset[Count], LastChild))
                                {
                                    IsSuccess = false;
                                }
                            }
                        }
                                              
                    }
                    else
                    {
                        IsSuccess = false;
                    }                  

                }

                if (!IsSuccess)
                {
                    ThreadContextObj.IsSuccess = false;
                    return;
                }

                // This SelectNodes is required again because changes made in CreateDigitalAsset only exists in ParentDigitalAssetNode.
                // And is not reflected in ChildAssets.
                ChildDigitalAssetNodes  = ParentDigitalAssetNode.SelectNodes("PARENT_CHILDASSET/DIGITALASSET");
                LastChild               = false;
                for (int Count = 0; Count <= ChildDigitalAssetNodes.Count; Count++)
                {
                    if (Count == ChildDigitalAssetNodes.Count)
                    {
                        LastChild = true;
                        ThreadContextObjForEachChildAsset[Count].ClonedDigitalAssetNode = ParentDigitalAssetNode;
                    }
                    else
                    {
                        ThreadContextObjForEachChildAsset[Count].ClonedDigitalAssetNode = ChildDigitalAssetNodes[Count];
                    }

                    if ((!ThreadContextObjForEachChildAsset[Count].PreviewExists) && (!ThreadContextObjForEachChildAsset[Count].UseOriginalAsPreview))
                    {   
                        GenerateAssetPreview(ThreadContextObjForEachChildAsset[Count]);
                    }
                    else if (ThreadContextObjForEachChildAsset[Count].PreviewExists)
                    {
                         //Call to Get Preview Attributes
                         GetAttributes(ThreadContextObjForEachChildAsset[Count], RenditionType.PREVIEW);  
                    }

                    if (!ThreadContextObjForEachChildAsset[Count].ThumbnailExists)
                    {                       
                        GenerateAssetThumbnail(ThreadContextObjForEachChildAsset[Count]);
                    }
                    else if (ThreadContextObjForEachChildAsset[Count].ThumbnailExists)
                    {
                        //Call to Get Preview Attributes
                        GetAttributes(ThreadContextObjForEachChildAsset[Count], RenditionType.THUMBNAIL);
                    }
                }

                if (!IsSuccess || !XMPReadSuccessfully)
                {
                    ThreadContextObj.IsSuccess = false;
                }
                              
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                ErrorMessage.AppendFormat( "Exception thrown in ProcessDigitalAsset for {0} : Message : {1}", ThreadContextObj.ParentDigitalAssetName, ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PROCESSDIGITALASSETFAILED", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
                UpdateCatalogJobProcessingDetails(ThreadContextObj, null, false);
                MarkJobAsError( ThreadContextObj.JobNode);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
            }
        }

        private Boolean GetAssetAttributes( ThreadContext ThreadContextObj)
        {
            Boolean IsSuccess = true;
            try
            {
                if (ThreadContextObj.PreviewExists)
                    if (ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator != null)
                        GetAttributes( ThreadContextObj,RenditionType.PREVIEW);

                if (ThreadContextObj.ThumbnailExists)
                    if (ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator != null)
                        GetAttributes( ThreadContextObj, RenditionType.THUMBNAIL);

                IsSuccess = GetAttributes( ThreadContextObj, RenditionType.ORIGINAL);

                return IsSuccess;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean GetAttributes( ThreadContext ThreadContextObj, RenditionType RenditionTypeValue)
        {
            InvokeMediaProcessorClass   InvokeMediaProcessorClassObj            = null;
            FileInfo                    FileInfoObj                             = null; 
            StringBuilder               TempFilePath                            = null;
            ENTITYID                    FileID                                  = 0;
            StringBuilder               ErrorMessage                            = null;
            String                      FileExtension                           = null;
            MediaProcessorWrapper       MediaProcessorWrapperObj                = null;    
            String                      str_ExtraInformationForMediaProcessor   = null;
            String                      TempPathFromJob                         = null;
            
           
            try
            {
                InvokeMediaProcessorClassObj    = new InvokeMediaProcessorClass();
                MediaProcessorWrapperObj        = new MediaProcessorWrapper(SessionID,UserID,TenantName,Language);
                TempFilePath                    = new StringBuilder();
                ErrorMessage                    = new StringBuilder();
                ALCommon.GenerateUniqueID(ref FileID);
                
                if (RenditionTypeValue == RenditionType.ORIGINAL)
                {
                    #region ORIGINAL
                    if (!String.IsNullOrEmpty(ThreadContextObj.OriginalAttrObj.OriginalGenerator))
                    {
                        if (ThreadContextObj.OriginalAttrObj.OriginalFileExtension == null)
                            FileExtension               = ThreadContextObj.OriginalAttrObj.OriginalFileName.Remove(0, ThreadContextObj.OriginalAttrObj.OriginalFileName.LastIndexOf('.') + 1);
                        else
                            FileExtension               = ThreadContextObj.OriginalAttrObj.OriginalFileExtension;

                        switch (ThreadContextObj.OriginalAttrObj.OriginalGenerator)
                        {
                            case "IllustratorCS3Wrapper.dll":
                            case "MSOfficeProcessorWrapper.dll":
                            case "CatalogerGenericQWrapper.dll":
                            case "FLVWrapper.dll":
                                {
                                    if (IsFilePathContainsSpecialCharacters(ThreadContextObj.OriginalAttrObj.OriginalFilePath))
                                    {
                                        TempFilePath.AppendFormat("{0}\\Temp", MountPoint);
                                        if (!Directory.Exists(TempFilePath.ToString()))
                                            Directory.CreateDirectory(TempFilePath.ToString());
                                        TempFilePath.AppendFormat("\\{0}.{1}", FileID, FileExtension);

                                        FileInfoObj = new FileInfo(ThreadContextObj.OriginalAttrObj.OriginalFilePath);
                                        FileInfoObj.CopyTo(TempFilePath.ToString());

                                        TempPathFromJob = TempFilePath.ToString();
                                        TempFilePath.Remove(0, TempFilePath.Length);
                                        TempFilePath.AppendFormat(GetUNCPath(TempPathFromJob, ThreadContextObj.MediaProcessor));
                                    }
                                    else
                                    {
                                        ThreadContextObj.OriginalAttrObj.IsFileCopiedToTempLocation = false;
                                        TempFilePath.AppendFormat(GetUNCPath(ThreadContextObj.OriginalAttrObj.OriginalFilePath, ThreadContextObj.MediaProcessor));
                                    }
                                    ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath = TempFilePath.ToString();                                                
                                    str_ExtraInformationForMediaProcessor                    = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.ORIGINAL, ThreadContextObj.OriginalAttrObj.OriginalGenerator);                        
                                    
                                } break;
                            default:
                                {
                                    TempFilePath.AppendFormat("{0}\\{1}.{2}", ThreadContextObj.TempFolderPath, FileID, FileExtension);
                                    FileInfoObj = new FileInfo(ThreadContextObj.OriginalAttrObj.OriginalFilePath);
                                    FileInfoObj.CopyTo(TempFilePath.ToString());
                                } break;
                        }

                        if (!ThreadContextObj.OriginalAttrObj.ManagedGenerator)
                        {
                            if (ThreadContextObj.IsBinHex)
                            {
                                InvokeMediaProcessorClassObj.SetFileAttributes(TempFilePath.ToString(), ThreadContextObj.OriginalAttrObj.TempOrgFilePath,
                                   null, null, FileExtension, 0, 0, 0, 0, 0, null);
                            }
                            else
                            {
                                InvokeMediaProcessorClassObj.SetFileAttributes(TempFilePath.ToString(), null,
                                   null, null, FileExtension, 0, 0, 0, 0, 0, null);
                            }
                            InvokeMediaProcessorClassObj.GeneratorName                  = ThreadContextObj.OriginalAttrObj.OriginalGenerator;
                            InvokeMediaProcessorClassObj.ThreadTimeout                  = ThreadTimeOut;
                            InvokeMediaProcessorClassObj.GetInfo();
                            
                            ThreadContextObj.OriginalAttrObj.OriginalHeight             = InvokeMediaProcessorClassObj.Height;
                            ThreadContextObj.OriginalAttrObj.OriginalWidth              = InvokeMediaProcessorClassObj.Width;
                            ThreadContextObj.OriginalAttrObj.OriginalDepth              = InvokeMediaProcessorClassObj.Depth;
                            ThreadContextObj.OriginalAttrObj.OriginalDPI                = InvokeMediaProcessorClassObj.DPI;
                           
                        }
                        else
                        {
                            if (ThreadContextObj.IsBinHex)
                            {
                                MediaProcessorWrapperObj.SetFileAttributes(TempFilePath.ToString(), ThreadContextObj.OriginalAttrObj.TempOrgFilePath,
                                   null, null, FileExtension, 0, 0, 0, 0, 0, null,str_ExtraInformationForMediaProcessor,ThreadContextObj.MediaProcessor);
                            }
                            else
                            {
                                MediaProcessorWrapperObj.SetFileAttributes(TempFilePath.ToString(), null,
                                   null, null, FileExtension, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor, ThreadContextObj.MediaProcessor);
                            }
                            MediaProcessorWrapperObj.GeneratorName                      = ThreadContextObj.OriginalAttrObj.OriginalGenerator;
                            MediaProcessorWrapperObj.ThreadTimeout                      = ThreadTimeOut;

                            //Set Processing Status in AssetInfo
                            CatalogerHelper.CatalogerHelper.SetProcessingStatusInAssetInfo(ref CatalogerContextObj, "GetOrgInfoRequestSent", "True", CatalogerContextObj.AssetInformationObj.AssetNode);

                            MediaProcessorWrapperObj.GetInfo();                            
                        }
                    }
                    #endregion
                }
                else if (RenditionTypeValue == RenditionType.PREVIEW)
                {
                    #region PREVIEW
                    if (!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewGenerator))
                    {
                        if (ThreadContextObj.PreviewAttrObj.PreviewFileExtension== null)
                            FileExtension               = ThreadContextObj.PreviewAttrObj.PreviewFileName.Remove(0, ThreadContextObj.PreviewAttrObj.PreviewFileName.LastIndexOf('.') + 1);
                        else
                            FileExtension               = ThreadContextObj.PreviewAttrObj.PreviewFileExtension;

                        switch (ThreadContextObj.PreviewAttrObj.PreviewGenerator)
                        {
                            case "IllustratorCS3Wrapper.dll":
                            case "MSOfficeProcessorWrapper.dll":
                            case "CatalogerGenericQWrapper.dll":
                            case "FLVWrapper.dll":
                                {
                                    if (IsFilePathContainsSpecialCharacters(ThreadContextObj.PreviewAttrObj.PreviewFilePath))
                                    {
                                        TempFilePath.AppendFormat("{0}\\Temp", MountPoint);
                                        if (!Directory.Exists(TempFilePath.ToString()))
                                            Directory.CreateDirectory(TempFilePath.ToString());
                                        TempFilePath.AppendFormat("\\{0}.{1}", FileID, FileExtension);

                                        FileInfoObj = new FileInfo(ThreadContextObj.PreviewAttrObj.PreviewFilePath);
                                        FileInfoObj.CopyTo(TempFilePath.ToString());

                                        TempPathFromJob = TempFilePath.ToString();
                                        TempFilePath.Remove(0, TempFilePath.Length);
                                        TempFilePath.AppendFormat(GetUNCPath(TempPathFromJob, ThreadContextObj.MediaProcessor));
                                    }
                                    else
                                    {
                                        TempFilePath.AppendFormat(GetUNCPath(ThreadContextObj.PreviewAttrObj.PreviewFilePath, ThreadContextObj.MediaProcessor));
                                        ThreadContextObj.PreviewAttrObj.IsFileCopiedToTempLocation = false;
                                    }
                                        str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.PREVIEW, ThreadContextObj.PreviewAttrObj.PreviewGenerator);
                                    
                                } break;
                            default:
                                {
                                    TempFilePath.AppendFormat("{0}\\{1}.{2}", ThreadContextObj.TempFolderPath, FileID, FileExtension);
                                    FileInfoObj = new FileInfo(ThreadContextObj.PreviewAttrObj.PreviewFilePath);
                                    FileInfoObj.CopyTo(TempFilePath.ToString());
                                } break;
                        }

                        

                        if (!ThreadContextObj.PreviewAttrObj.ManagedGenerator)
                        {
                            if (ThreadContextObj.IsPrvBinHex)
                            {
                                InvokeMediaProcessorClassObj.SetFileAttributes(TempFilePath.ToString(), ThreadContextObj.PreviewAttrObj.TempPrvFilePath,
                                null, null, FileExtension, 0, 0, 0, 0, 0, null);
                            }
                            else
                            {
                                InvokeMediaProcessorClassObj.SetFileAttributes(TempFilePath.ToString(), null,
                                 null, null, FileExtension, 0, 0, 0, 0, 0, null);
                            }
                           
                            InvokeMediaProcessorClassObj.GeneratorName                  = ThreadContextObj.PreviewAttrObj.PreviewGenerator;
                            InvokeMediaProcessorClassObj.ThreadTimeout                  = ThreadTimeOut;
                            InvokeMediaProcessorClassObj.GetInfo();
                           
                            ThreadContextObj.PreviewAttrObj.PreviewHeight               = InvokeMediaProcessorClassObj.Height;
                            ThreadContextObj.PreviewAttrObj.PreviewWidth                = InvokeMediaProcessorClassObj.Width;
                            ThreadContextObj.PreviewAttrObj.PreviewDepth                = InvokeMediaProcessorClassObj.Depth;
                            ThreadContextObj.PreviewAttrObj.PreviewDPI                  = InvokeMediaProcessorClassObj.DPI;
                            ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath     = TempFilePath.ToString();
                            
                         }
                        else
                        {
                            if (ThreadContextObj.IsPrvBinHex)
                            {
                                MediaProcessorWrapperObj.SetFileAttributes(TempFilePath.ToString(), ThreadContextObj.PreviewAttrObj.TempPrvFilePath,
                                null, null, FileExtension, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                            }
                            else
                            {
                                MediaProcessorWrapperObj.SetFileAttributes(TempFilePath.ToString(), null,
                                 null, null, FileExtension, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                            }

                            ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath     = TempFilePath.ToString();
                            MediaProcessorWrapperObj.GeneratorName                      = ThreadContextObj.PreviewAttrObj.PreviewGenerator;
                            MediaProcessorWrapperObj.ThreadTimeout                      = ThreadTimeOut;

                            //Set Processing Status in AssetInfo
                            CatalogerHelper.CatalogerHelper.SetProcessingStatusInAssetInfo(ref CatalogerContextObj, "GeneratePrvRequestSent", "True", CatalogerContextObj.AssetInformationObj.AssetNode);

                            MediaProcessorWrapperObj.GetInfo();     
                        }
                    }
                    #endregion
                }
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                {
                    #region THUMBNAIL
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator))
                    {
                        if (ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension == null)
                            FileExtension               = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.Remove(0, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.LastIndexOf('.') + 1);
                        else
                            FileExtension               = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension;

                        switch (ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator)
                        {
                            case "IllustratorCS3Wrapper.dll":
                            case "MSOfficeProcessorWrapper.dll":
                            case "CatalogerGenericQWrapper.dll":
                            case "FLVWrapper.dll":
                                {
                                    if (IsFilePathContainsSpecialCharacters(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath))
                                    {
                                        TempFilePath.AppendFormat("{0}\\Temp", MountPoint);
                                        if (!Directory.Exists(TempFilePath.ToString()))
                                            Directory.CreateDirectory(TempFilePath.ToString());
                                        TempFilePath.AppendFormat("\\{0}.{1}", FileID, FileExtension);

                                        FileInfoObj = new FileInfo(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath);
                                        FileInfoObj.CopyTo(TempFilePath.ToString());

                                        TempPathFromJob = TempFilePath.ToString();
                                        TempFilePath.Remove(0, TempFilePath.Length);
                                        TempFilePath.AppendFormat(GetUNCPath(TempPathFromJob, ThreadContextObj.MediaProcessor));
                                    }
                                    else
                                    {
                                        TempFilePath.AppendFormat(GetUNCPath(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath, ThreadContextObj.MediaProcessor));
                                        ThreadContextObj.ThumbnailAttrObj.IsFileCopiedToTempLocation = false;
                                    }
                                    str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.THUMBNAIL, ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator);
                                } break;
                            default:
                                {
                                    TempFilePath.AppendFormat("{0}\\{1}.{2}", ThreadContextObj.TempFolderPath, FileID, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.Remove(0, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.LastIndexOf('.') + 1));
                                    FileInfoObj = new FileInfo(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath);
                                    FileInfoObj.CopyTo(TempFilePath.ToString());
                                } break;
                        }                        

                        if (!ThreadContextObj.ThumbnailAttrObj.ManagedGenerator)
                        {
                            if (ThreadContextObj.IsThumbBinHex)
                            {

                                InvokeMediaProcessorClassObj.SetFileAttributes(TempFilePath.ToString(), ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath,
                                    null, null, FileExtension, 0, 0, 0, 0, 0, null);
                            }
                            else
                            {

                                InvokeMediaProcessorClassObj.SetFileAttributes(TempFilePath.ToString(), null,
                                    null, null, FileExtension, 0, 0, 0, 0, 0, null);
                            }

                            InvokeMediaProcessorClassObj.GeneratorName                  = ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator;
                            InvokeMediaProcessorClassObj.ThreadTimeout                  = ThreadTimeOut;
                            InvokeMediaProcessorClassObj.GetInfo();

                            ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight           = InvokeMediaProcessorClassObj.Height;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth            = InvokeMediaProcessorClassObj.Width;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth            = InvokeMediaProcessorClassObj.Depth;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI              = InvokeMediaProcessorClassObj.DPI;
                            ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath  = TempFilePath.ToString();    
                        }
                        else
                        {
                            if (ThreadContextObj.IsThumbBinHex)
                            {

                                MediaProcessorWrapperObj.SetFileAttributes(TempFilePath.ToString(), ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath,
                                    null, null, FileExtension, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                            }
                            else
                            {

                                MediaProcessorWrapperObj.SetFileAttributes(TempFilePath.ToString(), null,
                                    null, null, FileExtension, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                            }

                            ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath  = TempFilePath.ToString();
                            MediaProcessorWrapperObj.GeneratorName                      = ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator;
                            MediaProcessorWrapperObj.ThreadTimeout                      = ThreadTimeOut;

                            //Set Processing Status in AssetInfo
                            CatalogerHelper.CatalogerHelper.SetProcessingStatusInAssetInfo(ref CatalogerContextObj, "GenerateThbRequestSent", "True", CatalogerContextObj.AssetInformationObj.AssetNode);                            

                            MediaProcessorWrapperObj.GetInfo();
                        }
                    }
                    #endregion
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }           
        }
       
        private Boolean GetDigitalAssetInfo( ThreadContext ThreadContextObj)
        {
            FileInfo                FileInfoObj                 = null;
            StringBuilder           FilePath                    = null;
            String                  OriginalFileName            = null;
            String                  OriginalFileNameWithoutExt  = null;
            StringBuilder           Pattern                     = null;
            String[]                FileNames                   = null;
            StringBuilder           ErrorMessage                = null;
            Boolean                 IsBinHex                    = false;
            BinHeXtractWrapperClass BinHeXtractWrapperClassObj  = null;
                            
            try
            {
                FilePath                    = new StringBuilder();
                Pattern                     = new StringBuilder();
                ErrorMessage                = new StringBuilder();
                BinHeXtractWrapperClassObj  = new BinHeXtractWrapperClass();
                OriginalFileName            = ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("NAME").InnerText;
                
                FilePath.AppendFormat("{0}\\Originals\\{1}", JobPath, OriginalFileName);
                if (File.Exists(FilePath.ToString()))
                {
                    FileInfoObj                                         = new FileInfo(FilePath.ToString());
                    ThreadContextObj.OriginalAttrObj.OriginalFileName   = FileInfoObj.Name;
                    ThreadContextObj.OriginalAttrObj.OriginalSize       = FileInfoObj.Length;
                    ThreadContextObj.OriginalAttrObj.OriginalFilePath   = FilePath.ToString();
                }
                else
                {
                    strdict_AdditionalInfo.Clear();
                    strdict_AdditionalInfo.Add("ORIGINALFILENAME", OriginalFileName);
                    ErrorMessage.AppendFormat("Original file {0} not found.", OriginalFileName);
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_ORGFILENOTFOUND", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                    UpdateCatalogJobProcessingDetails(ThreadContextObj, null, false);
                    return false;
                }
                
                #region Check whether Thumnail or Preview exists
                if (OriginalFileName.Contains("."))
                    OriginalFileNameWithoutExt = OriginalFileName.Remove(OriginalFileName.LastIndexOf('.'));
                else
                    OriginalFileNameWithoutExt = OriginalFileName;

                Pattern.AppendFormat("{0}.*",OriginalFileNameWithoutExt);

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Previews", JobPath);

                if (Directory.Exists(FilePath.ToString()))
                {
                    FileNames   = Directory.GetFiles(FilePath.ToString(), Pattern.ToString());
                    if (FileNames.Length > 0)
                    {
                        FileInfoObj                                             = new FileInfo(FileNames[0]);
                        CatalogerContextObj.AssetInformationObj.PreviewFileName = FileInfoObj.Name;
                        ThreadContextObj.PreviewAttrObj.PreviewFileName         = FileInfoObj.Name;    
                        ThreadContextObj.PreviewAttrObj.PreviewSize             = FileInfoObj.Length;
                        ThreadContextObj.PreviewAttrObj.PreviewFilePath         = FileInfoObj.FullName;
                        ThreadContextObj.PreviewExists                          = true;
                    }
                }

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Thumbnails", JobPath);
                if (Directory.Exists(FilePath.ToString()))
                {
                    FileNames           = Directory.GetFiles(FilePath.ToString(),Pattern.ToString());
                    if (FileNames.Length > 0)
                    {
                        FileInfoObj                                                 = new FileInfo(FileNames[0]);
                        CatalogerContextObj.AssetInformationObj.ThumbnailFileName   = FileInfoObj.Name;  
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName         = FileInfoObj.Name;    
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailSize             = FileInfoObj.Length;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath         = FileInfoObj.FullName;
                        ThreadContextObj.ThumbnailExists                            = true;
                    }
                }

                FilePath.Remove(0, FilePath.Length);
                FilePath.AppendFormat("{0}\\Temporary", JobPath);
                ThreadContextObj.TempFolderPath = System.Environment.GetEnvironmentVariable("Temp", EnvironmentVariableTarget.Machine);
                
                #endregion

                // read the file and attempt to get mac Type and Creator
                BinHeXtractWrapperClassObj.GetMacTypeAndCreator(ThreadContextObj.OriginalAttrObj.OriginalFilePath);
                IsBinHex = Convert.ToBoolean(BinHeXtractWrapperClassObj.IsBinHex);
                if (IsBinHex)
                {
                    ThreadContextObj.OriginalAttrObj.OrgMacType     = BinHeXtractWrapperClassObj.Type.Trim();
                    ThreadContextObj.OriginalAttrObj .OrgMacCreator = BinHeXtractWrapperClassObj.Creator.Trim();
                    ThreadContextObj.IsBinHex                       = true;
                }
                #region Get File Extensions
                if (!ThreadContextObj.IsBinHex)
                {
                    if (ThreadContextObj.OriginalAttrObj.OriginalFileName.Contains("."))
                        ThreadContextObj.OriginalAttrObj.OriginalFileExtension      = ThreadContextObj.OriginalAttrObj.OriginalFileName.Remove(0, ThreadContextObj.OriginalAttrObj.OriginalFileName.LastIndexOf('.') + 1);
                    else
                        ThreadContextObj.OriginalAttrObj.OriginalFileExtension      = "";

                    if (ThreadContextObj.PreviewExists)
                    {
                        if (ThreadContextObj.PreviewAttrObj.PreviewFileName.Contains("."))
                            ThreadContextObj.PreviewAttrObj.PreviewFileExtension    = ThreadContextObj.PreviewAttrObj.PreviewFileName.Remove(0, ThreadContextObj.PreviewAttrObj.PreviewFileName.LastIndexOf('.') + 1);
                        else
                            ThreadContextObj.PreviewAttrObj.PreviewFileExtension    = "";
                    }
                    if (ThreadContextObj.ThumbnailExists)
                    {
                        if (ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.Contains("."))
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.Remove(0, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.LastIndexOf('.') + 1);
                        else
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension = "";
                        
                    }

                    return true;
                }
                else
                {
                    // BinHex Processing
                    FilePath.Remove(0, FilePath.Length);
                    FilePath.AppendFormat("{0}\\Originals\\$CatTemp$", JobPath);
                    if (!Directory.Exists(FilePath.ToString()))
                    {
                        Directory.CreateDirectory(FilePath.ToString());
                    }
                    FilePath.AppendFormat("\\{0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);

                    // extract the data fork
                    ThreadContextObj.OriginalAttrObj.TempOrgFilePath = FilePath.ToString();
                    if (!File.Exists(ThreadContextObj.OriginalAttrObj.TempOrgFilePath)) // This condition is checked as their is retry facility.
                        BinHeXtractWrapperClassObj.GetDataForkFromFile(ThreadContextObj.OriginalAttrObj.OriginalFilePath, ThreadContextObj.OriginalAttrObj.TempOrgFilePath);

                    if (ThreadContextObj.PreviewExists)
                    {
                        // read the file and attempt to get mac Type and Creator
                        BinHeXtractWrapperClassObj.GetMacTypeAndCreator(ThreadContextObj.PreviewAttrObj.PreviewFilePath);
                        IsBinHex = Convert.ToBoolean(BinHeXtractWrapperClassObj.IsBinHex);
                        if (!IsBinHex)
                        {
                            ThreadContextObj.PreviewAttrObj.PreviewFileExtension = ThreadContextObj.PreviewAttrObj.PreviewFileName.Remove(0, ThreadContextObj.PreviewAttrObj.PreviewFileName.LastIndexOf('.') + 1);
                        }
                        else
                        {
                            ThreadContextObj.IsPrvBinHex                    = true;
                            ThreadContextObj.PreviewAttrObj.PrvMacCreator   = BinHeXtractWrapperClassObj.Creator.Trim();
                            ThreadContextObj.PreviewAttrObj.PrvMacType      = BinHeXtractWrapperClassObj.Type.Trim();

                            // BinHex Processing
                            FilePath.Remove(0, FilePath.Length);
                            FilePath.AppendFormat("{0}\\Previews\\$CatTemp$", JobPath);
                            if (!Directory.Exists(FilePath.ToString()))
                            {
                                Directory.CreateDirectory(FilePath.ToString());
                            }
                            FilePath.AppendFormat("\\{0}", ThreadContextObj.PreviewAttrObj.PreviewFileName);

                            // extract the data fork
                            ThreadContextObj.PreviewAttrObj.TempPrvFilePath = FilePath.ToString();
                            if (!File.Exists(ThreadContextObj.PreviewAttrObj.TempPrvFilePath)) // This condition is checked as their is retry facility.
                                BinHeXtractWrapperClassObj.GetDataForkFromFile(ThreadContextObj.PreviewAttrObj.PreviewFilePath, ThreadContextObj.PreviewAttrObj.TempPrvFilePath);

                        }
                    }

                    if (ThreadContextObj.ThumbnailExists)
                    {
                        // read the file and attempt to get mac Type and Creator
                        BinHeXtractWrapperClassObj.GetMacTypeAndCreator(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath);
                        IsBinHex = Convert.ToBoolean(BinHeXtractWrapperClassObj.IsBinHex);
                        if (!IsBinHex)
                        {
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.Remove(0, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName.LastIndexOf('.') + 1);
                        }
                        else
                        {
                            ThreadContextObj.IsThumbBinHex                      = true;
                            ThreadContextObj.ThumbnailAttrObj.ThumbMacCreator   = BinHeXtractWrapperClassObj.Creator.Trim();
                            ThreadContextObj.ThumbnailAttrObj.ThumbMacType      = BinHeXtractWrapperClassObj.Type.Trim();

                            // BinHex Processing
                            FilePath.Remove(0, FilePath.Length);
                            FilePath.AppendFormat("{0}\\Thumbnails\\$CatTemp$", JobPath);
                            if (!Directory.Exists(FilePath.ToString()))
                            {
                                Directory.CreateDirectory(FilePath.ToString());
                            }
                            FilePath.AppendFormat("\\{0}", ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName);

                            // extract the data fork
                            ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath = FilePath.ToString();
                            if (!File.Exists(ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath)) // This condition is checked as their is retry facility.
                                BinHeXtractWrapperClassObj.GetDataForkFromFile(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath, ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath);

                        }
                    }

                }
                #endregion

                return true;
                
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                ErrorMessage.AppendFormat("Exception thrown in GetDigitalAssetInfo for DigitalAsset: {0} : Message : {1}",ThreadContextObj.ParentDigitalAssetName, ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_GETDIGITALASSETINFOFAILED", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
                UpdateCatalogJobProcessingDetails(ThreadContextObj, null, false);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }
                 
        private Boolean LoadGenerator( ThreadContext ThreadContextObj)
        {
            StringBuilder   XPathQuery              = null;
            XmlNode         FileFormatNode          = null;
            StringBuilder   ErrorMessage            = null;
            String          PartialQuery            = null;
            String          TranslateLower          = null;
            XmlNode         UseOrgAsPrvNode         = null;
            XmlNode         MetadataProcessorNode   = null;
            XmlNode         MediaProcessorNode      = null;

            try
            {
                XPathQuery              = new StringBuilder();
                ErrorMessage            = new StringBuilder();
                if (FileFormatXMLDoc.SelectSingleNode("//FILEFORMAT") == null)
                {
                    ErrorMessage.Append("FileFormats not found.");
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FILEFORMATSNOTFOUND", ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                    return false;
                }

                TranslateLower      = "translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')=";
                PartialQuery        = "//FILEFORMAT[FILEEXTENSION[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')=";

                if (!ThreadContextObj.IsBinHex)
                {
                    #region Check whether XMP can be extracted fron the file based on file type
                    if (XMPSupportRequired.ToUpper() == "YES")
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            MetadataProcessorNode = FileFormatNode.SelectSingleNode("METADATAPROCESSOR");
                            if ((MetadataProcessorNode != null) && (!String.IsNullOrEmpty(MetadataProcessorNode.InnerText)))
                            {
                                ThreadContextObj.OriginalAttrObj.MetadataProcessor = MetadataProcessorNode.InnerText;
                            }
                        }
                    }
                    #endregion

                    #region Get media processor name
                    XPathQuery.Remove(0, XPathQuery.Length);
                    XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                    if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                    {
                        MediaProcessorNode = FileFormatNode.SelectSingleNode("APPLICATIONTYPE");
                        if ((MediaProcessorNode != null) && (!String.IsNullOrEmpty(MediaProcessorNode.InnerText)))
                        {
                            ThreadContextObj.MediaProcessor = MediaProcessorNode.InnerText;
                        }
                    }
                    #endregion

                    if (ThreadContextObj.PreviewExists && ThreadContextObj.ThumbnailExists)
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]",PartialQuery, ThreadContextObj.PreviewAttrObj.PreviewFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.PreviewAttrObj.PreviewFileFormat       = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;

                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                }
                                else
                                {
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                }
                            }
                        }
                        
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                }
                                else
                                {
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                }
                            }
                        }

                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;

                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                            {
                                ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                            }
                            else
                            {
                                ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                            }
                        }

                        return true;
                    }

                    if ((!ThreadContextObj.PreviewExists) && (!ThreadContextObj.ThumbnailExists))
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.FormatDetails                          = FileFormatNode.CloneNode(true);

                            if ((UseOrgAsPrvNode = FileFormatNode.SelectSingleNode("USEORIGINALASPREVIEW")) != null)
                            {
                                if (UseOrgAsPrvNode.InnerText == "1")
                                    ThreadContextObj.UseOriginalAsPreview = true;
                            }

                            ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;

                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                }
                                else
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                }
                            }
                        }
                        return true;
                    }

                    if ((ThreadContextObj.PreviewExists) && (!ThreadContextObj.ThumbnailExists))
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.PreviewAttrObj.PreviewFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.FormatDetails                              = FileFormatNode.CloneNode(true);// Cloning is done bec if garbage collection happens it might garbage collect FileFormatNode

                            ThreadContextObj.PreviewAttrObj.PreviewFileFormat           = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator        = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                            ThreadContextObj.PreviewAttrObj.PreviewGenerator            = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            ThreadContextObj.GenThumbFromPreview                        = true;

                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                }
                                else
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                }
                            }

                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;

                                // This Condition Added because in some cases preview entry can be present in DB with no
                                // Thumbnail extension in such case we should try to generate the thumbnail from original.
                                if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator))
                                {
                                    ThreadContextObj.ThbFromOrgGenerator = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                    {
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                        {
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                        }
                                        else
                                        {
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator = false;
                                        }
                                    }
                                }
                                else
                                {
                                    ThreadContextObj.GenThumbFromPreview                    = false;
                                    ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                    {
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                        {
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                        }
                                        else
                                        {
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                        }
                                    }
                                }
                            }
                            return true;
                        }
                        else
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.FormatDetails                          = FileFormatNode.CloneNode(true);

                                ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                ThreadContextObj.GenThumbFromPreview                    = false;
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                        ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                        ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                    }
                                }
                            }
                            return true;
                        }
                        
                    }

                    if ((!ThreadContextObj.PreviewExists) && (ThreadContextObj.ThumbnailExists))
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.FormatDetails                          = FileFormatNode.CloneNode(true);

                            if ((UseOrgAsPrvNode = FileFormatNode.SelectSingleNode("USEORIGINALASPREVIEW")) != null)
                            {
                                if (UseOrgAsPrvNode.InnerText == "1")
                                    ThreadContextObj.UseOriginalAsPreview = true;
                            }

                            ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;

                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                }
                                else
                                {
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                }
                            }
                        }

                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                }
                                else
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = false;
                                }
                            }
                        }
                        return true;
                    }
                    return false;
                }
                else
                {
                    #region Check whether XMP can be extracted fron the file based on file type
                    if (XMPSupportRequired.ToUpper() == "YES")
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.OriginalAttrObj.OrgMacCreator.ToLower(), ThreadContextObj.OriginalAttrObj.OrgMacType.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            MetadataProcessorNode = FileFormatNode.SelectSingleNode("METADATAPROCESSOR");
                            if ((MetadataProcessorNode != null) && (!String.IsNullOrEmpty(MetadataProcessorNode.InnerText)))
                            {
                                ThreadContextObj.OriginalAttrObj.MetadataProcessor = MetadataProcessorNode.InnerText;
                            }
                        }
                    }
                    #endregion

                    #region Get media processor name
                    XPathQuery.Remove(0, XPathQuery.Length);
                    XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                    if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                    {
                        MediaProcessorNode = FileFormatNode.SelectSingleNode("APPLICATIONTYPE");
                        if ((MediaProcessorNode != null) && (!String.IsNullOrEmpty(MediaProcessorNode.InnerText)))
                        {
                            ThreadContextObj.MediaProcessor = MediaProcessorNode.InnerText;
                        }
                    }
                    #endregion

                    // Binhex Processing
                    if ((ThreadContextObj.PreviewExists) && (ThreadContextObj.ThumbnailExists))
                    {
                        if(!ThreadContextObj.IsPrvBinHex)
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.PreviewAttrObj.PreviewFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.PreviewAttrObj.PreviewFileFormat       = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;

                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator = false;
                                    }
                                }
                            }
                        }

                        if(!ThreadContextObj.IsThumbBinHex)
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;

                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = false;
                                    }
                                }
                            }
                        }

                        if (ThreadContextObj.IsPrvBinHex)
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'2']]", TranslateLower, ThreadContextObj.PreviewAttrObj.PrvMacCreator.ToLower(), ThreadContextObj.PreviewAttrObj.PrvMacType.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.PreviewAttrObj.PreviewFileFormat       = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator = false;
                                    }
                                }
                            }
                        }

                        if (ThreadContextObj.IsThumbBinHex)
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'2']]", TranslateLower, ThreadContextObj.ThumbnailAttrObj.ThumbMacCreator.ToLower(), ThreadContextObj.ThumbnailAttrObj.ThumbMacType.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = false;
                                    }
                                }
                            }
                        }
                       
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'2']]", TranslateLower, ThreadContextObj.OriginalAttrObj.OrgMacCreator.ToLower(), ThreadContextObj.OriginalAttrObj.OrgMacType.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.OriginalAttrObj.OriginalFileFormat = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.OriginalAttrObj.OriginalGenerator = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                }
                                else
                                {
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator = false;
                                }
                            }
                        }
                        return true;
                    }
                    else if((!ThreadContextObj.PreviewExists) && (!ThreadContextObj.ThumbnailExists))
                    {
                        XPathQuery.Remove(0, XPathQuery.Length);
                        XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.OriginalAttrObj.OrgMacCreator.ToLower(), ThreadContextObj.OriginalAttrObj.OrgMacType.ToLower());
                        if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                        {
                            ThreadContextObj.FormatDetails                              = FileFormatNode.CloneNode(true);
                            if ((UseOrgAsPrvNode = FileFormatNode.SelectSingleNode("USEORIGINALASPREVIEW")) != null)
                            {
                                if (UseOrgAsPrvNode.InnerText == "1")
                                    ThreadContextObj.UseOriginalAsPreview = true;
                            }
                            ThreadContextObj.OriginalAttrObj.OriginalFileFormat         = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                            ThreadContextObj.OriginalAttrObj.OriginalGenerator          = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            ThreadContextObj.PreviewAttrObj.PreviewGenerator            = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                            ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator        = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;

                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                }
                                else
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                    ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                    ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                }
                            }

                            return true;
                        }
                    }
                    else if ((ThreadContextObj.PreviewExists) && (!ThreadContextObj.ThumbnailExists))
                    {
                        if (ThreadContextObj.IsPrvBinHex)
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.PreviewAttrObj.PrvMacCreator.ToLower(), ThreadContextObj.PreviewAttrObj.PrvMacType.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.FormatDetails = FileFormatNode.CloneNode(true);

                                ThreadContextObj.PreviewAttrObj.PreviewFileFormat           = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator        = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                ThreadContextObj.PreviewAttrObj.PreviewGenerator            = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                ThreadContextObj.GenThumbFromPreview                        = true;

                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                    }
                                }

                                XPathQuery.Remove(0, XPathQuery.Length);
                                XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.OriginalAttrObj.OrgMacCreator.ToLower(), ThreadContextObj.OriginalAttrObj.OrgMacType.ToLower());
                                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                                {
                                    ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                    ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                    
                                    // This Condition Added because in some cases preview entry can be present in DB with no
                                    // Thumbnail extension in such case we should try to generate the thumbnail from original.
                                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator))
                                    {
                                        ThreadContextObj.ThbFromOrgGenerator = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                        {
                                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                            }
                                            else
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator = false;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        ThreadContextObj.GenThumbFromPreview                    = false;
                                        ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                        {
                                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                                ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                            }
                                            else
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                                ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                            }
                                        }
                                    }

                                    return true;
                                }
                                return true;
                            }
                            else
                            {
                                XPathQuery.Remove(0, XPathQuery.Length);
                                XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.OriginalAttrObj.OrgMacCreator.ToLower(), ThreadContextObj.OriginalAttrObj.OrgMacType.ToLower());
                                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                                {
                                    ThreadContextObj.FormatDetails = FileFormatNode.CloneNode(true);

                                    ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                    ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                    ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                    ThreadContextObj.GenThumbFromPreview                    = false;
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                    {
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                        {
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                        }
                                        else
                                        {
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = false;
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator = false;
                                        }
                                    }
                                    return true;
                                }
                            }
                        }
                        else
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.PreviewAttrObj.PreviewFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.FormatDetails                          = FileFormatNode.CloneNode(true);
                                ThreadContextObj.PreviewAttrObj.PreviewFileFormat       = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                ThreadContextObj.GenThumbFromPreview                    = true;
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                    }
                                }

                                XPathQuery.Remove(0, XPathQuery.Length);
                                XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                                {
                                    ThreadContextObj.OriginalAttrObj.OriginalFileFormat = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                    ThreadContextObj.OriginalAttrObj.OriginalGenerator  = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                   
                                    // This Condition Added because in some cases preview entry can be present in DB with no
                                    // Thumbnail extension in such case we should try to generate the thumbnail from original.
                                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator))
                                    {
                                        ThreadContextObj.ThbFromOrgGenerator = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                        {
                                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                            }
                                            else
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator = false;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        ThreadContextObj.GenThumbFromPreview = false;
                                        ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                        {
                                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                                ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = true;
                                            }
                                            else
                                            {
                                                ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                                ThreadContextObj.ThumbnailAttrObj.ManagedGenerator  = false;
                                            }
                                        }
                                    }

                                    return true;
                                }
                                return true;
                            }
                            else
                            {
                                XPathQuery.Remove(0, XPathQuery.Length);
                                XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                                {
                                    ThreadContextObj.FormatDetails = FileFormatNode.CloneNode(true);

                                    ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                    ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                    ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                    ThreadContextObj.GenThumbFromPreview = false;
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                    {
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                        {
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                        }
                                        else
                                        {
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                            ThreadContextObj.OriginalAttrObj.ManagedGenerator = true;
                                        }
                                    } 
                                    return true;
                                }
                           }

                        }

                    }
                    else if ((!ThreadContextObj.PreviewExists) && (ThreadContextObj.ThumbnailExists))
                    {
                        if (!ThreadContextObj.IsThumbBinHex)
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.OriginalAttrObj.OriginalFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.FormatDetails                          = FileFormatNode.CloneNode(true);

                                if ((UseOrgAsPrvNode = FileFormatNode.SelectSingleNode("USEORIGINALASPREVIEW")) != null)
                                {
                                    if (UseOrgAsPrvNode.InnerText == "1")
                                        ThreadContextObj.UseOriginalAsPreview = true;
                                }

                                ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;

                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                        ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                        ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                    }
                                }
                            }

                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                    }
                                } 
                                return true;
                            }
                        }
                        else
                        {
                            XPathQuery.Remove(0, XPathQuery.Length);
                            XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.OriginalAttrObj.OrgMacCreator.ToLower(), ThreadContextObj.OriginalAttrObj.OrgMacType.ToLower());
                            if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                            {
                                ThreadContextObj.FormatDetails                          = FileFormatNode.CloneNode(true);

                                if ((UseOrgAsPrvNode = FileFormatNode.SelectSingleNode("USEORIGINALASPREVIEW")) != null)
                                {
                                    if (UseOrgAsPrvNode.InnerText == "1")
                                        ThreadContextObj.UseOriginalAsPreview = true;
                                }

                                ThreadContextObj.OriginalAttrObj.OriginalFileFormat     = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                ThreadContextObj.PreviewAttrObj.PreviewGenerator        = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;
                                ThreadContextObj.OriginalAttrObj.OriginalGenerator      = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;

                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                {
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = true;
                                        ThreadContextObj.OriginalAttrObj.ManagedGenerator   = true;
                                    }
                                    else
                                    {
                                        ThreadContextObj.PreviewAttrObj.ManagedGenerator    = false;
                                        ThreadContextObj.OriginalAttrObj.ManagedGenerator   = false;
                                    }
                                }
                                
                                XPathQuery.Remove(0, XPathQuery.Length);
                                XPathQuery.AppendFormat("//FILEFORMAT[MACCREATOR[{0}'{1}'] and MACTYPE[{0}'{2}']]", TranslateLower, ThreadContextObj.ThumbnailAttrObj.ThumbMacCreator.ToLower(), ThreadContextObj.ThumbnailAttrObj.ThumbMacType.ToLower());
                                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                                {
                                    ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                                    ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                                    if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                                    {
                                        if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                        {
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                        }
                                        else
                                        {
                                            ThreadContextObj.ThumbnailAttrObj.ManagedGenerator = true;
                                        }
                                    } 
                                    return true;
                                }
                                return true;
                            }
                        }
                    }
                             
                }
                return false;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in LoadGenerator : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_LOADGENERATORFAILED",ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }
                
        private Boolean GetUserPreferences( CatalogerContext CatalogerContextObj)
        {
            XML             UserCatalogerPreferences        = null;
            XmlDocument     UserCatalogerPreferencesDoc     = null;
            XmlNode         CatalogerPreferencesNode        = null;
            ERRORXML        ErrorXML                        = null;
            StringBuilder   ErrorMessage                    = null;
            XPathDocument   TenantCatalogingPreferencesDoc  = null;
            XPathNavigator  RootNode                        = null;
            XPathNavigator  PreferenceValueNode             = null;  

            try
            {

                ErrorMessage            = new StringBuilder();
                ErrorXML                = DAMWSObj.GetUserCatalogerPreferences(ref UserCatalogerPreferences);

                if ((ErrorXML == null) && (UserCatalogerPreferences.Length > 0))
                {
                    UserCatalogerPreferencesDoc         = new XmlDocument();
                    UserCatalogerPreferencesDoc.LoadXml(UserCatalogerPreferences);
                    
                    CatalogerPreferencesNode            = UserCatalogerPreferencesDoc.SelectSingleNode("//USER_DAMPERSONALISATION/DAMPERSONALIZATION/CONFIGURATIONXML/APPLICATIONSETTINGS/CATALOGERPREFERENCES");
                    // User preferences
                    if (CatalogerPreferencesNode != null)
                    {
                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWHEIGHT")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewHeight = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWHEIGHT").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewHeight = 768;

                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWWIDTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewWidth = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWWIDTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewWidth = 1024;

                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWDEPTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewDepth     = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWDEPTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewDepth     = 24;

                        if (CatalogerPreferencesNode.SelectSingleNode("PREVIEWDPI")!=null)
                            CatalogerContextObj.UserPreferencesObj.PreviewDPI       = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("PREVIEWDPI").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.PreviewDPI       = 72;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILHEIGHT")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILHEIGHT").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailHeight = 125;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILWIDTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILWIDTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDEPTH")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDEPTH").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;

                        if(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILQUALITY")!=null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailQuality  = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILQUALITY").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailQuality  = 75;

                        if (CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDPI") != null)
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDPI      = Convert.ToInt32(CatalogerPreferencesNode.SelectSingleNode("THUMBNAILDPI").InnerText);
                        else
                            CatalogerContextObj.UserPreferencesObj.ThumbnailDPI      = 72;

                        if (CatalogerPreferencesNode.SelectSingleNode("EMAILNOTIFICATION") != null)
                        {                            
                            switch  (CatalogerPreferencesNode.SelectSingleNode("EMAILNOTIFICATION").InnerText)
                            {
                                case "ALWAYS":
                                            UserNotificationPref.SendMailSettings = MailSettings.ALWAYS;
                                            break ;
                                case "ONERROR":
                                            UserNotificationPref.SendMailSettings = MailSettings.ONERROR;
                                            break ;
                                case "NEVER":
                                default     : UserNotificationPref.SendMailSettings = MailSettings.NEVER;
                                            break;
                            }
                        }

                        if (CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS") != null)
                            UserNotificationPref.SendMailTo = CatalogerPreferencesNode.SelectSingleNode("EMAILADDRESS").InnerText;
                        
                    }
                    else if(!String.IsNullOrEmpty(TenantCatalogingPreferencesXML))
                    {
                        // Tenant preferences
                        TenantCatalogingPreferencesDoc  = new XPathDocument(new StringReader(TenantCatalogingPreferencesXML));
                        RootNode                        = TenantCatalogingPreferencesDoc.CreateNavigator();

                        if (RootNode != null)
                        {
                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWHEIGHT"))!= null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewHeight    = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewHeight    = 768;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewHeight        = 768;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWWIDTH")) != null)
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewWidth = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewWidth = 1024;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewWidth     = 1024;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWDEPTH")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewDepth = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewDepth = 24;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewDepth     = 24;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/PREVIEWDPI")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.PreviewDPI   = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.PreviewDPI   = 72;
                            else
                                CatalogerContextObj.UserPreferencesObj.PreviewDPI       = 72;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILHEIGHT")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = 125;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailHeight      = 125;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILWIDTH")) != null )
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = 125;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailWidth       = 125;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILDEPTH")) != null) 
                                 if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                     CatalogerContextObj.UserPreferencesObj.ThumbnailDepth  = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailDepth       = 24;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILQUALITY")) != null)
                                 if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailQuality = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailQuality = 75;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailQuality     = 75;

                            if ((PreferenceValueNode = RootNode.SelectSingleNode("CATALOGINGPREFERENCES/THUMBNAILDPI")) != null)
                                if (!String.IsNullOrEmpty(PreferenceValueNode.Value))
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailDPI     = Convert.ToInt32(PreferenceValueNode.Value);
                                else
                                    CatalogerContextObj.UserPreferencesObj.ThumbnailDPI     = 72;
                            else
                                CatalogerContextObj.UserPreferencesObj.ThumbnailDPI         = 72;
                        }

                    }
                    else
                    {
                        CatalogerContextObj.UserPreferencesObj.PreviewHeight    = 768;
                        CatalogerContextObj.UserPreferencesObj.PreviewWidth     = 1024;
                        CatalogerContextObj.UserPreferencesObj.PreviewDepth     = 24;
                        CatalogerContextObj.UserPreferencesObj.PreviewDPI       = 72;

                        CatalogerContextObj.UserPreferencesObj.ThumbnailHeight  = 125;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailWidth   = 125;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailDepth   = 24;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailDPI     = 72;
                        CatalogerContextObj.UserPreferencesObj.ThumbnailQuality = 75;
                    }
                }
                else
                {
                    ErrorMessage.Append("Getting user preferences failed.");
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_USERPREFERENCESREADFAILED", ErrorMessage.ToString(), ErrorXML,null, ErrorType.ERROR);
                    return false;
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in GetUserPreferences : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_GETUSERPREFERENCESFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }

        private Boolean FillUserPreferences( ThreadContext ThreadContextObj, CatalogerContext CatalogerContextObj)
        {
            try
            {
                ThreadContextObj.PreviewAttrObj.PreviewDepth            = CatalogerContextObj.UserPreferencesObj.PreviewDepth;
                ThreadContextObj.PreviewAttrObj.PreviewDPI              = CatalogerContextObj.UserPreferencesObj.PreviewDPI;
                ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth        = CatalogerContextObj.UserPreferencesObj.ThumbnailDepth;
                ThreadContextObj.ThumbnailAttrObj.ThumbnailQuality      = CatalogerContextObj.UserPreferencesObj.ThumbnailQuality;
                ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI          = CatalogerContextObj.UserPreferencesObj.ThumbnailDPI;

                if (CatalogerContextObj.UserPreferencesObj.PreviewHeight > MIN_HT_ATTRIBUTE && CatalogerContextObj.UserPreferencesObj.PreviewHeight <= MAX_HT_ATTRIBUTE)
                    ThreadContextObj.PreviewAttrObj.PreviewHeight       = CatalogerContextObj.UserPreferencesObj.PreviewHeight;

                if (CatalogerContextObj.UserPreferencesObj.PreviewWidth > MIN_WT_ATTRIBUTE && CatalogerContextObj.UserPreferencesObj.PreviewWidth <= MAX_WT_ATTRIBUTE)
                    ThreadContextObj.PreviewAttrObj.PreviewWidth        = CatalogerContextObj.UserPreferencesObj.PreviewWidth;


                if (CatalogerContextObj.UserPreferencesObj.ThumbnailHeight > MIN_HT_ATTRIBUTE && CatalogerContextObj.UserPreferencesObj.ThumbnailHeight <= MAX_HT_ATTRIBUTE)
                    ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight   = CatalogerContextObj.UserPreferencesObj.ThumbnailHeight;

                if (CatalogerContextObj.UserPreferencesObj.ThumbnailWidth > MIN_WT_ATTRIBUTE && CatalogerContextObj.UserPreferencesObj.ThumbnailWidth <= MAX_WT_ATTRIBUTE)
                    ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth    = CatalogerContextObj.UserPreferencesObj.ThumbnailWidth;
                return true;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean GenerateAssetPreview( ThreadContext ThreadContextObj)
        {
            String                      DestinationFormat               = null;
            StringBuilder               PreviewFileName                 = null;
            StringBuilder               PreviewFilePath                 = null;
            String                      ActualFileName                  = null;
            String                      DestinationName                 = null;
            String                      CurrentFileName                 = null;
            StringBuilder               TemporaryFilePath               = null;
            InvokeMediaProcessorClass   InvokeMediaProcessorClassObj    = null;            
            StringBuilder               ErrorMessage                    = null;            
            MediaProcessorWrapper       MediaProcessorWrapperObj        = null;
            String                      str_ExtraInformationForMediaProcessor = null;            
                        
            try
            {
                ErrorMessage            = new StringBuilder();
                
                if (ThreadContextObj.PreviewAttrObj.PreviewGenerator != null)
                {
                    if (ThreadContextObj.PreviewAttrObj.PreviewGenerator.Length > 0)
                    {
                        PreviewFileName                                     = new StringBuilder();
                        PreviewFilePath                                     = new StringBuilder();
                        TemporaryFilePath                                   = new StringBuilder();
                        ErrorMessage                                        = new StringBuilder();

                        GetPreviewFormatFromID(ThreadContextObj.FormatDetails.SelectSingleNode("PREVIEWFORMATID").InnerText,ref DestinationFormat);
                         
                        PreviewFileName.AppendFormat("{0}.{1}",ThreadContextObj.OriginalAttrObj.OriginalFileName, DestinationFormat);
                        PreviewFilePath.AppendFormat("{0}\\Previews\\{1}",JobPath,PreviewFileName.ToString());
                        ThreadContextObj.PreviewAttrObj.PreviewFileName     = PreviewFileName.ToString();
                        ThreadContextObj.PreviewAttrObj.PreviewFilePath     = PreviewFilePath.ToString();
                        ThreadContextObj.PreviewAttrObj.PreviewFileFormat   = DestinationFormat.ToString();
                        ActualFileName                                      = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                        DestinationName                                     = PreviewFilePath.ToString();
                        GetDestinationFileName(ThreadContextObj, DestinationFormat, RenditionType.ORIGINAL, ThreadContextObj.PreviewAttrObj.PreviewGenerator, ref CurrentFileName, ref DestinationName);
                        str_ExtraInformationForMediaProcessor               = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.ORIGINAL, ThreadContextObj.PreviewAttrObj.PreviewGenerator);                        
                        
                        if(!ThreadContextObj.PreviewAttrObj.ManagedGenerator)
                        {
                            #region UnManagedGenerator
                            /* Handling of Unmanaged processing is not currently handled in the Cataloger processing changes*/
                            InvokeMediaProcessorClassObj                        = new InvokeMediaProcessorClass();

                            if (ThreadContextObj.IsBinHex)
                            {
                                InvokeMediaProcessorClassObj.SetFileAttributes(CurrentFileName, ThreadContextObj.OriginalAttrObj.TempOrgFilePath, DestinationName, DestinationFormat.ToString(), null,
                                    ThreadContextObj.PreviewAttrObj.PreviewHeight, ThreadContextObj.PreviewAttrObj.PreviewWidth, ThreadContextObj.PreviewAttrObj.PreviewDepth,
                                    ThreadContextObj.PreviewAttrObj.PreviewDPI, 0, ActualFileName);
                            }
                            else
                            {
                                InvokeMediaProcessorClassObj.SetFileAttributes(CurrentFileName, null, DestinationName, DestinationFormat.ToString(), null,
                                    ThreadContextObj.PreviewAttrObj.PreviewHeight, ThreadContextObj.PreviewAttrObj.PreviewWidth, ThreadContextObj.PreviewAttrObj.PreviewDepth,
                                    ThreadContextObj.PreviewAttrObj.PreviewDPI, 0, ActualFileName);
                            }



                            InvokeMediaProcessorClassObj.GeneratorName          = ThreadContextObj.PreviewAttrObj.PreviewGenerator;
                            InvokeMediaProcessorClassObj.ThreadTimeout          = ThreadTimeOut;
                            InvokeMediaProcessorClassObj.GeneratePreviewFile();

                            if (InvokeMediaProcessorClassObj.ReturnValue == 0)
                            {
                                ThreadContextObj.PreviewAttrObj.PreviewHeight   = InvokeMediaProcessorClassObj.Height;
                                ThreadContextObj.PreviewAttrObj.PreviewWidth    = InvokeMediaProcessorClassObj.Width;
                                ThreadContextObj.PreviewAttrObj.PreviewDepth    = InvokeMediaProcessorClassObj.Depth;
                                ThreadContextObj.PreviewAttrObj.PreviewDPI      = InvokeMediaProcessorClassObj.DPI;
                            }
                            else
                            {
                                strdict_AdditionalInfo.Clear();
                                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                                strdict_AdditionalInfo.Add("MESSAGE", InvokeMediaProcessorClassObj.ErrorMessage);
                                ErrorMessage.AppendFormat("Preview generation failed for {0} : Message : {1}", ThreadContextObj.ParentDigitalAssetName, InvokeMediaProcessorClassObj.ErrorMessage);
                                
                                if ((!String.IsNullOrEmpty(InvokeMediaProcessorClassObj.ErrorMessage)) && ((InvokeMediaProcessorClassObj.ErrorMessage.Contains("IMAGEMAGIC_NOT_INSTALLED")) || (InvokeMediaProcessorClassObj.ErrorMessage.Contains("GHOSTSCRIPT_NOT_INSTALLED")) || (InvokeMediaProcessorClassObj.ErrorMessage.Contains("OFFICEADDINS_NOT_INSTALLED"))))
                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED", ErrorMessage.ToString(), InvokeMediaProcessorClassObj.ErrorMessage, strdict_AdditionalInfo, ErrorType.ERROR);
                                else
                                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);

                                return false;
                            }
                            #endregion
                        }
                        else
                        {
                            MediaProcessorWrapperObj = new MediaProcessorWrapper(SessionID, UserID, TenantName, Language);
                            if (ThreadContextObj.IsBinHex)
                            {
                                MediaProcessorWrapperObj.SetFileAttributes(CurrentFileName, ThreadContextObj.OriginalAttrObj.TempOrgFilePath, DestinationName, DestinationFormat.ToString(), null,
                                     ThreadContextObj.PreviewAttrObj.PreviewHeight, ThreadContextObj.PreviewAttrObj.PreviewWidth, ThreadContextObj.PreviewAttrObj.PreviewDepth,
                                     ThreadContextObj.PreviewAttrObj.PreviewDPI, 0, ActualFileName, str_ExtraInformationForMediaProcessor, ThreadContextObj.MediaProcessor);
                            }
                            else
                            {
                                MediaProcessorWrapperObj.SetFileAttributes(CurrentFileName, null, DestinationName, DestinationFormat.ToString(), null,
                                    ThreadContextObj.PreviewAttrObj.PreviewHeight, ThreadContextObj.PreviewAttrObj.PreviewWidth, ThreadContextObj.PreviewAttrObj.PreviewDepth,
                                    ThreadContextObj.PreviewAttrObj.PreviewDPI, 0, ActualFileName, str_ExtraInformationForMediaProcessor,ThreadContextObj.MediaProcessor);
                            }
                            

                            MediaProcessorWrapperObj.GeneratorName = ThreadContextObj.PreviewAttrObj.PreviewGenerator;
                            MediaProcessorWrapperObj.ThreadTimeout = ThreadTimeOut;
                            //Set Processing Status in AssetInfo
                            CatalogerHelper.CatalogerHelper.SetProcessingStatusInAssetInfo(ref CatalogerContextObj, "GeneratePrvRequestSent", "True", CatalogerContextObj.AssetInformationObj.AssetNode);                           
                            MediaProcessorWrapperObj.GeneratePreviewFile();
                        }
                       
                    }
                    else
                    {
                        //ErrorMessage.AppendFormat("Could not generate preview for {0} : Message : preview generator not available.", ThreadContextObj.ParentDigitalAssetName);
                        //LogError(ErrorMessage.ToString(),null, ErrorType.ERROR);
                        //return false;
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in GenerateAssetPreview : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETPREVIEWFAILED", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }

        private Boolean GenerateAssetThumbnail( ThreadContext ThreadContextObj)
        {
            String                      DestinationFormat                       = null;
            StringBuilder               ThumbnailFileName                       = null;
            StringBuilder               ThumbnailFilePath                       = null;
            String                      ActualFileName                          = null;
            String                      DestinationName                         = null;
            String                      CurrentFileName                         = null;
            StringBuilder               TemporaryFilePath                       = null;
            InvokeMediaProcessorClass   InvokeMediaProcessorClassObj            = null;            
            StringBuilder               ErrorMessage                            = null;
            String                      DataForkName                            = null;
            MediaProcessorWrapper       MediaProcessorWrapperObj                = null;
            
            String                      str_ExtraInformationForMediaProcessor   = null;                
            RenditionType               enum_RenditionType;
            try
            {
                ErrorMessage            = new StringBuilder();
               
                if (ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator != null)
                {
                    if (ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator.Length > 0)
                    {
                        ThumbnailFileName                                       = new StringBuilder();
                        ThumbnailFilePath                                       = new StringBuilder();
                        TemporaryFilePath                                       = new StringBuilder();
                        ErrorMessage                                            = new StringBuilder();

                        DestinationFormat                                       = "JPEG";

                        ThumbnailFileName.AppendFormat("{0}.{1}", ThreadContextObj.OriginalAttrObj.OriginalFileName, DestinationFormat);
                        ThumbnailFilePath.AppendFormat("{0}\\Thumbnails\\{1}", JobPath, ThumbnailFileName.ToString());
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName = ThumbnailFileName.ToString();
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath = ThumbnailFilePath.ToString();
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat = DestinationFormat.ToString();

                        if ((ThreadContextObj.PreviewExists) && (ThreadContextObj.GenThumbFromPreview))
                        {
                            //Is Binhex processing
                            if (ThreadContextObj.IsPrvBinHex)
                                DataForkName = ThreadContextObj.PreviewAttrObj.TempPrvFilePath;

                            DestinationName                     = ThumbnailFilePath.ToString();
                            GetDestinationFileName(ThreadContextObj, DestinationFormat, RenditionType.PREVIEW, ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator, ref CurrentFileName, ref DestinationName);
                            ActualFileName                      = ThreadContextObj.PreviewAttrObj.PreviewFilePath;                            
                            enum_RenditionType                  = RenditionType.PREVIEW;
                            Is_GenerateThb_From_Preview_Call    = true;
                        }
                        else
                        {
                            //Is Binhex processing
                            if (ThreadContextObj.IsBinHex)
                                DataForkName = ThreadContextObj.OriginalAttrObj.TempOrgFilePath;

                            DestinationName = ThumbnailFilePath.ToString();
                            GetDestinationFileName(ThreadContextObj, DestinationFormat, RenditionType.ORIGINAL, ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator, ref CurrentFileName, ref DestinationName);
                            ActualFileName     = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                            enum_RenditionType = RenditionType.ORIGINAL;
                        }                        
                                                
                       
                        str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, enum_RenditionType, ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator); 
                        if(!ThreadContextObj.ThumbnailAttrObj.ManagedGenerator)
                        {
                            /* Handling of Unmanaged processing is not currently handled in the Cataloger processing changes*/
                            InvokeMediaProcessorClassObj                            = new InvokeMediaProcessorClass();
                            InvokeMediaProcessorClassObj.SetFileAttributes(CurrentFileName, DataForkName, DestinationName, DestinationFormat.ToString(), "THUMBNAIL",
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight, ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth, ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth,
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI, ThreadContextObj.ThumbnailAttrObj.ThumbnailQuality, ActualFileName);
                            InvokeMediaProcessorClassObj.GeneratorName              = ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator;
                            InvokeMediaProcessorClassObj.ThreadTimeout              = ThreadTimeOut;
                            InvokeMediaProcessorClassObj.GenerateThumbnailFile();

                            if (InvokeMediaProcessorClassObj.ReturnValue == 0)
                            {
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight   = InvokeMediaProcessorClassObj.Height;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth    = InvokeMediaProcessorClassObj.Width;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth    = InvokeMediaProcessorClassObj.Depth;
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI      = InvokeMediaProcessorClassObj.DPI;
                                //ReturnValue = true;
                            }
                            else
                            {
                                //ReturnValue = false;
                            }

                        }
                        else
                        {
                            MediaProcessorWrapperObj                            = new MediaProcessorWrapper(SessionID,UserID,TenantName,Language);
                            MediaProcessorWrapperObj.SetFileAttributes(CurrentFileName, DataForkName, DestinationName, DestinationFormat.ToString(), "THUMBNAIL",
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight, ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth, ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth,
                                ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI, ThreadContextObj.ThumbnailAttrObj.ThumbnailQuality, ActualFileName, str_ExtraInformationForMediaProcessor);
                            MediaProcessorWrapperObj.GeneratorName              = ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator;
                            MediaProcessorWrapperObj.ThreadTimeout              = ThreadTimeOut;
                            //Set Processing Status in AssetInfo
                            CatalogerHelper.CatalogerHelper.SetProcessingStatusInAssetInfo(ref CatalogerContextObj, "GenerateThbRequestSent", "True", CatalogerContextObj.AssetInformationObj.AssetNode);                                                       
                            MediaProcessorWrapperObj.GenerateThumbnailFile();                           
                        }    
                    }
                    else
                    {
                        //ErrorMessage.AppendFormat("Could not generate thumbnail for {0} : Message : Thumbnail generator not available.", ThreadContextObj.ParentDigitalAssetName);
                        //LogError(ErrorMessage.ToString(), null,ErrorType.ERROR);
                        //return false;
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                ErrorMessage.AppendFormat("Exception thrown in GenerateAssetThumbnail : Message : {0}", ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETTHUMBNAILFAILED", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }
        
        /*public Boolean GenerateUniqueID(ref ulong UniqueID)
        {
           String  GUID = null;
           try
            {
                lock (Lock)
                {
                    GUID        = Guid.NewGuid().ToString();
                    GUID        = GUID.Remove(0, 24);
                    UniqueID    = Convert.ToUInt64(GUID, 16);
                }
            }
            catch (Exception )
            {
                return false;
            }

            return true;
        }*/

        private Boolean GetThumbnailAttributes(String DestinationName,  ThreadContext ThreadContextObj)
        {
            StringBuilder               XPathQuery                      = null;
            XmlNode                     FileFormatNode                  = null;
            StringBuilder               ErrorMessage                    = null;
            String                      PartialQuery                    = null;
            String                      JpegExt                         = "JPEG";
            InvokeMediaProcessorClass   InvokeMediaProcessorClassObj    = null;
            MediaProcessorWrapper       MediaProcessorWrapperObj        = null; 
            Boolean                     ManagedGenerator                = false;
            String                      str_ExtraInformationForMediaProcessor = null;

            try
            {
                if (!ThreadContextObj.ThumbnailExists)
                {
                    XPathQuery                      = new StringBuilder();
                    ErrorMessage                    = new StringBuilder();
                    
                    if (FileFormatXMLDoc.SelectSingleNode("//FILEFORMAT") == null)
                    {
                        ErrorMessage.Append("FileFormats not found.");
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FILEFORMATSNOTFOUND",ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                        return false;
                    }

                    PartialQuery        = "//FILEFORMAT[FILEEXTENSION[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')=";
                    XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, JpegExt.ToLower());
                    if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                    {
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat   = FileFormatNode.SelectSingleNode("FILEFORMAT").InnerText;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator    = FileFormatNode.SelectSingleNode("THUMBNAILGENERATOR").InnerText;
                        ThreadContextObj.MediaProcessor                         = FileFormatNode.SelectSingleNode("APPLICATIONTYPE").InnerText;

                        if (FileFormatNode != null)
                        {
                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    ManagedGenerator = true;
                                else
                                    ManagedGenerator = false;
                            }
                        }
                    }

                    if (!ManagedGenerator)
                    {
                        InvokeMediaProcessorClassObj                                = new InvokeMediaProcessorClass();

                        InvokeMediaProcessorClassObj.SetFileAttributes(DestinationName, null,
                        null, null, null, 0, 0, 0, 0, 0, null);
                        
                        InvokeMediaProcessorClassObj.GeneratorName                  = ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator;
                        InvokeMediaProcessorClassObj.ThreadTimeout                  = ThreadTimeOut;
                        InvokeMediaProcessorClassObj.GetInfo();

                        ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight           = InvokeMediaProcessorClassObj.Height;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth            = InvokeMediaProcessorClassObj.Width;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth            = InvokeMediaProcessorClassObj.Depth;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI              = InvokeMediaProcessorClassObj.DPI;
                    }
                    else
                    {
                        switch (ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator)
                        {
                            case "IllustratorCS3Wrapper.dll":
                            case "MSOfficeProcessorWrapper.dll":
                            case "CatalogerGenericQWrapper.dll":
                            case "FLVWrapper.dll":
                                {
                                    str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.THUMBNAIL, ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator);
                                } break;
                        }

                        MediaProcessorWrapperObj                                    = new MediaProcessorWrapper(SessionID,UserID,TenantName,Language);
                        MediaProcessorWrapperObj.SetFileAttributes(DestinationName, null,
                        null, null, null, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                        
                        MediaProcessorWrapperObj.GeneratorName                      = ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator;
                        MediaProcessorWrapperObj.ThreadTimeout                      = ThreadTimeOut;
                        MediaProcessorWrapperObj.GetInfo();

                        ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight           = MediaProcessorWrapperObj.Height;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth            = MediaProcessorWrapperObj.Width;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth            = MediaProcessorWrapperObj.Depth;
                        ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI              = MediaProcessorWrapperObj.DPI;
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean CreateDigitalAsset(ThreadContext ThreadContextObj,Boolean LastChild)
        {
            ERRORXML                    ErrorXML                        = null;
            XmlDocument                 DigitalAssetDoc                 = null;
            XmlNode                     DigitalAssetNode                = null;
            XML                         DigitalAssetXML                 = null;
            Boolean                     Success                         = false;
            XmlNode                     CRRenditionRelNode              = null;
            XmlNode                     ParentNode                      = null;
            StringBuilder               ErrorMessage                    = null;
            XmlNodeList                 AssetRenditionNodes             = null;
            String                      DigitalAssetName                = null;
            StringBuilder               XPath                           = null;
            String                      DigitalAssetID                  = null;
            bool                        CatalogJobProcessingDetailsPopulated = false;
                                                
            try
            {
                ErrorMessage            = new StringBuilder();
                XPath                   = new StringBuilder();
              
                Success             = GenerateAssetRenditionNode( ThreadContextObj, RenditionType.ORIGINAL);
                if (!Success)
                {
                    strdict_AdditionalInfo.Clear();
                    strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                    ErrorMessage.AppendFormat("DigitalAsset creation failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                    LogError("CATALOGJOBPROCESSOR_JOBHANDLER_CREATEDIGITALASSETFAILED", ErrorMessage.ToString(), null, strdict_AdditionalInfo, ErrorType.ERROR);
                    return false;
                }

                if (ErrorXML == null)
                {
                    if (ThreadContextObj.PreviewExists)
                    {
                        Success = GenerateAssetRenditionNode( ThreadContextObj, RenditionType.PREVIEW);
                        if (!Success)
                        {
                            DigitalAssetDoc     = ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument;
                            DigitalAssetNode    = ThreadContextObj.ClonedDigitalAssetNode;
                            CRRenditionRelNode  = DigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION[ASSETRENDITION/RENDITIONTYPE=PREVIEW]");
                            if (CRRenditionRelNode != null)
                            {
                                ParentNode      = CRRenditionRelNode.ParentNode;
                                ParentNode.RemoveChild(CRRenditionRelNode);
                            }
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            ErrorMessage.AppendFormat("Preview creation failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_PREVIEWGENERATIONFAILED", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                            return false;
                        }
                    }
                    else if (ThreadContextObj.UseOriginalAsPreview)
                    {
                        ThreadContextObj.PreviewAttrObj.PreviewFileName         = ThreadContextObj.OriginalAttrObj.OriginalFileName;
                        ThreadContextObj.PreviewAttrObj.PreviewFilePath         = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                        ThreadContextObj.PreviewAttrObj.PreviewSize             = ThreadContextObj.OriginalAttrObj.OriginalSize;
                        ThreadContextObj.PreviewAttrObj.PreviewFileExtension    = ThreadContextObj.OriginalAttrObj.OriginalFileExtension;
                        ThreadContextObj.PreviewAttrObj.PreviewFileFormat       = ThreadContextObj.OriginalAttrObj.OriginalFileFormat;                                                
                        ThreadContextObj.PreviewAttrObj.PrvMacCreator           = ThreadContextObj.OriginalAttrObj.OrgMacCreator;
                        ThreadContextObj.PreviewAttrObj.PrvMacType              = ThreadContextObj.OriginalAttrObj.OrgMacType;
                        ThreadContextObj.PreviewAttrObj.TempPrvFilePath         = ThreadContextObj.OriginalAttrObj.TempOrgFilePath;

                        Success = GenerateAssetRenditionNode( ThreadContextObj, RenditionType.PREVIEW);
                        if (!Success)
                        {
                            DigitalAssetDoc     = ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument;
                            DigitalAssetNode    = ThreadContextObj.ClonedDigitalAssetNode;
                            CRRenditionRelNode  = DigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION[ASSETRENDITION/RENDITIONTYPE=PREVIEW]");
                            if (CRRenditionRelNode != null)
                            {
                                ParentNode      = CRRenditionRelNode.ParentNode;
                                ParentNode.RemoveChild(CRRenditionRelNode);
                            }
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            ErrorMessage.AppendFormat("Setting Original as Preview failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_SETTINGORGASPRVFAILED", ErrorMessage.ToString(), null,strdict_AdditionalInfo, ErrorType.ERROR);
                            return false;
                        }
                    }

                    if (ThreadContextObj.ThumbnailExists)
                    {
                        Success = GenerateAssetRenditionNode( ThreadContextObj, RenditionType.THUMBNAIL);
                        if (!Success)
                        {
                            DigitalAssetDoc     = ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument;
                            DigitalAssetNode    = ThreadContextObj.ClonedDigitalAssetNode;
                            CRRenditionRelNode  = DigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION[ASSETRENDITION/RENDITIONTYPE=THUMBNAIL]");
                            if (CRRenditionRelNode != null)
                            {
                                ParentNode      = CRRenditionRelNode.ParentNode;
                                ParentNode.RemoveChild(CRRenditionRelNode);
                            }
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            ErrorMessage.AppendFormat("Thumbnail creation failed for {0}", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_THUMBNAILGENERATIONFAILED",ErrorMessage.ToString(),null,strdict_AdditionalInfo, ErrorType.ERROR);
                            return false;
                        }
                    }
                }

                
                DigitalAssetDoc                 = ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument;
                DigitalAssetNode                = ThreadContextObj.ClonedDigitalAssetNode;

                if (!CatalogerAndOMTogether)
                {

                    AssetRenditionNodes         = DigitalAssetNode.SelectNodes("DIGITALASSET_CONTENTREVISION/descendant::ASSETRENDITION");
                    foreach (XmlNode AssetRenditionNode in AssetRenditionNodes)
                    {
                        TransferFileAndUpdatePath(AssetRenditionNode);
                    }
                }

                if (LastChild)
                {
                    IsBusinessInfoValid = true;
                    DigitalAssetXML     = DigitalAssetNode.OuterXml;
                    ErrorXML            = DAMWSObj.CreateDigitalAsset(EnableVersoning, ref DigitalAssetXML);
                    
                    //Pravin
                    //Date: 28 Jan 2008
                    #region Check for BusinessInformation validation error
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        XPath.Remove(0, XPath.Length);
                        XPath.Append("//ERRORCODE[text()='DAMENGINE_DIGITALASSET_FACETINFOVALIDATIONFAILED']");

                        XPathDocument xpDoc_Error = new XPathDocument(new StringReader(ErrorXML));
                        XPathNavigator xpNav_Error = xpDoc_Error.CreateNavigator();
                        if (xpNav_Error.SelectSingleNode(XPath.ToString())!= null)
                        {
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.OriginalAttrObj.OriginalFileName);
                            strdict_AdditionalInfo.Add("TARGETSECTIONPATH", ThreadContextObj.str_SectionPath);
                            ErrorMessage.AppendFormat("Business information for DigitalAsset {0} in Section: {1} is not valid.", ThreadContextObj.ParentDigitalAssetName, ThreadContextObj.str_SectionPath);
                            LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FACETINFOVALIDATIONFAILED", ErrorMessage.ToString(), ErrorXML,strdict_AdditionalInfo, ErrorType.ERROR);
                            IsBusinessInfoValid = false;                            
                            ErrorXML            = null;
                        }
                    }
                    #endregion
                    //End - Pravin.

                    if (ErrorXML == null)
                    {
                        DigitalAssetDoc                                     = new XmlDocument();
                        DigitalAssetDoc.LoadXml(DigitalAssetXML);
                        ThreadContextObj.ClonedDigitalAssetNode.InnerXml    = DigitalAssetDoc.DocumentElement.InnerXml;

                        XPath.Remove(0, XPath.Length);
                        DigitalAssetName = ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("NAME").InnerText;
                        XPath.AppendFormat("//DIGITALASSET[NAME={0}]",ALXmlUtil.GetXpathValueString(DigitalAssetName));
                        DigitalAssetNode = ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument.SelectSingleNode(XPath.ToString());

                        if (DigitalAssetNode != null)
                        {
                            // Set the Cataloged attribute of the DigitalAsset to true.
                            ((XmlElement)DigitalAssetNode).SetAttribute("Cataloged", "TRUE");
                            ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument.Save(AssetInfoPath);

                            ENTITYID AssetID = 0;
                            DigitalAssetID                      = ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("DIGITALASSETID").InnerText;                            
                            ulong.TryParse(DigitalAssetID, out AssetID);
                            ThreadContextObj.DigitalAssetID     = AssetID;
                            if (ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISIONID") != null)
                                ThreadContextObj.ContentRevisionID  = ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISIONID").InnerText;

                            if (UpdateCatalogJobProcessingDetails(ThreadContextObj, DigitalAssetID, true) == false)
                                return false;
                            else
                                CatalogJobProcessingDetailsPopulated = true;
                        }

                    }
                    else
                    {
                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                        ErrorMessage.AppendFormat("DigitalAsset creation failed for {0}", ThreadContextObj.ParentDigitalAssetName);
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_CREATEDIGITALASSETFAILED",ErrorMessage.ToString(), ErrorXML,strdict_AdditionalInfo, ErrorType.ERROR);

                        DigitalAssetID = "";
                        if (UpdateCatalogJobProcessingDetails(ThreadContextObj, DigitalAssetID, false) == false)
                            return false;
                        else
                            CatalogJobProcessingDetailsPopulated = true;

                        IsProcessingComplete    = true;
                        isJobSuccessed          = false;
                        return false;
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                ErrorMessage.AppendFormat("Exception thrown in CreateDigitalAsset for {0} : Message : {1}",ThreadContextObj.ParentDigitalAssetName, ExceptionObj.Message);
                LogError("CATALOGJOBPROCESSOR_JOBHANDLER_CREATEDIGITALASSETFAILED",ErrorMessage.ToString(),null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
            finally
            {
                if (CatalogJobProcessingDetailsPopulated == false)
                {
                    if (String.IsNullOrEmpty(DigitalAssetID))
                        DigitalAssetID = "";
                    if (UpdateCatalogJobProcessingDetails(ThreadContextObj, DigitalAssetID, false) == false)
                    {
                        CatalogerHelper.CatalogerHelper.WriteLogToNTFS("Error occured in UpdateCatalogJobProcessingDetails for Digital Asset - " + ThreadContextObj.ParentDigitalAssetName, null);
                    }
                }
            }
        }

        private Boolean TransferFileAndUpdatePath(XmlNode AssetRenditionNode)
        {
            FileTransferInfrastructure  FileTransferInfrastructureObj   = null;
            XML                         FileListXML                     = null;
            XmlNode                     FileCollectionNode              = null;
            XmlDocument                 FileTransferXMLDoc              = null;
            ERRORXML                    ErrorXML                        = null;
            StringBuilder               ErrorMessage                    = null;
            XmlNode                     FileNode                        = null;
            try
            {
                FileNode = AssetRenditionNode.SelectSingleNode("FILESYSTEM/FILE");
                if (FileNode != null)
                {
                    //If Destination loaction for file Media Server then don't transfer the
                    //file to services Machine. (Since the will copied form catalger mount point
                    //location to vault using Media server at OM Level.
                    if (FileNode.SelectSingleNode("ONMEDIASERVER") != null)
                    {
                        if (Boolean.Parse(FileNode.SelectSingleNode("ONMEDIASERVER").InnerText))
                            return true; //No need to Physically Transefer the File
                    }

                    FileTransferXMLDoc              = new XmlDocument();
                    ErrorMessage                    = new StringBuilder();
                    FileTransferInfrastructureObj   = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                    FileCollectionNode              = FileTransferXMLDoc.CreateElement("FILECOLLECTION");
                    FileTransferInfrastructureObj   = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                   
                    FileCollectionNode.InnerXml = FileNode.OuterXml;
                    FileListXML                 = FileCollectionNode.OuterXml;
                    if ((ErrorXML = FileTransferInfrastructureObj.SaveFile(ref FileListXML)) == null)
                    {
                        FileCollectionNode.InnerXml = FileListXML;
                        AssetRenditionNode.SelectSingleNode("FILESYSTEM").InnerXml = FileCollectionNode.SelectSingleNode("FILECOLLECTION").InnerXml;
                    }
                    else
                    {
                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("FILENAME", FileNode.SelectSingleNode("NAME").InnerText);
                        ErrorMessage.Remove(0,ErrorMessage.Length);
                        ErrorMessage.AppendFormat("Failed to transfer file {0}",AssetRenditionNode.SelectSingleNode("FILESYSTEM/FILE/NAME").InnerText);
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FAILEDTOTRANSFERFILE", ErrorMessage.ToString(), ErrorXML, strdict_AdditionalInfo, ErrorType.ERROR);
                    }
                }
                return true;

            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean GenerateAssetRenditionNode( ThreadContext ThreadContextObj,RenditionType RenditionTypeValue)
        {
            XmlDocument     DigitalAssetDoc             = null; 
            XmlNode         DAContentRevisionRelNode    = null;
            XmlNode         CRRenditionRelNode          = null;
            XmlNode         DigitalAssetNode            = null;
            XmlNode         ColoumnNode                 = null;
            XmlNode         ContentRevisionNode         = null;
            XmlNode         RenditionNode               = null;
            XmlNode         FileSystemNode              = null;
            XmlNode         FileNode                    = null;
            StringBuilder   ErrorMessage                = null;
            XmlNode         DigitalAssetSectionRelNode  = null;
            XmlNode         MarkedAsDeletedNode         = null;
            String          ErrorCode                   = null;

            try
            {
                ErrorMessage                = new StringBuilder();
                #region Generate Rendition node 

                DigitalAssetDoc             = ThreadContextObj.ClonedDigitalAssetNode.OwnerDocument;
                DigitalAssetNode            = ThreadContextObj.ClonedDigitalAssetNode;

                if (DigitalAssetNode.SelectSingleNode("MARKEDASDELETED") == null)
                {
                    MarkedAsDeletedNode             = DigitalAssetDoc.CreateElement("MARKEDASDELETED");
                    MarkedAsDeletedNode.InnerText   = "0";
                    DigitalAssetNode.AppendChild(MarkedAsDeletedNode);
                }

                if (DigitalAssetNode.SelectSingleNode("SECTION_DIGITALASSET") == null)
                {
                    DigitalAssetSectionRelNode              = DigitalAssetDoc.CreateElement("SECTION_DIGITALASSET");
                    ((XmlElement)DigitalAssetSectionRelNode).SetAttribute("TYPE", "RELATION");
                    ((XmlElement)DigitalAssetSectionRelNode).SetAttribute("PARENTID", "TRUE");
                    DigitalAssetSectionRelNode.InnerText    = ThreadContextObj.SectionID.ToString();
                    DigitalAssetNode.AppendChild(DigitalAssetSectionRelNode);
                }

                if (DigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION") == null)
                {
                    DAContentRevisionRelNode    = DigitalAssetDoc.CreateElement("DIGITALASSET_CONTENTREVISION");
                    ((XmlElement)DAContentRevisionRelNode).SetAttribute("TYPE", "RELATION");

                    ContentRevisionNode         = DigitalAssetDoc.CreateElement("CONTENTREVISION");
                    DigitalAssetNode.AppendChild(DAContentRevisionRelNode);
                    DAContentRevisionRelNode.AppendChild(ContentRevisionNode);

                    CRRenditionRelNode          = DigitalAssetDoc.CreateElement("CONTENTREVISION_RENDITION");
                    ((XmlElement)CRRenditionRelNode).SetAttribute("TYPE", "RELATION");
                    ContentRevisionNode.AppendChild(CRRenditionRelNode);
                }
                else
                {
                    DAContentRevisionRelNode    = DigitalAssetNode.SelectSingleNode("DIGITALASSET_CONTENTREVISION");
                    ContentRevisionNode         = DAContentRevisionRelNode.SelectSingleNode("CONTENTREVISION");
                    CRRenditionRelNode          = ContentRevisionNode.SelectSingleNode("CONTENTREVISION_RENDITION");
                }
        
                RenditionNode               = DigitalAssetDoc.CreateElement("ASSETRENDITION");
                CRRenditionRelNode.AppendChild(RenditionNode);  
             
                ColoumnNode                 = DigitalAssetDoc.CreateElement("NAME");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = ThreadContextObj.OriginalAttrObj.OriginalFileName;
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = ThreadContextObj.PreviewAttrObj.PreviewFileName;
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName;
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("RENDITIONTYPE");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = "Original";
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = "Preview";
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = "Thumbnail";
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode = DigitalAssetDoc.CreateElement("FILEFORMAT");
                if (RenditionTypeValue == RenditionType.ORIGINAL)
                   ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalFileFormat;
                else if (RenditionTypeValue == RenditionType.PREVIEW)
                   ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewFileFormat;
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                   ColoumnNode.InnerText = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat;

                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("FILESIZE");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = ThreadContextObj.OriginalAttrObj.OriginalSize.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = ThreadContextObj.PreviewAttrObj.PreviewSize.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailSize.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("DPI");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalDPI.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewDPI.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailDPI.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("WIDTH");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalWidth.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewWidth.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailWidth.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("HEIGHT");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalHeight.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewHeight.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailHeight.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("DEPTH");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText = ThreadContextObj.OriginalAttrObj.OriginalDepth.ToString();
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText = ThreadContextObj.PreviewAttrObj.PreviewDepth.ToString();
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailDepth.ToString();
                RenditionNode.AppendChild(ColoumnNode);

                FileSystemNode              = DigitalAssetDoc.CreateElement("FILESYSTEM");
                FileNode                    = DigitalAssetDoc.CreateElement("FILE");
                
                ColoumnNode                 = DigitalAssetDoc.CreateElement("NAME");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = ThreadContextObj.OriginalAttrObj.OriginalFileName;
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = ThreadContextObj.PreviewAttrObj.PreviewFileName;
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName;
                FileNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("PATH");
                if(RenditionTypeValue == RenditionType.ORIGINAL)
                    ColoumnNode.InnerText   = ThreadContextObj.OriginalAttrObj.OriginalFilePath;
                else if(RenditionTypeValue == RenditionType.PREVIEW)
                    ColoumnNode.InnerText   = ThreadContextObj.PreviewAttrObj.PreviewFilePath;
                else if (RenditionTypeValue == RenditionType.THUMBNAIL)
                    ColoumnNode.InnerText   = ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath;
                FileNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("FIELDNAME");
                ColoumnNode.InnerText       = "ACTUALFILE";
                FileNode.AppendChild(ColoumnNode);

                ColoumnNode                 = DigitalAssetDoc.CreateElement("ONMEDIASERVER");
                ColoumnNode.InnerText       = ThreadContextObj.OnMediaServer.ToString().ToUpper();
                FileNode.AppendChild(ColoumnNode);

                FileSystemNode.AppendChild(FileNode);
                RenditionNode.AppendChild(FileSystemNode);
                

                #endregion
                return true;
            }
            catch (Exception ExceptionObj)
            {
                if (RenditionTypeValue == RenditionType.ORIGINAL)
                {
                    ErrorMessage.AppendFormat("Exception thrown in GenerateAssetRenditionNode for ORIGINAL : Message : {0}", ExceptionObj.Message);
                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETRENDITIONNODEFORORGFAILED";
                }
                if (RenditionTypeValue == RenditionType.PREVIEW)
                {
                    ErrorMessage.AppendFormat("Exception thrown in GenerateAssetRenditionNode for PREVIEW : Message : {0}", ExceptionObj.Message);
                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETRENDITIONNODEFORPREVFAILED";
                }
                if (RenditionTypeValue == RenditionType.THUMBNAIL)
                {
                    ErrorMessage.AppendFormat("Exception thrown in GenerateAssetRenditionNode for THUMBNAIL : Message : {0}", ExceptionObj.Message);
                    ErrorCode = "CATALOGJOBPROCESSOR_JOBHANDLER_GENERATEASSETRENDITIONNODEFORTHUMBFAILED";
                }

                strdict_AdditionalInfo.Clear();
                strdict_AdditionalInfo.Add("EXCEPTIONMESSAGE", ExceptionObj.Message);
                LogError(ErrorCode,ErrorMessage.ToString(),null,strdict_AdditionalInfo, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
        }

        private Boolean MarkJobAsError( XmlNode JobNode)
        {
            StringBuilder ErrorMessage = null;
            try
            {
                ErrorMessage                                        = new StringBuilder();
                if (JobNode.SelectSingleNode("STATUS").InnerText != "Error")
                {
                    JobNode.SelectSingleNode("STATUS").InnerText    = "Error";
                    if (!UpdateCatalogJob(JobNode))
                    {
                        ErrorMessage.Append("Marking job status as Error failed.");
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_MARKINGJOBSTATUSASERRORFAILED", ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }

        private Boolean InvokePlugin(String CodePoint,CatalogerContext CatalogerContextObj)
        {
            Assembly            PluginAssembly          = null;
            Type[]              Types                   = null;
            Type                ClassType               = null;
            Object              ClassInstance           = null;
            XPathNavigator      XPathNavigatorObj       = null;
            XPathNodeIterator   PluginNodes             = null; 
            XPathNavigator      PluginNode              = null;
            StringBuilder       XPath                   = null;
            XPathExpression     XPathExpressionObj      = null;
            XPathNavigator      CodePointNode           = null;
            String              MethodeToInvoke         = null;
            String              ClassName               = null;
          
            try
            {
                XPath                       = new StringBuilder();
                if (PluginsXMLDoc != null)
                {
					XPathNavigatorObj       = PluginsXMLDoc.CreateNavigator();
                    XPath.AppendFormat("CODEPOINTCOLLECTION/CODEPOINT[(translate(@Name, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='{0}')]", CodePoint.ToUpper());
                    XPathExpressionObj      = XPathExpression.Compile(XPath.ToString());

                    if ((CodePointNode      = XPathNavigatorObj.SelectSingleNode(XPathExpressionObj)) != null)
                    {
                        PluginNodes         = CodePointNode.Select("PLUGIN");
                        while (PluginNodes.MoveNext())
                        {
                            PluginNode      = PluginNodes.Current; ;

                            //Load the Database Assembly
                            PluginAssembly  = Assembly.LoadFrom(PluginNode.SelectSingleNode("DLLPATH").Value);
                            MethodeToInvoke = PluginNode.SelectSingleNode("METHODTOINVOKE").Value;
                            Types           = PluginAssembly.GetTypes();

                            ClassName       = PluginNode.SelectSingleNode("TYPENAME").Value;
                            foreach (Type assmType in Types)
                            {
                                if (assmType.Name.ToUpper() == ClassName.ToUpper())
                                {
                                    ClassType = assmType;
                                    break;
                                }
                            }

                            if (ClassType != null)
                            {
                                ClassInstance = Activator.CreateInstance(ClassType);

                                //Invoke InitializeDatabaseSetup method of the Database Library
                                ClassInstance.GetType().InvokeMember(MethodeToInvoke,
                                                       BindingFlags.InvokeMethod, null, ClassInstance,
                                                       new Object[] { CodePoint, CatalogerContextObj });
                            }
                        }
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				throw;
            }
        }

        private Boolean GetPreviewFormatFromID(String PreviewFormatID,ref String PreviewFormat)
        {
            StringBuilder XPathQuery        = null;
            XmlNode       FileFormatNode    = null;
            try
            {
                XPathQuery = new StringBuilder();
                XPathQuery.AppendFormat("//FILEFORMAT[FILEFORMATID={0}]",PreviewFormatID);
                if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                {
                    PreviewFormat = FileFormatNode.SelectSingleNode("FILEEXTENSION").InnerText;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
            return true;
        }

        public static int GetWebServiceTimeout()
        {
            int WebserviceTimeout = 0;

            if (Int32.TryParse(ConfigurationManager.AppSettings["WebServiceTimeout"], out WebserviceTimeout))
                return WebserviceTimeout;
            else
                return 3600000;
        }

        private String GenerateExtraInformationForMediaProcessor(ThreadContext ThreadContextObj, RenditionType enum_RenditionType, String GeneratorName)
        {
            StringBuilder   sb_ExtraInformationForMediaProcessor    = null;
            String          str_OutputFolderPath                    = null;
            //String          str_FileFormat                          = null;
            String          str_MediaProcessor                      = null;
            String          str_UpdateDetailsQPath                  = null;
            String          str_JobProcessingDetails                = null;
            String          ApplicationNameinMediaRequestRouter     = null;
			sb_ExtraInformationForMediaProcessor = new StringBuilder(500);
            try
			{
            ////if (enum_RenditionType == RenditionType.PREVIEW)
            ////    str_FileFormat = ThreadContextObj.PreviewAttrObj.PreviewFileFormat.ToUpper();
            ////else
            ////    str_FileFormat = ThreadContextObj.OriginalAttrObj.OriginalFileFormat.ToUpper();

            str_UpdateDetailsQPath      = (String)ConfigurationManager.AppSettings["UpdateAssetProcessJobQueue"];
            str_JobProcessingDetails    = GetJobDetails(ThreadContextObj);

            if (!String.IsNullOrEmpty(GeneratorName))
            {
                switch (GeneratorName)
                {
                    case "MSOfficeProcessorWrapper.dll":
                    case "CatalogerGenericQWrapper.dll":
                    case "FLVWrapper.dll":
                        {
                            if (IsFilePathContainsSpecialCharacters(JobPath))
                            {
                                str_OutputFolderPath = MountPoint + "\\Temp";
                            }
                            else
                            {
                                str_OutputFolderPath = JobPath + "\\Temp";
                            }

                            if (!Directory.Exists(str_OutputFolderPath))
                                Directory.CreateDirectory(str_OutputFolderPath);

                            sb_ExtraInformationForMediaProcessor.Append("<EXTRAINFORMATION>");
                            sb_ExtraInformationForMediaProcessor.Append("<REQUESTGENERATOR>CATALOGER</REQUESTGENERATOR>");
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<OUTPUTFOLDERPATH>{0}</OUTPUTFOLDERPATH>", ALCommon.TrimAndReplaceSpecialChars(str_OutputFolderPath));
                            switch (GeneratorName)
                            {
                                    case "MSOfficeProcessorWrapper.dll":
                                     {
                                         str_MediaProcessor = "MSOFFICEPROCESSOR";
                                     } break;
                                    case "CatalogerGenericQWrapper.dll":
                                     {
                                         str_MediaProcessor = "IMAGEPROCESSOR";
                                         
                                     }break;
                                    case "FLVWrapper.dll":
                                     {
                                        str_MediaProcessor = "FLVGENERATOR";
                                        if (String.Compare(ThreadContextObj.MediaProcessor, "ffmpeg.exe", true) == 0)
                                            ApplicationNameinMediaRequestRouter = "ffmpeg";
                                        else
                                            ApplicationNameinMediaRequestRouter = "mencoder";
                                     }break;
                             }
                            
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<ISMEDIAPROCESSORONSAMENETWORK>{0}</ISMEDIAPROCESSORONSAMENETWORK>", IsMediaProcessorOnSameNetwork(str_MediaProcessor));

                            if (String.Compare(str_MediaProcessor, "FLVGENERATOR", true) == 0)
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<APPLICATIONTYPE>{0}</APPLICATIONTYPE>", ALCommon.TrimAndReplaceSpecialChars(ApplicationNameinMediaRequestRouter));
                            else
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<APPLICATIONTYPE>{0}</APPLICATIONTYPE>", ALCommon.TrimAndReplaceSpecialChars(str_MediaProcessor));

                            sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBID>{0}</JOBID>", CatalogJobID);
                            if (ThreadContextObj.JobNode.SelectSingleNode("JOBNAME") != null)
                                sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERUNCPATH_WITHJOBNAME>{0}\\{1}\\{2}</CATALOGERUNCPATH_WITHJOBNAME>", ALCommon.TrimAndReplaceSpecialChars(CatalogerMountPointUNCPath), LoginName, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.JobNode.SelectSingleNode("JOBNAME").InnerText));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERUNCPATH>{0}</CATALOGERUNCPATH>", ALCommon.TrimAndReplaceSpecialChars(CatalogerMountPointUNCPath));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBPATH>{0}</JOBPATH>", ALCommon.TrimAndReplaceSpecialChars(JobPath));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<DIGITALASSETID>{0}</DIGITALASSETID>", ThreadContextObj.DigitalAssetID.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERANDOMTOGETHER>{0}</CATALOGERANDOMTOGETHER>", CatalogerAndOMTogether.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<UPDATEPROCESSQPATH>{0}</UPDATEPROCESSQPATH>", str_UpdateDetailsQPath);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<RENDITIONTYPE>{0}</RENDITIONTYPE>", enum_RenditionType.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<GEN_THB_FRM_PRV_CALL>{0}</GEN_THB_FRM_PRV_CALL>", Is_GenerateThb_From_Preview_Call.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<MOUNTPOINT>{0}</MOUNTPOINT>", ALCommon.TrimAndReplaceSpecialChars(MountPoint));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBPROCESSINGDETAILS>{0}</JOBPROCESSINGDETAILS>", str_JobProcessingDetails);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<LOGINNAME>{0}</LOGINNAME>", LoginName);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<NETWORKFOLDERPATH>{0}</NETWORKFOLDERPATH>", NetworkFolderPath);                            
                            sb_ExtraInformationForMediaProcessor.Append("</EXTRAINFORMATION>");
                        } break;

                    case "IllustratorCS3Wrapper.dll":
                        {
                            if (IsFilePathContainsSpecialCharacters(JobPath))
                            {
                                str_OutputFolderPath = MountPoint + "\\Temp";
                            }
                            else
                            {
                                str_OutputFolderPath = JobPath + "\\Temp";
                            }
                            
                            if (!Directory.Exists(str_OutputFolderPath))
                                Directory.CreateDirectory(str_OutputFolderPath);

                            sb_ExtraInformationForMediaProcessor.Append("<EXTRAINFORMATION>");
                            sb_ExtraInformationForMediaProcessor.Append("<REQUESTGENERATOR>CATALOGER</REQUESTGENERATOR>");
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<OUTPUTFOLDERPATH>{0}</OUTPUTFOLDERPATH>", ALCommon.TrimAndReplaceSpecialChars(str_OutputFolderPath));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERINSTALLED>{0}</CATALOGERINSTALLED>", ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerInstalled"));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<WEBSERVICESINSTALLED>{0}</WEBSERVICESINSTALLED>", ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "WebServicesInstalled"));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<ISMEDIAPROCESSORONSAMENETWORK>{0}</ISMEDIAPROCESSORONSAMENETWORK>", ConfigurationManager.AppSettings["ISMEDIAPROCESSORONSAMENETWORK"]);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<THUMBNAILEXISTS>{0}</THUMBNAILEXISTS>", ThreadContextObj.ThumbnailExists.ToString().ToLower());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<MOUNTPOINT>{0}</MOUNTPOINT>", ALCommon.TrimAndReplaceSpecialChars(MountPoint));
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<DIGITALASSETID>{0}</DIGITALASSETID>", ThreadContextObj.DigitalAssetID.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<CATALOGERANDOMTOGETHER>{0}</CATALOGERANDOMTOGETHER>", CatalogerAndOMTogether.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<UPDATEPROCESSQPATH>{0}</UPDATEPROCESSQPATH>", str_UpdateDetailsQPath);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<RENDITIONTYPE>{0}</RENDITIONTYPE>", enum_RenditionType.ToString());
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<GEN_THB_FRM_PRV_CALL>{0}</GEN_THB_FRM_PRV_CALL>", Is_GenerateThb_From_Preview_Call.ToString());                            
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<JOBPROCESSINGDETAILS>{0}</JOBPROCESSINGDETAILS>", str_JobProcessingDetails);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<LOGINNAME>{0}</LOGINNAME>", LoginName);
                            sb_ExtraInformationForMediaProcessor.AppendFormat("<NETWORKFOLDERPATH>{0}</NETWORKFOLDERPATH>", NetworkFolderPath);                            
                            sb_ExtraInformationForMediaProcessor.Append("</EXTRAINFORMATION>");
                        } break;
                }

            }
			}
			catch(Exception ExceptionObj)
			{
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
			}
            return sb_ExtraInformationForMediaProcessor.ToString();
        }

        internal void GetDestinationFileName(ThreadContext ThreadContextObj, String DestinationFormat, RenditionType enum_RenditionType, String GeneratorName, ref String CurrentFileName, ref String DestinationFileName)
        {
            ENTITYID            FileID = 0;
            StringBuilder       TemporaryFilePath = null;
            StringBuilder       sb_CatalogerMountPointUNCPath = null;
            String              str_MediaProcessor = null;

            sb_CatalogerMountPointUNCPath = new StringBuilder(100);
            TemporaryFilePath = new StringBuilder();

			try
			{
            ALCommon.GenerateUniqueID(ref FileID);
            if (!String.IsNullOrEmpty(GeneratorName))
            {
                switch (GeneratorName)
                {
                    case "IllustratorCS3Wrapper.dll":
                    case "MSOfficeProcessorWrapper.dll":
                    case "CatalogerGenericQWrapper.dll":
                    case "FLVWrapper.dll":
                        {
                            if (IsFilePathContainsSpecialCharacters(DestinationFileName))
                            {
                               
                                TemporaryFilePath.AppendFormat("{0}\\Temp\\{1}.{2}", MountPoint, FileID, DestinationFormat);
                                DestinationFileName = TemporaryFilePath.ToString();
                            }

                            switch (GeneratorName)
                            {
                                case "MSOfficeProcessorWrapper.dll":
                                    {
                                        str_MediaProcessor = "MSOFFICEPROCESSOR";
                                    } break;
                                case "CatalogerGenericQWrapper.dll":
                                    {
                                        str_MediaProcessor = "IMAGEPROCESSOR";

                                    } break;
                                case "FLVWrapper.dll":
                                    {
                                        str_MediaProcessor = "FLVGENERATOR";                                       
                                    } break;
                                default:
                                    str_MediaProcessor = ThreadContextObj.MediaProcessor;
                                    break;
                            }

                            DestinationFileName = GetUNCPath(DestinationFileName, str_MediaProcessor);

                            switch (enum_RenditionType)
                            {
                                case RenditionType.ORIGINAL:
                                    {
                                        if (IsFilePathContainsSpecialCharacters(ThreadContextObj.OriginalAttrObj.OriginalFilePath))
                                            CurrentFileName = GetUNCPath(ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath, str_MediaProcessor);
                                        else
                                            CurrentFileName = GetUNCPath(ThreadContextObj.OriginalAttrObj.OriginalFilePath, str_MediaProcessor);
                                    } break;
                                case RenditionType.PREVIEW:
                                    {
                                        if (IsFilePathContainsSpecialCharacters(ThreadContextObj.PreviewAttrObj.PreviewFilePath))
                                            CurrentFileName = GetUNCPath(ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath, str_MediaProcessor);
                                        else
                                            CurrentFileName = GetUNCPath(ThreadContextObj.PreviewAttrObj.PreviewFilePath, str_MediaProcessor);
                                    }
                                    break;
                            }
                        } break;
                    default:
                        {
                            ALCommon.GenerateUniqueID(ref FileID);
                            TemporaryFilePath.AppendFormat("{0}\\{1}.{2}", ThreadContextObj.TempFolderPath, FileID, DestinationFormat);
                            DestinationFileName = TemporaryFilePath.ToString();

                            switch (enum_RenditionType)
                            {
                                case RenditionType.PREVIEW:
                                    CurrentFileName = ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath;
                                    break;
                                case RenditionType.ORIGINAL:
                                default:
                                    CurrentFileName = ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath;
                                    break;
                            }
                        } break;
                }
            }
			}
			catch(Exception ExceptionObj)
			{
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
			}
        }

        private Boolean IsMediaProcessorInstalled(String ApplicationName)
        {
            String IsApplicationInstalled = null;
            StringBuilder sb_ApplicationInstalledKey = new StringBuilder(100);
            sb_ApplicationInstalledKey.AppendFormat("{0} MediaProcessor Installed", ApplicationName);

            IsApplicationInstalled = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", sb_ApplicationInstalledKey.ToString());

            if (!String.IsNullOrEmpty(IsApplicationInstalled))
            {
                if (IsApplicationInstalled == "1")
                    return true;
                else
                    return false;
            }
            return false;
        }

        private Boolean IsMediaProcessorOnSameNetwork(String ApplicationName)
        {
            String IsMediaProcessorOnSameNetwork = null;
            StringBuilder sb_IsMediaProcessorOnSameNetworkKey = new StringBuilder(100);
            sb_IsMediaProcessorOnSameNetworkKey.AppendFormat("{0}_ISONSAMENETWORK", ApplicationName);
            IsMediaProcessorOnSameNetwork = ConfigurationManager.AppSettings[sb_IsMediaProcessorOnSameNetworkKey.ToString()];

            if (!String.IsNullOrEmpty(IsMediaProcessorOnSameNetwork))
            {
                if (string.Compare(IsMediaProcessorOnSameNetwork, "TRUE", true) == 0)
                    return true;
                else
                    return false;
            }
            return false;
        }

        private String GetCatlogerMachineIPAddress()
        {
            String str_CatMachineIPAddress = null;
            str_CatMachineIPAddress = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "CatalogerMachineIP");
            return str_CatMachineIPAddress;
        }

        private Boolean IsFilePathContainsSpecialCharacters(String SourceFilePath)
        {
            String FileName = Path.GetFileName(SourceFilePath);
            String FileNameValidatorRegex = ConfigurationManager.AppSettings["FileNameValidatorRegexString"];//[^A-Za-z0-9-_&.:\\\\]
            Match match = Regex.Match(SourceFilePath, FileNameValidatorRegex);
            if (!string.IsNullOrEmpty(match.Value))
            {
                return true;
            }
            return false;
        }

        private String GetUNCPath(String FilePath, String ApplicationType)
        {
            if (!IsMediaProcessorInstalled(ApplicationType))
            {
                if (IsMediaProcessorOnSameNetwork(ApplicationType))
                {
                    FilePath = FilePath.Replace(MountPoint, CatalogerMountPointUNCPath);
                }
            }
            return FilePath;
        }

        private String GetPhysicalfilePath(String FilePath)
        {
            FilePath = FilePath.Replace(CatalogerMountPointUNCPath, MountPoint);
            return FilePath;
        }

        private bool ExtractXMP(ThreadContext ThreadContextObjForEachChildAsset, XPathNavigator XMPFacetConfigXMLDocNav)
        {
            StringBuilder   ErrorMessage                = null;
            String          ErrorXML                    = null;
            XPathNavigator  XPathNav                    = null;
            XML             XMPFacetCollectionXML       = null;
            String          XPath                       = null; 
            XPathDocument   XMPFacetCollectionXMLDoc    = null;


            try
            {
                ErrorMessage                        = new StringBuilder(100);
                XPath                               = "*/"+ TenantName.ToUpper();
                XPathNav                            = XMPFacetConfigXMLDocNav.SelectSingleNode(XPath);
                if (XPathNav != null)
                {
                    ErrorXML                        = MetadataHandler.ExtractXMPFromFile(ThreadContextObjForEachChildAsset.OriginalAttrObj.MetadataProcessor
                        ,ThreadContextObjForEachChildAsset.OriginalAttrObj.OriginalFilePath,XPathNav.InnerXml, ref XMPFacetCollectionXML);
                    if (String.IsNullOrEmpty(ErrorXML))
                    {
                        if(!String.IsNullOrEmpty(XMPFacetCollectionXML))
                        {
                            XMPFacetCollectionXMLDoc    = new XPathDocument(new StringReader(XMPFacetCollectionXML));
                            XPathNav                    = XMPFacetCollectionXMLDoc.CreateNavigator();
                            XPathNav                    = XPathNav.SelectSingleNode("Root");
                            ErrorXML                    = MergeStandardFacetAndXMPFields(XPathNav,ThreadContextObjForEachChildAsset.ClonedDigitalAssetNode);
                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                ErrorMessage.Remove(0, ErrorMessage.Length);
                                ErrorMessage.AppendFormat("XMP extraction for \"{0}\" succedded, but failed to update Assetinfo. Error Message: {1}", ThreadContextObjForEachChildAsset.OriginalAttrObj.OriginalFilePath, ErrorXML);
                                //LogError("CATALOGJOBPROCESSOR_JOBHANDLER_XMPEXTRACTIONFAILED", ErrorMessage.ToString(), null, null, ErrorType.ERROR);
                                LogError(null, ErrorMessage.ToString(), null, null, ErrorType.ERROR);
                                return false;
                            }
                        }
                    }
                    else
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.AppendFormat("Failed to extract XMP from \"{0}\". Error Message: {1}", ThreadContextObjForEachChildAsset.OriginalAttrObj.OriginalFilePath, ErrorXML);
                        LogError(null, ErrorMessage.ToString(), null, null, ErrorType.ERROR);
                        return false;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.AppendFormat("Failed to extract XMP from \"{0}\". Exception: {1}", ThreadContextObjForEachChildAsset.OriginalAttrObj.OriginalFilePath, ExceptionObj.Message);
                LogError(null, ErrorMessage.ToString(), null, null, ErrorType.EXCEPTION);
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return false;
            }
            return true;
        }

        private String MergeStandardFacetAndXMPFields(XPathNavigator XMPFacetCollectionNav,XmlNode DigitalAssetNode)
        {
            XPathNodeIterator   RelationshipNodes   = null; 
            XPathNodeIterator   FieldNodes          = null;
            XPathNavigator      FieldNodeNav        = null;
            XPathNavigator      RelationshipNode    = null;
            StringBuilder       XPath               = null;
            XmlNode             DA_FacetRelNode     = null;
            XmlNode             FacetNode           = null;    

            try
            {
                XPath                   = new StringBuilder(20);
                RelationshipNodes       = XMPFacetCollectionNav.SelectChildren(XPathNodeType.Element);
                while (RelationshipNodes.MoveNext())
                {
                    RelationshipNode    = RelationshipNodes.Current;
                    XPath.Remove(0, XPath.Length);
                    XPath.Append("descendant::" + RelationshipNode.Name);
                    if ((DA_FacetRelNode=DigitalAssetNode.SelectSingleNode(XPath.ToString())) != null)
                    {
                        FieldNodes      = RelationshipNode.Select("*/*");
                        while (FieldNodes.MoveNext())
                        {
                            FieldNodeNav    = FieldNodes.Current;
                            XPath.Remove(0, XPath.Length);
                            XPath.Append("descendant::" + FieldNodeNav.Name);
                            FacetNode       = DA_FacetRelNode.SelectSingleNode(XPath.ToString());
                            if (FacetNode   != null)
                            {
                                if (FieldNodeNav.SelectSingleNode("*")!=null)
                                    FacetNode.InnerXml  = FieldNodeNav.InnerXml;
                                else
                                    FacetNode.InnerText = FieldNodeNav.Value;
                            }
                            else
                            {
                                FacetNode               = DA_FacetRelNode.OwnerDocument.CreateElement(FieldNodeNav.Name);
                                if (FieldNodeNav.SelectSingleNode("*") != null)
                                    FacetNode.InnerXml  = FieldNodeNav.InnerXml;
                                else
                                    FacetNode.InnerText = FieldNodeNav.Value;
                                DA_FacetRelNode.LastChild.AppendChild(FacetNode);
                            }
                        }
                    }
                    else
                    {
                        DigitalAssetNode.InnerXml += RelationshipNode.OuterXml;
                    }

                }
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message,new StackTrace(true));
				return ExceptionObj.Message;
            }
            return null;
        }

        private Boolean GetPreviewAttributes(String DestinationName,  ThreadContext ThreadContextObj)
        {
            StringBuilder               XPathQuery                      = null;
            XmlNode                     FileFormatNode                  = null;
            StringBuilder               ErrorMessage                    = null;
            String                      PartialQuery                    = null;            
            InvokeMediaProcessorClass   InvokeMediaProcessorClassObj    = null;
            MediaProcessorWrapper       MediaProcessorWrapperObj        = null; 
            Boolean                     ManagedGenerator                = false;
            String                      str_ExtraInformationForMediaProcessor = null;
            String                      previewExt                      = null;
            String                      GeneratorName                   = null;
            try
            {
                //if (!ThreadContextObj.PreviewExists)
                {
                    XPathQuery                      = new StringBuilder();
                    ErrorMessage                    = new StringBuilder();
                    previewExt                      = DestinationName.Substring(DestinationName.LastIndexOf(".") + 1);

                    if (FileFormatXMLDoc.SelectSingleNode("//FILEFORMAT") == null)
                    {
                        ErrorMessage.Append("FileFormats not found.");
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_FILEFORMATSNOTFOUND",ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                        return false;
                    }

                    PartialQuery        = "//FILEFORMAT[FILEEXTENSION[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')=";
                    XPathQuery.AppendFormat("{0}'{1}']]", PartialQuery, previewExt.ToLower());
                    if ((FileFormatNode = FileFormatXMLDoc.SelectSingleNode(XPathQuery.ToString())) != null)
                    {
                        GeneratorName   = FileFormatNode.SelectSingleNode("PREVIEWGENERATOR").InnerText;                        
                        if (FileFormatNode != null)
                        {
                            if (FileFormatNode.SelectSingleNode("GENERATORTYPE") != null)
                            {
                                if (FileFormatNode.SelectSingleNode("GENERATORTYPE").InnerText.ToUpper() == ".NET")
                                    ManagedGenerator = true;
                                else
                                    ManagedGenerator = false;
                            }
                        }
                    }

                    if (!ManagedGenerator)
                    {
                        InvokeMediaProcessorClassObj                                = new InvokeMediaProcessorClass();

                        InvokeMediaProcessorClassObj.SetFileAttributes(DestinationName, null,
                        null, null, null, 0, 0, 0, 0, 0, null);
                        
                        InvokeMediaProcessorClassObj.GeneratorName      = GeneratorName;
                        InvokeMediaProcessorClassObj.ThreadTimeout      = ThreadTimeOut;
                        InvokeMediaProcessorClassObj.GetInfo();

                        ThreadContextObj.PreviewAttrObj.PreviewHeight   = InvokeMediaProcessorClassObj.Height;
                        ThreadContextObj.PreviewAttrObj.PreviewWidth    = InvokeMediaProcessorClassObj.Width;
                        ThreadContextObj.PreviewAttrObj.PreviewDepth    = InvokeMediaProcessorClassObj.Depth;
                        ThreadContextObj.PreviewAttrObj.PreviewDPI      = InvokeMediaProcessorClassObj.DPI;
                    }
                    else
                    {
                        switch (GeneratorName)
                        {
                            case "IllustratorCS3Wrapper.dll":
                            case "MSOfficeProcessorWrapper.dll":
                            case "CatalogerGenericQWrapper.dll":
                            case "FLVWrapper.dll":
                                {
                                    str_ExtraInformationForMediaProcessor = GenerateExtraInformationForMediaProcessor(ThreadContextObj, RenditionType.ORIGINAL, GeneratorName);
                                } break;
                        }

                        MediaProcessorWrapperObj                                    = new MediaProcessorWrapper(SessionID,UserID,TenantName,Language);
                        MediaProcessorWrapperObj.SetFileAttributes(DestinationName, null,
                        null, null, null, 0, 0, 0, 0, 0, null, str_ExtraInformationForMediaProcessor);
                        
                        MediaProcessorWrapperObj.GeneratorName         = GeneratorName;
                        MediaProcessorWrapperObj.ThreadTimeout         = ThreadTimeOut;
                        MediaProcessorWrapperObj.GetInfo();

                        ThreadContextObj.PreviewAttrObj.PreviewHeight  = MediaProcessorWrapperObj.Height;
                        ThreadContextObj.PreviewAttrObj.PreviewWidth   = MediaProcessorWrapperObj.Width;
                        ThreadContextObj.PreviewAttrObj.PreviewDepth   = MediaProcessorWrapperObj.Depth;
                        ThreadContextObj.PreviewAttrObj.PreviewDPI     = MediaProcessorWrapperObj.DPI;
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
				CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }


        #region Methods related to Cataloger Changed Process
        private Boolean UpdateCatalogJobProcessingDetails(ThreadContext ThreadContextObj, String AssetID, bool IsAssetCreatedSuccessfully)
        {
            #region member variables
            StringBuilder   ErrorMessage            = null;
            String          str_JobDetailsXML       = null;
            XmlNode         JobNode                 = null;
            XmlDocument     JobDetailsXml           = null;
            String          str_PreviewGenerated    = null;
            String          str_ThumbnailGenerated  = null;
            String          JobDetailsXML           = null;
            String          str_OrgattributeUpdated = null;
            String          str_PrvAttributeUpdated = null;
            String          str_ThbAttributeUpdated = null;
            String          str_ProcessingStatus    = null;            
            String          CatalogJobStatus        = null;
            String          AssetCreated            = null;
            String          str_BusinessInfoValid   = null;
            #endregion

            try
            {
                JobDetailsXml       = new XmlDocument();
                ErrorMessage        = new StringBuilder();
                JobDetailsXML       = GetJobDetails(ThreadContextObj);

                if (IsAssetCreatedSuccessfully)
                {
                    if (String.Compare(ThreadContextObj.OriginalAttrObj.OriginalGenerator, "MSOfficeProcessorWrapper.dll", true) == 0
                        || String.Compare(ThreadContextObj.OriginalAttrObj.OriginalGenerator, "FLVWrapper.dll", true) == 0)
                        str_OrgattributeUpdated = "2";//No Generator/Not Required
                    else if (!String.IsNullOrEmpty(ThreadContextObj.OriginalAttrObj.OriginalGenerator))
                        str_OrgattributeUpdated = "0";
                    else
                        str_OrgattributeUpdated = "2"; //No Generator/Not Required

                    str_PrvAttributeUpdated = "0";
                    if (ThreadContextObj.UseOriginalAsPreview || ThreadContextObj.PreviewExists)
                    {
                        str_PreviewGenerated = "1";
                        if (ThreadContextObj.UseOriginalAsPreview)
                        {
                            str_PrvAttributeUpdated = "1";
                        }
                        else
                        {
                            if (String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewGenerator))
                                str_PrvAttributeUpdated = "2";
                        }
                    }
                    else if (String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewGenerator))
                    {
                        str_PreviewGenerated = "2"; //No Generator/Not Required
                        str_PrvAttributeUpdated = "2";
                    }
                    else
                        str_PreviewGenerated = "0";

                    if (ThreadContextObj.PreviewExists && (String.Compare(ThreadContextObj.PreviewAttrObj.PreviewGenerator, "MSOfficeProcessorWrapper.dll", true) == 0
                        || String.Compare(ThreadContextObj.PreviewAttrObj.PreviewGenerator, "FLVWrapper.dll", true) == 0))
                        str_PrvAttributeUpdated = "2";

                    str_ThbAttributeUpdated = "0";
                    if (ThreadContextObj.ThumbnailExists)
                    {
                        str_ThumbnailGenerated = "1";
                        if(String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator))
                            str_ThbAttributeUpdated = "2";
                    }
                    else if (String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator))
                    {
                        str_ThumbnailGenerated = "2"; //No Generator/Not Required
                        str_ThbAttributeUpdated = "2";
                    }
                    else
                        str_ThumbnailGenerated = "0";

                    if (IsBusinessInfoValid)
                        str_BusinessInfoValid = "1";
                    else
                        str_BusinessInfoValid = "3";

                    if (String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator) && String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewGenerator)
                        && String.IsNullOrEmpty(ThreadContextObj.OriginalAttrObj.OriginalGenerator))
                    {
                        if(String.Compare(str_BusinessInfoValid, "1", true) == 0)
                            str_ProcessingStatus = "Done";
                        else if (String.Compare(str_BusinessInfoValid, "3", true) == 0)
                            str_ProcessingStatus = "Error";
                    }
                    else
                        str_ProcessingStatus = "In-Progress";

                    AssetCreated = "1";
                }
                else
                {
                    str_OrgattributeUpdated     = "3";
                    str_PreviewGenerated        = "3"; 
                    str_PrvAttributeUpdated     = "3";
                    str_ThumbnailGenerated      = "3"; 
                    str_ThbAttributeUpdated     = "3";
                    AssetCreated                = "3";
                    str_BusinessInfoValid       = "3";
                    str_ProcessingStatus        = "Error";
                }

                CatalogJobStatus = GetCatalogerJobStatus(CatalogJobID, str_ProcessingStatus);                                     

                //If the Current Asset's processing is complete as there is No Generator Present
                if (String.Compare(str_ProcessingStatus, "Done", true) == 0)
                {
                    //mark the Job status as Done , since all assets are processed.
                    if (NoOfProcessedAssets == iTotalAssets)
                    {                           
                        if (String.Compare(CatalogJobStatus, "Done", true) == 0)
                        {
                            IsProcessingComplete = true;
                            isJobSuccessed       = true;
                        }
                        else if (String.Compare(CatalogJobStatus, "Error", true) == 0)
                        {
                            IsProcessingComplete = true;
                            isJobSuccessed       = false;
                        }
                    }
                    else
                        CatalogJobStatus = "In-Progress";                                                
                }
                else if (String.Compare(str_ProcessingStatus, "Error", true) == 0)
                {
                    if (NoOfProcessedAssets == iTotalAssets)
                    {
                        CatalogJobStatus        = "Error";
                        IsProcessingComplete    = true;
                        isJobSuccessed          = false;
                    }
                    else
                        CatalogJobStatus = "In-Progress";   
                }
                else
                    CatalogJobStatus = "In-Progress";   

                str_JobDetailsXML   = String.Format(@"<CATALOGJOB>
                                                        <CATALOGJOBID>{0}</CATALOGJOBID>
                                                        <STATUS>{10}</STATUS>
                                                        <NOOFPROCESSEDASSETS>{11}</NOOFPROCESSEDASSETS>
                                                        <CATALOGJOB_PROCESSINGDETAILS TYPE='RELATION'>
                                                            <CATALOGJOBDETAILS>
                                                                <ASSETID>{1}</ASSETID>
                                                                <ASSETNAME>{2}</ASSETNAME>
                                                                <ASSETCREATED>{12}</ASSETCREATED>
                                                                <ORGATTRIBUTEUPDATED>{6}</ORGATTRIBUTEUPDATED>
                                                                <PREVIEWGENERATED>{3}</PREVIEWGENERATED>
                                                                <PRVATTRIBUTEUPDATED>{7}</PRVATTRIBUTEUPDATED>
                                                                <THUMBNAILGENERATED>{4}</THUMBNAILGENERATED>
                                                                <THBATTRIBUTEUPDATED>{8}</THBATTRIBUTEUPDATED>
                                                                <PROCESSINGSTATUS>{9}</PROCESSINGSTATUS>
                                                                <JOBDETAILSXML></JOBDETAILSXML>
                                                                <UPDATEDON>{5}</UPDATEDON>
                                                                <ERRORIDLIST>{13}</ERRORIDLIST> 
                                                                <ISVALIDBUSINESSINFO>{14}</ISVALIDBUSINESSINFO>  
                                                            </CATALOGJOBDETAILS>
                                                        </CATALOGJOB_PROCESSINGDETAILS>
                                                    </CATALOGJOB>"
                                                    , CatalogJobID, AssetID, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFileName), str_PreviewGenerated
                                                    , str_ThumbnailGenerated, ALCommon.DateTimeToXml(DateTime.Now), str_OrgattributeUpdated, str_PrvAttributeUpdated
                                                    , str_ThbAttributeUpdated, str_ProcessingStatus, CatalogJobStatus, NoOfProcessedAssets.ToString(), AssetCreated, LoggedERRORID, str_BusinessInfoValid);

                LoggedERRORID   = null;
                JobDetailsXml.LoadXml(str_JobDetailsXML);
                JobNode         = JobDetailsXml.SelectSingleNode("CATALOGJOB");

                if (JobNode != null)
                {
                    //IMP: This is done explicitly to removed the ClonedDigitalAsset node from the details xml as
                    //it contains XML used for CreateDA call, hence if not removed then files gets created in every UpdateCatalogJob call.
                    Int32 StartIndex = JobDetailsXML.IndexOf("<CLONEDDIGITALASSETNODE>");
                    Int32 EndIndex   = JobDetailsXML.LastIndexOf("</CLONEDDIGITALASSETNODE>") + ("</CLONEDDIGITALASSETNODE>").Length;
                    if(StartIndex != -1 && EndIndex != -1)
                        JobDetailsXML = JobDetailsXML.Remove(StartIndex, (EndIndex - StartIndex));

                    JobNode.SelectSingleNode("CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS/JOBDETAILSXML").InnerXml = JobDetailsXML;
                    if (!UpdateCatalogJob(JobNode))
                    {
                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("DIGITALASSETNAME", ThreadContextObj.ParentDigitalAssetName);
                        ErrorMessage.AppendFormat("DigitalAsset creation failed for {0}, Updating Job processing details have failed", ThreadContextObj.ParentDigitalAssetName);                        
                        LogError("CATALOGJOBPROCESSOR_JOBHANDLER_UPDATEJOBPROCESSINGDETAILSFAILED", ErrorMessage.ToString(), null,null, ErrorType.ERROR);
                        return false;
                    }
                }
                return true;
            }
            catch (Exception ExceptionObj)
            {
                CatalogerHelper.CatalogerHelper.WriteLogToNTFS("Exception Occured while Updating Job Processing Details in Method ( " + MethodInfo.GetCurrentMethod().Name + " )", new StackTrace(true));
                CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
				return false;
            }
        }

        private String GetJobDetails(ThreadContext ThreadContextObj)
        {
            #region member variable
            String     str_Details              = null;
            String     str_PreviewAttributes    = null;
            String     str_OriginalAttributes   = null;
            String     str_ThumbnailAttributes  = null;
            String     FormatDetailsXML         = null;
            #endregion


            try
            {
                #region Preview Details
                if (ThreadContextObj.PreviewExists)
                {
                    str_PreviewAttributes = String.Format(@"<PREVIEWFILENAME>{0}</PREVIEWFILENAME>           
                                                    <PREVIEWFILEPATH>{1}</PREVIEWFILEPATH>           
                                                    <PREVIEWFILEEXTENSION>{2}</PREVIEWFILEEXTENSION>                                                    
                                                    <ASSETRENDITIONID>{3}</ASSETRENDITIONID>
                                                    <PREVIEWFILEFORMAT>{4}</PREVIEWFILEFORMAT>"
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.PreviewFileName)
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.PreviewFilePath)
                                                    , ThreadContextObj.PreviewAttrObj.PreviewFileExtension
                                                    , ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("//CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')='preview']]/ASSETRENDITIONID").InnerText
                                                    , ThreadContextObj.PreviewAttrObj.PreviewFileFormat);

                }
                else if (ThreadContextObj.UseOriginalAsPreview)
                {
                    str_PreviewAttributes = String.Format(@"{0}<PREVIEWFILENAME>{1}</PREVIEWFILENAME>           
                                                    <PREVIEWFILEPATH>{2}</PREVIEWFILEPATH>           
                                                    <PREVIEWFILEEXTENSION>{3}</PREVIEWFILEEXTENSION>
                                                    <ASSETRENDITIONID>{4}</ASSETRENDITIONID>
                                                    <PREVIEWFILEFORMAT>{5}</PREVIEWFILEFORMAT>"
                                                    , str_PreviewAttributes
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFileName)
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                    , ThreadContextObj.OriginalAttrObj.OriginalFileExtension
                                                    , ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("//CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')='preview']]/ASSETRENDITIONID").InnerText
                                                    , ThreadContextObj.OriginalAttrObj.OriginalFileFormat);
                }
                else
                {
                    if(!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewFileName))
                        str_PreviewAttributes = String.Format("<PREVIEWFILENAME>{0}</PREVIEWFILENAME>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.PreviewFileName));
                    if (!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewFilePath))
                        str_PreviewAttributes = String.Format("{0}<PREVIEWFILEPATH>{1}</PREVIEWFILEPATH>", str_PreviewAttributes,ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.PreviewFilePath));
                    if (!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.PreviewFileExtension))
                        str_PreviewAttributes = String.Format("{0}<PREVIEWFILEEXTENSION>{1}</PREVIEWFILEEXTENSION>", str_PreviewAttributes, ThreadContextObj.PreviewAttrObj.PreviewFileExtension);
                    if (!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath))
                        str_PreviewAttributes = String.Format("{0)<INTERMIDIATEPRVFILEPATH>{1}</INTERMIDIATEPRVFILEPATH>", str_PreviewAttributes, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.IntermidiatePrvFilePath));
                    if (!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.TempPrvFilePath))
                        str_PreviewAttributes = String.Format("{0}<TEMPPRVFILEPATH>{1}</TEMPPRVFILEPATH>", str_PreviewAttributes, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.PreviewAttrObj.TempPrvFilePath));
                    if (!String.IsNullOrEmpty(ThreadContextObj.PreviewAttrObj.IsFileCopiedToTempLocation.ToString()))
                        str_PreviewAttributes = String.Format("{0}<ISFILECOPIEDTOTEMPLOCATION>{1}</ISFILECOPIEDTOTEMPLOCATION>", str_PreviewAttributes, ThreadContextObj.PreviewAttrObj.IsFileCopiedToTempLocation.ToString().ToUpper());
                    str_PreviewAttributes = String.Format("{0}<PREVIEWFILEFORMAT>{1}</PREVIEWFILEFORMAT>", str_PreviewAttributes, ThreadContextObj.PreviewAttrObj.PreviewFileFormat);
                }

                str_PreviewAttributes = String.Format(@"{0}<PREVIEWGENERATOR>{1}</PREVIEWGENERATOR>       
                                                           <PRVMACTYPE>{2}</PRVMACTYPE>   
                                                           <PRVMACCREATOR>{3}</PRVMACCREATOR>
                                                           <MANAGEDGENERATOR>{4}</MANAGEDGENERATOR>"
                                                         , str_PreviewAttributes                                                         
                                                         , ThreadContextObj.PreviewAttrObj.PreviewGenerator
                                                         , ThreadContextObj.PreviewAttrObj.PrvMacType
                                                         , ThreadContextObj.PreviewAttrObj.PrvMacCreator
                                                         , ThreadContextObj.PreviewAttrObj.ManagedGenerator.ToString().ToUpper());
                #endregion
                
                #region Thumbnail Details
                if (ThreadContextObj.ThumbnailExists)
                {
                    str_ThumbnailAttributes = String.Format(@"<THUMBNAILFILENAME>{0}</THUMBNAILFILENAME>
                                                    <THUMBNAILFILEPATH>{1}</THUMBNAILFILEPATH>
                                                    <THUMBNAILFILEEXTENSION>{2}</THUMBNAILFILEEXTENSION>
                                                    <ASSETRENDITIONID>{3}</ASSETRENDITIONID>"
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName)
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath)
                                                    , ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension
                                                    , ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("//CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')='thumbnail']]/ASSETRENDITIONID").InnerText);
                }
                else
                {
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName))
                        str_ThumbnailAttributes = String.Format("<THUMBNAILFILENAME>{0}</THUMBNAILFILENAME>", ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbnailFileName));
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath))
                        str_ThumbnailAttributes = String.Format("{0}<THUMBNAILFILEPATH>{1}</THUMBNAILFILEPATH>", str_ThumbnailAttributes, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbnailFilePath));
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension))
                        str_ThumbnailAttributes = String.Format("{0}<THUMBNAILFILEEXTENSION>{1}</THUMBNAILFILEEXTENSION>", str_ThumbnailAttributes, ThreadContextObj.ThumbnailAttrObj.ThumbnailFileExtension);
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath))
                        str_ThumbnailAttributes = String.Format("{0}<INTERMIDIATETHUMFILEPATH>{1}</INTERMIDIATETHUMFILEPATH>", str_ThumbnailAttributes, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.IntermidiateThumFilePath));
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath))
                        str_ThumbnailAttributes = String.Format("{0}<TEMPTHUMBFILEPATH>{1}</TEMPTHUMBFILEPATH>", str_ThumbnailAttributes, ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.TempThumbFilePath));
                    if (!String.IsNullOrEmpty(ThreadContextObj.ThumbnailAttrObj.IsFileCopiedToTempLocation.ToString()))
                        str_ThumbnailAttributes = String.Format("{0}<ISFILECOPIEDTOTEMPLOCATION>{1}</ISFILECOPIEDTOTEMPLOCATION>", str_ThumbnailAttributes, ThreadContextObj.ThumbnailAttrObj.IsFileCopiedToTempLocation.ToString());
                }

                str_ThumbnailAttributes = String.Format(@"{0}<THUMBNAILFILEFORMAT>{1}</THUMBNAILFILEFORMAT>
                                                    <THUMBNAILGENERATOR>{2}</THUMBNAILGENERATOR>
                                                    <THUMBMACTYPE>{3}</THUMBMACTYPE>
                                                    <THUMBMACCREATOR>{4}</THUMBMACCREATOR>
                                                    <MANAGEDGENERATOR>{5}</MANAGEDGENERATOR>", str_ThumbnailAttributes
                                                    , ThreadContextObj.ThumbnailAttrObj.ThumbnailFileFormat
                                                    , ThreadContextObj.ThumbnailAttrObj.ThumbnailGenerator
                                                    , ThreadContextObj.ThumbnailAttrObj.ThumbMacType
                                                    , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ThumbnailAttrObj.ThumbMacCreator)
                                                    , ThreadContextObj.ThumbnailAttrObj.ManagedGenerator.ToString().ToUpper());                
                #endregion

                #region Original Details
                str_OriginalAttributes = String.Format(@"<ORIGINALFILENAME>{0}</ORIGINALFILENAME>
                                                    <ORIGINALFILEPATH>{1}</ORIGINALFILEPATH>
                                                    <ORIGINALFILEEXTENSION>{2}</ORIGINALFILEEXTENSION>
                                                    <ORIGINALFILEFORMAT>{3}</ORIGINALFILEFORMAT>
                                                    <ORIGINALSIZE>{4}</ORIGINALSIZE>                                                    
                                                    <ORGMACTYPE>{5}</ORGMACTYPE>
                                                    <ORGMACCREATOR>{6}</ORGMACCREATOR>
                                                    <ORIGINALGENERATOR>{7}</ORIGINALGENERATOR>
                                                    <MANAGEDGENERATOR>{8}</MANAGEDGENERATOR>
                                                    <ASSETRENDITIONID>{9}</ASSETRENDITIONID>
                                                    <TEMPORGFILEPATH>{10}</TEMPORGFILEPATH>
                                                    <INTERMIDIATEORGFILEPATH>{11}</INTERMIDIATEORGFILEPATH>"
                                                     , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFileName)
                                                     , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OriginalFilePath)
                                                     , ThreadContextObj.OriginalAttrObj.OriginalFileExtension
                                                     , ThreadContextObj.OriginalAttrObj.OriginalFileFormat
                                                     , ThreadContextObj.OriginalAttrObj.OriginalSize
                                                     , ThreadContextObj.OriginalAttrObj.OrgMacType
                                                     , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.OrgMacCreator)
                                                     , ThreadContextObj.OriginalAttrObj.OriginalGenerator
                                                     , ThreadContextObj.OriginalAttrObj.ManagedGenerator.ToString().ToUpper()
                                                     , ThreadContextObj.ClonedDigitalAssetNode.SelectSingleNode("//CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE[translate(text(), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')='original']]/ASSETRENDITIONID").InnerText
                                                     , ThreadContextObj.OriginalAttrObj.TempOrgFilePath
                                                     , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.OriginalAttrObj.IntermidiateOrgFilePath));
                #endregion

                #region Job Details
                if (ThreadContextObj.FormatDetails != null)
                    FormatDetailsXML = ThreadContextObj.FormatDetails.OuterXml;
                else
                    FormatDetailsXML = "";

                str_Details = String.Format(@"<JOBDATA>
                                                <CATALOGJOBID>{0}</CATALOGJOBID>
                                                <PREVIEWEXISTS>{1}</PREVIEWEXISTS>
                                                <THUMBNAILEXISTS>{2}</THUMBNAILEXISTS>
                                                <ISBINHEX>{3}</ISBINHEX>
                                                <ISTHUMBBINHEX>{4}</ISTHUMBBINHEX>
                                                <ISPRVBINHEX>{5}</ISPRVBINHEX>
                                                <GENTHUMBFROMPREVIEW>{6}</GENTHUMBFROMPREVIEW>
                                                <THBFROMORGGENERATOR>{7}</THBFROMORGGENERATOR>
                                                <FORMATDETAILS>{8}</FORMATDETAILS>
                                                <PREVIEWATTROBJ>{17}</PREVIEWATTROBJ>
                                                <THUMBNAILATTROBJ>{18}</THUMBNAILATTROBJ>
                                                <ORIGINALATTROBJ>{19}</ORIGINALATTROBJ>
                                                <DIGITALASSETID>{9}</DIGITALASSETID>
                                                <CONTENTREVISIONID>{10}</CONTENTREVISIONID>
                                                <SECTIONID>{11}</SECTIONID>
                                                <ONMEDIASERVER>{12}</ONMEDIASERVER>
                                                <TEMPFOLDERPATH>{13}</TEMPFOLDERPATH>
                                                <PARENTDIGITALASSETNAME>{14}</PARENTDIGITALASSETNAME>
                                                <SECTIONPATH>{15}</SECTIONPATH>
                                                <MEDIAPROCESSOR>{16}</MEDIAPROCESSOR>
                                                <USEORIGINALASPREVIEW>{20}</USEORIGINALASPREVIEW>
                                                <CLONEDDIGITALASSETNODE>{21}</CLONEDDIGITALASSETNODE>
                                                <TENANTCATALOGINGPREFERENCESXML>{22}</TENANTCATALOGINGPREFERENCESXML>
                                           </JOBDATA>", CatalogJobID
                                          , ThreadContextObj.PreviewExists.ToString().ToUpper()
                                          , ThreadContextObj.ThumbnailExists.ToString().ToUpper()
                                          , ThreadContextObj.IsBinHex.ToString().ToUpper()
                                          , ThreadContextObj.IsThumbBinHex.ToString().ToUpper()
                                          , ThreadContextObj.IsPrvBinHex.ToString().ToUpper()
                                          , ThreadContextObj.GenThumbFromPreview.ToString().ToUpper()
                                          , ThreadContextObj.ThbFromOrgGenerator
                                          , FormatDetailsXML
                                          , ThreadContextObj.DigitalAssetID.ToString()
                                          , ThreadContextObj.ContentRevisionID
                                          , ThreadContextObj.SectionID.ToString()
                                          , ThreadContextObj.OnMediaServer.ToString().ToUpper()
                                          , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.TempFolderPath)
                                          , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.ParentDigitalAssetName)
                                          , ALCommon.TrimAndReplaceSpecialChars(ThreadContextObj.str_SectionPath)
                                          , ThreadContextObj.MediaProcessor
                                          , str_PreviewAttributes
                                          , str_ThumbnailAttributes
                                          , str_OriginalAttributes
                                          , ThreadContextObj.UseOriginalAsPreview.ToString().ToUpper()
                                          , ThreadContextObj.ClonedDigitalAssetNode.OuterXml
                                          , TenantCatalogingPreferencesXML);
                  #endregion
            }
            catch (Exception ExceptionObj)
            {
                CatalogerHelper.CatalogerHelper.WriteLogToNTFS("Exception Occured while getting Job Processing Details in Method ( " + MethodInfo.GetCurrentMethod().Name + " )", new StackTrace(true));
                CatalogerHelper.CatalogerHelper.WriteLogToNTFS(ExceptionObj.Message, new StackTrace(true));
                return "";
            }
            return str_Details;
        }

        private String GetCatalogerJobStatus(String JobID, String CurrentAssetProcessingStatus)
        {
            #region member variables
            String          Status              = null;
            Search          ObjSearchWS         = null;
            String          SearchInput         = null;
            String          SearchResult        = null;
            XPathDocument   xp_SearchResult     = null;
            XPathNavigator  xpNav_SearchRoot    = null;
            String          ErrorXML            = null;
            XPathNodeIterator xpItr_JobDetailsList      = null;
            String          str_tmpStatus               = null;
            XPathNavigator xpNav_ProcessingStatus       = null;
            XPathNavigator xpNav_NoOfProcessedAssets    = null;
            XPathNavigator xpNav_TotalAssets            = null;
            #endregion

            try
            {
                #region Initialization
                ObjSearchWS         = new Search();
                ObjSearchWS.Url     = ALCommon.GetWebServiceURL("ALSearchWS");
                ObjSearchWS.Timeout = GetWebServiceTimeout();
                Status              = "In-Progress";
                #endregion

                #region Prepare Search Xml
                SearchInput = String.Format(@"<SEARCH>
                                                <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}'/>
                                                <TARGETOBJECT>CATALOGJOB</TARGETOBJECT>
                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                <MAXRESULTSIZE>1</MAXRESULTSIZE>
                                                <TARGETENTITY>
                                                  <FILTERLIST>
                                                    <FILTER>
                                                      <EXPRESSION>CATALOGJOBID</EXPRESSION>
                                                      <OPERATOR>EQUALTO</OPERATOR>
                                                      <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
                                                    </FILTER>            
                                                  </FILTERLIST>
                                                  <SELECTLIST>
                                                    <EXPRESSION>
                                                      <FIELDNAME>CATALOGJOBID</FIELDNAME>
                                                      <ALIAS>CATALOGJOBID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                      <FIELDNAME>NOOFPROCESSEDASSETS</FIELDNAME>
                                                      <ALIAS>NOOFPROCESSEDASSETS</ALIAS>
                                                    </EXPRESSION> 
                                                    <EXPRESSION>
                                                      <FIELDNAME>TOTALASSETS</FIELDNAME>
                                                      <ALIAS>TOTALASSETS</ALIAS>
                                                    </EXPRESSION>                                                     
                                                  </SELECTLIST>           
                                                  <RELATIONSHIP NAME='CATALOGJOB_PROCESSINGDETAILS'>
                                                    <SELECTLIST>
                                                      <EXPRESSION>
                                                        <FIELDNAME>PROCESSINGSTATUS</FIELDNAME>
                                                        <ALIAS>PROCESSINGSTATUS</ALIAS>
                                                      </EXPRESSION>
                                                    </SELECTLIST>
                                                  </RELATIONSHIP>
                                                </TARGETENTITY>
                                              </SEARCH>", SessionID, UserID, TenantName, Language, JobID);
                #endregion

                #region Get JobDetails and check for Job Status
                ErrorXML    = ObjSearchWS.GetHierarchialData(SearchInput, ref SearchResult);
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    xp_SearchResult             = new XPathDocument(new StringReader(SearchResult));
                    xpNav_SearchRoot            = xp_SearchResult.CreateNavigator();
                    xpNav_NoOfProcessedAssets   = xpNav_SearchRoot.SelectSingleNode("CATALOGJOBCOLLECTION/CATALOGJOB/NOOFPROCESSEDASSETS");                 
                    xpNav_TotalAssets           = xpNav_SearchRoot.SelectSingleNode("CATALOGJOBCOLLECTION/CATALOGJOB/TOTALASSETS");

                    if ((xpNav_TotalAssets != null && !String.IsNullOrEmpty(xpNav_TotalAssets.Value)) && 
                        (xpNav_NoOfProcessedAssets != null && !String.IsNullOrEmpty(xpNav_NoOfProcessedAssets.Value)))
                    {
                        Int32.TryParse(xpNav_NoOfProcessedAssets.Value, out NoOfProcessedAssets);
                        Int32.TryParse(xpNav_TotalAssets.Value, out iTotalAssets);

                        if(String.Compare(CurrentAssetProcessingStatus, "In-Progress", true) != 0)
                            NoOfProcessedAssets++;

                        if (xpNav_TotalAssets.Value != NoOfProcessedAssets.ToString())
                            Status = "In-Progress";
                        else
                        {
                            xpItr_JobDetailsList = xpNav_SearchRoot.Select("CATALOGJOBCOLLECTION/CATALOGJOB/CATALOGJOB_PROCESSINGDETAILS/CATALOGJOBDETAILS");

                            if (xpItr_JobDetailsList.Count == 0)
                                return CurrentAssetProcessingStatus;

                            foreach (XPathNavigator xpNav_JodDetails in xpItr_JobDetailsList)
                            {
                                xpNav_ProcessingStatus = xpNav_JodDetails.SelectSingleNode("PROCESSINGSTATUS");
                                if (xpNav_ProcessingStatus != null)
                                {
                                    str_tmpStatus = xpNav_ProcessingStatus.Value;
                                    if (String.Compare(str_tmpStatus, "Done", true) == 0)
                                        Status = "Done";
                                    else if (String.Compare(str_tmpStatus, "In-Progress", true) == 0)
                                    {
                                        Status = "In-Progress";
                                        break;
                                    }
                                    else if (String.Compare(str_tmpStatus, "Error", true) == 0)
                                    {
                                        Status = "Error";
                                        break;
                                    }
                                }
                            }//for each asset
                        }
                    }
                }
                #endregion
            }
            catch (Exception)
            {
                Status = "In-Progress";
            }
            return Status;
        }
        #endregion
    }
}
