#region System Include
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Threading;
using System.Collections;
#endregion

#region Assetlink Include
using Assetlink;
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Solutions.Facade.DAMFacade;
using Assetlink.WebServices.ALArtworkProducerWS;
using Assetlink.WebServices.ALSearchWS;
using Assetlink.Services.Plugins.PlugInsWrapper;
using Assetlink.Solutions.Facade.TMSFacade;
using Assetlink.IOProxy;
#endregion
namespace Assetlink.Solutions.Facade.ArtworkProducerFacade
{
    using ERRORXML = System.String;
    using XML = System.String;
    public class FormatFacade : BaseFacade
    {
        #region variable
        ALArtworkProducerWS ArtworkProducerWSobj = null;
        ALServiceHeader RequestHeader = null;
        ERRORXML ErrorXML = null;
        Hashtable ImportFormatIDs = null;
        private String FormatToImport = null;
        private String FormatPathForZip = null;
        private String FormatNameForZip = null;
        ALErrorCommon objError = null;

        #endregion

        System.Security.Principal.WindowsIdentity OrgIdentity;

        #region Constructor
        /// <alsummary>
        ///     <createddate>27 November 2006</createddate>
        ///     <author>Pallavi Pitla</author>
        /// </alsummary>
        /// <param name="SessionID"></param>
        /// <param name="UserID"></param>
        /// <param name="TenantName"></param>
        /// <param name="Language"></param>
        public FormatFacade(String SessionID, String UserID, String TenantName, String Language)
            : base(SessionID, UserID, TenantName, Language)
        {
            try
            {
                RequestHeader = new ALServiceHeader();
                RequestHeader.SessionID = Convert.ToUInt64(SessionID);
                RequestHeader.UserID = Convert.ToUInt64(UserID);
                RequestHeader.TenantName = TenantName;
                RequestHeader.Language = CurrentLanguage;
                ArtworkProducerWSobj = new ALArtworkProducerWS();
                ArtworkProducerWSobj.ALServiceHeaderValue = RequestHeader;
                ArtworkProducerWSobj.Url = ALCommon.GetWebServiceURL("ALArtworkProducerWS");
                objError = new ALErrorCommon();
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
        }


        #endregion

        public enum FormatDownloadStatus
        {
            RetrivingFormatFiles = 2,
            RetrivingLinkedFiles = 3,
            PreparingZip = 5,
            Done = 6,
            Error = 7
        }

        public enum FormatExportStatus
        {
            RetrivingFormatInfo = 1,
            RetrivingFormatFiles = 2,
            RetrivingLinkedFiles = 3,
            RetrivingDAMItems = 4,
            PreparingZip = 5,
            Done = 6,
            Error = 7
        }

        public enum FormatImportStatus
        {
            UnzippingFormatFile = 1,
            CreatingFormat = 2,
            CreatingElements = 3,
            Done = 4,
            Error = 5,
            InvalidZip = 6
        }

        public enum CreateIndesignFormatStatus
        {
            UnzippingFormatFile = 1,
            ValidatingJob = 2,
            CreatingFormat = 3,
            Done = 4,
            Error = 5
        }

        /// <alsummary>
        /// <createddate>27 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to create new Format based on the input XML.</purpose>
        ///  <xmlstructure><![CDATA[
        /// 
        ///     Input XML :         |-  [FORMAT]
        ///                                       - $Attributes$
        ///                                       |-    [$RelationShipName$ @TYPE="RELATION"  @DISASSOCIATE="TRUE/FALSE"]text()=$Associated Entity/Facet Id$     // Associate or Disassociate existing instance with TargetEntity                    
        ///                                                  |-  [$Associated Entity/Facet Name$]        //it will create new instance of associated entity 
        ///                                                           |-  $Attributes                       // or facet , and associate it with target entity
        /// 
        /// ]]></xmlstructure>
        /// <param name="FormatXML"></param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML CreateFormat(ref String RelationShipName, ref String RelatedObjectID, ref XML FormatXML, ref String FormatID, ref String PageID, ref String GroupID)
        {
            #region variable
            StringBuilder sbCreatedOn = null;
            String[] Separator = { "$AL$" };
            StringBuilder sbRelationShip = null;


            #endregion
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin CreateFormat");
                if (RelationShipName == null)
                {
                    RelationShipName = "";
                    RelatedObjectID = "";
                }
                #region To add 'Creator' , if FormatXML doesn't contain it.
                if (!FormatXML.Contains("FORMAT_CREATOR"))
                {
                    if (!RelationShipName.Contains("FORMAT_CREATOR"))
                    {
                        RelationShipName = RelationShipName + "$AL$" + "FORMAT_CREATOR";
                        RelatedObjectID = RelatedObjectID + "$AL$" + UserID;
                    }
                }
                #endregion

                #region To add 'CreatedOn' date, if ArtworkXML doesn't contain it.
                if (!FormatXML.Contains("<CREATEDON>"))
                {
                    sbCreatedOn = new StringBuilder();
                    sbCreatedOn.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                            , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), sbCreatedOn.ToString());
                }
                #endregion

                #region Associate Format with Given Related Entity/Facet i.e RelationShipName and RelatedObjectID
                if ((RelationShipName != null) && (RelationShipName != "") && (RelatedObjectID != null) && (RelatedObjectID != ""))
                {
                    String[] RelationNameArray = RelationShipName.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                    String[] RelatedObjectIDArray = RelatedObjectID.Split(Separator, StringSplitOptions.RemoveEmptyEntries);

                    if (RelationNameArray.Length == RelatedObjectIDArray.Length)
                    {
                        sbRelationShip = new StringBuilder();
                        for (int i = 0; i < RelationNameArray.Length; i++)
                        {
                            sbRelationShip.AppendFormat("<{0} TYPE=\"RELATION\">{1}</{2}>", RelationNameArray[i], RelatedObjectIDArray[i], RelationNameArray[i]);
                        }
                        FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), sbRelationShip.ToString());
                    }
                    //else
                    //  return "Mismath RelationShip Names and RelationShip IDs";
                }
                #endregion

                ErrorXML = ArtworkProducerWSobj.CreateFormat(ref FormatXML);
                if (ErrorXML == null)
                {
                    FormatID = ALCommon.GetInnerValue(ref FormatXML, "<FORMATID>", "</FORMATID>");
                    PageID = ALCommon.GetInnerValue(ref FormatXML, "<ARTWORKPAGEID>", "</ARTWORKPAGEID>");
                    GroupID = ALCommon.GetInnerValue(ref FormatXML, "<ELEMENTGROUPID>", "</ELEMENTGROUPID>");
                }
                return ErrorXML;

            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_CREATEFORMAT_FAILED", this.GetType().ToString(), "CreateFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End CreateFormat");
            }
        }

        public ERRORXML CreateFormat_HTML(ref String RelationShipName, ref String RelatedObjectID, ref XML FormatXML, ref String FormatID, ref String PageID, ref String GroupID, out String FormatStatus)
        {
            #region variable
            StringBuilder sbCreatedOn = null;
            String[] Separator = { "$AL$" };
            StringBuilder sbRelationShip = null;

            FormatStatus = null;

            #endregion
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin CreateFormat");
                if (RelationShipName == null)
                {
                    RelationShipName = "";
                    RelatedObjectID = "";
                }
                #region To add 'Creator' , if FormatXML doesn't contain it.
                if (!FormatXML.Contains("FORMAT_CREATOR"))
                {
                    if (!RelationShipName.Contains("FORMAT_CREATOR"))
                    {
                        RelationShipName = RelationShipName + "$AL$" + "FORMAT_CREATOR";
                        RelatedObjectID = RelatedObjectID + "$AL$" + UserID;
                    }
                }
                #endregion

                #region To add 'CreatedOn' date, if ArtworkXML doesn't contain it.
                if (!FormatXML.Contains("<CREATEDON>"))
                {
                    sbCreatedOn = new StringBuilder();
                    sbCreatedOn.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                            , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), sbCreatedOn.ToString());
                }
                #endregion

                #region Associate Format with Given Related Entity/Facet i.e RelationShipName and RelatedObjectID
                if ((RelationShipName != null) && (RelationShipName != "") && (RelatedObjectID != null) && (RelatedObjectID != ""))
                {
                    String[] RelationNameArray = RelationShipName.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                    String[] RelatedObjectIDArray = RelatedObjectID.Split(Separator, StringSplitOptions.RemoveEmptyEntries);

                    if (RelationNameArray.Length == RelatedObjectIDArray.Length)
                    {
                        sbRelationShip = new StringBuilder();
                        for (int i = 0; i < RelationNameArray.Length; i++)
                        {
                            sbRelationShip.AppendFormat("<{0} TYPE=\"RELATION\">{1}</{2}>", RelationNameArray[i], RelatedObjectIDArray[i], RelationNameArray[i]);
                        }
                        FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), sbRelationShip.ToString());
                    }
                    //else
                    //  return "Mismath RelationShip Names and RelationShip IDs";
                }
                #endregion

                ErrorXML = ArtworkProducerWSobj.CreateFormat_HTML(ref FormatXML, out FormatStatus);
                if (ErrorXML == null)
                {
                    FormatID = ALCommon.GetInnerValue(ref FormatXML, "<FORMATID>", "</FORMATID>");
                    PageID = ALCommon.GetInnerValue(ref FormatXML, "<ARTWORKPAGEID>", "</ARTWORKPAGEID>");
                    GroupID = ALCommon.GetInnerValue(ref FormatXML, "<ELEMENTGROUPID>", "</ELEMENTGROUPID>");
                }
                return ErrorXML;

            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_CREATEFORMAT_FAILED", this.GetType().ToString(), "CreateFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End CreateFormat");
            }
        }

        /// <alsummary>
        /// <createddate>27 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to update Format based on the input XML.</purpose>
        ///  <xmlstructure><![CDATA[
        /// 
        ///     Input XML :         |-  [FORMAT @MODIFIED="TRUE"]
        ///                                       - $Attributes$
        ///                                       |-    [$RelationShipName$ @TYPE="RELATION"  @DISASSOCIATE="TRUE/FALSE"]text()=$Associated Entity/Facet Id$     // Associate or Disassociate existing instance with TargetEntity                    
        ///                                                  |-  [$Associated Entity/Facet Name$  @MODIFIED="TRUE"]        //it will create new instance of associated entity or facet
        ///                                                                                                                 //,if Modified is not specified.
        ///                                                           |-  $Attributes                                       // and associates it with target entity
        /// 
        /// ]]></xmlstructure>
        /// <param name="FormatXML"></param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML UpdateFormat(ref XML FormatXML)
        {
            try
            {
                ErrorXML = UpdateFormat(ref FormatXML, true);

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_UPDATEFORMAT_FAILED", this.GetType().ToString(), "UpdateFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End UpdateFormat");
            }
        }

        public ERRORXML UpdateFormat(ref XML FormatXML, bool UpdateCache)
        {
            #region variable
            StringBuilder StringBuilderObj = null;

            #endregion
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin UpdateFormat");
                #region To add 'ModifiedOn' date, if ArtworkXML doesn't contain it.
                XmlReader objReader = XmlReader.Create(new StringReader(FormatXML));
                objReader.MoveToContent();
                if (!objReader.ReadToDescendant("MODIFIEDON"))
                {
                    StringBuilderObj = new StringBuilder();
                    StringBuilderObj.AppendFormat("<MODIFIEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></MODIFIEDON>"
                                                            , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), StringBuilderObj.ToString());
                    StringBuilderObj.Remove(0, StringBuilderObj.Length);
                }
                objReader.Close();

                #endregion

                #region To add 'Modifier' , if ArtworkXML doesn't contain it.
                if (!FormatXML.Contains("FORMAT_MODIFIER"))
                {
                    StringBuilderObj.AppendFormat("<FORMAT_MODIFIER TYPE=\"RELATION\">{0}</FORMAT_MODIFIER>", UserID);
                    FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), StringBuilderObj.ToString());
                    StringBuilderObj.Remove(0, StringBuilderObj.Length);
                }
                #endregion

                ErrorXML = ArtworkProducerWSobj.UpdateFormat(ref FormatXML, UpdateCache);

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_UPDATEFORMAT_FAILED", this.GetType().ToString(), "UpdateFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End UpdateFormat");
            }
        }

        public ERRORXML UpdateFormat_HTML(ref XML FormatXML, bool UpdateCache, out String FormatStatus)
        {
            #region variable
            StringBuilder StringBuilderObj = null;
            FormatStatus = null;
            #endregion
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin UpdateFormat");
                #region To add 'ModifiedOn' date, if ArtworkXML doesn't contain it.
                XmlReader objReader = XmlReader.Create(new StringReader(FormatXML));
                objReader.MoveToContent();
                if (!objReader.ReadToDescendant("MODIFIEDON"))
                {
                    StringBuilderObj = new StringBuilder();
                    StringBuilderObj.AppendFormat("<MODIFIEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></MODIFIEDON>"
                                                            , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), StringBuilderObj.ToString());
                    StringBuilderObj.Remove(0, StringBuilderObj.Length);
                }
                objReader.Close();

                #endregion

                #region To add 'Modifier' , if ArtworkXML doesn't contain it.
                if (!FormatXML.Contains("FORMAT_MODIFIER"))
                {
                    StringBuilderObj.AppendFormat("<FORMAT_MODIFIER TYPE=\"RELATION\">{0}</FORMAT_MODIFIER>", UserID);
                    FormatXML = FormatXML.Insert(FormatXML.IndexOf("</FORMAT>"), StringBuilderObj.ToString());
                    StringBuilderObj.Remove(0, StringBuilderObj.Length);
                }
                #endregion

                ErrorXML = ArtworkProducerWSobj.UpdateFormat_HTML(ref FormatXML, UpdateCache, out FormatStatus);

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_UPDATEFORMAT_FAILED", this.GetType().ToString(), "UpdateFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End UpdateFormat");
            }
        }

        /// <alsummary>
        /// <createddate>27 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to Get Formats based on the FilterXML and FieldCollectionXML.</purpose>
        ///  <xmlstructure><![CDATA[
        /// 
        /// FilterXML       :   |-[SEARCH]
        ///                         |-  [TARGETOBJECT]
        ///                         |-  [TARGETOBJECTTYPE]
        ///                         |-  [ISCASESENSITIVE]
        ///                         |-  [FILTERLIST]
        ///                            |-  [FILTER]
        ///                                         |-  [FIELDNAME]
        ///                                         |-  [OPERATOR]
        ///                                         |-  [FIELDVALUE]
        ///                                         |-  [NAVIGATIONPATH]
        ///                                         |-  [ISCASESENSITIVE]
        ///                            |-   ...
        ///                            |_   ...
        /// 
        /// 
        /// FieldCollectiionXML :   |-[FIELDCOLLECTION]
        ///                                 |-  [TARGETENTITYFIELDS]
        ///                                             |-  [FIELD]
        ///                                             |-  ...
        ///                                 |-  [RELATEDENTITYFIELDS @NavigationPath] 
        ///                                             |-  [FIELD]
        ///                                             |-  ...
        ///                                 |-  [RELATEDFACETFIELDS @NavigationPath]           
        ///                                             |-  [FIELD]
        ///                                             |-  ...
        ///                                 |-      ...
        /// 
        /// FormatListXML      :   -[FORMATCOLLECTION]
        ///                                 |-  [FORMAT]
        ///                                 |-  ...
        /// ]]></xmlstructure>
        /// <param name="FilterXML">Specifies Filters , used in search</param>
        /// <param name="FieldCollectionXML">Specifies Target ,Related entity /facet fields</param>
        /// <param name="FormatListXML">Collection of Formats</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML GetAllFormats(XML FilterXML, XML FieldCollectionXML, out XML FormatListXML)
        {
            #region variable

            #endregion
            FormatListXML = null;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin GetAllFormats");
                ErrorXML = ArtworkProducerWSobj.GetAllFormats(FilterXML, FieldCollectionXML, out FormatListXML);
                if (ErrorXML != null)
                {
                    FormatListXML = null;
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_GETALLFORMATS_FAILED", this.GetType().ToString(), "GetAllFormats", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End GetAllFormats");
            }
        }

        /// <alsummary>
        ///     <createddate>13 December 2006</createddate>
        ///     <author>Abhijeet Yadwadkar</author>
        /// </alsummary>
        /// <param name="FormatID"></param>
        /// <param name="FieldCollectionXML"></param>
        /// <param name="FormatDetailstXML"></param>
        /// <returns></returns>
        public ERRORXML GetFormat(String FormatID, ref XML FieldCollectionXML, out XML FormatDetailsXML)
        {
            FormatDetailsXML = null;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin GetFormat");
                ErrorXML = ArtworkProducerWSobj.GetFormat(FormatID, ref FieldCollectionXML, out FormatDetailsXML);
                if (ErrorXML != null)
                {
                    FormatDetailsXML = null;
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_GETFORMAT_FAILED", this.GetType().ToString(), "GetFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End GetFormat");
            }

        }

        /// <summary>
        ///     <createddate>22 December 2006</createddate>
        ///     <author>Pranjali Bhide</author>
        /// </summary>
        /// <param name="FormatID">Format ID</param>
        /// <param name="TEPrefix">Text Elements Prefix</param> 
        /// <param name="maxTECount">Total No. of TextElements present in format depending upon TEPrefix</param>
        /// <param name="minTECount">Minimum no of TextElements that should be present in format</param>
        /// <param name="FormatDetailsXML">Format Details XML contains TextElements starting with given TEPrefix.
        ///     if TEPrefix = null, then it will return all TextElements.
        /// </param> 
        /// <returns></returns>
        public ERRORXML GetTextElements(String FormatID, String TEPrefix, out int maxTECount, out XML FormatDetailsXML)
        {
            #region Variables
            String ErrorXML = null;
            XPathDocument xpDoc_OutputXML = null;
            XPathNavigator XNav = null;
            XPathNodeIterator TEListNodes = null;
            String convertor = "'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'";
            StringBuilder SB_FormatDetailsXML = null;
            XmlTextReader xReader = null;
            StringBuilder sb_TEXTTAGNAMEFilter = null;
            #endregion
            maxTECount = 0;
            FormatDetailsXML = null;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin GetTextElements");
                //Get Format Details using Nsearch
                #region Create Nsearch input XML

                sb_TEXTTAGNAMEFilter = new StringBuilder();
                sb_TEXTTAGNAMEFilter.AppendFormat("");
                if (TEPrefix != null)
                {
                    sb_TEXTTAGNAMEFilter.AppendFormat(@"<FILTERLIST>
                                                            <FILTER>
                                                                <EXPRESSION>TEXTTAGNAME</EXPRESSION>
                                                                <OPERATOR>STARTSWITH</OPERATOR>
                                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                                <FILTERID>1</FILTERID>
                                                                <ISCASESENSITIVE>FALSE</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>", TEPrefix);
                }
                else
                    sb_TEXTTAGNAMEFilter.AppendFormat("");
                StringBuilder sb_NsearchInputXML = new StringBuilder();
                sb_NsearchInputXML.AppendFormat(@"<SEARCH>
                    <TARGETOBJECT>FORMAT</TARGETOBJECT>
                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                    <SEARCHSTRING></SEARCHSTRING>
                    <TARGETENTITY>
                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                        <FILTERLIST>
                            <FILTER>
                                <EXPRESSION>FORMATID</EXPRESSION>
                                <OPERATOR>EQUALTO</OPERATOR>
                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                <FILTERID>1</FILTERID>
                                <ISCASESENSITIVE>FALSE</ISCASESENSITIVE>
                            </FILTER>
                        </FILTERLIST>
                        <SELECTLIST>
			                <EXPRESSION>
				                <FIELDNAME>FORMATID</FIELDNAME>
				                <ALIAS>FORMATID</ALIAS>
			                </EXPRESSION>				                
		                </SELECTLIST>
                        <RELATIONSHIP NAME='FORMAT_ARTWORKPAGE'>
                            <SELECTLIST>
				                <EXPRESSION>
					                <FIELDNAME>ARTWORKPAGEID</FIELDNAME>
					                <ALIAS>ARTWORKPAGEID</ALIAS>
				                </EXPRESSION>				                
			                </SELECTLIST>
                            <RELATIONSHIP NAME='ARTWORKPAGE_ELEMENTGROUP'>  
                                <SELECTLIST>
				                    <EXPRESSION>
					                    <FIELDNAME>ELEMENTGROUPID</FIELDNAME>
					                    <ALIAS>ELEMENTGROUPID</ALIAS>
				                    </EXPRESSION>				                
			                    </SELECTLIST>                             
                                <RELATIONSHIP NAME='ELEMENTGROUP_TEXTELEMENT'>                                    
                                    {1}
                                </RELATIONSHIP>                                            
                            </RELATIONSHIP>
                        </RELATIONSHIP>
                    </TARGETENTITY>
                </SEARCH>", FormatID, sb_TEXTTAGNAMEFilter.ToString());

                String OutputXML = null;
                ErrorXML = ArtworkProducerWSobj.SearchFormatElements(sb_NsearchInputXML.ToString(), out OutputXML);
                //Thread.Sleep(5000);   
                if (ErrorXML != null)
                    OutputXML = null;

                #endregion


                //ErrorXML = GetFormatDetails(ref OutputXML);


                //if (ErrorXML != null && OutputXML == null)
                if (OutputXML == null)
                {
                    maxTECount = 0;
                    FormatDetailsXML = null;
                    return ErrorXML;
                    //ErrorXML = ArtworkProducerWSobj.SearchFormatElements(sb_NsearchInputXML.ToString(), out OutputXML);     
                }

                SB_FormatDetailsXML = new StringBuilder();
                //xpDoc_OutputXML = new XPathDocument(OutputXML);
                //xReader = new XmlTextReader("c:\\FormatDetailsXML.xml");
                Stream strXML = new System.IO.MemoryStream(ASCIIEncoding.Default.GetBytes(OutputXML));
                xReader = new XmlTextReader(strXML);
                xpDoc_OutputXML = new XPathDocument(new StringReader(OutputXML));
                XNav = xpDoc_OutputXML.CreateNavigator();

                if (XNav.SelectSingleNode("//FORMAT") != null)
                {
                    SB_FormatDetailsXML.AppendFormat("<Format FormatID='{0}'>", XNav.SelectSingleNode("//FORMAT/FORMATID").Value);

                    SB_FormatDetailsXML.AppendFormat("<TEXTELEMENTLIST>");
                    if (TEPrefix == null)
                        TEListNodes = XNav.Select("descendant::TEXTELEMENT");
                    else
                    {
                        //String xPathQuery = "(starts-with(translate(TEXTTAGNAME/text()," + convertor + "),'" + TEPrefix.ToUpper() + "')) ";
                        //TEListNodes = XNav.Select("descendant::TEXTELEMENT['" + xPathQuery + "']");
                        TEListNodes = XNav.Select("descendant::TEXTELEMENT[(starts-with(translate(TEXTTAGNAME/text()," + convertor + "),'" + TEPrefix.ToUpper() + "') )]");
                    }

                    maxTECount = TEListNodes.Count;
                    while (TEListNodes.MoveNext())
                    {
                        if (TEListNodes.Current != null)
                        {
                            SB_FormatDetailsXML.Append(TEListNodes.Current.OuterXml);
                        }
                    }

                    SB_FormatDetailsXML.AppendFormat("</TEXTELEMENTLIST>");
                    SB_FormatDetailsXML.AppendFormat("</Format>");
                }
                if (TEListNodes != null)
                    maxTECount = TEListNodes.Count;
                else
                    maxTECount = 0;
                if (SB_FormatDetailsXML != null)
                    FormatDetailsXML = SB_FormatDetailsXML.ToString();
                else
                    FormatDetailsXML = null;
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_GETTEXTELEMENTS_FAILED", this.GetType().ToString(), "GetTextElements", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End GetTextElements");
            }
        }

        /// <summary>
        ///     <createddate>26 December 2006</createddate>
        ///     <author>Pranjali Bhide</author>
        /// </summary>
        /// <param name="FormatID">Format ID</param>
        /// <param name="GEPrefix">Graphic Elements Prefix</param>
        /// <param name="FormatDetailsXML">Format Details XML contains GraphicElements starting with given GEPrefix.
        ///     if GEPrefix = null, then it will return all GraphicElements.
        /// </param>
        /// <returns></returns>
        public ERRORXML GetGraphicElements(String FormatID, String GEPrefix, out XML FormatDetailsXML)
        {
            #region Variables
            String ErrorXML = null;
            XPathDocument xpDoc_OutputXML = null;
            XPathNavigator XNav = null;
            XPathNodeIterator GEListNodes = null;
            String convertor = "'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'";
            StringBuilder SB_FormatDetailsXML = null;
            XmlTextReader xReader = null;
            StringBuilder sb_GRAPHICTAGNAMEFilter = null;
            #endregion
            FormatDetailsXML = null;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin GetGraphicElements");
                #region Create Nsearch input XML
                sb_GRAPHICTAGNAMEFilter = new StringBuilder();
                sb_GRAPHICTAGNAMEFilter.AppendFormat("");
                if (GEPrefix != null)
                {
                    sb_GRAPHICTAGNAMEFilter.AppendFormat(@"<FILTERLIST>
                                                            <FILTER>
                                                                <EXPRESSION>GRAPHICTAGNAME</EXPRESSION>
                                                                <OPERATOR>STARTSWITH</OPERATOR>
                                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                                <FILTERID>1</FILTERID>
                                                                <ISCASESENSITIVE>FALSE</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>", GEPrefix);
                }
                else
                    sb_GRAPHICTAGNAMEFilter.AppendFormat("");

                StringBuilder sb_NsearchInputXML = new StringBuilder();
                sb_NsearchInputXML.AppendFormat(@"<SEARCH>
                                                <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                <SEARCHSTRING></SEARCHSTRING>
                                                <TARGETENTITY>
                                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <FILTERLIST>
                                                        <FILTER>
                                                            <EXPRESSION>FORMATID</EXPRESSION>
                                                            <OPERATOR>EQUALTO</OPERATOR>
                                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                            <FILTERID>1</FILTERID>
                                                            <ISCASESENSITIVE>FALSE</ISCASESENSITIVE>
                                                        </FILTER>
                                                    </FILTERLIST>
                                                    <SELECTLIST>
			                                            <EXPRESSION>
				                                            <FIELDNAME>FORMATID</FIELDNAME>
				                                            <ALIAS>FORMATID</ALIAS>
			                                            </EXPRESSION>				                
		                                            </SELECTLIST>
                                                    <RELATIONSHIP NAME='FORMAT_ARTWORKPAGE'>
                                                        <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>ARTWORKPAGEID</FIELDNAME>
					                                            <ALIAS>ARTWORKPAGEID</ALIAS>
				                                            </EXPRESSION>				                
			                                            </SELECTLIST>
                                                        <RELATIONSHIP NAME='ARTWORKPAGE_ELEMENTGROUP'>
                                                            <SELECTLIST>
				                                                <EXPRESSION>
					                                                <FIELDNAME>ELEMENTGROUPID</FIELDNAME>
					                                                <ALIAS>ELEMENTGROUPID</ALIAS>
				                                                </EXPRESSION>				                
			                                                </SELECTLIST>
                                                            <RELATIONSHIP NAME='ELEMENTGROUP_GRAPHICELEMENT'>
                                                                <SELECTLIST>
				                                                    <EXPRESSION>
					                                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
					                                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
				                                                    </EXPRESSION>
                                                                    <EXPRESSION>
					                                                    <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
					                                                    <ALIAS>GRAPHICTAGNAME</ALIAS>
				                                                    </EXPRESSION>				                
			                                                    </SELECTLIST>    
                                                            {1}    
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                    </RELATIONSHIP>
                                                </TARGETENTITY>
                                            </SEARCH>", FormatID, sb_GRAPHICTAGNAMEFilter.ToString());

                String OutputXML = null;
                ErrorXML = ArtworkProducerWSobj.SearchFormatElements(sb_NsearchInputXML.ToString(), out OutputXML);
                //Thread.Sleep(5000);  
                if (ErrorXML != null)
                    OutputXML = null;

                #endregion
                //ErrorXML = GetFormatDetails(ref OutputXML);

                //if (ErrorXML != null && OutputXML == null)
                if (OutputXML == null)
                {
                    FormatDetailsXML = null;
                    return ErrorXML;
                    //ErrorXML = ArtworkProducerWSobj.SearchFormatElements(sb_NsearchInputXML.ToString(), out OutputXML);
                }

                SB_FormatDetailsXML = new StringBuilder();
                //xpDoc_OutputXML = new XPathDocument(OutputXML);
                //xReader = new XmlTextReader("c:\\FormatDetailsXML.xml");
                Stream strXML = new System.IO.MemoryStream(ASCIIEncoding.Default.GetBytes(OutputXML));
                xReader = new XmlTextReader(strXML);
                xpDoc_OutputXML = new XPathDocument(new StringReader(OutputXML));
                XNav = xpDoc_OutputXML.CreateNavigator();

                if (XNav.SelectSingleNode("//FORMAT") != null)
                {
                    SB_FormatDetailsXML.AppendFormat("<Format FormatID='{0}'>", XNav.SelectSingleNode("//FORMAT/FORMATID").Value);

                    SB_FormatDetailsXML.AppendFormat("<GRAPHICELEMENTLIST>");
                    if (GEPrefix == null)
                        GEListNodes = XNav.Select("descendant::GRAPHICELEMENT");
                    else
                    {
                        //String xPathQuery = "(starts-with(translate(TEXTTAGNAME/text()," + convertor + "),'" + TEPrefix.ToUpper() + "')) ";
                        //TEListNodes = XNav.Select("descendant::TEXTELEMENT['" + xPathQuery + "']");
                        GEListNodes = XNav.Select("descendant::GRAPHICELEMENT[(starts-with(translate(GRAPHICTAGNAME/text()," + convertor + "),'" + GEPrefix.ToUpper() + "') )]");
                    }


                    while (GEListNodes.MoveNext())
                    {
                        if (GEListNodes.Current != null)
                        {
                            SB_FormatDetailsXML.Append(GEListNodes.Current.OuterXml);
                        }
                    }

                    SB_FormatDetailsXML.AppendFormat("</GRAPHICELEMENTLIST>");
                    SB_FormatDetailsXML.AppendFormat("</Format>");
                }


                FormatDetailsXML = SB_FormatDetailsXML.ToString();
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_GETGRAPHICELEMENTS_FAILED", this.GetType().ToString(), "GetGraphicElements", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End GetGraphicElements");
            }
        }

        public ERRORXML DeleteFormat(String FormatIDList)
        {
            #region Local Variables

            ERRORXML ErrorXML = null;

            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin DeleteFormat");
                ErrorXML = ArtworkProducerWSobj.DeleteFormat(FormatIDList);
                if (ErrorXML != null)
                {
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_DELETEFORMAT_FAILED", this.GetType().ToString(), "DeleteFormat", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End DeleteFormat");
            }
        }

        public String DownloadFormatFiles(String FormatID)
        {
            String RequestID = Guid.NewGuid().ToString();
            String str_DownloadDirPath = null;

            str_DownloadDirPath = PrepareDownloadFolder(RequestID, true);

            UpdateDownloadFormatStatus(RequestID, FormatID, FormatDownloadStatus.RetrivingFormatFiles);

            ArrayList objParams = new ArrayList(3);
            objParams.Add(RequestID);
            objParams.Add(FormatID);
            objParams.Add(str_DownloadDirPath);

            OrgIdentity = System.Security.Principal.WindowsIdentity.GetCurrent();
            ThreadPool.QueueUserWorkItem(new WaitCallback(ProcessDownloadFormatRequest), objParams);
            return RequestID;
        }

        public FormatDownloadStatus GetDownloadFormatStatus(String RequestID, String FormatID, out String ZipFilePath)
        {
            ZipFilePath = null;
            String str_DownloadPath = PrepareDownloadFolder(RequestID, false);
            FormatDownloadStatus status = GetDownloadFormatStatus(RequestID, FormatID);
            if (status == FormatDownloadStatus.Done)
            {
                String ZipFileName = null;
                ZipFileName = str_DownloadPath + "\\ZipName.txt";

                ZipFilePath = str_DownloadPath + "\\" + File.ReadAllText(ZipFileName);

                return FormatDownloadStatus.Done;
            }

            return status;
        }

        public ERRORXML GetPageListTaskPaneXML(ref String TaskPaneName, String RedirectURL, String EventHandling, String JSName, String FormatID, String PrimaryKeyField, String RelationshipNameWithFormat, String SelectedPageID, String SolutionName, String Mode, String ArtworkID, ref String OutputXML)
        {
            #region Variables
            ERRORXML ErrorXML = null;
            String URL = null;
            bool IsTagDetailsXMLPresent = false;
            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin GetPageListTaskPaneXML");
                ArtworkPageFacade objPageFacade = new ArtworkPageFacade(SessionID, UserID, TenantName, CurrentLanguage);

                String FieldCollectionXML = @"<FIELDCOLLECTION>
                                            <TARGETENTITYFIELDS>
                                            <FIELD>ARTWORKPAGEID</FIELD>
                                            <FIELD>ARTWORKPAGETITLE</FIELD>
                                            <FIELD>ARTWORKPAGENAME</FIELD>
                                            <FIELD>ARTWORKPAGEORDER</FIELD>
                                            </TARGETENTITYFIELDS>
                                            <RELATEDENTITYFIELDS NavigationPath='ARTWORKPAGE.FORMAT_ARTWORKPAGE'>
                                            <FIELD>TAGDETAILS</FIELD>
                                            </RELATEDENTITYFIELDS>
                                            </FIELDCOLLECTION>";

                String PageListXML = null;

                ErrorXML = objPageFacade.GetArtworkPages(FormatID, PrimaryKeyField, RelationshipNameWithFormat, null, FieldCollectionXML, out PageListXML);

                if (ErrorXML == null && PageListXML != null)
                {
                    XPathDocument xdoc = new XPathDocument(new StringReader(PageListXML));
                    XPathExpression xpExpr = XPathExpression.Compile("//ARTWORKPAGE");
                    xpExpr.AddSort("ARTWORKPAGEORDER/text()", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Text);

                    XPathNodeIterator Page_Iter = xdoc.CreateNavigator().Select(xpExpr);
                    StringBuilder sbPaneXML = new StringBuilder();
                    if (String.IsNullOrEmpty(TaskPaneName))
                        TaskPaneName = "AdDirect";

                    sbPaneXML.Append("<TaskPanes><TaskPane TaskPaneName='" + TaskPaneName + "' VAlignment='Top' Visible='false' Order='1' InputXMLType='InnerXML'>");
                    sbPaneXML.Append("<TaskPaneDetails TaskPaneTitle='Customize' DisplayTextID='AP_FormatList_Customize' IsMinMaxRequired='True' IsHeaderPresent='True' DisplayMode='Maximized' ImageAlignment='left' VAlignment='right' HeadingAlignment='left'>");

                    XPathNavigator xpNav_Temp = xdoc.CreateNavigator().SelectSingleNode("ARTWORKPAGECOLLECTION/ARTWORKPAGE/FORMAT_ARTWORKPAGE/FORMAT/FILESYSTEM/FILE[FIELDNAME='TAGDETAILS']");
                    if (xpNav_Temp != null)
                    {
                        IsTagDetailsXMLPresent = true;
                    }
                    else
                    {
                        IsTagDetailsXMLPresent = false;
                    }

                    while (Page_Iter.MoveNext())
                    {
                        if ((RedirectURL.Contains("&CustomizePrevPage")) && (Page_Iter.CurrentPosition != 1))
                            // RedirectURL=RedirectURL.Remove(RedirectURL.IndexOf("&CustomizePrevPage"));
                            RedirectURL = RedirectURL.Replace("&CustomizePrevPage", "&CustomizeFirstPage");


                        XPathNavigator nav_Current = Page_Iter.Current;
                        String SectionName = nav_Current.SelectSingleNode("ARTWORKPAGENAME").InnerXml.ToString().Replace(" ", "_");
                        // if ((RedirectURL == null) || (RedirectURL == ""))
                        String Param = "UserID=" + UserID + "&TenantName=" + TenantName + "&Language=" + CurrentLanguage + "&SessionID=" + SessionID + "&SolutionName=" + SolutionName
                                                                                + "&Mode=" + Mode + "&FormatID=" + FormatID + "&RelationShipName=" + RelationshipNameWithFormat + "&PageId=" + nav_Current.SelectSingleNode("ARTWORKPAGEID").InnerXml + "&PageName=" + System.Web.HttpUtility.UrlEncode(nav_Current.SelectSingleNode("ARTWORKPAGENAME").Value) + "&ArtworkID=" + ArtworkID + "&PageTitle=" + System.Web.HttpUtility.UrlEncode(nav_Current.SelectSingleNode("ARTWORKPAGETITLE").Value) + "&IsTagDetailsXMLPresent=" + IsTagDetailsXMLPresent;
                        ALSolutionCommon ALSolCmnObj = new ALSolutionCommon();

                        if (RedirectURL.Contains("?"))
                            URL = @"/" + TenantName + "/" + SolutionName + "/ASPX/" + RedirectURL + "&PageId=" + nav_Current.SelectSingleNode("ARTWORKPAGEID").InnerXml + "&PageName=" + System.Web.HttpUtility.UrlEncode(nav_Current.SelectSingleNode("ARTWORKPAGENAME").Value) + "&PageTitle=" + System.Web.HttpUtility.UrlEncode(nav_Current.SelectSingleNode("ARTWORKPAGETITLE").Value) + "&IsTagDetailsXMLPresent=" + IsTagDetailsXMLPresent;
                        else
                            URL = @"/" + TenantName + "/" + SolutionName + "/ASPX/" + RedirectURL + "?" + Param;

                        URL = ALSolCmnObj.EncodeURLForResponseRedirects(URL);
                        if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                        {
                            // URL= "javascript:SubmitPage('" + RedirectURL + "_" + SectionName + "_" + Page_Iter.CurrentPosition.ToString() + "');";
                            if ((RedirectURL == null) || (RedirectURL == ""))
                                sbPaneXML.Append("<Section SectionType='Text' Order='" + Page_Iter.CurrentPosition + "' Target='_self' URL='' SectionName='" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(nav_Current.SelectSingleNode("ARTWORKPAGENAME").Value) + "' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + nav_Current.SelectSingleNode("ARTWORKPAGEID").InnerXml + "$AL$" + Page_Iter.CurrentPosition + "'");
                            else
                                sbPaneXML.Append("<Section SectionType='Text' Order='" + Page_Iter.CurrentPosition + "' Target='_self' URL='' SectionName='" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(nav_Current.SelectSingleNode("ARTWORKPAGENAME").Value) + "' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + URL + "'");
                            // else
                            //   sbPaneXML.Append("<Section SectionType='Text' Order='" + Page_Iter.CurrentPosition + "' Target='_self' URL='' SectionName='" + nav_Current.SelectSingleNode("ARTWORKPAGENAME").InnerXml + "' Handling='JavascriptEvent' Javascript='SubmitPage' Event='" + nav_Current.SelectSingleNode("ARTWORKPAGEID").InnerXml + "'");
                        }
                        else if (EventHandling.ToUpper().Equals("URL"))
                        {


                            sbPaneXML.Append("<Section SectionType='Text' Order='" + Page_Iter.CurrentPosition + "' Target='_self' URL='" + URL + "' SectionName='" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(nav_Current.SelectSingleNode("ARTWORKPAGENAME").Value) + "' Handling='URL' Event=''");
                        }
                        if (SelectedPageID != null)
                        {
                            if ((Page_Iter.CurrentPosition == 1) && (SelectedPageID.ToUpper().Equals("DEFAULT")))
                                sbPaneXML.Append(" Selected='true' ");
                            else if (nav_Current.SelectSingleNode("ARTWORKPAGEID").InnerXml == SelectedPageID)
                                sbPaneXML.Append(" Selected='true' ");
                        }
                        sbPaneXML.Append(">");
                        sbPaneXML.Append("<Text Title='" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(nav_Current.SelectSingleNode("ARTWORKPAGETITLE").Value) + "' Order='2'/> </Section>");

                    }

                    sbPaneXML.Append("</TaskPaneDetails></TaskPane></TaskPanes>");

                    OutputXML = sbPaneXML.ToString();
                }
                else
                {
                    OutputXML = null;
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_GETPAGELISTTASKPANEXML_FAILED", this.GetType().ToString(), "GetPageListTaskPaneXML", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End GetPageListTaskPaneXML");
            }

        }

        public ERRORXML CreateTaskPaneXML(String TaskPaneName, String SelectedPageID, String SolutionName, String FormatID, String ArtworkID, String Mode, String ProfileID, String UserInfoChoice, String ContactInfoChoice, String RedirectURLForPage, ref String OutputXML)
        {
            #region Variables
            StringBuilder sbSectionTagXML = null;
            String ALParamString = "";
            String SelectedURL = "";
            String ExistingTaskPaneXML = null;
            ALSolutionCommon ObjAlCommon = new ALSolutionCommon();
            String ErrorXML;
            String CustomizePrevPage = null;
            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin CreateTaskPaneXML");
                #region Common Parameters for all pages
                ALParamString = ALParamString + "?TenantName=" + TenantName + "&UserID=" + UserID + "&SessionID=" + SessionID + "&Language=" + CurrentLanguage;
                ALParamString = ALParamString + "&SolutionName=" + SolutionName;
                ALParamString = ALParamString + "&ArtworkID=" + ArtworkID;
                ALParamString = ALParamString + "&FormatID=" + FormatID;
                ALParamString = ALParamString + "&ProfileID=" + ProfileID;
                //   ALParamString = ALParamString + "&Mode=" + Mode;
                if ((UserInfoChoice != null) && (UserInfoChoice != ""))
                    ALParamString = ALParamString + "&UserInfoChoice=" + UserInfoChoice;
                if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                    ALParamString = ALParamString + "&ContactInfoChoice=" + ContactInfoChoice;

                #endregion
                sbSectionTagXML = new StringBuilder();

                #region Edit Ad
                SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("/" + TenantName + "/" + SolutionName + "/ASPX/Artwork_Generation_ArtworkEdit.aspx" + ALParamString + "&Mode=EDIT&ActualMode=" + Mode);
                sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='{0}' ", SelectedURL);
                if (SelectedPageID != null && SelectedPageID.ToUpper().Equals("EDITAD"))
                    sbSectionTagXML.Append(" Selected='true'");
                sbSectionTagXML.Append(">");


                sbSectionTagXML.AppendFormat("<Text Title='Name Your Ad'/>");
                //sbSectionTagXML.AppendFormat(ALParamString.ToString());    
                sbSectionTagXML.AppendFormat("</Section>");

                #endregion
                ALParamString = ALParamString + "&Mode=" + Mode;

                #region Create UserChoice Page
                if ((UserInfoChoice == null) || (!UserInfoChoice.ToUpper().Equals("NONE")))
                {

                    SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_Choices.aspx" + ALParamString);
                    CustomizePrevPage = "&CustomizePrevPage=UserInformation_Choices.aspx";
                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0)
                        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='{0}' Selected='true'>", SelectedURL);
                    else
                        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='{0}'>", SelectedURL);
                    sbSectionTagXML.AppendFormat("<Text Title='Ad Information Options'/>");
                    //sbSectionTagXML.AppendFormat(ALParamString.ToString());    
                    sbSectionTagXML.AppendFormat("</Section>");
                }
                #endregion

                #region Create AdvisorInformation
                if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                {
                    SelectedURL = "";
                    if (ContactInfoChoice.ToUpper().CompareTo("SINGLEADVISOR") == 0)
                    {
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_SingleAdv_Other.aspx" + ALParamString);
                        CustomizePrevPage = "&CustomizePrevPage=UserInformation_SingleAdv_Other.aspx";
                    }
                    else if (ContactInfoChoice.ToUpper().CompareTo("TWOADVISORS_SAME") == 0)
                    {
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_TwoAdv_Same.aspx" + ALParamString);
                        CustomizePrevPage = "&CustomizePrevPage=UserInformation_TwoAdv_Same.aspx";
                    }
                    else if (ContactInfoChoice.ToUpper().CompareTo("TWOADVISORS_DIFFERENT") == 0)
                    {
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_TwoAdv_Different.aspx" + ALParamString);
                        CustomizePrevPage = "&CustomizePrevPage=UserInformation_TwoAdv_Different.aspx";
                    }

                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0)
                        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Target='_self' URL='{0}' Selected='true'>", SelectedURL);
                    else
                        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Target='_self' URL='{0}'>", SelectedURL);
                    sbSectionTagXML.AppendFormat("<Text Title='Advisor Information'/>");
                    //sbSectionTagXML.AppendFormat(ALParamString.ToString());    
                    sbSectionTagXML.AppendFormat("</Section>");
                }
                #endregion

                #region Create AdvisorPhoto1
                if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                {
                    SelectedURL = "";
                    SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UploadPhoto_Page.aspx" + ALParamString);
                    CustomizePrevPage = "&CustomizePrevPage=UploadPhoto_Page.aspx";
                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORPHOTO1") == 0)
                        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Target='_self' URL='{0}' Selected='true'>", SelectedURL);
                    else
                        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Target='_self' URL='{0}'>", SelectedURL);
                    sbSectionTagXML.AppendFormat("<Text Title='Upload Photo'/>");
                    //sbSectionTagXML.AppendFormat(ALParamString.ToString());
                    sbSectionTagXML.AppendFormat("</Section>");
                }
                #endregion

                #region Create AdvisorPhoto2
                //if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                //{
                //    if (ContactInfoChoice.ToUpper().Contains("TWO") == true)
                //    {
                //        SelectedURL = "";
                //        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("ContactInfo_UploadPhoto_Second_Page.aspx" + ALParamString);
                //        CustomizePrevPage = "&CustomizePrevPage=ContactInfo_UploadPhoto_Second_Page.aspx";
                //        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORPHOTO2") == 0)
                //            sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Target='_self' URL='{0}' Selected='true'>", SelectedURL);                        
                //        else
                //        sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Target='_self' URL='{0}'>", SelectedURL);
                //        sbSectionTagXML.AppendFormat("<Text Title='Upload Photo for Second Advisor'/>");
                //        //sbSectionTagXML.AppendFormat(ALParamString.ToString());
                //        sbSectionTagXML.AppendFormat("</Section>");
                //    }
                //}
                #endregion

                if ((UserInfoChoice != null) && (UserInfoChoice.ToUpper().Equals("NONE")))
                {
                    CustomizePrevPage = "&CustomizePrevPage=Artwork_Generation_ArtworkEdit.aspx";
                    ALParamString = ALParamString.Replace("&Mode=" + Mode, "&Mode=EDIT");
                }

                if (CustomizePrevPage != null)
                    ALParamString += CustomizePrevPage;

                #region Get Existing TaskPaneXML
                if ((RedirectURLForPage == null) || (RedirectURLForPage == ""))
                    ErrorXML = GetPageListTaskPaneXML(ref TaskPaneName, "", "JAVASCRIPTEVENT", "SubmitPage", FormatID, "FORMATID", "FORMAT_ARTWORKPAGE", SelectedPageID, SolutionName, Mode, ArtworkID, ref ExistingTaskPaneXML);
                else
                    ErrorXML = GetPageListTaskPaneXML(ref TaskPaneName, RedirectURLForPage + ALParamString, "URL", null, FormatID, "FORMATID", "FORMAT_ARTWORKPAGE", SelectedPageID, SolutionName, Mode, ArtworkID, ref ExistingTaskPaneXML);
                #endregion
                ExistingTaskPaneXML = ExistingTaskPaneXML.Insert(ExistingTaskPaneXML.IndexOf("<Section"), sbSectionTagXML.ToString());

                #region Assign Order to Section
                if (ExistingTaskPaneXML != null)
                {
                    XmlDocument xDocTaskPane = new XmlDocument();
                    xDocTaskPane.LoadXml(ExistingTaskPaneXML);
                    XmlNodeList xNodeList = xDocTaskPane.SelectNodes("TaskPanes/TaskPane/TaskPaneDetails/Section");
                    int cnt = 0;
                    foreach (XmlNode xNode in xNodeList)
                    {
                        try
                        {
                            if (((XmlElement)xNode).GetAttribute("ORDER") != null)
                            {
                                ((XmlElement)xNode).RemoveAttribute("Order");
                                ((XmlElement)xNode).SetAttribute("Order", (cnt = cnt + 1).ToString());
                            }
                        }
                        catch (Exception)
                        {

                        }
                    }
                    ExistingTaskPaneXML = xDocTaskPane.InnerXml;
                    xNodeList = null;
                    xDocTaskPane = null;
                }
                #endregion

                OutputXML = ExistingTaskPaneXML;
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_CREATETASKPANEXML_FAILED", this.GetType().ToString(), "CreateTaskPaneXML", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End CreateTaskPaneXML");
            }
        }

        public ERRORXML CreateTaskPaneXML(String TaskPaneName, String SelectedPageID, String VisitedLinks, String SolutionName, String FormatID, String ArtworkID, String Mode, String ProfileID, String UserInfoChoice, String ContactInfoChoice, String RedirectURLForPage, String EventHandling, String JSName, String IsGraphicElementPresent, String CustomizeParameters, ref String OutputXML)
        {

            #region Variables
            StringBuilder sbSectionTagXML = null;
            String ALParamString = "";
            String SelectedURL = "";
            String ExistingTaskPaneXML = null;
            ALSolutionCommon ObjAlCommon = new ALSolutionCommon();
            String ErrorXML = null; ;
            String CustomizePrevPage = null;
            String URLParmForEachSec = null;
            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin CreateTaskPaneXML");
                if (VisitedLinks == null)
                    VisitedLinks = "";
                #region Common Parameters for all pages
                ALParamString = ALParamString + "?TenantName=" + TenantName + "&UserID=" + UserID + "&SessionID=" + SessionID + "&Language=" + CurrentLanguage;
                ALParamString = ALParamString + "&SolutionName=" + SolutionName;
                ALParamString = ALParamString + "&ArtworkID=" + ArtworkID;
                ALParamString = ALParamString + "&FormatID=" + FormatID;
                ALParamString = ALParamString + "&ProfileID=" + ProfileID;
                ALParamString = ALParamString + CustomizeParameters;
                //   ALParamString = ALParamString + "&Mode=" + Mode;
                if ((UserInfoChoice != null) && (UserInfoChoice != ""))
                    ALParamString = ALParamString + "&UserInfoChoice=" + UserInfoChoice;
                if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                    ALParamString = ALParamString + "&ContactInfoChoice=" + ContactInfoChoice;



                #endregion
                sbSectionTagXML = new StringBuilder();

                #region Edit Ad
                SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("/" + TenantName + "/" + SolutionName + "/ASPX/Artwork_Generation_ArtworkEdit.aspx" + ALParamString + "&Mode=EDIT&ActualMode=" + Mode + "&VisitedLink=" + VisitedLinks);
                if (EventHandling.ToUpper().Equals("URL"))
                    sbSectionTagXML.AppendFormat("<Section SectionType='Text' SectionName='NAME_ARTWORK' Order='' Handling='URL' Target='_self' URL='{0}' ", SelectedURL);
                else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                    sbSectionTagXML.Append("<Section SectionType='Text' SectionName='NAME_ARTWORK' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");
                if (SelectedPageID != null && SelectedPageID.ToUpper().Equals("EDITAD"))
                    sbSectionTagXML.Append(" Selected='true'");
                sbSectionTagXML.Append(">");


                sbSectionTagXML.AppendFormat("<Text Title='Name Your Artwork'/>");
                sbSectionTagXML.AppendFormat("</Section>");

                #endregion
                //  ALParamString = ALParamString + "&Mode=" + Mode;

                #region Create UserChoice Page
                if ((UserInfoChoice == null) || (!UserInfoChoice.ToUpper().Equals("NONE")))
                {
                    if (VisitedLinks.Contains("#CHOICE#"))
                        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                    else
                        URLParmForEachSec = ALParamString + "&ActualMode=" + Mode + "&Mode=" + Mode;

                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0 && (!VisitedLinks.Contains("#CHOICE#")))
                    {
                        VisitedLinks += "#CHOICE#";
                        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                    }

                    // ALParamString += "&VisitedLink=" + VisitedLinks;
                    SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_Choices.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                    CustomizePrevPage = "&CustomizePrevPage=UserInformation_Choices.aspx";

                    if (EventHandling.ToUpper().Equals("URL"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO_OPTIONS' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                    else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO_OPTIONS' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");

                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0)
                        sbSectionTagXML.AppendFormat(" Selected='true'");

                    sbSectionTagXML.Append(">");

                    sbSectionTagXML.AppendFormat("<Text Title='Partner Information Options'/>");
                    sbSectionTagXML.AppendFormat("</Section>");
                }
                #endregion

                #region Create AdvisorInformation
                if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                {
                    SelectedURL = "";
                    if (VisitedLinks.Contains("#ADVISORINFO#"))
                        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                    else
                        URLParmForEachSec = ALParamString + "&ActualMode=" + Mode + "&Mode=" + Mode;

                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0 && (!VisitedLinks.Contains("#ADVISORINFO#")))
                    {
                        VisitedLinks += "#ADVISORINFO#";
                        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                    }

                    //  ALParamString += "&VisitedLink=" + VisitedLinks;


                    if (ContactInfoChoice.ToUpper().CompareTo("SINGLEADVISOR") == 0)
                    {
                        if (UserInfoChoice.ToUpper().Equals("OTHER"))
                        {
                            SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_SingleAdv_Other.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                            CustomizePrevPage = "&CustomizePrevPage=UserInformation_SingleAdv_Other.aspx";
                        }
                        else
                        {
                            SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_SingleAdv_Self.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                            CustomizePrevPage = "&CustomizePrevPage=UserInformation_SingleAdv_Self.aspx";
                        }

                    }
                    else if (ContactInfoChoice.ToUpper().CompareTo("TWOADVISORS_SAME") == 0)
                    {
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_TwoAdv_Same.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                        CustomizePrevPage = "&CustomizePrevPage=UserInformation_TwoAdv_Same.aspx";

                    }
                    else if (ContactInfoChoice.ToUpper().CompareTo("TWOADVISORS_DIFFERENT") == 0)
                    {
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_TwoAdv_Different.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                        CustomizePrevPage = "&CustomizePrevPage=UserInformation_TwoAdv_Different.aspx";
                    }


                    if (EventHandling.ToUpper().Equals("URL"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                    else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");


                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0)
                        sbSectionTagXML.AppendFormat(" Selected='true'");

                    sbSectionTagXML.Append(">");

                    sbSectionTagXML.AppendFormat("<Text Title='Partner Information'/>");
                    //sbSectionTagXML.AppendFormat(ALParamString.ToString());    
                    sbSectionTagXML.AppendFormat("</Section>");
                }
                #endregion

                #region Create AdvisorPhoto1
                if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                {
                    if ((IsGraphicElementPresent != null) && (!IsGraphicElementPresent.ToUpper().Equals("FALSE")))
                    {
                        SelectedURL = "";
                        if (VisitedLinks.Contains("#ADVISORPHOTO1#"))
                            URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                        else
                            URLParmForEachSec = ALParamString + "&ActualMode=" + Mode + "&Mode=" + Mode;

                        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0 && (!VisitedLinks.Contains("#ADVISORPHOTO1#")))
                        {
                            VisitedLinks += "#ADVISORPHOTO1#";
                            URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                        }

                        // ALParamString += "&VisitedLink=" + VisitedLinks;


                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UploadPhoto_Page.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                        CustomizePrevPage = "&CustomizePrevPage=UploadPhoto_Page.aspx";



                        if (EventHandling.ToUpper().Equals("URL"))
                            sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_PHOTO' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                        else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                            sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_PHOTO' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");


                        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORPHOTO1") == 0)
                            sbSectionTagXML.AppendFormat(" Selected='true'");

                        sbSectionTagXML.Append(">");
                        sbSectionTagXML.AppendFormat("<Text Title='Upload Photo'/>");
                        //sbSectionTagXML.AppendFormat(ALParamString.ToString());
                        sbSectionTagXML.AppendFormat("</Section>");
                    }
                }
                #endregion

                #region Create AdvisorPhoto2
                //if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                //{
                //    if (ContactInfoChoice.ToUpper().Contains("TWO") == true)
                //    {
                //        SelectedURL = "";
                //        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("ContactInfo_UploadPhoto_Second_Page.aspx" + ALParamString);
                //        CustomizePrevPage = "&CustomizePrevPage=ContactInfo_UploadPhoto_Second_Page.aspx";

                //        if (EventHandling.ToUpper().Equals("URL"))
                //            sbSectionTagXML.Append("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                //        else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                //            sbSectionTagXML.Append("<Section SectionType='Text' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");

                //        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORPHOTO2") == 0)
                //            sbSectionTagXML.AppendFormat(" Selected='true'");

                //        sbSectionTagXML.Append(">"); 
                //        sbSectionTagXML.AppendFormat("<Text Title='Upload Photo for Second Advisor'/>");
                //        //sbSectionTagXML.AppendFormat(ALParamString.ToString());
                //        sbSectionTagXML.AppendFormat("</Section>");
                //    }
                //}
                #endregion

                if ((UserInfoChoice != null) && (UserInfoChoice.ToUpper().Equals("NONE")))
                {
                    CustomizePrevPage = "&CustomizePrevPage=Artwork_Generation_ArtworkEdit.aspx";
                    ALParamString = ALParamString.Replace("&Mode=" + Mode, "&Mode=EDIT");
                }

                if (CustomizePrevPage != null)
                    ALParamString += CustomizePrevPage;

                #region Get Existing TaskPaneXML
                //   if ((RedirectURLForPage == null) || (RedirectURLForPage == ""))
                //     ErrorXML = GetPageListTaskPaneXML(ref TaskPaneName, "",EventHandling,JSName, FormatID, "FORMATID", "FORMAT_ARTWORKPAGE", SelectedPageID, SolutionName, Mode, ArtworkID, ref ExistingTaskPaneXML);
                //else
                if (!ALParamString.Contains("&VisitedLink"))
                    ALParamString += "&VisitedLink=" + VisitedLinks;
                if (!ALParamString.Contains("&Mode"))
                    ALParamString = ALParamString + "&Mode=" + Mode;
                if ((RedirectURLForPage != null) && (RedirectURLForPage != ""))
                    RedirectURLForPage += ALParamString;
                else
                    RedirectURLForPage = "";
                ErrorXML = GetPageListTaskPaneXML(ref TaskPaneName, RedirectURLForPage, EventHandling, JSName, FormatID, "FORMATID", "FORMAT_ARTWORKPAGE", SelectedPageID, SolutionName, Mode, ArtworkID, ref ExistingTaskPaneXML);
                #endregion
                ExistingTaskPaneXML = ExistingTaskPaneXML.Insert(ExistingTaskPaneXML.IndexOf("<Section"), sbSectionTagXML.ToString());

                #region Assign Order to Section
                if (ExistingTaskPaneXML != null)
                {
                    XmlDocument xDocTaskPane = new XmlDocument();
                    xDocTaskPane.LoadXml(ExistingTaskPaneXML);
                    XmlNodeList xNodeList = xDocTaskPane.SelectNodes("TaskPanes/TaskPane/TaskPaneDetails/Section");
                    int cnt = 0;
                    foreach (XmlNode xNode in xNodeList)
                    {
                        try
                        {
                            if (((XmlElement)xNode).GetAttribute("ORDER") != null)
                            {
                                ((XmlElement)xNode).RemoveAttribute("Order");
                                ((XmlElement)xNode).SetAttribute("Order", (cnt = cnt + 1).ToString());
                            }
                        }
                        catch (Exception)
                        {

                        }
                    }
                    ExistingTaskPaneXML = xDocTaskPane.InnerXml;
                    xNodeList = null;
                    xDocTaskPane = null;
                }
                #endregion

                OutputXML = ExistingTaskPaneXML;
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_CREATETASKPANEXML_FAILED", this.GetType().ToString(), "CreateTaskPaneXML", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End CreateTaskPaneXML");
            }

        }

        public ERRORXML CreateTaskPaneXML(String TaskPaneName, String SelectedPageID, String VisitedLinks, String FormatID, String ArtworkID, String Mode, String RedirectURLForPage, String EventHandling, String JSName, ref String OutputXML)
        {

            #region Variables
            StringBuilder sbSectionTagXML = null;
            String ALParamString = "";
            String SelectedURL = "";
            String ExistingTaskPaneXML = null;
            //int paramCnt = 0;
            ALSolutionCommon ObjAlCommon = new ALSolutionCommon();
            String ErrorXML = null; ;
            String CustomizePrevPage = null;
            //String URLParmForEachSec = null;
            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "Begin CreateTaskPaneXML");
                if (VisitedLinks == null)
                    VisitedLinks = "";
                #region Common Parameters for all pages
                ALParamString = ALParamString + "?TenantName=" + TenantName + "&UserID=" + UserID + "&SessionID=" + SessionID + "&Language=" + CurrentLanguage;
                ALParamString = ALParamString + "&SolutionName=" + SolutionName;
                ALParamString = ALParamString + "&ArtworkID=" + ArtworkID;
                ALParamString = ALParamString + "&FormatID=" + FormatID;
                //ALParamString = ALParamString + CustomizeParameters;
                #endregion
                sbSectionTagXML = new StringBuilder();

                #region Edit Artwork
                SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("/" + TenantName + "/" + SolutionName + "/ASPX/Artwork_Generation_ArtworkEdit.aspx" + ALParamString + "&Mode=EDIT&ActualMode=" + Mode + "&VisitedLink=" + VisitedLinks);
                if (EventHandling.ToUpper().Equals("URL"))
                    sbSectionTagXML.AppendFormat("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='{0}' ", SelectedURL);
                else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                    sbSectionTagXML.Append("<Section SectionType='Text' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");
                if (SelectedPageID != null && SelectedPageID.ToUpper().Equals("EDITAD"))
                    sbSectionTagXML.Append(" Selected='true'");
                sbSectionTagXML.Append(">");
                sbSectionTagXML.AppendFormat("<Text Title='" + Localization.GetString("ALResource", TenantName, "AP_LeftPane_NameYourArtwork") + "'/>");
                sbSectionTagXML.AppendFormat("</Section>");
                #endregion


                //#region Create UserChoice Page
                //if ((UserInfoChoice == null) || (!UserInfoChoice.ToUpper().Equals("NONE")))
                //{
                //    if (VisitedLinks.Contains("#CHOICE#"))
                //        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                //    else
                //        URLParmForEachSec = ALParamString + "&ActualMode=" + Mode + "&Mode=" + Mode;

                //    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0 && (!VisitedLinks.Contains("#CHOICE#")))
                //    {
                //        VisitedLinks += "#CHOICE#";
                //        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                //    }

                //    // ALParamString += "&VisitedLink=" + VisitedLinks;
                //    SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_Choices.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                //    CustomizePrevPage = "&CustomizePrevPage=UserInformation_Choices.aspx";

                //    if (EventHandling.ToUpper().Equals("URL"))
                //        sbSectionTagXML.Append("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                //    else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                //        sbSectionTagXML.Append("<Section SectionType='Text' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");

                //    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0)
                //        sbSectionTagXML.AppendFormat(" Selected='true'");

                //    sbSectionTagXML.Append(">");

                //    sbSectionTagXML.AppendFormat("<Text Title='Ad Information Options'/>");
                //    sbSectionTagXML.AppendFormat("</Section>");
                //}
                //#endregion

                //#region Create AdvisorInformation
                //if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                //{
                //    SelectedURL = "";
                //    if (VisitedLinks.Contains("#ADVISORINFO#"))
                //        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                //    else
                //        URLParmForEachSec = ALParamString + "&ActualMode=" + Mode + "&Mode=" + Mode;

                //    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0 && (!VisitedLinks.Contains("#ADVISORINFO#")))
                //    {
                //        VisitedLinks += "#ADVISORINFO#";
                //        URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                //    }

                //    //  ALParamString += "&VisitedLink=" + VisitedLinks;


                //    if (ContactInfoChoice.ToUpper().CompareTo("SINGLEADVISOR") == 0)
                //    {
                //        if (UserInfoChoice.ToUpper().Equals("OTHER"))
                //        {
                //            SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_SingleAdv_Other.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                //            CustomizePrevPage = "&CustomizePrevPage=UserInformation_SingleAdv_Other.aspx";
                //        }
                //        else
                //        {
                //            SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_SingleAdv_Self.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                //            CustomizePrevPage = "&CustomizePrevPage=UserInformation_SingleAdv_Self.aspx";
                //        }

                //    }
                //    else if (ContactInfoChoice.ToUpper().CompareTo("TWOADVISORS_SAME") == 0)
                //    {
                //        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_TwoAdv_Same.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                //        CustomizePrevPage = "&CustomizePrevPage=UserInformation_TwoAdv_Same.aspx";

                //    }
                //    else if (ContactInfoChoice.ToUpper().CompareTo("TWOADVISORS_DIFFERENT") == 0)
                //    {
                //        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_TwoAdv_Different.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                //        CustomizePrevPage = "&CustomizePrevPage=UserInformation_TwoAdv_Different.aspx";
                //    }


                //    if (EventHandling.ToUpper().Equals("URL"))
                //        sbSectionTagXML.Append("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                //    else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                //        sbSectionTagXML.Append("<Section SectionType='Text' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");


                //    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0)
                //        sbSectionTagXML.AppendFormat(" Selected='true'");

                //    sbSectionTagXML.Append(">");

                //    sbSectionTagXML.AppendFormat("<Text Title='Advisor Information'/>");
                //    //sbSectionTagXML.AppendFormat(ALParamString.ToString());    
                //    sbSectionTagXML.AppendFormat("</Section>");
                //}
                //#endregion

                //#region Create AdvisorPhoto1
                //if ((ContactInfoChoice != null) && (ContactInfoChoice != ""))
                //{
                //    if ((IsGraphicElementPresent != null) && (!IsGraphicElementPresent.ToUpper().Equals("FALSE")))
                //    {
                //        SelectedURL = "";
                //        if (VisitedLinks.Contains("#ADVISORPHOTO1#"))
                //            URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                //        else
                //            URLParmForEachSec = ALParamString + "&ActualMode=" + Mode + "&Mode=" + Mode;

                //        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORINFO") == 0 && (!VisitedLinks.Contains("#ADVISORPHOTO1#")))
                //        {
                //            VisitedLinks += "#ADVISORPHOTO1#";
                //            URLParmForEachSec = ALParamString + "&DisplayMode=EDIT&Mode=EDIT&ActualMode=" + Mode;
                //        }

                //        // ALParamString += "&VisitedLink=" + VisitedLinks;


                //        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UploadPhoto_Page.aspx" + URLParmForEachSec + "&VisitedLink=" + VisitedLinks);
                //        CustomizePrevPage = "&CustomizePrevPage=UploadPhoto_Page.aspx";



                //        if (EventHandling.ToUpper().Equals("URL"))
                //            sbSectionTagXML.Append("<Section SectionType='Text' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                //        else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                //            sbSectionTagXML.Append("<Section SectionType='Text' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript='" + JSName + "' Event='" + SelectedURL + "'");


                //        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("ADVISORPHOTO1") == 0)
                //            sbSectionTagXML.AppendFormat(" Selected='true'");

                //        sbSectionTagXML.Append(">");
                //        sbSectionTagXML.AppendFormat("<Text Title='Upload Photo'/>");
                //        //sbSectionTagXML.AppendFormat(ALParamString.ToString());
                //        sbSectionTagXML.AppendFormat("</Section>");
                //    }
                //}
                //#endregion

                //if ((UserInfoChoice != null) && (UserInfoChoice.ToUpper().Equals("NONE")))
                //{
                CustomizePrevPage = "&CustomizePrevPage=Artwork_Generation_ArtworkEdit.aspx";
                //    ALParamString = ALParamString.Replace("&Mode=" + Mode, "&Mode=EDIT");
                //}

                if (CustomizePrevPage != null)
                    ALParamString += CustomizePrevPage;

                #region Get Existing TaskPaneXML
                if (!ALParamString.Contains("&VisitedLink"))
                    ALParamString += "&VisitedLink=" + VisitedLinks;
                if (!ALParamString.Contains("&Mode"))
                    ALParamString = ALParamString + "&Mode=" + Mode;
                if ((RedirectURLForPage != null) && (RedirectURLForPage != ""))
                    RedirectURLForPage += ALParamString;
                else
                    RedirectURLForPage = "";
                ErrorXML = GetPageListTaskPaneXML(ref TaskPaneName, RedirectURLForPage, EventHandling, JSName, FormatID, "FORMATID", "FORMAT_ARTWORKPAGE", SelectedPageID, SolutionName, Mode, ArtworkID, ref ExistingTaskPaneXML);
                #endregion
                ExistingTaskPaneXML = ExistingTaskPaneXML.Insert(ExistingTaskPaneXML.IndexOf("<Section"), sbSectionTagXML.ToString());

                #region Assign Order to Section
                if (ExistingTaskPaneXML != null)
                {
                    XmlDocument xDocTaskPane = new XmlDocument();
                    xDocTaskPane.LoadXml(ExistingTaskPaneXML);
                    //int NoOfSections = 0;
                    XmlNodeList xNodeList = xDocTaskPane.SelectNodes("TaskPanes/TaskPane/TaskPaneDetails/Section");
                    int cnt = 0;
                    foreach (XmlNode xNode in xNodeList)
                    {
                        try
                        {
                            if (((XmlElement)xNode).GetAttribute("ORDER") != null)
                            {
                                ((XmlElement)xNode).RemoveAttribute("Order");
                                ((XmlElement)xNode).SetAttribute("Order", (cnt = cnt + 1).ToString());
                            }
                        }
                        catch (Exception)
                        {

                        }
                    }
                    ExistingTaskPaneXML = xDocTaskPane.InnerXml;
                    xNodeList = null;
                    xDocTaskPane = null;
                }
                #endregion

                OutputXML = ExistingTaskPaneXML;
                return null;
            }
            catch (Exception ExcepObj)
            {
                //objError.AppendError("APFACADE_FORMAT_CREATETASKPANEXML_FAILED", this.GetType().ToString(), "CreateTaskPaneXML", ExcepObj, null);
                //ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                //return ErrorXML;
                throw ExcepObj;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End CreateTaskPaneXML");
            }

        }

        /// <summary>
        /// Used in Advertisement.
        /// </summary>
        /// <param name="TaskPaneName"></param>
        /// <param name="SelectedPageID"></param>
        /// <param name="FormatID"></param>
        /// <param name="ArtworkID"></param>
        /// <param name="Mode"></param>
        /// <param name="EventHandling"></param>
        /// <param name="JSName"></param>
        /// <param name="SelectedField_UniqueNames"></param>
        /// <param name="ProfileInfoFlag"></param>
        /// <param name="PhotoFlag"></param>
        /// <param name="RedirectURLForPage"></param>
        /// <param name="CustomizeParameters"></param>
        /// <param name="OutputXML"></param>
        /// <returns></returns>
        public ERRORXML CreateTaskPaneXML(String TaskPaneName, String SelectedPageID, String FormatID, String ArtworkID, String Mode, String EventHandling, String JSName, String SelectedField_UniqueNames, String ProfileInfoFlag, String PhotoFlag, String EDITAD_DisplayText, String CHOICE_DisplayText, String PARTNERINFO_DisplayText, String PARTNERPHOTO_DisplayText, String RedirectURLForPage, String CustomizeParameters, ref String OutputXML)
        {
            #region Variables
            StringBuilder sbSectionTagXML = null;
            StringBuilder ALParamString = null;
            String SelectedURL = "";
            String ExistingTaskPaneXML = null;
            ALSolutionCommon ObjAlCommon = new ALSolutionCommon();
            String ErrorXML = null; ;
            String CustomizePrevPage = null;
            #endregion

            try
            {
                #region Common Parameters for all pages
                ALParamString = new StringBuilder(16);
                ALParamString.Append("?TenantName=" + TenantName + "&UserID=" + UserID + "&SessionID=" + SessionID + "&Language=" + CurrentLanguage);
                ALParamString.Append("&SolutionName=" + SolutionName);
                ALParamString.Append("&ArtworkID=" + ArtworkID);
                ALParamString.Append("&FormatID=" + FormatID + "&SelectedField_UniqueNames=" + SelectedField_UniqueNames + "&ProfileInfoFlag=" + ProfileInfoFlag + "&PhotoFlag=" + PhotoFlag + "&Mode=" + Mode);
                ALParamString.Append(CustomizeParameters);
                #endregion

                sbSectionTagXML = new StringBuilder();

                #region Edit Ad
                SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("/" + TenantName + "/" + SolutionName + "/ASPX/Artwork_Generation_ArtworkEdit.aspx" + ALParamString.ToString() + "");
                if (EventHandling.ToUpper().Equals("URL"))
                    sbSectionTagXML.AppendFormat("<Section SectionType='Text' SectionName='NAME_ARTWORK' Order='' Handling='URL' Target='_self' URL='{0}' ", SelectedURL);
                else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                    sbSectionTagXML.Append("<Section SectionType='Text' SectionName='NAME_ARTWORK' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + SelectedURL + "'");
                if (SelectedPageID != null && SelectedPageID.ToUpper().Equals("EDITAD"))
                    sbSectionTagXML.Append(" Selected='true'");
                sbSectionTagXML.Append(">");

                sbSectionTagXML.AppendFormat("<Text Title='" + EDITAD_DisplayText + "'/>");
                sbSectionTagXML.AppendFormat("</Section>");
                #endregion

                bool bShowPartnerChoicePages = false;
                if ((SelectedPageID != "PARTNERINFO" && SelectedPageID != "CHOICE" && SelectedPageID != "PARTNERPHOTO"))
                {
                    if (!String.IsNullOrEmpty(SelectedField_UniqueNames))
                    {
                        bShowPartnerChoicePages = CheckIfArtworkValueSelected(SelectedField_UniqueNames);
                    }
                }
                else
                    bShowPartnerChoicePages = true;


                #region Create UserChoice Page

                if (ProfileInfoFlag != null && ProfileInfoFlag == "1")
                {
                    SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_Choices.aspx" + ALParamString.ToString() + "");
                    CustomizePrevPage = "&CustomizePrevPage=UserInformation_Choices.aspx";

                    if (EventHandling.ToUpper().Equals("URL"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO_OPTIONS' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                    else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO_OPTIONS' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + SelectedURL + "'");

                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0)
                        sbSectionTagXML.AppendFormat(" Selected='true'");

                    sbSectionTagXML.Append(">");

                    sbSectionTagXML.AppendFormat("<Text Title='" + CHOICE_DisplayText + "'/>");
                    sbSectionTagXML.AppendFormat("</Section>");

                    #region Create Partner Information Node
                    if (bShowPartnerChoicePages)
                    {
                        SelectedURL = "";
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("ProfileField_Selection.aspx" + ALParamString.ToString() + "");
                        CustomizePrevPage = "&CustomizePrevPage=ProfileField_Selection.aspx";

                        if (EventHandling.ToUpper().Equals("URL"))
                            sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                        else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                            sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + SelectedURL + "'");


                        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("PARTNERINFO") == 0)
                            sbSectionTagXML.AppendFormat(" Selected='true'");

                        sbSectionTagXML.Append(">");

                        sbSectionTagXML.AppendFormat("<Text Title='" + PARTNERINFO_DisplayText + "'/>");
                        //sbSectionTagXML.AppendFormat(ALParamString.ToString());    
                        sbSectionTagXML.AppendFormat("</Section>");
                    }
                    #endregion

                }
                else if (PhotoFlag != null && PhotoFlag == "1")
                {
                    SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("UserInformation_Choices.aspx" + ALParamString.ToString() + "");
                    CustomizePrevPage = "&CustomizePrevPage=UserInformation_Choices.aspx";

                    if (EventHandling.ToUpper().Equals("URL"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO_OPTIONS' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                    else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                        sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_INFO_OPTIONS' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + SelectedURL + "'");

                    if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("CHOICE") == 0)
                        sbSectionTagXML.AppendFormat(" Selected='true'");

                    sbSectionTagXML.Append(">");

                    sbSectionTagXML.AppendFormat("<Text Title='" + CHOICE_DisplayText + "'/>");
                    sbSectionTagXML.AppendFormat("</Section>");
                }

                if (PhotoFlag != null && PhotoFlag == "1")
                {
                    #region Create Partner Photo node
                    if (bShowPartnerChoicePages)
                    {
                        SelectedURL = "";
                        SelectedURL = ObjAlCommon.EncodeURLForResponseRedirects("Artwork_Generation_ProfilePhotoSelection.aspx" + ALParamString.ToString() + "");
                        CustomizePrevPage = "&CustomizePrevPage=Artwork_Generation_ProfilePhotoSelection.aspx";

                        if (EventHandling.ToUpper().Equals("URL"))
                            sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_PHOTO' Order='' Handling='URL' Target='_self' URL='" + SelectedURL + "' ");
                        else if (EventHandling.ToUpper().Equals("JAVASCRIPTEVENT"))
                            sbSectionTagXML.Append("<Section SectionType='Text' SectionName='PARTNER_PHOTO' Order='' Target='_self' URL='' Handling='JavascriptEvent' Javascript=\"" + JSName + "\" Event='" + SelectedURL + "'");


                        if (SelectedPageID != null && SelectedPageID.ToUpper().CompareTo("PARTNERPHOTO") == 0)
                            sbSectionTagXML.AppendFormat(" Selected='true'");

                        sbSectionTagXML.Append(">");
                        sbSectionTagXML.AppendFormat("<Text Title='" + PARTNERPHOTO_DisplayText + "'/>");
                        //sbSectionTagXML.AppendFormat(ALParamString.ToString());
                        sbSectionTagXML.AppendFormat("</Section>");
                    }
                    #endregion
                }

                #endregion

                if (CustomizePrevPage != null)
                    ALParamString.Append(CustomizePrevPage);

                if ((RedirectURLForPage != null) && (RedirectURLForPage != ""))
                    RedirectURLForPage += ALParamString.ToString();
                else
                    RedirectURLForPage = "";

                #region Get Existing TaskPaneXML
                ErrorXML = GetPageListTaskPaneXML(ref TaskPaneName, RedirectURLForPage, EventHandling, JSName, FormatID, "FORMATID", "FORMAT_ARTWORKPAGE", SelectedPageID, SolutionName, Mode, ArtworkID, ref ExistingTaskPaneXML);
                ExistingTaskPaneXML = ExistingTaskPaneXML.Insert(ExistingTaskPaneXML.IndexOf("<Section"), sbSectionTagXML.ToString());
                #endregion

                #region Assign Order to Section
                if (ExistingTaskPaneXML != null)
                {
                    XmlDocument xDocTaskPane = new XmlDocument();
                    xDocTaskPane.LoadXml(ExistingTaskPaneXML);
                    XmlNodeList xNodeList = xDocTaskPane.SelectNodes("TaskPanes/TaskPane/TaskPaneDetails/Section");
                    int cnt = 0;
                    foreach (XmlNode xNode in xNodeList)
                    {
                        try
                        {
                            if (((XmlElement)xNode).GetAttribute("ORDER") != null)
                            {
                                ((XmlElement)xNode).RemoveAttribute("Order");
                                ((XmlElement)xNode).SetAttribute("Order", (cnt = cnt + 1).ToString());
                            }
                        }
                        catch (Exception)
                        {

                        }
                    }
                    ExistingTaskPaneXML = xDocTaskPane.InnerXml;
                    xNodeList = null;
                    xDocTaskPane = null;
                }
                #endregion

                OutputXML = ExistingTaskPaneXML;
                return null;
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }

        }

        public bool CheckIfArtworkValueSelected(String SelectedField_UniqueNames)
        {
            bool bArtworkValuesSelected = false;
            if (!String.IsNullOrEmpty(SelectedField_UniqueNames))
            {
                XPathDocument xpDoc_Config = null;
                XPathNavigator xpNav_Root = null;
                String[] srr_Separator = { "$AL$" };
                String[] str_UniqueIDs = SelectedField_UniqueNames.Split(srr_Separator, StringSplitOptions.RemoveEmptyEntries);

                if (str_UniqueIDs.Length > 0)
                {
                    String str_AdvtConfigPath = ALSolutionCommon.GetConfigurationXMLPath(TenantName, SolutionName, null, SolutionName + "_Configuration");
                    xpDoc_Config = new XPathDocument(str_AdvtConfigPath);
                    xpNav_Root = xpDoc_Config.CreateNavigator();
                }

                foreach (String UniqueID in str_UniqueIDs)
                {
                    XPathNavigator xpNav_Unique = xpNav_Root.SelectSingleNode("CONFIGURATION/FIELDVALUELIST/FIELDVALUE[@UNIQUENAME='" + UniqueID + "']");
                    if (xpNav_Unique != null)
                    {
                        String TargetName = xpNav_Unique.GetAttribute("TARGETOBJECTNAME", "");
                        if (String.Compare(TargetName, "ARTWORK", true) == 0)
                        {
                            bArtworkValuesSelected = true;
                            break;
                        }
                    }
                }
            }
            return bArtworkValuesSelected;
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to create new Format based on the input XML.</purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="FolderPath"></param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>

        public String ImportFormat(String ZipFilePath, String DesignID, String CampaignID)
        {
            String RequestID = null;
            bool IsZippedContent = true;
            String NextFormatId = null;

            RequestID = Guid.NewGuid().ToString();
            ImportFormat(IsZippedContent, ZipFilePath, DesignID, CampaignID, RequestID, out NextFormatId);

            return RequestID;
        }

        public String ImportFormat(bool IsZippedContent, String ZipFilePath, String DesignID, String CampaignID, String RequestID, out String NextFormatId)
        {
            if (String.IsNullOrEmpty(RequestID))
                RequestID = Guid.NewGuid().ToString();
            String str_ExportDirPath = null;
            NextFormatId = null;

            str_ExportDirPath = PrepareImportFolder(RequestID, true);

            UpdateImportFormatStatus(RequestID, FormatImportStatus.UnzippingFormatFile);

            ArrayList objParams = new ArrayList(4);
            objParams.Add(RequestID);
            objParams.Add(ZipFilePath);
            objParams.Add(DesignID);
            objParams.Add(str_ExportDirPath);
            objParams.Add(CampaignID);
            objParams.Add(IsZippedContent);


            if (IsZippedContent == false)
                ProcessImportFormatRequest(objParams, out NextFormatId);
            else
            {
                OrgIdentity = System.Security.Principal.WindowsIdentity.GetCurrent();
                ThreadPool.QueueUserWorkItem(new WaitCallback(ProcessImportFormatRequest), objParams);
            }


            return RequestID;
        }

        public String ExportFormat(String FormatID)
        {
            String ExportDirPath = null;
            String RequestID = null;
            bool ZipRequired = true;

            RequestID = Guid.NewGuid().ToString();
            ExportFormat(FormatID, ZipRequired, RequestID, out ExportDirPath);

            return RequestID;
        }

        public String ExportFormat(String FormatID, bool ZipRequired, String RequestID, out String ExportDirPath)
        {
            if (String.IsNullOrEmpty(RequestID))
                RequestID = Guid.NewGuid().ToString();
            String str_ExportDirPath = null;

            str_ExportDirPath = PrepareExportFolder(RequestID, true);

            UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.RetrivingFormatInfo);

            ArrayList objParams = new ArrayList(3);
            objParams.Add(RequestID);
            objParams.Add(FormatID);
            objParams.Add(str_ExportDirPath);
            objParams.Add(ZipRequired);

            if (ZipRequired == true)
            {
                OrgIdentity = System.Security.Principal.WindowsIdentity.GetCurrent();
                ThreadPool.QueueUserWorkItem(new WaitCallback(ProcessExportFormatRequest), objParams);
            }
            else
                ProcessExportFormatRequest(objParams);

            ExportDirPath = str_ExportDirPath;
            return RequestID;
        }

        public FormatExportStatus GetExportFormatStatus(String RequestID, String FormatID, out String ZipFilePath)
        {
            ZipFilePath = null;
            String str_DownloadPath = PrepareExportFolder(RequestID, false);
            FormatExportStatus status = GetExportFormatStatus(RequestID, FormatID);
            if (status == FormatExportStatus.Done)
            {
                String ExportPath = null;

                ExportPath = str_DownloadPath + "\\ZipName.txt";

                ZipFilePath = str_DownloadPath + "\\" + File.ReadAllText(ExportPath);

                return FormatExportStatus.Done;
            }

            return status;
        }

        public FormatImportStatus GetImportFormatStatus(String RequestID)
        {
            String ImportPath = null;

            ImportPath = PrepareImportFolder(RequestID, false);

            ImportPath = ImportPath + "\\status.txt";
            if (File.Exists(ImportPath))
            {
                switch (File.ReadAllText(ImportPath))
                {
                    case "UnzippingFormatFile":
                        return FormatImportStatus.UnzippingFormatFile;

                    case "CreatingFormat":
                        return FormatImportStatus.CreatingFormat;

                    case "CreatingElements":
                        return FormatImportStatus.CreatingElements;

                    case "Error":
                        return FormatImportStatus.Error;

                    case "Done":
                        return FormatImportStatus.Done;

                    case "InvalidZip":
                        return FormatImportStatus.InvalidZip;
                }
            }
            return FormatImportStatus.UnzippingFormatFile;
        }

        #region Private Methods

        private void ProcessDownloadFormatRequest(object Params)
        {
            System.Security.Principal.WindowsImpersonationContext NewContext = null;
            String RequestID = null;
            String str_FormatID = null;
            try
            {
                System.Threading.Thread.CurrentPrincipal = new System.Security.Principal.WindowsPrincipal(OrgIdentity);
                NewContext = OrgIdentity.Impersonate();

                Search objSearchWS = new Search();
                StringBuilder sbSearchXML = new StringBuilder(300);
                String str_SearchOutput = null;
                String str_DownloadDirPath = null;

                objSearchWS.Url = ALCommon.GetWebServiceURL("ALSearchWS");
                RequestID = (String)((ArrayList)Params)[0];
                str_FormatID = (String)((ArrayList)Params)[1];
                str_DownloadDirPath = (String)((ArrayList)Params)[2];

                sbSearchXML.AppendFormat(@" <SEARCH>
                                        <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{4}'></SESSIONINFO>
                                        <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
	                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
	                                                <EXPRESSION>FORMATID</EXPRESSION>
	                                                <OPERATOR>EQUALTO</OPERATOR>
	                                                <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
	                                                <FILTERID>1</FILTERID>
	                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                    <ALIAS>ORIGINALFILE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>PREVIEWFILE</FIELDNAME>
                                                    <ALIAS>PREVIEWFILE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>THUMBNAILFILE</FIELDNAME>
                                                    <ALIAS>THUMBNAILFILE</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>FORMATNAME</FIELDNAME>
                                                    <ALIAS>FORMATNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
	                                                <FIELDNAME>ADDITIONALFILES</FIELDNAME>
	                                                <ALIAS>ADDITIONALFILES</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='FORMAT_ARTWORKPAGE'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>ARTWORKPAGEID</FIELDNAME>
                                                        <ALIAS>ARTWORKPAGEID</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='ARTWORKPAGE_ELEMENTGROUP'> 
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ELEMENTGROUPID</FIELDNAME>
                                                            <ALIAS>ELEMENTGROUPID</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                    <RELATIONSHIP NAME='ELEMENTGROUP_GRAPHICELEMENT'> 
                                                        <SELECTLIST>
	                                                        <EXPRESSION>
		                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
		                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
	                                                        </EXPRESSION>
	                                                        <EXPRESSION>
		                                                        <FIELDNAME>ORIGINALFILE</FIELDNAME>
		                                                        <ALIAS>ORIGINALFILE</ALIAS>
	                                                        </EXPRESSION>
	                                                        <EXPRESSION>
		                                                        <FIELDNAME>PREVIEWFILE</FIELDNAME>
		                                                        <ALIAS>PREVIEWFILE</ALIAS>
	                                                        </EXPRESSION>
	                                                        <EXPRESSION>
		                                                        <FIELDNAME>THUMBNAILFILE</FIELDNAME>
		                                                        <ALIAS>THUMBNAILFILE</ALIAS>
	                                                        </EXPRESSION>
                                                        </SELECTLIST> 
                                                    </RELATIONSHIP>
                                                </RELATIONSHIP>
                                            </RELATIONSHIP>
                                        </TARGETENTITY>
                                    </SEARCH>", CurrentLanguage, TenantName, UserID, str_FormatID, SessionID);
                String ErrorXML = objSearchWS.GetHierarchialData(sbSearchXML.ToString(), ref str_SearchOutput);

                if (ErrorXML != null)
                    UpdateDownloadFormatStatus(RequestID, str_FormatID, FormatDownloadStatus.Error);

                XPathDocument xpDoc_FormatDetails = new XPathDocument(new StringReader(str_SearchOutput));
                XPathNavigator xpNav_Root = xpDoc_FormatDetails.CreateNavigator();
                XPathNavigator xpNav_File = null;


                #region Get Format Files

                xpNav_File = xpNav_Root.SelectSingleNode("/FORMATCOLLECTION/FORMAT/FILESYSTEM [FILE/FIELDNAME/text()='ORIGINALFILE']");
                if (xpNav_File != null)
                {
                    GetFile(xpNav_File.OuterXml, "ORIGINALFILE", str_DownloadDirPath);
                }

                xpNav_File = xpNav_Root.SelectSingleNode("/FORMATCOLLECTION/FORMAT/FILESYSTEM [FILE/FIELDNAME/text()='PREVIEWFILE']");
                if (xpNav_File != null)
                {
                    GetFile(xpNav_File.OuterXml, "PREVIEWFILE", str_DownloadDirPath);
                }

                xpNav_File = xpNav_Root.SelectSingleNode("/FORMATCOLLECTION/FORMAT/FILESYSTEM [FILE/FIELDNAME/text()='THUMBNAILFILE']");
                if (xpNav_File != null)
                {
                    GetFile(xpNav_File.OuterXml, "THUMBNAILFILE", str_DownloadDirPath);
                }

                xpNav_File = xpNav_Root.SelectSingleNode("/FORMATCOLLECTION/FORMAT/FILESYSTEM/BINDER");
                if (xpNav_File != null)
                {
                    GetFile(xpNav_File.OuterXml, "ADDITIONALFILES", str_DownloadDirPath);
                }

                #endregion

                UpdateDownloadFormatStatus(RequestID, str_FormatID, FormatDownloadStatus.RetrivingLinkedFiles);

                #region Get Linked Files

                XPathNodeIterator xpIter_GEs = xpNav_Root.Select("/FORMATCOLLECTION/FORMAT/FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_GRAPHICELEMENT");
                while (xpIter_GEs.MoveNext())
                {
                    xpNav_File = xpIter_GEs.Current;
                    if (xpNav_File != null)
                    {
                        GetFile(xpNav_File.OuterXml, "ORIGINALFILE", str_DownloadDirPath);
                    }

                    if (xpNav_File != null)
                    {
                        GetFile(xpNav_File.OuterXml, "PREVIEWFILE", str_DownloadDirPath);
                    }

                    if (xpNav_File != null)
                    {
                        GetFile(xpNav_File.OuterXml, "THUMBNAILFILE", str_DownloadDirPath);
                    }
                }
                #endregion

                UpdateDownloadFormatStatus(RequestID, str_FormatID, FormatDownloadStatus.PreparingZip);

                #region Prepare Zip

                String FormatName = xpNav_Root.SelectSingleNode("/FORMATCOLLECTION/FORMAT/FORMATNAME").Value;
                String FormatName_FileName = ReplaceSpecialChars_InvalidOnFileSystem(FormatName);
                FormatName_FileName = FormatName_FileName.Replace(" ", "_");  //Reason for doing this is when we download save dialog box replaces space with underscore character, so while importing it searches for the XML which is not present since name has been changed.
                FormatName_FileName = FormatName_FileName.Replace("#", "_");  //Reason for doing this is upload control gives error for #,; and + characters
                FormatName_FileName = FormatName_FileName.Replace(";", "_");
                FormatName_FileName = FormatName_FileName.Replace("+", "_");

                String ZipName = FormatName_FileName + "_" + str_FormatID;

                File.WriteAllText(str_DownloadDirPath + "\\ZipName.txt", ZipName + ".zip");

                if (PrepareZip(str_DownloadDirPath, str_DownloadDirPath, ZipName) != null)
                    UpdateDownloadFormatStatus(RequestID, str_FormatID, FormatDownloadStatus.Error);
                else
                #endregion
                    UpdateDownloadFormatStatus(RequestID, str_FormatID, FormatDownloadStatus.Done);
            }
            catch (Exception)
            {
                UpdateDownloadFormatStatus(RequestID, str_FormatID, FormatDownloadStatus.Error);
            }
            finally
            {
                NewContext.Undo();
            }
            return;
        }

        private String ReplaceSpecialChars_InvalidOnFileSystem(String Name)
        {
            Name = Name.Replace('/', '_');
            Name = Name.Replace('\\', '_');
            Name = Name.Replace(':', '_');
            Name = Name.Replace('*', '_');
            Name = Name.Replace('?', '_');
            Name = Name.Replace('<', '_');
            Name = Name.Replace('>', '_');
            Name = Name.Replace('|', '_');
            Name = Name.Replace('"', '_');
            return Name;
        }

        private String PrepareDownloadFolder(String RequestID, bool CheckFolder)
        {
            String str_DownloadPath = null;            
            StringBuilder sbDownloadPath = new StringBuilder(50);

            Context ContextObj = null;
            ulong uid = 0;
            ulong sid = 0;
            try
            {
                uid = ulong.Parse(UserID);
            }
            catch (Exception)
            {
                uid = 0;
            }
            try
            {
                sid = ulong.Parse(SessionID);
            }
            catch (Exception)
            {
                sid = 0;
            }
            StringBuilder ContextAdditionalInfo = null;
            ContextAdditionalInfo = new StringBuilder(20);
            ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerFacade.FormatFacade.PrepareDownloadFolder</APINAME><REQUESTID>{0}</REQUESTID></ADDITIOALINFO>", RequestID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

            ContextObj = new Context(uid, sid, TenantName, SolutionName, ContextAdditionalInfo.ToString());
            sbDownloadPath.AppendFormat("{0}\\DownloadedFormats\\{1}", ALSolutionCommon.GetDownloadFilePath(TenantName), RequestID);

            str_DownloadPath = sbDownloadPath.ToString();
            if (CheckFolder)
            {

                if (Directory.Exists(str_DownloadPath))
                {
                    //Directory.Delete(str_DownloadPath, true);
                    SASMOMDirectory.DeleteFolder(str_DownloadPath, true, ContextObj);
                }
                Directory.CreateDirectory(str_DownloadPath);

            }
            return str_DownloadPath;
        }

        private void GetFile(String FileXML, String FieldName, String DownloadPath)
        {
            #region variable
            FileRetrival FileRetrivalObj = null;
            FileTransferInfrastructure FileTransferObj = null;
            String FileSystemXML = null;
            String ErrorXML = null;
            XPathDocument XPathDoc = null;
            StringReader XPathDocXML = null;
            XPathNavigator XPathNav = null;
            XPathNavigator FileNode = null;
            XPathNodeIterator FileNodeList = null;

            //String FilePath = null;
            String FileName = null;
            String DisplayFileName = null;
            String ObjectID = null;
            //int Index = 0;
            #endregion



            FileRetrivalObj = new FileRetrival(SessionID.ToString(), UserID.ToString(), TenantName, CurrentLanguage);
            FileTransferObj = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, CurrentLanguage);


            XPathDocXML = new StringReader(FileXML);
            XPathDoc = new XPathDocument(XPathDocXML);
            XPathNav = XPathDoc.CreateNavigator();

            FileNodeList = XPathNav.Select("//FIELDNAME[text()='" + FieldName + "']");

            if (FileNodeList.Count > 0)
            {
                while (FileNodeList.MoveNext())
                {

                    FileNode = FileNodeList.Current;
                    FileNode.MoveToParent();
                    FileSystemXML = FileNode.OuterXml;
                    if (FileSystemXML.StartsWith("<FILE>"))
                    {
                        //Get File Name
                        if (FileNode.SelectSingleNode("NAME") != null)
                            FileName = FileNode.SelectSingleNode("NAME").InnerXml;

                        if (FileNode.SelectSingleNode("DISPLAYNAME") != null)
                            DisplayFileName = FileNode.SelectSingleNode("DISPLAYNAME").InnerXml;

                        #region Get File to Outbox

                        ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                        ObjectID = ObjectID.Replace("<OBJECTID>", "");
                        FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                        FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                        try
                        {
                            ErrorXML = FileRetrivalObj.GetFile(ref FileSystemXML);
                        }
                        catch (Exception)
                        {
                            throw;
                        }

                        #endregion

                        #region Get File to solution machine

                        if (ErrorXML == null)
                        {
                            FileSystemXML = FileSystemXML.Replace("<PATH>", "<SOURCEPATH>");
                            FileSystemXML = FileSystemXML.Replace("</PATH>", "</SOURCEPATH>");
                            FileTransferObj.GetFilePath = DownloadPath;
                            ErrorXML = FileTransferObj.GetFile(ref FileSystemXML);
                        }

                        #endregion

                    }
                }//end of while

            }

        }

        private void UpdateDownloadFormatStatus(String RequestID, String FormatID, FormatDownloadStatus status)
        {
            String DownloadPath = null;
            DownloadPath = PrepareDownloadFolder(RequestID, false);
            File.WriteAllText(DownloadPath + "\\status.txt", status.ToString());
        }

        private FormatDownloadStatus GetDownloadFormatStatus(String RequestID, String FormatID)
        {
            String DownloadPath = null;

            DownloadPath = PrepareDownloadFolder(RequestID, false);
            DownloadPath = DownloadPath + "\\status.txt";
            if (File.Exists(DownloadPath))
            {
                switch (File.ReadAllText(DownloadPath))
                {
                    case "RetrivingFormatFiles":
                        return FormatDownloadStatus.RetrivingFormatFiles;

                    case "RetrivingLinkedFiles":
                        return FormatDownloadStatus.RetrivingLinkedFiles;

                    case "PreparingZip":
                        return FormatDownloadStatus.PreparingZip;

                    case "Error":
                        return FormatDownloadStatus.Error;

                    case "Done":
                        return FormatDownloadStatus.Done;
                }
            }
            return FormatDownloadStatus.RetrivingFormatFiles;
        }

        private String PrepareZip(String SourceFilePath, String DestinationDirectory, String ZipFileName)
        {
            Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper objZipWrapper = new Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper();
            objZipWrapper.FilesToExclude = "status.txt|ZipName.txt";
            String ErrorXML = objZipWrapper.ZipFiles(SourceFilePath, DestinationDirectory, ZipFileName);
            if (ErrorXML == "SUCCESS")
                return null;
            else
                return ErrorXML;
        }


        private String PrepareExportFolder(String RequestID, bool CheckFolder)
        {
            String str_ExportPath = null;
            StringBuilder sbExportPath = new StringBuilder(50);

            Context ContextObj = null;
            ulong uid = 0;
            ulong sid = 0;
            try
            {
                uid = ulong.Parse(UserID);
            }
            catch (Exception)
            {
                uid = 0;
            }
            try
            {
                sid = ulong.Parse(SessionID);
            }
            catch (Exception)
            {
                sid = 0;
            }
            StringBuilder ContextAdditionalInfo = null;
            ContextAdditionalInfo = new StringBuilder(20);
            ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerFacade.FormatFacade.PrepareExportFolder</APINAME><REQUESTID>{0}</REQUESTID></ADDITIOALINFO>", RequestID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

            ContextObj = new Context(uid, sid, TenantName, SolutionName, ContextAdditionalInfo.ToString());

            sbExportPath.AppendFormat("{0}\\ExportedFormats\\{1}", ALSolutionCommon.GetDownloadFilePath(TenantName), RequestID);
            str_ExportPath = sbExportPath.ToString();
            if (CheckFolder)
            {

                if (Directory.Exists(str_ExportPath))
                {
                    //Directory.Delete(str_ExportPath, true);
                    SASMOMDirectory.DeleteFolder(str_ExportPath, true, ContextObj);
                }
                Directory.CreateDirectory(str_ExportPath + "\\ADDITIONALFILES");
                Directory.CreateDirectory(str_ExportPath + "\\FORMATPREVIEWFILES");
                Directory.CreateDirectory(str_ExportPath + "\\FORMATTHUMBNAILFILES");

            }
            return str_ExportPath;
        }

        private void UpdateExportFormatStatus(String RequestID, String FormatID, FormatExportStatus status)
        {
            String ExportPath = null;

            ExportPath = PrepareExportFolder(RequestID, false);
            File.WriteAllText(ExportPath + "\\status.txt", status.ToString());
        }

        private FormatExportStatus GetExportFormatStatus(String RequestID, String FormatID)
        {
            String ExportPath = null;

            ExportPath = PrepareExportFolder(RequestID, false);
            ExportPath = ExportPath + "\\status.txt";
            if (File.Exists(ExportPath))
            {
                switch (File.ReadAllText(ExportPath))
                {
                    case "RetrivingFormatInfo":
                        return FormatExportStatus.RetrivingFormatInfo;

                    case "RetrivingFormatFiles":
                        return FormatExportStatus.RetrivingFormatFiles;

                    case "RetrivingLinkedFiles":
                        return FormatExportStatus.RetrivingLinkedFiles;

                    case "RetrivingDAMItems":
                        return FormatExportStatus.RetrivingDAMItems;

                    case "PreparingZip":
                        return FormatExportStatus.PreparingZip;

                    case "Error":
                        return FormatExportStatus.Error;

                    case "Done":
                        {
                            return FormatExportStatus.Done;
                        }
                }
            }
            return FormatExportStatus.RetrivingFormatInfo;
        }

        private void ProcessImportFormatRequest(Object Params)
        {
            String NextFormatId = null;
            ProcessImportFormatRequest(Params, out NextFormatId);
        }

        private void ProcessImportFormatRequest(Object Params, out String NextFormatId)
        {
            #region variable
            String FormatName = null;
            //String          CampaignID      =   null;
            //String          DesignID        =   null;
            String FormatID = null;

            XML FormatInputXML = null;
            ERRORXML MainErrorXML = null;

            ERRORXML ErrorXML = null;
            XPathDocument xpDoc = null;
            XPathNavigator xpNav = null;
            XPathNavigator xpNodeNav = null;
            StringBuilder sbInputXML = null;
            String OldPath = null;
            String ExtactPath = null;
            String RequestID = null;
            String FolderPath = null;
            String DesignID = null;
            String CampaignID = null;
            String FormatToImport_Status = null;
            bool IsZippedContent = true;
            NextFormatId = null;
            #endregion

            System.Security.Principal.WindowsImpersonationContext NewContext = null;

            IsZippedContent = (bool)((ArrayList)Params)[5];

            try
            {
                if (IsZippedContent == true)
                {
                    System.Threading.Thread.CurrentPrincipal = new System.Security.Principal.WindowsPrincipal(OrgIdentity);
                    NewContext = OrgIdentity.Impersonate();
                }

                RequestID = (String)((ArrayList)Params)[0];
                FolderPath = (String)((ArrayList)Params)[1];
                DesignID = (String)((ArrayList)Params)[2];
                ExtactPath = (String)((ArrayList)Params)[3];
                CampaignID = (String)((ArrayList)Params)[4];

                ImportFormatIDs = new Hashtable();

                if (IsZippedContent == true)
                {
                    //if (FolderPath.Contains(".zip") || FolderPath.Contains(".ZIP"))
                    #region Unzip that Folder
                    {
                        String str_OldFormatID = null;

                        FormatName = FolderPath.Substring(FolderPath.LastIndexOf("\\") + 1);
                        FormatName = FormatName.Replace(".zip", "");
                        int iPos = FormatName.LastIndexOf("_");
                        str_OldFormatID = FormatName.Substring(iPos + 1, FormatName.Length - iPos - 1);
                        FormatName = FormatName.Substring(0, FormatName.LastIndexOf("_"));
                        //String TempPath = FolderPath.Replace(".zip", "");
                        //**********************************************************************
                        //ErrorXML = UnZipFormatFolder(FolderPath, ref TempPath);
                        //**********************************************************************
                        #region Unzip File

                        Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper objZipWrapper = new Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper();
                        objZipWrapper.UncompressZipFile(FolderPath, ExtactPath);

                        #endregion

                        UpdateImportFormatStatus(RequestID, FormatImportStatus.CreatingFormat);

                        FolderPath = ExtactPath;
                        if (ErrorXML != null)
                        {
                            UpdateImportFormatStatus(RequestID, FormatImportStatus.Error);
                        }

                    }
                    //else
                    //    FormatName = FolderPath.Substring(FolderPath.LastIndexOf("\\") + 1);
                    #endregion
                }
                else
                {
                    //copying all the files from the export directiry to import directiry
                    //Creating all the subdirectories
                    foreach (string dirPath in Directory.GetDirectories(FolderPath, "*", SearchOption.AllDirectories))
                        Directory.CreateDirectory(dirPath.Replace(FolderPath, ExtactPath));

                    //Copy all the files
                    foreach (string newPath in Directory.GetFiles(FolderPath, "*.*", SearchOption.AllDirectories))
                        File.Copy(newPath, newPath.Replace(FolderPath, ExtactPath), true);
                }
                FormatToImport = FormatName;

                String TempDirPath = ALCommon.GetTempPath() + SessionID;
                Directory.CreateDirectory(TempDirPath);

                FormatToImport_Status = TempDirPath + "\\" + FormatToImport + "_Status.xml";
                if (File.Exists(FormatToImport_Status))
                    File.Delete(FormatToImport_Status);


                if (File.Exists(FolderPath + "\\Format_Import.xml"))
                {
                    FormatInputXML = File.ReadAllText(FolderPath + "\\Format_Import.xml");
                    OldPath = ALCommon.GetInnerValue(ref FormatInputXML, "<EXPORTPATH>", "</EXPORTPATH>");
                    if (OldPath != null)
                        FormatInputXML = FormatInputXML.Replace(OldPath, FolderPath + "\\");
                    else
                    {

                        SaveStatusFile("ERROR", "Export path is not specified");
                        //return ALSolutionCommon.PrepareErrorXML("Export path is not specified in input xml");
                        UpdateImportFormatStatus(RequestID, FormatImportStatus.Error);
                        return;
                    }


                    sbInputXML = new StringBuilder();

                    MainErrorXML = "<ERRORLIST></ERRORLIST>";

                    xpDoc = new XPathDocument(new StringReader(FormatInputXML));
                    xpNodeNav = xpDoc.CreateNavigator();

                    #region Create Format

                    xpNav = xpNodeNav.SelectSingleNode("//FORMAT");
                    if (xpNav != null)
                    {
                        ErrorXML = CreateFormat(xpNav, DesignID, CampaignID, out FormatID);
                        if (ErrorXML == null)
                            NextFormatId = FormatID;

                        if (ErrorXML != null)
                        {
                            //  MainErrorXML = MainErrorXML.Insert(MainErrorXML.IndexOf("</ERRORLIST>"), ErrorXML);
                            ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                            SaveStatusFile("ERROR", ErrorXML);
                            //return ErrorXML;
                            UpdateImportFormatStatus(RequestID, FormatImportStatus.Error);

                            return;
                        }
                        UpdateImportFormatStatus(RequestID, FormatImportStatus.CreatingElements);

                        SaveStatusFile("Format Creation is over", "");
                        xpNav = xpNav.SelectSingleNode("FORMAT_ARTWORKPAGE");
                        //xpNodeItr=xpNav.Select("FORMAT_ARTWORKPAGE
                        if (xpNav != null)
                        {
                            ErrorXML = CreateArtworkPages(xpNav, FormatID, FolderPath);
                            if (ErrorXML != null)
                            {
                                MainErrorXML = MainErrorXML.Insert(MainErrorXML.IndexOf("</ERRORLIST>"), ErrorXML);
                                ArtworkProducerWSobj.LogError("FormatFacade", MainErrorXML, ErrorType.ERROR, null, null);
                                SaveStatusFile("ERROR", ErrorXML);
                                //return MainErrorXML;
                                UpdateImportFormatStatus(RequestID, FormatImportStatus.Error);
                                String UpdateXML = "<FORMAT><FORMATID>" + FormatID + "</FORMATID><FORMATSTATUS>Error</FORMATSTATUS></FORMAT>";
                                UpdateFormat(ref UpdateXML);
                                return;
                            }
                        }

                    }
                    else
                    {
                        ErrorXML = ALSolutionCommon.PrepareErrorXML("Format Details are not specified in Format Input XML");
                        ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                        SaveStatusFile("ERROR", ErrorXML);
                        //return ErrorXML;
                        UpdateImportFormatStatus(RequestID, FormatImportStatus.Error);
                        return;
                    }

                    #endregion

                }
                else
                {
                    ErrorXML = ALSolutionCommon.PrepareErrorXML("Format input xml does not exist");
                    ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                    SaveStatusFile("ERROR", ErrorXML);
                    //return ErrorXML;
                    UpdateImportFormatStatus(RequestID, FormatImportStatus.InvalidZip);
                    return;
                }
                SaveStatusFile("Done", "");
                UpdateImportFormatStatus(RequestID, FormatImportStatus.Done);
                return;

            }
            catch (Exception ExcepObj)
            {
                //ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                //ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.EXCEPTION, null, null);
                objError.AppendError("APFACADE_FORMAT_IMPORTFORMAT_FAILED", this.GetType().ToString(), "ImportFormat", ExcepObj, null);
                ErrorXML = objError.ErrorCollectionXML;
                //objError.LogError(TenantName, ErrorXML);
                SaveStatusFile("ERROR", ErrorXML);
                //return ErrorXML;
                UpdateImportFormatStatus(RequestID, FormatImportStatus.Error);
                if (!String.IsNullOrEmpty(FormatID))
                {
                    String UpdateXML = "<FORMAT><FORMATID>" + FormatID + "</FORMATID><FORMATSTATUS>Error</FORMATSTATUS></FORMAT>";
                    UpdateFormat(ref UpdateXML);
                }
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "FormatFacade.cs", SessionID, "End ImportFormat");
                if (IsZippedContent == true)
                    NewContext.Undo();
            }
        }

        private String PrepareImportFolder(String RequestID, bool CheckFolder)
        {
            String str_ImportPath = null;
            StringBuilder sbImportPath = new StringBuilder(50);
            Context ContextObj = null;
            ulong uid = 0;
            ulong sid = 0;
            try
            {
                uid = ulong.Parse(UserID);
            }
            catch (Exception)
            {
                uid = 0;
            }
            try
            {
                sid = ulong.Parse(SessionID);
            }
            catch (Exception)
            {
                sid = 0;
            }
            StringBuilder ContextAdditionalInfo = null;
            ContextAdditionalInfo = new StringBuilder(20);
            ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{1}</THREADID><APINAME>ArtworkProducerFacade.FormatFacade.PrepareImportFolder</APINAME><REQUESTID>{0}</REQUESTID></ADDITIOALINFO>", RequestID, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

            ContextObj = new Context(uid, sid, TenantName, SolutionName, ContextAdditionalInfo.ToString());

            sbImportPath.AppendFormat("{0}\\ImportFormats\\{1}", ALSolutionCommon.GetDownloadFilePath(TenantName), RequestID);
            str_ImportPath = sbImportPath.ToString();
            if (CheckFolder)
            {

                if (Directory.Exists(str_ImportPath))
                {
                    //Directory.Delete(str_ImportPath, true);
                    SASMOMDirectory.DeleteFolder(str_ImportPath, true, ContextObj);
                }
                Directory.CreateDirectory(str_ImportPath);

            }
            return str_ImportPath;
        }

        private void UpdateImportFormatStatus(String RequestID, FormatImportStatus status)
        {
            String ImportPath = null;

            ImportPath = PrepareImportFolder(RequestID, false);
            File.WriteAllText(ImportPath + "\\status.txt", status.ToString());
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,
        /// to check whether campaign exist or not, if not it creates campaign.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="xpNav">Campaign Details Node</param>
        /// <param name="CampaignID">CampaignID</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateCampaign(XPathNavigator xpNav, out String CampaignID)
        {
            #region variable
            String CampaignName = null;
            String CampaignXML = null;

            StringBuilder sbInputXML = null;
            String InputXML = null;
            String OutputXML = null;

            ERRORXML ErrorXML = null;
            SearchConfiguration SearchConfigObj = null;




            #endregion

            try
            {

                sbInputXML = new StringBuilder();
                CampaignXML = xpNav.OuterXml;
                CampaignID = null;
                CampaignName = ALCommon.GetInnerValue(ref CampaignXML, "<CAMPAIGNNAME>", "</CAMPAIGNNAME>");
                SearchConfigObj = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);

                if (CampaignName != null)
                {
                    #region Check whether campaign exist or not
                    sbInputXML.Remove(0, sbInputXML.Length);
                    sbInputXML.AppendFormat(@"<INPUTXML>
                                                        <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}'/>
                                                        <SEARCHCOLLECTION>
                                                             <SEARCH>
                                                                <TARGETOBJECT>ARTWORKCAMPAIGN</TARGETOBJECT>
                                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                                <FILTERLIST>
	                                                                    <FILTER>
		                                                                    <FIELDNAME>CAMPAIGNNAME</FIELDNAME>
		                                                                    <OPERATOR>EQUALTO</OPERATOR>
		                                                                    <FIELDVALUE>{4}</FIELDVALUE>
		                                                                    <NAVIGATIONPATH>ARTWORKCAMPAIGN</NAVIGATIONPATH>
		                                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                                    </FILTER>
                                                                 </FILTERLIST>
                                                      </SEARCH></SEARCHCOLLECTION></INPUTXML>", SessionID, UserID, TenantName, CurrentLanguage, CampaignName);
                    //ErrorXML        =   .GetData(sbInputXML.ToString(),null,ref OutputXML);
                    ErrorXML = SearchConfigObj.DoSearch(sbInputXML.ToString(), ref OutputXML);
                    if (ErrorXML == null)
                    {
                        CampaignID = ALCommon.GetInnerValue(ref OutputXML, "<CAMPAIGNID>", "</CAMPAIGNID>");

                        if (CampaignID == null)
                        {
                            //Create Campaign
                            sbInputXML.Remove(0, sbInputXML.Length);
                            sbInputXML.Append("<ARTWORKCAMPAIGN>");
                            sbInputXML.AppendFormat("<CAMPAIGNNAME>{0}</CAMPAIGNNAME>", CampaignName);
                            if (xpNav.SelectSingleNode("CAMPAIGNNUMBER") != null)
                                sbInputXML.Append(xpNav.SelectSingleNode("CAMPAIGNNUMBER").OuterXml);
                            if (xpNav.SelectSingleNode("VALIDFROM ") != null)
                                sbInputXML.Append(xpNav.SelectSingleNode("VALIDFROM").OuterXml);
                            if (xpNav.SelectSingleNode("VALIDTO") != null)
                                sbInputXML.AppendFormat(xpNav.SelectSingleNode("VALIDTO").OuterXml);
                            if (xpNav.SelectSingleNode("CAMPAIGNDESCRIPTION ") != null)
                                sbInputXML.AppendFormat(xpNav.SelectSingleNode("CAMPAIGNDESCRIPTION ").OuterXml);
                            if (xpNav.SelectSingleNode("FILESYSTEM") != null)
                            {
                                String FileCollectionXML = null;
                                ErrorXML = SaveFiles(xpNav, out FileCollectionXML);
                                if (ErrorXML == null && FileCollectionXML != null)
                                    sbInputXML.AppendFormat(FileCollectionXML);
                                //else log error code
                            }

                            sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                      , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                            sbInputXML.AppendFormat("<CAMPAIGNSTATUS>0</CAMPAIGNSTATUS>");
                            sbInputXML.Append("</ARTWORKCAMPAIGN>");
                            InputXML = sbInputXML.ToString();


                            ErrorXML = ArtworkProducerWSobj.CreateCampaign(ref InputXML);
                            if (ErrorXML == null)
                            {
                                CampaignID = ALCommon.GetInnerValue(ref InputXML, "<CAMPAIGNID>", "</CAMPAIGNID>");
                                SaveStatusFile("Campaign creation is over", "");
                                return null;
                            }
                            else
                            {
                                // ApCommonObj.LogError("FormatEngine-CreateFormatNLevel", ErrorXML, ErrorType.ERROR);
                                return ErrorXML;
                            }
                        }
                        else
                        {
                            //Campaign already Exist
                            SaveStatusFile("Campaign is already Exist, Campaign check is over", "");
                            return null;
                        }
                    }
                    else
                    {
                        //ApCommonObj.LogError("FormatEngine-CreateFormatNLevel", ErrorXML, ErrorType.ERROR);
                        return ErrorXML;
                    }

                    #endregion

                }
                return ALSolutionCommon.PrepareErrorXML("Invalid Campaign Name");

            }
            catch (Exception ExcepObj)
            {
                CampaignID = null;
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                throw ExcepObj;
            }
        }



        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,
        /// to check whether Design exist or not, if not it creates Design.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="CampaignID">CampaignID</param>
        ///  <param name="CampaignID">DesignID as out param.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateDesign(XPathNavigator xpNav, String CampaignID, out String DesignID)
        {
            #region variable
            String DesignName = null;
            String DesignXML = null;


            StringBuilder sbInputXML = null;
            String InputXML = null;
            String OutputXML = null;

            ERRORXML ErrorXML = null;

            SearchConfiguration SearchConfigObj = null;

            #endregion

            try
            {

                sbInputXML = new StringBuilder();
                //OMIObj                  = new OMInterface(UserID, Language, TenantName);
                SearchConfigObj = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                DesignXML = xpNav.OuterXml;
                DesignID = null;
                DesignName = ALCommon.GetInnerValue(ref DesignXML, "<DESIGNNAME>", "</DESIGNNAME>");
                if (DesignName != null)
                {
                    #region Check whether Design exist or not
                    sbInputXML.Remove(0, sbInputXML.Length);
                    sbInputXML.AppendFormat(@"<INPUTXML>
                                                        <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}'/>
                                                        <SEARCHCOLLECTION>
                                                            <SEARCH>
                                                            <TARGETOBJECT>DESIGN</TARGETOBJECT>
                                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                            <FILTERLIST>
	                                                                <FILTER>
		                                                                <FIELDNAME>DESIGNNAME</FIELDNAME>
		                                                                <OPERATOR>EQUALTO</OPERATOR>
		                                                                <FIELDVALUE>" + DesignName + @"</FIELDVALUE>
		                                                                <NAVIGATIONPATH>DESIGN</NAVIGATIONPATH>
		                                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                                </FILTER>
                                                                    <FILTER>
		                                                               <FIELDNAME>CAMPAIGNID</FIELDNAME>
		                                                                <OPERATOR>EQUALTO</OPERATOR>
		                                                                <FIELDVALUE>" + CampaignID + @"</FIELDVALUE>
		                                                                <NAVIGATIONPATH>DESIGN.CAMPAIGN_DESIGN</NAVIGATIONPATH>
		                                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                                </FILTER>
                                                             </FILTERLIST>
                                                          </SEARCH></SEARCHCOLLECTION></INPUTXML>", SessionID, UserID, TenantName, CurrentLanguage);
                    //ErrorXML        =   OMIObj.GetData(sbInputXML.ToString(),null,ref OutputXML);
                    ErrorXML = SearchConfigObj.DoSearch(sbInputXML.ToString(), ref OutputXML);
                    if (ErrorXML == null)
                    {
                        DesignID = ALCommon.GetInnerValue(ref OutputXML, "<DESIGNID>", "</DESIGNID>");
                        if (DesignID == null)
                        {
                            //Create Campaign
                            sbInputXML.Remove(0, sbInputXML.Length);
                            sbInputXML.Append("<DESIGN>");
                            sbInputXML.AppendFormat("<DESIGNNAME>{0}</DESIGNNAME>", DesignName);
                            if (xpNav.SelectSingleNode("DESIGNSTATUS") != null)
                                sbInputXML.Append(xpNav.SelectSingleNode("DESIGNSTATUS").OuterXml);
                            if (xpNav.SelectSingleNode("DESIGNTYPE ") != null)
                                sbInputXML.Append(xpNav.SelectSingleNode("DESIGNTYPE").OuterXml);
                            if (xpNav.SelectSingleNode("VALIDFROM") != null)
                                sbInputXML.AppendFormat(xpNav.SelectSingleNode("VALIDFROM").OuterXml);
                            if (xpNav.SelectSingleNode("VALIDTO") != null)
                                sbInputXML.AppendFormat(xpNav.SelectSingleNode("VALIDTO").OuterXml);
                            if (xpNav.SelectSingleNode("DESIGNDESCRIPTION ") != null)
                                sbInputXML.AppendFormat(xpNav.SelectSingleNode("DESIGNDESCRIPTION ").OuterXml);
                            if (xpNav.SelectSingleNode("FILESYSTEM") != null)
                            {
                                // sbInputXML.AppendFormat(xpNav.SelectSingleNode("FILESYSTEM").OuterXml);
                                String FileCollectionXML = null;
                                ErrorXML = SaveFiles(xpNav, out FileCollectionXML);
                                if (ErrorXML == null && FileCollectionXML != null)
                                    sbInputXML.AppendFormat(FileCollectionXML);
                                //else log error code
                            }

                            //  sbInputXML.AppendFormat("<CAMPAIGNSTATUS>0</CAMPAIGNSTATUS>");
                            sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                        , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                            sbInputXML.AppendFormat("<CAMPAIGN_DESIGN TYPE='RELATION'>{0}</CAMPAIGN_DESIGN>", CampaignID);
                            sbInputXML.Append("</DESIGN>");
                            InputXML = sbInputXML.ToString();

                            // DesignEngineObj         =   new DesignEngine(UserID,Language,TenantName);
                            //ErrorXML                =   DesignEngineObj.CreateDesign(ref InputXML);
                            ErrorXML = ArtworkProducerWSobj.CreateDesign(ref InputXML);
                            if (ErrorXML == null)
                            {
                                DesignID = ALCommon.GetInnerValue(ref InputXML, "<DESIGNID>", "</DESIGNID>");
                                SaveStatusFile("Design Creation is over", "");
                                return null;
                            }
                            else
                            {
                                // ApCommonObj.LogError("FormatEngine-CreateFormatNLevel", ErrorXML, ErrorType.ERROR);
                                return ErrorXML;
                            }
                        }
                        else
                        {
                            //Campaign already Exist
                            SaveStatusFile("Design already exist, Design check is over", "");
                            return null;
                        }
                    }
                    else
                    {
                        //ApCommonObj.LogError("FormatEngine-CreateFormatNLevel", ErrorXML, ErrorType.ERROR);
                        return ErrorXML;
                    }

                    #endregion

                }
                return ALSolutionCommon.PrepareErrorXML("Invalid Design Name");

            }
            catch (Exception ExcepObj)
            {
                DesignID = null;
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                throw ExcepObj;
            }
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Format.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="xpNav">Format Details</param>
        /// <param name="DesignID">DesignID</param>
        /// <param name="FormatID">FormatID as out param.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateFormat(XPathNavigator xpNav, String DesignID, String CampaignID, out String FormatID)
        {
            #region variable
            String FormatName = null;
            String FormatXML = null;

            StringBuilder sbInputXML = null;
            String InputXML = null;
            ERRORXML ErrorXML = null;
            String FileCollectionXML = null;
            #endregion

            try
            {

                sbInputXML = new StringBuilder();
                FormatXML = xpNav.OuterXml;
                FormatID = null;

                //Create Format
                sbInputXML.Remove(0, sbInputXML.Length);
                sbInputXML.Append("<FORMAT>");
                if (xpNav.SelectSingleNode("FORMATNAME") != null)
                {
                    sbInputXML.Append(xpNav.SelectSingleNode("FORMATNAME").OuterXml);
                    FormatName = xpNav.SelectSingleNode("FORMATNAME").InnerXml;
                }
                sbInputXML.Append("<FORMATSTATUS>Draft</FORMATSTATUS>");
                if (xpNav.SelectSingleNode("FORMATTYPE") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("FORMATTYPE").OuterXml);
                if (xpNav.SelectSingleNode("VALIDFROM ") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("VALIDFROM ").OuterXml);
                if (xpNav.SelectSingleNode("VALIDTO") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("VALIDTO").OuterXml);
                if (xpNav.SelectSingleNode("FORMATDESCRIPTION  ") != null)
                    sbInputXML.Append(xpNav.SelectSingleNode("FORMATDESCRIPTION  ").OuterXml);
                if (xpNav.SelectSingleNode("ISGRAPHICELEMENTPRESENT  ") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("ISGRAPHICELEMENTPRESENT  ").OuterXml);
                if (xpNav.SelectSingleNode("THUMBNAILACTUALHEIGHT") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("THUMBNAILACTUALHEIGHT  ").OuterXml);
                if (xpNav.SelectSingleNode("THUMBNAILACTUALWIDTH") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("THUMBNAILACTUALWIDTH  ").OuterXml);
                if (xpNav.SelectSingleNode("CATEGORY") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("CATEGORY").OuterXml);
                if (xpNav.SelectSingleNode("RESERVEDFORBOOK") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("RESERVEDFORBOOK").OuterXml);
                if (xpNav.SelectSingleNode("THUMBNAILFILEUPLOADED") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("THUMBNAILFILEUPLOADED").OuterXml);
                if (xpNav.SelectSingleNode("PREVIEWFILEUPLOADED") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("PREVIEWFILEUPLOADED").OuterXml);
                if (xpNav.SelectSingleNode("ISMULTIPAGE") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("ISMULTIPAGE").OuterXml);
                if (xpNav.SelectSingleNode("ISSPREADWISE") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("ISSPREADWISE").OuterXml);
                if (xpNav.SelectSingleNode("USAGE") != null)
                    sbInputXML.AppendFormat(xpNav.SelectSingleNode("USAGE").OuterXml);

                if (xpNav.SelectSingleNode("FILESYSTEM") != null)
                {
                    ErrorXML = SaveFiles(xpNav, out FileCollectionXML);
                    if (ErrorXML == null && FileCollectionXML != null)
                        sbInputXML.AppendFormat(FileCollectionXML);
                    else if (ErrorXML != null)
                        return ErrorXML;
                }
                sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                        , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                #region FormatInfo and Additional facet information

                XPathDocument xpAssociationXMLDoc = null;
                XPathNavigator xpAssociationXMLNav = null;
                XPathNavigator xpRelationshipNav = null;
                XPathNodeIterator xpAssociationXMLNodeList = null;

                XPathDocument xpTypeInfoXMLDoc = null;
                XPathNavigator xpTypeInfoXMLNav = null;

                String TypeInfoXML = "";
                String AssociationXML = "";

                String RelationShipName = null;
                String FacetTypeName = null;
                String FacetTypeID = null;
                String FacetPrimaryKey = null;
                String RelationInfo = null;

                TMSFacade.TMFacade TMFacadeObj = new Assetlink.Solutions.Facade.TMSFacade.TMFacade(SessionID, UserID, TenantName, CurrentLanguage);

                ErrorXML = TMFacadeObj.GetAssociatedObjectTypes("ENTITY", "FORMAT", ref AssociationXML);

                if (ErrorXML == null)
                {
                    xpAssociationXMLDoc = new XPathDocument(new StringReader(AssociationXML));
                    xpAssociationXMLNav = xpAssociationXMLDoc.CreateNavigator();

                    xpAssociationXMLNodeList = xpAssociationXMLNav.Select("RELATIONSHIPTYPECOLLECTION/FACETCOLLECTION/RELATIONSHIPTYPE");

                    while (xpAssociationXMLNodeList.MoveNext())
                    {
                        RelationShipName = xpAssociationXMLNodeList.Current.SelectSingleNode("RELATIONSHIPNAME").InnerXml;
                        FacetTypeName = xpAssociationXMLNodeList.Current.SelectSingleNode("CONTAINSTYPENAME").InnerXml;
                        FacetTypeID = xpAssociationXMLNodeList.Current.SelectSingleNode("CONTAINSTYPEID").InnerXml;

                        xpRelationshipNav = xpNav.SelectSingleNode(RelationShipName);

                        if (xpRelationshipNav != null)
                        {
                            TypeInfoXML = null;

                            ErrorXML = TMFacadeObj.GetTypeInfo("FACET", FacetTypeID, ref TypeInfoXML);

                            if (ErrorXML == null)
                            {
                                xpTypeInfoXMLDoc = new XPathDocument(new StringReader(TypeInfoXML));
                                xpTypeInfoXMLNav = xpTypeInfoXMLDoc.CreateNavigator();

                                FacetPrimaryKey = xpTypeInfoXMLNav.SelectSingleNode("TYPE/PRIMARYKEY").InnerXml;
                                RelationInfo = xpRelationshipNav.InnerXml;
                                RelationInfo = RelationInfo.Remove(RelationInfo.IndexOf("<" + FacetPrimaryKey + ">"), RelationInfo.IndexOf("</" + FacetPrimaryKey + ">") - RelationInfo.IndexOf("<" + FacetPrimaryKey));
                                RelationInfo = RelationInfo.Replace("</" + FacetPrimaryKey + ">", "");
                                sbInputXML.AppendFormat("<{0} TYPE='RELATION'>{1}</{0}>", RelationShipName, RelationInfo);
                            }
                        }

                    }
                }

                //String FormatInfo   =   null;;
                //if (xpNav.SelectSingleNode("FORMAT_FORMATINFO") != null)
                //{
                //    FormatInfo = xpNav.SelectSingleNode("FORMAT_FORMATINFO").InnerXml;
                //    FormatInfo = FormatInfo.Remove(FormatInfo.IndexOf("<FORMATINFOID>"), FormatInfo.IndexOf("</FORMATINFOID>") - FormatInfo.IndexOf("<FORMATINFOID"));
                //    FormatInfo = FormatInfo.Replace("</FORMATINFOID>", "");
                //    sbInputXML.AppendFormat("<FORMAT_FORMATINFO TYPE='RELATION'>{0}</FORMAT_FORMATINFO>", FormatInfo);
                //}
                #endregion

                sbInputXML.AppendFormat("<DESIGN_FORMAT TYPE='RELATION'>{0}</DESIGN_FORMAT>", DesignID);
                sbInputXML.AppendFormat("<CAMPAIGN_FORMAT TYPE='RELATION'>{0}</CAMPAIGN_FORMAT>", CampaignID);
                sbInputXML.AppendFormat("<FORMAT_CREATOR TYPE='RELATION'>{0}</FORMAT_CREATOR>", UserID);
                sbInputXML.Append("</FORMAT>");
                InputXML = sbInputXML.ToString();
                ErrorXML = ArtworkProducerWSobj.CreateFormatWODefPageAndGroup(ref InputXML);

                if (ErrorXML == null)
                {
                    FormatID = ALCommon.GetInnerValue(ref InputXML, "<FORMATID>", "</FORMATID>");
                    return null;
                }
                else
                {
                    return ErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                FormatID = null;
                throw ExcepObj;
            }
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Artwork Pages.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="xpNav">Artwork Page Details</param>
        /// <param name="FormatID">FormatID</param>
        /// <param name="Path">Format Folder Path.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateArtworkPages(XPathNavigator xpNav, String FormatID, String Path)
        {
            #region variable
            XPathNodeIterator xpNodeItr = null;
            StringBuilder sbInputXML = null;
            XML ArtworkPageXML = null;
            ERRORXML ErrorXML = null;
            String ArtworkPageId = null;
            StringBuilder MainErrorXML = null;


            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXML = new StringBuilder();
                MainErrorXML.AppendFormat("<ARTWORKPAGE_ERRORLIST></ARTWORKPAGE_ERRORLIST>");
                SaveStatusFile("Creating ArtworkPages", "");
                xpNodeItr = xpNav.Select("ARTWORKPAGE");
                while (xpNodeItr.MoveNext())
                {
                    sbInputXML.Remove(0, sbInputXML.Length);
                    ErrorXML = null;
                    ArtworkPageXML = null;
                    ArtworkPageId = null;

                    sbInputXML.AppendFormat("<ARTWORKPAGE>");

                    if (xpNodeItr.Current.SelectSingleNode("ARTWORKPAGENAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("ARTWORKPAGENAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("ARTWORKPAGETITLE ") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("ARTWORKPAGETITLE ").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("DESCRIPTION") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("DESCRIPTION").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("ISDEFAULT") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISDEFAULT").OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISDEFAULT>{0}</ISDEFAULT>");

                    if (xpNodeItr.Current.SelectSingleNode("ARTWORKPAGEORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ARTWORKPAGEORDER").OuterXml);

                    sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                              , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    sbInputXML.AppendFormat("<FORMAT_ARTWORKPAGE TYPE='RELATION'>{0}</FORMAT_ARTWORKPAGE>", FormatID);
                    sbInputXML.AppendFormat("</ARTWORKPAGE>");
                    ArtworkPageXML = sbInputXML.ToString();
                    ErrorXML = ArtworkProducerWSobj.CreateArtworkPageWODefGroup(ref ArtworkPageXML);
                    if (ErrorXML != null)
                    {
                        MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ARTWORKPAGE_ERRORLIST>"), ErrorXML);
                    }
                    else
                    {
                        ArtworkPageId = ALCommon.GetInnerValue(ref ArtworkPageXML, "<ARTWORKPAGEID>", "</ARTWORKPAGEID>");
                        if (ArtworkPageId != null)
                        {
                            ErrorXML = CreateElementGroups(xpNodeItr.Current, Path, ArtworkPageId, FormatID);
                            if (ErrorXML != null)
                                // return ErrorXML;
                                MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ARTWORKPAGE_ERRORLIST>"), ErrorXML);
                        }
                        else
                            MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ARTWORKPAGE_ERRORLIST>"), ALSolutionCommon.PrepareErrorXML("Invalid Artwork Page Id in createArtworkPages method of FormatEngine" + ArtworkPageXML));

                    }
                }


                if (MainErrorXML.ToString().Equals("<ARTWORKPAGE_ERRORLIST></ARTWORKPAGE_ERRORLIST>"))
                {
                    SaveStatusFile("Artwork Pages are created successfully", "");
                    return null;

                }
                else
                    return MainErrorXML.ToString();
                //return null;

            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.EXCEPTION, null, null);
                return ExcepObj.Message;
            }

        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Element Groups.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="xpNav">Element Group Details</param>
        /// <param name="Path">Format Folder Path</param>
        /// <param name="ArtworkPageID">Artwork Page ID.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateElementGroups(XPathNavigator xpNav, String Path, String ArtworkPageID, String FormatID)
        {

            #region variable
            XPathNodeIterator xpNodeItr = null;
            StringBuilder sbInputXML = null;
            XML ElementGroupXML = null;
            ERRORXML ErrorXML = null;
            String ElementGroupId = null;
            StringBuilder MainErrorXML = null;

            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXML = new StringBuilder();
                MainErrorXML.AppendFormat("<ELEMENTGROUP_ERRORLIST></ELEMENTGROUP_ERRORLIST>");
                xpNodeItr = xpNav.Select("ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP");
                SaveStatusFile("Creating ElementGroups", "");
                while (xpNodeItr.MoveNext())
                {
                    ElementGroupId = null;
                    ErrorXML = null;
                    ElementGroupXML = null;
                    sbInputXML.Remove(0, sbInputXML.Length);
                    sbInputXML.AppendFormat("<ELEMENTGROUP>");

                    if (xpNodeItr.Current.SelectSingleNode("ELEMENTGROUPNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("ELEMENTGROUPNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("DESCRIPTION") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("DESCRIPTION").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("ISDEFAULT") != null && xpNodeItr.Current.SelectSingleNode("ISDEFAULT").Value != "")
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISDEFAULT").OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISDEFAULT>{0}</ISDEFAULT>");

                    if (xpNodeItr.Current.SelectSingleNode("ELEMENTGROUPORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ELEMENTGROUPORDER").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("CONTENTTYPE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("CONTENTTYPE").OuterXml);


                    sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                              , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                    sbInputXML.AppendFormat("<ARTWORKPAGE_ELEMENTGROUP TYPE='RELATION'>{0}</ARTWORKPAGE_ELEMENTGROUP>", ArtworkPageID);
                    sbInputXML.AppendFormat("</ELEMENTGROUP>");
                    ElementGroupXML = sbInputXML.ToString();
                    ErrorXML = ArtworkProducerWSobj.CreateElementGroup(ref ElementGroupXML);
                    if (ErrorXML != null)
                    {
                        //   return ErrorXML;
                        MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ELEMENTGROUP_ERRORLIST>"), ErrorXML);
                    }
                    else
                    {
                        ElementGroupId = ALCommon.GetInnerValue(ref ElementGroupXML, "<ELEMENTGROUPID>", "</ELEMENTGROUPID>");
                        if (!String.IsNullOrEmpty(ElementGroupId))
                        {
                            ErrorXML = CreateTextElements(xpNodeItr.Current, ElementGroupId, FormatID);
                            if (ErrorXML != null)
                            {
                                // return ErrorXML;
                                MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ELEMENTGROUP_ERRORLIST>"), ErrorXML);
                            }
                            ErrorXML = CreateGraphicElements(xpNodeItr.Current, Path, ElementGroupId, FormatID);
                            if (ErrorXML != null)
                                //return ErrorXML;
                                MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ELEMENTGROUP_ERRORLIST>"), ErrorXML);
                            ErrorXML = CreateSetCollections(xpNodeItr.Current, ElementGroupId);
                            if (ErrorXML != null)
                                //return ErrorXML;
                                MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ELEMENTGROUP_ERRORLIST>"), ErrorXML);

                        }
                        else
                            MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</ELEMENTGROUP_ERRORLIST>"), ALSolutionCommon.PrepareErrorXML("Invalide Element Group ID" + ElementGroupXML));

                    }

                }
                // return null;
                if (MainErrorXML.ToString().Equals("<ELEMENTGROUP_ERRORLIST></ELEMENTGROUP_ERRORLIST>"))
                {
                    SaveStatusFile("ElementGroups are created successfully", "");
                    return null;

                }
                else
                    return MainErrorXML.ToString();
            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.EXCEPTION, null, null);
                return ExcepObj.Message;

            }
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Text Elements.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="TextElementNav">Text Element Details</param>
        /// <param name="ElementGroupID">ElementGroupID.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateTextElements(XPathNavigator TextElementNav, String ElementGroupID, String FormatID)
        {

            #region variable
            XPathNodeIterator xpNodeItr = null;
            XPathNodeIterator TONodeItr = null;
            StringBuilder sbInputXML = null;
            XML TextElementXML = null;
            ERRORXML ErrorXML = null;
            StringBuilder MainErrorXL = null;
            // TextElementEngine       TElementEngineObj=   null;
            String OldTextElementID = null;
            String TextElementID = null;
            String TextElementOptionXML = null;
            String TextElementOptID = null;
            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXL = new StringBuilder();
                MainErrorXL.AppendFormat("<TEXTELEMENT_ERRORLIST></TEXTELEMENT_ERRORLIST>");
                xpNodeItr = TextElementNav.Select("ELEMENTGROUP_TEXTELEMENT/TEXTELEMENT");
                SaveStatusFile("Creating Text Elements", "");
                while (xpNodeItr.MoveNext())
                {
                    sbInputXML.Remove(0, sbInputXML.Length);
                    OldTextElementID = null;
                    TextElementID = null;
                    ErrorXML = null;
                    TextElementOptionXML = null;
                    TextElementXML = null;
                    TextElementOptID = "$AL$";
                    sbInputXML.AppendFormat("<TEXTELEMENT>");
                    if (xpNodeItr.Current.SelectSingleNode("TEXTELEMENTID") != null)
                        OldTextElementID = xpNodeItr.Current.SelectSingleNode("TEXTELEMENTID").Value;
                    if (xpNodeItr.Current.SelectSingleNode("TEXTTAGNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("TEXTTAGNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("TEXTELEMENTNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("TEXTELEMENTNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("DESCRIPTION ") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("DESCRIPTION ").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("EDITABLE") != null && xpNodeItr.Current.SelectSingleNode("EDITABLE").Value != "")
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("EDITABLE").OuterXml);
                    else
                        sbInputXML.AppendFormat("<EDITABLE>0</EDITABLE>");

                    if (xpNodeItr.Current.SelectSingleNode("ELEMENTDISPLAYORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ELEMENTDISPLAYORDER").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("BELONGSTOSET") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("BELONGSTOSET").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("TEXTELEMENTTYPE") != null && xpNodeItr.Current.SelectSingleNode("TEXTELEMENTTYPE").Value != "")
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("TEXTELEMENTTYPE").OuterXml);
                    else
                        sbInputXML.AppendFormat("<TEXTELEMENTTYPE>0</TEXTELEMENTTYPE>");

                    if (xpNodeItr.Current.SelectSingleNode("ISMANDATORY") != null && xpNodeItr.Current.SelectSingleNode("ISMANDATORY").Value != "")
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISMANDATORY").OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISMANDATORY>0</ISMANDATORY>");

                    if (xpNodeItr.Current.SelectSingleNode("MAXCHARSALLOWED") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("MAXCHARSALLOWED").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ISHIDDEN") != null && xpNodeItr.Current.SelectSingleNode("ISHIDDEN").Value != "")
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISHIDDEN").OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISHIDDEN>0</ISHIDDEN>");
                    sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                              , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                    if (xpNodeItr.Current.SelectSingleNode("FIELDNODE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("FIELDNODE").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("DATATYPE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("DATATYPE").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ISMULTIPLEALLOWED") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISMULTIPLEALLOWED").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ISTAXONOMYSEARCHPRESENT") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISTAXONOMYSEARCHPRESENT").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("TAGCONFIGURATIONXML") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("TAGCONFIGURATIONXML").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("DATASOURCE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("DATASOURCE").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("AUTOPOPULATE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("AUTOPOPULATE").OuterXml);

                    sbInputXML.AppendFormat("<FORMAT_TEXTELEMENT TYPE='RELATION'>{0}</FORMAT_TEXTELEMENT>", FormatID);

                    TONodeItr = null;
                    TONodeItr = xpNodeItr.Current.Select("TEXTELEMENT_OPTION/TEXTELEMENTOPTION");
                    while (TONodeItr.MoveNext())
                    {
                        String TEOptionID = null;
                        TextElementOptionXML = TONodeItr.Current.OuterXml;
                        TEOptionID = ALCommon.GetInnerValue(ref TextElementOptionXML, "<TEXTELEMENTOPTIONID>", "</TEXTELEMENTOPTIONID>");
                        if (!String.IsNullOrEmpty(TEOptionID))
                            TextElementOptID += TEOptionID + "$AL$";

                        if (TextElementOptionXML.Contains("<TEXTELEMENTOPTIONID>"))
                        {
                            TextElementOptionXML = TextElementOptionXML.Remove(TextElementOptionXML.IndexOf("<TEXTELEMENTOPTIONID>"), TextElementOptionXML.IndexOf("</TEXTELEMENTOPTIONID>") - TextElementOptionXML.IndexOf("<TEXTELEMENTOPTIONID"));
                            TextElementOptionXML = TextElementOptionXML.Replace("</TEXTELEMENTOPTIONID>", "");
                        }
                        //  sbInputXML.AppendFormat("<TEXTELEMENT_OPTION TYPE='RELATION'>{0}</TEXTELEMENT_OPTION>",TONodeItr.Current.OuterXml);
                        sbInputXML.AppendFormat("<TEXTELEMENT_OPTION TYPE='RELATION'>{0}</TEXTELEMENT_OPTION>", TextElementOptionXML);
                    }
                    sbInputXML.AppendFormat("<ELEMENTGROUP_TEXTELEMENT TYPE='RELATION'>{0}</ELEMENTGROUP_TEXTELEMENT>", ElementGroupID);
                    sbInputXML.AppendFormat("</TEXTELEMENT>");

                    TextElementXML = null;
                    TextElementXML = sbInputXML.ToString();
                    // ErrorXML        =   TElementEngineObj.CreateTextElement(ref TextElementXML);
                    ErrorXML = ArtworkProducerWSobj.CreateTextElement(ref TextElementXML);
                    if (ErrorXML != null)
                        MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</TEXTELEMENT_ERRORLIST>"), ErrorXML);
                    else
                    {
                        TextElementID = ALCommon.GetInnerValue(ref TextElementXML, "<TEXTELEMENTID>", "</TEXTELEMENTID>");
                        if (!String.IsNullOrEmpty(TextElementID) && !String.IsNullOrEmpty(OldTextElementID))
                            ImportFormatIDs.Add(OldTextElementID, TextElementID);

                        #region Add Text ElementOption Id in Import Format List
                        String[] TEOptID = TextElementOptID.Split(new String[] { "$AL$" }, StringSplitOptions.RemoveEmptyEntries);
                        if (TEOptID.Length > 0)
                        {
                            XPathDocument xpOptDoc = new XPathDocument(new StringReader(TextElementXML));
                            XPathNodeIterator xpOptNodeItr = xpOptDoc.CreateNavigator().Select("TEXTELEMENT/TEXTELEMENT_OPTION/TEXTELEMENTOPTION/TEXTELEMENTOPTIONID");
                            while (xpOptNodeItr.MoveNext())
                            {
                                if (xpOptNodeItr.CurrentPosition - 1 < TEOptID.Length)
                                    ImportFormatIDs.Add(TEOptID[xpOptNodeItr.CurrentPosition - 1], xpOptNodeItr.Current.Value);
                            }

                        }
                        #endregion


                    }

                }

                if (MainErrorXL.ToString().Equals("<TEXTELEMENT_ERRORLIST></TEXTELEMENT_ERRORLIST>"))
                {
                    SaveStatusFile("TextElements created successfully", "");
                    return null;
                }
                else
                    return MainErrorXL.ToString();
            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                return ExcepObj.Message;
            }

        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Set Collections.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="SectCollectionNav">Set Collection Detail</param>
        /// <param name="ElementGroupID">ElementGroupID.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateSetCollections(XPathNavigator SectCollectionNav, String ElementGroupID)
        {
            #region variable
            StringBuilder sbInputXML = null;
            XML SetCollectionXML = null;
            ERRORXML ErrorXML = null;
            StringBuilder MainErrorXL = null;
            XPathNodeIterator xpNodeItr = null;
            String SetCollectionID = null;
            SetCollectionFacade SetCollectionFacadeObj = null;

            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXL = new StringBuilder();
                SetCollectionFacadeObj = new SetCollectionFacade(SessionID, UserID, TenantName, CurrentLanguage);
                MainErrorXL.AppendFormat("<SETCOLLECTION_ERRORLIST></SETCOLLECTION_ERRORLIST>");
                xpNodeItr = SectCollectionNav.Select("ELEMENTGROUP_SETCOLLECTION/SETCOLLECTION");
                SaveStatusFile("Creating SetCollections", "");
                while (xpNodeItr.MoveNext())
                {
                    sbInputXML.Remove(0, sbInputXML.Length);
                    ErrorXML = null;
                    SetCollectionXML = null;
                    sbInputXML.AppendFormat("<SETCOLLECTION>");
                    if (xpNodeItr.Current.SelectSingleNode("SETCOLLECTIONNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("SETCOLLECTIONNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETCOLLECTIONDESCRIPTION") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("SETCOLLECTIONDESCRIPTION").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETCOLLECTIONORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("SETCOLLECTIONORDER").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("LAYOUT") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("LAYOUT").OuterXml);
                    sbInputXML.AppendFormat("</SETCOLLECTION>");
                    SetCollectionXML = sbInputXML.ToString();
                    ErrorXML = SetCollectionFacadeObj.CreateSetCollection(ElementGroupID, "ELEMENTGROUP_SETCOLLECTION", ref SetCollectionXML);
                    if (ErrorXML != null)
                        MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SETCOLLECTION_ERRORLIST>"), ErrorXML);
                    else
                    {
                        SetCollectionID = ALCommon.GetInnerValue(ref SetCollectionXML, "<SETCOLLECTIONID>", "</SETCOLLECTIONID>");

                        if (SetCollectionID != null)
                        {
                            ErrorXML = CreateSets(xpNodeItr.Current, SetCollectionID);
                            if (ErrorXML != null)
                                MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SETCOLLECTION_ERRORLIST>"), ErrorXML);
                        }
                        else

                            MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SETCOLLECTION_ERRORLIST>"), ALSolutionCommon.PrepareErrorXML("Invalid Set Collection Id" + SetCollectionXML));
                    }



                }

                if (MainErrorXL.ToString().Equals("<SETCOLLECTION_ERRORLIST></SETCOLLECTION_ERRORLIST>"))
                {
                    SaveStatusFile("Set collections are created successfully", "");
                    return null;
                }
                else
                    return MainErrorXL.ToString();

            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                return ExcepObj.Message;

            }
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Set .
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="SetNav">Set Detail</param>
        /// <param name="SetCollectionID">SetCollectionID.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateSets(XPathNavigator SetNav, String SetCollectionID)
        {

            #region variable
            StringBuilder sbInputXML = null;
            XML SetXML = null;
            ERRORXML ErrorXML = null;
            StringBuilder MainErrorXL = null;
            XPathNodeIterator xpNodeItr = null;
            String SetID = null;
            SetFacade SetFacadeObj = null;

            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXL = new StringBuilder();
                SetFacadeObj = new SetFacade(SessionID, UserID, TenantName, CurrentLanguage);
                MainErrorXL.AppendFormat("<SET_ERRORLIST></SET_ERRORLIST>");
                xpNodeItr = SetNav.Select("SETCOLLECTION_SET/SET");
                SaveStatusFile("Creating Sets", "");
                while (xpNodeItr.MoveNext())
                {

                    ErrorXML = null;
                    SetXML = null;
                    sbInputXML.Remove(0, sbInputXML.Length);
                    sbInputXML.AppendFormat("<SET>");
                    if (xpNodeItr.Current.SelectSingleNode("SETNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("SETNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETDESCRIPTION") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("SETDESCRIPTION").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("SETORDER").OuterXml);
                    sbInputXML.AppendFormat("</SET>");
                    SetXML = sbInputXML.ToString();
                    ErrorXML = SetFacadeObj.CreateSet(SetCollectionID, "SETCOLLECTION_SET", ref SetXML);
                    if (ErrorXML != null)
                        MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SET_ERRORLIST>"), ErrorXML);
                    else
                    {
                        SetID = ALCommon.GetInnerValue(ref SetXML, "<SETID>", "</SETID>");
                        if (SetID != null)
                        {
                            ErrorXML = CreateSetElements(xpNodeItr.Current, SetID);
                            if (ErrorXML != null)
                                MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SET_ERRORLIST>"), ErrorXML);
                        }
                        else
                            MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SET_ERRORLIST>"), ALSolutionCommon.PrepareErrorXML("Invalid Set ID" + SetXML));



                    }


                }
                if (MainErrorXL.ToString().Equals("<SET_ERRORLIST></SET_ERRORLIST>"))
                {
                    SaveStatusFile("Sets created successfully", "");
                    return null;
                }
                else
                    return MainErrorXL.ToString();


            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                return ExcepObj.Message;
            }

        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create SetElements.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="SetElementNav">SetElements Detail</param>
        /// <param name="SetID">SetID.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateSetElements(XPathNavigator SetElementNav, String SetID)
        {
            #region variable
            StringBuilder sbInputXML = null;
            XML SetElementXML = null;
            ERRORXML ErrorXML = null;
            StringBuilder MainErrorXL = null;
            XPathNodeIterator xpNodeItr = null;
            XPathNodeIterator xpRelNodeItr = null;
            // String                  SetID           =   null;
            SetElementFacade SetEleFacadeObj = null;
            XPathNavigator xpTempNav = null;

            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXL = new StringBuilder();
                SetEleFacadeObj = new SetElementFacade(SessionID, UserID, TenantName, CurrentLanguage);
                MainErrorXL.AppendFormat("<SETELEMENT_ERRORLIST></SETELEMENT_ERRORLIST>");
                xpNodeItr = SetElementNav.Select("SET_SETELEMENT/SETELEMENT");
                SaveStatusFile("Creating Set Elements", "");
                while (xpNodeItr.MoveNext())
                {

                    ErrorXML = null;
                    SetElementXML = null;
                    sbInputXML.Remove(0, sbInputXML.Length);
                    sbInputXML.AppendFormat("<SETELEMENT>");

                    if (xpNodeItr.Current.SelectSingleNode("SETELEMENTNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("SETELEMENTNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETELEMENTDESCRIPTION") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("SETELEMENTDESCRIPTION").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETELEMENTORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("SETELEMENTORDER").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("SETELEMENTTYPE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("SETELEMENTTYPE").OuterXml);
                    xpTempNav = xpNodeItr.Current.SelectSingleNode("ISHIDDEN");
                    if (xpTempNav != null && xpTempNav.Value != "")
                        sbInputXML.AppendFormat(xpTempNav.OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISHIDDEN>0</ISHIDDEN>");
                    xpTempNav = xpNodeItr.Current.SelectSingleNode("ISMULTISELECT");
                    //  if (xpNodeItr.Current.SelectSingleNode("ISMULTISELECT") != null)
                    //    sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISMULTISELECT").OuterXml);
                    if (xpTempNav != null && xpTempNav.Value != "")
                        sbInputXML.AppendFormat(xpTempNav.OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISMULTISELECT>0</ISMULTISELECT>");

                    if (xpNodeItr.Current.SelectSingleNode("WIDTH") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("WIDTH").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("MINOPTIONS") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("MINOPTIONS").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("MAXOPTIONS") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("MAXOPTIONS").OuterXml);
                    xpRelNodeItr = xpNodeItr.Current.Select("SETELEMENT_TEXTELEMENT/TEXTELEMENT");
                    while (xpRelNodeItr.MoveNext())
                    {

                        sbInputXML.AppendFormat("<SETELEMENT_TEXTELEMENT TYPE='RELATION'>{0}</SETELEMENT_TEXTELEMENT>", ImportFormatIDs[xpRelNodeItr.Current.SelectSingleNode("TEXTELEMENTID").Value]);
                    }
                    xpRelNodeItr = null;

                    xpRelNodeItr = xpNodeItr.Current.Select("SETELEMENT_TEXTELEMENTOPTION/TEXTELEMENTOPTION");
                    while (xpRelNodeItr.MoveNext())
                        sbInputXML.AppendFormat("<SETELEMENT_TEXTELEMENTOPTION TYPE='RELATION'>{0}</SETELEMENT_TEXTELEMENTOPTION>", ImportFormatIDs[xpRelNodeItr.Current.SelectSingleNode("TEXTELEMENTOPTIONID").Value]);
                    xpRelNodeItr = null;

                    xpRelNodeItr = xpNodeItr.Current.Select("SETELEMENT_DIGITALASSET/DIGITALASSET");
                    while (xpRelNodeItr.MoveNext())
                    {
                        XPathNavigator xnav = xpRelNodeItr.Current.SelectSingleNode("DIGITALASSETID");
                        String PrevDigitalAssetID = null;
                        if (xnav != null)
                            PrevDigitalAssetID = xnav.Value;

                        if (PrevDigitalAssetID != null)
                            sbInputXML.AppendFormat("<SETELEMENT_DIGITALASSET TYPE='RELATION'>{0}</SETELEMENT_DIGITALASSET>", ImportFormatIDs[PrevDigitalAssetID]);
                    }
                    xpRelNodeItr = null;

                    xpRelNodeItr = xpNodeItr.Current.Select("SETELEMENT_GRAPHICELEMENT/GRAPHICELEMENT");
                    while (xpRelNodeItr.MoveNext())
                        sbInputXML.AppendFormat("<SETELEMENT_GRAPHICELEMENT TYPE='RELATION'>{0}</SETELEMENT_GRAPHICELEMENT>", ImportFormatIDs[xpRelNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").Value]);
                    xpRelNodeItr = null;

                    sbInputXML.AppendFormat("<SET_SETELEMENT TYPE='RELATION'>{0}</SET_SETELEMENT>", SetID);
                    sbInputXML.AppendFormat("</SETELEMENT>");
                    SetElementXML = sbInputXML.ToString();
                    ErrorXML = SetEleFacadeObj.CreateSetElement(ref  SetElementXML);
                    if (ErrorXML != null)
                        MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</SETELEMENT_ERRORLIST>"), ErrorXML);


                }
                if (MainErrorXL.ToString().Equals("<SETELEMENT_ERRORLIST></SETELEMENT_ERRORLIST>"))
                {
                    SaveStatusFile("Set Elements Created Successfully", "");
                    return null;
                }
                else
                    return MainErrorXL.ToString();


            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);

                return ExcepObj.Message;
            }
        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to create Graphic Elements.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="GraphicElementNav">Graphic Element Detail</param>
        /// <param name="Path">Format Folder path to get Files.</param>
        ///  <param name="ElementGroupID">Format Folder path to get Files.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CreateGraphicElements(XPathNavigator GraphicElementNav, String Path, String ElementGroupID, String FormatID)
        {

            #region variable
            XPathNodeIterator xpNodeItr = null;
            XPathNodeIterator xpGELinkedAssets = null;
            StringBuilder sbInputXML = null;
            XML GraphicElementXML = null;
            ERRORXML ErrorXML = null;
            StringBuilder MainErrorXL = null;
            // GraphicElementEngine    GEEngineObj     =   null;
            //Boolean                 DamItemSelection=   false;
            String GraphicElemntTagName = null;
            String GraphicElementID = null;
            String OldGEID = null;
            StringBuilder sbGELinkedOptionsAssociationXML = null;
            String GELinkedOptionsAssociationXML = null;
            ulong LinkedAssetId = 0;

            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                MainErrorXL = new StringBuilder();
                // GEEngineObj         =   new GraphicElementEngine(UserID,Language,TenantName);
                MainErrorXL.AppendFormat("<GRAPHICELEMENT_ERRORLIST></GRAPHICELEMENT_ERRORLIST>");
                xpNodeItr = GraphicElementNav.Select("ELEMENTGROUP_GRAPHICELEMENT/GRAPHICELEMENT");
                SaveStatusFile("Creating Graphic Elements", "");
                while (xpNodeItr.MoveNext())
                {
                    sbInputXML.Remove(0, sbInputXML.Length);
                    GraphicElementXML = null;
                    ErrorXML = null;
                    //DamItemSelection        =   false;
                    OldGEID = null;

                    sbInputXML.AppendFormat("<GRAPHICELEMENT>");
                    if (xpNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID") != null)
                        OldGEID = xpNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").Value;
                    if (xpNodeItr.Current.SelectSingleNode("GRAPHICTAGNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("GRAPHICTAGNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("GRAPHICELEMENTNAME") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("GRAPHICELEMENTNAME").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("DESCRIPTION  ") != null)
                        sbInputXML.Append(xpNodeItr.Current.SelectSingleNode("DESCRIPTION").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("BELONGSTOSET  ") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("BELONGSTOSET").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("THUMBNAILACTUALWIDTH") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("THUMBNAILACTUALWIDTH").OuterXml);
                    if (xpNodeItr.Current.SelectSingleNode("THUMBNAILACTUALHEIGHT") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("THUMBNAILACTUALHEIGHT").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ALLOWEXTERNALFILEUPLOAD") != null && xpNodeItr.Current.SelectSingleNode("ALLOWEXTERNALFILEUPLOAD").Value != "")
                    {
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ALLOWEXTERNALFILEUPLOAD").OuterXml);

                    }
                    else
                        sbInputXML.AppendFormat("<ALLOWEXTERNALFILEUPLOAD>0</ALLOWEXTERNALFILEUPLOAD>");

                    if (xpNodeItr.Current.SelectSingleNode("ISHIDDEN") != null && xpNodeItr.Current.SelectSingleNode("ISHIDDEN").Value != "")
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISHIDDEN").OuterXml);
                    else
                        sbInputXML.AppendFormat("<ISHIDDEN>0</ISHIDDEN>");

                    if (xpNodeItr.Current.SelectSingleNode("ELEMENTDISPLAYORDER") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ELEMENTDISPLAYORDER").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ISTAXONOMYSEARCHPRESENT") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ISTAXONOMYSEARCHPRESENT").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ALLOWIMAGEEDITING") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ALLOWIMAGEEDITING").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("TAGCONFIGURATIONXML") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("TAGCONFIGURATIONXML").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("AUTOPOPULATE") != null)
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("AUTOPOPULATE").OuterXml);

                    if (xpNodeItr.Current.SelectSingleNode("ALLOWASSETSELECTION") != null && xpNodeItr.Current.SelectSingleNode("ALLOWASSETSELECTION").Value != "")
                    {
                        sbInputXML.AppendFormat(xpNodeItr.Current.SelectSingleNode("ALLOWASSETSELECTION").OuterXml);
                        //if (xpNodeItr.Current.SelectSingleNode("ALLOWASSETSELECTION").InnerXml=="1")
                        //    DamItemSelection    =   true;

                    }
                    else
                        sbInputXML.AppendFormat("<ALLOWASSETSELECTION>0</ALLOWASSETSELECTION>");

                    if (xpNodeItr.Current.SelectSingleNode("FILESYSTEM") != null)
                    {
                        String FileSystemXML = null;
                        ErrorXML = SaveFiles(xpNodeItr.Current, out FileSystemXML);
                        if (ErrorXML == null && FileSystemXML != null)
                            sbInputXML.Append(FileSystemXML);
                        else if (ErrorXML != null)
                            MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</GRAPHICELEMENT_ERRORLIST>"), ErrorXML);
                        ErrorXML = null;
                    }


                    sbInputXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>"
                                                              , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                    sbInputXML.AppendFormat("<ELEMENTGROUP_GRAPHICELEMENT TYPE='RELATION'>{0}</ELEMENTGROUP_GRAPHICELEMENT>", ElementGroupID);

                    sbInputXML.AppendFormat("<FORMAT_GRAPHICELEMENT TYPE='RELATION'>{0}</FORMAT_GRAPHICELEMENT>", FormatID);

                    sbInputXML.AppendFormat("</GRAPHICELEMENT>");

                    GraphicElementXML = sbInputXML.ToString();
                    //ErrorXML                =   GEEngineObj.CreateGraphicElement(ref GraphicElementXML);
                    ErrorXML = ArtworkProducerWSobj.CreateGraphicElement(ref GraphicElementXML);
                    if (ErrorXML != null)
                        MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</GRAPHICELEMENT_ERRORLIST>"), ErrorXML);
                    else
                    {
                        #region Dam Item Catlog
                        //if (DamItemSelection)
                        {
                            GraphicElemntTagName = ALCommon.GetInnerValue(ref GraphicElementXML, "<GRAPHICTAGNAME>", "</GRAPHICTAGNAME>");
                            GraphicElementID = ALCommon.GetInnerValue(ref GraphicElementXML, "<GRAPHICELEMENTID>", "</GRAPHICELEMENTID>");
                            if (!String.IsNullOrEmpty(GraphicElementID) && !String.IsNullOrEmpty(OldGEID))
                                ImportFormatIDs.Add(OldGEID, GraphicElementID);
                            ErrorXML = CatlogGraphicElementDamItem(GraphicElemntTagName, Path, GraphicElementID);
                            if (ErrorXML != null)
                                MainErrorXL.Insert(MainErrorXL.ToString().IndexOf("</GRAPHICELEMENT_ERRORLIST>"), ErrorXML);
                        }
                        #endregion
                    }

                    #region - Creating relations for the Linked Options
                    if (ErrorXML == null)
                    {
                        if (xpNodeItr.Current.Select("GRAPHICELEMENT_LINKEDOPTIONS/DIGITALASSET/DIGITALASSETID").Count != 0)
                        {
                            xpGELinkedAssets = xpNodeItr.Current.Select("GRAPHICELEMENT_LINKEDOPTIONS/DIGITALASSET/DIGITALASSETID");
                            GraphicElementID = ALCommon.GetInnerValue(ref GraphicElementXML, "<GRAPHICELEMENTID>", "</GRAPHICELEMENTID>");
                            sbGELinkedOptionsAssociationXML = new StringBuilder();
                            sbGELinkedOptionsAssociationXML.AppendFormat("<GRAPHICELEMENT MODIFIED=\"TRUE\"><GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", GraphicElementID);

                            while (xpGELinkedAssets.MoveNext())
                            {
                                LinkedAssetId = Convert.ToUInt64(xpGELinkedAssets.Current.InnerXml.ToString());
                                sbGELinkedOptionsAssociationXML.AppendFormat("<GRAPHICELEMENT_LINKEDOPTIONS TYPE=\"RELATION\">{0}</GRAPHICELEMENT_LINKEDOPTIONS>", LinkedAssetId);
                            }

                            sbGELinkedOptionsAssociationXML.Append("</GRAPHICELEMENT>");
                            GELinkedOptionsAssociationXML = sbGELinkedOptionsAssociationXML.ToString();
                            ErrorXML = ArtworkProducerWSobj.UpdateGraphicElement(ref GELinkedOptionsAssociationXML);
                        }
                    }
                    #endregion
                }
                if (MainErrorXL.ToString().Equals("<GRAPHICELEMENT_ERRORLIST></GRAPHICELEMENT_ERRORLIST>"))
                {
                    SaveStatusFile("Graphic Elements created successfully", "");
                    return null;
                }
                else
                    return MainErrorXL.ToString();

            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);
                return ExcepObj.Message;
            }

        }


        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in Import Format,to Catlog Graphic Element options into DAM.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="GraphicElementTagName">Graphic Element Tag Name</param>
        /// <param name="Path">Format Folder path to get Files.</param>
        ///  <param name="GraphciElementID">GraphciElementID</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML CatlogGraphicElementDamItem(String GraphicElementTagName, String Path, String GraphciElementID)
        {
            #region variable
            StringBuilder sbInputXML = null;
            String SectionID = null;
            String SectionXML = null;
            String GEPath = null;
            String[] GeDamFiles = null;
            String FileName = null;
            String PrevFileName = null;
            StringBuilder DigitalAssetXML = null;
            // DigitalAsset        DigitalAssetObj =   null;
            AssetWidget AssetWidgetObj = null;

            ERRORXML ErrorXML = null;
            StringBuilder AssetRenditionXML = null;
            StringBuilder MainErrorXML = null;
            //OMInterface         OMIObj          =   null;
            SearchConfiguration SearchConfigObj = null;

            #endregion

            try
            {
                #region Get Required Section
                SectionID = null;
                //OMIObj          =   new OMInterface(UserID, Language, TenantName);
                AssetWidgetObj = new AssetWidget(SessionID, UserID, TenantName, CurrentLanguage);
                SearchConfigObj = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                sbInputXML = new StringBuilder();
                MainErrorXML = new StringBuilder();
                MainErrorXML.AppendFormat("<GEDAMITEM_ERRORLIST></GEDAMITEM_ERRORLIST>");
                sbInputXML.Append(@"<INPUTXML>
	                                    <SESSIONINFO SessionID='" + SessionID + "' UserID='" + UserID + "' TenantName='" + TenantName + "' Language='" + CurrentLanguage + @"'/>
	                                    <SEARCHCOLLECTION>
                                        <SEARCH>
			                                    <TARGETOBJECT>SECTION</TARGETOBJECT>
			                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                <FILTERLIST>
				                                        <FILTER>
					                                        <FIELDNAME>NAME</FIELDNAME>
					                                        <OPERATOR>EQUALTO</OPERATOR>
					                                        <FIELDVALUE>" + GraphicElementTagName.Replace('\\', '_') + "_" + GraphciElementID + @"</FIELDVALUE>
					                                        <NAVIGATIONPATH>SECTION</NAVIGATIONPATH>
					                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                        </FILTER>
			                                     </FILTERLIST>
                                              </SEARCH></SEARCHCOLLECTION></INPUTXML>");
                //ErrorXML = OMIObj.GetData(sbInputXML.ToString(), null, ref SectionXML);
                ErrorXML = SearchConfigObj.DoSearch(sbInputXML.ToString(), ref SectionXML);
                sbInputXML.Remove(0, sbInputXML.Length);

                if (ErrorXML != null)
                {
                    // return ErrorXML;
                    MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</GEDAMITEM_ERRORLIST>"), ErrorXML);
                }
                SectionID = ALCommon.GetInnerValue(ref SectionXML, "<SECTIONID>", "</SECTIONID>");

                #endregion
                if (!String.IsNullOrEmpty(SectionID))
                {
                    GraphicElementTagName = ALSpecialCharHandling.DecodeSpecialCharacters(GraphicElementTagName);
                    GraphicElementTagName = ReplaceSpecialChars_InvalidOnFileSystem(GraphicElementTagName);
                    GEPath = Path + "\\" + GraphicElementTagName;
                    AssetRenditionXML = new StringBuilder();
                    if (Directory.Exists(GEPath))
                    {
                        XPathDocument xpDoc_AssetDetails = new XPathDocument(GEPath + "\\AssetDetails.xml");
                        GeDamFiles = Directory.GetFiles(GEPath);

                        Array.Sort(GeDamFiles);
                        // foreach (String DamFile in GeDamFiles)
                        int Cnt = 0;
                        SaveStatusFile("Catloging Graphic Element options into Dam", "");
                        bool bCreatePending = false;
                        String PrevDigitalAssetID = null;
                        while (Cnt < GeDamFiles.Length)
                        {
                            String DamFile = GeDamFiles[Cnt].ToString().Substring(GeDamFiles[Cnt].ToString().LastIndexOf("\\"));

                            DamFile = DamFile.Replace("\\", "");
                            if (DamFile == "AssetDetails.xml")
                            {
                                Cnt++;
                                if (Cnt < GeDamFiles.Length || !bCreatePending)
                                    continue;
                            }
                            else
                            {
                                bCreatePending = true;
                                FileName = null;
                                FileName = DamFile.Substring(0, DamFile.IndexOf("."));
                                // DamFile             =   GeDamFiles

                                if (PrevFileName == null)
                                    PrevFileName = FileName;

                                if (FileName.Equals(PrevFileName))
                                {
                                    XPathNavigator nav_OldAssetDetails = xpDoc_AssetDetails.CreateNavigator().SelectSingleNode("/SECTIONCOLLECTION/SECTION/SECTION_DIGITALASSET/DIGITALASSET/DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION [FILENAME/text()='" + DamFile + "']");
                                    if (nav_OldAssetDetails == null)
                                    {
                                        nav_OldAssetDetails = xpDoc_AssetDetails.CreateNavigator().SelectSingleNode("/SECTIONCOLLECTION/SECTION/SECTION_DIGITALASSET/DIGITALASSET/DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION [NAME/text()='" + DamFile + "']");
                                        PrevDigitalAssetID = xpDoc_AssetDetails.CreateNavigator().SelectSingleNode("/SECTIONCOLLECTION/SECTION/SECTION_DIGITALASSET/DIGITALASSET [DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION/NAME/text()='" + DamFile + "']/DIGITALASSETID").Value;
                                    }
                                    else
                                    {
                                        PrevDigitalAssetID = xpDoc_AssetDetails.CreateNavigator().SelectSingleNode("/SECTIONCOLLECTION/SECTION/SECTION_DIGITALASSET/DIGITALASSET [DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION/FILENAME/text()='" + DamFile + "']/DIGITALASSETID").Value;
                                    }

                                    AssetRenditionXML.Append(GetAssetRenditionXML(DamFile, GEPath, nav_OldAssetDetails));
                                    Cnt++;

                                    if (Cnt < GeDamFiles.Length)
                                        continue;
                                }
                            }
                            //else 
                            {
                                bCreatePending = false;
                                FileName = PrevFileName;
                                PrevFileName = null;
                                ErrorXML = null;

                                ALSpecialCharHandling.EncodeSpecialCharacters(ref FileName);

                                DigitalAssetXML = new StringBuilder();
                                DigitalAssetXML.Append("<DIGITALASSET>");
                                DigitalAssetXML.AppendFormat("<NAME>{0}</NAME>", FileName);
                                DigitalAssetXML.AppendFormat("<DESCRIPTION>Asset Created For {0}</DESCRIPTION>", FileName);
                                DigitalAssetXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>",
                                                                DateTime.Now.Day.ToString(), DateTime.Now.Month.ToString(), DateTime.Now.Year.ToString(), DateTime.Now.Hour.ToString(), DateTime.Now.Minute.ToString(), DateTime.Now.Second.ToString());
                                DigitalAssetXML.Append("<MARKEDASDELETED>0</MARKEDASDELETED>");
                                DigitalAssetXML.Append("<DIGITALASSET_CONTENTREVISION  TYPE='RELATION'>");
                                DigitalAssetXML.Append("<CONTENTREVISION>");
                                DigitalAssetXML.Append("<VERSIONNUMBER>1</VERSIONNUMBER>");
                                DigitalAssetXML.Append("<CONTENTREVISION_RENDITION TYPE='RELATION'>");
                                DigitalAssetXML.Append(AssetRenditionXML);
                                DigitalAssetXML.Append("</CONTENTREVISION_RENDITION>");

                                DigitalAssetXML.Append("</CONTENTREVISION>");
                                DigitalAssetXML.Append("</DIGITALASSET_CONTENTREVISION>");
                                DigitalAssetXML.AppendFormat("<SECTION_DIGITALASSET TYPE='RELATION' PARENTID='TRUE'>{0}</SECTION_DIGITALASSET>", SectionID);
                                DigitalAssetXML.Append("</DIGITALASSET>");

                                XML DigitalAssetInputXML = DigitalAssetXML.ToString();

                                //ErrorXML = DigitalAssetObj.CreateDigitalAsset(false, ref DigitalAssetInputXML);
                                ErrorXML = AssetWidgetObj.CreateDigitalAsset(false, ref DigitalAssetInputXML);
                                AssetRenditionXML.Remove(0, AssetRenditionXML.Length);


                                if (ErrorXML == null)
                                {
                                    String DigitalAssetID = ALCommon.GetInnerValue(ref  DigitalAssetInputXML, "<DIGITALASSETID>", "</DIGITALASSETID>");
                                    if (!String.IsNullOrEmpty(DigitalAssetID) && !String.IsNullOrEmpty(PrevDigitalAssetID))
                                        ImportFormatIDs.Add(PrevDigitalAssetID, DigitalAssetID);

                                }
                                else
                                    MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</GEDAMITEM_ERRORLIST>"), ErrorXML);
                                PrevDigitalAssetID = null;

                                DigitalAssetInputXML = null;
                                FileName = null;

                            }



                        }
                    }
                    //else
                    //    MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</GEDAMITEM_ERRORLIST>"), ALSolutionCommon.PrepareErrorXML("Directory does not exist :" + GEPath));
                }
                //else
                //        MainErrorXML.Insert(MainErrorXML.ToString().IndexOf("</GEDAMITEM_ERRORLIST>"), ALSolutionCommon.PrepareErrorXML("Section does not exist for given Graphic Element :"+GraphicElementTagName));

                //  return null;
                if (MainErrorXML.ToString().Equals("<GEDAMITEM_ERRORLIST></GEDAMITEM_ERRORLIST>"))
                {
                    SaveStatusFile("Graphic Element Options are catloged into Dam Successfully ", "");
                    return null;
                }
                else
                    return MainErrorXML.ToString();

            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);
                return ExcepObj.Message;
            }

        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This function is used in CatlogGraphicElementDamItem,to get Asset RenditionXML.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="GeDamFileName">File name to be catalog</param>
        /// <param name="FilePath">path of file.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private XML GetAssetRenditionXML(String GeDamFileName, String FilePath, XPathNavigator xpNav_OldAssetDetails)
        {
            #region variable
            StringBuilder sbInputXML = null;
            String FileFormat = null;
            StringBuilder FileSystem = null;
            FileTransferInfrastructure FTObj = null;
            String FileListXML = null;

            #endregion

            FTObj = new FileTransferInfrastructure(SessionID, UserID, TenantName, CurrentLanguage);
            FileFormat = GeDamFileName.Substring(GeDamFileName.LastIndexOf(".") + 1);
            sbInputXML = new StringBuilder();

            ALSpecialCharHandling.EncodeSpecialCharacters(ref GeDamFileName);

            // sbInputXML.Append("<CONTENTREVISION_RENDITION TYPE='RELATION'>");
            sbInputXML.Append("<ASSETRENDITION>");
            sbInputXML.AppendFormat("<NAME>{0}</NAME>", GeDamFileName);

            if (xpNav_OldAssetDetails != null)
            {
                sbInputXML.AppendFormat("<HEIGHT>{0}</HEIGHT>", xpNav_OldAssetDetails.SelectSingleNode("HEIGHT").Value); ;
                sbInputXML.AppendFormat("<WIDTH>{0}</WIDTH>", xpNav_OldAssetDetails.SelectSingleNode("WIDTH").Value); ;
                sbInputXML.AppendFormat("<RENDITIONTYPE>{0}</RENDITIONTYPE>", xpNav_OldAssetDetails.SelectSingleNode("RENDITIONTYPE").Value);
            }
            else
            {
                if (GeDamFileName.ToUpper().Contains(".ORIGINAL."))
                    sbInputXML.Append("<RENDITIONTYPE>Original</RENDITIONTYPE>");
                else if (GeDamFileName.ToUpper().Contains(".PREVIEW."))
                    sbInputXML.AppendFormat("<RENDITIONTYPE>Preview</RENDITIONTYPE>");
                else if (GeDamFileName.ToUpper().Contains(".THUMBNAIL."))
                    sbInputXML.AppendFormat("<RENDITIONTYPE>Thumbnail</RENDITIONTYPE>");
            }


            sbInputXML.AppendFormat("<FILENAME>{0}</FILENAME>", GeDamFileName);
            if (FileFormat != null)
                sbInputXML.AppendFormat("<FILEFORMAT>{0}</FILEFORMAT>", FileFormat);
            sbInputXML.Append("<MEDIACREATEDIN>Printer</MEDIACREATEDIN>");

            //Filesystem
            FileSystem = new StringBuilder();
            FileSystem.Append("<FILESYSTEM><FILE>");
            FileSystem.Append("<FIELDNAME>ACTUALFILE</FIELDNAME>");
            FileSystem.AppendFormat("<DESCRIPTION>File For {0}</DESCRIPTION>", GeDamFileName);
            FileSystem.AppendFormat("<NAME>{0}</NAME>", GeDamFileName);
            FileSystem.AppendFormat("<DISPLAYNAME>{0}</DISPLAYNAME>", GeDamFileName);
            ALSpecialCharHandling.EncodeSpecialCharacters(ref FilePath);
            FileSystem.AppendFormat("<PATH>{0}</PATH>", FilePath + "\\" + GeDamFileName);
            FileSystem.Append("</FILE></FILESYSTEM>");
            FileListXML = FileSystem.ToString();
            FileListXML = FileListXML.Replace("<FILESYSTEM>", "<FILECOLLECTION>");
            FileListXML = FileListXML.Replace("</FILESYSTEM>", "</FILECOLLECTION>");

            ErrorXML = FTObj.SaveFile(ref FileListXML);
            if (ErrorXML == null)
            {
                FileListXML = FileListXML.Replace("<FILECOLLECTION>", "<FILESYSTEM>");
                FileListXML = FileListXML.Replace("</FILECOLLECTION>", "</FILESYSTEM>");
                sbInputXML.Append(FileListXML);
            }
            else
                ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
            sbInputXML.Append("</ASSETRENDITION>");
            //  sbInputXML.Append("</CONTENTREVISION_RENDITION>");
            return sbInputXML.ToString();


        }

        /// <alsummary>
        /// <createddate>10th March 2007</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>
        /// This helper function is used to save files on om machine.
        /// </purpose>
        ///  <xmlstructure>
        /// ]]></xmlstructure>
        /// <param name="xpNavFileSystem">File System Node</param>
        /// <param name="FileSystemXML">returns updated File System xml.</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        private ERRORXML SaveFiles(XPathNavigator xpNavFileSystem, out String FileSystemXML)
        {
            #region variable
            String BinderXML = null;
            String FileCollectionXML = null;
            FileTransferInfrastructure FTIObj = null;
            #endregion

            try
            {

                //Save Binder Files
                XPathNodeIterator BinderNodeIter = xpNavFileSystem.SelectSingleNode("FILESYSTEM").Select("BINDER");
                FTIObj = new FileTransferInfrastructure(SessionID, UserID, TenantName, CurrentLanguage);
                FileSystemXML = null;
                StringBuilder sbFileSystemXML = new StringBuilder(20);
                sbFileSystemXML.Append("<FILESYSTEM>");

                while (BinderNodeIter.MoveNext())
                {
                    BinderXML = BinderNodeIter.Current.OuterXml;
                    BinderXML = BinderXML.Replace("<BINDER>", "<FILECOLLECTION>");
                    BinderXML = BinderXML.Replace("</BINDER>", "</FILECOLLECTION>");
                    ErrorXML = FTIObj.SaveFile(ref BinderXML);
                    if (ErrorXML != null)
                        return ErrorXML;
                    else
                    {
                        BinderXML = BinderXML.Replace("FILECOLLECTION", "BINDER");
                        sbFileSystemXML.Append(BinderXML);
                    }
                }

                //To Save Normal Files
                XPathNodeIterator FileNodeIter = xpNavFileSystem.SelectSingleNode("FILESYSTEM").Select("FILE");
                StringBuilder sbFileCollectionXML = new StringBuilder(20);
                sbFileCollectionXML.Append("<FILECOLLECTION>");

                while (FileNodeIter.MoveNext())
                    sbFileCollectionXML.Append(FileNodeIter.Current.OuterXml);

                sbFileCollectionXML.Append("</FILECOLLECTION>");
                FileCollectionXML = sbFileCollectionXML.ToString();

                ErrorXML = FTIObj.SaveFile(ref FileCollectionXML);
                if (ErrorXML != null)
                    return ErrorXML;
                else
                {
                    FileCollectionXML = FileCollectionXML.Replace("<FILECOLLECTION>", "");
                    FileCollectionXML = FileCollectionXML.Replace("</FILECOLLECTION>", "");
                    sbFileSystemXML.Append(FileCollectionXML);
                }
                sbFileSystemXML.Append("</FILESYSTEM>");

                FileSystemXML = sbFileSystemXML.ToString();
                return null;
            }
            catch (Exception ExcepObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Exception Message :" + ExcepObj.Message + " Exception  Stack  Trace :" + ExcepObj.StackTrace);
                ArtworkProducerWSobj.LogError("FormatFacade:ImportFormat", ErrorXML, ErrorType.EXCEPTION, null, null);
                FileSystemXML = null;
                throw ExcepObj;
            }
        }

        private void SaveStatusFile(String Status, String Error)
        {
            #region variable
            StringBuilder sbInputXML = null;
            #endregion

            try
            {
                sbInputXML = new StringBuilder();
                sbInputXML.AppendFormat(@"<FORMATIMPORTSTATUS>
                                                <IMPORTSTATUS>{0}</IMPORTSTATUS>
                                                <IMPORTERROR>{1}</IMPORTERROR>
                                        </FORMATIMPORTSTATUS>", Status, Error);



                File.WriteAllText(ALSolutionCommon.GetTempPath(SessionID) + FormatToImport + "_Status.xml", sbInputXML.ToString());

            }
            catch (Exception ExcepObj)
            {
                ArtworkProducerWSobj.LogError("FormatFacade", ExcepObj.Message, ErrorType.EXCEPTION, null, null);
            }



        }

        #endregion



        # region Varuns_Changes
        ///-------------------------------------------------------------------------------------------------------///
        //********************************************************************************************************//
        /// CHANGES MADE BY VARUN
        /// <summary>
        /// <changeson> 7 March 2007</changeson>
        /// <createdon> 2 March 2007</createdon>
        /// <createdby> Varun Dighe</createdby>
        /// <purpose>   Exporting Formats </purpose> 
        /// <USAGE>The methods for exporting formats are called by a thread in FORMAT_DESIGN_DETAILS.aspx, in this file a call 
        /// is given to ASPX page(Export_Design.aspx) which reads the status XML written by the methods below which depict EXPORT STATUS</USAGE>
        /// </summary>




        /// <summary>
        /// <purpose>This method Retrives information for Format using GetHierarchialData()</purpose>
        /// </summary>
        // public void GetAllFormatInfo(String FID)
        private void ProcessExportFormatRequest(object Params)
        {
            ERRORXML Err_XML = null;
            System.Security.Principal.WindowsImpersonationContext NewContext = null;
            String RequestID = null;
            String FormatID = null;
            String str_ExportDirPath = null;
            bool ZipRequired = true;
            StringBuilder SB_FormatInfo = null;
            StringBuilder SB_SetCollection = null;
            StringBuilder SB_SetCollectionIDs = null;
            XmlDocument SelCollectionDoc = null;
            XmlNodeList SetCollectionIDNodes = null;
            string seperator = "";
            String Outputsearchresult_SetCollection = null;

            ZipRequired = (bool)((ArrayList)Params)[3];
            try
            {
                if (ZipRequired == true)
                {
                    System.Threading.Thread.CurrentPrincipal = new System.Security.Principal.WindowsPrincipal(OrgIdentity);
                    NewContext = OrgIdentity.Impersonate();
                }

                RequestID = (String)((ArrayList)Params)[0];
                FormatID = (String)((ArrayList)Params)[1];
                str_ExportDirPath = (String)((ArrayList)Params)[2];

                SB_FormatInfo = new StringBuilder();
                UpdateExportStatusXML("Processing....");



                if (File.Exists(ALSolutionCommon.GetTempPath(SessionID) + "ExportStatus.xml"))
                    File.Delete(ALSolutionCommon.GetTempPath(SessionID) + "ExportStatus.xml");

                SB_FormatInfo.AppendFormat(@"
                                                <SEARCH>
                                                    <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{4}'></SESSIONINFO>
                                                    <TARGETOBJECT>FORMAT</TARGETOBJECT>
	                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                    <SEARCHALLFACETS>TRUE</SEARCHALLFACETS>
	                                                <TARGETENTITY>
		                                                <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                        <FILTERLIST>
                                                            <FILTER>
	                                                            <EXPRESSION>FORMATID</EXPRESSION>
	                                                            <OPERATOR>EQUALTO</OPERATOR>
	                                                            <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
	                                                            <FILTERID>1</FILTERID>
	                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
                                                            </FILTER>
                                                        </FILTERLIST>                                                    
                                                        <RELATIONSHIP NAME='FORMAT_ARTWORKPAGE'>
                                                            <RELATIONSHIP NAME='ARTWORKPAGE_ELEMENTGROUP'>
                                                                <RELATIONSHIP NAME='ELEMENTGROUP_TEXTELEMENT'>
                                                                    <RELATIONSHIP NAME='TEXTELEMENT_OPTION' />
                                                                </RELATIONSHIP> 
                                                                <RELATIONSHIP NAME='ELEMENTGROUP_GRAPHICELEMENT'>
                                                                   <RELATIONSHIP NAME='GRAPHICELEMENT_LINKEDOPTIONS'>
                                                                    <SELECTLIST>
									                                    <EXPRESSION>
										                                    <FIELDNAME>DIGITALASSETID</FIELDNAME>
										                                    <ALIAS>DIGITALASSETID</ALIAS>
									                                    </EXPRESSION>
								                                    </SELECTLIST>
                                                                   </RELATIONSHIP>
                                                                </RELATIONSHIP>
                                                                <RELATIONSHIP NAME='ELEMENTGROUP_SETCOLLECTION' >
                                                                    <SELECTLIST>
									                                    <EXPRESSION>
										                                    <FIELDNAME>SETCOLLECTIONID</FIELDNAME>
										                                    <ALIAS>SETCOLLECTIONID</ALIAS>
									                                    </EXPRESSION>
								                                    </SELECTLIST>
                                                                </RELATIONSHIP>
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                    </TARGETENTITY>
                                                </SEARCH>
                                       ", CurrentLanguage, TenantName, UserID, FormatID, SessionID);


                String Outputsearchresult = null;
                SearchConfiguration ObjSearchConfiguration = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);

                Err_XML = ObjSearchConfiguration.GetHierarchialData(SB_FormatInfo.ToString(), ref Outputsearchresult);

                if (string.IsNullOrEmpty(Err_XML))
                {
                    SelCollectionDoc = new XmlDocument();
                    SB_SetCollectionIDs = new StringBuilder();

                    SelCollectionDoc.LoadXml(Outputsearchresult);
                    SetCollectionIDNodes = SelCollectionDoc.SelectNodes("FORMATCOLLECTION/FORMAT/FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_SETCOLLECTION/SETCOLLECTION/SETCOLLECTIONID");

                    if (SetCollectionIDNodes.Count != 0)
                    {
                        SB_SetCollection = new StringBuilder();
                        SB_SetCollectionIDs.Append("(");
                        //Get all SETCOLLECTIONID's for next search
                        foreach (XmlNode SetCollectionIDNode in SetCollectionIDNodes)
                        {
                            SB_SetCollectionIDs.AppendFormat("{0}{1}", seperator, SetCollectionIDNode.InnerText);
                            seperator = ",";
                        }
                        SB_SetCollectionIDs.Append(")");

                        SB_SetCollection.AppendFormat(@"
                                      <SEARCH>
                                            <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{4}'></SESSIONINFO>
                                            <TARGETOBJECT>SETCOLLECTION</TARGETOBJECT>
	                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <SEARCHALLFACETS>TRUE</SEARCHALLFACETS>
	                                        <TARGETENTITY>
		                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                <FILTERLIST>
                                                    <FILTER>
	                                                    <EXPRESSION>SETCOLLECTIONID</EXPRESSION>
	                                                    <OPERATOR>IN</OPERATOR>
	                                                    <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
	                                                    <FILTERID>1</FILTERID>
	                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>                                                    
                                                <RELATIONSHIP NAME='SETCOLLECTION_SET' >
                                                    <RELATIONSHIP NAME='SET_SETELEMENT' >
                                                        <RELATIONSHIP NAME='SETELEMENT_DIGITALASSET' >
                                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTREVISION' >
                                                                    <RELATIONSHIP NAME='CONTENTREVISION_RENDITION' >
                                                                        <SELECTLIST>
                                                                            <EXPRESSION>
                                                                                <FIELDNAME>NAME</FIELDNAME>
                                                                                <ALIAS>NAME</ALIAS>
                                                                            </EXPRESSION>
                                                                            <EXPRESSION>
                                                                                <FIELDNAME>FILENAME</FIELDNAME>
                                                                                <ALIAS>FILENAME</ALIAS>
                                                                            </EXPRESSION>
                                                                            <EXPRESSION>
                                                                                <FIELDNAME>RENDITIONTYPE</FIELDNAME>
                                                                                <ALIAS>RENDITIONTYPE</ALIAS>
                                                                            </EXPRESSION>
                                                                            <EXPRESSION>
                                                                                <FIELDNAME>WIDTH</FIELDNAME>
                                                                                <ALIAS>WIDTH</ALIAS>
                                                                            </EXPRESSION>
                                                                            <EXPRESSION>
                                                                                <FIELDNAME>HEIGHT</FIELDNAME>
                                                                                <ALIAS>HEIGHT</ALIAS>
                                                                            </EXPRESSION>
                                                                        </SELECTLIST>
                                                                    </RELATIONSHIP>
                                                                </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='SETELEMENT_GRAPHICELEMENT' >
                                                            <SELECTLIST>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                                </EXPRESSION>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                                    <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                                </EXPRESSION>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
                                                                    <ALIAS>GRAPHICELEMENTNAME</ALIAS>
                                                                </EXPRESSION>
                                                            </SELECTLIST>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='SETELEMENT_TEXTELEMENT' />
                                                        <RELATIONSHIP NAME='SETELEMENT_TEXTELEMENTOPTION' />
                                                    </RELATIONSHIP>
                                                </RELATIONSHIP>
                                            </TARGETENTITY>
                                        </SEARCH>
                                       ", CurrentLanguage, TenantName, UserID, SB_SetCollectionIDs.ToString(), SessionID);

                        Err_XML = ObjSearchConfiguration.GetHierarchialData(SB_SetCollection.ToString(), ref Outputsearchresult_SetCollection);

                        if (string.IsNullOrEmpty(Err_XML))
                        {
                            //Consolidate two search result.
                            Err_XML = ConsolidateSearchResults(ref Outputsearchresult, Outputsearchresult_SetCollection);
                        }
                    }
                }

                if (Err_XML != null)
                {
                    UpdateExportStatusXML(Err_XML);
                    UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                    ErrorXML = ALSolutionCommon.PrepareErrorXML("Get HierarchialData returned Error");
                    ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                    return;
                }
                else
                {
                    String SearchComplete = null;
                    SearchComplete = "Searching is Complete";
                    UpdateExportStatusXML(SearchComplete);
                    UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.RetrivingFormatFiles);
                    //GetFormatDetailsAndFiles(RequestID, FormatID, Outputsearchresult);
                    GetAllFormatRelatedFiles(RequestID, FormatID, Outputsearchresult, str_ExportDirPath, ZipRequired);

                }
                ObjSearchConfiguration = null;




            }
            catch (Exception e)
            {
                String ExceptionMSG = e.Message;
                ArtworkProducerWSobj.LogError("FormatFacade", e.Message, ErrorType.EXCEPTION, null, null);
                UpdateExportStatusXML(ExceptionMSG);
                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
            }
            finally
            {
                if (ZipRequired == true)
                    NewContext.Undo();
            }

        }

        /// <summary>
        /// <createddate>27 November 2006</createddate>
        /// <author>Mr.Satish S. Gaikwad</author>
        /// <purpose> This method is used to consolidate two search results from GetHierarchicalData
        /// </summary>
        /// <param name="Outputsearchresult">1st SearchOutPutXML from GetHierarchicalData. This is 'ref' parameter and stores final consolidated result</param>
        /// <param name="Outputsearchresult_SetCollection">2nd SearchOutPutXML from GetHierarchicalData</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        public String ConsolidateSearchResults(ref String Outputsearchresult, String Outputsearchresult_SetCollection)
        {
            XmlDocument FinalConsolidationResultDoc = null;
            XPathDocument xpDoc_SetCollection = null;
            XPathNavigator xpNav_SetCollection = null;
            XmlNodeList setCollectionNodes = null;
            XmlNode setCollectionNode_FinalConsolidationResult = null;
            XmlNode setCollectionNode_SetCollectionDoc = null;

            try
            {
                FinalConsolidationResultDoc = new XmlDocument();
                //SetCollectionDoc = new XmlDocument();
                xpDoc_SetCollection = new XPathDocument(new StringReader(Outputsearchresult_SetCollection));
                xpNav_SetCollection = xpDoc_SetCollection.CreateNavigator();
                FinalConsolidationResultDoc.LoadXml(Outputsearchresult);

                //Select all SETCOLLECTION nodes
                setCollectionNodes = FinalConsolidationResultDoc.SelectNodes("FORMATCOLLECTION/FORMAT/FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_SETCOLLECTION/SETCOLLECTION");

                //Replaces related SETCOLLECTIONID node with SETCOLLECTION DATA
                foreach (XmlNode setCollectionNode in setCollectionNodes)
                {
                    setCollectionNode_FinalConsolidationResult = setCollectionNode.SelectSingleNode("SETCOLLECTIONID");
                    setCollectionNode_SetCollectionDoc = FinalConsolidationResultDoc.CreateElement("SETCOLLECTION");
                    setCollectionNode_SetCollectionDoc.InnerXml = xpNav_SetCollection.SelectSingleNode("SETCOLLECTIONCOLLECTION/SETCOLLECTION [SETCOLLECTIONID=" + setCollectionNode_FinalConsolidationResult.InnerText + "]").InnerXml;
                    setCollectionNode.ParentNode.ReplaceChild(setCollectionNode_SetCollectionDoc, setCollectionNode);
                }
                //Consolidated search result.
                Outputsearchresult = FinalConsolidationResultDoc.OuterXml;
                return null;
            }
            catch (Exception exObj)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML("Error in ConsolidateSearchResults of GetHierarchicalData ", exObj.Message);
                return ErrorXML;
            }
        }

        #region For Creating Folder

        /// <summary>
        /// <purpose>This Method Creates a Folder for a given Format on the Filesystem</purpose>
        /// <modifiedTo>This method will now read PATH from ConfigurationXML and create folder at the specified path
        /// </summary>
        /// <param name="FormatInputXML"></param>
        public void GetFormatDetailsAndFiles(String RequestID, String FormatID, String FormatInputXML)
        {
            #region Variables

            //XPathDocument       XPathFDoc       = null;
            //XPathNavigator xNav = null;
            //XPathNodeIterator Iterator = null;
            //XmlDocument FormatXml = null;
            String FSName = null;

            #endregion

            try
            {

                FSName = ALCommon.GetInnerValue(ref FormatInputXML, "<FORMATNAME>", "</FORMATNAME>");

                StringBuilder FolderPath = new StringBuilder();
                String FPath = null;

                StringBuilder sbTenantXMLpath = new StringBuilder(60);
                sbTenantXMLpath.Append(ALCommon.GetTenantXMLPath(TenantName));
                XPathDocument xpDoc_TenantXML = new XPathDocument(sbTenantXMLpath.ToString());
                String str_ExportPath = xpDoc_TenantXML.CreateNavigator().SelectSingleNode("/CONFIGURATION/ARTWORKPRODUCER/IMPORTEXPORT/EXPORTPATH").Value;

                FPath = str_ExportPath;

                CreateDirectory(new System.IO.DirectoryInfo(FPath));
                FormatPathForZip = FPath;
                FormatNameForZip = FPath;
            }
            catch (Exception e)
            {
                ArtworkProducerWSobj.LogError("FormatFacade", e.Message, ErrorType.EXCEPTION, null, null);
            }


        }



        #endregion

        #region  To get FORMAT FILES

        /// <summary>
        /// <purpose>This Method uses FileFacde , FileTransfer Object to get the specified file from 
        /// Engine to WebService & WebService to LocalMachine respectively</purpose>
        /// </summary>
        /// <param name="FileXML"></param>
        /// <param name="DownloadPath"></param>

        public void GetAllFormatRelatedFiles(String RequestID, String FormatID, String FileXML, String DownloadPath, bool ZipRequired)
        {
            #region variable

            XPathDocument XPathDoc = null;
            XmlDocument FormatFile = null;
            XmlDocument Copy_FormatFile = null;
            XmlDocument DamFormatFile = null;
            StringReader XPathDocXML = null;
            XPathNavigator XPathNav = null;
            XmlNodeList XMLFileNodeList = null;




            FileFacade FileFacadeObj = null;
            FileTransfer FileTransferObj = null;
            String FileSystemXML = null;
            StringBuilder FileFacadeXML = null;
            String ErrorXML = null;
            String ObjectID = null;
            String FileName = null;
            String Spath = null;
            XmlNode FileSystemNode = null;
            String FormatName = null;





            #endregion

            #region Get File

            //Get Field Node
            #region Get Format and Graphic Element Files
            try
            {
                FileFacadeObj = new FileFacade(SessionID, UserID, TenantName, CurrentLanguage);
                FileTransferObj = new FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);
                XPathDocXML = new StringReader(FileXML);
                XPathDoc = new XPathDocument(XPathDocXML);
                FormatFile = new XmlDocument();
                Copy_FormatFile = new XmlDocument();
                FileFacadeXML = new StringBuilder();
                DamFormatFile = new XmlDocument();


                FormatFile.LoadXml(FileXML);

                XPathNav = XPathDoc.CreateNavigator();


                FileSystemNode = FormatFile.SelectSingleNode("FORMATCOLLECTION/FORMAT/FILESYSTEM");
                FormatName = FormatFile.SelectSingleNode("FORMATCOLLECTION/FORMAT/FORMATNAME").InnerText;


                XmlNode FormatCollectionNode = FormatFile.SelectSingleNode("FORMATCOLLECTION");
                if (FormatCollectionNode != null)
                {

                    XmlNode ExportPathNode = FormatFile.CreateElement("EXPORTPATH");
                    ExportPathNode.InnerText = DownloadPath + "\\";
                    FormatCollectionNode.AppendChild(ExportPathNode);
                }



                XMLFileNodeList = FormatFile.SelectNodes("//FILE");

                int cntr = 0;
                foreach (XmlNode SelectedNode in XMLFileNodeList)
                {

                    FileSystemXML = SelectedNode.OuterXml;
                    if (FileSystemXML.StartsWith("<FILE>"))
                    {
                        //Get File Name
                        if (SelectedNode.SelectSingleNode("DISPLAYNAME") != null)
                            FileName = SelectedNode.SelectSingleNode("DISPLAYNAME").InnerXml;

                        if (String.IsNullOrEmpty(FileName))
                        {
                            if (SelectedNode.SelectSingleNode("NAME") != null)
                                FileName = SelectedNode.SelectSingleNode("NAME").InnerXml;
                        }

                        #region Get File From Engine to WebService
                        ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                        ObjectID = ObjectID.Replace("<OBJECTID>", "");
                        FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                        FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                        FileFacadeXML.Append(FileSystemXML);
                        #endregion
                        ErrorXML = FileFacadeObj.GetFile(ref FileSystemXML);
                        if (ErrorXML != null)
                        {
                            UpdateExportStatusXML(ErrorXML);
                            UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                            ErrorXML = ALSolutionCommon.PrepareErrorXML("Error while Format Files From OM to WebService");
                            ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                            return;
                        }
                        else
                        {
                            String GetFileStatus = null;
                            GetFileStatus = "Successful In Getting Files From OM to WebService";
                            UpdateExportStatusXML(GetFileStatus);
                        }
                        #region Get File from WebService to local Machine
                        if (ErrorXML == null)
                        {
                            UpdateExportStatusXML("Getting Files From WebService");
                            FileSystemXML = FileSystemXML.Replace("<PATH>", "<SOURCEPATH>");
                            FileSystemXML = FileSystemXML.Replace("</PATH>", "</SOURCEPATH>");

                            if (SelectedNode.ParentNode.Name.ToUpper() == "BINDER")
                            {
                                if (SelectedNode.SelectSingleNode("FIELDNAME") != null)
                                {
                                    if (String.IsNullOrEmpty(SelectedNode.SelectSingleNode("FIELDNAME").InnerText) == false)
                                        FileTransferObj.GetFilePath = DownloadPath + "\\" + SelectedNode.SelectSingleNode("FIELDNAME").InnerText;
                                    else
                                        FileTransferObj.GetFilePath = DownloadPath + "\\ADDITIONALFILES";
                                }
                                else
                                    FileTransferObj.GetFilePath = DownloadPath + "\\ADDITIONALFILES";
                            }
                            else
                                FileTransferObj.GetFilePath = DownloadPath;

                            ErrorXML = FileTransferObj.GetFile(ref FileSystemXML);
                            if (ErrorXML != null)
                            {
                                UpdateExportStatusXML(ErrorXML);
                                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                                ErrorXML = ALSolutionCommon.PrepareErrorXML("Error while Format Files From WebService to LocalMachine");
                                ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                                return;
                            }
                            else
                            {
                                String GetFileStatus_FT = null;
                                GetFileStatus_FT = "Successful In Getting Files From WebService to Local Machine";
                                UpdateExportStatusXML(GetFileStatus_FT);

                            }


                            Spath = ALCommon.GetInnerValue(ref FileSystemXML, "<DESTINATIONPATH>", "</DESTINATIONPATH>");

                            if (SelectedNode.SelectSingleNode("PATH") != null)
                                SelectedNode.SelectSingleNode("PATH").InnerText = Spath + "\\" + FileName;
                        }
                        #endregion


                    }
                    cntr++;

                    if (cntr == 3)
                        UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.RetrivingLinkedFiles);

                }//end of for
            }
            catch (Exception e)
            {
                String ExMsg = null;
                ExMsg = e.Message;
                UpdateExportStatusXML(ExMsg);
                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                ArtworkProducerWSobj.LogError("FormatFacade", e.Message, ErrorType.EXCEPTION, null, null);
                return;
            }
            #endregion

            UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.RetrivingDAMItems);

            #region Get Dam Items
            try
            {

                XMLFileNodeList = FormatFile.SelectNodes("FORMATCOLLECTION/FORMAT/FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_GRAPHICELEMENT//GRAPHICELEMENT");
                UpdateExportStatusXML("Searching For DAM ITEMS ...");
                foreach (XmlNode SelectedNode in XMLFileNodeList)
                {


                    # region Variables For DAM ITEM SEARCH
                    String GEName_encoded = null;
                    String GEName = null;
                    String GEID = null;
                    StringBuilder Dam_Search = null;
                    String GE = null;
                    ERRORXML Er_XML = null;
                    # endregion
                    Dam_Search = new StringBuilder();


                    //if ((SelectedNode.SelectSingleNode("ALLOWASSETSELECTION") != null) && (SelectedNode.SelectSingleNode("ALLOWASSETSELECTION").InnerText == "1"))
                    {
                        if ((SelectedNode.SelectSingleNode("GRAPHICTAGNAME") != null))
                        {
                            GEName_encoded = SelectedNode.SelectSingleNode("GRAPHICTAGNAME").InnerXml;
                            GEName = SelectedNode.SelectSingleNode("GRAPHICTAGNAME").InnerText;
                        }

                        if ((SelectedNode.SelectSingleNode("GRAPHICELEMENTID") != null))
                        {
                            GEID = SelectedNode.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                            GE = GEName_encoded + "_" + GEID;
                            GE = GE.Replace('\\', '_');
                        }


                        Dam_Search.AppendFormat(@"
                                      <SEARCH>
                                            <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{4}'></SESSIONINFO>
                                            <TARGETOBJECT>SECTION</TARGETOBJECT>
	                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                        <TARGETENTITY>
		                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <FILTERLIST>
                                                        <FILTER>
			                                                <EXPRESSION>NAME</EXPRESSION>
			                                                <OPERATOR>EQUALTO</OPERATOR>
			                                                <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
			                                                <FILTERID>1</FILTERID>
			                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
		                                                </FILTER>
	                                                </FILTERLIST>
                                                 <RELATIONSHIP NAME='SECTION_DIGITALASSET'> 
                                                     <FILTERLIST>
                                                        <FILTER>
			                                                <EXPRESSION>MARKEDASDELETED</EXPRESSION>
			                                                <OPERATOR>EQUALTO</OPERATOR>
			                                                <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
			                                                <FILTERID>1</FILTERID>
			                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
		                                                </FILTER>
	                                                </FILTERLIST>                                                                                                               
                                                     <RELATIONSHIP NAME='DIGITALASSET_CURRENTREVISION'>
                                                    <RELATIONSHIP NAME='CONTENTREVISION_RENDITION'>
                                                    </RELATIONSHIP>
                                                </RELATIONSHIP>
                                              </RELATIONSHIP>
                                            </TARGETENTITY>
                                        </SEARCH>                                               
                                       ", CurrentLanguage, TenantName, UserID, GE, SessionID);
                        String Damsearchresult = null;
                        SearchConfiguration ObjSearchConfiguration = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);

                        Er_XML = ObjSearchConfiguration.GetHierarchialData(Dam_Search.ToString(), ref Damsearchresult);
                        if (Er_XML != null)
                        {
                            UpdateExportStatusXML(Er_XML);
                            UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                            ErrorXML = ALSolutionCommon.PrepareErrorXML("Searching for DAM items with GetHierarchicalData returned ERROR");
                            ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                            return;
                        }
                        else
                        {
                            UpdateExportStatusXML("Searching For DAM Items Complete.");
                        }

                        ObjSearchConfiguration = null;



                        StringBuilder Path = new StringBuilder();
                        GEName = ReplaceSpecialChars_InvalidOnFileSystem(GEName);

                        Path.AppendFormat(@"{0}\{1}", DownloadPath, GEName);
                        System.IO.Directory.CreateDirectory(Path.ToString());
                        UpdateExportStatusXML("Getting DAM Items");
                        File.WriteAllText(Path + "\\AssetDetails.xml", Damsearchresult);
                        ErrorXML = GetDamItems(RequestID, FormatID, Damsearchresult, Path.ToString());
                        if (ErrorXML != null)
                        {
                            UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                            return;
                        }
                        
                    }

                } // end of For

                String FormatName_FileName = ReplaceSpecialChars_InvalidOnFileSystem(FormatName);
                FormatName_FileName = FormatName_FileName.Replace(" ", "_");  //Reason for doing this is when we download save dialog box replaces space with underscore character, so while importing it searches for the XML which is not present since name has been changed.
                FormatName_FileName = FormatName_FileName.Replace("#", "_");  //Reason for doing this is upload control gives error for #,; and + characters
                FormatName_FileName = FormatName_FileName.Replace(";", "_");
                FormatName_FileName = FormatName_FileName.Replace("+", "_");
                FormatFile.Save(DownloadPath + "\\Format_Import.xml");

                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.PreparingZip);


            #endregion

                #region Prepare Zip
                if (ZipRequired == true)
                {
                    String ZipName = FormatName_FileName + "_" + FormatID;

                    File.WriteAllText(DownloadPath + "\\ZipName.txt", ZipName + ".zip");

                    ErrorXML = PrepareZip(DownloadPath, DownloadPath, ZipName);
                    if (ErrorXML != null)
                    {
                        UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                        return;
                    }
                }
                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Done);
                #endregion

                return;
            }
            catch (Exception e)
            {
                String EMs = e.Message;
                UpdateExportStatusXML(EMs);
                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                ArtworkProducerWSobj.LogError("FormatFacade", e.Message, ErrorType.EXCEPTION, null, null);
                return;
            }
            #endregion

        }

        #endregion



        #region To get All DAM ITEMS

        /// <summary>
        /// <purpose>This method retrieves the Graphic Elements for Format.</purpose>
        /// </summary>
        /// <param name="DamXml"></param>
        /// <param name="Path"></param>

        public ERRORXML GetDamItems(String RequestID, String FormatID, String DamXml, String Path)
        {


            #region variable
            XPathDocument XPathDoc = null;
            XmlDocument DamFormatFile = null;
            StringReader XPathDocXML = null;
            XPathNavigator XPathNav = null;
            XPathNavigator FileNode = null;
            XPathNodeIterator FileNodeList = null;
            XPathNodeIterator DamFileNodeList = null;


            FileFacade FileFacadeObj = null;
            FileTransfer FileTransferObj = null;
            String FileSystemXML = null;
            StringBuilder FileFacadeXML = null;
            String ErrorXML = null;
            String ObjectID = null;

            # endregion

            FileFacadeObj = new FileFacade(SessionID, UserID, TenantName, CurrentLanguage);
            FileTransferObj = new FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);
            XPathDocXML = new StringReader(DamXml);
            XPathDoc = new XPathDocument(XPathDocXML);
            DamFormatFile = new XmlDocument();
            FileFacadeXML = new StringBuilder();


            try
            {
                DamFormatFile.LoadXml(DamXml);
                XPathNav = XPathDoc.CreateNavigator();
                XPathNavigator XNavD = XPathDoc.CreateNavigator();
                FileNodeList = XNavD.Select("SECTIONCOLLECTION/SECTION/SECTION_DIGITALASSET/DIGITALASSET/DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION");


                if (FileNodeList.Count > 0)
                {
                    while (FileNodeList.MoveNext())
                    {


                        DamFileNodeList = FileNodeList.Current.Select("FILESYSTEM/FILE");
                        while (DamFileNodeList.MoveNext())
                        {
                            FileNode = DamFileNodeList.Current;

                            FileSystemXML = FileNode.OuterXml;
                            if (FileSystemXML.StartsWith("<FILE>"))
                            {
                                //Get File Name

                                #region Get File From Engine to WebService
                                ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                                ObjectID = ObjectID.Replace("<OBJECTID>", "");
                                FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                                FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                                FileFacadeXML.Append(FileSystemXML);
                                #endregion
                                ErrorXML = FileFacadeObj.GetFile(ref FileSystemXML);
                                if (ErrorXML != null)
                                {
                                    UpdateExportStatusXML(ErrorXML);
                                    UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                                    ErrorXML = ALSolutionCommon.PrepareErrorXML("Error while Format Files From OM to WebService");
                                    ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                                    return ErrorXML;
                                }
                                else
                                {
                                    String GetFileStatus = null;
                                    GetFileStatus = "Successful In Getting DAM Items From OM to WebService";
                                    UpdateExportStatusXML(GetFileStatus);
                                }


                                #region Get File from WebService to local Machine
                                if (ErrorXML == null)
                                {
                                    FileSystemXML = FileSystemXML.Replace("<PATH>", "<SOURCEPATH>");
                                    FileSystemXML = FileSystemXML.Replace("</PATH>", "</SOURCEPATH>");
                                    FileTransferObj.GetFilePath = Path;
                                    //  FileTransferObj.GetFilePath = ;
                                    ErrorXML = FileTransferObj.GetFile(ref FileSystemXML);
                                    if (ErrorXML != null)
                                    {
                                        UpdateExportStatusXML(ErrorXML);
                                        UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                                        ErrorXML = ALSolutionCommon.PrepareErrorXML("Error while Format Files From WebService to LocalMachine");
                                        ArtworkProducerWSobj.LogError("FormatFacade", ErrorXML, ErrorType.ERROR, null, null);
                                        return ErrorXML;
                                    }
                                    else
                                    {
                                        String GetFileStatus_FT = null;
                                        GetFileStatus_FT = "Successful In Getting DAM Items From WebService to Local Machine";
                                        UpdateExportStatusXML(GetFileStatus_FT);
                                        UpdateExportStatusXML("DONE");


                                    }

                                }
                                #endregion

                            }
                        }


                    } // end of while

                } // end of IF
                //   ZipFormatFolder();
                return null;
            }
            catch (Exception e)
            {
                String DExMsg = null;
                DExMsg = e.Message;
                UpdateExportStatusXML(DExMsg);
                UpdateExportFormatStatus(RequestID, FormatID, FormatExportStatus.Error);
                ArtworkProducerWSobj.LogError("FormatFacade", e.Message, ErrorType.EXCEPTION, null, null);
                return "ERROR";
            }

        }




        #endregion

        #region Updating EXPORT Status XML



        public void UpdateExportStatusXML(String FStatus)
        {
            StringBuilder sbInputXML = null;
            try
            {
                sbInputXML = new StringBuilder();

                sbInputXML.AppendFormat(@"<FORMATEXPORTSTATUS>
                                    <EXPORTSTATUS>{0}</EXPORTSTATUS>
                                    </FORMATEXPORTSTATUS>", FStatus);
                File.WriteAllText(ALSolutionCommon.GetTempPath(SessionID) + "ExportStatus.xml", sbInputXML.ToString());
            }
            catch (Exception ExcepObj)
            {
                ArtworkProducerWSobj.LogError("FormatFacade", ExcepObj.Message, ErrorType.EXCEPTION, null, null);
            }
        }

        #endregion


        private void CreateDirectory(System.IO.DirectoryInfo oDirInfo)
        {
            if (oDirInfo.Parent != null)
                CreateDirectory(oDirInfo.Parent);
            if (!oDirInfo.Exists)
            {
                oDirInfo.Create();
            }
        }

        //********************************************************************************************************//
        ///-------------------------------------------------------------------------------------------------------///
        #endregion VARUNs_Changes


        #region Methods Related to InDesign Format Creation

        public string CreateInDesignFormat(String ZipFilePath, String CampaignID, String DesignID, String FormatDetailsXML)
        {
            String FormatName = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref FormatDetailsXML, "<FORMATNAME>", "</FORMATNAME>"));
            ReplaceSpecialCharacters(ref FormatName);
            String JobPath = CreateNTFSHierarchy(CampaignID, DesignID, FormatName);

            if (JobPath == null)
            {
                //ERROR
            }

            ArrayList objParams = new ArrayList(5);
            String RequestID = Guid.NewGuid().ToString();

            objParams.Add(JobPath);
            objParams.Add(ZipFilePath);
            objParams.Add(FormatDetailsXML);
            objParams.Add(RequestID);

            String TempPathToUnZip = PrepareJobPath_CreateInDesignFormat(RequestID);
            objParams.Add(TempPathToUnZip);
            Directory.CreateDirectory(TempPathToUnZip);
            Update_CreateInDesignFormatJobStatus(RequestID, CreateIndesignFormatStatus.UnzippingFormatFile, null);
            File.WriteAllText(TempPathToUnZip + "RequestID.txt", RequestID);

            OrgIdentity = System.Security.Principal.WindowsIdentity.GetCurrent();
            ThreadPool.QueueUserWorkItem(new WaitCallback(SubmitJobToFormatWatcher), objParams);

            return RequestID;
        }

        String CreateNTFSHierarchy(String CampaignID, String DesignID, String FormatName)
        {
            //            String str_SearchOP = null;
            //            Search objSearch = new Search();
            //            StringBuilder sbSearchXML = new StringBuilder(300);
            //            sbSearchXML.AppendFormat(@" <SEARCH>
            //                                        <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}'></SESSIONINFO>
            //                                        <TARGETOBJECT>ARTWORKCAMPAIGN</TARGETOBJECT>
            //                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
            //                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
            //                                        <TARGETENTITY>
            //	                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
            //                                            <FILTERLIST>
            //                                                <FILTER>
            //	                                                <EXPRESSION>CAMPAIGNID</EXPRESSION>
            //	                                                <OPERATOR>EQUALTO</OPERATOR>
            //	                                                <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
            //	                                                <FILTERID>1</FILTERID>
            //	                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
            //                                                </FILTER>
            //                                            </FILTERLIST>
            //                                            <SELECTLIST>
            //                                                <EXPRESSION>
            //                                                    <FIELDNAME>CAMPAIGNNAME</FIELDNAME>
            //                                                    <ALIAS>CAMPAIGNNAME</ALIAS>
            //                                                </EXPRESSION>
            //                                            </SELECTLIST>
            //                                            <RELATIONSHIP NAME='CAMPAIGN_DESIGN'>
            //                                                <FILTERLIST>
            //                                                    <FILTER>
            //	                                                    <EXPRESSION>DESIGNID</EXPRESSION>
            //	                                                    <OPERATOR>EQUALTO</OPERATOR>
            //	                                                    <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
            //	                                                    <FILTERID>1</FILTERID>
            //	                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
            //                                                    </FILTER>
            //                                                </FILTERLIST>
            //                                                <SELECTLIST>
            //                                                    <EXPRESSION>
            //                                                        <FIELDNAME>DESIGNNAME</FIELDNAME>
            //                                                        <ALIAS>DESIGNNAME</ALIAS>
            //                                                    </EXPRESSION>
            //                                                </SELECTLIST>
            //                                            </RELATIONSHIP>
            //                                        </TARGETENTITY>
            //                                    </SEARCH>", CurrentLanguage, TenantName, UserID, CampaignID, DesignID);

            //            ErrorXML = objSearch.GetHierarchialData(sbSearchXML.ToString(), ref str_SearchOP);

            //            if (ErrorXML != null)
            //            { 
            //                //ERROR
            //            }

            //            String CampaignName = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref str_SearchOP, "<CAMPAIGNNAME>", "</CAMPAIGNNAME>"));
            //            CampaignName = ReplaceSpecialChars_InvalidOnFileSystem(CampaignName);

            //            String DesignName = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref str_SearchOP, "<DESIGNNAME>", "</DESIGNNAME>"));
            //            DesignName = ReplaceSpecialChars_InvalidOnFileSystem(DesignName);

            XPathDocument xpDoc_TenantXML = new XPathDocument(ALCommon.GetTenantXMLPath(TenantName));
            String FWMountPoint = xpDoc_TenantXML.CreateNavigator().SelectSingleNode("/CONFIGURATION/ARTWORKPRODUCER/FORMATWATCHERCONFIGURATION/FORMATWATCHERMOUNTPOINT").Value;
            StringBuilder sbMountPoint = new StringBuilder(80);

            sbMountPoint.Append(FWMountPoint);

            if (!Directory.Exists(sbMountPoint.ToString()))
            {
                //ERROR
            }

            //String UserName = GetUserName(UserID);
            //sbMountPoint.AppendFormat("\\{0}", UserName);

            //String tmpFolder = sbMountPoint.ToString() + "\\Created through UI\\" + CampaignID + "\\" + DesignID + "\\" + FormatName;
            sbMountPoint.AppendFormat("\\ThroughUI\\{3}\\{0}\\{1}\\{2}", CampaignID, DesignID, FormatName, UserID);
            String tmpFolder = sbMountPoint.ToString();
            if (Directory.Exists(tmpFolder))
            {
                ulong iUniqueID = 0;
                ALCommon.GenerateUniqueID(ref iUniqueID);
                tmpFolder += "_" + iUniqueID;
            }
            Directory.CreateDirectory(tmpFolder);
            return tmpFolder;
        }

        void SubmitJobToFormatWatcher(object Params)
        {
            System.Security.Principal.WindowsImpersonationContext NewContext = null;

            try
            {
                System.Threading.Thread.CurrentPrincipal = new System.Security.Principal.WindowsPrincipal(OrgIdentity);
                NewContext = OrgIdentity.Impersonate();

                ArrayList lstParams = (ArrayList)Params;
                String JobPath = (String)lstParams[0];
                String ZipFilePath = (String)lstParams[1];
                String FormatDetailsXML = (String)lstParams[2];
                String RequestID = (String)lstParams[3];
                String TempPathToUnZip = (String)lstParams[4];

                //String TempPathToUnZip = PrepareJobPath_CreateInDesignFormat(RequestID);

                Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper objZipWrapper = new Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper();

                String result = objZipWrapper.UncompressZipFile(ZipFilePath, TempPathToUnZip);
                if (result != "SUCCESS")
                {
                    //ERROR
                }

                DirectoryInfo objTempDir = new DirectoryInfo(TempPathToUnZip);
                FileInfo[] objFileList = objTempDir.GetFiles();

                foreach (FileInfo objFile in objFileList)
                {
                    if (objFile.Name.EndsWith("_INFO.XML", StringComparison.InvariantCultureIgnoreCase))
                        continue;

                    if (objFile.Name == "status.txt")
                        continue;

                    File.Move(objFile.FullName, JobPath + "\\" + objFile.Name);
                }


                #region Drop the FormatDetailsXML

                File.WriteAllText(JobPath + "\\FormatDetails.xml", FormatDetailsXML);

                #endregion


                objFileList = objTempDir.GetFiles();

                if (objFileList.Length < 2)
                {
                    Update_CreateInDesignFormatJobStatus(RequestID, CreateIndesignFormatStatus.Error, null);
                    return;
                }

                foreach (FileInfo objFile in objFileList)
                {
                    if (objFile.Name == "status.txt")
                        continue;

                    File.Move(objFile.FullName, JobPath + "\\" + objFile.Name);
                }
            }
            finally
            {
                NewContext.Undo();
            }
        }

        String GetUserName(String UserID)
        {
            String UserXML = null;
            StringBuilder sbUserXMLPath = new StringBuilder(30);
            sbUserXMLPath.AppendFormat("{0}{1}.xml", ALSolutionCommon.GetDataFolderPath(TenantName, UserID), UserID);
            String UserXMLPath = sbUserXMLPath.ToString();
            if (File.Exists(UserXMLPath))
            {
                UserXML = File.ReadAllText(UserXMLPath);
                return ALCommon.GetInnerValue(ref UserXML, "<LOGINNAME>", "</LOGINNAME>");
            }
            else
            {
                StringBuilder sbSearchXML = new StringBuilder(80);
                sbSearchXML.AppendFormat(@" <SEARCH>
                                        <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'></SESSIONINFO>
                                        <TARGETOBJECT>USER</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
	                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
	                                                <EXPRESSION>USERID</EXPRESSION>
	                                                <OPERATOR>EQUALTO</OPERATOR>
	                                                <EXPRESSIONVALUE>{2}</EXPRESSIONVALUE>
	                                                <FILTERID>1</FILTERID>
	                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                        
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>LOGINNAME</FIELDNAME>
                                                    <ALIAS>LOGINNAME</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </TARGETENTITY>
                                    </SEARCH>", CurrentLanguage, TenantName, UserID, SessionID);
                Search objSearch = new Search();
                objSearch.Url = ALCommon.GetWebServiceURL("ALSearchWS");
                ErrorXML = objSearch.GetHierarchialData(sbSearchXML.ToString(), ref UserXML);
                if (ErrorXML != null)
                {
                    //ERROR
                }

                return ALCommon.GetInnerValue(ref UserXML, "<LOGINNAME>", "</LOGINNAME>");
            }
        }

        static String PrepareJobPath_CreateInDesignFormat(String RequestID)
        {
            return ALCommon.GetTempPath() + "\\" + RequestID + "\\";
        }

        public static void Update_CreateInDesignFormatJobStatus(String RequestID, CreateIndesignFormatStatus oStatus, String FormatID)
        {
            String TempPath = PrepareJobPath_CreateInDesignFormat(RequestID);
            File.WriteAllText(TempPath + "status.txt", oStatus.ToString());

            if (FormatID != null)
            {
                File.WriteAllText(TempPath + "FormatID.txt", FormatID);
            }
        }

        public static CreateIndesignFormatStatus Get_CreateInDesignFormatJobStatus(String RequestID)
        {
            String str_Path = PrepareJobPath_CreateInDesignFormat(RequestID) + "status.txt";

            if (!File.Exists(str_Path))
                return CreateIndesignFormatStatus.Error;

            String str_status = File.ReadAllText(str_Path);

            switch (str_status)
            {
                case "UnzippingFormatFile":
                    return CreateIndesignFormatStatus.UnzippingFormatFile;

                case "CreatingFormat":
                    return CreateIndesignFormatStatus.CreatingFormat;

                case "Done":
                    return CreateIndesignFormatStatus.Done;

                case "Error":
                    return CreateIndesignFormatStatus.Error;

                case "ValidatingJob":
                    return CreateIndesignFormatStatus.ValidatingJob;

                default:
                    return CreateIndesignFormatStatus.Error;
            }
        }

        public static String GetFormatID(String RequestID)
        {
            String str_Path = PrepareJobPath_CreateInDesignFormat(RequestID) + "FormatID.txt";
            if (File.Exists(str_Path))
                return File.ReadAllText(str_Path);
            else
                return null;
        }

        public static void ReplaceSpecialCharacters(ref String InputString)
        {
            String CurrentStr = "";
            for (int charCnt = 0; charCnt < InputString.Length; charCnt++)
            {
                if ((InputString[charCnt] >= 'a' && InputString[charCnt] <= 'z') ||
                    (InputString[charCnt] >= 'A' && InputString[charCnt] <= 'Z') ||
                    (InputString[charCnt] >= '0' && InputString[charCnt] <= '9'))
                {
                    CurrentStr = CurrentStr + InputString[charCnt].ToString();
                }
                else
                    CurrentStr = CurrentStr + "_";
            }
            InputString = CurrentStr.ToString();
        }

        #endregion

    }
}